/*  Module Name: @(#)ACSE.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: DBV-OSI II API code to gain information from the ACSE
	configuration file.
 
*/

#include "ACSE.h"

/* Check to see if the ACSE cnofiguration file exists. Used by the target
	which reacts in a more complex manner than the origin */
int DBVDoesFileExist(DbvError **ppsError)
{
	int return_value = SI_OK;
	TBL InTbl;

	if((InTbl = TBL_Create(DBVACSEFileName, TBL_Read))==NULL)
	{
		/* Missing ACSE_FILE */
		DBVfill_error_structure(0, SI_Interface_Error, 
			0, DBV_ACSE_MISSING_FILE, DBV_ACSE_MISSING_FILE_S, 0, 
			Association_Open, ppsError);
		return_value = SI_NOTOK;
	}
	else
		TBL_Destruct(InTbl);

	return return_value;
}

/* The functionality of the target is slightly different to that of the origin
*/
int DBVGetACNAtTarget( char **ppcContext, DbvAssociationState eAssociationState,
	DbvError **ppsError)
{
	int return_value = SI_OK;
	char pcSR[] = BASIC_SR_OID;

	DBVbSR = DBVbZ3950 = DBV_FALSE;

	/* Check to see if the configuration file exists */
	return_value = DBVDoesFileExist(ppsError);
	if(return_value == SI_NOTOK)
	{

		/* The file does not exist so use the default */
		DBVWriteLog("Warning: Using default Application Context: Basic_SR_Application_Context (1.0.10163.1.1)");
		DBVbSR=DBV_TRUE;
		/* Use the default Context */
		if(DBVAllocCheck((*ppcContext=(char *)
			DBVAlloc(strlen(BASIC_SR_OID)+1,
			ppsError)))==SI_NOTOK)
	        { return SI_NOTOK; }
		strcpy(*ppcContext, BASIC_SR_OID);
	}
	else
	{
		return_value = DBVGetACN(ppcContext, eAssociationState, T,
			ppsError);
/*
		if(return_value == SI_NOTOK)
		{
			DBVWriteLog("Warning: Using default Application Context: Basic_SR_Application_Context (1.0.10163.1.1)");
			DBVbSR=DBV_TRUE;
			** Use the default Context **
			if(DBVAllocCheck((*ppcContext=(char *)
				DBVAlloc(strlen(BASIC_SR_OID)+1,
				ppsError)))==SI_NOTOK)
	        	{ return SI_NOTOK; }
			strcpy(*ppcContext, BASIC_SR_OID);
		}
*/
	}
	/* Make sure the error structure indicates the correct state of the 
		association (TBL_GetRecord() always returns Association_Closed*/
	if(return_value == SI_NOTOK && *ppsError != NULL)
		(*ppsError)->eAssociationState = Association_Open;

	return return_value;
}

/* This function retrieves the Application Context Name from the ACSE 
	configuration file. Used by the Origin. */
int DBVGetACN(char **ppcContext, DbvAssociationState eAssociationState,
	DBVOriginOrTarget eOorT, DbvError **ppsError)
{
	int return_value = SI_OK;
	char *pcRecordTitle;

	char pcZ3950[] = BASIC_Z3950_OID;
	char pcSR[] = BASIC_SR_OID;
	long lACNIndex=0;
	DbvBool bCompleted=DBV_FALSE;
	TBL_Return T_Return = NULL;
	TBL InTbl = NULL;

	DBVbSR = DBVbZ3950 = DBV_FALSE;

	T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
	/* if((InTbl = TBL_Create(DBVACSEFileName, TBL_Read))==NULL) */
	if(1==1)
	{
		/* File Does not exist */
		/* Use the default ACN */
		sprintf(DBVErrorBuffer,"Warning: ACSE configuration file %s does not exist", DBVACSEFileName);
		DBVWriteLog(DBVErrorBuffer);
	}
	else
	{
		/* Read records until we find the title 
			"Application_Context_Name" */
		while(bCompleted == DBV_FALSE && return_value == SI_OK && 
			TBL_GetRecordDbv(InTbl, ppsError)!=DBV_FALSE)
		{
			if(TBL_GetField(InTbl, T_Return)!=FALSE)
			{
			if(T_Return->Tag != TBL_UText)
			{
				/* FORMAT IS WRONG */
				sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
					"Characters", DBVGetFieldTypeString(
					T_Return->Tag));
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
					eAssociationState, ppsError);

				return_value = SI_NOTOK;
			}
			else
			{
				pcRecordTitle=T_Return->Value.UText;
				if(strcmp(pcRecordTitle, 
					"Application_Context_Name")==0)
				{
					return_value = DBVReadACNs(InTbl, 
						ppcContext, eAssociationState,
						eOorT, ppsError);
					bCompleted = DBV_TRUE;
				}
				free(pcRecordTitle);
			}
			}
		}
		/* Check if TBL_GetRecord filled out the error structure */
		if(*ppsError != NULL)
			return_value = SI_NOTOK;
	}
	if((*ppcContext == NULL && return_value == SI_OK) || 
		(*ppcContext == NULL && eOorT == T))
	{
		DBVWriteLog("Warning: Using default Application Context: Basic_SR_Application_Context (1.0.10163.1.1)");
		DBVbSR=DBV_TRUE;
		/* Use the default Context */
		if(DBVAllocCheck((*ppcContext=(char *)
			DBVAlloc(strlen(BASIC_SR_OID)+1,
			ppsError)))==SI_NOTOK)
	        { return SI_NOTOK; }
		strcpy(*ppcContext, BASIC_SR_OID);
	}

	if(InTbl!=NULL)
		TBL_Destruct(InTbl);
	free(T_Return);

	return return_value;
}

/* Read in the Application Contexts from the file */
int DBVReadACNs(TBL InTbl, char **ppcContext, DbvAssociationState eAssociationState,
	DBVOriginOrTarget eOorT, DbvError **ppsError)
{
	int return_value = SI_OK;

	char pcZ3950[] = BASIC_Z3950_OID;
	char pcSR[] = BASIC_SR_OID;

	char **ppcACNs=NULL;
	long lACNIndex=0;
	char *pcTempContext=NULL;

	TBL_Return T_Return;

	ppcACNs = (char **)DbvAlloc(sizeof(char *)*2);
	T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));

	while(return_value == SI_OK && lACNIndex < 2 && 
		(TBL_GetField(InTbl, T_Return)!=FALSE))
	{
		if(T_Return->Tag != TBL_QText)
		{
			/* FORMAT IS WRONG */
			sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
				"String", DBVGetFieldTypeString(
				T_Return->Tag));
			DBVfill_error_structure(0, SI_Interface_Error, 
				0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
				eAssociationState, ppsError);
		 	return_value = SI_NOTOK;
		}
		else
		{
			pcTempContext=T_Return->Value.UText;
			ppcACNs[lACNIndex]=DBVConvertToOID(pcTempContext);
/*
			if(DBVAllocCheck((ppcACNs[lACNIndex]=(char *)
				DBVAlloc(strlen(pcTempContext)+1, 
				ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }
			strcpy(ppcACNs[lACNIndex], 
				DBVConvertToOID(pcTempContext));
*/
			lACNIndex++;
		}
	}
	/* There should not be more than two entries */
	if(lACNIndex>2)
	{
		sprintf(DBVErrorBuffer, DBV_ACSE_FORMATERROR_S, "No more than two Application Contexts should be supplied");
		DBVfill_error_structure(0, SI_Interface_Error, 0,
			DBV_ACSE_FORMATERROR, DBVErrorBuffer, 0, 
			eAssociationState,ppsError);
		return_value = SI_NOTOK;
	}
	if(lACNIndex==0)
	{
		if(DBVAllocCheck((ppcACNs[0]=(char *)DBVAlloc(
			strlen(BASIC_SR_OID)+1, ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
		strcpy(ppcACNs[lACNIndex], BASIC_SR_OID);
		DBVWriteLog("Warning: Using default Application Context: Basic_SR_Application_Context (1.0.10163.1.1)");
	}
	/* Is what we've got a valid ACN? */
	if(return_value == SI_OK)
		return_value = DBVValidateACN(ppcACNs, eAssociationState, 
			eOorT, ppsError);

	/* If everything's OK, copy the ACN to *ppcContext */
	if(return_value == SI_OK || eOorT == T)
	{
		*ppcContext=(char *)DbvAlloc(strlen(*ppcACNs)+1);
		strcpy(*ppcContext, *ppcACNs);
	}

	if(ppcACNs[0]!=NULL) DbvFree(ppcACNs[0]);
	if(ppcACNs[1]!=NULL) DbvFree(ppcACNs[1]);
	DbvFree(ppcACNs);
	free(T_Return);
	return return_value;
}

/* Take the list of ACNs (as OIDs), verify them, and return the one to use */
/* If we are in a target, the behaviour is more complicated, because we can't */
/* just return NOT_OK. */
int DBVValidateACN(char **ppcACNs, DbvAssociationState eAssociationState,
	DBVOriginOrTarget eOorT, DbvError **ppsError)
{
	char *pcSelectedACS;
	long lACNIndex=0;
	DbvBool bTargetError = DBV_FALSE;
	long lInvalidACNs[2];
	int return_value = SI_OK;

	DBVbSR = DBVbZ3950 = DBV_FALSE;
	lInvalidACNs[0]=0; lInvalidACNs[1]=0;
	
	/* There will only be two! */
	while(lACNIndex<2 && return_value == SI_OK)
	{
		if(ppcACNs[lACNIndex]!=NULL)
		{
			if(strcmp(ppcACNs[lACNIndex],BASIC_Z3950_OID)==0)
				DBVbZ3950=DBV_TRUE;
			else if(strcmp(ppcACNs[lACNIndex],BASIC_SR_OID)==0)
				DBVbSR=DBV_TRUE;
			else
			{
				/* ACN is not valid */
				 sprintf(DBVErrorBuffer, 
					DBV_ACSE_INVALID_ACN_VALUE_S,
					ppcACNs[lACNIndex]);
				 DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_ACSE_INVALID_ACN_VALUE, 
					DBVErrorBuffer, 0, eAssociationState,
					ppsError);
				if(eOorT == O)
				{
				 return_value = SI_NOTOK;
				}
				else
				{
					lInvalidACNs[lACNIndex]=1;
					bTargetError = DBV_TRUE;
				}
			}
		}
		lACNIndex++;
	}
	if(eOorT == T)
	{
		if(lInvalidACNs[0] == 0)
		{
			/* First Entry is valid */
		}
		else if(lInvalidACNs[1] == 0)
		{
			/* Seconds entry is valid, so move it to the first
				entry */
			pcSelectedACS=ppcACNs[1];
			if(ppcACNs[0]!=NULL)
				DbvFree(ppcACNs[0]);
			if(DBVAllocCheck((ppcACNs[0]=(char *)
                                DBVAlloc(strlen(pcSelectedACS)+1,ppsError)))
				==SI_NOTOK)
		        { return SI_NOTOK; }
				strcpy(ppcACNs[0],pcSelectedACS);
		}
		else
		{
			/* Both Invalid */
			DBVWriteLog("Warning: Using default Application Context: Basic_SR_Application_Context (1.0.10163.1.1)");
			if(ppcACNs[0]!=NULL)
				DbvFree(ppcACNs[0]);
			if(DBVAllocCheck((ppcACNs[0]=(char *)
				DBVAlloc(strlen(BASIC_SR_OID)+1,
				ppsError)))==SI_NOTOK)
		        { return SI_NOTOK; }
			strcpy(ppcACNs[0], BASIC_SR_OID);
		}
	}
	if(bTargetError == DBV_TRUE)
		return_value = SI_NOTOK;

	if(return_value==SI_OK)
	{
	
		if(strcmp(ppcACNs[0],BASIC_SR_OID)==0)
		{
			if(ppcACNs[0]!=NULL)
				DbvFree(ppcACNs[0]);
			if(DBVAllocCheck((ppcACNs[0]=(char *)
				DBVAlloc(strlen(BASIC_SR_OID)+1,
				ppsError)))==SI_NOTOK)
		        { return SI_NOTOK; }
			strcpy(ppcACNs[0], BASIC_SR_OID);

		}
		if(strcmp(ppcACNs[0],BASIC_Z3950_OID)==0)
		{
			if(ppcACNs[0]!=NULL)
				DbvFree(ppcACNs[0]);
			if(DBVAllocCheck((ppcACNs[0]=(char *)
				DBVAlloc(strlen(BASIC_Z3950_OID)+1,
				ppsError)))==SI_NOTOK)
		        { return SI_NOTOK; }
			strcpy(ppcACNs[0], BASIC_Z3950_OID);
		}
	}
	/* Free the array */
	if(ppcACNs[1]!=NULL)
	{
		DbvFree(ppcACNs[1]);
		ppcACNs[1] = NULL;
	}

	return return_value; 
} 
/* Scans and Converts the strings Basic_Z3950_Application_Context and 
	Basic_SR_Application_Context to their OID equivalents */
char *DBVConvertToOID(char *pcTempContext)
{
	char *pcReturn=NULL;

	if(strcmp(pcTempContext, "Basic_Z3950_Application_Context") == 0)
	{
		pcReturn=(char *)DbvAlloc(strlen(BASIC_Z3950_OID)+1);
		strcpy(pcReturn,BASIC_Z3950_OID);
	}
	if(strcmp(pcTempContext, "Basic_SR_Application_Context") == 0)
	{
		pcReturn=(char *)DbvAlloc(strlen(BASIC_SR_OID)+1);
		strcpy(pcReturn,BASIC_SR_OID);
	}

	/* If the ACN was neither of the above, just return the string */
	if(pcReturn==NULL)
	{
		pcReturn=(char *)DbvAlloc(strlen(pcTempContext)+1);
		strcpy(pcReturn,pcTempContext);
	}

	return pcReturn;
}


#ifndef NO_OSI
/* Fills out the Presentation Context Definition List by reading the ACSE config
	file. Adds the application contexts depending on what has already been 
	read by the ACN functions. Used by the Origin */
int DBVGetPCDL(struct PSAPctxlist *psPresentationContextList,
	DbvAssociationState eAssociationState, DbvError **ppsError)
{
	int return_value = SI_OK;
	char *pcRecordTitle;
	DbvBool bReadList=DBV_FALSE;

	TBL_Return T_Return;
	TBL InTbl;

	psPresentationContextList->pc_nctx=0;
	T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
	if((InTbl = TBL_Create(DBVACSEFileName, TBL_Read))==NULL)
	{
		/* File Does not exist */
		/* Use the default PCDL */
		sprintf(DBVErrorBuffer,"Warning: ACSE configuration file %s does not exist", DBVACSEFileName);
		DBVWriteLog(DBVErrorBuffer);
	}
	else
	{
		/* Read records until we find the title 
			"Presentation_Context_Definition_List" */
		while( bReadList==DBV_FALSE && return_value == SI_OK && 
			TBL_GetRecordDbv(InTbl, ppsError)!=DBV_FALSE)
		{
			if(TBL_GetField(InTbl, T_Return)!=FALSE)
			{
			if(T_Return->Tag != TBL_UText)
			{
				/* FORMAT IS WRONG */
				sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
					"Characters", DBVGetFieldTypeString(
					T_Return->Tag));
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
					eAssociationState, ppsError);
				return_value = SI_NOTOK;
			}
			else
			{
			pcRecordTitle=(char *)DbvAlloc(strlen(T_Return->Value.
				UText)+1);
				strcpy(pcRecordTitle,T_Return->Value.UText);
				if(strcmp(pcRecordTitle, 
				    "Presentation_Context_Definition_List")==0)
				{
					bReadList=DBV_TRUE;
					return_value = DBVReadPCDLs(InTbl, 
						psPresentationContextList, 
						eAssociationState, ppsError);
				}
			}
			}
		}
		/* Check if TBL_GetRecord filled out the error structure */
		if(*ppsError != NULL)
			return_value = SI_NOTOK;
	}

	if(return_value==SI_OK && bReadList==DBV_FALSE)
	{
		/* There was no PCDL entry in the config file */
		DBVWriteLog("Warning: No Presentation Context Definition list was supplied. Only ASCE PCI version and ACN oid(s) will be sent");
		
	}

	if(return_value==SI_OK)
	{
		/* Add the application contexts */
		return_value = DBVAddTheApplicationContexts(
			psPresentationContextList, ppsError);
	}
	if(InTbl != NULL)
		TBL_Destruct(InTbl);
	free(T_Return);

	return return_value;
}


int DBVReadPCDLs(TBL InTbl, struct PSAPctxlist *psPresentationContextList,
	DbvAssociationState eAssociationState, DbvError **ppsError)
{
	long lEntries=0;
	char *pcTempPC;
	char *pcErrorString;

	int return_value = SI_OK;
	TBL_Return T_Return;

	T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
	while(return_value == SI_OK && lEntries < NPCTX && 
		(TBL_GetField(InTbl, T_Return)!=FALSE))
	{
		if(T_Return->Tag != TBL_QText)
		{
			/* FORMAT IS WRONG */
			sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
				"String", DBVGetFieldTypeString(
				T_Return->Tag));
			DBVfill_error_structure(0, SI_Interface_Error, 
				0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
				eAssociationState, ppsError);
			return_value = SI_NOTOK;
		}
		else
		{
			pcTempPC=T_Return->Value.UText;
			/* Add OID to list. If the OID is valid add the 
				string to the ppcDBVProposedPresContexts list
				(used in AssociateRequest()) */
			if((psPresentationContextList->pc_ctx[lEntries].pc_asn=
				str2oid(pcTempPC))==NULLOID)
			{
				/* Construct the error string */
				pcErrorString=(char *)DbvAlloc(strlen(
					": invalid object descriptor ")+
					strlen(pcTempPC)+1);
				sprintf(pcErrorString,
					"%s: invalid object descriptor",
					pcTempPC);
				sprintf(DBVErrorBuffer, DBV_ACSE_FORMATERROR_S,
					pcErrorString);
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_ACSE_FORMATERROR, DBVErrorBuffer,
					0, eAssociationState,ppsError);
				DbvFree(pcErrorString);
				return_value=SI_NOTOK;
			}
			if(return_value == SI_OK && (psPresentationContextList->
				pc_ctx[lEntries].pc_asn=oid_cpy(
				psPresentationContextList->pc_ctx[lEntries].
				pc_asn))== NULLOID)
			{
				sprintf(DBVErrorBuffer, DBV_ACSE_FORMATERROR_S,
					"ISODE function oid_cpy() failed");
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_ACSE_FORMATERROR, DBVErrorBuffer,
					0, eAssociationState,ppsError);
				return_value=SI_NOTOK;
			}
			if(return_value == SI_OK)
			{
				psPresentationContextList->pc_ctx[lEntries].
					pc_id=1;
				psPresentationContextList->pc_ctx[lEntries].
					pc_atn=NULLOID;
				ppcDBVProposedPresContexts[lEntries]=pcTempPC;
			}
			lEntries++;
		}
	}
	if(lEntries == NPCTX)
	{
		/* There were too many entries in the file. ISODE only creates
			enough space for NPCTX entries !! */
		pcErrorString=(char *)DbvAlloc(128);
		sprintf(pcErrorString,"The limit for Presentation Contexts (NPCTX), as defined in your version of ISODE, is: %d (Refer to ISODE header file psap.h).", NPCTX);
		sprintf(DBVErrorBuffer, DBV_ACSE_FORMATERROR_S, pcErrorString);
		DBVfill_error_structure(0, SI_Interface_Error, 0,
			DBV_ACSE_FORMATERROR, DBVErrorBuffer, 0, 
			eAssociationState,ppsError);
		DbvFree(pcErrorString);
		return_value = SI_NOTOK;
	}

	if(lEntries == 0 && return_value == SI_OK)
	{
		DBVWriteLog("Warning: No Presentation Context Definition list was supplied. Only ASCE PCI version and ACN oid(s) will be sent");
	}

	/* Add the number of entries in the list to the structure */
	if(return_value == SI_OK)
		psPresentationContextList->pc_nctx=lEntries;
	

	free(T_Return);
	return return_value;
}



int DBVAddTheApplicationContexts(struct PSAPctxlist *psPresentationContextList,
        DbvError **ppsSIError)

{
	int return_value = SI_OK;
	long lEntries=psPresentationContextList->pc_nctx;

	if(DBVbSR == DBV_TRUE)
	{
		if((psPresentationContextList->pc_ctx[lEntries].pc_asn=
			str2oid(BASIC_SR_OID))==NULLOID)
		{
			printf("%s: unknown object descriptor\n",
				BASIC_SR_OID);
			return_value=SI_NOTOK;
		}
		if(return_value == SI_OK && (psPresentationContextList->
			pc_ctx[lEntries].pc_asn=oid_cpy(
			psPresentationContextList->pc_ctx[lEntries].
			pc_asn))== NULLOID)
		{
			printf("oid_cpy failed\n");
			return_value=SI_NOTOK;
		}
		if(return_value == SI_OK)
		{
			psPresentationContextList->pc_ctx[lEntries].
				pc_id=1;
			psPresentationContextList->pc_ctx[lEntries].
				pc_atn=NULLOID;
			ppcDBVProposedPresContexts[lEntries]=BASIC_SR_OID;
		}
		lEntries++;
	}
	if(DBVbZ3950 == DBV_TRUE)
	{
		if((psPresentationContextList->pc_ctx[lEntries].pc_asn=
			str2oid(BASIC_Z3950_OID))==NULLOID)
		{
			printf("%s: unknown object descriptor\n",
				BASIC_Z3950_OID);
			return_value=SI_NOTOK;
		}
		if(return_value == SI_OK && (psPresentationContextList->
			pc_ctx[lEntries].pc_asn=oid_cpy(
			psPresentationContextList->pc_ctx[lEntries].
			pc_asn))== NULLOID)
		{
			printf("oid_cpy failed, mate\n");
			return_value=SI_NOTOK;
		}
		if(return_value == SI_OK)
		{
			psPresentationContextList->pc_ctx[lEntries].
				pc_id=1;
			psPresentationContextList->pc_ctx[lEntries].
				pc_atn=NULLOID;
			ppcDBVProposedPresContexts[lEntries]=BASIC_Z3950_OID;
		}
		lEntries++;
	}
	psPresentationContextList->pc_nctx=lEntries;

	return return_value;
}

#endif

/* Returns which Contexts are supported by the application. Used by the Target
*/
void DBVGetSupportedContexts(DbvBool *bSR, DbvBool *bZ3950)
{
	*bSR=DBVbSR;
	*bZ3950=DBVbZ3950;
}

/* Compare the given presentation context with the ones in the file. If a 
	match is found return pbMatched as DBV_TRUE, else return DBV_FALSE.
	used by the target. */
int DBVMatchPresentationContext(char *pcProposedContext, DbvBool *pbMatched, 
	DbvError **ppsError)
{
	int return_value = SI_OK;
        DbvBool bReadList=DBV_FALSE;
	char *pcRecordTitle=NULL;
 
        TBL_Return T_Return;
        TBL InTbl;
 
	*pbMatched = DBV_FALSE;
        T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
        if((InTbl = TBL_Create(DBVACSEFileName, TBL_Read))==NULL)
        {
		/* This error should be trapped earlier */
		/* File Does not exist */
		/* Use the default PCDL */
		sprintf(DBVErrorBuffer,"Warning: ACSE configuration file %s does not exist", DBVACSEFileName);
		DBVWriteLog(DBVErrorBuffer);
        }
        else
        {
                /* Read records until we find the title
                        "Presentation_Context_Definition_List" */
                while( bReadList==DBV_FALSE && return_value == SI_OK &&
                        TBL_GetRecordDbv(InTbl, ppsError)!=DBV_FALSE)
                {
                        if(TBL_GetField(InTbl, T_Return)!=FALSE)
                        {
                        if(T_Return->Tag != TBL_UText)
                        {
				/* FORMAT IS WRONG */
				sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
					"Characters", DBVGetFieldTypeString(
					T_Return->Tag));
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
					Association_Open, ppsError);
                                return_value = SI_NOTOK;
                        }
                        else
                        {
                        	pcRecordTitle=T_Return->Value.UText;
                                if(strcmp(pcRecordTitle,
                                    "Presentation_Context_Definition_List")==0)
                                {
                                        bReadList=DBV_TRUE;
                                        return_value = DBVComparePC(InTbl,
                                                pcProposedContext, pbMatched,
                                                ppsError);
                                }
				free(pcRecordTitle);
                        }
                        }
                }
		/* Check if TBL_GetRecord filled out the error structure */
		if(*ppsError != NULL)
			return_value = SI_NOTOK;
        }
	if(return_value == SI_OK && bReadList == DBV_FALSE)
	{
		/* There was no PCDL entry in the config file */
		DBVfill_error_structure(0, SI_Interface_Error, 
			0, DBV_ACSE_MISSING_PCDL, DBV_ACSE_MISSING_PCDL_S, 0, 
			Association_Open, ppsError);
                return_value = SI_NOTOK;
	}

	if(InTbl != NULL)
		TBL_Destruct(InTbl);
	free(T_Return);

	return return_value;
}

/* Reads the list of OIDs. If a match is found pbMatched is set to DBV_TRUE,
	otherwise it is set to DBV_FALSE */
int DBVComparePC(TBL InTbl, char *pcPropsedContext, DbvBool *pbMatched,
	DbvError **ppsError)
{
	int return_value=SI_OK;
	char *pcTempPC;
	int lCount = 0; /* If no PCs are supplied, an error occurred! */
	TBL_Return T_Return;

	*pbMatched = DBV_FALSE;
        T_Return = (struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
        while(return_value == SI_OK && *pbMatched == DBV_FALSE &&
		(TBL_GetField(InTbl, T_Return)!=FALSE))
        {
		lCount ++;
                if(T_Return->Tag != TBL_QText)
                {
                        /* FORMAT IS WRONG */
			sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, 
				"String", DBVGetFieldTypeString(
				T_Return->Tag));
			DBVfill_error_structure(0, SI_Interface_Error, 
				0, DBV_FILE_FORMAT, DBVErrorBuffer, 0, 
				Association_Open, ppsError);
                        return_value = SI_NOTOK;
                }
                else
                {
			if(strcmp(T_Return->Value.UText, pcPropsedContext)==0)
			{
				/* The context is in the file, so we indicate
					DBV_TRUE */
				*pbMatched = DBV_TRUE;
			}
			free(T_Return->Value.UText);
		}
	}
	free(T_Return);
	
	if(return_value == SI_OK && lCount == 0)
	{
		/* There was no PCDL entry in the config file */
		DBVfill_error_structure(0, SI_Interface_Error, 
			0, DBV_ACSE_MISSING_PCDL, DBV_ACSE_MISSING_PCDL_S, 0, 
			Association_Open, ppsError);
                return_value = SI_NOTOK;
	}
	return return_value;
}

char *DBVGetFieldTypeString(int dFieldType)
{
        char *pcType;
 
        pcType=(char *)DbvAlloc(32);
 
        switch(dFieldType)
        {
        case TBL_QText:
                strcpy(pcType,"String");
                break;
        case TBL_UText:
                strcpy(pcType,"Characters");
                break;
        case TBL_Int:
                strcpy(pcType,"Integer");
                break;
        case TBL_Double:
                strcpy(pcType,"Double");
                break;
        case TBL_Comment:
                strcpy(pcType,"Comment");
                break;
        case TBL_NullField:
                strcpy(pcType,"Null");
                break;
        default:
                strcpy(pcType,"Undefined type");
                break;
        }
 
        return pcType;
 
}

