/*  Module Name: @(#)address_by_name.c	7.1
    Version: 11/15/96
    Author: John Hough/Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: DBV-OSI II API Functions used to read in the file
	target_names.tbl and map a given name onto a complete address 
 
*/

#include "address_by_name.h"

/* This function opens the name file, reads the entries, find the first one
	which matches the requested name (if possible) and fills out the
	address structure. The error structure is filled out if the
	name does not exist in the file (or if any errors occur) */
int DBVConvertNameToAddress(char *pcTargetName, 
	DbvTargetInformation **ppsTargetInformation, DbvError **ppsError)
{
	int return_value=SI_OK;
	char *pcTitle = NULL;
	DbvBool lFinished = DBV_FALSE;

	TBL_Return T_Return = NULL;
	TBL InTbl = NULL;
	
	T_Return=(struct tbl_return *)DbvAlloc(sizeof(struct tbl_return));
	if((InTbl = TBL_Create(DBV_NAME_FILE, TBL_Read))==NULL)
	{
#ifdef NO_OSI
	/* Only included if OSI database is not being searched */
		/* The file could not be opened */
		sprintf(DBVErrorBuffer, DBV_FILE_S, DBV_NAME_FILE);
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_FILE, DBVErrorBuffer,0,Association_Closed, 
			ppsError);
		return_value = SI_NOTOK;
#endif
	}
	else if(return_value == SI_OK)
	{
	
		while(TBL_GetRecordDbv(InTbl,ppsError)!=DBV_FALSE && 
			lFinished == DBV_FALSE && return_value == SI_OK)
		{
			if((pcTitle=DBVGetStringField(InTbl, T_Return, 
			    ppsError)) == NULL)
			{
				return_value = SI_NOTOK;
			}
			else if(strcmp(pcTargetName,pcTitle)==0)
			{
				/* We've got a match */
				return_value=DBVCompleteAddress(
					ppsTargetInformation, InTbl, T_Return, 
					ppsError);
				lFinished=DBV_TRUE;
			}
			if(pcTitle != NULL)
				DbvFree(pcTitle);
		}
		/* Check if TBL_GetRecord filled out the error structure */
		if(*ppsError != NULL)
			return_value = SI_NOTOK;
	}
#ifndef NO_OSI
	if(lFinished == DBV_FALSE && return_value == SI_OK)
	{
		/* Check for entries in the isoentities file */
		return_value = DBVCheckISOENTITIESFile(pcTargetName,
			ppsTargetInformation, &lFinished, ppsError);
	}
#endif
	if(lFinished == DBV_FALSE && return_value == SI_OK)
	{
		/* Failed to find a match in the database */
		sprintf(DBVErrorBuffer,DBV_NAME_ENTRY_S, pcTargetName);
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_NAME_ENTRY, DBVErrorBuffer, 0, 
			Association_Closed, ppsError);
		return_value = SI_NOTOK;
		
	}

	if(InTbl!=NULL)
		TBL_Destruct(InTbl);
	free(T_Return);
	return return_value;
}

/* Read in the rest of the fields. Adresses in here will always be for TCP */
int DBVCompleteAddress(DbvTargetInformation **ppsTargetInformation, TBL InTbl, 
	TBL_Return T_Return,DbvError **ppsError)
{
	int return_value=SI_OK;
	char *pcData = NULL;
	DbvTargetInformation *psLocalTargetInformation = NULL;

	/* Malloc the basic bits */
	psLocalTargetInformation=(DbvTargetInformation *)
		DbvAlloc(sizeof(DbvTargetInformation));

	/* It's a TCP/IP address */
	psLocalTargetInformation->eOsiOrInternetAddress = 
		Internet_Address;
	psLocalTargetInformation->uOsiOrInternetAddress.
		psInternetAddress=(DbvInternetAddress *)
		DbvAlloc(sizeof(DbvInternetAddress));

	if((psLocalTargetInformation->uOsiOrInternetAddress.
		psInternetAddress->pcHostAddress=
		DBVGetStringField(InTbl, T_Return, ppsError))
		==NULL)
	{
		/* Error */
		return_value = SI_NOTOK;
	}
		
	if(return_value == SI_OK && (pcData=DBVGetStringField(
		InTbl, T_Return, ppsError)) != NULL)
	{
		psLocalTargetInformation->
			uOsiOrInternetAddress.psInternetAddress
			->ulPortNumber=atoi(pcData);
		DbvFree(pcData);
	}
	else
	{
		/* Error */
		return_value = SI_NOTOK;
	}
	if(return_value == SI_OK)
	{
		*ppsTargetInformation=psLocalTargetInformation;
	}

	return return_value;
}

#ifndef NO_OSI
/* Scan the isoentities file to make sure that the requested entry exists. */
int DBVCheckISOENTITIESFile(char *pcTargetName, DbvTargetInformation 
	**ppsTargetInformation, DbvBool *plFinished, DbvError **ppsError)
{
        int return_value=SI_OK;
        char *pcAddressType;
        char *pcData;
	char pcService[] = DBV_SERVICE;
        DbvTargetInformation *psLocalTargetInformation=NULL;
	AEI psAppEntityInfo;
 

	if((psAppEntityInfo=str2aeinfo(pcTargetName, pcService,0,
		NULLCP,NULLCP))!=NULLAEI)
	{
		*plFinished = DBV_TRUE;

        	psLocalTargetInformation=(DbvTargetInformation *)
                	DbvAlloc(sizeof(DbvTargetInformation));
		psLocalTargetInformation->eOsiOrInternetAddress = DBV_OSI_NAME;
		psLocalTargetInformation->uOsiOrInternetAddress.psOsiAddress =
			(DbvOsiAddress *)DbvAlloc(sizeof(DbvOsiAddress));
		psLocalTargetInformation->uOsiOrInternetAddress.psOsiAddress->
			pcPsel = (char *)DbvAlloc(strlen(pcTargetName)+1);
		strcpy(psLocalTargetInformation->uOsiOrInternetAddress.
			psOsiAddress->pcPsel, pcTargetName);
	}

	*ppsTargetInformation=psLocalTargetInformation;

	if(psAppEntityInfo != NULL)
		AEIFREE(psAppEntityInfo);

	return return_value;
}
#endif

char *DBVGetStringField(TBL Table, TBL_Return T_Return, DbvError **ppsError)
{
	char *pcData=NULL;

	if(TBL_GetField(Table, T_Return) == FALSE)
	{
		sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, "String", 
			"Nothing");
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_FILE_FORMAT, DBVErrorBuffer, 0, Association_Closed,
			ppsError);
		return NULL;
	}

	if(T_Return->Tag != TBL_QText)
	{
		/* Format in file is wrong! */
		sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, "String", 
			DBVGetFieldTypeString(T_Return->Tag));
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_FILE_FORMAT, DBVErrorBuffer, 0, Association_Closed,
			ppsError);
	}
	else
	{
		pcData=(char *)DbvAlloc(strlen(T_Return->Value.QText)+1);
		strcpy(pcData, T_Return->Value.QText);
	}
		
	return pcData;
}

char *DBVGetCharField(TBL Table, TBL_Return T_Return, DbvError **ppsError)
{
        char *pcData=NULL;
 
        if(TBL_GetField(Table, T_Return) == FALSE)
        {
		sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, "Characters", 
			"Nothing");
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_FILE_FORMAT, DBVErrorBuffer, 0, Association_Closed,
			ppsError);
                return NULL;
        }
 
        if(T_Return->Tag != TBL_UText)
        {
                /* Format in file is wrong! */
                sprintf(DBVErrorBuffer, DBV_FILE_FORMAT_S, "Characters",
                        DBVGetFieldTypeString(T_Return->Tag));
                DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_FILE_FORMAT, DBVErrorBuffer, 0, Association_Closed, 
			ppsError);
        }
        else
        {
                pcData=(char *)DbvAlloc(strlen(T_Return->Value.UText)+1);
                strcpy(pcData, T_Return->Value.UText);
        }
                
        return pcData;
}

