/*  Module Name: @(#)logfiles.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: DBV-OSI II API code to write information to the API system
	log files. This is necessary when there is either no existing 
	association, or it has not been initialised yet 
	(ReceiveAssociateRequest() for instance).
 
*/

#include "logfiles.h"

/* Passes the file descriptor between the Open and Close functions */
long lFileDescriptor;

void DBVWriteOriginLog(char *pcLogData)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

	if(DBVOpenLogFile()==SI_NOTOK)
		return;
/*
	printf("DBV-OSI II ORIGIN ProcessId: %d\n\t: %s\n", getpid(),pcLogData);
*/
	sprintf(pcLogBuffer,
		"DBV-OSI II ORIGIN ProcessId: %d\n\t: %s\n",getpid(),pcLogData);
	write(lFileDescriptor, pcLogBuffer, strlen(pcLogBuffer));

	DBVCloseLogFile();
}

void DBVWriteTargetLog(char *pcLogData)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

	if(DBVOpenLogFile()==SI_NOTOK)
		return;
/*
	printf("DBV-OSI II TARGET ProcessId: %d\n\t: %s\n", getpid(),pcLogData);
*/
	sprintf(pcLogBuffer, 
		"DBV-OSI II TARGET ProcessId: %d\n\t: %s\n",getpid(),pcLogData);
	write(lFileDescriptor, pcLogBuffer, strlen(pcLogBuffer));
	DBVCloseLogFile();

}

void DBVWriteLog(char *pcLogData)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

	if(DBVOpenLogFile()==SI_NOTOK)
		return;

/*
	printf("DBV-OSI II LogFile ProcessId: %d\n\t: %s\n",getpid(),pcLogData);
*/
	sprintf(pcLogBuffer, 
	    "DBV-OSI II LogFile ProcessId: %d\n\t: %s\n",getpid(),pcLogData);
	write(lFileDescriptor, pcLogBuffer, strlen(pcLogBuffer));
	DBVCloseLogFile();
}

/* This function should not be called externally */
void DBVWriteLogData(char *pcLogData)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

/*
	printf("%s",pcLogData);
*/
	sprintf(pcLogBuffer, "%s",pcLogData);
	write(lFileDescriptor, pcLogBuffer, strlen(pcLogBuffer));
}

/* If there is no trace file, write the error to the log file */
void DBVWriteErrorToLog(DbvError **ppsError, char *pcDiagnostic)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

	if(DBVOpenLogFile()==SI_NOTOK)
		return;

	sprintf(pcLogBuffer,
		"DBV-OSI II LogFile ProcessId: %d produced the following error\n",
		getpid());
	DBVWriteLogData(pcLogBuffer);
	
	DBVWriteLogData("\teErrorClass == ");
	if((*ppsError)->eErrorClass == SI_Communication_Error)
		DBVWriteLogData("SI_Communication_Error\n");
	if((*ppsError)->eErrorClass == SI_Interface_Error)
		DBVWriteLogData("SI_Interface_Error\n");
	if((*ppsError)->eErrorClass == SI_OperatingSystem_Error)
		DBVWriteLogData("SI_OperatingSystem_Error\n");

	if((*ppsError)->eErrorClass == SI_Abort)
	{
		DBVWriteLogData("SI_Abort\n");
		sprintf(pcLogBuffer, "\tuErrorCode.psAbort->eAbortSource == %d\n",(*ppsError)->uErrorCode.psAbort->eAbortSource);
		DBVWriteLogData(pcLogBuffer);
		sprintf(pcLogBuffer, "\tuErrorCode.psAbort->lAbortReason == %d\n",(*ppsError)->uErrorCode.psAbort->lAbortReason);
		DBVWriteLogData(pcLogBuffer);
		sprintf(pcLogBuffer, "\tuErrorCode.psAbort->pcDiagnostics == %s\n", (*ppsError)->uErrorCode.psAbort->pcDiagnostics);
		DBVWriteLogData(pcLogBuffer);
	}
	else
	{
		sprintf(pcLogBuffer, "\tuErrorCode.lIfOsCoError == %d\n", (*ppsError)->uErrorCode.lIfOsCoError);
		DBVWriteLogData(pcLogBuffer);
		if(pcDiagnostic != NULL)
		{
			DBVWriteLogData(pcDiagnostic);
			DBVWriteLogData("\n");
		}
	}
	DBVWriteLogData("eAssociationState == ");
	if((*ppsError)->eAssociationState == Association_Open)
		DBVWriteLogData("Association_Open\n");
	if((*ppsError)->eAssociationState == Association_Closed)
		DBVWriteLogData("Association_Closed\n");
	if((*ppsError)->eAssociationState == Association_Unknown_State)
		DBVWriteLogData("Association_Unknown_State\n");

	DBVWriteLogData("\n");
	DBVCloseLogFile();
}

#ifndef NO_OSI
/* Write additional error information produced by ISODE produced at the ACSE
	level to the log file. This is only done when there is no association 
	trace file (see si_utilities.[ch]). */
void DBVLogISODEACSEError(char *pcFunction, struct AcSAPabort 
	*psAssocControlAbort)
{
	char pcLogBuffer[SA_ARRAY_SIZE];

	if(DBVOpenLogFile()==SI_NOTOK)
		return;

	sprintf(pcLogBuffer,
		"DBV-OSI II LogFile ProcessId: %d produced the following error\n",
		getpid());
	DBVWriteLogData(pcLogBuffer);
	DBVWriteLogData("ISODE function ");
	DBVWriteLogData(pcFunction);
	DBVWriteLogData(" reported the following ACSE Level error:\n");
	DBVWriteLogData("aca_source==");
	switch(psAssocControlAbort->aca_source)
	{
	case ACA_USER:
		DBVWriteLogData("ACA_USER, A-U-ABORT.INDICATION\n");
		break;
	case ACA_PROVIDER:
		DBVWriteLogData("ACA_PROVIDER, A-P-ABORT.INDICATION\n");
		break;
	case ACA_LOCAL:
		DBVWriteLogData("ACA_LOCAL, UNOFFICIAL ERROR\n");
		break;
	default:
		DBVWriteLogData("Unknown type!\n");
		break;
	}
	sprintf(pcLogBuffer,"\taca_reason==%d\n",psAssocControlAbort->
		aca_reason);
	DBVWriteLogData(pcLogBuffer);
	DBVWriteLogData("\taca_data==");
	DBVWriteLogData(psAssocControlAbort->aca_data);
	DBVWriteLogData("\n\tAcErrorString==");
	DBVWriteLogData(AcErrString(psAssocControlAbort->aca_reason));
	DBVWriteLogData("\n");

	DBVCloseLogFile();
}

/* Write additional error information produced by ISODE produced at the 
	presentation level to the log file. This is only done when there is no 
	association trace file (see si_utilities.[ch]). */
void DBVLogISODEPError(char *pcFunction, struct PSAPabort 
	*psPresentationAbort)
{
	char pcLogBuffer[SA_ARRAY_SIZE];
	
	if(DBVOpenLogFile()==SI_NOTOK)
		return;

	sprintf(pcLogBuffer,
		"DBV-OSI II LogFile ProcessId: %d produced the following error\n", 
		getpid());
	DBVWriteLogData(pcLogBuffer);
	DBVWriteLogData("ISODE function ");
	DBVWriteLogData(pcFunction);
	DBVWriteLogData(" reported the following Presentation Level error:\n");
	DBVWriteLogData("pa_peer==");
	if(psPresentationAbort->pa_peer == 0)
		DBVWriteLogData("0, P-P-ABORT.INDICATION\n");
	else
		DBVWriteLogData("TRUE, P-U-ABORT.INDICATION\n");
	sprintf(pcLogBuffer,"\tpa_reason==%d\n",psPresentationAbort->
		pa_reason);
	DBVWriteLogData(pcLogBuffer);
	DBVWriteLogData("\tpa_data==");
	DBVWriteLogData(psPresentationAbort->pa_data);
	DBVWriteLogData("\n\tAcErrorString==");
	DBVWriteLogData(AcErrString(psPresentationAbort->pa_reason));
	DBVWriteLogData("\n");
	
	DBVCloseLogFile();
}
#endif

/* Open or create a file and put a lock on it */
int DBVOpenLogFile(void)
{
	struct flock sLockData;
	int value;

	if(access(DBV_API_LOGFILE, F_OK) == -1)
	{
		/* the file does not exist, we had better create it */
		value = umask(0);
		if ((lFileDescriptor = creat(DBV_API_LOGFILE, 0644)) == -1)
		{
			/* the system must be in a real state. */
			fprintf(stderr, "DBV-OSI II API Error: Could not create logfile %s\n", DBV_API_LOGFILE);
			umask(value);
			return SI_NOTOK;
		}
		umask(value);
	}
	else
	{
		/* open the file */
		if ((lFileDescriptor = open(DBV_API_LOGFILE, O_WRONLY)) == -1)
		{
			/* the system must be in a real state. */
			fprintf(stderr, 
			   "DBV-OSI II API Error: Could not open logfile %s\n", 
				DBV_API_LOGFILE);
			return SI_NOTOK;
		}
	}

	sLockData.l_type = F_WRLCK;
	sLockData.l_start = 0;
	sLockData.l_whence = SEEK_SET;
	sLockData.l_len = 0;

	/* Lock the file. This call will block until the file becomes free. */
	if (fcntl(lFileDescriptor, F_SETLKW, &sLockData) == -1)
	{
		/* We haven't been able to create a lock on the file. Normally 
		we would log this, but seeing as this is the log file that we 
		are trying to lock write an error and exit */
		close(lFileDescriptor);
		return SI_NOTOK;
	}
	lseek(lFileDescriptor, 0, SEEK_END);     /* go to the end of the file */

	return SI_OK;
}

/* Release the lock on the log file */
void DBVCloseLogFile(void)
{
	struct flock sLockData;

	sLockData.l_type = F_UNLCK;
	sLockData.l_start = 0;
	sLockData.l_whence = SEEK_SET;
	sLockData.l_len = 0;
	fcntl(lFileDescriptor, F_SETLKW, &sLockData);
	close(lFileDescriptor);
}

