/*  Module Name: @(#)oid_tool.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Oid related routines. All routines are derived from
		 code in the public domain Cansearch application.
		  

*/

#include "oid_tool.h"


/*************************************************************/
/* STRING (char*) TO OID */

void ASN_MakeOid(char *dottedString, AsnOid *oidEntry, jmp_buf env)
{
    OID_SNACC_STRUCT *oidList;
    OID_SNACC_STRUCT *curOid;
    OID_SNACC_STRUCT *lastOid;
    char done = FALSE;
    long arcNum;
    char *s;
    char tmpBuf[50];
    short i;
    AsnLen oidByteLen;
    short toolStat;

    oidList = (OID_SNACC_STRUCT *) NULL;
    curOid = oidList;
    s = dottedString;
    while (done == FALSE)
    {

        /* Skip the dot if currently positioned over one */
        while (*s == '.')
        {
            s++;
        }
        /* Extract an entry from the dotted string notation */
        i = 0;
        while ((*s != '.') && (*s != '\0'))
        {
            tmpBuf[i++] = *s++;
        }
        /* Check for last entry */
        if ((*s == '\0') && (i == 0))
        {
            done = TRUE;
        }
        else
        {
            /* Terminate the arc num string */
            tmpBuf[i] = '\0';
            /* Convert the arc num string to a long */
            arcNum = atol(tmpBuf);

            /* Malloc the OID_SNACC_STRUCT entry. Clearing the mem. */
            curOid = (OID_SNACC_STRUCT *) RescIR_Zmalloc(stdout, sizeof(OID_SNACC_STRUCT));
	    CheckAsn1Alloc(curOid, env);

            /* Set the arc num into the OID_SNACC_STRUCT */
            curOid->arcNum = (long int) arcNum;

            /* Create linked list of OID_SNACC_STRUCTs */
            if (oidList == (OID_SNACC_STRUCT *) NULL)
            {
                /* Save the start of the list */
                oidList = curOid;
                lastOid = curOid;

		/* Terminate the list properly */
		curOid->next = NULL; 
            }
            else
            {
                /* Link the OID_SNACC_STRUCT into the list of OID_SNACC_STRUCTs */
                lastOid->next = curOid;
                lastOid = lastOid->next;

		/* Terminate the list properly */
		curOid->next = NULL; 

            }
        }
    }

    oidByteLen = EncodedOidLen(oidList);

    /* Create the oid but do not populate it with oid data */
    ASN_OctsCreate((unsigned long) oidByteLen, oidEntry, env);

    /* Now populate it with oid data */
    BuildEncodedOid(oidList, oidEntry);
    ASN_FreeOid(oidList);
}



char *RescIR_Zmalloc(FILE *f, int size)
{
    return ((char *) malloc(size));
}



int ASN_OctsCreate(AsnLen oidByteLen, AsnOid *oidEntry, jmp_buf env)
{
    oidEntry->octetLen = oidByteLen;
    oidEntry->octs = (char *) malloc(oidByteLen);
    CheckAsn1Alloc(oidEntry->octs, env);

    return (0);
}




void ASN_FreeOid(OID_SNACC_STRUCT *oidList)
{
    OID_SNACC_STRUCT *tmpOid;
    OID_SNACC_STRUCT *lastOid;

    for (tmpOid = oidList; tmpOid->next != (OID_SNACC_STRUCT *) NULL;
            tmpOid = lastOid)
    {
        lastOid = tmpOid->next;
        RescIR_Free(stdout, (char *) tmpOid);
    }
    /* Free the last one in the list */
    RescIR_Free(stdout, (char *) tmpOid);
}


void RescIR_Free(FILE *f, char *str)
{
    free(str);
}



/*************************************************************/
/* OID TO STRING (char *) */

char *ASN_EncodedToString(AsnOid *v)
{

	static char stringformed[SA_ARRAY_SIZE];
	char tempstring[SA_ARRAY_SIZE];
	

	unsigned short int firstArcNum;
	unsigned long int arcNum;
	int i;
 
	
	/* Clear the string which will hold the result */
	memset(stringformed, 0, sizeof(stringformed));

	/* Check if the AsnOid is avlid */
	if ((v->octetLen == 0) || (v->octs == NULL)) {
		DBVsa_strcpy(stringformed, ERROR_OID);
		return stringformed;
	}
 
	/* un-munge first two arc numbers */
	for (arcNum = 0, i=0; (i < v->octetLen) && (v->octs[i] & 0x80);i++)
		arcNum = (arcNum << 7) + (v->octs[i] & 0x7f);
 
	arcNum = (arcNum << 7) + (v->octs[i] & 0x7f);
	i++;
	firstArcNum = arcNum/40;
	if (firstArcNum > 2)
		firstArcNum = 2;
 
	sprintf(tempstring, "%u.%u", firstArcNum, arcNum - (firstArcNum * 40));
	DBVsa_strcat(stringformed, tempstring);
 
	for (; i < v->octetLen ; )
	{
		for (arcNum = 0; (i < v->octetLen) && (v->octs[i] & 0x80);i++)
			arcNum = (arcNum << 7) + (v->octs[i] & 0x7f);
 
		arcNum = (arcNum << 7) + (v->octs[i] & 0x7f);
		i++;
		sprintf(tempstring, ".%u", arcNum);
		DBVsa_strcat(stringformed, tempstring);
	}


	return stringformed;
 
} 
