/*  Module Name: @(#)origin-sm.c	7.1
    Version: 11/15/96
    Author: Chris Attewell 
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: Origin state machine code for DBV-OSI II project.
*/
 

#include "origin-sm.h"



char DBVOriginStateErrorString[][DBV_SM_STRLEN] =
        { "", "DBV_OS_CLOSED","DBV_OS_ASSOCREQ_INPROGRESS","DBV_OS_ASSOCIATION",
          "DBV_OS_INITSENT","DBV_OS_OPEN","DBV_OS_SEARCH_SENT",
          "DBV_OS_PRESENT_SENT","DBV_OS_DELETE_SENT","DBV_OS_SCAN_SENT",
          "DBV_OS_RESOURCE_REPORT_SENT","DBV_OS_SORT_REQUEST_SENT",
          "DBV_OS_EXTENDED_SERVICE_SENT","DBV_OS_RELEASEREQ_SENT",
          "DBV_OS_CLOSE_SENT","DBV_OS_CLOSE_RCVD","DBV_OS_ERROR", "DBV_OS_DEAD", 

	    "DBV_OS_ACCCNT_RCVD", "DBV_OS_RESCNT_RCVD", "DBV_OS_TRIGRESCNT_SENT"};


char DBVOriginEventErrorString[][DBV_SM_STRLEN] =
        {"","DBV_OE_IASSOCREQ","DBV_OE_AASSOCIATERESP_IND_TRUE",
         "DBV_OE_AASSOCIATERESP_IND_FALSE","DBV_OE_INITREQ",
         "DBV_OE_INITRESP_PDU_TRUE","DBV_OE_INITRESP_PDU_FALSE",
         "DBV_OE_SEARCHREQ","DBV_OE_SEARCHRESP_PDU","DBV_OE_PRESENTREQ",
         "DBV_OE_PRESENTRESP_PDU","DBV_OE_DELETEREQ","DBV_OE_DELETERESP_PDU",
         "DBV_OE_SCANREQ","DBV_OE_SCANRESP_PDU","DBV_OE_RESOURCEREQ",
         "DBV_OE_RESOURCERESP_PDU","DBV_OE_SORTREQ","DBV_OE_SORTRESP_PDU",
         "DBV_OE_SEGMENTREQ_PDU","DBV_OE_EXTENDEDREQ","DBV_OE_EXTENDEDRESP_PDU",
         "DBV_OE_IRELREQ","DBV_OE_ARELREQ_IND","DBV_OE_ARELRESP_IND",
         "DBV_OE_AABORT_IND","DBV_OE_IABORTREQ","DBV_OE_CLOSEREQ_PDU",
         "DBV_OE_CLOSEREQ","DBV_OE_CLOSERESP","DBV_OE_DECODE_FAILED",

	   "DBV_OE_ACCCNTREQ_PDU", "DBV_OE_ACCCNTRSP", 
         "DBV_OE_RESCNTREQ_PDU", "DBV_OE_RESCNTRSP", 
	   "DBV_OE_TRIGRESCNTREQ" };


/* 0 transition = protocol failed when forming PDU (state machine error) */
/* -1 transition = impossible (states and events start at 1)*/
/* 98 transition = revert to previous state before access control request arrived */
/* 99 transition = revert to previous state before resource control request arrived 
 (99s for INITRESP_PDU_TRUE etc. allow reversion without resource control response
being formed (depends upons values in PDU!) */ 



/* start in a state (column) .. get an event (row) and move to new state (column) */

int DBVOriginStateTable[DBV_O_EVENTS][DBV_O_STATES] =
{
/* State ->               0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20  */
/* Event \/          */ {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1},
/* IASSOCREQ         */ {-1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* AASSOCRESP_TRUE   */ {-1, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* AASSOCRESP_FALSE  */ {-1, 0,17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* INITREQ           */ {-1, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* INITRESP_PDU_TRUE */ {-1, 0, 0,16, 5,16,16,16,16,16,16,16,16,16,16,16, 0, 0, 0, 0, 0},
/* INITRESP_PDU_FALSE*/ {-1, 0, 0,16, 3,16,16,16,16,16,16,16,16,16,16,16, 0, 0, 0, 0, 0},
/* SEARCHREQ         */ {-1, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SEARCHRESP_PDU    */ {-1, 0, 0,16,16,16, 5,16,16,16,16,16,16,13,14,16, 0, 0, 0, 0, 0},
/* PRESENTREQ        */ {-1, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* PRESENTRESP_PDU   */ {-1, 0, 0,16,16,16,16, 5,16,16,16,16,16,13,14,16, 0, 0, 0, 0, 0},
/* DELETEREQ         */ {-1, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* DELETERESP_PDU    */ {-1, 0, 0,16,16,16,16,16, 5,16,16,16,16,13,14,16, 0, 0, 0, 0, 0},
/* SCANREQ           */ {-1, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SCANRESP_PDU      */ {-1, 0, 0,16,16,16,16,16,16, 5,16,16,16,13,14,16, 0, 0, 0, 0, 0},
/* RESOURCEREQ       */ {-1, 0, 0, 0, 0,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* RESOURCERESP_PDU  */ {-1, 0, 0,16,16,16,16,16,16,16, 5,16,16,13,14,16, 0, 0, 0, 0, 0},
/* SORTREQ           */ {-1, 0, 0, 0, 0,11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SORTRESP_PDU      */ {-1, 0, 0,16,16,16,16,16,16,16,16, 5,16,13,14,16, 0, 0, 0, 0, 0},
/* SEGMENTREQ_PDU    */ {-1, 0, 0,16,16,16,16, 7,16,16,16,16,16,13,14,16, 0, 0, 0, 0, 0},
/* EXTENDEDREQ       */ {-1, 0, 0, 0, 0,12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* EXTENDEDRESP_PDU  */ {-1, 0, 0,16,16,16,16,16,16,16,16,16, 5,13,14,16, 0, 0, 0, 0, 0},
/* IRELREQ           */ {-1, 0, 0,13, 0,13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* ARELREQ_IND       */ {-1, 0, 0,16,16,16,16,16,16,16,16,16,16,16,16,16, 0, 0, 0, 0, 0},
/* ARELRESP_IND      */ {-1, 0, 0,16,16,16,16,16,16,16,16,16,16,17,16,16, 0, 0, 0, 0, 0},
/* AABORT_IND        */ {-1, 0,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17, 0, 0, 0, 0},
/* ABORTREQ          */ {-1, 0,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17, 0, 0, 0, 0},
/* CLOSEREQ_PDU      */ {-1, 0, 0,16,16,15,15,15,15,15,15,15,15,13, 3, 0,15, 0, 0, 0, 0},
/* CLOSEREQ          */ {-1, 0, 0, 0, 0,14,14,14,14,14,14,14,14, 0, 0, 0,14, 0, 0, 0, 0},
/* CLOSERESP         */ {-1, 0,17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0},
/* DECODE_FAILED     */ {-1, 0, 0,16,16,16,16,16,16,16,16,16,16, 0,16,16, 0, 0, 0, 0, 0},

/* ACCCNTREQ_PDU     */ {-1, 0, 0, 0,18, 0,18,18,18,18,18,18,18, 0, 0, 0, 0, 0, 0, 0, 0},
/* ACCCNTRSP         */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,98, 0, 0},

/* RESCNTREQ_PDU     */ {-1, 0, 0, 0,19, 0,19,19,19,19,19,19,19, 0, 0, 0, 0, 0, 0, 0, 0},
/* RESCNTRSP         */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,99, 0},

/* TRIGRESCNTREQ     */ {-1, 0, 0, 0, 0, 0, 6, 7, 8, 9,10,11,12,13,14,15, 0, 0, 0, 0, 0}

};




