/*  Module Name: @(#)origin_z3950_access.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Origin end routines to create and send an 
		  AccessControlResponse PDU to a target and process a 
		  received AccessControlRequest PDU sent by a target 
		  

*/


#include "origin_z3950_access.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE AccessControlResponse PDU TO SEND TO TARGET AND SEND IT 
*/

/* one of the few responses sent from an origin to a target */

int DbvAccessControlResponse(DbvAssocId lAssocId, DbvAccessControlRsp *psAccessControlRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVAccessControlResponse", lAssocId);

	if (DBVCheckDbvAccessControlRspFields(lAssocId, psAccessControlRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_ACCCNTRSP, ppsSIError) == SI_OK)
			 if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK) {
		DBVSendingTrace(lAssocId);
                DBVAccRspTrace(lAssocId, psAccessControlRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.accessControlResponse = (struct AccessControlResponse *) Asn1Alloc (sizeof(struct AccessControlResponse));
			CheckAsn1Alloc(PDU_ptr->a.accessControlResponse, env);
        		PDU_ptr->choiceId = PDU_ACCESSCONTROLRESPONSE;
			DBVsetup_snacc_AccessControlResponse(PDU_ptr, psAccessControlRsp, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVAccessControlResponse", lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvAccessControlRspFields(DbvAssocId lAssocId, DbvAccessControlRsp *psAccessControlRsp, DbvError **ppsSIError)
{

	/* o */
	/* Dont have to check psReferenceId */

	/* o */
	if (DBVcheck_DbvSecurityChallengeRsp_opt(lAssocId, psAccessControlRsp->psSecurityChallengeRsp, "psAccessControlRsp->psSecurityChallengeRsp->", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
	if (DBVcheck_DbvDiagRec_opt(lAssocId, psAccessControlRsp->psDiagnostic, "psAccessControlRsp->psDiagnostic->", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvOtherInformation_opt(lAssocId, psAccessControlRsp->ppsOtherInformation, "psAccessControlRsp->ppsOtherInformation", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvSecurityChallengeRsp_opt(DbvAssocId lAssocId, DbvSecurityChallengeRsp *psSecurityChallengeRsp, char *error_buffer, DbvError **ppsSIError)
{
	if (psSecurityChallengeRsp)
		return(DBVcheck_DbvSecurityChallengeRsp(lAssocId, psSecurityChallengeRsp, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvSecurityChallengeRsp(DbvAssocId lAssocId, DbvSecurityChallengeRsp *psSecurityChallengeRsp, char *error_buffer, DbvError **ppsSIError)
{

        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psSecurityChallengeRsp->eSecurityChallengeRspType) {
	case SECT_SimpleForm:
		DBVsa_strcat(local_error_buffer, "uSecurityChallengeRsp.psSimpleForm");
		if (psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

                break;


	case SECT_ExternallyDefined:
		DBVsa_strcat(local_error_buffer, "uSecurityChallengeRsp.psExternallyDefinedInfo");
		if (psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return (DBVcheck_dbv_external(lAssocId, psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo, local_error_buffer, ppsSIError));

                break;
	default:
		DBVsa_strcat(local_error_buffer, "eSecurityChallengeRspType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
        }


        return SI_OK;
}



int DBVcheck_DbvDiagRec_opt(DbvAssocId lAssocId, DbvDiagRec *psDiagnostic, char *error_buffer, DbvError **ppsSIError)
{
	if (psDiagnostic)
		return (DBVcheck_dbv_surrdiag(lAssocId, psDiagnostic, error_buffer, ppsSIError));

	return SI_OK;
}





/* snacc set up */

int DBVsetup_snacc_AccessControlResponse(PDU *PDU_ptr, DbvAccessControlRsp *psAccessControlRsp, jmp_buf env)
{
	/* o */
	DBVset_AccessControlResponse_referenceId(PDU_ptr, psAccessControlRsp->psReferenceId, env);

	/* o */
        DBVset_AccessControlResponse_securityChallengeResponse(PDU_ptr, psAccessControlRsp->psSecurityChallengeRsp, env);

	/* o */
        DBVset_AccessControlResponse_diagnostic(PDU_ptr, psAccessControlRsp->psDiagnostic, env);

	/* o */
	DBVset_AccessControlResponse_otherinfo(PDU_ptr, psAccessControlRsp->ppsOtherInformation, env);
}



int DBVset_AccessControlResponse_referenceId(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvOctetString_opt(&(PDU_ptr->a.accessControlResponse->referenceId), psReferenceId, env);
	
}



int DBVset_AccessControlResponse_securityChallengeResponse(PDU *PDU_ptr, DbvSecurityChallengeRsp *psSecurityChallengeRsp, jmp_buf env)
{
	/* o */
	if (psSecurityChallengeRsp) {
		PDU_ptr->a.accessControlResponse->securityChallengeResponse = ( struct AccessControlResponseChoice *) Asn1Alloc (sizeof (struct AccessControlResponseChoice));
		CheckAsn1Alloc(PDU_ptr->a.accessControlResponse->securityChallengeResponse, env);
	
		switch (psSecurityChallengeRsp->eSecurityChallengeRspType) {
		case SECT_SimpleForm:
		PDU_ptr->a.accessControlResponse->securityChallengeResponse->choiceId = ACCESSCONTROLRESPONSECHOICE_SIMPLEFORM;
			DBVdbv_to_snacc_octetstring_comp(&PDU_ptr->a.accessControlResponse->securityChallengeResponse->a.simpleForm, psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm, env);
	
			break;
	
		case SECT_ExternallyDefined:
			PDU_ptr->a.accessControlResponse->securityChallengeResponse->choiceId = ACCESSCONTROLRESPONSECHOICE_EXTERNALLYDEFINED;
			DBVdbv_to_snacc_external_opt(&PDU_ptr->a.accessControlResponse->securityChallengeResponse->a.externallyDefined, psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo, env);
			break;
		}
		
	}
}



int DBVset_AccessControlResponse_diagnostic(PDU *PDU_ptr, DbvDiagRec *psDiagnostic, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvDiagRec_opt(&PDU_ptr->a.accessControlResponse->diagnostic, psDiagnostic, env);
}




int DBVset_AccessControlResponse_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	DBVdbv_to_snacc_otherinfo_opt(&PDU_ptr->a.accessControlResponse->otherInfo, ppsOtherInformation, env);

}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS AccessControlRequest PDU RECEIVED FROM TARGET 
*/

int DBVset_dbv_fields_DbvAccessControlReq(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{
 
        psOriginData->eOriginEvent = OR_AccessCntrlInd;
        psOriginData->uOriginData.psAccessControlReq = (DbvAccessControlReq *) DbvAlloc ( sizeof (DbvAccessControlReq));
	CheckDbvAlloc(psOriginData->uOriginData.psAccessControlReq, env);

	/* o */
        DBVset_DbvAccessControlReq_psReferenceId(psOriginData->uOriginData.psAccessControlReq, PDU_ptr, env);

	/* c */
        DBVset_DbvAccessControlReq_psSecurityChallengeReq(psOriginData->uOriginData.psAccessControlReq, PDU_ptr, env);

	/* o */
	DBVset_DbvAccessControlReq_ppsOtherInformation(psOriginData->uOriginData.psAccessControlReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}




int DBVset_DbvAccessControlReq_psReferenceId(DbvAccessControlReq *psAccessControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */	
	DBVsnacc_to_dbv_DbvOctetString_opt(&psAccessControlReq->psReferenceId, &PDU_ptr->a.accessControlRequest->referenceId, env);
}



int DBVset_DbvAccessControlReq_psSecurityChallengeReq(DbvAccessControlReq *psAccessControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* c */

	switch (PDU_ptr->a.accessControlRequest->securityChallenge->choiceId) {
	case ACCESSCONTROLREQUESTCHOICE_SIMPLEFORM:
		psAccessControlReq->sSecurityChallengeReq.eSecurityChallengeReqType = SECT_SimpleForm;
		DBVsnacc_to_dbv_DbvOctetString_opt(&psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psSimpleForm, PDU_ptr->a.accessControlRequest->securityChallenge->a.simpleForm, env);
		break;

	case ACCESSCONTROLREQUESTCHOICE_EXTERNALLYDEFINED:
		psAccessControlReq->sSecurityChallengeReq.eSecurityChallengeReqType = SECT_ExternallyDefined;
		DBVsnacc_to_dbv_external_opt(&psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psExternallyDefinedInfo, PDU_ptr->a.accessControlRequest->securityChallenge->a.externallyDefined, env);
		break;
	}

}




int DBVset_DbvAccessControlReq_ppsOtherInformation(DbvAccessControlReq *psAccessControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_otherinfo_opt(&psAccessControlReq->ppsOtherInformation, PDU_ptr->a.accessControlRequest->otherInfo, env);
}



