/*  Module Name: @(#)origin_z3950_delete.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a 
		 DeleteResultSetRequest PDU 
		 to a target and process
		 a received DeleteResultSetResponse PDU sent by a target 

*/

#include "origin_z3950_delete.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE DeleteResultSetRequest PDU TO SEND TO TARGET AND SEND IT
*/


int DbvDeleteResultSetRequest(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVDeleteResultSetRequest",lAssocId);

	if (DBVCheckDbvDelReqFields(lAssocId, psDeleteResultSetReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_DELETEREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVDelReqTrace(lAssocId, psDeleteResultSetReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {
			PDU_ptr->a.deleteResultSetRequest = (struct DeleteResultSetRequest *) Asn1Alloc(sizeof(struct DeleteResultSetRequest));
			CheckAsn1Alloc(PDU_ptr->a.deleteResultSetRequest, env);

			PDU_ptr->choiceId = PDU_DELETERESULTSETREQUEST;
			
			DBVsetup_snacc_delrequest(PDU_ptr, psDeleteResultSetReq, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);

	}

	DBVLeaveFuncTrace("DBVDeleteResultSetRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvDelReqFields(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_delreq_delfunc(lAssocId, psDeleteResultSetReq, ppsSIError))
		return SI_NOTOK;

	if (DBVcheck_dbv_delreq_resset(lAssocId, psDeleteResultSetReq, ppsSIError))
		return SI_NOTOK;


	if (DBVcheck_dbv_delreq_otherinfo(lAssocId, psDeleteResultSetReq, ppsSIError))
		return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_dbv_delreq_delfunc(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq, DbvError **ppsSIError)
{

	if (DBVcheck_dbv_enum(psDeleteResultSetReq->eDeleteFunction, DF_List, DF_All) == SI_NOTOK)
                return(DBVdbv_invalid_enum_error(lAssocId, "psDeleteResultSetReq->eDeleteFunction", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_delreq_resset(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq, DbvError **ppsSIError)
{
 
	/* Can be NULL because optional */
	if (psDeleteResultSetReq->ppsResultSetId == NULL)
		return SI_OK;

	return SI_OK;
}



int DBVcheck_dbv_delreq_otherinfo(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psDeleteResultSetReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psDeleteResultSetReq->ppsOtherInformation, "psDeleteResultSetReq->ppsOtherInformation", ppsSIError));
 
}



int DBVsetup_snacc_delrequest(PDU *PDU_ptr, DbvDeleteResultSetReq *psDeleteResultSetReq, jmp_buf env)
{
	DBVset_snacc_delreq_refid(PDU_ptr, psDeleteResultSetReq->psReferenceId, env);
	DBVset_snacc_delreq_deletefunc(PDU_ptr, psDeleteResultSetReq->eDeleteFunction, env);
	DBVset_snacc_delreq_ressetid(PDU_ptr, psDeleteResultSetReq->ppsResultSetId, env);
	DBVset_snacc_delreq_otherinfo(PDU_ptr, psDeleteResultSetReq->ppsOtherInformation, env);
}



int DBVset_snacc_delreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.deleteResultSetRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_delreq_deletefunc(PDU *PDU_ptr, DbvDeleteFunction eDeleteFunction, jmp_buf env)
{
	switch (eDeleteFunction) {
	case DF_List:
		PDU_ptr->a.deleteResultSetRequest->deleteFunction = LIST;
		break;
	case DF_All:
		PDU_ptr->a.deleteResultSetRequest->deleteFunction = ALL;
		break;
	}
}



int DBVset_snacc_delreq_ressetid(PDU *PDU_ptr, DbvInternationalString **ppsResultSetId, jmp_buf env)
{
	ResultSetId *data_ptr;
	ResultSetId **data_ptr_address;

        int current_listnode = 0;

	if (ppsResultSetId != NULL) {
        	PDU_ptr->a.deleteResultSetRequest->resultSetList = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetRequest->resultSetList, env);
 
        	while(ppsResultSetId[current_listnode] != NULL) {
                	data_ptr_address = AsnListAppend(PDU_ptr->a.deleteResultSetRequest->resultSetList);
			CheckAsn1Alloc(data_ptr_address, env);
                	*data_ptr_address = Asn1Alloc(sizeof(ResultSetId));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;
 
                	DBVdbv_to_snacc_inter_string(data_ptr, ppsResultSetId[current_listnode], env);
                	current_listnode++;
		}
        }
 
}



int DBVset_snacc_delreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.deleteResultSetRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetRequest->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.deleteResultSetRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS DeleteResultSetResponse PDU RECEIVED FROM TARGET 
*/




int DBVset_dbv_fields_deleteresultsetresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_DeleteResultSetConf;
	psOriginData->uOriginData.psDeleteResultSetRsp = (DbvDeleteResultSetRsp *) DbvAlloc (sizeof (DbvDeleteResultSetRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psDeleteResultSetRsp, env);


	DBVset_dbv_delresp_refid(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_opstat(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_delliststat(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_numnotdel(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_bulkliststat(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_delmess(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);
	DBVset_dbv_delresp_otherinfo(psOriginData->uOriginData.psDeleteResultSetRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_delresp_refid(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.deleteResultSetResponse->referenceId.octetLen != 0){
		delresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(delresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(delresp->psReferenceId, &(PDU_ptr->a.deleteResultSetResponse->referenceId), env); 
	}
}



int DBVset_dbv_delresp_opstat(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_delsetstat(&delresp->eDeleteOperationStatus, &PDU_ptr->a.deleteResultSetResponse->deleteOperationStatus, env);
}



int DBVsnacc_to_dbv_delsetstat(DbvDeleteSetStatus *dbv_stat, DeleteSetStatus *snacc_stat, jmp_buf env)
{
	switch (*snacc_stat) {
	case SUCCESS1:
		*dbv_stat = DSS_Success;
		break;	
	case RESULTSETDIDNOTEXIST:
		*dbv_stat = DSS_ResultSetDidNotExist;
		break;
	case PREVIOUSLYDELETEDBYTARGET:
		*dbv_stat = DSS_PreviouslyDeletedByTarget;
		break;
	case SYSTEMPROBLEMATTARGET:
		*dbv_stat = DSS_SystemProblemAtTarget;
		break;
	case ACCESSNOTALLOWED:
		*dbv_stat = DSS_AccessNotAllowed;
		break;
	case RESOURCECONTROLATORIGIN:
		*dbv_stat = DSS_ResourceControlAtOrigin;
		break;
	case RESOURCECONTROLATTARGET:
		*dbv_stat = DSS_ResourceControlAtTarget;
		break;
	case BULKDELETENOTSUPPORTED:
		*dbv_stat = DSS_BulkDeleteNotSupported;
		break;
	case NOTALLRSLTSETSDELETEDONBULKDLTE:
		*dbv_stat = DSS_NotAllSetsDeletedOnBulkDlt;
		break;
	case NOTALLREQUESTEDRESULTSETSDELETED:
		*dbv_stat = DSS_NotAllRequestedSetsDeleted;
		break;
	case RESULTSETINUSE:
		*dbv_stat = DSS_ResultSetInUse;
		break;
	default:
		*dbv_stat = DSS_Invalid;
		break;
	}
}



int DBVset_dbv_delresp_delliststat(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
        ListStatusesSeq *data_ptr;
        int num_elements = 0;
        int index = 0;
 
	if (PDU_ptr->a.deleteResultSetResponse->deleteListStatuses != NULL) {
        	num_elements = AsnListCount(PDU_ptr->a.deleteResultSetResponse->deleteListStatuses);
        	delresp->ppsDeleteListStatuses = (DbvListStatus **) DbvAlloc (sizeof(DbvListStatus *) * (num_elements + 1));
		CheckDbvAlloc(delresp->ppsDeleteListStatuses, env);

        	FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.deleteResultSetResponse->deleteListStatuses) {
                	delresp->ppsDeleteListStatuses[index] = (DbvListStatus *) DbvAlloc (sizeof(DbvListStatus));
			CheckDbvAlloc(delresp->ppsDeleteListStatuses[index], env);

                	DBVsnacc_to_dbv_liststat(delresp->ppsDeleteListStatuses[index], data_ptr, env);
                	index++;
        	}
        	delresp->ppsDeleteListStatuses[index] = NULL;
	}

}



int DBVsnacc_to_dbv_liststat(DbvListStatus *psDeleteListStatuses, ListStatusesSeq *data_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_inter_string(&psDeleteListStatuses->sResultSetId, &data_ptr->id, env); 

	DBVsnacc_to_dbv_delsetstat(&psDeleteListStatuses->eDeleteSetStatus, &data_ptr->status, env);
	
}



int DBVset_dbv_delresp_numnotdel(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.deleteResultSetResponse->numberNotDeleted != NULL) {
		delresp->pulNumberNotDeleted = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(delresp->pulNumberNotDeleted, env);

		*delresp->pulNumberNotDeleted = *PDU_ptr->a.deleteResultSetResponse->numberNotDeleted;
	}
}



int DBVset_dbv_delresp_bulkliststat(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
        ListStatusesSeq *data_ptr;
        int num_elements = 0;
        int index = 0;
 
	if (PDU_ptr->a.deleteResultSetResponse->bulkStatuses != NULL) {
        	num_elements = AsnListCount(PDU_ptr->a.deleteResultSetResponse->bulkStatuses);
        	delresp->ppsBulkStatuses = (DbvListStatus **) DbvAlloc (sizeof(DbvListStatus *) * (num_elements + 1));
		CheckDbvAlloc(delresp->ppsBulkStatuses, env);


        	FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.deleteResultSetResponse->bulkStatuses) {
                	delresp->ppsBulkStatuses[index] = (DbvListStatus *) DbvAlloc (sizeof(DbvListStatus));
			CheckDbvAlloc(delresp->ppsBulkStatuses[index], env);

                	DBVsnacc_to_dbv_liststat(delresp->ppsBulkStatuses[index], data_ptr, env);
                	index++;
        	}
        	delresp->ppsBulkStatuses[index] = NULL;
	}
}



int DBVset_dbv_delresp_delmess(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If optional field exists */
	if (PDU_ptr->a.deleteResultSetResponse->deleteMessage.octetLen != 0) {
               	delresp->psDeleteMessage = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(delresp->psDeleteMessage, env);

		DBVsnacc_to_dbv_inter_string(delresp->psDeleteMessage, &PDU_ptr->a.deleteResultSetResponse->deleteMessage, env);
	}
}



int DBVset_dbv_delresp_otherinfo(DbvDeleteResultSetRsp *delresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.deleteResultSetResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.deleteResultSetResponse->otherInfo);

		delresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(delresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(delresp->ppsOtherInformation, PDU_ptr->a.deleteResultSetResponse->otherInfo, env);
        }
}



