/*  Module Name: @(#)origin_z3950_extend.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send an 
		 ExtendedServicesRequest PDU to a target and process
		 a received ExtendedServicesResponse PDU sent by a target 

*/

#include "origin_z3950_extend.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE ExtendedServicesRequest PDU TO SEND TO TARGET AND SEND IT
*/


int DbvExtendedServicesRequest(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVExtendedServicesRequest",lAssocId);
	if (DBVCheckDbvExtendReqFields(lAssocId, psExtendedServicesReq, ppsSIError) == SI_OK)

		if (DBVChangeState(lAssocId, DBV_OE_EXTENDEDREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVExtendReqTrace(lAssocId, psExtendedServicesReq);

		PDU_ptr = &pdu_to_encode;
	
		if ((val = setjmp(env)) == 0) {
			PDU_ptr->a.extendedServicesRequest = (struct ExtendedServicesRequest *) Asn1Alloc(sizeof(struct ExtendedServicesRequest));
			CheckAsn1Alloc(PDU_ptr->a.extendedServicesRequest, env);
			PDU_ptr->choiceId = PDU_EXTENDEDSERVICESREQUEST;
			DBVsetup_snacc_extendrequest(PDU_ptr, psExtendedServicesReq, env);
                } else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVExtendedServicesRequest",lAssocId, return_value);
	return return_value;
}


int DBVCheckDbvExtendReqFields(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_extendreq_funct(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_packtype(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_rettime(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_permiss(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_origspec(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_waact(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendreq_otherinfo(lAssocId, psExtendedServicesReq, ppsSIError))
                return SI_NOTOK;
 
	return SI_OK;
}



int DBVcheck_dbv_extendreq_funct(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_enum(psExtendedServicesReq->eFunction, ESF_Create, ESF_Modify) == SI_NOTOK) 
                return(DBVdbv_invalid_enum_error(lAssocId, "psExtendedServicesReq->eFunction", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_extendreq_packtype(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	/* Cant be NULL as compulsory */
	if (psExtendedServicesReq->pcPackageType == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psExtendedServicesReq->pcPackageType", ppsSIError));

	if (DBVcheck_dbv_validoid(psExtendedServicesReq->pcPackageType) == SI_NOTOK)
                return(DBVdbv_oid_error(lAssocId, "psExtendedServicesReq->pcPackageType", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_extendreq_rettime(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	/* Can be NULL as optional */
	if (psExtendedServicesReq->psRetentionTime == NULL)
                return SI_OK;

	return(DBVcheck_DbvIntUnit(lAssocId, psExtendedServicesReq->psRetentionTime, "psExtendedServicesReq->psRetentionTime->", ppsSIError));
}



int DBVcheck_dbv_extendreq_permiss(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];


	/* Can be NULL as optional */
	if (psExtendedServicesReq->ppsPermissions == NULL)
                return SI_OK;


	DBVsa_strcpy(local_error_buffer, "psExtendedServicesReq->");

	while (psExtendedServicesReq->ppsPermissions[index] != NULL) {
		 
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsPermissions[");
	
		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");
		 
		if (DBVcheck_dbv_allowfuncs(lAssocId, psExtendedServicesReq->ppsPermissions[index]->ppeAllowableFunctions, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}

	return SI_OK;
	
}



int DBVcheck_dbv_extendreq_origspec(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
        if (psExtendedServicesReq->psOriginSpecificParameter == NULL)
		return SI_OK;

        return(DBVcheck_dbv_external(lAssocId, psExtendedServicesReq->psOriginSpecificParameter, "psExtendedServicesReq->psOriginSpecificParameter->", ppsSIError));

}



int DBVcheck_dbv_extendreq_waact(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_enum(psExtendedServicesReq->eWaitAction, ESWA_Wait, ESWA_DontReturnPackage) == SI_NOTOK) 
                return(DBVdbv_invalid_enum_error(lAssocId, "psExtendedServicesReq->eWaitAction", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_extendreq_otherinfo(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq, DbvError **ppsSIError)
{
       /* Can be null because optional */
        if (psExtendedServicesReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psExtendedServicesReq->ppsOtherInformation, "psExtendedServicesReq->ppsOtherInformation", ppsSIError));

}



int DBVsetup_snacc_extendrequest(PDU *PDU_ptr, DbvExtendedServicesReq *psExtendedServicesReq, jmp_buf env)
{

	DBVset_snacc_extendreq_refid(PDU_ptr, psExtendedServicesReq->psReferenceId, env);
	DBVset_snacc_extendreq_function(PDU_ptr, psExtendedServicesReq->eFunction, env);
	DBVset_snacc_extendreq_packtype(PDU_ptr, psExtendedServicesReq->pcPackageType, env);
	DBVset_snacc_extendreq_packname(PDU_ptr, psExtendedServicesReq->psPackageName, env);
	DBVset_snacc_extendreq_userid(PDU_ptr, psExtendedServicesReq->psUserId, env);
	DBVset_snacc_extendreq_retenttime(PDU_ptr, psExtendedServicesReq->psRetentionTime, env);
	DBVset_snacc_extendreq_permission(PDU_ptr, psExtendedServicesReq->ppsPermissions, env);
	DBVset_snacc_extendreq_description(PDU_ptr, psExtendedServicesReq->psDescription, env);
	DBVset_snacc_extendreq_ospecoparam(PDU_ptr, psExtendedServicesReq->psOriginSpecificParameter, env);
	DBVset_snacc_extendreq_waitaction(PDU_ptr, psExtendedServicesReq->eWaitAction, env);
	DBVset_snacc_extendreq_elements(PDU_ptr, psExtendedServicesReq->psElements, env);
	DBVset_snacc_extendreq_otherinfo(PDU_ptr, psExtendedServicesReq->ppsOtherInformation, env);
}



int DBVset_snacc_extendreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.extendedServicesRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_extendreq_function(PDU *PDU_ptr, DbvExtServicesFunctions eFunction, jmp_buf env)
{
	switch (eFunction) {
	case ESF_Create:
		PDU_ptr->a.extendedServicesRequest->function = CREATE;
		break;
	case ESF_Delete:
		PDU_ptr->a.extendedServicesRequest->function = DELETE1; 
		break;
	case ESF_Modify:
		PDU_ptr->a.extendedServicesRequest->function = MODIFY; 
		break;
	}
}



int DBVset_snacc_extendreq_packtype(PDU *PDU_ptr, DbvObjectIdentifier pcPackageType, jmp_buf env) 
{
	ASN_MakeOid((char *) pcPackageType, &PDU_ptr->a.extendedServicesRequest->packageType, env); 
}



int DBVset_snacc_extendreq_packname(PDU *PDU_ptr, DbvInternationalString *psPackageName, jmp_buf env)
{
	if (psPackageName != NULL) 
		DBVdbv_to_snacc_inter_string(&PDU_ptr->a.extendedServicesRequest->packageName, psPackageName, env);
}



int DBVset_snacc_extendreq_userid(PDU *PDU_ptr, DbvInternationalString *psUserId, jmp_buf env)
{
	if (psUserId != NULL) 
		DBVdbv_to_snacc_inter_string(&PDU_ptr->a.extendedServicesRequest->userId, psUserId, env);
}



int DBVset_snacc_extendreq_retenttime(PDU *PDU_ptr, DbvIntUnit *psRetentionTime, jmp_buf env)
{
	if (psRetentionTime != NULL) {
		PDU_ptr->a.extendedServicesRequest->retentionTime = (struct IntUnit*) Asn1Alloc (sizeof (struct IntUnit));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesRequest->retentionTime, env);
		DBVdbv_to_snacc_intunit(PDU_ptr->a.extendedServicesRequest->retentionTime, psRetentionTime, env);
	}
}



int DBVset_snacc_extendreq_permission(PDU *PDU_ptr, DbvPermissions **ppsPermissions, jmp_buf env)
{
	if (ppsPermissions != NULL) {
		PDU_ptr->a.extendedServicesRequest->permissions = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesRequest->permissions, env);

		DBVdbv_to_snacc_permissions(PDU_ptr->a.extendedServicesRequest->permissions, ppsPermissions, env);
	}
}



int DBVset_snacc_extendreq_description(PDU *PDU_ptr,  DbvInternationalString *psDescription, jmp_buf env)
{
	if (psDescription != NULL) 
		DBVdbv_to_snacc_inter_string(&PDU_ptr->a.extendedServicesRequest->description, psDescription, env);
}



int DBVset_snacc_extendreq_ospecoparam(PDU *PDU_ptr, DbvASN1External *psOriginSpecificParameter, jmp_buf env)
{
	if (psOriginSpecificParameter != NULL) {
		PDU_ptr->a.extendedServicesRequest->taskSpecificParameters = (struct NEWEXTERNAL*) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesRequest->taskSpecificParameters, env);
		DBVdbv_to_snacc_external(PDU_ptr->a.extendedServicesRequest->taskSpecificParameters, psOriginSpecificParameter, env);
	}
}



int DBVset_snacc_extendreq_waitaction(PDU *PDU_ptr, DbvExtServicesWaitAction eWaitAction, jmp_buf env)
{
	switch (eWaitAction) {
	case ESWA_Wait:
		PDU_ptr->a.extendedServicesRequest->waitAction = WAIT;
		break;
	case ESWA_WaitIfPossible:
		PDU_ptr->a.extendedServicesRequest->waitAction = WAITIFPOSSIBLE;
		break;
	case ESWA_DontWait:
		PDU_ptr->a.extendedServicesRequest->waitAction = DONTWAIT;
		break;
	case ESWA_DontReturnPackage:
		PDU_ptr->a.extendedServicesRequest->waitAction = DONTRETURNPACKAGE;
		break;
	}
}



int DBVset_snacc_extendreq_elements(PDU *PDU_ptr, DbvInternationalString  *psElements, jmp_buf env)
{
	if (psElements != NULL) 
		DBVdbv_to_snacc_inter_string(&PDU_ptr->a.extendedServicesRequest->elements, psElements, env);
}



int DBVset_snacc_extendreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.extendedServicesRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesRequest->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.extendedServicesRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS ExtendedServicesResponse PDU RECEIVED FROM TARGET
*/




int DBVset_dbv_fields_extendresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_ExtendedServicesConf;
	psOriginData->uOriginData.psExtendedServicesRsp = (DbvExtendedServicesRsp *) DbvAlloc (sizeof (DbvExtendedServicesRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psExtendedServicesRsp, env);

	DBVset_dbv_extendresp_refid(psOriginData->uOriginData.psExtendedServicesRsp, PDU_ptr, env);
	DBVset_dbv_extendresp_opstat(psOriginData->uOriginData.psExtendedServicesRsp, PDU_ptr, env);
	DBVset_dbv_extendresp_diagnost(psOriginData->uOriginData.psExtendedServicesRsp, PDU_ptr, env);
	DBVset_dbv_extendresp_taskpack(psOriginData->uOriginData.psExtendedServicesRsp, PDU_ptr, env);
	DBVset_dbv_extendresp_otherinfo(psOriginData->uOriginData.psExtendedServicesRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_extendresp_refid(DbvExtendedServicesRsp *extendresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.extendedServicesResponse->referenceId.octetLen != 0){
		extendresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(extendresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(extendresp->psReferenceId, &(PDU_ptr->a.extendedServicesResponse->referenceId), env); 
	}
}



int DBVset_dbv_extendresp_opstat(DbvExtendedServicesRsp *extendresp, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.extendedServicesResponse->operationStatus) {
	case DONE:
		extendresp->eOperationStatus = ESOS_Done;
		break;
	case ACCEPTED:
		extendresp->eOperationStatus = ESOS_Queued;
		break;
	case FAILURE1:
		extendresp->eOperationStatus = ESOS_Failure;
		break;
	default:
		extendresp->eOperationStatus = ESOS_Invalid;
		break;
	}
}



int DBVset_dbv_extendresp_diagnost(DbvExtendedServicesRsp *extendresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	if (PDU_ptr->a.extendedServicesResponse->diagnostics != NULL) {
		num_elements = AsnListCount (PDU_ptr->a.extendedServicesResponse->diagnostics);
		extendresp->ppsDiagnostics = (DbvDiagRec **) DbvAlloc (sizeof (DbvDiagRec *) * (num_elements + 1));
		CheckDbvAlloc(extendresp->ppsDiagnostics, env);

		DBVsnacc_to_dbv_multdiag(extendresp->ppsDiagnostics, PDU_ptr->a.extendedServicesResponse->diagnostics, env);
	}
}



int DBVset_dbv_extendresp_taskpack(DbvExtendedServicesRsp *extendresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesResponse->taskPackage != NULL) {
		extendresp->psTaskPackage = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(extendresp->psTaskPackage, env);

		DBVsnacc_to_dbv_external(extendresp->psTaskPackage, PDU_ptr->a.extendedServicesResponse->taskPackage, env);
	}
}




int DBVset_dbv_extendresp_otherinfo(DbvExtendedServicesRsp *extendresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.extendedServicesResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.extendedServicesResponse->otherInfo);
		extendresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(extendresp->ppsOtherInformation, env);
	
		DBVsnacc_to_dbv_otherinfo(extendresp->ppsOtherInformation, PDU_ptr->a.extendedServicesResponse->otherInfo, env);
        }
}


