/*  Module Name: @(#)origin_z3950_init.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send an InitRequest PDU 
		 to a target and process a received InitResponse PDU 
		 sent by a target

*/

#include "origin_z3950_init.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE InitRequest PDU TO SEND TO TARGET AND SEND IT
*/


int DbvInitializeRequest(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVInitializeRequest",lAssocId);

	if (DBVCheckDbvInitReqFields(lAssocId, psInitializeReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_INITREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;


	if (return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId); 
		DBVInitReqTrace(lAssocId, psInitializeReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
		   a longjump to here if allocing fails */
		if ((val = setjmp(env)) == 0) {

			PDU_ptr->a.initRequest = (struct InitializeRequest *) Asn1Alloc(sizeof(struct InitializeRequest));
			CheckAsn1Alloc(PDU_ptr->a.initRequest, env);
			PDU_ptr->choiceId = PDU_INITREQUEST;

			DBVsetup_snacc_initrequest(PDU_ptr, psInitializeReq, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                	return SI_NOTOK;
		}

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);

	}

	DBVLeaveFuncTrace("DBVInitializeRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvInitReqFields(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{

	if (DBVcheck_dbv_initreq_prover(lAssocId, psInitializeReq, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_dbv_initreq_options(lAssocId, psInitializeReq, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_dbv_initreq_auth(lAssocId, psInitializeReq, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;
	
	if (DBVcheck_dbv_initreq_userinf(lAssocId, psInitializeReq, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	if (DBVcheck_dbv_initreq_otherinfo(lAssocId, psInitializeReq, ppsSIError) == SI_NOTOK)

		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_initreq_prover(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{

	/* If field doesnt exist */
	if(psInitializeReq->pcProtocolVersion == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psInitializeReq->pcProtocolVersion", ppsSIError));


	/* pcProtocolVersion exists - check its value */

	if (DBVcheck_bit_string(psInitializeReq->pcProtocolVersion) == SI_NOTOK)
		return(DBVdbv_bit_string_error(lAssocId, "psInitializeReq->pcProtocolVersion", psInitializeReq->pcProtocolVersion, ppsSIError));


	return SI_OK;
}



int DBVcheck_dbv_initreq_options(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{

	if(psInitializeReq->pcOptions == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psInitializeReq->pcOptions", ppsSIError));

	
	if(DBVcheck_bit_string(psInitializeReq->pcOptions) == SI_NOTOK)
		return(DBVdbv_bit_string_error(lAssocId, "psInitializeReq->pcOptions", psInitializeReq->pcOptions, ppsSIError));

			
	/* Check that the options chosen are supported */
	if (DBVcheck_chosen_options(psInitializeReq->pcOptions) == SI_NOTOK)
		return(DBVdbv_options_chosen_error(lAssocId, "psInitializeReq->pcOptions", psInitializeReq->pcOptions, ppsSIError));


	return SI_OK;

}





int DBVcheck_dbv_initreq_auth(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{
	/* Can be null because optional */
	if (psInitializeReq->psAuthentication == NULL)
		return SI_OK;

	return (DBVcheck_idauth_struct(lAssocId, psInitializeReq->psAuthentication, "psInitializeReq->psAuthentication->", ppsSIError));
}



int DBVcheck_idauth_struct(DbvAssocId lAssocId, DbvAuthentication *psAuthentication, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psAuthentication->eAuthenticationType) {
	case AT_Open:
		DBVsa_strcat(local_error_buffer, "uAuthentication.psOpen");

		if (psAuthentication->uAuthentication.psOpen == NULL) 
			return (DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		
		break;
	case AT_IdPassword:
		DBVsa_strcat(local_error_buffer, "uAuthentication.psIdPassword");

		if (psAuthentication->uAuthentication.psIdPassword == NULL) 
			return (DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		break;
	case AT_Anonymous:
		return SI_OK;
		break;
	case AT_Other:
		DBVsa_strcat(local_error_buffer, "uAuthentication.psOther");

		if (psAuthentication->uAuthentication.psOther == NULL) 
			return (DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_external(lAssocId, psAuthentication->uAuthentication.psOther, local_error_buffer, ppsSIError));

		break;
	default:
		DBVsa_strcat(local_error_buffer, "eAuthenticationType");
		return (DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;

}



int DBVcheck_dbv_initreq_userinf(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{
	/* Can be null because optional */
	if (psInitializeReq->psUserInformationField == NULL) 
		return SI_OK;

	return(DBVcheck_dbv_external(lAssocId, psInitializeReq->psUserInformationField, "psInitializeReq->psUserInformationField->", ppsSIError));

}



int DBVcheck_dbv_initreq_otherinfo(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psInitializeReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psInitializeReq->ppsOtherInformation, "psInitializeReq->ppsOtherInformation", ppsSIError));

}



int DBVsetup_snacc_initrequest(PDU *PDU_ptr, DbvInitializeReq *psInitializeReq, jmp_buf env)
{

	DBVset_snacc_initreq_refid(PDU_ptr, psInitializeReq->psReferenceId, env);
	DBVset_snacc_initreq_protocolversion(PDU_ptr, psInitializeReq->pcProtocolVersion, env);
	DBVset_snacc_initreq_options(PDU_ptr, psInitializeReq->pcOptions, env);
	DBVset_snacc_initreq_prefmessize(PDU_ptr, psInitializeReq->ulPreferredMessageSize, env);
	DBVset_snacc_initreq_maxmessize(PDU_ptr, psInitializeReq->ulMaximumMessageSize, env);
	DBVset_snacc_initreq_authentication(PDU_ptr, psInitializeReq->psAuthentication, env);
	DBVset_snacc_initreq_impid(PDU_ptr, psInitializeReq->psImplementationId, env);
	DBVset_snacc_initreq_impname(PDU_ptr, psInitializeReq->psImplementationName, env);
	DBVset_snacc_initreq_impver(PDU_ptr, psInitializeReq->psImplementationVersion, env);
	DBVset_snacc_initreq_userinfo(PDU_ptr, psInitializeReq->psUserInformationField, env);
	DBVset_snacc_initreq_otherinfo(PDU_ptr, psInitializeReq->ppsOtherInformation, env);
}



int DBVset_snacc_initreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.initRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_initreq_protocolversion(PDU *PDU_ptr, DbvProtocolVersion pcProtocolVersion, jmp_buf env)
{

	AsnBits *a1;
	unsigned long i;
	unsigned long bytes_to_alloc = 0;

	/* Allocate space for 2 bit groups (bytes).
           Each bit group has 8 bits in the order 128, 64, 32, 16, 8, 4, 2, 1
           The first bit in a1 is the 128 bit of group 1.
           The last bit in a1 is the 1 bit of group 2.
 
           Initially all bits are reset (0).
 
           SetAsnBit sets a relevant bit.
           ClrAsnBit resets a relevant bit.
           GetAsnBits gets the decimal value of a bit (either 128, 64, 32, 16,
                8, 4, 2 or 1 if the bit is set, 0 if the bit is reset)
        */

	a1 = &PDU_ptr->a.initRequest->protocolVersion;

	/* e.g. 27 bits / 8 = 3 bytes so add 1 */
	bytes_to_alloc = (strlen(pcProtocolVersion) / 8) + 1;

	a1->bits = Asn1Alloc(bytes_to_alloc);
	CheckAsn1Alloc(a1->bits, env);

	/* THE FOLLOWING LINE IS ESSENTIAL BEFORE SETTING UP THE BITS -
	   total number of bits in all bit groups */
	a1->bitLen = 8 * bytes_to_alloc;         

	for (i = 0; i < strlen(pcProtocolVersion); i++){
		if (pcProtocolVersion[i] == '1')
			SetAsnBit(a1, i);
		else
			ClrAsnBit(a1, i);
	}

	a1->bitLen = i;

}



int DBVset_snacc_initreq_options(PDU *PDU_ptr, DbvOptions pcOptions, jmp_buf env)
{
	AsnBits *a1;
	unsigned long i;
	unsigned long bytes_to_alloc = 0;


	a1 = &PDU_ptr->a.initRequest->options;

	bytes_to_alloc = (strlen(pcOptions) / 8) + 1;

	a1->bits = Asn1Alloc(bytes_to_alloc);
	CheckAsn1Alloc(a1->bits, env);

	a1->bitLen = 8 * bytes_to_alloc;

	for (i = 0; i < strlen(pcOptions); i++){
		if (pcOptions[i] == '1')
			SetAsnBit(a1, i);
		else
			ClrAsnBit(a1, i);
	}

	a1->bitLen = i;
}



int DBVset_snacc_initreq_prefmessize(PDU *PDU_ptr, Ulong ulPreferredMessageSize, jmp_buf env)
{

	PDU_ptr->a.initRequest->preferredMessageSize = ulPreferredMessageSize;

}



int DBVset_snacc_initreq_maxmessize(PDU *PDU_ptr, Ulong ulMaximumMessageSize, jmp_buf env)
{

	PDU_ptr->a.initRequest->exceptionalRecordSize = ulMaximumMessageSize;

}



int DBVset_snacc_initreq_authentication(PDU *PDU_ptr, DbvAuthentication *psAuthentication, jmp_buf env)
{

	if (psAuthentication != NULL) {
		PDU_ptr->a.initRequest->idAuthentication = (struct IdAuthentication *) Asn1Alloc(sizeof(struct IdAuthentication));
		CheckAsn1Alloc(PDU_ptr->a.initRequest->idAuthentication, env);

		switch(psAuthentication->eAuthenticationType) {
		case AT_Open:
			DBVset_snacc_initreq_authentication_open(PDU_ptr, psAuthentication, env);
			break;
		case AT_IdPassword:
			DBVset_snacc_initreq_authentication_idpass(PDU_ptr, psAuthentication, env);
			break;
		case AT_Anonymous:
			PDU_ptr->a.initRequest->idAuthentication->choiceId = IDAUTHENTICATION_ANONYMOUS;
			break;
		case AT_Other:
			DBVset_snacc_initreq_authentication_other(PDU_ptr, psAuthentication, env);
			break;
		}
	}
}

int DBVset_snacc_initreq_authentication_open(PDU *PDU_ptr, DbvAuthentication *psAuthentication, jmp_buf env)
{

	PDU_ptr->a.initRequest->idAuthentication->choiceId = IDAUTHENTICATION_OPEN;

	PDU_ptr->a.initRequest->idAuthentication->a.open = (VisibleString *) Asn1Alloc(sizeof(VisibleString));
	CheckAsn1Alloc(PDU_ptr->a.initRequest->idAuthentication->a.open, env);

	DBVdbv_to_snacc_vis_string(PDU_ptr->a.initRequest->idAuthentication->a.open, psAuthentication->uAuthentication.psOpen, env);

}



int DBVset_snacc_initreq_authentication_idpass(PDU *PDU_ptr, DbvAuthentication *psAuthentication, jmp_buf env)
{
	PDU_ptr->a.initRequest->idAuthentication->choiceId = IDAUTHENTICATION_IDPASS;

	PDU_ptr->a.initRequest->idAuthentication->a.idPass = (struct IdAuthenticationSeq *) Asn1Alloc(sizeof(struct IdAuthenticationSeq));
	CheckAsn1Alloc(PDU_ptr->a.initRequest->idAuthentication->a.idPass, env);

	/* groupId */
	if (psAuthentication->uAuthentication.psIdPassword->psGroupId != NULL)
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->idAuthentication->a.idPass->groupId), psAuthentication->uAuthentication.psIdPassword->psGroupId, env);

	/* userId */
	if (psAuthentication->uAuthentication.psIdPassword->psUserId != NULL)
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->idAuthentication->a.idPass->userId), psAuthentication->uAuthentication.psIdPassword->psUserId, env);

	/* password */
	if (psAuthentication->uAuthentication.psIdPassword->psPassword != NULL)
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->idAuthentication->a.idPass->password), psAuthentication->uAuthentication.psIdPassword->psPassword, env);
}



int DBVset_snacc_initreq_authentication_other(PDU *PDU_ptr, DbvAuthentication *psAuthentication, jmp_buf env)
{
	char *oid = NULL;


	PDU_ptr->a.initRequest->idAuthentication->choiceId = IDAUTHENTICATION_OTHER;
	PDU_ptr->a.initRequest->idAuthentication->a.other = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
	CheckAsn1Alloc(PDU_ptr->a.initRequest->idAuthentication->a.other, env);

	DBVdbv_to_snacc_external(PDU_ptr->a.initRequest->idAuthentication->a.other, psAuthentication->uAuthentication.psOther, env);
}



int DBVset_snacc_initreq_impid(PDU *PDU_ptr, DbvInternationalString *psImplementationId, jmp_buf env)
{
	/* Check if a ImplementationId is given(it is an optional field) */
	if  (psImplementationId != NULL) {
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->implementationId), psImplementationId, env);
	}
}



int DBVset_snacc_initreq_impname(PDU *PDU_ptr, DbvInternationalString *psImplementationName, jmp_buf env)
{
	/* Check if a ImplementationName is given (it is an optional field) */
	if  (psImplementationName != NULL) {
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->implementationName), psImplementationName, env);
	}
}



int DBVset_snacc_initreq_impver(PDU *PDU_ptr, DbvInternationalString *psImplementationVersion, jmp_buf env)
{
	/* Check if a ImplementationVersion is given (it is an optional field) */
	if  (psImplementationVersion != NULL) {
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initRequest->implementationVersion), psImplementationVersion, env);
	}
}



int DBVset_snacc_initreq_userinfo(PDU *PDU_ptr, DbvASN1External *psUserInformationField, jmp_buf env)
{

	/* Check if psUserInformationField exists - it is an optional field */
	if (psUserInformationField != NULL)  {
		PDU_ptr->a.initRequest->userInformationField = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
		CheckAsn1Alloc(PDU_ptr->a.initRequest->userInformationField, env);

               DBVdbv_to_snacc_external(PDU_ptr->a.initRequest->userInformationField, psUserInformationField, env);
        }
}


int DBVset_snacc_initreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if psOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.initRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.initRequest->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.initRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS InitResponse PDU RECEIVED FROM TARGET
*/




int DBVset_dbv_fields_initresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_InitializeConf;


	psOriginData->uOriginData.psInitializeRsp = (DbvInitializeRsp *) DbvAlloc (sizeof (DbvInitializeRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psInitializeRsp, env);

	DBVset_dbv_initresp_refid(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_prover(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_options(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_prefmes(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_maxmes(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_result(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);

	DBVset_dbv_initresp_impid(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_impname(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_impver(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_userinfo(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);
	DBVset_dbv_initresp_otherinfo(psOriginData->uOriginData.psInitializeRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_initresp_refid(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.initResponse->referenceId.octetLen != 0){
		initresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(initresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(initresp->psReferenceId, &(PDU_ptr->a.initResponse->referenceId), env); 
	}
}



int DBVset_dbv_initresp_prover(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	char *prover_string = NULL;
	int i;
	AsnBits *a1;

	a1 = &PDU_ptr->a.initResponse->protocolVersion;

	prover_string = (char *) DbvAlloc (a1->bitLen  + 1);
	CheckDbvAlloc(prover_string, env);

	for (i = 0; i < (a1->bitLen); i++){
		if (GetAsnBit(a1, i)) {
			prover_string[i] =  '1';
		} else {
			prover_string[i] =  '0';
		}
	}

	/* Terminate the string */
	prover_string[i] = 0;

	/* Address of pointer to character */
	initresp->pcProtocolVersion =  (DbvProtocolVersion) DbvAlloc (strlen(prover_string) + 1);
	CheckDbvAlloc(initresp->pcProtocolVersion, env);
	memcpy(initresp->pcProtocolVersion, prover_string, strlen(prover_string));

        if (prover_string != NULL)
                DbvFree(prover_string);
}



int DBVset_dbv_initresp_options(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	char *options_string = NULL;
	int i;
	AsnBits *a1;

	a1 = &PDU_ptr->a.initResponse->options;

	options_string = (char *) DbvAlloc (a1->bitLen  + 1);

	for (i = 0; i < (a1->bitLen); i++){
		if (GetAsnBit(a1, i)) {
			options_string[i] = '1';
		} else {
			options_string[i] = '0';
		}
	}

	/* Terminate the string */
	options_string[i] = 0;

	initresp->pcOptions =  (DbvOptions) DbvAlloc (strlen(options_string) + 1);
	CheckDbvAlloc(initresp->pcOptions, env);
	memcpy(initresp->pcOptions, options_string, strlen(options_string));

        if (options_string != NULL)
                DbvFree(options_string);
}



int DBVset_dbv_initresp_prefmes(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	initresp->ulPreferredMessageSize = PDU_ptr->a.initResponse->preferredMessageSize;
}



int DBVset_dbv_initresp_maxmes(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	initresp->ulMaximumMessageSize = PDU_ptr->a.initResponse->exceptionalRecordSize;

}



int DBVset_dbv_initresp_result(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	if(PDU_ptr->a.initResponse->result == TRUE)
		initresp->fResult = DBV_TRUE;
	else
		initresp->fResult = DBV_FALSE;
}



int DBVset_dbv_initresp_impid(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If implementation id given (it is an optional field) */
	if (PDU_ptr->a.initResponse->implementationId.octetLen != 0){
		initresp->psImplementationId = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(initresp->psImplementationId, env);

		DBVsnacc_to_dbv_inter_string(initresp->psImplementationId, &(PDU_ptr->a.initResponse->implementationId), env); 
	}

}



int DBVset_dbv_initresp_impname(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If implementation name given (it is an optional field) */
	if (PDU_ptr->a.initResponse->implementationName.octetLen != 0){
		initresp->psImplementationName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(initresp->psImplementationName, env);

		DBVsnacc_to_dbv_inter_string(initresp->psImplementationName, &(PDU_ptr->a.initResponse->implementationName), env); 
	}
}

int DBVset_dbv_initresp_impver(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If implementation version given (it is an optional field) */
	if (PDU_ptr->a.initResponse->implementationVersion.octetLen != 0){
		initresp->psImplementationVersion = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(initresp->psImplementationVersion, env);

		DBVsnacc_to_dbv_inter_string(initresp->psImplementationVersion, &(PDU_ptr->a.initResponse->implementationVersion), env); 
	}

}


int DBVset_dbv_initresp_userinfo(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{

	if (PDU_ptr->a.initResponse->userInformationField != NULL) {
		initresp->psUserInformationField = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(initresp->psUserInformationField, env);

		DBVsnacc_to_dbv_external(initresp->psUserInformationField, PDU_ptr->a.initResponse->userInformationField, env);
	}
}




int DBVset_dbv_initresp_otherinfo(DbvInitializeRsp *initresp, PDU *PDU_ptr, jmp_buf env)
{
        int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.initResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.initResponse->otherInfo); 
		initresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));

		CheckDbvAlloc(initresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(initresp->ppsOtherInformation, PDU_ptr->a.initResponse->otherInfo, env);
        }
}



