/*  Module Name: @(#)origin_z3950_main.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: DBV-OSI II client main routines 
		 (common for all client functions)
*/

#include "origin_z3950_main.h"

 
int DbvReceiveDataOrigin( DbvAssocId lAssocId, DbvSecs lSeconds, 
			  DbvOriginData **ppsOriginData, DbvError **ppsError)
{
	PDU PDU_to_decode;
	PDU *PDU_ptr;
	int return_value = SI_OK;
	DbvProtocolStack eProtocolStack;
	DBVAssociateDetails *psAssociateDetails;
	int val;
	jmp_buf env;
	char *ber_data = NULL;
	unsigned long ber_data_length = 0;
	DBVTraceDetails *psTraceInformation;
 
	PDU_ptr = &PDU_to_decode;

	DBVEnterFuncTrace("ReceiveDataOrigin",lAssocId);

        /* Get the response */
	if(SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
	{
		psAssociateDetails = SL_Current(DBVAssociateList);
		if (eProtocolStack == TCP_Stack)
		{
        		return_value = DBVreceive_data_over_socket(lAssocId, 
					PDU_ptr, 
			    		psAssociateDetails->ulTcpSocket, 
					lSeconds, 
					&ber_data, &ber_data_length, ppsError);
	
				/* If the value returned is neither SI_OK or 
				   SI_NOTOK then the socket timed out */
		}
		else
		{
#ifndef NO_OSI
			return_value = DBVreceive_data_over_OSI(lAssocId,
				PDU_ptr, 
				psAssociateDetails->lOSIAssocId,
				lSeconds, 
				&ber_data, &ber_data_length, ppsError);
#else
			printf("OSI not supported\n");
#endif

		}
	}
	else
		/* Error structure filled out by SIWhichStack() */
		return_value = SI_NOTOK;


	if(return_value == SI_OK)
	{

		/* Do a setjump - the following structure setup 
		   routines do a longjump here if allocing fails */
		if ((val = setjmp(env)) == 0) {

			/* Allocate space for the PDU returned - ppsOriginData 
			   is the address of a pointer to OriginData, 
			   so derefence to get and set the pointer */
			*ppsOriginData = (struct DbvOriginData *) DbvAlloc (sizeof (struct DbvOriginData)); 
			CheckDbvAlloc(*ppsOriginData, env);
			return_value = DBVconvert_pdu_to_origin_data(lAssocId,*ppsOriginData, PDU_ptr, &ber_data, ber_data_length, ppsError, env); 

		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
			return SI_NOTOK;
                }
 
	}

	/* Timed out with no response */
	if (return_value == TIMED_OUT)
		return_value = SI_OK;

	DBVLeaveFuncTrace("ReceiveDataOrigin",lAssocId, return_value);

        if(return_value == SI_NOTOK)
                if((*ppsError)->eErrorClass == SI_Abort)
                {
			if(DBVTraceList != NULL)
			{
			psTraceInformation = SL_Locate(DBVTraceList,
                	    (SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
			if(psTraceInformation != NULL)
			{
				psTraceInformation = SL_RemoveCurrent(DBVTraceList);
                		if(psTraceInformation != NULL)
                        		free(psTraceInformation);

			}
			}

                /* Remove the association if it's finished */
                        psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                        free(psAssociateDetails);
                }

	/* Free the ber data  */
	if (ber_data != NULL) {
		DbvFree(ber_data);
		ber_data = NULL;
	}

	return return_value;
}



int DBVconvert_pdu_to_origin_data(DbvAssocId lAssocId, DbvOriginData *psOriginData, PDU *PDU_ptr, char **ber_data, unsigned long ber_data_length, DbvError **ppsError, jmp_buf env) 
{
	int return_value = SI_OK;

	switch (PDU_ptr->choiceId) {
		case PDU_INITRESPONSE:
			return_value = DBVsnacc_to_dbv_initrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_SEARCHRESPONSE:
			return_value = DBVsnacc_to_dbv_srchrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_PRESENTRESPONSE:
			return_value = DBVsnacc_to_dbv_presrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_DELETERESULTSETRESPONSE:
			return_value = DBVsnacc_to_dbv_delrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;

		case PDU_RESOURCEREPORTRESPONSE:
			return_value = DBVsnacc_to_dbv_resreprsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_SCANRESPONSE:
			return_value = DBVsnacc_to_dbv_scanrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_SORTRESPONSE:
			return_value = DBVsnacc_to_dbv_sortrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		/* Segmentation from server to client is a REQUEST */
		case PDU_SEGMENTREQUEST:
			return_value = DBVsnacc_to_dbv_segreq(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_EXTENDEDSERVICESRESPONSE:
			return_value = DBVsnacc_to_dbv_extendrsp(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_CLOSE:
			return_value = DBVsnacc_to_dbv_orclose(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
                case PDU_UNKNOWN:
			return_value = DBVsnacc_to_dbv_orunknown(PDU_ptr, lAssocId, ppsError, psOriginData, ber_data, ber_data_length, env);
			break;
		case PDU_ACCESSCONTROLREQUEST:
			return_value = DBVsnacc_to_dbv_accessreq(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;
		case PDU_RESOURCECONTROLREQUEST:
			return_value = DBVsnacc_to_dbv_rescntrlreq(PDU_ptr, lAssocId, ppsError, psOriginData, env);
			break;

		default:
			break;
	}

	return return_value;
}



int DBVsnacc_to_dbv_initrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	int proposed_state = DBV_OE_INITRESP_PDU_TRUE;

	/* Set proposed state based on value in the result 
	   field in the snacc structure */
	proposed_state = DBVget_snacc_initrsp_state(PDU_ptr);

	/* If state change allowed */
	if (DBVChangeState(lAssocId, proposed_state, ppsError) == SI_OK) {
		DBVset_dbv_fields_initresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVInitRspTrace(lAssocId, psOriginData->uOriginData.psInitializeRsp);
		return SI_OK;

	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received InitRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVget_snacc_initrsp_state(PDU *PDU_ptr)
{
	/* Get the snacc initrsp result field */
	if (PDU_ptr->a.initResponse->result == TRUE)
		return DBV_OE_INITRESP_PDU_TRUE;
	else
		return DBV_OE_INITRESP_PDU_FALSE;
}



int DBVsnacc_to_dbv_srchrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_SEARCHRESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_searchresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVSrchRspTrace(lAssocId, psOriginData->uOriginData.psSearchRsp);
		return SI_OK;

	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received SrchRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}


int DBVsnacc_to_dbv_presrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{

	if (DBVChangeState(lAssocId, DBV_OE_PRESENTRESP_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_presresp(psOriginData, PDU_ptr, env);
                DBVReceivingTrace(lAssocId);
                DBVPresRspTrace(lAssocId, psOriginData->uOriginData.psPresentRsp);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received PresRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_delrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_DELETERESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_deleteresultsetresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVDelRspTrace(lAssocId, psOriginData->uOriginData.psPresentRsp);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received DelRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_resreprsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_RESOURCERESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_resrepresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVResRepRspTrace(lAssocId, psOriginData->uOriginData.psResourceReportRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ResourceReportRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_scanrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_SCANRESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_scanresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVScanRspTrace(lAssocId, psOriginData->uOriginData.psScanRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ScanRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_sortrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_SORTRESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_sortresp(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVSortRspTrace(lAssocId, psOriginData->uOriginData.psSortRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received SortRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_segreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_SEGMENTREQ_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_segreq(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVSegReqTrace(lAssocId, psOriginData->uOriginData.psSegmentReq);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received SegmentReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_extendrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_EXTENDEDRESP_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_extendresp(psOriginData, PDU_ptr, env);
                DBVReceivingTrace(lAssocId);
                DBVExtendRspTrace(lAssocId, psOriginData->uOriginData.psExtendedServicesRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ExtendedServicesRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}


int DBVsnacc_to_dbv_orclose(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	DBVAssociateDetails *psAssociateDetails = NULL;
	long current_state = DBV_OS_DEAD;
	DbvOriginEvent eOriginEvent = OR_Invalid_APDU;

	/* Ind if close request */
	/* Conf if close response */

        /* Get the current state of the state machine */
        psAssociateDetails = SL_Current(DBVAssociateList);
	current_state = psAssociateDetails->lCurrentState;

	/* If previously JUST sent a CLOSE PDU */
	if (current_state == DBV_OS_CLOSE_SENT)

		/* Assume that have got a 'closeResponse' */
		eOriginEvent = OR_CloseConf;
	else
		eOriginEvent = OR_CloseInd;

	
	if (DBVChangeState(lAssocId, DBV_OE_CLOSEREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_originfields_closereq(psOriginData, PDU_ptr, eOriginEvent, env);
		DBVReceivingTrace(lAssocId);
                DBVCloseTraceOrigin(lAssocId, psOriginData->uOriginData.psCloseReq, eOriginEvent);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received Close PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVCloseTraceOrigin(DbvAssocId lAssocId, DbvCloseReq *psCloseReq, DbvOriginEvent eOriginEvent)
{
	if (eOriginEvent == OR_CloseInd)
                DBVCloseReqTrace(lAssocId, psCloseReq);
        else
                DBVCloseRspTrace(lAssocId, psCloseReq);
}



int DBVsnacc_to_dbv_orunknown(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, char **ber_data, unsigned long ber_data_length, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_Invalid_APDU;
	psOriginData->uOriginData.psInvalidAPDU = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
	CheckDbvAlloc(psOriginData->uOriginData.psInvalidAPDU, env);
	psOriginData->uOriginData.psInvalidAPDU->pcValue = (char *) DbvAlloc(ber_data_length);
	CheckDbvAlloc(psOriginData->uOriginData.psInvalidAPDU->pcValue, env);
	memcpy(psOriginData->uOriginData.psInvalidAPDU->pcValue, *ber_data, ber_data_length);
	psOriginData->uOriginData.psInvalidAPDU->ulLength = ber_data_length;

	DBVReceivingTrace(lAssocId);
        DBVUnknownPDUTrace(lAssocId, psOriginData->uOriginData.psInvalidAPDU);

	/* Output the Unknown PDU to a special file */
	DBVoutput_invalid_pdu(psOriginData->uOriginData.psInvalidAPDU);
	
	/* Now the ber data has been assigned to the DBV structure free it */
	if (*ber_data != NULL) {
		free(*ber_data);
		*ber_data = NULL;
	}

	return SI_OK;
}




/* AccessControlRequest from target */
int DBVsnacc_to_dbv_accessreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_OE_ACCCNTREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_DbvAccessControlReq(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVAccReqTrace(lAssocId, psOriginData->uOriginData.psAccessControlReq);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received AccReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



/* ResourceControlRequest from target */
int DBVsnacc_to_dbv_rescntrlreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvOriginData *psOriginData, jmp_buf env)
{
	/* set a global variable used to indicate if a 
	   ResourceControlResponse PDU is required. This variable
	   will be used by the state machine */

	/* Set initially to DBV_FALSE */
	ResourceControlResponseRequired = DBV_FALSE;

	if (PDU_ptr)
		if (PDU_ptr->a.resourceControlRequest)
			if (PDU_ptr->a.resourceControlRequest->responseRequired == TRUE)
				ResourceControlResponseRequired = DBV_TRUE;


	if (DBVChangeState(lAssocId, DBV_OE_RESCNTREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_DbvResourceControlReq(psOriginData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVResCntrlReqTrace(lAssocId, psOriginData->uOriginData.psResourceControlReq);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ResCntrlReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



