/*  Module Name: @(#)origin_z3950_present.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a PresentRequest PDU 
		 to a target and process a received PresentResponse PDU 
		 sent by a target 

*/

#include "origin_z3950_present.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE PresentRequest PDU TO SEND TO TARGET AND SEND IT
*/


int DbvPresentRequest(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVPresentRequest",lAssocId);

	if (DBVCheckDbvPresReqFields(lAssocId, psPresentReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_PRESENTREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVPresReqTrace(lAssocId, psPresentReq);

		PDU_ptr = &pdu_to_encode;

/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {

			PDU_ptr->a.presentRequest = (struct PresentRequest *) Asn1Alloc(sizeof(struct PresentRequest));
			CheckAsn1Alloc(PDU_ptr->a.presentRequest, env);
			PDU_ptr->choiceId = PDU_PRESENTREQUEST;
			DBVsetup_snacc_presrequest(PDU_ptr, psPresentReq, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVPresentRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvPresReqFields(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError)
{

        if (DBVcheck_dbv_presreq_addrang(lAssocId, psPresentReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_presreq_reccomp(lAssocId, psPresentReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_presreq_prefsyn(lAssocId, psPresentReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_presreq_othinfo(lAssocId, psPresentReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_dbv_presreq_addrang(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError) 
{

	/* Can be NULL because optional */
	if (psPresentReq->ppsAdditionalRanges == NULL)
		return SI_OK;

	
	return SI_OK;
}

	

int DBVcheck_dbv_presreq_reccomp(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError) 
{
	/* Can be NULL because optional */
	if (psPresentReq->psRecordComposition == NULL)
		return SI_OK;

	return(DBVcheck_DbvRecordComposition(lAssocId, psPresentReq->psRecordComposition, "psPresentReq->psRecordComposition->", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_presreq_prefsyn(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError) 
{

	/* Optional so can be NULL */
	if (psPresentReq->pcPreferredRecordSyntax == NULL) 
		return SI_OK;

	if (DBVcheck_dbv_recsyn(psPresentReq->pcPreferredRecordSyntax) == SI_NOTOK) 
		return(DBVdbv_field_missing_error(lAssocId, "psPresentReq->pcPreferredRecordSyntax", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_presreq_othinfo(DbvAssocId lAssocId, DbvPresentReq *psPresentReq, DbvError **ppsSIError)
{
        if (psPresentReq->ppsOtherInformation == NULL)
                return SI_OK;

	return(DBVcheck_dbv_otherinfo(lAssocId, psPresentReq->ppsOtherInformation, "psPresentReq->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_presrequest(PDU *PDU_ptr, DbvPresentReq *psPresentReq, jmp_buf env)
{
	DBVset_snacc_presreq_refid(PDU_ptr, psPresentReq->psReferenceId, env);
	DBVset_snacc_presreq_ressetid(PDU_ptr, psPresentReq->sResultSetId, env);
	DBVset_snacc_presreq_ressetstart(PDU_ptr, psPresentReq->ulResultSetStartPoint, env);
	DBVset_snacc_presreq_numrecsreq(PDU_ptr, psPresentReq->ulNumberOfRecordsRequested, env);
	DBVset_snacc_presreq_addranges(PDU_ptr, psPresentReq->ppsAdditionalRanges, env);
	DBVset_snacc_presreq_reccomp(PDU_ptr, psPresentReq->psRecordComposition, env);
	DBVset_snacc_presreq_prefrecsyn(PDU_ptr, psPresentReq->pcPreferredRecordSyntax, env);
	DBVset_snacc_presreq_maxsegcnt(PDU_ptr, psPresentReq->pulMaxSegmentCount, env);
	DBVset_snacc_presreq_maxrecsze(PDU_ptr, psPresentReq->pulMaxRecordSize, env);
	DBVset_snacc_presreq_maxsegsze(PDU_ptr, psPresentReq->pulMaxSegmentSize, env);
	DBVset_snacc_presreq_otherinfo(PDU_ptr, psPresentReq->ppsOtherInformation, env);
}



int DBVset_snacc_presreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.presentRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_presreq_ressetid(PDU *PDU_ptr, DbvInternationalString sResultSetId, jmp_buf env)
{
	DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.presentRequest->resultSetId), &sResultSetId, env);

}



int DBVset_snacc_presreq_ressetstart(PDU *PDU_ptr, Ulong ulResultSetStartPoint, jmp_buf env)
{
	PDU_ptr->a.presentRequest->resultSetStartPoint = ulResultSetStartPoint;
}



int DBVset_snacc_presreq_numrecsreq(PDU *PDU_ptr, Ulong ulNumberOfRecordsRequested, jmp_buf env)
{
	PDU_ptr->a.presentRequest->numberOfRecordsRequested = ulNumberOfRecordsRequested;
}



int DBVset_snacc_presreq_addranges(PDU *PDU_ptr, DbvRange **ppsAdditionalRanges, jmp_buf env)
{

	Range *data_ptr;
	Range **data_ptr_address;
	int current_listnode = 0;


	/* If the Dbv list is not NULL (the field is optional) */
	if (ppsAdditionalRanges != NULL) {

		/* Create a snacc list */
		PDU_ptr->a.presentRequest->additionalRanges = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->additionalRanges, env);

		/* while not at end of Dbv List */
		while (ppsAdditionalRanges[current_listnode] != NULL) {


			/* create a snacc list node */
			data_ptr_address = AsnListAppend(PDU_ptr->a.presentRequest->additionalRanges);
			CheckAsn1Alloc(data_ptr_address, env);

                	*data_ptr_address = Asn1Alloc(sizeof(struct Range));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;

			/* Now add the Dbv list item to the snacc list node */
			data_ptr->startingPosition = ppsAdditionalRanges[current_listnode]->ulStartingPosition; 
			data_ptr->numberOfRecords = ppsAdditionalRanges[current_listnode]->ulNumberOfRecords; 

			current_listnode++;
		}
	}
}



int DBVset_snacc_presreq_reccomp(PDU *PDU_ptr, DbvRecordComposition *psRecordComposition, jmp_buf env)
{
	/* If the field is not NULL (the field is optional) */
	if (psRecordComposition != NULL) {
		PDU_ptr->a.presentRequest->recordComposition = (struct PresentRequestChoice *) Asn1Alloc(sizeof(struct PresentRequestChoice));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->recordComposition, env);
		DBVdbv_to_snacc_DbvRecordComposition(PDU_ptr->a.presentRequest->recordComposition, psRecordComposition, env);
	}
}



int DBVset_snacc_presreq_prefrecsyn(PDU *PDU_ptr, DbvObjectIdentifier pcPreferredRecordSyntax, jmp_buf env)
{
	if(pcPreferredRecordSyntax != NULL)
                ASN_MakeOid((char *) pcPreferredRecordSyntax, &PDU_ptr->a.presentRequest->preferredRecordSyntax, env);

}



int DBVset_snacc_presreq_maxsegcnt(PDU *PDU_ptr, Ulong *pulMaxSegmentCount, jmp_buf env)
{
	if (pulMaxSegmentCount != NULL) {	
		PDU_ptr->a.presentRequest->maxSegmentCount = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->maxSegmentCount, env);

		*PDU_ptr->a.presentRequest->maxSegmentCount = *pulMaxSegmentCount;
	}
}



int DBVset_snacc_presreq_maxrecsze(PDU *PDU_ptr, Ulong *pulMaxRecordSize, jmp_buf env)
{
	if (pulMaxRecordSize != NULL) 	{
		PDU_ptr->a.presentRequest->maxRecordSize = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->maxRecordSize, env);

		*PDU_ptr->a.presentRequest->maxRecordSize = *pulMaxRecordSize;
	}
}



int DBVset_snacc_presreq_maxsegsze(PDU *PDU_ptr, Ulong *pulMaxSegmentSize, jmp_buf env)
{
	if (pulMaxSegmentSize != NULL) 	{
		PDU_ptr->a.presentRequest->maxSegmentSize = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->maxSegmentSize, env);
		*PDU_ptr->a.presentRequest->maxSegmentSize = *pulMaxSegmentSize;
	}

}



int DBVset_snacc_presreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.presentRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.presentRequest->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.presentRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS PresentResponse PDU RECEIVED FROM TARGET 
*/




int DBVset_dbv_fields_presresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_PresentConf;
	psOriginData->uOriginData.psPresentRsp = (DbvPresentRsp *) DbvAlloc (sizeof (DbvPresentRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psPresentRsp, env);

	DBVset_dbv_presresp_refid(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);
	DBVset_dbv_presresp_numrecs(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);
	DBVset_dbv_presresp_nxtresset(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);
	DBVset_dbv_presresp_presstat(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);
	DBVset_dbv_presresp_records(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);
	DBVset_dbv_presresp_otherinfo(psOriginData->uOriginData.psPresentRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_presresp_refid(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.presentResponse->referenceId.octetLen != 0){
		presresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(presresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(presresp->psReferenceId, &(PDU_ptr->a.presentResponse->referenceId), env); 
	}
}


int DBVset_dbv_presresp_numrecs(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{

	presresp->ulNumberOfRecordsReturned = PDU_ptr->a.presentResponse->numberOfRecordsReturned;
}



int DBVset_dbv_presresp_nxtresset(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{
	presresp->ulNextResultSetPosition = PDU_ptr->a.presentResponse->nextResultSetPosition;
}



int DBVset_dbv_presresp_presstat(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{
	/* In present, presentStatus is a value */
	DBVsnacc_to_dbv_presstat(&presresp->ePresentStatus, PDU_ptr->a.presentResponse->presentStatus);
}



int DBVset_dbv_presresp_records(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.presentResponse->records != NULL) {
		presresp->psRecords = (DbvRecords *) DbvAlloc (sizeof(struct DbvRecords));
		CheckDbvAlloc(presresp->psRecords, env);

		DBVsnacc_to_dbv_records(presresp->psRecords, PDU_ptr->a.presentResponse->records, env);
        }
}



int DBVset_dbv_presresp_otherinfo(DbvPresentRsp *presresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.presentResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.presentResponse->otherInfo);
		presresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(presresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(presresp->ppsOtherInformation, PDU_ptr->a.presentResponse->otherInfo, env);
        }
}



