/*  Module Name: @(#)origin_z3950_rescntrl.c	7.2
    Version: 4/16/97
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Origin end routines to create and send a 
		  ResourceControlResponse PDU to a target and process a 
		  received ResourceControlRequest PDU sent by a target 
		  

*/


#include "origin_z3950_rescntrl.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE ResourceControlResponse PDU TO SEND TO TARGET AND SEND IT 
*/

/* one of the few responses sent from an origin to a target */

int DbvResourceControlResponse(DbvAssocId lAssocId, DbvResourceControlRsp *psResourceControlRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVResourceControlResponse", lAssocId);

        if (DBVCheckDbvResourceControlRspFields(lAssocId, psResourceControlRsp, ppsSIError) == SI_OK)

		if (DBVChangeState(lAssocId, DBV_OE_RESCNTRSP, ppsSIError) == SI_OK)

        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if (return_value == SI_OK) {
		DBVSendingTrace(lAssocId);
                DBVResCntrlRspTrace(lAssocId, psResourceControlRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.resourceControlResponse = (struct ResourceControlResponse *) Asn1Alloc (sizeof(struct ResourceControlResponse));
			CheckAsn1Alloc(PDU_ptr->a.resourceControlResponse, env);
        		PDU_ptr->choiceId = PDU_RESOURCECONTROLRESPONSE;
			DBVsetup_snacc_ResourceControlResponse(PDU_ptr, psResourceControlRsp, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVResourceControlResponse", lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvResourceControlRspFields(DbvAssocId lAssocId, DbvResourceControlRsp *psResourceControlRsp, DbvError **ppsSIError)
{

	/* o */
	/* Dont have to check psReferenceId */

	/* c */
	if (DBVcheck_DbvBool(lAssocId, &psResourceControlRsp->fContinueFlag, "psResourceControlRsp->fContinueFlag", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
	if (DBVcheck_DbvBool_opt(lAssocId, psResourceControlRsp->pfResultSetWanted, "*psResourceControlRsp->pfResultSetWanted", ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvOtherInformation_opt(lAssocId, psResourceControlRsp->ppsOtherInformation, "psResourceControlRsp->ppsOtherInformation", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}




/* snacc set up */

int DBVsetup_snacc_ResourceControlResponse(PDU *PDU_ptr, DbvResourceControlRsp *psResourceControlRsp, jmp_buf env)
{
	/* o */
	DBVset_ResourceControlResponse_referenceId(PDU_ptr, psResourceControlRsp->psReferenceId, env);

	/* c */
        DBVset_ResourceControlResponse_continueFlag(PDU_ptr, &psResourceControlRsp->fContinueFlag, env);

	/* o */
        DBVset_ResourceControlResponse_resultSetWanted(PDU_ptr, psResourceControlRsp->pfResultSetWanted, env);

	/* o */
	DBVset_ResourceControlResponse_otherinfo(PDU_ptr, psResourceControlRsp->ppsOtherInformation, env);
}



int DBVset_ResourceControlResponse_referenceId(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvOctetString_opt(&(PDU_ptr->a.resourceControlResponse->referenceId), psReferenceId, env);
	
}



int DBVset_ResourceControlResponse_continueFlag(PDU *PDU_ptr, DbvBool *pfContinueFlag, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvBool(&PDU_ptr->a.resourceControlResponse->continueFlag, pfContinueFlag, env);
}



int DBVset_ResourceControlResponse_resultSetWanted(PDU *PDU_ptr, DbvBool *pfResultSetWanted, jmp_buf env)
{
	DBVdbv_to_snacc_DbvBool_opt(&PDU_ptr->a.resourceControlResponse->resultSetWanted, pfResultSetWanted, env);
}



int DBVset_ResourceControlResponse_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	DBVdbv_to_snacc_otherinfo_opt(&PDU_ptr->a.resourceControlResponse->otherInfo, ppsOtherInformation, env);

}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS ResourceControlRequest PDU RECEIVED FROM TARGET 
*/

int DBVset_dbv_fields_DbvResourceControlReq(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{
 
        psOriginData->eOriginEvent = OR_ResourCntrlInd;
        psOriginData->uOriginData.psResourceControlReq = (DbvResourceControlReq *) DbvAlloc ( sizeof (DbvResourceControlReq));
	CheckDbvAlloc(psOriginData->uOriginData.psResourceControlReq, env);

	/* o */
        DBVset_DbvResourceControlReq_psReferenceId(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);

	/* o */
        DBVset_DbvResourceControlReq_pfSuspendedFlag(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);

	/* o */
        DBVset_DbvResourceControlReq_psResourceReport(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);


	/* o */
        DBVset_DbvResourceControlReq_pePartialResultsAvailable(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);


	/* c */
        DBVset_DbvResourceControlReq_fResponseRequired(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);


	/* o */
        DBVset_DbvResourceControlReq_pfTriggeredRequestFlag(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);

	/* o */
	DBVset_DbvResourceControlReq_ppsOtherInformation(psOriginData->uOriginData.psResourceControlReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}




int DBVset_DbvResourceControlReq_psReferenceId(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */	
	DBVsnacc_to_dbv_DbvOctetString_opt(&psResourceControlReq->psReferenceId, &PDU_ptr->a.resourceControlRequest->referenceId, env);
}



int DBVset_DbvResourceControlReq_pfSuspendedFlag(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvBool_opt(&psResourceControlReq->pfSuspendedFlag, PDU_ptr->a.resourceControlRequest->suspendedFlag, env);

}


int DBVset_DbvResourceControlReq_psResourceReport(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_external_opt(&psResourceControlReq->psResourceReport, PDU_ptr->a.resourceControlRequest->resourceReport, env);
}



int DBVset_DbvResourceControlReq_pePartialResultsAvailable(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{

	/* o */
	DBVsnacc_to_dbv_DbvPartialResultsAvailable_opt(&psResourceControlReq->pePartialResultsAvailable, PDU_ptr->a.resourceControlRequest->partialResultsAvailable, env);
}



int DBVsnacc_to_dbv_DbvPartialResultsAvailable_opt(DbvPartialResultsAvailable **ppePartialResultsAvailable, ResourceControlRequestInt *partialResultsAvailable, jmp_buf env)
{
	if (partialResultsAvailable) {
		*ppePartialResultsAvailable = (DbvPartialResultsAvailable *) DbvAlloc (sizeof (DbvPartialResultsAvailable));
		CheckDbvAlloc(*ppePartialResultsAvailable, env);
		DBVsnacc_to_dbv_DbvPartialResultsAvailable(*ppePartialResultsAvailable, partialResultsAvailable, env);
	}
}



int DBVsnacc_to_dbv_DbvPartialResultsAvailable(DbvPartialResultsAvailable *pePartialResultsAvailable, ResourceControlRequestInt *partialResultsAvailable, jmp_buf env)
{
	switch (*partialResultsAvailable) {
	case SUBSET1:
		*pePartialResultsAvailable = PRA_Subset;
		break;
	case INTERIM2:
		*pePartialResultsAvailable = PRA_Interim2;
		break;
	case NONE2:
		*pePartialResultsAvailable = PRA_None2;
		break;
	default:
		*pePartialResultsAvailable = PRA_Invalid;
		break;
	}
}



int DBVset_DbvResourceControlReq_fResponseRequired(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvBool(&psResourceControlReq->fResponseRequired, &PDU_ptr->a.resourceControlRequest->responseRequired, env);

}



int DBVset_DbvResourceControlReq_pfTriggeredRequestFlag(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvBool_opt(&psResourceControlReq->pfTriggeredRequestFlag, PDU_ptr->a.resourceControlRequest->triggeredRequestFlag, env);

}



int DBVset_DbvResourceControlReq_ppsOtherInformation(DbvResourceControlReq *psResourceControlReq, PDU *PDU_ptr, jmp_buf env)
{
        /* o */
        DBVsnacc_to_dbv_otherinfo_opt(&psResourceControlReq->ppsOtherInformation, PDU_ptr->a.resourceControlRequest->otherInfo, env); 
}





