/*  Module Name: @(#)origin_z3950_resrep.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a ResourceReportRequest
	  	 PDU to a target and process a received ResourceReportResponse 
		 PDU sent by a target 

*/

#include "origin_z3950_resrep.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE ResourceReportRequest PDU TO SEND TO TARGET AND SEND IT
*/


int DbvResourceReportRequest(DbvAssocId lAssocId, DbvResourceReportReq *psResourceReportReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVResourceReportRequest",lAssocId);

	if (DBVCheckDbvResRepReqFields(lAssocId, psResourceReportReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_RESOURCEREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVResRepReqTrace(lAssocId, psResourceReportReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {
			PDU_ptr->a.resourceReportRequest = (struct ResourceReportRequest *) Asn1Alloc(sizeof(struct ResourceReportRequest));
			CheckAsn1Alloc(PDU_ptr->a.resourceReportRequest, env);
			PDU_ptr->choiceId = PDU_RESOURCEREPORTREQUEST;
			DBVsetup_snacc_resreprequest(PDU_ptr, psResourceReportReq, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVResourceReportRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvResRepReqFields(DbvAssocId lAssocId, DbvResourceReportReq *psResourceReportReq, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_rerereq_preffmt(lAssocId, psResourceReportReq, ppsSIError))
                return SI_NOTOK;
	
        if (DBVcheck_dbv_rerereq_otherinfo(lAssocId, psResourceReportReq, ppsSIError))
                return SI_NOTOK;
	
	return SI_OK;
}



int DBVcheck_dbv_rerereq_preffmt(DbvAssocId lAssocId, DbvResourceReportReq *psResourceReportReq, DbvError **ppsSIError)
{
        if (psResourceReportReq->pcPrefResourceReportFormat == NULL)
                return SI_OK;
 
        if (DBVcheck_dbv_resrepoid(psResourceReportReq->pcPrefResourceReportFormat) == SI_NOTOK)
                return(DBVdbv_oid_error(lAssocId, "psResourceReportReq->pcPrefResourceReportFormat" , ppsSIError));
 
 
        return SI_OK;

}



int DBVcheck_dbv_rerereq_otherinfo(DbvAssocId lAssocId, DbvResourceReportReq *psResourceReportReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psResourceReportReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psResourceReportReq->ppsOtherInformation, "psResourceReportReq->ppsOtherInformation", ppsSIError));

}




int DBVsetup_snacc_resreprequest(PDU *PDU_ptr, DbvResourceReportReq *psResourceReportReq, jmp_buf env)
{
	DBVset_snacc_resrepreq_refid(PDU_ptr, psResourceReportReq->psReferenceId, env);
	DBVset_snacc_resrepreq_opid(PDU_ptr, psResourceReportReq->psOperationId, env);
	DBVset_snacc_resrepreq_prefformat(PDU_ptr,psResourceReportReq->pcPrefResourceReportFormat, env);
	DBVset_snacc_resrepreq_otherinfo(PDU_ptr, psResourceReportReq->ppsOtherInformation, env);
}



int DBVset_snacc_resrepreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given (it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.resourceReportRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_resrepreq_opid(PDU *PDU_ptr, DbvOctetString *psOperationId, jmp_buf env)
{
	/* Check if an opId is given (it is an optional field) */
	if  (psOperationId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.resourceReportRequest->opId), psOperationId, env);
	}

}



int DBVset_snacc_resrepreq_prefformat(PDU *PDU_ptr, DbvObjectIdentifier pcPrefResourceReportFormat, jmp_buf env)
{
	/* pcPrefResourceReportFormat is a typedefed char* */
	if (pcPrefResourceReportFormat != NULL)
		ASN_MakeOid((char *)pcPrefResourceReportFormat, &(PDU_ptr->a.resourceReportRequest->prefResourceReportFormat), env);

}



int DBVset_snacc_resrepreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.resourceReportRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.resourceReportRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.resourceReportRequest->otherInfo, ppsOtherInformation, env);
	}
}




/*****************************************************************
******************************************************************
******************************************************************
PROCESS ResourceReportResponse PDU RECEIVED FROM TARGET
*/




int DBVset_dbv_fields_resrepresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_ResourceReportConf;
	psOriginData->uOriginData.psResourceReportRsp = (DbvResourceReportRsp *) DbvAlloc (sizeof (DbvResourceReportRsp));
	CheckAsn1Alloc(psOriginData->uOriginData.psResourceReportRsp, env);

	DBVset_dbv_resrepresp_refid(psOriginData->uOriginData.psResourceReportRsp, PDU_ptr, env);
	DBVset_dbv_resrepresp_repstat(psOriginData->uOriginData.psResourceReportRsp, PDU_ptr, env);
	DBVset_dbv_resrepresp_report(psOriginData->uOriginData.psResourceReportRsp, PDU_ptr, env);
	DBVset_dbv_resrepresp_otherinfo(psOriginData->uOriginData.psResourceReportRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_resrepresp_refid(DbvResourceReportRsp *resrepresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.resourceReportResponse->referenceId.octetLen != 0) {
		resrepresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(resrepresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(resrepresp->psReferenceId, &(PDU_ptr->a.resourceReportResponse->referenceId), env); 
	}
}



int DBVset_dbv_resrepresp_repstat(DbvResourceReportRsp *resrepresp, PDU *PDU_ptr, jmp_buf env)
{
	/* Compulosry field */
	switch (PDU_ptr->a.resourceReportResponse->resourceReportStatus) {
	case SUCCESS4:
		resrepresp->eResourceReportStatus = RRS_Success;
		break;
	case PARTIAL:
		resrepresp->eResourceReportStatus = RRS_Partial;
		break;
	case FAILURE_1:
		resrepresp->eResourceReportStatus = RRS_Failure_1;
		break;
	case FAILURE_2:
		resrepresp->eResourceReportStatus = RRS_Failure_2;
		break;
	case FAILURE_3:
		resrepresp->eResourceReportStatus = RRS_Failure_3;
		break;
	case FAILURE_4:
		resrepresp->eResourceReportStatus = RRS_Failure_4;
		break;
	case FAILURE_5:
		resrepresp->eResourceReportStatus = RRS_Failure_5;
		break;
	case FAILURE_6:
		resrepresp->eResourceReportStatus = RRS_Failure_6;
		break;
	default:
		resrepresp->eResourceReportStatus = RRS_Invalid;
		break;
	}
}



int DBVset_dbv_resrepresp_report(DbvResourceReportRsp *resrepresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.resourceReportResponse->resourceReport != NULL) {
		resrepresp->psResourceReport = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(resrepresp->psResourceReport, env);

		DBVsnacc_to_dbv_external(resrepresp->psResourceReport, PDU_ptr->a.resourceReportResponse->resourceReport, env);
        }
}



int DBVset_dbv_resrepresp_otherinfo(DbvResourceReportRsp *resrepresp, PDU *PDU_ptr, jmp_buf env)
{

	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.resourceReportResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.resourceReportResponse->otherInfo);
		resrepresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(resrepresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(resrepresp->ppsOtherInformation, PDU_ptr->a.resourceReportResponse->otherInfo, env);
        }
}




