/*  Module Name: @(#)origin_z3950_scan.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a ScanRequest PDU 
		 to a target and process a received ScanResponse PDU 
		 sent by a target 

*/

#include "origin_z3950_scan.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE ScanRequest PDU TO SEND TO TARGET AND SEND IT 
*/


int DbvScanRequest(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVScanRequest",lAssocId);

	if (DBVCheckDbvScanReqFields(lAssocId, psScanReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_SCANREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;
				
	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVScanReqTrace(lAssocId, psScanReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {
			PDU_ptr->a.scanRequest = (struct ScanRequest *) Asn1Alloc(sizeof(struct ScanRequest));
			CheckAsn1Alloc(PDU_ptr->a.scanRequest, env);
			PDU_ptr->choiceId = PDU_SCANREQUEST;
			DBVsetup_snacc_scanrequest(PDU_ptr, psScanReq, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVScanRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvScanReqFields(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_scanreq_dbnames(lAssocId, psScanReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanreq_attset(lAssocId, psScanReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanreq_atttrm(lAssocId, psScanReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanreq_otherinfo(lAssocId, psScanReq, ppsSIError))
                return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_scanreq_dbnames(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
	/* Cant be NULL because compulsory */
	if (psScanReq->ppsDatabaseNames == NULL)
                return(DBVdbv_field_missing_error(lAssocId, "psScanReq->ppsDatabaseNames", ppsSIError));
 

	/* No need to check ppsDatabaseNames because it is a list of
	   DbvInternationalString */

        return SI_OK;
}



int DBVcheck_dbv_scanreq_attset(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
        if (psScanReq->pcAttributeSetId == NULL)
                return SI_OK;
 
        if (DBVcheck_dbv_attsetoid(psScanReq->pcAttributeSetId) == SI_NOTOK)
                return(DBVdbv_oid_error(lAssocId, "psScanReq->pcAttributeSetId" , ppsSIError));
 
        return SI_OK;

}



int DBVcheck_dbv_scanreq_atttrm(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
	return(DBVcheck_dbv_attsplusterm(lAssocId, &psScanReq->sAttributesPlusTerm, "psScanReq->sAttributesPlusTerm.",ppsSIError));
}



int DBVcheck_dbv_scanreq_otherinfo(DbvAssocId lAssocId, DbvScanReq *psScanReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psScanReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psScanReq->ppsOtherInformation, "psScanReq->ppsOtherInformation", ppsSIError));

}



int DBVsetup_snacc_scanrequest(PDU *PDU_ptr, DbvScanReq *psScanReq, jmp_buf env)
{
	DBVset_snacc_scanreq_refid(PDU_ptr, psScanReq->psReferenceId, env);
	DBVset_snacc_scanreq_dbnames(PDU_ptr, psScanReq->ppsDatabaseNames, env);
	DBVset_snacc_scanreq_attsetid(PDU_ptr, psScanReq->pcAttributeSetId, env); 
	DBVset_snacc_scanreq_attplusterm(PDU_ptr, psScanReq->sAttributesPlusTerm, env);
	DBVset_snacc_scanreq_stepsize(PDU_ptr, psScanReq->pulStepSize, env);
	DBVset_snacc_scanreq_numterms(PDU_ptr, psScanReq->ulNumberOfTermsRequested, env);
	DBVset_snacc_scanreq_prefpos(PDU_ptr, psScanReq->pulPreferredPositionInResponse, env);
	DBVset_snacc_scanreq_otherinfo(PDU_ptr, psScanReq->ppsOtherInformation, env);
}


int DBVset_snacc_scanreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.scanRequest->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_scanreq_dbnames(PDU *PDU_ptr, DbvInternationalString **ppsDatabaseNames, jmp_buf env)
{
	/* Compulsory */
        DatabaseName *data_ptr;
        DatabaseName **data_ptr_address;
 
        int current_listnode = 0;
 
        PDU_ptr->a.scanRequest->databaseNames = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.scanRequest->databaseNames, env);
 
        while(ppsDatabaseNames[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(PDU_ptr->a.scanRequest->databaseNames);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(DatabaseName));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_inter_string(data_ptr, ppsDatabaseNames[current_listnode], env);
                current_listnode++;
        }
}



int DBVset_snacc_scanreq_attsetid(PDU *PDU_ptr, DbvObjectIdentifier pcAttributeSetId, jmp_buf env)
{
	/* pcAttributeSetId is a typedefed char* */
	if (pcAttributeSetId != NULL)
		ASN_MakeOid((char *)pcAttributeSetId, &(PDU_ptr->a.scanRequest->attributeSet), env);
}



int DBVset_snacc_scanreq_attplusterm(PDU *PDU_ptr, DbvAttributesPlusTerm sAttributesPlusTerm, jmp_buf env)
{
	/* Compulsory */
	PDU_ptr->a.scanRequest->termListAndStartPoint = (struct AttributesPlusTerm*) Asn1Alloc (sizeof(struct AttributesPlusTerm));
	CheckAsn1Alloc(PDU_ptr->a.scanRequest->termListAndStartPoint, env);

	DBVdbv_to_snacc_attrterm(PDU_ptr->a.scanRequest->termListAndStartPoint, &sAttributesPlusTerm, env);
}



int DBVset_snacc_scanreq_stepsize(PDU *PDU_ptr, Ulong *pulStepSize, jmp_buf env)
{
	if (pulStepSize != NULL) {
		PDU_ptr->a.scanRequest->stepSize = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.scanRequest->stepSize, env);

		*PDU_ptr->a.scanRequest->stepSize = *pulStepSize;
	}
}



int DBVset_snacc_scanreq_numterms(PDU *PDU_ptr, Ulong ulNumberOfTermsRequested, jmp_buf env)
{
	PDU_ptr->a.scanRequest->numberOfTermsRequested = ulNumberOfTermsRequested;
}



int DBVset_snacc_scanreq_prefpos(PDU *PDU_ptr, Ulong *pulPreferredPositionInResponse, jmp_buf env)
{
	if (pulPreferredPositionInResponse != NULL) {
		PDU_ptr->a.scanRequest->preferredPositionInResponse = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.scanRequest->preferredPositionInResponse, env);

		*PDU_ptr->a.scanRequest->preferredPositionInResponse = *pulPreferredPositionInResponse;
	}
}




int DBVset_snacc_scanreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.scanRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.scanRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.scanRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS ScanResponse PDU RECEIVED FROM SERVER
*/




int DBVset_dbv_fields_scanresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_ScanConf;
	psOriginData->uOriginData.psScanRsp = (DbvScanRsp *) DbvAlloc (sizeof (DbvScanRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psScanRsp, env);

	DBVset_dbv_scanresp_refid(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_stepsize(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_scanstat(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_numentries(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_posterm(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_entries(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_attsetid(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);
	DBVset_dbv_scanresp_otherinfo(psOriginData->uOriginData.psScanRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_scanresp_refid(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.scanResponse->referenceId.octetLen != 0){
		scanresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(scanresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(scanresp->psReferenceId, &(PDU_ptr->a.scanResponse->referenceId), env); 
	}
}



int DBVset_dbv_scanresp_stepsize(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.scanResponse->stepSize != NULL) {
		scanresp->pulStepSize = (Ulong *) DbvAlloc (sizeof (Ulong));
		CheckDbvAlloc(scanresp->pulStepSize, env);

		*scanresp->pulStepSize = *PDU_ptr->a.scanResponse->stepSize;
	}
}



int DBVset_dbv_scanresp_scanstat(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.scanResponse->scanStatus) {
	case SUCCESS3:
		scanresp->eScanStatus = SCS_Success;
		break;
	case PARTIAL_12:
		scanresp->eScanStatus = SCS_Partial_1;
		break;
	case PARTIAL_21:
		scanresp->eScanStatus = SCS_Partial_2;
		break;
	case PARTIAL_31:
		scanresp->eScanStatus = SCS_Partial_3;
		break;
	case PARTIAL_41:
		scanresp->eScanStatus = SCS_Partial_4;
		break;
	case PARTIAL_5:
		scanresp->eScanStatus = SCS_Partial_5;
		break;
	case FAILURE3:
		scanresp->eScanStatus = SCS_Failure;
		break;
	default:
		scanresp->eScanStatus = SCS_Invalid;
		break;
	}
}



int DBVset_dbv_scanresp_numentries(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	scanresp->ulNumberOfEntriesReturned = PDU_ptr->a.scanResponse->numberOfEntriesReturned;
}



int DBVset_dbv_scanresp_posterm(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.scanResponse->positionOfTerm != NULL) {
		scanresp->pulPositionOfTerm = (Ulong *) DbvAlloc (sizeof (Ulong));
		CheckDbvAlloc(scanresp->pulPositionOfTerm, env);

		*scanresp->pulPositionOfTerm = *PDU_ptr->a.scanResponse->positionOfTerm;
	}
		
}



int DBVset_dbv_scanresp_entries(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	if (PDU_ptr->a.scanResponse->entries != NULL) {
		scanresp->psEntries = (DbvListEntries *) DbvAlloc (sizeof (DbvListEntries));
		CheckDbvAlloc(scanresp->psEntries, env);


		/* If entries exists (it is optional) */
		if (PDU_ptr->a.scanResponse->entries->entries != NULL) {
			num_elements = AsnListCount(PDU_ptr->a.scanResponse->entries->entries);

			/* +1 for the NULL terminator */
			scanresp->psEntries->ppsEntry = (DbvEntry **) DbvAlloc (sizeof(DbvEntry *) * (num_elements + 1));
			CheckDbvAlloc(scanresp->psEntries->ppsEntry, env);

			DBVsnacc_to_dbv_entries(scanresp->psEntries->ppsEntry, PDU_ptr->a.scanResponse->entries->entries, env);
		}


		/* If nonsurrogateDiagnostics exists (it is optional) */
		if (PDU_ptr->a.scanResponse->entries->nonsurrogateDiagnostics != NULL) {
			num_elements = AsnListCount(PDU_ptr->a.scanResponse->entries->nonsurrogateDiagnostics);
			scanresp->psEntries->ppsNonSurrogateDiagnostics = (DbvDiagRec **) DbvAlloc (sizeof(DbvDiagRec *) * (num_elements + 1));
			CheckDbvAlloc(scanresp->psEntries->ppsNonSurrogateDiagnostics, env);
			DBVsnacc_to_dbv_multdiag(scanresp->psEntries->ppsNonSurrogateDiagnostics, PDU_ptr->a.scanResponse->entries->nonsurrogateDiagnostics, env);
		}
	}

}



int DBVsnacc_to_dbv_entries(DbvEntry **ppsEntry, ListEntriesSeqOf *entries, jmp_buf env)
{

	Entry *data_ptr;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, entries) {
                ppsEntry[index] = (DbvEntry *) DbvAlloc (sizeof(DbvEntry));
		CheckDbvAlloc(ppsEntry[index], env);

                DBVsnacc_to_dbv_entry(ppsEntry[index], data_ptr, env);
                index++;
        }
        ppsEntry[index] = NULL;
}



int DBVsnacc_to_dbv_entry(DbvEntry *psEntry, Entry *data_ptr, jmp_buf env)
{
	switch (data_ptr->choiceId) {
	case ENTRY_TERMINFO:
		psEntry->eEntryType = ET_TermInfo;
		psEntry->uEntry.psTermInfo = (DbvTermInfo *) DbvAlloc (sizeof(DbvTermInfo));
		CheckDbvAlloc(psEntry->uEntry.psTermInfo, env);

		DBVsnacc_to_dbv_terminfo(psEntry->uEntry.psTermInfo, data_ptr->a.termInfo, env);
		break;
	case ENTRY_SURROGATEDIAGNOSTIC:
		psEntry->eEntryType = ET_SurrogateDiagnostic;
		psEntry->uEntry.psSurrogateDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof(DbvDiagRec));
		CheckDbvAlloc(psEntry->uEntry.psSurrogateDiagnostic, env);

		DBVsnacc_to_dbv_diag_rec(psEntry->uEntry.psSurrogateDiagnostic, data_ptr->a.surrogateDiagnostic, env);
		break;
	}
}



int DBVsnacc_to_dbv_terminfo(DbvTermInfo *psTermInfo, struct TermInfo* termInfo, jmp_buf env)
{
	int num_elements = 0;

	/* Term compulsory */
	DBVsnacc_to_dbv_term(&psTermInfo->sTerm, termInfo->term, env);

	if (termInfo->displayTerm.octetLen != 0) {
		psTermInfo->psDisplayTerm = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psTermInfo->psDisplayTerm, env);
		DBVsnacc_to_dbv_inter_string(psTermInfo->psDisplayTerm, &termInfo->displayTerm, env);
	}

		
	if (termInfo->suggestedAttributes != NULL) {
		num_elements = AsnListCount(termInfo->suggestedAttributes);
		psTermInfo->ppsSuggestedAttribute = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * (num_elements + 1));
		CheckDbvAlloc(psTermInfo->ppsSuggestedAttribute, env);

		DBVsnacc_to_dbv_attribute_list(psTermInfo->ppsSuggestedAttribute, termInfo->suggestedAttributes, env);
	}

		
	if (termInfo->alternativeTerm != NULL) {
		num_elements = AsnListCount(termInfo->alternativeTerm);
		psTermInfo->ppsAlternativeTerm = (DbvAttributesPlusTerm **) DbvAlloc (sizeof(DbvAttributesPlusTerm *) * (num_elements + 1));
		CheckDbvAlloc(psTermInfo->ppsAlternativeTerm, env);

		DBVsnacc_to_dbv_attrtermlist(psTermInfo->ppsAlternativeTerm, termInfo->alternativeTerm, env);
	}



	if (termInfo->globalOccurrences != NULL) {
		psTermInfo->pulGlobalOccurrences = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(psTermInfo->pulGlobalOccurrences, env);

		*psTermInfo->pulGlobalOccurrences = *termInfo->globalOccurrences;
	}
		
	
	if (termInfo->byAttributes != NULL) {
		num_elements = AsnListCount(termInfo->byAttributes);
		psTermInfo->ppsByAttributes = (DbvOccurrenceByAttributes **) DbvAlloc (sizeof(DbvOccurrenceByAttributes *) * (num_elements + 1));
		CheckDbvAlloc(psTermInfo->ppsByAttributes, env);

		DBVsnacc_to_dbv_occurbyatt_list(psTermInfo->ppsByAttributes, termInfo->byAttributes, env);
	}

	
	if (termInfo->otherTermInfo != NULL) {
		num_elements = AsnListCount(termInfo->otherTermInfo);
		psTermInfo->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(psTermInfo->ppsOtherInformation, env);
		DBVsnacc_to_dbv_otherinfo(psTermInfo->ppsOtherInformation, termInfo->otherTermInfo, env);
	}
}


	
int DBVsnacc_to_dbv_attrtermlist(DbvAttributesPlusTerm **ppsAlternativeTerm, TermInfoSeqOf *alternativeTerm, jmp_buf env)
{
	AttributesPlusTerm *data_ptr;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, alternativeTerm) {
                ppsAlternativeTerm[index] = (DbvAttributesPlusTerm *) DbvAlloc (sizeof(DbvAttributesPlusTerm));
		CheckDbvAlloc(ppsAlternativeTerm[index], env);

		DBVsnacc_to_dbv_attrterm(ppsAlternativeTerm[index], data_ptr, env);
                index++;
        }
        ppsAlternativeTerm[index] = NULL;
}



int DBVsnacc_to_dbv_occurbyatt_list(DbvOccurrenceByAttributes **ppsByAttributes, OccurrenceByAttributes *byAttributes, jmp_buf env)
{
	OccurrenceByAttributesSeq *data_ptr;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, byAttributes) {
		ppsByAttributes[index] = (DbvOccurrenceByAttributes *) DbvAlloc (sizeof(DbvOccurrenceByAttributes));
		CheckDbvAlloc(ppsByAttributes[index], env);
                DBVsnacc_to_dbv_occurbyattseq(ppsByAttributes[index], data_ptr, env);
                index++;
        }
}



int DBVsnacc_to_dbv_occurbyattseq(DbvOccurrenceByAttributes *psByAttributes, OccurrenceByAttributesSeq *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* attributes compulsory */
	num_elements = AsnListCount(data_ptr->attributes);
	psByAttributes->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * (num_elements + 1));
	CheckDbvAlloc(psByAttributes->ppsAttributeElement, env);

	DBVsnacc_to_dbv_attribute_list(psByAttributes->ppsAttributeElement, data_ptr->attributes, env);


	if (data_ptr->occurrences != NULL) {
		psByAttributes->psOccurrence = (DbvOccurrence *) DbvAlloc (sizeof (DbvOccurrence));
		CheckDbvAlloc(psByAttributes->psOccurrence, env);

		DBVsnacc_to_dbv_occurrence(psByAttributes->psOccurrence, data_ptr->occurrences, env);
	}


	if (data_ptr->otherOccurInfo != NULL) {
		num_elements = AsnListCount(data_ptr->otherOccurInfo);
		psByAttributes->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(psByAttributes->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(psByAttributes->ppsOtherInformation, data_ptr->otherOccurInfo, env);
	}
}



int DBVsnacc_to_dbv_occurrence(DbvOccurrence *psOccurrence, struct OccurrenceByAttributesSeqChoice* occurrences, jmp_buf env)
{
	int num_elements = 0;

	switch (occurrences->choiceId) {
	case OCCURRENCEBYATTRIBUTESSEQCHOICE_GLOBAL:
		psOccurrence->eOccurrenceType = OT_Global;
		psOccurrence->uOccurence.ulGlobal = occurrences->a.global;
		break;
	case OCCURRENCEBYATTRIBUTESSEQCHOICE_BYDATABASE:
		psOccurrence->eOccurrenceType = OT_ByDatabase;
		num_elements = AsnListCount(occurrences->a.byDatabase);
		psOccurrence->uOccurence.ppsOccurrenceByDb = (DbvOccurrenceByDb **) DbvAlloc (sizeof(DbvOccurrenceByDb *) * (num_elements + 1));
		CheckDbvAlloc(psOccurrence->uOccurence.ppsOccurrenceByDb, env);

		DBVsnacc_to_dbv_bydatabase(psOccurrence->uOccurence.ppsOccurrenceByDb, occurrences->a.byDatabase, env);
		break;
	}
}



int DBVsnacc_to_dbv_bydatabase(DbvOccurrenceByDb **ppsOccurrenceByDb, OccurrenceByAttributesSeqChoiceSeqOf *byDatabase, jmp_buf env)
{
	/* OccurrenceByAttributesSeqChoiceSeqOf is a typedefed AsnList - SEQUENCE OF OccurrenceByAttributesSeqChoiceSeqOfSeq */

	OccurrenceByAttributesSeqChoiceSeqOfSeq *data_ptr;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, byDatabase) {
                ppsOccurrenceByDb[index] = (DbvOccurrenceByDb *) DbvAlloc (sizeof(DbvOccurrenceByDb));
		CheckDbvAlloc(ppsOccurrenceByDb[index], env);

                DBVsnacc_to_dbv_bigoccurrthing(ppsOccurrenceByDb[index], data_ptr, env);
                index++;
        }
        ppsOccurrenceByDb[index] = NULL;
}



int DBVsnacc_to_dbv_bigoccurrthing(DbvOccurrenceByDb *psOccurrenceByDb, OccurrenceByAttributesSeqChoiceSeqOfSeq *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* DatabaseName is compulsory */
	DBVsnacc_to_dbv_inter_string(&psOccurrenceByDb->sDatabaseName, &data_ptr->db, env);

	if (data_ptr->num != NULL) {
		psOccurrenceByDb->pulNumber = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(psOccurrenceByDb->pulNumber, env);

		*psOccurrenceByDb->pulNumber = *data_ptr->num;
	}

	if (data_ptr->otherDbInfo != NULL) {
		num_elements = AsnListCount(data_ptr->otherDbInfo);
		psOccurrenceByDb->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(psOccurrenceByDb->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(psOccurrenceByDb->ppsOtherInformation, data_ptr->otherDbInfo, env);
	}

}



int DBVset_dbv_scanresp_attsetid(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
        char *oid = NULL;
 
        if (PDU_ptr->a.scanResponse->attributeSet.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&(PDU_ptr->a.scanResponse->attributeSet));
                scanresp->pcAttributeSetId = (char *) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(scanresp->pcAttributeSetId, env);

                strcpy(scanresp->pcAttributeSetId, oid);
        }
}




int DBVset_dbv_scanresp_otherinfo(DbvScanRsp *scanresp, PDU *PDU_ptr, jmp_buf env)
{
        int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.scanResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.scanResponse->otherInfo);
		scanresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(scanresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(scanresp->ppsOtherInformation, PDU_ptr->a.scanResponse->otherInfo, env);
        }
}




