/*  Module Name: @(#)origin_z3950_search.c	7.1
    Version: 11/15/96
    Author: John Hough/Chris Attewell
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a SearchRequest PDU 
		 to a target and process a received SearchResponse PDU 
		 sent by a target

*/

#include "origin_z3950_search.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE SearchRequest PDU TO SEND TO TARGET AND SEND IT
*/



int DbvSearchRequest(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVSearchRequest",lAssocId);

	if (DBVCheckDbvSearchReqFields(lAssocId, psSearchReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_SEARCHREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSrchReqTrace(lAssocId, psSearchReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {
 
			PDU_ptr->a.searchRequest = (struct SearchRequest *) Asn1Alloc(sizeof(struct SearchRequest));
                        CheckAsn1Alloc(PDU_ptr->a.searchRequest, env);
			PDU_ptr->choiceId = PDU_SEARCHREQUEST;
 
                        DBVsetup_snacc_srchrequest(PDU_ptr, psSearchReq, env);
                } else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSearchRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvSearchReqFields(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{

        if (DBVcheck_dbv_srchreq_replace(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_dbnames(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_sselnam(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_mselnam(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_prefsyn(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_query(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_addinf(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_othinfo(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_dbv_srchreq_replace(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

        if (DBVcheck_dbv_bool(psSearchReq->fReplaceIndicator) == SI_NOTOK) 
		return(DBVdbv_bool_error(lAssocId, "psSearchReq->fReplaceIndicator", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_srchreq_dbnames(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

	/* If no list at all */
	if (psSearchReq->ppsDatabaseNames == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->ppsDatabaseNames", ppsSIError));
	
	
	return SI_OK;
}



int DBVcheck_dbv_srchreq_sselnam(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{
	/* If null then ok because optional */
	if (psSearchReq->psSmallSetElementNames == NULL)
		return SI_OK;

	switch (psSearchReq->psSmallSetElementNames->eElementSetType) {
	case EST_Generic:

		if (psSearchReq->psSmallSetElementNames->uElementSet.psGeneric == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psSmallSetElementNames->uElementSet.psGeneric", ppsSIError));

		break;
	case EST_DatabaseSpecific:

		if (psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific", ppsSIError));


		return(DBVcheck_dbspecific(lAssocId, psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific, "psSearchReq->psSmallSetElementNames->uElementSet.", ppsSIError));
		break;
	default:
		return(DBVdbv_invalid_enum_error(lAssocId, "psSearchReq->psSmallSetElementNames->eElementSetType", ppsSIError));
		break;
	}

	return SI_OK;
}





int DBVcheck_dbv_srchreq_mselnam(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

	/* If null then ok because optional */
	if (psSearchReq->psMediumSetElementNames == NULL)
		return SI_OK;

	switch (psSearchReq->psMediumSetElementNames->eElementSetType) {
	case EST_Generic:

              if (psSearchReq->psMediumSetElementNames->uElementSet.psGeneric == NULL)
                        return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psMediumSetElementNames->uElementSet.psGeneric", ppsSIError));

		break;
	case EST_DatabaseSpecific:

              if (psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific == NULL)
                        return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific", ppsSIError));

		return(DBVcheck_dbspecific(lAssocId, psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific, "psSearchReq->psMediumSetElementNames->uElementSet.", ppsSIError));
		break;
	default:
		return(DBVdbv_invalid_enum_error(lAssocId, "psSearchReq->psMediumSetElementNames->eElementSetType", ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_srchreq_prefsyn(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{
	/* Can be null because optional */
	if (psSearchReq->pcPreferredRecordSyntax == NULL) 
		return SI_OK;


	if (DBVcheck_dbv_recsyn(psSearchReq->pcPreferredRecordSyntax) == SI_NOTOK) 
		return(DBVdbv_oid_error(lAssocId, "psSearchReq->pcPreferredRecordSyntax", ppsSIError));



	return SI_OK;
	
}



int DBVcheck_dbv_srchreq_query(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

	if(psSearchReq->psQuery == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery", ppsSIError));


	switch (psSearchReq->psQuery->eQueryType) {
	case QT_Private:
		if (psSearchReq->psQuery->uQuery.psPrivateQuery == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psPrivateQuery", ppsSIError));

		break;
	case QT_Rpn:
		if (psSearchReq->psQuery->uQuery.psRpnQuery == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psRpnQuery", ppsSIError));

		return(DBVcheck_dbv_rpn(lAssocId, psSearchReq->psQuery->uQuery.psRpnQuery, "psSearchReq->psQuery->uQuery.psRpnQuery->", ppsSIError));

		break;
	case QT_Iso8777:
		if (psSearchReq->psQuery->uQuery.psIso8777Query == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psIso8777Query", ppsSIError));
		break;
	case QT_Z3958:
		if (psSearchReq->psQuery->uQuery.psZ3958Query == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psZ3958Query", ppsSIError));

		break;
	case QT_Erpn:
		if (psSearchReq->psQuery->uQuery.psErpnQuery == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psErpnQuery", ppsSIError));

		return(DBVcheck_dbv_rpn(lAssocId, psSearchReq->psQuery->uQuery.psErpnQuery, "psSearchReq->psQuery->uQuery.psErpnQuery->", ppsSIError));

		break;
	case QT_NaturalLanguage:
		if (psSearchReq->psQuery->uQuery.psNaturalLanguageQuery == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psQuery->uQuery.psNaturalLanguageQuery", ppsSIError));

		break;
	default:
		return(DBVdbv_invalid_enum_error(lAssocId, "psSearchReq->psQuery->eQueryType", ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_rpn(DbvAssocId lAssocId, DbvRPNQuery *psRpnQuery, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

        if (DBVcheck_dbv_attsetoid(psRpnQuery->pcAttributeSetId) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "pcAttributeSet");
		return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
	}
	

	DBVsa_strcat(local_error_buffer, "sRpnStructure.");
	return(DBVcheck_dbv_rpnstruct(lAssocId, &psRpnQuery->sRpnStructure, local_error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_rpnstruct(DbvAssocId lAssocId, DbvRPNStructure *psRpnStructure, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];


	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psRpnStructure->eRpnStructureType) {
	case RST_RpnLopRopOperator:

		DBVsa_strcat(local_error_buffer, "uRPNStructure.psRpnLopRopOperator");

		if (psRpnStructure->uRPNStructure.psRpnLopRopOperator == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "->psLeftOperand");
		if (psRpnStructure->uRPNStructure.psRpnLopRopOperator->psLeftOperand == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, temp_error_buffer, ppsSIError));

		DBVsa_strcat(temp_error_buffer, "->");
		if (DBVcheck_dbv_rpnstruct(lAssocId, psRpnStructure->uRPNStructure.psRpnLopRopOperator->psLeftOperand, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "->psRightOperand");
		if (psRpnStructure->uRPNStructure.psRpnLopRopOperator->psRightOperand == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, temp_error_buffer, ppsSIError));

		DBVsa_strcat(temp_error_buffer, "->");
		if (DBVcheck_dbv_rpnstruct(lAssocId, psRpnStructure->uRPNStructure.psRpnLopRopOperator->psRightOperand, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "->sOperator");
		DBVsa_strcat(temp_error_buffer, ".");
		return(DBVcheck_dbv_operator(lAssocId, &psRpnStructure->uRPNStructure.psRpnLopRopOperator->sOperator, temp_error_buffer, ppsSIError));

		break;
	case RST_RpnOperand:

		DBVsa_strcat(local_error_buffer, "uRPNStructure.psRpnOperand");
		if (psRpnStructure->uRPNStructure.psRpnOperand == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_operand(lAssocId, psRpnStructure->uRPNStructure.psRpnOperand, local_error_buffer, ppsSIError));
		
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eRpnStructureType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));

		break;
        }


	return SI_OK;
}



int DBVcheck_dbv_operator(DbvAssocId lAssocId, DbvOperator *psOperator, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);



	if (DBVcheck_dbv_enum(psOperator->eOperatorType, OT_Or, OT_Proximity)== SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eOperatorType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* Only need to do futher checks if proximity */
	if (psOperator->eOperatorType == OT_Proximity) {
		DBVsa_strcat(local_error_buffer, "psProximityOperator");

		if (psOperator->psProximityOperator == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_prox(lAssocId, psOperator->psProximityOperator, local_error_buffer, ppsSIError));
	}


	return SI_OK;
}



int DBVcheck_dbv_prox(DbvAssocId lAssocId, DbvProximityOperator *psProximityOperator, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

		
	/* Can be null because optional */
	if (psProximityOperator->pfExclusion != NULL) 
		if (DBVcheck_dbv_bool(*psProximityOperator->pfExclusion) == SI_NOTOK) {
			DBVsa_strcat(local_error_buffer, "pfExclusion");
			return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
		}


	/* No need to check distance */
	

	if (DBVcheck_dbv_bool(psProximityOperator->fOrdered) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fOrdered");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	if (DBVcheck_dbv_enum(psProximityOperator->eProximityRelationType, PRT_LessThan, PRT_NotEqual) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eProximityRelationType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* sProximityUnitCode */
	switch (psProximityOperator->sProximityUnitCode.eProximityUnitType) {
	case PUT_Known:
		
		DBVsa_strcat(local_error_buffer, "sProximityUnitCode.uProximityUnit.eKnownProximityUnit");
		if (DBVcheck_dbv_enum(psProximityOperator->sProximityUnitCode.uProximityUnit.eKnownProximityUnit, KPU_Character, KPU_Byte) == SI_NOTOK) 
			return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
			
		break;
	case PUT_Private:
			

		/* no checking needed here */
		break;
	default:
		DBVsa_strcat(local_error_buffer, "sProximityUnitCode.eProximityUnitType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_operand(DbvAssocId lAssocId, DbvOperand *psRpnOperand, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psRpnOperand->eOperandType) {
	case OT_AttributePlusTerm:

		DBVsa_strcat(local_error_buffer, "uOperand.psAttributesPlusTerm");

		if (psRpnOperand->uOperand.psAttributesPlusTerm == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_attsplusterm(lAssocId, psRpnOperand->uOperand.psAttributesPlusTerm, local_error_buffer, ppsSIError));

		break;


	case OT_ResultSetId:
		DBVsa_strcat(local_error_buffer, "uOperand.psResultSetId");

		if (psRpnOperand->uOperand.psResultSetId == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		break;


	case OT_ResultSetPlusAttributes:
		DBVsa_strcat(local_error_buffer, "uOperand.psResultSetPlusAttributes");

		if (psRpnOperand->uOperand.psResultSetPlusAttributes == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_resplusatts(lAssocId, psRpnOperand->uOperand.psResultSetPlusAttributes, local_error_buffer, ppsSIError));
		break;


	default:
		DBVsa_strcat(local_error_buffer, "eOperandType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_resplusatts(DbvAssocId lAssocId, DbvResultSetPlusAttributes *psResultSetPlusAttributes, char *error_buffer, DbvError **ppsSIError)
{
	
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];


	DBVsa_strcpy(local_error_buffer, error_buffer);

        /* if no list nodes */
        if (psResultSetPlusAttributes->ppsAttributeElement == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsAttributeElement");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}
 

        /* Check all items in the list */
        while (psResultSetPlusAttributes->ppsAttributeElement[index] != NULL) {
		
                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsAttributeElement[");
                sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");

 
                if (DBVcheck_dbv_attelement(lAssocId, psResultSetPlusAttributes->ppsAttributeElement[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
 
        index++;
        }

	return SI_OK;

	
}



int DBVcheck_dbv_srchreq_addinf(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
        if (psSearchReq->ppsAdditionalSearchInfo == NULL)
                return SI_OK;


	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchReq->ppsAdditionalSearchInfo, "psSearchReq->ppsAdditionalSearchInfo", ppsSIError));
}



int DBVcheck_dbv_srchreq_othinfo(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
	if (psSearchReq->ppsOtherInformation == NULL)
                return SI_OK;


	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchReq->ppsOtherInformation, "psSearchReq->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_srchrequest(PDU *PDU_ptr, DbvSearchReq *psSearchReq, jmp_buf env)
{
	DBVset_snacc_searchreq_refid(PDU_ptr, psSearchReq->psReferenceId, env);
	DBVset_snacc_searchreq_ssetupper(PDU_ptr,  psSearchReq->ulSmallSetUpperBound, env);
	DBVset_snacc_searchreq_lsetlower(PDU_ptr, psSearchReq->ulLargeSetLowerBound, env);
	DBVset_snacc_searchreq_msetpres(PDU_ptr, psSearchReq->ulMediumSetPresentNumber, env);
	DBVset_snacc_searchreq_replace(PDU_ptr, psSearchReq->fReplaceIndicator, env);
	DBVset_snacc_searchreq_resultset(PDU_ptr, psSearchReq->sResultSetName, env);
	DBVset_snacc_searchreq_dbnames(PDU_ptr, psSearchReq->ppsDatabaseNames, env);
	DBVset_snacc_searchreq_ssetelname(PDU_ptr, psSearchReq->psSmallSetElementNames, env);
	DBVset_snacc_searchreq_msetelname(PDU_ptr, psSearchReq->psMediumSetElementNames, env);
	DBVset_snacc_searchreq_prefrecsyn(PDU_ptr, psSearchReq->pcPreferredRecordSyntax, env);
	DBVset_snacc_searchreq_query(PDU_ptr, psSearchReq->psQuery, env);
	DBVset_snacc_searchreq_addinfo(PDU_ptr, psSearchReq->ppsAdditionalSearchInfo, env);
	DBVset_snacc_searchreq_otherinfo(PDU_ptr, psSearchReq->ppsOtherInformation, env);
}


int DBVset_snacc_searchreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.searchRequest->referenceId), psReferenceId, env);
	}

}


int DBVset_snacc_searchreq_ssetupper(PDU *PDU_ptr,  Ulong ulSmallSetUpperBound, jmp_buf env)
{
	PDU_ptr->a.searchRequest->smallSetUpperBound = ulSmallSetUpperBound;
}



int DBVset_snacc_searchreq_lsetlower(PDU *PDU_ptr, Ulong ulLargeSetLowerBound, jmp_buf env)
{
	PDU_ptr->a.searchRequest->largeSetLowerBound = ulLargeSetLowerBound;
}



int DBVset_snacc_searchreq_msetpres(PDU *PDU_ptr, Ulong ulMediumSetPresentNumber, jmp_buf env)
{
	PDU_ptr->a.searchRequest->mediumSetPresentNumber = ulMediumSetPresentNumber;
}



int DBVset_snacc_searchreq_replace(PDU *PDU_ptr, DbvBool fReplaceIndicator, jmp_buf env)
{
	if (fReplaceIndicator == DBV_TRUE)
		PDU_ptr->a.searchRequest->replaceIndicator = TRUE;
	else
		PDU_ptr->a.searchRequest->replaceIndicator = FALSE;
}



int DBVset_snacc_searchreq_resultset(PDU *PDU_ptr, DbvInternationalString sResultSetName, jmp_buf env)
{
	/* This field is compulsory */
	DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.searchRequest->resultSetName), &sResultSetName, env);
}



int DBVset_snacc_searchreq_dbnames(PDU *PDU_ptr, DbvInternationalString **ppsDatabaseNames, jmp_buf env)
{
	DatabaseName *data_ptr;
        DatabaseName **data_ptr_address;

	int current_listnode = 0;

	PDU_ptr->a.searchRequest->databaseNames = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.searchRequest->databaseNames, env);
	
	while (ppsDatabaseNames[current_listnode] != NULL) {
		data_ptr_address = AsnListAppend(PDU_ptr->a.searchRequest->databaseNames);
		CheckAsn1Alloc(data_ptr_address, env);

		*data_ptr_address = Asn1Alloc(sizeof(DatabaseName));
		CheckAsn1Alloc(*data_ptr_address, env);

		data_ptr = *data_ptr_address;

		DBVdbv_to_snacc_inter_string(data_ptr, ppsDatabaseNames[current_listnode], env);
		current_listnode++;
	}
		
}



int DBVset_snacc_searchreq_ssetelname(PDU *PDU_ptr, DbvElementSetNames *psSmallSetElementNames, jmp_buf env)
{
	if(psSmallSetElementNames != NULL) {
		PDU_ptr->a.searchRequest->smallSetElementSetNames = (struct ElementSetNames *) Asn1Alloc (sizeof(struct ElementSetNames));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->smallSetElementSetNames, env);

		DBVdbv_to_snacc_el_set_names(PDU_ptr->a.searchRequest->smallSetElementSetNames, psSmallSetElementNames, env);
	}
}



int DBVset_snacc_searchreq_msetelname(PDU *PDU_ptr, DbvElementSetNames *psMediumSetElementNames, jmp_buf env)
{
	if(psMediumSetElementNames != NULL) {
		PDU_ptr->a.searchRequest->mediumSetElementSetNames = (struct ElementSetNames*) Asn1Alloc (sizeof(struct ElementSetNames));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->mediumSetElementSetNames, env);

		DBVdbv_to_snacc_el_set_names(PDU_ptr->a.searchRequest->mediumSetElementSetNames, psMediumSetElementNames, env);
	}
}



int DBVset_snacc_searchreq_prefrecsyn(PDU *PDU_ptr, DbvObjectIdentifier pcPreferredRecordSyntax, jmp_buf env)
{

	if(pcPreferredRecordSyntax != NULL) 
		ASN_MakeOid((char *)pcPreferredRecordSyntax, &(PDU_ptr->a.searchRequest->preferredRecordSyntax), env);
}



int DBVset_snacc_searchreq_query(PDU *PDU_ptr, DbvQuery *psQuery, jmp_buf env)
{
	PDU_ptr->a.searchRequest->query= (Query*) Asn1Alloc (sizeof(Query));
	CheckAsn1Alloc(PDU_ptr->a.searchRequest->query, env);

	switch(psQuery->eQueryType) {
	case QT_Private:
		PDU_ptr->a.searchRequest->query->a.type_0 = (AsnOcts *) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_0, env);

		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_0;
		DBVdbv_to_snacc_octet_string(PDU_ptr->a.searchRequest->query->a.type_0, psQuery->uQuery.psPrivateQuery, env);
		break;

	case QT_Rpn:
		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_1;
		PDU_ptr->a.searchRequest->query->a.type_1 = (struct RPNQuery *) Asn1Alloc(sizeof(struct RPNQuery));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_1, env);

		DBVdbv_to_snacc_rpnquery(PDU_ptr->a.searchRequest->query->a.type_1, psQuery->uQuery.psRpnQuery, env);
		break;
	case QT_Iso8777:
		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_2;
		PDU_ptr->a.searchRequest->query->a.type_2 = (AsnOcts *) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_2, env);

		DBVdbv_to_snacc_octet_string(PDU_ptr->a.searchRequest->query->a.type_2, psQuery->uQuery.psIso8777Query, env);
		break;

	case QT_Z3958:
		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_100;
		PDU_ptr->a.searchRequest->query->a.type_100 = (AsnOcts *) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_100, env);

		DBVdbv_to_snacc_octet_string(PDU_ptr->a.searchRequest->query->a.type_100, psQuery->uQuery.psZ3958Query, env);
		break;

	case QT_Erpn:
		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_101;
		PDU_ptr->a.searchRequest->query->a.type_101 = (struct RPNQuery *) Asn1Alloc (sizeof(struct RPNQuery));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_101, env);

		DBVdbv_to_snacc_rpnquery(PDU_ptr->a.searchRequest->query->a.type_101, psQuery->uQuery.psErpnQuery, env);
		break;
	case QT_NaturalLanguage:
		PDU_ptr->a.searchRequest->query->choiceId = QUERY_TYPE_102;
		PDU_ptr->a.searchRequest->query->a.type_102 = (AsnOcts *) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->query->a.type_102, env);

		DBVdbv_to_snacc_octet_string(PDU_ptr->a.searchRequest->query->a.type_102, psQuery->uQuery.psNaturalLanguageQuery, env);
		break;

	default:
		break;
	}
}



int DBVdbv_to_snacc_rpnquery(struct RPNQuery* type_1, DbvRPNQuery *psRpnQuery, jmp_buf env)
{
	/* attributeSet is compulsory */
	ASN_MakeOid((char *) psRpnQuery->pcAttributeSetId, &type_1->attributeSet, env);
	type_1->rpn = (struct RPNStructure*) Asn1Alloc (sizeof(struct RPNStructure));
	CheckAsn1Alloc(type_1->rpn, env);

	DBVdbv_to_snacc_rpnstruct(type_1->rpn, &psRpnQuery->sRpnStructure, env);
}



int DBVdbv_to_snacc_rpnstruct(struct RPNStructure* rpn, DbvRPNStructure *psRpnStructure, jmp_buf env)
{
	switch (psRpnStructure->eRpnStructureType) {
	case RST_RpnLopRopOperator:
		rpn->choiceId = RPNSTRUCTURE_RPNRPNOP;
		rpn->a.rpnRpnOp = (struct RPNStructureSeq*) Asn1Alloc (sizeof(struct RPNStructureSeq));
		CheckAsn1Alloc(rpn->a.rpnRpnOp, env);

		DBVdbv_to_snacc_rpnrpnop(rpn->a.rpnRpnOp, psRpnStructure->uRPNStructure.psRpnLopRopOperator, env);
		break;
	case RST_RpnOperand:
		rpn->choiceId = RPNSTRUCTURE_OP;
		rpn->a.op = (struct Operand*) Asn1Alloc (sizeof(struct Operand));
		CheckAsn1Alloc(rpn->a.op, env);

		DBVdbv_to_snacc_operand(rpn->a.op, psRpnStructure->uRPNStructure.psRpnOperand, env);
		break;
	}
}



int DBVdbv_to_snacc_rpnrpnop(struct RPNStructureSeq* rpnRpnOp, DbvRPNLopRopOperator *psRpnLopRopOperator, jmp_buf env)
{

	/* Compulsory */
	if (psRpnLopRopOperator->psLeftOperand != NULL) {
		rpnRpnOp->rpn1 = (struct RPNStructure*) Asn1Alloc (sizeof(struct RPNStructure));
		CheckAsn1Alloc(rpnRpnOp->rpn1, env);

		DBVdbv_to_snacc_rpnstruct(rpnRpnOp->rpn1, psRpnLopRopOperator->psLeftOperand, env);
	}


	/* Compulsory */
	if (psRpnLopRopOperator->psRightOperand != NULL) {
		rpnRpnOp->rpn2 = (struct RPNStructure*) Asn1Alloc (sizeof(struct RPNStructure));
		CheckAsn1Alloc(rpnRpnOp->rpn2, env);

		DBVdbv_to_snacc_rpnstruct(rpnRpnOp->rpn2, psRpnLopRopOperator->psRightOperand, env);
	}


	/* Compulsory */
	rpnRpnOp->op = (struct Operator*) Asn1Alloc (sizeof(struct Operator));
	CheckAsn1Alloc(rpnRpnOp->op, env);
	DBVdbv_to_snacc_operator(rpnRpnOp->op, psRpnLopRopOperator->sOperator, env);
	
}



int DBVdbv_to_snacc_operator(struct Operator* op, DbvOperator sOperator, jmp_buf env)
{
	switch (sOperator.eOperatorType) {
	case OT_Or:
		op->choiceId = OPERATOR_OR;
		break;
	case OT_And:
		op->choiceId = OPERATOR_AND;
		break;
	case OT_AndNot:
		op->choiceId = OPERATOR_AND_NOT;
		break;
	case OT_Proximity:
		op->choiceId = OPERATOR_PROX;
		op->a.prox = (struct ProximityOperator*) Asn1Alloc (sizeof(ProximityOperator));
		CheckAsn1Alloc(op->a.prox, env);
		DBVdbv_to_snacc_prox(op->a.prox, sOperator.psProximityOperator, env);
		break;
	}
}



int DBVdbv_to_snacc_prox(struct ProximityOperator* prox, DbvProximityOperator *psProximityOperator, jmp_buf env)
{
	if (psProximityOperator->pfExclusion != NULL) {
		prox->exclusion = (AsnBool*) Asn1Alloc (sizeof(AsnBool));
		CheckAsn1Alloc(prox->exclusion, env);

		switch (*psProximityOperator->pfExclusion) {
		case DBV_FALSE:
			*prox->exclusion = FALSE;
			break;
		case DBV_TRUE:
			*prox->exclusion = TRUE;
			break;
		}
	}


	prox->distance = psProximityOperator->ulDistance;

	switch (psProximityOperator->fOrdered) {
	case DBV_FALSE:
		prox->ordered = FALSE;
		break;
	case DBV_TRUE:
		prox->ordered = TRUE;
		break;
	}

	switch (psProximityOperator->eProximityRelationType) {
	case PRT_LessThan:
		prox->relationType = LESSTHAN;
		break;
	case PRT_LessThanOrEqual:
		prox->relationType = LESSTHANOREQUAL;
		break;
	case PRT_Equal:
		prox->relationType = EQUAL;
		break;
	case PRT_GreaterThanOrEqual:
		prox->relationType = GREATERTHANOREQUAL;
		break;
	case PRT_GreaterThan:
		prox->relationType = GREATERTHAN;
		break;
	case PRT_NotEqual:
		prox->relationType = NOTEQUAL;
		break;
	}
	
	prox->proximityUnitCode = (struct ProximityOperatorChoice*) Asn1Alloc (sizeof(struct ProximityOperatorChoice));
	CheckAsn1Alloc(prox->proximityUnitCode, env);

	DBVdbv_to_snacc_proxunitcode(prox->proximityUnitCode, psProximityOperator->sProximityUnitCode, env);
}



int DBVdbv_to_snacc_proxunitcode(struct ProximityOperatorChoice* proximityUnitCode, DbvProximityUnitCode sProximityUnitCode, jmp_buf env)
{
	switch (sProximityUnitCode.eProximityUnitType) {
	case PUT_Known:
		proximityUnitCode->choiceId = PROXIMITYOPERATORCHOICE_KNOWN;
		switch (sProximityUnitCode.uProximityUnit.eKnownProximityUnit) {
		case KPU_Character:
			proximityUnitCode->a.known = CHARACTER;
			break;
		case KPU_Word:
			proximityUnitCode->a.known = WORD;
			break;
		case KPU_Sentence:
			proximityUnitCode->a.known = SENTENCE;
			break;
		case KPU_Paragraph:
			proximityUnitCode->a.known = PARAGRAPH;
			break;
		case KPU_Section:
			proximityUnitCode->a.known = SECTION;
			break;
		case KPU_Chapter:
			proximityUnitCode->a.known = CHAPTER;
			break;
		case KPU_Document:
			proximityUnitCode->a.known = DOCUMENT;
			break;
		case KPU_Element:
			proximityUnitCode->a.known = ELEMENT;
			break;
		case KPU_SubElement:
			proximityUnitCode->a.known = SUBELEMENT;
			break;
		case KPU_ElementType:
			proximityUnitCode->a.known = ELEMENTTYPE;
			break;
		case KPU_Byte:
			proximityUnitCode->a.known = BYTE;
			break;
		}
		break;
	case PUT_Private:
		proximityUnitCode->choiceId = PROXIMITYOPERATORCHOICE_PRIVATE;
		proximityUnitCode->a.private = sProximityUnitCode.uProximityUnit.ulPrivate;
		break;
	}
}



int DBVdbv_to_snacc_operand(struct Operand* op, DbvOperand* psRpnOperand, jmp_buf env)
{
	switch (psRpnOperand->eOperandType) {
	case OT_AttributePlusTerm:
		op->choiceId = OPERAND_ATTRTERM;
		op->a.attrTerm = (struct AttributesPlusTerm*) Asn1Alloc (sizeof(struct AttributesPlusTerm));
		CheckAsn1Alloc(op->a.attrTerm, env);

		DBVdbv_to_snacc_attrterm(op->a.attrTerm, psRpnOperand->uOperand.psAttributesPlusTerm, env);
		break;
	case OT_ResultSetId:
		op->choiceId = OPERAND_RESULTSET;

		/* ResultSetId is a typdefed InternationalString */
		op->a.resultSet = (ResultSetId*) Asn1Alloc (sizeof(ResultSetId));
		CheckAsn1Alloc(op->a.resultSet, env);

		DBVdbv_to_snacc_inter_string(op->a.resultSet, psRpnOperand->uOperand.psResultSetId, env);
		break;
	case OT_ResultSetPlusAttributes:
		op->choiceId = OPERAND_RESULTATTR;
		op->a.resultAttr = (struct ResultSetPlusAttributes*) Asn1Alloc (sizeof(struct ResultSetPlusAttributes));
		CheckAsn1Alloc(op->a.resultAttr, env);

		DBVdbv_to_snacc_resulattr(op->a.resultAttr, psRpnOperand->uOperand.psResultSetPlusAttributes, env);
		break;
	}
}





int DBVdbv_to_snacc_resulattr(struct ResultSetPlusAttributes* resultAttr, DbvResultSetPlusAttributes *psResultSetPlusAttributes, jmp_buf env)
{

	/* Both compulsory */
	DBVdbv_to_snacc_inter_string(&resultAttr->resultSet, &psResultSetPlusAttributes->sResultSetId, env);
	
	/* Attributes list */
	if (psResultSetPlusAttributes->ppsAttributeElement != NULL) {
		resultAttr->attributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(resultAttr->attributes, env);

		DBVdbv_to_snacc_attribute_list(resultAttr->attributes, psResultSetPlusAttributes->ppsAttributeElement, env);
	}

}




int DBVset_snacc_searchreq_addinfo(PDU *PDU_ptr, DbvOtherInformation **ppsAdditionalSearchInfo, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsAdditionalSearchInfo != NULL)  {
		PDU_ptr->a.searchRequest->additionalSearchInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->additionalSearchInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchRequest->additionalSearchInfo, ppsAdditionalSearchInfo, env);
	}
}



int DBVset_snacc_searchreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if psOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.searchRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS SearchResponse PDU RECEIVED FROM TARGET 
*/




int DBVset_dbv_fields_searchresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_SearchConf;
	psOriginData->uOriginData.psSearchRsp = (DbvSearchRsp *) DbvAlloc (sizeof (DbvSearchRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psSearchRsp, env);

	DBVset_dbv_searchresp_refid(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_rescount(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_numrecs(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_nextres(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_srchstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_resstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_presstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_records(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_addinfo(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_otherinfo(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_searchresp_refid(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->referenceId.octetLen != 0){
		searchresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(searchresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(searchresp->psReferenceId, &(PDU_ptr->a.searchResponse->referenceId), env); 
	}
}



int DBVset_dbv_searchresp_rescount(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulResultCount = PDU_ptr->a.searchResponse->resultCount;
}



int DBVset_dbv_searchresp_numrecs(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulNumberOfRecordsReturned = PDU_ptr->a.searchResponse->numberOfRecordsReturned;
}



int DBVset_dbv_searchresp_nextres(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulNextResultSetPosition = PDU_ptr->a.searchResponse->nextResultSetPosition;
}



int DBVset_dbv_searchresp_srchstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->searchStatus == TRUE)
		searchresp->fSearchStatus = DBV_TRUE;
	else
		searchresp->fSearchStatus = DBV_FALSE;
}



int DBVset_dbv_searchresp_resstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->resultSetStatus != NULL) {
		searchresp->peResultSetStatus = (DbvResultSetStatus *) DbvAlloc (sizeof(DbvResultSetStatus));
		CheckDbvAlloc(searchresp->peResultSetStatus, env);

		switch (*PDU_ptr->a.searchResponse->resultSetStatus) {
		case SUBSET:
			*searchresp->peResultSetStatus = RSS_Subset;
			break;
		case INTERIM:
			*searchresp->peResultSetStatus = RSS_Interim;
			break;
		case NONE:
			*searchresp->peResultSetStatus = RSS_None;
			break;
		default:
			*searchresp->peResultSetStatus = RSS_Invalid;
			break;
		}
	}
}



int DBVset_dbv_searchresp_presstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->presentStatus != NULL) {
		searchresp->pePresentStatus = (DbvPresentStatus *) DbvAlloc (sizeof(DbvPresentStatus));
		CheckDbvAlloc(searchresp->pePresentStatus, env);

		/* In search, presentStatus is a pointer */
		DBVsnacc_to_dbv_presstat(searchresp->pePresentStatus, *PDU_ptr->a.searchResponse->presentStatus);
	}
}



int DBVset_dbv_searchresp_records(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->records != NULL) {
		searchresp->psRecords = (DbvRecords *) DbvAlloc (sizeof(struct DbvRecords));
		CheckDbvAlloc(searchresp->psRecords, env);

		DBVsnacc_to_dbv_records(searchresp->psRecords, PDU_ptr->a.searchResponse->records, env);
        }
}



int DBVset_dbv_searchresp_addinfo(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->additionalSearchInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchResponse->additionalSearchInfo);
		searchresp->ppsAdditionalSearchInfo = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchresp->ppsAdditionalSearchInfo, env);

		DBVsnacc_to_dbv_otherinfo(searchresp->ppsAdditionalSearchInfo, PDU_ptr->a.searchResponse->additionalSearchInfo, env);
        }
}



int DBVset_dbv_searchresp_otherinfo(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchResponse->otherInfo);
		searchresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(searchresp->ppsOtherInformation, PDU_ptr->a.searchResponse->otherInfo, env);
        }
}



