/*  Module Name: @(#)origin_z3950_segment.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to 
		 process a received SegmentRequest PDU sent by a target 
		 (a SegmentRequest PDU may be sent from a target 
                 to an origin as part of a response to a
                 SearchRequest/PresentRequest query.
                 There is no SegmentResponse PDU)



*/

#include "origin_z3950_segment.h"


/*****************************************************************
******************************************************************
******************************************************************
PROCESS SegmentRequest PDU RECEIVED FROM TARGET 
*/




int DBVset_dbv_fields_segreq(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_SegmentationInd;
	psOriginData->uOriginData.psSegmentReq = (DbvSegmentReq *) DbvAlloc (sizeof (DbvSegmentReq));
	CheckDbvAlloc(psOriginData->uOriginData.psSegmentReq, env);

	DBVset_dbv_segreq_refid(psOriginData->uOriginData.psSegmentReq, PDU_ptr, env);
	DBVset_dbv_segreq_numrecs(psOriginData->uOriginData.psSegmentReq, PDU_ptr, env);
	DBVset_dbv_segreq_segrecs(psOriginData->uOriginData.psSegmentReq, PDU_ptr, env);
	DBVset_dbv_segreq_otherinfo(psOriginData->uOriginData.psSegmentReq, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);
}



int DBVset_dbv_segreq_refid(DbvSegmentReq *segreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.segmentRequest->referenceId.octetLen != 0){
		segreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(segreq->psReferenceId , env);

		DBVsnacc_to_dbv_octet_string(segreq->psReferenceId, &(PDU_ptr->a.segmentRequest->referenceId), env); 
	}
}



int DBVset_dbv_segreq_numrecs(DbvSegmentReq *segreq, PDU *PDU_ptr, jmp_buf env)
{
	segreq->ulNumberOfRecordsReturned = PDU_ptr->a.segmentRequest->numberOfRecordsReturned;
}



int DBVset_dbv_segreq_segrecs(DbvSegmentReq *segreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	num_elements = AsnListCount(PDU_ptr->a.segmentRequest->segmentRecords);
	segreq->ppsSegmentRecords = (DbvNamePlusRecord **) DbvAlloc (sizeof(DbvNamePlusRecord *) * (num_elements + 1));
	CheckDbvAlloc(segreq->ppsSegmentRecords, env);

	DBVsnacc_to_dbv_resprec(segreq->ppsSegmentRecords, PDU_ptr->a.segmentRequest->segmentRecords, env);
}





int DBVset_dbv_segreq_otherinfo(DbvSegmentReq *segreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.segmentRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.segmentRequest->otherInfo);
		segreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(segreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(segreq->ppsOtherInformation, PDU_ptr->a.segmentRequest->otherInfo, env);
        }
}


