/*  Module Name: @(#)origin_z3950_sort.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a SortRequest PDU 
		 to a target and process a received SortResponse PDU 
		 sent by a target 

*/

#include "origin_z3950_sort.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE SortRequest PDU TO SEND TO SERVER AND SEND IT
*/


int DbvSortRequest(DbvAssocId lAssocId, DbvSortReq *psSortReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVSortRequest",lAssocId);

	if (DBVCheckDbvSortReqFields(lAssocId, psSortReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_SORTREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSortReqTrace(lAssocId, psSortReq);

		PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
			PDU_ptr->a.sortRequest = (struct SortRequest *) Asn1Alloc(sizeof(struct SortRequest));
			CheckAsn1Alloc(PDU_ptr->a.sortRequest, env);
			PDU_ptr->choiceId = PDU_SORTREQUEST;
			DBVsetup_snacc_sortrequest(PDU_ptr, psSortReq, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSortRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvSortReqFields(DbvAssocId lAssocId, DbvSortReq *psSortReq, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_sortreq_inresname(lAssocId, psSortReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_sortreq_sequence(lAssocId, psSortReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_sortreq_otherinfo(lAssocId, psSortReq, ppsSIError))
                return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_sortreq_inresname(DbvAssocId lAssocId, DbvSortReq *psSortReq, DbvError **ppsSIError)
{
        /* Cant be NULL because compulsory */
        if (psSortReq->ppsInputResultSetName == NULL)
                return(DBVdbv_field_missing_error(lAssocId, "psSortReq->ppsInputResultSetName", ppsSIError));
 
	return SI_OK;
}



int DBVcheck_dbv_sortreq_sequence(DbvAssocId lAssocId, DbvSortReq *psSortReq, DbvError **ppsSIError)
{
        int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        /* Cant be NULL because compulsory */
        if (psSortReq->ppsSortSequence == NULL)
                return(DBVdbv_field_missing_error(lAssocId, "psSortReq->ppsSortSequence", ppsSIError));
 
 
	DBVsa_strcpy(local_error_buffer, "psSortReq->");

        while (psSortReq->ppsSortSequence[index] != NULL) {
	 
                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsSortSequence[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_sortkeyspec(lAssocId, psSortReq->ppsSortSequence[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
                index++;
        }

	return SI_OK;
}



int DBVcheck_sortkeyspec(DbvAssocId lAssocId, DbvSortKeySpec *psSortSequence, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sSortElement.");
	if (DBVcheck_sortelmnt(lAssocId, &psSortSequence->sSortElement, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "eSortRelationType");
	if (DBVcheck_dbv_enum(psSortSequence->eSortRelationType, SRT_Ascending, SRT_FrequencyDesc) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, temp_error_buffer, ppsSIError));
	

	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "eSortCaseType");
	if (DBVcheck_dbv_enum(psSortSequence->eSortCaseType, SCT_CaseSensitive, SCT_CaseInsensitive) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, temp_error_buffer, ppsSIError));

	
	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "psMVActionType->");
	if (DBVcheck_sortmvaction(lAssocId, psSortSequence->psMVActionType, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_sortelmnt(DbvAssocId lAssocId, DbvSortElement *psSortElement, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
        switch (psSortElement->eSortElementType) {
        case SET_Generic:
                DBVsa_strcat(local_error_buffer, "uSortElement.psGeneric");
        	if (psSortElement->uSortElement.psGeneric == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
                return(DBVcheck_dbv_sortkey(lAssocId, psSortElement->uSortElement.psGeneric, local_error_buffer, ppsSIError));
                break;
        case SET_DbSpecific:

		DBVsa_strcat(local_error_buffer, "uSortElement.");

		/* if no list nodes */
        	if (psSortElement->uSortElement.ppsDatabaseSpecific == NULL) {
                	DBVsa_strcat(local_error_buffer, "ppsDatabaseSpecific");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
        	}

                return(DBVcheck_dbspecsortkey(lAssocId, psSortElement->uSortElement.ppsDatabaseSpecific, local_error_buffer, ppsSIError));
                break;
        default:
                DBVsa_strcat(local_error_buffer, "eSortElementType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }
 
	
	return SI_OK;
}



int DBVcheck_dbv_sortkey(DbvAssocId lAssocId, DbvSortKey *psGeneric, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
        switch (psGeneric->eSortKeyType) {
        case SKT_SortField:
                DBVsa_strcat(local_error_buffer, "uSortElement.psSortField");

		if (psGeneric->uSortElement.psSortField == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        case SKT_ElementSpec:
                DBVsa_strcat(local_error_buffer, "uSortElement.psElementSpec");

		if (psGeneric->uSortElement.psElementSpec == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		
		DBVsa_strcat(local_error_buffer, "->");
                return(DBVcheck_DbvSpecification(lAssocId, psGeneric->uSortElement.psElementSpec, local_error_buffer, ppsSIError));
                break;
        case SKT_SortAttributes:
                DBVsa_strcat(local_error_buffer, "uSortElement.psSortAttributes");

		if (psGeneric->uSortElement.psSortAttributes == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		
		DBVsa_strcat(local_error_buffer, "->");
                return(DBVcheck_sortatts(lAssocId, psGeneric->uSortElement.psSortAttributes, local_error_buffer, ppsSIError));
                break;

        default:
                strcat(local_error_buffer, "eSortKeyType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }
	return SI_OK;
}



int DBVcheck_sortatts(DbvAssocId lAssocId, DbvSortAttributes *psSortAttributes, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
	if (DBVcheck_sortattsetid(lAssocId, psSortAttributes->pcAttributeSetId, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_sortattelmnt(lAssocId, psSortAttributes->ppsAttributeElement, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;
}




int DBVcheck_sortattsetid(DbvAssocId lAssocId, DbvObjectIdentifier pcAttributeSetId, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "->pcAttributeSetId");

	if (pcAttributeSetId == NULL) 
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

 
        if (DBVcheck_dbv_attsetoid(pcAttributeSetId) == SI_NOTOK) 
                return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));

	return SI_OK;

}



int DBVcheck_sortattelmnt(DbvAssocId lAssocId, DbvAttributeElement **ppsAttributeElement, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        /* Cant be NULL because compulsory */
        if (ppsAttributeElement == NULL)
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
 
 
        while (ppsAttributeElement[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsAttributeElement[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
		
                if (DBVcheck_dbv_attelement(lAssocId, ppsAttributeElement[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
                index++;
        }

	return SI_OK;
}



int DBVcheck_dbspecsortkey(DbvAssocId lAssocId, DbvDbSpecificSortKey **ppsDatabaseSpecific, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 

	/* This field is compulsory */


        while (ppsDatabaseSpecific[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsDatabaseSpecific[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_specsortkey(lAssocId, ppsDatabaseSpecific[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
                index++;
        }
	return SI_OK;
}



int DBVcheck_specsortkey(DbvAssocId lAssocId, DbvDbSpecificSortKey *psDatabaseSpecific, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	DBVsa_strcat(local_error_buffer, "sDbSort.");
	return(DBVcheck_dbv_sortkey(lAssocId, &psDatabaseSpecific->sDbSort, local_error_buffer, ppsSIError));
}



int DBVcheck_sortmvaction(DbvAssocId lAssocId, DbvSortMVAction *psMVActionType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* Can be NULL cos optional */
	if (psMVActionType == NULL)
		return SI_OK;

        switch (psMVActionType->eSortMVActionType) {
        case SMT_Abort:
        case SMT_Null:
        case SMT_Constant:
                break;

        default:
                DBVsa_strcat(local_error_buffer, "eSortMVActionType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }
	return SI_OK;
}



int DBVcheck_dbv_sortreq_otherinfo(DbvAssocId lAssocId, DbvSortReq *psSortReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psSortReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psSortReq->ppsOtherInformation, "psSortReq->ppsOtherInformation", ppsSIError));

}




int DBVsetup_snacc_sortrequest(PDU *PDU_ptr, DbvSortReq *psSortReq, jmp_buf env)
{
	DBVset_snacc_sortreq_refid(PDU_ptr, psSortReq->psReferenceId, env);
	DBVset_snacc_sortreq_inresset(PDU_ptr, psSortReq->ppsInputResultSetName, env);
	DBVset_snacc_sortreq_sortedres(PDU_ptr, psSortReq->sSortedResultSetName, env);
	DBVset_snacc_sortreq_sortseq(PDU_ptr, psSortReq->ppsSortSequence, env);
	DBVset_snacc_sortreq_otherinfo(PDU_ptr, psSortReq->ppsOtherInformation, env);
}



int DBVset_snacc_sortreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.sortRequest->referenceId), psReferenceId, env);
	}

}



int DBVset_snacc_sortreq_inresset(PDU *PDU_ptr, DbvInternationalString **ppsInputResultSetName, jmp_buf env)
{
	/* Compulsory */

	InternationalString *data_ptr;
	InternationalString **data_ptr_address;
 
        int current_listnode = 0;
 
        PDU_ptr->a.sortRequest->inputResultSetNames = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.sortRequest->inputResultSetNames, env);
 
        while(ppsInputResultSetName[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(PDU_ptr->a.sortRequest->inputResultSetNames);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(InternationalString));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_inter_string(data_ptr, ppsInputResultSetName[current_listnode], env);
                current_listnode++;
        }
}



int DBVset_snacc_sortreq_sortedres(PDU *PDU_ptr, DbvInternationalString sSortedResultSetName, jmp_buf env)
{
	/* Compulsory */
	DBVdbv_to_snacc_inter_string(&PDU_ptr->a.sortRequest->sortedResultSetName, &sSortedResultSetName, env);
}



int DBVset_snacc_sortreq_sortseq(PDU *PDU_ptr, DbvSortKeySpec **ppsSortSequence, jmp_buf env)
{
	/* Compulsory */
	SortKeySpec *data_ptr;
	SortKeySpec **data_ptr_address;
 
        int current_listnode = 0;
 
        PDU_ptr->a.sortRequest->sortSequence = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.sortRequest->sortSequence, env);
 
        while(ppsSortSequence[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(PDU_ptr->a.sortRequest->sortSequence);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(SortKeySpec));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_sortkeyspec(data_ptr, ppsSortSequence[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_sortkeyspec(SortKeySpec *data_ptr, DbvSortKeySpec *psSortSequence, jmp_buf env)
{
	/* All compulsory except last */
	data_ptr->sortElement = (struct SortElement*) Asn1Alloc (sizeof(struct SortElement));
	CheckAsn1Alloc(data_ptr->sortElement, env);

	DBVdbv_to_snacc_sortelmnt(data_ptr->sortElement, &psSortSequence->sSortElement, env);
	
	DBVdbv_to_snacc_sortrel(data_ptr, psSortSequence, env);
	DBVdbv_to_snacc_sortcase(data_ptr, psSortSequence, env);

	if (psSortSequence->psMVActionType != NULL) {
		data_ptr->missingValueAction = (struct SortKeySpecChoice*) Asn1Alloc (sizeof(struct SortKeySpecChoice));
		CheckAsn1Alloc(data_ptr->missingValueAction, env);
		DBVdbv_to_snacc_sortmv(data_ptr->missingValueAction, psSortSequence->psMVActionType, env);
	}
}



int DBVdbv_to_snacc_sortelmnt(struct SortElement* sortElement, DbvSortElement *psSortElement, jmp_buf env)
{
	switch (psSortElement->eSortElementType) {
	case SET_Generic:
		sortElement->choiceId = SORTELEMENT_GENERIC;
		sortElement->a.generic = (struct SortKey*) Asn1Alloc (sizeof(struct SortKey));
		CheckAsn1Alloc(sortElement->a.generic, env);

		DBVdbv_to_snacc_sortkey(sortElement->a.generic, psSortElement->uSortElement.psGeneric, env);
		break;
	case SET_DbSpecific:
		sortElement->choiceId = SORTELEMENT_DATBASESPECIFIC;
		sortElement->a.datbaseSpecific = AsnListNew(sizeof(void *)); 
		CheckAsn1Alloc(sortElement->a.datbaseSpecific, env);

		DBVdbv_to_snacc_dbspecific(sortElement->a.datbaseSpecific, psSortElement->uSortElement.ppsDatabaseSpecific, env);
		break;
	}
	
}



int DBVdbv_to_snacc_sortkey(struct SortKey* generic, DbvSortKey *psGeneric, jmp_buf env)
{
	switch (psGeneric->eSortKeyType) {
	case SKT_SortField:
		generic->choiceId = SORTKEY_SORTFIELD;
		generic->a.sortfield = (InternationalString *) Asn1Alloc(sizeof (InternationalString));
		CheckAsn1Alloc(generic->a.sortfield, env);

		DBVdbv_to_snacc_inter_string(generic->a.sortfield, psGeneric->uSortElement.psSortField, env); 
		break;
	case SKT_ElementSpec:
		generic->choiceId = SORTKEY_ELEMENTSPEC;
		generic->a.elementSpec = (struct Specification *) Asn1Alloc(sizeof (struct Specification));
		CheckAsn1Alloc(generic->a.elementSpec, env);

		DBVdbv_to_snacc_specification(generic->a.elementSpec, psGeneric->uSortElement.psElementSpec, env);
		break;
	case SKT_SortAttributes:
		generic->choiceId = SORTKEY_SORTATTRIBUTES;
		generic->a.sortAttributes = (struct SortKeySeq *) Asn1Alloc (sizeof(struct SortKeySeq));
		CheckAsn1Alloc(generic->a.sortAttributes, env);

		DBVdbv_to_snacc_sortatts(generic->a.sortAttributes, psGeneric->uSortElement.psSortAttributes, env);
		break;
	}
}



int DBVdbv_to_snacc_sortatts(struct SortKeySeq* sortAttributes, DbvSortAttributes *psSortAttributes, jmp_buf env)
{
	/* Both compulsory */

	ASN_MakeOid((char *) psSortAttributes->pcAttributeSetId, &sortAttributes->id, env);

	sortAttributes->list = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(sortAttributes->list, env);

	DBVdbv_to_snacc_attribute_list(sortAttributes->list, psSortAttributes->ppsAttributeElement, env);

}



int DBVdbv_to_snacc_dbspecific(SortElementSeqOf *datbaseSpecific, DbvDbSpecificSortKey **ppsDatabaseSpecific, jmp_buf env)
{
	SortElementSeqOfSeq *data_ptr;
	SortElementSeqOfSeq **data_ptr_address;
 
        int current_listnode = 0;
 
        while(ppsDatabaseSpecific[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(datbaseSpecific);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(SortElementSeqOfSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_specsortkey(data_ptr, ppsDatabaseSpecific[current_listnode], env);
                current_listnode++;
        }
}

	

int DBVdbv_to_snacc_specsortkey(SortElementSeqOfSeq *data_ptr, DbvDbSpecificSortKey *psDatabaseSpecific, jmp_buf env)
{
	DBVdbv_to_snacc_inter_string(&data_ptr->databaseName, &psDatabaseSpecific->sDatabaseName, env);

	data_ptr->dbSort = (struct SortKey*) Asn1Alloc (sizeof(struct SortKey));
	CheckAsn1Alloc(data_ptr->dbSort, env);

	DBVdbv_to_snacc_sortkey(data_ptr->dbSort, &psDatabaseSpecific->sDbSort, env);
	
}



int DBVdbv_to_snacc_sortrel(SortKeySpec *data_ptr, DbvSortKeySpec *psSortSequence, jmp_buf env)
{
	/* Compulsory field */
	switch (psSortSequence->eSortRelationType) {
	case SRT_Ascending:
		data_ptr->sortRelation = ASCENDING;
		break;
	case SRT_Descending:
		data_ptr->sortRelation = DESCENDING;
		break;
	case SRT_FrequencyAsc:
		data_ptr->sortRelation = ASCENDINGBYFREQUENCY;
		break;
	case SRT_FrequencyDesc:
		data_ptr->sortRelation = DESCENDINGBYFREQUENCY;
		break;
	}
}



int DBVdbv_to_snacc_sortcase(SortKeySpec *data_ptr, DbvSortKeySpec *psSortSequence, jmp_buf env)
{
	/* Compulsory field */
	switch (psSortSequence->eSortCaseType) {
	case SCT_CaseSensitive:
		data_ptr->caseSensitivity = CASESENSITIVE;
		break;
	case SCT_CaseInsensitive:
		data_ptr->caseSensitivity = CASEINSENSITIVE;
		break;
	}
}



int DBVdbv_to_snacc_sortmv(struct SortKeySpecChoice* missingValueAction, DbvSortMVAction *psMVActionType, jmp_buf env)
{
	switch (psMVActionType->eSortMVActionType) {
	case SMT_Abort:
		missingValueAction->choiceId = SORTKEYSPECCHOICE_ABORT;
		break;
	case SMT_Null:
		missingValueAction->choiceId = SORTKEYSPECCHOICE_NULL;
		break;
	case SMT_Constant:
		missingValueAction->choiceId = SORTKEYSPECCHOICE_MISSINGVALUEDATA;
		missingValueAction->a.missingValueData = (AsnOcts *) Asn1Alloc (sizeof (AsnOcts));
		CheckAsn1Alloc(missingValueAction->a.missingValueData, env);
		DBVdbv_to_snacc_octet_string(missingValueAction->a.missingValueData, psMVActionType->psOctetString, env);
		break;
	}
}



int DBVset_snacc_sortreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.sortRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.sortRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.sortRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS SortResponse PDU RECEIVED FROM TARGET 

*/




int DBVset_dbv_fields_sortresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_SortConf;
	psOriginData->uOriginData.psSortRsp = (DbvSortRsp *) DbvAlloc (sizeof (DbvSortRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psSortRsp, env);

	DBVset_dbv_sortresp_refid(psOriginData->uOriginData.psSortRsp, PDU_ptr, env);
	DBVset_dbv_sortresp_sortstat(psOriginData->uOriginData.psSortRsp, PDU_ptr, env);
	DBVset_dbv_sortresp_resstat(psOriginData->uOriginData.psSortRsp, PDU_ptr, env);
	DBVset_dbv_sortresp_diag(psOriginData->uOriginData.psSortRsp, PDU_ptr, env);
	DBVset_dbv_sortresp_otherinfo(psOriginData->uOriginData.psSortRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_sortresp_refid(DbvSortRsp *sortresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.sortResponse->referenceId.octetLen != 0){
		sortresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(sortresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(sortresp->psReferenceId, &(PDU_ptr->a.sortResponse->referenceId), env); 
	}
}



int DBVset_dbv_sortresp_sortstat(DbvSortRsp *sortresp, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.sortResponse->sortStatus) {
	case SUCCESS2:
		sortresp->eSortStatus = SOS_Success;
		break;
	case PARTIAL_11:
		sortresp->eSortStatus = SOS_Partial_1;
		break;
	case FAILURE2:
		sortresp->eSortStatus = SOS_Failure;
		break;
	}
}



int DBVset_dbv_sortresp_resstat(DbvSortRsp *sortresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.sortResponse->resultSetStatus != NULL) {
		sortresp->peResultSetStatus = (DbvSortResultSetStatus *) DbvAlloc (sizeof (DbvSortResultSetStatus));
		CheckDbvAlloc(sortresp->peResultSetStatus, env);

		switch (*PDU_ptr->a.sortResponse->resultSetStatus) {
		case EMPTY:
			*sortresp->peResultSetStatus = SRSS_Empty;
			break;
		case INTERIM1:
			*sortresp->peResultSetStatus = SRSS_Interim;
			break;
		case UNCHANGED:
			*sortresp->peResultSetStatus = SRSS_Unchanged;
			break;
		case NONE1:
			*sortresp->peResultSetStatus = SRSS_None;
			break;
		defualt:
			*sortresp->peResultSetStatus = SRSS_Invalid;
			break;
		}
	}
}



int DBVset_dbv_sortresp_diag(DbvSortRsp *sortresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	if (PDU_ptr->a.sortResponse->diagnostics != NULL) {
		num_elements = AsnListCount (PDU_ptr->a.sortResponse->diagnostics);
		sortresp->ppsDiagnostics = (DbvDiagRec **) DbvAlloc (sizeof(DbvDiagRec *) * (num_elements +1));
		CheckDbvAlloc(sortresp->ppsDiagnostics, env);

		DBVsnacc_to_dbv_multdiag(sortresp->ppsDiagnostics, PDU_ptr->a.sortResponse->diagnostics, env);
	}
}



int DBVset_dbv_sortresp_otherinfo(DbvSortRsp *sortresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.sortResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.sortResponse->otherInfo);
		sortresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(sortresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(sortresp->ppsOtherInformation, PDU_ptr->a.sortResponse->otherInfo, env);
        }
}




