/*  Module Name: @(#)origin_z3950_trigger.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Origin end routines to create and send a 
		  TriggerResourceControlRequest PDU to a target.
		  There's no TriggerResourceControlResponse.
		  

*/


#include "origin_z3950_trigger.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE TriggerResourceControlRequest PDU TO SEND TO TARGET AND SEND IT 
*/


int DbvTriggerResourceControlRequest(DbvAssocId lAssocId, DbvTriggerResourceControlReq *psTriggerResourceControlReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVTriggerResourceControlResponse", lAssocId);

        if (DBVCheckDbvTriggerResourceControlRspFields(lAssocId, psTriggerResourceControlReq, ppsSIError) == SI_OK)

		if (DBVChangeState(lAssocId, DBV_OE_TRIGRESCNTREQ, ppsSIError) == SI_OK)

        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK) {
		DBVSendingTrace(lAssocId);
                DBVTrigResCntrlReqTrace(lAssocId, psTriggerResourceControlReq);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.triggerResourceControlRequest = (struct TriggerResourceControlRequest *) Asn1Alloc (sizeof(struct TriggerResourceControlRequest));
			CheckAsn1Alloc(PDU_ptr->a.triggerResourceControlRequest, env);
        		PDU_ptr->choiceId = PDU_TRIGGERRESOURCECONTROLREQUEST;
			DBVsetup_snacc_TriggerResourceControlRequest(PDU_ptr, psTriggerResourceControlReq, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVTriggerResourceControlResponse", lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvTriggerResourceControlRspFields(DbvAssocId lAssocId, DbvTriggerResourceControlReq *psTriggerResourceControlReq, DbvError **ppsSIError)
{

	/* o */
	/* Dont have to check psReferenceId */

	/* c */
	if (DBVcheck_DbvRequestedAction(lAssocId, &psTriggerResourceControlReq->eRequestedAction, "psTriggerResourceControlReq->eRequestedAction", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
	if (DBVcheck_DbvObjectIdentifier_opt(lAssocId, psTriggerResourceControlReq->pcPreferredResourceReportFormat, "psTriggerResourceControlReq->pcPreferredResourceReportFormat", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvBool_opt(lAssocId, psTriggerResourceControlReq->pfResultSetWanted, "*psTriggerResourceControlReq->pfResultSetWanted", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvOtherInformation_opt(lAssocId, psTriggerResourceControlReq->ppsOtherInformation, "psTriggerResourceControlReq->ppsOtherInformation", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvRequestedAction(DbvAssocId lAssocId, DbvRequestedAction *peRequestedAction, char *error_buffer, DbvError **ppsSIError)
{
	/* c */ 
	if (DBVcheck_dbv_enum(*peRequestedAction, RA_ResourceReport1, RA_Cancel) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, error_buffer, ppsSIError));


	return SI_OK;

}





/* snacc set up */

int DBVsetup_snacc_TriggerResourceControlRequest(PDU *PDU_ptr, DbvTriggerResourceControlReq *psTriggerResourceControlReq, jmp_buf env)
{
	/* o */
	DBVset_TriggerResourceControlRequest_referenceId(PDU_ptr, psTriggerResourceControlReq->psReferenceId, env);

	/* c */
        DBVset_TriggerResourceControlRequest_requestedAction(PDU_ptr, &psTriggerResourceControlReq->eRequestedAction, env);

	/* o */
        DBVset_TriggerResourceControlRequest_prefResourceReportFormat(PDU_ptr, psTriggerResourceControlReq->pcPreferredResourceReportFormat, env);

	/* o */
        DBVset_TriggerResourceControlRequest_resultSetWanted(PDU_ptr, psTriggerResourceControlReq->pfResultSetWanted, env);


	/* o */
	DBVset_TriggerResourceControlRequest_otherinfo(PDU_ptr, psTriggerResourceControlReq->ppsOtherInformation, env);
}



int DBVset_TriggerResourceControlRequest_referenceId(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvOctetString_opt(&PDU_ptr->a.triggerResourceControlRequest->referenceId, psReferenceId, env);
	
}





int DBVset_TriggerResourceControlRequest_requestedAction(PDU *PDU_ptr, DbvRequestedAction *peRequestedAction, jmp_buf env)
{
	/* c */
	switch (*peRequestedAction) {
	case RA_ResourceReport1:
		PDU_ptr->a.triggerResourceControlRequest->requestedAction = RESOURCEREPORT1;
		break;
	
	case RA_ResourceControl:
		PDU_ptr->a.triggerResourceControlRequest->requestedAction = RESOURCECONTROL;
		break;

	case RA_Cancel:
		PDU_ptr->a.triggerResourceControlRequest->requestedAction = CANCEL;
		break;

	}
}



int DBVset_TriggerResourceControlRequest_prefResourceReportFormat(PDU *PDU_ptr, DbvObjectIdentifier pcPreferredResourceReportFormat, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_objectident_opt(&PDU_ptr->a.triggerResourceControlRequest->prefResourceReportFormat, pcPreferredResourceReportFormat, env);
}



int DBVset_TriggerResourceControlRequest_resultSetWanted(PDU *PDU_ptr, DbvBool *pfResultSetWanted, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvBool_opt(&PDU_ptr->a.triggerResourceControlRequest->resultSetWanted, pfResultSetWanted, env);
}



int DBVset_TriggerResourceControlRequest_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	DBVdbv_to_snacc_otherinfo_opt(&PDU_ptr->a.triggerResourceControlRequest->otherInfo, ppsOtherInformation, env);

}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS TriggerResourceControlResponse PDU RECEIVED FROM TARGET 
*/


/* THERE AINT SUCH A PDU !! */





