/*  Module Name: @(#)osi-funcs.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: DBV-OSI II OSI specific utility functions
 
*/

#ifndef NO_OSI

#include "osi-funcs.h"

/* This function checks errors returned by ISODE at the presentation level. The 
	state-machine is queried for release requests and Abort requests and 
	the error structure filled out according to the error.
	NOTE: The return value from this function will be 
	(DBV_RELEASE_REQUEST+reason) if a release request is received and it 
	does not invalidate the state machine */
int DBVPOSICheckISODEError(	DbvAssocId lAssocId, long lOSIAssocId, 
			 	struct PSAPindication *psPresentationIndication,
				DbvError **ppsError)
{
	int return_value=SI_OK;

	struct AcSAPindication *psAssocControlIndication=NULL;
	struct AcSAPabort *psAssocControlAbort=NULL;
	struct PSAPabort *psPresentationAbort=
		&psPresentationIndication->pi_abort;

	DBVEnterFuncTrace("DBVPOSICheckISODEError",lAssocId);

	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort=&psAssocControlIndication->aci_abort;

	/* Check for Abort */
	if(psPresentationIndication->pi_type == PI_ABORT)
	{
		if(psPresentationAbort->pa_reason == PC_ABORTED)
		{
		  AcABORTser(lOSIAssocId, psPresentationAbort, 
			psAssocControlIndication);
		  if(psAssocControlIndication->aci_abort.aca_reason == 
			ACS_ABORTED)
		  {
			DBVfill_error_structure(lAssocId, SI_Abort, 
				DBVACSEUserOrProvider(psAssocControlAbort->
				aca_source), DBV_ABORT, DBV_ABORT_S, 0, 
				Association_Closed, ppsError);
			return_value = SI_NOTOK;
		  }
		  else
		  {
			DBVTraceISODEACSEError(lAssocId, 
				"DBVPOSICheckISODEError()", 
				psAssocControlAbort);
			DBVfill_error_structure(lAssocId, SI_Abort, 
				DBVACSEUserOrProvider(psAssocControlAbort->
				aca_source), DBV_ASSOCDEAD, DBV_ASSOCDEAD_S, 0, 
				Association_Closed, ppsError);
			return_value = SI_NOTOK;
		  }
		}
		else
		{
			/* An error has occurred on the association */
			DBVTraceISODEPError(lAssocId, 
				"DBVPOSICheckISODEError()", 
				psPresentationAbort);
			DBVfill_error_structure(lAssocId, SI_Abort, 
				DBVPUserOrProvider(psPresentationAbort->pa_peer)
				,DBV_ASSOCDEAD, DBV_ASSOCDEAD_S, 0, 
				Association_Closed, ppsError);
			return_value = SI_NOTOK;
		}
	}
	else if(psPresentationIndication->pi_type == PI_FINISH)
	{
		if(AcFINISHser(lOSIAssocId,
			&psPresentationIndication->
			pi_un.pi_un_finish,
			psAssocControlIndication)
			==NOTOK)
		{
			sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, "In function DBVPOSICheckISODEError(): ISODE function AcFINISHser() failed\nCould not map a P-RELEASE.INDICATION to A-RELEASE.INDICATION");
			DBVfill_error_structure(lAssocId, SI_Abort,
				DBVACSEUserOrProvider(psAssocControlAbort->
				aca_source), DBV_OSI_ISODE_ERROR, 
				DBVErrorBuffer, 0, Association_Closed,ppsError);
			DBVTraceISODEACSEError(lAssocId, "AcFINISHser()", psAssocControlAbort);
			return_value=SI_NOTOK;
		}
		else
		{
			return_value = DBV_RELEASE_REQUEST+
				psAssocControlIndication->aci_un.aci_un_finish.
				acf_reason;
		}
	}
	else
	{
		/* This should never appear. Eh? */
		sprintf(DBVErrorBuffer, DBV_INTERNAL_S, "In function DBVPOSICheckISODEError(): psPresentationIndication->pi_type was neither PI_FINISH or PI_ABORT");
		DBVfill_error_structure(lAssocId, SI_OperatingSystem_Error, 0, 
			DBV_INTERNAL, DBVErrorBuffer, 0, 
			Association_Unknown_State, ppsError);
		return_value=SI_NOTOK;
	}
	DBVLeaveFuncTrace("DBVPOSICheckISODEError",lAssocId, return_value);

	return return_value;
}

/* This function checks errors returned by ISODE at the ACSE level. The
        state-machine is queried for release requests and Abort requests and
        the error structure filled out according to the error.
        NOTE: The return value from this function will be
        (DBV_RELEASE_REQUEST+reason) if a release request is received and it
        does not invalidate the state machine */
int DBVACSEOSICheckISODEError( DbvAssocId lAssocId, long lOSIAssocId,
			       struct AcSAPindication *psAssocControlIndication,
			       DbvError **ppsError)
{
        int return_value=SI_OK;
 
	struct AcSAPabort *psAssocControlAbort=&psAssocControlIndication->
		aci_abort;

	DBVEnterFuncTrace("DBVACSEOSICheckISODEError",lAssocId);

	/* Check for abort */
	if(psAssocControlIndication->aci_type == ACI_ABORT)
	{
		if(psAssocControlIndication->aci_abort.aca_reason ==
			ACS_ABORTED)
		{
			/* The Association has received an ABORT.INDICATION */
			DBVfill_error_structure(lAssocId, SI_Abort,
				DBVACSEUserOrProvider(psAssocControlAbort->
				aca_source), DBV_ABORT, DBV_ABORT_S, 0,
				Association_Closed, ppsError);
			return_value = SI_NOTOK;
		}
		else
		{
			/* The association has been aborted for some other
				reason */
			DBVTraceISODEACSEError(lAssocId, 
				"DBVACSEOSICheckISODEError()", 
				psAssocControlAbort);
			DBVfill_error_structure(lAssocId, SI_Abort,
				SI_Presentation_Provider, DBV_ASSOCDEAD,
				DBV_ASSOCDEAD_S, 0, Association_Closed,
				ppsError);
			return_value = SI_NOTOK;
		}
	}	
	else if(psAssocControlIndication->aci_type == ACI_FINISH)
	{
		return_value = DBV_RELEASE_REQUEST+
			psAssocControlIndication->aci_un.aci_un_finish.
			acf_reason;
	}
	else
	{
		sprintf(DBVErrorBuffer, DBV_INTERNAL_S, "In function DBVACSEOSICheckISODEError(): psAssocControlIndication->pi_type was neither ACI_FINISH or ACI_ABORT");
		DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, 
			DBV_INTERNAL, DBVErrorBuffer, 0, 
			Association_Unknown_State, ppsError);
		return_value=SI_NOTOK;
	}
	DBVLeaveFuncTrace("DBVACSEOSICheckISODEError",lAssocId, return_value);

	return return_value;
}

DbvAbortSource DBVACSEUserOrProvider(int aca_source)
{
	if(aca_source == 0)
		return SI_ACSE_User;
	else
		return SI_ACSE_Provider;
}

DbvAbortSource DBVPUserOrProvider(int pa_peer)
{
	if(pa_peer != 0)
		return SI_Presentation_User;
	else
		return SI_Presentation_Provider;
}

#endif


/* ANSI C forbids an empty source file */
int dummy_for_tcpip(void)
{
	return 1;
}



