/*  Module Name: @(#)si_utilities.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
	DBV-OSI II Utility functions
 
*/

#include "si_utilities.h"

int SISetACSEFile(char *pcACSEFile, DbvError **ppsError)
{
	int return_value=SI_OK;
	FILE *psFd;
	
	if((psFd=fopen(pcACSEFile, "r"))==NULL)
	{
		/* Can't open file */
		sprintf(DBVErrorBuffer, DBV_FILE_S, pcACSEFile);
		DBVfill_error_structure(0, SI_Interface_Error, 0, DBV_FILE,
			DBVErrorBuffer,0,Association_Closed, ppsError);
		return_value=SI_NOTOK;
	}
	else
	{
		fclose(psFd);
		strcpy(DBVACSEFileName, pcACSEFile);
	}

	return return_value;
}

/* Note: writes to trace file only on errors for specific associations */
int SIGetEventLocation( DbvAssocId **pplAssocId, DbvSecs lSeconds,
                        DbvFds **ppulFileDescriptor,
                        DbvEventLocation **ppsEventLocation, DbvError **ppsError)
{
	int count, seconds, time_up, incoming;
	int select_result, osi_select_result;
	int return_value = SI_OK;
	struct timeval time_out;
	time_t start_time, current_time;
	/* AssocId	**pplOSIAssocs; For separating the OSI associations when they're implemented */
	fd_set read_ready,set_read_ready, osi_read_ready, osi_set_read_ready;
	int osi_nfds=0;
	DbvProtocolStack eProtocolStack;
	DBVAssociateDetails *psAssociateDetails;

#ifndef NO_OSI
	/* ISODE */
	struct PSAPindication *psPresentationIndication=NULL;
#endif

	time_up = incoming = FALSE;
	seconds = count = 0;

#ifndef NO_OSI
	if(DBVAllocCheck((psPresentationIndication=(struct PSAPindication *)
		DBVAlloc(sizeof(struct PSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
#endif


	if(pplAssocId == NULL)
	{
		/* Error - No file descriptors given at all! */
		/* Fill out error structure */
		sprintf(DBVErrorBuffer,DBV_INVALIDPARAMS_S, "SIGetEventLocation: pplAssocId must be NULL terminated list");
		DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_INVALIDPARAMS, DBVErrorBuffer,0,Association_Closed, ppsError);
		return SI_NOTOK;
	}

	if(ppulFileDescriptor == NULL)
	{
		/* Error - No file descriptors given at all! */
		/* Fill out error structure */
		sprintf(DBVErrorBuffer,DBV_INVALIDPARAMS_S, "SIGetEventLocation: ppulFileDescriptor must be NULL terminated list");
		DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_INVALIDPARAMS, DBVErrorBuffer,0,Association_Closed, ppsError);
		return SI_NOTOK;
	}


/* Make sure that there are some Associations to check for */
	if((*pplAssocId == NULL) && (*ppulFileDescriptor == NULL))
	{
		/* Error - No file descriptors given at all! */
		/* Fill out error structure */

		sprintf(DBVErrorBuffer,DBV_INVALIDPARAMS_S, "SIGetEventLocation: Association list and File Descriptor list cannot both be empty");
		DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_INVALIDPARAMS, DBVErrorBuffer,0,Association_Unknown_State, ppsError);

		return SI_NOTOK;
	}
/*
	if(*pplAssocId != NULL && DBVAssociateList == NULL)
	{
		** There are no valid associations to check **
		DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_NOASSOC, DBV_NOASSOC_S,0,Association_Closed, ppsError);
		return SI_NOTOK;
	}
*/
/* Run through the Associations and make sure that they are valid ...*/

	/* Set the timeout for tcp/ip (1/100th sec). This is so that we can 
		alternate checking between TCP and OSI associations */
	time_out.tv_usec = 10000;
	time_out.tv_sec = 0;

	if(lSeconds == CALL_ASYNCHRONOUS)
		seconds = 0;
	if(lSeconds > 0)
		seconds = lSeconds;

	count = 0;
	/* Set the TCP/OSI associations and file descriptors */
	FD_ZERO(&read_ready);
	FD_ZERO(&set_read_ready);
	FD_ZERO(&osi_read_ready);
	FD_ZERO(&osi_set_read_ready);
	while(return_value == SI_OK && pplAssocId[count] != NULL)
	{
		if(SIWhichStack(*pplAssocId[count],&eProtocolStack, ppsError)
			== SI_OK)
		{
			/* Write some trace information for a specific 
				association */
			if(DBVTraceLevel(*pplAssocId[count]) != Trace_Off)
        		{
                		sprintf(DBVTraceBuffer,"Inside function SIGetEventLocation\n");
                		DBVTrace(*pplAssocId[count], Trace_Low,
					DBVTraceBuffer);
        		}

			psAssociateDetails = SL_Current(DBVAssociateList);
			if(eProtocolStack == TCP_Stack)
			{	
				FD_SET(psAssociateDetails->ulTcpSocket, 
					&read_ready);
			}
			if(eProtocolStack == OSI_Stack)
			{
#ifndef NO_OSI
				if(PSelectMask(psAssociateDetails->lOSIAssocId,
					&osi_read_ready, &osi_nfds, 
					psPresentationIndication)!=OK)
				{
				     DBVPOSICheckISODEError(*pplAssocId[count],
					    psAssociateDetails->lOSIAssocId,
					    psPresentationIndication,
					    ppsError);
					return_value = SI_NOTOK;
				}
#else
				printf("SIGetEventLocation: OSI not supported\n");
#endif /* NO_OSI */
			}
		}
		else
			/* Error structure filled out by SIWhichStack */
			return_value = SI_NOTOK;
		count++;
	}
	count = 0;
	if(return_value == SI_OK)
	{
		while(ppulFileDescriptor[count] != NULL)
		{
			FD_SET(*ppulFileDescriptor[count], &read_ready);
			count++;
		};

		set_read_ready = read_ready;
		osi_set_read_ready = osi_read_ready;
        	start_time = time(NULL);
	}
	while(return_value == SI_OK && !time_up && !incoming)
	{
		/* Do the select */
		select_result = select(FD_SETSIZE, &read_ready, 
			(fd_set *)0, (fd_set *)0, &time_out);
		if(select_result < 0)
		{
			/* Error in select */
			/* Fill error structure */
			if(errno == EINTR)
				DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_INTERRUPT, DBV_INTERRUPT_S,0,Association_Open, ppsError);
			else
			{
				sprintf(DBVErrorBuffer,DBV_SYSERROR_S, "select()", sys_errlist[errno]);
				DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_SYSERROR, DBVErrorBuffer,0,Association_Unknown_State, ppsError);
			}
			return_value = SI_NOTOK;
				
		}
		/* select equivalent for OSI associations */
		/* Poll's assoc descriptors because we cannot do a timeout of
			less than a second */
#ifndef NO_OSI
		if(return_value == SI_OK)
		{
			osi_select_result = xselect(osi_nfds, &osi_read_ready, 
				(fd_set *)0, (fd_set *)0, 0);
			if(select_result < 0)
			{
				/* error in xselect */
				printf("error in xselect!!!\n");
				return_value = SI_NOTOK;
			}
			if(osi_select_result > 0 && select_result == 0)
				select_result = osi_select_result;
		}
#endif

		if(select_result > 0)
		{
			incoming = TRUE;
		}
		if(select_result == 0)
		{
			read_ready = set_read_ready;
			osi_read_ready = osi_set_read_ready;
			fflush(stdout);
		}
		current_time = time(NULL);
		if(lSeconds != CALL_BLOCKING)
                	if (current_time >= (start_time + seconds)) 
			{
                        	time_up = TRUE;
			}
		/* Have to reset read_ready each time */
	}
	if(return_value == SI_OK && incoming == TRUE)
	{
		*ppsEventLocation = (struct DbvEventLocation *)
			DbvAlloc(sizeof(struct DbvEventLocation ));

		/* At least (only ever?) one fd is ready for reading */
		DBVfill_out_event_location_structure(pplAssocId, 
			ppulFileDescriptor, &read_ready, &osi_read_ready, 
			ppsEventLocation);
	}

	return return_value;
}

int SIWhichStack(       DbvAssocId lAssocId, DbvProtocolStack *peProtocolStack,
                        DbvError **ppsError)
{
	int return_value = SI_OK;
	DBVAssociateDetails *psAssociateDetails;

	DBVEnterFuncTrace("SIWhichStack",lAssocId);

	if(DBVAssociateList == (struct SList *)NULL)
	{
		/* There are no associations */
		/* Fill out error structure */
		sprintf(DBVErrorBuffer,DBV_INVALIDASSOC_S, lAssocId);
		DBVfill_error_structure(lAssocId, SI_Communication_Error, 0, 
			DBV_INVALIDASSOC, DBVErrorBuffer, 0, 
			Association_Closed, ppsError);
		return_value = SI_NOTOK;
	}
	else
	{
		psAssociateDetails = SL_Locate(DBVAssociateList, 
			(SL_LocateFunc)DBVLocateAssociate, &lAssocId);
		if( psAssociateDetails == NULL)
		{
			/* Association was not in list! */
			sprintf(DBVErrorBuffer,DBV_INVALIDASSOC_S, lAssocId);
			DBVfill_error_structure(lAssocId, 
				SI_Communication_Error, 0, DBV_INVALIDASSOC, 
				DBVErrorBuffer, 0, Association_Closed, 
				ppsError);
			return SI_NOTOK;
		}
		else
		{
			if(psAssociateDetails->eOsiOrInternetAddress == 
				Internet_Address)
			{
				*peProtocolStack = TCP_Stack;
			}
			else
			{
				if(psAssociateDetails->eOsiOrInternetAddress 
					== Osi_Address)
				{
					*peProtocolStack = OSI_Stack;
				}
				else
				{
					/* Association has unknown stack ! */
					sprintf(DBVErrorBuffer,
						DBV_INVALIDSTYPE_S, lAssocId);
					DBVfill_error_structure(lAssocId, 
						SI_OperatingSystem_Error, 0, 
						DBV_INVALIDSTYPE, 
						DBVErrorBuffer, 0, 
						Association_Closed, ppsError);
					return_value = SI_NOTOK;
				}
			}
	
		}
	}
	if(return_value == SI_OK)
	{
		DBVTrace(lAssocId, Trace_High, "\nOutput Function Arguments\n");
		Trace_DbvProtocolStack(peProtocolStack, lAssocId);
	}
	DBVLeaveFuncTrace("SIWhichStack",lAssocId, return_value);

	return return_value;
}

/* Set a trace parameter for the given association */
int SISetTraceParameter(   DbvAssocId lAssocId, FILE *psTraceFp, 
                        DbvTraceLevel eTraceLevel, DbvError **ppsError)
{
	int return_value = SI_OK;
	int new_trace = FALSE;
	DbvProtocolStack eProtocolStack;
	DBVTraceDetails *psTraceInformation;
	time_t the_time;
	struct tm *time_struct;

	/* Create a list if one doesn't exist */
        if(DBVTraceList == (struct SList *)NULL)
                DBVTraceList = SL_Create( (SL_Free)NULL );

	/* See if the Association is already in the list */
	psTraceInformation = SL_Locate(DBVTraceList, 
		(SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
	if(psTraceInformation == NULL)
	{
		/* Associate is not in trace list so we can't close file */
		new_trace = TRUE;
	}

	if(new_trace == TRUE)
	{
		/* Allocate memory for new Trace */
		psTraceInformation = (DBVTraceDetails *)
			DbvAlloc(sizeof(DBVTraceDetails));
	
		psTraceInformation->DBVTraceLevel = eTraceLevel;
		psTraceInformation->DBVTraceFile = psTraceFp;
		psTraceInformation->lAssocId = lAssocId;
	}
	else
	{
		psTraceInformation->DBVTraceFile = psTraceFp;
		psTraceInformation->DBVTraceLevel = eTraceLevel;
	}

	the_time = time(NULL);
	time_struct = localtime(&the_time);
	/* Is this Association valid? */
        if(SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
	{
	if((psTraceInformation->DBVTraceLevel == Trace_Off) && 
		(new_trace == FALSE))
	{
		DBVWrite("Ending current trace.\n",psTraceInformation->
			DBVTraceFile);
		strftime(DBVTraceBuffer, 128, "Ended at:%T %t%d/%m/%Y %n %n", 
			time_struct);
		DBVWrite(DBVTraceBuffer,psTraceInformation->DBVTraceFile);
		psTraceInformation = SL_RemoveCurrent(DBVTraceList);
		if(psTraceInformation != NULL)
			free(psTraceInformation);
	}
	else
	{
	if((psTraceInformation->DBVTraceLevel == Trace_Off) &&
		(new_trace == TRUE))
	{
		sprintf(DBVErrorBuffer,DBV_INVALIDPARAMS_S, "SISetTraceParameter: There is no trace file for this Association, so it cannot be closed");
		DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_INVALIDPARAMS, DBVErrorBuffer,0,Association_Closed, ppsError);
		return_value = SI_NOTOK;
	}
	else
	{
		if(new_trace == TRUE)
			SL_Add(DBVTraceList, psTraceInformation);
		DBVWrite("DBV-OSI II API Trace Information\n", 
			psTraceInformation->DBVTraceFile);
		DBVWrite("--------------------------------\n",
			psTraceInformation->DBVTraceFile);
		sprintf(DBVTraceBuffer,"\nBeginning trace for Association:%d\n",
			psTraceInformation->lAssocId);
		DBVWrite(DBVTraceBuffer,psTraceInformation->DBVTraceFile);
		strftime(DBVTraceBuffer, 128, "Started at %T %t%d/%m/%Y %n %n",
			time_struct);
		DBVWrite(DBVTraceBuffer,psTraceInformation->DBVTraceFile);
		if(eProtocolStack == TCP_Stack)
			DBVWrite("Association uses stack: TCP_Stack\n",
				psTraceInformation->DBVTraceFile);
		if(eProtocolStack == OSI_Stack)
			DBVWrite("Association uses stack: OSI_Stack\n",
				psTraceInformation->DBVTraceFile);
		DBVWrite("\nBeginning Trace at Level: ", psTraceInformation->
			DBVTraceFile);
		if(psTraceInformation->DBVTraceLevel == Trace_Low)
			DBVWrite("Trace_Low\n",psTraceInformation->DBVTraceFile);
		if(psTraceInformation->DBVTraceLevel == Trace_High)
			DBVWrite("Trace_High\n",psTraceInformation->DBVTraceFile);
	}
	}
	}
	else
	{
		/* Error structure already filled by SIWhichStack */
		return_value = SI_NOTOK;
	}

	if(return_value == SI_NOTOK && new_trace == TRUE)
	{
		/* Trace was new and SIWhichStack returned NOTOK */
		free(psTraceInformation);
	}
	 
	return return_value;
}

void DBVWrite(char *data, FILE *fd)
{
/* Write it, then fflush it straight away */
	fputs(data, fd);
	fflush(fd);
}

DbvTraceLevel DBVTraceLevel(DbvAssocId lAssocId)
{
	DBVTraceDetails *psTraceInformation;

        if(DBVTraceList == (struct SList *)NULL)
                return Trace_Off;

	/* Get the Association from the list of configured trace files*/
	psTraceInformation = SL_Locate(DBVTraceList, 
		(SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
	if(psTraceInformation == NULL)
		return Trace_Off;

	return psTraceInformation->DBVTraceLevel;
}

int DBVTrace(DbvAssocId lAssocId, int level, char *trace)
{
	DBVTraceDetails *psTraceInformation;

        if(DBVTraceList == (struct SList *)NULL)
                return TRUE;

	/* Get the Association from the list of configured trace files*/
	psTraceInformation = SL_Locate(DBVTraceList, 
		(SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
	if(psTraceInformation == NULL)
		return TRUE;
	if(psTraceInformation->DBVTraceLevel == Trace_Off)
		return TRUE;
	if((psTraceInformation->DBVTraceLevel == Trace_Low) && 
		(level == Trace_High))
		return TRUE;

	DBVWrite(trace, psTraceInformation->DBVTraceFile);

	return TRUE;
}

void DBVErrorTrace(DbvAssocId lAssocId, DbvError **ppsError, char *pcDiagnostic)
{
	DbvError *psError;

	psError = *ppsError;
	if(DBVTraceLevel(lAssocId) != Trace_Off)
	{
		DBVTrace(lAssocId, Trace_Low, "An error has occured, Error structure is:\n");
                DBVTrace(lAssocId, Trace_Low, "eErrorClass == ");
                if(psError->eErrorClass == SI_Communication_Error)
                        DBVTrace(lAssocId, Trace_Low, "SI_Communication_Error\n");
                if(psError->eErrorClass == SI_Interface_Error)
                        DBVTrace(lAssocId, Trace_Low, "SI_Interface_Error\n");
                if(psError->eErrorClass == SI_OperatingSystem_Error)
                        DBVTrace(lAssocId, Trace_Low, "SI_OperatingSystem_Error\n");
                if(psError->eErrorClass == SI_Abort)
                {
                        DBVTrace(lAssocId, Trace_Low, "SI_Abort\n");
                        sprintf(DBVTraceBuffer, "uErrorCode.psAbort->eAbortSource == %d\n",psError->uErrorCode.psAbort->eAbortSource);
                        DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);
                        sprintf(DBVTraceBuffer, "uErrorCode.psAbort->lAbortReason == %d\n",psError->uErrorCode.psAbort->lAbortReason);
                        DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);
                        sprintf(DBVTraceBuffer, "uErrorCode.psAbort->pcDiagnostics == %s\n", psError->uErrorCode.psAbort->pcDiagnostics);
                        DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);
                }
		else
		{
			sprintf(DBVTraceBuffer, "uErrorCode.lIfOsCoError == %d\n", psError->uErrorCode.lIfOsCoError);
                        DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);

			sprintf(DBVTraceBuffer, "Diagnostic: %s\n", pcDiagnostic);
                        DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);


		}
                DBVTrace(lAssocId, Trace_Low, "eAssociationState == ");
		if(psError->eAssociationState == Association_Open)
                       	DBVTrace(lAssocId, Trace_Low, "Association_Open\n");
		if(psError->eAssociationState == Association_Closed)
                       	DBVTrace(lAssocId, Trace_Low, "Association_Closed\n");
		if(psError->eAssociationState == Association_Unknown_State)
                       	DBVTrace(lAssocId, Trace_Low, "Association_Unknown_State\n");
        }
}

#ifndef NO_OSI
/* Store error reported from ISODE to the respective trace file/log file.
	This function deals with errors at the ACSE level. */
void DBVTraceISODEACSEError(DbvAssocId lAssocId, char *pcFunction, 
	struct AcSAPabort *psAssocControlAbort)
{
	if(lAssocId == 0 || DBVTraceLevel(lAssocId) == Trace_Off)
		DBVLogISODEACSEError(pcFunction, psAssocControlAbort);
	else
	{
		DBVTrace(lAssocId, Trace_Low, "ISODE function ");
		DBVTrace(lAssocId, Trace_Low, pcFunction);
		DBVTrace(lAssocId, Trace_Low, 
			" reported the following ACSE Level error:\n");
		DBVTrace(lAssocId, Trace_Low, "aca_source==");
		switch(psAssocControlAbort->aca_source)
		{
		case ACA_USER:
			DBVTrace(lAssocId, Trace_Low, 
				"ACA_USER, A-U-ABORT.INDICATION\n");
			break;
		case ACA_PROVIDER:
			DBVTrace(lAssocId, Trace_Low, 
				"ACA_PROVIDER, A-P-ABORT.INDICATION\n");
			break;
		case ACA_LOCAL:
			DBVTrace(lAssocId, Trace_Low, 
				"ACA_LOCAL, UNOFFICIAL ERROR\n");
			break;
		default:
			DBVTrace(lAssocId, Trace_Low, "Unknown type!\n");
			break;
		}
		sprintf(DBVTraceBuffer,"aca_reason==%d\n",psAssocControlAbort->
			aca_reason);
		DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);
		DBVTrace(lAssocId, Trace_Low, "aca_data==");
		DBVTrace(lAssocId, Trace_Low, psAssocControlAbort->aca_data);
		DBVTrace(lAssocId, Trace_Low, "\n");
		DBVTrace(lAssocId, Trace_Low, "\nAcErrorString==");
		DBVTrace(lAssocId, Trace_Low, AcErrString(psAssocControlAbort->
			aca_reason));
		DBVTrace(lAssocId, Trace_Low, "\n");

	}
}

/* Store error reported from ISODE to the respective trace file/log file.
	This function deals with errors at the Presentation level. */
void DBVTraceISODEPError(DbvAssocId lAssocId, char *pcFunction, 
	struct PSAPabort *psPresentationAbort)
{
	if(lAssocId == 0 || DBVTraceLevel(lAssocId) == Trace_Off)
		DBVLogISODEACSEError(pcFunction, psPresentationAbort);
	else
	{
		DBVTrace(lAssocId, Trace_Low, "ISODE function ");
		DBVTrace(lAssocId, Trace_Low, pcFunction);
		DBVTrace(lAssocId, Trace_Low, 
			" reported the following ACSE Level error:\n");
		DBVTrace(lAssocId, Trace_Low, "pa_peer==");
		if(psPresentationAbort->pa_peer == 0)
			DBVTrace(lAssocId, Trace_Low, 
				"0, P-P-ABORT.INDICATION\n");
		else
			DBVTrace(lAssocId, Trace_Low, 
				"TRUE, P-U-ABORT.INDICATION\n");
		sprintf(DBVTraceBuffer,"pa_reason==%d\n",psPresentationAbort->
			pa_reason);
		DBVTrace(lAssocId, Trace_Low, DBVTraceBuffer);
		DBVTrace(lAssocId, Trace_Low, "pa_data==");
		DBVTrace(lAssocId, Trace_Low, psPresentationAbort->pa_data);
		DBVTrace(lAssocId, Trace_Low, "\n");
		DBVTrace(lAssocId, Trace_Low, "AcErrorString==");
		DBVTrace(lAssocId, Trace_Low, AcErrString(psPresentationAbort->
			pa_reason));
		DBVTrace(lAssocId, Trace_Low, "\n");

	}
}
#endif

void DBVEnterFuncTrace(char *function_name, DbvAssocId lAssocId)
{
	if(DBVTraceLevel(lAssocId) != Trace_Off)
        {
                sprintf(DBVTraceBuffer,"DBV-OSI II Trace File Association: %d\n",lAssocId);
                DBVTrace(lAssocId, Trace_Low,DBVTraceBuffer);
                sprintf(DBVTraceBuffer,"**Entering function:%s\n",function_name);
                DBVTrace(lAssocId, Trace_Low,DBVTraceBuffer);
        }
}

void DBVLeaveFuncTrace(char *function_name, DbvAssocId lAssocId, int return_value)
{
        if(DBVTraceLevel(lAssocId) != Trace_Off)
        {
                sprintf(DBVTraceBuffer,"DBV-OSI II Trace File Association: %d\n",lAssocId);
                DBVTrace(lAssocId, Trace_Low,DBVTraceBuffer);
                sprintf(DBVTraceBuffer,"**Leaving function:%s\n",function_name);
		if(return_value == SI_OK)
			strcat(DBVTraceBuffer,"Function returned: SI_OK\n\n");
		if(return_value == SI_NOTOK)
			strcat(DBVTraceBuffer,"Function returned: SI_NOTOK\n\n");
                DBVTrace(lAssocId, Trace_Low,DBVTraceBuffer);
        }
}

void DBVfill_out_event_location_structure( DbvAssocId **pplAssocId, 
					   DbvFds **ppulFileDescriptor, 
					   fd_set *pread_ready, 
					   fd_set *posi_read_ready, 
					   DbvEventLocation **ppsEventLocation )

{
	int count, Eventcount;
	DbvEventLocation *psLocalEventLocation = *ppsEventLocation;
	DBVAssociateDetails *psAssociateDetails;

	count = Eventcount = 0;

	/* Malloc enough memory for all the file descriptors */
	if(*pplAssocId != NULL)
	{
	do{
		psAssociateDetails = SL_Locate(DBVAssociateList,
			(SL_LocateFunc)DBVLocateAssociate, pplAssocId[count]);
		if(psAssociateDetails->eOsiOrInternetAddress==Internet_Address)
		{
			if(FD_ISSET(psAssociateDetails->ulTcpSocket, 
			  pread_ready))
				Eventcount++;
		}
#ifndef NO_OSI
		if(psAssociateDetails->eOsiOrInternetAddress==Osi_Address)
		{
			if(FD_ISSET(psAssociateDetails->lOSIAssocId, 
			  posi_read_ready))
				Eventcount++;
		}
#endif
		count++;
	}
	while(pplAssocId[count] != NULL);
	}
	Eventcount++; /* Room for closing NULL */
	psLocalEventLocation->pplProviderEvent = (DbvAssocId **)
		DbvAlloc(Eventcount * sizeof(DbvAssocId *));
	count = Eventcount = 0;

	/* Check the users file descriptors */
	if(*ppulFileDescriptor != NULL)
	{
	do{
		if( FD_ISSET(*ppulFileDescriptor[count], pread_ready))
			Eventcount++;
		count++;
	}
	while(ppulFileDescriptor[count] != NULL);
	}
	Eventcount++; /* Room for closing NULL */
	psLocalEventLocation->ppulUserEvent = (DbvFds **)
		DbvAlloc(Eventcount * sizeof(DbvFds *));

	/* Run through each file descriptor and see if it's been set */
	count = 0; Eventcount = 0;
	if(*pplAssocId != NULL)
	{
	/* Fill out the EventLocation structure */
	do{
		psAssociateDetails = SL_Locate(DBVAssociateList,
			(SL_LocateFunc)DBVLocateAssociate, pplAssocId[count]);

		/* TCP/IP association check - both could be compressed into
			one big if, I suppose */
		if((psAssociateDetails->eOsiOrInternetAddress==Internet_Address)		  && (FD_ISSET(psAssociateDetails->ulTcpSocket, pread_ready)))
		{
			/* Add to ppsEventLocation->pplProviderEvent */
			psLocalEventLocation->pplProviderEvent[Eventcount] = 
				(DbvAssocId *)DbvAlloc(sizeof(DbvAssocId));
			*(psLocalEventLocation->pplProviderEvent[Eventcount])=
				*pplAssocId[count];
			Eventcount++;
		}
#ifndef NO_OSI
		/* OSI asssociation check */
		if((psAssociateDetails->eOsiOrInternetAddress==Osi_Address) && 
		  (FD_ISSET(psAssociateDetails->lOSIAssocId, posi_read_ready)))
		{
			/* Add to ppsEventLocation->pplProviderEvent */
			psLocalEventLocation->pplProviderEvent[Eventcount] =
				(DbvAssocId *)DbvAlloc(sizeof(DbvAssocId));
			*(psLocalEventLocation->pplProviderEvent[Eventcount])=
				*pplAssocId[count];
			Eventcount++;
		}
#endif
		count++;
	}
	while(pplAssocId[count] != NULL);
	}
	/* End the list with a NULL */
	psLocalEventLocation->pplProviderEvent[Eventcount] = 
		(DbvAssocId *)NULL;

	count = Eventcount = 0;
	if(*ppulFileDescriptor != NULL)
	{
	do{
		if( FD_ISSET(*ppulFileDescriptor[count], pread_ready))
		{
			/* Add to ppsEventLocation->ppulUserEvent */
			psLocalEventLocation->ppulUserEvent[Eventcount] =
				(unsigned long *)DbvAlloc(sizeof(unsigned long));
			*(psLocalEventLocation->ppulUserEvent[Eventcount]) =
					*ppulFileDescriptor[count];
			Eventcount++;
		}
		count++;
	}
	while(ppulFileDescriptor[count] != NULL);
	}
	/* End the list with a 0x0 */
	psLocalEventLocation->ppulUserEvent[Eventcount] = 
		NULL;

}

