/*  Module Name: @(#)sl_list.c	1.4
    Version: 9/19/96
    Author: M. Bush
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue: 

    QA Acceptance:  
    Date of Acceptance:  

    Description:  
		  Singly Linked List source code.
*/

#include <stdlib.h>
#include <stdio.h>
#include "memcheck.h"
#include "sl_list.h"


typedef
struct ListElement *ListElementPtr;

struct ListElement {
	void	    *Value;
	ListElementPtr  Succ;
};

struct SList {
	ListElementPtr  Head;
	ListElementPtr  Tail;
	ListElementPtr  Current;
	ListElementPtr  CurrentHold;
	SL_Free	    Dispose;
	Bool_	    Assign;
};

Local_ void   sl_Scrap(ListElementPtr ThisElement, SL_Free Dispose);
Local_ Bool_  OneElementIn(SL List);

/*
 * Returns True if there is one and only one item in the list, otherwise 
 * returns False.
 */

Local_ Bool_   OneElementIn(SL List)
{
	if ((List->Head == List->Tail) && (List->Head != NULL)) {
		return True;
	} else {
		return False;
	}
}

/*
 * Returns True if the list is empty, otherwise returns False.
 */

Bool_  SL_EmptyList(SL List)
{
	if ((List->Head == NULL)) {
		return True;
	} else {
		return False;
	}
}

/*
 * Returns a pointer to a singly linked list (SL) object. The Dispose argument is a 
 * pointer to a function which will deallocate memory used to store items stored
 * in the list.
 */


SL SL_Create(SL_Free Dispose)
{
	SL  List;

	List = SMALLOC_(struct SList , "SL");
	List->Head =    NULL;
	List->Tail =    NULL;
	List->Current = NULL;
	List->Dispose = Dispose;
	return List;
}


/*
 * De-allocates memory used by the list object and memory used to store items on the 
 * list.
 */

void SL_Destruct(SL List)
{
/*	SL_RemoveDuplicates( List );  */
	if ( List->Dispose != NULL) {
		sl_Scrap(List->Head, List->Dispose);
	}
	FREE_(List);
}


/*
 * Called by SL_Destruct, it de-allocates the memory allocated to one item in the list.
 */

Local_ void sl_Scrap(ListElementPtr ThisElement, SL_Free Dispose)
{
	ListElementPtr NextElement;

	if (ThisElement != NULL) {
		NextElement = ThisElement->Succ;
		if( Dispose != NULL ) {
			Dispose(ThisElement->Value);
		}
		FREE_(ThisElement);
		sl_Scrap(NextElement, Dispose);
	};
}


/*
 * Removes the first item from the list and returns a pointer to this item.
 */

void* SL_Remove(SL List)
{
	ListElementPtr  LElement;
	void * 	Value;

	if (SL_EmptyList(List)) {
		return NULL;
	} else {
		if (OneElementIn(List)) {
			List->Tail = NULL;
		}
		LElement = List->Head;
		Value = LElement->Value;
		List->Head = LElement->Succ;
		FREE_(LElement);
		return Value;
	};
}


/*
 * Removes the current item from the list and returns a pointer to this item.
 * Current item becomes the previous item to the one deleted in the list.
 * If deleted item is at the head of the list, new
 * head of the list  becomes the current item.
 */

void* SL_RemoveCurrent(SL List)
{
	ListElementPtr  LElement;
	void * 	Value;

	if (SL_EmptyList(List)) {
		return NULL;
	} else if ( OneElementIn(List) == True ) {
		LElement = List->Current;
		List->Head =    NULL;
		List->Tail =    NULL;
		List->Current = NULL;
	} else if ( List->Current == List->Tail ) {
		LElement = List->Current;
		SL_Prev( List );
		List->Current->Succ = NULL;
		List->Tail = List->Current;
	} else if ( List->Current == List->Head ) {
		LElement = List->Current;
		List->Head = List->Current->Succ;
		List->Current = List->Head;
	} else {
		LElement = List->Current;
		SL_Prev( List );
		List->Current->Succ = LElement->Succ;
	}
	Value = LElement->Value;
	FREE_(LElement);
	return Value;
}

/*
 * Removes the current item from the list and returns a pointer to this item.
 * Current item becomes the suceeding item to the one deleted in the list.
 * If deleted item is at the tail of the list, new
 * tail of the list  becomes the current item.
 */

void* SL_RemoveCurrentN(SL List)
{
	ListElementPtr  LElement;
	void * 	Value;

	if (SL_EmptyList(List)) {
		return NULL;
	} else if ( OneElementIn(List) == True ) {
		LElement = List->Current;
		List->Head =    NULL;
		List->Tail =    NULL;
		List->Current = NULL;
	} else if ( List->Current == List->Tail ) {
		LElement = List->Current;
		SL_Prev( List );
		List->Current->Succ = NULL;
		List->Tail = List->Current;
	} else if ( List->Current == List->Head ) {
		LElement = List->Current;
		List->Head = List->Current->Succ;
		List->Current = List->Head;
	} else {
		LElement = List->Current;
		SL_Prev( List );
		List->Current->Succ = LElement->Succ;
		SL_Succ( List );
	}
	Value = LElement->Value;
	FREE_(LElement);
	return Value;
}


/*
 * Makes the last item in the list the current item.
 */

void SL_End(SL List)
{
	List->Current = List->Tail;
}


/*
 * Makes the first item in the list the current item.
 */

void SL_Start(SL List)
{
	List->Current = List->Head;
}


/*
 * Returns 'True' if the current item is the first in the list. Otherwise returns 
 * False.
 */

Bool_ SL_IsStart(SL List)
{
	if (List->Current == List->Head) {
		return True;
	} else {
		return False;
	}
}


/*
 * Returns 'True' if the current item is the last in the list. Otherwise returns False.
 */

Bool_ SL_IsEnd(SL List)
{
	if (List->Current == List->Tail) {
		return True;
	} else {
		return False;
	}
}


/*
 * Returns a pointer to the current item in the list.
 */

void* SL_Current(SL List)
{
	if (List->Current == NULL) {
		return NULL;
	} else {
		return List->Current->Value;
	}
}


/*
 * Makes the next item the current item in the list.
 */

void SL_Succ(SL List)
{
	if( SL_IsEnd( List ) == True ) {
		return;
	}
	if (List->Current == NULL) {
		SError_(Fatal_, "Impossible program route", "SL");
	} else {
		List->Current = List->Current->Succ;
	}
}


/*
 * Makes the previous item the current item in the list.
 */

void SL_Prev(SL List)
{
	ListElementPtr  Compare;

	if( SL_IsStart( List ) == True ) {
		return;
	}
	Compare = List->Current;
	List->Current = List->Head;
	while ( List->Current->Succ != Compare ) {
		List->Current = List->Current->Succ;
	}
}


/*
 * Deletes the head of the list.
 */

void SL_DeleteHead(SL List)
{
	ListElementPtr  LElement;

	if (SL_EmptyList(List) == False) {
		if (OneElementIn(List)) {
			List->Dispose(List->Current->Value);
			FREE_(List->Current);
			List->Current = NULL;
			List->Tail = NULL;
			List->Head = NULL;
		} else {
			if (SL_IsStart(List)) {
				List->Current = List->Current->Succ;
			}
			LElement = List->Head;
			List->Head = List->Head->Succ;
			List->Dispose(LElement->Value);
			FREE_(LElement);
		}
	}
}


/*
 * Adds an item to the list at current position. Current is added item.
 */

void SL_Add(SL List, void* Item)
{
	ListElementPtr LElement;

	LElement = SMALLOC_(struct ListElement , "SL");
	if (SL_EmptyList(List)) {
		List->Tail = LElement;
		List->Head = LElement;
		LElement->Succ = NULL;
	} else {
		LElement->Succ = List->Current->Succ;
		List->Current->Succ = LElement;
		if (List->Current == List->Tail) {
			List->Tail = LElement;
		}
	}
	List->Current = LElement;
	LElement->Value = Item;
}

/*
 * Adds an item to the list before the current item. Item added becomes the current item.
 */

void SL_AddBefore(SL List, void* Item)
{
	ListElementPtr LElement;

	if (SL_EmptyList(List)) {
		LElement = SMALLOC_(struct ListElement , "SL");
		List->Tail = LElement;
		List->Head = LElement;
		LElement->Succ = NULL;
		List->Current = LElement;
		LElement->Value = Item;
	} else if( SL_IsStart( List ) == False ) {
		SL_Prev( List );
		SL_Add( List, Item );
	} else {
		LElement = SMALLOC_(struct ListElement , "SL");
		LElement->Succ = List->Current;
		List->Head = LElement;
		List->Current = LElement;
		LElement->Value = Item;
	}
}

/*
 * Sets a boolean flag on the object. Used to mark that the List has been assigned
 * and should not be destroyed.
 */

void SL_Assign( SL List, Bool_ Assign )
{
	List->Assign = Assign;
}


/*
 * Returns the last boolean setting made on the call of SL_Assign. If SL_Assign 
 * has not been called then returns False;
 */

Bool_ SL_IsAssign( SL List )
{
	return List->Assign;
}


/*
 * Returns an item in the list the calling routine requires. A pointer to the location 
 * function 'LocateFunc' along with a pointer to a comparison item 'Item' of the type 
 * stored in the list are passed as arguments. Thus, as an example, if it was required 
 * to locate a gui_WindowHand structure with identifier number 2, then the 'Item'
 * passed would be a WINDOWHAND with the setting Item->Id = 2. The locate
 * function would be as follows :
 *
 * SL_LocateFunc gui_WindowHand_Locate( WINDOWHAND Window, WINDOWHAND WindowLoc ) {
 *     return ( Window->Id == WindowLoc->Id );
 * }
 */

void*	SL_Locate( SL List, SL_LocateFunc LocateFunc, void*Item )
{
	void * 	ItemRet;
	Bool_	First;


	if( List == NULL ) {
		return NULL;
	}
	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsEnd ( List ) != True );
	return NULL;
}


/* Same as SL_Locate though two arguments rather than one are used in location */

void*	SL_Locate2( SL List, SL_LocateFunc2 LocateFunc, void*Item1, void*Item2 )
{
	void * 	ItemRet;
	Bool_	First;


	if( List == NULL ) {
		return NULL;
	}
	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item1, Item2, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsEnd ( List ) != True );
	return NULL;
}

/* Locates the next item in the list which satisfies the given search criteria */

void*	SL_LocateNext( SL List, SL_LocateFunc LocateFunc, void*Item )
{
	void * 	ItemRet;
	Bool_	First;


	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	if(SL_IsEnd(List) == True) {
		return NULL;
	}
	do {
		SL_Succ( List );
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsEnd ( List ) != True );
	return NULL;
}

/* Locates the next item in the list which satisfies the given search criteria */


void*	SL_LocateNext2( SL List, SL_LocateFunc2 LocateFunc, void*Item1, void*Item2 )
{
	void * 	ItemRet;
	Bool_	First;


	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	if(SL_IsEnd(List) == True) {
		return NULL;
	}
	do {
		SL_Succ( List );
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item1, Item2, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsEnd ( List ) != True );
	return NULL;
}



/* Locates the next item back in the list which satisfies the given search criteria */

void*	SL_LocateNextBack( SL List, SL_LocateFunc LocateFunc, void*Item )
{
	void * 	ItemRet;
	Bool_	First;


	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	if(SL_IsStart(List) == True) {
		return NULL;
	}
	do {
		SL_Prev( List );
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsStart ( List ) != True );
	return NULL;
}

/* Locates the next item back in the list which satisfies the given search criteria */

void*	SL_LocateNextBack2( SL List, SL_LocateFunc2 LocateFunc, void*Item1, void*Item2 )
{
	void * 	ItemRet;
	Bool_	First;


	First = True;
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	if(SL_IsStart(List) == True) {
		return NULL;
	}
	do {
		SL_Prev( List );
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return NULL;
		if ( (*LocateFunc)(Item1, Item2, ItemRet) == True ) 
			return ItemRet;
	} while ( SL_IsStart ( List ) != True );
	return NULL;
}

/* Copies the list of objects from 'List' into 'ListRet'. Uses same list items if 'CopyFunc' is NULL or else uses 'CopyFunc' to reproduce list items ( i.e. allocate memory ) for a new similar set of list items */

void	SL_Copy( SL List, SL_CopyFunc CopyFunc, SL ListRet )
{
	void * 	Item;
	void * 	ItemRet;
	Bool_	First;

	First = True;
	if(SL_EmptyList(List) == True) {
		return;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Item = SL_Current( List );
		if ( Item == NULL ) 
			return;
		if( CopyFunc != NULL ) {
			ItemRet = (*CopyFunc)(Item);
			SL_Add( ListRet, ItemRet );
		} else {
			SL_Add( ListRet, Item );
		}
	} while ( SL_IsEnd ( List ) != True );
}



/*
 * Performs the function 'TraverseFunc' on every item in the list.
 */

void  SL_Traverse( SL List, SL_TraverseFunc TraverseFunc )
{
	void * 	Item;
	Bool_	First;

	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) {
		return;
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Item = SL_Current( List );
		if ( Item == NULL )
			break;
		(*TraverseFunc)(Item);
	} while ( SL_IsEnd ( List ) != True );
}



/*
 * Performs the function 'TraverseFunc' on every item in the list. Allows up
 * to two items to be included in the traverse function
 */


void  SL_Traverse3( SL List, SL_TraverseFunc3 TraverseFunc, void*Item1, void*Item2, void*Item3 )
{
	void * 	Item;
	Bool_	First;

	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) {
		return;
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Item = SL_Current( List );
		if ( Item == NULL )
			break;
		(*TraverseFunc)( Item1, Item2, Item3, Item );
	} while ( SL_IsEnd ( List ) != True );
}




/* Adds AddList to the end of List */

void	SL_AddList( SL List, SL AddList, Bool_ DestructAddList )
{
	void * 	Item;
	Bool_	First;

	if( AddList == NULL ) {
		return;
	}
	if(SL_EmptyList(AddList) == True) {
  		if( DestructAddList == True ) {
			SL_Destruct( AddList );
		}
		return;
	}
	if(SL_EmptyList(List) == False) {
		SL_End( List );
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( AddList );
			First = False;
		} else {
			SL_Succ( AddList );
		}
		Item = SL_Current( AddList );
		SL_Add( List, Item );
	} while ( SL_IsEnd ( AddList ) == False );
	AddList->Dispose = NULL;
  	if( DestructAddList == True ) {
		SL_Destruct( AddList );
	}
}

/* Adds AddList, where AddList is a list of strings to the end of List allocating memory space to each string added */

void	SL_AddStringList( SL List, SL AddList, Bool_ DestructAddList )
{
	char* 	Item;
	char* 	ItemAdded;
	Bool_	First;

	if( AddList == NULL ) {
		return;
	}
	if(SL_EmptyList(AddList) == True) {
		return;
	}
	if(SL_EmptyList(List) == False) {
		SL_End( List );
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( AddList );
			First = False;
		} else {
			SL_Succ( AddList );
		}
		Item = SL_Current( AddList );
		ItemAdded = SBYTES_( strlen( Item ) + 2, "SL" );
		strcpy( ItemAdded, Item );
		SL_Add( List, ItemAdded );
	} while ( SL_IsEnd ( AddList ) == False );
	AddList->Dispose = NULL;
  	if( DestructAddList == True ) {
		SL_Destruct( AddList );
	}
}


/* Conditionaly adds AddList to the end of List */

void	SL_AddListConditional( SL List, SL AddList, SL_LocateFunc LocateFunc )
{
	void * 	Item;
	Bool_	First;

	if( AddList == NULL ) {
		return;
	}
	if(SL_EmptyList(AddList) == True) {
		return;
	}
	if(SL_EmptyList(List) == False) {
		SL_End( List );
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( AddList );
			First = False;
		} else {
			SL_Succ( AddList );
		}
		Item = SL_Current( AddList );
		if ( (*LocateFunc)(List, Item) == True ) {
			SL_End( List );
			SL_Add( List, Item );
		}
	} while ( SL_IsEnd ( AddList ) == False );
}

/* Inserts 'AddList' into 'List' before current item in 'List' if 'Before' is set
   to True and after current item in list if 'Before' is set to False. If 'CurrentRemain' is set to True, then the item initially the current remains the current, otherwise the current becomes the item last inserted into the list from the inserted list. */

void	SL_InsertList( SL List, SL AddList, Bool_ Before, Bool_ DestructAddList, Bool_ CurrentRemain )
{
	void * 	Item;
	Bool_	First;
	int	Count;
	int	i;

	First = True;
	Count = 0;
	do {
		if ( First == True ) {
			SL_Start( AddList );
			Item = SL_Current( AddList );
			First = False;
			if( Before == True ) {
				SL_AddBefore( List, Item );
			} else {
				SL_Add( List, Item );
			}
		} else {
			SL_Succ( AddList );
			Item = SL_Current( AddList );
			SL_Add( List, Item );
		}
		Count++;
	} while ( SL_IsEnd ( AddList ) == False );
	AddList->Dispose = NULL;
  	if( DestructAddList == True ) {
		SL_Destruct( AddList );
	}
	if( CurrentRemain == True && Before == False ) {
		for( i=0; i<Count; i++ ) {
			SL_Prev( List );
		}
	} else if( CurrentRemain == True && Before == True )  {
		SL_Succ( List );
	}
}

/* Returns the number of items in the 'List' */

int	SL_NumberOfItemsInList( SL List )
{
	Bool_	First;
	int	Count;

	Count = 0;
	if(SL_EmptyList(List) == True) {
		return Count;
	}
	First = True;
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Count++;
	} while ( SL_IsEnd ( List ) == False );
	return Count;
}

/* Reverses the order of the items in the 'List' */

void	SL_ReverseList( SL List )
{
	void * 	Item;
	SL	ListTemp;

	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) return;
	ListTemp = SL_Create( (SL_Free) NULL);

 	SL_End(List);

	while(SL_EmptyList(List) == False) {
 		Item = SL_RemoveCurrent(List);
	  	SL_Add(ListTemp, Item);
	}

 	SL_Start(ListTemp);

	while(SL_EmptyList(ListTemp) == False) {
 		Item = SL_RemoveCurrent(ListTemp);
	  	SL_Add(List, Item);
	}
	SL_Destruct( ListTemp );
}

/* Orders the items in the 'List' according to the function defined in the 
   'OrderFunc'.
*/

void	SL_OrderList( SL List, SL_OrderFunc OrderFunc )
{
	Bool_	First;
	void*	Item1;
	void*	Item2;

	First = True;
	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) {
		return;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
			if( SL_IsEnd ( List ) == True ){ 
				break;
			} else {
				Item1 = SL_Current( List );
			}
		} else {
			Item1 = Item2;
		}
		SL_Succ( List );
		Item2 = SL_Current( List );
		if ( (*OrderFunc)(Item1, Item2) == True ) {
			First = True;
			Item2 = SL_RemoveCurrent( List );
			SL_AddBefore( List, Item2 );
		}
	} while ( SL_IsEnd ( List ) == False || First == True );
}

/* Returns the position of an item in the list defined by the function 'PositionFunc' */


int	SL_Position( SL List, SL_PositionFunc PositionFunc, void*Item )
{
	void * 	ItemRet;
	Bool_	First;
	int	Count;


	First = True;
	if( List == NULL ) {
		return (-1);
	}
	if(SL_EmptyList(List) == True) {
		return (-1);
	}
	Count = 0;
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Count++;
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return (-1);
		if ( (*PositionFunc)(Item, ItemRet) == True ) 
			return Count;
	} while ( SL_IsEnd( List ) != True );
	return (-1);
}

/* Clears all items from 'List' and frees memory allocated to them if 'FreeItems' is True */

void		SL_Clear( SL List, Bool_ FreeItems )
{
  void*		Item;

  while( SL_EmptyList( List ) == False ) {
	Item = SL_RemoveCurrent( List );
	if( FreeItems == True ) {
		List->Dispose( Item );
	}
  }
}

/* Returns the number of a particular type in 'List' where type is defined by the function LocateFunc */

int	SL_NumberOfTypeInList( SL List, SL_LocateFunc LocateFunc, void*Item )
{
	void * 	ItemRet;
	Bool_	First;
	int	Count;


	if( List == NULL ) {
		return 0;
	}
	First = True;
	if(SL_EmptyList(List) == True) {
		return 0;
	}
	Count = 0;
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		ItemRet = SL_Current( List );
		if ( ItemRet == NULL ) 
			return 0;
		if ( (*LocateFunc)(Item, ItemRet) == True ) 
			Count++;
	} while ( SL_IsEnd ( List ) != True );
	return Count;
}

/* Removes all duplicated items from the list */


void	SL_RemoveDuplicates( SL List )
{
	void * 	Item;
	void * 	ItemRet;
	Bool_	First;


	First = True;
	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) {
		return;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		Item = SL_Current( List );
		ItemRet = SL_LocateNext( List, (SL_LocateFunc)SL_Identical_Locate, Item );
		if ( ItemRet != NULL ) {
			SL_RemoveCurrent( List );
			SL_Start( List );
			First = True;
		}
	} while ( SL_IsEnd ( List ) != True );

}

/* Locates a duplicated item */

SL_LocateFunc	SL_Identical_Locate( void* Item, void* ItemRet )
{
	if( Item == ItemRet ) {
		return (SL_LocateFunc)True;
	} else {
		return (SL_LocateFunc)False;
	}
}

/* Locates and returns a similar item in 'List' to 'Item' where the similarity is defined by the function LocateFunc, else returns NULL if one does not exist. */

void*	SL_LocateAlike( SL List, SL_LocateFunc2 LocateFunc, void* Item )
{
	void * 	ItemRet;
	void * 	ItemNext;
	Bool_	First;


	First = True;
	if( List == NULL ) {
		return NULL;
	}
	if(SL_EmptyList(List) == True) {
		return NULL;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		ItemRet = SL_Current( List );
		SL_HoldPosition( List );
		ItemNext = SL_LocateNext2( List, (SL_LocateFunc2)LocateFunc, ItemRet, Item );
		if( ItemNext != NULL ) {
			return ItemNext;
		}
		SL_ReturnPosition( List );
	} while ( SL_IsEnd ( List ) != True );
	return NULL;
}

/* Locates and removes a similar item in 'List' to 'Item' where the similarity is defined by the function LocateFunc. */

void	SL_RemoveAlike( SL List, SL_LocateFunc2 LocateFunc, void* Item )
{
	void * 	ItemRet;
	void * 	ItemNext;
	Bool_	First;


	First = True;
	if( List == NULL ) {
		return;
	}
	if(SL_EmptyList(List) == True) {
		return;
	}
	do {
		if ( First == True ) {
			SL_Start( List );
			First = False;
		} else {
			SL_Succ( List );
		}
		ItemRet = SL_Current( List );
		ItemNext = SL_LocateNext2( List, (SL_LocateFunc2)LocateFunc, ItemRet, Item );
		if( ItemNext != NULL ) {
			SL_RemoveCurrent( List );
			SL_Start( List );
			First = True;
		}
	} while ( SL_IsEnd ( List ) != True && First != True );
}

/* Retains the current position in the list for returning to when routine SL_ReturnPosition is called */

void	SL_HoldPosition( SL List )
{
	List->CurrentHold = List->Current;

}

/* Current position in list becomes the one that was current when routine SL_HoldPosition was called */

void	SL_ReturnPosition( SL List )
{
	List->Current = List->CurrentHold;
}
