/*  Module Name: @(#)str_hand.c	1.7
    Version: 11/11/96
    Author:  M. Bush
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance: 

    Description:  
		  String handling functions.
*/

#include <stdio.h> 
#include <string.h>
#include <ctype.h>
#include "memcheck.h"
#include "sl_list.h"
#include "str_hand.h"

#define QUOTE '"'
#define EOS   '\0'
#define SPACE ' '
#define TAB   '\t'
#define LEGAL  " -"

static int	MaxWordSize = 0;
static Bool_ 	GSpeedOn;
static Bool_ 	GBlankOutCalled = False;
static char*	GString1;
static char*  	GString2;
static int	GEndPos;





/*
 * Returns a pointer to a null terminated string derived from 'InString' by the 
 * following rules:
 *
 * i) The first character of the returned string is a ".
 *
 * ii) The characters of 'InString' are copied to the return string. If a " is
 * found in the string, it is doubled to "".
 *
 * iii) The last character of the returned string (before a null) is ".
 *
 * iv) If the length of the returned string would exceed 'BufLength', then it is 
 * truncated before the trailing ".
 *
 * For example, This is a "text" string \0     becomes:
 * "This is a ""text"" string "\0.
 */

char*STR_Quote	( const char *InString , char *Buf , int BufLength )
{
	int	InputIndex, OutputIndex;

	InputIndex = OutputIndex = 0;
	BufLength -= 3;		    /* room for start ", end " and \0 */
	Buf[OutputIndex] = QUOTE;	    /* start " */
	while ( InString[InputIndex] != EOS && OutputIndex < BufLength ) {
		OutputIndex++;
		Buf[OutputIndex] = InString[InputIndex];
		if ( Buf[OutputIndex] == QUOTE && OutputIndex <= BufLength) {
			Buf[++OutputIndex] = QUOTE;
		}
		InputIndex++;
	}
	Buf[++OutputIndex] = QUOTE;	    /* end " */
	Buf[++OutputIndex] = EOS;	    /* \0 */
	return Buf ;
}

/*
 * The returned string points to a null terminated string derived from Instring by the 
 * following rules:
 *
 * i) If the first character of 'InString' is not a " then a pointer is returned to 
 * InString, otherwise:
 *
 * ii) The characters of 'InString' are copied to the returned string, but the
 * leading " and trailing " (if any) are not copied.
 *
 * iii) If two quotes are found together in 'InString', only one is copied to the 
 * returned string.
 *
 * iv) If the length of the returned string would exceed 'BufLength' then it is 
 * truncated before the trailing null.
 * For example, "This is a ""text"" string "\0   becomes:
 */


char*STR_UnQuote	( const char *InString , char *Buf , int BufLength )
{
	int	InputIndex, OutputIndex;

	InputIndex = OutputIndex = 0;
	if ( InString[0] != QUOTE ) {
		do {
			Buf [InputIndex] = InString[InputIndex] ;
			InputIndex++;
		} while ( InputIndex <= BufLength && InString[InputIndex] != EOS ) ;
		Buf [InputIndex] = EOS ;
	} else {
		InputIndex = OutputIndex = (-1);
		do {
			OutputIndex++;
			InputIndex++;
			if ( InString[InputIndex] == QUOTE ) {
				InputIndex++;
			}
			Buf[OutputIndex] = InString[InputIndex];
		} while ( InString[InputIndex] != EOS && OutputIndex <= BufLength );
	}
	return Buf ;
}


/*
 * Memory is allocated for X to be copied into. A pointer to the copied X is returned.
 */

char*STR_Save	(char*X)
{
	char * Y;

	Y = (char * )SBYTES_(strlen(X) + 1, "STR1");
	strcpy(Y, X);
	return(Y);
}


/*
 * Deletes all occurences of '\r' and '\n' from 'Text'.
 */

void  STR_DeleteReturns( char*Text )
{
	int	i;

	i = 0;
	while ( Text[i] != '\0' ) {
		if ( Text[i] == '\r' || Text[i] == '\n' ) {
			Text[i] = ' ';
		}
		i++;
	}
	if( i != 0 ) {
		if ( Text[i-1] == ' ' ) {
			Text[i-1] = '\0';
		}
	}
}


/*
 * Converts all characters in 'Text' to lower case except for the first which is 
 * converted to upper case.
 */

void  STR_FirstToUpper( char*Text )
{
	int	i;
	Bool_ NewWord;

	NewWord = True;
	for ( i = 0; i < strlen( Text ); i++) {
		if ( isalpha( Text[i] ) != 0 && NewWord == True ) {
			NewWord = False;
			Text[i] = toupper( Text[i] );
		} else if ( isalnum( Text[i] ) == 0 ) {
			NewWord = True;
		} else {
			if ( isalpha( Text[i] ) != 0 ) {
				Text[i] = tolower( Text[i] );
			}
			NewWord = False;
		}
	}
}


/*
 * Converts all characters in 'Text' to lower case.
 */

void  STR_ToLower( char*Text )
{
	int	i;

	for ( i = 0; i < strlen( Text ); i++) {
		if ( isalpha( Text[i] ) != 0 ) {
			Text[i] = tolower( Text[i] );
		}
	}
}


/*
 * Converts all characters in 'Text' to upper case.
 */

void  STR_ToUpper( char*Text )
{
	int	i;

	for ( i = 0; i < strlen( Text ); i++) {
		if ( isalpha( Text[i] ) != 0 ) {
/*			if( i != 0 ) {
				if( Text[i-1] == '\\' ) {
					continue;
				}
			}  */
			Text[i] = toupper( Text[i] );
		}
	}
}


/*
 * Removes tabs and spaces from the beginning of 'String'
 */

void  STR_Shuffle( char* String )
{
	int	i,j;

	i = 0;
	j = 0;
	while( String[i] == ' ' || String[i] == '\t' ) {
		i++;
	}
	if( i != 0 ) {
		while( String[i] != '\0' ) {
			String[j] = String[i];
			i++;   j++;
		}
		String[j] = '\0';
	}
}

/*
 * Removes tabs and spaces from the end of 'String'
 */

void	STR_RemoveEndSpaces( char* String )
{
	int	Length;
	int	i;

	Length = strlen( String );
	i = Length - 1;
	while( String[i] == ' ' || String[i] == '\t' ) {
		i--;
		if( i < 0 ) {
			break;
		}
	}
	i++;
	String[i] = '\0';
}



/*
 * Removes mask characters from the end of 'String'
 */

void	STR_RemoveEndMaskChars( char* String, char Mask )
{
	int	Length;
	int	i;

	Length = strlen( String );
	i = Length - 1;
	while( String[i] == Mask ) {
		i--;
	}
	i++;
	String[i] = '\0';
}



/*
 * Removes tabs, spaces and end of line markers from the end of 'String'
 */


void	STR_RemoveEndSpacesAndEOLMs( char* String )
{
	int	Length;
	int	i;

	Length = strlen( String );
	i = Length - 1;
	while( String[i] == ' ' || String[i] == '\t' ||
	       String[i] == '\r' || String[i] == '\n' ) {
		i--;
		if( i < 0 ) {
			break;
		}
	}
	i++;
	String[i] = '\0';
}


/* Compares the beginning of string 'Main' with 'Sub' and returns the position of 'Sub' blanking of 'Main' or (-1) if 'Sub' does not exist in 'Main'. Blanks are ignored in main */



int	STR_CompareBeginningNoBlank( char* Main, char* Sub )
{
  int	i;
  int	j;

  i = 0;
  j = 0;
  while( Sub[j] != '\0' ) {
	if( Main[i] == '\0' ) {
		return (-1);
	}
	if( Main[i] == ' ' ) {
		i++;
	} else if( Main[i] == Sub[j] ) {
		i++;
		j++;
	} else {
		return (-1);
	}
  }
  return i;
}

/* Compares the beginning of string 'Main' with 'Sub' and returns the position of 'Sub' blanking of 'Main' or (-1) if 'Sub' does not exist in 'Main' */

int	STR_CompareBeginning( char* Main, char* Sub )
{
  int	i;

  i = 0;
  while( Sub[i] != '\0' ) {
	if( Main[i] == '\0' ) {
		return (-1);
	}
	if( Main[i] == Sub[i] ) {
		i++;
	} else {
		return (-1);
	}
  }
  return i;
}

/* Copies first integer number embedded in 'String' and places in 'Number' */

Bool_	STR_ExtractNumber( char* String, int *Pos, char* Number )
{
  int		i;
  Bool_		Found;

  i = 0;
  Found = False;
  if( (int)strlen( String ) < *Pos ) {
	SError_( Fatal_, "String length less than position given", "STR1" );
  }
  while( String[*Pos] != '\0' ) {
	if( isdigit( String[*Pos] ) != 0 ) {
		Number[i] =  String[*Pos];
		Found = True;
		i++;
	} else {
		if( Found == True ) {
			break;
		}
	}
	(*Pos)++;
  }
  Number[i] =  '\0';
  return Found;
}

/* Replaces first occurrencs of '~' in 'String' with 'Number' */

void	STR_InsertNumber( char* String, char* Number )
{
  int		i,j,k;
  char*		Hold;

  i = 0;
  j = 0;
  k = 0;
  Hold = SBYTES_( strlen( String ) + strlen( Number ) + 10, "STR2" );
  while( String[i] != '~' ) {
	Hold[j] = String[i];
	i++; j++;
  }
  i++;
  while( Number[k] != '\0' ) {
	Hold[j] = Number[k];
	j++; k++;
  }
  while( String[i] != '\0' ) {
	Hold[j] = String[i];
	i++; j++;
  }
  Hold[j] = '\0';
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Inserts 'Argument' into 'String' where 'InsertChar' occurs */

void	STR_InsertArgument( char* String, char* Argument, char InsertChar )
{
  int		i,j,k;
  char*		Hold;

  i = 0;
  j = 0;
  k = 0;
  Hold = SBYTES_( strlen( String ) + 10000, "STR3" );
  while( String[i] != InsertChar ) {
	Hold[j] = String[i];
	if( String[i] == '\0' ) {
		strcat( String, " " );
		strcat( String, Argument );
  		FREE_( Hold );
		return;
	}
	i++; j++;
  }
  i++;
  while( Argument[k] != '\0' ) {
	Hold[j] = Argument[k];
	j++; k++;
  }
  while( String[i] != '\0' ) {
	Hold[j] = String[i];
	i++; j++;
  }
  Hold[j] = '\0';
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Copies from position 'Pos' in string 'From' into 'To'. Returns True if
   alphanumeric character(s) have been copied. If EndPos is -1 then copy to end of string. */

Bool_	STR_CopyFrom( char* To, char* From, int Pos, int EndPos )
{
  int	i,j;
  Bool_ AlphaNum;

  i = Pos;
  if( i < 0 ) {
	i = 0;
  }
  j = 0;
  AlphaNum = False;
  if( EndPos == (-1) ) EndPos = (-5);
  if( (int)strlen( From ) < Pos ) {
	SError_( Fatal_, "String length less than position given", "STR2" );
  }
  while( From[i] != '\0' && i != EndPos + 1 ) {
	To[j] = From[i];
	if( isalnum( From[i] ) != 0 ) {
		AlphaNum = True;
	}
	i++; j++;
  }
  To[j] = '\0';
  return AlphaNum;
}

/* Copies 'String' into 'Buffer' from 'Pos' upto the first occurrence of a 'Mask' character or '\0'. */

void	STR_CopyToMask( char* String, int Pos, char Mask, char* Buffer )
{
  int	i;
  int	j;

  j = 0;
  i = Pos;
  while( String[i] != Mask && String[i] != '\0' ) {
	Buffer[j] = String[i];
	i++;  j++;
  }
  Buffer[j] = '\0';
}

/* Converts two or more adjacent spaces in 'String' to just one space */

void	STR_ElimMultiSpaces( char* String )
{
  int	i,j;
  Bool_	LastSpace;


  i = 0;
  while( String[i] != '\0' ) {
	if( String[i] == '\t' ) {
		String[i] = ' ';
	}
	i++;
  }
  i = 0;
  j = 0;
  LastSpace = False;
  while( String[i] != '\0' ) {
	if( String[i] == ' ' ) {
		if( LastSpace == False ) {
			String[j] = String[i];
			j++;
		}
		LastSpace = True;
	} else {
		LastSpace = False;
		String[j] = String[i];
		j++;
	}
	i++;
  }
  String[j] = '\0';
}

/* Returns True if 'String' consists of only spaces */

Bool_	STR_AllSpace( char* String )
{
  int	i;

  i = 0;
  while( String[i] != '\0' ) {
	if( String[i] == ' ' || String[i] == '\t' ) {
		i++;
	} else {
		return False;
	}
  }
  return True;
}

/* If any of the characters in'String' can be found in string 'Illegal' return False */

Bool_	STR_LegalChars( char* String, char* Illegal )
{
  int	i;

  i = 0;
  while( String[i] != '\0' ) {
	if( strchr( Illegal, String[i] ) != NULL ){
		return False;
	} else {
		i++;
	}
  }
  return True;
}

/* If 'c' occurs in the string LEGAL return True */

Bool_	STR_LegalChar( char c )
{
	if( strchr( LEGAL, c ) == NULL ){
		return False;
	} else {
		return True;
	}
}

/* Adds a space to the beginning of a string and end of string */

void	STR_AddSpaces( char* String )
{
  char*	Hold;

  Hold = SBYTES_( strlen( String ) + 10, "STR6" );
  strcpy( Hold, " " );
  strcat( Hold, String );
  strcat( Hold, " " );
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Returns in hold the first number that occurs in String and the position at which it occurs */
Bool_	STR_GetIntString( char* String, int *Pos, char* Hold )
{
  int i,j;
  i = 0;

  j = *Pos;
  if( (int)strlen( String ) < *Pos ) {
	SError_( Fatal_, "String length less than position given", "STR3" );
  }
  while( isdigit( String[j] ) == 0 ) {
	if( String[j] == '\0' ) return False;
	j++;
  }

  while( isdigit( String[j] ) != 0 ) {
	if( String[j] == '\0' ) return False;
	if( i < INTSIZE ) {
		Hold[i] = String[j];
	}
	i++;    j++;
  }

  if( i < INTSIZE ) {
	Hold[i] = '\0';
  } else {
	Hold[INTSIZE-1] = '\0';
  }
  *Pos = j;
  return True;
}

/* Puts text in 'Hold' from position 'Pos' in 'String' up to the point where
   'TextMarker' is found. Returns in 'Pos' the position in 'String' immediately
   after the 'TextMarker' found, or the same value of Pos is retained if
   'TextMarker' is not found. Returns True if 'TextMarker' is found. */

Bool_	STR_GetTextToMarkerExact( char* String, int *Pos, char* Hold, char* TextMarker )
{
  int	i,j,k,l;

  i = *Pos;
  j = 0;
  k = 0;
  if( (int)strlen( String ) < *Pos ) {
	SError_( Fatal_, "String length less than position given", "STR4" );
  }
  while( String[i] != '\0' ) {



	if( STR_WhiteSpacesToEOL( String, i ) == True && STR_WhiteSpacesToEOL( TextMarker, j ) ) {
		while( String[i] != '\n' ) {
			Hold[k] = String[i];
			i++;  k++;
			if( k == XLINESIZE ){
				return False;
			}
		}
		while( TextMarker[j] != '\n' ) {
			j++;
		}
	}


	if( String[i] == TextMarker[j] ) {
		if( j == 0 ) l = k;
		j++;
		if( TextMarker[j] == '\0' ) {
			Hold[l] = '\0';
			i++;
			*Pos = i;
			return True;
		}
	} else if( String[i] == TextMarker[0] ) {
		j = 0;
		l = k;
		j++;
		if( TextMarker[j] == '\0' ) {
			Hold[l] = '\0';
			i++;
			*Pos = i;
			return True;
		}
	} else {
		j = 0;
	}
	Hold[k] = String[i];

	if( Hold[k] == '\n' && j != 0 ) {
		i++;  k++;
		while( isspace( String[i] ) != 0 && String[i] != '\n' ) {
			Hold[k] = String[i];
			i++;  k++;
		}
		while( isspace( TextMarker[j] ) != 0 && TextMarker[j] != '\n' ) {
			j++;
		}
	} else {
		i++;  k++;
	}
	if( k == XLINESIZE ){
		return False;
	}
  }
  Hold[k] = '\0';
  return False;
}

/* Performs in the same way as STR_GetTextToMarkerExact except that if 
   'TextMarker' consists of just whitespace(s), will look for the next 
   'TextMarker' in 'String' that will require copying non white space
   characters into 'Hold' */

Bool_	STR_GetTextToMarker( char* String, int *Pos, char* Hold, char* TextMarker )
{
  Bool_		WhiteMarker;
  Bool_		WhiteText;

  WhiteMarker = STR_SpaceTab( TextMarker );
  do {
	if( STR_GetTextToMarkerExact( String, Pos, Hold, TextMarker ) == False ) {
		return False;
	}
	WhiteText = STR_WhiteSpace( Hold );
  } while( WhiteMarker == True && WhiteText == True );
  return True;
}

/* Returns True if ther are just white spaces or nothing up to the next 
   '\n' in String */

Bool_	STR_WhiteSpacesToEOL( char* String, int Pos )
{
  while( String[Pos] != '\n' ) {
	if( String[Pos] == '\0' )return False;
	if( isspace( String[Pos] ) == 0 ) {
		return False;
	}
	Pos++;
  }
  return True;
}






/* This function returns in 'Hold' the text from 'Pos' in 'String' upto 
   the nearest Z3950 RPN Marker (or '\0') and returns the type of marker
   found. Returns in 'Pos' the position in 'String' immediately
   after the 'TextMarker' found. */

STR_MarkerType	STR_GetTextToZ3950RPNMarkers( char* String, int *Pos, char* Hold )
{
  char*			Hold1;
  char*			Hold2;
  char*			Hold3;
  int			Pos1;
  int			Pos2;
  int			Pos3;
  char			ListMarker[3];
  char			SeqMarker[3];
  char			InternalMarker[3];
  Bool_			More1;
  Bool_			More2;
  Bool_			More3;
  STR_MarkerType	Type;
  int			FPos;

  ListMarker[0] = '\xb6';
  ListMarker[1] = '\xb6';
  ListMarker[2] = '\0';

  SeqMarker[0] = '\xb5';
  SeqMarker[1] = '\xb5';
  SeqMarker[2] = '\0';

  InternalMarker[0] = '\xb9';
  InternalMarker[1] = '\xb9';
  InternalMarker[2] = '\0';

  Pos1 = *Pos;
  Pos2 = *Pos;
  Pos3 = *Pos;

  Hold1 = SBYTES_( strlen( String ) + 5, "STR7" );
  Hold2 = SBYTES_( strlen( String ) + 5, "STR8" );
  Hold3 = SBYTES_( strlen( String ) + 5, "STR9" );

  More1 = STR_GetTextToMarkerExact( String, &Pos1, Hold1, ListMarker );
  More2 = STR_GetTextToMarkerExact( String, &Pos2, Hold2, SeqMarker );
  More3 = STR_GetTextToMarkerExact( String, &Pos3, Hold3, InternalMarker );

  FPos = (-1);
  Type = STR_EndOfString;
  strcpy( Hold, Hold1 );
  if( More1 == True ) {
	FPos = Pos1;
	Type = STR_ListMarker;
	strcpy( Hold, Hold1 );
  }
  if( More2 == True ) {
	if( FPos == (-1) || FPos > Pos2 ) {
		FPos = Pos2;
		Type = STR_SeqMarker;
		strcpy( Hold, Hold2 );
	}
  }
  if( More3 == True ) {
	if( FPos == (-1) || FPos > Pos3 ) {
		FPos = Pos3;
		Type = STR_InternalMarker;
		strcpy( Hold, Hold3 );
	}
  }
  if( FPos != (-1) ) {
	*Pos = FPos;
  }


  FREE_( Hold1 );
  FREE_( Hold2 );
  FREE_( Hold3 );
  return Type;
}

/* Converts '\r' and '\n' to return and line feed characters respectively */

void	STR_TransformEscSeq( char* String )
{
  char*		pString;
  int		i,j;

  char*		EscSeq[] = { "\\n", "\\r" };
  char		ConvEscSeq[] = { '\n', '\r' };

  for ( i=0; i<2; i++ ) {
	while( ( pString = strstr( String, EscSeq[i] ) ) != NULL ) {
		pString[0] = ConvEscSeq[i];
		j = 1;
		do {
			pString[j] = pString[j+1];
			j++;
		} while( pString[j-1] != '\0' );
	}
  }
}

/* Returns True if 'String' contains only spaces or tabs */

Bool_	STR_SpaceTab( char* String )
{
  int	i;

  i = 0;
  if( String[i] == '\0' ) return True;
  while( String[i] == ' ' || String[i] == '\t' ) {
	i++;
	if( String[i] == '\0' ) return True;
  }
  return False;
}

/* Returns True if 'String' contains only white spaces */

Bool_	STR_WhiteSpace( char* String )
{
  int	i;

  i = 0;
  if( String[i] == '\0' ) return True;
  while( isspace( String[i] ) != 0 ) {
	i++;
	if( String[i] == '\0' ) return True;
  }
  return False;
}

/* Returns True if 'String' contains only from white spaces from position 'Pos' */

Bool_	STR_WhiteSpaceFrom( char* String, int Pos )
{
  int	i;

  if( (int)strlen( String ) <= Pos ) {
	return True;
  }
  i = Pos;
  if( String[i] == '\0' ) return True;
  while( isspace( String[i] ) != 0 ) {
	i++;
	if( String[i] == '\0' ) return True;
  }
  return False;
}


/* Returns False if the string contains any characters which are not alpha numeric */
Bool_	STR_IsStringAlnum( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isalnum( String[i] ) == 0 ) {
		return False;
	} else {
		i++;
	}
  }
  return True;
}

/* Returns True if the string contains only characters which are alpha or white space */
Bool_	STR_IsStringAlphaWS( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isalpha( String[i] ) == 0 && isspace( String[i] ) == 0 ) {
		return False;
	} else {
		i++;
	}
  }
  return True;
}


/* Returns True if the string contains only characters which are alphanumeric or white space */
Bool_	STR_IsStringAlnumWS( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isalnum( String[i] ) == 0 && isspace( String[i] ) == 0 ) {
		return False;
	} else {
		i++;
	}
  }
  return True;
}


/* Returns True if the string contains any characters which are alpha numeric */
Bool_	STR_AnyAlnumInString( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isalnum( String[i] ) != 0 ) {
		return True;
	} else {
		i++;
	}
  }
  return False;
}

/* Returns True if the string contains any characters which are alpha */
Bool_	STR_AnyAlphaInString( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isalpha( String[i] ) != 0 ) {
		return True;
	} else {
		i++;
	}
  }
  return False;
}

/* Returns True if the string contains any characters which are numeric */
Bool_	STR_AnyNumericInString( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isdigit( String[i] ) != 0 ) {
		return True;
	} else {
		i++;
	}
  }
  return False;
}

/* Returns True if the string only contains numbers and white spaces */
Bool_	STR_IsStringNumeric( char* String )
{
  int i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isdigit( String[i] ) == 0 && isspace( String[i] ) == 0 ){
		return False;
	} else {
		i++;
	}
  }
  return True;
}

/* Returns True if the string contains only one number and white spaces at beginning and end */

Bool_	STR_IsStringNumeric1( char* String )
{
  int 		i;
  char*		Buffer;

  i = 0;
  Buffer = SBYTES_( strlen( String ) + 5, "STR" );
  strcpy( Buffer, String );
  STR_Shuffle( Buffer );
  STR_RemoveEndSpaces( Buffer );
  while( Buffer[i] != '\0' ) {
	if( isdigit( Buffer[i] ) == 0 ){
		FREE_( Buffer );
		return False;
	} else {
		i++;
	}
  }
  FREE_( Buffer );
  return True;
}

/* Removes string2 from string1 and returns the position at the point where the removal has occurred */


Bool_	STR_RemoveSubString( char* String1, char* String2, int *Pos )
{
  int	i,j,k;
  char*	Buffer;

  i = 0;
  j = 0;

  if( String1 == NULL || String2 == NULL ) {
	return False;
  }
  if( String2[0] == '\0' ) {
	return False;
  }
  Buffer = SBYTES_( strlen( String1 ) + strlen( String2 ) + 10, "STR10" );
  if( strlen( String1 ) < strlen( String2 ) ) {
	FREE_( Buffer );
	return False;
  }
  strcpy( Buffer, String1 );
  do{
	if( String2[j] == '\0' ) {
		i = *Pos;
		do {
			Buffer[i] = Buffer[i+j];
			i++;
		} while( Buffer[i-1] != '\0' );
		strcpy( String1, Buffer );
		FREE_( Buffer );
		return True;
	}
	if( Buffer[i] == '\0' ) {
		break;
	}
	if( Buffer[i] == String2[j] ) {
		if( j == 0 ) {
			*Pos = i;
		}
		j++;
	} else {
		j = 0;
	}
	i++; 
  } while( 1 );
  FREE_( Buffer );
  return False;
}

/* Removes the first occurrence of String2 from String1 after position 'Pos'. Returns position of removed String in 'Pos'. Returns False if String2 does not occur in String1 after 'Pos'. */

Bool_	STR_RemoveSubStringFromPos( char* String1, char* String2, int *Pos )
{
  int	i,j,k;
  char*	Buffer;

  i = *Pos;
  j = 0;

  if( String1 == NULL || String2 == NULL ) {
	return False;
  }
  if( String2[0] == '\0' ) {
	return False;
  }
  Buffer = SBYTES_( strlen( String1 ) + strlen( String2 ) + 10, "STR11" );
  if( strlen( String1 ) < strlen( String2 ) ) {
	FREE_( Buffer );
	return False;
  }
  strcpy( Buffer, String1 );
  do{
	if( String2[j] == '\0' ) {
		i = *Pos;
		do {
			Buffer[i] = Buffer[i+j];
			i++;
		} while( Buffer[i-1] != '\0' );
		strcpy( String1, Buffer );
		FREE_( Buffer );
		return True;
	}
	if( Buffer[i] == '\0' ) {
		break;
	}
	if( Buffer[i] == String2[j] ) {
		if( j == 0 ) {
			*Pos = i;
		}
		j++;
	} else {
		j = 0;
	}
	i++; 
  } while( 1 );
  FREE_( Buffer );
  return False;
}


/* Inserts 'String2' into 'String1' at position 'Pos' */
void	STR_InsertString( char* String1, char* String2, int Pos )
{
  char*	Hold;
  int	i,j,k;

  i = 0;
  j = 0;
  k = 0;
  Hold = SBYTES_( strlen( String1 ) + strlen( String2 ) + 10, "STR12" );
  while( i != Pos ){
	Hold[j] = String1[i];
	i++;
	j++;
  }
  while( String2[k] != '\0') {
	Hold[j] = String2[k];
	j++;
	k++;
  }
  do {
	Hold[j] = String1[i];
	i++;
	j++;
  } while( String1[i-1] != '\0');
  strcpy( String1, Hold );
  FREE_( Hold );
}


/* Replaces all occurrences of 'Old' in 'String' with 'New'. */
void	STR_Replace( char* String, char* Old, char* New )
{
  int Pos;

  Pos = 0;
  if( strcmp( Old, New ) == 0 )return;
  while( STR_RemoveSubStringFromPos( String, Old, &Pos ) == True ) {
  	STR_InsertString( String, New, Pos );
	Pos = Pos + strlen( New );
  }
}


/* Replaces first occurrence of 'Old' in 'String' with 'New'. */
void	STR_ReplaceJustOnce( char* String, char* Old, char* New )
{
  int Pos;

  Pos = 0;
  if( strcmp( Old, New ) == 0 )return;
  if( STR_RemoveSubString( String, Old, &Pos ) == True ) {
  	STR_InsertString( String, New, Pos );
  }
}


/* Replaces first occurrence of 'Old' in 'String' with characters 'C'. */
void	STR_ReplaceJustOnceC( char* String, char* Old, char C )
{
  int 		Pos;
  char*		New;
  int		Length;

  Length = strlen( Old );
  New = SBYTES_( strlen( Old ) + 2, "STR13" );
  memset( New, C, Length );
  New[ Length ] = '\0';
  Pos = 0;
  if( strcmp( Old, New ) == 0 )return;
  if( STR_RemoveSubString( String, Old, &Pos ) == True ) {
  	STR_InsertString( String, New, Pos );
  }
}



/* Returns the number in a character 'String' in integer form */
int	STR_CharToInt( char* String )
{
	int i;

	sscanf( String, "%d", &i );
	return i;
}

/* Returns the integer in 'i' in a character string */ 
void	STR_IntToChar( int i, char* Buffer )
{
	sprintf( Buffer, "%d", i );
	STR_Shuffle( Buffer );
	STR_RemoveEndSpaces( Buffer );
}





/* Returns the position in the string1 buffer at which string2 points to */
int	STR_Position( char* String1, char* String2 )
{
  int	Position;

  Position = 0;
  while( String1 != String2 ) {
	if( *String1 == '\0' ) return (-1);
	Position++;
	String1++;
  }
  return Position;
}

/* Returns position of occurrence of String2 in String1.  'End' is set to True if String2 occurs 
   at the end of string1. Position and length of substring are returned
   in 'Position' and 'length'. If no occurence, position is set to -1.If 'Next' is set to True, looks for occurence
   after the last one found in the last call. A list of format % entries
   is returned from the routine if they exist. */





/* Returns the position at which 'String2' occurrs in 'String1' or -1 if no occurrence */

int	STR_PositionOfSubStringBasic( char* String1, char* String2 )
{
  int		i;
  int		j;
  int		Position;

  i = 0;
  j = 0;
  while( String1[i] != '\0' ) {
	if( String1[i] == String2[j] ) {
		if( j == 0 ) {
			Position = i;
		}
		j++;
	} else {
		if( j != 0 ) {
			i = Position;
		}
		j = 0;
	}
	i++;
	if( String2[j] == '\0' ) {
		return Position;
	}
  }
  return (-1);
}



/* Returns True if alphanumeric terms are seperated by white spaces */

Bool_	STR_IsStringPhrase( char* String )
{
  int	i;
  Bool_ Letter;
  Bool_ Space;

  i = 0;
  Letter = False;
  Space = False;
  while( String[i] != '\0' ) {
	if( isspace( String[i] ) == 0 ) {
		if( Space == True ) return True;
		Letter = True;
	} else {
		if( Letter == True ) Space = True;
	}
	i++;
  }
  return False;

}

/* Adds two numbers together in character form and returns the result in character form */

char*	STR_NumberStringAddition( char* NoString1, char* NoString2 )
{
  char		ResultC[80];
  int		No1I, No2I, ResultI;
  double	No1D, No2D, ResultD;

  if( strstr( NoString1, "." ) != NULL ) {
	sscanf( NoString1, "%lf", &No1D );
  	if( strstr( NoString2, "." ) != NULL ) {
		sscanf( NoString2, "%lf", &No2D );
		ResultD = No1D + No2D;
		sprintf( ResultC, "%lf", ResultD );
  	} else {
		sscanf( NoString2, "%d", &No2I );
		ResultD = No1D + No2I;
		sprintf( ResultC, "%lf", ResultD );
  	}
  } else {
	sscanf( NoString1, "%d", &No1I );
  	if( strstr( NoString2, "." ) != NULL ) {
		sscanf( NoString2, "%lf", &No2D );
		ResultD = No1I + No2D;
		sprintf( ResultC, "%lf", ResultD );
  	} else {
		sscanf( NoString2, "%d", &No2I );
		ResultI = No1I + No2I;
		sprintf( ResultC, "%d", ResultI );
  	}
  }

  STR_Shuffle( ResultC );
  STR_RemoveEndSpaces( ResultC );
  return STR_Save( ResultC );
}

/* Returns True if z3950 attribute String1 is numericaly greater than String2 */

Bool_	STR_Z3950NumberCompare( char* String1, char* String2 )
{
  int	i1,i2,j1,j2;

  sscanf( String1, "%d,%d", &i1, &j1 );
  sscanf( String2, "%d,%d", &i2, &j2 );
  if( i1 == i2 ) {
    	if( j1 > j2 ) {
		return True;
	} else {
		return False;
	}
  } else if( i1 < i2 ) {
	return False;
  } else {
	return True;
  }
}

/* Encloses 'String' inside a pair of substrings 'Enclose' */
void	STR_Enclose( char* String, char* Enclose )
{
  char*		Hold;

  Hold = (char*)SBYTES_( strlen( String ) + 3 * strlen( Enclose ), "STR16" );
  strcpy( Hold, Enclose );
  strcat( Hold, String );
  strcat( Hold, Enclose );
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Doubles all occurrences of string 'Double' in 'String' */

void	STR_Double( char* String, char* Double )
{
  int	i,j,k;
  char*	Buffer;

  i = 0;
  j = 0;
  k = 0;

  Buffer = SBYTES_( strlen( String ) + ( 20 * strlen( Double ) ) + 1000, "STR17" ); 
  while( String[i] != '\0' ) {
	Buffer[k] = String[i];
	k++;
	if( String[i] == Double[j] ) {
		j++;
		if( Double[j] == '\0' ) {
			j = 0;
			while( Double[j] != '\0' ) {
				Buffer[k] = Double[j];
				k++;  j++;
			}
		}
	} else {
		j = 0;
	}
	i++;
  }
  Buffer[k] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/*
   Replaces with 'Mask's the first occurrence of 'String2' in 'String1'. 
   Like occurrences of String2 in String1 are registered if whites spaces 
   correspond in position regardless of makeup or number. Returns position
   'Pos' of occurrence of String2 in String1. Returns True if masking has
   been performed. Also gives end position 'EndPos' of 'String2' in 'String1'.
   A list of format terms is returned in 'List'.
*/


/* Masks characters in 'String' from 'StartPos' to 'EndPos' inclusive 
   with 'Mask' */

void	STR_Blank( char* String, int StartPos, int EndPos, char Mask )
{
  int	i;

  for( i=StartPos; i<=EndPos; i++ ) {
	String[i] = Mask;
  }

}

/* Returns position in string of first non white space character (including \n)
   after 'StartPos'
*/

int	STR_FirstNonWS( char* String, int StartPos )
{
   int		i;

  i = StartPos;
  while( isspace( String[i] ) != 0 && String[i] != '\n' ) {
	if( String[i] == '\0' ) {
		return (-1);
	}
	i++;
  }
  return i;
}

/* Gets the next word from 'String' from position 'Pos' and places in 'Word'.
   If more potential words available in 'String' returns True else returns 
   False. A '\n' is treated as a character. If WSPrecede is True a white space
   precedes the word. If WSSucceed is True a white space suceeds the word.
*/

Bool_	STR_GetNextWord( char* String, int *Pos, char* Word, Bool_ *WSPrecede, Bool_ *WSSucceed )
{
  int		i;
  int		j;

  i = *Pos;
  j = 0;
  *WSPrecede = False;
  *WSSucceed = False;

/* Go to start of word */

  while( isspace( String[i] ) != 0 && String[i] != '\n' ) {   /* White Space */

	if( String[i] == '\0' ) {
		Word[0] = '\0';
		return False;
	} 
	*WSPrecede = True;
	i++;
  }

/* Place word into 'Word' */

  while( isspace( String[i] ) == 0 || String[i] == '\n' ) {   /* Character or EOL */
	Word[j] = String[i];
	if( String[i] == '\0' ) {
		return False;
	}
	i++; j++;
  	if( j == MaxWordSize && GBlankOutCalled == True ) {
		Word[j-1] = '\0';
		printf( "1 Word  %s Size %d\n", Word, MaxWordSize );
		SError_( Fatal_, "Word size bigger than size allocated", "STR21" );
  	}
  }
  *Pos = i;
  if( j>= MaxWordSize && GBlankOutCalled == True ) {
	printf( "2 Word  %s Size %d\n", Word, MaxWordSize );
	SError_( Fatal_, "Word size bigger than size allocated", "STR22" );
  }
  Word[j] = '\0';
  *WSSucceed = True;

/* Test to see if ther are any more words to the end of the string */

  while( isspace( String[i] ) != 0 ) {
	i++;
	if( String[i] == '\0' ) {
		break;
	}
  }
  if( String[i] == '\0' ) {
	return False;
  } else {
	return True;
  }
}



/* Gets the next word from 'String' from position 'Pos' and places in 'Word',
   where word in this instance is defined as being all the characters upto
   the next EOL marker.
   If more potential words available in 'String' returns True else returns 
   False. A '\n' is treated as a word. If WSPrecede is True a white space
   precedes the word. If WSSucceed is True a white space suceeds the word.
*/

Bool_	STR_GetToEOL( char* String, int *Pos, char* Word, Bool_ *WSPrecede, Bool_ *WSSucceed )
{
  int		i;
  int		j;

  i = *Pos;
  j = 0;
  *WSPrecede = False;
  *WSSucceed = False;
  while( isspace( String[i] ) != 0 && String[i] != '\n' ) {   /* White Space */

	if( String[i] == '\0' ) {
		Word[0] = '\0';
		return False;
	} 
	*WSPrecede = True;
	i++;
  }
  while( String[i] != '\n' ) {   /* Character or EOL */
	Word[j] = String[i];
	if( String[i] == '\0' ) {
		return False;
	}
	i++; j++;
  }
  *Pos = i;
  Word[j] = '\0';
  *WSSucceed = True;
  while( isspace( String[i] ) != 0 ) {
	i++;
  }
  if( String[i] == '\0' ) {
	return False;
  } else {
	return True;
  }
}



/* Makes all end of line markers in 'String' a single '\n' */

void	STR_StandardizeEOL( char* String )
{
  int		i;
  int		j;
  char*		Hold;
  int		PosHeld;
  int		PosHeldj;
  int		PrevLineStart;
  Bool_		AllWSLine;
  char		PrevChar;
  char		Line[300];
  Bool_		RealLine;

  i = 0;
  j = 0;
  PosHeldj = j;
  RealLine = False;
  Hold = SBYTES_( strlen( String ) + 100, "STR23" );
  while( String[i] != '\0' ) {
	if( String[i] == '\r' ) {
		if( RealLine == False ) {
			j = PosHeldj;
		}
		Hold[j] = '\n';
		i++;  j++;
		PosHeld = i;
		PosHeldj = j;
		while( 1 ) {
			if( String[i] == '\r' ) {
/* Correction */
/*				break;  */
				i++;
			} else if( String[i] == '\n' ) {
				i++;
				break;
			} else if( String[i] == '\0' ) {
				break;
			} else if( isspace( String[i] ) != 0 ) {
				i++;
			} else {
				i = PosHeld;
				break;
			}
		}
		RealLine = False;
	} else if( String[i] == '\n' ) {
		if( RealLine == False ) {
			j = PosHeldj;
		}
		Hold[j] = '\n';
		i++;  j++;
		PosHeld = i;
		PosHeldj = j;
		while( 1 ) {
			if( String[i] == '\n' ) {
				break;
			} else if( String[i] == '\r' ) {
				i++;
/* Correction */
/*				break;  */
			} else if( String[i] == '\0' ) {
				break;
			} else if( isspace( String[i] ) != 0 ) {
				i++;
			} else {
				i = PosHeld;
				break;
			}
		}
		RealLine = False;
	} else {
		if( isspace( String[i] ) == 0 ) {
			RealLine = True;
		}
		Hold[j] = String[i];
		i++;  j++;
	}
  }
  Hold[j] = '\0';
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Removes from 'String' characters from 'Start' to 'End' inclusive */

void		STR_RemoveLocsIncl( char* String, int Start, int End )
{
  char*	Buffer;
  int	i;
  int	j;

  Buffer = SBYTES_( strlen( String ) + 10, "STR24" );
  i = 0;
  j = 0;
  while( String[i] != '\0' ) {
	if( i < Start || i > End ) {
		Buffer[j] = String[i];
		j++;
	}
	i++;
  }
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* Removes the 2nd and further consecutive occurrences of a character 'C'
   in 'String'   */

void		STR_RemoveMultiples( char* String, char C )
{
  char*		Hold;
  int		i,j;
  Bool_		First;

  i = 0;
  j = 0;
  Hold = SBYTES_( strlen( String ) + 10, "STR25" );
  First = True;
  do {
	if( String[i] == C && First == True ) {
		First = False;
		Hold[j] = String[i];
		i++;	j++;
	} else if( String[i] == C && First == False ) {
		i++;
	} else {
		First = True;
		Hold[j] = String[i];
		i++;	j++;
	}
  } while( String[i-1] != '\0' );
  strcpy( String, Hold );
  FREE_( Hold );
}

/* Removes the 2nd consecutive occurrences of a character 'C'
   in 'String'   */

void		STR_RemoveDoubles( char* String, char C )
{
  char*		Hold;
  int		i,j;
  Bool_		First;

  i = 0;
  j = 0;
  Hold = SBYTES_( strlen( String ) + 10, "STR26" );
  First = True;
  do {
	if( String[i] == C && First == True ) {
		First = False;
		Hold[j] = String[i];
		i++;	j++;
	} else if( String[i] == C && First == False ) {
		i++;
		First = True;
	} else {
		First = True;
		Hold[j] = String[i];
		i++;	j++;
	}
  } while( String[i-1] != '\0' );
  strcpy( String, Hold );
  FREE_( Hold );
}




/* Inserts end of line markers '\n' such that no line is greater in length
   than LineSize and no single word is split across two lines.  */


void		STR_InsertEOLs( char* String, int LineSize, int StartSpaces )
{
  int		i,j,k,l;
  int		Starti;
  int		Startk;
  char*		Buffer;
  char*		pField;
  char*		pChange;
  char*		pEndField;
  int		Pos;
  int		Length;

  i = 0;
  k = 0;

/* Check for any words greater than 'LineSize' - 'StartSpaces' and insert spaces if so */

  STR_RemoveSpacesFromEOLs( String );

  STR_InsertSpacesInLongWords( String, LineSize - StartSpaces - 2 );

  Length = 2 * strlen( String );
  Buffer = SBYTES_( Length, "STR27" );
  memset( Buffer, '\0', Length );
  pField = Buffer;
  do {
	if( String[i] == '\n' || String[i]  == '\r' ) {
		j = 0;
  		pField = Buffer + k;
	} else {
		j++;
	}
	if( j == LineSize ) {
		while( String[i] != ' ' ) {
			i--;
			k--;
			j--;
		}
		Buffer[k] = '\0';
		pChange=pField;
		pChange=STR_PointerToFirstNonWhiteSpace(pChange);
		while ( j < LineSize ) { 
			pChange=strstr(pChange," ");	
			if (pChange == NULL ) {
					pChange=pField;
					pChange=STR_PointerToFirstNonWhiteSpace(pChange);
					pChange=strstr(pChange," ");	
				}
			if (pChange != NULL ) {
				pChange=STR_PointerToFirstNonWhiteSpace(pChange);
				STR_InsertString(pChange," ",0);
				k++;
				pChange++;
				pChange++;
				j++;
				} else	{ 
						i--;
						i--;
						k--;
						j=LineSize;
					}
			}
		Buffer[k] = '\n';
		pField=Buffer+k;
		j = 0;
		k++;
		i++;
	} else {
		Buffer[k] = String[i];
		i++;
		k++;
	}
  } while( String[i] != '\0' );
  Buffer[k] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}








/* Inserts into String1 format marker (%d or %s) String2 */

void	STR_InsertInFormatMarkerLeftOverWrite( char* String1, char* String2 )
{
  char		Text[10];
  char*		pString1;
  int		i;
  char*		Hold;
  char		Message[100];

  if( STR_ReturnFirstFormatType( String1, Text ) == False ) {
	sprintf( Message, "Failed to find format marker in string %s for %s", String1, String2 );
	SError_( Fatal_, Message, "STR32" );
  }

  pString1 = strstr( String1, Text );
  *pString1 = ' ';
  pString1++; 
  *pString1 = ' ';
  for( i=strlen(String2)-1; i>=0; i-- ) {
	*pString1 = String2[i];
  	pString1--;
  }
}

/* Inserts into String1 format marker (%d or %s) String2 */

void	STR_InsertInFormatMarkerRightOverWrite( char* String1, char* String2 )
{
  char		Text[10];
  char*		pString1;
  int		i;
  char*		Hold;
  char		Message[100];

  if( STR_ReturnFirstFormatType( String1, Text ) == False ) {
	sprintf( Message, "Failed to find format marker in string %s for %s", String1, String2 );
	SError_( Fatal_, Message, "STR33" );
  }
  pString1 = strstr( String1, Text );
  *pString1 = ' ';
  pString1++; 
  *pString1 = ' ';
  pString1--;
  Hold = SBYTES_( strlen( String2 ) + 2, "STR34" );
  strcpy( Hold, String2 );
  STR_Shuffle( Hold );
  for( i=0; i<strlen(Hold); i++ ) {
	*pString1 = Hold[i];
  	pString1++;
  }
  FREE_( Hold );
}

/* Returns the length of the longest string in the array 'Fields' where 'ArrayNo' is the number in the array */

int	STR_LengthOfLongestStringInArray( int ArrayNo, char *Fields[] )
{
  int	i;
  int	Longest;
  int	Length;

  Longest = 0;
  for( i=0; i<ArrayNo; i++ ) {
	Length = strlen( Fields[i] );
	if( Length > Longest ) {
		Longest = Length;
	}
  }
  return Longest;
}

/* Removes prompt characters after last '\n' in 'String' */

void	STR_RemoveCharactersAfterLastEOL( char* String )
{
  char		C;
  int		Length;

  if( STR_PromptAtEnd( String ) == False ) {
	return;
  }
  Length = strlen( String );
  do{
	Length--;
	C = String[ Length ];
	String[ Length ] = '\0';
  }while( C != '\n' && Length != 0 );
} 

/* Removes '.' from integers in 'String' e.g. "aaa 123.456 bbb" becomes "aaa 123456 bbb" */

void	STR_RemovePuncsFromNumbers( char* String )
{
   char*	Buffer;
   int		i,j;
   int		Length;

   Length = strlen( String );
   Buffer = SBYTES_( Length + 10 , "STR35" );
   j=0;
   i=0;
   while( String[i+2] != '\0' ){
	Buffer[j] = String[i];
	if( isdigit( String[i] ) != 0 && isdigit( String[i+2] ) != 0 && ( String[i+1] == '.' /* || String[i+1] == ',' */ ) ) {
		i++;
	}
	i++;
	j++;
   }
   Buffer[j] = String[i];
   Buffer[j+1] = String[i+1];
   Buffer[j+2] = String[i+2];
   strcpy( String, Buffer );
   FREE_( Buffer );
}

/* If any character in String2 is 'WildCard', this can be compared as like to any character in the equivalent position in String1. */

int	STR_StrcmpWildCard( char* String1, char* String2, char WildCard )
{
  int		i;
  char		Buffer[200];

  if( strlen( String1 ) != strlen( String2 ) ) {
	return 1;
  }
  strcpy( Buffer, String2 );
  i = 0;
  while( Buffer[i] != '\0' ) {
	if( Buffer[i] == WildCard ) {
		Buffer[i] = String1[i];
	}
	i++;
  }
  return strcmp( String1, Buffer );
}

/* Returns true if string is a number */

Bool_	STR_IsNumber( char* String )
{
  int	i;

  i = 0;
  while( String[i] != '\0' ){
	if( isdigit( String[i] ) == 0 && isspace( String[i] ) == 0 ) {
		return False;
	}
	i++;
  }
  if( i == 0 ) {
	return False;
  } else {
	return True;
  }
}

/* Puts the contents of a file 'FileName' into 'Buffer' */

void	STR_ContentsOfFileToString( char* FileName, char* Buffer )
{
  FILE		*StreamR;
  int		i;
  int		C;
  char		Message[80];
  char		ErrorMessage[100];

  i = 0;
  StreamR = fopen( FileName, "r" );
  if( StreamR == NULL ) {
	perror( ErrorMessage );
	SError_( Warning_, ErrorMessage, "STR36" );
	sprintf( Message, "Failed to open file %s", FileName );
	SError_( Warning_, Message, "STR37" );
	strcpy( Buffer, " " );
	return;
  }
  while( 1 ) {
	C = getc( StreamR );
	if( C == EOF )break;
	if( C == '\0' ) {
		C = ' ';
	}
	Buffer[i] = C;
	i++;
  }
  Buffer[i] = '\0';
  fclose( StreamR );
}


/* Puts the contents of a buffer 'Buffer' into 'FileName' */

void	STR_ContentsOfStringToFile( char* FileName, char* Buffer )
{
  FILE		*StreamW;
  int		i;
  int		C;
  char		Message[80];
  char		ErrorMessage[100];

  i = 0;
  StreamW = fopen( FileName, "a" );
  if( StreamW == NULL ) {
	perror( ErrorMessage );
	SError_( Warning_, ErrorMessage, "STR57" );
	sprintf( Message, "Failed to open file %s", FileName );
	SError_( Warning_, Message, "STR56" );
	strcpy( Buffer, " " );
	return;
  }
  while( 1 ) {
	C = Buffer[i];
	if( C == '\0' )break;
	fputc( C, StreamW );
	i++;
  }
  fclose( StreamW );
}

/* Remove lines in 'String' beginning with the sequence 'Sequence' */

void	STR_RemoveLinesBeginningWith( char* String, char* Sequence )
{
  char*		Line;
  char*		LineNoSpaceBeg;
  char*		Buffer;
  Bool_		Copy;
  Bool_		Exit;
  int		i;
  int		j;
  int		k;

  Line = SBYTES_( strlen( String ) + 10, "STR38" );
  LineNoSpaceBeg = SBYTES_( strlen( String ) + 10, "STR39" );
  Buffer = SBYTES_( strlen( String ) + 10, "STR40" );
  i = 0;
  Exit = False;
  Buffer[0] = '\0';
  do {
	k = 0;
	do {
		Line[k] = String[i];
		i++;  k++;
	} while( String[i-1] != '\n' && String[i-1] != '\0' );
	if( Line[k-1] == '\0' ) {
		Exit = True;
	} else {
		Line[k] = '\0';
	}
	strcpy( LineNoSpaceBeg, Line );
	STR_Shuffle( LineNoSpaceBeg );
	j = 0;
	Copy = False;
	while( Sequence[j] != '\0' ) {
		if( Sequence[j] == LineNoSpaceBeg[j] ) {
			j++;
		} else {
			Copy = True;
			break;
		}
	}
	if( Copy == True ) {
		strcat( Buffer, Line );
	}
  } while( Exit == False );
  strcpy( String, Buffer );
  FREE_( Buffer );
  FREE_( LineNoSpaceBeg );
  FREE_( Line );
}

/*  Overwrites in 'String' at location 'Loc' the text in string 'Over' */

void		STR_OverWriteLocInLine( char* String, char* Over, int Loc )
{
  Bool_		Exit;
  int		i;
  int		j;
  int		k;
  int		l;
  Bool_		Copy;

  i = 0;
  Exit = False;
  do {
	k = 0;
	do {
		if( k == Loc ) {
			Copy = True;
			for( l=i; l< i + strlen( Over ); l++ ) {
				if( String[l] != ' ' && String[l+1] != ' ' ) {
					Copy = False;
					break;
				}
			}
			j = 0;
			if( Copy == True ) {
				while( Over[j] != '\0' ) {
					String[i] = Over[j];
					i++; j++;
				}
			}
		}
		i++; k++;
	} while( String[i-1] != '\n' && String[i-1] != '\0' );
	if( String[i-1] == '\0' ) {
		Exit = True;
	}
  } while( Exit == False );
}

/* Returns True if 'MinorString' exists in 'MainString' without characters
   being adjacent to it in 'MainString' */

Bool_		STR_DetectUniqueAlphaString( char* MainString, char* MinorString )
{
  Bool_		Found;
  int		i;
  int		j;

  i = 0;
  j = 0;
  Found = False;
  while( MainString[i] != '\0' ) {
	if( MainString[i] == MinorString[j] && Found == False ) {
		i++; j++;
	} else if( isalpha( MainString[i] ) != 0 ) {
		return False;
	} else {
		if( j != 0 && Found == False ) {
			return False;
		}
		i++;
	}
	if( MinorString[j] == '\0' ) {
		Found = True;
	}
  }
  return Found;
}

/* Returns True if the next is a blank line after 'Pos' */

Bool_		STR_IsNextBlankLine( char* String, int Pos )
{
  int	i;

  i = Pos;
  while( isspace( String[i] ) != 0 && String[i] != '\0' ){
	if( String[i] == '\n' && String[i+1] == '\n' ) {
		return True;
	} else if( String[i] == '\n' ) {
		return False;
	}
	i++;
  }
  return False;
}

/* Returns True if there are no set characters in the format given */

Bool_		STR_SetCharsInFormat( char* Format )
{
  int		i;

  i = 0;
  while( Format[i] != '\0' ) {
	if( Format[i] == '%' ) {
		i++;
		continue;
	}
	if( isspace( Format[i] ) == 0 ) {
		return True;
	}
	i++;
  }
  return False;
}

/* Returns the length of the first occurrence of a sequence of 'C' characters in 'String' */

int		STR_LengthOfOccurrenceOfCharacter( char* String, char C )
{
  int	Count;
  int	i;

  Count = 0;
  i = 0;
  while( String[i] != '\0' ) {
	if( String[i] == C ) {
		Count++;
		i++;
		while( String[i] == C ) {
			Count++;
			i++;
		}
		return Count;
	} else {
		i++;
	}
  }
  return Count;
}

/* Returns False if any the characters in 'String' are alphanumeric or punctuation or symbol */

Bool_		STR_AllNonePrintChars( char* String )
{
  int	i;

  i = 0;
  while( String[i] != '\0' ) {
	if( isgraph( String[i] ) != 0 ) {
		return False;
	}
	i++;
  }
  return True;
}

/* Returns the maximun number of contiguous characters in a string that do not
   include a space or a tab */

int	STR_MaxTermLength( char* String ) 
{
  int	Count;
  int	CumulativeCount;
  int	i;

  Count = 0;
  CumulativeCount = 0;
  i = 0;
  while( String[i] != '\0' ) {
	if( isspace( String[i] ) == 0 ) {
		Count++;
		if( CumulativeCount < Count ) {
			CumulativeCount = Count;
		}
	} else {
		Count = 0;
	}
	i++;
  }
  return CumulativeCount;
}


/* Removes all characters including 'Marker' to the point in 'String' where marker occurs */

void	STR_RemoveCharsUpToMarker( char* String, char* Marker )
{
  int		i;
  int		j;
  int		k;
  Bool_		Found;
  char*		Buffer;

  Found = False;
  i = 0;
  j = 0;
  k = 0;
  Buffer = SBYTES_( strlen( String ) + 2, "STR41" );
  while( String[i] != '\0' ) {
	if( Marker[j] == '\0' ) {
		Found = True;
		Buffer[k] = String[i];
		k++;
	} else if( Marker[j] == String[i] ) {
		j++;
	} else {
		j = 0;
	}
	i++;
  }
  if( Found == True ) {
	Buffer[k] = '\0';
  	strcpy( String, Buffer );
  }
}

/* Returns first format type ( %d, %s or %n in 'String' ) in 'Format'. Returns False if no format type in 'String' */

Bool_		STR_ReturnFirstFormatType( char* String, char* Format )
{
  int		i;

  i = 0;
  if( String[0] == '\0' ) {
	return False;
  }
  while( String[i + 1] != '\0' ) {
	if( String[i] == '%' && String[i+1] == 'd' ) {
		strcpy( Format, "%d" );
		return True;
	}else if( String[i] == '%' && String[i+1] == 's' ) {
		strcpy( Format, "%s" );
		return True;
	}else if( String[i] == '%' && String[i+1] == 'n' ) {
		strcpy( Format, "%n" );
		return True;
	}
	i++;
  }

  return False;
}

/* STR_AddCharacterToEnd: This take two parameters.  The first one is a
   string, and the second one is the character that is to be added to the
   end of the string.  This function is preferable to strncat(string1,
   string2, 1) as it will maintain the '\0' at the end of the string */
void
STR_AddCharacterToEnd(char *s, char c)
{
    while(*s)
    {
	s++;
    }
    *s++ = c;
    *s = '\0';
}


/* STR_RemoveLeadingSpaces: remove all the leading spaces and tabs from the
   input string.  Return a pointer to the first non white space character */
char * 
STR_RemoveLeadingSpaces(char *s)
{
    char * ptr = s;
    
    while (*ptr == ' ' || *ptr == '\t')
    {
	ptr++;
    }
    return ptr;
}

/* STR_CaseChar: This is an non case sensitive version of strchr */
char * 
STR_CaseChar(char *s, char c)
{
    int lowerC = tolower(c);
    
    while (*s != '\0' && tolower(*s) != lowerC)
    {
	s++;
    }
    return (*s=='\0')?NULL:s;
}

/* STR_StringNotIn: This checks all the characters in the second string to
   make sure they are the only characters in the first string */
int
STR_StringNotIn(char *s1, char *s2)
{
    int found = 0;

    while (*s1 != '\0')
    {
	if (strchr(s2, *s1) == NULL)
	{
	    found = 1;
	    break;
	}
	s1++;
    }
    return found;
}

/* Replaces character 'ToReplace' with 'ReplaceWith' 'NumberOfChanges' of times in 'String', where 'ToReplace' is not enclosed by 'Encloser'. Replacing  starts from the end of the string if 'Backward' is True.  */

void	STR_ChangeChars( char* String, int NumberOfChanges, Bool_ Backward, char* Encloser, char ToReplace, char ReplaceWith )
{
  Bool_		Enclosed;
  int		i;
  int		j;
  int		Length;

  j = 0;
  if( Backward == True ) {
	i = strlen( String ) - 1 ;
	Enclosed = False;
	while( i >= 0 ) {
		if( String[i] == ToReplace && Enclosed == False ) {
			String[i] = ReplaceWith;
			j++;
		} else if( Encloser == NULL ) {
			Enclosed = False;
		} else if( String[i] == Encloser[0] ) {
			if( Enclosed == False ) {
				Enclosed = True;
			} else {
				Enclosed = False;
			}
		}
		if( j == NumberOfChanges ) {
			break;
		}
		i--;
	}
  } else if( Backward == False ) {
	i = 0;
	Enclosed = False;
	Length = strlen( String );
	while( i < Length ) {
		if( String[i] == ToReplace && Enclosed == False ) {
			String[i] = ReplaceWith;
			j++;
		} else if( Encloser == NULL ) {
			Enclosed = False;
		} else if( String[i] == Encloser[0] ) {
			if( Enclosed == False ) {
				Enclosed = True;
			} else {
				Enclosed = False;
			}
		}
		if( j == NumberOfChanges ) {
			break;
		}
		i++;
	}
  }
}

/* Extracts the last integer number in the String and places in LastNumberC */

void		STR_GetLastNumber( char* String, char* LastNumberC )
{
  int	i;
  int	j;

  j = 0;
  i = strlen( String ) - 1;
  while( isdigit( String[i] ) == 0 ) {
	i--;
	if( i < 0 ) {
		return;
	}
  }
  while( isdigit( String[i] ) != 0 ) {
	LastNumberC[j] = String[i];
	i--;
	j++;
	if( i < 0 ) {
		break;
	}
  }
  LastNumberC[j] = '\0';
  STR_Reverse( LastNumberC );
}


/* Reverses the order of the characters in the String */

void		STR_Reverse( char* String )
{
  int		i;
  int		j;
  char*		Buffer;

  Buffer = SBYTES_( strlen( String ) + 2, "STR42" );
  j = 0;
  i = strlen( String ) - 1;
  while( i >= 0 ) {
	Buffer[j] = String[i];
	j++;	i--;
  }
  Buffer[j] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* If the length of 'Command' is greater than 'MaxCharsInACommandline', then returns the position before 'MaxCharsInACommandline' where the last blank occurred, else returns -1 */ 

int	STR_LocateBlank( char* Command, int Start, char Encloser, int MaxCharsInACommandline )
{
  int	i;
  int	LastPos;
  Bool_	Enclosed;

  i = Start;
  Enclosed = False;
  LastPos = Start + MaxCharsInACommandline - 3;
  while( Command[i] != '\0' ) {
	if( Enclosed == False && Command[i] == ' ' ) {
		if( i - Start > MaxCharsInACommandline ) {
			return LastPos;
		} else {
			LastPos = i;
		}
	}
	i++;
	if( Command[i] == Encloser ) {
		if( Enclosed == True ) {
			Enclosed = False;
		} else {
			Enclosed = True;
		}
	}
  }
  return (-1);
}

/* Removes each occurrence of character 'C' in 'String' */

void		STR_RemoveGivenChar( char* String, char C )
{
  char*		Buffer;
  int		i;
  int		j;

  Buffer = SBYTES_( strlen( String ) + 2, "STR43" );
  i = 0;
  j = 0;
  do {
	if( String[i] != C ) {
		Buffer[j] = String[i];
		j++;
	}
	i++;
  } while( String[i-1] != '\0' );
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* Removes enclose characters 'Enclose' from 'String' */

void		STR_RemoveEncloseChars( char* String, char Enclose )
{
  int		StartEnc;
  int		EndEnc;
  int		i;
  Bool_		AlnumFound;

  i = 0;
  StartEnc = (-1);
  EndEnc = (-1);
  AlnumFound = False;
  STR_RemoveDoubles( String, Enclose );
  while( String[i] != '\0' ) {
	if( String[i] == Enclose ) {
		if( StartEnc == (-1) && AlnumFound == False ) {
			StartEnc = i;
		} else {
			EndEnc = i;
		}
	} else if( isspace( String[i] ) == 0 ) {
		AlnumFound = True;
	}
	i++;
  }
  if( StartEnc != (-1) && EndEnc != (-1) ) {
	String[ StartEnc ] = ' ';
	String[ EndEnc ] = ' ';
	STR_Shuffle( String );
	STR_RemoveEndSpaces( String );
  }
}

/* Removes zeros before the character representation of an integer in 'String'  e.g. String is "000055" : String becomes "55" */

void		STR_RemoveLeadingZeros( char* String )
{
  int		i;
  int		j;
  char*		Buffer;
  Bool_		PrevDigit;

  i = 0;
  j = 0;
  PrevDigit = False;
  Buffer = SBYTES_( strlen( String ) + 2, "STR44" );
  while( String[i] != '\0' ) {
	if( String[i] == '0' && PrevDigit == False ) {
		PrevDigit = False;
	} else if( isdigit( String[i] ) != 0 ) {
		Buffer[j] = String[i];
		PrevDigit = True;
		j++;
	} else {
		Buffer[j] = String[i];
		PrevDigit = False;
		j++;
	}
	i++;
  }
  Buffer[j] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}


/* Inserts zeros before the character representation of an integer in 'String' such that the total length of the character representation of the number is 'Number'  e.g. String is "55" Number is 6 : String becomes "000055" */

void		STR_InsertZeros( char* String, int Number )
{
  char*		Buffer;
  int		i,j,k;
  int		Count;
  int		NoZeros;

  Buffer = SBYTES_( strlen( String ) + 10, "STR45" );

  STR_Reverse( String );
  i = 0;
  j = 0;
  Count = 0;
  do{
	if( isdigit( String[i] ) != 0 ) {
		Count++;
	} else {
		if( Count != 0 ) {
			NoZeros = Number - Count;
			if( NoZeros >= 1 ) {
				for( k = 1; k<= NoZeros; k++ ) {
					Buffer[j] = '0';
					j++;
				}
			}
			Count = 0;
		}
	}
	Buffer[j] = String[i];
	i++;  j++;
  }while( String[i-1] != '\0' );
  strcpy( String, Buffer );
  STR_Reverse( String );
  FREE_( Buffer );
}



/*
  copies n charachters from s2 to s1 and puts a '\0' after the n characters
  on s1 that is s1 is cleaned of what it contained before
*/
void	STR_ncpy(char *s1,char *s2,int n)
{
int counter;
char *ptr1,*ptr2;

	ptr1 = s1;
	ptr2 = s2;
	counter = 0;
	while(*ptr2 != '\0' && counter < n) {
		*ptr1 = *ptr2;
		++ptr1;
		++ptr2;
		++counter;
	}
	*ptr1 = '\0';
}


/*------------
1) The next is a routine to get a field inside a 
string.
The strtok routine has not been used because we are 
not sure on field separators.
	I added the follwing lines in the str_hand.c 
file.
*/
/*
Returns the nth field in the string s
*/
char *STR_GetField(char *s,int n)
{
int field_counter,len,ss_len;
char *ptr,*substr,*base_ptr,*ss_ptr;

	if(s == NULL || n == 0 || substr == NULL)
		return(NULL);
/* reach the requested field */
	ptr = s;
	field_counter = 0;
	while(*ptr != '\0' && field_counter < n) {
		while(isgraph(*ptr) == 0 && *ptr != '\0') 
			++ptr;
		if(*ptr == '\0')
			return(NULL);
		++field_counter;
		if(field_counter < n) {
			while(isgraph(*ptr) != 0 && *ptr != '\0') 
				++ptr;
		}
	}
	if(*ptr == '\0')
		return(NULL);
/* compute the field length */
	base_ptr = ptr;
	len =0;
	while(isgraph(*ptr) != 0 && *ptr != '\0') {
		++len;
		++ptr;
	}
/* get the field */
	ptr = base_ptr;
	substr = (char * )SBYTES_(len + 1, "STR46");
	ss_ptr = substr;
	ss_len = 0;
	while(ss_len < len) {
		*ss_ptr = *ptr;
		++ss_len;
		++ss_ptr;
		++ptr;
	}
	*ss_ptr = '\0';
	return(substr);
}

/* Returns a numeric character representation of a month in 'MonthNo' when given a 3 letter representation of the month in 'Month' */

void	  STR_MonthToNumber( char* Month, char* MonthNo )
{
  int	i;
  char*	MonthC[]={ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", 
		   "Aug", "Sep", "Oct", "Nov", "Dec" };

  char*	MonthN[]={ "01", "02", "03", "04", "05", "06", "07", 
			"08", "09", "10", "11", "12" };

  strcpy( MonthNo, "00" );
  for( i=0; i<12; i++ ) {
	if( strcmp( Month, MonthC[i] ) == 0 ) {
		strcpy( MonthNo, MonthN[i] );
		return;
	}
  }
}

/* Place text from 'String' upto the first character 'Mask' or '\0' into 'Text'. If text is placed into 'Text' return True. */

Bool_		STR_GetNonMask( char* String, char Mask, char* Text )
{
  Bool_		Fill;
  int		i;
  int		j;

  i = 0;
  j = 0;
  Fill = False;
  while( String[i] != '\0' ) {
	if( String[i] == Mask ) {
		if( Fill == True ) {
			Text[j] = '\0';
			return True;
		} else {
			i++;
			continue;
		}
	} else {
		Text[j] = String[i];
		i++;
		j++;
		Fill = True;
	}
  }
  if( Fill == True ) {
	Text[j] = '\0';
	return True;
  } else {
	return False;
  }
}

/* If a word(s) in 'String' is/are greater than 'Size', (a) space(s) is/are inserted into 'String' such that no word is greater than 'Size' */ 

void	STR_InsertSpacesInLongWords( char* String, int Size )
{
  int		i;
  int		j;
  int		k;
  char*		Buffer;

  i = 0;
  j = 0;
  k = 0;
  Buffer = SBYTES_( 2 * strlen( String ), "STR47" );
  while( String[i] != '\0' ) {
	if( String[i] == ' ' || String[i] == '\t' ) {
		k = 0;
	} else if( k == Size ) {
		Buffer[j] = ' ';
		j++;
		k = 0;
	} else {
		k++;
	}
	Buffer[j] = String[i];
	i++; j++;
  }
  Buffer[j] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* Returns a pointer to the first non white space character in 'String' */

char*	STR_PointerToFirstNonWhiteSpace( char* String )
{
  char*		pString;

  pString = String;
  if( *pString == '\0' ) {
	return String;
  }
  while( isspace( *pString ) != 0 ) {
	pString++;
	if( *pString == '\0' ) {
		return String;
	}
  }
  return pString;
}

/* Returns True if the last line of 'String' contains a prompt of the form "?" or ">" */

Bool_	STR_PromptAtEnd( char* String )
{
  int		Length;
  char*		LastLine;
  int		i,j,k;
  char*		PosPrompts[]={ "?", ">" };

  Length = strlen( String );
  if( Length == 0 ) {
	return False;
  }
  LastLine = SBYTES_( strlen( String ) + 2, "STR48" );
  i = Length - 1;
  j = 0;
  while( i >= 0 ) {
	if( String[i] == '\n' ) {
		break;
	}
	LastLine[j] = String[i];
	i--;  j++;
  }
  LastLine[j] = '\0';
  STR_Reverse( LastLine );
  STR_Shuffle( LastLine );
  STR_RemoveEndSpaces( LastLine );
  for( k=0; k<2; k++ ) {
	if( strcmp( LastLine, PosPrompts[k] ) == 0 ){
		FREE_( LastLine );
		return True;
	}
  }
  FREE_( LastLine );
  return False;
}

/* Returns the position that the pointer at the start of 'String2' occurrs in 'String1' */

int	  STR_PositionOfPointer( char* String1, char* String2 )
{
  int	i;

  i = 0;
  while( *String1 != '\0' ) {
	if( String1 == String2 ) {
		return i;
	}
	String1++;  i++;
  }
  return (-1);
}

/* Inserts character 'C' at the beginning of each line of 'String' before position 'Pos' */

void	STR_InsertCAtBeginningOfLine( char* String, int Pos, char C )
{
  char		Insert[5];

  Insert[0] = C;
  Insert[1] = '\0';
  while( Pos >= 0 ) {
	if( String[ Pos ] == '\n' ) {
		Pos++;
		STR_InsertString( String, Insert, Pos );
		return;
	}
	Pos--;
  }
  STR_InsertString( String, Insert, 0 );
}
/* Truncates lines greater than 'LineSize' in 'String' */

void	STR_LineTruncation( char* String, int LineSize )
{
  char*		Buffer;
  int		i,j,k;

  Buffer = SBYTES_( strlen( String ) + 10, "STR49" );
  i = 0;
  j = 0;
  k = 0;
  while( String[i] != '\0' ){
	if( String[i] == '\n' ) {
		k = 0;
		Buffer[j] = String[i];
		j++;
	} else if( k < LineSize ) {
		Buffer[j] = String[i];
		j++;
		k++;
	}
	i++;
  }
  Buffer[j] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}


/* Calculates the maximum length a format string e.g. "text %d %s %n" is can be */

int	STR_CalculateMaxLength( char* String )
{
  int		i;
  int		Length;

  i = 0;
  Length = strlen( String );
  while( String[i] != '\0' ) {
	if( String[i] == '%' ) {
		if( String[i+1] == 'd' ) {
			Length = Length + 20;
		} else if( String[i+1] == 's' ) {
			Length = Length + 80;
		} else if( String[i+1] == 'n' ) {
			Length = Length + 300;
		}
	}
	i++;
  }
  return Length;
}

/* Removes spaces after character 'C' on each line */

void	STR_RemoveSpacesFollowing1stSpecCharOnEachLine( char* String, char C )
{
  char*		Buffer;
  Bool_		NewLine;
  int		i,j;

  Buffer = SBYTES_( strlen( String ) + 10, "STR50" );
  i = 0;
  j = 0;
  NewLine = True;
  while( String[i] != '\0' ){
	Buffer[j] = String[i];
	if( String[i] == C && NewLine == True ) {
		while( String[i+1] == ' ' || String[i+1] == '\t' ) {
			if( String[i+1] != '\n' ) {
				i++;
			}
		}
		NewLine = False;
	} else if( String[i] == '\n' ) {
		NewLine = True;
	}
	i++;  j++;
  }
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* Returns the last printable character apart from WS in 'String' */


char	STR_LastPrintCharacterInString( char* String )
{
  int	i;

  i = strlen( String ) - 1;
  while( i >= 0 ) {
	if( isspace( String[i] ) == 0 ) {
		return String[i];
	}
	i--;
  }
  return '\0';
}

/* Replaces the last printable character apart from WS in 'String' with 'C' */

void	STR_ReplaceLastPrintCharacterInString( char* String, char C )
{
  int	i;

  i = strlen( String ) - 1;
  while( i >= 0 ) {
	if( isspace( String[i] ) == 0 ) {
		String[i] = C;
		return;
	}
	i--;
  }
}

/* Changes occurrences of 'Old' for 'New' in 'String' where 'Old' is not enlosed by 'Encloser' */


void	STR_ExchangeChars( char* String, char Old, char New, char Encloser )
{


  int		i;

  i = 0;
  while( String[i] != '\0' ){
	if( String[i] == Encloser ) {
		i++;
		while( String[i] != Encloser ) {
			i++;
			if( String[i] == '\0' ) {
				return;
			}
		}
	}
	if( String[i] == Old ) {
		String[i] = New;
	}
	i++;
  }
}

/* Concatonates text from a file 'Stream' being read up to the next '\n' or EOF and concatonates onto 'String' */

void	STR_GetToEndOfLineAndConcatonate( FILE* Stream, char* String )
{
	int	i;
	int	C;

	i = strlen( String );
	while( 1 ) {
		C = fgetc( Stream );
		if( C == '\n' || C == EOF ) {
			break;
		} else {
			String[i] = C;
			i++;
		}
	}
	String[i] = '\0';
}

/* Changes occurrences of 'Old' for 'New' in 'String' where 'Old' is not enlosed by 'Encloser' */

void	STR_ExchangeStrings( char* String, char* Old, char* New, char Encloser )
{


  int		i,j,k;
  char*		Buffer;
  int		Holdi;
  Bool_		Found;

  Buffer = SBYTES_( strlen( String ) + 500, "STR51" );
  i = 0;
  k = 0;
  while( String[i] != '\0' ){
	if( String[i] == Encloser ) {
		Buffer[k] = String[i];
		i++;   k++;
		while( String[i] != Encloser ) {
			Buffer[k] = String[i];
			i++;   k++;
			if( String[i] == '\0' ) {
				Buffer[k] = '\0';
				strcpy( String, Buffer );
				FREE_( Buffer );
				return;
			}
		}
	}
	Found = False;
	Holdi = i;
	j = 0;
	while( String[i] == Old[j] ) {
		i++; j++;
		if( Old[j] == '\0' ) {
			Found = True;
		} else if( String[i] == '\0' ) {
			break;
		}
	}
	if( Found == False ) {
		i = Holdi;
		Buffer[k] = String[i];
		i++;   k++;
	} else {
		Buffer[k] = '\0';
		strcat( Buffer, New );
		k = strlen( Buffer );
	}
  }
  Buffer[k] = '\0';
  strcpy( String, Buffer );
  FREE_( Buffer );
}

/* Performs in the same way as 'C' function 'strstr' but registers a comparison if case of the letters are different in compared strings */


char*	STR_strstrCaseInsensitive( char* String1, char* String2 )
{
  char*		Save1;
  char*		Save2;
  char*		Point;

  Save1 = STR_Save( String1 );
  String2 = STR_Save( String2 );
  STR_ToLower( String1 );
  STR_ToLower( String2 );
  Point = strstr( String1, String2 );
  strcpy( String1, Save1 );
  FREE_( Save1 );
  FREE_( String2 );
  return Point;
}

/* Performs in the same way as 'C' function 'strcmp' but registers a comparison if case of the letters are different in compared strings */


int	STR_strcmpCaseInsensitive( char* String1, char* String2 )
{
  char*		Save1;
  char*		Save2;
  int		Point;

  Save1 = STR_Save( String1 );
  Save2 = STR_Save( String2 );
  STR_ToLower( Save1 );
  STR_ToLower( Save2 );
  Point = strcmp( Save1, Save2 );
  FREE_( Save1 );
  FREE_( Save2 );
  return Point;
}

/* Checks to see if the file 'FileName' exists in the directory 'Path'. 'FileName' may include wild characters '*'. If it does it returns the actual name of the file. If not returns NULL */

char*	STR_FindFileName( char* Path, char* FileName )
{
  int  i,j,k;
  char* FNActual;
  int fd,nread;
  int	CharNo;

  char*		Buffer;

  Buffer = SBYTES_( 100002, "STR52" );
  if((fd=open(Path, 0))== (-1)){
	printf( "Could not open\n");
	exit(-1);
  }
  nread = read(fd, Buffer, 100000 );
  CharNo = nread;
/*  printf( "nread %d\n", nread  );  */
  FNActual = SBYTES_( strlen( FileName ) + 256, "STR53" );
  i = 0;
  j = 0;
  k = 0;
  for( i=0; i<CharNo; i++ ) {
/*	if( isprint( Buffer[i] ) != 0 ) {
		printf( " %c %c\n", Buffer[i], FileName[j]  );
	}  */
	if( FileName[j] == '*' ) {
		if( FileName[j+1] == '\0' && isprint( Buffer[i] ) == 0 ) {
			FNActual[k] = '\0';
			FREE_( Buffer );
			return FNActual;
		} else if( STR_Correspond( &Buffer[i], &FileName[j+1] ) == True ) {
			FNActual[k] = Buffer[i];
			j++;  j++; k++;
		} else if( isprint( Buffer[i] ) != 0 ) {
			FNActual[k] = Buffer[i];
			k++;
		} else {
			j = 0;
			k = 0;
		}
	} else if( FileName[j] == '\0' ) {
		FNActual[k] = '\0';
		FREE_( Buffer );
		return FNActual;
	} else {
		if( Buffer[i] == FileName[j] ) {
			FNActual[k] = Buffer[i];
			k++;  j++;
		} else {
			j = 0;
			k = 0;
		}
	}
  }
  FREE_( Buffer );
  return NULL;
}

/* Returns True if if String2 corresponds from the start of String1 up to the first '\0' or '*' character in String2 */

Bool_   STR_Correspond( char* String1, char* String2 )
{
  int 	i;

  i = 0;
  while( String2[i] != '\0' && String2[i] != '*') {
	if( String2[i] != String1[i] ) {
		return False;
	}
	i++;
  }
  return True;

}


char*	STR_AnyFormatMarkers( char* String )
{
  char*		CPoint;

  if( ( CPoint = strstr( String, "%d" ) ) != NULL ) { 
	return CPoint;
  } else if( ( CPoint = strstr( String, "%s" ) ) != NULL ) { 
	return CPoint;
  } else if( ( CPoint = strstr( String, "%n" ) ) != NULL ) { 
	return CPoint;
  }
  return NULL;
}

/* Removes a file which contains no printing characters */

void	STR_RemoveEmptyFile( char* FileName )
{
  FILE		*StreamR;
  int		i;
  int		C;
  char		Message[100];

  i = 0;
  StreamR = fopen( FileName, "r" );
  if( StreamR == NULL ) {
	return;
  }
  while( 1 ) {
	C = getc( StreamR );
	if( C == EOF )break;
	if( isgraph( C ) != 0 ) {
		fclose( StreamR );
		return;
	}
  }
  fclose( StreamR );
  if( unlink( FileName ) != 0 ) {
	sprintf( Message, "Could not remove file %s", FileName );
	SError_( Fatal_, Message, "STR54" );
  }
}

/* Returns number of day in 'Year' from given 'Day' and 'Month' */


int	STR_MonthDayToDayInYear( char* Month, char* Day, char* Year )
{
  int		Leap;
  int		YearI;


  YearI = STR_CharToInt( Year );
  if( YearI > 94 && YearI < 100 ) {
	YearI = YearI + 1900;
  } else if( YearI < 100 ) {
	YearI = YearI + 2000;
  }
  if( ( YearI % 4 == 0 && YearI % 100 != 0 ) || YearI % 400 == 0 ) {
	Leap = 1;
  } else {
	Leap = 0;
  }

  if( STR_strcmpCaseInsensitive( Month, "JAN" ) == 0 || strcmp( Month, "1" ) == 0 ) {
	return STR_CharToInt( Day );
  } else if( STR_strcmpCaseInsensitive( Month, "FEB" ) == 0 || strcmp( Month, "2" ) == 0 ) {
	return ( 31 + STR_CharToInt( Day ) );
  } else if( STR_strcmpCaseInsensitive( Month, "MAR" ) == 0 || strcmp( Month, "3" ) == 0 ) {
	return ( 59 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "APR" ) == 0 || strcmp( Month, "4" ) == 0 ) {
	return ( 90 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "MAY" ) == 0 || strcmp( Month, "5" ) == 0 ) {
	return ( 120 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "JUN" ) == 0 || strcmp( Month, "6" ) == 0 ) {
	return ( 151 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "JUL" ) == 0 || strcmp( Month, "7" ) == 0 ) {
	return ( 181 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "AUG" ) == 0 || strcmp( Month, "8" ) == 0 ) {
	return ( 212 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "SEPT" ) == 0 || strcmp( Month, "9" ) == 0 ) {
	return ( 243 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "OCT" ) == 0 || strcmp( Month, "10" ) == 0 ) {
	return ( 273 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "NOV" ) == 0 || strcmp( Month, "11" ) == 0 ) {
	return ( 304 + STR_CharToInt( Day ) + Leap );
  } else if( STR_strcmpCaseInsensitive( Month, "DEC" ) == 0 || strcmp( Month, "12" ) == 0 ) {
	return ( 334 + STR_CharToInt( Day ) + Leap );
  }
  return 0;
}

/* Gets line number 'LineNumber' from 'Buffer' and copies it into 'LineText' */

Bool_	STR_GetLineNumber( char* Buffer, int LineNumber, char* LineText )
{
  int		i;
  int		j;
  int		LineCount;

  i = 0;
  j = 0;
  LineCount = 1;
  do {
	if( Buffer[i] == '\n' || Buffer[i] == '!' ){
		if( LineNumber == LineCount ){
			LineText[j] = '\0';
			return True;
		} else {
			LineCount++;
			j = 0;
		}
	} else if( Buffer[i] == '\0' ){
		if( LineNumber == LineCount ){
			LineText[j] = '\0';
			return True;
		} else {
			return False;
		}
	} else {
		LineText[j] = Buffer[i];
		j++;
	}
	i++;
  } while( 1 );

}


/* Encloses all occurrences of 'Word' in 'String' with 'Enclose'. e.g. where String = "abcdefgabcfghj", Word = "abc" and Enclose = "'" then 'String' becomes 
"'abc'defg'abc'fghj". Spaces at the front and end of word are treated in the following way demonstrated by the example :  String = " abc defg abc fghj", Word = " abc " and Enclose = "'" then 'String' becomes 
" 'abc' defg 'abc' fghj". */


void	STR_EncloseWord( char* String, char* Word, char* Enclose )
{
  int	i,j,k,l;

  i = 0;
  j = 0;
  k = (-1);
  while( String[i] != '\0' ) {
	if( String[i] == Word[j] ) {
		if( k == (-1) ) {
			k = i;
		}
		j++;
	} else if( Word[j] == '\0' ) {
		if( Word[0] == ' ' || Word[0] == '\t' ) {
			k = k + 1;
		}
		if( Word[j-1] == ' ' || Word[j-1] == '\t' ) {
			l = i;
		} else {
			l = i + 1;
		}
		STR_InsertString( String, Enclose, k );
		STR_InsertString( String, Enclose, l );
		j = 0;
		k = (-1);
	} else {
		j = 0;
		k = (-1);
	}
	i++;
  }
}

/* Copies 'String' int 'StringEndIntsRemoved' removing any digits from the end of 'StringEndIntsRemoved' */

void	STR_RemoveIntegersAtEnd( char* String, char* StringEndIntsRemoved )
{
  int	i;

  strcpy( StringEndIntsRemoved, String );
  i = strlen( String ) - 1;
  while( isdigit( StringEndIntsRemoved[i] ) != 0 ) {
	StringEndIntsRemoved[i] = '\0';
	i--;
	if( i == 0 ) {
		SError_( Fatal_, "Unexpectedly input string is all digits", "STR55" );
	}
  }
}

/* Returns the number of times character 'C' occurs in 'String' */


int		STR_NumberOfOcurrencesChar( char* String, char C )
{
  int		i;
  int		Count;

  i = 0;
  Count = 0;
  while( String[i] != '\0' ) {
	if( String[i] == C ) {
		Count++;
	}
	i++;
  }
  return Count;
}

/* Removes spaces from the end of lines */

void		STR_RemoveSpacesFromEOLs( char* String )
{
  int		i;
  int		j;
  Bool_		EndOfLine;
  Bool_		Space;

  STR_Reverse( String );

  i = 0;
  j = 0;
  EndOfLine = True;
  while( String[i] != '\0' ) {
	if( String[i] == '\n' ) {
		EndOfLine = True;
		Space = False;
	} else if( String[i] == ' ' ) {
		Space = True;
	} else {
		Space = False;
		EndOfLine = False;
	}
	if( EndOfLine == True && Space == True ) {
		i++;
	} else {
		String[j] = String[i];
		i++;  j++;
	}
  }
  String[j] = '\0';
  STR_Reverse( String );
}




/*
12345678901234567890123456789012345678901234567890123456789012345678901234567890
         1         2         3         4         5         6         7         8
*/
