/*  Module Name: @(#)target-sm.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: Target state machine code for DBV-OSI II project.
*/

#include "target-sm.h"



char DBVTargetStateErrorString[][DBV_SM_STRLEN] =
        {"","DBV_TS_CLOSED","DBV_TS_ASSOC_INPROGRESS","DBV_TS_ASSOCIATION",
         "DBV_TS_INIT_RECVD","DBV_TS_OPEN","DBV_TS_SEARCH_RECVD",
         "DBV_TS_PRESENT_RECVD","DBV_TS_DELETE_RECVD","DBV_TS_SCAN_RECVD",
         "DBV_TS_RESOURCE_REPORT_RECVD","DBV_TS_SORT_RECVD",
         "DBV_TS_EXTENDED_SERVICE_RECVD","DBV_TS_ARELEASE_REQ_RECVD",
         "DBV_TS_CLOSE_SENT","DBV_TS_CLOSE_RECVD","DBV_TS_ERROR","DBV_TS_DEAD",
         
	   "DBV_TS_ACCCNT_SENT", "DBV_TS_RESCNT_SENT", "DBV_TS_TRIGRESCNT_RECVD"};


char DBVTargetEventErrorString[][DBV_SM_STRLEN] =
        {"","DBV_TE_AASOCREQ_IND","DBV_TE_IASSOCRESP_TRUE",
         "DBV_TE_IASSOCRESP_FALSE","DBV_TE_INITREQ_PDU","DBV_TE_INITRESP_TRUE",
         "DBV_TE_INITRESP_FALSE","DBV_TE_SEARCHREQ_PDU","DBV_TE_SEARCHRESP",
         "DBV_TE_PRESENTREQ_PDU","DBV_TE_PRESENTRESP","DBV_TE_DELETEREQ_PDU",
         "DBV_TE_DELETERESP","DBV_TE_SCANREQ_PDU","DBV_TE_SCANRESP",
         "DBV_TE_RESOURCEREQ_PDU","DBV_TE_RESOURCERESP","DBV_TE_SORTREQ_PDU",
         "DBV_TE_SORTRESP","DBV_TE_SEGMENTREQ","DBV_TE_EXTENDEDREQ_PDU",
         "DBV_TE_EXTENDEDRESP","DBV_TE_AREL_IND","DBV_TE_IRELRESP",
         "DBV_TE_AABORT_IND","DBV_TE_IABORTREQ","DBV_TE_CLOSEREQ_PDU",
         "DBV_TE_CLOSEREQ","DBV_TE_CLOSERESP","DBV_TE_DECODE_FAILED",

         "DBV_OE_ACCCNTREQ", "DBV_OE_ACCCNTRSP_PDU", 
	   "DBV_OE_RESCNTREQ", "DBV_OE_RESCNTRSP_PDU", 
	   "DBV_OE_TRIGRESCNTREQ_PDU"};


/* 0 transition = protocol failed (state machine error) */
/* -1 transition = impossible (states and events start at 1) */
/* 98 transition = revert to previous state before access control request arrived */
/* 99 transition = revert to previous state before resource control request arrived 
 (99s for INITRESP_PDU_TRUE etc. allow reversion without resource control response
being formed (depends upons values in PDU!) */ 


/* start in a state (column) .. get an event (row) and move to new state (column) */

int DBVTargetStateTable[DBV_T_EVENTS][DBV_T_STATES] =
{
/* State ->            0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 */
/* Event \/       */ {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1},
/* AASOCREQ_IND   */ {-1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* IASSOCRESP_TRUE*/ {-1, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* ASSOCRESP_FALSE*/ {-1, 0,17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* INITREQ_PDU    */ {-1, 0, 0, 4,16,16,16,16,16,16,16,16,16, 0, 0,16, 0, 0, 0, 0, 0},
/* INITRESP_TRUE  */ {-1, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* INITRESP_FALSE */ {-1, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SEARCHREQ_PDU  */ {-1, 0, 0,16,16, 6,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* SEARCHRESP     */ {-1, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* PRESENTREQ_PDU */ {-1, 0, 0,16,16, 7,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* PRESENTRESP    */ {-1, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* DELETEREQ_PDU  */ {-1, 0, 0,16,16, 8,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* DELETERESP     */ {-1, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SCANREQ_PDU    */ {-1, 0, 0,16,16, 9,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* SCANRESP       */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* RESOURCEREQ_PDU*/ {-1, 0, 0,16,16,10,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* RESOURCERESP   */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SORTREQ_PDU    */ {-1, 0, 0,16,16,11,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* SORTRESP       */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* SEGMENTREQ     */ {-1, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
/* EXTENDEDREQ_PDU*/ {-1, 0, 0,16,16,12,16,16,16,16,16,16,16, 0,14,16, 0, 0, 0, 0, 0},
/* EXTENDEDRESP   */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0},
/* AREL_IND       */ {-1, 0, 0,13,16,13,16,16,16,16,16,16,16,16,16,16, 0, 0, 0, 0, 0},
/* IRELRESP       */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,17, 0, 0, 0, 0, 0, 0, 0},
/* AABORT_IND     */ {-1, 0,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17, 0, 0, 0, 0},
/* IABORTREQ      */ {-1, 0,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17, 0, 0, 0, 0},
/* CLOSEREQ_PDU   */ {-1, 0, 0,16,16,15,15,15,15,15,15,15,15, 0, 3, 0,15, 0, 0, 0, 0},
/* CLOSEREQ       */ {-1, 0, 0, 0, 0,14,14,14,14,14,14,14,14, 0, 0, 0,14, 0, 0, 0, 0},
/* CLOSERESP      */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0},
/* DECODE_FAILED  */ {-1, 0, 0,16,16,16,16,16,16,16,16,16,16, 0,16,16, 0, 0, 0, 0, 0},

/* ACCCNTREQ      */ {-1, 0, 0, 0,18, 0,18,18,18,18,18,18,18, 0, 0, 0, 0, 0, 0, 0, 0},
/* ACCCNTRSP_PDU  */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,98, 0, 0},

/* RESCNTREQ      */ {-1, 0, 0, 0,19, 0,19,19,19,19,19,19,19, 0, 0, 0, 0, 0, 0, 0, 0},
/* RESCNTRSP_PDU  */ {-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,99, 0},

/*TRIGRESCNTRE_PDU*/ {-1, 0, 0, 0, 0, 0, 6, 7, 8, 9,10,11,12,13,14,15, 0, 0, 0, 0, 0}
};




