/*  Module Name: @(#)target_z3950_access.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send an 
		  AccessControlRequest PDU to an origin and process a 
		  received AccessControlResponse PDU sent by an origin
		  

*/


#include "target_z3950_access.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE AccessControlRequest PDU TO SEND TO ORIGIN AND SEND IT 
*/

/* one of the few requests sent from a target to an origin */

int DbvAccessControlRequest(DbvAssocId lAssocId, DbvAccessControlReq *psAccessControlReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;


	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVAccessControlRequest",lAssocId);


        if (DBVCheckDbvAccessControlReqFields(lAssocId, psAccessControlReq, ppsSIError) == SI_OK)

		if (DBVChangeState(lAssocId, DBV_TE_ACCCNTREQ, ppsSIError) == SI_OK)

        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK) {
		DBVSendingTrace(lAssocId);
                DBVAccReqTrace(lAssocId, psAccessControlReq);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.accessControlRequest = (struct AccessControlRequest *) Asn1Alloc (sizeof(struct AccessControlRequest));
			CheckAsn1Alloc(PDU_ptr->a.accessControlRequest, env);
        		PDU_ptr->choiceId = PDU_ACCESSCONTROLREQUEST;
			DBVsetup_snacc_AccessControlRequest(PDU_ptr, psAccessControlReq, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVAccessControlRequest",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvAccessControlReqFields(DbvAssocId lAssocId, DbvAccessControlReq *psAccessControlReq, DbvError **ppsSIError)
{

	/* o */
	/* Dont have to check psReferenceId */

	/* c */
	if (DBVcheck_DbvSecurityChallengeReq(lAssocId, &psAccessControlReq->sSecurityChallengeReq, "psAccessControlReq->sSecurityChallengeReq.", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
	if (DBVcheck_DbvOtherInformation_opt(lAssocId, psAccessControlReq->ppsOtherInformation, "psAccessControlReq->ppsOtherInformation", ppsSIError))
                return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvSecurityChallengeReq(DbvAssocId lAssocId, DbvSecurityChallengeReq *psSecurityChallengeReq, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psSecurityChallengeReq->eSecurityChallengeReqType) {
	case SECT_SimpleForm:
		DBVsa_strcat(local_error_buffer, "uSecurityChallengeReq.psSimpleForm");
		if (psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

                break;


	case SECT_ExternallyDefined:
		DBVsa_strcat(local_error_buffer, "uSecurityChallengeReq.psExternallyDefinedInfo");
		if (psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return (DBVcheck_dbv_external(lAssocId, psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo, local_error_buffer, ppsSIError));

                break;
	default:
		DBVsa_strcat(local_error_buffer, "eSecurityChallengeReqType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
        }


        return SI_OK;
}



/* snacc set up */

int DBVsetup_snacc_AccessControlRequest(PDU *PDU_ptr, DbvAccessControlReq *psAccessControlReq, jmp_buf env)
{
	DBVset_AccessControlRequest_referenceId(PDU_ptr, psAccessControlReq->psReferenceId, env);
        DBVset_AccessControlRequest_securityChallenge(PDU_ptr, &psAccessControlReq->sSecurityChallengeReq, env);
	DBVset_AccessControlRequest_otherinfo(PDU_ptr, psAccessControlReq->ppsOtherInformation, env);
}



int DBVset_AccessControlRequest_referenceId(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvOctetString_opt(&(PDU_ptr->a.accessControlRequest->referenceId), psReferenceId, env);
	
}



	

int DBVset_AccessControlRequest_securityChallenge(PDU *PDU_ptr, DbvSecurityChallengeReq *psSecurityChallengeReq, jmp_buf env)
{
	PDU_ptr->a.accessControlRequest->securityChallenge = ( struct AccessControlRequestChoice*) Asn1Alloc (sizeof (struct AccessControlRequestChoice));
	CheckAsn1Alloc(PDU_ptr->a.accessControlRequest->securityChallenge, env);

	switch (psSecurityChallengeReq->eSecurityChallengeReqType) {
	case SECT_SimpleForm:
		PDU_ptr->a.accessControlRequest->securityChallenge->choiceId = ACCESSCONTROLREQUESTCHOICE_SIMPLEFORM;
		DBVdbv_to_snacc_octetstring_comp(&PDU_ptr->a.accessControlRequest->securityChallenge->a.simpleForm, psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm, env);

		break;

	case SECT_ExternallyDefined:
		PDU_ptr->a.accessControlRequest->securityChallenge->choiceId = ACCESSCONTROLREQUESTCHOICE_EXTERNALLYDEFINED;
		DBVdbv_to_snacc_external_opt(&PDU_ptr->a.accessControlRequest->securityChallenge->a.externallyDefined, psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo, env);
		break;
	}
	
}



int DBVset_AccessControlRequest_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	DBVdbv_to_snacc_otherinfo_opt(&PDU_ptr->a.accessControlRequest->otherInfo, ppsOtherInformation, env);

}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS AccessControlResponse PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_DbvAccessControlRsp(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_AccessCntrlConf;
        psTargetData->uTargetData.psAccessControlRsp = (DbvAccessControlRsp *) DbvAlloc ( sizeof (DbvAccessControlRsp));
	CheckDbvAlloc(psTargetData->uTargetData.psAccessControlRsp, env);

	/* o */
        DBVset_DbvAccessControlRsp_psReferenceId(psTargetData->uTargetData.psAccessControlRsp, PDU_ptr, env);

	/* o */
        DBVset_DbvAccessControlRsp_psSecurityChallengeRsp(psTargetData->uTargetData.psAccessControlRsp, PDU_ptr, env);

	/* o */
        DBVset_DbvAccessControlRsp_psDiagnostic(psTargetData->uTargetData.psAccessControlRsp, PDU_ptr, env);

	/* o */
	DBVset_DbvAccessControlRsp_ppsOtherInformation(psTargetData->uTargetData.psAccessControlRsp, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



int DBVset_DbvAccessControlRsp_psReferenceId(DbvAccessControlRsp *psAccessControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* o */	
	DBVsnacc_to_dbv_DbvOctetString_opt(&psAccessControlRsp->psReferenceId, &PDU_ptr->a.accessControlResponse->referenceId, env);
}



int DBVset_DbvAccessControlRsp_psSecurityChallengeRsp(DbvAccessControlRsp *psAccessControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	if (PDU_ptr->a.accessControlResponse->securityChallengeResponse) {
		psAccessControlRsp->psSecurityChallengeRsp = (DbvSecurityChallengeRsp *) DbvAlloc (sizeof (DbvSecurityChallengeRsp));
		CheckDbvAlloc(psAccessControlRsp->psSecurityChallengeRsp, env);

		switch (PDU_ptr->a.accessControlResponse->securityChallengeResponse->choiceId) {
		case ACCESSCONTROLRESPONSECHOICE_SIMPLEFORM:
			psAccessControlRsp->psSecurityChallengeRsp->eSecurityChallengeRspType = SECT_SimpleForm;
			DBVsnacc_to_dbv_DbvOctetString_opt(&psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm, PDU_ptr->a.accessControlResponse->securityChallengeResponse->a.simpleForm, env);
			break;

		case ACCESSCONTROLRESPONSECHOICE_EXTERNALLYDEFINED:
			psAccessControlRsp->psSecurityChallengeRsp->eSecurityChallengeRspType = SECT_ExternallyDefined;
			DBVsnacc_to_dbv_external_opt(&psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo, PDU_ptr->a.accessControlResponse->securityChallengeResponse->a.externallyDefined, env);
			break;
		}
	}
}



int DBVset_DbvAccessControlRsp_psDiagnostic(DbvAccessControlRsp *psAccessControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* o */	
	DBVsnacc_to_dbv_DbvDiagRec_opt(&psAccessControlRsp->psDiagnostic, PDU_ptr->a.accessControlResponse->diagnostic, env);
}



int DBVset_DbvAccessControlRsp_ppsOtherInformation(DbvAccessControlRsp *psAccessControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_otherinfo_opt(&psAccessControlRsp->ppsOtherInformation, PDU_ptr->a.accessControlResponse->otherInfo, env);
}



