/*  Module Name: @(#)target_z3950_delete.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a 
		  DeleteResultSetResponse PDU to an origin and process
		  a received DeleteResultSetRequest PDU sent by an origin
		  

*/

#include "target_z3950_delete.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE DeleteResultSetResponse PDU TO SEND TO CLIENT AND SEND IT
*/

int DbvDeleteResultSetResponse(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;


	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVDeleteResultSetResponse",lAssocId);

	if (DBVCheckDbvDelRspFields(lAssocId, psDeleteResultSetRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_DELETERESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVDelRspTrace(lAssocId, psDeleteResultSetRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.deleteResultSetResponse = (struct DeleteResultSetResponse *) Asn1Alloc(sizeof(struct DeleteResultSetResponse));
			CheckAsn1Alloc(PDU_ptr->a.deleteResultSetResponse, env);
        		PDU_ptr->choiceId = PDU_DELETERESULTSETRESPONSE;
			DBVsetup_snacc_delresponse(PDU_ptr, psDeleteResultSetRsp, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVDeleteResultSetResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvDelRspFields(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
	
        if (DBVcheck_dbv_delrsp_opstat(lAssocId, psDeleteResultSetRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_delrsp_liststat(lAssocId, psDeleteResultSetRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_delrsp_bulkstat(lAssocId, psDeleteResultSetRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_delrsp_otherinfo(lAssocId, psDeleteResultSetRsp, ppsSIError))
                return SI_NOTOK;
 
        return SI_OK;
}
 


int DBVcheck_dbv_delrsp_opstat(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
 
        if (DBVcheck_dbv_enum(psDeleteResultSetRsp->eDeleteOperationStatus, DSS_Success, DSS_ResultSetInUse) == SI_NOTOK)
                return(DBVdbv_invalid_enum_error(lAssocId, "psDeleteResultSetRsp->eDeleteOperationStatus", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_delrsp_liststat(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
 
        /* Can be NULL because optional */
        if (psDeleteResultSetRsp->ppsDeleteListStatuses == NULL)
                return SI_OK;
 

	return(DBVcheck_liststat(lAssocId, psDeleteResultSetRsp->ppsDeleteListStatuses, "psDeleteResultSetRsp->ppsDeleteListStatuses", ppsSIError));

}



int DBVcheck_liststat(DbvAssocId lAssocId, DbvListStatus **ppsDeleteListStatuses, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	while (ppsDeleteListStatuses[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "[");

		if (index < MAX_INDEX)
	                sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]");


        	if (DBVcheck_dbv_enum(ppsDeleteListStatuses[index]->eDeleteSetStatus, DSS_Success, DSS_ResultSetInUse) == SI_NOTOK)
                	return(DBVdbv_invalid_enum_error(lAssocId, temp_error_buffer, ppsSIError));


		index++;
	}
 
        return SI_OK;
}



int DBVcheck_dbv_delrsp_bulkstat(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
        /* Can be NULL because optional */
        if (psDeleteResultSetRsp->ppsBulkStatuses == NULL)
                return SI_OK;
 

	return(DBVcheck_liststat(lAssocId, psDeleteResultSetRsp->ppsBulkStatuses, "psDeleteResultSetRsp->ppsBulkStatuses", ppsSIError));

}



int DBVcheck_dbv_delrsp_otherinfo(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psDeleteResultSetRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psDeleteResultSetRsp->ppsOtherInformation, "psDeleteResultSetRsp->ppsOtherInformation", ppsSIError));
 

}



int DBVsetup_snacc_delresponse(PDU *PDU_ptr, DbvDeleteResultSetRsp *psDeleteResultSetRsp, jmp_buf env)
{
	DBVset_snacc_delresp_refid(PDU_ptr, psDeleteResultSetRsp->psReferenceId, env);
	DBVset_snacc_delresp_opstat(PDU_ptr, psDeleteResultSetRsp->eDeleteOperationStatus, env);
	DBVset_snacc_delresp_delliststat(PDU_ptr, psDeleteResultSetRsp->ppsDeleteListStatuses, env);
	DBVset_snacc_delresp_numnotdel(PDU_ptr,  psDeleteResultSetRsp->pulNumberNotDeleted, env);
	DBVset_snacc_delresp_bulkliststat(PDU_ptr,  psDeleteResultSetRsp->ppsBulkStatuses, env);
	DBVset_snacc_delresp_delmess(PDU_ptr,  psDeleteResultSetRsp->psDeleteMessage, env);

	DBVset_snacc_delresp_otherinfo(PDU_ptr, psDeleteResultSetRsp->ppsOtherInformation, env);
}



int DBVset_snacc_delresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
 
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.deleteResultSetResponse->referenceId), psReferenceId, env);
        }
}
 


int DBVset_snacc_delresp_opstat(PDU *PDU_ptr, DbvDeleteSetStatus eDeleteOperationStatus, jmp_buf env)
{
	DBVdbv_to_snacc_delstat(&PDU_ptr->a.deleteResultSetResponse->deleteOperationStatus, &eDeleteOperationStatus, env); 
}


/* Following function is called later on also */
int DBVdbv_to_snacc_delstat(DeleteSetStatus *snacc_status, DbvDeleteSetStatus *dbv_status, jmp_buf env)
{


	switch (*dbv_status) {
	case DSS_Success:
		*snacc_status = SUCCESS1;
		break;
	case DSS_ResultSetDidNotExist:
		*snacc_status = RESULTSETDIDNOTEXIST;
		break;
	case DSS_PreviouslyDeletedByTarget:
		*snacc_status = PREVIOUSLYDELETEDBYTARGET;
		break;
	case DSS_SystemProblemAtTarget:
		*snacc_status = SYSTEMPROBLEMATTARGET;
		break;
	case DSS_AccessNotAllowed:
		*snacc_status = ACCESSNOTALLOWED;
		break;
	case DSS_ResourceControlAtOrigin:
		*snacc_status = RESOURCECONTROLATORIGIN;
		break;
	case DSS_ResourceControlAtTarget:
		*snacc_status = RESOURCECONTROLATTARGET;
		break;
	case DSS_BulkDeleteNotSupported:
		*snacc_status = BULKDELETENOTSUPPORTED;
		break;
	case DSS_NotAllSetsDeletedOnBulkDlt:
		*snacc_status = NOTALLRSLTSETSDELETEDONBULKDLTE;
		break;
	case DSS_NotAllRequestedSetsDeleted:
		*snacc_status = NOTALLREQUESTEDRESULTSETSDELETED;
		break;
	case DSS_ResultSetInUse:
		*snacc_status = RESULTSETINUSE;
		break;
	}
}




int DBVset_snacc_delresp_delliststat(PDU *PDU_ptr, DbvListStatus **ppsDeleteListStatuses, jmp_buf env)
{
        ListStatusesSeq *data_ptr;
        ListStatusesSeq **data_ptr_address;
 
        int current_listnode = 0;

	if (ppsDeleteListStatuses != NULL) {
        	PDU_ptr->a.deleteResultSetResponse->deleteListStatuses = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetResponse->deleteListStatuses, env);
 
        	while(ppsDeleteListStatuses[current_listnode] != NULL) {
                	data_ptr_address = AsnListAppend(PDU_ptr->a.deleteResultSetResponse->deleteListStatuses);
			CheckAsn1Alloc(data_ptr_address, env);

                	*data_ptr_address = Asn1Alloc(sizeof(ListStatusesSeq));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;
	 
                	DBVdbv_to_snacc_liststat(data_ptr, ppsDeleteListStatuses[current_listnode], env);
                	current_listnode++;
        	}
	}
}



int DBVdbv_to_snacc_liststat(ListStatusesSeq *data_ptr, DbvListStatus *psDeleteListStatuses, jmp_buf env)
{
	/* Both the following are compulsory */
	DBVdbv_to_snacc_inter_string(&data_ptr->id, &psDeleteListStatuses->sResultSetId, env);

	DBVdbv_to_snacc_delstat(&data_ptr->status, &psDeleteListStatuses->eDeleteSetStatus, env); 
}



int DBVset_snacc_delresp_numnotdel(PDU *PDU_ptr, Ulong *pulNumberNotDeleted, jmp_buf env)
{
	if (pulNumberNotDeleted != NULL) {
       		PDU_ptr->a.deleteResultSetResponse->numberNotDeleted = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetResponse->numberNotDeleted, env);

       		*PDU_ptr->a.deleteResultSetResponse->numberNotDeleted = *pulNumberNotDeleted;
	}
}



int DBVset_snacc_delresp_bulkliststat(PDU *PDU_ptr, DbvListStatus **ppsBulkStatuses, jmp_buf env)
{
        ListStatusesSeq *data_ptr;
        ListStatusesSeq **data_ptr_address;
        int current_listnode = 0;

	if (ppsBulkStatuses != NULL) {
        	PDU_ptr->a.deleteResultSetResponse->bulkStatuses = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetResponse->bulkStatuses, env);
 
        	while (ppsBulkStatuses[current_listnode] != NULL) {
                	data_ptr_address = AsnListAppend(PDU_ptr->a.deleteResultSetResponse->bulkStatuses);
			CheckAsn1Alloc(data_ptr_address, env);

                	*data_ptr_address = Asn1Alloc(sizeof(ListStatusesSeq));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;
	 
                	DBVdbv_to_snacc_liststat(data_ptr, ppsBulkStatuses[current_listnode], env);
                	current_listnode++;
        	}
	}
}



int DBVset_snacc_delresp_delmess(PDU *PDU_ptr, DbvInternationalString *psDeleteMessage, jmp_buf env)
{
	if (psDeleteMessage != NULL)
		DBVdbv_to_snacc_inter_string(&PDU_ptr->a.deleteResultSetResponse->deleteMessage, psDeleteMessage, env);
}



int DBVset_snacc_delresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
 
        /* Check if ppsOtherInformation exists (it is an optional field) */
        if (ppsOtherInformation != NULL)  {
                PDU_ptr->a.deleteResultSetResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.deleteResultSetResponse->otherInfo, env);
		
                DBVdbv_to_snacc_otherinfo(PDU_ptr->a.deleteResultSetResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS DeleteResultSetRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_delreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_DeleteResultSetInd;
        psTargetData->uTargetData.psDeleteResultSetReq = (DbvDeleteResultSetReq *) DbvAlloc ( sizeof (DbvDeleteResultSetReq));
	CheckDbvAlloc(psTargetData->uTargetData.psDeleteResultSetReq, env);
 
        DBVset_dbv_delreq_refid(psTargetData->uTargetData.psDeleteResultSetReq, PDU_ptr, env);
        DBVset_dbv_delreq_deletefunc(psTargetData->uTargetData.psDeleteResultSetReq, PDU_ptr, env);
        DBVset_dbv_delreq_ressetid(psTargetData->uTargetData.psDeleteResultSetReq, PDU_ptr, env);
	DBVset_dbv_delreq_otherinfo(psTargetData->uTargetData.psDeleteResultSetReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_delreq_refid(DbvDeleteResultSetReq *delreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.deleteResultSetRequest->referenceId.octetLen != 0){
                delreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(delreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(delreq->psReferenceId, &(PDU_ptr->a.deleteResultSetRequest->referenceId), env);

        }

}



int DBVset_dbv_delreq_deletefunc(DbvDeleteResultSetReq *delreq, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.deleteResultSetRequest->deleteFunction) {
	case LIST:
                delreq->eDeleteFunction = DF_List;
		break;
	case ALL:
                delreq->eDeleteFunction = DF_All;
		break;
	default:
                delreq->eDeleteFunction = DF_Invalid;
		break;
	}
}



int DBVset_dbv_delreq_ressetid(DbvDeleteResultSetReq *delreq, PDU *PDU_ptr, jmp_buf env)
{
	ResultSetId *data_ptr;
	int num_elements = 0;
        int index = 0;
 

	if (PDU_ptr->a.deleteResultSetRequest->resultSetList != NULL) {
        	num_elements = AsnListCount(PDU_ptr->a.deleteResultSetRequest->resultSetList);
        	delreq->ppsResultSetId = (DbvInternationalString **) DbvAlloc (sizeof(DbvInternationalString *) * (num_elements + 1));
		CheckDbvAlloc(delreq->ppsResultSetId, env);
	
        	FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.deleteResultSetRequest->resultSetList) {
                	delreq->ppsResultSetId[index] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
			CheckDbvAlloc(delreq->ppsResultSetId[index], env);

                	DBVsnacc_to_dbv_inter_string(delreq->ppsResultSetId[index], data_ptr, env);
                	index++;
        	}
        	delreq->ppsResultSetId[index] = NULL;
	}
}




int DBVset_dbv_delreq_otherinfo(DbvDeleteResultSetReq *delreq, PDU *PDU_ptr, jmp_buf env)
{
	 int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.deleteResultSetRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.deleteResultSetRequest->otherInfo);
		delreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(delreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(delreq->ppsOtherInformation, PDU_ptr->a.deleteResultSetRequest->otherInfo, env);
	}
}



