/*  Module Name:@(#)target_z3950_extend.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send an 
		  ExtendedServicesResponse PDU to an origin and process 
		  a received ExtendedServicesRequest PDU 
		  send by an origin
		  

*/

#include "target_z3950_extend.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE ExtendedServicesResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvExtendedServicesResponse(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVExtendedServicesResponse",lAssocId);

	if (DBVCheckDbvExtendRspFields(lAssocId, psExtendedServicesRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_EXTENDEDRESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVExtendRspTrace(lAssocId, psExtendedServicesRsp);

        	PDU_ptr = &pdu_to_encode;
		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.extendedServicesResponse = (struct ExtendedServicesResponse *) Asn1Alloc(sizeof(struct ExtendedServicesResponse));
			CheckAsn1Alloc(PDU_ptr->a.extendedServicesResponse, env);
        		PDU_ptr->choiceId = PDU_EXTENDEDSERVICESRESPONSE;
			DBVsetup_snacc_extendresponse(PDU_ptr, psExtendedServicesRsp, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVExtendedServicesResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvExtendRspFields(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_extendrsp_opstat(lAssocId, psExtendedServicesRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendrsp_diags(lAssocId, psExtendedServicesRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendrsp_packtype(lAssocId, psExtendedServicesRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_extendrsp_otherinfo(lAssocId, psExtendedServicesRsp, ppsSIError))
                return SI_NOTOK;
 
        return SI_OK;
}
 


int DBVcheck_dbv_extendrsp_opstat(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_enum(psExtendedServicesRsp->eOperationStatus, ESOS_Done, ESOS_Failure) == SI_NOTOK)
                return(DBVdbv_invalid_enum_error(lAssocId, "psExtendedServicesRsp->eOperationStatus", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_extendrsp_diags(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
        /* Can be NULL because optional */
        if (psExtendedServicesRsp->ppsDiagnostics == NULL)
                return SI_OK;
 

	return(DBVcheck_dbv_multdiag(lAssocId, psExtendedServicesRsp->ppsDiagnostics, "psExtendedServicesRsp->ppsDiagnostics", ppsSIError));

}



int DBVcheck_dbv_extendrsp_packtype(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
        /* Can be NULL because optional */
        if (psExtendedServicesRsp->psTaskPackage == NULL)
		return SI_OK;

        return(DBVcheck_dbv_external(lAssocId, psExtendedServicesRsp->psTaskPackage, "psExtendedServicesRsp->psTaskPackage->", ppsSIError));
 
        return SI_OK;

}



int DBVcheck_dbv_extendrsp_otherinfo(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psExtendedServicesRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psExtendedServicesRsp->ppsOtherInformation, "psExtendedServicesRsp->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_extendresponse(PDU *PDU_ptr, DbvExtendedServicesRsp *psExtendedServicesRsp, jmp_buf env)
{
 
	DBVset_snacc_extendresp_refid(PDU_ptr, psExtendedServicesRsp->psReferenceId, env);
	DBVset_snacc_extendresp_opstat(PDU_ptr, psExtendedServicesRsp->eOperationStatus, env);
	DBVset_snacc_extendresp_diagnost(PDU_ptr, psExtendedServicesRsp->ppsDiagnostics, env);
	DBVset_snacc_extendresp_taskpack(PDU_ptr, psExtendedServicesRsp->psTaskPackage, env);
	DBVset_snacc_extendresp_otherinfo(PDU_ptr, psExtendedServicesRsp->ppsOtherInformation, env);
}



int DBVset_snacc_extendresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.extendedServicesResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_extendresp_opstat(PDU *PDU_ptr, DbvExtServicesOperationStatus eOperationStatus, jmp_buf env)
{
	switch (eOperationStatus) {
	case ESOS_Done:
		PDU_ptr->a.extendedServicesResponse->operationStatus = DONE;
		break;
	case ESOS_Queued:
		PDU_ptr->a.extendedServicesResponse->operationStatus = ACCEPTED;
		break;
	case ESOS_Failure:
		PDU_ptr->a.extendedServicesResponse->operationStatus = FAILURE1;
		break;
	}
}



int DBVset_snacc_extendresp_diagnost(PDU *PDU_ptr, DbvDiagRec **ppsDiagnostics, jmp_buf env)
{
	if (ppsDiagnostics != NULL) {
		PDU_ptr->a.extendedServicesResponse->diagnostics = AsnListNew (sizeof (void *));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesResponse->diagnostics, env);
		DBVdbv_to_snacc_multdiag(PDU_ptr->a.extendedServicesResponse->diagnostics, ppsDiagnostics, env);
	}
}



int DBVset_snacc_extendresp_taskpack(PDU *PDU_ptr, DbvASN1External *psTaskPackage, jmp_buf env)
{
	if (psTaskPackage != NULL) {
		PDU_ptr->a.extendedServicesResponse->taskPackage = (struct NEWEXTERNAL*) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesResponse->taskPackage, env);

		DBVdbv_to_snacc_external(PDU_ptr->a.extendedServicesResponse->taskPackage, psTaskPackage, env);
	}

}



int DBVset_snacc_extendresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{

	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.extendedServicesResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.extendedServicesResponse->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.extendedServicesResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS ExtendedServicesRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_extendreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_ExtendedServicesInd;
        psTargetData->uTargetData.psExtendedServicesReq = (DbvExtendedServicesReq *) DbvAlloc ( sizeof (DbvExtendedServicesReq));
	CheckDbvAlloc(psTargetData->uTargetData.psExtendedServicesReq, env);
 
        DBVset_dbv_extendreq_refid(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_function(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_packtype(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_packname(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_userid(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_retenttime(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_permission(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_description(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_ospecoparam(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_waitaction(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
        DBVset_dbv_extendreq_elements(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
	DBVset_dbv_extendreq_otherinfo(psTargetData->uTargetData.psExtendedServicesReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_extendreq_refid(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.extendedServicesRequest->referenceId.octetLen != 0){
                extendreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(extendreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(extendreq->psReferenceId, &(PDU_ptr->a.extendedServicesRequest->referenceId), env);
        }
}



int DBVset_dbv_extendreq_function(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.extendedServicesRequest->function) {
	case CREATE:
		extendreq->eFunction = ESF_Create;
		break;
	case DELETE1:
		extendreq->eFunction = ESF_Delete;
		break;
	case MODIFY:
		extendreq->eFunction = ESF_Modify;
		break;
	default:
		extendreq->eFunction = ESF_Invalid;
		break;
	}
}



int DBVset_dbv_extendreq_packtype(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
        char *oid = NULL;
 
        oid = (char *) ASN_EncodedToString(&(PDU_ptr->a.extendedServicesRequest->packageType));
        extendreq->pcPackageType = (char *) DbvAlloc (strlen(oid) + 1);
	CheckDbvAlloc(extendreq->pcPackageType, env);

        memcpy(extendreq->pcPackageType, oid, strlen(oid));
}



int DBVset_dbv_extendreq_packname(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If packageName given (it is an optional field) */
	if (PDU_ptr->a.extendedServicesRequest->packageName.octetLen != 0) {
                extendreq->psPackageName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(extendreq->psPackageName, env);

		DBVsnacc_to_dbv_inter_string(extendreq->psPackageName, &(PDU_ptr->a.extendedServicesRequest->packageName), env);
        }
}



int DBVset_dbv_extendreq_userid(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesRequest->userId.octetLen != 0) {
                extendreq->psUserId = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(extendreq->psUserId , env);

		DBVsnacc_to_dbv_inter_string(extendreq->psUserId, &(PDU_ptr->a.extendedServicesRequest->userId), env);
	}
}



int DBVset_dbv_extendreq_retenttime(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesRequest->retentionTime != NULL) {
		extendreq->psRetentionTime = (DbvIntUnit *) DbvAlloc (sizeof(DbvIntUnit));
		CheckDbvAlloc(extendreq->psRetentionTime, env);
		DBVsnacc_to_dbv_intunit(extendreq->psRetentionTime, PDU_ptr->a.extendedServicesRequest->retentionTime, env);
	}
}



int DBVset_dbv_extendreq_permission(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	if (PDU_ptr->a.extendedServicesRequest->permissions != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.extendedServicesRequest->permissions);
		extendreq->ppsPermissions = (DbvPermissions **) DbvAlloc (sizeof(DbvPermissions *) * (num_elements + 1));
		CheckDbvAlloc(extendreq->ppsPermissions, env);
		DBVsnacc_to_dbv_permissions(extendreq->ppsPermissions, PDU_ptr->a.extendedServicesRequest->permissions, env);
	}
}



int DBVset_dbv_extendreq_description(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesRequest->description.octetLen != 0) {
		extendreq->psDescription = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(extendreq->psDescription, env);

		DBVsnacc_to_dbv_inter_string(extendreq->psDescription, &PDU_ptr->a.extendedServicesRequest->description, env);
	}
}



int DBVset_dbv_extendreq_ospecoparam(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesRequest->taskSpecificParameters != NULL) {
		extendreq->psOriginSpecificParameter = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(extendreq->psOriginSpecificParameter, env);

		DBVsnacc_to_dbv_external(extendreq->psOriginSpecificParameter, PDU_ptr->a.extendedServicesRequest->taskSpecificParameters, env);
	}
}



int DBVset_dbv_extendreq_waitaction(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	switch (PDU_ptr->a.extendedServicesRequest->waitAction) {
	case WAIT:
		extendreq->eWaitAction = ESWA_Wait;
		break;
	case WAITIFPOSSIBLE:	
		extendreq->eWaitAction = ESWA_WaitIfPossible;
		break;
	case DONTWAIT:
		extendreq->eWaitAction = ESWA_DontWait;
		break;
	case DONTRETURNPACKAGE:
		extendreq->eWaitAction = ESWA_DontReturnPackage;
		break;
	default:
		extendreq->eWaitAction = ESWA_Invalid;
		break;
	}
}



int DBVset_dbv_extendreq_elements(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.extendedServicesRequest->elements.octetLen != 0) {
		extendreq->psElements = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(extendreq->psElements, env);

		DBVsnacc_to_dbv_inter_string(extendreq->psElements, &PDU_ptr->a.extendedServicesRequest->elements, env);
	}
}



int DBVset_dbv_extendreq_otherinfo(DbvExtendedServicesReq *extendreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.extendedServicesRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.extendedServicesRequest->otherInfo);
		extendreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(extendreq->ppsOtherInformation, env);
		DBVsnacc_to_dbv_otherinfo(extendreq->ppsOtherInformation, PDU_ptr->a.extendedServicesRequest->otherInfo, env);
	}
}






