/*  Module Name: @(#)target_z3950_init.c	7.2
    Version: 7/4/97
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send an InitResponse PDU 
		  to an origin and process a received InitRequest PDU 
		  sent by an origin
		  

*/

#include "target_z3950_init.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE InitResponse PDU TO SEND TO ORIGIN AND SEND IT 
*/

int DbvInitializeResponse(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	int proposed_state = DBV_TE_INITRESP_TRUE;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVInitializeResponse",lAssocId);

	/* Set propsed state based on value in the fResult field 
	   in the dbv structure */
	proposed_state = DBVget_dbv_initrsp_state(psInitializeRsp);
	
        if (DBVCheckDbvInitRspFields(lAssocId, psInitializeRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, proposed_state, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVInitRspTrace(lAssocId, psInitializeRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.initResponse = (struct InitializeResponse *) Asn1Alloc (sizeof(struct InitializeResponse));
			CheckAsn1Alloc(PDU_ptr->a.initResponse, env);
        		PDU_ptr->choiceId = PDU_INITRESPONSE;
			DBVsetup_snacc_initresponse(PDU_ptr, psInitializeRsp, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVInitializeResponse",lAssocId, return_value);
        return return_value;
}



/* A special initrsp operation - get next propsed state for state machine */
int DBVget_dbv_initrsp_state(DbvInitializeRsp *psInitializeRsp)
{
	if (psInitializeRsp->fResult == DBV_TRUE)
		return DBV_TE_INITRESP_TRUE;
	else
		return DBV_TE_INITRESP_FALSE;
}



int DBVCheckDbvInitRspFields(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{


	if (DBVcheck_dbv_initrsp_prover(lAssocId, psInitializeRsp, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_dbv_initrsp_options(lAssocId, psInitializeRsp, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_dbv_initrsp_result(lAssocId, psInitializeRsp, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;
	
	if (DBVcheck_dbv_initrsp_userinf(lAssocId, psInitializeRsp, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	if (DBVcheck_dbv_initrsp_otherinfo(lAssocId, psInitializeRsp, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_initrsp_prover(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{

	/* If field doesnt exist */
	if(psInitializeRsp->pcProtocolVersion == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psInitializeRsp->pcProtocolVersion", ppsSIError));


	/* pcProtocolVersion exists - check its value */
	if (DBVcheck_bit_string(psInitializeRsp->pcProtocolVersion) == SI_NOTOK)
		return(DBVdbv_bit_string_error(lAssocId, "psInitializeRsp->pcProtocolVersion", psInitializeRsp->pcProtocolVersion, ppsSIError));


	return SI_OK;
}



int DBVcheck_dbv_initrsp_options(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{

	if(psInitializeRsp->pcOptions == NULL)
		return(DBVdbv_field_missing_error(lAssocId, "psInitializeRsp->pcOptions", ppsSIError));

	
	if(DBVcheck_bit_string(psInitializeRsp->pcOptions) == SI_NOTOK)
		return(DBVdbv_bit_string_error(lAssocId, "psInitializeRsp->pcOptions", psInitializeRsp->pcOptions, ppsSIError));

			
	/* Check that the options chosen are supported */
	if (DBVcheck_chosen_options(psInitializeRsp->pcOptions) == SI_NOTOK)
                return(DBVdbv_options_chosen_error(lAssocId, "psInitializeRsp->pcOptions", psInitializeRsp->pcOptions, ppsSIError));

	return SI_OK;

}




int DBVcheck_dbv_initrsp_result(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{

	if (DBVcheck_dbv_bool(psInitializeRsp->fResult) == SI_NOTOK) 
		return(DBVdbv_bool_error(lAssocId, "psInitializeRsp->pcOptions", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_initrsp_userinf(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{
	/* Can be null because optional */
	if (psInitializeRsp->psUserInformationField == NULL) 
		return SI_OK;

	return(DBVcheck_dbv_external(lAssocId, psInitializeRsp->psUserInformationField, "psInitializeRsp->psUserInformationField->", ppsSIError));

}



int DBVcheck_dbv_initrsp_otherinfo(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psInitializeRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psInitializeRsp->ppsOtherInformation, "psInitializeRsp->ppsOtherInformation", ppsSIError));

}


 


int DBVsetup_snacc_initresponse(PDU *PDU_ptr, DbvInitializeRsp *psInitializeRsp, jmp_buf env)
{
	DBVset_snacc_initresp_refid(PDU_ptr, psInitializeRsp->psReferenceId, env);
        DBVset_snacc_initresp_protocolversion(PDU_ptr, psInitializeRsp->pcProtocolVersion, env);
        DBVset_snacc_initresp_options(PDU_ptr, psInitializeRsp->pcOptions, env);
        DBVset_snacc_initresp_prefmessize(PDU_ptr, psInitializeRsp->ulPreferredMessageSize, env);
        DBVset_snacc_initresp_maxmessize(PDU_ptr, psInitializeRsp->ulMaximumMessageSize, env);
	DBVset_snacc_initresp_result(PDU_ptr, psInitializeRsp->fResult, env);
	DBVset_snacc_initresp_impid(PDU_ptr, psInitializeRsp->psImplementationId, env);
	DBVset_snacc_initresp_impname(PDU_ptr, psInitializeRsp->psImplementationName, env);
	DBVset_snacc_initresp_impver(PDU_ptr, psInitializeRsp->psImplementationVersion, env);
	DBVset_snacc_initresp_userinfo(PDU_ptr, psInitializeRsp->psUserInformationField, env);
	DBVset_snacc_initresp_otherinfo(PDU_ptr, psInitializeRsp->ppsOtherInformation, env);
}



int DBVset_snacc_initresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.initResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_initresp_protocolversion(PDU *PDU_ptr, DbvProtocolVersion pcProtocolVersion, jmp_buf env)
{
 
        AsnBits *a1;
        unsigned long i;
	unsigned long bytes_to_alloc = 0;
 
 
        /* Allocate space for 2 bit groups (bytes).
           Each bit group has 8 bits in the order 128, 64, 32, 16, 8, 4, 2, 1
           The first bit in a1 is the 128 bit of group 1.
           The last bit in a1 is the 1 bit of group 2.
 
           Initially all bits are reset (0).
 
           SetAsnBit sets a relevant bit.
           ClrAsnBit resets a relevant bit.
           GetAsnBits gets the decimal value of a bit (either 128, 64, 32, 16,
                8, 4, 2 or 1 if the bit is set, 0 if the bit is reset)
        */

        a1 = &PDU_ptr->a.initResponse->protocolVersion;

	bytes_to_alloc = (strlen(pcProtocolVersion) / 8) + 1;

        a1->bits = Asn1Alloc(bytes_to_alloc);
	CheckAsn1Alloc(a1->bits, env);
 
        /* Total number of bits in all bit groups */
        a1->bitLen = 8 * bytes_to_alloc;
 
        for (i = 0; i < strlen(pcProtocolVersion); i++){
                if (pcProtocolVersion[i] == '1')
                        SetAsnBit(a1, i);
                else
                        ClrAsnBit(a1, i);
        }
 
        a1->bitLen = i;
 
 
}



int DBVset_snacc_initresp_options(PDU *PDU_ptr, DbvOptions pcOptions, jmp_buf env)
{
        AsnBits *a1;
        unsigned long i;
	unsigned long bytes_to_alloc = 0;
 
        a1 = &PDU_ptr->a.initResponse->options;

	bytes_to_alloc = (strlen(pcOptions) / 8) + 1;
 
        a1->bits = Asn1Alloc(bytes_to_alloc);
	CheckAsn1Alloc(a1->bits, env);
 
        /* Total number of bits in all bit groups */
        a1->bitLen = 8 * bytes_to_alloc;
 
	/* Allow 16 bits max (0 - 15) */
        for (i = 0; i < strlen(pcOptions); i++){
                if (pcOptions[i] == '1')
                        SetAsnBit(a1, i);
                else
                        ClrAsnBit(a1, i);
        }

        a1->bitLen = i;
}
 
 
 
int DBVset_snacc_initresp_prefmessize(PDU *PDU_ptr, Ulong ulPreferredMessageSize, jmp_buf env)
{
        PDU_ptr->a.initResponse->preferredMessageSize = ulPreferredMessageSize;
 
}



int DBVset_snacc_initresp_maxmessize(PDU *PDU_ptr, Ulong ulMaximumMessageSize, jmp_buf env)
{
       PDU_ptr->a.initResponse->exceptionalRecordSize = ulMaximumMessageSize;
}



int DBVset_snacc_initresp_impid(PDU *PDU_ptr, DbvInternationalString *psImplementationId, jmp_buf env)
{
        /* Check if a ImplementationId is given(it is an optional field) */
        if  (psImplementationId != NULL) 
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initResponse->implementationId), psImplementationId, env);
}



int DBVset_snacc_initresp_result(PDU *PDU_ptr, DbvBool fResult, jmp_buf env)
{
	if(fResult == DBV_TRUE)
		PDU_ptr->a.initResponse->result = TRUE;
	else
		PDU_ptr->a.initResponse->result = FALSE;
}



int DBVset_snacc_initresp_impname(PDU *PDU_ptr, DbvInternationalString *psImplementationName, jmp_buf env)
{
        /* Check if a ImplementationName is given(it is an optional field) */
        if  (psImplementationName != NULL) 
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initResponse->implementationName), psImplementationName, env);
}



int DBVset_snacc_initresp_impver(PDU *PDU_ptr, DbvInternationalString *psImplementationVersion, jmp_buf env)
{
        /* Check if a ImplementationVersion is given(it is an optional field) */
        if  (psImplementationVersion != NULL) 
		DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.initResponse->implementationVersion), psImplementationVersion, env);
}



int DBVset_snacc_initresp_userinfo(PDU *PDU_ptr, DbvASN1External *psUserInformationField, jmp_buf env)
{
 
	
	/* If psUserInformationField exists (it is an optional field) */
	if (psUserInformationField != NULL) {
        	PDU_ptr->a.initResponse->userInformationField = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
		CheckAsn1Alloc(PDU_ptr->a.initResponse->userInformationField, env);

		DBVdbv_to_snacc_external(PDU_ptr->a.initResponse->userInformationField, psUserInformationField, env);
	}
}




int DBVset_snacc_initresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.initResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.initResponse->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.initResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS InitRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_initreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_InitializeInd;
        psTargetData->uTargetData.psInitializeReq = (DbvInitializeReq *) DbvAlloc ( sizeof (DbvInitializeReq));
	CheckDbvAlloc(psTargetData->uTargetData.psInitializeReq, env);

 
        DBVset_dbv_initreq_refid(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
        DBVset_dbv_initreq_prover(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
        DBVset_dbv_initreq_options(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env)
;
        DBVset_dbv_initreq_prefmes(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env)
;
        DBVset_dbv_initreq_maxmes(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_authentication(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_impid(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_impname(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_impver(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_userinfo(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
	DBVset_dbv_initreq_otherinfo(psTargetData->uTargetData.psInitializeReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_initreq_refid(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.initRequest->referenceId.octetLen != 0){
                initreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(initreq->psReferenceId, env);
		DBVsnacc_to_dbv_octet_string(initreq->psReferenceId, &(PDU_ptr->a.initRequest->referenceId), env);

        }

}


int DBVset_dbv_initreq_prover(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
 
	char *prover_string = NULL;
        int i;
        AsnBits *a1;
 
        a1 = &PDU_ptr->a.initRequest->protocolVersion;

        prover_string = (char *) DbvAlloc (a1->bitLen  + 1);
	CheckDbvAlloc(prover_string, env);

        for (i = 0; i < (a1->bitLen); i++){
                if (GetAsnBit(a1, i)) {
                        prover_string[i] =  '1';
                } else {
                        prover_string[i] =  '0';
                }
        }


	/* Terminate the string */
        prover_string[i] = 0;


       	/* Address of pointer to character */
       	initreq->pcProtocolVersion = (DbvProtocolVersion) DbvAlloc(strlen(prover_string) + 1);
	CheckDbvAlloc(initreq->pcProtocolVersion, env);
	memcpy(initreq->pcProtocolVersion, prover_string, strlen(prover_string));

	if (prover_string != NULL)
		DbvFree(prover_string);
}

	


int DBVset_dbv_initreq_options(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        char *options_string = NULL;
        int i;
        AsnBits *a1;


        a1 = &PDU_ptr->a.initRequest->options;

	options_string = (char *) DbvAlloc (a1->bitLen  + 1);

        for (i = 0; i < (a1->bitLen); i++){
                if (GetAsnBit(a1, i)) {
                        options_string[i] = '1';
                } else {
                        options_string[i] = '0';
                }
        }
 

	/* Terminate the string */
        options_string[i] = 0;

	initreq->pcOptions = (DbvOptions) DbvAlloc (strlen(options_string) + 1);
	CheckDbvAlloc(initreq->pcOptions, env);
	memcpy(initreq->pcOptions, options_string, strlen(options_string));

	if (options_string != NULL)
		DbvFree(options_string);
}



int DBVset_dbv_initreq_prefmes(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        initreq->ulPreferredMessageSize = PDU_ptr->a.initRequest->preferredMessageSize;
}



int DBVset_dbv_initreq_maxmes(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        initreq->ulMaximumMessageSize = PDU_ptr->a.initRequest->exceptionalRecordSize;
}



int DBVset_dbv_initreq_authentication(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
	if(PDU_ptr->a.initRequest->idAuthentication != NULL)
	{
		initreq->psAuthentication = (DbvAuthentication *)DbvAlloc(sizeof(DbvAuthentication));
		CheckDbvAlloc(initreq->psAuthentication, env);

		switch(PDU_ptr->a.initRequest->idAuthentication->choiceId)
		{
		case IDAUTHENTICATION_OPEN:
			DBVset_dbv_initreq_authentication_open(PDU_ptr, initreq, env);
			break;
		case IDAUTHENTICATION_IDPASS:
			DBVset_dbv_initreq_authentication_pass(PDU_ptr, initreq, env);
			break;
		case IDAUTHENTICATION_ANONYMOUS:
			initreq->psAuthentication->eAuthenticationType = AT_Anonymous;
			break;
		case IDAUTHENTICATION_OTHER:
			DBVset_dbv_initreq_authentication_other(PDU_ptr, initreq, env);
			break;
		}
	}
}



int DBVset_dbv_initreq_authentication_open(PDU *PDU_ptr, DbvInitializeReq *initreq, jmp_buf env)
{
	initreq->psAuthentication->eAuthenticationType = AT_Open;
	
	initreq->psAuthentication->uAuthentication.psOpen = NULL;

	/* Following function allocs space for dbv initreq->psAuthentication->uAuthentication.psOpen - pass in char ** */

	DBVsnacc_to_dbv_vis_string(&initreq->psAuthentication->uAuthentication.psOpen, PDU_ptr->a.initRequest->idAuthentication->a.open, env); 

}



int DBVset_dbv_initreq_authentication_pass(PDU *PDU_ptr, DbvInitializeReq *initreq, jmp_buf env)
{
	initreq->psAuthentication->eAuthenticationType = AT_IdPassword;

	initreq->psAuthentication->uAuthentication.psIdPassword = (DbvIdPassword *)DbvAlloc(sizeof(DbvIdPassword));
	CheckDbvAlloc(initreq->psAuthentication->uAuthentication.psIdPassword, env);



	/* All the following three are optional... */

	/* Group id */
	if (PDU_ptr->a.initRequest->idAuthentication->a.idPass->groupId.octetLen != 0) {
		initreq->psAuthentication->uAuthentication.psIdPassword->psGroupId = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
		CheckDbvAlloc(initreq->psAuthentication->uAuthentication.psIdPassword->psGroupId, env);
		DBVsnacc_to_dbv_inter_string(initreq->psAuthentication->uAuthentication.psIdPassword->psGroupId, &(PDU_ptr->a.initRequest->idAuthentication->a.idPass->groupId), env);
	}


	/* User id */
	if (PDU_ptr->a.initRequest->idAuthentication->a.idPass->userId.octetLen != 0) {
		initreq->psAuthentication->uAuthentication.psIdPassword->psUserId = (DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		CheckDbvAlloc(initreq->psAuthentication->uAuthentication.psIdPassword->psUserId, env);
		DBVsnacc_to_dbv_inter_string(initreq->psAuthentication->uAuthentication.psIdPassword->psUserId, &(PDU_ptr->a.initRequest->idAuthentication->a.idPass->userId), env);
	}


	/* Password */
	if (PDU_ptr->a.initRequest->idAuthentication->a.idPass->password.octetLen != 0) {
		initreq->psAuthentication->uAuthentication.psIdPassword->psPassword = (DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		CheckDbvAlloc(initreq->psAuthentication->uAuthentication.psIdPassword->psPassword, env);
		DBVsnacc_to_dbv_inter_string(initreq->psAuthentication->uAuthentication.psIdPassword->psPassword, &(PDU_ptr->a.initRequest->idAuthentication->a.idPass->password), env);
	}

}



/* Other is an external */
int DBVset_dbv_initreq_authentication_other(PDU *PDU_ptr, DbvInitializeReq *initreq, jmp_buf env)
{
	initreq->psAuthentication->eAuthenticationType = AT_Other;
	initreq->psAuthentication->uAuthentication.psOther = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
	CheckDbvAlloc(initreq->psAuthentication->uAuthentication.psOther, env);
	
	/* DBV external, snacc external */
	DBVsnacc_to_dbv_external(initreq->psAuthentication->uAuthentication.psOther, PDU_ptr->a.initRequest->idAuthentication->a.other, env);
}



int DBVset_dbv_initreq_impid(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        /* If implementation id given (it is an optional field) */
        if (PDU_ptr->a.initRequest->implementationId.octetLen != 0){
                initreq->psImplementationId = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(initreq->psImplementationId, env);

		DBVsnacc_to_dbv_inter_string(initreq->psImplementationId, &(PDU_ptr->a.initRequest->implementationId), env);
        }
 
}



int DBVset_dbv_initreq_impname(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        /* If implementation name given (it is an optional field) */
        if (PDU_ptr->a.initRequest->implementationName.octetLen != 0){
                initreq->psImplementationName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(initreq->psImplementationName, env);

		DBVsnacc_to_dbv_inter_string(initreq->psImplementationName, &(PDU_ptr->a.initRequest->implementationName), env);
        }
 
}



int DBVset_dbv_initreq_impver(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
        /* If implementation version given (it is an optional field) */
        if (PDU_ptr->a.initRequest->implementationVersion.octetLen != 0){
                initreq->psImplementationVersion = (DbvInternationalString *) DbvAlloc(sizeof (DbvInternationalString));
		CheckDbvAlloc(initreq->psImplementationVersion, env);

		DBVsnacc_to_dbv_inter_string(initreq->psImplementationVersion, &(PDU_ptr->a.initRequest->implementationVersion), env);
        }
}



int DBVset_dbv_initreq_userinfo(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{

	if (PDU_ptr->a.initRequest->userInformationField != NULL) {
		initreq->psUserInformationField = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(initreq->psUserInformationField, env);
	
                DBVsnacc_to_dbv_external(initreq->psUserInformationField, PDU_ptr->a.initRequest->userInformationField, env);
        }
}


int DBVset_dbv_initreq_otherinfo(DbvInitializeReq *initreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.initRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.initRequest->otherInfo);
		initreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(initreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(initreq->ppsOtherInformation, PDU_ptr->a.initRequest->otherInfo, env);
	}
}


