/*  Module Name: @(#)target_z3950_main.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: DBV-OSI II server main routines 
	         (common for all server functions)
	         

*/

#include "target_z3950_main.h"


int DbvReceiveDataTarget( DbvAssocId lAssocId, DbvSecs lSeconds, 
			  DbvTargetData **ppsTargetData, DbvError **ppsError)
{
	int return_value = SI_OK;
        PDU PDU_to_decode;
        PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        DBVAssociateDetails *psAssociateDetails;
	int val;
	jmp_buf env;
	char *ber_data = NULL;
	unsigned long ber_data_length = 0;
	DBVTraceDetails *psTraceInformation;
 
        PDU_ptr = &PDU_to_decode;
 
	DBVEnterFuncTrace("ReceiveDataTarget",lAssocId);

        /* Get the request */
        if(SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
        {
        	psAssociateDetails = SL_Current(DBVAssociateList);
		if(eProtocolStack == TCP_Stack)
		{
        		return_value = DBVreceive_data_over_socket(lAssocId, 
		        	         PDU_ptr, 
				         psAssociateDetails->ulTcpSocket, 
					 lSeconds, 
					 &ber_data, &ber_data_length, ppsError);

			/* If the value returned is neither SI_OK or
                           SI_NOTOK then the socket timed out */
		}
		else
		{
#ifndef NO_OSI
			return_value = DBVreceive_data_over_OSI(lAssocId,
				PDU_ptr, psAssociateDetails->lOSIAssocId,
				lSeconds, 
				&ber_data, &ber_data_length, ppsError);
#else
			printf("OSI not supported\n");
#endif
        	}
	}
	else
                /* Error structure filled out by SIWhichStack() */
		return_value = SI_NOTOK;

	if(return_value == SI_OK)
	{

		/* Do a setjump - the following structure setup
                   routines do a longjump here if allocing fails */
		if ((val = setjmp(env)) == 0) {

        		/* Allocate space for the PDU recieved - ppsTargetData 
			   is the address of a pointer to TargetData, 
			   so derefence to get and set the pointer */
        		*ppsTargetData = (struct DbvTargetData *) DbvAlloc (sizeof (struct DbvTargetData));
			CheckDbvAlloc(*ppsTargetData, env);
        		return_value = DBVconvert_pdu_to_target_data(lAssocId, *ppsTargetData, PDU_ptr, ber_data, ber_data_length, ppsError, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                        return SI_NOTOK;
                }

	}


/* OSI and TCP/IP */
#ifndef NO_OSI
	/* Release Request? */
	if(return_value >= DBV_RELEASE_REQUEST)
	{
	    if (DBVChangeState(lAssocId, DBV_TE_AREL_IND, ppsError) 
		== SI_OK) {
		if(DBVAllocCheck((*ppsTargetData = (struct DbvTargetData *) 
		    DBVAlloc (sizeof (struct DbvTargetData), ppsError)))
		    ==SI_NOTOK)
			{ return SI_NOTOK; }

		if(DBVAllocCheck(((*ppsTargetData)->uTargetData.peReleaseReq = 
		    (DbvReleaseReq *)DBVAlloc(sizeof(DbvReleaseReq),
		    ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }

		(*ppsTargetData)->eTargetEvent = TA_ReleaseInd;
		return_value -=10;
		switch(return_value)
		{
		case ACF_NORMAL:
			*(*ppsTargetData)->uTargetData.peReleaseReq=
				ReleaseReq_Normal;
			return_value = SI_OK;
			break;
		case ACF_URGENT:
			*(*ppsTargetData)->uTargetData.peReleaseReq=
				ReleaseReq_Urgent;
			return_value = SI_OK;
			break;
		case ACF_USERDEFINED: /* ???? */
			*(*ppsTargetData)->uTargetData.peReleaseReq=
				ReleaseReq_Undefined;
			return_value = SI_OK;
			break;
		default:
			/* Error: value is not understood!!! */
			printf("Fill out an error structure here!\n");
			return_value = SI_NOTOK;
			break;
		}
		
	    }
	    else
		return_value = SI_NOTOK;
	}


/* TCP/IP only */
#else
	if (return_value >= DBV_RELEASE_REQUEST) {
	    if (DBVChangeState(lAssocId, DBV_TE_AREL_IND, ppsError) 
		== SI_OK) {
		if(DBVAllocCheck((*ppsTargetData = (struct DbvTargetData *) 
		    DBVAlloc (sizeof (struct DbvTargetData), ppsError)))
		    ==SI_NOTOK)
			{ return SI_NOTOK; }

		if(DBVAllocCheck(((*ppsTargetData)->uTargetData.peReleaseReq = 
		    (DbvReleaseReq *)DBVAlloc(sizeof(DbvReleaseReq),
		    ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }

		(*ppsTargetData)->eTargetEvent = TA_ReleaseInd;
                *(*ppsTargetData)->uTargetData.peReleaseReq= ReleaseReq_Normal;
                return_value = SI_OK;
	    }
	    else
	    	return_value = SI_NOTOK;
	}

#endif

	/* Timed out with no response! */
	if (return_value == TIMED_OUT)
		return_value = SI_OK;
	
	DBVLeaveFuncTrace("ReceiveDataTarget",lAssocId, return_value);

        if(return_value == SI_NOTOK)
                if((*ppsError)->eErrorClass == SI_Abort)
                {
                /* Remove the association if it's finished */
			if(DBVTraceList != NULL)
                        {
                        psTraceInformation = SL_Locate(DBVTraceList,
                            (SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
                        if(psTraceInformation != NULL)
                        {
                                psTraceInformation = SL_RemoveCurrent(DBVTraceList);
                                if(psTraceInformation != NULL)
                                        free(psTraceInformation);
 
                        }
			}
                        psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                        free(psAssociateDetails);
                }
 

	/* Free the ber data */
        if (ber_data != NULL)
                DbvFree(ber_data);

	return return_value;

}



int DBVconvert_pdu_to_target_data(DbvAssocId lAssocId, DbvTargetData *psTargetData, PDU *PDU_ptr, char *ber_data, unsigned long ber_data_length, DbvError **ppsError, jmp_buf env)
{
	int return_value = SI_OK;

        switch (PDU_ptr->choiceId) {
                case PDU_INITREQUEST:
			return_value = DBVsnacc_to_dbv_initreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_SEARCHREQUEST:
			return_value = DBVsnacc_to_dbv_srchreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_PRESENTREQUEST:
			return_value = DBVsnacc_to_dbv_presreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_DELETERESULTSETREQUEST:
			return_value = DBVsnacc_to_dbv_delreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;


                case PDU_RESOURCEREPORTREQUEST:
			return_value = DBVsnacc_to_dbv_resrepreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_SCANREQUEST:
			return_value = DBVsnacc_to_dbv_scanreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_SORTREQUEST:
			return_value = DBVsnacc_to_dbv_sortreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_EXTENDEDSERVICESREQUEST:
			return_value = DBVsnacc_to_dbv_extendreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
                case PDU_CLOSE:
			return_value = DBVsnacc_to_dbv_taclose(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;
		case PDU_UNKNOWN:
			return_value = DBVsnacc_to_dbv_taunknown(PDU_ptr, lAssocId, ppsError, psTargetData, ber_data, ber_data_length, env);
			break;
                case PDU_ACCESSCONTROLRESPONSE:
			return_value = DBVsnacc_to_dbv_accessrsp(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;

                case PDU_RESOURCECONTROLRESPONSE:
			return_value = DBVsnacc_to_dbv_rescntrlrsp(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;

                case PDU_TRIGGERRESOURCECONTROLREQUEST:
			return_value = DBVsnacc_to_dbv_trigrescntrlreq(PDU_ptr, lAssocId, ppsError, psTargetData, env);
			break;

		default:
			break;
        }

	return return_value;
}



int DBVsnacc_to_dbv_initreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	/* If state change allowed */
	if (DBVChangeState(lAssocId, DBV_TE_INITREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_initreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
		DBVInitReqTrace(lAssocId, psTargetData->uTargetData.psInitializeReq);
		return SI_OK;

	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received InitReq PDU to DBV format\n");
		return SI_NOTOK;
	}

}



int DBVsnacc_to_dbv_srchreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_SEARCHREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_searchreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVSrchReqTrace(lAssocId, psTargetData->uTargetData.psSearchReq);
		return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received SrchReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_presreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_PRESENTREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_presreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVPresReqTrace(lAssocId, psTargetData->uTargetData.psPresentReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received PresReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_delreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_DELETEREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_delreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVDelReqTrace(lAssocId, psTargetData->uTargetData.psDeleteResultSetReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received DeleteResultSetReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}





int DBVsnacc_to_dbv_resrepreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_RESOURCEREQ_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_resrepreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVResRepReqTrace(lAssocId, psTargetData->uTargetData.psResourceReportReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ResourceReportReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_scanreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_SCANREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_scanreq(psTargetData, PDU_ptr, env);
                DBVReceivingTrace(lAssocId);
                DBVScanReqTrace(lAssocId, psTargetData->uTargetData.psScanReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ScanReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_sortreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_SORTREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_fields_sortreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVSortReqTrace(lAssocId, psTargetData->uTargetData.psSortReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received SortReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVsnacc_to_dbv_extendreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	if (DBVChangeState(lAssocId, DBV_TE_EXTENDEDREQ_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_extendreq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVExtendReqTrace(lAssocId, psTargetData->uTargetData.psExtendedServicesReq);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ExtendedServicesReq PDU to DBV format\n");
		return SI_NOTOK;
	}
}


int DBVsnacc_to_dbv_taclose(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{
	DBVAssociateDetails *psAssociateDetails = NULL;
        long current_state = DBV_OS_DEAD;
        DbvTargetEvent eTargetEvent = TA_Invalid_APDU;

	/* Ind if close request */
	/* Conf if close response */

	if(DBVTraceLevel(lAssocId) == Trace_High)
		DBVTrace(lAssocId,Trace_High, "Close (in Target)\n");

	/* Get the current state of the state machine */
	psAssociateDetails = SL_Current(DBVAssociateList);
	current_state = psAssociateDetails->lCurrentState;     

	/* If previously JUST sent a CLOSE PDU */
        if (current_state == DBV_TS_CLOSE_SENT)
 
                /* Assume that have got a 'closeResponse' */
                eTargetEvent = TA_CloseConf;
        else
                eTargetEvent = TA_CloseInd;
 
 

	if (DBVChangeState(lAssocId, DBV_TE_CLOSEREQ_PDU, ppsError) == SI_OK) {
		DBVset_dbv_targetfields_closereq(psTargetData, PDU_ptr, eTargetEvent, env);
		DBVReceivingTrace(lAssocId);
		DBVCloseTraceTarget(lAssocId, psTargetData->uTargetData.psCloseReq, eTargetEvent);

                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received Close PDU to DBV format\n");
		return SI_NOTOK;
	}
}



int DBVCloseTraceTarget(DbvAssocId lAssocId, DbvCloseReq *psCloseReq, DbvTargetEvent eTargetEvent)
{
	if (eTargetEvent == TA_CloseInd)
		DBVCloseReqTrace(lAssocId, psCloseReq);
	else
		DBVCloseRspTrace(lAssocId, psCloseReq);
}



int DBVsnacc_to_dbv_taunknown(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, char *ber_data, unsigned long ber_data_length, jmp_buf env)
{
	psTargetData->eTargetEvent = TA_Invalid_APDU;
	psTargetData->uTargetData.psInvalidAPDU = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
	CheckDbvAlloc(psTargetData->uTargetData.psInvalidAPDU, env);

	psTargetData->uTargetData.psInvalidAPDU->pcValue = (char *) DbvAlloc(ber_data_length);
	CheckDbvAlloc(psTargetData->uTargetData.psInvalidAPDU->pcValue, env);
	memcpy(psTargetData->uTargetData.psInvalidAPDU->pcValue, ber_data, ber_data_length);
	psTargetData->uTargetData.psInvalidAPDU->ulLength = ber_data_length;

        DBVReceivingTrace(lAssocId);
        DBVUnknownPDUTrace(lAssocId, psTargetData->uTargetData.psInvalidAPDU);

	/* Output the Unknown PDU to a special file */
        DBVoutput_invalid_pdu(psTargetData->uTargetData.psInvalidAPDU);

	/* Now the ber data has been assigned to the DBV structure free it */
	if (ber_data != NULL)
		free(ber_data);

	return SI_OK;
}




/* AccessControlResponse sent from origin */
int DBVsnacc_to_dbv_accessrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{

	if (DBVChangeState(lAssocId, DBV_TE_ACCCNTRSP_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_DbvAccessControlRsp(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVAccRspTrace(lAssocId, psTargetData->uTargetData.psAccessControlRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received AccessControlRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



/* ResourceControlResponse sent from origin */
int DBVsnacc_to_dbv_rescntrlrsp(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{

	if (DBVChangeState(lAssocId, DBV_TE_RESCNTRSP_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_DbvResourceControlRsp(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVResCntrlRspTrace(lAssocId, psTargetData->uTargetData.psResourceControlRsp);
                return SI_OK;
	} else {
		DBVTrace(lAssocId, Trace_High, "Failed to convert received ResourceControlRsp PDU to DBV format\n");
		return SI_NOTOK;
	}
}



/* TriggerResourceControlRequest sent from origin */
int DBVsnacc_to_dbv_trigrescntrlreq(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError, DbvTargetData *psTargetData, jmp_buf env)
{

	if (DBVChangeState(lAssocId, DBV_TE_TRIGRESCNTREQ_PDU, ppsError) == SI_OK){
		DBVset_dbv_fields_DbvTriggerResourceControlReq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVTrigResCntrlReqTrace(lAssocId, psTargetData->uTargetData.psTriggerResourceControlReq);
                return SI_OK;
	} else {

		/* Special field setting and tracing for 
	           TriggerResourceControlRequest */
		DBVset_dbv_fields_DbvTriggerResourceControlReq(psTargetData, PDU_ptr, env);
		DBVReceivingTrace(lAssocId);
                DBVTrigResCntrlReqTrace(lAssocId, psTargetData->uTargetData.psTriggerResourceControlReq);

		return SI_NOTOK;
	}
}


