/*  Module Name: @(#)target_z3950_present.c	7.2
    Version: 4/18/97
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a PresentResponse PDU 
		  to an origin and process a received PresentRequest PDU
		  sent by an origin
		  

*/

#include "target_z3950_present.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE PresentResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvPresentResponse(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVPresentResponse",lAssocId);

	if (DBVCheckDbvPresRspFields(lAssocId, psPresentRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_PRESENTRESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVPresRspTrace(lAssocId, psPresentRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.presentResponse = (struct PresentResponse *) Asn1Alloc(sizeof(struct PresentResponse));
                        CheckAsn1Alloc(PDU_ptr->a.presentResponse, env);
        		PDU_ptr->choiceId = PDU_PRESENTRESPONSE;
                        DBVsetup_snacc_presresponse(PDU_ptr, psPresentRsp, env);
                } else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,
DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVPresentResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvPresRspFields(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_presrsp_pstat(lAssocId, psPresentRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_presrsp_records(lAssocId, psPresentRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_presrsp_othinfo(lAssocId, psPresentRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;
}
 


int DBVcheck_dbv_presrsp_pstat(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp, DbvError **ppsSIError) 
{
	
        if (DBVcheck_dbv_enum(psPresentRsp->ePresentStatus, PS_Success, PS_Failure) == SI_NOTOK) 
                return(DBVdbv_invalid_enum_error(lAssocId, "psPresentRsp->ePresentStatus", ppsSIError));
 
        return SI_OK;

}



int DBVcheck_dbv_presrsp_records(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp, DbvError **ppsSIError)
{
 
        /* Can be null because optional */
        if (psPresentRsp->psRecords == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_records(lAssocId, psPresentRsp->psRecords, "psPresentRsp->psRecords->", ppsSIError));
        return SI_OK;
}



int DBVcheck_dbv_presrsp_othinfo(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp, DbvError **ppsSIError)
{
        if (psPresentRsp->ppsOtherInformation == NULL)
                return SI_OK;

	return(DBVcheck_dbv_otherinfo(lAssocId, psPresentRsp->ppsOtherInformation, "psPresentRsp->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_presresponse(PDU *PDU_ptr, DbvPresentRsp *psPresentRsp, jmp_buf env)
{
	DBVset_snacc_presresp_refid(PDU_ptr, psPresentRsp->psReferenceId, env);
	DBVset_snacc_presresp_numrecs(PDU_ptr, psPresentRsp->ulNumberOfRecordsReturned, env);
	DBVset_snacc_presresp_nxtresset(PDU_ptr, psPresentRsp->ulNextResultSetPosition, env);
	DBVset_snacc_presresp_presstat(PDU_ptr, psPresentRsp->ePresentStatus, env);
	DBVset_snacc_presresp_records(PDU_ptr, psPresentRsp->psRecords, env);
	DBVset_snacc_presresp_otherinfo(PDU_ptr, psPresentRsp->ppsOtherInformation, env);
}


int DBVset_snacc_presresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.presentResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_presresp_numrecs(PDU *PDU_ptr, Ulong ulNumberOfRecordsReturned, jmp_buf env)
{
	PDU_ptr->a.presentResponse->numberOfRecordsReturned = ulNumberOfRecordsReturned;
}



int DBVset_snacc_presresp_nxtresset(PDU *PDU_ptr, Ulong ulNextResultSetPosition, jmp_buf env)
{
	PDU_ptr->a.presentResponse->nextResultSetPosition = ulNextResultSetPosition;
}



int DBVset_snacc_presresp_presstat(PDU *PDU_ptr, DbvPresentStatus ePresentStatus, jmp_buf env)
{
	/* presentStatus in snacc SearchResponse is PresentStatus* 
           presentStatus in snacc PresentResponse is PresentStatus 
        */

	DBVdbv_to_snacc_presstat(&PDU_ptr->a.presentResponse->presentStatus, ePresentStatus);
}



int DBVset_snacc_presresp_records(PDU *PDU_ptr, DbvRecords *psRecords, jmp_buf env)
{
	if (psRecords != NULL) {
		PDU_ptr->a.presentResponse->records = (struct Records *) Asn1Alloc (sizeof (struct Records));
		CheckAsn1Alloc(PDU_ptr->a.presentResponse->records, env);

		DBVdbv_to_snacc_records(PDU_ptr->a.presentResponse->records, psRecords, env);
	}
		
	
}




int DBVset_snacc_presresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.presentResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.presentResponse->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.presentResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS PresentRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_presreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_PresentInd;
        psTargetData->uTargetData.psPresentReq = (DbvPresentReq *) DbvAlloc ( sizeof (DbvPresentReq));
	CheckDbvAlloc(psTargetData->uTargetData.psPresentReq, env);
 
        DBVset_dbv_presreq_refid(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_ressetid(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_ressetstart(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_numrecsreq(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_addranges(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_reccomp(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_prefrecsyn(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_maxsegcnt(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_maxrecsze(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
        DBVset_dbv_presreq_maxsegsze(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
	DBVset_dbv_presreq_otherinfo(psTargetData->uTargetData.psPresentReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_presreq_refid(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.presentRequest->referenceId.octetLen != 0){
		presreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(presreq->psReferenceId, env);
		DBVsnacc_to_dbv_octet_string(presreq->psReferenceId, &(PDU_ptr->a.presentRequest->referenceId), env);
        }

}



int DBVset_dbv_presreq_ressetid(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_inter_string(&(presreq->sResultSetId), &(PDU_ptr->a.presentRequest->resultSetId), env);
	
}



int DBVset_dbv_presreq_ressetstart(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{

	presreq->ulResultSetStartPoint = PDU_ptr->a.presentRequest->resultSetStartPoint;

}



int DBVset_dbv_presreq_numrecsreq(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	presreq->ulNumberOfRecordsRequested = PDU_ptr->a.presentRequest->numberOfRecordsRequested;
}



int DBVset_dbv_presreq_addranges(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{

	Range *data_ptr;
	int num_elements = 0;
	int index = 0;
 

	/* If an additional ranges list exists */
	if (PDU_ptr->a.presentRequest->additionalRanges != NULL) {

		/* Find the number of elements in the AsnList so that we can 
	   	malloc a correct size for ppsAdditionalRanges */
		num_elements = AsnListCount(PDU_ptr->a.presentRequest->additionalRanges);

		/* +1 to allocate space for the final NULL */
		presreq->ppsAdditionalRanges = (DbvRange **) DbvAlloc (sizeof(DbvRange *) * (num_elements + 1));
		CheckDbvAlloc(presreq->ppsAdditionalRanges, env);
	
		FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.presentRequest->additionalRanges) {
			presreq->ppsAdditionalRanges[index] = (DbvRange *) DbvAlloc(sizeof (DbvRange));
			CheckDbvAlloc(presreq->ppsAdditionalRanges[index], env);

			presreq->ppsAdditionalRanges[index]->ulStartingPosition = data_ptr->startingPosition;
			presreq->ppsAdditionalRanges[index]->ulNumberOfRecords = data_ptr->numberOfRecords;
			index++;
		}

		presreq->ppsAdditionalRanges[index] = NULL;
	}

}



int DBVset_dbv_presreq_reccomp(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env) 
{
        if (PDU_ptr->a.presentRequest->recordComposition != NULL) {
		presreq->psRecordComposition = (DbvRecordComposition *) DbvAlloc (sizeof (struct DbvRecordComposition));
		CheckDbvAlloc(presreq->psRecordComposition, env);
		DBVsnacc_to_dbv_DbvRecordComposition(presreq->psRecordComposition, PDU_ptr->a.presentRequest->recordComposition, env);
	}
}


	
int DBVset_dbv_presreq_prefrecsyn(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{

	char *oid;

        if(PDU_ptr->a.presentRequest->preferredRecordSyntax.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&(PDU_ptr->a.presentRequest->preferredRecordSyntax));
		presreq->pcPreferredRecordSyntax = (char *) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(presreq->pcPreferredRecordSyntax, env);

                memcpy(presreq->pcPreferredRecordSyntax, oid, strlen(oid));
        }

}



int DBVset_dbv_presreq_maxsegcnt(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.presentRequest->maxSegmentCount != NULL) {
		presreq->pulMaxSegmentCount = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(presreq->pulMaxSegmentCount, env);
		*presreq->pulMaxSegmentCount = *PDU_ptr->a.presentRequest->maxSegmentCount;
	}
}



int DBVset_dbv_presreq_maxrecsze(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.presentRequest->maxRecordSize != NULL) {
		presreq->pulMaxRecordSize = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(presreq->pulMaxRecordSize, env);
		*presreq->pulMaxRecordSize = *PDU_ptr->a.presentRequest->maxRecordSize;
	}
}



int DBVset_dbv_presreq_maxsegsze(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.presentRequest->maxSegmentSize != NULL) {
		presreq->pulMaxSegmentSize = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(presreq->pulMaxSegmentSize, env);

		*presreq->pulMaxSegmentSize = *PDU_ptr->a.presentRequest->maxSegmentSize;
	}
}




int DBVset_dbv_presreq_otherinfo(DbvPresentReq *presreq, PDU *PDU_ptr, jmp_buf env)
{

       int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.presentRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.presentRequest->otherInfo);
		presreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(presreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(presreq->ppsOtherInformation, PDU_ptr->a.presentRequest->otherInfo, env);
	}
}



