/*  Module Name: @(#)target_z3950_rescntrl.c	7.2
    Version: 4/16/97
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a 
		  ResourceControlRequest PDU to an origin and process a 
		  received ResourceControlResponse PDU sent by an origin
		  

*/


#include "target_z3950_rescntrl.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE ResourceControlRequest PDU TO SEND TO ORIGIN AND SEND IT 
*/

/* one of the few requests sent from a target to an origin */

int DbvResourceControlRequest(DbvAssocId lAssocId, DbvResourceControlReq *psResourceControlReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;


	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVResourceControlRequest", lAssocId);

        if (DBVCheckDbvResourceControlReqFields(lAssocId, psResourceControlReq, ppsSIError) == SI_OK) {

		/* Set a global variable used to indicate if a
		   ResourceControlResponse PDU is required. This variable 
		   will be used by the state machine */
		ResourceControlResponseRequired = psResourceControlReq->fResponseRequired;
		if (DBVChangeState(lAssocId, DBV_TE_RESCNTREQ, ppsSIError) == SI_OK)

        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK) 
				/* PDU can be created and sent */
				return_value = SI_OK;
	}

	if (return_value == SI_OK) {
		DBVSendingTrace(lAssocId);
                DBVResCntrlReqTrace(lAssocId, psResourceControlReq);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.resourceControlRequest = (struct ResourceControlRequest *) Asn1Alloc (sizeof(struct ResourceControlRequest));
			CheckAsn1Alloc(PDU_ptr->a.resourceControlRequest, env);
        		PDU_ptr->choiceId = PDU_RESOURCECONTROLREQUEST;
			DBVsetup_snacc_ResourceControlRequest(PDU_ptr, psResourceControlReq, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }


		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVResourceControlRequest", lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvResourceControlReqFields(DbvAssocId lAssocId, DbvResourceControlReq *psResourceControlReq, DbvError **ppsSIError)
{
	/* o */
	/* Dont have to check psReferenceId */

	/* o */
	if (DBVcheck_DbvBool_opt(lAssocId, psResourceControlReq->pfSuspendedFlag, "*psResourceControlReq->pfSuspendedFlag", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
	if (DBVcheck_DbvASN1External_opt(lAssocId, psResourceControlReq->psResourceReport, "psResourceControlReq->psResourceReport->", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvPartialResultsAvailable_opt(lAssocId, psResourceControlReq->pePartialResultsAvailable, "*psResourceControlReq->pePartialResultsAvailable", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
	if (DBVcheck_DbvBool(lAssocId, &psResourceControlReq->fResponseRequired, "psResourceControlReq->fResponseRequired", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvBool_opt(lAssocId, psResourceControlReq->pfTriggeredRequestFlag, "*psResourceControlReq->pfTriggeredRequestFlag", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (DBVcheck_DbvOtherInformation_opt(lAssocId, psResourceControlReq->ppsOtherInformation, "psResourceControlReq->ppsOtherInformation", ppsSIError))
                return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvPartialResultsAvailable_opt(DbvAssocId lAssocId, DbvPartialResultsAvailable *pePartialResultsAvailable, char *error_buffer, DbvError **ppsSIError)
{
	if (pePartialResultsAvailable)
		return(DBVcheck_DbvPartialResultsAvailable(lAssocId, pePartialResultsAvailable, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvPartialResultsAvailable(DbvAssocId lAssocId, DbvPartialResultsAvailable *pePartialResultsAvailable, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
                
	if (DBVcheck_dbv_enum(*pePartialResultsAvailable, PRA_Subset, PRA_None2) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));


	return SI_OK;
}





/* snacc set up */

int DBVsetup_snacc_ResourceControlRequest(PDU *PDU_ptr, DbvResourceControlReq *psResourceControlReq, jmp_buf env)
{
	/* o */
	DBVset_ResourceControlRequest_referenceId(PDU_ptr, psResourceControlReq->psReferenceId, env);

	/* o */
	DBVset_ResourceControlRequest_suspendedFlag(PDU_ptr, psResourceControlReq->pfSuspendedFlag, env);

	/* o */
	DBVset_ResourceControlRequest_resourceReport(PDU_ptr, psResourceControlReq->psResourceReport, env);


	/* o */
	DBVset_ResourceControlRequest_partialResultsAvailable(PDU_ptr, psResourceControlReq->pePartialResultsAvailable, env);


	/* c */
	DBVset_ResourceControlRequest_responseRequired(PDU_ptr, &psResourceControlReq->fResponseRequired, env);


	/* o */
	DBVset_ResourceControlRequest_triggeredRequestFlag(PDU_ptr, psResourceControlReq->pfTriggeredRequestFlag, env);


	/* o */
	DBVset_ResourceControlRequest_otherinfo(PDU_ptr, psResourceControlReq->ppsOtherInformation, env);
}



int DBVset_ResourceControlRequest_referenceId(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvOctetString_opt(&(PDU_ptr->a.resourceControlRequest->referenceId), psReferenceId, env);
	
}




int DBVset_ResourceControlRequest_suspendedFlag(PDU *PDU_ptr, DbvBool *pfSuspendedFlag, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvBool_opt(&PDU_ptr->a.resourceControlRequest->suspendedFlag, pfSuspendedFlag, env);
}



int DBVset_ResourceControlRequest_resourceReport(PDU *PDU_ptr, DbvASN1External *psResourceReport, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_external_opt(&PDU_ptr->a.resourceControlRequest->resourceReport, psResourceReport, env);
}



int DBVset_ResourceControlRequest_partialResultsAvailable(PDU *PDU_ptr, DbvPartialResultsAvailable *pePartialResultsAvailable, jmp_buf env)
{

	/* o */
	if (pePartialResultsAvailable) {

		PDU_ptr->a.resourceControlRequest->partialResultsAvailable = (ResourceControlRequestInt *) Asn1Alloc (sizeof (ResourceControlRequestInt));
		CheckAsn1Alloc(PDU_ptr->a.resourceControlRequest->partialResultsAvailable, env);

		switch (*pePartialResultsAvailable) {
		case PRA_Subset:
			*PDU_ptr->a.resourceControlRequest->partialResultsAvailable = SUBSET1;
			break;
		case PRA_Interim2:
			*PDU_ptr->a.resourceControlRequest->partialResultsAvailable = INTERIM2;
			break;
		case PRA_None2:
			*PDU_ptr->a.resourceControlRequest->partialResultsAvailable = NONE2;
			break;
		}
	}
}



int DBVset_ResourceControlRequest_responseRequired(PDU *PDU_ptr, DbvBool *pfResponseRequired, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvBool(&PDU_ptr->a.resourceControlRequest->responseRequired, pfResponseRequired, env);
}



int DBVset_ResourceControlRequest_triggeredRequestFlag(PDU *PDU_ptr, DbvBool *pfTriggeredRequestFlag, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvBool_opt(&PDU_ptr->a.resourceControlRequest->triggeredRequestFlag, pfTriggeredRequestFlag, env);
}



int DBVset_ResourceControlRequest_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_otherinfo_opt(&PDU_ptr->a.resourceControlRequest->otherInfo, ppsOtherInformation, env);

}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS ResourceControlResponse PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_DbvResourceControlRsp(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_ResourCntrlConf;
        psTargetData->uTargetData.psResourceControlRsp = (DbvResourceControlRsp *) DbvAlloc ( sizeof (DbvResourceControlRsp));
	CheckDbvAlloc(psTargetData->uTargetData.psResourceControlRsp, env);

	/* o */
        DBVset_DbvResourceControlRsp_psReferenceId(psTargetData->uTargetData.psResourceControlRsp, PDU_ptr, env);

	/* c */
        DBVset_DbvResourceControlRsp_fContinueFlag(psTargetData->uTargetData.psResourceControlRsp, PDU_ptr, env);

	/* o */
        DBVset_DbvResourceControlRsp_pfResultSetWanted(psTargetData->uTargetData.psResourceControlRsp, PDU_ptr, env);

	/* o */
	DBVset_DbvResourceControlRsp_ppsOtherInformation(psTargetData->uTargetData.psResourceControlRsp, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



int DBVset_DbvResourceControlRsp_psReferenceId(DbvResourceControlRsp *psResourceControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* o */	
	DBVsnacc_to_dbv_DbvOctetString_opt(&psResourceControlRsp->psReferenceId, &PDU_ptr->a.resourceControlResponse->referenceId, env);
}



int DBVset_DbvResourceControlRsp_fContinueFlag(DbvResourceControlRsp *psResourceControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvBool(&psResourceControlRsp->fContinueFlag, &PDU_ptr->a.resourceControlResponse->continueFlag, env);
}



int DBVset_DbvResourceControlRsp_pfResultSetWanted(DbvResourceControlRsp *psResourceControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvBool_opt(&psResourceControlRsp->pfResultSetWanted, PDU_ptr->a.resourceControlResponse->resultSetWanted, env);
}



int DBVset_DbvResourceControlRsp_ppsOtherInformation(DbvResourceControlRsp *psResourceControlRsp, PDU *PDU_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_otherinfo_opt(&psResourceControlRsp->ppsOtherInformation, PDU_ptr->a.resourceControlResponse->otherInfo, env);
}



