/*  Module Name: @(#)target_z3950_resrep.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a 
		  ResourceReportResponse PDU to an origin and process a 
		  received ResourceReportRequest PDU sent by an origin
		  

*/

#include "target_z3950_resrep.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE ResourceReportResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvResourceReportResponse(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;
	

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVResourceReportResponse",lAssocId);

	if (DBVCheckDbvResRepRspFields(lAssocId, psResourceReportRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_RESOURCERESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVResRepRspTrace(lAssocId, psResourceReportRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.resourceReportResponse = (struct ResourceReportResponse *) Asn1Alloc (sizeof(struct ResourceReportResponse));
			CheckAsn1Alloc(PDU_ptr->a.resourceReportResponse, env);
        		PDU_ptr->choiceId = PDU_RESOURCEREPORTRESPONSE;
			DBVsetup_snacc_resrepresponse(PDU_ptr, psResourceReportRsp, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

 
		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVResourceReportResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvResRepRspFields(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_rerersp_repstat(lAssocId, psResourceReportRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_rerersp_resrep(lAssocId, psResourceReportRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_rerersp_otherinfo(lAssocId, psResourceReportRsp, ppsSIError))
                return SI_NOTOK;
 
        return SI_OK;
}



int DBVcheck_dbv_rerersp_repstat(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp, DbvError **ppsSIError)
{
 
        if (DBVcheck_dbv_enum(psResourceReportRsp->eResourceReportStatus, RRS_Success, RRS_Failure_6) == SI_NOTOK)
                return(DBVdbv_invalid_enum_error(lAssocId, "psResourceReportRsp->eResourceReportStatus", ppsSIError));
 
        return SI_OK;

}



int DBVcheck_dbv_rerersp_resrep(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psResourceReportRsp->psResourceReport == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_external(lAssocId, psResourceReportRsp->psResourceReport, "psResourceReportRsp->psResourceReport->", ppsSIError));
 

}



int DBVcheck_dbv_rerersp_otherinfo(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psResourceReportRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psResourceReportRsp->ppsOtherInformation, "psResourceReportRsp->ppsOtherInformation", ppsSIError));

}





int DBVsetup_snacc_resrepresponse(PDU *PDU_ptr, DbvResourceReportRsp *psResourceReportRsp, jmp_buf env)
{
	DBVset_snacc_resrepresp_refid(PDU_ptr, psResourceReportRsp->psReferenceId, env);
	DBVset_snacc_resrepresp_repstat(PDU_ptr, psResourceReportRsp->eResourceReportStatus, env);
	DBVset_snacc_resrepresp_report(PDU_ptr, psResourceReportRsp->psResourceReport, env);

        DBVset_snacc_resrepresp_otherinfo(PDU_ptr, psResourceReportRsp->ppsOtherInformation, env);
}



int DBVset_snacc_resrepresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.resourceReportResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_resrepresp_repstat(PDU *PDU_ptr, DbvResourceReportStatus eResourceReportStatus, jmp_buf env)
{
	/* Following compulsory */
	switch (eResourceReportStatus) {
	case RRS_Success:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = SUCCESS4;
		break;
	case RRS_Partial:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = PARTIAL;
		break;
	case RRS_Failure_1:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_1;
		break;
	case RRS_Failure_2:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_2;
		break;
	case RRS_Failure_3:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_3;
		break;
	case RRS_Failure_4:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_4;
		break;
	case RRS_Failure_5:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_5;
		break;
	case RRS_Failure_6:
		PDU_ptr->a.resourceReportResponse->resourceReportStatus = FAILURE_6;
		break;
	}
}



int DBVset_snacc_resrepresp_report(PDU *PDU_ptr, DbvASN1External *psResourceReport, jmp_buf env) 
{
	if  (psResourceReport != NULL) {
		PDU_ptr->a.resourceReportResponse->resourceReport = (ResourceReport *) Asn1Alloc (sizeof(ResourceReport));
		CheckAsn1Alloc(PDU_ptr->a.resourceReportResponse->resourceReport, env);

		DBVdbv_to_snacc_external(PDU_ptr->a.resourceReportResponse->resourceReport, psResourceReport, env);
	}
}



int DBVset_snacc_resrepresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.resourceReportResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.resourceReportResponse->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.resourceReportResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS ResourceReportRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_resrepreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_ResourceReportInd;
        psTargetData->uTargetData.psResourceReportReq = (DbvResourceReportReq *) DbvAlloc ( sizeof (DbvResourceReportReq));
	CheckDbvAlloc(psTargetData->uTargetData.psResourceReportReq, env);
 
        DBVset_dbv_resrepreq_refid(psTargetData->uTargetData.psResourceReportReq, PDU_ptr, env);
        DBVset_dbv_resrepreq_opid(psTargetData->uTargetData.psResourceReportReq, PDU_ptr, env);
        DBVset_dbv_resrepreq_prefformat(psTargetData->uTargetData.psResourceReportReq, PDU_ptr, env);
	DBVset_dbv_resrepreq_otherinfo(psTargetData->uTargetData.psResourceReportReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_resrepreq_refid(DbvResourceReportReq *resrepreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.resourceReportRequest->referenceId.octetLen != 0){
                resrepreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(resrepreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(resrepreq->psReferenceId, &(PDU_ptr->a.resourceReportRequest->referenceId), env);

        }
}
 


int DBVset_dbv_resrepreq_opid(DbvResourceReportReq *resrepreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.resourceReportRequest->opId.octetLen != 0){
                resrepreq->psOperationId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(resrepreq->psOperationId, env);

		DBVsnacc_to_dbv_octet_string(resrepreq->psOperationId, &(PDU_ptr->a.resourceReportRequest->opId), env);

        }
}



int DBVset_dbv_resrepreq_prefformat(DbvResourceReportReq *resrepreq, PDU *PDU_ptr, jmp_buf env)
{
	char *oid = NULL;
 
        if (PDU_ptr->a.resourceReportRequest->prefResourceReportFormat.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&(PDU_ptr->a.resourceReportRequest->prefResourceReportFormat));
                resrepreq->pcPrefResourceReportFormat = (char *) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(resrepreq->pcPrefResourceReportFormat, env);
                strcpy(resrepreq->pcPrefResourceReportFormat, oid);
        }
}



int DBVset_dbv_resrepreq_otherinfo(DbvResourceReportReq *resrepreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.resourceReportRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.resourceReportRequest->otherInfo);
		resrepreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(resrepreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(resrepreq->ppsOtherInformation, PDU_ptr->a.resourceReportRequest->otherInfo, env);
	}
}




