/*  Module Name: @(#)target_z3950_scan.c	7.2
    Version: 4/16/97
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a ScanResponse PDU 
		  to an origin and process a received ScanRequest PDU
		  sent by an origin
		  

*/

#include "target_z3950_scan.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE ScanResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvScanResponse(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVScanResponse",lAssocId);

	if (DBVCheckDbvScanRspFields(lAssocId, psScanRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_SCANRESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVScanRspTrace(lAssocId, psScanRsp);

        	PDU_ptr = &pdu_to_encode;
		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.scanResponse = (struct ScanResponse *) Asn1Alloc(sizeof(struct ScanResponse));
			CheckAsn1Alloc(PDU_ptr->a.scanResponse, env);
        		PDU_ptr->choiceId = PDU_SCANRESPONSE;
			DBVsetup_snacc_scanresponse(PDU_ptr, psScanRsp, env);
		} else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVScanResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvScanRspFields(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_scanrsp_stat(lAssocId, psScanRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanrsp_entries(lAssocId, psScanRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanrsp_attset(lAssocId, psScanRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_scanrsp_otherinfo(lAssocId, psScanRsp, ppsSIError))
                return SI_NOTOK;
 
        return SI_OK;
}



int DBVcheck_dbv_scanrsp_stat(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_enum(psScanRsp->eScanStatus, SCS_Success, SCS_Failure) == SI_NOTOK)
                return(DBVdbv_invalid_enum_error(lAssocId, "psScanRsp->eScanStatus", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_scanrsp_entries(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psScanRsp->psEntries == NULL)
                return SI_OK;
 

        return(DBVcheck_dbv_entries(lAssocId, psScanRsp->psEntries, "psScanRsp->psEntries->", ppsSIError));
}



int DBVcheck_dbv_entries(DbvAssocId lAssocId, DbvListEntries *psEntries, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psEntries->ppsEntry != NULL) 
		if (DBVcheck_dbv_entry_list(lAssocId, psEntries->ppsEntry, local_error_buffer, ppsSIError) == SI_NOTOK)

			return SI_NOTOK;


	if (psEntries->ppsNonSurrogateDiagnostics != NULL) {


		DBVsa_strcat(local_error_buffer, "ppsNonSurrogateDiagnostic");
		return(DBVcheck_dbv_multdiag(lAssocId, psEntries->ppsNonSurrogateDiagnostics, local_error_buffer, ppsSIError));
	}
		

	return SI_OK;
}




int DBVcheck_dbv_entry_list(DbvAssocId lAssocId, DbvEntry **ppsEntry, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	

	while (ppsEntry[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsEntry[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");

		if (DBVcheck_dbv_entry(lAssocId, ppsEntry[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}
			
	return SI_OK;
}



int DBVcheck_dbv_entry(DbvAssocId lAssocId, DbvEntry *psEntry, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psEntry->eEntryType) {
	case ET_TermInfo:
		DBVsa_strcat(local_error_buffer, "uEntry.psTermInfo");

		if (psEntry->uEntry.psTermInfo == NULL)
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_terminfo(lAssocId, psEntry->uEntry.psTermInfo, local_error_buffer, ppsSIError));
		break;
	case ET_SurrogateDiagnostic:
		DBVsa_strcat(local_error_buffer, "uEntry.psSurrogateDiagnostic");
		if (psEntry->uEntry.psSurrogateDiagnostic == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		
		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_surrdiag(lAssocId, psEntry->uEntry.psSurrogateDiagnostic, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eEntryType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
	}
	

	return SI_OK;
}



int DBVcheck_dbv_terminfo(DbvAssocId lAssocId, DbvTermInfo *psTermInfo, char* error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);

	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sTerm.");
	if (DBVcheck_DbvTerm(lAssocId, &psTermInfo->sTerm, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	if (psTermInfo->ppsSuggestedAttribute != NULL) {
		if (DBVcheck_dbv_attel_list(lAssocId, psTermInfo->ppsSuggestedAttribute, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


	if (psTermInfo->ppsAlternativeTerm != NULL) {
		if (DBVcheck_attsplusterm_list(lAssocId, psTermInfo->ppsAlternativeTerm,local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


	if (psTermInfo->ppsByAttributes != NULL) {
		if (DBVcheck_dbv_occurbyatt_list(lAssocId, psTermInfo->ppsByAttributes, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


        /* Can be null because optional */
        if (psTermInfo->ppsOtherInformation == NULL)
                return SI_OK;
 
	DBVsa_strcat(error_buffer, "ppsOtherInformation");
        return(DBVcheck_dbv_otherinfo(lAssocId, psTermInfo->ppsOtherInformation, local_error_buffer, ppsSIError));
}




int DBVcheck_dbv_attel_list(DbvAssocId lAssocId, DbvAttributeElement **ppsAttributeElement, char *error_buffer, DbvError **ppsSIError)
{

	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);

 
	
	/* Check all items in the list */
        while (ppsAttributeElement[index] != NULL) {
	
                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsAttributeElement[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");

                if (DBVcheck_dbv_attelement(lAssocId, ppsAttributeElement[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}
 

	return SI_OK;
}



int DBVcheck_attsplusterm_list(DbvAssocId lAssocId, DbvAttributesPlusTerm **ppsAlternativeTerm, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);
 
 
 
        /* Check all items in the list */
        while (ppsAlternativeTerm[index] != NULL) {
	
                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsAlternativeTerm[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_dbv_attsplusterm(lAssocId, ppsAlternativeTerm[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
 
        	index++;
        }
 
        return SI_OK;
}



int DBVcheck_dbv_occurbyatt_list(DbvAssocId lAssocId, DbvOccurrenceByAttributes **ppsByAttributes, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);
 
 
 
        /* Check all items in the list */
        while (ppsByAttributes[index] != NULL) {
	
                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsByAttributes[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_byattributes(lAssocId, ppsByAttributes[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
 
        	index++;
        }
 
        return SI_OK;
}



int DBVcheck_byattributes(DbvAssocId lAssocId, DbvOccurrenceByAttributes *psByAttributes, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

        /* if no list nodes then error as compulsory */
        if (psByAttributes->ppsAttributeElement == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsAttributeElement");
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	if (DBVcheck_attel_list_comp(lAssocId, psByAttributes->ppsAttributeElement, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* optional */
	if (psByAttributes->psOccurrence != NULL) {
		DBVsa_strcat(local_error_buffer, "psOccurrence->");
		if (DBVcheck_occurrence(lAssocId, psByAttributes->psOccurrence, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


        /* Can be null because optional */
        if (psByAttributes->ppsOtherInformation != NULL) {
		DBVsa_strcat(local_error_buffer, "ppsOtherInformation");
        	return(DBVcheck_dbv_otherinfo(lAssocId, psByAttributes->ppsOtherInformation, local_error_buffer, ppsSIError));
	}


	return SI_OK;
}



int DBVcheck_attel_list_comp(DbvAssocId lAssocId, DbvAttributeElement **ppsAttributeElement, char *error_buffer, DbvError **ppsSIError)
{

	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];


	DBVsa_strcpy(local_error_buffer, error_buffer);
 

	
	/* Check all items in the list */
        while (ppsAttributeElement[index] != NULL) {
 
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsAttributeElement[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]");

                if (DBVcheck_dbv_attelement(lAssocId, ppsAttributeElement[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}
	return SI_OK;
}






int DBVcheck_occurrence(DbvAssocId lAssocId, DbvOccurrence *psOccurrence, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);
 
 
	switch (psOccurrence->eOccurrenceType) {
	case OT_Global:
		break;
	case OT_ByDatabase:

        	if (psOccurrence->uOccurence.ppsOccurrenceByDb == NULL) {
			DBVsa_strcat(local_error_buffer, "ppsOccurrenceByDb");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		return(DBVcheck_occurbydb_list(lAssocId, psOccurrence->uOccurence.ppsOccurrenceByDb, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eOccurrenceType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
	}

	return SI_OK;
}



int DBVcheck_occurbydb_list(DbvAssocId lAssocId, DbvOccurrenceByDb **ppsOccurrenceByDb, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];


	DBVsa_strcpy(local_error_buffer, error_buffer);


	
	/* Check all items in the list */
        while (ppsOccurrenceByDb[index] != NULL) {

                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsOccurrenceByDb[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_occurbydb(lAssocId, ppsOccurrenceByDb[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}
	return SI_OK;
}



int DBVcheck_occurbydb(DbvAssocId lAssocId, DbvOccurrenceByDb *psOccurrenceByDb, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

        /* Can be null because optional */
	if (psOccurrenceByDb->ppsOtherInformation != NULL) {
		DBVsa_strcat(local_error_buffer, "ppsOtherInformation");
        	return(DBVcheck_dbv_otherinfo(lAssocId, psOccurrenceByDb->ppsOtherInformation, local_error_buffer, ppsSIError));
	}

	return SI_OK;
}



int DBVcheck_dbv_scanrsp_attset(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
        if (psScanRsp->pcAttributeSetId == NULL)
                return SI_OK;
 
        if (DBVcheck_dbv_attsetoid(psScanRsp->pcAttributeSetId) == SI_NOTOK)
                return(DBVdbv_oid_error(lAssocId, "psScanRsp->pcAttributeSetId" , ppsSIError));
 
        return SI_OK;

}



int DBVcheck_dbv_scanrsp_otherinfo(DbvAssocId lAssocId, DbvScanRsp *psScanRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psScanRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psScanRsp->ppsOtherInformation, "psScanRsp->ppsOtherInformation", ppsSIError));

}



int DBVsetup_snacc_scanresponse(PDU *PDU_ptr, DbvScanRsp *psScanRsp, jmp_buf env)
{
	DBVset_snacc_scanresp_refid(PDU_ptr, psScanRsp->psReferenceId, env);
	DBVset_snacc_scanresp_stepsize(PDU_ptr, psScanRsp->pulStepSize, env);
	DBVset_snacc_scanresp_scanstat(PDU_ptr, psScanRsp->eScanStatus, env);
	DBVset_snacc_scanresp_numentries(PDU_ptr, psScanRsp->ulNumberOfEntriesReturned, env);
	DBVset_snacc_scanresp_posterm(PDU_ptr, psScanRsp->pulPositionOfTerm, env);
	DBVset_snacc_scanresp_entries(PDU_ptr, psScanRsp->psEntries, env);
	DBVset_snacc_scanresp_attsetid(PDU_ptr, psScanRsp->pcAttributeSetId, env);
	DBVset_snacc_scanresp_otherinfo(PDU_ptr, psScanRsp->ppsOtherInformation, env);
}



int DBVset_snacc_scanresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.scanResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_scanresp_stepsize(PDU *PDU_ptr, Ulong *pulStepSize, jmp_buf env)
{
	if (pulStepSize != NULL) {
		PDU_ptr->a.scanResponse->stepSize = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.scanResponse->stepSize, env);

		*PDU_ptr->a.scanResponse->stepSize = *pulStepSize;
	}
}



int DBVset_snacc_scanresp_scanstat(PDU *PDU_ptr, DbvScanStatus eScanStatus, jmp_buf env)
{
	/* Compulsory */
	switch (eScanStatus) {
	case SCS_Success:
		PDU_ptr->a.scanResponse->scanStatus = SUCCESS3;
		break;
	case SCS_Partial_1:
		PDU_ptr->a.scanResponse->scanStatus = PARTIAL_12;
		break;
	case SCS_Partial_2:
		PDU_ptr->a.scanResponse->scanStatus = PARTIAL_21;
		break;
	case SCS_Partial_3:
		PDU_ptr->a.scanResponse->scanStatus = PARTIAL_31;
		break;
	case SCS_Partial_4:
		PDU_ptr->a.scanResponse->scanStatus = PARTIAL_41;
		break;
	case SCS_Partial_5:
		PDU_ptr->a.scanResponse->scanStatus = PARTIAL_5;
		break;
	case SCS_Failure:
		PDU_ptr->a.scanResponse->scanStatus = FAILURE3;
		break;
	}
}



int DBVset_snacc_scanresp_numentries(PDU *PDU_ptr,Ulong ulNumberOfEntriesReturned, jmp_buf env)
{
	PDU_ptr->a.scanResponse->numberOfEntriesReturned = ulNumberOfEntriesReturned;
}



int DBVset_snacc_scanresp_posterm(PDU *PDU_ptr, Ulong *pulPositionOfTerm, jmp_buf env)
{
	if (pulPositionOfTerm != NULL) {
		PDU_ptr->a.scanResponse->positionOfTerm = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(PDU_ptr->a.scanResponse->positionOfTerm, env);

		*PDU_ptr->a.scanResponse->positionOfTerm = *pulPositionOfTerm;
	}
}



int DBVset_snacc_scanresp_entries(PDU *PDU_ptr, DbvListEntries *psEntries, jmp_buf env)
{
	if (psEntries != NULL) {
		PDU_ptr->a.scanResponse->entries = (struct ListEntries *) Asn1Alloc (sizeof(struct ListEntries));
		CheckAsn1Alloc(PDU_ptr->a.scanResponse->entries, env);

		DBVdbv_to_snacc_listent(PDU_ptr->a.scanResponse->entries, psEntries, env);
	}
}



int DBVdbv_to_snacc_listent(struct ListEntries *entries, DbvListEntries *psEntries, jmp_buf env)
{
	/* Both optional */

	if (psEntries->ppsEntry != NULL) {
		entries->entries = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(entries->entries, env);

		DBVdbv_to_snacc_entries(entries->entries, psEntries->ppsEntry, env);
	}

	if (psEntries->ppsNonSurrogateDiagnostics != NULL) {
		entries->nonsurrogateDiagnostics = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(entries->nonsurrogateDiagnostics, env);

		DBVdbv_to_snacc_multdiag(entries->nonsurrogateDiagnostics, psEntries->ppsNonSurrogateDiagnostics, env);
	}
}



int DBVdbv_to_snacc_entries(ListEntriesSeqOf *entries, DbvEntry **ppsEntry, jmp_buf env)
{
	Entry *data_ptr;
        Entry **data_ptr_address;
 
        int current_listnode = 0;
 
        while(ppsEntry[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(entries);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Entry));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_entry(data_ptr, ppsEntry[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_entry(Entry *data_ptr, DbvEntry *psEntry, jmp_buf env)
{
	switch (psEntry->eEntryType) {
	case ET_TermInfo:
		data_ptr->choiceId = ENTRY_TERMINFO;
		data_ptr->a.termInfo = (struct TermInfo*) Asn1Alloc (sizeof (struct TermInfo));
		CheckAsn1Alloc(data_ptr->a.termInfo, env);

		DBVdbv_to_snacc_terminfo(data_ptr->a.termInfo, psEntry->uEntry.psTermInfo, env);
		break;
	case ET_SurrogateDiagnostic:
		data_ptr->choiceId = ENTRY_SURROGATEDIAGNOSTIC;
		data_ptr->a.surrogateDiagnostic = (struct DiagRec*) Asn1Alloc (sizeof (struct DiagRec));
		CheckAsn1Alloc(data_ptr->a.surrogateDiagnostic, env);

		DBVdbv_to_snacc_diag_rec(data_ptr->a.surrogateDiagnostic, psEntry->uEntry.psSurrogateDiagnostic, env);
		break;
	}
}



int DBVdbv_to_snacc_terminfo(struct TermInfo* termInfo, DbvTermInfo *psTermInfo, jmp_buf env)
{
	/* Term compulsory */
	termInfo->term = (struct Term*) Asn1Alloc (sizeof (struct Term));
	CheckAsn1Alloc(termInfo->term, env);
	DBVdbv_to_snacc_term(termInfo->term, &psTermInfo->sTerm, env);


	if (psTermInfo->psDisplayTerm != NULL) 
		DBVdbv_to_snacc_inter_string(&termInfo->displayTerm, psTermInfo->psDisplayTerm, env);
	

	if (psTermInfo->ppsSuggestedAttribute != NULL) {
		termInfo->suggestedAttributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(termInfo->suggestedAttributes, env);

		DBVdbv_to_snacc_attribute_list(termInfo->suggestedAttributes, psTermInfo->ppsSuggestedAttribute, env);
	}


	if (psTermInfo->ppsAlternativeTerm != NULL) {
		termInfo->alternativeTerm = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(termInfo->alternativeTerm, env);

		DBVdbv_to_snacc_attrtermlist(termInfo->alternativeTerm, psTermInfo->ppsAlternativeTerm, env);
	}


			
	if (psTermInfo->pulGlobalOccurrences != NULL) {
		termInfo->globalOccurrences = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(termInfo->globalOccurrences, env);

		*termInfo->globalOccurrences = *psTermInfo->pulGlobalOccurrences;
	}

	
	if (psTermInfo->ppsByAttributes != NULL) {
		termInfo->byAttributes = AsnListNew (sizeof (void *));
		CheckAsn1Alloc(termInfo->byAttributes, env);

		DBVdbv_to_snacc_occurbyatt_list(termInfo->byAttributes, psTermInfo->ppsByAttributes, env);
	}


	if (psTermInfo->ppsOtherInformation != NULL) {
		termInfo->otherTermInfo = AsnListNew (sizeof (void *));
		CheckAsn1Alloc(termInfo->otherTermInfo, env);

 		DBVdbv_to_snacc_otherinfo(termInfo->otherTermInfo, psTermInfo->ppsOtherInformation, env);
	}
}




int DBVdbv_to_snacc_attrtermlist(TermInfoSeqOf *alternativeTerm, DbvAttributesPlusTerm **ppsAlternativeTerm, jmp_buf env)
{
	AttributesPlusTerm *data_ptr;
        AttributesPlusTerm **data_ptr_address;
 
        int current_listnode = 0;
 
        while (ppsAlternativeTerm[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(alternativeTerm);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributesPlusTerm));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_attrterm(data_ptr, ppsAlternativeTerm[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_occurbyatt_list(OccurrenceByAttributes *byAttributes, DbvOccurrenceByAttributes **ppsByAttributes, jmp_buf env)
{
	OccurrenceByAttributesSeq *data_ptr;
        OccurrenceByAttributesSeq **data_ptr_address;
 
        int current_listnode = 0;
 
	while (ppsByAttributes[current_listnode] != NULL) {
        	data_ptr_address = AsnListAppend(byAttributes);
		CheckAsn1Alloc(data_ptr_address, env);

        	*data_ptr_address = Asn1Alloc(sizeof(OccurrenceByAttributesSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

        	data_ptr = *data_ptr_address;

        	DBVdbv_to_snacc_occurbyattseq(data_ptr, ppsByAttributes[current_listnode], env);
        	current_listnode++;
	}
}



int DBVdbv_to_snacc_occurbyattseq(OccurrenceByAttributesSeq *data_ptr, DbvOccurrenceByAttributes *psByAttributes, jmp_buf env)
{
	/* attribute list compulsory */
	data_ptr->attributes = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->attributes, env);

	DBVdbv_to_snacc_attribute_list(data_ptr->attributes, psByAttributes->ppsAttributeElement, env);

	if (psByAttributes->psOccurrence != NULL) {
		data_ptr->occurrences = (struct OccurrenceByAttributesSeqChoice*) Asn1Alloc (sizeof (struct OccurrenceByAttributesSeqChoice));
		CheckAsn1Alloc(data_ptr->occurrences, env);

		DVBVdbv_to_snacc_occurrence(data_ptr->occurrences, psByAttributes->psOccurrence, env);
	}


	if (psByAttributes->ppsOtherInformation != NULL) {
		data_ptr->otherOccurInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->otherOccurInfo, env);

		DBVdbv_to_snacc_otherinfo(data_ptr->otherOccurInfo, psByAttributes->ppsOtherInformation, env);
	}
		
}



int DVBVdbv_to_snacc_occurrence(struct OccurrenceByAttributesSeqChoice* occurrences, DbvOccurrence *psOccurrence, jmp_buf env)
{
	switch (psOccurrence->eOccurrenceType) {
	case OT_Global:
		occurrences->choiceId = OCCURRENCEBYATTRIBUTESSEQCHOICE_GLOBAL;
		occurrences->a.global = psOccurrence->uOccurence.ulGlobal;
		break;
	case OT_ByDatabase:
		occurrences->choiceId = OCCURRENCEBYATTRIBUTESSEQCHOICE_BYDATABASE;
		occurrences->a.byDatabase = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(occurrences->a.byDatabase, env);

		DBVdbv_to_snacc_bydatabase(occurrences->a.byDatabase, psOccurrence->uOccurence.ppsOccurrenceByDb, env);
		break;
	}
}



int DBVdbv_to_snacc_bydatabase(OccurrenceByAttributesSeqChoiceSeqOf *byDatabase, DbvOccurrenceByDb **ppsOccurrenceByDb, jmp_buf env)
{
	OccurrenceByAttributesSeqChoiceSeqOfSeq *data_ptr;
        OccurrenceByAttributesSeqChoiceSeqOfSeq **data_ptr_address;
 
        int current_listnode = 0;
 
        while (ppsOccurrenceByDb[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(byDatabase);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(OccurrenceByAttributesSeqChoiceSeqOfSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_bigoccurrthing(data_ptr, ppsOccurrenceByDb[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_bigoccurrthing(OccurrenceByAttributesSeqChoiceSeqOfSeq *data_ptr, DbvOccurrenceByDb *psOccurrenceByDb, jmp_buf env)
{
	/* Database name (a typedefed char* ) is compulsory */
	DBVdbv_to_snacc_inter_string(&data_ptr->db, &psOccurrenceByDb->sDatabaseName, env);
	
	if (psOccurrenceByDb->pulNumber != NULL) {
		data_ptr->num = (AsnInt *) Asn1Alloc (sizeof (AsnInt));
		CheckAsn1Alloc(data_ptr->num, env);

		*data_ptr->num = *psOccurrenceByDb->pulNumber;
	}


	if (psOccurrenceByDb->ppsOtherInformation != NULL) {
		data_ptr->otherDbInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->otherDbInfo, env);
		DBVdbv_to_snacc_otherinfo(data_ptr->otherDbInfo, psOccurrenceByDb->ppsOtherInformation, env);
	}
		
}



int DBVset_snacc_scanresp_attsetid(PDU *PDU_ptr, DbvObjectIdentifier pcAttributeSetId, jmp_buf env)
{
	/* Optional */
	if (pcAttributeSetId != NULL)
		ASN_MakeOid((char *) pcAttributeSetId, &(PDU_ptr->a.scanResponse->attributeSet), env);	
}



int DBVset_snacc_scanresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.scanResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.scanResponse->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.scanResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS ScanRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_scanreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_ScanInd;
        psTargetData->uTargetData.psScanReq = (DbvScanReq *) DbvAlloc ( sizeof (DbvScanReq));
	CheckDbvAlloc(psTargetData->uTargetData.psScanReq, env);
 
        DBVset_dbv_scanreq_refid(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_dbnames(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_attsetid(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_attplusterm(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_stepsize(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_numterms(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
        DBVset_dbv_scanreq_prefpos(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
	DBVset_dbv_scanreq_otherinfo(psTargetData->uTargetData.psScanReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_scanreq_refid(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.scanRequest->referenceId.octetLen != 0){
                scanreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(scanreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(scanreq->psReferenceId, &(PDU_ptr->a.scanRequest->referenceId), env);

        }

}



int DBVset_dbv_scanreq_dbnames(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
        DatabaseName *data_ptr;
        int num_elements = 0;
        int index = 0;

        num_elements = AsnListCount(PDU_ptr->a.scanRequest->databaseNames);
        scanreq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc (sizeof(DbvInternationalString *) * (num_elements + 1));
	CheckDbvAlloc(scanreq->ppsDatabaseNames, env);

        FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.scanRequest->databaseNames) {
                scanreq->ppsDatabaseNames[index] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(scanreq->ppsDatabaseNames[index], env);

                DBVsnacc_to_dbv_inter_string(scanreq->ppsDatabaseNames[index], data_ptr, env);
                index++;
        }
        scanreq->ppsDatabaseNames[index] = NULL;
}



int DBVset_dbv_scanreq_attsetid(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
        char *oid = NULL;
 
        if (PDU_ptr->a.scanRequest->attributeSet.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&(PDU_ptr->a.scanRequest->attributeSet));
                scanreq->pcAttributeSetId = (char *) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(scanreq->pcAttributeSetId, env);

                strcpy(scanreq->pcAttributeSetId, oid);
        }
}



int DBVset_dbv_scanreq_attplusterm(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	/* Compulsory */
	DBVsnacc_to_dbv_attrterm(&scanreq->sAttributesPlusTerm, PDU_ptr->a.scanRequest->termListAndStartPoint, env);
}



int DBVset_dbv_scanreq_stepsize(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.scanRequest->stepSize != NULL) {
                scanreq->pulStepSize = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(scanreq->pulStepSize, env);
                *scanreq->pulStepSize = *PDU_ptr->a.scanRequest->stepSize;
	}
		
}



int DBVset_dbv_scanreq_numterms(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	scanreq->ulNumberOfTermsRequested = PDU_ptr->a.scanRequest->numberOfTermsRequested;
}



int DBVset_dbv_scanreq_prefpos(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.scanRequest->preferredPositionInResponse != NULL) {
                scanreq->pulPreferredPositionInResponse = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(scanreq->pulPreferredPositionInResponse, env);

                *scanreq->pulPreferredPositionInResponse = *PDU_ptr->a.scanRequest->preferredPositionInResponse;
	}
}




int DBVset_dbv_scanreq_otherinfo(DbvScanReq *scanreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.scanRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.scanRequest->otherInfo);
		scanreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(scanreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(scanreq->ppsOtherInformation, PDU_ptr->a.scanRequest->otherInfo, env);
	}
}





