/*  Module Name: @(#)target_z3950_search.c	7.1
    Version: 11/15/96
    Author: John Hough / Chris Attewell
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a SearchResponse PDU 
		  to an origin and process a received SearchRequest PDU 
		  sent by an origin
		  

*/

#include "target_z3950_search.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE SearchResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvSearchResponse(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVSearchResponse",lAssocId);

	if (DBVCheckDbvSearchRspFields(lAssocId, psSearchRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_SEARCHRESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSrchRspTrace(lAssocId, psSearchRsp);

        	PDU_ptr = &pdu_to_encode;
	
		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.searchResponse = (struct SearchResponse *) Asn1Alloc(sizeof(struct SearchResponse));
                        CheckAsn1Alloc(PDU_ptr->a.searchResponse, env);
        		PDU_ptr->choiceId = PDU_SEARCHRESPONSE;
                        DBVsetup_snacc_srchresponse(PDU_ptr, psSearchRsp, env);
                } else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }
 
		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSearchResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvSearchRspFields(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError)
{

        if (DBVcheck_dbv_srchrsp_sstat(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchrsp_ressetst(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchrsp_presst(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchrsp_records(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchrsp_addinf(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchrsp_othinfo(lAssocId, psSearchRsp, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_dbv_srchrsp_sstat(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError) 
{
        if (DBVcheck_dbv_bool(psSearchRsp->fSearchStatus) == SI_NOTOK) 
		return(DBVdbv_bool_error(lAssocId, "psSearchRsp->fSearchStatus", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_srchrsp_ressetst(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError) 
{
	/* Can be null because optional */	
	if (psSearchRsp->peResultSetStatus == NULL)
		return SI_OK;

	if (DBVcheck_dbv_enum(*psSearchRsp->peResultSetStatus, RSS_Subset, RSS_None) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, "*psSearchRsp->peResultSetStatus", ppsSIError));
		
        return SI_OK;
}



int DBVcheck_dbv_srchrsp_presst(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError) 
{
	/* Can be null because optional */	
	if (psSearchRsp->pePresentStatus == NULL)
		return SI_OK;

	if (DBVcheck_dbv_enum(*psSearchRsp->pePresentStatus, PS_Success, PS_Failure) == SI_NOTOK) 
		return(DBVdbv_invalid_enum_error(lAssocId, "*psSearchRsp->pePresentStatus", ppsSIError));
		
        return SI_OK;
}




int DBVcheck_dbv_srchrsp_records(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError) 
{

	/* Can be null because optional */	
	if (psSearchRsp->psRecords == NULL)
		return SI_OK;
 
	return(DBVcheck_dbv_records(lAssocId, psSearchRsp->psRecords, "psSearchRsp->psRecords->", ppsSIError));

        return SI_OK;
}



int DBVcheck_dbv_srchrsp_addinf(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError)
{
        if (psSearchRsp->ppsAdditionalSearchInfo == NULL)
                return SI_OK;

	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchRsp->ppsAdditionalSearchInfo, "psSearchRsp->ppsAdditionalSearchInfo", ppsSIError));
}



int DBVcheck_dbv_srchrsp_othinfo(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp, DbvError **ppsSIError)
{
        if (psSearchRsp->ppsOtherInformation == NULL)
                return SI_OK;

	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchRsp->ppsOtherInformation, "psSearchRsp->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_srchresponse(PDU *PDU_ptr, DbvSearchRsp *psSearchRsp, jmp_buf env)
{
	DBVset_snacc_searchresp_refid(PDU_ptr, psSearchRsp->psReferenceId, env);
	DBVset_snacc_searchresp_rescount(PDU_ptr, psSearchRsp->ulResultCount, env);
	DBVset_snacc_searchresp_numrecs(PDU_ptr, psSearchRsp->ulNumberOfRecordsReturned, env);
	DBVset_snacc_searchresp_nextres(PDU_ptr, psSearchRsp->ulNextResultSetPosition, env);
	DBVset_snacc_searchresp_srchstat(PDU_ptr, psSearchRsp->fSearchStatus, env);
	DBVset_snacc_searchresp_resstat(PDU_ptr, psSearchRsp->peResultSetStatus, env);
	DBVset_snacc_searchresp_presstat(PDU_ptr, psSearchRsp->pePresentStatus, env);
	DBVset_snacc_searchresp_records(PDU_ptr, psSearchRsp->psRecords, env);
	DBVset_snacc_searchresp_addinfo(PDU_ptr, psSearchRsp->ppsAdditionalSearchInfo, env);
	DBVset_snacc_searchresp_otherinfo(PDU_ptr, psSearchRsp->ppsOtherInformation, env);
}



int DBVset_snacc_searchresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given (it is an optional field) */
	if  (psReferenceId != NULL) 
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.searchResponse->referenceId), psReferenceId, env);
}


int DBVset_snacc_searchresp_rescount(PDU *PDU_ptr, Ulong ulResultCount, jmp_buf env) 
{
	PDU_ptr->a.searchResponse->resultCount = ulResultCount;
}



int DBVset_snacc_searchresp_numrecs(PDU *PDU_ptr, Ulong ulNumberOfRecordsReturned, jmp_buf env)
{
	PDU_ptr->a.searchResponse->numberOfRecordsReturned = ulNumberOfRecordsReturned;
}



int DBVset_snacc_searchresp_nextres(PDU *PDU_ptr, Ulong ulNextResultSetPosition, jmp_buf env)
{
	PDU_ptr->a.searchResponse->nextResultSetPosition = ulNextResultSetPosition;
}



int DBVset_snacc_searchresp_srchstat(PDU *PDU_ptr, DbvBool fSearchStatus, jmp_buf env)
{
	if (fSearchStatus == DBV_TRUE)
		PDU_ptr->a.searchResponse->searchStatus = TRUE;
	else
		PDU_ptr->a.searchResponse->searchStatus = FALSE;
}



int DBVset_snacc_searchresp_resstat(PDU *PDU_ptr, DbvResultSetStatus *peResultSetStatus, jmp_buf env)
{
	if (peResultSetStatus != NULL) {
		PDU_ptr->a.searchResponse->resultSetStatus = (SearchResponseInt *)Asn1Alloc(sizeof(SearchResponseInt));
		CheckAsn1Alloc(PDU_ptr->a.searchResponse->resultSetStatus, env);

		switch (*peResultSetStatus) {
		case RSS_Subset:
			*PDU_ptr->a.searchResponse->resultSetStatus = SUBSET;
			break;
		case RSS_Interim:
			*PDU_ptr->a.searchResponse->resultSetStatus = INTERIM;
			break;
		case RSS_None:
			*PDU_ptr->a.searchResponse->resultSetStatus = NONE;
			break;
		}
	}
}


int DBVset_snacc_searchresp_presstat(PDU *PDU_ptr, DbvPresentStatus *pePresentStatus, jmp_buf env)
{

	/* presentStatus in snacc SearchResponse is PresentStatus* 
	   presentStatus in snacc PresentResponse is PresentStatus 
	*/

	if (pePresentStatus != NULL) {
		PDU_ptr->a.searchResponse->presentStatus = (PresentStatus *)Asn1Alloc(sizeof(PresentStatus));
		CheckAsn1Alloc(PDU_ptr->a.searchResponse->presentStatus, env);

		DBVdbv_to_snacc_presstat(PDU_ptr->a.searchResponse->presentStatus, *pePresentStatus);
	}
}



int DBVset_snacc_searchresp_records(PDU *PDU_ptr, DbvRecords *psRecords, jmp_buf env)
{
        if (psRecords != NULL) {
                PDU_ptr->a.searchResponse->records = (struct Records *) Asn1Alloc (sizeof (struct Records));
		CheckAsn1Alloc(PDU_ptr->a.searchResponse->records, env);

                DBVdbv_to_snacc_records(PDU_ptr->a.searchResponse->records, psRecords, env);
        }
}



int DBVset_snacc_searchresp_addinfo(PDU *PDU_ptr, DbvOtherInformation **ppsAdditionalSearchInfo, jmp_buf env)
{
	/* Check if psAdditionalSearchInfo exists (it is an optional field) */
	if (ppsAdditionalSearchInfo != NULL)  {
        	PDU_ptr->a.searchResponse->additionalSearchInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchResponse->additionalSearchInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchResponse->additionalSearchInfo, ppsAdditionalSearchInfo, env);
        }
}


int DBVset_snacc_searchresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.searchResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchResponse->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS SearchRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_searchreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_SearchInd;
        psTargetData->uTargetData.psSearchReq = (DbvSearchReq *) DbvAlloc ( sizeof (DbvSearchReq));
	CheckDbvAlloc(psTargetData->uTargetData.psSearchReq, env);
 
        DBVset_dbv_searchreq_refid(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_ssetupper(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_lsetlower(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_msetpres(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_replace(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_resultset(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_dbnames(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_ssetelname(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_msetelname(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_prefrecsyn(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
        DBVset_dbv_searchreq_query(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
	DBVset_dbv_searchreq_addinfo(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
	DBVset_dbv_searchreq_otherinfo(psTargetData->uTargetData.psSearchReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
}



/* referenceId - an optional field */
int DBVset_dbv_searchreq_refid(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.searchRequest->referenceId.octetLen != 0) {
                searchreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(searchreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(searchreq->psReferenceId, &(PDU_ptr->a.searchRequest->referenceId), env);
        }

}


int DBVset_dbv_searchreq_ssetupper(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	searchreq->ulSmallSetUpperBound = PDU_ptr->a.searchRequest->smallSetUpperBound;
}



int DBVset_dbv_searchreq_lsetlower(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	searchreq->ulLargeSetLowerBound = PDU_ptr->a.searchRequest->largeSetLowerBound;
}



int DBVset_dbv_searchreq_msetpres(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	searchreq->ulMediumSetPresentNumber = PDU_ptr->a.searchRequest->mediumSetPresentNumber;
}



int DBVset_dbv_searchreq_replace(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchRequest->replaceIndicator == TRUE)
		searchreq->fReplaceIndicator = DBV_TRUE;
	else
		searchreq->fReplaceIndicator = DBV_FALSE;
}



int DBVset_dbv_searchreq_resultset(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_inter_string(&(searchreq->sResultSetName), &(PDU_ptr->a.searchRequest->resultSetName), env);
}



int DBVset_dbv_searchreq_dbnames(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	DatabaseName *data_ptr;
	int num_elements = 0;
	int index = 0;

	num_elements = AsnListCount(PDU_ptr->a.searchRequest->databaseNames);
	searchreq->ppsDatabaseNames=(DbvInternationalString **)DbvAlloc(sizeof(DbvInternationalString *)*(num_elements+1));
	CheckDbvAlloc(searchreq->ppsDatabaseNames, env);

	FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.searchRequest->databaseNames) {
		searchreq->ppsDatabaseNames[index]=(DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		CheckDbvAlloc(searchreq->ppsDatabaseNames[index], env);

		DBVsnacc_to_dbv_inter_string(searchreq->ppsDatabaseNames[index], data_ptr, env);
		index++;
	}
	searchreq->ppsDatabaseNames[index]=NULL;
}



int DBVset_dbv_searchreq_ssetelname(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	if(PDU_ptr->a.searchRequest->smallSetElementSetNames != NULL) {
		searchreq->psSmallSetElementNames = (DbvElementSetNames *) DbvAlloc (sizeof(struct DbvElementSetNames));
		CheckDbvAlloc(searchreq->psSmallSetElementNames, env);

		DBVsnacc_to_dbv_elset_name(searchreq->psSmallSetElementNames, PDU_ptr->a.searchRequest->smallSetElementSetNames, env);
	}
}



int DBVset_dbv_searchreq_msetelname(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	if(PDU_ptr->a.searchRequest->mediumSetElementSetNames != NULL) {
		searchreq->psMediumSetElementNames = (DbvElementSetNames *) DbvAlloc (sizeof(struct DbvElementSetNames));
		CheckDbvAlloc(searchreq->psMediumSetElementNames, env);

		DBVsnacc_to_dbv_elset_name(searchreq->psMediumSetElementNames, PDU_ptr->a.searchRequest->mediumSetElementSetNames, env);
	}
}



int DBVset_dbv_searchreq_prefrecsyn(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	char *oid;

	if(PDU_ptr->a.searchRequest->preferredRecordSyntax.octetLen != 0) {
		oid = (char *)ASN_EncodedToString(&(PDU_ptr->a.searchRequest->preferredRecordSyntax));
		searchreq->pcPreferredRecordSyntax=(char *)DbvAlloc(strlen(oid)+1);
		CheckDbvAlloc(searchreq->pcPreferredRecordSyntax, env);

		strcpy(searchreq->pcPreferredRecordSyntax, oid);
	}
}



int DBVset_dbv_searchreq_query(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	searchreq->psQuery = (DbvQuery *) DbvAlloc (sizeof(DbvQuery));
	CheckDbvAlloc(searchreq->psQuery, env);


	switch(PDU_ptr->a.searchRequest->query->choiceId) {
	case QUERY_TYPE_0:
		searchreq->psQuery->eQueryType = QT_Private;
		searchreq->psQuery->uQuery.psPrivateQuery = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psPrivateQuery, env);

		DBVsnacc_to_dbv_octet_string(searchreq->psQuery->uQuery.psPrivateQuery, PDU_ptr->a.searchRequest->query->a.type_0, env);
		break;

	case QUERY_TYPE_1:
		searchreq->psQuery->eQueryType = QT_Rpn;
		searchreq->psQuery->uQuery.psRpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psRpnQuery, env);

		DBVsnacc_to_dbv_query(searchreq->psQuery->uQuery.psRpnQuery, PDU_ptr->a.searchRequest->query->a.type_1, env);
		break;

	case QUERY_TYPE_2:
		searchreq->psQuery->eQueryType = QT_Iso8777;
		searchreq->psQuery->uQuery.psIso8777Query = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psIso8777Query, env);

		DBVsnacc_to_dbv_octet_string(searchreq->psQuery->uQuery.psIso8777Query, PDU_ptr->a.searchRequest->query->a.type_2, env);
		break;

	case QUERY_TYPE_100:
		searchreq->psQuery->eQueryType = QT_Z3958;
		searchreq->psQuery->uQuery.psZ3958Query = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psZ3958Query, env);

		DBVsnacc_to_dbv_octet_string(searchreq->psQuery->uQuery.psZ3958Query, PDU_ptr->a.searchRequest->query->a.type_100, env);
		break;

	case QUERY_TYPE_101:
		searchreq->psQuery->eQueryType = QT_Erpn;
		searchreq->psQuery->uQuery.psErpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psErpnQuery, env);

		DBVsnacc_to_dbv_query(searchreq->psQuery->uQuery.psErpnQuery, PDU_ptr->a.searchRequest->query->a.type_101, env);
		break;

	case QUERY_TYPE_102:
		searchreq->psQuery->eQueryType = QT_NaturalLanguage;
		searchreq->psQuery->uQuery.psNaturalLanguageQuery = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(searchreq->psQuery->uQuery.psNaturalLanguageQuery, env);

		DBVsnacc_to_dbv_octet_string(searchreq->psQuery->uQuery.psNaturalLanguageQuery, PDU_ptr->a.searchRequest->query->a.type_102, env);
		break;

	}
}



int DBVsnacc_to_dbv_query(DbvRPNQuery *psRpnQuery, struct RPNQuery* type_1, jmp_buf env)
{
	/* Both compulsory */
	char *oid;

	oid = (char *) ASN_EncodedToString(&type_1->attributeSet);
	psRpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(oid)+ 1) ;
	CheckDbvAlloc(psRpnQuery->pcAttributeSetId, env);

	memcpy(psRpnQuery->pcAttributeSetId, oid, strlen(oid));

	DBVsnacc_to_dbv_rpn(&psRpnQuery->sRpnStructure, type_1->rpn, env);
}



int DBVsnacc_to_dbv_rpn(DbvRPNStructure *psRpnStructure, struct RPNStructure* rpn, jmp_buf env)
{
	switch (rpn->choiceId) {
	case RPNSTRUCTURE_OP:
		psRpnStructure->eRpnStructureType = RST_RpnOperand;
		psRpnStructure->uRPNStructure.psRpnOperand = (DbvOperand *) DbvAlloc (sizeof(DbvOperand));
		CheckDbvAlloc(psRpnStructure->uRPNStructure.psRpnOperand, env);

		DBVsnacc_to_dbv_operand(psRpnStructure->uRPNStructure.psRpnOperand, rpn->a.op, env);
		break;
	case RPNSTRUCTURE_RPNRPNOP:
		psRpnStructure->eRpnStructureType = RST_RpnLopRopOperator;
		psRpnStructure->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *) DbvAlloc (sizeof(DbvRPNLopRopOperator));
		CheckDbvAlloc(psRpnStructure->uRPNStructure.psRpnLopRopOperator, env);

		DBVsnacc_to_dbv_lopropop(psRpnStructure->uRPNStructure.psRpnLopRopOperator, rpn->a.rpnRpnOp, env);
		break;
	}	
}


int DBVsnacc_to_dbv_operand(DbvOperand *psRpnOperand, struct Operand* op, jmp_buf env)
{
	switch (op->choiceId) {
	case OPERAND_ATTRTERM:
		psRpnOperand->eOperandType = OT_AttributePlusTerm;
		psRpnOperand->uOperand.psAttributesPlusTerm = (DbvAttributesPlusTerm *) DbvAlloc (sizeof(DbvAttributesPlusTerm));
		CheckDbvAlloc(psRpnOperand->uOperand.psAttributesPlusTerm, env);

		DBVsnacc_to_dbv_attrterm(psRpnOperand->uOperand.psAttributesPlusTerm, op->a.attrTerm, env);
		break;
	case OPERAND_RESULTSET:
		psRpnOperand->eOperandType = OT_ResultSetId;
		psRpnOperand->uOperand.psResultSetId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psRpnOperand->uOperand.psResultSetId, env);

		DBVsnacc_to_dbv_inter_string(psRpnOperand->uOperand.psResultSetId, op->a.resultSet, env);
		break;
	case OPERAND_RESULTATTR:
		psRpnOperand->eOperandType = OT_ResultSetPlusAttributes;
		psRpnOperand->uOperand.psResultSetPlusAttributes = (DbvResultSetPlusAttributes *) DbvAlloc (sizeof(DbvResultSetPlusAttributes));
		CheckDbvAlloc(psRpnOperand->uOperand.psResultSetPlusAttributes, env);

		DBVsnacc_to_dbv_resulattr(psRpnOperand->uOperand.psResultSetPlusAttributes, op->a.resultAttr, env);
		break;
	}
}



int DBVsnacc_to_dbv_resulattr(DbvResultSetPlusAttributes *psResuLtSetPlusAttributes, struct ResultSetPlusAttributes* resultAttr, jmp_buf env)
{

	int num_elements = 0;

	/* Both compulsory */
	DBVsnacc_to_dbv_inter_string(&psResuLtSetPlusAttributes->sResultSetId, &resultAttr->resultSet, env);

	/* Get the number of records in the list */
	num_elements = AsnListCount(resultAttr->attributes);

	/* +1 to allocate space for the \0 at the end */
	psResuLtSetPlusAttributes->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * (num_elements +1 ) );
	CheckDbvAlloc(psResuLtSetPlusAttributes->ppsAttributeElement, env);

	DBVsnacc_to_dbv_attribute_list(psResuLtSetPlusAttributes->ppsAttributeElement, resultAttr->attributes, env);
}



int DBVsnacc_to_dbv_lopropop(DbvRPNLopRopOperator *psRpnLopRopOperator, struct RPNStructureSeq* rpnRpnOp, jmp_buf env)
{
	
	/* All compulsory */
	psRpnLopRopOperator->psLeftOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	CheckDbvAlloc(psRpnLopRopOperator->psLeftOperand, env);

	DBVsnacc_to_dbv_rpn(psRpnLopRopOperator->psLeftOperand, rpnRpnOp->rpn1, env);
 
	psRpnLopRopOperator->psRightOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	CheckDbvAlloc(psRpnLopRopOperator->psRightOperand, env);

	DBVsnacc_to_dbv_rpn(psRpnLopRopOperator->psRightOperand, rpnRpnOp->rpn2, env);
 
	DBVsnacc_to_dbv_operator(&psRpnLopRopOperator->sOperator, rpnRpnOp->op, env);
}



int DBVsnacc_to_dbv_operator(DbvOperator *psOperator, struct Operator* op, jmp_buf env)
{
	switch (op->choiceId) {
	case OPERATOR_AND:
		psOperator->eOperatorType = OT_And;
		break;
	case OPERATOR_OR:
		psOperator->eOperatorType = OT_Or;
		break;
	case OPERATOR_AND_NOT:
		psOperator->eOperatorType = OT_AndNot;
		break;
	case OPERATOR_PROX:
		psOperator->eOperatorType = OT_Proximity;
		psOperator->psProximityOperator = (DbvProximityOperator *) DbvAlloc (sizeof(DbvProximityOperator));
		CheckDbvAlloc(psOperator->psProximityOperator, env);

		DBVsnacc_to_dbv_prox(psOperator->psProximityOperator, op->a.prox, env);
		break;
	}
}



int DBVsnacc_to_dbv_prox(DbvProximityOperator *psProximityOperator, struct ProximityOperator* prox, jmp_buf env)
{
	if (prox->exclusion != NULL) {
		psProximityOperator->pfExclusion = (DbvBool *) DbvAlloc (sizeof(DbvBool));
		CheckDbvAlloc(psProximityOperator->pfExclusion, env);

		switch (*prox->exclusion) {
		case FALSE:	
			*psProximityOperator->pfExclusion = DBV_FALSE;
			break;
		case TRUE:	
			*psProximityOperator->pfExclusion = DBV_TRUE;
			break;
		}
	}

	psProximityOperator->ulDistance = prox->distance;

	switch (prox->ordered) {
	case FALSE:
		psProximityOperator->fOrdered = DBV_FALSE;
		break;
	case TRUE:
		psProximityOperator->fOrdered = DBV_TRUE;
		break;
	}

	switch (prox->relationType) {
	case LESSTHAN:
		psProximityOperator->eProximityRelationType =  PRT_LessThan;
		break;
	case LESSTHANOREQUAL:
		psProximityOperator->eProximityRelationType =  PRT_LessThanOrEqual;
		break;
	case EQUAL:
		psProximityOperator->eProximityRelationType =  PRT_Equal;
		break;
	case GREATERTHANOREQUAL:
		psProximityOperator->eProximityRelationType =  PRT_GreaterThanOrEqual;
		break;
	case GREATERTHAN:
		psProximityOperator->eProximityRelationType =  PRT_GreaterThan;
		break;
	case NOTEQUAL:
		psProximityOperator->eProximityRelationType =  PRT_NotEqual;
		break;
	default:
		psProximityOperator->eProximityRelationType =  PRT_Invalid;
		break;
	}

	DBVsnacc_to_dbv_proxunitcode(&psProximityOperator->sProximityUnitCode, prox->proximityUnitCode, env); 
}



int DBVsnacc_to_dbv_proxunitcode(DbvProximityUnitCode *psProximityUnitCode, struct ProximityOperatorChoice* proximityUnitCode, jmp_buf env) 
{
	switch (proximityUnitCode->choiceId) {
	case PROXIMITYOPERATORCHOICE_KNOWN:
		psProximityUnitCode->eProximityUnitType = PUT_Known;
	
		switch (proximityUnitCode->a.known) {
		case CHARACTER:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Character;
			break;
		case WORD:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Word;
			break;
		case SENTENCE:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Sentence;
			break;
		case PARAGRAPH:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Paragraph;
			break;
		case SECTION:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Section;
			break;
		case CHAPTER:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Chapter;
			break;
		case DOCUMENT:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Document;
			break;
		case ELEMENT:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Element;
			break;
		case SUBELEMENT:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_SubElement;
			break;
		case ELEMENTTYPE:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_ElementType;
			break;
		case BYTE:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Byte;
			break;
		default:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Invalid;
			break;

		}
		break;
	case PROXIMITYOPERATORCHOICE_PRIVATE:
		psProximityUnitCode->eProximityUnitType = PUT_Private;
		psProximityUnitCode->uProximityUnit.ulPrivate = proximityUnitCode->a.private;
		break;
	}
}



int DBVset_dbv_searchreq_addinfo(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchRequest->additionalSearchInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchRequest->additionalSearchInfo);
		searchreq->ppsAdditionalSearchInfo = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchreq->ppsAdditionalSearchInfo, env);

		DBVsnacc_to_dbv_otherinfo(searchreq->ppsAdditionalSearchInfo, PDU_ptr->a.searchRequest->additionalSearchInfo, env);
	}
}



int DBVset_dbv_searchreq_otherinfo(DbvSearchReq *searchreq, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchRequest->otherInfo);
		searchreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchreq->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(searchreq->ppsOtherInformation, PDU_ptr->a.searchRequest->otherInfo, env);
	}
}



