/*  Module Name: @(#)target_z3950_segment.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a 
		  SegmentRequest PDU to an origin
		  (a SegmentRequest PDU may be sent from a target 
		  to an origin as part of a response to a 
		  SearchRequest/PresentRequest query. 
	   	  There is no SegmentResponse PDU)
		  

*/

#include "target_z3950_segment.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE SegmentRequest PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvSegmentRequest(DbvAssocId lAssocId, DbvSegmentReq *psSegmentReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVSegmentRequest",lAssocId);

	if (DBVCheckDbvSegReqFields(lAssocId, psSegmentReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_SEGMENTREQ, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSegReqTrace(lAssocId, psSegmentReq);

        	PDU_ptr = &pdu_to_encode;
	
		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.segmentRequest = (struct Segment *) Asn1Alloc(sizeof(struct Segment));
			CheckAsn1Alloc(PDU_ptr->a.segmentRequest, env);
        		PDU_ptr->choiceId = PDU_SEGMENTREQUEST;
			DBVsetup_snacc_segrequest(PDU_ptr, psSegmentReq, env);
		} else {
                       DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSegmentRequest",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvSegReqFields(DbvAssocId lAssocId, DbvSegmentReq *psSegmentReq, DbvError **ppsSIError)
{
        if (DBVcheck_dbv_segreq_segrecs(lAssocId, psSegmentReq, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_segreq_otherinfo(lAssocId, psSegmentReq, ppsSIError))
                return SI_NOTOK;

	
        return SI_OK;
}
 


int DBVcheck_dbv_segreq_segrecs(DbvAssocId lAssocId, DbvSegmentReq *psSegmentReq, DbvError **ppsSIError)
{
	/* Cant be NULL because compulsory */
	if (psSegmentReq->ppsSegmentRecords == NULL)
                return(DBVdbv_field_missing_error(lAssocId, "psSegmentReq->ppsSegmentRecords", ppsSIError));

	

        return(DBVcheck_response_rec(lAssocId, psSegmentReq->ppsSegmentRecords, "psSegmentReq->ppsSegmentRecords", ppsSIError));

        return SI_OK;

}



int DBVcheck_dbv_segreq_otherinfo(DbvAssocId lAssocId, DbvSegmentReq *psSegmentReq, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psSegmentReq->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psSegmentReq->ppsOtherInformation, "psSegmentReq->ppsOtherInformation", ppsSIError));

}



int DBVsetup_snacc_segrequest(PDU *PDU_ptr, DbvSegmentReq *psSegmentReq, jmp_buf env)
{
	DBVset_snacc_segreq_refid(PDU_ptr, psSegmentReq->psReferenceId, env);
	DBVset_snacc_segreq_numrecs(PDU_ptr, psSegmentReq->ulNumberOfRecordsReturned, env);
	DBVset_snacc_segreq_segrecs(PDU_ptr, psSegmentReq->ppsSegmentRecords, env);
	DBVset_snacc_segreq_otherinfo(PDU_ptr, psSegmentReq->ppsOtherInformation, env);
}



int DBVset_snacc_segreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.segmentRequest->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_segreq_numrecs(PDU *PDU_ptr, Ulong ulNumberOfRecordsReturned, jmp_buf env)
{
	PDU_ptr->a.segmentRequest->numberOfRecordsReturned = ulNumberOfRecordsReturned;
}


int DBVset_snacc_segreq_segrecs(PDU *PDU_ptr, DbvNamePlusRecord **ppsSegmentRecords, jmp_buf env)
{
	/* Compulsory */
	PDU_ptr->a.segmentRequest->segmentRecords = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.segmentRequest->segmentRecords, env);

        DBVdbv_to_snacc_resprec(PDU_ptr->a.segmentRequest->segmentRecords, ppsSegmentRecords, env);
}



int DBVset_snacc_segreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.segmentRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.segmentRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.segmentRequest->otherInfo, ppsOtherInformation, env);
        }
}




