/*  Module Name: @(#)target_z3950_sort.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description:  Target end routines to create and send a SortResponse PDU
		  to an origin and process a received SortRequest PDU 
		  send by an origin
		  

*/

#include "target_z3950_sort.h"


/*****************************************************************
******************************************************************
******************************************************************
CREATE SortResponse PDU TO SEND TO ORIGIN AND SEND IT
*/

int DbvSortResponse(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)

{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
        DbvProtocolStack eProtocolStack;
        int return_value = SI_NOTOK;
	int val;
	jmp_buf env;


	return_value = SI_NOTOK;
        DBVEnterFuncTrace("DBVSortResponse",lAssocId);

	if (DBVCheckDbvSortRspFields(lAssocId, psSortRsp, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_TE_SORTRESP, ppsSIError) == SI_OK)
        		if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSortRspTrace(lAssocId, psSortRsp);

        	PDU_ptr = &pdu_to_encode;

		if ((val = setjmp(env)) == 0) {
        		PDU_ptr->a.sortResponse = (struct SortResponse *) Asn1Alloc(sizeof(struct SortResponse));
			CheckAsn1Alloc(PDU_ptr->a.sortResponse, env);
        		PDU_ptr->choiceId = PDU_SORTRESPONSE;
			DBVsetup_snacc_sortresponse(PDU_ptr, psSortRsp, env);
		} else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }
 

		/* Assume that the file descriptor of the socket is in 
		   lAssocId */
        	return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSortResponse",lAssocId, return_value);
        return return_value;
}



int DBVCheckDbvSortRspFields(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_sortrsp_status(lAssocId, psSortRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_sortrsp_ressetid(lAssocId, psSortRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_sortrsp_diag(lAssocId, psSortRsp, ppsSIError))
                return SI_NOTOK;
 
        if (DBVcheck_dbv_sortrsp_otherinfo(lAssocId, psSortRsp, ppsSIError))
                return SI_NOTOK;
 
        return SI_OK;
}
 


int DBVcheck_dbv_sortrsp_status(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_enum(psSortRsp->eSortStatus, SOS_Success, SOS_Failure) == SI_NOTOK)
		return(DBVdbv_invalid_enum_error(lAssocId, "psSortRsp->eSortStatus", ppsSIError));


	return SI_OK;

}



int DBVcheck_dbv_sortrsp_ressetid(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
	if (psSortRsp->peResultSetStatus == NULL)
		return SI_OK;

	if (DBVcheck_dbv_enum(*(psSortRsp->peResultSetStatus), SRSS_Empty, SRSS_None) == SI_NOTOK)
		return(DBVdbv_invalid_enum_error(lAssocId, "*psSortRsp->peResultSetStatus", ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_sortrsp_diag(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)
{
 
	/* Can be NULL because optional */
	if (psSortRsp->ppsDiagnostics == NULL)
		return SI_OK;

 
	if (DBVcheck_dbv_multdiag(lAssocId, psSortRsp->ppsDiagnostics, "psSortRsp->ppsDiagnostics", ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

        return SI_OK;
}



int DBVcheck_dbv_sortrsp_otherinfo(DbvAssocId lAssocId, DbvSortRsp *psSortRsp, DbvError **ppsSIError)
{
        /* Can be null because optional */
        if (psSortRsp->ppsOtherInformation == NULL)
                return SI_OK;
 
        return(DBVcheck_dbv_otherinfo(lAssocId, psSortRsp->ppsOtherInformation, "psSortRsp->ppsOtherInformation", ppsSIError));

}




int DBVsetup_snacc_sortresponse(PDU *PDU_ptr, DbvSortRsp *psSortRsp, jmp_buf env)
{
	DBVset_snacc_sortresp_refid(PDU_ptr, psSortRsp->psReferenceId, env);
	DBVset_snacc_sortresp_sortstat(PDU_ptr, psSortRsp->eSortStatus, env);
	DBVset_snacc_sortresp_resstat(PDU_ptr, psSortRsp->peResultSetStatus, env);
	DBVset_snacc_sortresp_diag(PDU_ptr, psSortRsp->ppsDiagnostics, env);
	DBVset_snacc_sortresp_otherinfo(PDU_ptr, psSortRsp->ppsOtherInformation, env);
}



int DBVset_snacc_sortresp_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{
	
	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.sortResponse->referenceId), psReferenceId, env);
	}
}



int DBVset_snacc_sortresp_sortstat(PDU *PDU_ptr, DbvSortStatus eSortStatus, jmp_buf env)
{
	switch (eSortStatus) {
	case SOS_Success:
		PDU_ptr->a.sortResponse->sortStatus = SUCCESS2;
		break;
	case SOS_Partial_1:
		PDU_ptr->a.sortResponse->sortStatus = PARTIAL_11;
		break;
	case SOS_Failure:
		PDU_ptr->a.sortResponse->sortStatus = FAILURE2;
		break;
	}
}



int DBVset_snacc_sortresp_resstat(PDU *PDU_ptr, DbvSortResultSetStatus *peResultSetStatus, jmp_buf env)
{
	if (peResultSetStatus != NULL) {

		PDU_ptr->a.sortResponse->resultSetStatus = (SortResponseInt1 *) Asn1Alloc (sizeof(SortResponseInt1));
		CheckAsn1Alloc(PDU_ptr->a.sortResponse->resultSetStatus, env);
	
		switch (*peResultSetStatus) {
		case SRSS_Empty:
			*PDU_ptr->a.sortResponse->resultSetStatus = EMPTY;
			break;
		case SRSS_Interim:
			*PDU_ptr->a.sortResponse->resultSetStatus = INTERIM1;
			break;
		case SRSS_Unchanged:
			*PDU_ptr->a.sortResponse->resultSetStatus = UNCHANGED;
			break;
		case SRSS_None:
			*PDU_ptr->a.sortResponse->resultSetStatus = NONE1;
			break;
		}
	}
}



int DBVset_snacc_sortresp_diag(PDU *PDU_ptr, DbvDiagRec **ppsDiagnostics, jmp_buf env)
{
	if (ppsDiagnostics != NULL) {
		PDU_ptr->a.sortResponse->diagnostics = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.sortResponse->diagnostics, env);

		DBVdbv_to_snacc_multdiag(PDU_ptr->a.sortResponse->diagnostics, ppsDiagnostics, env);
	}
}



int DBVset_snacc_sortresp_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
        	PDU_ptr->a.sortResponse->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.sortResponse->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.sortResponse->otherInfo, ppsOtherInformation, env);
        }
}



/*****************************************************************
******************************************************************
******************************************************************
PROCESS SortRequest PDU RECEIVED FROM ORIGIN 
*/

int DBVset_dbv_fields_sortreq(DbvTargetData *psTargetData, PDU *PDU_ptr, jmp_buf env)
{
 
        psTargetData->eTargetEvent = TA_SortInd;
        psTargetData->uTargetData.psSortReq = (DbvSortReq *) DbvAlloc ( sizeof (DbvSortReq));
	CheckDbvAlloc(psTargetData->uTargetData.psSortReq, env);
 
        DBVset_dbv_sortreq_refid(psTargetData->uTargetData.psSortReq, PDU_ptr, env);
        DBVset_dbv_sortreq_inresset(psTargetData->uTargetData.psSortReq, PDU_ptr, env);
        DBVset_dbv_sortreq_sortedres(psTargetData->uTargetData.psSortReq, PDU_ptr, env);
        DBVset_dbv_sortreq_sortseq(psTargetData->uTargetData.psSortReq, PDU_ptr, env);
	DBVset_dbv_sortreq_otherinfo(psTargetData->uTargetData.psSortReq, PDU_ptr, env);
 
        /* Following frees elements in PDU_ptr, not PDU_ptr itself */
        FreePDU(PDU_ptr);
 
}



/* referenceId - an optional field */
int DBVset_dbv_sortreq_refid(DbvSortReq *sortreq, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.sortRequest->referenceId.octetLen != 0){
                sortreq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(sortreq->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(sortreq->psReferenceId, &(PDU_ptr->a.sortRequest->referenceId), env);

        }

}


int DBVset_dbv_sortreq_inresset(DbvSortReq *sortreq, PDU *PDU_ptr, jmp_buf env)
{
        InternationalString *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        num_elements = AsnListCount(PDU_ptr->a.sortRequest->inputResultSetNames);
        sortreq->ppsInputResultSetName = (DbvInternationalString **) DbvAlloc (sizeof(DbvInternationalString *) * (num_elements +1));
	CheckDbvAlloc(sortreq->ppsInputResultSetName, env);

        FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.sortRequest->inputResultSetNames) {
                sortreq->ppsInputResultSetName[index] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
		CheckDbvAlloc(sortreq->ppsInputResultSetName[index], env);

                DBVsnacc_to_dbv_inter_string(sortreq->ppsInputResultSetName[index], data_ptr, env);
                index++;
        }
        sortreq->ppsInputResultSetName[index] = NULL;
}



int DBVset_dbv_sortreq_sortedres(DbvSortReq *sortreq, PDU *PDU_ptr, jmp_buf env)
{
	/* Compulsory */
	DBVsnacc_to_dbv_inter_string(&sortreq->sSortedResultSetName, &PDU_ptr->a.sortRequest->sortedResultSetName, env);
}



int DBVset_dbv_sortreq_sortseq(DbvSortReq *sortreq, PDU *PDU_ptr, jmp_buf env)
{
	/* Compulsory */
	
        SortKeySpec *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        num_elements = AsnListCount(PDU_ptr->a.sortRequest->sortSequence);
        sortreq->ppsSortSequence = (DbvSortKeySpec **) DbvAlloc (sizeof(DbvSortKeySpec *) * (num_elements +1));
	CheckDbvAlloc(sortreq->ppsSortSequence, env);

        FOR_EACH_LIST_ELMT(data_ptr, PDU_ptr->a.sortRequest->sortSequence) {
                sortreq->ppsSortSequence[index] = (DbvSortKeySpec *) DbvAlloc(sizeof(DbvSortKeySpec));
		CheckDbvAlloc(sortreq->ppsSortSequence[index], env);

                DBVsnacc_to_dbv_sortkeyspec(sortreq->ppsSortSequence[index], data_ptr, env);
                index++;
        }
        sortreq->ppsSortSequence[index] = NULL;
}



int DBVsnacc_to_dbv_sortkeyspec(DbvSortKeySpec *psSortSequence, SortKeySpec *data_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_sortelmnt(&psSortSequence->sSortElement, data_ptr->sortElement, env);
	DBVsnacc_to_dbv_sortrel(psSortSequence, data_ptr, env);
	DBVsnacc_to_dbv_sortcase(psSortSequence, data_ptr, env);

	if (data_ptr->missingValueAction != NULL) {
		psSortSequence->psMVActionType = (DbvSortMVAction*) DbvAlloc (sizeof (DbvSortMVAction));
		CheckDbvAlloc(psSortSequence->psMVActionType, env);

		DBVsnacc_to_dbv_sortmv(psSortSequence->psMVActionType, data_ptr->missingValueAction, env);
	}
}



int DBVsnacc_to_dbv_sortelmnt(DbvSortElement *psSortElement, struct SortElement* sortElement, jmp_buf env)
{

	int num_elements = 0;

	switch (sortElement->choiceId) {
	case SORTELEMENT_GENERIC:
		psSortElement->eSortElementType = SET_Generic;
		psSortElement->uSortElement.psGeneric = (DbvSortKey *) DbvAlloc (sizeof (DbvSortKey));
		CheckDbvAlloc(psSortElement->uSortElement.psGeneric, env);
		DBVsnacc_to_dbv_sortkey(psSortElement->uSortElement.psGeneric, sortElement->a.generic, env);
		break;
	case SORTELEMENT_DATBASESPECIFIC:
		psSortElement->eSortElementType = SET_DbSpecific;
		num_elements = AsnListCount(sortElement->a.datbaseSpecific);
        	psSortElement->uSortElement.ppsDatabaseSpecific = (DbvDbSpecificSortKey **) DbvAlloc (sizeof(DbvDbSpecificSortKey *) * (num_elements +1));
		CheckDbvAlloc(psSortElement->uSortElement.ppsDatabaseSpecific, env);
		
		DBVsnacc_to_dbv_dbspecific(psSortElement->uSortElement.ppsDatabaseSpecific, sortElement->a.datbaseSpecific, env);
		break;
	}
}



int DBVsnacc_to_dbv_sortkey(DbvSortKey *psGeneric, struct SortKey* generic, jmp_buf env)
{
	switch (generic->choiceId) {
	case SORTKEY_SORTFIELD:
		psGeneric->eSortKeyType = SKT_SortField;
		psGeneric->uSortElement.psSortField = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psGeneric->uSortElement.psSortField, env);

		DBVsnacc_to_dbv_inter_string(psGeneric->uSortElement.psSortField, generic->a.sortfield, env);
		break;
	case SORTKEY_ELEMENTSPEC:
		psGeneric->eSortKeyType = SKT_ElementSpec;
		psGeneric->uSortElement.psElementSpec = (DbvSpecification *) DbvAlloc (sizeof(DbvSpecification));
		CheckDbvAlloc(psGeneric->uSortElement.psElementSpec, env);

		DBVsnacc_to_dbv_specification(psGeneric->uSortElement.psElementSpec, generic->a.elementSpec, env);
		break;
	case SORTKEY_SORTATTRIBUTES:
		psGeneric->eSortKeyType = SKT_SortAttributes;
		psGeneric->uSortElement.psSortAttributes = (DbvSortAttributes *) DbvAlloc (sizeof(DbvSortAttributes));
		CheckDbvAlloc(psGeneric->uSortElement.psSortAttributes, env);

		DBVsnacc_to_dbv_soratts(psGeneric->uSortElement.psSortAttributes, generic->a.sortAttributes, env);
		break;
	}

}



int DBVsnacc_to_dbv_soratts(DbvSortAttributes *psSortAttributes, struct SortKeySeq* sortAttributes, jmp_buf env)
{
	/* Both compulosory */
	char *oid = NULL;
	int num_elements = 0;

	oid = (char *) ASN_EncodedToString(&sortAttributes->id);

	psSortAttributes->pcAttributeSetId = (char *) DbvAlloc (strlen(oid) + 1);
	CheckDbvAlloc(psSortAttributes->pcAttributeSetId, env);

	memcpy(psSortAttributes->pcAttributeSetId, oid, strlen(oid));

        num_elements = AsnListCount(sortAttributes->list);
        psSortAttributes->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * (num_elements +1));
	CheckDbvAlloc(psSortAttributes->ppsAttributeElement, env);

	DBVsnacc_to_dbv_attribute_list(psSortAttributes->ppsAttributeElement, sortAttributes->list, env);
}



int DBVsnacc_to_dbv_dbspecific(DbvDbSpecificSortKey **ppsDatabaseSpecific, SortElementSeqOf *datbaseSpecific, jmp_buf env)
{
        SortElementSeqOfSeq *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, datbaseSpecific) {
                ppsDatabaseSpecific[index] = (DbvDbSpecificSortKey *) DbvAlloc(sizeof(DbvDbSpecificSortKey));
		CheckDbvAlloc(ppsDatabaseSpecific[index], env);

                DBVsnacc_to_dbv_specsortkey(ppsDatabaseSpecific[index], data_ptr, env);
                index++;
        }
        ppsDatabaseSpecific[index] = NULL;
}



int DBVsnacc_to_dbv_specsortkey(DbvDbSpecificSortKey *psDatabaseSpecific, SortElementSeqOfSeq *data_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_inter_string(&psDatabaseSpecific->sDatabaseName, &data_ptr->databaseName, env);
	
	DBVsnacc_to_dbv_sortkey(&psDatabaseSpecific->sDbSort, data_ptr->dbSort, env);
	
}



int DBVsnacc_to_dbv_sortrel(DbvSortKeySpec *psSortSequence, SortKeySpec *data_ptr, jmp_buf env)
{
	switch (data_ptr->sortRelation) {
	case ASCENDING:
		psSortSequence->eSortRelationType = SRT_Ascending;
		break;
	case DESCENDING:
		psSortSequence->eSortRelationType = SRT_Descending;
		break;
	case ASCENDINGBYFREQUENCY:
		psSortSequence->eSortRelationType = SRT_FrequencyAsc;
		break;
	case DESCENDINGBYFREQUENCY:
		psSortSequence->eSortRelationType = SRT_FrequencyDesc;
		break;
	default:
		psSortSequence->eSortRelationType = SRT_Invalid;
		break;
	}
}



int DBVsnacc_to_dbv_sortcase(DbvSortKeySpec *psSortSequence, SortKeySpec *data_ptr, jmp_buf env)
{
	switch(data_ptr->caseSensitivity) {
	case CASESENSITIVE:
		psSortSequence->eSortCaseType = SCT_CaseSensitive;
		break;
	case CASEINSENSITIVE:
		psSortSequence->eSortCaseType = SCT_CaseInsensitive;
		break;
	default:
		psSortSequence->eSortCaseType = SCT_Invalid;
		break;
	}
	
}



int DBVsnacc_to_dbv_sortmv(DbvSortMVAction *psMVActionType, struct SortKeySpecChoice* missingValueAction, jmp_buf env)
{
	switch (missingValueAction->choiceId) {
	case SORTKEYSPECCHOICE_ABORT:
		psMVActionType->eSortMVActionType = SMT_Abort;
		break;
	case SORTKEYSPECCHOICE_NULL:
		psMVActionType->eSortMVActionType = SMT_Null;
		break;
	case SORTKEYSPECCHOICE_MISSINGVALUEDATA:
		psMVActionType->eSortMVActionType = SMT_Constant;
		psMVActionType->psOctetString = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(psMVActionType->psOctetString, env);

		DBVsnacc_to_dbv_octet_string(psMVActionType->psOctetString, missingValueAction->a.missingValueData, env);
		break;
	}
}




int DBVset_dbv_sortreq_otherinfo(DbvSortReq *sortreq, PDU *PDU_ptr, jmp_buf env)
{

	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.sortRequest->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.sortRequest->otherInfo);
		sortreq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(sortreq->ppsOtherInformation, env);
		DBVsnacc_to_dbv_otherinfo(sortreq->ppsOtherInformation, PDU_ptr->a.sortRequest->otherInfo, env);
	}
}




