/*  Module Name: @(#)tbl.h	1.2
    Version: 9/19/96
    Author:  M. Bush
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue: 

    QA Acceptance:  
    Date of Acceptance:  

    Description:  
		  Header file for Tabular file parser objects.
*/

#ifndef TBL_Include
#define TBL_Include

#include "sl_list.h" /* For SL reference below */


typedef enum tbl_Types {	TBL_Record,
				TBL_Field } TBL_Type;

typedef enum tbl_md {	TBL_Read,
			TBL_Write,
			TBL_Append } TBL_Mode;

typedef enum tbl_fieldType {	TBL_QText,
				TBL_UText,
				TBL_Int,
				TBL_Double,
				TBL_Comment,
				TBL_NullField} TBL_FieldType;

typedef struct tbl_Table* TBL;

typedef struct tbl_return *TBL_Return;
struct tbl_return {
    TBL_FieldType	Tag;
    union {
	long int     Int;
	char    *QText;
	char    *UText;
	char	*Text;
	double  Double;
	char	*Comment;
    }	Value;
};



#define FIELD_SEP           ','
#define RECORD_TERM         ';'
#define FILE_TERM           EOF
#define COMMENT_START       '/'
#define NULL_FIELD          '*'
#define STORE_BUFFER_SIZE   2048



struct tbl_Table {
	SL          FieldParseTree;
	FILE *   Stream;
};



typedef enum tbl_tokenType {
	tbl_FieldSep,
	tbl_RecTerm,
	tbl_FileTerm,
	tbl_QText,
	tbl_UText,
	tbl_Int,
	tbl_Double,
	tbl_Comment,
	tbl_NullField,
	tbl_TokenError } 
tbl_TokenType;



typedef struct tbl_Token *tbl_TokenPtr;
	struct tbl_Token {
	tbl_TokenType       Type;
	union {
		char    FieldSep;
		char    RecTerm;
		char    FileTerm;
		char *  QText;
		char *  UText;
		char *  Comment;
		long int        AnInt;
		double  ADouble;
        } Value;
};



TBL	TBL_Create(char* Filename, TBL_Mode Mode);
void    TBL_Destruct(TBL Table);

Bool_   TBL_GetField(TBL Table, TBL_Return Return);
Bool_   TBL_PutRecord(TBL Table);
Bool_   TBL_PutField(TBL Table, TBL_Return Return);


/* Cant have local (static) declarations I'm afraid because functions
   need to be called from other parts of DBV-OSI II */

Global_ tbl_TokenPtr    tbl_Token(TBL Table);
Global_ Bool_           tbl_ParseFieldIntoTree(TBL Table); 
Global_ tbl_TokenType   tbl_NumberType(double Num);
Global_ char*           tbl_GetUText(FILE*Stream);
Global_ char*           tbl_GetQText(FILE*Stream);
Global_ void            tbl_GetComment(FILE*Stream);
Global_ void            tbl_GetNumber(FILE*Stream, tbl_TokenPtr AToken);
Global_ void            tbl_ClearBuffer(TBL Table);
Global_ void            tbl_ThrowToken(tbl_TokenPtr AToken);
Global_ void            tbl_WriteField(FILE *Stream, tbl_TokenPtr AToken);


#endif
