/*  Module Name: @(#)trace_assocutil_params.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Prints the input parameters of the association and utility
	functions to the trace file.
 
        Used by the origin and target.
 
*/

#include "trace_assocutil_params.h"

void Trace_writeDbvAssocId(DbvAssocId lAssocId)
{ 
	sprintf(DBVTraceBuffer, "\tDbvAssocId: %d\n", lAssocId);
	DBVTrace(lAssocId, Trace_High, DBVTraceBuffer);
}

void Trace_writeDbvSecs(DbvSecs lSeconds, DbvAssocId lAssocId)
{
	if(lSeconds == CALL_BLOCKING)
		sprintf(DBVTraceBuffer, "\tDbvSecs: CALL_BLOCKING\n");
	else if(lSeconds == CALL_ASYNCHRONOUS)
		sprintf(DBVTraceBuffer, "\tDbvSecs: CALL_ASYNCHRONOUS\n");
	else
		sprintf(DBVTraceBuffer, "\tDbvSecs: %d\n", lSeconds);

	DBVTrace(lAssocId, Trace_High, DBVTraceBuffer);
}

void Trace_writeDbvAssociateState(DbvAssociateState *peAssociateState, 
	DbvAssocId lAssocId)
{
	if(peAssociateState == NULL)
	{
		DBVTrace(lAssocId, Trace_High, "\tDbvAssociateState: NULL\n");
		return;
	}

	switch(*peAssociateState)
	{
	case Associate_Null:
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateState: Associate_Null\n");
		break;
	case Associate_Accepted:
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateState: Associate_Accepted\n");
		break;
	case Associate_Rejected:
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateState: Associate_Rejected\n");
		break;
	case Associate_Pending:
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateState: Associate_Pending\n");
		break;
	default:
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateState: Unknown/Invalid\n");
		break;
	}

}

void Trace_writeDbvAssociateResult(DbvAssociateResult *puAssociateResult, 
	DbvAssociateState eAssociateState, DbvAssocId lAssocId)
{
	char *stringfield=NULL;
	int intfield=0;

        if(eAssociateState == Associate_Rejected)
        {
		DBVTrace(lAssocId, Trace_High, "\tDbvAssociateResult\n");

                Trace_writeDbvAssociateRejectSource(puAssociateResult->
                        psAssociateReject->peAssociateRejectSource,
                        lAssocId);
                Trace_writeDbvAssociateRejectResult(puAssociateResult->
                        psAssociateReject->peAssociateRejectResult,
                        lAssocId);
                Trace_writeDbvAssociateRejectReason(puAssociateResult->
                        psAssociateReject->peAssociateRejectReason,
                        lAssocId);
        }
        if(eAssociateState == Associate_Pending)
		DBVTrace(lAssocId, Trace_High, "\tAssociateResult: NULL\n");

        if(eAssociateState == Associate_Accepted)
        {
                DBVTrace(lAssocId, Trace_High, "\nAssociateResult.psProtocolAndSyntaxes->peApplicationProtocol\n   AssociateResult.psProtocolAndSyntaxes->peAbstractSyntax\n   AssociateResult.psProtocolAndSyntaxes->ppcCommonSyntaxes\n");
		
		if( puAssociateResult->psProtocolAndSyntaxes->
			peApplicationProtocol == NULL)
		{
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvApplicationProtocol: NULL\n");
		}
		else
		{
			switch(*puAssociateResult->psProtocolAndSyntaxes->
                        	peApplicationProtocol)
			{
			case Unknown_Protocol:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvApplicationProtocol: Unknown_Protocol\n");
				break;
			case Unidentified_Protocol:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvApplicationProtocol: Unidentified_Protocol\n");
				break;
			case SR_Protocol:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvApplicationProtocol: Z3950_Protocol\n");
				break;
			case Z3950_Protocol:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvApplicationProtocol: Z3950_Protocol\n");
				break;
			default:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvApplicationProtocol: Unknown/Invalid\n");
				break;
			}
		}

		if( puAssociateResult->psProtocolAndSyntaxes->
			peAbstractSyntax == NULL)
		{
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAbstractSyntax: NULL\n");
		}
		else
		{
			switch(*puAssociateResult->psProtocolAndSyntaxes->
                        	peAbstractSyntax)
			{
			case No_Syntaxes_Provided:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvAbstractSyntax: No_Syntaxes_Provided\n");
				break;
			case No_Common_Syntax:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvAbstractSyntax: No_Common_Syntax\n");
				break;
			case Common_Syntaxes:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvAbstractSyntax: Common_Syntaxes\n");
				break;
			default:
				DBVTrace(lAssocId, Trace_High, 
					"\tDbvAbstractSyntax: Unknown/Invalid\n");
				break;
			}
		}
                if(puAssociateResult->psProtocolAndSyntaxes->
                    ppcCommonSyntaxes == NULL)
			DBVTrace(lAssocId, Trace_High, "NULL\n");
                else
                {
                while(puAssociateResult->psProtocolAndSyntaxes->
                        ppcCommonSyntaxes[intfield] != NULL)
                {
                        sprintf(DBVTraceBuffer,"\t%s\n",(puAssociateResult->
                                psProtocolAndSyntaxes->
                                ppcCommonSyntaxes[intfield]));
			DBVTrace(lAssocId, Trace_High, DBVTraceBuffer);
                        intfield++;
                }
                }

        }
}

void Trace_writeDbvAssociateRsp(DbvAssociateRsp eAssociateRsp, 
	DbvAssocId lAssocId)
{
	char *stringfield = NULL;

	switch( eAssociateRsp )
	{
		case Accepted:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRsp: Accepted\n");
			break;
		case Rejected_Perm:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRsp: Rejected_Perm\n");
			break;
		case Rejected_Tran:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRsp: Rejected_Tran\n");
			break;
		default:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRsp: Unknown/Invalid\n");
			break;
	}
}

void Trace_writeDbvReleaseRsp(DbvReleaseRsp *psReleaseRsp, DbvAssocId lAssocId)
{
	char *stringfield = NULL;

	if(psReleaseRsp == NULL)
	{
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvReleaseRsp: NULL\n");
		return;
	}

	if(psReleaseRsp->peReleaseRspReason == NULL)
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvReleaseRsp->peReleaseRspReason: NULL\n");
	else
	{
		switch(*psReleaseRsp->peReleaseRspReason)
		{
		case Normal:
			DBVTrace(lAssocId, Trace_High, "psReleaseRsp->peReleaseRspReason: Normal\n");
			break;
		case Not_Finished:
			DBVTrace(lAssocId, Trace_High, "psReleaseRsp->peReleaseRspReason: Not_Finished\n");
			break;
		default:
			DBVTrace(lAssocId, Trace_High, "psReleaseRsp->peReleaseRspReason: Unknown/Invalid\n");
			break;
		}
	}

	switch(psReleaseRsp->eReleaseRspResult)
	{
	case Affirmative:
		DBVTrace(lAssocId, Trace_High, "psReleaseRsp->eReleaseRspResult: Affirmative\n");
		break;
	case Negative:
		DBVTrace(lAssocId, Trace_High, "psReleaseRsp->eReleaseRspResult: Negative\n");
		break;
	default:
		DBVTrace(lAssocId, Trace_High, "psReleaseRsp->eReleaseRspResult: Unknown/Invalid\n");
		break;
	}
}

void Trace_writeDbvAssociateRejectSource(DbvAssociateRejectSource
        *peAssociateRejectSource, DbvAssocId lAssocId)
{
        if(peAssociateRejectSource == NULL)
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateRejectSource: NULL\n");
        else
        {
                switch(*peAssociateRejectSource)
                {
                case ACSE_User:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectSource: ACSE_User\n");
                        break;
                case ACSE_Provider:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectSource: ACSE_Provider\n");
                        break;
                default:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectSource: Unknown/Invalid\n");
                        break;
 
                }
        }
}

void Trace_writeDbvAssociateRejectResult(DbvAssociateRejectResult
        *peAssociateRejectResult, DbvAssocId lAssocId)
{
        if(peAssociateRejectResult == NULL)
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateRejectResult: NULL\n");
        else
        {
                switch(*peAssociateRejectResult)
                {
                case Rejected_Permanent:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectResult: Rejected_Permanent\n");
                        break;
                case Rejected_Transient:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectResult: Rejected_Transient\n");
                        break;
                default:
			DBVTrace(lAssocId, Trace_High, 
				"\tDbvAssociateRejectResult: Unknown/Invalid\n");
                        break;
                }
        }
 
}

void Trace_writeDbvAssociateRejectReason(DbvAssociateRejectReason
        *peAssociateRejectReason, DbvAssocId lAssocId)
{
 
        if(peAssociateRejectReason == NULL)
		DBVTrace(lAssocId, Trace_High, 
			"\tDbvAssociateRejectReason: NULL\n");
        else
        {
                switch(*peAssociateRejectReason)
                {
                case No_Reason_Given:
			DBVTrace(lAssocId, Trace_High,
                        	"\tDbvAssociateRejectReason: No_Reason_Given\n");
                        break;
                case No_Common_ACSE_Version:
			DBVTrace(lAssocId, Trace_High,
                        	"\tDbvAssociateRejectReason: No_Common_ACSE_Version\n");
                        break;
                case Application_Context_Not_Supported:
			DBVTrace(lAssocId, Trace_High,
                        	"\tDbvAssociateRejectReason: Application_Context_Not_Supported\n");
                        break;
                default:
			DBVTrace(lAssocId, Trace_High,
                        	"\tDbvAssociateRejectReason: Unknown/Invalid\n");
                        break;
                }
        }
}

void Trace_writeDbvReleaseState(DbvReleaseState *peReleaseState, 
	DbvAssocId lAssocId)
{
	if(peReleaseState == NULL)
	{
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: NULL\n");
		return;
	}
	switch(*peReleaseState)
	{
	case Release_Null:
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: Release_Null\n");
		break;
	case Release_Accepted:
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: Release_Accepted\n");
		break;
	case Release_Rejected:
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: Release_Rejected\n");
		break;
	case Release_Pending:
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: Release_Pending\n");
		break;
	default:
		DBVTrace(lAssocId, Trace_High, "\tDbvReleaseState: Unknown/Invalid\n");
		break;

	}
}

void Trace_DbvProtocolStack(DbvProtocolStack *peProtocolStack, 
	DbvAssocId lAssocId)
{
	if(peProtocolStack == NULL)
	{
		DBVTrace(lAssocId, Trace_High, "\tDbvProtocolStack: NULL\n");
		return;
	}
	switch(*peProtocolStack)
	{
	case Unknown_Stack:
		DBVTrace(lAssocId, Trace_High, "\tDbvProtocolStack: Unknown_Stack\n");
		break;
	case OSI_Stack:
		DBVTrace(lAssocId, Trace_High, "\tDbvProtocolStack: OSI_Stack\n");
		break;
	case TCP_Stack:
		DBVTrace(lAssocId, Trace_High, "\tDbvProtocolStack: TCP_Stack\n");
		break;
	default:
		DBVTrace(lAssocId, Trace_High, "\tDbvProtocolStack: Unknown/Invalid\n");
		break;
	}
}

