/*  Module Name: @(#)z3950_enc_dec.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: DBV-OSI II PDU<->Byte String Encoding And Decoding Routines

*/


#include "z3950_enc_dec.h"


int DBVencode_pdu_to_bytes(DbvAssocId lAssocId, PDU *pdu_ptr, char **encoded_pdu, long *encoded_pdu_length, DbvError **ppsSIError)
{

        AsnLen encodedlen;
        static SBuf encodebuffer;
	int return_value = SI_OK;

	/* This array is used by the encoding - various pointers in the SBuf 
	   struture point at it  */
	static char *work_array = NULL;

	/* Size (bytes) of each realloc increment */
	int work_array_increment = WORK_ARRAY_INCREMENT;

	/* Work array initial size (bytes) */
	int work_array_size = WORK_ARRAY_SIZE;
	int out_of_memory = 0;
	int done_encoding = 0;

	static int done_hash_table_setup = 0;
 
	/* Initialise the hash tables used for encoding/decoding
	   AnyDefinedBy externals - this only needs to be done
	   once at the start of the program */
	if (!done_hash_table_setup) {
		DBVinit_external_data();
		done_hash_table_setup = 1;
	}


	encodedlen = 0;

	/* In some Z39.50 implementations (e.g. Cansearch), 
	   SBufInstallData is used here rather than SBufInit, which is ok
	   because SBufInstallData doesnt actually 'install' data, it just
	   calls SBufInit then changes a few pointers in the Sbuf structure 
	   (see sbuf.h). SBufInit is all that is needed to initialize	
	   prior to an encoding */  


	/* Free the work array (to reinitialise after any previous 
	   encoding) */ 
	if (work_array != NULL)
		free(work_array);

	work_array = (char *) malloc (work_array_size);

	/* Implement an expanding SBuf to cater for any length of
	   encoding (seems faster than using ExpBufs) */

	/* while not enough space to encode and not done the encoding */
	while (!out_of_memory && !done_encoding) {
		work_array = (char *) realloc (work_array, work_array_size);
		if (work_array == NULL) {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,
 				DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, 
				Association_Open, ppsSIError);
			out_of_memory = 1;
			return_value = SI_NOTOK;
			break;
		}

		SBufInit(&encodebuffer, work_array, work_array_size);	
        	SBufResetInWriteRvsMode(&encodebuffer);
        	encodedlen = BEncPDU(&encodebuffer, pdu_ptr);
 
        	if ((encodedlen <= 0) || SBufWriteError(&encodebuffer)) {
			DBVTrace(lAssocId, Trace_Low, 
				"Encode Buffer too small - Reallocing...\n");
			work_array_size += work_array_increment;	
		} else
			done_encoding = 1;
        }

	/* FreePDU recursively frees all Asn1Alloced elements in the structure
	   to which pdu_ptr points but not the structure itself (assumes
	   Asn1Alloc(size) is calloc(1, size) and not the snacc 1.1 
	   nibble memory form) */
	if (pdu_ptr != NULL)
        	FreePDU(pdu_ptr);


	/* Because encodebuffer and work_array are static 
	   the following assignment is fine */
	if (encodedlen > 0) {
		*encoded_pdu = encodebuffer.dataStart; 
		*encoded_pdu_length = encodedlen;
	}
  
        return return_value;
}



int DBVdecode_bytes_to_pdu(DbvAssocId lAssocId, char *encoded_pdu, int encoded_pdu_length, PDU *pdu_ptr, DbvError **ppsSIError) 
{

	AsnLen decodedlen;
	SBuf decodebuffer; 
	int val;
	jmp_buf env;
	int return_value = SI_OK;
	static int done_hash_table_setup = 0;
 
 
	/* Initialise the hash tables used for encoding/decoding
	   AnyDefinedBy externals - this only needs to be done
	   once at the start of the program */
	if (!done_hash_table_setup) {
		DBVinit_external_data();
		done_hash_table_setup = 1;
	}
	


	decodedlen = 0;		

	/* Put the data 'encoded_pdu' of size 'encoded_pdu_length'
	   into SBuf decodebuffer and set the SBuf up for reading
	   'encoded_pdu' from the beginning */
	SBufInstallData(&decodebuffer, encoded_pdu, encoded_pdu_length);
 
	/* Do a setjump - snacc 1.1 generated decoding routines do a
	   longjmp if decoding fails */
	if ((val = setjmp(env)) == 0) {
		BDecPDU(&decodebuffer, pdu_ptr, &decodedlen, env);
	} else {

		if (val == -262)

			/* Unknown PDU received !! */
			;
		else {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_DECODEFAILED, DBV_PDU_DECODEFAILED_S, 0, Association_Open, ppsSIError);
			return_value = SI_NOTOK;
		}
        }


	return return_value;
}

