/*  Module Name: @(#)z3950_otherinfo.c	7.1
    Version: 11/15/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Otherinfo set up/get code

*/

#include "z3950_otherinfo.h"


/* OtherInformation fields  dbv -> snacc structure setting before encoding */


/* dbv -> snacc */
int DBVdbv_to_snacc_otherinfo(AsnList *snacc_otherinfo, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	char *oid = NULL;
	char *string = NULL;
	long length = 0;
	int index = 0;

	struct OtherInformationSeq       *data_ptr = NULL;
	struct OtherInformationSeq      **data_ptr_address = NULL;


	while (ppsOtherInformation[index] != NULL) {

		/* Create a list node, append it to the list and return
           	   the address of the void *data pointer in the node created */

		/* The return pointer values in the following are not cast
           	   as they are in c_examples/any/genber.c, they are
           	   simply returned as they are in c_examples/simple/genber.c */
		data_ptr_address = AsnListAppend(snacc_otherinfo);
		CheckAsn1Alloc(data_ptr_address, env);

		*data_ptr_address = Asn1Alloc(sizeof(struct OtherInformationSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

		/* Point to the node data just created [ 'data_ptr->' is 
	   	   easier to read than '(*data_ptr_address)->' ] */
		data_ptr = *data_ptr_address;

		DBVdbv_to_snacc_otherinfo_node(data_ptr, ppsOtherInformation[index], env);
		index++;
	}
}



/***************************/
/* Now add data to the list node created */

int DBVdbv_to_snacc_otherinfo_node(struct OtherInformationSeq *data_ptr, DbvOtherInformation *psOtherInformation, jmp_buf env)
{
	/* If psInfoCategory exists (it is an optional field) */
	if (psOtherInformation->psInfoCategory != NULL) {
		data_ptr->category = (struct InfoCategory *) Asn1Alloc(sizeof(struct InfoCategory));
		CheckAsn1Alloc(data_ptr->category, env);

		/* If pcCategoryTypeId exists (it is an optional field) */
		if (psOtherInformation->psInfoCategory->pcCategoryTypeId != NULL) {
			ASN_MakeOid((char *) psOtherInformation->psInfoCategory->pcCategoryTypeId, &data_ptr->category->categoryTypeId, env);
		}

		/* Add ulCategoryValue */
		data_ptr->category->categoryValue = psOtherInformation->psInfoCategory->ulCategoryValue;
	}


        data_ptr->information = (OtherInformationSeqChoice *) Asn1Alloc(sizeof (struct OtherInformationSeqChoice));
	CheckAsn1Alloc(data_ptr->information, env);

	switch (psOtherInformation->eInformationType) {
	case IT_CharacterInfo:
		DBVdbv_to_snacc_otherinfo_char(data_ptr, psOtherInformation->uInformation.psCharacterInfo, env);
		break;
	case IT_BinaryInfo:
		DBVdbv_to_snacc_otherinfo_bin(data_ptr, psOtherInformation->uInformation.psBinaryInfo, env);
		break;
	case IT_ExternalInfo:
		DBVdbv_to_snacc_otherinfo_ext(data_ptr, psOtherInformation->uInformation.psExternallyDefinedInfo, env);
		break;
	case IT_ObjectIdentifier:
		DBVdbv_to_snacc_otherinfo_oid(data_ptr, psOtherInformation->uInformation.pcObjectIdentifier, env);
		break;
	}

}



int DBVdbv_to_snacc_otherinfo_char(struct OtherInformationSeq *data_ptr, DbvInternationalString *psCharacterInfo, jmp_buf env)
{

        data_ptr->information->choiceId = OTHERINFORMATIONSEQCHOICE_CHARACTERINFO;
        data_ptr->information->a.characterInfo = (InternationalString *) Asn1Alloc(sizeof (InternationalString));
	CheckAsn1Alloc(data_ptr->information->a.characterInfo, env);
	DBVdbv_to_snacc_inter_string(data_ptr->information->a.characterInfo, psCharacterInfo, env);
}
 


int DBVdbv_to_snacc_otherinfo_bin(struct OtherInformationSeq *data_ptr, DbvOctetString *psBinaryInfo, jmp_buf env)
{
        data_ptr->information->choiceId = OTHERINFORMATIONSEQCHOICE_BINARYINFO;

        data_ptr->information->a.binaryInfo = (AsnOcts *) Asn1Alloc(sizeof (AsnOcts));
	CheckAsn1Alloc(data_ptr->information->a.binaryInfo, env);
	DBVdbv_to_snacc_octet_string(data_ptr->information->a.binaryInfo, psBinaryInfo, env);
}
 


int DBVdbv_to_snacc_otherinfo_ext(struct OtherInformationSeq *data_ptr, DbvASN1External *psExternallyDefinedInfo, jmp_buf env)
{

	char *oid = NULL;

        data_ptr->information->choiceId = OTHERINFORMATIONSEQCHOICE_EXTERNALLYDEFINEDINFO;
 
        /* Set up the external */
        data_ptr->information->a.externallyDefinedInfo = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
	CheckAsn1Alloc(data_ptr->information->a.externallyDefinedInfo, env);        
	DBVdbv_to_snacc_external(data_ptr->information->a.externallyDefinedInfo, psExternallyDefinedInfo, env);
}



int DBVdbv_to_snacc_otherinfo_oid(struct OtherInformationSeq *data_ptr, DbvObjectIdentifier pcObjectIdentifier, jmp_buf env)
{

	char *oid = NULL;

        data_ptr->information->choiceId = OTHERINFORMATIONSEQCHOICE_OID_SNACC;
        data_ptr->information->a.oid = (AsnOid *) Asn1Alloc(sizeof (AsnOid));
	CheckAsn1Alloc(data_ptr->information->a.oid, env);

	oid = pcObjectIdentifier;


        /* NOTE - no & before data_ptr below */
        ASN_MakeOid((char *) oid, data_ptr->information->a.oid, env);
}


/*********************************************************************/
/* OtherInformation fields  snacc -> dbv structure setting after decoding */

/* snacc -> dbv */
int DBVsnacc_to_dbv_otherinfo(DbvOtherInformation **ppsOtherInformation, AsnList *snacc_otherinfo, jmp_buf env)
{
	int index = 0;

	/* OtherInformation structure */
        OtherInformationSeq *data_ptr = NULL;


	/* In DBV there will only ever be one node in the
	   otherInformation list */
	FOR_EACH_LIST_ELMT(data_ptr, snacc_otherinfo) {

		ppsOtherInformation[index] = (DbvOtherInformation *) DbvAlloc (sizeof(DbvOtherInformation));
		
		DBVsnacc_to_dbv_otherinfo_node(ppsOtherInformation[index], data_ptr, env);
		index++;
	}
}



int DBVsnacc_to_dbv_otherinfo_node(DbvOtherInformation *psOtherInformation, OtherInformationSeq *data_ptr, jmp_buf env)
{
	char direct_reference[SA_ARRAY_SIZE];
	long length = 0;
	char *sutrs_str = NULL;

	memset(direct_reference, 0, sizeof(direct_reference));

	/* If category exists (it is optional) */
	if (data_ptr->category != NULL) {
		psOtherInformation->psInfoCategory = (struct DbvInfoCategory *) DbvAlloc (sizeof (struct DbvInfoCategory));
		CheckDbvAlloc(psOtherInformation->psInfoCategory, env);

		/* If category type exists (it is optional) */
		if (data_ptr->category->categoryTypeId.octs != NULL) {
			DBVsa_strcpy(direct_reference, (char *) ASN_EncodedToString(&data_ptr->category->categoryTypeId));
					
			/* Alloc +1 (to alloc a \0 at the end) */
			psOtherInformation->psInfoCategory->pcCategoryTypeId = (char *) DbvAlloc (strlen(direct_reference) + 1);
			CheckDbvAlloc(psOtherInformation->psInfoCategory->pcCategoryTypeId, env);

			memcpy(psOtherInformation->psInfoCategory->pcCategoryTypeId, direct_reference, strlen(direct_reference));
		}

		/* Set ulCategoryValue */
		psOtherInformation->psInfoCategory->ulCategoryValue = data_ptr->category->categoryValue;
	}

	/* Now get the other info data */
	switch(data_ptr->information->choiceId) {
	case OTHERINFORMATIONSEQCHOICE_CHARACTERINFO:
               	DBVsnacc_to_dbv_otherinfo_char(psOtherInformation, data_ptr, env);
		break;
	case OTHERINFORMATIONSEQCHOICE_BINARYINFO:
		DBVsnacc_to_dbv_otherinfo_bin(psOtherInformation, data_ptr, env);
		break;
	case OTHERINFORMATIONSEQCHOICE_EXTERNALLYDEFINEDINFO:
		DBVsnacc_to_dbv_otherinfo_ext(psOtherInformation, data_ptr, env);
		break;
	case OTHERINFORMATIONSEQCHOICE_OID_SNACC:
		DBVsnacc_to_dbv_otherinfo_oid(psOtherInformation, data_ptr, env);
		break;
	}
}



int DBVsnacc_to_dbv_otherinfo_char(DbvOtherInformation *psOtherInformation, OtherInformationSeq *data_ptr, jmp_buf env)
{
	psOtherInformation->eInformationType = IT_CharacterInfo;
        psOtherInformation->uInformation.psCharacterInfo = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
	CheckDbvAlloc(psOtherInformation->uInformation.psCharacterInfo, env);
 
	DBVsnacc_to_dbv_inter_string(psOtherInformation->uInformation.psCharacterInfo, data_ptr->information->a.characterInfo, env);

}
 


int DBVsnacc_to_dbv_otherinfo_bin(DbvOtherInformation *psOtherInformation, OtherInformationSeq *data_ptr, jmp_buf env)
{
	psOtherInformation->eInformationType = IT_BinaryInfo;
        psOtherInformation->uInformation.psBinaryInfo = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));

	DBVsnacc_to_dbv_octet_string(psOtherInformation->uInformation.psBinaryInfo, data_ptr->information->a.binaryInfo, env);

}
 


int DBVsnacc_to_dbv_otherinfo_ext(DbvOtherInformation *psOtherInformation, OtherInformationSeq *data_ptr, jmp_buf env)
{
	psOtherInformation->eInformationType = IT_ExternalInfo;
        psOtherInformation->uInformation.psExternallyDefinedInfo = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
	CheckDbvAlloc(psOtherInformation->uInformation.psExternallyDefinedInfo, env);
 
	DBVsnacc_to_dbv_external(psOtherInformation->uInformation.psExternallyDefinedInfo, data_ptr->information->a.externallyDefinedInfo, env);

}



int DBVsnacc_to_dbv_otherinfo_oid(DbvOtherInformation *psOtherInformation, OtherInformationSeq *data_ptr, jmp_buf env)
{
	char direct_reference[SA_ARRAY_SIZE];

	/* Get the direct reference (oid) of the external */
	memset(direct_reference, 0, sizeof(direct_reference));

        /* Note - no & needed in below because a.oid is an address */
	DBVsa_strcpy(direct_reference, (char *) ASN_EncodedToString(data_ptr->information->a.oid));

	psOtherInformation->eInformationType = IT_ObjectIdentifier;

	/* Add the oid to the dbv structure : +1 to copy the \0 */
        psOtherInformation->uInformation.pcObjectIdentifier = (char *) DbvAlloc (strlen(direct_reference) + 1);
	CheckDbvAlloc(psOtherInformation->uInformation.pcObjectIdentifier, env);
 
       	memcpy(psOtherInformation->uInformation.pcObjectIdentifier, direct_reference, strlen(direct_reference) + 1);
}



int DBVcheck_dbv_otherinfo(DbvAssocId lAssocId, DbvOtherInformation **ppsOtherInformation, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	while (ppsOtherInformation[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");
 
		if (DBVcheck_dbv_otherinfo_node(lAssocId, ppsOtherInformation[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
 
                index++;
	}

	return SI_OK;
}



int DBVcheck_dbv_otherinfo_node(DbvAssocId lAssocId, DbvOtherInformation *psOtherInformation, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];
 
	DBVsa_strcpy(local_error_buffer, error_buffer);


	/* psInfoCategory can be NULL */ 
	if (psOtherInformation->psInfoCategory != NULL) 

		/* pcCategoryTypeId can be NULL */
		if (psOtherInformation->psInfoCategory->pcCategoryTypeId != NULL)
			if (DBVcheck_dbv_validoid(psOtherInformation->psInfoCategory->pcCategoryTypeId) == SI_NOTOK) {
				DBVsa_strcat(local_error_buffer, "psInfoCategory->pcCategoryTypeId");
				return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
			}
        


	/* No need to check ulCategoryValue */


	switch (psOtherInformation->eInformationType) {
	case IT_CharacterInfo:

		if (psOtherInformation->uInformation.psCharacterInfo == NULL) {
			DBVsa_strcat(local_error_buffer, "uInformation.psCharacterInfo");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}
		break;
	case IT_BinaryInfo:
		if (psOtherInformation->uInformation.psBinaryInfo == NULL) {
			DBVsa_strcat(local_error_buffer, "uInformation.psBinaryInfo");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
	case IT_ExternalInfo:
		if (psOtherInformation->uInformation.psExternallyDefinedInfo == NULL) {
			DBVsa_strcat(local_error_buffer, "uInformation.psExternallyDefinedInfo");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		return(DBVcheck_dbv_external(lAssocId, psOtherInformation->uInformation.psExternallyDefinedInfo, local_error_buffer, ppsSIError));	
		break;
	case IT_ObjectIdentifier:
		if (psOtherInformation->uInformation.pcObjectIdentifier == NULL) {
			strcat(local_error_buffer, "uInformation.pcObjectIdentifier");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		if (DBVcheck_dbv_validoid(psOtherInformation->uInformation.pcObjectIdentifier) == SI_NOTOK) {
			strcat(local_error_buffer, "uInformation.pcObjectIdentifier");
			return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
	default:
		strcat(local_error_buffer, "eInformationType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}
		

	return SI_OK;
}
 




