/* -----------  itemordr.h -------------------------------------------*/

/* When using this file with other DBV-OSI II includes, include as follows

#include "sidefs.h"
#include "sitypes.h"
#include "sitypes2.h"
#include "dbvproto.h"
#include "iord_ext.h"
#include "itemordr.h"
*/





#if !defined(ITEMORDER_H)
#define ITEMORDER_H


typedef enum DbvEsIoAuxiliaryStatus {
             IOAS_NotReceived       = 1,
             IOAS_LoanQueue         = 2,
             IOAS_Forwarded         = 3,
             IOAS_UnfilledCopyright = 4,
             IOAS_FilledCopyright   = 5,
             IOAS_Invalid           = -1
} DbvEsIoAuxiliaryStatus;

typedef enum DbvEsIoPaymentMethodType {
             IOPM_BillInvoice          = 1,
             IOPM_Prepay               = 2,
             IOPM_DepositAccount       = 3,
             IOPM_CreditCard           = 4,
             IOPM_CardInfoPrevSupplied = 5,
             IOPM_PrivateKnown         = 6,
             IOPM_PrivateNotKnown      = 7,
             IOPM_Invalid              = -1
} DbvEsIoPaymentMethodType;


typedef struct DbvEsIoTargetPart {
DbvASN1External             *psItemRequest;        /* ILL ItemOrder PDU */
DbvASN1External             *psStatusOrErrorReport;
DbvEsIoAuxiliaryStatus      *peAuxiliaryStatus;
} DbvEsIoTargetPart;


typedef struct DbvEsIoResultSetItem {
DbvInternationalString    sResultSetId;
Ulong                     ulItem;
} DbvEsIoResultSetItem;


typedef struct DbvEsIoOriginPartNotToKeep {
DbvEsIoResultSetItem     *psResultSetItem;
DbvASN1External          *psItemRequest;     /* ILL ItemOrder */
} DbvEsIoOriginPartNotToKeep;


typedef struct DbvEsIoCreditCardInfo {
DbvInternationalString   sNameOnCard;
DbvInternationalString   sExpirationDate;
DbvInternationalString   sCardNumber;
} DbvEsIoCreditCardInfo;



typedef union uPaymentMethod {
     DbvEsIoCreditCardInfo  *psCreditCardInfo;
     DbvASN1External        *psPrivateNotKnown;
} uPaymentMethod;

typedef struct DbvEsIoPaymentMethod {
DbvEsIoPaymentMethodType          ePaymentMethodType;
uPaymentMethod                   *puPaymentMethod;
} DbvEsIoPaymentMethod;


typedef struct DbvEsIoAdditionalBilling {
DbvEsIoPaymentMethod         sPaymentMethod;
DbvInternationalString      *psCustomerReference;
DbvInternationalString      *psCustomerPONumber;
} DbvEsIoAdditionalBilling;


typedef struct DbvEsIoContact {
DbvInternationalString    *psName;
DbvInternationalString    *psPhone;
DbvInternationalString    *psEmail;
} DbvEsIoContact;


typedef struct DbvEsIoOriginPartToKeep {
DbvASN1External          *psSupplementalDescription;
DbvEsIoContact           *psContact;
DbvEsIoAdditionalBilling *psAdditionalBilling;
} DbvEsIoOriginPartToKeep;


typedef struct DbvEsIoRequest {
DbvEsIoOriginPartToKeep		*psToKeep;
DbvEsIoOriginPartNotToKeep	 sNotToKeep;
} DbvEsIoRequest;


typedef struct DbvEsIoTaskPackage {
DbvEsIoOriginPartToKeep		*psOriginPart;
DbvEsIoTargetPart		 sTargetPart;
} DbvEsIoTaskPackage;



typedef enum DbvItemOrderReqOrRsp {
                    IOROR_Request  = 1,
                    IOROR_Response = 2
} DbvItemOrderReqOrRsp;


typedef struct DbvItemOrderExtendedService {
DbvItemOrderReqOrRsp     eItemOrderReqRsp;
union {
    DbvEsIoRequest              *psItemOrderRequest;
    DbvEsIoTaskPackage          *psItemOrderResponse;
  }                uItemOrderReqOrRsp;
} DbvItemOrderExtendedService;



/* -----------------   ILL-Definitions ------------------------- */

typedef enum  DbvIllStringType {
		ST_GeneralString = 1,
		ST_EdifactString = 2
} DbvIllStringType;



typedef struct DbvIllString {
DbvIllStringType	eStringType;
union {
	DbvGeneralString   *psGeneralString;
	DbvVisibleString    pcVisibleString;
    }                   uString;
} DbvIllString;




/* enumeration types IllRequest */

typedef char*  DbvPrintableString;

typedef enum DbvIllProtocolVersion {
		PV_Version1 = 1,
		PV_Version2 = 2,
		PV_Invalid  = -1 
}DbvIllProtocolVersion;

typedef enum DbvNameOfPersOrInstType {
		NPI_NameOfPerson      = 1,
                NPI_NameOfInstitution = 2
} DbvNameOfPersOrInstType;


typedef enum  DbvPersOrInstSymbolType {
		SPI_PersonSymbol      = 1,
                SPI_InstitutionSymbol = 2
} DbvPersOrInstSymbolType;


typedef enum  DbvTransactionType {
		TT_Simple      = 1,
                TT_Chained     = 2,
                TT_Partitioned = 3,
                TT_Invalid     = -1
} DbvTransactionType;


typedef enum  DbvDeliveryServiceType {
		DST_PhysicalDelivery   = 1,
                DST_ElectronicDelivery = 2,
                DST_Invalid            = -1
} DbvDeliveryServiceType;


typedef enum DbvElDeliveryDetailsType {
		DDT_SystemAddress = 1, 
		DDT_SystemId      = 2
} DbvElDeliveryDetailsType;


typedef enum  DbvServiceType {
		ST_Loan              = 1,
                ST_CopyNonReturnable = 2,
		ST_Locations         = 3,
		ST_Estimate          = 4,
		ST_ResponderSpecific = 5,
		ST_Invalid           = -1
} DbvServiceType;


typedef enum  DbvOptionalMsgRequirements {
		OMR_Requires = 1,
		OMR_Desires = 2,
		OMR_Neither = 3,
		OMR_Invalid = -1
} DbvOptionalMsgRequirements;


typedef enum  DbvExpiry {
		EXP_NeedBeforeDate = 1,
		EXP_OtherDate      = 2,
		EXP_NoExpiry       = 3,
		EXP_Invalid        = -1
} DbvExpiry;


typedef enum DbvSupplyMediumType {
		SMT_Printed              = 1,
		SMT_Photocopy            = 2,
		SMT_Microform            = 3,
		SMT_FilmOrVideoRecording = 4,
		SMT_AudioRecording       = 5,
		SMT_MachineReadable      = 6,
		SMT_Other                = 7,
		SMT_Invalid              = -1
} DbvSupplyMediumType;


typedef enum  DbvPlaceOnHold {
		POH_Yes             = 1,
		POH_No              = 2,
		POH_ResponderPolicy = 3,
		POH_Invalid         = -1
} DbvPlaceOnHold;


typedef enum DbvItemType {
		IT_Monograph = 1,
		IT_Serial    = 2,
		IT_Other     = 3,
		IT_Invalid   = -1
} DbvItemType;


typedef enum  DbvMediumType {
		MT_Printed              = 1,
		MT_Microform            = 3,
		MT_FilmOrVideoRecording = 4, 
                MT_AudioRecording       = 5, 
                MT_MachineReadable      = 6,
		MT_Other                = 7,
		MT_Invalid              = -1
} DbvMediumType;


typedef enum  DbvPreference {
		PRE_Ordered   = 1,
		PRE_Unordered = 2,
		PRE_Invalid   = -1
} DbvPreference;


/* enumeration types StatusOrErrorReport */

typedef enum  DbvTransactionResults {
		TR_Conditional       = 1,
		TR_Retry             = 2,
		TR_Unfilled          = 3,
		TR_LocationsProvided = 4,
		TR_WillSupply        = 5,
		TR_HoldPlaced        = 6,
		TR_Estimate          = 7,
		TR_Invalid           = -1
} DbvTransactionResults;


typedef enum  DbvMostRecentService {
		MRS_IllRequest          = 1,
		MRS_Forward             = 21,
		MRS_ForwardNotification = 2,
		MRS_Shipped             = 3,
		MRS_IllAnswer           = 4,
		MRS_ConditionalReply    = 5,
		MRS_Cancel              = 6,
		MRS_CancelReply         = 7,
		MRS_Received            = 8,
		MRS_Recall              = 9,
		MRS_Returned            = 10,
		MRS_CheckedIn           = 11,
		MRS_Overdue             = 12,
		MRS_Renew               = 13,
		MRS_RenewAnswer         = 14,
		MRS_Lost                = 15,
		MRS_Damaged             = 16,
		MRS_Message             = 17,
		MRS_StatusQuery         = 18,
		MRS_StatusOrErrorReport = 19,
		MRS_Expired             = 20,
		MRS_Invalid             = -1
} DbvMostRecentService;


typedef enum  DbvShippedServiceType {
		SST_Loan              = 1,
		SST_CopyNonReturnable = 2,
		SST_Invalid           = -1
} DbvShippedServiceType;


typedef enum  DbvCurrentState {
		CS_NotSupplied        = 1,
		CS_Pending            = 2,
		CS_InProcess          = 3,
		CS_Forward            = 4,
		CS_Conditional        = 5,
		CS_CancelPending      = 6,
		CS_Cancelled          = 7,
		CS_Shipped            = 8,
		CS_Received           = 9,
		CS_RenewPending       = 10,
		CS_NotReceivedOverdue = 11,
		CS_RenewOverdue       = 12,
		CS_Overdue            = 13,
		CS_Returned           = 14,
		CS_CheckedIn          = 15,
		CS_Recall             = 16,
		CS_Lost               = 17,
		CS_Unknown            = 18,
		CS_Invalid            = -1
} DbvCurrentState;

typedef enum  DbvIllAPDUType {
		IAT_IllRequest          = 1,
		IAT_ForwardNotification = 2,
		IAT_Shipped             = 3,
		IAT_IllAnswer           = 4,
		IAT_ConditionalReply    = 5,
		IAT_Cancel              = 6,
		IAT_CancelReply         = 7,
		IAT_Received            = 8,
		IAT_Recall              = 9,
		IAT_Returned            = 10,
		IAT_CheckedIn           = 11,
		IAT_Overdue             = 12,
		IAT_Renew               = 13,
		IAT_RenewAnswer         = 14,
		IAT_Lost                = 15,
		IAT_Damaged             = 16,
		IAT_Message             = 17,
		IAT_StatusQuery         = 18,
		IAT_StatusOrErrorReport = 19,
		IAT_Expired             = 20,
		IAT_Invalid             = -1
} DbvIllAPDUType;


typedef enum  DbvReportSource {
		RS_User     = 1,
		RS_Provider = 2
} DbvReportSource;


typedef enum  DbvIntermediaryProblem {
		IP_CannotSendOnward = 1,
		IP_Invalid          = -1
} DbvIntermediaryProblem;


typedef enum  DbvUnableToPerform {
		UTP_NotAvailable       = 1,
		UTP_ResourceLimitation = 2,
		UTP_Other              = 3,
		UTP_Invalid            = -1
} DbvUnableToPerform;


typedef enum  DbvUserErrorReportType {
		UER_AlreadyForwarded    = 1,
		UER_IntermediaryProblem = 2,
		UER_SecurityProblem     = 3,
		UER_UnableToPerform     = 4
} DbvUserErrorReportType;


typedef enum  DbvProviderErrorReportType {
		PER_GeneralProblem            = 1,
		PER_TransactionIdProblem      = 2,
		PER_StateTransitionProhibited = 3
} DbvProviderErrorReportType;


typedef enum  DbvTransactionIdProblem {
		TIP_DuplicateTransactionId = 1,
		TIP_InvalidTransactionId   = 2,
		TIP_UnknownTransactionId   = 3,
		TIP_Invalid                = -1
} DbvTransactionIdProblem;

typedef enum DbvReasonNoReport {
		RNR_Temporary = 1,
		RNR_Permanent = 2,
		RNR_Invalid   = -1
} DbvReasonNoReport;

typedef enum  DbvGeneralProblem {
		GP_UnrecognizedAPDU            = 1,
		GP_MistypedAPDU                = 2,
		GP_BadlyStructuredAPDU         = 3,
		GP_ProtocolVersionNotSupported = 4,
		GP_Other                       = 5,
		GP_Invalid                     = -1
} DbvGeneralProblem;

/* end enumeration types StatusOrErrorReport */




/* structures IllRequest */

/* SystemId  */

typedef struct DbvPersonOrInstitutionSymbol {
DbvPersOrInstSymbolType		eSymbolType;
DbvIllString			sSymbol;
} DbvPersonOrInstitutionSymbol;


typedef struct  DbvNameOfPersonOrInstitution {
DbvNameOfPersOrInstType		eNameType;
DbvIllString			sName;
} DbvNameOfPersonOrInstitution;


typedef struct  DbvSystemId {
DbvPersonOrInstitutionSymbol	*psSymbol;
DbvNameOfPersonOrInstitution	*psName;
} DbvSystemId;


/* TransactionId */

typedef struct  DbvTransactionId {
DbvSystemId		*psInitialRequesterId;
DbvIllString		 sTransactionGroupQualifier;
DbvIllString		 sTransactionQualifier;
DbvIllString		*psSubTransactionQualifier;
} DbvTransactionId;


/* ServiceDateTime */

typedef struct  DbvDateTime {
DbvVisibleString	pcDate;  /* YYYYMMDD,  mand. */
DbvVisibleString	pcTime;  /* HHMMSS, opt. */
} DbvDateTime;


typedef struct  DbvServiceDateTime {
DbvDateTime		 sThisService;
DbvDateTime		*psOriginalService;
} DbvServiceDateTime;


/* DeliveryAddress, SystemAddress */

typedef struct  DbvPostalAddress {
DbvNameOfPersonOrInstitution	*psNameOfPersonOrInstitution;
DbvIllString			*psExtendedPostalDeliveryAddress;
DbvIllString			*psStreetAndNumber;
DbvIllString			*psPostOfficeBox;
DbvIllString			*psCity;
DbvIllString			*psRegion;
DbvIllString			*psCountry;
DbvIllString			*psPostalCode;
} DbvPostalAddress;


typedef struct  DbvSystemAddress {
DbvIllString		*psTelecomServiceIdentifier;
DbvIllString		*psTelecomServiceAddress;
} DbvSystemAddress;


typedef struct  DbvDeliveryAddress {
DbvPostalAddress	*psPostalAddress;
DbvSystemAddress	*psElectronicAddress;
} DbvDeliveryAddress;


/* DeliveryService */

typedef struct  DbvElDeliveryDetails {
DbvElDeliveryDetailsType	eDeliveryDetailsType;
union {
	DbvSystemAddress   *psSystemAddress;
	DbvSystemId	   *psSystemId;
    }				uDeliveryDetails;
} DbvElDeliveryDetails;



typedef union uDeliveryParameter {
	DbvASN1External	     *psDummy;  /* mand. */
} uDeliveryParameter;


typedef struct  DbvElDeliveryParameter {
DbvObjectIdentifier	 pcDeliveryMode;       /* opt. */
uDeliveryParameter      *puDeliveryParameter;  /* opt. */
} DbvElDeliveryParameter;


typedef union uDocumentTypeParameter {
	DbvASN1External	     *psDummy;  /* mand. */
} uDocumentTypeParameter;


typedef struct  DbvDocumentTypeParameter {
DbvObjectIdentifier	 pcDocumentTypeId;        /* opt. */
uDocumentTypeParameter  *puDocumentTypeParameter; /* opt. */
} DbvDocumentTypeParameter;


typedef struct  DbvElectronicDeliveryService {
DbvElDeliveryParameter		*psDeliveryParameter;
DbvDocumentTypeParameter	*psDocumentTypeParameter;
DbvIllString			*psDeliveryDescription;
DbvElDeliveryDetails		 sDeliveryDetails;
DbvIllString			*psNameOrCode;
DbvVisibleString		 pcDeliveryTime;    /* opt. */
} DbvElectronicDeliveryService;


typedef struct  DbvDeliveryService {
DbvDeliveryServiceType		eDeliveryServiceType;
union {
	DbvIllString			 *psPhysicalDelivery;
	DbvElectronicDeliveryService	**ppsElectronicDelivery;
    }				uDeliveryService;
} DbvDeliveryService;


/* RequesterOptionalMessageType */

typedef struct  DbvRequesterOptionalMessages {
DbvBool				fCanSendReceived;
DbvBool				fCanSendReturned;
DbvOptionalMsgRequirements	eRequesterShipped;
DbvOptionalMsgRequirements	eRequesterCheckedIn;
} DbvRequesterOptionalMessages;


typedef struct  DbvSearchType {
DbvIllString		*psLevelOfService;
DbvVisibleString	 pcNeedBeforeDate;  /* opt. */
DbvExpiry		*peExpiryFlag;      /* default 3 */
DbvVisibleString	 pcExpiryDate;      /* opt. */
} DbvSearchType;


/* SupplyMediumInfoType */

typedef struct  DbvSupplyMediumInfo {
DbvSupplyMediumType	 eSupplyMediumType;
DbvIllString		*psMediumCharacteristics;
} DbvSupplyMediumInfo;


/* ClientId */

typedef struct  DbvClientId {
DbvIllString		*psClientName;
DbvIllString		*psClientStatus;
DbvIllString		*psClientIdentifier;
} DbvClientId;


/* ItemId */

typedef struct  DbvItemId {
DbvItemType		*peItemType;
DbvMediumType		*peMediumType;
DbvIllString		*psCallNumber;
DbvIllString		*psAuthor;
DbvIllString		*psTitle;
DbvIllString		*psSubTitle;
DbvIllString		*psSponsoringBody;
DbvIllString		*psPlaceOfPublication;
DbvIllString		*psPublisher;
DbvIllString		*psSeriesTitleNumber;
DbvIllString		*psVolumeIssue;
DbvIllString		*psEdition;
DbvIllString		*psPublicationDate;
DbvIllString		*psPublicationDateOfComponent;
DbvIllString		*psAuthorOfArticle;
DbvIllString		*psTitleOfArticle;
DbvIllString		*psPagination;
DbvASN1External		*psNationalBibliographyNo;
DbvIllString		*psIsbn;
DbvIllString		*psIssn;
DbvASN1External		*psSystemNo;
DbvIllString		*psAdditionalNoLetters;
DbvIllString		*psVerificationReferenceSource;
} DbvItemId;


/* CostInfoType */

typedef struct DbvAmount {
DbvPrintableString	pcCurrencyCode;    /* opt. */
DbvPrintableString	pcMonetaryValue;   /* mand */
} DbvAmount;


typedef struct DbvCostInfo {
DbvIllString		*psAccountNumber;
DbvAmount		*psMaximumCost;
DbvBool			*pfReciprocalAgreement;  /* default FALSE */
DbvBool			*pfWillPayFee;           /* default FALSE */
DbvBool			*pfPaymentProvided;      /* default FALSE */
} DbvCostInfo;


/* ThirdPartyInfoType */

typedef struct DbvSendToList {
DbvSystemId		 sSystemId;
DbvIllString		*psAccountNumber;
DbvSystemAddress	*psSystemAddress;
} DbvSendToList;


typedef struct DbvThirdPartyInfo {
DbvBool			 *pfPermissionToForward;          /* default FALSE */
DbvBool			 *pfPermissionToChain;            /* default FALSE */ 
DbvBool			 *pfPermissionToPartition;        /* default FALSE */
DbvBool			 *pfPermissionToChangeSendToList; /* default FALSE */
DbvSystemAddress	 *psInitialRequesterAddress;
DbvPreference		 *pePreference;                   /* default 2 */
DbvSendToList		**ppsSendToList;
DbvSystemId		**ppsAlreadyTriedList;
} DbvThirdPartyInfo;

typedef struct DbvExtension {
Ulong		 ulIdentifier;
DbvBool		*pfCritical;     /* default FALSE */
union {
	DbvSubitoAdditionalInfo *psAddIllReqInfo;
    }            uItem;
} DbvExtension;


typedef struct DbvIllRequest {
DbvIllProtocolVersion		  eProtocolVersion;
DbvTransactionId		  sTransactionId;
DbvServiceDateTime		  sServiceDateTime;
DbvSystemId			 *psRequesterId;
DbvSystemId			 *psResponderId;
DbvTransactionType		 *peTransactionType;     /* default 1 */
DbvDeliveryAddress		 *psDeliveryAddress;
DbvDeliveryService		 *psDeliveryService;
DbvDeliveryAddress		 *psBillingAddress;
DbvServiceType			**ppeServiceType;
DbvASN1External			 *psResponderSpecificService;
DbvRequesterOptionalMessages	  sRequesterOptionalMessages;
DbvSearchType			 *psSearchType;
DbvSupplyMediumInfo		**ppsSupplyMediumInfo;
DbvPlaceOnHold			 *pePlaceOnHold;        /* default 3 */
DbvClientId			 *psClientId;
DbvItemId			  sItemId;
DbvASN1External			**ppsSupplementalItemDescription;
DbvCostInfo			 *psCostInfo;
DbvIllString			 *psCopyrightCompliance;
DbvThirdPartyInfo		 *psThirdPartyInfo;
DbvBool				 *pfRetry;	/* default FALSE */
DbvBool				 *pfForward;	/* default FALSE */
DbvIllString			 *psRequesterNote;
DbvIllString			 *psForwardNote;
DbvExtension			**ppsRequestExtension;
} DbvIllRequest;



/* structures StatusOrErrorReport */


typedef struct  DbvHistoryReport {
DbvVisibleString		 pcDateRequested;           /* opt. */
DbvIllString			*psAuthor;
DbvIllString			*psTitle;
DbvIllString			*psAuthorOfArticle;
DbvIllString			*psTitleOfArticle;
DbvVisibleString		 pcDateOfLastTransition;    /* opt. */
DbvMostRecentService		 eMostRecentService;
DbvVisibleString		 pcDateOfMostRecentService; /* mand. */
DbvSystemId			 sSystemId;
DbvShippedServiceType		*peShippedServiceType;
DbvTransactionResults		*peTransactionResults;
DbvIllString			*psMostRecentServiceNote;
} DbvHistoryReport;


typedef struct  DbvStateTransitionProhibited {
DbvIllAPDUType		eAPDUType;
DbvCurrentState		eCurrentState;
} DbvStateTransitionProhibited;


typedef struct  DbvAlreadyForwarded {
DbvSystemId		 sResponderId;
DbvSystemAddress	*psResponderAddress;
} DbvAlreadyForwarded;


typedef struct  DbvProviderErrorReport {
DbvProviderErrorReportType	eReportType;
union {
	DbvGeneralProblem	      eGeneralProblem;
	DbvTransactionIdProblem	      eTransactionIdProblem;
	DbvStateTransitionProhibited *psStateTransitionProblem;
    }				uProviderErrorReport;
} DbvProviderErrorReport;


typedef struct  DbvUserErrorReport {
DbvUserErrorReportType		eReportType;
union {
	DbvAlreadyForwarded	   *psAlreadyForwarded;
	DbvIntermediaryProblem	    eIntermediaryProblem;
	DbvIllString		   *psSecurityProblem;
	DbvUnableToPerform	    eUnableToPerform;
    }                           uUserErrorReport;
} DbvUserErrorReport;

typedef struct DbvUserOrProviderErrorReport {
DbvReportSource		eReportSource;
union {
	DbvProviderErrorReport	*psProviderErrorReport;
	DbvUserErrorReport	*psUserErrorReport;
    }			uReport;
} DbvUserOrProviderErrorReport;

typedef struct  DbvErrorReport {
DbvIllString	     		 sCorrelationInfo;
DbvUserOrProviderErrorReport	 sReport;
} DbvErrorReport;


typedef struct  DbvStatusReport {
DbvHistoryReport	sUserStatusReport;
DbvCurrentState		eProviderStatusReport;
} DbvStatusReport;


typedef struct  DbvStatusOrErrorReport {
DbvIllProtocolVersion		  eProtocolVersion;
DbvTransactionId		  sTransactionId;
DbvServiceDateTime		  sServiceDateTime;
DbvSystemId			 *psRequesterId;
DbvSystemId			 *psResponderId;
DbvReasonNoReport		 *peReasonNoReport;
DbvStatusReport			 *psStatusReport;
DbvErrorReport		 	 *psErrorReport;
DbvIllString			 *psNote;
DbvExtension			**ppsStOrErReportExtension;
} DbvStatusOrErrorReport;



/************************************************/
/* ILL APDU Definition */

/* Note ILL_Invalid in the typedef enum below.
   An Invalid option is not normally
   used in DBV-OSI II union enums but is needed here because
   DbvIllReport only supports two of the many possible
   ILL Report types which may be received. 

   ILL_Invalid should never be used when setting up
   an ILL request for sending, it will be set 
   internally by the API if an ILL report unsupported
   by DBV-OSI II is received.
  
*/ 

typedef enum DbvIllPduType {
             ILL_IllRequest             = 1,
             ILL_StatusOrErrorReport    = 19,
             ILL_Invalid                = -1   
} DbvIllPduType;



typedef struct DbvIllPdu {
DbvIllPduType          eIllPduType;
union {
    struct DbvIllRequest               *psIllRequest;
    struct DbvStatusOrErrorReport      *psStatusOrErrorReport;
  }               uIllPdu;
} DbvIllPdu;



#endif

