/* -----------  sil_defs.h -------------------------------------------*/

/* DO NOT INCLUDE THIS FILE IN AN APPLICATION PROGRAM. IT IS USED 
   WHEN BUILDING THE DBV-OSI II LIBRARIES */

/*  Module Name: @(#)sil_defs.h	7.2
    Version: 12/2/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
 
*/

/* SIL definitions */
#if !defined(SIL_DEFS_H)
#define SIL_DEFS_H

#ifndef NO_OSI
/* ISODE include file */
#include <ul/psap.h>
#include <ul/acsap.h>
#endif

/* Standard port for TCP connections to Z.39.50 server */
#define DEFAULT_PORT_NO 210

/* This is the filename of the name file */
#define DBV_NAME_FILE	"./target_names.tbl"

/* The default ACSE configuration file */
#define DBV_ACSE_FILE 	"./ACSE_configuration_file.tbl"

/* The default API log file */
#define DBV_API_LOGFILE	"./DBV_API_logfile"

typedef enum DBVOriginOrTarget {
	O = 1, T = 2
} DBVOriginOrTarget;

/* store details about connections which have been initiated/completed */
/* OSI fields need to be put in here also */
typedef struct DBVAssociateDetails {
	DbvAssocId lAssocId;
	DbvOsiOrInternetAddress eOsiOrInternetAddress;
	DbvFds ulTcpSocket;
	DbvAssociateState eAssociateState;
	DbvReleaseState eReleaseState;
	struct sockaddr_in sa;
	long lCurrentState;	/* Stores the current state of the association.
				Used (obviously) in the state machine */

#ifndef NO_OSI
	long lAssocResult; 	/* lAssocResult stores the ISODE value for the 
				association state:  OK/DONE, NOTOK, 
				CONNECTING_1, CONNECTING_2 */

	long lOSIAssocId; 	/* lOSIAssocId contains the association 
				ID which ISODE assigns the association */
	struct AcSAPstart *psAssocControlStart; /* Used to transfer information 
				from DbvReceiveAssociateRequest to 
				DbvAssociateResponse */
#endif
	DbvApplicationProtocol eApplicationProtocol; /* Used between 
				DbvReceiveAssociateRequest and 
				DbvAssociateResponse */

	DBVOriginOrTarget eOriginOrTarget;

} DBVAssociateDetails;

/* Used as a return value when a release request is detected */
#define DBV_RELEASE_REQUEST	10

/* Stores the proposed presentation context definition list OID's in char *
	format so that they can be compared with the negotiated list in the
	AAsociate.Confirmation service primitive. NPCTX is defined in the Isode 
	header "psap.h" and is the maximum number of contexts allowed in the 
	list */
#ifndef NO_OSI
char *ppcDBVProposedPresContexts[NPCTX];
#endif

#ifndef NO_OSI
/* Defines the session requirements for an association */
#define DBV_SESSION_REQUIREMENTS (SR_DUPLEX | SR_EXPEDITED)
#endif

/* Defines the object identifiers for Z39.50 and SR protocols */
/* Used in the OSI association functions (Application Contexts) */
#define BASIC_Z3950_OID	"1.2.840.10003.1.1"
#define BASIC_SR_OID	"1.0.10163.1.1" /* Default ACN */

/* Defines the default service name (for entries in the isobjects file) */
/* See originassoc.c and address_by_name.c */
#define DBV_SERVICE "SR/Z39.50 Service"

/* Defines the standard port on which the target will listen */
#define TARGET_PORT "2020"

/* Store trace details for Asoociations */
typedef struct DBVTraceDetails {
        DbvTraceLevel DBVTraceLevel;
        FILE *DBVTraceFile;
        DbvAssocId lAssocId;
} DBVTraceDetails;

#endif
