/*  Module Name: @(#)FreeC_DbvFuncs.c	7.1
    Version: 11/15/96
    Author: Dan Maskell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for compulsory structure types.
	The pointer passed is a pointer to an embedded structure and is 
	therefore not freed as it would be inappropriate
        Used by the origin and target.

    05-02-96 Dan Maskell Added Functions for ILL & Item order stuff
*/

#include "testy_calls.h"
#include "Free_DbvFuncs.h"

void Free_compulsory_DbvAdditionalInfo( DbvAdditionalInfo *psAdditionalInfo )
   {

   if ( psAdditionalInfo == NULL )
      return;

   switch(  psAdditionalInfo->eProtocolVersionInForce )
      {
      case PVIF_Version2:
         Free_DbvVisibleString( psAdditionalInfo->uAdditionalInfo.pcV2 );
	 break;

      case PVIF_Version3:
         FreeDbvInternationalString( psAdditionalInfo->uAdditionalInfo.psV3 );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvAdditionalInfo() */ 

void Free_compulsory_DbvAttributeValue( DbvAttributeValue *psAttributeValue )
   {

   if ( psAttributeValue == NULL )
      return;

   switch(  psAttributeValue->eAttributeValueType )
      {
      case AVT_Numeric:
	 /* No space allocated */
	 break;

      case AVT_Complex:
         Free_DbvComplexValue( psAttributeValue->
			       uAttributeValue.psComplexValue );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvAttributeValue() */ 

void Free_compulsory_DbvHoldingsAndCircData( DbvHoldingsAndCircData 
					*psHoldingsAndCircData )
   {

   /* psTypeOfRecord */

   if ( psHoldingsAndCircData->psTypeOfRecord != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psTypeOfRecord );

   /* psEncodingLevel */

   if ( psHoldingsAndCircData->psEncodingLevel != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psEncodingLevel );

   /* psFormat */

   if ( psHoldingsAndCircData->psFormat != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psFormat );

   /* psReceiptAcqStatus */

   if ( psHoldingsAndCircData->psReceiptAcqStatus != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psReceiptAcqStatus );

   /* psGeneralRetention */

   if ( psHoldingsAndCircData->psGeneralRetention != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psGeneralRetention );

   /* psCompleteness */

   if ( psHoldingsAndCircData->psCompleteness != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCompleteness );

   /* psDateOfReport */

   if ( psHoldingsAndCircData->psDateOfReport != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psDateOfReport );

   /* psNucCode */

   if ( psHoldingsAndCircData->psNucCode != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psNucCode );

   /* psLocalLocation */

   if ( psHoldingsAndCircData->psLocalLocation != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psLocalLocation );

   /* psShelvingLocation */

   if ( psHoldingsAndCircData->psShelvingLocation != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psShelvingLocation );

   /* psCallNumber */

   if ( psHoldingsAndCircData->psCallNumber != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCallNumber );

   /* psShelvingData */

   if ( psHoldingsAndCircData->psShelvingData != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psShelvingData );

   /* psCopyNumber */

   if ( psHoldingsAndCircData->psCopyNumber != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCopyNumber );

   /* psPublicNote */

   if ( psHoldingsAndCircData->psPublicNote != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psPublicNote );

   /* psReproductionNote */

   if ( psHoldingsAndCircData->psReproductionNote != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psReproductionNote );

   /* psTermsUseRepro */

   if ( psHoldingsAndCircData->psTermsUseRepro != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psTermsUseRepro );

   /* psEnumAndChron */

   if ( psHoldingsAndCircData->psEnumAndChron != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psEnumAndChron );

   /* ppsVolume */

   FreeN_DbvVolume( psHoldingsAndCircData->ppsVolume );

   /* ppsCircRecord*/

   FreeN_DbvCircRecord( psHoldingsAndCircData->ppsCircRecord );

   /* Note No final free as compulsory */

   } /* End Free_DbvHoldingsAndCircData() */ 

void Free_compulsory_DbvInternationalString( DbvInternationalString 
					*psInternationalString)
   {
   DbvFree(psInternationalString->pcValue);
   } /* Free_compulsory_DbvInternationalString() */

void Free_compulsory_DbvOriginPartToKeep( DbvOriginPartToKeep 
						*psOriginPartToKeep )
   {

   if ( psOriginPartToKeep == NULL )
      return;

   /* sDatabaseName */

   Free_compulsory_DbvInternationalString( &psOriginPartToKeep->sDatabaseName ); 
   /* pcSchema */

   if ( psOriginPartToKeep->pcSchema != NULL )
      FreeDbvObjectIdentifier( psOriginPartToKeep->pcSchema );

   /* psElementSetName */

   if ( psOriginPartToKeep->psElementSetName != NULL )
      FreeDbvInternationalString( psOriginPartToKeep->psElementSetName ); 


   } /* End Free_compulsory_DbvOriginPartToKeep() */ 

void Free_compulsory_DbvTerm( DbvTerm *psTerm )
   {

   if ( psTerm == NULL )
      return;

   switch( psTerm->eTermType )
      {
      case TT_General:
         if ( psTerm->uTerm.psGeneral != NULL )
	    FreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
         /* No Space allocated */
	 break;

      case TT_SimpleCharacter:
         if ( psTerm->uTerm.psSimpleCharacter != NULL )
            FreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter ); 
	 break;

      case TT_ObjectIdentifier:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            FreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            Free_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
         FreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 Free_DbvIntUnit(  psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_compulsory_DbvTerm() */ 

void Free_compulsory_DbvUnit( DbvUnit *psUnit )
   {

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */

   if ( psUnit->psUnitSystem != NULL )
      FreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */

   if ( psUnit->psUnitType != NULL )
      Free_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */

   if ( psUnit->psUnit != NULL )
      Free_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */

   if ( psUnit->pulScaleFactor != NULL )
      Free_Ulong( psUnit->pulScaleFactor );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End Free_compulsory_DbvUnit() */

void Free_compulsory_DbvUpdateTargetPart( DbvUpdateTargetPart 
						*psUpdateTargetPart )
   {

   if ( psUpdateTargetPart == NULL )
      return;

   /* ppsGlobalDiagnostics */

   FreeN_DbvDiagRec( psUpdateTargetPart->ppsGlobalDiagnostics );

   /* ppsTaskPackageRecord */

   FreeN_DbvTaskPackageRecord( psUpdateTargetPart->ppsTaskPackageRecord );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End Free_compulsory_DbvUpdateTargetPart() */



void Free_compulsory_DbvSpecification(DbvSpecification *psSpecification)
{
	FreeDbvObjectIdentifier(psSpecification->pcSchema);

	/* optional */
	Free_DbvElementSpec(psSpecification->psElementSpec);
} /* End Free_compulsory_DbvSpecification */


/* 12-2-96 The following function where added by Dan Maskell */

void Free_compulsory_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      do_Free_DbvEsIoTargetPart( psEsIoTargetPart );

   } /* End Free_compulsory_DbvEsIoTargetPart() */ 

void Free_compulsory_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      do_Free_DbvIllString( psIllString );

   } /* End Free_compulsory_DbvIllString() */ 

void Free_compulsory_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      do_Free_DbvDateTime( psDateTime );

   } /* End Free_compulsory_DbvDateTime() */ 


void Free_compulsory_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      do_Free_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );

   } /* End Free_compulsory_DbvUserOrProviderErrorReport() */ 

void Free_compulsory_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      do_Free_DbvHistoryReport( psHistoryReport );

   } /* End Free_compulsory_DbvHistoryReport() */ 

void Free_compulsory_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      do_Free_DbvItemId( psItemId );

   } /* End Free_compulsory_DbvItemId() */ 

void Free_compulsory_DbvElDeliveryDetails( 
		DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      do_Free_DbvElDeliveryDetails( psElDeliveryDetails );

   } /* End Free_compulsory_DbvElDeliveryDetails() */ 

void Free_compulsory_DbvSystemId( DbvSystemId *psSystemId )
   {

   if ( psSystemId != NULL )
      do_Free_DbvSystemId( psSystemId );

   } /* End Free_compulsory_DbvSystemId() */ 

void Free_compulsory_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      do_Free_DbvTransactionId( psTransactionId );

   } /* End Free_compulsory_DbvTransactionId() */ 

void Free_compulsory_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      do_Free_DbvServiceDateTime( psServiceDateTime );

   } /* End Free_compulsory_DbvServiceDateTime() */ 


void Free_compulsory_DbvEsIoPaymentMethod( 
		DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod != NULL )
      do_Free_DbvEsIoPaymentMethod( psEsIoPaymentMethod );

   } /* End Free_compulsory_DbvEsIoPaymentMethod() */ 

void Free_compulsory_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      do_Free_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );

   } /* End Free_compulsory_DbvEsIoOriginPartNotToKeep() */ 





/* Explain Record Support */


void Free_compulsory_DbvCharge( DbvCharge *psCharge )
   {

   if ( psCharge != NULL )
      do_Free_DbvCharge( psCharge );

   } /* End Free_compulsory_DbvCharge() */ 

void Free_compulsory_DbvBodyType( DbvBodyType *psBodyType )
   {

   if ( psBodyType != NULL )
      do_Free_DbvBodyType( psBodyType );

   } /* End Free_compulsory_DbvBodyType() */ 

void Free_compulsory_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {

   if ( psAttributeOccurrenceChoice != NULL )
      do_Free_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );

   } /* End Free_compulsory_DbvAttributeOccurrenceChoice() */ 


#ifdef NOT_YET_IMPLEMENTED

/* Generic Record Support */

void Free_compulsory_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

   if ( psElementDataG != NULL )
      do_Free_DbvElementDataG( psElementDataG );

   } /* End Free_compulsory_DbvElementDataG() */ 


#endif 



/*******************************************************/
/* Access, Resource and Trigger Control */


void Free_compulsory_DbvSecurityChallengeReq( DbvSecurityChallengeReq *psSecurityChallengeReq )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSecurityChallengeReq()\n" );
#endif

   if ( psSecurityChallengeReq == NULL )
      return;

   /* uSecurityChallengeReq */

   switch( psSecurityChallengeReq->eSecurityChallengeReqType ) 
      {
      case SECT_SimpleForm:
   	 FreeDbvOctetString( psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 FreeDbvASN1External( psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvSecurityChallengeReq */



void Free_compulsory_DbvPromptId( DbvPromptId *psPromptId )
   {
#ifdef DEBUG
   printf( "D - Free_compulsory_DbvPromptId()\n" );
#endif

   if ( psPromptId == NULL )
      return;


   /* uPromptId */

   switch( psPromptId->ePromptIdType ) 
      {
      case PIT_EnumeratedPrompt:
   	 Free_DbvEnumeratedPrompt( psPromptId->uPromptId.psEnumeratedPrompt );
	 break;

      case PIT_NoneEnumeratedPrompt:
         FreeDbvInternationalString( psPromptId->uPromptId.psNonEnumeratedPrompt );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvPromptId */



void Free_compulsory_DbvPromptResponse( DbvPromptResponse *psPromptResponse )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPromptResponse()\n" );
#endif

   if ( psPromptResponse == NULL )
      return;


   /* uResponse */

   switch( psPromptResponse->eResponseType ) 
      {
      case RT_String:
   	 FreeDbvInternationalString( psPromptResponse->uResponse.psString );
	 break;

      case RT_Accept:
	 break;

      case RT_Acknowledge:
	 break;

      case RT_Diagnostic:
   	 Free_DbvDiagRec( psPromptResponse->uResponse.psDiagnostic );
	 break;

      case RT_Encrypted:
   	 Free_DbvEncryption( psPromptResponse->uResponse.psEncryption );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvPromptResponse() */


