/*  Module Name: @(#)Free_DbvFuncs.c	7.2
    Version: 3/3/97
    Author: Dan Maskell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for structure types.
        Used by the origin and target.

    Modification History.

    Implementation completed and de-bugged by Chris Attewell

    31-01-96 Dan Maskell Changed Resourse Report 2 to psMessage from sMessage
    12-02-96 Dan Maskell Added Functions for ILL & Item order stuff
 
*/

#include "Free_DbvFuncs.h"

void Free_DbvAttributeElement( DbvAttributeElement *psAttributeElement )
   {

   if ( psAttributeElement == NULL )
      return;

   /* pcAttributeSetId */

   if ( psAttributeElement->pcAttributeSetId != NULL )
      FreeDbvObjectIdentifier( psAttributeElement->pcAttributeSetId );

   /* sAttributeValue */

   Free_compulsory_DbvAttributeValue( &psAttributeElement->sAttributeValue );

   DbvFree( psAttributeElement );
   } /* End Free_DbvAttributeElement() */ 



void Free_DbvAttributesPlusTerm( DbvAttributesPlusTerm *psAttributesPlusTerm )
   {

   if ( psAttributesPlusTerm == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psAttributesPlusTerm->ppsAttributeElement );

   /* sTerm */

   Free_compulsory_DbvTerm( &psAttributesPlusTerm->sTerm );

   DbvFree( psAttributesPlusTerm );
   } /* End Free_DbvAttributesPlusTerm() */ 



void Free_DbvCircRecord( DbvCircRecord *psCircRecord )
   {

   if ( psCircRecord == NULL )
      return;

   /* psAvailabilityDate */

   if ( psCircRecord->psAvailabilityDate != NULL )
      FreeDbvInternationalString( psCircRecord->psAvailabilityDate );

   /* psAvailableThru */

   if ( psCircRecord->psAvailableThru != NULL )
      FreeDbvInternationalString( psCircRecord->psAvailableThru );

   /* psRestrictions */

   if ( psCircRecord->psRestrictions != NULL )
      FreeDbvInternationalString( psCircRecord->psRestrictions );

   /* psItemId */

   if ( psCircRecord->psItemId != NULL )
      FreeDbvInternationalString( psCircRecord->psItemId );

   /* psEnumAndChron */

   if ( psCircRecord->psEnumAndChron != NULL )
      FreeDbvInternationalString( psCircRecord->psEnumAndChron );

   /* psMidspine */

   if ( psCircRecord->psMidspine != NULL )
      FreeDbvInternationalString( psCircRecord->psMidspine );

   /* psTemporaryLocation */

   if ( psCircRecord->psTemporaryLocation != NULL )
      FreeDbvInternationalString( psCircRecord->psTemporaryLocation );

   DbvFree( psCircRecord );
   } /* End Free_DbvCircRecord() */ 


void Free_DbvCloseReq( DbvCloseReq *psCloseReq )
   {

   /* psReferenceId */

   if( psCloseReq->psReferenceId != NULL)
      FreeDbvOctetString( psCloseReq->psReferenceId );

   /* psDiagnosticInformation */

   if( psCloseReq->psDiagnosticInformation != NULL)
      FreeDbvInternationalString( psCloseReq->psDiagnosticInformation );

   /* pcResourceReportFormat */

   if( psCloseReq->pcResourceReportFormat != NULL)
      FreeDbvObjectIdentifier( psCloseReq->pcResourceReportFormat );

   /* psResourceReport */

   FreeDbvASN1External( psCloseReq->psResourceReport );

   /* ppsOtherInformation */

   if( psCloseReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psCloseReq->ppsOtherInformation );

   DbvFree( psCloseReq );
   } /* End Free_DbvCloseReq() */ 


void Free_DbvComplexValue( DbvComplexValue *psComplexValue )
   {

   if ( psComplexValue == NULL )
      return;

   /* ppsList */

   FreeN_DbvStringOrNumeric( psComplexValue->ppsList );

   /* ppulSemanticAction */

   FreeN_Ulong( psComplexValue->ppulSemanticAction );

   DbvFree( psComplexValue );
   } /* End Free_DbvComplexValue() */ 

void Free_DbvCorrelationInfo( DbvCorrelationInfo *psCorrelationInfo )
   {

   if ( psCorrelationInfo == NULL )
      return;

   /* psNote */

   if ( psCorrelationInfo->psNote != NULL )
      FreeDbvInternationalString( psCorrelationInfo->psNote );

   /* pulId */

   if ( psCorrelationInfo->pulId != NULL )
      Free_Ulong( psCorrelationInfo->pulId );

   DbvFree( psCorrelationInfo );
   } /* End Free_DbvCorrelationInfo() */ 

void Free_DbvDefaultDiagFormat( DbvDefaultDiagFormat *psDefaultDiagFormat )
   {

   if ( psDefaultDiagFormat == NULL )
      return;

   if ( psDefaultDiagFormat->pcDiagnosticSetId != NULL )
      FreeDbvObjectIdentifier( psDefaultDiagFormat->pcDiagnosticSetId );

   Free_compulsory_DbvAdditionalInfo( &psDefaultDiagFormat->sAdditionalInfo );

   DbvFree( psDefaultDiagFormat );
   } /* End Free_DbvDefaultDiagFormat() */ 

void Free_DbvDeleteResultSetReq( DbvDeleteResultSetReq *psDeleteResultSetReq )
   {

   /* psReferenceId */
   if( psDeleteResultSetReq->psReferenceId != NULL)
      FreeDbvOctetString(psDeleteResultSetReq->psReferenceId);
			 
   /* ppsResultSetId */

   FreeN_DbvInternationalString( psDeleteResultSetReq->ppsResultSetId );

   /* ppsOtherInformation */

   if( psDeleteResultSetReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetReq->ppsOtherInformation );

   DbvFree( psDeleteResultSetReq );
   } /* End Free_DbvDeleteResultSetReq() */ 

void Free_DbvDeleteResultSetRsp( DbvDeleteResultSetRsp *psDeleteResultSetRsp )
   {

   /* psReferenceId */

   if( psDeleteResultSetRsp->psReferenceId != NULL)
      FreeDbvOctetString(psDeleteResultSetRsp->psReferenceId);
			 
   /* ppsDeleteListStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsDeleteListStatuses );

   /* pulNumberNotDeleted */

   if ( psDeleteResultSetRsp->pulNumberNotDeleted != NULL )
      Free_Ulong( psDeleteResultSetRsp->pulNumberNotDeleted );

   /* ppsBulkStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsBulkStatuses );

   /* psDeleteMessage */

   if ( psDeleteResultSetRsp->psDeleteMessage != NULL )
      FreeDbvInternationalString( psDeleteResultSetRsp->psDeleteMessage );

   /* ppsOtherInformation */
   if( psDeleteResultSetRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetRsp->ppsOtherInformation );

   DbvFree( psDeleteResultSetRsp );
   } /* End Free_DbvDeleteResultSetRsp() */ 

void Free_DbvDiagRec( DbvDiagRec *psDiagRec )
   {

   if ( psDiagRec == NULL )
      return;

   switch( psDiagRec->eDiagnosticsType )
      {
      case DT_DefaultFormat:
	 Free_DbvDefaultDiagFormat( psDiagRec->uDiagnostics.psDefault );
	 break;

      case DT_ExternallyDefined:
	 FreeDbvASN1External( psDiagRec->uDiagnostics.psExternallyDefined );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDiagRec );
   } /* End Free_DbvDiagRec() */ 

void Free_DbvElementSetNames( DbvElementSetNames *psElementSetNames )
   {

   if ( psElementSetNames == NULL )
      return;

   switch( psElementSetNames->eElementSetType )
      {
      case EST_Generic:
	 FreeDbvInternationalString( psElementSetNames->uElementSet.psGeneric );
	 break;

      case EST_DatabaseSpecific:
	 FreeN_DbvElementSetsDbSpecific( psElementSetNames->
					uElementSet.ppsElementSetsDbSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementSetNames );
   } /* End Free_DbvElementSetNames() */ 

void Free_DbvElementSetsDbSpecific( DbvElementSetsDbSpecific 
						*psElementSetsDbSpecific )
   {

   /* psDatabaseName */

   if ( psElementSetsDbSpecific->psDatabaseName != NULL )
      FreeDbvInternationalString( psElementSetsDbSpecific->psDatabaseName );

   /* psElementSetName */

   if ( psElementSetsDbSpecific->psElementSetName != NULL )
      FreeDbvInternationalString( psElementSetsDbSpecific->psElementSetName );

   DbvFree( psElementSetsDbSpecific );
   } /* End Free_DbvElementSetsDbSpecific() */ 

void Free_DbvEntry( DbvEntry *psEntry )
   {

   if ( psEntry == NULL )
      return;

   switch( psEntry->eEntryType )
      {
      case ET_TermInfo:
	 Free_DbvTermInfo( psEntry->uEntry.psTermInfo );
	 break;

      case ET_SurrogateDiagnostic:
	 Free_DbvDiagRec( psEntry->uEntry.psSurrogateDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psEntry );
   } /* End Free_DbvEntry() */ 

void Free_DbvEsTaskPackage( DbvEsTaskPackage *psEsTaskPackage )
   {

   if ( psEsTaskPackage == NULL )
      return;

   /* pcPackageType */

   FreeDbvObjectIdentifier( psEsTaskPackage->pcPackageType );

   /* psPackageName */

   if ( psEsTaskPackage->psPackageName != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psPackageName );

   /* psUserId */

   if ( psEsTaskPackage->psUserId != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psUserId );

   /* psRetentionTime */

   Free_DbvIntUnit( psEsTaskPackage->psRetentionTime );

   /* ppsPermissions */

   FreeN_DbvPermissions( psEsTaskPackage->ppsPermissions );

   /* psDescription */

   if ( psEsTaskPackage->psDescription != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psDescription );

   /* psTargetReference */

   if( psEsTaskPackage->psTargetReference != NULL)
      FreeDbvOctetString( psEsTaskPackage->psTargetReference );

   /* pcCreationDateTime */

   Free_DbvGeneralizedTime( psEsTaskPackage->pcCreationDateTime );

   /* ppsPackageDiagnostics */

   FreeN_DbvDiagRec( psEsTaskPackage->ppsPackageDiagnostics );

   /* sTaskSpecificParameters */

   Free_compulsory_DbvASN1External( &psEsTaskPackage->sTaskSpecificParameters );

   DbvFree( psEsTaskPackage );
   } /* End Free_DbvEsTaskPackage() */ 


void Free_DbvEstimate1( DbvEstimate1 *psEstimate1 )
   {

   if ( psEstimate1 == NULL )
	 return;

   /* pulCurrencyCode */

   if ( psEstimate1->pulCurrencyCode != NULL )
      Free_Ulong( psEstimate1->pulCurrencyCode );
	   
   DbvFree( psEstimate1 );
   } /* End Free_DbvEstimate1() */ 


void Free_DbvEstimate2( DbvEstimate2 *psEstimate2 )
   {

   if ( psEstimate2 == NULL )
	 return;

   Free_compulsory_DbvStringOrNumeric(&psEstimate2->sType);
   Free_compulsory_DbvIntUnit(&psEstimate2->sValue);

   DbvFree( psEstimate2 );
   } /* End Free_DbvEstimate2() */ 

void Free_DbvFormatSpec( DbvFormatSpec *psFormatSpec )
   {

   if ( psFormatSpec == NULL )
	 return;

   Free_compulsory_DbvInternationalString( &psFormatSpec->sType );

   /* pulSize */

   if ( psFormatSpec->pulSize != NULL )
      Free_Ulong( psFormatSpec->pulSize );
	   
   /* pulBestPosn */

   if ( psFormatSpec->pulBestPosn != NULL )
      Free_Ulong( psFormatSpec->pulBestPosn );
	   
   DbvFree( psFormatSpec );
   } /* End Free_DbvFormatSpec() */ 

void Free_DbvFragmentSyntax( DbvFragmentSyntax *psFragmentSyntax )
   {

   if ( psFragmentSyntax == NULL )
      return;

   switch( psFragmentSyntax->eFragmentSyntaxType )
      {
      case FST_ExternallyTagged:
	 FreeDbvASN1External( psFragmentSyntax->
			      uFragmentSyntax.psExternallyTagged );
	 break;

      case FST_NotExternallyTagged:
	 if ( psFragmentSyntax-> uFragmentSyntax.psNotExternallyTagged != NULL )
	    FreeDbvOctetString( psFragmentSyntax->
			     uFragmentSyntax.psNotExternallyTagged );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psFragmentSyntax );
   } /* End Free_DbvFragmentSyntax() */ 

void Free_DbvGeneralizedTime( DbvGeneralizedTime psGeneralizedTime )
   {

   if ( psGeneralizedTime == NULL )
      return;

   DbvFree( psGeneralizedTime );
   } /* End Free_DbvGeneralizedTime() */ 

void Free_DbvHoldingsRecord( DbvHoldingsRecord *psHoldingsRecord )
   {

   if ( psHoldingsRecord == NULL )
      return;

   switch( psHoldingsRecord->eHoldingsRecType )
      {
      case HRT_MarcHoldingsRecord:
         Free_compulsory_DbvASN1External( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sMarcHoldingsRecord );
	 break;

      case HRT_HoldingsAndCirc:
         Free_compulsory_DbvHoldingsAndCircData( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sHoldingsAndCirc );
	 break;
      default:
	 break;
      } /* End switch */

   DbvFree( psHoldingsRecord );
   } /* End Free_DbvHoldingsRecord() */ 

void Free_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      {
      do_Free_DbvIntUnit( psIntUnit );
      DbvFree( psIntUnit );
      } /* End if */

   } /* End Free_DbvIntUnit() */

void Free_compulsory_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      do_Free_DbvIntUnit( psIntUnit );

   } /* End Free_compulsory_DbvIntUnit() */


void do_Free_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   Free_compulsory_DbvUnit( &psIntUnit->sUnitUsed );
   } /* End Free_DbvIntUnit() */ 

void Free_DbvListEntries( DbvListEntries *psListEntries )
   {

   if ( psListEntries == NULL )
      return;

   FreeN_DbvEntry( psListEntries->ppsEntry );
   FreeN_DbvDiagRec( psListEntries->ppsNonSurrogateDiagnostics );

   DbvFree( psListEntries );
   } /* End Free_DbvListEntries() */ 

void Free_DbvListStatus( DbvListStatus *psListStatus )
   {
   Free_compulsory_DbvInternationalString( &psListStatus->sResultSetId );
   DbvFree( psListStatus );
   } /* End Free_DbvListStatus() */ 

void Free_DbvNamePlusRecord( DbvNamePlusRecord *psNamePlusRecord )
   {

   if ( psNamePlusRecord == NULL )
      return;

   if ( psNamePlusRecord->psDatabaseName != NULL )
      FreeDbvInternationalString( psNamePlusRecord->psDatabaseName );

   switch( psNamePlusRecord->eDbRecordType )
      {
      case DRT_Retrieval:
	 FreeDbvASN1External( psNamePlusRecord->uDbRecord.psRetrievalRecord );
	 break;

      case DRT_SurrogateDiagnostics:
	 Free_DbvDiagRec( psNamePlusRecord->uDbRecord.psSurrogateDiagnostic );
	 break;

      case DRT_StartingFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psStartingFragment );
	 break;

      case DRT_IntermediateFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psIntermediateFragment );
	 break;

      case DRT_FinalFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psFinalFragment);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNamePlusRecord );
   } /* End Free_DbvNamePlusRecord() */ 

void Free_DbvOccurrence( DbvOccurrence *psOccurrence )
   {

   if ( psOccurrence == NULL )
      return;

   switch( psOccurrence->eOccurrenceType )
      {
      case OT_Global:
	 /* No space allocated */
	 break;

      case OT_ByDatabase:
	 FreeN_DbvOccurrenceByDb( psOccurrence->uOccurence.ppsOccurrenceByDb );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psOccurrence );
   } /* End Free_DbvOccurrence() */ 


void Free_DbvOccurrenceByAttributes( DbvOccurrenceByAttributes 
			*psOccurrenceByAttributes )
   {

   if ( psOccurrenceByAttributes == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psOccurrenceByAttributes->ppsAttributeElement );

   /* psOccurrence */

   Free_DbvOccurrence( psOccurrenceByAttributes->psOccurrence );

   /* ppsOtherInformation */
   if( psOccurrenceByAttributes->ppsOtherInformation != NULL)
      FreeN_DbvOtherInformation(psOccurrenceByAttributes->ppsOtherInformation );


   DbvFree( psOccurrenceByAttributes );
   } /* End Free_DbvOccurrenceByAttributes() */ 


void Free_DbvOccurrenceByDb( DbvOccurrenceByDb *psOccurrenceByDb )
   {

   if ( psOccurrenceByDb == NULL )
      return;

   /* sDatabaseName */

   Free_compulsory_DbvInternationalString( &psOccurrenceByDb->sDatabaseName );

   /* pulNumber */

   if ( psOccurrenceByDb->pulNumber ) 
      Free_Ulong( psOccurrenceByDb->pulNumber );

   /* ppsOtherInformation */
   if ( psOccurrenceByDb->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psOccurrenceByDb->ppsOtherInformation );

   DbvFree( psOccurrenceByDb );
   } /* End Free_DbvOccurrenceByDb() */ 


void Free_DbvOpacRecord( DbvOpacRecord *psOpacRecord )
   {

   if ( psOpacRecord == NULL )
      return;

   FreeDbvASN1External( psOpacRecord->psBibliographicRecord );
   FreeN_DbvHoldingsRecord( psOpacRecord->ppsHoldingsData );

   DbvFree( psOpacRecord );
   } /* End Free_DbvOpacRecord() */ 

void Free_DbvPermAllowableFunctions( DbvPermAllowableFunctions 
					*psPermAllowableFunctions )
   {

   if ( psPermAllowableFunctions == NULL )
      return;

   DbvFree( psPermAllowableFunctions );
   } /* End Free_DbvPermAllowableFunctions() */ 

void Free_DbvPermissions( DbvPermissions *psPermissions )
   {

   if ( psPermissions == NULL )
      return;

   Free_compulsory_DbvInternationalString( &psPermissions->sUserId );
   FreeN_DbvPermAllowableFunctions( psPermissions->ppeAllowableFunctions );

   DbvFree( psPermissions );
   } /* End Free_DbvPermissions() */ 

void Free_DbvPresentReq( DbvPresentReq *psPresentReq )
   {
   /* psReferenceId */
   if( psPresentReq->psReferenceId != NULL)
      FreeDbvOctetString(psPresentReq->psReferenceId);

   /* sResultSetId */

   Free_compulsory_DbvInternationalString( &psPresentReq->sResultSetId );

   /* ppsAdditionalRanges */

   FreeN_DbvRange( psPresentReq->ppsAdditionalRanges );

   /* DbvRecordComposition */

   if( psPresentReq->psRecordComposition != NULL)
      Free_DbvRecordComposition( psPresentReq->psRecordComposition );

   /* pcPreferredRecordSyntax */

   if( psPresentReq->pcPreferredRecordSyntax != NULL)
      FreeDbvObjectIdentifier( psPresentReq->pcPreferredRecordSyntax );

   /* pulMaxSegmentCount */

   if( psPresentReq->pulMaxSegmentCount != NULL)
      Free_Ulong( psPresentReq->pulMaxSegmentCount );

   /* pulMaxRecordSize */

   if( psPresentReq->pulMaxRecordSize!= NULL)
      Free_Ulong( psPresentReq->pulMaxRecordSize );

   /* pulMaxSegmentSize */

   if( psPresentReq->pulMaxSegmentSize != NULL)
      Free_Ulong( psPresentReq->pulMaxSegmentSize );

   /* ppsOtherInformation */

   if( psPresentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentReq->ppsOtherInformation );

   DbvFree( psPresentReq );
   } /* End Free_DbvPresentReq() */ 

void Free_DbvPresentRsp( DbvPresentRsp *psPresentRsp )
   {
   /* psReferenceId */
   if( psPresentRsp->psReferenceId != NULL)
      FreeDbvOctetString( psPresentRsp->psReferenceId );

   /* psRecords */

   Free_DbvRecords( psPresentRsp->psRecords );

   /* ppsOtherInformation */

   if( psPresentRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentRsp->ppsOtherInformation );

   DbvFree( psPresentRsp );
   } /* End Free_DbvPresentRsp() */ 

void Free_DbvPresentStatus( DbvPresentStatus *pePresentStatus )
   {

   if ( pePresentStatus == NULL )
      return;

   DbvFree( pePresentStatus );
   } /* End Free_DbvPresentStatus() */ 

void Free_DbvQuery( DbvQuery *psQuery )
   {

   if ( psQuery == NULL )
      return;

   switch( psQuery->eQueryType )
      {
      case QT_Private:
	 if ( psQuery->uQuery.psPrivateQuery )
	    FreeDbvOctetString( psQuery->uQuery.psPrivateQuery);
	 break;

      case QT_Iso8777:
	 if ( psQuery->uQuery.psIso8777Query )
	    FreeDbvOctetString( psQuery->uQuery.psIso8777Query);
	 break;

      case QT_Z3958:
	 if ( psQuery->uQuery.psZ3958Query )
	    FreeDbvOctetString( psQuery->uQuery.psZ3958Query);
	 break;

      case QT_NaturalLanguage:
	 if ( psQuery->uQuery.psNaturalLanguageQuery )
	    FreeDbvOctetString( psQuery->uQuery.psNaturalLanguageQuery);
	 break;

      case QT_Erpn:
         if (psQuery->uQuery.psErpnQuery != NULL)
                Free_DbvRPNQuery(psQuery->uQuery.psErpnQuery);
         break;

      case QT_Rpn:
         if (psQuery->uQuery.psRpnQuery != NULL)
                Free_DbvRPNQuery(psQuery->uQuery.psRpnQuery);
 
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQuery );
   } /* End Free_DbvQuery() */ 

void Free_DbvRange( DbvRange *psRange )
   {
   DbvFree( psRange );
   } /* End Free_DbvRange() */ 

void Free_DbvRecords( DbvRecords *psRecords )
   {

   if ( psRecords == NULL )
      return;

   switch( psRecords->eRecordType )
      {
      case RT_Response:
	 FreeN_DbvNamePlusRecord( psRecords->uRecord.ppsResponseRecord );
	 break;

      case RT_NonSurrDiagnostics:
	 Free_DbvDefaultDiagFormat( psRecords->uRecord.psNonSurrogateDiag );
	 break;

      case RT_MultipleNonSurrDiagnostics:
	 FreeN_DbvDiagRec( psRecords->uRecord.ppsMultipleNonSurrogateDiag );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psRecords );
   } /* End Free_DbvRecord() */ 

void Free_DbvResourceReport1( DbvResourceReport1 *psResourceReport1 )
   {

   if ( psResourceReport1 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate1( psResourceReport1->ppsEstimate );

   /* sMessage */

   Free_compulsory_DbvInternationalString( &psResourceReport1->sMessage );

   DbvFree( psResourceReport1 );
   } /* End Free_DbvResourceReport1() */ 

void Free_DbvResourceReport2( DbvResourceReport2 *psResourceReport2 )
   {

   if ( psResourceReport2 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate2( psResourceReport2->ppsEstimate );

   /* psMessage */

   if ( psResourceReport2->psMessage != NULL )
      FreeDbvInternationalString( psResourceReport2->psMessage );

   DbvFree( psResourceReport2 );
   } /* End Free_DbvResourceReport2() */ 

void Free_DbvResourceReportReq( DbvResourceReportReq *psResourceReportReq )
   {

   /* psReferenceId */

   if( psResourceReportReq->psReferenceId != NULL)
      FreeDbvOctetString(psResourceReportReq->psReferenceId);

   /* psOperationId */

   if( psResourceReportReq->psOperationId != NULL)
      FreeDbvOctetString(psResourceReportReq->psOperationId );

   /* pcPrefResourceReportFormat */

   if( psResourceReportReq->pcPrefResourceReportFormat != NULL)
      FreeDbvObjectIdentifier(psResourceReportReq->pcPrefResourceReportFormat );

   /* ppsOtherInformation */

   if( psResourceReportReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportReq->ppsOtherInformation );

   DbvFree( psResourceReportReq );
   } /* End Free_DbvResourceReportReq() */ 

void Free_DbvResourceReportRsp( DbvResourceReportRsp *psResourceReportRsp )
   {

   /* psReferenceId */

   if( psResourceReportRsp->psReferenceId != NULL)
      FreeDbvOctetString(psResourceReportRsp->psReferenceId );

   /* psResourceReport*/

   FreeDbvASN1External( psResourceReportRsp->psResourceReport );

   /* ppsOtherInformation */

   if( psResourceReportRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportRsp->ppsOtherInformation );

   DbvFree( psResourceReportRsp );
   } /* End Free_DbvResourceReportRsp() */ 

void Free_DbvResultSetStatus( DbvResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End Free_DbvResultSetStatus() */ 

void Free_DbvScanReq( DbvScanReq *psScanReq )
   {
   /* psReferenceId */
   if( psScanReq->psReferenceId != NULL)
      FreeDbvOctetString(psScanReq->psReferenceId);
			 
   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psScanReq->ppsDatabaseNames );

   /* pcAttributeSetId  */

   if( psScanReq->pcAttributeSetId != NULL)
      FreeDbvObjectIdentifier( psScanReq->pcAttributeSetId );

   /* pulStepSize */

   if( psScanReq->pulStepSize != NULL)
      Free_Ulong( psScanReq->pulStepSize );

   /* pulPreferredPositionInResponse */

   if( psScanReq->pulPreferredPositionInResponse != NULL)
      Free_Ulong( psScanReq->pulPreferredPositionInResponse );

   /* ppsOtherInformation */

   if( psScanReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanReq->ppsOtherInformation );

   DbvFree( psScanReq );
   } /* End Free_DbvScanReq() */ 

void Free_DbvScanRsp( DbvScanRsp *psScanRsp )
   {

   /* psReferenceId */

   if( psScanRsp->psReferenceId != NULL )
      FreeDbvOctetString(psScanRsp->psReferenceId );

   /* pulStepSize */

   if ( psScanRsp->pulStepSize )
      Free_Ulong( psScanRsp->pulStepSize );

   /* pulPositionOfTerm */

   if ( psScanRsp->pulPositionOfTerm )
      Free_Ulong( psScanRsp->pulPositionOfTerm );

   /* psEntries */

   Free_DbvListEntries( psScanRsp->psEntries );

   /* pcAttributeSetId */

   if( psScanRsp->pcAttributeSetId != NULL)
      FreeDbvObjectIdentifier( psScanRsp->pcAttributeSetId );

   /* ppsOtherInformation */

   if( psScanRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanRsp->ppsOtherInformation );

   DbvFree( psScanRsp );
   } /* End Free_DbvScanRsp() */ 

void Free_DbvSearchReq( DbvSearchReq *psSearchReq )
   {

   /* psReferenceId */

   if( psSearchReq->psReferenceId != NULL)
      FreeDbvOctetString(psSearchReq->psReferenceId);

   /* sResultSetName */

   Free_compulsory_DbvInternationalString( &psSearchReq->sResultSetName );

   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psSearchReq->ppsDatabaseNames );

   /* psSmallSetElementNames */

   Free_DbvElementSetNames( psSearchReq->psSmallSetElementNames );

   /* psMediumSetElementNames */

   Free_DbvElementSetNames( psSearchReq->psMediumSetElementNames );

   /* pcPreferredRecordSyntax */

   if( psSearchReq->pcPreferredRecordSyntax != NULL)
      FreeDbvObjectIdentifier( psSearchReq->pcPreferredRecordSyntax );

   /* psQuery */

   Free_DbvQuery( psSearchReq->psQuery );

   /* psAdditionalSearchInfo */

   if( psSearchReq->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsOtherInformation );

   DbvFree( psSearchReq );
   } /* End Free_DbvSearchReq() */ 

void Free_DbvSearchRsp( DbvSearchRsp *psSearchRsp )
   {

   /* psReferenceId */

   if( psSearchRsp->psReferenceId != NULL )
      FreeDbvOctetString(psSearchRsp->psReferenceId );

   /* peResultSetStatus */

   if ( psSearchRsp->peResultSetStatus )
      Free_DbvResultSetStatus( psSearchRsp->peResultSetStatus );

   /* pePresentStatus */

   if ( psSearchRsp->pePresentStatus )
      Free_DbvPresentStatus( psSearchRsp->pePresentStatus );

   /* psRecords */

   Free_DbvRecords( psSearchRsp->psRecords );

   /* psAdditionalSearchInfo */

   if( psSearchRsp->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsOtherInformation );

   DbvFree( psSearchRsp );
   } /* End Free_DbvSearchRsp() */ 

void Free_DbvSegmentReq( DbvSegmentReq *psSegmentReq )
   {

   /* psReferenceId */

   if( psSegmentReq->psReferenceId != NULL)
      FreeDbvOctetString(psSegmentReq->psReferenceId);

   /* ppsSegmentRecords */

   FreeN_DbvNamePlusRecord( psSegmentReq->ppsSegmentRecords );

   /* ppsOtherInformation */

   if( psSegmentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSegmentReq->ppsOtherInformation );

   DbvFree( psSegmentReq );
   } /* End Free_DbvSegmentReq() */ 

void Free_DbvSingleASN1Type( DbvSingleASN1Type *psSingleASN1Type )
   {

   if ( psSingleASN1Type == NULL )
      return;

   switch( psSingleASN1Type->eASN1Type )
      {
      case AT_UnstructuredText:
	 if ( psSingleASN1Type-> uASN1.psUnstructuredText != NULL )
	    FreeDbvInternationalString( psSingleASN1Type->
				 uASN1.psUnstructuredText );
	 break;

      case AT_OpacRecord:
	 Free_DbvOpacRecord( psSingleASN1Type->uASN1.psOpacRecord );
	 break;

      case AT_SummaryRecord:
	 Free_DbvSummaryRecord( psSingleASN1Type->uASN1.psSummaryRecord );
	 break;

      case AT_ResourceReport1:
	 Free_DbvResourceReport1( psSingleASN1Type->uASN1.psResourceReport1 );
	 break;

      case AT_ResourceReport2:
	 Free_DbvResourceReport2( psSingleASN1Type->uASN1.psResourceReport2 );
	 break;

      case AT_EsTaskPackage:
	 Free_DbvEsTaskPackage( psSingleASN1Type->uASN1.psEsTaskPackage );
	 break;

      case AT_ItemOrderExtendedService:
	 Free_DbvItemOrderExtendedService( psSingleASN1Type->
					   uASN1.psItemOrderExtendedService );
	 break;

      case AT_UpdateExtendedService:
	 Free_DbvUpdateExtendedService( psSingleASN1Type->
					uASN1.psUpdateExtendedService );
	 break;

      case AT_IllPdu:
	 Free_DbvIllPdu( psSingleASN1Type->uASN1.psIllPdu );
	 break;

	
      case AT_ExplainRecord:
	 Free_DbvExplainRecord( psSingleASN1Type->uASN1.psExplainRecord );
         break;



	#ifdef NOT_YET_IMPLEMENTED
      case AT_GenericRecord: 
	 Free_DbvGenericRecord( psSingleASN1Type->uASN1.psGenericRecord );
	 break;

	case AT_AccessControlDes1:
	case AT_AccessControlKrb1:
	case AT_PerResSetExtendedService:
	case AT_PerQueryExtendedService:
	case AT_PerQuerySchExtendedService:
	case AT_ExpSpecExtendedService:
	case AT_ExpInvoExtendedService:
	case AT_UserInfoSearchResult:
	case AT_ElementSpec1:
		break;
	#endif

      case AT_PromptObject:
         Free_DbvPromptObject( psSingleASN1Type->uASN1.psPromptObject );
         break;

      case AT_CharSetAndLanguage:
	 if ( psSingleASN1Type->uASN1.psCharSetAndLanguage != NULL )
	    Free_DbvCharSetAndLanguage( psSingleASN1Type->uASN1.psCharSetAndLanguage );
         break;

      case AT_UnknownType:
	 if ( psSingleASN1Type->uASN1.psUnknownType != NULL )
	    FreeDbvOctetString( psSingleASN1Type->uASN1.psUnknownType );
         break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSingleASN1Type );
   } /* End Free_DbvSingleASN1Type() */ 

void Free_DbvSortKeySpec( DbvSortKeySpec *psSortKeySpec )
   {

   if ( psSortKeySpec->psMVActionType != NULL )
      Free_DbvSortMVAction( psSortKeySpec->psMVActionType );

   DbvFree( psSortKeySpec );
   } /* End Free_DbvSortKeySpec() */ 

void Free_DbvSortMVAction( DbvSortMVAction *psSortMVAction )
   {

   if ( psSortMVAction->psOctetString!= NULL )
      FreeDbvOctetString( psSortMVAction->psOctetString );

   DbvFree( psSortMVAction );
   } /* End Free_DbvSortMVAction() */ 

void Free_DbvSortReq( DbvSortReq *psSortReq )
   {
   /* psReferenceId */
   if( psSortReq->psReferenceId != NULL)
      FreeDbvOctetString(psSortReq->psReferenceId);
			 
   /* ppsInputResultSetName */

   FreeN_DbvInternationalString( psSortReq->ppsInputResultSetName );

   /* sSortedResultSetName */

   Free_compulsory_DbvInternationalString( &psSortReq->sSortedResultSetName );

   /* ppsSortSequence */

   FreeN_DbvSortKeySpec( psSortReq->ppsSortSequence );

   /* ppsOtherInformation */

   if( psSortReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortReq->ppsOtherInformation );

   DbvFree( psSortReq );
   } /* End Free_DbvSortReq() */ 

void Free_DbvSortResultSetStatus( DbvSortResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End Free_DbvSortResultSetStatus() */ 

void Free_DbvSortRsp( DbvSortRsp *psSortRsp )
   {

   /* psReferenceId */

   if( psSortRsp->psReferenceId != NULL )
      FreeDbvOctetString(psSortRsp->psReferenceId );

   /* peResultSetStatus  */

   Free_DbvSortResultSetStatus( psSortRsp->peResultSetStatus );

   /* ppsDiagnostics */

   FreeN_DbvDiagRec( psSortRsp->ppsDiagnostics );

   /* ppsOtherInformation */

   if( psSortRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortRsp->ppsOtherInformation );

   DbvFree( psSortRsp );
   } /* End Free_DbvSortRsp() */ 

void Free_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      {
      do_Free_DbvStringOrNumeric( psStringOrNumeric );
      DbvFree( psStringOrNumeric );
      } /* End if */

   } /* End FreeDbvStringOrNumeric() */

void Free_compulsory_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      do_Free_DbvStringOrNumeric( psStringOrNumeric );

   } /* End Free_compulsory_DbvStringOrNumeric() */


void do_Free_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {


   switch( psStringOrNumeric->eStringOrNumeric )
      {
      case SNT_String:
	 FreeDbvInternationalString( psStringOrNumeric->
				     uStringOrNumeric.psString);
	 break;

      case SNT_Numeric:
      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvStringOrNumeric() */ 

void Free_DbvSummaryRecord( DbvSummaryRecord *psSummaryRecord )
   {

   if ( psSummaryRecord == NULL )
      return;

   /* sTitle */

   Free_compulsory_DbvInternationalString( &psSummaryRecord->sTitle );

   /* psAuthor */

   if ( psSummaryRecord->psAuthor != NULL )
      FreeDbvInternationalString( psSummaryRecord->psAuthor );

   /* psCallNumber */

   if ( psSummaryRecord->psCallNumber != NULL )
      FreeDbvInternationalString( psSummaryRecord->psCallNumber );

   /* psRecordType */

   if ( psSummaryRecord->psRecordType != NULL )
      FreeDbvInternationalString( psSummaryRecord->psRecordType );

   /* psBibliographicLevel */

   if ( psSummaryRecord->psBibliographicLevel != NULL )
      FreeDbvInternationalString( psSummaryRecord->psBibliographicLevel );

   /* ppsFormat */

   FreeN_DbvFormatSpec( psSummaryRecord->ppsFormat );

   /* psPublicationPlace */

   if ( psSummaryRecord->psPublicationPlace != NULL )
      FreeDbvInternationalString( psSummaryRecord->psPublicationPlace );

   /* psPublicationDate */

   if ( psSummaryRecord->psPublicationDate != NULL )
      FreeDbvInternationalString( psSummaryRecord->psPublicationDate );

   /* psTargetSystemKey */

   if ( psSummaryRecord->psTargetSystemKey != NULL )
      FreeDbvInternationalString( psSummaryRecord->psTargetSystemKey );

   /* psSatisfyingElement */

   if ( psSummaryRecord->psSatisfyingElement != NULL )
      FreeDbvInternationalString( psSummaryRecord->psSatisfyingElement );

   /* pulRank */

   if ( psSummaryRecord->pulRank )
      Free_Ulong( psSummaryRecord->pulRank );

   /* psDocumentId */

   if ( psSummaryRecord->psDocumentId != NULL )
      FreeDbvInternationalString( psSummaryRecord->psDocumentId );

   /* psAbstract */

   if ( psSummaryRecord->psAbstract != NULL )
      FreeDbvInternationalString( psSummaryRecord->psAbstract );

   /* ppsOtherInformation */

   if ( psSummaryRecord->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psSummaryRecord->ppsOtherInformation );

   DbvFree( psSummaryRecord );
   } /* End Free_DbvSummaryRecord() */ 

void Free_DbvSuppliedRecord( DbvSuppliedRecord *psSuppliedRecord )
   {

   if ( psSuppliedRecord == NULL )
      return;

   /* psUpdtRecordId  */

   Free_DbvUpdtRecordId( psSuppliedRecord->psUpdtRecordId ); 

   /* psUpdtSupplementalId */

   Free_DbvUpdtSupplementalId( psSuppliedRecord->psUpdtSupplementalId  );

   /* psCorrelationInfo */

   Free_DbvCorrelationInfo( psSuppliedRecord->psCorrelationInfo  );

   /* sRecord */

   Free_compulsory_DbvASN1External( &psSuppliedRecord->sRecord );

   DbvFree( psSuppliedRecord );

   } /* End Free_compulsory_DbvSuppliedRecord() */ 

void Free_DbvTaskPackageRecord( DbvTaskPackageRecord *psTaskPackageRecord )
   {

   if ( psTaskPackageRecord == NULL )
      return;

   /* psUpdtRecordOrDiag */

   Free_DbvUpdtRecordOrDiag( psTaskPackageRecord->psUpdtRecordOrDiag );

   /* psCorrelationInfo */

   Free_DbvCorrelationInfo( psTaskPackageRecord->psCorrelationInfo );

   DbvFree( psTaskPackageRecord );
   } /* End Free_DbvTaskPackageRecord() */ 

void Free_DbvTermInfo( DbvTermInfo *psTermInfo )
   {

   if ( psTermInfo == NULL )
      return;

   /* sTerm */

   Free_compulsory_DbvTerm( &psTermInfo->sTerm  );

   /* psDisplayTerm */
   FreeDbvInternationalString( psTermInfo->psDisplayTerm );

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psTermInfo->ppsSuggestedAttribute );

   /* ppsAlternativeTerm */

   FreeN_DbvAttributesPlusTerm( psTermInfo->ppsAlternativeTerm );

   /* pulGlobalOccurrences */

   if ( psTermInfo->pulGlobalOccurrences != NULL )
      Free_Ulong( psTermInfo->pulGlobalOccurrences );

   /* psByAttributes */

   FreeN_DbvOccurrenceByAttributes( psTermInfo->ppsByAttributes );

   /* ppsOtherInformation */

   if ( psTermInfo->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psTermInfo->ppsOtherInformation );

   DbvFree( psTermInfo );
   } /* End Free_DbvTermInfo() */ 

void Free_DbvUpdateExtendedService( DbvUpdateExtendedService 
						*psUpdateExtendedService )
   {

   if ( psUpdateExtendedService == NULL )
      return;

   switch( psUpdateExtendedService->eUpdateReqRsp )
      {
      case UROR_Request:
	 Free_DbvUpdateRequestPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateRequest );
	 break;

      case UROR_Response:
	 Free_DbvUpdateTaskPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateResponse );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psUpdateExtendedService );
   } /* End Free_DbvUpdateExtendedService() */

void Free_DbvUpdateRequestPackage( DbvUpdateRequestPackage 
						*psUpdateRequestPackage )
   {

   if ( psUpdateRequestPackage == NULL )
      return;

   Free_compulsory_DbvOriginPartToKeep( &psUpdateRequestPackage->
							sOriginPartToKeep );
   FreeN_DbvSuppliedRecord( psUpdateRequestPackage->ppsSuppliedRecord );

   DbvFree( psUpdateRequestPackage );
   } /* End Free_DbvUpdateRequestPackage() */ 

void Free_DbvUpdateTaskPackage( DbvUpdateTaskPackage 
						*psUpdateTaskPackage )
   {

   if ( psUpdateTaskPackage == NULL )
      return;

   Free_compulsory_DbvOriginPartToKeep( &psUpdateTaskPackage->
							sOriginPartToKeep );
   Free_compulsory_DbvUpdateTargetPart( &psUpdateTaskPackage->sTargetPart );

   DbvFree( psUpdateTaskPackage );
   } /* End Free_DbvUpdateTaskPackage() */ 

void Free_DbvUpdtRecordId( DbvUpdtRecordId *psUpdtRecordId )
   {

   if ( psUpdtRecordId == NULL )
      return;

   switch( psUpdtRecordId->eRecordIdType )
      {
      case URIT_Number:
	 /* No Space allocated */
	 break;

      case URIT_String:
	 if ( psUpdtRecordId->uUpdtRecordId.psString != NULL )
	    FreeDbvInternationalString( psUpdtRecordId->
					uUpdtRecordId.psString );
	 break;

      case URIT_Opaque:
	 if ( psUpdtRecordId->uUpdtRecordId.psOpaque != NULL )
	    FreeDbvOctetString( psUpdtRecordId-> uUpdtRecordId.psOpaque);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordId );
   } /* End Free_DbvUpdtRecordId() */ 

void Free_DbvUpdtRecordOrDiag( DbvUpdtRecordOrDiag *psUpdtRecordOrDiag )
   {

   if ( psUpdtRecordOrDiag == NULL )
      return;

   switch( psUpdtRecordOrDiag->eUpdtRecordOrDiag )
      {
      case URDT_Record:
	 FreeDbvASN1External( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord );
	 break;

      case URDT_Diagnostic:
	 Free_DbvDiagRec( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordOrDiag );
   } /* End Free_DbvUpdtRecordOrDiag() */ 

void Free_DbvUpdtSupplementalId( DbvUpdtSupplementalId *psUpdtSupplementalId )
   {

   if ( psUpdtSupplementalId == NULL )
      return;

   switch( psUpdtSupplementalId->eSupplementalIdType )
      {
      case USIT_Time:
	 Free_DbvGeneralizedTime( psUpdtSupplementalId->
					uUpdtSupplementalId.pcTime );
	 break;

      case USIT_Version:
	 if ( psUpdtSupplementalId->uUpdtSupplementalId.psVersion != NULL )
	    FreeDbvInternationalString( psUpdtSupplementalId->
					uUpdtSupplementalId.psVersion );
	 break;

      case USIT_PreviousVersion:
	 FreeDbvASN1External( psUpdtSupplementalId->
					uUpdtSupplementalId.psPreviousVersion );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtSupplementalId );
   } /* End Free_DbvUpdtSupplementalId() */ 

void Free_DbvVisibleString( DbvVisibleString psVisibleString )
   {

   if ( psVisibleString == NULL )
      return;

   DbvFree( psVisibleString );
   } /* End Free_DbvVisibleString() */ 

void Free_DbvVolume( DbvVolume *psVolume )
   {

   if ( psVolume == NULL )
	 return;

   /* psEnumeration */

   if ( psVolume->psEnumeration != NULL )
      FreeDbvInternationalString( psVolume->psEnumeration );

   /* psChronology */

   if ( psVolume->psChronology != NULL )
      FreeDbvInternationalString( psVolume->psChronology );

   /* psEnumAndChron */

   if ( psVolume->psEnumAndChron != NULL )
      FreeDbvInternationalString( psVolume->psEnumAndChron );

   DbvFree( psVolume );

   } /* End Free_DbvVolume() */ 

void Free_Ulong( Ulong *pulong )
   {
   DbvFree( pulong );
   } /* End Free_Ulong() */ 


void FreeDbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      {
      do_FreeDbvASN1External( psASN1External );
      DbvFree( psASN1External );
      } /* End if */

   } /* End FreeDbvASN1External() */

void Free_compulsory_DbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      do_FreeDbvASN1External( psASN1External );

   } /* End Free_compulsory_DbvASN1External() */


void do_FreeDbvASN1External( DbvASN1External *psASN1External )
   {


   if ( psASN1External->pcObjectIdentifier != NULL )
      FreeDbvObjectIdentifier( psASN1External->pcObjectIdentifier );

   switch( psASN1External->eValueType )
      {
      case EVT_SingleASN1Type:
	 Free_DbvSingleASN1Type( psASN1External->
				 uExternalValue.psSingleASN1Type );
	 break;

      case EVT_OctetAligned:
	 FreeDbvOctetString( psASN1External->
			     uExternalValue.psOctetAligned );
	 break;

      case EVT_Arbitrary:
	 FreeDbvOctetString( psASN1External->
			     uExternalValue.psArbitrary );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End FreeDbvASN1External() */ 

void Free_DbvRecordComposition(DbvRecordComposition *psRecordComposition)
   {
   printf("Free_DbvRecordComposition has not been done yet!\n");

   if(psRecordComposition != NULL)
   {
	switch(psRecordComposition->eRecordCompositionType)
	{
	case RCT_ElementSetNames:
		Free_DbvElementSetNames(psRecordComposition->uRecordComposition.
			psElementSetNames);
		break;
	case RCT_CompSpec:
		break;
	default:
		Free_DbvCompSpec(psRecordComposition->uRecordComposition.
			psCompSpec);
		break;
	} /* End switch */
	DbvFree(psRecordComposition);
   } /* End if */

   } /* End Free_DbvRecordComposition */

void Free_DbvCompSpec(DbvCompSpec *psCompSpec)
   {
   if(psCompSpec != NULL)
	{
	Free_DbvSpecification(psCompSpec->psGeneric);
	FreeN_DbvDbSpecificComp(psCompSpec->ppsDbSpecific);
	FreeN_DbvObjectIdentifier(psCompSpec->ppcRecordSyntax);
	DbvFree(psCompSpec);
	}
   } /* End Free_DbvCompSpec */

void Free_DbvSpecification(DbvSpecification *psSpecification)
{
	if(psSpecification != NULL)
	{
		Free_compulsory_DbvSpecification(psSpecification);
		DbvFree(psSpecification);
	}
} /* End Free_DbvSpecification */

void Free_DbvDbSpecificComp(DbvDbSpecificComp *psDbSpecific)
{
	if(psDbSpecific != NULL)
	{
		Free_compulsory_DbvInternationalString(&psDbSpecific->
			sDatabaseName);
		Free_compulsory_DbvSpecification(&psDbSpecific->
			sSpecification);
	} /* End if */
} /* End Free_DbvDbSpecificComp */

void Free_DbvRPNQuery(DbvRPNQuery *psDbvRpnQuery)
{
	if(psDbvRpnQuery->pcAttributeSetId != NULL)
		DbvFree(psDbvRpnQuery->pcAttributeSetId);

        Free_rpn_structure(&psDbvRpnQuery->sRpnStructure);
        DbvFree(psDbvRpnQuery);
}

void Free_rpn_structure(DbvRPNStructure *psDbvRPNStructure)
{
 
        switch (psDbvRPNStructure->eRpnStructureType) {
        case RST_RpnLopRopOperator:
 
                if (psDbvRPNStructure->uRPNStructure.psRpnLopRopOperator != NULL)
                        Free_rpnlopropoperator(psDbvRPNStructure->uRPNStructure.
				psRpnLopRopOperator);
                break;
        case RST_RpnOperand:
                if (psDbvRPNStructure->uRPNStructure.psRpnOperand != NULL)
 
                        Free_rpnoperand(psDbvRPNStructure->uRPNStructure.
				psRpnOperand);
                break;
        }
 
}

void Free_rpnlopropoperator(DbvRPNLopRopOperator *psDbvRPNLopRopOperator)
{
 
        if (psDbvRPNLopRopOperator->psLeftOperand != NULL)
                Free_rpn_structure(psDbvRPNLopRopOperator->psLeftOperand);
 
        if (psDbvRPNLopRopOperator->psRightOperand != NULL)
                Free_rpn_structure(psDbvRPNLopRopOperator->psRightOperand);
 
	Free_DbvOperator(&psDbvRPNLopRopOperator->sOperator);
 
        DbvFree(psDbvRPNLopRopOperator);
}

void Free_rpnoperand(DbvOperand *psRpnOperand)
{
 
        switch (psRpnOperand->eOperandType) {
        case OT_AttributePlusTerm:
 
                if (psRpnOperand->uOperand.psAttributesPlusTerm != NULL)
                        Free_attsplusterm(psRpnOperand->uOperand.
				psAttributesPlusTerm);
                break;
        }
 
        DbvFree(psRpnOperand);
}

void Free_attsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm)
{
        int index = 0;
 
 
        /* Print all attributes */
 
        while (psAttributesPlusTerm->ppsAttributeElement[index] != NULL) {
                Free_att_element(psAttributesPlusTerm->
			ppsAttributeElement[index]);
                index++;
        }
 
        /* then the term */
        Free_att_term(psAttributesPlusTerm->sTerm);
 
 
        DbvFree(psAttributesPlusTerm);
}

void Free_att_element(DbvAttributeElement *psAttributeElement)
{
        if (psAttributeElement->pcAttributeSetId != NULL)
                DbvFree(psAttributeElement->pcAttributeSetId);
 
        DbvFree(psAttributeElement);
}
 
void Free_att_term(DbvTerm sTerm)
{
        /* Only general catered for !! */
        switch (sTerm.eTermType) {
        case TT_General:
 
                if (sTerm.uTerm.psGeneral != NULL)
                        DbvFree(sTerm.uTerm.psGeneral);
                break;
        }
 
}

void Free_DbvOperator(DbvOperator *psOperator)
{
	
	/* Operator is not dynamically allocated */
	if(psOperator->psProximityOperator != NULL)
	{
		if(psOperator->psProximityOperator->pfExclusion != NULL)
			DbvFree(psOperator->psProximityOperator->pfExclusion);
		DbvFree(psOperator->psProximityOperator);
	}
}

void Free_DbvExtendedServicesReq(DbvExtendedServicesReq *psExtendedServicesReq)
{
   /* psReferenceId */

   if( psExtendedServicesReq->psReferenceId != NULL)
      FreeDbvOctetString(psExtendedServicesReq->psReferenceId);

   /* pcPackageType */
   FreeDbvObjectIdentifier( psExtendedServicesReq->pcPackageType);

   /* psPackageName */
   FreeDbvInternationalString(psExtendedServicesReq->psPackageName);

   /* psUserId */
   FreeDbvInternationalString(psExtendedServicesReq->psUserId);

   /* psRetentionTime */
   Free_DbvIntUnit(psExtendedServicesReq->psRetentionTime);

   /* ppsPermissions */
   FreeN_DbvPermissions(psExtendedServicesReq->ppsPermissions);

   /* psDescription */
   FreeDbvInternationalString(psExtendedServicesReq->psDescription);

   /* psOriginSpecificParameter */
   FreeDbvASN1External(psExtendedServicesReq->psOriginSpecificParameter);

   /* psElements */
   FreeDbvInternationalString(psExtendedServicesReq->psElements);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesReq->ppsOtherInformation);

   DbvFree(psExtendedServicesReq);
} /* End Free_DbvExtendedServicesRsp */

void Free_DbvExtendedServicesRsp(DbvExtendedServicesRsp *psExtendedServicesRsp)
{
   /* psReferenceId */
 
   FreeDbvOctetString(psExtendedServicesRsp->psReferenceId);
 
   /* ppsDiagnostics */
   FreeN_DbvDiagRec(psExtendedServicesRsp->ppsDiagnostics);

   /* psTaskPackage */
   FreeDbvASN1External(psExtendedServicesRsp->psTaskPackage);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesRsp->ppsOtherInformation);

} /* End Free_DbvExtendedServicesRsp */

void Free_DbvAssociateResult(DbvAssociateResult *puAssociateResult,
        DbvAssociateState eAssociateState)
{
        int place = 0;
        if(eAssociateState == Associate_Null || eAssociateState ==
          Associate_Pending)
                return;
 
        if(eAssociateState == Associate_Accepted)
        {
                if(puAssociateResult->psProtocolAndSyntaxes->
                  peApplicationProtocol != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peApplicationProtocol);
                if(puAssociateResult->psProtocolAndSyntaxes->peAbstractSyntax 
                  != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peAbstractSyntax);
                if(puAssociateResult->psProtocolAndSyntaxes->
			ppcCommonSyntaxes != NULL)
                {
                while(puAssociateResult->psProtocolAndSyntaxes->
                  ppcCommonSyntaxes[place] != NULL)
                {
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          ppcCommonSyntaxes[place]);
                        place++;
                }
		}
                DbvFree(puAssociateResult->psProtocolAndSyntaxes);
        }
        else
        {       
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectSource != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectSource);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectResult != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectResult);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectReason != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectReason);
                DbvFree(puAssociateResult->psAssociateReject);

        }
        DbvFree(puAssociateResult);
}

/* 12-2-96 The following function where added by Dan Maskell  */

void Free_DbvGeneralString( DbvGeneralString *psGeneralString )
   {
   if( psGeneralString == NULL)
      return;

   if(psGeneralString->pcValue != NULL )
      DbvFree( psGeneralString->pcValue );

   DbvFree( psGeneralString );
   } /* End Free_DbvGeneralString() */
 
 
void Free_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      {
      do_Free_DbvEsIoTargetPart( psEsIoTargetPart );
      DbvFree( psEsIoTargetPart );
      } /* End if */

   } /* End Free_DbvEsIoTargetPart() */ 

void do_Free_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   /* psItemRequest */
   FreeDbvASN1External( psEsIoTargetPart->psItemRequest );

   /* psStatusOrErrorReport */
   FreeDbvASN1External( psEsIoTargetPart->psStatusOrErrorReport );

   /* peAuxiliaryStatus */
   Free_DbvEsIoAuxiliaryStatus( psEsIoTargetPart->peAuxiliaryStatus );

   } /* End do_Free_DbvEsIoTargetPart() */ 

void Free_DbvEsIoResultSetItem( DbvEsIoResultSetItem *psEsIoResultSetItem )
   {

   if ( psEsIoResultSetItem == NULL )
      return;

   /* sResultSetId */
   Free_compulsory_DbvInternationalString( &psEsIoResultSetItem->sResultSetId);

   DbvFree( psEsIoResultSetItem );
   } /* End Free_DbvEsIoResultSetItem() */ 

void Free_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      {
      do_Free_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );
      DbvFree( psEsIoOriginPartNotToKeep );
      } /* End if */

   } /* End Free_DbvEsIoOriginPartNotToKeep() */ 

void Free_DbvEsIoCreditCardInfo( DbvEsIoCreditCardInfo *psEsIoCreditCardInfo)
   {
   if( psEsIoCreditCardInfo == NULL)
      return;

   /* sNameOnCard */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sNameOnCard );

   /* sExpirationDate */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->
					   sExpirationDate );

   /* sCardNumber */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sCardNumber );

   DbvFree( psEsIoCreditCardInfo );
   } /* End Free_DbvEsIoCreditCardInfo() */
 
 
void do_Free_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   /* psResultSetItem */
   Free_DbvEsIoResultSetItem( psEsIoOriginPartNotToKeep->psResultSetItem );

   /* psItemRequest */
   FreeDbvASN1External( psEsIoOriginPartNotToKeep->psItemRequest );

   } /* End do_Free_DbvEsIoOriginPartNotToKeep() */ 

void Free_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      {
      do_Free_DbvIllString( psIllString );
      DbvFree( psIllString );
      } /* End if */

   } /* End Free_DbvIllString() */ 


void do_Free_DbvIllString( DbvIllString *psIllString )
   {

   switch( psIllString->eStringType)
      {
      case ST_GeneralString:
	 Free_DbvGeneralString( psIllString->uString.psGeneralString);
	 break;

      case ST_EdifactString:
	 Free_DbvVisibleString( psIllString->uString.pcVisibleString);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvIllString() */ 

void Free_DbvPersonOrInstitutionSymbol( DbvPersonOrInstitutionSymbol 
					*psPersonOrInstitutionSymbol )
   {
   if ( psPersonOrInstitutionSymbol == NULL )
      return;

   /* sSymbol */
   Free_compulsory_DbvIllString( &psPersonOrInstitutionSymbol->sSymbol );

   DbvFree( psPersonOrInstitutionSymbol );
   } /* End Free_DbvPersonOrInstitutionSymbol() */ 

void Free_DbvNameOfPersonOrInstitution( DbvNameOfPersonOrInstitution 
					*psNameOfPersonOrInstitution )
   {
   if ( psNameOfPersonOrInstitution == NULL )
      return;

   /* sName */
   Free_compulsory_DbvIllString( &psNameOfPersonOrInstitution->sName );

   DbvFree( psNameOfPersonOrInstitution );
   } /* End Free_DbvNameOfPersonOrInstitution() */ 

void Free_DbvSystemId( DbvSystemId *psSystemId )
   {
   if ( psSystemId != NULL )
      {
      do_Free_DbvSystemId( psSystemId );
      DbvFree( psSystemId );
      } /* End if */

   } /* End Free_DbvSystemId() */ 

void do_Free_DbvSystemId( DbvSystemId *psSystemId )
   {
   /* psSymbol */
   Free_DbvPersonOrInstitutionSymbol( psSystemId->psSymbol );

   /* psName */
   Free_DbvNameOfPersonOrInstitution( psSystemId->psName );

   } /* End do_Free_DbvSystemId() */ 

void Free_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      {
      do_Free_DbvTransactionId( psTransactionId );
      DbvFree( psTransactionId );
      } /* End if */

   } /* End Free_DbvTransactionId() */ 

void do_Free_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   /* psInitialRequesterId */
   Free_DbvSystemId( psTransactionId->psInitialRequesterId );

   /* sTransactionGroupQualifier */
   Free_compulsory_DbvIllString( &psTransactionId->sTransactionGroupQualifier );

   /* sTransactionQualifier */
   Free_compulsory_DbvIllString( &psTransactionId->sTransactionQualifier );

   /* psSubTransactionQualifier */
   Free_DbvIllString( psTransactionId->psSubTransactionQualifier );

   } /* End do_Free_DbvTransactionId() */ 

void Free_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      {
      do_Free_DbvDateTime(  psDateTime );
      DbvFree( psDateTime );
      } /* End if */
   } /* End Free_DbvDateTime() */ 

void do_Free_DbvDateTime( DbvDateTime *psDateTime )
   {

   /* pcDate */
   Free_DbvVisibleString( psDateTime->pcDate );

   /* pcTime */
   Free_DbvVisibleString( psDateTime->pcTime );

   } /* End do_Free_DbvDateTime() */ 
void Free_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      {
      do_Free_DbvServiceDateTime( psServiceDateTime );
      DbvFree( psServiceDateTime );
      } /* End if */

   } /* End Free_DbvServiceDateTime() */ 

void do_Free_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   /* sThisService */
   Free_compulsory_DbvDateTime( &psServiceDateTime->sThisService );

   /* psOriginalService */
   Free_DbvDateTime( psServiceDateTime->psOriginalService );

   } /* End do_Free_DbvServiceDateTime() */ 


void Free_DbvPostalAddress( DbvPostalAddress *psPostalAddress )
   {

   if ( psPostalAddress == NULL )
      return;

   /* psNameOfPersonOrInstitution */
   Free_DbvNameOfPersonOrInstitution( psPostalAddress->
				      psNameOfPersonOrInstitution );

   /* psExtendedPostalDeliveryAddress */
   Free_DbvIllString( psPostalAddress->psExtendedPostalDeliveryAddress );

   /* psStreetAndNumber */
   Free_DbvIllString( psPostalAddress->psStreetAndNumber );

   /* psPostOfficeBox */
   Free_DbvIllString( psPostalAddress->psPostOfficeBox );

   /* psCity */
   Free_DbvIllString( psPostalAddress->psCity );

   /* psRegion */
   Free_DbvIllString( psPostalAddress->psRegion );

   /* psCountry */
   Free_DbvIllString( psPostalAddress->psCountry );

   /* psPostalCode */
   Free_DbvIllString( psPostalAddress->psPostalCode );

   DbvFree( psPostalAddress );
   } /* End Free_DbvPostalAddress() */ 

void Free_DbvSystemAddress( DbvSystemAddress *psSystemAddress )
   {

   if ( psSystemAddress == NULL )
      return;

   /* psTelecomServiceIdentifier  */
   Free_DbvIllString( psSystemAddress->psTelecomServiceIdentifier );

   /* psTelecomServiceAddress */
   Free_DbvIllString( psSystemAddress->psTelecomServiceAddress );

   DbvFree( psSystemAddress );
   } /* End Free_DbvSystemAddress() */ 

void Free_DbvDeliveryAddress( DbvDeliveryAddress *psDeliveryAddress )
   {

   if ( psDeliveryAddress == NULL )
      return;

   /* psPostalAddress */
   Free_DbvPostalAddress( psDeliveryAddress->psPostalAddress );

   /* psElectronicAddress */
   Free_DbvSystemAddress( psDeliveryAddress->psElectronicAddress);

   DbvFree( psDeliveryAddress );
   } /* End Free_DbvDeliveryAddress() */ 

void Free_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      {
      do_Free_DbvElDeliveryDetails( psElDeliveryDetails );
      DbvFree( psElDeliveryDetails );
      } /* End if */

   } /* End Free_DbvElDeliveryDetails() */ 

void do_Free_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   switch( psElDeliveryDetails->eDeliveryDetailsType )
      {
      case DDT_SystemAddress:
	 Free_DbvSystemAddress( psElDeliveryDetails->
				uDeliveryDetails.psSystemAddress);
	 break;

      case DDT_SystemId:
	 Free_DbvSystemId( psElDeliveryDetails->
			   uDeliveryDetails.psSystemId);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvElDeliveryDetails() */ 

void Free_DbvElDeliveryParameter( 
			DbvElDeliveryParameter *psElDeliveryParameter )
   {

   if ( psElDeliveryParameter == NULL )
      return;

   /* pcDeliveryMode */
   FreeDbvObjectIdentifier( psElDeliveryParameter->pcDeliveryMode );

   /* psDummy */
   if ( psElDeliveryParameter->puDeliveryParameter )
      {
/*
      FreeDbvASN1External( psElDeliveryParameter->
			   puDeliveryParameter->psDummy );
*/

      /* puDeliveryParameter */
      DbvFree( psElDeliveryParameter->puDeliveryParameter );
      } /* End if */

   DbvFree( psElDeliveryParameter );
   } /* End Free_DbvElDeliveryParameter() */ 


void Free_DbvDocumentTypeParameter( 
			DbvDocumentTypeParameter *psDocumentTypeParameter )
   {

   if ( psDocumentTypeParameter == NULL )
      return;

   /* pcDeliveryMode */
   FreeDbvObjectIdentifier( psDocumentTypeParameter->pcDocumentTypeId );

   /* psDummy */
   if ( psDocumentTypeParameter->puDocumentTypeParameter)
      {
      FreeDbvASN1External( psDocumentTypeParameter->
			   puDocumentTypeParameter->psDummy );

      /* puDeliveryParameter */
      DbvFree( psDocumentTypeParameter->puDocumentTypeParameter);
      } /* End if */

   DbvFree( psDocumentTypeParameter );
   } /* End Free_DbvDocumentTypeParameter() */ 


void Free_DbvElectronicDeliveryService( 
		DbvElectronicDeliveryService *psElectronicDeliveryService )
   {

   if ( psElectronicDeliveryService == NULL )
      return;

   /* psDeliveryParameter */
   Free_DbvElDeliveryParameter( psElectronicDeliveryService->
				psDeliveryParameter);

   /* psDocumentTypeParameter */
   Free_DbvDocumentTypeParameter( psElectronicDeliveryService->
				  psDocumentTypeParameter );
   /* psDeliveryDescription */
   Free_DbvIllString( psElectronicDeliveryService->psDeliveryDescription );

   /* sDeliveryDetails */
   Free_compulsory_DbvElDeliveryDetails( &psElectronicDeliveryService->
					 sDeliveryDetails );


   /* psNameOrCode */
   Free_DbvIllString( psElectronicDeliveryService->psNameOrCode );

   /* pcDeliveryTime */
   Free_DbvVisibleString( psElectronicDeliveryService->pcDeliveryTime );
   
   DbvFree( psElectronicDeliveryService );
   } /* End Free_DbvElectronicDeliveryService() */ 


void Free_DbvDeliveryService( DbvDeliveryService *psDeliveryService )
   {

   if ( psDeliveryService == NULL )
      return;

   switch( psDeliveryService->eDeliveryServiceType )
      {
      case DST_PhysicalDelivery:
	 Free_DbvIllString( psDeliveryService->
			    uDeliveryService.psPhysicalDelivery);
	 break;

      case DST_ElectronicDelivery:
	 FreeN_DbvElectronicDeliveryService( psDeliveryService->
				uDeliveryService.ppsElectronicDelivery);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDeliveryService );
   } /* End Free_DbvDeliveryService() */ 

void Free_DbvRequesterOptionalMessages( 
		DbvRequesterOptionalMessages *psRequesterOptionalMessages )
   {

   if ( psRequesterOptionalMessages == NULL )
      return;

   DbvFree( psRequesterOptionalMessages );
   } /* End Free_DbvRequesterOptionalMessages() */ 

void Free_DbvSearchType( DbvSearchType *psSearchType )
   {

   if ( psSearchType == NULL )
      return;

   /* psLevelOfService */
   Free_DbvIllString( psSearchType->psLevelOfService );

   /* pcNeedBeforeDate */
   Free_DbvVisibleString( psSearchType->pcNeedBeforeDate );

   /* peExpiryFlag */
   Free_DbvExpiry( psSearchType->peExpiryFlag );

   /* pcExpiryDate */
   Free_DbvVisibleString( psSearchType->pcExpiryDate );

   DbvFree( psSearchType );
   } /* End Free_DbvSearchType() */ 

void Free_DbvSupplyMediumInfo( DbvSupplyMediumInfo *psSupplyMediumInfo )
   {

   if ( psSupplyMediumInfo == NULL )
      return;

   /* psMediumCharacteristics */
   Free_DbvIllString( psSupplyMediumInfo->psMediumCharacteristics );

   DbvFree( psSupplyMediumInfo );
   } /* End Free_DbvSupplyMediumInfo() */ 

void Free_DbvClientId( DbvClientId *psClientId )
   {

   if ( psClientId == NULL )
      return;

   /* psClientName */
   Free_DbvIllString( psClientId->psClientName);

   /* psClientStatus */
   Free_DbvIllString( psClientId->psClientStatus);

   /* psClientIdentifier */
   Free_DbvIllString( psClientId->psClientIdentifier);

   DbvFree( psClientId );
   } /* End Free_DbvClientId() */ 

void Free_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      {

      /* Free the fields in psItemId */
      do_Free_DbvItemId( psItemId );

      /* Dont attempt a DbvFree on psItemId as psItemId points to a structure
	 allocated at compile time, not one malloced at run time */

      } /* End if */

   } /* End Free_DbvItemId() */ 

void do_Free_DbvItemId( DbvItemId *psItemId )
   {

   /* peItemType */
   Free_DbvItemType( psItemId->peItemType );

   /* peMediumType */
   Free_DbvMediumType( psItemId->peMediumType );

   /* psCallNumber */
   Free_DbvIllString( psItemId->psCallNumber );

   /* psAuthor */
   Free_DbvIllString( psItemId->psAuthor );

   /* psTitle */
   Free_DbvIllString( psItemId->psTitle );

   /* psSubTitle */
   Free_DbvIllString( psItemId->psSubTitle );

   /* psSponsoringBody */
   Free_DbvIllString( psItemId->psSponsoringBody );

   /* psPlaceOfPublication */
   Free_DbvIllString( psItemId->psPlaceOfPublication );

   /* psPublisher */
   Free_DbvIllString( psItemId->psPublisher);

   /* psSeriesTitleNumber */
   Free_DbvIllString( psItemId->psSeriesTitleNumber);

   /* psVolumeIssue */
   Free_DbvIllString( psItemId->psVolumeIssue );

   /* psEdition */
   Free_DbvIllString( psItemId->psEdition );

   /* psPublicationDate */
   Free_DbvIllString( psItemId->psPublicationDate );

   /* psPublicationDateOfComponent */
   Free_DbvIllString( psItemId->psPublicationDateOfComponent );

   /* psAuthorOfArticle */
   Free_DbvIllString( psItemId->psAuthorOfArticle );

   /* psTitleOfArticle */
   Free_DbvIllString( psItemId->psTitleOfArticle );

   /* psPagination */
   Free_DbvIllString( psItemId->psPagination );

   /* psNationalBibliographyNo */
   FreeDbvASN1External( psItemId->psNationalBibliographyNo );

   /* psIsbn */
   Free_DbvIllString( psItemId->psIsbn );

   /* psIssn */
   Free_DbvIllString( psItemId->psIssn );

   /* psSystemNo */
   FreeDbvASN1External( psItemId->psSystemNo );

   /* psAdditionalNoLetters */
   Free_DbvIllString( psItemId->psAdditionalNoLetters );

   /* psVerificationReferenceSource */
   Free_DbvIllString( psItemId->psVerificationReferenceSource );

   } /* End do_Free_DbvItemId() */ 

void Free_DbvAmount( DbvAmount *psAmount )
   {

   if ( psAmount == NULL )
      return;

   /* pcCurrencyCode */
   Free_DbvPrintableString( psAmount->pcCurrencyCode );

   /* pcMonetaryValue */
   Free_DbvPrintableString( psAmount->pcMonetaryValue );

   DbvFree( psAmount );
   } /* End Free_DbvAmount() */ 

void Free_DbvCostInfo( DbvCostInfo *psCostInfo )
   {

   if ( psCostInfo == NULL )
      return;

   /* psAccountNumber */
   Free_DbvIllString( psCostInfo->psAccountNumber );

   /* psMaximumCost */
   Free_DbvAmount( psCostInfo->psMaximumCost );

   /* pfReciprocalAgreement */
   Free_DbvBool( psCostInfo->pfReciprocalAgreement );

   /* pfWillPayFee */
   Free_DbvBool( psCostInfo->pfWillPayFee );

   /* pfPaymentProvided */
   Free_DbvBool( psCostInfo->pfPaymentProvided );

   DbvFree( psCostInfo );
   } /* End Free_DbvCostInfo() */ 

void Free_DbvSendToList( DbvSendToList *psSendToList )
   {

   if ( psSendToList == NULL )
      return;

   /* sSystemId */
   Free_compulsory_DbvSystemId( &psSendToList->sSystemId );

   /* psAccountNumber */
   Free_DbvIllString( psSendToList->psAccountNumber );

   /* psSystemAddress */
   Free_DbvSystemAddress( psSendToList->psSystemAddress );

   DbvFree( psSendToList );
   } /* End Free_DbvSendToList() */ 

void Free_DbvThirdPartyInfo( DbvThirdPartyInfo *psThirdPartyInfo )
   {

   if ( psThirdPartyInfo == NULL )
      return;

   /* pfPermissionToForward */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToForward );

   /* pfPermissionToChain */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToChain );

   /* pfPermissionToPartition */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToPartition );

   /* pfPermissionToChangeSendToList */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToChangeSendToList );

   /* psInitialRequesterAddress */
   Free_DbvSystemAddress( psThirdPartyInfo->psInitialRequesterAddress );

   /* pePreference */
   Free_DbvPreference( psThirdPartyInfo->pePreference );

   /* ppsSendToList */
   FreeN_DbvSendToList( psThirdPartyInfo->ppsSendToList );

   /* ppsAlreadyTriedList */
   FreeN_DbvSystemId( psThirdPartyInfo->ppsAlreadyTriedList );

   DbvFree( psThirdPartyInfo );
   } /* End Free_DbvThirdPartyInfo() */ 


void Free_DbvRequesterGroup( DbvRequesterGroup *peRequesterGroup )
{
   if ( peRequesterGroup == NULL )
      return;

   DbvFree( peRequesterGroup );

} /* End Free_DbvRequesterGroup() */



void Free_DbvAddRequesterInfo( DbvAddRequesterInfo *psAddRequesterInfo )
{
   if ( psAddRequesterInfo == NULL )
      return;

   /* peRequesterGroup */
   Free_DbvRequesterGroup( psAddRequesterInfo->peRequesterGroup );

   /* pcRequesterEmailAddress */
   Free_DbvVisibleString( psAddRequesterInfo->pcRequesterEmailAddress );

   DbvFree( psAddRequesterInfo );
 
} /* End Free_DbvAddRequesterInfo() */


void Free_DbvStatusLevelUser( DbvStatusLevelUser *peStatusLevelUser )
{
   if ( peStatusLevelUser == NULL )
      return;

   DbvFree( peStatusLevelUser );
 
} /* End Free_DbvStatusLevelUser() */


 
void Free_DbvStatusLevelRequester( DbvStatusLevelRequester *peStatusLevelOrigin )
{
   if ( peStatusLevelOrigin == NULL )
      return;

   DbvFree( peStatusLevelOrigin );
 
} /* End Free_DbvStatusLevelRequester() */


 
void Free_DbvAddDeliveryInfo( DbvAddDeliveryInfo *psAddDeliveryInfo )
{
   if ( psAddDeliveryInfo == NULL )
      return;

   /* peStatusLevelUser */
   Free_DbvStatusLevelUser( psAddDeliveryInfo->peStatusLevelUser );

   /* peStatusLevelOrigin */
   Free_DbvStatusLevelRequester( psAddDeliveryInfo->peStatusLevelOrigin );

   /* pcDeliveryNotes */
   Free_DbvVisibleString( psAddDeliveryInfo->pcDeliveryNotes );

   /* pcCountryDeliveryTarget */
   Free_DbvVisibleString( psAddDeliveryInfo->pcCountryDeliveryTarget );

   /* pfAlternativePostalDelivery */
   Free_DbvBool( psAddDeliveryInfo->pfAlternativePostalDelivery );
    
   DbvFree( psAddDeliveryInfo );
 
} /* End Free_DbvAddDeliveryInfo() */



void Free_DbvBillingBankInfo( DbvBillingBankInfo *psBillingBankInfo)
{
   if ( psBillingBankInfo == NULL )
      return;

   /* pcBankAccountHouse */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountHouse );

   /* pcBankAccountRoutingSymbol */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountRoutingSymbol );

   /* pcBankAccountNo */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountNo );

   /* fBankAuthority ok */

   DbvFree( psBillingBankInfo );

} /* End Free_DbvBillingBankInfo */



void Free_DbvBillingCreditCardInfo( DbvBillingCreditCardInfo *psBillingCreditCardInfo)
{
   if ( psBillingCreditCardInfo == NULL )
      return;

   /* pcCardCompany */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardCompany );

   /* pcCardNo */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardNo );

   /* pcCardValidFrom */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardValidFrom );

   /* pcCardValidThru */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardValidThru );


   DbvFree( psBillingCreditCardInfo );

} /* End Free_DbvBillingCreditCardInfo */



void Free_DbvAddBillingInfo( DbvAddBillingInfo *psAddBillingInfo )
{
   if ( psAddBillingInfo == NULL )
      return;

   switch( psAddBillingInfo->eAddBillingType )
      {
      case ABT_BankInfo:
	 Free_DbvBillingBankInfo( psAddBillingInfo->
			    uAddBillingInfo.psBillingBankInfo);
	 break;

      case ABT_CreditCardInfo:
	 Free_DbvBillingCreditCardInfo( psAddBillingInfo->
				uAddBillingInfo.psBillingCreditCardInfo);
	 break;

      case ABT_InvoiceType:

         /* nothing required here because no pointer */
	 break;

      case ABT_PrepaidType:
         Free_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcPrepaidId); 
	 break;

      case ABT_DepositType:
         Free_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcDepositId); 
	 break;


      default:
	 break;
      } /* End switch */

   DbvFree( psAddBillingInfo );
 
} /* End Free_DbvAddBillingInfo() */



 


void Free_DbvSubitoAdditionalInfo( DbvSubitoAdditionalInfo *psAddIllReqInfo )
{
   if ( psAddIllReqInfo == NULL )
      return;

   /* pfCustomerInfoToKeep */
   Free_DbvBool( psAddIllReqInfo->pfCustomerInfoToKeep );

   /* psAddRequesterInfo */
   Free_DbvAddRequesterInfo( psAddIllReqInfo->psAddRequesterInfo );

 
   /* psAddDeliveryInfo */
   Free_DbvAddDeliveryInfo( psAddIllReqInfo->psAddDeliveryInfo );

 
   /* psAddBillingInfo */
   Free_DbvAddBillingInfo( psAddIllReqInfo->psAddBillingInfo );

 

   DbvFree( psAddIllReqInfo );
   } /* End Free_DbvSubitoAdditionalInfo() */ 


void Free_DbvExtension( DbvExtension *psExtension )
   {

   if ( psExtension == NULL )
      return;

   /* pfCritical */
   Free_DbvBool( psExtension->pfCritical );

   /* uItem.psAddIllReqInfo */

   Free_DbvSubitoAdditionalInfo( psExtension->uItem.psAddIllReqInfo );

   DbvFree( psExtension );
   } /* End Free_DbvExtension() */ 


void Free_DbvIllRequest( DbvIllRequest *psIllRequest )
   {

   if ( psIllRequest == NULL )
      return;

   /* eProtocolVersion : No Free required */
   /* sTransactionId */
   Free_compulsory_DbvTransactionId( &psIllRequest->sTransactionId );

   /* sServiceDateTime */
   Free_compulsory_DbvServiceDateTime( &psIllRequest->sServiceDateTime );

   /* psRequesterId */
   Free_DbvSystemId( psIllRequest->psRequesterId );

   /* psResponderId */
   Free_DbvSystemId( psIllRequest->psResponderId );

   /* peTransactionType */
   Free_DbvTransactionType( psIllRequest->peTransactionType );

   /* psDeliveryAddress */
   Free_DbvDeliveryAddress( psIllRequest->psDeliveryAddress );

   /* psDeliveryService */
   Free_DbvDeliveryService( psIllRequest->psDeliveryService );

   /* psBillingAddress */
   Free_DbvDeliveryAddress( psIllRequest->psBillingAddress );

   /* ppeServiceType */
   FreeN_DbvServiceType( psIllRequest->ppeServiceType );

   /* psResponderSpecificService */
   FreeDbvASN1External( psIllRequest->psResponderSpecificService );

   /* sRequesterOptionalMessages : This is currently defined as a    */
   /* macro which converts to a null as the stucture has no pointers */
   Free_compulsory_DbvRequesterOptionalMessages( 
			&psIllRequest->sRequesterOptionalMessages );

   /* psSearchType */
   Free_DbvSearchType( psIllRequest->psSearchType );

   /* ppsSupplyMediumInfo */
   FreeN_DbvSupplyMediumInfo( psIllRequest->ppsSupplyMediumInfo );

   /* pePlaceOnHold */
   Free_DbvPlaceOnHold( psIllRequest->pePlaceOnHold );

   /* psClientId */
   Free_DbvClientId( psIllRequest->psClientId );

   /* sItemId */
   Free_compulsory_DbvItemId( &psIllRequest->sItemId );

   /* ppsSupplementalItemDescription */
   FreeN_DbvASN1External( psIllRequest->ppsSupplementalItemDescription );

   /* psCostInfo */
   Free_DbvCostInfo( psIllRequest->psCostInfo );

   /* psCopyrightCompliance */
   Free_DbvIllString( psIllRequest->psCopyrightCompliance );

   /* psThirdPartyInfo */
   Free_DbvThirdPartyInfo( psIllRequest->psThirdPartyInfo );

   /* pfRetry */
   Free_DbvBool( psIllRequest->pfRetry );

   /* pfForward */
   Free_DbvBool( psIllRequest->pfForward );

   /* psRequesterNote */
   Free_DbvIllString( psIllRequest->psRequesterNote );

   /* psForwardNote */
   Free_DbvIllString( psIllRequest->psForwardNote );

   /* ppsRequestExtension */
   FreeN_DbvExtension( psIllRequest->ppsRequestExtension );

   DbvFree( psIllRequest );
   } /* End Free_DbvIllRequest() */ 


void Free_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      {
      do_Free_DbvHistoryReport( psHistoryReport );
      DbvFree( psHistoryReport );
      } /* End if */

   } /* End Free_DbvHistoryReport() */ 

void do_Free_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   /* pcDateRequested  */
   Free_DbvVisibleString( psHistoryReport->pcDateRequested );

   /* psAuthor */
   Free_DbvIllString( psHistoryReport->psAuthor );

   /* psTitle */
   Free_DbvIllString( psHistoryReport->psTitle );

   /* psAuthorOfArticle */
   Free_DbvIllString( psHistoryReport->psAuthorOfArticle );

   /* psTitleOfArticle */
   Free_DbvIllString( psHistoryReport->psTitleOfArticle );

   /* pcDateOfLastTransition */
   Free_DbvVisibleString( psHistoryReport->pcDateOfLastTransition );

   /* pcDateOfMostRecentService */
   Free_DbvVisibleString( psHistoryReport->pcDateOfMostRecentService );

   /* sSystemId */
   Free_compulsory_DbvSystemId( &psHistoryReport->sSystemId );

   /* peShippedServiceType */
   Free_DbvShippedServiceType( psHistoryReport->peShippedServiceType );

   /* peTransactionResults */
   Free_DbvTransactionResults( psHistoryReport->peTransactionResults );

   /* psMostRecentServiceNote */
   Free_DbvIllString( psHistoryReport->psMostRecentServiceNote );

   } /* End do_Free_DbvHistoryReport() */ 

void Free_DbvStateTransitionProhibited( 
		DbvStateTransitionProhibited *psStateTransitionProhibited )
   {

   if ( psStateTransitionProhibited == NULL )
      return;

   DbvFree( psStateTransitionProhibited );
   } /* End Free_DbvStateTransitionProhibited() */ 


void Free_DbvAlreadyForwarded( DbvAlreadyForwarded *psAlreadyForwarded )
   {

   if ( psAlreadyForwarded == NULL )
      return;

   /* sResponderId */
   Free_compulsory_DbvSystemId( &psAlreadyForwarded->sResponderId );

   /* psResponderAddress */
   Free_DbvSystemAddress( psAlreadyForwarded->psResponderAddress );


   DbvFree( psAlreadyForwarded );
   } /* End Free_DbvAlreadyForwarded() */ 


void Free_DbvProviderErrorReport( 
			DbvProviderErrorReport *psProviderErrorReport )
   {

   if ( psProviderErrorReport == NULL )
      return;

   /* uProviderErrorReport */

   switch ( psProviderErrorReport->eReportType )
      {
      case PER_GeneralProblem:
      case PER_TransactionIdProblem:
	 break;

      case PER_StateTransitionProhibited:
	 Free_DbvStateTransitionProhibited( psProviderErrorReport->
					    uProviderErrorReport.
					    psStateTransitionProblem );
         break;

      default:
	 break;

      } /* End if */
   DbvFree( psProviderErrorReport );
   } /* End Free_DbvProviderErrorReport() */ 

void Free_DbvUserErrorReport( DbvUserErrorReport *psUserErrorReport )
   {

   if ( psUserErrorReport == NULL )
      return;

   /* uUserErrorReport */

   switch ( psUserErrorReport->eReportType )
      {
      case UER_AlreadyForwarded:
	 Free_DbvAlreadyForwarded( psUserErrorReport->
				   uUserErrorReport.
				   psAlreadyForwarded );
	 break;
      case UER_IntermediaryProblem:
	 break;

      case UER_SecurityProblem:
	 Free_DbvIllString( psUserErrorReport->
			    uUserErrorReport.
			    psSecurityProblem );
	 break;

      case UER_UnableToPerform:
      default:
	 break;

      } /* End if */
   DbvFree( psUserErrorReport );
   } /* End Free_DbvUserErrorReport() */ 

void Free_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      {
      do_Free_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );
      DbvFree( psUserOrProviderErrorReport );
      } /* End if */

   } /* End Free_DbvUserOrProviderErrorReport() */ 

void do_Free_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   /* uReport */

   switch ( psUserOrProviderErrorReport->eReportSource )
      {
      case RS_Provider:
	 Free_DbvProviderErrorReport( psUserOrProviderErrorReport->
				      uReport.psProviderErrorReport);
	 break;

      case RS_User:
	 Free_DbvUserErrorReport( psUserOrProviderErrorReport->
			    uReport.psUserErrorReport );
	 break;

      default:
	 break;

      } /* End if */

   } /* End do_Free_DbvUserOrProviderErrorReport() */ 

void Free_DbvErrorReport( DbvErrorReport *psErrorReport )
   {

   if ( psErrorReport == NULL )
      return;

   /* sCorrelationInfo */
   Free_compulsory_DbvIllString( &psErrorReport->sCorrelationInfo );

   /* sReport */
   Free_compulsory_DbvUserOrProviderErrorReport(  &psErrorReport->sReport );

   DbvFree( psErrorReport );
   } /* End Free_DbvErrorReport() */ 

void Free_DbvStatusReport( DbvStatusReport *psStatusReport )
   {

   if ( psStatusReport == NULL )
      return;

   /* sUserStatusReport */
   Free_compulsory_DbvHistoryReport( &psStatusReport->sUserStatusReport );

   DbvFree( psStatusReport );
   } /* End Free_DbvStatusReport() */ 

void Free_DbvStatusOrErrorReport( 
			DbvStatusOrErrorReport *psStatusOrErrorReport )
   {

   if ( psStatusOrErrorReport == NULL )
      return;

   /* eProtocolVersion */

   /* sTransactionId */
   Free_compulsory_DbvTransactionId( &psStatusOrErrorReport->sTransactionId );

   /* sServiceDateTime */
   Free_compulsory_DbvServiceDateTime( 
				&psStatusOrErrorReport->sServiceDateTime );
   /* psRequesterId */
   Free_DbvSystemId( psStatusOrErrorReport->psRequesterId );

   /* psResponderId */
   Free_DbvSystemId( psStatusOrErrorReport->psResponderId );

   /* peReasonNoReport */
   Free_DbvReasonNoReport( psStatusOrErrorReport->peReasonNoReport );

   /* psStatusReport */
   Free_DbvStatusReport( psStatusOrErrorReport->psStatusReport );

   /* psErrorReport */
   Free_DbvErrorReport( psStatusOrErrorReport->psErrorReport );

   /* psNote */
   Free_DbvIllString( psStatusOrErrorReport->psNote );

   /* ppsStOrErReportExtension */
   FreeN_DbvExtension( psStatusOrErrorReport->ppsStOrErReportExtension );

   DbvFree( psStatusOrErrorReport );
   } /* End Free_DbvStatusOrErrorReport() */ 


void Free_DbvIllPdu( DbvIllPdu *psIllPdu )
   {

   if ( psIllPdu == NULL )
      return;

   /* uIllPdu */

   switch( psIllPdu->eIllPduType )
      {
      case ILL_IllRequest:
	 Free_DbvIllRequest( psIllPdu->uIllPdu.psIllRequest );
	 break;

      case ILL_StatusOrErrorReport:
	 Free_DbvStatusOrErrorReport( psIllPdu->
				      uIllPdu.psStatusOrErrorReport);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psIllPdu );
   } /* End Free_DbvIllPdu() */ 

void Free_DbvBool( DbvBool *peBool )
   {

   if ( peBool == NULL )
      return;

   DbvFree( peBool );
   } /* End Free_DbvBool() */ 

void Free_DbvPreference( DbvPreference *pePreference )
   {

   if ( pePreference == NULL )
      return;

   DbvFree( pePreference );
   } /* End Free_DbvPreference() */ 

void Free_DbvTransactionType( DbvTransactionType *peTransactionType )
   {

   if ( peTransactionType == NULL )
      return;

   DbvFree( peTransactionType );
   } /* End Free_DbvTransactionType() */ 

void Free_DbvReasonNoReport( DbvReasonNoReport *peReasonNoReport )
   {

   if ( peReasonNoReport == NULL )
      return;

   DbvFree( peReasonNoReport );
   } /* End Free_DbvReasonNoReport() */ 

void Free_DbvPrintableString( DbvPrintableString pcPrintableString )
   {

   if ( pcPrintableString == NULL )
      return;

   DbvFree( pcPrintableString );
   } /* End Free_DbvPrintableString() */ 

void Free_DbvItemType( DbvItemType *peItemType )
   {

   if ( peItemType == NULL )
      return;

   DbvFree( peItemType );
   } /* End Free_DbvItemType() */ 

void Free_DbvExpiry( DbvExpiry *peExpiry )
   {

   if ( peExpiry == NULL )
      return;

   DbvFree( peExpiry );
   } /* End Free_DbvExpiry() */ 

void Free_DbvMediumType( DbvMediumType *peMediumType )
   {

   if ( peMediumType == NULL )
      return;

   DbvFree( peMediumType );
   } /* End Free_DbvMediumType() */ 

void Free_DbvServiceType( DbvServiceType *peServiceType )
   {

   if ( peServiceType == NULL )
      return;

   DbvFree( peServiceType );
   } /* End Free_DbvServiceType() */ 

void Free_DbvPlaceOnHold( DbvPlaceOnHold *pePlaceOnHold )
   {

   if ( pePlaceOnHold == NULL )
      return;

   DbvFree( pePlaceOnHold );
   } /* End Free_DbvPlaceOnHold() */ 

void Free_DbvShippedServiceType( DbvShippedServiceType *peShippedServiceType )
   {

   if ( peShippedServiceType == NULL )
      return;

   DbvFree( peShippedServiceType );
   } /* End Free_DbvShippedServiceType() */ 

void Free_DbvTransactionResults( DbvTransactionResults *peTransactionResults )
   {

   if ( peTransactionResults == NULL )
      return;

   DbvFree( peTransactionResults );
   } /* End Free_DbvTransactionResults() */ 

void Free_DbvEsIoAuxiliaryStatus( 
			DbvEsIoAuxiliaryStatus *peEsIoAuxiliaryStatus )
   {

   if ( peEsIoAuxiliaryStatus == NULL )
      return;

   DbvFree( peEsIoAuxiliaryStatus );
   } /* End Free_DbvEsIoAuxiliaryStatus() */ 

void Free_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod == NULL )
      {
      do_Free_DbvEsIoPaymentMethod( psEsIoPaymentMethod );
      DbvFree( psEsIoPaymentMethod );
      } /* End if */

   } /* End Free_DbvEsIoPaymentMethod() */ 


void do_Free_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod->puPaymentMethod )
      {
      switch( psEsIoPaymentMethod->ePaymentMethodType )
         {
         case IOPM_CreditCard:
	    Free_DbvEsIoCreditCardInfo( psEsIoPaymentMethod->
				       puPaymentMethod->psCreditCardInfo );
	    break;
   
         case IOPM_PrivateNotKnown:
	    FreeDbvASN1External( psEsIoPaymentMethod->
				 puPaymentMethod->psPrivateNotKnown );
	    break;
   
	 case IOPM_BillInvoice:
	 case IOPM_Prepay:
	 case IOPM_DepositAccount:
	 case IOPM_CardInfoPrevSupplied:
	 case IOPM_PrivateKnown:
	 case IOPM_Invalid:
         default:
	    break;
         } /* End switch */
      } /* End if */

   } /* End do_Free_DbvEsIoPaymentMethod() */ 


void Free_DbvEsIoAdditionalBilling( 
			DbvEsIoAdditionalBilling *psEsIoAdditionalBilling )
   {

   if ( psEsIoAdditionalBilling == NULL )
      return;

   /* sPaymentMethod */

   Free_compulsory_DbvEsIoPaymentMethod( 
				&psEsIoAdditionalBilling->sPaymentMethod );
   /* psCustomerReference */
   if ( psEsIoAdditionalBilling->psCustomerReference != NULL )
      FreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerReference);

   /* psCustomerPONumber */
   if ( psEsIoAdditionalBilling->psCustomerPONumber != NULL )
      FreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerPONumber);

   DbvFree( psEsIoAdditionalBilling );
   } /* End Free_DbvEsIoAdditionalBilling() */ 


void Free_DbvEsIoContact( DbvEsIoContact *psEsIoContact )
   {

   if ( psEsIoContact == NULL )
      return;

   /* psName */
   if ( psEsIoContact->psName != NULL )
      FreeDbvInternationalString( psEsIoContact->psName );

   /* psPhone */
   if ( psEsIoContact->psPhone != NULL )
      FreeDbvInternationalString( psEsIoContact->psPhone );

   /* psEmail */
   if ( psEsIoContact->psEmail != NULL )
      FreeDbvInternationalString( psEsIoContact->psEmail );

   DbvFree( psEsIoContact );
   } /* End Free_DbvEsIoContact() */ 

void Free_DbvEsIoOriginPartToKeep( 
			DbvEsIoOriginPartToKeep *psEsIoOriginPartToKeep )
   {

   if ( psEsIoOriginPartToKeep == NULL )
      return;

   /* psSupplementalDescription  */
   FreeDbvASN1External( psEsIoOriginPartToKeep->psSupplementalDescription );

   /* psContact */
   Free_DbvEsIoContact( psEsIoOriginPartToKeep->psContact );

   /* psAdditionalBilling */
   Free_DbvEsIoAdditionalBilling( psEsIoOriginPartToKeep->psAdditionalBilling );

   DbvFree( psEsIoOriginPartToKeep );
   } /* End Free_DbvEsIoOriginPartToKeep() */ 

void Free_DbvEsIoRequest( DbvEsIoRequest *psEsIoRequest )
   {

   if ( psEsIoRequest == NULL )
      return;

   /* psToKeep */
   Free_DbvEsIoOriginPartToKeep( psEsIoRequest->psToKeep );

   /* sNotToKeep */
   Free_compulsory_DbvEsIoOriginPartNotToKeep( &psEsIoRequest->sNotToKeep );

   DbvFree( psEsIoRequest );
   } /* End Free_DbvEsIoRequest() */ 

void Free_DbvEsIoTaskPackage( DbvEsIoTaskPackage *psEsIoTaskPackage )
   {

   if ( psEsIoTaskPackage == NULL )
      return;

   /* psOriginPart */
   Free_DbvEsIoOriginPartToKeep( psEsIoTaskPackage->psOriginPart );

   /* sTargetPart */
   Free_compulsory_DbvEsIoTargetPart( &psEsIoTaskPackage->sTargetPart );

   DbvFree( psEsIoTaskPackage );
   } /* End Free_DbvEsIoTaskPackage() */ 

void Free_DbvItemOrderExtendedService( 
		DbvItemOrderExtendedService *psItemOrderExtendedService )
   {

   if ( psItemOrderExtendedService == NULL )
      return;

   /* uItemOrderReqOrRsp */

   switch( psItemOrderExtendedService->eItemOrderReqRsp )
      {
      case IOROR_Request:
	 Free_DbvEsIoRequest( psItemOrderExtendedService->
			      uItemOrderReqOrRsp.psItemOrderRequest);
	 break;

      case IOROR_Response:
	 Free_DbvEsIoTaskPackage( psItemOrderExtendedService->
				  uItemOrderReqOrRsp.psItemOrderResponse);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psItemOrderExtendedService );
   } /* End Free_DbvItemOrderExtendedService() */ 
