/*  Module Name: @(#)Free_DbvFuncs2.c	7.1
    Version: 11/15/96
    Author: Dan Maskell
 
    Crossnet Systems Ltd,
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for structure types.
        Used by the origin and target.

    Modification History.

 
*/

#include "Free_DbvFuncs.h"



void Free_DbvCommonInfo( DbvCommonInfo *psCommonInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCommonInfo()\n" );
#endif

   if ( psCommonInfo == NULL )
      return;

   /* pcDateAdded */
   Free_DbvGeneralizedTime( psCommonInfo->pcDateAdded );

   /* pcDateChanged */
   Free_DbvGeneralizedTime( psCommonInfo->pcDateChanged );

   /* pcExpiry */
   Free_DbvGeneralizedTime( psCommonInfo->pcExpiry );

   /* psHumanStringLanguage */
   Free_DbvLanguageCode( psCommonInfo->psHumanStringLanguage );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psCommonInfo->ppsOtherInformation );

   DbvFree( psCommonInfo );
   } /* End Free_DbvCommonInfo() */ 

void Free_DbvHumanString( DbvHumanString *psHumanString )
   {
#ifdef DEBUG
   printf( "D - Free_DbvHumanString()\n" );
#endif

   if ( psHumanString == NULL )
      return;

   /* psLanguage */
   Free_DbvLanguageCode( psHumanString->psLanguage );

   /* sText */
   Free_compulsory_DbvInternationalString( &psHumanString->sText );

   DbvFree( psHumanString );
   } /* End Free_DbvHumanString() */ 

void Free_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvBodyType()\n" );
#endif

   if ( psBodyType != NULL )
      {
      do_Free_DbvBodyType( psBodyType );
      DbvFree( psBodyType );
      } /* End if */

   } /* End Free_DbvBodyType() */ 

void do_Free_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUG
   printf( "D - do_Free_DbvBodyType()\n" );
#endif

   if ( psBodyType == NULL )
      return;

   /* uBodyTypeType */

   switch( psBodyType->eBodyTypeType )
      {
      case BTT_IanaType:
	 FreeDbvInternationalString( psBodyType->uBodyType.psIanaType );
	 break;

      case BTT_Z3950Type:
	 FreeDbvInternationalString( psBodyType->uBodyType.psZ3950Type );
	 break;

      case BTT_OtherType:
	 FreeDbvInternationalString( psBodyType->uBodyType.psOtherType );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_DbvBodyType() */ 

void Free_DbvIconObject( DbvIconObject *psIconObject )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIconObject()\n" );
#endif

   if ( psIconObject == NULL )
      return;

   /* sBodyType */
   Free_compulsory_DbvBodyType( &psIconObject->sBodyType );

   /* sContent */
   Free_compulsory_DbvOctetString( &psIconObject->sContent );

   DbvFree( psIconObject );
   } /* End Free_DbvIconObject() */ 

void Free_DbvContactInfo( DbvContactInfo *psContactInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvContactInfo()\n" );
#endif

   if ( psContactInfo == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psContactInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psContactInfo->ppsDescription );

   /* ppsAddress */
   FreeN_DbvHumanString( psContactInfo->ppsAddress );

   /* psEmail */
   FreeDbvInternationalString( psContactInfo->psEmail );

   /* psPhone */
   FreeDbvInternationalString( psContactInfo->psPhone );

   DbvFree( psContactInfo );
   } /* End Free_DbvContactInfo() */ 

void Free_DbvDatabaseList( DbvDatabaseList *psDatabaseList )
   {
#ifdef DEBUG
   printf( "D - Free_DbvDatabaseList()\n" );
#endif

   if ( psDatabaseList == NULL )
      return;

   /* ppsInnerDatabases */
   FreeN_DbvInternationalString( psDatabaseList->ppsInnerDatabases );

   DbvFree( psDatabaseList );
   } /* End Free_DbvDatabaseList() */ 

void Free_DbvInterAddress( DbvInterAddress *psInterAddress )
   {
#ifdef DEBUG
   printf( "D - Free_DbvInterAddress()\n" );
#endif

   if ( psInterAddress == NULL )
      return;

   /* sHostAddress */
   Free_compulsory_DbvInternationalString( &psInterAddress->sHostAddress );

   DbvFree( psInterAddress );
   } /* End Free_DbvInterAddress() */ 

void Free_DbvOsiPresentationAddress( 
		DbvOsiPresentationAddress *psOsiPresentationAddress )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOsiPresentationAddress()\n" );
#endif

   if ( psOsiPresentationAddress == NULL )
      return;

   /* sPsel */
   Free_compulsory_DbvInternationalString( &psOsiPresentationAddress->sPsel );

   /* psSSel */
   FreeDbvInternationalString( psOsiPresentationAddress->psSSel );

   /* psTSel */
   FreeDbvInternationalString( psOsiPresentationAddress->psTSel );

   /* sNSap */
   Free_compulsory_DbvInternationalString( &psOsiPresentationAddress->sNSap );

   DbvFree( psOsiPresentationAddress );
   } /* End Free_DbvOsiPresentationAddress() */ 

void Free_DbvOtherAddress( DbvOtherAddress *psOtherAddress )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOtherAddress()\n" );
#endif

   if ( psOtherAddress == NULL )
      return;

   /* sType */
   Free_compulsory_DbvInternationalString( &psOtherAddress->sType );

   /* sAddress */
   Free_compulsory_DbvInternationalString( &psOtherAddress->sAddress );

   DbvFree( psOtherAddress );
   } /* End Free_DbvOtherAddress() */ 

void Free_DbvNetworkAddress( DbvNetworkAddress *psNetworkAddress )
   {
#ifdef DEBUG
   printf( "D - Free_DbvNetworkAddress()\n" );
#endif

   if ( psNetworkAddress == NULL )
      return;

   /* uNetworkAddressType */

   switch( psNetworkAddress->eNetworkAddressType )
      {
      case NAT_InternetAddress:
	 Free_DbvInterAddress( psNetworkAddress->
				uNetworkAddress.psInternetAddress );
	 break;

      case NAT_OsiPresentationAddress:
	 Free_DbvOsiPresentationAddress( psNetworkAddress->
				uNetworkAddress.psOsiPresentationAddress );
	 break;

      case NAT_OtherAddress:
	 Free_DbvOtherAddress( psNetworkAddress->
				uNetworkAddress.psOtherAddress );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNetworkAddress );
   } /* End Free_DbvNetworkAddress() */ 

void Free_DbvOperators( DbvOperators *psOperators )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOperators()\n" );
#endif

   if ( psOperators == NULL )
      return;

   /* sOperator */
   Free_compulsory_DbvInternationalString( &psOperators->sOperator );

   /* ppsDescription */
   FreeN_DbvHumanString( psOperators->ppsDescription );

   DbvFree( psOperators );
   } /* End Free_DbvOperators() */ 

void Free_DbvSearchKey( DbvSearchKey *psSearchKey )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSearchKey()\n" );
#endif

   if ( psSearchKey == NULL )
      return;

   /* sSearchKey */
   Free_compulsory_DbvInternationalString( &psSearchKey->sSearchKey );

   /* ppsDescription */
   FreeN_DbvHumanString( psSearchKey->ppsDescription );

   DbvFree( psSearchKey );
   } /* End Free_DbvSearchKey() */ 


void Free_DbvHumanStringSeq( DbvHumanStringSeq *psDescription )
   {
#ifdef DEBUG
   printf( "D - Free_DbvHumanStringSeq()\n" );
#endif

   if ( psDescription == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psDescription->ppsHumanString );

   DbvFree( psDescription );
   } /* End Free_DbvHumanStringSeq() */ 



void Free_DbvPrivateCapabilities( 
				DbvPrivateCapabilities *psPrivateCapabilities )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPrivateCapabilities()\n" );
#endif

   if ( psPrivateCapabilities == NULL )
      return;

   /* ppsOperators */
   FreeN_DbvOperators( psPrivateCapabilities->ppsOperators );

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psPrivateCapabilities->ppsSearchKeys );

   /* ppsDescription */
   FreeN_DbvHumanStringSeq( psPrivateCapabilities->ppsDescription );

   DbvFree( psPrivateCapabilities );
   } /* End Free_DbvPrivateCapabilities() */ 

void Free_DbvPrivate( DbvPrivate *psPrivate )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPrivate()\n" );
#endif

   if ( psPrivate == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psPrivate->ppsDescription );

   DbvFree( psPrivate );
   } /* End Free_DbvPrivate() */ 

void Free_DbvUnitsSupported( DbvUnitsSupported *psUnitsSupported )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUnitsSupported()\n" );
#endif

   if ( psUnitsSupported == NULL )
      return;

   /* uUnitsSupportedType */

   switch( psUnitsSupported->eUnitsSupportedType )
      {
      case UST_Private:
	 Free_DbvPrivate( psUnitsSupported->uUnitsSupported.psPrivate );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUnitsSupported );
   } /* End Free_DbvUnitsSupported() */ 

void Free_DbvProximitySupport( DbvProximitySupport *psProximitySupport )
   {
#ifdef DEBUG
   printf( "D - Free_DbvProximitySupport()\n" );
#endif

   if ( psProximitySupport == NULL )
      return;

   FreeN_DbvUnitsSupported( psProximitySupport->ppsUnitsSupported );

   DbvFree( psProximitySupport );
   } /* End Free_DbvProximitySupport() */ 

void Free_DbvRpnCapabilities( DbvRpnCapabilities *psRpnCapabilities )
   {
#ifdef DEBUG
   printf( "D - Free_DbvRpnCapabilities()\n" );
#endif

   if ( psRpnCapabilities == NULL )
      return;

   /* ppulOperators */
   FreeN_Ulong( psRpnCapabilities->ppulOperators );

   /* psProximity */
   Free_DbvProximitySupport( psRpnCapabilities->psProximity );

   DbvFree( psRpnCapabilities );
   } /* End Free_DbvRpnCapabilities() */ 

void Free_DbvIso8777Capabilities(
			DbvIso8777Capabilities *psIso8777Capabilities )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIso8777Capabilities()\n" );
#endif

   if ( psIso8777Capabilities == NULL )
      return;

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psIso8777Capabilities->ppsSearchKeys );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psIso8777Capabilities->ppsRestrictions );

   DbvFree( psIso8777Capabilities );
   } /* End Free_DbvIso8777Capabilities() */ 

void Free_DbvQueryTypeDetails( DbvQueryTypeDetails *psQueryTypeDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvQueryTypeDetails()\n" );
#endif

   if ( psQueryTypeDetails == NULL )
      return;

   /* uQueryTypeDetailsType */

   switch( psQueryTypeDetails->eQueryTypeDetailsType )
      {
      case QTDT_Private:
	 Free_DbvPrivateCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psPrivate );
	 break;

      case QTDT_Rpn:
	 Free_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psRpn );
	 break;

      case QTDT_Iso8777:
	 Free_DbvIso8777Capabilities( psQueryTypeDetails->
				uQueryTypeDetails.psIso8777 );
	 break;

      case QTDT_Z39_58:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsZ39_58 );
	 break;

      case QTDT_Erpn:
	 Free_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psErpn );
	 break;

      case QTDT_RankedList:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsRankedList );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQueryTypeDetails );
   } /* End Free_DbvQueryTypeDetails() */ 

void Free_DbvAccessRestrictions( DbvAccessRestrictions *psAccessRestrictions )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAccessRestrictions()\n" );
#endif

   if ( psAccessRestrictions == NULL )
      return;

   /* ppsAccessText */
   FreeN_DbvHumanString( psAccessRestrictions->ppsAccessText );

   /* ppcAccessChallenges */
   FreeN_DbvObjectIdentifier( psAccessRestrictions->ppcAccessChallenges );

   DbvFree( psAccessRestrictions );
   } /* End Free_DbvAccessRestrictions() */ 

void Free_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCharge()\n" );
#endif

   if ( psCharge != NULL )
      {
      do_Free_DbvCharge( psCharge );
      DbvFree( psCharge );
      } /* End if */

   } /* End Free_DbvCharge() */ 

void do_Free_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUG
   printf( "D - do_Free_DbvCharge()\n" );
#endif

   /* sCost */
   Free_compulsory_DbvIntUnit( &psCharge->sCost );

   /* psPerWhat */
   Free_DbvUnit( psCharge->psPerWhat );

   /* ppsText */
   FreeN_DbvHumanString( psCharge->ppsText );

   } /* End do_Free_DbvCharge() */ 

void Free_DbvOtherCharge( DbvOtherCharge *psOtherCharge )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOtherCharge()\n" );
#endif

   if ( psOtherCharge == NULL )
      return;

   /* ppsForWhat */
   FreeN_DbvHumanString( psOtherCharge->ppsForWhat );

   /* sCharge */
   Free_compulsory_DbvCharge( &psOtherCharge->sCharge );

   DbvFree( psOtherCharge );
   } /* End Free_DbvOtherCharge() */ 

void Free_DbvCosts( DbvCosts *psCosts )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCosts()\n" );
#endif

   if ( psCosts == NULL )
      return;

   /* ppsForWhat */
   Free_DbvCharge( psCosts->psConnectCharge );

   /* psConnectTime */
   Free_DbvCharge( psCosts->psConnectTime );

   /* psDisplayCharge */
   Free_DbvCharge( psCosts->psDisplayCharge );

   /* psSearchCharge */
   Free_DbvCharge( psCosts->psSearchCharge );

   /* psSubscriptCharge */
   Free_DbvCharge( psCosts->psSubscriptCharge );

   /* ppsOtherCharge */
   FreeN_DbvOtherCharge( psCosts->ppsOtherCharge );

   DbvFree( psCosts );
   } /* End Free_DbvCosts() */ 

void Free_DbvAccessInfo( DbvAccessInfo *psAccessInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAccessInfo()\n" );
#endif

   if ( psAccessInfo == NULL )
      return;

   /* ppsQueryTypesSupported */
   FreeN_DbvQueryTypeDetails( psAccessInfo->ppsQueryTypesSupported );

   /* ppcDiagnosticsSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcDiagnosticsSets );

   /* ppcAttributeSetIds */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcAttributeSetIds );

   /* ppcSchemas */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcSchemas );

   /* ppcRecordSyntaxes */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcRecordSyntaxes );

   /* ppcResourceChallenges */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcResourceChallenges );

   /* ppsRestrictedAccess */
   FreeN_DbvAccessRestrictions( psAccessInfo->ppsRestrictedAccess );

   /* psCostInfo */
   Free_DbvCosts( psAccessInfo->psCostInfo );

   /* ppcVariantSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcVariantSets );

   /* ppsElementSetNames */
   FreeN_DbvInternationalString( psAccessInfo->ppsElementSetNames );

   /* ppsUnitSystems */
   FreeN_DbvInternationalString( psAccessInfo->ppsUnitSystems );

   DbvFree( psAccessInfo );
   } /* End Free_DbvAccessInfo() */ 

void Free_DbvTargetInfo( DbvTargetInfo *psTargetInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTargetInfo()\n" );
#endif

   if ( psTargetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTargetInfo->psCommonInfo );

   /* sName */
   Free_compulsory_DbvInternationalString( &psTargetInfo->sName );

   /* ppsRecentNews */
   FreeN_DbvHumanString( psTargetInfo->ppsRecentNews );

   /* ppsIcon */
   FreeN_DbvIconObject( psTargetInfo->ppsIcon );

   /* pulMaxResultSets */
   Free_Ulong( psTargetInfo->pulMaxResultSets );

   /* pulMaxResultSize */
   Free_Ulong( psTargetInfo->pulMaxResultSize );

   /* pulMaxTerms */
   Free_Ulong( psTargetInfo->pulMaxTerms );

   /* psTimeoutInterval */
   Free_DbvIntUnit( psTargetInfo->psTimeoutInterval );

   /* ppsWelcomeMessage */
   FreeN_DbvHumanString( psTargetInfo->ppsWelcomeMessage );

   /* psContactInfo */
   Free_DbvContactInfo( psTargetInfo->psContactInfo );

   /* ppsDescription */
   FreeN_DbvHumanString( psTargetInfo->ppsDescription );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psTargetInfo->ppsNicknames );

   /* ppsUsageRest */
   FreeN_DbvHumanString( psTargetInfo->ppsUsageRest );

   /* ppsPaymentAddr */
   FreeN_DbvHumanString( psTargetInfo->ppsPaymentAddr );

   /* ppsHours */
   FreeN_DbvHumanString( psTargetInfo->ppsHours  );

   /* ppsDbCombinations */
   FreeN_DbvDatabaseList( psTargetInfo->ppsDbCombinations );

   /* ppsAddresses */
   FreeN_DbvNetworkAddress( psTargetInfo->ppsAddresses );

   /* ppsLanguages */
   FreeN_DbvInternationalString( psTargetInfo->ppsLanguages );

   /* psCommonAccessInfo */
   Free_DbvAccessInfo( psTargetInfo->psCommonAccessInfo );

   DbvFree( psTargetInfo );
   } /* End Free_DbvTargetInfo() */ 

void Free_DbvRecordCount( DbvRecordCount *psRecordCount )
   {
#ifdef DEBUG
   printf( "D - Free_DbvRecordCount()\n" );
#endif

   if ( psRecordCount == NULL )
      return;

   DbvFree( psRecordCount );
   } /* End Free_DbvRecordCount() */ 

void Free_DbvDatabaseInfo( DbvDatabaseInfo *psDatabaseInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvDatabaseInfo()\n" );
#endif

   if ( psDatabaseInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psDatabaseInfo->psCommonInfo );

   /* sName */
   Free_compulsory_DbvInternationalString( &psDatabaseInfo->sName );

   /* pvExplainDatabase */
   Free_DbvNull( psDatabaseInfo->pvExplainDatabase );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsNicknames );

   /* ppsIcon */
   FreeN_DbvIconObject( psDatabaseInfo->ppsIcon );

   /* ppsTitleString */
   FreeN_DbvHumanString( psDatabaseInfo->ppsTitleString );

   /* ppsKeywords */
   FreeN_DbvDataBaseInfoSeq( psDatabaseInfo->ppsKeywords );

   /* ppsDescription */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDescription );

   /* ppsAssociatedDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsAssociatedDbs );

   /* ppsSubDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsSubDbs );

   /* ppsDisclaimers */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDisclaimers );

   /* ppsNews */
   FreeN_DbvHumanString( psDatabaseInfo->ppsNews );

   /* psRecordCount */
   Free_DbvRecordCount( psDatabaseInfo->psRecordCount );

   /* ppsDefaultOrder */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDefaultOrder );

   /* pulAvRecordSize */
   Free_Ulong( psDatabaseInfo->pulAvRecordSize );

   /* pulMaxRecordSize */
   Free_Ulong( psDatabaseInfo->pulMaxRecordSize );

   /* ppsHours */
   FreeN_DbvHumanString( psDatabaseInfo->ppsHours );

   /* ppsBestTime */
   FreeN_DbvHumanString( psDatabaseInfo->ppsBestTime );

   /* pcLastUpdate */
   Free_DbvGeneralizedTime( psDatabaseInfo->pcLastUpdate );

   /* psUpdateInterval */
   Free_DbvIntUnit( psDatabaseInfo->psUpdateInterval );

   /* ppsCoverage */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCoverage );

   /* pfProprietary */
   Free_DbvBool( psDatabaseInfo->pfProprietary );

   /* ppsCopyrightText */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightText );

   /* ppsCopyrightNotice */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightNotice );

   /* psProducerContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psProducerContactInfo );

   /* psSupplierContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psSupplierContactInfo );

   /* psSubmissionContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psSubmissionContactInfo );

   /* psAccessInfo */
   Free_DbvAccessInfo( psDatabaseInfo->psAccessInfo );

   DbvFree( psDatabaseInfo );
   } /* End Free_DbvDatabaseInfo() */ 

void Free_DbvTagTypeMapping( DbvTagTypeMapping *psTagTypeMapping )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTagTypeMapping()\n" );
#endif

   if ( psTagTypeMapping == NULL )
      return;

   /* pcTagSet */
   FreeDbvObjectIdentifier( psTagTypeMapping->pcTagSet );

   /* pvDefaultTagType */
   Free_DbvNull( psTagTypeMapping->pvDefaultTagType );

   DbvFree( psTagTypeMapping );
   } /* End Free_DbvTagTypeMapping() */ 

void Free_DbvPath( DbvPath *psPath )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPath()\n" );
#endif

   if ( psPath == NULL )
      return;

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psPath->sTagValue );

   DbvFree( psPath );
   } /* End Free_DbvPath() */ 

void Free_DbvPathA( DbvPathA *psPathA )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPathA()\n" );
#endif

   if ( psPathA == NULL )
      return;

   /* ppsSchemaTags */
   FreeN_DbvPath( psPathA->ppsSchemaTags );

   DbvFree( psPathA );
   } /* End Free_DbvPathA() */ 

void Free_DbvElementInfo( DbvElementInfo *psElementInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvElementInfo()\n" );
#endif

   if ( psElementInfo == NULL )
      return;

   /* sElementName */
   Free_compulsory_DbvInternationalString( &psElementInfo->sElementName );

   /* ppsElementTagPath */
   FreeN_DbvPath( psElementInfo->ppsElementTagPath );

   /* psDataType */
   Free_DbvElementDataType( psElementInfo->psDataType );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementInfo->ppsDescription );

   DbvFree( psElementInfo );
   } /* End Free_DbvElementInfo() */ 

void Free_DbvElementDataType( DbvElementDataType *psElementDataType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvElementDataType()\n" );
#endif

   if ( psElementDataType == NULL )
      return;

   /* uElementDataTypeType */

   switch( psElementDataType->eElementDataTypeType )
      {
      case EDTT_Primitive:
	 break;

      case EDTT_Structured:
	 FreeN_DbvElementInfo( psElementDataType->
				uElementDataType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementDataType );
   } /* End Free_DbvElementDataType() */ 

void Free_DbvSchemaInfo( DbvSchemaInfo *psSchemaInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSchemaInfo()\n" );
#endif

   if ( psSchemaInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psSchemaInfo->psCommonInfo );

   /* pcSchema */
   FreeDbvObjectIdentifier( psSchemaInfo->pcSchema );

   /* sName */
   Free_compulsory_DbvInternationalString( &psSchemaInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psSchemaInfo->ppsDescription );

   /* ppsTagTypeMapping */
   FreeN_DbvTagTypeMapping( psSchemaInfo->ppsTagTypeMapping );

   /* ppsRecordStructure */
   FreeN_DbvElementInfo( psSchemaInfo->ppsRecordStructure );

   DbvFree( psSchemaInfo );
   } /* End Free_DbvSchemaInfo() */ 

void Free_DbvElements( DbvElements *psElements )
   {
#ifdef DEBUG
   printf( "D - Free_DbvElements()\n" );
#endif

   if ( psElements == NULL )
      return;

   /* sElementName */
   Free_compulsory_DbvInternationalString( &psElements->sElementName );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psElements->ppsNicknames );

   /* sElementTag */
   Free_compulsory_DbvStringOrNumeric( &psElements->sElementTag );

   /* ppsDescription */
   FreeN_DbvHumanString( psElements->ppsDescription );

   /* peDataType */
   Free_DbvPrimitiveDataType( psElements->peDataType );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psElements->ppsOtherInformation );

   DbvFree( psElements );
   } /* End Free_DbvElements() */ 

void Free_DbvPrimitiveDataType( DbvPrimitiveDataType *pePrimitiveDataType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPrimitiveDataType()\n" );
#endif

   if ( pePrimitiveDataType == NULL )
      return;

   DbvFree( pePrimitiveDataType );
   } /* End Free_DbvPrimitiveDataType() */ 

void Free_DbvTagSetInfo( DbvTagSetInfo *psTagSetInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTagSetInfo()\n" );
#endif

   if ( psTagSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTagSetInfo->psCommonInfo );

   /* pcTagSet */
   FreeDbvObjectIdentifier( psTagSetInfo->pcTagSet );

   /* sName */
   Free_compulsory_DbvInternationalString( &psTagSetInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTagSetInfo->ppsDescription );

   /* ppsElements */
   FreeN_DbvElements( psTagSetInfo->ppsElements );

   DbvFree( psTagSetInfo );
   } /* End Free_DbvTagSetInfo() */ 

void Free_DbvRecordSyntaxInfo( DbvRecordSyntaxInfo *psRecordSyntaxInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvRecordSyntaxInfo()\n" );
#endif

   if ( psRecordSyntaxInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psRecordSyntaxInfo->psCommonInfo );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psRecordSyntaxInfo->pcRecordSyntax );

   /* sName */
   Free_compulsory_DbvInternationalString( &psRecordSyntaxInfo->sName );

   /* ppcTransferSyntaxes */
   FreeN_DbvObjectIdentifier( psRecordSyntaxInfo->ppcTransferSyntaxes );

   /* ppsDescription */
   FreeN_DbvHumanString( psRecordSyntaxInfo->ppsDescription );

   /* psAsn1Module */
   FreeDbvInternationalString( psRecordSyntaxInfo->psAsn1Module );

   /* ppsAbstractStructure */
   FreeN_DbvElementInfo( psRecordSyntaxInfo->ppsAbstractStructure );

   DbvFree( psRecordSyntaxInfo );
   } /* End Free_DbvRecordSyntaxInfo() */ 

void Free_DbvAttributeDescription( 
		DbvAttributeDescription *psAttributeDescription )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeDescription()\n" );
#endif

   if ( psAttributeDescription == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psAttributeDescription->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeDescription->ppsDescription );

   /* sAttributeValue */
   Free_compulsory_DbvStringOrNumeric( 
				&psAttributeDescription->sAttributeValue );

   /* ppsEquivalentAttributes */
   FreeN_DbvStringOrNumeric( psAttributeDescription->ppsEquivalentAttributes );

   DbvFree( psAttributeDescription );
   } /* End Free_DbvAttributeDescription() */ 

void Free_DbvAttributeType( DbvAttributeType *psAttributeType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeType()\n" );
#endif

   if ( psAttributeType == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psAttributeType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeType->ppsDescription );

   /* ppsAttributeValues */
   FreeN_DbvAttributeDescription( psAttributeType->ppsAttributeValues );

   DbvFree( psAttributeType );
   } /* End Free_DbvAttributeType() */ 

void Free_DbvAttributeSetInfo( DbvAttributeSetInfo *psAttributeSetInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeSetInfo()\n" );
#endif

   if ( psAttributeSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psAttributeSetInfo->psCommonInfo );

   /* pcAttributeSetId */
   FreeDbvObjectIdentifier( psAttributeSetInfo->pcAttributeSetId );

   /* sName */
   Free_compulsory_DbvInternationalString( &psAttributeSetInfo->sName );

   /* ppsAttributes */
   FreeN_DbvAttributeType( psAttributeSetInfo->ppsAttributes );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeSetInfo->ppsDescription );

   DbvFree( psAttributeSetInfo );
   } /* End Free_DbvAttributeSetInfo() */ 

void Free_DbvTermList( DbvTermList *psTermList )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTermList()\n" );
#endif

   if ( psTermList == NULL )
      return;

   /* sName */
   Free_compulsory_DbvInternationalString( &psTermList->sName );

   /* ppsTitle */
   FreeN_DbvHumanString( psTermList->ppsTitle );

   /* peSearchCost */
   Free_DbvSearchCost( psTermList->peSearchCost );

   /* ppsBroader */
   FreeN_DbvInternationalString( psTermList->ppsBroader );

   /* ppsNarrower */
   FreeN_DbvInternationalString( psTermList->ppsNarrower );

   DbvFree( psTermList );
   } /* End Free_DbvTermList() */ 

void Free_DbvTermListInfo( DbvTermListInfo *psTermListInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTermListInfo()\n" );
#endif

   if ( psTermListInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTermListInfo->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psTermListInfo->sDatabaseName );

   /* ppsTermLists */
   FreeN_DbvTermList( psTermListInfo->ppsTermLists );

   DbvFree( psTermListInfo );
   } /* End Free_DbvTermListInfo() */ 

void Free_DbvSearchCost( DbvSearchCost *peSearchCost )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSearchCost()\n" );
#endif

   if ( peSearchCost == NULL )
      return;

   DbvFree( peSearchCost );
   } /* End Free_DbvSearchCost() */ 

void Free_DbvExtendedServicesInfo( 
			DbvExtendedServicesInfo *psExtendedServicesInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvExtendedServicesInfo()\n" );
#endif

   if ( psExtendedServicesInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psExtendedServicesInfo->psCommonInfo );

   /* pcType */
   FreeDbvObjectIdentifier( psExtendedServicesInfo->pcType );

   /* psName */
   FreeDbvInternationalString( psExtendedServicesInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psExtendedServicesInfo->ppsDescription );

   /* psSpecificExplain */
   FreeDbvASN1External( psExtendedServicesInfo->psSpecificExplain );

   /* psEsASN */
   FreeDbvInternationalString( psExtendedServicesInfo->psEsASN );

   DbvFree( psExtendedServicesInfo );
   } /* End Free_DbvExtendedServicesInfo() */ 

void Free_DbvOmittedAttributeInterpretation( 
	DbvOmittedAttributeInterpretation *psOmittedAttributeInterpretation )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOmittedAttributeInterpretation()\n" );
#endif

   if ( psOmittedAttributeInterpretation == NULL )
      return;

   /* psDefaultValue */
   Free_DbvStringOrNumeric( psOmittedAttributeInterpretation->
			    psDefaultValue );

   /* ppsDefaultDescription */
   FreeN_DbvHumanString( psOmittedAttributeInterpretation->
			 ppsDefaultDescription );

   DbvFree( psOmittedAttributeInterpretation );
   } /* End Free_DbvOmittedAttributeInterpretation() */ 

void Free_DbvAttributeValues( DbvAttributeValues *psAttributeValues )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeValues()\n" );
#endif

   if ( psAttributeValues == NULL )
      return;

   /* sValue */
   Free_compulsory_DbvStringOrNumeric( &psAttributeValues->sValue );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeValues->ppsDescription );

   /* ppsSubAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSubAttributes );

   /* ppsSuperAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSuperAttributes );

   /* pvPartialSupport */
   Free_DbvNull( psAttributeValues->pvPartialSupport );

   DbvFree( psAttributeValues );
   } /* End Free_DbvAttributeValues() */ 

void Free_DbvAttributesByType( DbvAttributesByType *psAttributesByType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributesByType()\n" );
#endif

   if ( psAttributesByType == NULL )
      return;

   /* psDefaultIfOmitted */
   Free_DbvOmittedAttributeInterpretation( psAttributesByType->psDefaultIfOmitted );

   /* ppsAttributeValues */
   FreeN_DbvAttributeValues( psAttributesByType->ppsAttributeValues );

   DbvFree( psAttributesByType );
   } /* End Free_DbvAttributesByType() */ 

void Free_DbvAttributesBySet( DbvAttributesBySet *psAttributesBySet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributesBySet()\n" );
#endif

   if ( psAttributesBySet == NULL )
      return;

   /* pcAttributeSet */
   FreeDbvObjectIdentifier( psAttributesBySet->pcAttributeSet );

   /* ppsAttributesByType */
   FreeN_DbvAttributesByType( psAttributesBySet->ppsAttributesByType );

   DbvFree( psAttributesBySet );
   } /* End Free_DbvAttributesBySet() */ 

void Free_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeOccurrenceChoice()\n" );
#endif

   if ( psAttributeOccurrenceChoice != NULL )
      {
      do_Free_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );
      DbvFree( psAttributeOccurrenceChoice );
      } /* End if */

   } /* End Free_DbvAttributeOccurrenceChoice() */ 

void do_Free_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUG
   printf( "D - do_Free_DbvAttributeOccurrenceChoice()\n" );
#endif

   if ( psAttributeOccurrenceChoice == NULL )
      return;

   /* uAttributeOccurrenceChoiceType */

   switch( psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType )
      {
      case AOT_AnyOrNone:
	 Free_DbvNull( psAttributeOccurrenceChoice->
		       uAttributeOccurrenceChoiceType.
		       pvAnyOrNone );
	 break;

      case AOT_Specific:
	 FreeN_DbvStringOrNumeric( psAttributeOccurrenceChoice->
				       uAttributeOccurrenceChoiceType.
				       ppsSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_DbvAttributeOccurrenceChoice() */ 

void Free_DbvAttributeOccurrence(
			DbvAttributeOccurrence *psAttributeOccurrence )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeOccurrence()\n" );
#endif

   if ( psAttributeOccurrence == NULL )
      return;

   /* pcAttributeSet */
   FreeDbvObjectIdentifier( psAttributeOccurrence->pcAttributeSet );

   /* pvMustBeSupplied */
   Free_DbvNull( psAttributeOccurrence->pvMustBeSupplied );

   /* sAttributeValues */
   Free_compulsory_DbvAttributeOccurrenceChoice(
				&psAttributeOccurrence->sAttributeValues );

   DbvFree( psAttributeOccurrence );
   } /* End Free_DbvAttributeOccurrence() */ 

void Free_DbvAttributeCombination( 
			DbvAttributeCombination *psAttributeCombination )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeCombination()\n" );
#endif

   if ( psAttributeCombination == NULL )
      return;

   /* ppsAttributeOccurrence */
   FreeN_DbvAttributeOccurrence( 
			psAttributeCombination->ppsAttributeOccurrence );

   DbvFree( psAttributeCombination );
   } /* End Free_DbvAttributeCombination() */ 

void Free_DbvAttributeCombinations( 
			DbvAttributeCombinations *psAttributeCombinations )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeCombinations()\n" );
#endif

   if ( psAttributeCombinations == NULL )
      return;

   /* pcDefaultAttributeSet */
   FreeDbvObjectIdentifier( psAttributeCombinations->pcDefaultAttributeSet );

   /* ppsLegalCombinations */
   FreeN_DbvAttributeCombination( psAttributeCombinations->
				   ppsLegalCombinations );

   DbvFree( psAttributeCombinations );
   } /* End Free_DbvAttributeCombinations() */ 

void Free_DbvAttributeDetails( DbvAttributeDetails *psAttributeDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAttributeDetails()\n" );
#endif

   if ( psAttributeDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psAttributeDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psAttributeDetails->sDatabaseName );

   /* ppsAttributesBySet */
   FreeN_DbvAttributesBySet( psAttributeDetails->ppsAttributesBySet );

   /* psAttributeCombinations */
   Free_DbvAttributeCombinations( psAttributeDetails->psAttributeCombinations );

   DbvFree( psAttributeDetails );
   } /* End Free_DbvAttributeDetails() */ 

void Free_DbvScanInfo( DbvScanInfo *psScanInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvScanInfo()\n" );
#endif

   if ( psScanInfo == NULL )
      return;

   /* pulMaxStepSize */
   Free_Ulong( psScanInfo->pulMaxStepSize );

   /* ppsCollatingSequence */
   FreeN_DbvHumanString( psScanInfo->ppsCollatingSequence );

   /* pfIncreasing */
   Free_DbvBool( psScanInfo->pfIncreasing );

   DbvFree( psScanInfo );
   } /* End Free_DbvScanInfo() */ 

void Free_DbvTermListDetails( DbvTermListDetails *psTermListDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTermListDetails()\n" );
#endif

   if ( psTermListDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTermListDetails->psCommonInfo );

   /* sTermListName */
   Free_compulsory_DbvInternationalString( &psTermListDetails->sTermListName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTermListDetails->ppsDescription );

   /* psAttributes */
   Free_DbvAttributeCombinations( psTermListDetails->psAttributes );

   /* psScanInfo */
   Free_DbvScanInfo( psTermListDetails->psScanInfo );

   /* pulEstNumberTerms */
   Free_Ulong( psTermListDetails->pulEstNumberTerms );

   /* ppsSampleTerms */
   FreeN_DbvTerm( psTermListDetails->ppsSampleTerms );

   DbvFree( psTermListDetails );
   } /* End Free_DbvTermListDetails() */ 

void Free_DbvRecordTag( DbvRecordTag *psRecordTag )
   {
#ifdef DEBUG
   printf( "D - Free_DbvRecordTag()\n" );
#endif

   if ( psRecordTag == NULL )
      return;

   /* psQualifier */
   Free_DbvStringOrNumeric( psRecordTag->psQualifier );

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psRecordTag->sTagValue );

   DbvFree( psRecordTag );
   } /* End Free_DbvRecordTag() */ 

void Free_DbvDetailsPerElement( DbvDetailsPerElement *psDetailsPerElement )
   {
#ifdef DEBUG
   printf( "D - Free_DbvDetailsPerElement()\n" );
#endif

   if ( psDetailsPerElement == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psDetailsPerElement->psName );

   /* psRecordTag */
   Free_DbvRecordTag( psDetailsPerElement->psRecordTag );

   /* ppsSchemaTags */
   FreeN_DbvPathA( psDetailsPerElement->ppsSchemaTags );

   /* pulMaxSize */
   Free_Ulong(  psDetailsPerElement->pulMaxSize );

   /* pulMinSize */
   Free_Ulong(  psDetailsPerElement->pulMinSize );

   /* pulAvgSize */
   Free_Ulong(  psDetailsPerElement->pulAvgSize );

   /* pulFixedSize */
   Free_Ulong(  psDetailsPerElement->pulFixedSize );

   /* ppsDescription*/
   FreeN_DbvHumanString( psDetailsPerElement->ppsDescription );

   /* ppsContents */
   FreeN_DbvHumanString( psDetailsPerElement->ppsContents );

   /* ppsBillingInfo */
   FreeN_DbvHumanString( psDetailsPerElement->ppsBillingInfo );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psDetailsPerElement->ppsRestrictions );

   /* ppsAlternateNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsAlternateNames );

   /* ppsGenericNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsGenericNames );

   /* psSearchAccess */
   Free_DbvAttributeCombinations( psDetailsPerElement->psSearchAccess );

   DbvFree( psDetailsPerElement );
   } /* End Free_DbvDetailsPerElement() */ 

void Free_DbvElementSetDetails( DbvElementSetDetails *psElementSetDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvElementSetDetails()\n" );
#endif

   if ( psElementSetDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psElementSetDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psElementSetDetails->sDatabaseName);

   /* sElementSetName */
   Free_compulsory_DbvInternationalString( 
					&psElementSetDetails->sElementSetName );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psElementSetDetails->pcRecordSyntax );

   /* pcSchema */
   FreeDbvObjectIdentifier( psElementSetDetails->pcSchema );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementSetDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psElementSetDetails->ppsDetailsPerElement );

   DbvFree( psElementSetDetails );
   } /* End Free_DbvElementSetDetails() */ 

void Free_DbvRetrievalRecordDetails( 
			DbvRetrievalRecordDetails *psRetrievalRecordDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvRetrievalRecordDetails()\n" );
#endif

   if ( psRetrievalRecordDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psRetrievalRecordDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psRetrievalRecordDetails->
					   sDatabaseName);
   /* pcSchema */
   FreeDbvObjectIdentifier( psRetrievalRecordDetails->pcSchema );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psRetrievalRecordDetails->pcRecordSyntax );

   /* ppsDescription */
   FreeN_DbvHumanString( psRetrievalRecordDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psRetrievalRecordDetails->ppsDetailsPerElement );

   DbvFree( psRetrievalRecordDetails );
   } /* End Free_DbvRetrievalRecordDetails() */ 

void Free_DbvSortType( DbvSortType *psSortType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSortType()\n" );
#endif

   if ( psSortType == NULL )
      return;

   /* uSortTypeType */

   switch( psSortType->eSortTypeType )
      {
      case STT_Structured:
	 FreeN_DbvHumanString( psSortType->uSortType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSortType );
   } /* End Free_DbvSortType() */ 

void Free_DbvCaseSensitivity( DbvCaseSensitivity *peCaseSensitivity )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCaseSensitivity()\n" );
#endif

   if ( peCaseSensitivity == NULL )
      return;

   DbvFree( peCaseSensitivity );
   } /* End Free_DbvCaseSensitivity() */ 

void Free_DbvSortKeys( DbvSortKeys *psSortKeys )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSortKeys()\n" );
#endif

   if ( psSortKeys == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psSortKeys->ppsDescription );

   /* ppsElementSpecifications */
   FreeN_DbvSpecification( psSortKeys->ppsElementSpecifications );

   /* psAttributeSpecifications */
   Free_DbvAttributeCombinations( psSortKeys->psAttributeSpecifications );

   /* psSortType */
   Free_DbvSortType( psSortKeys->psSortType );

   /* peCaseSensitivity */
   Free_DbvCaseSensitivity( psSortKeys->peCaseSensitivity );
   
   DbvFree( psSortKeys );
   } /* End Free_DbvSortKeys() */ 

void Free_DbvSortDetails( DbvSortDetails *psSortDetails )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSortDetails()\n" );
#endif

   if ( psSortDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psSortDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psSortDetails->sDatabaseName );

   /* ppsSortKeys */
   FreeN_DbvSortKeys( psSortDetails->ppsSortKeys );

   DbvFree( psSortDetails );
   } /* End Free_DbvSortDetails() */ 

void Free_DbvProcessingInformation( 
			DbvProcessingInformation *psProcessingInformation )
   {
#ifdef DEBUG
   printf( "D - Free_DbvProcessingInformation()\n" );
#endif

   if ( psProcessingInformation == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psProcessingInformation->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( 
				&psProcessingInformation->sDatabaseName );
   /* sName */
   Free_compulsory_DbvInternationalString( &psProcessingInformation->sName );

   /* pcOid */
   FreeDbvObjectIdentifier( psProcessingInformation->pcOid );

   /* ppsDescription */
   FreeN_DbvHumanString( psProcessingInformation->ppsDescription );

   /* psInstructions */
   FreeDbvASN1External( psProcessingInformation->psInstructions );

   DbvFree( psProcessingInformation );
   } /* End Free_DbvProcessingInformation() */ 

void Free_DbvValueDescription( DbvValueDescription *psValueDescription )
   {
#ifdef DEBUG
   printf( "D - Free_DbvValueDescription()\n" );
#endif

   if ( psValueDescription == NULL )
      return;

   /* uValueDescriptionType */

   switch( psValueDescription->eValueDescriptionType )
      {
      case VDT_Integer:
	 break;

      case VDT_String:
	 FreeDbvInternationalString( psValueDescription->
					uValueDescription.psString );
	 break;

      case VDT_Octets:
	 FreeDbvOctetString( psValueDescription->
					uValueDescription.psOctets );
	 break;

      case VDT_Oid:
	 FreeDbvObjectIdentifier( psValueDescription->
					uValueDescription.pcOid );
	 break;

      case VDT_Unit:
	 Free_DbvUnit( psValueDescription->
					uValueDescription.psUnit );
	 break;

      case VDT_ValueAndUnit:
	 Free_DbvIntUnit( psValueDescription->
					uValueDescription.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueDescription );
   } /* End Free_DbvValueDescription() */ 

void Free_DbvValueRange( DbvValueRange *psValueRange )
   {
#ifdef DEBUG
   printf( "D - Free_DbvValueRange()\n" );
#endif

   if ( psValueRange == NULL )
      return;

   /* psLower */
   Free_DbvValueDescription( psValueRange->psLower );

   /* psUpper */
   Free_DbvValueDescription( psValueRange->psUpper );

   DbvFree( psValueRange );
   } /* End Free_DbvValueRange() */ 

void Free_DbvValueSet( DbvValueSet *psValueSet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvValueSet()\n" );
#endif

   if ( psValueSet == NULL )
      return;

   /* uValueSetType */

   switch( psValueSet->eValueSetType )
      {
      case VT_Range:
	 Free_DbvValueRange( psValueSet->uValueSet.psRange );
	 break;

      case VT_Enumerated:
	 FreeN_DbvValueDescription( psValueSet->uValueSet.ppsEnumerated );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueSet );
   } /* End Free_DbvValueSet() */ 

void Free_DbvVariantValue( DbvVariantValue *psVariantValue )
   {
#ifdef DEBUG
   printf( "D - Free_DbvVariantValue()\n" );
#endif

   if ( psVariantValue == NULL )
      return;

   /* psValues */
   Free_DbvValueSet( psVariantValue->psValues );

   DbvFree( psVariantValue );
   } /* End Free_DbvVariantValue() */ 

void Free_DbvVariantTypes( DbvVariantTypes *psVariantTypes )
   {
#ifdef DEBUG
   printf( "D - Free_DbvVariantTypes()\n" );
#endif

   if ( psVariantTypes == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psVariantTypes->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantTypes->ppsDescription );

   /* psVariantValue */
   Free_DbvVariantValue( psVariantTypes->psVariantValue );

   DbvFree( psVariantTypes );
   } /* End Free_DbvVariantTypes() */ 

void Free_DbvVariantClass( DbvVariantClass *psVariantClass )
   {
#ifdef DEBUG
   printf( "D - Free_DbvVariantClass()\n" );
#endif

   if ( psVariantClass == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psVariantClass->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantClass->ppsDescription );

   /* ppsVariantTypes */
   FreeN_DbvVariantTypes( psVariantClass->ppsVariantTypes );

   DbvFree( psVariantClass );
   } /* End Free_DbvVariants() */ 

void Free_DbvVariantSetInfo( DbvVariantSetInfo *psVariantSetInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvVariantSetInfo()\n" );
#endif

   if ( psVariantSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psVariantSetInfo->psCommonInfo );

   /* pcVariantSet */
   FreeDbvObjectIdentifier( psVariantSetInfo->pcVariantSet );

   /* sName */
   Free_compulsory_DbvInternationalString( &psVariantSetInfo->sName );

   /* ppsVariants */
   FreeN_DbvVariantClass( psVariantSetInfo->ppsVariants );

   DbvFree( psVariantSetInfo );
   } /* End Free_DbvVariantSetInfo() */ 

void Free_DbvUnits( DbvUnits *psUnits )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUnits()\n" );
#endif

   if ( psUnits == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psUnits->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnits->ppsDescription );

   /* sUnit */
   Free_compulsory_DbvStringOrNumeric( &psUnits->sUnit );

   DbvFree( psUnits );
   } /* End Free_DbvUnits() */ 

void Free_DbvUnitType( DbvUnitType *psUnitType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUnitType()\n" );
#endif

   if ( psUnitType == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psUnitType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitType->ppsDescription );

   /* sUnitType */
   Free_compulsory_DbvStringOrNumeric( &psUnitType->sUnitType );

   /* ppsUnits */
   FreeN_DbvUnits( psUnitType->ppsUnits );

   DbvFree( psUnitType );
   } /* End Free_DbvUnitType() */ 

void Free_DbvUnitInfo( DbvUnitInfo *psUnitInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUnitInfo()\n" );
#endif

   if ( psUnitInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psUnitInfo->psCommonInfo );

   /* sUnitSystem */
   Free_compulsory_DbvInternationalString( &psUnitInfo->sUnitSystem );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitInfo->ppsDescription );

   /* ppsUnitTypes */
   FreeN_DbvUnitType( psUnitInfo->ppsUnitTypes );

   DbvFree( psUnitInfo );
   } /* End Free_DbvUnitInfo() */ 

void Free_DbvCategoryInfo( DbvCategoryInfo *psCategoryInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCategoryInfo()\n" );
#endif

   if ( psCategoryInfo == NULL )
      return;

   /* sCategory */
   Free_compulsory_DbvInternationalString( &psCategoryInfo->sCategory );

   /* psOriginalCategory */
   FreeDbvInternationalString( psCategoryInfo->psOriginalCategory );

   /* ppsDescription */
   FreeN_DbvHumanString( psCategoryInfo->ppsDescription );

   /* psAsn1Module */
   FreeDbvInternationalString( psCategoryInfo->psAsn1Module );

   DbvFree( psCategoryInfo );
   } /* End Free_DbvCategorInfo() */ 

void Free_DbvCategoryList( DbvCategoryList *psCategoryList )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCategoryList()\n" );
#endif

   if ( psCategoryList == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psCategoryList->psCommonInfo );

   /* ppsCategories */
   FreeN_DbvCategoryInfo( psCategoryList->ppsCategories );

   DbvFree( psCategoryList );
   } /* End Free_DbvCategoryList() */ 

void Free_DbvExplainRecord( DbvExplainRecord *psExplainRecord )
   {
#ifdef DEBUG
   printf( "D - Free_DbvExplainRecord()\n" );
#endif

   if ( psExplainRecord == NULL )
      return;

   /* uExplainRecordType */

   switch( psExplainRecord->eExplainRecordType )
      {
      case ERT_TargetInfo:
	 Free_DbvTargetInfo( psExplainRecord->uExplainRecord.psTargetInfo);
	 break;

      case ERT_DatabaseInfo:
	 Free_DbvDatabaseInfo( psExplainRecord->
					uExplainRecord.psDatabaseInfo);
	 break;

      case ERT_SchemaInfo:
	 Free_DbvSchemaInfo( psExplainRecord->
					uExplainRecord.psSchemaInfo);
	 break;

      case ERT_TagSetInfo:
	 Free_DbvTagSetInfo( psExplainRecord->
					uExplainRecord.psTagSetInfo);
	 break;

      case ERT_RecordSyntaxInfo:
	 Free_DbvRecordSyntaxInfo( psExplainRecord->
					uExplainRecord.psRecordSyntaxInfo);
	 break;

      case ERT_AttributeSetInfo:
	 Free_DbvAttributeSetInfo( psExplainRecord->
					uExplainRecord.psAttributeSetInfo);
	 break;

      case ERT_TermListInfo:
	 Free_DbvTermListInfo( psExplainRecord->
					uExplainRecord.psTermListInfo);
	 break;

      case ERT_ExtendedServicesInfo:
	 Free_DbvExtendedServicesInfo( psExplainRecord->
					uExplainRecord.psExtendedServicesInfo);
	 break;

      case ERT_AttributeDetails:
	 Free_DbvAttributeDetails( psExplainRecord->
					uExplainRecord.psAttributeDetails);
	 break;

      case ERT_TermListDetails:
	 Free_DbvTermListDetails( psExplainRecord->
					uExplainRecord.psTermListDetails);
	 break;

      case ERT_ElementSetDetails:
	 Free_DbvElementSetDetails( psExplainRecord->
					uExplainRecord.psElementSetDetails);
	 break;

      case ERT_RetrievalRecordDetails:
	 Free_DbvRetrievalRecordDetails( psExplainRecord->
			 	uExplainRecord.psRetrievalRecordDetails);
	 break;

      case ERT_SortDetails:
	 Free_DbvSortDetails( psExplainRecord->
					uExplainRecord.psSortDetails);
	 break;

      case ERT_Processing:
	 Free_DbvProcessingInformation( psExplainRecord->
					uExplainRecord.psProcessing);
	 break;

      case ERT_Variants:
	 Free_DbvVariantSetInfo( psExplainRecord->
					uExplainRecord.psVariants);
	 break;

      case ERT_Units:
	 Free_DbvUnitInfo( psExplainRecord->
					uExplainRecord.psUnits);
	 break;

      case ERT_CategoryList:
	 Free_DbvCategoryList( psExplainRecord->
					uExplainRecord.psCategoryList);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psExplainRecord );
   } /* End Free_DbvExplainRecord() */ 

void Free_DbvLanguageCode( DbvLanguageCode *psLanguageCode )
   {
#ifdef DEBUG
   printf( "D - Free_DbvLanguageCode()\n" );
#endif

   FreeDbvOctetString( (DbvOctetString*)psLanguageCode );

   } /* End Free_DbvLanguageCode() */ 

void Free_DbvNull( DbvNull *psNull )
   {
#ifdef DEBUG
   printf( "D - Free_DbvNull()\n" );
#endif

   if ( psNull == NULL )
      return;

   DbvFree( psNull );

   } /* End Free_DbvNull() */ 

void Free_DbvUnit( DbvUnit *psUnit )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUnit()\n" );
#endif

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */
   FreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */
   Free_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */
   Free_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */
   Free_Ulong( psUnit->pulScaleFactor );

   DbvFree( psUnit );
   } /* End Free_DbvUnit() */ 

void Free_DbvTerm( DbvTerm *psTerm )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTerm()\n" );
#endif

   if ( psTerm == NULL )
      return;

   /* uTermType */

   switch( psTerm->eTermType )
      {
      case TT_General:
	 FreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
	 break;

      case TT_SimpleCharacter:
	 FreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter );
	 break;

      case TT_ObjectIdentifier:
	 FreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
	 Free_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
	 FreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 Free_DbvIntUnit( psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
      default:
	 break;
      } /* End switch */

   DbvFree( psTerm );
   } /* End Free_DbvTerm() */ 

void Free_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq *psDataBaseInfoSeq )
   {
#ifdef DEBUG
   printf( "D - Free_DbvDataBaseInfoSeq()\n" );
#endif

   if ( psDataBaseInfoSeq == NULL )
      return;

   /* ppsDataBaseInfo */
   FreeN_DbvHumanString( psDataBaseInfoSeq->ppsDataBaseInfo );

   DbvFree( psDataBaseInfoSeq );
   } /* End Free_DbvDataBaseInfoSeq() */ 

