/*  Module Name: @(#)Free_DbvFuncs3.c	7.1
    Version: 11/15/96
    Author: Dan Maskell
 
    Crossnet Systems Ltd, 
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for structure types.
        Used by the origin and target.

    Modification History.

 
*/

#include "Free_DbvFuncs.h"

#ifdef NOT_YET_IMPLEMENTED

void Free_DbvOrder( DbvOrder *psOrder )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOrder()\n" );
#endif

   if ( psOrder == NULL )
      return;

   DbvFree( psOrder );
   } /* End Free_DbvOrder() */ 

void Free_DbvUsage( DbvUsage *psUsage )
   {
#ifdef DEBUG
   printf( "D - Free_DbvUsage()\n" );
#endif

   if ( psUsage == NULL )
      return;

   /* psRestriction */
   FreeDbvInternationalString( psUsage->psRestriction );

   DbvFree( psUsage );
   } /* End Free_DbvUsage() */ 

void Free_DbvHits( DbvHits *psHits )
   {
#ifdef DEBUG
   printf( "D - Free_DbvHits()\n" );
#endif

   if ( psHits == NULL )
      return;

   /* psSatisfier */
   Free_DbvTerm( psHits->psSatisfier );

   /* psOffsetIntoElement */
   Free_DbvIntUnit( psHits->psOffsetIntoElement );

   /* psLength */
   Free_DbvIntUnit( psHits->psLength );

   /* pulHitRank */
   Free_Ulong( psHits->pulHitRank );

   /* psTargetToken */
   FreeDbvOctetString( psHits->psTargetToken );

   DbvFree( psHits );
   } /* End Free_DbvHits() */ 



void Free_DbvValue( DbvValue *psValue )
   {
#ifdef DEBUG
   printf( "D - Free_DbvValue()\n" );
#endif

   if ( psValue == NULL )
      return;

   /* uValueType */

   switch( psValue->eVariantValueType )
      {
      case VVT_Int:
	 break;

      case VVT_InternationalString:
	 FreeDbvInternationalString( psValue->
				      uVariantValueType.psInternationalString );
	 break;

      case VVT_Octs:
	 FreeDbvOctetString( psValue->uVariantValueType.psOcts );
	 break;

      case VVT_Oid:
	 FreeDbvObjectIdentifier( psValue->uVariantValueType.pcOid );
	 break;

      case VVT_Bool:
      case VVT_Null:
	 break;

      case VVT_Unit:
	 Free_DbvUnit( psValue->uVariantValueType.psUnit );
	 break;

      case VVT_ValueAndUnit:
	 Free_DbvIntUnit( psValue->uVariantValueType.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValue );
   } /* End Free_DbvValue() */ 



void Free_DbvTriples( DbvTriples *psTriples )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTriples()\n" );
#endif

   if ( psTriples == NULL )
      return;

   /* pcVariantSetId */
   FreeDbvObjectIdentifier( psTriples->pcVariantSetId );

   /* ppsValue */
   FreeN_DbvValue( psTriples->ppsValue );

   DbvFree( psTriples );
   } /* End Free_DbvTriples() */ 

void Free_DbvVariant( DbvVariant *psVariant )
   {
#ifdef DEBUG
   printf( "D - Free_DbvVariant()\n" );
#endif

   if ( psVariant == NULL )
      return;

   /* pcGlobalVariantSetId */
   FreeDbvObjectIdentifier( psVariant->pcGlobalVariantSetId );

   /* ppsTriples */
   FreeN_DbvTriples( psVariant->ppsTriples );

   DbvFree( psVariant );
   } /* End Free_DbvVariant() */ 

void Free_DbvTagPath( DbvTagPath *psTagPath )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTagPath()\n" );
#endif

   if ( psTagPath == NULL )
      return;

   /* pulTagType */
   Free_Ulong( psTagPath->pulTagType );

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psTagPath->sTagValue );

   /* pulTagOccurrence */
   Free_Ulong( psTagPath->pulTagOccurrence );

   DbvFree( psTagPath );
   } /* End Free_DbvTagPath() */ 

void Free_DbvElementMetaData( DbvElementMetaData *psElementMetaData )
   {
#ifdef DEBUG
   printf( "D - Free_DbvElementMetaData()\n" );
#endif

   if ( psElementMetaData == NULL )
      return;

   /* psSeriesOrder */
   Free_DbvOrder( psElementMetaData->psSeriesOrder );

   /* psUsageRight */
   Free_DbvUsage( psElementMetaData->psUsageRight );

   /* ppsHits */
   FreeN_DbvHits( psElementMetaData->ppsHits );

   /* psDisplayName */
   FreeDbvInternationalString( psElementMetaData->psDisplayName );

   /* ppsSupportedVariants */
   FreeN_DbvVariant( psElementMetaData->ppsSupportedVariants );

   /* psMessage */
   FreeDbvInternationalString( psElementMetaData->psMessage );

   /* psElementDescriptor */
   FreeDbvOctetString( psElementMetaData->psElementDescriptor );

   /* ppsSurrogateFor */
   FreeN_DbvTagPath( psElementMetaData->ppsSurrogateFor );

   /* ppsSurrogateElement */
   FreeN_DbvTagPath( psElementMetaData->ppsSurrogateElement );

   /* psOther */
   FreeDbvASN1External( psElementMetaData->psOther );

   DbvFree( psElementMetaData );
   } /* End Free_DbvElementMetaData() */ 

void Free_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

#ifdef DEBUG
   printf( "D - Free_DbvElementDataG()\n" );
#endif

   if ( psElementDataG == NULL )
      {
      do_Free_DbvElementDataG( psElementDataG );
      DbvFree( psElementDataG );
      } /* End if */

   } /* End Free_DbvElementData() */ 

void do_Free_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

#ifdef DEBUG
   printf( "D - do_Free_DbvElementDataG()\n" );
#endif

   /* uElementDataGType */

   switch( psElementDataG->eElementDataGType )
      {
      case EDT_Octets:
	 FreeDbvOctetString( psElementDataG->uElementDataG.psOctets );
	 break;

      case EDT_Numeric:
	 break;

      case EDT_Date:
	 Free_DbvGeneralizedTime( psElementDataG->uElementDataG.pcDate );
	 break;

      case EDT_Ext:
	 FreeDbvASN1External( psElementDataG->uElementDataG.psExt );
	 break;

      case EDT_String:
	 FreeDbvInternationalString( psElementDataG->uElementDataG.psString );
	 break;

      case EDT_TrueOrFalse:
	 break;

      case EDT_Oid:
	 FreeDbvObjectIdentifier( psElementDataG->uElementDataG.pcOid );
	 break;

      case EDT_IntUnit:
	 Free_DbvIntUnit( psElementDataG->uElementDataG.psIntUnit);
	 break;

      case EDT_ElementNotThere:
      case EDT_ElementEmpty:
      case EDT_NoDataRequested:
	 break;

      case EDT_Diagnostic:
	 FreeDbvASN1External( psElementDataG->uElementDataG.psDiagnostic );
	 break;

      case EDT_Subtree:
	 FreeN_DbvTaggedElement( psElementDataG->uElementDataG.ppsSubTree);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_DbvElementDataG() */ 

void Free_DbvTaggedElement( DbvTaggedElement *psTaggedElement )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTaggedElement()\n" );
#endif

   if ( psTaggedElement == NULL )
      return;


   /* pulTagType */
   Free_Ulong( psTaggedElement->pulTagType );

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psTaggedElement->sTagValue );

   /* pulTagOccurrence */
   Free_Ulong( psTaggedElement->pulTagOccurrence );

   /* sContent */
   Free_compulsory_DbvElementDataG( &psTaggedElement->sContent );

   /* psMetaData */
   Free_DbvElementMetaData( psTaggedElement->psMetaData );

   /* psAppliedVariant */
   Free_DbvVariant( psTaggedElement->psAppliedVariant );

   DbvFree( psTaggedElement );
   } /* End Free_DbvTaggedElement() */ 

void Free_DbvGenericRecord( DbvGenericRecord *psGenericRecord )
   {
#ifdef DEBUG
   printf( "D - Free_DbvGenericRecord()\n" );
#endif

   if ( psGenericRecord == NULL )
      return;

   /* ppsTaggedElement */
   FreeN_DbvTaggedElement( psGenericRecord->ppsTaggedElement );

   DbvFree( psGenericRecord );
   } /* End Free_DbvGenericRecord() */ 

#endif
