/*  Module Name: @(#)Free_DbvFuncs4.c	7.1
    Version: 11/15/96
    Author: John Hough
 
    Crossnet Systems Ltd, 
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for structure types.
        Used by the origin and target.

    Modification History.

*/


#include "Free_DbvFuncs.h"



/*-------------------- ACCESSCONTROL --------------------*/

void Free_DbvSecurityChallengeRsp( DbvSecurityChallengeRsp *psSecurityChallengeRsp )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSecurityChallengeRsp()\n" );
#endif

   if ( psSecurityChallengeRsp == NULL )
      return;

   /* uSecurityChallengeRsp */

   switch( psSecurityChallengeRsp->eSecurityChallengeRspType ) 
      {
      case SECT_SimpleForm:
   	 FreeDbvOctetString( psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 FreeDbvASN1External( psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSecurityChallengeRsp );
   } /* End Free_DbvSecurityChallengeRsp */



void Free_DbvAccessControlReq( DbvAccessControlReq *psAccessControlReq )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAccessControlReq()\n" );
#endif

   if ( psAccessControlReq == NULL )
      return;

   /* psReferenceId */  
   FreeDbvOctetString( psAccessControlReq->psReferenceId );

   /* sSecurityChallengeReq */
   Free_compulsory_DbvSecurityChallengeReq( &psAccessControlReq->sSecurityChallengeReq );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlReq->ppsOtherInformation );

   DbvFree( psAccessControlReq );
   } /* End Free_DbvAccessControlReq() */ 



void Free_DbvAccessControlRsp( DbvAccessControlRsp *psAccessControlRsp )
   {
#ifdef DEBUG
   printf( "D - Free_DbvAccessControlRsp()\n" );
#endif

   if ( psAccessControlRsp == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psAccessControlRsp->psReferenceId );

   /* psSecurityChallengeRsp */
   Free_DbvSecurityChallengeRsp( psAccessControlRsp->psSecurityChallengeRsp );

   /* psDiagnostic */
   Free_DbvDiagRec( psAccessControlRsp->psDiagnostic );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlRsp->ppsOtherInformation );

   DbvFree( psAccessControlRsp );
   } /* End Free_DbvAccessControlRsp() */ 



/* -------------- Access Control Format "Prompt 1" ----------------*/


void Free_DbvEnumeratedPrompt( DbvEnumeratedPrompt *psEnumeratedPrompt )
   {
#ifdef DEBUG
   printf( "D - Free_DbvEnumeratedPrompt()\n" );
#endif

   if ( psEnumeratedPrompt == NULL )
      return;


   /* compulsory eType */

   /* psSuggestedString */
   FreeDbvInternationalString( psEnumeratedPrompt->psSuggestedString );

   DbvFree( psEnumeratedPrompt );
   } /* End Free_DbvEnumeratedPrompt() */ 



void Free_DbvEncryption( DbvEncryption *psEncryption )
   {
#ifdef DEBUG
   printf( "D - Free_DbvEncryption()\n" );
#endif

   if ( psEncryption == NULL )
      return;

   /* psCryptType */
   FreeDbvOctetString( psEncryption->psCryptType );

   /* psCredential */
   FreeDbvOctetString( psEncryption->psCredential );

   /* sData */
   Free_compulsory_DbvOctetString( &psEncryption->sData);

   DbvFree( psEncryption );
   } /* End Free_DbvEncryption() */ 



void Free_DbvPromptInfo( DbvPromptInfo *psPromptInfo )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPromptInfo()\n" );
#endif

   if ( psPromptInfo == NULL )
      return;


   /* uPromptInfo */

   switch( psPromptInfo->ePromptInfoType ) 
      {
      case PIIT_Character:
   	 FreeDbvInternationalString( psPromptInfo->uPromptInfo.psCharacter );
	 break;

      case PIIT_Encrypted:
         Free_DbvEncryption( psPromptInfo->uPromptInfo.psEncrypted );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptInfo );
   } /* End Free_DbvPromptInfo */



void Free_DbvDataType( DbvDataType *peDataType )
   {
#ifdef DEBUG
   printf( "D - Free_DbvDataType()\n" );
#endif
     
   if ( peDataType == NULL )
      return;

   DbvFree( peDataType );
   } /* End Free_DbvDataType() */



void Free_DbvChallenge( DbvChallenge *psChallenge )
   {
#ifdef DEBUG
   printf( "D - Free_DbvChallenge()\n" );
#endif

   if ( psChallenge == NULL )
      return;

   /* sPromptId */
   Free_compulsory_DbvPromptId( &psChallenge->sPromptId );

   /* psDefaultResponse */
   FreeDbvInternationalString( psChallenge->psDefaultResponse );

   /* psPromptInfo */
   Free_DbvPromptInfo( psChallenge->psPromptInfo );

   /* psRegExpr */
   FreeDbvInternationalString( psChallenge->psRegExpr );

   /* pvResponseRequired */
   Free_DbvNull( psChallenge->pvResponseRequired );

   /* ppsAllowedValues */
   FreeN_DbvInternationalString( psChallenge->ppsAllowedValues );

   /* pvShouldSave */
   Free_DbvNull( psChallenge->pvShouldSave );

   /* peDataType */
   Free_DbvDataType( psChallenge->peDataType );

   /* psDiagnostic */
   FreeDbvASN1External( psChallenge->psDiagnostic );

   DbvFree( psChallenge );
   } /* End Free_DbvChallenge() */ 



void Free_DbvResponse( DbvResponse *psResponse )
   {
#ifdef DEBUG
   printf( "D - Free_DbvResponse()\n" );
#endif

   if ( psResponse == NULL )
      return;

   /* sPromptId */
   Free_compulsory_DbvPromptId( &psResponse->sPromptId );

   /* sPromptResponse */
   Free_compulsory_DbvPromptResponse( &psResponse->sPromptResponse );

   DbvFree( psResponse );
   } /* Free_DbvResponse() */



void Free_DbvPromptObject( DbvPromptObject *psPromptObject )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPromptObject()\n" );
#endif

   if ( psPromptObject == NULL )
      return;


   /* uPromptObject */

   switch( psPromptObject->ePromptObjectType ) 
      {
      case POT_Challenge:
   	 FreeN_DbvChallenge( psPromptObject->uPromptObject.ppsChallenge );
	 break;

      case POT_Response:
   	 FreeN_DbvResponse( psPromptObject->uPromptObject.ppsResponse );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptObject );
   } /* End Free_DbvPromptObject() */



/*-------------------- RESOURCECONTROL --------------------*/


void Free_DbvPartialResultsAvailable( DbvPartialResultsAvailable *pePartialResultsAvailable )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPartialResultsAvailable()\n" );
#endif
     
   if ( pePartialResultsAvailable == NULL )
      return;

   DbvFree( pePartialResultsAvailable );
   } /* End Free_DbvPartialResultsAvailable() */



void Free_DbvResourceControlReq( DbvResourceControlReq *psResourceControlReq )
   {
#ifdef DEBUG
   printf( "D - Free_DbvResourceControlReq()\n" );
#endif

   if ( psResourceControlReq == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psResourceControlReq->psReferenceId );

   /* pfSuspendedFlag */
   Free_DbvBool( psResourceControlReq->pfSuspendedFlag );

   /* psResourceReport */
   FreeDbvASN1External( psResourceControlReq->psResourceReport );

   /* pePartialResultsAvailable */
   Free_DbvPartialResultsAvailable( psResourceControlReq->pePartialResultsAvailable );

   /* compulsory fResponseRequired */

   /* pfTriggeredRequestFlag */
   Free_DbvBool( psResourceControlReq->pfTriggeredRequestFlag );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlReq->ppsOtherInformation );

   DbvFree( psResourceControlReq );
   } /* End Free_DbvResourceControlReq() */ 




void Free_DbvResourceControlRsp( DbvResourceControlRsp *psResourceControlRsp )
   {
#ifdef DEBUG
   printf( "D - Free_DbvResourceControlRsp()\n" );
#endif

   if ( psResourceControlRsp == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psResourceControlRsp->psReferenceId );

   /* compulsory fContinueFlag */

   /* pfResultSetWanted */
   Free_DbvBool( psResourceControlRsp->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlRsp->ppsOtherInformation );

   DbvFree( psResourceControlRsp );
   } /* End Free_DbvResourceControlRsp() */ 




/*-------------------- TRIGGERRESOURCECONTROL --------------------*/


void Free_DbvTriggerResourceControlReq( DbvTriggerResourceControlReq *psTriggerResourceControlReq )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTriggerResourceControlReq()\n" );
#endif

   if ( psTriggerResourceControlReq == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psTriggerResourceControlReq->psReferenceId );

   /* compulsory eRequestedAction */

   /* pcPreferredResourceReportFormat */
   FreeDbvObjectIdentifier( psTriggerResourceControlReq->pcPreferredResourceReportFormat );

   /* pfResultSetWanted */
   Free_DbvBool( psTriggerResourceControlReq->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psTriggerResourceControlReq->ppsOtherInformation );

   DbvFree( psTriggerResourceControlReq );
   } /* End Free_DbvTriggerResourceControlReq() */ 


/*------------------------------------*/


void Free_DbvElementSpec(DbvElementSpec *psElementSpec)                 
{
#ifdef DEBUG
   printf( "D - Free_DbvElementSpec()\n" );
#endif

   if ( psElementSpec == NULL )
      return;

   switch(psElementSpec->eElementSpecType)
   {
   case EST_Primitive:
           FreeDbvInternationalString(psElementSpec->
                   uElementSpec.psPrimitive);
           break;
   case EST_External:
           FreeDbvASN1External(psElementSpec->
                   uElementSpec.psExternal);
           break;
   default:
           break;
   }
} /* Free_DbvElementSpec() */




