/*  Module Name: @(#)Free_DbvFuncs5.c	1.3
    Version: 3/3/97
    Author: John Hough
 
    Crossnet Systems Ltd, 
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to free space allocated for structure types.
        Used by the origin and target.

    Modification History.

*/


#include "Free_DbvFuncs.h"


/* ---------------------------------------------------------------------*/
/* ---- User Info Format "Character Set And Language Negotiation" ----- */



void Free_DbvEnvironment( DbvEnvironment *psEnvironment )
   {
#ifdef DEBUG
   printf( "D - Free_DbvEnvironment()\n" );
#endif

   if ( psEnvironment == NULL )
      return;

   DbvFree( psEnvironment );
   } /* End Free_DbvEnvironment */



void Free_DbvInitialSet( DbvInitialSet *psInitialSet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvInitialSet()\n" );
#endif

   if ( psInitialSet == NULL )
      return;

   DbvFree( psInitialSet );
   } /* End Free_DbvInitialSet */



void Free_DbvLeftAndRight( DbvLeftAndRight *psLeftAndRight )
   {
#ifdef DEBUG
   printf( "D - Free_DbvLeftAndRight()\n" );
#endif

   if ( psLeftAndRight == NULL )
      return;

   DbvFree( psLeftAndRight );
   } /* End Free_DbvLeftAndRight */



void Free_DbvIso2022Proposal( DbvIso2022Proposal *psIso2022Proposal )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIso2022Proposal()\n" );
#endif

   if ( psIso2022Proposal == NULL )
      return;


   /* psProposedEnvironment */  
   Free_DbvEnvironment( psIso2022Proposal->psProposedEnvironment );

   /* ppulProposedSets */
   FreeN_Ulong(psIso2022Proposal->ppulProposedSets);

   /* ppsProposedInitialSets */  
   FreeN_DbvInitialSet( psIso2022Proposal->ppsProposedInitialSets );

   /* psProposedLeftAndRight */
   Free_DbvLeftAndRight(psIso2022Proposal->psProposedLeftAndRight);


   DbvFree( psIso2022Proposal );
   } /* End Free_DbvIso2022Proposal() */ 



void Free_DbvIso2022Response( DbvIso2022Response *psIso2022Response )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIso2022Response()\n" );
#endif

   if ( psIso2022Response == NULL )
      return;

   /* psSelectedEnvironment */  
   Free_DbvEnvironment( psIso2022Response->psSelectedEnvironment );

   /* ppulSelectedSets */
   FreeN_Ulong(psIso2022Response->ppulSelectedSets);

   /* psSelectedInitialSet */  
   Free_DbvInitialSet( psIso2022Response->psSelectedInitialSet );

   /* psSelectedLeftAndRight */
   Free_DbvLeftAndRight(psIso2022Response->psSelectedLeftAndRight);


   DbvFree( psIso2022Response );
   } /* End Free_DbvIso2022Response() */ 




void Free_DbvIso2022( DbvIso2022 *psIso2022 )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIso2022()\n" );
#endif

   if ( psIso2022 == NULL )
      return;


   /* uIso2022Type */

   switch( psIso2022->eIso2022Type ) 
      {
      case I2022T_OriginProposal:
   	 Free_DbvIso2022Proposal( psIso2022->uIso2022Type.psOriginProposal );
	 break;

      case I2022T_TargetResponse:
   	 Free_DbvIso2022Response( psIso2022->uIso2022Type.psTargetResponse );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psIso2022 );
   } /* End Free_DbvIso2022 */



void Free_DbvIso10646( DbvIso10646 *psIso10646 )
   {
#ifdef DEBUG
   printf( "D - Free_DbvIso10646()\n" );
#endif

   if ( psIso10646 == NULL )
      return;

   /* pcCollections */  
   FreeDbvObjectIdentifier( psIso10646->pcCollections );

   /* pcEncodingLevel */
   FreeDbvObjectIdentifier( psIso10646->pcEncodingLevel );


   DbvFree( psIso10646 );
   } /* End Free_DbvIso10646() */ 



void Free_DbvPrivateCharacterSet( DbvPrivateCharacterSet *psPrivateCharacterSet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvPrivateCharacterSet()\n" );
#endif

   if ( psPrivateCharacterSet == NULL )
      return;


   /* uPrivateCharacterSet */

   switch( psPrivateCharacterSet->ePrivateCharacterSetType ) 
      {
      case PCST_ViaOid:
   	 FreeN_DbvObjectIdentifier( psPrivateCharacterSet->uPrivateCharacterSet.ppcViaOid );
	 break;

      case PCST_ExternallySpecified:
   	 FreeDbvASN1External( psPrivateCharacterSet->uPrivateCharacterSet.psExternallySpecified );
	 break;

      case PCST_PreviouslyAgreedUpon:
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPrivateCharacterSet );
   } /* End Free_DbvPrivateCharacterSet */



void Free_DbvProposedCharSet( DbvProposedCharSet *psProposedCharSet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvProposedCharSet()\n" );
#endif

   if ( psProposedCharSet == NULL )
      return;


   /* uProposedCharSet */

   switch( psProposedCharSet->eProposedCharSetType ) 
      {
      case PRCST_Iso2022:
   	 Free_DbvIso2022( psProposedCharSet->uProposedCharSet.psIso2022);
	 break;

      case PRCST_Iso10646:
   	 Free_DbvIso10646( psProposedCharSet->uProposedCharSet.psIso10646);
	 break;

      case PRCST_Private:
   	 Free_DbvPrivateCharacterSet( psProposedCharSet->uProposedCharSet.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psProposedCharSet );
   } /* End Free_DbvProposedCharSet */



void Free_DbvOriginProposal( DbvOriginProposal *psOriginProposal )
   {
#ifdef DEBUG
   printf( "D - Free_DbvOriginProposal()\n" );
#endif

   if ( psOriginProposal == NULL )
      return;

   /* ppsProposedCharSets */  
   FreeN_DbvProposedCharSet( psOriginProposal->ppsProposedCharSets );

   /* ppsProposedLanguages */
   FreeN_DbvLanguageCode( psOriginProposal->ppsProposedLanguages );


   DbvFree( psOriginProposal );
   } /* End Free_DbvOriginProposal() */ 



void Free_DbvSelectedCharSet( DbvSelectedCharSet *psSelectedCharSet )
   {
#ifdef DEBUG
   printf( "D - Free_DbvSelectedCharSet()\n" );
#endif

   if ( psSelectedCharSet == NULL )
      return;


   /* uSelectedCharSet */

   switch( psSelectedCharSet->eSelectedCharSetType ) 
      {
      case SCST_Iso2022:
   	 Free_DbvIso2022( psSelectedCharSet->uSelectedCharSet.psIso2022);
	 break;

      case SCST_Iso10646:
   	 Free_DbvIso10646( psSelectedCharSet->uSelectedCharSet.psIso10646);
	 break;

      case SCST_Private:
   	 Free_DbvPrivateCharacterSet( psSelectedCharSet->uSelectedCharSet.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSelectedCharSet );
   } /* End Free_DbvSelectedCharSet */



void Free_DbvTargetResponse( DbvTargetResponse *psTargetResponse )
   {
#ifdef DEBUG
   printf( "D - Free_DbvTargetResponse()\n" );
#endif

   if ( psTargetResponse == NULL )
      return;

   /* psSelectedCharSets */  
   Free_DbvSelectedCharSet( psTargetResponse->psSelectedCharSets );

   /* psSelectedLanguage */
   Free_DbvGeneralString( psTargetResponse->psSelectedLanguage );


   DbvFree( psTargetResponse );
   } /* End Free_DbvTargetResponse() */ 



void Free_DbvCharSetAndLanguage( DbvCharSetAndLanguage *psCharSetAndLanguage )
   {
#ifdef DEBUG
   printf( "D - Free_DbvCharSetAndLanguage()\n" );
#endif

   if ( psCharSetAndLanguage == NULL )
      return;


   /* uCharSetAndLanguage */

   switch( psCharSetAndLanguage->eCharSetAndLanguageType ) 
      {
      case CSALT_Proposal:
   	 Free_DbvOriginProposal( psCharSetAndLanguage->uCharSetAndLanguage.psProposal);
	 break;

      case CSALT_Response:
   	 Free_DbvTargetResponse( psCharSetAndLanguage->uCharSetAndLanguage.psResponse);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psCharSetAndLanguage );
   } /* End Free_DbvCharSetAndLanguage */



