/*  Module Name: @(#)apifree.c	7.2
    Version: 3/5/97
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
	Routines to free DBV PDU structures
 
        Used by the origin and target.
 
*/

#include "apifree.h"

void FreeDbvOriginData(DbvOriginData *psOriginData)
{
	printf("Freeing the APDU received from the target\n");

	if(psOriginData == NULL)
		return;

	switch(psOriginData->eOriginEvent)
	{
	case OR_InitializeConf:
		FreeDbvInitializeRsp(psOriginData->uOriginData.psInitializeRsp);
		break;
	case OR_SearchConf:
		Free_DbvSearchRsp(psOriginData->uOriginData.psSearchRsp);
		break;
	case OR_PresentConf:
		Free_DbvPresentRsp(psOriginData->uOriginData.psPresentRsp);
		break;
	case OR_DeleteResultSetConf:
		Free_DbvDeleteResultSetRsp(psOriginData->uOriginData.
			psDeleteResultSetRsp);
		break;
	case OR_ScanConf:
		Free_DbvScanRsp(psOriginData->uOriginData.psScanRsp);
		break;
	case OR_ResourceReportConf:
		Free_DbvResourceReportRsp(psOriginData->uOriginData.
			psResourceReportRsp);
		break;
	case OR_SegmentationInd:
		Free_DbvSegmentReq(psOriginData->uOriginData.psSegmentReq);
		break;
	case OR_SortConf:
		Free_DbvSortRsp(psOriginData->uOriginData.psSortRsp);
		break;
	case OR_ExtendedServicesConf:
		Free_DbvExtendedServicesRsp(psOriginData->uOriginData.
			psExtendedServicesRsp);
		break;
	case OR_CloseConf:
		Free_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_CloseInd:
		Free_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_AccessCntrlInd:
		Free_DbvAccessControlReq(psOriginData->uOriginData.psAccessControlReq);
		break;
	case OR_ResourCntrlInd:
		Free_DbvResourceControlReq(psOriginData->uOriginData.psResourceControlReq);
		break;
	default:
		printf("apifree: Not done yet!\n");
		break;
	}

	printf("Completed the free\n");
	
}

void FreeDbvTargetData(DbvTargetData *psTargetData)
{
	printf("Freeing the APDU received from the target\n");
	if(psTargetData == NULL)
		return;

	switch(psTargetData->eTargetEvent)
	{
	case TA_InitializeInd:
		FreeDbvInitializeReq(psTargetData->uTargetData.psInitializeReq);
		break;
	case TA_SearchInd:
		Free_DbvSearchReq(psTargetData->uTargetData.psSearchReq);
		break;
	case TA_PresentInd: 
		Free_DbvPresentReq(psTargetData->uTargetData.psPresentReq);
		break;
	case TA_DeleteResultSetInd: 
		Free_DbvDeleteResultSetReq(psTargetData->uTargetData.
			psDeleteResultSetReq);
		break;
	case TA_ScanInd:
		Free_DbvScanReq(psTargetData->uTargetData.psScanReq);
		break;
	case TA_ResourceReportInd: 
		Free_DbvResourceReportReq(psTargetData->uTargetData.
			psResourceReportReq);
		break;
	case TA_SortInd: 
		Free_DbvSortReq(psTargetData->uTargetData.psSortReq);
		break;
	case TA_ExtendedServicesInd: 
		Free_DbvExtendedServicesReq(psTargetData->uTargetData.
			psExtendedServicesReq);
		break;
	case TA_CloseInd:
                Free_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_CloseConf:
                Free_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_ReleaseInd:
		DbvFree(psTargetData->uTargetData.peReleaseReq);
		break;
	case TA_AccessCntrlConf:
		Free_DbvAccessControlRsp(psTargetData->uTargetData.psAccessControlRsp);
		break;
	case TA_ResourCntrlConf:
		Free_DbvResourceControlRsp(psTargetData->uTargetData.psResourceControlRsp);
		break;
	case TA_TrigResourCntrlInd:
		Free_DbvTriggerResourceControlReq(psTargetData->uTargetData.psTriggerResourceControlReq);
		break;
	default:
		printf("Not done yet!\n");
		break;
	}
	
	printf("Completed the free\n");
}

void FreeDbvInitializeRsp(DbvInitializeRsp *psInitializeRsp)
{
	if(psInitializeRsp->psReferenceId != NULL)
		FreeDbvOctetString(psInitializeRsp->psReferenceId);
	FreeDbvProtocolVersion(psInitializeRsp->pcProtocolVersion);
	FreeDbvOptions(psInitializeRsp->pcOptions);
	if(psInitializeRsp->psImplementationId != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationId);
	if(psInitializeRsp->psImplementationName != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationName);
	if(psInitializeRsp->psImplementationVersion != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationVersion);
	if(psInitializeRsp->psUserInformationField != NULL)
		FreeDbvASN1External(psInitializeRsp->psUserInformationField);
	if(psInitializeRsp->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeRsp->ppsOtherInformation);
        DbvFree(psInitializeRsp);
}

void FreeDbvInitializeReq(DbvInitializeReq *psInitializeReq)
{
	if(psInitializeReq->psReferenceId != NULL)
		FreeDbvOctetString(psInitializeReq->psReferenceId);
	FreeDbvProtocolVersion(psInitializeReq->pcProtocolVersion);
	FreeDbvOptions(psInitializeReq->pcOptions);
	if(psInitializeReq->psAuthentication != NULL)
		FreeDbvAuthentication(psInitializeReq->psAuthentication);
	if(psInitializeReq->psImplementationId != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationId);
	if(psInitializeReq->psImplementationName != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationName);
	if(psInitializeReq->psImplementationVersion != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationVersion);
	if(psInitializeReq->psUserInformationField != NULL)
		FreeDbvASN1External(psInitializeReq->psUserInformationField);
	if(psInitializeReq->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeReq->ppsOtherInformation);
	DbvFree(psInitializeReq);
}

void FreeDbvOctetString(DbvOctetString *psOctetString)
{
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
		DbvFree(psOctetString);
	}
}



void Free_compulsory_DbvOctetString(DbvOctetString *psOctetString)
{   
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
	}
}



void FreeDbvProtocolVersion(DbvProtocolVersion pcProtocolVersion)
{
	if(pcProtocolVersion != NULL)
		DbvFree(pcProtocolVersion);
}

void FreeDbvOptions(DbvOptions pcOptions)
{
	if(pcOptions != NULL)
		DbvFree(pcOptions);
}

void FreeDbvInternationalString(DbvInternationalString *psInternationalString)
{
	if(psInternationalString != NULL)
	{
		if(psInternationalString->pcValue != NULL)
			DbvFree(psInternationalString->pcValue);
		DbvFree(psInternationalString);
	}
}

void FreeDbvObjectIdentifier(DbvObjectIdentifier pcObjectIdentifier)
{
	if(pcObjectIdentifier != NULL)
		DbvFree(pcObjectIdentifier);
}

void FreeDbvOtherInformation(DbvOtherInformation *psOtherInformation)
{
	if(psOtherInformation == NULL)
		return;

	switch(psOtherInformation->eInformationType)
	{
	case IT_CharacterInfo:
		FreeDbvInternationalString(psOtherInformation->uInformation.psCharacterInfo);
		break;
	case IT_BinaryInfo:
		FreeDbvOctetString(psOtherInformation->uInformation.psBinaryInfo);
		break;
	case IT_ExternalInfo:
		FreeDbvASN1External(psOtherInformation->uInformation.psExternallyDefinedInfo);
		break;
	case IT_ObjectIdentifier:
		FreeDbvObjectIdentifier(psOtherInformation->uInformation.pcObjectIdentifier);
		break;
	default:
		printf("OtherInformation: error\n");
		break;
	}
	if(psOtherInformation->psInfoCategory != NULL)
		FreeDbvInfoCategory(psOtherInformation->psInfoCategory);
	DbvFree(psOtherInformation);
}

void FreeDbvInfoCategory(DbvInfoCategory *psInfoCategory)
{
	FreeDbvObjectIdentifier(psInfoCategory->pcCategoryTypeId);
	DbvFree(psInfoCategory);
}

void FreeDbvIdPassword(DbvIdPassword *psIdPassword)
{
	if(psIdPassword != NULL)
	{
		FreeDbvInternationalString(psIdPassword->psGroupId);
		FreeDbvInternationalString(psIdPassword->psUserId);
		FreeDbvInternationalString(psIdPassword->psPassword);
	}
}

void FreeDbvAuthentication(DbvAuthentication *psAuthentication)
{
	if(psAuthentication == NULL)
		return;

	switch(psAuthentication->eAuthenticationType)
	{
	case AT_Open:
		DbvFree(psAuthentication->uAuthentication.psOpen);
		break;
	case AT_IdPassword:
		FreeDbvIdPassword(psAuthentication->uAuthentication.psIdPassword);
		break;
	case AT_Anonymous:
		break;
	case  AT_Other:
		FreeDbvASN1External(psAuthentication->uAuthentication.psOther);
		break;
	default:
		printf("FreeAuthentication: error\n");
		break;
	}

	DbvFree(psAuthentication);
}

