/*  Module Name: @(#)assocutil_params.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Prints out the input parameters to the association and utility
	functions.
 
        Used by the origin.
 
*/

#include "assocutil_params.h"

extern TBL_Return T_Return;

void TT_writeDbvAssocId(DbvAssocId lAssocId, TBL OutTable)
{ 
	write_char_field("\tDbvAssocId", OutTable, T_Return);
	write_int_field(lAssocId, OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void TT_writeDbvSecs(DbvSecs lSeconds, TBL OutTable)
{
	write_char_field("\tDbvSecs", OutTable, T_Return);
	if(lSeconds == CALL_BLOCKING)
		write_char_field("CALL_BLOCKING", OutTable, T_Return);
	else if(lSeconds == CALL_ASYNCHRONOUS)
		write_char_field("CALL_ASYNCHRONOUS", OutTable, T_Return);
	else
		write_int_field(lSeconds, OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void TT_writeDbvTarget(DbvTarget *psTarget, TBL OutTable)
{
	write_char_field("\tDbvTarget", OutTable, T_Return);
	TBL_PutRecord(OutTable);
	if(psTarget->eTargetNameOrAddress == Target_Name)
	{
		write_char_field("\tDbvTargetNameOrAddress",OutTable,T_Return);
		write_char_field("Target_Name", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_char_field("\tpcTargetName",OutTable,T_Return);
		write_char_field(psTarget->uNameOrAddress.pcTargetName, 
			OutTable, T_Return);
	}	
	else
	{
		write_char_field("\tDbvTargetNameOrAddress",OutTable,T_Return);
		write_char_field("Target_Address", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		TT_writeDbvTargetInformation(psTarget->uNameOrAddress.
			psTargetInformation, OutTable);
	}
	TBL_PutRecord(OutTable);
}

void TT_writeDbvTargetInformation(DbvTargetInformation *psTargetInformation,
	TBL OutTable)
{
	write_char_field("DbvTargetInformation", OutTable, T_Return);
	TBL_PutRecord(OutTable);
	/* Forget the DbvApplicationProtocol for now */
	if(psTargetInformation->eOsiOrInternetAddress == Osi_Address)
	{
		write_char_field("\tDbvTargetInformation->eOsiOrInternetAddress"
			,OutTable,T_Return);
		write_char_field("Osi_Address", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		TT_writeDbvOsiAddress(psTargetInformation->uOsiOrInternetAddress
			.psOsiAddress, OutTable);
	}
	else
	{
		write_char_field("\tDbvTargetInformation->eOsiOrInternetAddress"
			,OutTable,T_Return);
		write_char_field("Internet_Address", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_char_field("DbvInternetAddress", OutTable, T_Return);
		write_char_field(psTargetInformation->uOsiOrInternetAddress.
			psInternetAddress->pcHostAddress, OutTable, T_Return);
		write_int_field(psTargetInformation->uOsiOrInternetAddress.
                        psInternetAddress->ulPortNumber, OutTable, T_Return);
		
	}
	TBL_PutRecord(OutTable);
}

void TT_writeDbvOsiAddress(DbvOsiAddress *psOsiAddress, TBL OutTable)
{
	write_char_field("DbvOsiAddress", OutTable, T_Return);
	TBL_PutRecord(OutTable);

	write_char_field("pcPsel", OutTable, T_Return);
	write_char_field("psOsiAddress->pcPsel", OutTable, T_Return);
	TBL_PutRecord(OutTable);

	write_char_field("pcSsel", OutTable, T_Return);
	write_char_field("psOsiAddress->pcSsel", OutTable, T_Return);
	TBL_PutRecord(OutTable);

	write_char_field("pcTsel", OutTable, T_Return);
	write_char_field("psOsiAddress->pcTsel", OutTable, T_Return);
	TBL_PutRecord(OutTable);

	write_char_field("pcNsap", OutTable, T_Return);
	write_char_field("psOsiAddress->pcNsap", OutTable, T_Return);
	TBL_PutRecord(OutTable);

}

void TT_writeDbvAssociateState(DbvAssociateState eAssociateState, TBL OutTable)
{
	char *stringfield = NULL;

        convert_DbvAssociateState(&stringfield, &eAssociateState);
        write_string_field(stringfield, OutTable,T_Return);
        DbvFree(stringfield);
        TBL_PutRecord(OutTable);
}

void TT_writeDbvAssociateResult(int out_record, DbvAssociateResult *puAssociateResult,
	DbvAssociateState eAssociateState, TBL OutTable)
{
	char *stringfield=NULL;
	int intfield;

        if(eAssociateState == Associate_Rejected)
        {
                write_comment_field(" AssociateResult: ", OutTable, T_Return);
                write_comment_field(" puAssociateResult->psAssociateReject->peAssociateRejectSource\n   puAssociateResult->psAssociateReject->peAssociateRejectResult\n   puAssociateResult->psAssociateReject->peAssociateRejectReason", OutTable, T_Return);
                write_DbvAssociateRejectSource(puAssociateResult->
                        psAssociateReject->peAssociateRejectSource,
                        OutTable);
                write_DbvAssociateRejectResult(puAssociateResult->
                        psAssociateReject->peAssociateRejectResult,
                        OutTable);
                write_DbvAssociateRejectReason(puAssociateResult->
                        psAssociateReject->peAssociateRejectReason,
                        OutTable);
                TBL_PutRecord(OutTable);
        }
        if(eAssociateState == Associate_Pending)
        {
                if(puAssociateResult != NULL)
                {
                        printf("Dbv(Confirm)AssociateRequest has returned Associate_Pending and puAssociateResult is not NULL. This indicates an error in the API");
                        write_comment_field("DbvAssociateRequest has returned Associate_Pending and puAssociateResult is not NULL. This indicates an error in the API", OutTable, T_Return);
                }
                else
                {
                        write_comment_field(" AssociateResult: ", OutTable, T_Return);
                        write_int_field(out_record++,OutTable,T_Return);
                        write_string_field("NULL", OutTable,T_Return);
                        TBL_PutRecord(OutTable);
                }
        }
        if(eAssociateState == Associate_Accepted)
        {
                write_comment_field(" AssociateResult.psProtocolAndSyntaxes->peApplicationProtocol,\n   AssociateResult.psProtocolAndSyntaxes->peAbstractSyntax;\n   AssociateResult.psProtocolAndSyntaxes->ppcCommonSyntaxes ", OutTable, T_Return);
                write_int_field(out_record++,OutTable,T_Return);
                stringfield = NULL;
                convert_DbvApplicationProtocol(&stringfield, puAssociateResult->psProtocolAndSyntaxes->peApplicationProtocol);
                write_string_field(stringfield, OutTable,T_Return);
                DbvFree(stringfield);
                stringfield = NULL;
                convert_DbvAbstractSyntax(&stringfield, puAssociateResult->psProtocolAndSyntaxes->peAbstractSyntax);
                write_string_field(stringfield, OutTable, T_Return);
                DbvFree(stringfield);
                TBL_PutRecord(OutTable);
                stringfield = NULL;
                intfield = 0;
                if(puAssociateResult->psProtocolAndSyntaxes->
                        ppcCommonSyntaxes == NULL)
                        write_string_field("NULL",OutTable, T_Return);
                else
                {
                while(puAssociateResult->psProtocolAndSyntaxes->
                        ppcCommonSyntaxes[intfield] != NULL)
                {
                        write_string_field(puAssociateResult->
                                psProtocolAndSyntaxes->
                                ppcCommonSyntaxes[intfield], OutTable,
                                T_Return);
                        intfield++;
                }
                }

                TBL_PutRecord(OutTable);
        }
}

void TT_writeDbvAssociateRsp(DbvAssociateRsp eAssociateRsp, TBL OutTable)
{
	char *stringfield = NULL;

	write_char_field("\tDbvAssociateRsp", OutTable, T_Return);
	convert_DbvAssociateRsp(&stringfield, &eAssociateRsp);
	write_char_field(stringfield, OutTable, T_Return);
	TBL_PutRecord(OutTable);

}

void TT_writeDbvReleaseRsp(DbvReleaseRsp *psReleaseRsp, TBL OutTable)
{
	char *stringfield = NULL;

	if(psReleaseRsp == NULL)
	{
		write_char_field("\tDbvReleaseRsp", OutTable, T_Return);
		write_char_field("NULL", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		return;
	}

	write_char_field("\tDbvReleaseRsp->peReleaseRspReason, DbvReleaseRsp->eReleaseRspResult", OutTable, T_Return);
	TBL_PutRecord(OutTable);
	if(psReleaseRsp->peReleaseRspReason == NULL)
		write_char_field("NULL", OutTable, T_Return);
	else
	{
		switch(*psReleaseRsp->peReleaseRspReason)
		{
		case Normal:
			write_char_field("Normal", OutTable, T_Return);
			break;
		case Not_Finished:
			write_char_field("Not_Finished", OutTable, T_Return);
			break;
		default:
			write_char_field("Invalid Value!", OutTable, T_Return);
			break;
		}
	}
	switch(psReleaseRsp->eReleaseRspResult)
	{
	case Affirmative:
		write_char_field("Affirmative", OutTable, T_Return);
		break;
	case Negative:
		write_char_field("Negative", OutTable, T_Return);
		break;
	default:
		write_char_field("Invalid Value!", OutTable, T_Return);
		break;
	}
	TBL_PutRecord(OutTable);
}

void TT_writeDbvAssocIdList(DbvAssocId **pplAssocId, TBL OutTable)
{
	int loop=0;

	write_char_field("\tDbvAssocId list", OutTable, T_Return);
	while(pplAssocId[loop] != NULL)
	{
		write_int_field(*pplAssocId[loop], OutTable, T_Return);
		loop ++;
	}
	TBL_PutRecord(OutTable);
}

void TT_writeDbvTraceLevel(DbvTraceLevel eTraceLevel, TBL OutTable)
{ 
	write_char_field("\tDbvTraceLevel", OutTable, T_Return);
	switch(eTraceLevel)
	{
	case Trace_Off:
		write_char_field("Trace_Off", OutTable, T_Return);
		break;
	case Trace_Low:
		write_char_field("Trace_Low", OutTable, T_Return);
		break;
	case Trace_High:
		write_char_field("Trace_High", OutTable, T_Return);
		break;
	default:
		break;
	}
	TBL_PutRecord(OutTable);
}

void write_DbvAssociateRejectSource(DbvAssociateRejectSource
        *peAssociateRejectSource, TBL OutTable)
{
        if(peAssociateRejectSource == NULL)
                write_string_field("NULL", OutTable, T_Return);
        else
        {
                switch(*peAssociateRejectSource)
                {
                case ACSE_User:
                        write_string_field("ACSE_User",OutTable, T_Return);
                        break;
                case ACSE_Provider:
                        write_string_field("ACSE_Provider",OutTable, T_Return);
                        break;
                default:
                        write_string_field("OUT OF RANGE",OutTable, T_Return);
                        break;
 
                }
        }
}

void write_DbvAssociateRejectResult(DbvAssociateRejectResult
        *peAssociateRejectResult, TBL OutTable)
{
        if(peAssociateRejectResult == NULL)
                write_string_field("NULL", OutTable, T_Return);
        else
        {
                switch(*peAssociateRejectResult)
                {
                case Rejected_Permanent:
                        write_string_field("Rejected_Permanent",OutTable,
                                T_Return);
                        break;
                case Rejected_Transient:
                        write_string_field("Rejected_Transient",OutTable,
                                T_Return);
                        break;
                default:
                        write_string_field("OUT OF RANGE",OutTable, T_Return);
                        break;
                }
        }
 
}

void write_DbvAssociateRejectReason(DbvAssociateRejectReason
        *peAssociateRejectReason, TBL OutTable)
{
 
        if(peAssociateRejectReason == NULL)
                write_string_field("NULL", OutTable, T_Return);
        else
        {
                switch(*peAssociateRejectReason)
                {
                case No_Reason_Given:
                        write_string_field("No_Reason_Given",OutTable,
                                T_Return);
                        break;
                case No_Common_ACSE_Version:
                        write_string_field("No_Common_ACSE_Version",OutTable,
                                T_Return);
                        break;
                case Application_Context_Not_Supported:
                        write_string_field("Application_Context_Not_Supported",
                                OutTable, T_Return);
                        break;
                default:
                        write_string_field("OUT OF RANGE",OutTable, T_Return);
                        break;
                }
        }
}

