/*  Module Name: @(#)batch.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Routines to process the origin and target batch files.
 
*/
 
#include "batch.h"
 
extern TBL_Return T_Return;
extern TTMode eMode;
extern char *pcBatchFile;

char pcBatchFilename[SZ_FILENAME];

BatchFunctions psUtilityFunctions[NO_UTILITY_FUNCTIONS] = {
        { "SIGetEventLocation",         batch_ExecSIGetEventLocation },
        { "SISetACSEFile",              batch_ExecSISetACSEFile },
        { "SIWhichStack",               batch_ExecSIWhichStack },
        { "SISetTraceParameter",        batch_ExecSISetTraceParameter }
};
 

DbvBool batch_ParseUtilityEntries(char *pcEntry)
{
	long lSub = 0;
	DbvBool fFoundEntry = DBV_FALSE;

	while( fFoundEntry == DBV_FALSE && lSub < NO_UTILITY_FUNCTIONS)
	{
		if(strcmp(psUtilityFunctions[lSub].Name, pcEntry) == 0)
			fFoundEntry = DBV_TRUE;
		lSub ++;
	}

	return fFoundEntry;
}


void batch_CheckFile(char *pcFilename)
{

	if(access(pcFilename, F_OK) == -1)
	{
		printf("File %s does not exist\n",pcFilename);
		config_TargetQuit();
	}
}
 
void batch_ExecuteUtility(char *pcEntry, char *pcFilename)
{
        long lSub = 0;
        DbvBool fFoundEntry = DBV_FALSE;
	int return_value = SI_OK;
 
        while( fFoundEntry == DBV_FALSE && lSub < NO_UTILITY_FUNCTIONS)
        {
                if((strcmp(psUtilityFunctions[lSub].Name, pcEntry))==0)
                        fFoundEntry = DBV_TRUE;
                else
                        lSub ++;
        }
        return_value = psUtilityFunctions[lSub].function(
                pcFilename);
        if(return_value == SI_NOTOK)
                return_value = batch_QuitOrContinue();
}

/* Joint */
int batch_ExecDbvAbortRequest(char *pcFilename)
{
	int return_value;

	printf("***Executing DbvAbortRequest() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_DbvAbortRequest(Valid);

	return return_value;
}
int batch_ExecDbvCloseRequest(char *pcFilename)
{
	int return_value;

	printf("***Executing DbvCloseRequest() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_DbvCloseRequest(Valid);

	return return_value;
}
int batch_ExecDbvCloseResponse(char *pcFilename)
{
	int return_value;

	printf("***Executing DbvCloseResponse() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_DbvCloseResponse(Valid);

	return return_value;
}

/* Utility Functions */
int batch_ExecSIGetEventLocation(char *pcFilename)
{
	int return_value;

	printf("***Executing SIGetEventLocation() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_SIGetEventLocation(Valid);

	return return_value;
}
int batch_ExecSISetACSEFile(char *pcFilename)
{
	int return_value;

	printf("***Executing SISetACSEFile() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_SISetACSEFile(Valid);

	return return_value;
}
int batch_ExecSIWhichStack(char *pcFilename)
{
	int return_value;

	printf("***Executing SIWhichStack() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_SIWhichStack(Valid);

	return return_value;
}
int batch_ExecSISetTraceParameter(char *pcFilename)
{
	int return_value;

	printf("***Executing SISetTraceParameter() [%s]\n", pcFilename);
	strcpy(pcBatchFilename, pcFilename);
	free(pcFilename);
	return_value = do_SISetTraceParameter(Valid);

	return return_value;
}

int batch_QuitOrContinue(void)
{
	char pcInput[2];
        while(*pcInput != 'c' && *pcInput != 'q')
        {
		printf("*** An error occurred. Do you wish to:\n");
                printf("(c)ontinue or (q)uit\n");
                fgets(pcInput, 2, stdin);
        }
        if(*pcInput == 'q')
                config_OriginQuit();

	return SI_OK;
}

