/*  Module Name: @(#)config.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Routines to configure the origin and target.
 
        Used by the origin and target.
 
*/

#include "config.h"

extern TBL_Return T_Return;

TTMode eMode = Menu;
char *pcOWorkingDir = NULL;
char *pcOAssocOutput = NULL;
char *pcOCompletedStructures = NULL;
char *pcOUtilityOutput = NULL;
char *pcOReceivePDUs = NULL;

char *pcTWorkingDir = NULL;
char *pcTAssocOutput = NULL;
char *pcTCompletedStructures = NULL;
char *pcTUtilityOutput = NULL;
char *pcTReceivePDUs = NULL;

char *pcBatchFile = NULL;


DbvBool config_OriginConfigure(void)
{
	DbvBool fReturn = DBV_TRUE;
	TBL psInTable;

	config_OriginGetDefaults();
	psInTable = TBL_Create(ORIGIN_CONFIG_FILE, TBL_Read);
	if(psInTable == NULL)
	{
		printf("Configuration file %s does not exist. Using Defaults\n",
			ORIGIN_CONFIG_FILE);
	}
	else
	{
		TBL_Destruct(psInTable);
		/* Get the configuration details from the configuration file */
		config_GetMode();
		config_OriginGetAssocOutput();
		config_OriginGetCompletedStructures();
		config_OriginGetUtilityOutput();
		config_OriginGetReceivedPDUs();
		if(eMode == Batch)
			fReturn = config_GetBatchFile();
		config_OriginGetWorkingDir();
	}

}

DbvBool config_TargetConfigure(void)
{
        DbvBool fReturn = DBV_TRUE;
        TBL psInTable;
 
        config_TargetGetDefaults();
        psInTable = TBL_Create(TARGET_CONFIG_FILE, TBL_Read);
        if(psInTable == NULL)
        {
                printf("Configuration file %s does not exist. Using Defaults\n",
                        TARGET_CONFIG_FILE);
        }
        else
        {
                TBL_Destruct(psInTable);
                /* Get the configuration details from the configuration file */
                config_GetMode();
                config_TargetGetAssocOutput();
                config_TargetGetCompletedStructures();
                config_TargetGetUtilityOutput();
                config_TargetGetReceivedPDUs();
                if(eMode == Batch)
                        fReturn = config_GetBatchFile();
                config_TargetGetWorkingDir();
        }
 
	return fReturn;
}


DbvBool config_GetBatchFile(void)
{
	char *pcTempBatchFile = NULL;
	int fd;

	pcTempBatchFile = config_SearchForEntry("BatchFile");
	if(pcTempBatchFile != NULL)
	{
		pcBatchFile = pcTempBatchFile;
	}
	else
	{
		printf("Could not find BatchFile entry\n");
		config_TargetQuit();
	}
	
	return DBV_TRUE;
}

void config_GetMode(void)
{
	char *pcMode = NULL;

	pcMode = config_SearchForEntry("Mode");
	if(pcMode != NULL)
	{
		if(strcmp(pcMode, "Menu") == 0)
			eMode = Menu;
		else if(strcmp(pcMode, "Batch") == 0)
			eMode = Batch;
		else
		{
			printf("Error!\nMode: Entry %s is invalid\n", pcMode);
			printf("Using default: Menu\n");
			printf("\n");
			eMode = Menu;
		}
		free(pcMode);
	}
	else
	{
		printf("Using default: Menu\n");
		eMode = Menu;
	}
}

void config_OriginGetWorkingDir(void)
{
	char *pcWorkingDirectory = NULL;

	pcWorkingDirectory = config_SearchForEntry("WorkingDirectory");
	if(pcWorkingDirectory != NULL)
	{
		free(pcOWorkingDir);
		pcOWorkingDir = pcWorkingDirectory;
		printf("Working directory set to %s\n", pcOWorkingDir);
	}
	else
	{
		printf("Could not find entry WorkingDirectory\n");
		printf("Using default: %s\n\n", ORIGIN_WORKINGDIRECTORY_DEFAULT);
	}

	if(chdir(pcOWorkingDir) == -1)
	{
		printf("Error: Directory does not exist\n");
		config_OriginQuit();
	}
}

void config_TargetGetWorkingDir(void)
{
        char *pcWorkingDirectory = NULL;
 
        pcWorkingDirectory = config_SearchForEntry("WorkingDirectory");
        if(pcWorkingDirectory != NULL)
        {
                free(pcOWorkingDir);
                pcTWorkingDir = pcWorkingDirectory;
                printf("Working directory set to %s\n", pcTWorkingDir);
        }
        else
        {
                printf("Could not find entry WorkingDirectory\n");
                printf("Using default: %s\n\n", TARGET_WORKINGDIRECTORY_DEFAULT)
;
        }
 
        if(chdir(pcTWorkingDir) == -1)
        {
                printf("Error: Directory does not exist\n");
                config_TargetQuit();
        }
}

void config_OriginGetAssocOutput(void)
{
	char *pcAssociationOutput = NULL;

	pcAssociationOutput = config_SearchForEntry("AssociationOutputTo");
	if(pcAssociationOutput != NULL)
	{
		free(pcOAssocOutput);
		pcOAssocOutput = pcAssociationOutput;
		printf("Results from association functions to %s\n",
			pcOAssocOutput);
	}
	else
	{
                printf("Could not find entry AssociationOutputTo\n");
		printf("Using default: %s\n\n", ORIGIN_ASSOCOUTPUTTO_DEFAULT);
	}

}

void config_TargetGetAssocOutput(void)
{
        char *pcAssociationOutput = NULL;
 
        pcAssociationOutput = config_SearchForEntry("AssociationOutputTo");
        if(pcAssociationOutput != NULL)
        {
                free(pcTAssocOutput);
                pcTAssocOutput = pcAssociationOutput;
                printf("Results from association functions to %s\n",
                        pcTAssocOutput);
        }
        else
        {
                printf("Could not find entry AssociationOutputTo\n");
                printf("Using default: %s\n\n", TARGET_ASSOCOUTPUTTO_DEFAULT);
        }
 
}

void config_OriginGetCompletedStructures(void)
{
	char *pcCompletedStructures = NULL;

	pcCompletedStructures = config_SearchForEntry("CompletedStructuresTo");
	if(pcCompletedStructures != NULL)
	{
		free(pcOCompletedStructures);
		pcOCompletedStructures = pcCompletedStructures;
		printf("Completed Dbv structures to %s\n", 
			pcOCompletedStructures);
	}
	else
	{
		printf("Could not find entry CompletedStructuresTo\n");
		printf("Using default: %s\n\n", ORIGIN_COMPSTRUCTURESTO_DEFAULT);
	}
}

void config_TargetGetCompletedStructures(void)
{
        char *pcCompletedStructures = NULL;
 
        pcCompletedStructures = config_SearchForEntry("CompletedStructuresTo");
        if(pcCompletedStructures != NULL)
        {
                free(pcTCompletedStructures);
                pcTCompletedStructures = pcCompletedStructures;
                printf("Completed Dbv structures to %s\n",
                        pcTCompletedStructures);
        }
        else
        {
                printf("Could not find entry CompletedStructuresTo\n");
                printf("Using default: %s\n\n", TARGET_COMPSTRUCTURESTO_DEFAULT);
        }
}

void config_OriginGetUtilityOutput(void)
{
	char *pcUtilityOutput = NULL;

	pcUtilityOutput = config_SearchForEntry("UtilityOutputTo");
	if(pcUtilityOutput != NULL)
	{
		free(pcOUtilityOutput);
		pcOUtilityOutput = pcUtilityOutput;
		printf("Results from Utility functions to %s\n", 
			pcOUtilityOutput);
	}
	else
	{
		printf("Could not find entry UtilityOutputTo\n");
		printf("Using default: %s\n\n", ORIGIN_UTILITYOUTPUTTO_DEFAULT);
	}
}

void config_TargetGetUtilityOutput(void)
{
        char *pcUtilityOutput = NULL;
 
        pcUtilityOutput = config_SearchForEntry("UtilityOutputTo");
        if(pcUtilityOutput != NULL)
        {
                free(pcTUtilityOutput);
                pcTUtilityOutput = pcUtilityOutput;
                printf("Results from Utility functions to %s\n",
                        pcTUtilityOutput);
        }
        else
        {
                printf("Could not find entry UtilityOutputTo\n");
                printf("Using default: %s\n\n", TARGET_UTILITYOUTPUTTO_DEFAULT);
        }
}

void config_OriginGetReceivedPDUs(void)
{
	char *pcReceivedPDUs = NULL;

	pcReceivedPDUs = config_SearchForEntry("ReceivedPdusTo");
	if(pcReceivedPDUs != NULL)
	{
		free(pcOReceivePDUs);
		pcOReceivePDUs = pcReceivedPDUs;
		printf("Received PDUs to %s\n",pcOReceivePDUs);
	}
	else
	{
		printf("Could not find entry ReceivedPdusTo\n");
		printf("Using default: %s\n\n", ORIGIN_RECEIVEDPDUSTO_DEFAULT);
	}
}

void config_TargetGetReceivedPDUs(void)
{
        char *pcReceivedPDUs = NULL;
 
        pcReceivedPDUs = config_SearchForEntry("ReceivedPdusTo");
        if(pcReceivedPDUs != NULL)
        {
                free(pcTReceivePDUs);
                pcTReceivePDUs = pcReceivedPDUs;
                printf("Received PDUs to %s\n",pcTReceivePDUs);
        }
        else
        {
                printf("Could not find entry ReceivedPdusTo\n");
                printf("Using default: %s\n\n", TARGET_RECEIVEDPDUSTO_DEFAULT);
        }
}

void config_OriginGetDefaults(void)
{
	pcOWorkingDir = (char *)malloc(
		strlen(ORIGIN_WORKINGDIRECTORY_DEFAULT)+1);
	strcpy(pcOWorkingDir, ORIGIN_WORKINGDIRECTORY_DEFAULT);

	pcOAssocOutput = (char *)malloc(
		strlen(ORIGIN_ASSOCOUTPUTTO_DEFAULT)+1);
	strcpy(pcOAssocOutput , ORIGIN_ASSOCOUTPUTTO_DEFAULT);

	pcOCompletedStructures = (char *)malloc(
		strlen(ORIGIN_COMPSTRUCTURESTO_DEFAULT)+1);
	strcpy(pcOCompletedStructures , ORIGIN_COMPSTRUCTURESTO_DEFAULT);

	pcOUtilityOutput = (char *)malloc(
		strlen(ORIGIN_UTILITYOUTPUTTO_DEFAULT)+1);
	strcpy(pcOUtilityOutput , ORIGIN_UTILITYOUTPUTTO_DEFAULT);

	pcOReceivePDUs = (char *)malloc(
		strlen(ORIGIN_RECEIVEDPDUSTO_DEFAULT)+1);
	strcpy(pcOReceivePDUs , ORIGIN_RECEIVEDPDUSTO_DEFAULT);
}

void config_TargetGetDefaults(void)
{
        pcTWorkingDir = (char *)malloc(
                strlen(TARGET_WORKINGDIRECTORY_DEFAULT)+1);
        strcpy(pcTWorkingDir, TARGET_WORKINGDIRECTORY_DEFAULT);
 
        pcTAssocOutput = (char *)malloc(
                strlen(TARGET_ASSOCOUTPUTTO_DEFAULT)+1);
        strcpy(pcTAssocOutput , TARGET_ASSOCOUTPUTTO_DEFAULT);
 
        pcTCompletedStructures = (char *)malloc(
                strlen(TARGET_COMPSTRUCTURESTO_DEFAULT)+1);
        strcpy(pcTCompletedStructures , TARGET_COMPSTRUCTURESTO_DEFAULT);
 
        pcTUtilityOutput = (char *)malloc(
                strlen(TARGET_UTILITYOUTPUTTO_DEFAULT)+1);
        strcpy(pcTUtilityOutput , TARGET_UTILITYOUTPUTTO_DEFAULT);
 
        pcTReceivePDUs = (char *)malloc(
                strlen(TARGET_RECEIVEDPDUSTO_DEFAULT)+1);
        strcpy(pcTReceivePDUs , TARGET_RECEIVEDPDUSTO_DEFAULT);
}

char *config_SearchForEntry(char *pcEntry)
{
	char *pcReturnItem = NULL;
	char *pcString = NULL;
	DbvBool fFound = DBV_FALSE;
	TBL psInTable;

	psInTable = TBL_Create(ORIGIN_CONFIG_FILE, TBL_Read);

	while(TBL_GetRecord(psInTable) != DBV_FALSE)
	{
		pcString = config_GetCharField(psInTable);
		if(pcString != NULL && (strcmp(pcString, pcEntry)==0))
		{
			pcReturnItem = config_GetStringField(psInTable);
			if(pcReturnItem == NULL)
				printf("%s entry contains an invalid field\n", 
					pcEntry);
			fFound = DBV_TRUE;
		}

		if(pcString != NULL)
			free(pcString);
	}

	if(psInTable != NULL)
		TBL_Destruct(psInTable);

	if(fFound == DBV_FALSE)
		printf("Entry %s was not in the configuration file.\n");

	return pcReturnItem;
}

char *config_GetStringField(TBL psInTable)
{
        if( TBL_GetField(psInTable, T_Return) == FALSE )
	{
		printf("WARNING: There is an error in the configuration file!\n");
                return NULL;
        }

        if(T_Return->Tag != TBL_QText)
        {
		printf("WARNING: There is an error in the configuration file!\n");
                return NULL;
        }
 
        return T_Return->Value.QText;
}

char *config_GetCharField(TBL psInTable)
{
 
        /* Get the first field in the record read */
        if( TBL_GetField(psInTable, T_Return) == FALSE ) 
	{
		printf("WARNING: There is an error in the configuration file!\n");
                return NULL;
        }
        if(T_Return->Tag != TBL_UText)
        {
		printf("WARNING: There is an error in the configuration file!\n");
                return NULL;
        }

        return T_Return->Value.UText;
}

/* There has been a fatal error while configuring the origin */
void config_OriginQuit(void)
{
	exit(1);
}

/* There has been a fatal error while configuring the origin */
void config_TargetQuit(void)
{
	char temp[SML_BUFFER];
	
	printf("Press return to exit\n");
	fgets(temp, 31, stdin);
	exit(1);
}

