/*  Module Name: @(#)external_processing.c	7.3
    Version: 2/27/97
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        DBV-OSI II Origin Test Tool external functions. Covers the conversion
        of strings to enumerated types and reading and writing to/from tbl
        files. All specific to the external format. 
 
        Used by the origin and target.

    Modification History.

    14-02-96 	D.B.Maskell	Added support or Item Order and Ill Reports
*/

#include "external_processing.h"

extern TBL_Return T_Return;
extern char pcFileExtension[];
extern char pcTBLDirectory[];


int open_external_table( char *pcFilename, TBL *ExternalTable)
{
	int return_value = SI_OK;
	char buffer[STRING_LEN];

	sprintf(buffer,"%s%s",pcTBLDirectory, pcFilename, pcFileExtension);
	*ExternalTable = TBL_Create(buffer, TBL_Read);
	if(*ExternalTable==NULL)
	{
		printf("Error: Could not open file %s\n", buffer);
		return_value =  SI_NOTOK;
	}

	return return_value;
	
}

void get_DbvASN1External(DbvASN1External **ppsASN1External, TBL InTable)
{
        char *stringfield;
	char input[10];
	TBL ExternalTable;
 
	TBL_GetRecord(InTable);
	/* Ignore the title */
	stringfield=get_char_field(InTable, T_Return);
	DbvFree(stringfield);

        if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))==0)
        {
		return;
        }

	/* Open the external file */
        if(open_external_table(stringfield, &ExternalTable) == SI_NOTOK)
	{
		printf("Press return\n");
		gets(input);
	}

	DbvFree(stringfield);

	if(*ppsASN1External == NULL)
        	*ppsASN1External = (DbvASN1External *)DbvAlloc(sizeof(DbvASN1External));
        TBL_GetRecord(ExternalTable);
	/* Get EVT */
        stringfield=get_string_field(ExternalTable, T_Return);
        convert_DbvExternalValueType(&stringfield, &(*ppsASN1External)->eValueType);
	DbvFree(stringfield);
	/* Get oid */
	stringfield=get_string_field(ExternalTable, T_Return);
	if(strcmp(stringfield, NOTHING) != 0)
	{
		(*ppsASN1External)->pcObjectIdentifier=stringfield;
	}
	else
		DbvFree(stringfield);
	/* Ignore the title */
	TBL_GetRecord(ExternalTable);
        stringfield=get_char_field(ExternalTable, T_Return);
	
        switch((*ppsASN1External)->eValueType)
        {
        case EVT_SingleASN1Type:
                get_DbvSingleASN1Type(&(*ppsASN1External)->uExternalValue.psSingleASN1Type, ExternalTable);
                break;
        case EVT_OctetAligned:
                get_DbvOctetString(&(*ppsASN1External)->uExternalValue.psOctetAligned, ExternalTable);
                break;
        case EVT_Arbitrary:
                get_DbvOctetString(&(*ppsASN1External)->uExternalValue.psArbitrary, ExternalTable);
                break;
        default:
                break;
        }

		TBL_Destruct(ExternalTable);
}

void get_DbvSingleASN1Type(DbvSingleASN1Type **ppsSingleASN1Type, TBL ExternalTable)
{
        char *stringfield;
 
        *ppsSingleASN1Type = (DbvSingleASN1Type *)DbvAlloc(sizeof(DbvSingleASN1Type));
        stringfield = get_string_field(ExternalTable, T_Return);
        convert_DbvASN1Type(&stringfield, &(*ppsSingleASN1Type)->eASN1Type);
	DbvFree(stringfield);
 
        switch((*ppsSingleASN1Type)->eASN1Type)
        {
        case AT_UnstructuredText:
		/* Ignore the title */
		TBL_GetRecord(ExternalTable);
        	stringfield=get_char_field(ExternalTable, T_Return);
		get_DbvInternationalString(&(*ppsSingleASN1Type)->uASN1.psUnstructuredText, ExternalTable);
                break;
        case AT_OpacRecord:
		get_DbvOpacRecord(&(*ppsSingleASN1Type)->uASN1.psOpacRecord, ExternalTable);
                break;
        case AT_SummaryRecord:
		/* Ignore the title */
		TBL_GetRecord(ExternalTable);
        	stringfield=get_char_field(ExternalTable, T_Return);
		get_DbvSummary(&(*ppsSingleASN1Type)->uASN1.psSummaryRecord, 
			ExternalTable);
                break;
        case AT_ResourceReport1:
		/* Ignore the title */
		TBL_GetRecord(ExternalTable);
        	stringfield=get_char_field(ExternalTable, T_Return);
		get_DbvResourceReport1(&(*ppsSingleASN1Type)->uASN1.
			psResourceReport1, ExternalTable);
                break;
        case AT_ResourceReport2:
		get_DbvResourceReport2(&(*ppsSingleASN1Type)->uASN1.
			psResourceReport2, ExternalTable);
                break;
        case AT_EsTaskPackage:
		get_DbvEsTaskPackage(&(*ppsSingleASN1Type)->uASN1.
			psEsTaskPackage, ExternalTable);
                break;
        case AT_UpdateExtendedService:
		get_DbvUpdateExtendedService(&(*ppsSingleASN1Type)->uASN1.
			psUpdateExtendedService, ExternalTable);
                break;
        case AT_UnknownType:
		get_DbvOctetString(&(*ppsSingleASN1Type)->uASN1.psUnknownType,
			ExternalTable);
                break;
	case AT_ItemOrderExtendedService:
		get_DbvItemOrderExtendedService(&(*ppsSingleASN1Type)->uASN1.
			psItemOrderExtendedService, ExternalTable);
		break;
	case AT_IllPdu:
		get_DbvIllPdu(&(*ppsSingleASN1Type)->uASN1.psIllPdu,
			ExternalTable);
		break;
	case AT_ExplainRecord:
		get_DbvExplainRecord( &(*ppsSingleASN1Type)->uASN1.
			psExplainRecord, ExternalTable);
		break;


#ifdef NOT_YET_IMPLEMENTED
	case AT_GenericRecord:
		get_DbvGenericRecord( &(*ppsSingleASN1Type)->uASN1.
			psGenericRecord, ExternalTable);
		break;

#endif

	case AT_PromptObject:
		get_DbvPromptObject( &(*ppsSingleASN1Type)->uASN1.
			psPromptObject, ExternalTable);
		break;

	case AT_CharSetAndLanguage:
		get_DbvCharSetAndLanguage( &(*ppsSingleASN1Type)->uASN1.
			psCharSetAndLanguage, ExternalTable);
		break;

#ifdef NOT_YET_IMPLEMENTED
	case AT_AccessControlDes1:
	case AT_AccessControlKrb1:
	case AT_PerResSetExtendedService:
	case AT_PerQueryExtendedService:
	case AT_PerQuerySchExtendedService:
	case AT_ExpSpecExtendedService:
	case AT_ExpInvoExtendedService:
	case AT_UserInfoSearchResult:
	case AT_ElementSpec1:
#endif 
        default:
                break;
        }
}

void get_DbvOpacRecord(DbvOpacRecord **ppsOpacRecord, TBL ExternalTable)
{
	char *stringfield;
	int intfield;
	int loop=0;
	DbvOpacRecord *psOpacRecord;
	TBL AnotherExternalTable;

	*ppsOpacRecord = (DbvOpacRecord *)DbvAlloc(sizeof(DbvOpacRecord));
	psOpacRecord = *ppsOpacRecord;

	get_DbvASN1External(&(psOpacRecord->psBibliographicRecord), 
		ExternalTable);

	TBL_GetRecord(ExternalTable);
	/*Ignore the title */
        stringfield=get_char_field(ExternalTable, T_Return);
		DbvFree(stringfield);
	/* Get the number of DbvHoldingsRecords to read */
	intfield=get_int_field(ExternalTable,T_Return);
	if(intfield > 0)
	{
		psOpacRecord->ppsHoldingsData=(DbvHoldingsRecord **)DbvAlloc((intfield+1) * sizeof(DbvHoldingsRecord *));
		for(loop = 0; loop < intfield; loop ++)
		{
			psOpacRecord->ppsHoldingsData[loop]=(DbvHoldingsRecord *)DbvAlloc(sizeof(DbvHoldingsRecord));
			get_DbvHoldingsRecord(psOpacRecord->ppsHoldingsData[loop], ExternalTable);
		}
	}
}

void get_DbvHoldingsRecord(DbvHoldingsRecord *psHoldingsRecord, TBL ExternalTable)
{
	char *stringfield;
	int intfield;
	DbvASN1External *psExternal;

	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield = get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	stringfield = get_string_field(ExternalTable, T_Return);
	convert_DbvHoldingsRecType(&stringfield, &(psHoldingsRecord->eHoldingsRecType));
	DbvFree(stringfield);
	switch(psHoldingsRecord->eHoldingsRecType)
	{
	case HRT_MarcHoldingsRecord:
		/* Open the new external file */
		/*stringfield=get_char_field(ExternalTable, T_Return);*/
		/*DbvFree(stringfield);*/
		psExternal = &(psHoldingsRecord->uMarcOrHoldAndCirc.sMarcHoldingsRecord);
		get_DbvASN1External(&psExternal, ExternalTable);
		break;
	case HRT_HoldingsAndCirc:
		get_DbvHoldingsAndCircData(&(psHoldingsRecord->uMarcOrHoldAndCirc.sHoldingsAndCirc), ExternalTable);
		break;
	}

}

void get_DbvHoldingsAndCircData(DbvHoldingsAndCircData *psHoldingsAndCirc, TBL ExternalTable)
{
	char *stringfield;
	int loop,records;

	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable,T_Return);
	DbvFree(stringfield);

	get_DbvInternationalString(&(psHoldingsAndCirc->psTypeOfRecord), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psEncodingLevel), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psFormat), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psReceiptAcqStatus), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psGeneralRetention), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psCompleteness), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psDateOfReport), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psNucCode), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psLocalLocation), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psShelvingLocation), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psCallNumber), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psShelvingData), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psCopyNumber), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psPublicNote), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psReproductionNote), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psTermsUseRepro), ExternalTable);
	get_DbvInternationalString(&(psHoldingsAndCirc->psEnumAndChron), ExternalTable);
	
	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	records=get_int_field(ExternalTable, T_Return);
	if(records>0)
	{
		psHoldingsAndCirc->ppsVolume=(DbvVolume **)DbvAlloc((records+1) * sizeof(DbvVolume *));
		for(loop=0;loop<records;loop++)
		{
			TBL_GetRecord(ExternalTable);
			psHoldingsAndCirc->ppsVolume[loop]=(DbvVolume *)DbvAlloc(sizeof(DbvVolume));
			get_DbvVolume(psHoldingsAndCirc->ppsVolume[loop], ExternalTable);
		}
	}
	TBL_GetRecord(ExternalTable);
	/* Ignore Title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	records=get_int_field(ExternalTable, T_Return);
	if(records>0)
	{
		psHoldingsAndCirc->ppsCircRecord=(DbvCircRecord **)DbvAlloc((records+1) * sizeof(DbvCircRecord *));
		for(loop=0; loop<records;loop++)
		{
			TBL_GetRecord(ExternalTable);
			get_DbvCircRecord(&(psHoldingsAndCirc->ppsCircRecord[loop]), ExternalTable);
		}
	}

}

void get_DbvVolume(DbvVolume *psVolume, TBL ExternalTable)
{
	char *stringfield;

	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	get_DbvInternationalString(&(psVolume->psEnumeration), ExternalTable);
	get_DbvInternationalString(&(psVolume->psChronology), ExternalTable);
	get_DbvInternationalString(&(psVolume->psEnumAndChron), ExternalTable);
}

void get_DbvCircRecord(DbvCircRecord **ppsCircRecord, TBL ExternalTable)
{
	char *stringfield;
	DbvCircRecord *psCircRecord;

	*ppsCircRecord=(DbvCircRecord *)DbvAlloc(sizeof(DbvCircRecord));
	psCircRecord = *ppsCircRecord;
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	psCircRecord->fAvailableNow = get_DbvBool(ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psAvailabilityDate), ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psAvailableThru), ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psRestrictions), ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psItemId), ExternalTable);
	psCircRecord->fRenewable = get_DbvBool(ExternalTable);
	psCircRecord->fOnHold = get_DbvBool(ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psEnumAndChron), ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psMidspine), ExternalTable);
	get_DbvInternationalString(&(psCircRecord->psTemporaryLocation), ExternalTable);

}

void get_DbvSummary(DbvSummaryRecord **ppsSummaryRecord, TBL ExternalTable)
{
	int loop,records;
	char *stringfield;
	DbvSummaryRecord *psSummaryRecord;

	loop=records=0;
	*ppsSummaryRecord=(DbvSummaryRecord *)DbvAlloc(sizeof(DbvSummaryRecord));
	psSummaryRecord=*ppsSummaryRecord;
	get_compulsory_DbvInternationalString(&(psSummaryRecord->sTitle), ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psAuthor), ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psCallNumber), ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psRecordType), ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psBibliographicLevel), ExternalTable);
	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	records=get_int_field(ExternalTable, T_Return);
	if(records > 0)
	{
		psSummaryRecord->ppsFormat=(DbvFormatSpec **)DbvAlloc((records + 1) * sizeof(DbvFormatSpec *));
		for(loop=0;loop<records;loop++)
		{
			TBL_GetRecord(ExternalTable);
			psSummaryRecord->ppsFormat[loop]=(DbvFormatSpec *)DbvAlloc(sizeof(DbvFormatSpec));
		get_DbvFormatSpec(psSummaryRecord->ppsFormat[loop], ExternalTable);
		}
	}
	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	get_DbvInternationalString(&(psSummaryRecord->psPublicationPlace), 
		ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psPublicationDate), 
		ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psTargetSystemKey), 
		ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psSatisfyingElement), 
		ExternalTable);
	psSummaryRecord->pulRank=(Ulong *)get_integer_string(ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psDocumentId), 
		ExternalTable);
	get_DbvInternationalString(&(psSummaryRecord->psAbstract), 
		ExternalTable);
	getN_DbvOtherInformation(&(psSummaryRecord->ppsOtherInformation), 
		ExternalTable);

}
void get_DbvFormatSpec(DbvFormatSpec *psFormat, TBL ExternalTable)
{
	char *stringfield;

	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	get_compulsory_DbvInternationalString(&(psFormat->sType), ExternalTable);
	psFormat->pulSize=(Ulong *)get_integer_string(ExternalTable);
	psFormat->pulBestPosn=(Ulong *)get_integer_string(ExternalTable);
	
}

void get_DbvResourceReport1(DbvResourceReport1 **ppsResourceReport1, TBL ExternalTable)
{
	char *stringfield;
	int records, loop;
	DbvResourceReport1 *psResourceReport1;

	*ppsResourceReport1=(DbvResourceReport1 *)DbvAlloc(sizeof(DbvResourceReport1));
	psResourceReport1=*ppsResourceReport1;

	records=get_int_field(ExternalTable, T_Return);
	if(records>0)
	{
		psResourceReport1->ppsEstimate=(DbvEstimate1 **)DbvAlloc((records +1) * sizeof(DbvEstimate1 *));
		for(loop=0;loop<records;loop++)
		{
			TBL_GetRecord(ExternalTable);
			psResourceReport1->ppsEstimate[loop]=(DbvEstimate1 *)DbvAlloc(sizeof(DbvEstimate1));
			get_DbvEstimate1(psResourceReport1->ppsEstimate[loop], ExternalTable);
		}
	}
	TBL_GetRecord(ExternalTable);
	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	get_compulsory_DbvInternationalString(&(psResourceReport1->sMessage), ExternalTable);
		
}

void get_DbvEstimate1(DbvEstimate1 *psEstimate, TBL ExternalTable)
{
	char *stringfield;

	/* Ignore the title */
	stringfield=get_char_field(ExternalTable, T_Return);
	DbvFree(stringfield);
	stringfield=get_string_field(ExternalTable, T_Return);
	convert_DbvEstimateType(&stringfield, &(psEstimate->eEstimateType));
	psEstimate->ulValue=get_int_field(ExternalTable, T_Return);
	psEstimate->pulCurrencyCode=(Ulong *)get_integer_string(ExternalTable);
}

void write_DbvASN1External(DbvASN1External *psASN1External, TBL OutTable)
{
	char *temp=NULL;

	write_comment_field("START of External",OutTable, T_Return);
	if(psASN1External != NULL)
	{
		write_title_char(psASN1External->pcObjectIdentifier,
			"pcObjectIdentifier", OutTable);
		TBL_PutRecord(OutTable);
		convert_DbvExternalValueType(&temp, &(psASN1External->eValueType));
		write_title_char(temp, "eValueType", OutTable);
		DbvFree(temp);
		TBL_PutRecord(OutTable);
		switch (psASN1External->eValueType)
		{
		case EVT_SingleASN1Type:
			write_SingleASN1Type(psASN1External->uExternalValue.
				psSingleASN1Type, OutTable);
			break;
		case EVT_OctetAligned:
			write_comment_field("START of OctetAligned External", 
				OutTable, T_Return);
			write_DbvOctetString("psOctetAligned", psASN1External->
				uExternalValue.psOctetAligned, OutTable);
			write_comment_field("END of OctetAligned External", 
				OutTable, T_Return);
			TBL_PutRecord(OutTable);
			break;
		case EVT_Arbitrary:
			write_comment_field("START of Arbitrary External", 
				OutTable, T_Return);
			write_DbvOctetString("psArbitrary", psASN1External->
				uExternalValue.psArbitrary, OutTable);
			write_comment_field("END of Arbitrary External", 
				OutTable, T_Return);
			TBL_PutRecord(OutTable);
			break;
		}
	}
	TBL_PutRecord(OutTable);
	write_comment_field("END of External",OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void write_SingleASN1Type(DbvSingleASN1Type *psSingleASN1Type, TBL OutTable)
{
	char *temp=NULL;
	
	write_comment_field("START of DbvSingleASN1Type structure", OutTable, T_Return);
	convert_DbvASN1Type(&temp, &(psSingleASN1Type->eASN1Type));
	write_title_char(temp, "eASN1Type", OutTable);
	TBL_PutRecord(OutTable);
	DbvFree(temp);
	switch(psSingleASN1Type->eASN1Type)
	{
	case AT_UnstructuredText:
		write_comment_field("START of Unstructured Text Record", OutTable, T_Return);
		write_DbvInternationalString("psUnstructuredText", psSingleASN1Type->uASN1.psUnstructuredText, OutTable);
		write_comment_field("END of Unstructured Text Record", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		break;
	case AT_OpacRecord:
		write_DbvOpacRecord(psSingleASN1Type->uASN1.psOpacRecord, 
			OutTable);
		break;
	case AT_SummaryRecord:
		write_DbvSummaryRecord(psSingleASN1Type->uASN1.psSummaryRecord, 
			OutTable);
		break;
	case AT_ResourceReport1:
		write_DbvResourceReport1(psSingleASN1Type->uASN1.
			psResourceReport1, OutTable);
		break;
	case AT_ResourceReport2:
		write_DbvResourceReport2(psSingleASN1Type->uASN1.
			psResourceReport2, OutTable);
		break;
	case AT_EsTaskPackage:
		write_DbvEsTaskPackage(psSingleASN1Type->uASN1.psEsTaskPackage,
                                        OutTable);
		break;
	case AT_UpdateExtendedService:
		write_DbvUpdateExtendedService(psSingleASN1Type->uASN1.
			psUpdateExtendedService, OutTable);
		break;
	case AT_UnknownType:
		break;
	case AT_ItemOrderExtendedService:
		write_DbvItemOrderExtendedService(psSingleASN1Type->uASN1.
			psItemOrderExtendedService, OutTable);
		break;
	case AT_IllPdu:
		write_DbvIllPdu(psSingleASN1Type->uASN1.psIllPdu,
			OutTable);
		break;
	case AT_ExplainRecord:
		write_DbvExplainRecord( psSingleASN1Type->uASN1.
			psExplainRecord, OutTable);
		break;

#ifdef NOT_YET_IMPLEMENTED
	case AT_GenericRecord:
		write_DbvGenericRecord( psSingleASN1Type->uASN1.psGenericRecord,
					OutTable);
		break;
#endif

	case AT_PromptObject:
		write_DbvPromptObject( psSingleASN1Type->uASN1.
			psPromptObject, OutTable);
		break;


	case AT_CharSetAndLanguage:
		write_DbvCharSetAndLanguage( "psCharSetAndLanguage",
			psSingleASN1Type->uASN1.
			psCharSetAndLanguage, OutTable);
		break;



#ifdef NOT_YET_IMPLEMENTED
	case AT_AccessControlPrompt1:
	case AT_AccessControlDes1:
	case AT_AccessControlKrb1:
	case AT_PerResSetExtendedService:
	case AT_PerQueryExtendedService:
	case AT_PerQuerySchExtendedService:
	case AT_ExpSpecExtendedService:
	case AT_ExpInvoExtendedService:
	case AT_UserInfoSearchResult:
	case AT_ElementSpec1:
#endif 
        default:
		break;
	}

	write_comment_field("END of DbvSingleASN1Type structure", OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void write_DbvOpacRecord(DbvOpacRecord *psOpacRecord, TBL OutTable)
{
	char *temp;
	int loop;

	write_comment_field("START of Opac Record", OutTable, T_Return);
	if(psOpacRecord->psBibliographicRecord != NULL)
	{
		write_char_field("psBibliographicRecord External follows", OutTable,T_Return);
		TBL_PutRecord(OutTable);
		write_DbvASN1External(psOpacRecord->psBibliographicRecord, OutTable);
	}
	else
	{
		write_title_char("NULL", "psBibliographicRecord", OutTable);
	}
	TBL_PutRecord(OutTable);
	if(psOpacRecord->ppsHoldingsData != NULL)
	{
		loop=0;
		while(psOpacRecord->ppsHoldingsData[loop] != NULL)
		{
			write_DbvHoldingsRecord(psOpacRecord->ppsHoldingsData[loop], OutTable);
			loop++;
		}
	}
	else
		write_title_char("NULL", "ppsHoldingsData", OutTable);
	
	write_comment_field("END of Opac Record", OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void write_DbvHoldingsRecord(DbvHoldingsRecord *psHoldingsData, TBL OutTable)
{
	char *temp=NULL;

	write_comment_field("START of DbvHoldingsRecord", OutTable, T_Return);

	convert_DbvHoldingsRecType(&temp, &(psHoldingsData->eHoldingsRecType));
	write_title_char(temp, "eHoldingsRecType", OutTable);
	DbvFree(temp);
	switch(psHoldingsData->eHoldingsRecType)
	{
	case HRT_MarcHoldingsRecord:
		TBL_PutRecord(OutTable);
		write_char_field("sMarcHoldingsRecord External follows", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_DbvASN1External(&(psHoldingsData->uMarcOrHoldAndCirc.sMarcHoldingsRecord), OutTable);
		break;
	case HRT_HoldingsAndCirc:
		TBL_PutRecord(OutTable);
		write_DbvHoldingsAndCircData(&(psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc), OutTable);
		break;
	}
	write_comment_field("END of DbvHoldingsRecord", OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void write_DbvHoldingsAndCircData(DbvHoldingsAndCircData *psHoldingsAndCirc, TBL OutTable)
{
	int loop;

	write_comment_field("START of DbvHoldingsAndCircData", OutTable, T_Return);
	
	write_DbvInternationalString("psTypeOfRecord",psHoldingsAndCirc->psTypeOfRecord, OutTable);
	write_DbvInternationalString("psEncodingLevel",psHoldingsAndCirc->psEncodingLevel, OutTable);
	write_DbvInternationalString("psFormat",psHoldingsAndCirc->psFormat, OutTable);
	write_DbvInternationalString("psReceiptAcqStatus",psHoldingsAndCirc->psReceiptAcqStatus, OutTable);
	write_DbvInternationalString("psGeneralRetention",psHoldingsAndCirc->psGeneralRetention, OutTable);
	write_DbvInternationalString("psCompleteness",psHoldingsAndCirc->psCompleteness, OutTable);
	write_DbvInternationalString("psDateOfReport",psHoldingsAndCirc->psDateOfReport, OutTable);
	write_DbvInternationalString("psLocalLocation",psHoldingsAndCirc->psNucCode, OutTable);
	write_DbvInternationalString("psLocalLocation",psHoldingsAndCirc->psLocalLocation, OutTable);
	write_DbvInternationalString("psShelvingLocation",psHoldingsAndCirc->psShelvingLocation, OutTable);
	write_DbvInternationalString("psCallNumber",psHoldingsAndCirc->psCallNumber, OutTable);
	write_DbvInternationalString("psShelvingData",psHoldingsAndCirc->psShelvingData, OutTable);
	write_DbvInternationalString("psCopyNumber",psHoldingsAndCirc->psCopyNumber, OutTable);
	write_DbvInternationalString("psPublicNote",psHoldingsAndCirc->psPublicNote, OutTable);
	write_DbvInternationalString("psReproductionNote",psHoldingsAndCirc->psReproductionNote, OutTable);
	write_DbvInternationalString("psTermsUseRepro",psHoldingsAndCirc->psTermsUseRepro, OutTable);
	write_DbvInternationalString("psEnumAndChron",psHoldingsAndCirc->psEnumAndChron, OutTable);
	write_comment_field("END of DbvHoldingsAndCircData", OutTable, T_Return);
	TBL_PutRecord(OutTable);
	
	if(psHoldingsAndCirc->ppsVolume != NULL)
	{
		loop=0;
		while(psHoldingsAndCirc->ppsVolume[loop] != NULL)
		{
			write_DbvVolume(psHoldingsAndCirc->ppsVolume[loop], OutTable);
			loop++;
		}
	}
	else
		write_title_char("NULL", "ppsVolume", OutTable);
	TBL_PutRecord(OutTable);

	if(psHoldingsAndCirc->ppsCircRecord != NULL)
	{
		loop=0;
		while(psHoldingsAndCirc->ppsCircRecord[loop] != NULL)
		{
			write_DbvCircRecord(psHoldingsAndCirc->ppsCircRecord[loop], OutTable);
			loop++;
		}
	}
	else
		write_title_char("NULL", "ppsCircRecord", OutTable);
	TBL_PutRecord(OutTable);
}

void write_DbvVolume(DbvVolume *psVolume, TBL OutTable)
{
	write_comment_field("START of DbvVolume", OutTable, T_Return);
	write_DbvInternationalString("psEnumeration", psVolume->psEnumeration, OutTable);
	write_DbvInternationalString("psChronology", psVolume->psChronology, OutTable);
	write_DbvInternationalString("psEnumAndChron", psVolume->psEnumAndChron, OutTable);
	write_comment_field("END of DbvVolume", OutTable, T_Return);
}

void write_DbvCircRecord(DbvCircRecord *psCircRecord, TBL OutTable)
{
	write_comment_field("START of DbvCircRecord", OutTable, T_Return);
	write_DbvBool("fAvailableNow", psCircRecord->fAvailableNow, OutTable);
	TBL_PutRecord(OutTable);
	write_DbvInternationalString("psAvailabilityDate", psCircRecord->psAvailabilityDate, OutTable);
	write_DbvInternationalString("psAvailableThru", psCircRecord->psAvailableThru, OutTable);
	write_DbvInternationalString("psRestrictions", psCircRecord->psRestrictions, OutTable);
	write_DbvInternationalString("psItemId", psCircRecord->psItemId, OutTable);
	write_DbvBool("fRenewable", psCircRecord->fRenewable, OutTable);
	TBL_PutRecord(OutTable);
	write_DbvBool("fOnHold", psCircRecord->fOnHold, OutTable);
	TBL_PutRecord(OutTable);
	write_DbvInternationalString("psEnumAndChron", psCircRecord->psEnumAndChron, OutTable);
	write_DbvInternationalString("psMidspine", psCircRecord->psMidspine, OutTable);
	write_DbvInternationalString("psTemporaryLocation", psCircRecord->psTemporaryLocation, OutTable);
	
	write_comment_field("END of DbvCircRecord", OutTable, T_Return);
}

void write_DbvSummaryRecord(DbvSummaryRecord *psSummaryRecord, TBL OutTable)
{
	int loop;

	write_comment_field("START of Summary Record", OutTable, T_Return);

	write_DbvInternationalString("sTitle", &(psSummaryRecord->sTitle), OutTable);
	write_DbvInternationalString("psAuthor", psSummaryRecord->psAuthor, OutTable);
	write_DbvInternationalString("psCallNumber", psSummaryRecord->psCallNumber, OutTable);
	write_DbvInternationalString("psRecordType", psSummaryRecord->psRecordType, OutTable);
	write_DbvInternationalString("psBibliographicLevel", psSummaryRecord->psBibliographicLevel, OutTable);

	TBL_PutRecord(OutTable);
	if(psSummaryRecord->ppsFormat != NULL)
	{
		loop=0;
		while(psSummaryRecord->ppsFormat[loop] != NULL)
		{
			write_DbvFormatSpec(psSummaryRecord->ppsFormat[loop], OutTable);
			loop++;
		}
	}
	else
		write_title_char("NULL", "ppsFormat", OutTable);

	TBL_PutRecord(OutTable);

	write_DbvInternationalString("psPublicationPlace", psSummaryRecord->psPublicationPlace, OutTable);
	write_DbvInternationalString("psPublicationDate", psSummaryRecord->psPublicationDate, OutTable);
	write_DbvInternationalString("psTargetSystemKey", psSummaryRecord->psTargetSystemKey, OutTable);
	write_DbvInternationalString("psSatisfyingElement", psSummaryRecord->psSatisfyingElement, OutTable);
	write_title_int("pulRank", (int *)psSummaryRecord->pulRank, OutTable);
	write_DbvInternationalString("psDocumentId", psSummaryRecord->psDocumentId, OutTable);
	write_DbvInternationalString("psAbstract", psSummaryRecord->psAbstract, OutTable);
	
	/* Other Information */
	writeN_DbvOtherInformation(psSummaryRecord->ppsOtherInformation, 
		OutTable);

	write_comment_field("END of Summary Record", OutTable, T_Return);
	TBL_PutRecord(OutTable);
}

void write_DbvFormatSpec(DbvFormatSpec *psFormat, TBL OutTable)
{
	write_comment_field("Start of DbvFormatSpec", OutTable, T_Return);
	write_DbvInternationalString("sType", &(psFormat->sType), OutTable);
	write_title_int("pulSize", (int *)psFormat->pulSize, OutTable);
	write_title_int("pulBestPosn", (int *)psFormat->pulBestPosn, OutTable);

	write_comment_field("END of DbvFormatSpec", OutTable, T_Return);
}

void write_DbvResourceReport1(DbvResourceReport1 *psResourceReport1, TBL OutTable)
{
	int loop;

	write_comment_field("START of DbvResourceReport1", OutTable, T_Return);

	if(psResourceReport1->ppsEstimate != NULL)
	{
		loop=0;
		while(psResourceReport1->ppsEstimate[loop] != NULL)
		{
			write_DbvEstimate1(psResourceReport1->ppsEstimate[loop], OutTable);
			loop++;
		}
	}
	else
		write_title_char("NULL", "ppsEstimate", OutTable);

	TBL_PutRecord(OutTable);
	write_DbvInternationalString("sMessage", &(psResourceReport1->sMessage), OutTable);
	write_comment_field("END of DbvResourceReport1", OutTable, T_Return);
}

void write_DbvEstimate1(DbvEstimate1 *psEstimate, TBL OutTable)
{
	char *stringfield=NULL;

	write_comment_field("START of DbvEstimate1", OutTable, T_Return);
	convert_DbvEstimateType(&stringfield, &(psEstimate->eEstimateType));
	write_title_char(stringfield, "eEstimateType",OutTable);
	TBL_PutRecord(OutTable);
	DbvFree(stringfield);
	write_title_int("ulValue", (int *)&(psEstimate->ulValue), OutTable);
	write_title_int("pulCurrencyCode", (int *)psEstimate->pulCurrencyCode, OutTable);
	write_comment_field("END of DbvEstimate1", OutTable, T_Return);
}

