/*  Module Name: @(#)o_access.c	7.1
    Version: 11/15/96
    Author: John Hough

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to read arguments/write results from/to origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.
*/

#include "o_access.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvDbvAccessControlResponse() 					!*/
/*! Read in DbvAccessControlResponse tbl file 				!*/
/*+---------------------------------------------------------------------+*/
int do_DbvAccessControlResponse( TestType eTestType )
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   lAssocId;
   DbvAccessControlRsp 	   *psAccessControlRsp;
   DbvError		   *psError = NULL;


   return_value = open_tables("accessrsp", eTestType, InFilename, OutFilename , 
						 &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of AccessControlResponse\n", OutTable, T_Return);

   TBL_GetRecord( InTable );
   do {
   psAccessControlRsp = (DbvAccessControlRsp *) DbvAlloc (sizeof(DbvAccessControlRsp));

      /* Assoc Id */
      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* o ReferenceId */
      TBL_GetRecord( InTable );
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psAccessControlRsp->psReferenceId ), InTable );



      /* o SecurityChallengeRsp */
      TBL_GetRecord( InTable );

      /* Ignore the title */
      stringfield = get_char_field( InTable, T_Return );
      DbvFree(stringfield);

      stringfield = get_string_field( InTable, T_Return );
      if ( strcmp( stringfield, NOTHING ) != 0 )
         get_DbvSecurityChallengeRsp( &psAccessControlRsp->psSecurityChallengeRsp, InTable );
      DbvFree(stringfield);


      /* o Diagnostic */
      get_DbvDiagRec( &psAccessControlRsp->psDiagnostic, InTable );


      /* o OtherInformation */
      getN_DbvOtherInformation( &psAccessControlRsp->ppsOtherInformation, InTable );


        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvAccessControlResponse(psAccessControlRsp, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;


      /* Now execute the function */

      printf( "Press enter to execute DbvAccessControlRsp\n" );
      fflush( stdin );
      get_return();

      return_value = DbvAccessControlResponse(lAssocId, psAccessControlRsp, &psError);

      if( return_value == SI_OK )
         {
         printf( "AccessControlRsp Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psError, OutTable );
         } /* End else */

      Free_DbvAccessControlRsp(psAccessControlRsp);
      TBL_GetRecord( InTable );
      out_record = 1; 
      } while( TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DbvAccessControlRsp( psAccessControlRsp );	*/

   return return_value;
   } /* End do_DbvAccessControlResponse() */


