/*  Module Name: @(#)o_close.c	7.2
    Version: 2/27/97
    Author: Peter Watson

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to write results/read arguments to/from origin and target 
	specific tbl files associated with the Z39.50 functions. For use with 
	the testtool.

*/
#include "o_close.h"

extern TBL_Return T_Return;
extern char string[];

/* Read in DbvCloseRequest tbl file */
int do_DbvCloseRequest(TestType eTestType)
{
	char InFilename[STRING_LEN];
	char OutFilename[STRING_LEN];
        char pcResult[STRING_LEN];
	TBL InTable, OutTable, LocalTable;
	int place = 0;

	int intfield;
	int out_record = 1;
	int return_value = SI_OK;

	char *stringfield;
	char *reason;
	char *format;
        char *pcDbvCloseReason;
        DbvCloseReason eCloseReason;

	DbvAssocId         lAssocId;
	DbvCloseReq        *psCloseReq;
	DbvError           *psError=NULL;


	return_value = open_tables("closereq", eTestType, InFilename, OutFilename , &InTable, &OutTable);
	if(return_value == SI_NOTOK)
		return return_value;
	write_comment_field("\nResults of CloseRequest\n", OutTable, T_Return);
	TBL_GetRecord(InTable);
	do {
	psCloseReq = (DbvCloseReq *) DbvAlloc (sizeof(DbvCloseReq));
/* Assoc Id */
	intfield=get_int_field(InTable,T_Return);
	lAssocId=get_int_field(InTable,T_Return);
	TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
/* ReferenceId */
	get_DbvOctetString(&(psCloseReq->psReferenceId), InTable);
/* eCloseReason */
	TBL_GetRecord(InTable);
 	stringfield= get_char_field(InTable, T_Return);
        DbvFree(stringfield);
        pcDbvCloseReason = get_string_field(InTable, T_Return);
        convert_DbvCloseReason(&pcDbvCloseReason,&eCloseReason);
        psCloseReq->eCloseReason=eCloseReason;
	DbvFree(pcDbvCloseReason);
	TBL_GetRecord(InTable);

/* DiagnosticInformation  */
        stringfield= get_char_field(InTable, T_Return);
	DbvFree(stringfield);
	get_DbvInternationalString(&(psCloseReq->psDiagnosticInformation), InTable);

	TBL_GetRecord(InTable);
/* ResourceReportFormat */
	get_DbvObjectIdentifier(&psCloseReq->pcResourceReportFormat, InTable);

/* ResourceReport External */
	get_DbvASN1External(&(psCloseReq->psResourceReport), InTable);


/* OtherInformation */
	getN_DbvOtherInformation(&(psCloseReq->ppsOtherInformation), InTable);
 
        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvCloseRequest(psCloseReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

	printf("Press enter to execute DbvCloseRequest\n");
	fflush(stdin);
	get_return();

	return_value = DbvCloseRequest(lAssocId, psCloseReq, &psError);

	if(return_value == SI_OK)
	{
		printf("CloseRequest Ok\n");
		printf("Writing output to %s\n", OutFilename);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("IN_PARAM", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("IN_PARAM", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("NULL", OutTable, T_Return);
		TBL_PutRecord(OutTable);
	}
	else
	{
		printf("Call failed\n");
		write_DbvError(&psError, OutTable);
	}
	Free_DbvCloseReq(psCloseReq);
	TBL_GetRecord(InTable);
	out_record = 1; place = 0;
	}
	while(TBL_GetRecord(InTable)!=FALSE);

	TBL_Destruct(InTable);
	TBL_Destruct(OutTable);

	return return_value;
}

void write_DbvCloseResponse(DbvCloseReq *psCloseRsp, TBL OutTable)
{
	char temp[STRING_LEN];
        char *pcDbvCloseReason;

        if( bLocalResult == DBV_TRUE)
                write_comment_field(" Completed structure DbvCloseResponse ",
                        OutTable, T_Return);
        else
        {
		write_comment_field(" DbvCloseResponse received ", OutTable, 
			T_Return);
		write_comment_field("Structure detailed - DbvOrigin/TargetData.uOrgin/TargetEvent->psCloseRsp:", OutTable, T_Return);
	}

/* ReferenceId is an optional field */
	write_DbvOctetString("psReferenceId", psCloseRsp->psReferenceId, OutTable);

/* CloseReason */
        pcDbvCloseReason= NULL;
        convert_DbvCloseReason(&pcDbvCloseReason,&(psCloseRsp->eCloseReason));
	write_title_char(pcDbvCloseReason, "eCloseReason", OutTable);
	DbvFree(pcDbvCloseReason);
	TBL_PutRecord(OutTable);

/* DiagnosticInformation*/
	write_DbvInternationalString("psDiagnosticInformation", psCloseRsp->psDiagnosticInformation, OutTable);

/* ResourceReportFormat */

	write_title_char(psCloseRsp->pcResourceReportFormat, "pcResourceReportFormat", OutTable);
	TBL_PutRecord(OutTable);

/* ResourceReport ASN1External */

	write_comment_field("START of psResourceReport", OutTable, T_Return);
	write_DbvASN1External(psCloseRsp->psResourceReport, OutTable);
	write_comment_field("END of psResourceReport", OutTable, T_Return);

/* Other Information */
	writeN_DbvOtherInformation(psCloseRsp->ppsOtherInformation, OutTable);

        write_comment_field(" End ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
}
 
int do_DbvCloseResponse(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
        char pcResult[STRING_LEN];
        TBL InTable, OutTable, LocalTable;
        int place = 0;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        DbvCloseReason eCloseReason;
        char *stringfield;
        char *format;
        char *version;
        char *reason;
        char *pcDbvCloseReason;  
        DbvAssocId         lAssocId;
        DbvCloseReq      *psCloseRsp;
        DbvError         *psError=NULL;
 
        psCloseRsp = (DbvCloseReq *)DbvAlloc(sizeof(DbvCloseReq));
        return_value = open_tables("closeresp", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvCloseResponse\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
/* Assoc Id */
        intfield=get_int_field(InTable,T_Return);
        lAssocId=intfield=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
/* ReferenceId */
    get_DbvOctetString(&(psCloseRsp->psReferenceId), InTable);
 
/* eCloseReason */
	TBL_GetRecord(InTable);
 	stringfield= get_char_field(InTable, T_Return);
        DbvFree(stringfield);
        pcDbvCloseReason = get_string_field(InTable, T_Return);
        convert_DbvCloseReason(&pcDbvCloseReason,&eCloseReason);
        psCloseRsp->eCloseReason=eCloseReason;
	DbvFree(pcDbvCloseReason);
	TBL_GetRecord(InTable);
 
/* DiagnosticInformation  */
        stringfield= get_char_field(InTable, T_Return);
        DbvFree(stringfield);
        get_DbvInternationalString(&(psCloseRsp->psDiagnosticInformation), InTable);
 
        TBL_GetRecord(InTable);
 
/* ResourceReportFormat */
	get_DbvObjectIdentifier(&psCloseRsp->pcResourceReportFormat, InTable);
 
/* ResourceReport External */
        get_DbvASN1External(&(psCloseRsp->psResourceReport), InTable);
 
/* OtherInformation */
        getN_DbvOtherInformation(&(psCloseRsp->ppsOtherInformation), InTable);
 
        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvCloseResponse(psCloseRsp, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

        printf("Press enter to execute CloseRsp\n");
        fflush(stdin);
        get_return();
 
        return_value = DbvCloseResponse(lAssocId, psCloseRsp, &psError);
 
        if(return_value == SI_OK)
        {
                printf("CloseResponse Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL", OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}
 
void write_DbvCloseRequest(DbvCloseReq *psCloseReq,TBL OutTable)
{
        char temp[STRING_LEN];
        char *pcDbvCloseReason; 

	 
        if( bLocalResult == DBV_TRUE)
                write_comment_field("Completed structure DbvCloseRequest",
                        OutTable, T_Return);
        else
        {
        	write_comment_field("DbvCloseRequest received from Target", 
			OutTable, T_Return);
        	write_comment_field("Structure detailed - DbvOriginData.uOriginEvent->psCloseReq:", OutTable, T_Return);
 	}

/* ReferenceId is an optional field */
        write_DbvOctetString("psReferenceId", psCloseReq->psReferenceId,OutTable);
 
/* CloseReason */
        pcDbvCloseReason= NULL;
        convert_DbvCloseReason(&pcDbvCloseReason,&(psCloseReq->eCloseReason));
	write_title_char(pcDbvCloseReason, "eCloseReason", OutTable);
	DbvFree(pcDbvCloseReason);
	TBL_PutRecord(OutTable);
/* DiagnosticInformation*/
        write_DbvInternationalString("psDiagnosticInformation", psCloseReq->psDiagnosticInformation, OutTable);
 
/* ResourceReportFormat */

	write_title_char(psCloseReq->pcResourceReportFormat, "pcResourceReportFormat", OutTable);
        TBL_PutRecord(OutTable);
 
/* ResourceReport ASN1External */
 
        write_comment_field("START of psResourceReport", OutTable, T_Return);
        write_DbvASN1External(psCloseReq->psResourceReport, OutTable);
        write_comment_field("END of psResourceReport", OutTable, T_Return);
 
/* Other Information */
        writeN_DbvOtherInformation(psCloseReq->ppsOtherInformation, OutTable);
 
        write_comment_field(" End ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
}
 
