/*  Module Name: @(#)o_del.c	7.1
    Version: 11/15/96
    Author: Peter Watson

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to write results/read arguments to/from origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.

    Modification History.

    18-05-95	D.B.Maskell	Continued implementation.
*/

#include "o_del.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvDeleteResultSetRequest()					!*/
/*! Read in DbvDeleteResultSetRequest tbl file 				!*/
/*+---------------------------------------------------------------------+*/
int do_DbvDeleteResultSetRequest(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable,LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   lAssocId;
   DbvDeleteResultSetReq   *psDeleteResultSetReq;
   DbvError		   *psError = NULL;
   DbvInternationalString  **ppsResultSetId;
   DbvDeleteFunction	   eDeleteFunction;


   return_value = open_tables("delreq", eTestType, InFilename, OutFilename , &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of DeleteResultSetRequest\n", OutTable, T_Return);

   TBL_GetRecord(InTable);	/* Believe this should be in the loop */
   do {

      psDeleteResultSetReq = (DbvDeleteResultSetReq *) DbvAlloc (sizeof(DbvDeleteResultSetReq));
      /* Assoc Id */

      intfield = get_int_field(InTable,T_Return);
      lAssocId = get_int_field(InTable,T_Return);

      /* ReferenceId */

      TBL_GetRecord(InTable);
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psDeleteResultSetReq->psReferenceId ), InTable);

      /* Delete Fuction */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );

      stringfield = get_string_field( InTable, T_Return );
      convert_DbvDeleteFunction( &stringfield, &eDeleteFunction );
      psDeleteResultSetReq->eDeleteFunction = eDeleteFunction;
      DbvFree( stringfield );

      /* Get the number of ResultSedIds */
      TBL_GetRecord(InTable);

      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );

      intfield = get_int_field(InTable,T_Return);

      /* ResultSetId list  */

      ppsResultSetId = ( DbvInternationalString **)DbvAlloc( 
			( sizeof( DbvInternationalString* ) * (intfield + 1) ));
      psDeleteResultSetReq->ppsResultSetId = ppsResultSetId;


      for ( ; intfield; intfield-- )
	 {
         TBL_GetRecord(InTable);
         stringfield = get_char_field(InTable, T_Return); 
         DbvFree( stringfield );
         get_DbvInternationalString( ppsResultSetId, InTable );
	 ppsResultSetId++;
	 } /* End for */
 
      *ppsResultSetId = NULL;

      /* OtherInformation */
      getN_DbvOtherInformation(&(psDeleteResultSetReq->ppsOtherInformation), 
		InTable);

      /* Now execute the function */
        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvDeleteResultSetRequest(psDeleteResultSetReq, 
			LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;
 

      printf("Press enter to execute DbvDeleteResultSetRequest\n");
      fflush(stdin);
      get_return();

      return_value = DbvDeleteResultSetRequest(lAssocId, psDeleteResultSetReq, 
						&psError);

      if(return_value == SI_OK)
         {
         printf("DeleteResultSetRequest Ok\n");
         printf("Writing output to %s\n", OutFilename);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("NULL", OutTable, T_Return);
         TBL_PutRecord(OutTable);

	 /* I Think there should be more here But I known not what is wanted */
         } /* End if */
      else
         {
         printf("Call failed\n");
         write_DbvError(&psError, OutTable);
         } /* End else */

      Free_DbvDeleteResultSetReq(psDeleteResultSetReq);
      TBL_GetRecord(InTable);
      out_record = 1; 
      } while( TBL_GetRecord(InTable)!=FALSE );

   TBL_Destruct(InTable);
   TBL_Destruct(OutTable);

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DeleteResultRequest( psDeleteResultSetReq );	*/

   return return_value;
   } /* End do_DbvDeleteResultSetRequest() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvDeleteResultSetResponce()					!*/
/*+---------------------------------------------------------------------+*/
void write_DbvDeleteResultSetResponse(DbvDeleteResultSetRsp *psDeleteResultSetRsp, TBL OutTable)
   {
   char temp[STRING_LEN];
   char *pc_string = NULL;
   DbvListStatus	**ppsDeleteListStatuses;
   DbvListStatus	**ppsBulkStatuses;

        if( bLocalResult == DBV_TRUE)
                write_comment_field("Completed structure DbvDeleteResultSetResponse", OutTable, T_Return);
        else
        {

   		write_comment_field("DeleteResultSetResponse received from Target", OutTable, T_Return);
   		write_comment_field("Structure detailed - DbvOriginData.uOriginEvent->psDeleteResultSetRsp:", OutTable, T_Return);
	}

   /* ReferenceId is an optional field */
   write_DbvOctetString("psReferenceId",psDeleteResultSetRsp->psReferenceId, 
		OutTable);

   strcpy(temp, "eDeleteOperationStatus : ");
   convert_DbvDeleteSetStatus( &pc_string,&psDeleteResultSetRsp->eDeleteOperationStatus );

   strcat( temp, pc_string );
   DbvFree( pc_string );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord(OutTable);

   /* DeleteListStatuses : write all lists statuses for out table */

   ppsDeleteListStatuses = psDeleteResultSetRsp->ppsDeleteListStatuses;

   while( ppsDeleteListStatuses && *ppsDeleteListStatuses )
      {
      write_DbvInternationalString( "DeleteListStatuses", 
				    &(*ppsDeleteListStatuses)->sResultSetId,
				    OutTable );
      pc_string = NULL;
      convert_DbvDeleteSetStatus( &pc_string, 
				   &(*ppsDeleteListStatuses)->eDeleteSetStatus);

      sprintf( temp, "DeleteListStatuses->eDeleteStatus : %s", pc_string );
      DbvFree(pc_string);
      write_char_field(temp, OutTable, T_Return); 
      TBL_PutRecord(OutTable);
      ppsDeleteListStatuses++;
      } /* End while */

   /* NumberNotDelete */
  write_integer_string( "pulNumberNotDeleted", (int *)psDeleteResultSetRsp->pulNumberNotDeleted, 
	OutTable );
 
   TBL_PutRecord(OutTable);

   /* BulkStatuses : write all bulk statatuses to out table */

   ppsBulkStatuses = psDeleteResultSetRsp->ppsBulkStatuses;

   while( ppsBulkStatuses && *ppsBulkStatuses )
      {
      write_DbvInternationalString( "BulkStatuses", 
				    &(*ppsBulkStatuses)->sResultSetId,
				    OutTable );
      pc_string = NULL;
      convert_DbvDeleteSetStatus( &pc_string, 
				    &(*ppsBulkStatuses)->eDeleteSetStatus);
      sprintf( temp, "BulkStatuses->eDeleteStatus : %s", pc_string );
      DbvFree(pc_string);
      write_char_field(temp, OutTable, T_Return);
      TBL_PutRecord(OutTable);
      ppsBulkStatuses++;
      } /* End while */

   /* DeleteMessage */

   write_DbvInternationalString( "psDeleteMessage : ",
	psDeleteResultSetRsp->psDeleteMessage,OutTable );
   TBL_PutRecord(OutTable);

   writeN_DbvOtherInformation(psDeleteResultSetRsp->ppsOtherInformation, 
	OutTable);
   TBL_PutRecord(OutTable);
   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvDeleteResultSetResponce() */
