/*  Module Name: @(#)o_init.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments to/from origin specific tbl
        files associated with the Z39.50 functions. For use with the testtool.
 
*/

#include "o_init.h"

extern TBL_Return T_Return;
extern char string[];

/* Read in DbvInitializeRequest tbl file */
int do_DbvInitializeRequest(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
	char pcResult[STRING_LEN];
        TBL InTable, OutTable, LocalTable;
        int place = 0;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
        char *version;
        char *options;
 
        DbvAssocId         lAssocId;
        DbvSecs            lSeconds;
        DbvInitializeReq   *psInitializeReq;
        DbvError           *psError=NULL;

        return_value = open_tables("initreq", eTestType, InFilename, OutFilename , &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of InitializeRequest\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        psInitializeReq = (DbvInitializeReq *) DbvAlloc (sizeof(DbvInitializeReq));
/* Assoc Id */
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
/* ReferenceId */
	get_DbvOctetString(&(psInitializeReq->psReferenceId), InTable);
/* ProtocolVersion */
        if((strcmp(version=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
                psInitializeReq->pcProtocolVersion = version;
        }
        else
                free(version);
/* Options */
        if((strcmp(options=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
                psInitializeReq->pcOptions = options;
        }
        else
                free(version);
/* Pref mes size */
        psInitializeReq->ulPreferredMessageSize = get_int_field(InTable,T_Return);
/* Max mes size */
        psInitializeReq->ulMaximumMessageSize = get_int_field(InTable,T_Return);
/* Authentication External */
        TBL_GetRecord(InTable);
        get_Authentication(&psInitializeReq->psAuthentication, InTable);
        TBL_GetRecord(InTable);
/* ImplementationId */
	get_DbvInternationalString(&(psInitializeReq->psImplementationId), InTable);
/* ImplementationName */
	get_DbvInternationalString(&(psInitializeReq->psImplementationName), InTable);
/* ImplementationVersion */
	get_DbvInternationalString(&(psInitializeReq->psImplementationVersion), InTable);
/* UserInformationField External */
	get_DbvASN1External(&(psInitializeReq->psUserInformationField), InTable);

/* OtherInformation */
	getN_DbvOtherInformation(&(psInitializeReq->ppsOtherInformation), 
		InTable);
	
	bLocalResult = DBV_TRUE;
	if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
		write_DbvInitializeRequest(psInitializeReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
	bLocalResult = DBV_FALSE;
 
        printf("Press enter to execute DbvInitializeRequest\n");
        fflush(stdin);
        get_return();
 
        return_value = DbvInitializeRequest(lAssocId, psInitializeReq, &psError);
 
        if(return_value == SI_OK)
        {
                printf("InitializeRequest Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL", OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
	FreeDbvInitializeReq(psInitializeReq);
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}
 
void write_DbvInitializeResponse(DbvInitializeRsp *psInitializeRsp,TBL OutTable)
{
        char temp[STRING_LEN];
 
        if( bLocalResult == DBV_TRUE)
                write_comment_field("Completed structure DbvInitializeResponse",
                        OutTable, T_Return);
        else
        {
        	write_comment_field(
			"DbvInitializeResponse received from Target", OutTable,
			T_Return);
        	write_comment_field("Structure detailed - DbvOriginData.uOriginEvent->psInitializeRsp:", OutTable, T_Return);
	}
 
/* ReferenceId is an optional field */
        write_DbvOctetString("psReferenceId",psInitializeRsp->psReferenceId, OutTable);
 
        strcpy(temp, "pcProtocolVersion: ");
        strcat(temp, psInitializeRsp->pcProtocolVersion);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);

        strcpy(temp, "pcOptions: ");
        strcat(temp, psInitializeRsp->pcOptions);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        sprintf(temp, "ulPreferredMessageSize: %ld",psInitializeRsp->ulPreferredMessageSize);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        sprintf(temp, "ulMaxmiumMessageSize: %ld", psInitializeRsp->ulMaximumMessageSize);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
/* result */
        if(psInitializeRsp->fResult == TRUE)
                sprintf(temp, "fResult: true");
        else
        if(psInitializeRsp->fResult == FALSE)
                sprintf(temp, "fResult: false");
        else
                sprintf(temp, "fResult: %d - which is invalid, mate!\n", psInitializeRsp->fResult);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
/* Implementation Id */
        write_DbvInternationalString("psImplementationId", psInitializeRsp->psImplementationId, OutTable);
 
/* Implementation Name */
        write_DbvInternationalString("psImplementationName", psInitializeRsp->psImplementationName, OutTable);
 
/* Implementation Version */
        write_DbvInternationalString("psImplementationVersion", psInitializeRsp->psImplementationVersion, OutTable);


/* User Information field */
	write_comment_field("START of psUserInformationField", OutTable, T_Return);
	write_DbvASN1External(psInitializeRsp->psUserInformationField, OutTable);
	write_comment_field("END of psUserInformationField", OutTable, T_Return);

/* Other Information */
	writeN_DbvOtherInformation(psInitializeRsp->ppsOtherInformation, OutTable);
 
        write_comment_field(" End ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
}
 
