/*  Module Name: @(#)o_pres.c	7.1
    Version: 11/15/96
    Author: Peter Watson

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to write results/read arguments to/from origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.

*/
#include "o_pres.h"

extern TBL_Return T_Return;
extern char string[];

/* Read in DbvPresentRequest tbl file */
int do_DbvPresentRequest(TestType eTestType)
{
	char InFilename[STRING_LEN];
	char OutFilename[STRING_LEN];
        char pcResult[STRING_LEN];
	TBL InTable, OutTable, LocalTable;
	int loop;
	int place = 0;
	int  NumberOfAdditionalRanges;
	int intfield;
	int out_record = 1;
	int return_value = SI_OK;
	char *stringfield;
	char *version;
	char *options;

	DbvAssocId         lAssocId;
	DbvSecs            lSeconds;
	DbvPresentReq   *psPresentReq;
	DbvError           *psError=NULL;

	return_value = open_tables("presreq", eTestType, InFilename, OutFilename , &InTable, &OutTable);
	if(return_value == SI_NOTOK)
		return return_value;
	write_comment_field("\nResults of PresentRequest\n", OutTable, T_Return);
	TBL_GetRecord(InTable);
	do {
	psPresentReq = (DbvPresentReq *) DbvAlloc (sizeof(DbvPresentReq));
/* Assoc Id */
	intfield=get_int_field(InTable,T_Return);
	lAssocId=get_int_field(InTable,T_Return);
	TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
/* ReferenceId */
	get_DbvOctetString(&(psPresentReq->psReferenceId), InTable);


/* sResultSetId  */
	TBL_GetRecord(InTable);
        get_compulsory_DbvInternationalString( &psPresentReq->sResultSetId, 
					  InTable );

/* ulResultSetStartPoint */
	TBL_GetRecord(InTable);
	psPresentReq->ulResultSetStartPoint=get_int_field(InTable,T_Return);

/* ulNumberOfRecordsRequested */
	TBL_GetRecord(InTable);
	psPresentReq->ulNumberOfRecordsRequested=get_int_field(InTable,T_Return);


/* NumberOfAdditionalRanges */
	TBL_GetRecord(InTable);
        stringfield= get_char_field(InTable, T_Return);
	DbvFree(stringfield);	 
	NumberOfAdditionalRanges=get_int_field(InTable,T_Return);
 

/* AdditionalRanges */
	if(NumberOfAdditionalRanges > 0)
	{
		psPresentReq->ppsAdditionalRanges=(DbvRange **)DbvAlloc((NumberOfAdditionalRanges+1) * sizeof(DbvRange *));
		for(loop = 0; loop < NumberOfAdditionalRanges; loop ++)
		{
			/*psPresentReq->ppsAdditionalRanges[loop]=(DbvRange *)DbvAlloc(sizeof(DbvRange));*/
			get_DbvRange(&psPresentReq->ppsAdditionalRanges[loop],InTable);	
		}
	}

 
/* RecordComposition */
	get_DbvRecordComposition(&psPresentReq->psRecordComposition,InTable);	

/* PreferredRecordSyntax */
	TBL_GetRecord(InTable);
	get_DbvObjectIdentifier(&psPresentReq->pcPreferredRecordSyntax, 
		InTable);

	
/* pulMaxSegmentCount */
	TBL_GetRecord(InTable);
        stringfield= get_char_field(InTable, T_Return);
	DbvFree(stringfield);
        psPresentReq->pulMaxSegmentCount = (Ulong *)get_integer_string( InTable );

/* pulMaxRecordSize */
	TBL_GetRecord(InTable);
        stringfield= get_char_field(InTable, T_Return);
	DbvFree(stringfield);
        psPresentReq->pulMaxRecordSize =  (Ulong *)get_integer_string( InTable );
/* pulMaxSegmentSize */
	TBL_GetRecord(InTable);
        stringfield = get_char_field(InTable, T_Return);
	DbvFree(stringfield);
        psPresentReq->pulMaxSegmentSize =  (Ulong *)get_integer_string( InTable );

/* OtherInformation */
	getN_DbvOtherInformation(&(psPresentReq->ppsOtherInformation), InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvPresentRequest(psPresentReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

	printf("Press enter to execute DbvPresentRequest\n");
	fflush(stdin);
	get_return();

	return_value = DbvPresentRequest(lAssocId, psPresentReq, &psError);

	if(return_value == SI_OK)
	{
		printf("PresentRequest Ok\n");
		printf("Writing output to %s\n", OutFilename);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("IN_PARAM", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("IN_PARAM", OutTable, T_Return);
		TBL_PutRecord(OutTable);
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("NULL", OutTable, T_Return);
		TBL_PutRecord(OutTable);
	}
	else
	{
		printf("Call failed\n");
		write_DbvError(&psError, OutTable);
	}
	Free_DbvPresentReq(psPresentReq);
	TBL_GetRecord(InTable);
	out_record = 1; place = 0;
	}
	while(TBL_GetRecord(InTable)!=FALSE);

	TBL_Destruct(InTable);
	TBL_Destruct(OutTable);

	return return_value;
}

void write_DbvPresentResponse(DbvPresentRsp *psPresentRsp, TBL OutTable)
{
	char temp[STRING_LEN];
	int ulNumberOfRecordsReturned;
	char *pcDbvPresentStatus;
	DbvPresentStatus ePresentStatus;

        if( bLocalResult == DBV_TRUE)
                write_comment_field("Completed structure DbvPresentResponse",
                        OutTable, T_Return);
        else
        {
		write_comment_field("DbvPresentResponse received from Target", 
			OutTable, T_Return);
		write_comment_field("Structure detailed - DbvOriginData.uOriginEvent->psPresentRsp:", OutTable, T_Return);
	}

/* ReferenceId is an optional field */
	write_DbvOctetString("psReferenceId", psPresentRsp->psReferenceId, OutTable);

/* ulNumberOfRecordsReturned */
	write_title_int( "ulNumberOfRecordsReturned",(int *)&(psPresentRsp->ulNumberOfRecordsReturned), OutTable);
	TBL_PutRecord(OutTable);

/* ulNextResultSetPosition */
	write_title_int( "ulNextResultSetPosition",(int *)&(psPresentRsp->ulNextResultSetPosition), OutTable);
	TBL_PutRecord(OutTable);

/* ePresentStatus */
        pcDbvPresentStatus= NULL;
        convert_DbvPresentStatus(&pcDbvPresentStatus,&(psPresentRsp->ePresentStatus));
	write_title_char(pcDbvPresentStatus, "ePresentStatus", OutTable);
	DbvFree(pcDbvPresentStatus);
	TBL_PutRecord(OutTable);


/* Records */
	if(psPresentRsp->psRecords != NULL)
	{
	write_DbvRecords(psPresentRsp->psRecords, OutTable);
	}
	else
		write_title_char("NULL", "psRecords", OutTable);

/* Other Information */
   writeN_DbvOtherInformation(psPresentRsp->ppsOtherInformation, OutTable);

       write_comment_field(" End ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
}
 
