/*  Module Name: @(#)o_scan.c	7.1
    Version: 11/15/96
    Author: Peter Watson

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to write results/read arguments to/from origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.

    Modification History.

    29-05-95	D.B.Maskell	Continued implementation.
*/
#include "o_scan.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvScanRequest()							!*/
/*! Read in DbvScanRequest tbl file 					!*/
/*+---------------------------------------------------------------------+*/
int do_DbvScanRequest(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   lAssocId;
   DbvScanReq 	   	   *psScanReq;
   DbvInternationalString  **ppsDatabaseNames;
   DbvError		   *psError = NULL;
   DbvAttributesPlusTerm   *psAttributesPlusTerm;


   return_value = open_tables("scanreq", eTestType, InFilename, OutFilename , 
					&InTable, &OutTable);
   if( return_value == SI_NOTOK )
      return return_value;

   write_comment_field( "\nResults of ScanRequest\n", OutTable, T_Return );

   TBL_GetRecord( InTable );	
   do {
   psScanReq = ( DbvScanReq* )DbvAlloc( sizeof( DbvScanReq ));

      /* Assoc Id */

      intfield = get_int_field( InTable, T_Return );
      lAssocId = get_int_field( InTable, T_Return );

      /* ReferenceId */

      TBL_GetRecord( InTable );
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psScanReq->psReferenceId ), InTable);

      /* Get the Number of Database Names */

      TBL_GetRecord(InTable);

      /* Ignore title */
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree(stringfield);

      intfield = get_int_field(InTable,T_Return);

      /* Database Names */

      if ( intfield > 0 )
	 {
         ppsDatabaseNames = ( DbvInternationalString **)DbvAlloc( 
			( sizeof( DbvInternationalString* ) * intfield + 1 ));
         psScanReq->ppsDatabaseNames = ppsDatabaseNames;

         for ( ; intfield; intfield-- )
	    {
            TBL_GetRecord(InTable);
            get_DbvInternationalString( ppsDatabaseNames, InTable );
	    ppsDatabaseNames++;
	    } /* End for */
 
         *ppsDatabaseNames = NULL;

	 } /* End if */

      /* Attribute Set Id */

      TBL_GetRecord( InTable );
      get_DbvObjectIdentifier( &psScanReq->pcAttributeSetId, InTable );
       /* AtributesPlusTerm */

      psAttributesPlusTerm = &psScanReq->sAttributesPlusTerm;
      get_DbvAttributesPlusTerm( &psAttributesPlusTerm, InTable );

      /*  Step Size */

      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );
      psScanReq->pulStepSize = (Ulong*)get_integer_string( InTable );

      /* Number of terms requested */

      TBL_GetRecord( InTable );
      /* Ignore Title */
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );
      psScanReq->ulNumberOfTermsRequested = get_int_field( InTable,T_Return );

      /* Preferred position in response */
      TBL_GetRecord( InTable );
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );
      psScanReq->pulPreferredPositionInResponse = 
					(Ulong*)get_integer_string(InTable);

      /* OtherInformation */

      getN_DbvOtherInformation( &psScanReq->ppsOtherInformation, InTable );

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvScanRequest(psScanReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

      /* Now execute the function */

      printf( "Press enter to execute DbvScanRequest\n" );
      fflush( stdin );
      get_return();

      return_value = DbvScanRequest( lAssocId, psScanReq, &psError );

      if( return_value == SI_OK )
         {
         printf( "ScanRequest Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );

         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psError, OutTable );
         } /* End else */

      Free_DbvScanReq(psScanReq);
      TBL_GetRecord( InTable );
      out_record = 1; 
      } while( TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DbvScanRequest( psScanReq );	*/

   return return_value;
   } /* End do_DbvScanRequest() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvScanResponse()						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvScanResponse( DbvScanRsp *psScanRsp, TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   if( bLocalResult == DBV_TRUE)
          write_comment_field("Completed structure DbvScanResponse",
                  OutTable, T_Return);
   else
   {
   	write_comment_field("ScanResponse received from Target", OutTable, 
		T_Return );
   	write_comment_field("Structure detailed - psScanRsp:", OutTable, 
		T_Return );
   }

   /* ReferenceId is an optional field */
   write_DbvOctetString("psReferenceId", psScanRsp->psReferenceId, 
			OutTable );

   /* Step Size */

   write_integer_string( "pulStepSize", (int *)psScanRsp->pulStepSize,OutTable);
   TBL_PutRecord( OutTable );

   /* Scan Status */

   convert_DbvScanStatus( &pc_string, &psScanRsp->eScanStatus );
   sprintf(temp, "eScanStatus : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Number Of Entries Returned */

   sprintf( temp, "ulNumberOfEntriesReturned: %ld", 
		         psScanRsp->ulNumberOfEntriesReturned );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Position Of Term */

   write_integer_string( "pulPositionOfTerm", (int *)psScanRsp->
	pulPositionOfTerm, OutTable );
   TBL_PutRecord( OutTable );

   /* Entries */

   write_DbvListEntries( psScanRsp->psEntries, OutTable );

   /* Attribute Set Id */

   write_DbvObjectIdentifier( "pcAttributeSetId:",
			       psScanRsp->pcAttributeSetId,
			       OutTable );

   /* Write Other Information */
   writeN_DbvOtherInformation(psScanRsp->ppsOtherInformation, OutTable);

   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord( OutTable );

   } /* End write_DbvScanResponse() */

