/*  Module Name:  @(#)o_sort.c	7.1
    Version: 11/15/96
    Author: Peter Watson

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to write results/read arguments to/from origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.

    Modification History.

    25-05-95	D.B.Maskell	Continued implementation.
*/

#include "o_sort.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvSortRequest()							!*/
/*! Read in DbvSortRequest tbl file 					!*/
/*+---------------------------------------------------------------------+*/
int do_DbvSortRequest( TestType eTestType )
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   lAssocId;
   DbvSortReq 		   *psSortReq;
   DbvError		   *psError = NULL;
   DbvInternationalString  **ppsInputResultSetName;
   DbvInternationalString  *psSortedResultSetName;
   DbvSortKeySpec	   **ppsSortSequence;


   return_value = open_tables("sortreq", eTestType, InFilename, OutFilename , 
						 &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of SortRequest\n", OutTable, T_Return);

   TBL_GetRecord( InTable );
   do {
   psSortReq = (DbvSortReq *) DbvAlloc (sizeof(DbvSortReq));

      /* Assoc Id */

      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* ReferenceId */

      TBL_GetRecord( InTable );
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psSortReq->psReferenceId ), InTable );

      /* Get the number of Input Result Set Names */
      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      intfield = get_int_field( InTable,T_Return );

      /* InputResultSetName list  */

      if ( intfield )
	 {
         ppsInputResultSetName = ( DbvInternationalString **)DbvAlloc( 
			( sizeof( DbvInternationalString* ) * intfield + 1 ));
         psSortReq->ppsInputResultSetName = ppsInputResultSetName;

         for ( ; intfield; intfield-- )
	    {
            TBL_GetRecord( InTable );
            get_DbvInternationalString( ppsInputResultSetName, InTable );
	    ppsInputResultSetName++;
	    } /* End for */
 
         *ppsInputResultSetName = NULL;
	 } /* End if */

      /* Sorted Result Set Name */
      
      TBL_GetRecord( InTable );

      /* Ignore title */
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      get_DbvInternationalString( &psSortedResultSetName, InTable );
      psSortReq->sSortedResultSetName.ulLength = 
				psSortedResultSetName->ulLength; 
      psSortReq->sSortedResultSetName.pcValue  = 
				psSortedResultSetName->pcValue; 

      DbvFree( psSortedResultSetName );

      /* Get the number of Sort Sequences */
      TBL_GetRecord( InTable );

      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      intfield = get_int_field( InTable,T_Return );

      /* SortSequence list  */

      if ( intfield > 0)
	 {
         ppsSortSequence = ( DbvSortKeySpec **)DbvAlloc( 
			( sizeof( DbvSortKeySpec* ) * (intfield + 1 )));
         psSortReq->ppsSortSequence = ppsSortSequence;

         for ( ; intfield; intfield-- )
	    {
            get_DbvSortKeySpec( ppsSortSequence, InTable );
	    ppsSortSequence++;
	    } /* End for */
	 } /* End if */
 
      *ppsSortSequence = NULL;

      /* OtherInformation */

      getN_DbvOtherInformation( &psSortReq->ppsOtherInformation, InTable );

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvSortRequest(psSortReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

      /* Now execute the function */

      printf( "Press enter to execute DbvSortRequest\n" );
      fflush( stdin );
      get_return();

      return_value = DbvSortRequest(lAssocId, psSortReq, &psError);

      if( return_value == SI_OK )
         {
         printf( "SortRequest Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psError, OutTable );
         } /* End else */

      Free_DbvSortReq(psSortReq);
      TBL_GetRecord( InTable );
      out_record = 1; 
      } while( TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_SortReq( psSortReq );	*/

   return return_value;
   } /* End do_DbvSortRequest() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvSortResponse() 						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvSortResponse(DbvSortRsp *psSortRsp, TBL OutTable)
   {
   char temp[STRING_LEN];
   char *pc_string = NULL;
   DbvDiagRec	**ppsDiagnostics;

   if( bLocalResult == DBV_TRUE)
           write_comment_field("Completed structure DbvSortResponse",
                   OutTable, T_Return);
   else
   {
   	write_comment_field("SortResponse received from Target", OutTable, 
		T_Return);
   	write_comment_field("Structure detailed - DbvOriginData.uOriginEvent->psSortRsp:", OutTable, T_Return);
   }

   /* ReferenceId is an optional field */
   write_DbvOctetString("psReferenceId", psSortRsp->psReferenceId, OutTable);

   /* Sort Status */

   strcpy(temp, "eSortStatus : ");
   convert_DbvSortStatus( &pc_string, &psSortRsp->eSortStatus );
   strcat( temp, pc_string );
   DbvFree( pc_string );
   pc_string=NULL;

   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Result Set Status */
   if(psSortRsp->peResultSetStatus != NULL)
   {
        convert_DbvSortResultSetStatus( &pc_string, 
		psSortRsp->peResultSetStatus );
   	sprintf(temp, "peResultSetStatus: %s", pc_string);
        DbvFree( pc_string );
   }
   else
	sprintf(temp, "peResultSetStatus: NULL", pc_string);

   pc_string=NULL;
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Diagnostics : Optional */

   write_comment_field( "START of ppsDiagnostics", OutTable, T_Return );
   ppsDiagnostics = psSortRsp->ppsDiagnostics;
   while ( ppsDiagnostics && *ppsDiagnostics )
      {
      write_DbvDiagRec( *ppsDiagnostics, OutTable );
      ppsDiagnostics++;
      } /* End while */

   write_comment_field( "END of ppsDiagnostics", OutTable, T_Return );
   /* Other Information */

   writeN_DbvOtherInformation(psSortRsp->ppsOtherInformation, OutTable);
   
   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvSortResponce() */
