/*  Module Name: @(#)o_srch.c	7.1
    Version: 11/15/96
    Author: Peter Watson
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments to/from origin specific tbl
        files associated with the Z39.50 functions. For use with the testtool.
 
    Modification History.

    22-05-95	D.B.Maskell	Continued implementation.
*/
#include "o_srch.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvSearchRequest()						!*/
/*! Read in DbvSearchRequest tbl file					!*/
/*+---------------------------------------------------------------------+*/
int do_DbvSearchRequest(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   lAssocId;
   DbvSearchReq 	   *psSearchReq;
   DbvError		   *psError = NULL;
   DbvInternationalString  **ppsDatabaseNames;
   DbvInternationalString  *psResultSetName;


   return_value = open_tables("srchreq", eTestType, InFilename, OutFilename , &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of SearchRequest\n", OutTable, T_Return);

   TBL_GetRecord(InTable);	/* Believe this should be in the loop */
   do {
   psSearchReq = (DbvSearchReq *) DbvAlloc (sizeof(DbvSearchReq));

      /* Assoc Id */

      intfield = get_int_field( InTable, T_Return );
      lAssocId = get_int_field( InTable, T_Return );

      /* ReferenceId */

      TBL_GetRecord(InTable);
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psSearchReq->psReferenceId ), InTable);

      /* Small Set upper bound */

      psSearchReq->ulSmallSetUpperBound = get_int_field(InTable,T_Return);

      /* Large Set upper bound */
      psSearchReq->ulLargeSetLowerBound = get_int_field(InTable,T_Return);

      /* Medium Set Present Number */
      psSearchReq->ulMediumSetPresentNumber = get_int_field( InTable,
								T_Return );

      /* Replace Indicator */

      psSearchReq->fReplaceIndicator = get_DbvBool( InTable );

      /* Result Set Name  : Yuk */
      
      TBL_GetRecord(InTable);
      get_DbvInternationalString( &psResultSetName, InTable );
      psSearchReq->sResultSetName.ulLength = psResultSetName->ulLength; 
      psSearchReq->sResultSetName.pcValue  = psResultSetName->pcValue; 

      DbvFree( psResultSetName );

      /* Get the Number of Database Names */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree(stringfield);

      intfield = get_int_field(InTable,T_Return);

      /* Database Names */

      if(intfield > 0)
      {
      ppsDatabaseNames = ( DbvInternationalString **)DbvAlloc( 
			( sizeof( DbvInternationalString* ) * (intfield + 1)));
      psSearchReq->ppsDatabaseNames = ppsDatabaseNames;

      for ( ; intfield; intfield-- )
	 {
         TBL_GetRecord(InTable);
         get_DbvInternationalString( ppsDatabaseNames, InTable );
	 ppsDatabaseNames++;
	 } /* End for */
 
      *ppsDatabaseNames = NULL;
      } /* End if */

      /* Small Set Element Names */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree(stringfield);

      if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
	DbvFree(stringfield);
	get_DbvElementSetNames(&(psSearchReq->psSmallSetElementNames ),InTable);
	}
	
	else
	DbvFree(stringfield);
      /* Medium Set Element Names */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree(stringfield);

      if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))!=0)
	{
 	DbvFree(stringfield);
        get_DbvElementSetNames(&(psSearchReq->psMediumSetElementNames ),InTable);
	}
	else
	DbvFree(stringfield);
      /* Preferred Record Syntax */

      TBL_GetRecord(InTable);
      get_DbvObjectIdentifier(&psSearchReq->pcPreferredRecordSyntax, InTable);
      /* Query */

      TBL_GetRecord(InTable);
      /* Ignore the title */
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree(stringfield);
      if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))!=0)
         get_DbvQuery( &psSearchReq->psQuery, InTable );

      /* AdditionalSearchInfo */
      getN_DbvOtherInformation(&(psSearchReq->ppsAdditionalSearchInfo ),
					InTable);

      /* OtherInformation */
      getN_DbvOtherInformation(&(psSearchReq->ppsOtherInformation), InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvSearchRequest(psSearchReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

      /* Now execute the function */

      printf("Press enter to execute DbvSearchRequest\n");
      fflush(stdin);
      get_return();

      return_value = DbvSearchRequest(lAssocId, psSearchReq, &psError);

      if(return_value == SI_OK)
         {
         printf("SearchRequest Ok\n");
         printf("Writing output to %s\n", OutFilename);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("NULL", OutTable, T_Return);
         TBL_PutRecord(OutTable);

         } /* End if */
      else
         {
         printf("Call failed\n");
         write_DbvError(&psError, OutTable);
         } /* End else */

      Free_DbvSearchReq(psSearchReq);
      TBL_GetRecord(InTable);
      out_record = 1; 
      } while( TBL_GetRecord(InTable)!=FALSE );

   TBL_Destruct(InTable);
   TBL_Destruct(OutTable);

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DbvSearchRequest( psSearchReq );	*/

   return return_value;
   } /* End do_DbvSearchRequest() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvSearchResponse()						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvSearchResponse( DbvSearchRsp *psSearchRsp, TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   if( bLocalResult == DBV_TRUE)
           write_comment_field("Completed structure DbvSearchResponse",
		OutTable, T_Return);
   else
   {
   	write_comment_field("SearchResponse received from Target", OutTable, 
			T_Return );
   	write_comment_field("Structure detailed - psSearchRsp:", OutTable, 
			T_Return );
   }

   /* ReferenceId is an optional field */
   write_DbvOctetString("psReferenceId", psSearchRsp->psReferenceId, 
			OutTable );

   /* Result Count */

   sprintf( temp, "ulResultCount: %ld", psSearchRsp->ulResultCount );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Number Of Records Returned */

   sprintf(temp, "ulNumberOfRecordsReturned: %ld", 
				psSearchRsp->ulNumberOfRecordsReturned );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* Next Result Set Position */

   sprintf(temp, "ulNextResultSetPosition: %ld", 
				psSearchRsp->ulNextResultSetPosition );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Search Status */

   convert_DbvBool( &pc_string, &psSearchRsp->fSearchStatus );
   sprintf(temp, "fSearchStatus : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Result Set Status */

   pc_string = NULL;
   if(psSearchRsp->peResultSetStatus != NULL)
   {
   	convert_DbvResultSetStatus( &pc_string, 
		psSearchRsp->peResultSetStatus );
   	sprintf(temp, "psResultSetStatus : %s", pc_string );
        DbvFree( pc_string );
   }
   else
	sprintf(temp, "psResultSetStatus : NULL");
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* Present Status */

   pc_string = NULL;
   if(psSearchRsp->pePresentStatus != NULL)
   {
   	convert_DbvPresentStatus( &pc_string, psSearchRsp->pePresentStatus );
   	sprintf(temp, "pePresentStatus : %s", pc_string );
   	DbvFree( pc_string );
   }
   else
	sprintf(temp, "pePresentStatus : NULL", pc_string );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* Records */
   if(psSearchRsp->psRecords==NULL)
   {
     write_char_field("psSearchRsp->psRecords: NULL", OutTable, T_Return);
   }
   else
   write_DbvRecords( psSearchRsp->psRecords, OutTable );

   /* Write Additional Information */

   writeN_DbvOtherInformation( psSearchRsp->ppsAdditionalSearchInfo, OutTable);

   /* Write Other Information */
   writeN_DbvOtherInformation(psSearchRsp->ppsOtherInformation, OutTable);

   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord( OutTable );

   } /* End write_DbvSearchResponse() */
