/*  Module Name: @(#)o_trigger.c	7.1
    Version: 11/15/96
    Author: John Hough

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
	Functions to read arguments/write results from/to origin specific tbl
	files associated with the Z39.50 functions. For use with the testtool.
*/

#include "o_trigger.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvTriggerResourceControlRequest() 				!/
/*! Read in DbvTriggerResourceControlRequest tbl file 			!*/
/*+---------------------------------------------------------------------+*/
int do_DbvTriggerResourceControlRequest( TestType eTestType )
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;

   DbvAssocId		   			lAssocId;
   DbvTriggerResourceControlReq			*psTriggerResourceControlReq;
   DbvBool                 			*pBool;
   DbvRequestedAction				eRequestedAction;


   DbvError		   			*psError = NULL;



   return_value = open_tables("trigrescntrlreq", eTestType, InFilename, OutFilename , 
						 &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of TriggerResourceControlRequest\n", OutTable, T_Return);

   TBL_GetRecord( InTable );
   do {
   psTriggerResourceControlReq = (DbvTriggerResourceControlReq *) DbvAlloc (sizeof(DbvTriggerResourceControlReq));

      /* Assoc Id */
      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* o ReferenceId */
      TBL_GetRecord( InTable );
      intfield = get_int_field(InTable,T_Return);
      get_DbvOctetString( &(psTriggerResourceControlReq->psReferenceId ), InTable );

      /* c RequestedAction */
      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );

      stringfield = get_string_field( InTable, T_Return );
      convert_DbvRequestedAction( &stringfield, &eRequestedAction );
      psTriggerResourceControlReq->eRequestedAction = eRequestedAction;


      /* o PreferredResourceReportFormat */
      TBL_GetRecord( InTable );
      get_DbvObjectIdentifier( &psTriggerResourceControlReq->pcPreferredResourceReportFormat, InTable );

      /* o ResultSetWanted */
      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return );
      DbvFree(stringfield);
      stringfield = get_string_field( InTable, T_Return );
      if( (strcmp(stringfield, NOTHING)) != 0 )  
      {
      pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
      *pBool = get_DbvBool( InTable );
      psTriggerResourceControlReq->pfResultSetWanted = pBool;
      } /* End if */
      DbvFree(stringfield);


      /* o OtherInformation */
      getN_DbvOtherInformation( &psTriggerResourceControlReq->ppsOtherInformation, InTable );


        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvTriggerResourceControlRequest(psTriggerResourceControlReq, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;


      /* Now execute the function */

      printf( "Press enter to execute DbvTriggerResourceControlReq\n" );
      fflush( stdin );
      get_return();

      return_value = DbvTriggerResourceControlRequest(lAssocId, psTriggerResourceControlReq, &psError);

      if( return_value == SI_OK )
         {
         printf( "TriggerResourceControlReq Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psError, OutTable );
         } /* End else */

      Free_DbvTriggerResourceControlReq(psTriggerResourceControlReq);
      TBL_GetRecord( InTable );
      out_record = 1; 
      } while( TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DbvTriggerResourceControlReq( psTriggerResourceControlReq );	*/

   return return_value;
   } /* End do_DbvTriggerResourceControlRequest() */


