/*  Module Name: @(#)origin.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        DBV-OSI II Origin Test Tool. Includes Association control and covers
	both TCP and OSI stacks. For origin specific Z39.50 functions view the
	file origin_funcs.c.
 
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>

#include "sidefs.h"
#include "sitypes.h"
#include "sitypes2.h"
#include "dbvproto.h"

#include "memcheck.h"
#include "sl_list.h"
#include "tbl.h"
#include "str_hand.h"

#include "testy_calls.h"
#include "origin_funcs.h"
#include "o_init.h"
#include "o_srch.h"
#include "o_pres.h"
#include "o_del.h"
#include "o_scan.h"
#include "o_res.h"
#include "o_sort.h"
#include "o_ext.h"
#include "o_seg.h"
#include "o_close.h"
#include "apifree.h"


int test_association_handling(void);
int test_z3950(void);
int test_non_comms(void);

void tprintf(char *data);

/* Functions to test API functions */
int do_DbvAssociateRequest(TestType eTestType);
int do_DbvConfirmAssociateRequest(TestType eTestType);
int do_DbvReleaseRequest(TestType eTestType);


extern char pcFileExtension[];
extern char pcTBLDirectory[];
extern char string[];
extern DbvBool bPrintInput;

extern TBL_Return T_Return;
/* Look in sil_defs.h for DBVOriginOrTarget */
extern int eOriginOrTarget;

/* State machine off/on variable (DBV_FALSE | DBV_TRUE) */
extern int DBVStateMachine;

void main()
{
	char choice_string[100];
	int choice = 0;
	char *path;
	int result;

#ifdef MALLOC_DEBUG
	/*malloc_debug(2);*/
#endif

DBVStateMachine = DBV_TRUE;

	/* Set the global variable so that functions load correct tbl files */
	eOriginOrTarget = ORIGIN;
	/* This tells us if we're writing the results before they've been sent
		or after they've been received. */
	bLocalResult = DBV_FALSE;
	printf("DBV-OSI II Origin Test Tool (c) CSL\n");
	printf("-----------------------------------\n");

	if((path=getcwd((char *)NULL, 128)) == NULL)
	{
		perror("cwd");
		exit(1);
	}
	printf("\n\nMake sure that targetd has been started\n");
	printf("Using:\n\t-e %s/../target/target\nas non-interactive target process\n",path);

/*
	strcpy(pcTBLDirectory, TBLDIR);
	printf("Directory which holds origin TBL files is :%s\n", pcTBLDirectory);
*/
	T_Return = SMALLOC_(struct tbl_return , "Origin test tool");

	strcpy(pcFileExtension, "");
	/* Remove the old Origin results */
	remove_old_results(pcTBLDirectory);

	bPrintInput = DBV_TRUE;

	printf("\n\n");
	/* Configure the origin */
	config_OriginConfigure();

	/* Do the batch processing (if any) */
	batch_DoOriginBatchProcessing();

	printf("bPrintInput is currently set to TRUE\n");
	
GroupToTest:
	choice = 0; result = 0;
	while(choice < 1 || choice > 4)
	{
		printf("\nChoose Origin API Function Group To Test\n");
		printf("1) Association Handling\n");
		printf("2) SR-/Z39.50 and Communication Utility Functions\n");
		printf("3) Non Communication Utility Functions\n");
		printf("4) Exit\n");
		printf("Choice (1/2/3/4) :");
		fflush(stdout);
		gets(choice_string);
		choice = atoi(choice_string);
	}
	switch(choice)
	{
	case 1:
		result = test_association_handling();
		break;
	case 2:
		result = test_z3950();
		break;
	case 3:
		result = test_non_comms();
		break;
	case 4:
		goto Quit;
		break;
	default:
		break;
	}
	if(result == SI_PREV)
		goto GroupToTest;

Quit:
        DbvFree(T_Return);
}

int test_association_handling()
{
	int choice = 0;
	int return_value = SI_OK;
	char choice_string[100];

	while(choice < 1 || choice > 11)
	{
		printf("Choose Origin Association Handling Function\n");
		printf("1) Valid DbvAssociateRequest()\n");
		tprintf("Input From:\t assocreq_valid");
		tprintf("Output To:\t assocreq_valid_result");

		printf("2) Invalid DbvAssociateRequest()\n");
		tprintf("Input From:\t assocreq_invalid");
		tprintf("Output To:\t assocreq_invalid_result");

		printf("3) Valid DbvConfirmAssociateRequest()\n");
		tprintf("Input From:\t confasreq_valid");
		tprintf("Output To:\t confasreq_valid_result");

		printf("4) Invalid DbvConfirmAssociateRequest()\n");
		tprintf("Input From:\t confasreq_invalid");
		tprintf("Output To:\t confasreq_invalid_result");

		printf("5) Valid DbvReleaseRequest()\n");
		tprintf("Input From:\t relreq_valid");
		tprintf("Output To:\t relreq_valid_result");

		printf("6) Invalid DbvReleaseRequest()\n");
		tprintf("Input From:\t relreq_invalid");
		tprintf("Output To:\t relreq_invalid_result");

		printf("7) Valid DbvConfirmReleaseRequest()\n");
		tprintf("Input From:\t confrereq_valid");
		tprintf("Output To:\t confrereq_valid_result");

		printf("8) Invalid DbvConfirmReleaseRequest()\n");
		tprintf("Input From:\t confrereq_invalid");
		tprintf("Output To:\t confrereq_invalid_result");

		printf("9) Valid DbvAbortRequest()\n");
		tprintf("Input From:\t abortreq_valid");
		tprintf("Output To:\t abortreq_valid_result");

		printf("10) Invalid DbvAbortRequest()\n");
		tprintf("Input From:\t abortreq_invalid");
		tprintf("Output To:\t abortreq_invalid_result");

		printf("11) Previous Menu\n");
		printf("Choice (1/2/3/4/5/6/7/8/9/10/11) :");
		fflush(stdout);
		gets(choice_string);
		choice = atoi(choice_string);

		switch(choice)
		{
		case 1:
			return_value = do_DbvAssociateRequest(Valid);
			choice = -1;
			break;
		case 2: return_value = do_DbvAssociateRequest(Invalid);
			choice = -1;
			break;
		case 3: return_value = do_DbvConfirmAssociateRequest(Valid);
			choice = -1;
			break;
		case 4: return_value = do_DbvConfirmAssociateRequest(Invalid);
			choice = -1;
			break;
		case 5: return_value = do_DbvReleaseRequest(Valid);
			choice = -1;
			break;
		case 6: return_value = do_DbvReleaseRequest(Invalid);
			choice = -1;
			break;
		case 7: return_value = do_DbvConfirmReleaseRequest(Valid);
			choice = -1;
			break;
		case 8: return_value = do_DbvConfirmReleaseRequest(Invalid);
			choice = -1;
			break;
		case 9: return_value = do_DbvAbortRequest(Valid);
			choice = -1;
			break;
		case 10:return_value = do_DbvAbortRequest(Invalid);
			choice = -1;
			break;
		case 11:
			return_value = SI_PREV;
			break;
		default:
			printf("Not currently Implemented!\n");
			choice = -1;
			break;
		}
		printf("Press enter\n");
		fflush(stdin);
		gets(string);
	}

	return return_value;
}

int test_z3950()
{
	int choice = 0;
	int return_value = SI_OK;
	char choice_string[100];

	while(choice < 1 || choice > 35)
	{
		printf("Choose Origin SR-/Z39.50 / Communication Utility Function:\n");
		printf("SR-/Z39.50 Functions\n");
		
		printf("1) Valid DbvInitializeRequest()\n");
		tprintf("Input From:\tinitreq_valid");
		tprintf("Output To:\tinitreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tinitreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tinitresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tinitresp_valid_received");

		printf("2) Invalid DbvInitializeRequest()\n");
		tprintf("Input From:\tinitreq_invalid");
		tprintf("Output To:\tinitreq_invalid_result");
		printf("\n");

		printf("3) Valid DbvSearchRequest()\n");
		tprintf("Input From:\tsrchreq_valid");
		tprintf("Output To:\tsrchreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tsrchreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tsrchresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tsrchresp_valid_received");

		printf("4) Invalid DbvSearchRequest()\n");
		tprintf("Input From:\tsrchreq_invalid");
		tprintf("Output To:\tsrchreq_invalid_result");
		printf("\n");

		printf("5) Valid DbvPresentRequest()\n");
		tprintf("Input From:\tpresreq_valid");
		tprintf("Output To:\tpresreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tpresreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tpresresp_valid");
		printf("\tor\n");
		tprintf("\t\tsegreq_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tpresresp_valid_received");
		printf("\tor\n");
		tprintf("\t\tsegreq_valid_received");
		
		printf("6) Invalid DbvPresentRequest()\n");
		tprintf("Input From:\tpresreq_invalid");
		tprintf("Output To:\tpresreq_invalid_result");
		printf("\n");

		printf("7) Valid DbvDeleteResultSetRequest()\n");
		tprintf("Input From:\tdelreq_valid");
		tprintf("Output To:\tdelreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tdelreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tdelresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tdelresp_valid_received");
		
		printf("8) Invalid DbvDeleteResultSetRequest()\n");
		tprintf("Input From:\tdelreq_invalid");
		tprintf("Output To:\tdelreq_invalid_result");
		printf("\n");

		printf("9) Valid DbvScanRequest()\n");
		tprintf("Input From:\tscanreq_valid");
		tprintf("Output To:\tscanreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tscanreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tscanresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tscanresp_valid_received");
		
		printf("10) Invalid DbvScanRequest()\n");
		tprintf("Input From:\tscanreq_invalid");
		tprintf("Output To:\tscanreq_invalid_result");
		printf("\n");

		printf("11) Valid DbvResourceReportRequest()\n");
		tprintf("Input From:\tresreq_valid");
		tprintf("Output To:\tresreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tresnreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tresnresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tresresp_valid_received");
		
		printf("12) Invalid DbvResourceReportRequest()\n");
		tprintf("Input From:\tresreq_invalid");
		tprintf("Output To:\tresreq_invalid_result");
		printf("\n");

		printf("13) Valid DbvSortRequest()\n");
		tprintf("Input From:\tsortreq_valid");
		tprintf("Output To:\tsortreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tsortreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tsortresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tsortesp_valid_received");
		
		printf("14) Invalid DbvSortRequest()\n");
		tprintf("Input From:\tsortreq_invalid");
		tprintf("Output To:\tsortreq_invalid_result");
		printf("\n");

		printf("15) Valid DbvExtendedServicesRequest()\n");
		tprintf("Input From:\textreq_valid");
		tprintf("Output To:\textreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\textreq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\textresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\textresp_valid_received");
		
		printf("16) Invalid DbvExtendedServicesRequest()\n");
		tprintf("Input From:\textreq_invalid");
		tprintf("Output To:\textreq_invalid_result");
		printf("\n");

		printf("17) Valid DbvCloseRequest()\n");
		tprintf("Input From:\tclosereq_valid");
		tprintf("Output To:\tclosereq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tclosereq_valid_received");
		printf("\tTarget Response From:\n");
		tprintf("\t\tcloseresp_valid");
		printf("\tResponse From Target To:\n");
		tprintf("\t\tcloseresp_valid_received");
		
		printf("18) Invalid DbvCloseRequest()\n");
		tprintf("Input From:\tclosereq_invalid");
		tprintf("Output To:\tclosereq_invalid_result");
		printf("\n");

		printf("19) Valid DbvCloseResponse()\n");
		tprintf("Input From:\tcloseresp_valid");
		tprintf("Output To:\tcloseresp_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\tcloseresp_valid_received");
		
		printf("20) Invalid DbvCloseResponse()\n");
		tprintf("Input From:\tcloseresp_invalid");
		tprintf("Output To:\tcloseresp_invalid_result");
		printf("\n");

		printf("21) Valid DbvAccessControlRsp()\n");
		tprintf("Input From:\taccessresp_valid");
		tprintf("Output To:\taccessresp_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\taccessresp_valid_received");
		
		printf("22) Invalid DbvAccessControlRsp()\n");
		tprintf("Input From:\taccessresp_invalid");
		tprintf("Output To:\taccessresp_invalid_result");
		printf("\n");

		printf("23) Valid DbvResourceControlRsp()\n");
		tprintf("Input From:\trescntrlresp_valid");
		tprintf("Output To:\trescntrlresp_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\trescntrlresp_valid_received");
		
		printf("24) Invalid DbvResourceControlRsp()\n");
		tprintf("Input From:\trescntrlresp_invalid");
		tprintf("Output To:\trescntrlresp_invalid_result");
		printf("\n");

		printf("25) Valid DbvTriggerResourceControlReq()\n");
		tprintf("Input From:\ttrigrescntrlreq_valid");
		tprintf("Output To:\ttrigrescntrlreq_valid_result");
		printf("\tRequest at Target To:\n");
		tprintf("\t\ttrigrescntrl_valid_received");
		
		printf("26) Invalid DbvTriggerResourceControlReq()\n");
		tprintf("Input From:\ttrigrescntrl_invalid");
		tprintf("Output To:\ttrigrescntrl_invalid_result");
		printf("\n");


		printf("--------------------------------------------\n");
		printf("Communication Related Utility functions\n");

		printf("31) Valid SIGetEventLocation()\n");
		tprintf("Input From:\torigin_sievent_valid");
		tprintf("Output To:\torigin_sievent_valid_result");

		printf("32) Invalid SIGetEventLocation()\n");
		tprintf("Input From:\torigin_sievent_invalid");
		tprintf("Output To:\torigin_sievent_invalid_result");

		printf("33) Valid DbvReceiveDataOrigin()\n");
		tprintf("Input From:\torigin_receive_valid");
		tprintf("Output To:\torigin_receive_valid_result");

		printf("34) Invalid DbvReceiveDataOrigin()\n");
		tprintf("Input From:\torigin_receive_invalid");
		tprintf("Output To:\torigin_receivet_invalid_result");


		printf("35) Previous Menu\n");
		printf("Choice (1/2/3/4/5/6/7/8/9/10/11/12/13/14/15/16/17/18/19/20/21/22/23/24/25/26/31/32/33/34/35) :");
		fflush(stdout);
		gets(choice_string);
		choice = atoi(choice_string);
		switch(choice)
		{
		case 1:
			do_DbvInitializeRequest(Valid);
			choice = -1;
			break;
		case 2:
			do_DbvInitializeRequest(Invalid);
			choice = -1;
			break;
		case 3:
			do_DbvSearchRequest(Valid);
			choice = -1;
			break;
		case 4:
			do_DbvSearchRequest(Invalid);
			choice = -1;
			break;
		case 5:
			do_DbvPresentRequest(Valid);
			choice = -1;
			break;
		case 6:
			do_DbvPresentRequest(Invalid);
			choice = -1;
			break;
		case 7:
			do_DbvDeleteResultSetRequest(Valid);
			choice = -1;
			break;
		case 8:
			do_DbvDeleteResultSetRequest(Invalid);
			choice = -1;
			break;
		case 9:
			do_DbvScanRequest(Valid);
			choice = -1;
			break;
		case 10:
			do_DbvScanRequest(Invalid);
			choice = -1;
			break;
		case 11:
			do_DbvResourceReportRequest(Valid);
			choice = -1;
			break;
		case 12:
			do_DbvResourceReportRequest(Invalid);
			choice = -1;
			break;
		case 13:
			do_DbvSortRequest(Valid);
			choice = -1;
			break;
		case 14:
			do_DbvSortRequest(Invalid);
			choice = -1;
			break;
		case 15:
			do_DbvExtendedServicesRequest(Valid);
			choice = -1;
			break;
		case 16:
			do_DbvExtendedServicesRequest(Invalid);
			choice = -1;
			break;
		case 17:
			do_DbvCloseRequest(Valid);
			choice = -1;
			break;
		case 18:
			do_DbvCloseRequest(Invalid);
			choice = -1;
			break;
		case 19:
			do_DbvCloseResponse(Valid);
			choice = -1;
			break;
		case 20:
			do_DbvCloseResponse(Invalid);
			choice = -1;
			break;



		case 21:
			do_DbvAccessControlResponse(Valid);
			choice = -1;
			break;
		case 22:
			do_DbvAccessControlResponse(Invalid);
			choice = -1;
			break;
		case 23:
			do_DbvResourceControlResponse(Valid);
			choice = -1;
			break;
		case 24:
			do_DbvResourceControlResponse(Invalid);
			choice = -1;
			break;
		case 25:
			do_DbvTriggerResourceControlRequest(Valid);
			choice = -1;
			break;
		case 26:
			do_DbvTriggerResourceControlRequest(Invalid);
			choice = -1;
			break;

		case 31:
			do_SIGetEventLocation(Valid);
			choice = -1;
			break;
		case 32:
			do_SIGetEventLocation(Invalid);
			choice = -1;
			break;
		case 33:
			do_DbvReceiveDataOrigin(Valid);
			choice = -1;
			break;
		case 34:
			do_DbvReceiveDataOrigin(Invalid);
			choice = -1;
			break;
		case 35:
			return_value = SI_PREV;
			break;
		default:
			printf("Not currently Implemented!\n");
			choice = -1;
			break;
		}
		printf("Press enter\n");
		fflush(stdin);
		gets(string);
	}

	return return_value;
}

int test_non_comms()
{
	int choice = 0;
	int return_value = 0;
	char choice_string[100];

	while(choice < 1 || choice > 7)
	{
		printf("Choose Origin Non Communication Utility Function:\n");

		printf("1) Valid SISetACSEFile()\n");
		tprintf("Input From:\torigin_acse_valid");
		tprintf("Output To:\torigin_acse_valid_result");

		printf("2) Invalid SISetACSEFile()\n");
		tprintf("Input From:\torigin_acse_invalid");
		tprintf("Output To:\torigin_acse_invalid_result");

		printf("3) Valid SIWhichStack()\n");
		tprintf("Input From:\torigin_stack_valid");
		tprintf("Output To:\torigin_stack_valid_result");

		printf("4) Invalid SIWhichStack()\n");
		tprintf("Input From:\torigin_stack_invalid");
		tprintf("Output To:\torigin_stack_invalid_result");

		printf("5) Valid SISetTraceParameter()\n");
		tprintf("Input From:\torigin_trace_valid");
		tprintf("Output To:\torigin_trace_valid_result");

		printf("6) Invalid SISetTraceParameter()\n");
		tprintf("Input From:\torigin_trace_invalid");
		tprintf("Output To:\torigin_trace_invalid_result");
		
		printf("7) Previous Menu\n");
		printf("Choice (1/2/3/4/5/6/7) :");
		fflush(stdout);
		gets(choice_string);
		choice = atoi(choice_string);

		switch(choice)
		{
		case 1:
			do_SISetACSEFile(Valid);
			choice = -1;
			break;
		case 2:
			do_SISetACSEFile(Invalid);
			choice = -1;
			break;
		case 3:
			do_SIWhichStack(Valid);
			choice = -1;
			break;
		case 4:
			do_SIWhichStack(Invalid);
			choice = -1;
			break;
		case 5:
			do_SISetTraceParameter(Valid);
			choice = -1;
			break;
		case 6:
			do_SISetTraceParameter(Invalid);
			choice = -1;
			break;
		case 7:
			return_value = SI_PREV;
			break;
		default:
			printf("Not currently Implemented!\n");
			choice = -1;
			break;
		}
		printf("Press enter\n");
		fflush(stdin);
		gets(string);
	}

	return return_value;
}

void tprintf(char *data)
{
	printf("%s%s\n",data, pcFileExtension);
}

int do_DbvConfirmAssociateRequest(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;

	int intfield;
	char *stringfield;
	int return_value = SI_OK;
	int out_record = 1;

	DbvAssocId lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
	DbvSecs            lSeconds;
        DbvAssociateState  eAssociateState = Associate_Null;
        DbvAssociateResult *puAssociateResult = NULL;
        DbvError         *psError = NULL;

	return_value = open_assoc_tables("confasreq", eTestType, InFilename, 
		OutFilename, &InTable, &OutTable);

	if(return_value == SI_NOTOK)
		return return_value;
	TBL_GetRecord(InTable);
	write_comment_field("\nResults of DbvConfirmAssociateRequest\n", OutTable, T_Return);
	do {
	intfield=get_int_field(InTable,T_Return);
	lAssocId=intfield=get_int_field(InTable,T_Return);
	TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
	lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
	printf("Press enter to execute DbvConfirmAssociateRequest\n");
	fflush(stdin);
	get_return();
	
	return_value = DbvConfirmAssociateRequest(lAssocId, lSeconds, 
		&eAssociateState, &puAssociateResult, 
		&psError);

        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvSecs(lSeconds, OutTable);
        write_comment_field(" AssociateState: ", OutTable, T_Return);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssociateState(eAssociateState, OutTable);
 
        TT_writeDbvAssociateResult(out_record, puAssociateResult,
                eAssociateState, OutTable);
        if (return_value == SI_OK)
        {
		printf("Confirm Associate Request Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
		printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }

	TBL_GetRecord(InTable);
	TBL_GetRecord(InTable);
	TBL_GetRecord(InTable);
	out_record = 1;
	}
	while(TBL_GetRecord(InTable)!=FALSE);
		
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);

	return return_value;
}

int do_DbvAssociateRequest(TestType eTestType)
{
	
	char InFilename[128];
	char OutFilename[128];
	TBL InTable, OutTable;
	int test;

	int intfield;
	int out_record = 1;
	int return_value = SI_OK;
	char *stringfield;

	DbvAssocId         lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
	DbvSecs 		lSeconds;
	DbvTarget          sTargetId;
	DbvAssociateState  eAssociateState = Associate_Null;
	DbvAssociateResult *puAssociateResult = NULL;
	DbvError         *psError = NULL;

	return_value = open_assoc_tables("assocreq", eTestType, InFilename, OutFilename, &InTable, &OutTable);
	if(return_value == SI_NOTOK)
		return return_value;
		
	TBL_GetRecord(InTable);
	write_comment_field("\nResults of DbvAssociateRequest\n", OutTable, T_Return);
	do {
	TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
	lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
	DbvFree(stringfield);
	TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
	/*intfield = get_int_field(InTable, T_Return);*/
	stringfield=get_string_field(InTable,T_Return);
	convert_DbvTargetNameOrAddress(&stringfield, &(sTargetId.eTargetNameOrAddress));
	DbvFree(stringfield);

        /* Set up structure for TCP communication */
        /*sTargetId.eTargetNameOrAddress = intfield;*/
	if(sTargetId.eTargetNameOrAddress == Target_Address) 
	{
	/* Read in Target_address tbl format */
        sTargetId.uNameOrAddress.psTargetInformation =
            (struct DbvTargetInformation *)
            DbvAlloc(sizeof(struct DbvTargetInformation));
	stringfield=get_string_field(InTable,T_Return);
	convert_DbvOsiOrInternetAddress(&stringfield, &sTargetId.uNameOrAddress.
		psTargetInformation->eOsiOrInternetAddress);
	DbvFree(stringfield);
	if(sTargetId.uNameOrAddress.psTargetInformation->eOsiOrInternetAddress
		== Internet_Address)
	{
	/* Read in TCP format record */
		stringfield=get_string_field(InTable,T_Return);
        	sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psInternetAddress = 
			(struct DbvInternetAddress *) DbvAlloc
			(sizeof(struct DbvInternetAddress));
        	sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psInternetAddress->
			pcHostAddress =(char *)DbvAlloc(strlen(stringfield)+1);
        	strcpy(sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psInternetAddress->pcHostAddress,
			stringfield);
		intfield = get_int_field(InTable, T_Return);
        	sTargetId.uNameOrAddress.psTargetInformation->
		uOsiOrInternetAddress.psInternetAddress->ulPortNumber=intfield;
		DbvFree(stringfield);
	}
	if(sTargetId.uNameOrAddress.psTargetInformation->eOsiOrInternetAddress
		== Osi_Address)
	{
	/* Read in Osi Format record */
        	sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress = 
			(struct DbvOsiAddress *) DbvAlloc
			(sizeof(struct DbvOsiAddress));
		stringfield=get_string_field(InTable,T_Return);
		sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress->pcPsel=stringfield;
		stringfield=get_string_field(InTable,T_Return);
		sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress->pcSsel=stringfield;
		stringfield=get_string_field(InTable,T_Return);
		sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress->pcTsel=stringfield;
		stringfield=get_string_field(InTable,T_Return);
		sTargetId.uNameOrAddress.psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress->pcNsap=stringfield;
	}
	}
	else
	{
		stringfield=get_string_field(InTable,T_Return);
        	sTargetId.uNameOrAddress.pcTargetName = stringfield;
	}
	stringfield = NULL;
	printf("Press enter to execute DbvAssociateRequest\n");
	fflush(stdin);
	get_return();
	
	return_value = DbvAssociateRequest(&lAssocId, lSeconds, 
		&sTargetId, &eAssociateState, &puAssociateResult, 
		&psError);
	printf("Writing output to %s\n", OutFilename);
	write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssocId(lAssocId, OutTable);
	write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvSecs(lSeconds, OutTable);
	write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvTarget(&sTargetId, OutTable);
	write_comment_field(" AssociateState: ", OutTable, T_Return);
	write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssociateState(eAssociateState, OutTable);

	TT_writeDbvAssociateResult(out_record, puAssociateResult, 
		eAssociateState, OutTable);
	if (return_value == SI_OK)
	{
		printf("Associate Request Ok\n");
		write_int_field(out_record++, OutTable, T_Return);
		write_string_field("NULL",OutTable, T_Return);
		TBL_PutRecord(OutTable);
	}
	else
	{
		printf("Call failed\n");
		write_DbvError(&psError, OutTable);
	}

	Free_DbvAssociateResult(puAssociateResult, eAssociateState);

        /* Free sTargetId structure */
        if(sTargetId.eTargetNameOrAddress == Target_Name)
        {
                if(sTargetId.uNameOrAddress.pcTargetName != NULL)
                        DbvFree(sTargetId.uNameOrAddress.pcTargetName);
        }
        else
        {
                if(sTargetId.uNameOrAddress.psTargetInformation->
                        eOsiOrInternetAddress == Internet_Address)
                {
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psInternetAddress->
                                pcHostAddress);
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psInternetAddress);
                }
                else
                {
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psOsiAddress->
                                pcPsel);
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psOsiAddress->
                                pcSsel);
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psOsiAddress->
                                pcTsel);
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psOsiAddress->
                                pcNsap);
                        DbvFree(sTargetId.uNameOrAddress.psTargetInformation->
                                uOsiOrInternetAddress.psOsiAddress);
                }
                DbvFree(sTargetId.uNameOrAddress.psTargetInformation);
        }

	TBL_GetRecord(InTable);
	TBL_GetRecord(InTable);
	TBL_GetRecord(InTable);
	out_record = 1;
	}
	while(TBL_GetRecord(InTable)!=FALSE);

        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);

	return return_value;
}

int do_DbvReleaseRequest(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
 
        DbvAssocId         	lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        DbvSecs            	lSeconds;
	DbvReleaseState		eReleaseState = Release_Null;
	DbvError		*psError=NULL;
	
        return_value = open_assoc_tables("relreq", eTestType, InFilename, 
		OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
	write_comment_field("\nResults of DbvReleaseRequest\n", OutTable, T_Return);
	TBL_GetRecord(InTable);
	do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
        printf("Press enter to execute DbvReleaseRequest\n");
        fflush(stdin);
        get_return();

        return_value = DbvReleaseRequest(lAssocId, lSeconds,
                &eReleaseState, &psError);

        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssocId(lAssocId, OutTable);
	write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvSecs(lSeconds, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
        write_comment_field(" ReleaseState: ", OutTable, T_Return);
	stringfield = NULL;
	convert_DbvReleaseState(&stringfield, &eReleaseState);
	write_string_field(stringfield, OutTable,T_Return);
        TBL_PutRecord(OutTable);
	if(return_value == SI_OK)
	{
       		printf("DbvReleaseRequest Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);

        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);

	return return_value;
}

int do_DbvConfirmReleaseRequest(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
 
        DbvAssocId         lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        DbvSecs            lSeconds;
        DbvReleaseState    eReleaseState = Release_Null;
        DbvError         *psError=NULL;
 
        return_value = open_assoc_tables("confrereq", eTestType, InFilename, 
		OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvConfirmReleaseRequest\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=intfield=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
        printf("Press enter to execute DbvConfirmReleaseRequest\n");
        fflush(stdin);
        get_return();
 
        return_value = DbvConfirmReleaseRequest(lAssocId, lSeconds,
                &eReleaseState, &psError);
 
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvSecs(lSeconds, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
        write_comment_field(" ReleaseState: ", OutTable, T_Return);
        stringfield = NULL;
        convert_DbvReleaseState(&stringfield, &eReleaseState);
        write_string_field(stringfield, OutTable,T_Return);
        TBL_PutRecord(OutTable);
        if(return_value == SI_OK)
        {
        	printf("DbvConfirmReleaseRequest Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

