
#include "origin_batch.h"

extern char *pcBatchFile;
extern char pcBatchFilename[SZ_FILENAME];
extern TTMode eMode;

/* Process the Origin batch file */
int batch_DoOriginBatchProcessing(void)
{
        TBL psBatchTable = NULL;
        char pcInput[3]="  ";
 
        if(eMode != Batch)
                return SI_NOTOK;
 
        printf("Origin Batch Mode\n");
        psBatchTable = TBL_Create(pcBatchFile, TBL_Read);
        if(psBatchTable == NULL)
        {
                printf("Could not open file %s\n", psBatchTable);
                config_OriginQuit();
        }
 
        batch_ParseOriginBatchFile(psBatchTable);
        TBL_Destruct(psBatchTable);
 
        batch_ExecuteOriginBatchFile();
 
        while(*pcInput != 'm' && *pcInput != 'q')
        {
                printf("\n\nBatch run has completed. Do you wish to:\n");
                printf("Revert to (m)enu mode or (q)uit\n");
                fgets(pcInput, 2, stdin);
        }
        if(*pcInput == 'q')
                config_OriginQuit();
 
        eMode = Menu;
 
        return SI_OK;
}

/* run through the batch file and make sure that each entry is valid */
void batch_ParseOriginBatchFile(TBL psBatchTable)
{
        DbvBool fFoundEntry = DBV_FALSE;
        char *pcEntry;
        char *pcFilename;
        long lSub = 0;
 
        printf("Parsing Batch File\n");
        while(TBL_GetRecord(psBatchTable) != DBV_FALSE)
        {
                pcEntry = config_GetCharField(psBatchTable);
                if(pcEntry == NULL)
                        config_OriginQuit();
                lSub = 0; 
		fFoundEntry = DBV_FALSE;
                while( fFoundEntry == DBV_FALSE && lSub < NO_ORIGIN_FUNCTIONS)
                {
                        if((strcmp(psOriginFunctions[lSub].Name, pcEntry)) == 0)
                                fFoundEntry = DBV_TRUE;
                        lSub ++;
                }
                if(fFoundEntry == DBV_FALSE)
                        fFoundEntry = batch_ParseUtilityEntries(pcEntry);
 
                if(fFoundEntry == DBV_FALSE)
                {
                        printf("Entry %s is invalid\n", pcEntry);
                        config_OriginQuit();
                }
                else
                {
                        pcFilename = config_GetStringField(psBatchTable);
                        if(pcFilename != NULL)
                                batch_CheckFile(pcFilename);
                        else
                        {
                                printf("Entry %s has no filename", pcEntry);
                        }
                }
                free(pcFilename);
                free(pcEntry);
        }
        printf("Batch File Ok\n");
}

/* Execute the origin test functions */
void batch_ExecuteOriginBatchFile(void)
{
        TBL psBatchTable;
        char *pcEntry;
        long lSub = 0;
        DbvBool fFoundEntry;
        int return_value = SI_OK;
 
        psBatchTable = TBL_Create(pcBatchFile, TBL_Read);
 
        printf("\nExecuting Batch file\n");
        while(TBL_GetRecord(psBatchTable) != DBV_FALSE)
        {
                lSub = 0; fFoundEntry = DBV_FALSE;
                pcEntry = config_GetCharField(psBatchTable);
                if(pcEntry == NULL)
                        config_OriginQuit();
                while( fFoundEntry == DBV_FALSE && lSub < NO_ORIGIN_FUNCTIONS)
                {
                        if((strcmp(psOriginFunctions[lSub].Name, pcEntry)) == 0)
                                fFoundEntry = DBV_TRUE;
                        else
                                lSub ++;
                }

		if(fFoundEntry == DBV_FALSE)
			batch_ExecuteUtility(pcEntry, 
				config_GetStringField(psBatchTable));
		else
		{
                	return_value = psOriginFunctions[lSub].function(
                        	config_GetStringField(psBatchTable));
                	if(return_value == SI_NOTOK)
                        	return_value = batch_QuitOrContinue();
		}
               	free(pcEntry);
               	sleep(1);
        }
	TBL_Destruct(psBatchTable);
 
}
 
/* Origin Functions */
int batch_ExecDbvAssociateRequest(char *pcFilename)
{
        int return_value;
	char string[2];
 
        printf("***Executing DbvAssociateRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvAssociateRequest(Valid);
 
	printf("\nPress return to continue (or wait for target debugger)\n");
	fgets(string, 2, stdin);

        return return_value;
}
 
int batch_ExecDbvReleaseRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvReleaseRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvReleaseRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvConfirmAssociateRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvConfirmAssociateRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvConfirmAssociateRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvConfirmReleaseRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvConfirmReleaseRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvConfirmReleaseRequest(Valid);
 
        return return_value;
}
 
int batch_ExecDbvInitializeRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvInitializeRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvInitializeRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvSearchRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvSearchRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvSearchRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvPresentRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvPresentRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvPresentRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvDeleteResultSetRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvDeleteResultSetRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvDeleteResultSetRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvScanRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvScanRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvScanRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvResourceReportRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvResourceReportRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvResourceReportRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvSortRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvSortRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvSortRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvExtendedServicesRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvExtendedServicesRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvExtendedServicesRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvReceiveDataOrigin(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvReceiveDataOrigin() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvReceiveDataOrigin(Valid);
 
        return return_value;
}
 
int batch_ExecDbvAccessControlResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvAccessControlResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvAccessControlResponse(Valid);
 
        return return_value;
}
 
int batch_ExecDbvResourceControlResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvResourceControlResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvResourceControlResponse(Valid);
 
        return return_value;
}
 
int batch_ExecDbvTriggerResourceControlRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvTriggerResourceControlRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvTriggerResourceControlRequest(Valid);
 
        return return_value;
}
 


