/*  Module Name: @(#)origin_funcs.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments to/from origin specific tbl
        files associated with the Z39.50 functions. For use with the testtool.
 
*/

#include "origin_funcs.h"

extern TBL_Return T_Return;
extern char string[];

int do_DbvReceiveDataOrigin(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int place = 0;
 
        int intfield;
        char *stringfield;
        int out_record = 1;
        int return_value = SI_OK;
 
        DbvAssocId         lAssocId;
        DbvSecs            lSeconds;
        DbvOriginData      *psOriginData = NULL;
        DbvError           *psError=NULL;
 
        return_value = open_tables("origin_receive", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvReceiveDataOrigin\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId = get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
	DbvFree(stringfield);
        printf("Press enter to execute DbvReceiveDataOrigin\n");
        fflush(stdin);
        get_return();
        return_value = DbvReceiveDataOrigin(lAssocId, lSeconds,
                &psOriginData, &psError);
 
        if(return_value == SI_OK)
        {
                printf("DbvReceiveDataOrigin Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                if(psOriginData == NULL)
		{
			write_int_field(out_record, OutTable, T_Return);
            		write_string_field("NULL", OutTable, T_Return);
			write_comment_field("Function timed out", OutTable, 
				T_Return);
                        TBL_PutRecord(OutTable);
                }
		else
                        write_DbvOriginData(out_record, psOriginData, OutTable);

                out_record++;
                write_int_field(out_record, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
                /* Free the data */
                FreeDbvOriginData(psOriginData);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

/* Extract the fields of the incoming OriginData */
void write_DbvOriginData(int out_record, DbvOriginData *psOriginData,
        TBL OutTable)
{
        TBL ResultOutTable;
        char pcResult[STRING_LEN];
 
        switch (psOriginData->eOriginEvent)
        {
        /* InitResponse from server */
        case OR_InitializeConf:
                open_result_table("initresp", &ResultOutTable, pcResult);
                printf("Writing OR_InitializeConf PDU result to %s\n",
			pcResult); 
		write_comment_field(" InitializeResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvInitializeResponse(psOriginData->uOriginData.psInitializeRsp, ResultOutTable);
                break;

        case OR_SearchConf:
                open_result_table("srchresp", &ResultOutTable, pcResult);
                printf("Writing OR_SearchConf PDU result to %s\n",pcResult);
                write_comment_field(" SearchResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvSearchResponse(psOriginData->uOriginData.psSearchRsp, ResultOutTable);
                break;
        case OR_PresentConf:
                open_result_table("presresp", &ResultOutTable, pcResult);
                printf("Writing OR_PresentConf PDU result to %s\n",pcResult);
                write_comment_field(" PresentResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvPresentResponse(psOriginData->uOriginData.psPresentRsp, ResultOutTable);
                break;
       case OR_DeleteResultSetConf:
                open_result_table("delresp", &ResultOutTable, pcResult);
                printf("Writing OR_DeleteResultSetConf PDU result to %s\n",pcResult);
                write_comment_field(" DeleteResultSetResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvDeleteResultSetResponse(psOriginData->uOriginData.psDeleteResultSetRsp, ResultOutTable);
                break;

        case OR_ScanConf:
                open_result_table("scanresp", &ResultOutTable, pcResult);
                printf("Writing OR_ScanConf PDU result to %s\n",pcResult);
                write_comment_field(" ScanResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvScanResponse(psOriginData->uOriginData.psScanRsp, ResultOutTable);
                break;

        case OR_ResourceReportConf:
                open_result_table("rescresp", &ResultOutTable, pcResult);
                printf("Writing OR_ResourceReportConf PDU result to %s\n",pcResult);
                write_comment_field(" ResourceReportResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvResourceReportResponse(psOriginData->uOriginData.psResourceReportRsp, ResultOutTable);
                break;

        case OR_SegmentationInd:
                open_result_table("segresp", &ResultOutTable, pcResult);
                printf("Writing OR_SegmentationInd PDU result to %s\n",pcResult);
                write_comment_field(" SegmentRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvSegmentRequest(psOriginData->uOriginData.psSegmentReq, ResultOutTable);
                break;
        case OR_SortConf:
                open_result_table("sortresp", &ResultOutTable, pcResult);
                printf("Writing OR_SortConf PDU result to %s\n",pcResult);
                write_comment_field(" SortResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvSortResponse(psOriginData->uOriginData.psSortRsp, ResultOutTable);
                break;

        case OR_ExtendedServicesConf:
                open_result_table("extresp", &ResultOutTable, pcResult);
                printf("Writing OR_ExtendedServicesConf PDU result to %s\n",pcResult);
                write_comment_field(" ExtendedServicesResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvExtendedServicesResponse(psOriginData->uOriginData.psExtendedServicesRsp, ResultOutTable);
                break;

        case OR_CloseConf:
                open_result_table("closeresp", &ResultOutTable, pcResult);
                printf("Writing OR_CloseConf PDU result to %s\n",pcResult);
                write_comment_field(" CloseResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvCloseResponse(psOriginData->uOriginData.psCloseReq, ResultOutTable);
                break;
        case OR_CloseInd:
		open_result_table("closereq", &ResultOutTable, pcResult);
                printf("Writing OR_CloseInd PDU result to closereq_valid_received\n");
                write_comment_field(" CloseResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvCloseRequest(psOriginData->uOriginData.psCloseReq, ResultOutTable);
                break;
        case OR_Invalid_APDU:
                break;
        case OR_AccessCntrlInd:
		open_result_table("accessreq", &ResultOutTable, pcResult);
                printf("Writing OR_AccessCntrlInd PDU result to accessreq_valid_received\n");
                write_comment_field(" AccessControlRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvAccessControlRequest(psOriginData->uOriginData.psAccessControlReq, ResultOutTable);
                break;
        case OR_ResourCntrlInd:
		open_result_table("rescntrlreq", &ResultOutTable, pcResult);
                printf("Writing OR_ResourCntrlInd PDU result to rescntrlreq_valid_received\n");
                write_comment_field(" ResourceControlRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvResourceControlRequest(psOriginData->uOriginData.psResourceControlReq, ResultOutTable);
                break;
        default:
                break;
        }
        TBL_PutRecord(OutTable);
        TBL_Destruct(ResultOutTable);
}

int DbvInitializeResponse(      DbvAssocId lAssocId,
                                DbvInitializeRsp *psInitializeRsp,
                                DbvError **ppsDbvError)
{

}

int DbvSearchResponse(          DbvAssocId lAssocId,
                                DbvSearchRsp *psSearchRsp,
                                DbvError **ppsDbvError)
{

}
 
int DbvPresentResponse(         DbvAssocId lAssocId,
                                DbvPresentRsp *psPresentRsp,
                                DbvError **ppsDbvError)
{

}
 
int DbvDeleteResultSetResponse( DbvAssocId lAssocId,
                                DbvDeleteResultSetRsp *psDeleteResultSetRsp,
                                DbvError **ppsDbvError)
{

}
 
int DbvScanResponse(            DbvAssocId lAssocId,
                                DbvScanRsp *psScanRsp,
                                DbvError **ppsDbvError)
{

}
 
int DbvResourceReportResponse(  DbvAssocId lAssocId,
                                DbvResourceReportRsp *psResourceReportRsp,
                                DbvError **ppsDbvError)
{

}

int DbvSortResponse(            DbvAssocId lAssocId,
                                DbvSortRsp *psSortRsp,
                                DbvError **ppsDbvError)
{

}
 
int DbvExtendedServicesResponse(DbvAssocId lAssocId,
                                DbvExtendedServicesRsp *psExtendedServicesRsp,
                                DbvError **ppsDbvError)
{

}

