/*  Module Name: @(#)t_del.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments from target DeleteResultSet
        Req/Resp specific tbl files. For use with the testtool.

 
    Modification History.

    23-05-95	D.B.Maskell	Continued implementation.
*/
 
#include "t_del.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvDeleteResultSetResponse()					!*/
/*+---------------------------------------------------------------------+*/
int do_DbvDeleteResultSetResponse(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
 
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
    char *stringfield;
   char *psDeleteOperationStatus;
 
   DbvAssocId   	   lAssocId;
   DbvDeleteResultSetRsp   *psDeleteResultSetRsp;
   DbvDeleteSetStatus	   eDeleteOperationStatus;
   DbvListStatus	   **ppsDeleteListStatuses;
   DbvListStatus	   **ppsBulkStatuses;

   DbvError         *psSIError;
 

   return_value = open_tables("delresp", eTestType, 
			InFilename, OutFilename, &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of DbvDeleteResultSetResponse\n", 
				OutTable, T_Return);
   TBL_GetRecord( InTable );
   do {
   psDeleteResultSetRsp = (DbvDeleteResultSetRsp *)DbvAlloc(
						sizeof(DbvDeleteResultSetRsp) );
      /* Assoc Id */
      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* ReferenceId */
      TBL_GetRecord( InTable );
      intfield = get_int_field( InTable, T_Return );
      get_DbvOctetString( &psDeleteResultSetRsp->psReferenceId, InTable );

      /* Delete Operation Status */

      TBL_GetRecord( InTable );
      /* ignore title */
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );

      psDeleteOperationStatus = get_string_field( InTable, T_Return );
      convert_DbvDeleteSetStatus( &psDeleteOperationStatus, 
					&eDeleteOperationStatus );
      psDeleteResultSetRsp->eDeleteOperationStatus = eDeleteOperationStatus;

      /* Get the number of Delete List Statuses */

      TBL_GetRecord( InTable );
      /* ignore title */
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      intfield = get_int_field( InTable,T_Return );

      /* DeleteListStatuses list  */

      ppsDeleteListStatuses = ( DbvListStatus **)DbvAlloc( 
			( sizeof( DbvListStatus* ) * (intfield + 1) ));
      psDeleteResultSetRsp->ppsDeleteListStatuses = ppsDeleteListStatuses;
      for ( ; intfield; intfield-- )
	 {

         get_DbvListStatus( ppsDeleteListStatuses, InTable );
	 *ppsDeleteListStatuses++;
	 } /* End for */
 
      *ppsDeleteListStatuses = NULL;

      /* Number Not Deleted */
      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );
      psDeleteResultSetRsp->pulNumberNotDeleted =  
		(Ulong *)get_integer_string( InTable );
 
      /* Get the number of Bulk Statuses */
      TBL_GetRecord(InTable);

      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree(stringfield);

      intfield = get_int_field(InTable,T_Return);

      /* BulkStatuses list  */

      ppsBulkStatuses = ( DbvListStatus **)DbvAlloc( 
			( sizeof( DbvListStatus* ) * (intfield + 1) ));
      psDeleteResultSetRsp->ppsBulkStatuses = ppsBulkStatuses;


      for ( ; intfield; intfield-- )
	 {
 
         get_DbvListStatus( ppsBulkStatuses, InTable );
	 ppsBulkStatuses++;
	 } /* End for */
 
      *ppsBulkStatuses = NULL;

      /* Delete Message */

      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );
      get_DbvInternationalString( &psDeleteResultSetRsp->psDeleteMessage, 
				  InTable );
	 
      /* Other Information */
      getN_DbvOtherInformation( &psDeleteResultSetRsp->ppsOtherInformation,
					InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
        {
                write_DbvDeleteResultSetResponse(psDeleteResultSetRsp, 
			LocalTable);
                TBL_Destruct(LocalTable);
        }
        bLocalResult = DBV_FALSE;

      printf("Press enter to execute DeleteResultSetRsp\n");
      fflush( stdin );
      get_return();
 
      return_value = DbvDeleteResultSetResponse(lAssocId, psDeleteResultSetRsp, &psSIError);
 
      if(return_value == SI_OK)
         {
         printf("DeleteResultSetResponse Ok\n");
         printf("Writing output to %s\n", OutFilename);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("NULL", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         } /* End if */
      else
         {
         printf("Call failed\n");
         write_DbvError(&psSIError, OutTable);
         }
      Free_DbvDeleteResultSetRsp(psDeleteResultSetRsp);
      TBL_GetRecord(InTable);
      out_record = 1;
      } while(TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );
 
   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_DeleteRsp( psDeleteResultSetRsp );	*/

   return return_value;
   } /* End do_DbvDeleteResultSetResponse() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvDeleteResultSetRequest()					!*/
/*+---------------------------------------------------------------------+*/
void write_DbvDeleteResultSetRequest( 
	DbvDeleteResultSetReq *psDeleteResultSetReq,
        TBL OutTable)
   {
   char temp[STRING_LEN];
   char *pc_string = NULL;
   DbvInternationalString **ppsResultSetId;
 
   if( bLocalResult == DBV_TRUE)
          write_comment_field("Completed structure DbvDeleteResultSetRequest",
                  OutTable, T_Return);
   else
   {
   	write_comment_field("DbvDeleteResultSetRequest received from Origin", 
			OutTable, T_Return);
   	write_comment_field("Structure detailed - DbvTargetData.uTargetEvent->psDeleteResultSetReq:", OutTable, T_Return);
   }

   /* ReferenceId is an optional field */

   write_DbvOctetString("psReferenceId", psDeleteResultSetReq->psReferenceId,
			OutTable );

   /* Delete Function */
   convert_DbvDeleteFunction( &pc_string, 
			      &psDeleteResultSetReq->eDeleteFunction );

   sprintf( temp, "DbvDeleteFunction : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* ResultSetId : write all Result set ids to out table */

   write_comment_field( "START of ppsResultSetId", OutTable, T_Return );

   ppsResultSetId = psDeleteResultSetReq->ppsResultSetId;

   while( ppsResultSetId && *ppsResultSetId )
      {
      write_DbvInternationalString( "ResultSetId", *ppsResultSetId, OutTable );
      ppsResultSetId++;
      } /* End while */

   write_comment_field( "END of psResultSetId", OutTable, T_Return );

   /* Other Information */
   writeN_DbvOtherInformation(psDeleteResultSetReq->ppsOtherInformation, 
			     OutTable );
   TBL_PutRecord(OutTable);

   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord(OutTable);
   } /* End write_DbvDeleteResultSetRequest() */
