/*  Module Name: @(#)t_init.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments from target Initialize 
        Req/Resp specific tbl files. For use with the testtool.

 
*/
 
#include "t_init.h"

extern TBL_Return T_Return;
extern char string[];

int do_DbvInitializeResponse(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
	char pcResult[STRING_LEN];
        TBL InTable, OutTable, LocalTable;
        int place = 0;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
        char *version;
        char *options;
 
        DbvAssocId         lAssocId;
        DbvSecs            lSeconds;
        DbvInitializeRsp   *psInitializeRsp;
        DbvError         *psSIError;
 
        return_value = open_tables("initresp", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvInitializeResponse\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        psInitializeRsp = (DbvInitializeRsp *)DbvAlloc(sizeof(DbvInitializeRsp));
/* Assoc Id */
        intfield=get_int_field(InTable,T_Return);
        lAssocId=intfield=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
/* ReferenceId */
	get_DbvOctetString(&(psInitializeRsp->psReferenceId), InTable);
/* ProtocolVersion */
        if((strcmp(version=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
                psInitializeRsp->pcProtocolVersion = version;
        }
/* Options */
        if((strcmp(options=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
                psInitializeRsp->pcOptions = options;
        }
/* Pref mes size */
        psInitializeRsp->ulPreferredMessageSize = get_int_field(InTable, T_Return);
/* Max mes size */
        psInitializeRsp->ulMaximumMessageSize = get_int_field(InTable, T_Return);
/* Result (TRUE or FALSE) */
        psInitializeRsp->fResult=get_DbvBool(InTable);
 
/* ImplementationId */
	get_DbvInternationalString(&(psInitializeRsp->psImplementationId), InTable);
/* ImplementationName */
	get_DbvInternationalString(&(psInitializeRsp->psImplementationName), InTable);
/* ImplementationVersion */
	get_DbvInternationalString(&(psInitializeRsp->psImplementationVersion), InTable);
/* UserInformationField External */
        get_DbvASN1External(&(psInitializeRsp->psUserInformationField),
		 InTable);
       
/* OtherInformation External */
        getN_DbvOtherInformation(&(psInitializeRsp->ppsOtherInformation),
		InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
	{
                write_DbvInitializeResponse(psInitializeRsp, LocalTable);
		TBL_Destruct(LocalTable);
	}
        bLocalResult = DBV_FALSE;

        printf("Press enter to execute InitializeRsp\n");
        fflush(stdin);
        get_return();
 
        return_value = DbvInitializeResponse(lAssocId, psInitializeRsp, &psSIError);
 
        if(return_value == SI_OK)
        {
                printf("InitializeResponse Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL", OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psSIError, OutTable);
        }
	FreeDbvInitializeRsp(psInitializeRsp);
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);

        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

void write_DbvInitializeRequest(DbvInitializeReq *psInitializeReq,
        TBL OutTable)
{
        char temp[STRING_LEN];
 
	if( bLocalResult == DBV_TRUE)
		write_comment_field("Completed structure DbvInitializeRequest",
			OutTable, T_Return);
	else
	{
        	write_comment_field("DbvInitializeRequest received from Origin", OutTable, T_Return);
        	write_comment_field("Structure detailed - DbvTargetData.uTargetEvent->psInitializeReq:", OutTable, T_Return);
	}

/* ReferenceId is an optional field */
        write_DbvOctetString("psReferenceId", psInitializeReq->psReferenceId,OutTable);
        sprintf(temp, "pcProtocolVersion: >%s<",psInitializeReq->pcProtocolVersion);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        sprintf(temp, "pcOptions: >%s<", psInitializeReq->pcOptions);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        sprintf(temp, "ulPreferredMessageSize: %ld", psInitializeReq->
                ulPreferredMessageSize);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        sprintf(temp, "ulMaxmiumMessageSize: %ld", psInitializeReq->
                ulMaximumMessageSize);
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord(OutTable);

/* AuthenticationId */
        write_DbvAuthentication(psInitializeReq->psAuthentication, OutTable);
 
/* Implementation Id */
        write_DbvInternationalString("psImplementationId", psInitializeReq->psImplementationId, OutTable);
 
/* Implementation Name */
        write_DbvInternationalString("psImplementationName", psInitializeReq->psImplementationName, OutTable);
 
/* Implementation Version */
        write_DbvInternationalString("psImplementationVersion", psInitializeReq->psImplementationVersion, OutTable);
 
/* User Information field */
	write_comment_field("START of psUserInformationField", OutTable, T_Return);
	write_DbvASN1External(psInitializeReq->psUserInformationField, OutTable);
	write_comment_field("END of psUserInformationField", OutTable, T_Return);

/* Other Information */
	writeN_DbvOtherInformation(psInitializeReq->ppsOtherInformation, 
		OutTable);
        write_comment_field(" End ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
}


