/*  Module Name: @(#)t_scan.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments from target Scan
        Req/Resp specific tbl files. For use with the testtool.

    Modification History.

    31-05-95	D.B.Maskell	Continued implementation.
*/
 
#include "t_scan.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvScanResponse()						!*/
/*+---------------------------------------------------------------------+*/
int do_DbvScanResponse(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
   char pcResult[STRING_LEN];
   TBL InTable, OutTable, LocalTable;
 
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   Ulong  *pUlong;
   char *stringfield;
   char *pcScanStatus;
 
   DbvAssocId      lAssocId;
   DbvScanRsp 	  *psScanRsp;
   DbvScanStatus   eScanStatus;

   DbvError         *psSIError;
 

   return_value = open_tables("scanresp", eTestType, 
			InFilename, OutFilename, &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of DbvScanResponse\n", 
				OutTable, T_Return);
   TBL_GetRecord( InTable );
   do {
      psScanRsp = (DbvScanRsp *)DbvAlloc( sizeof(DbvScanRsp) );
      /* Assoc Id */
      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* ReferenceId */
      TBL_GetRecord( InTable );
      intfield = get_int_field( InTable, T_Return );
      get_DbvOctetString( &psScanRsp->psReferenceId, InTable );

      /* Step size */

      TBL_GetRecord( InTable );

      /* Ignore title */
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );
      
      psScanRsp->pulStepSize = (Ulong*)get_integer_string( InTable );

      /* Scan Status */

      TBL_GetRecord( InTable );

      /* Ignore title */
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );

      pcScanStatus = get_string_field( InTable, T_Return );
      convert_DbvScanStatus( &pcScanStatus, &eScanStatus );
      psScanRsp->eScanStatus = eScanStatus;
      DbvFree( pcScanStatus );

      /* Number Of Entries Returned */

      TBL_GetRecord( InTable );

      /* Ignore title */
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      psScanRsp->ulNumberOfEntriesReturned = get_int_field( InTable,T_Return );

      /* Position Of Term */

      TBL_GetRecord( InTable );

      /* Ignore title */
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      psScanRsp->pulPositionOfTerm = (Ulong*)get_integer_string( InTable );

      /* Entries */

         get_DbvListEntries( &psScanRsp->psEntries, InTable );

      /* Attribute Set Id */

      TBL_GetRecord( InTable );
         get_DbvObjectIdentifier( &psScanRsp->pcAttributeSetId, InTable );
/*
printf("doing malloc_verify()\n");
	printf("1 OK, 0 - BAAAAD!\n");
	printf("Returned %d\n",malloc_verify());
printf("DONE malloc_verify()\n");

      /* Other Information */

      getN_DbvOtherInformation( &psScanRsp->ppsOtherInformation, InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
        {
                write_DbvScanResponse(psScanRsp, LocalTable);
                TBL_Destruct(LocalTable);
        }
        bLocalResult = DBV_FALSE;

      printf("Press enter to execute ScanRsp\n");
      fflush( stdin );
      get_return();
 
      return_value = DbvScanResponse(lAssocId, psScanRsp, &psSIError );
 
      if( return_value == SI_OK )
         {
         printf( "ScanResponse Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psSIError, OutTable );
         }
      Free_DbvScanRsp(psScanRsp);
      TBL_GetRecord( InTable );
      out_record = 1;
      } while(TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );
 
   return return_value;
   } /* End do_DbvScanResponse() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvScanRequest()						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvScanRequest( DbvScanReq *psScanReq, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   DbvInternationalString **ppsDatabaseNames;
 
   if( bLocalResult == DBV_TRUE)
        write_comment_field("Completed structure DbvScanRequest",
               OutTable, T_Return);
   else
   {
   	write_comment_field("DbvScanRequest received from Origin", 
			OutTable, T_Return);
   	write_comment_field("Structure detailed - DbvTargetData.uTargetEvent->psScanReq:", OutTable, T_Return);
   }

   /* ReferenceId is an optional field */

   write_DbvOctetString( "psReferenceId", psScanReq->psReferenceId,
			OutTable );
   /* Database Names */

   write_comment_field( "START of ppsDatabaseNames", OutTable, T_Return );

   ppsDatabaseNames = psScanReq->ppsDatabaseNames;

   while( ppsDatabaseNames && *ppsDatabaseNames )
      {
      write_DbvInternationalString( "DatabaseNames", *ppsDatabaseNames, 
							OutTable );
      ppsDatabaseNames++;
      } /* End while */

   write_comment_field( "END of psDatabaseNames", OutTable, T_Return );

   /* AttributeSetId */

   write_DbvObjectIdentifier( "pcAttributeSetId:",
		 	      psScanReq->pcAttributeSetId,
			      OutTable );
								 
   /* AttributesPlusTerm */

   write_DbvAttributesPlusTerm( &psScanReq->sAttributesPlusTerm, OutTable );

   /* StepSize */ 

   write_integer_string( "pulStepSize", (int *)psScanReq->pulStepSize, OutTable );
   TBL_PutRecord( OutTable );

   /* NumberOfTermsRequested */ 
   sprintf( temp, "ulNumberOfEntriesReturned: %ld", 
				psScanReq->ulNumberOfTermsRequested);
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
			  
   /* PreferredPositionInResponse */

   write_integer_string( "pulPreferredPositionInResponse", 
			  (int *)psScanReq->pulPreferredPositionInResponse, 
			  OutTable );
   TBL_PutRecord( OutTable );
   
   /* Other Information */

   writeN_DbvOtherInformation(psScanReq->ppsOtherInformation, OutTable );

   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord(OutTable);
   } /* End write_DbvScanRequest() */
