/*  Module Name: @(#)t_sort.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments from target Sort
        Req/Resp specific tbl files. For use with the testtool.

 
    Modification History.

    27-05-95	D.B.Maskell	Continued implementation.
*/
 
#include "t_sort.h"

extern TBL_Return T_Return;
extern char string[];
 
/*+---------------------------------------------------------------------+*/
/*! do_DbvSortResponse() 						!*/
/*+---------------------------------------------------------------------+*/
int do_DbvSortResponse(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
	char pcResult[STRING_LEN];
	TBL InTable, OutTable, LocalTable;
 
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   char *stringfield;
   char *psSortStatus;
 
   DbvAssocId 		  lAssocId;
   DbvSortRsp 		  *psSortRsp;
   DbvSortStatus	  eSortStatus;
   DbvSortResultSetStatus *peResultSetStatus;
   DbvDiagRec		  **ppsDiagnostics;

   DbvError         *psSIError;
 

   return_value = open_tables("sortresp", eTestType, 
			InFilename, OutFilename, &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of DbvSortResponse\n", 
				OutTable, T_Return);
   TBL_GetRecord( InTable );
   do {
      psSortRsp = (DbvSortRsp *)DbvAlloc( sizeof(DbvSortRsp) );
      /* Assoc Id */
      intfield = get_int_field( InTable,T_Return );
      lAssocId = get_int_field( InTable,T_Return );

      /* ReferenceId */
      TBL_GetRecord( InTable );
      intfield = get_int_field( InTable, T_Return );
      get_DbvOctetString( &psSortRsp->psReferenceId, InTable );

      /* Sort Status */

      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );

      psSortStatus = get_string_field( InTable, T_Return );
      convert_DbvSortStatus( &psSortStatus, &eSortStatus );
      psSortRsp->eSortStatus = eSortStatus;

      /* Result Set Status */

      TBL_GetRecord( InTable );
      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      stringfield = get_string_field( InTable, T_Return );
      if((strcmp(stringfield, NOTHING)) != 0 )
         {
         peResultSetStatus = (DbvSortResultSetStatus*)DbvAlloc(
					sizeof( DbvSortResultSetStatus ));
         convert_DbvSortResultSetStatus( &stringfield, peResultSetStatus );
         psSortRsp->peResultSetStatus = peResultSetStatus;
         } /* End if */

      DbvFree( stringfield );
      /* Get the number of Diag Records */

      TBL_GetRecord( InTable );

      stringfield = get_char_field( InTable, T_Return ); 
      DbvFree( stringfield );

      intfield = get_int_field( InTable, T_Return );

       /* Diag list  */

       if ( intfield )
          {
          ppsDiagnostics = ( DbvDiagRec **)DbvAlloc( 
 			( sizeof( DbvDiagRec* ) * intfield + 1 ));
          psSortRsp->ppsDiagnostics = ppsDiagnostics;

          for ( ; intfield; intfield-- )
             {
             get_DbvDiagRec( ppsDiagnostics, InTable );
	     ppsDiagnostics++;
	     } /* End for */
 
          *ppsDiagnostics = NULL;

	  } /* End if */

      /* Other Information */

      getN_DbvOtherInformation( &psSortRsp->ppsOtherInformation, InTable);

      bLocalResult = DBV_TRUE;
      if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
      {
              write_DbvSortResponse(psSortRsp, LocalTable);
              TBL_Destruct(LocalTable);
      }
      bLocalResult = DBV_FALSE;

      printf( "Press enter to execute SortRsp\n" );
      fflush( stdin );
      get_return();
 
      return_value = DbvSortResponse(lAssocId, psSortRsp, &psSIError);
 
      if(return_value == SI_OK)
         {
         printf("SortResponse Ok\n");
         printf("Writing output to %s\n", OutFilename);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("IN_PARAM", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         write_int_field(out_record++, OutTable, T_Return);
         write_string_field("NULL", OutTable, T_Return);
         TBL_PutRecord(OutTable);
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError(&psSIError, OutTable);
         }
      Free_DbvSortRsp(psSortRsp);
      TBL_GetRecord(InTable);
      out_record = 1;
      } while( TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );

   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_SortRsp( psSortRsp );	*/

   return return_value;
   } /* End do_DbvSortResponse() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvSortRequest()						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvSortRequest( DbvSortReq *psSortReq, TBL OutTable)
   {
   DbvInternationalString **ppsInputResultSetName;
   DbvSortKeySpec	  **ppsSortSequence;

   if( bLocalResult == DBV_TRUE)
           write_comment_field("Completed structure DbvSortRequest",
                   OutTable, T_Return);
   else
   {
   	write_comment_field("DbvSortRequest received from Origin", 
			OutTable, T_Return);
   	write_comment_field("Structure detailed - DbvTargetData.uTargetEvent->psSortReq:", OutTable, T_Return);
   }

   /* ReferenceId is an optional field */

   write_DbvOctetString("psReferenceId", psSortReq->psReferenceId,
			OutTable );

   /* InputResultSetName : write all Result set ids to out table */

   write_comment_field( "START of ppsInputResultSetName", OutTable, T_Return );

   ppsInputResultSetName = psSortReq->ppsInputResultSetName;

   while( ppsInputResultSetName && *ppsInputResultSetName )
      {
      write_DbvInternationalString( "InputResultSetName", 
			*ppsInputResultSetName, OutTable );
      TBL_PutRecord(OutTable);
      ppsInputResultSetName++;
      } /* End while */

   write_comment_field( "END of psInputResultSetName", OutTable, T_Return );

   /* Sorted Result Set Name */

   write_DbvInternationalString( "sSortedResultSetName", 
			&psSortReq->sSortedResultSetName, OutTable );
   TBL_PutRecord( OutTable );

   /* Sort Sequence : write all Result set ids to out table */

   write_comment_field( "START of ppsSortSequence", OutTable, T_Return );

   ppsSortSequence = psSortReq->ppsSortSequence;

   while( ppsSortSequence && *ppsSortSequence )
      {
      write_DbvSortKeySpec( *ppsSortSequence, OutTable );
      ppsSortSequence++;
      } /* End while */

   write_comment_field( "END of psSortSequence", OutTable, T_Return );

   /* Other Information */
   writeN_DbvOtherInformation(psSortReq->ppsOtherInformation, 
			     OutTable );
   write_comment_field(" End ", OutTable, T_Return);
   TBL_PutRecord(OutTable);
   } /* End write_DbvSortRequest() */
