/*  Module Name: @(#)t_srch.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments from target Search
        Req/Resp specific tbl files. For use with the testtool.

 
    Modification History.

    24-05-95	D.B.Maskell	Continued implementation.
*/
 
#include "t_srch.h"

extern TBL_Return T_Return;
extern char string[];

/*+---------------------------------------------------------------------+*/
/*! do_DbvSearchResponse()						!*/
/*+---------------------------------------------------------------------+*/
int do_DbvSearchResponse(TestType eTestType)
   {
   char InFilename[ SZ_FILENAME ];
   char OutFilename[ SZ_FILENAME ];
	char pcResult[STRING_LEN];
	TBL InTable, OutTable, LocalTable;
 
   int intfield;
   int out_record = 1;
   int return_value = SI_OK;
   Ulong  *pUlong;
   char *stringfield;
 
   DbvAssocId   	   lAssocId;
   DbvSearchRsp   	   *psSearchRsp;
   DbvResultSetStatus	   *peResultSetStatus;
   DbvPresentStatus	   *pePresentStatus;

   DbvError         *psSIError;
 

   return_value = open_tables("srchresp", eTestType, 
			InFilename, OutFilename, &InTable, &OutTable);
   if(return_value == SI_NOTOK)
      return return_value;

   write_comment_field("\nResults of DbvSearchResponse\n", OutTable, T_Return );

   TBL_GetRecord( InTable );
   do {
      psSearchRsp = (DbvSearchRsp *)DbvAlloc( sizeof(DbvSearchRsp) );

      /* Assoc Id */

      intfield = get_int_field( InTable, T_Return );
      lAssocId = get_int_field( InTable, T_Return );

      /* ReferenceId */

      TBL_GetRecord( InTable );
      intfield = get_int_field( InTable, T_Return );
      get_DbvOctetString( &psSearchRsp->psReferenceId, InTable );

      /* Result Count */

      TBL_GetRecord(InTable);
      psSearchRsp->ulResultCount = get_int_field( InTable, T_Return );

      /* Number of records returned */


      psSearchRsp->ulNumberOfRecordsReturned = get_int_field(InTable,T_Return);

      /* Next Result Set Position */

 
      psSearchRsp->ulNextResultSetPosition = get_int_field( InTable, T_Return );

      /* Search Status */

      TBL_GetRecord(InTable);
      psSearchRsp->fSearchStatus = get_DbvBool( InTable );

      /* Result Set Status */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );

      stringfield = get_string_field( InTable, T_Return );
      if( ( strcmp( stringfield, NOTHING ) ) != 0 )
	 {
         peResultSetStatus = (DbvResultSetStatus*)DbvAlloc(
						sizeof( DbvResultSetStatus ));
         convert_DbvResultSetStatus( &stringfield, peResultSetStatus );
         psSearchRsp->peResultSetStatus = peResultSetStatus;
	 } /* End if */

      DbvFree( stringfield );

      /* Present Status */

      TBL_GetRecord(InTable);
      stringfield = get_char_field(InTable, T_Return); 
      DbvFree( stringfield );

      stringfield = get_string_field( InTable, T_Return );
      if( ( strcmp( stringfield, NOTHING ) ) != 0 )
	 {
         pePresentStatus = (DbvPresentStatus*)DbvAlloc(
						sizeof( DbvPresentStatus ));
         convert_DbvPresentStatus( &stringfield, pePresentStatus );
         psSearchRsp->pePresentStatus = pePresentStatus;
	 } /* End if */
      DbvFree( stringfield );

      /* Records */

      TBL_GetRecord( InTable );

      /* Ignore the title */
      stringfield = get_char_field( InTable, T_Return );
      DbvFree( stringfield );
      stringfield = get_string_field( InTable, T_Return );
      if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      {
      DbvFree( stringfield );
      get_DbvRecords( &psSearchRsp->psRecords, InTable );
      } /* End if */
      else
        DbvFree( stringfield );
 
      /* Additional Search Info */
      getN_DbvOtherInformation(&(psSearchRsp->ppsAdditionalSearchInfo ),
		InTable);
      /* Other Information */

      getN_DbvOtherInformation( &psSearchRsp->ppsOtherInformation,InTable);

        bLocalResult = DBV_TRUE;
        if(open_result_table("o", &LocalTable, pcResult)==SI_OK)
        {
                write_DbvSearchResponse(psSearchRsp, LocalTable);
                TBL_Destruct(LocalTable);
        }
        bLocalResult = DBV_FALSE;

     	printf( "Press enter to execute SearchRsp\n" );
      	fflush( stdin );
      	get_return();
 
      return_value = DbvSearchResponse( lAssocId, psSearchRsp, &psSIError );
 
      if( return_value == SI_OK )
         {
         printf( "SearchResponse Ok\n" );
         printf( "Writing output to %s\n", OutFilename );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "IN_PARAM", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         write_int_field( out_record++, OutTable, T_Return );
         write_string_field( "NULL", OutTable, T_Return );
         TBL_PutRecord( OutTable );
         } /* End if */
      else
         {
         printf( "Call failed\n" );
         write_DbvError( &psSIError, OutTable );
         }
      Free_DbvSearchRsp(psSearchRsp);
      TBL_GetRecord( InTable );
      out_record = 1;
      }
   while(TBL_GetRecord( InTable ) != FALSE );

   TBL_Destruct( InTable );
   TBL_Destruct( OutTable );
 
   /* Free : Special Function required : Must free all attached space */

/*   DbvFree_SearchRsp( psSearchRsp );	*/

   return return_value;
   } /* End do_DbvSearchResponse() */

/*+---------------------------------------------------------------------+*/
/*! write_DbvSearchRequest()						!*/
/*+---------------------------------------------------------------------+*/
void write_DbvSearchRequest( DbvSearchReq *psSearchReq, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   DbvInternationalString **ppsDatabaseNames;

 
   if( bLocalResult == DBV_TRUE)
           write_comment_field("Completed structure DbvSearchRequest",
                   OutTable, T_Return);
   else
   {
   	write_comment_field( "DbvSearchRequest received from Origin", 
			OutTable, T_Return );
   	write_comment_field( "Structure detailed - DbvTargetData.uTargetEvent->psSearchReq:", OutTable, T_Return );
   }

   /* ReferenceId is an optional field */

   write_DbvOctetString( "psReferenceId", psSearchReq->psReferenceId,
			OutTable );

   /* Small Set Upper Bound */

   sprintf(temp, "ulSmallSetUpperBound: %ld", 
			psSearchReq->ulSmallSetUpperBound );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* Large Set Lower Bound */

   sprintf(temp, "ulLargeSetLowerBound: %ld", 
			psSearchReq->ulLargeSetLowerBound );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Medium Set Present Number */

   sprintf(temp, "ulMediumSetPresentNumber : %ld", 
			psSearchReq->ulMediumSetPresentNumber );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Replace Indicator */

   convert_DbvBool( &pc_string, &psSearchReq->fReplaceIndicator );
   sprintf( temp, "fReplaceIndicator: %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Result Set Name */

   write_DbvInternationalString( "sResultSetName:", 
				&psSearchReq->sResultSetName, 
				OutTable );
   TBL_PutRecord(OutTable);
   /* Database Names */

   write_comment_field( "START of ppsDatabaseNames", OutTable, T_Return );

   ppsDatabaseNames = psSearchReq->ppsDatabaseNames;

   while( ppsDatabaseNames && *ppsDatabaseNames )
      {
      write_DbvInternationalString( "psDatabaseNames", *ppsDatabaseNames,
				    OutTable );
      TBL_PutRecord(OutTable);
      ppsDatabaseNames++;
      } /* End while */

   write_comment_field( "END of ppsDatabaseNames", OutTable, T_Return );

   /* Small Set Element Names */
   if(psSearchReq->psSmallSetElementNames==NULL)
   {
     write_char_field("psSearchReq->psSmallSetElementNames: NULL", OutTable, T_Return);
     TBL_PutRecord(OutTable);
   }
   else
   {
   write_comment_field( "START of psSmallSetElementNames", OutTable, T_Return );
   write_DbvElementSetNames( psSearchReq->psSmallSetElementNames, OutTable );
   write_comment_field( "END of psSmallSetElementNames", OutTable, T_Return );
   }
   /* Medium Set Element Names */
   if(psSearchReq->psMediumSetElementNames==NULL)
   {
     write_char_field("psSearchReq->psMediumSetElementNames: NULL", OutTable, T_Return);
     TBL_PutRecord(OutTable);
   }
   else
   {
   write_comment_field( "START of psMediumSetElementNames", OutTable, T_Return);
   write_DbvElementSetNames( psSearchReq->psMediumSetElementNames, OutTable );
   write_comment_field( "END of psMediumSetElementNames", OutTable, T_Return );
   }
   /* Preferred Record Syntax */

   write_DbvObjectIdentifier( "pcPreferredRecordSyntax:",
			      psSearchReq->pcPreferredRecordSyntax,
			      OutTable );

   /* Query */

   write_comment_field( "START of psQuery", OutTable, T_Return );
   write_DbvQuery( psSearchReq->psQuery, OutTable );
   write_comment_field( "END of psQuery", OutTable, T_Return );

   /* Additional Information */
   write_comment_field( "START of psAdditionalSearchInfo", OutTable, T_Return );
   writeN_DbvOtherInformation( psSearchReq->ppsAdditionalSearchInfo, OutTable );
   write_comment_field( "END of psAdditionalSearchInfo", OutTable, T_Return );

   /* Other Information */
   writeN_DbvOtherInformation(psSearchReq->ppsOtherInformation, OutTable );

   write_comment_field(" End of Search Request ", OutTable, T_Return);
   TBL_PutRecord(OutTable);
   } /* End write_DbvSearchRequest() */
