/*  Module Name: @(#)target.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        DBV-OSI II target. For use with the testtool
 
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
 
#include "sl_list.h" 
#include "memcheck.h"

#include "tbl.h"
#include "str_hand.h"

#include "testy_calls.h"
#include "target_funcs.h"
#include "t_init.h"
#include "t_srch.h"
#include "t_pres.h"
#include "t_del.h"
#include "t_scan.h"
#include "t_res.h"
#include "t_seg.h"
#include "t_sort.h"
#include "t_ext.h"
#include "t_close.h"
#include "dbvproto.h" 
#include <errno.h>

extern int errno;
extern char *sys_errlist[];

extern char pcFileExtension[];
extern char pcTBLDirectory[];
extern char string[];
extern DbvBool bPrintInput;
extern int eOriginOrTarget;
extern TBL_Return T_Return;

int Gargc;
char **Gargv;
int PDU_Received = 0;
pid_t ProcessId;

int test_association_handling(void);
int test_PDU_receive(void);
int test_non_comms(void);
 
void tprintf(char *data);

void interactive_target(void);
void process_target_fields(DbvTargetData *psTargetData, DbvAssocId assoc_id);

/* Functions to test API functions */
int do_DbvReceiveAssociateRequest(TestType eTestType);
int do_DbvAssociateResponse(TestType eTestType);
int do_DbvReleaseResponse(TestType eTestType);
int do_DbvTargetInitialize(TestType eTestType);

/* State machine off/on variable (DBV_FALSE | DBV_TRUE) */
extern int DBVStateMachine;

main(int argc, char **argv)
{
	int choice = 0;
	int file_number = 1;
	int length = 0;
	char Filename[128];
	char pcNewDirectory[256];
	char choice_string[50];
	FILE *psTraceFp;
	DbvTraceLevel eTraceLevel = Trace_High;
	int return_value = SI_OK;
	DbvError *psError=NULL;
	
DBVStateMachine = DBV_TRUE;

#ifdef MALLOC_DEBUG
        malloc_debug(2);
#endif

	/* Get out process id */
	ProcessId=getpid();

        /* Set the global variable so that functions load correct tbl files */
        eOriginOrTarget = TARGET;
        /* This tells us if we're writing the results before they've been sent
                or after they've been received. */
        bLocalResult = DBV_FALSE;
	strcpy(pcTBLDirectory, TBLDIR);
	
	printf("Debug? (y/n)");
       	fflush(stdout);
       	while(fgets(choice_string, 50, stdin)==NULL)
	{
		perror("Error reading stdin");
		clearerr(stdin);
		sleep(2);
	}
	if(*choice_string=='y' || *choice_string=='Y')
	{
		setup_gdb(argv[0]);
	}

	strcpy(pcNewDirectory, argv[0]);
	length = strlen(pcNewDirectory);
	length --;
	while(pcNewDirectory[length]!='/')
		{ length --; }
	pcNewDirectory[length] = '\0';

        strcpy(pcFileExtension, "");
	if(chdir(pcNewDirectory)==-1)
	{
		printf("Directory %s does not exist\n", pcNewDirectory);
		sleep(10);
		exit(-1);
	}
	else
		printf("Moving to directory %s\n", pcNewDirectory);

	if(argc == 1)
	{
		printf("No arguments!\n");
		exit(-1);
	}

	T_Return = SMALLOC_(struct tbl_return , "Target test tool");
	
	/* Remove old target results */
	remove_old_results(pcTBLDirectory);
	bPrintInput = DBV_TRUE;
	printf("bPrintInput is currently set to TRUE\n");

	printf("\n\n\n");
        printf("DBV-OSI II Target Test Tool (c) CSL\n");
        printf("-----------------------------------\n");
        printf("\n");

	printf("argc = %d\n", argc);
	for(choice = 0; choice < argc; choice ++)
		printf("argv[%d] = %s\n",choice, argv[choice]);
	choice =0;
	Gargc = argc;
	Gargv = argv;

	printf("\n\n");
	/* Configure the target */
	config_TargetConfigure();

	/* Do the batch processing (if any) */
	batch_DoTargetBatchProcessing();

/*
	printf("\nThe file extension for result files corresponding to this test run is: %d\n",ProcessId);
	sprintf(pcFileExtension, ".%d", ProcessId);

	while(choice<1 || choice>3)
	{
		printf("\n1) Non-interactive Target\n");
		printf("2) Interactive Target\n");
		printf("3) Exit \n");
        	printf("Choice (1/2/3) :");
        	fflush(stdout);
        	gets(choice_string);
        	choice = atoi(choice_string);
	}

	switch(choice)
	{
	case 1:
        	DbvTargetInitialize(argc, argv, &psError);
        	silent_target();
		break;
	case 2:
		interactive_target();
		break;
	case 3:
		break;
	default:
		break;
	}
*/
	
	interactive_target();
        DbvFree(T_Return);
}

setup_gdb(char *Filename)
{
 
        char gdb_str[200];
	char pcString[10];
 
        printf("Doing: xterm -e gdb target %d &\n", getpid());
	printf("Press return when ready\n");
 
        sprintf(gdb_str, "%s%s %d%s", "xterm -e gdb ", Filename, getpid(), " &");
        system(gdb_str);
	gets(pcString);
}

int silent_target(void)
{

	DbvAssocId	   lAssocId;
        DbvTargetData      *psTargetData = NULL;
        DbvError           *psError = NULL;
	DbvProtocolAndSyntaxes *psProtocolAndSyntaxes = NULL;

	char 		str[128];
	DbvReleaseRsp	*psReleaseRsp = NULL;
	int 		return_value = SI_OK;
	char *origin_address = NULL;
	int type_of_apdu_to_send = 0;

        if(DbvReceiveAssociateRequest(&lAssocId, CALL_BLOCKING, &origin_address,
                &psProtocolAndSyntaxes, &psError) == SI_NOTOK)
        {
                printf("Error in DbvReceiveAssociateRequest\n");
                /* Look inside the error structure */
                PrivateErrorHandling(&psError);
                free_DbvError(&psError);
                sleep(3);
                exit(-1);
        }
 
        if(DbvAssociateResponse(lAssocId, Accepted, &psError) == SI_NOTOK)
        {
                printf("Error in DbvAssociateResponse\n");
                /* Look inside the error structure */
                PrivateErrorHandling(&psError);
                free_DbvError(&psError);
                sleep(5);
                exit(-1);
        }

	printf("Received connection from %s\n", origin_address);

	while(return_value == SI_OK) 
	{
		printf("Waiting for data to be sent by the client...\n");

                /* Receive a Z39.50 PDU over the communication link from the client */
                if (DbvReceiveDataTarget(lAssocId, CALL_BLOCKING, &psTargetData, 
			&psError) == SI_NOTOK)
                {
                        PrivateErrorHandling(&psError);
                        free_DbvError(&psError);
                        printf("DbvReceiveDataTarget failed - Press return\n");
			fflush(stdout);
			gets(str);
			return_value = SI_NOTOK;
                }
 
		if(return_value == SI_OK)
                	process_target_fields(psTargetData, lAssocId);
	}
	return return_value;
}
 
void interactive_target(void)
{
        char choice_string[100];
        int choice = 0;
        int result;
 
        printf("DBV-OSI II Interactive Target Test Tool (c) CSL\n");
        printf("-----------------------------------------------\n");


GroupToTest:
        choice = 0; result = 0;
        while(choice < 1 | choice > 4)
        {
                printf("\nChoose Target API Function Group To Test\n");
                printf("1) Association Handling\n");
                printf("2) Non Communication Utility Functions\n");
                printf("3) Exit\n");
                printf("Choice (1/2/3) :");
                fflush(stdout);
                gets(choice_string);
                choice = atoi(choice_string);
        }
        switch(choice)
        {
        case 1:
                result = test_association_handling();
                break;
	case 2:
                result = test_non_comms();
                break;
        case 3:
                break;
        default:
                break;
        }
        if(result == SI_PREV)
                goto GroupToTest;
 
Quit:
        printf("Goodbye\n");
}

int test_association_handling(void)
{
        int choice = 0;
        int return_value = SI_OK;
        char choice_string[100];

        while(choice < 1 || choice > 10)
        {
                printf("Choose Target Association Handling Function\n");
                printf("1) Valid DbvReceiveAssociateRequest()\n");
                printf("Input From:\treassocreq_valid\n");
                tprintf("Output To:\treassocreq_valid_result");
 
                printf("2) Invalid DbvReceiveAssociateRequest()\n");
                printf("Input From:\treassocreq_invalid\n");
                tprintf("Output To:\treassocreq_invalid_result");

                printf("3) Valid DbvAssociateResponse()\n");
                printf("Input From:\tassocresp_valid\n");
                tprintf("Output To:\tassocresp_valid_result");
 
                printf("4) Invalid DbvAssociateResponse()\n");
                printf("Input From:\tassocresp_invalid\n");
                tprintf("Output To:\tassocresp_invalid_result");

                printf("5) Valid DbvReleaseResponse()\n");
                printf("Input From:\trelresp_valid\n");
                tprintf("Output To:\trelresp_valid_result");
 
                printf("6) Invalid DbvReleaseResponse()\n");
                printf("Input From:\trelresp_invalid\n");
                tprintf("Output To:\trelresp_invalid_result");

                printf("7) Valid DbvAbortRequest()\n");
                printf("Input From:\tabortreq_valid\n");
                tprintf("Output To:\tabortreq_valid_result");
 
                printf("8) Invalid DbvAbortRequest()\n");
                printf("Input From:\tabortreq_invalid\n");
                tprintf("Output To:\tabortreq_invalid_result");

		printf("9) PDU Reception from Origin (menu)\n");
		printf("10) Previous Menu\n");
                printf("Choice (1/2/3/4/5/6/7/8/9/10) :");
                fflush(stdout);
                gets(choice_string);
                choice = atoi(choice_string);
 
                switch(choice)
                {
                case 1:
			do_DbvReceiveAssociateRequest(Valid);
			choice = -1;
                        break;
                case 2:
			do_DbvReceiveAssociateRequest(Invalid);
			choice = -1;
                        break;
		case 3:
			do_DbvAssociateResponse(Valid);
			choice = -1;
			break;
		case 4:
			do_DbvAssociateResponse(Invalid);
			choice = -1;
			break;
		case 5:
			do_DbvReleaseResponse(Valid);
			choice = -1;
			break;
		case 6:
			do_DbvReleaseResponse(Invalid);
			choice = -1;
			break;
		case 7:
			do_DbvAbortRequest(Valid);
			choice = -1;
			break;
		case 8:
			do_DbvAbortRequest(Invalid);
			choice = -1;
			break;
		case 9:
			return_value = test_PDU_receive();
			if(return_value == SI_PREV)
				choice = -1;
			break;
			
                case 10:
                        return_value = SI_PREV;
                        break;
                default:
                        printf("Not currently Implemented!\n");
                        choice = -1;
                        break;
                }
		if(return_value!=SI_PREV)
		{
                	printf("Press enter\n");
                	fflush(stdin);
                	gets(string);
		}
        }


	return return_value;
}

int test_PDU_receive(void)
{
        int choice = 0;
        int return_value = SI_OK;
        char choice_string[100];

        while(choice < 1 || choice > 37)
        {
                printf("Choose Target SR-/Z39.50  Function\n\n");
                printf("1) Valid DbvInitializeResponse()\n");
                printf("Input From:\tinitresp_valid\n");
                tprintf("Output To:\tinitresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tinitresp_valid_received");

                printf("2) Invalid DbvInitializeResponse()\n");
                printf("Input From:\tinitresp_invalid\n");
                tprintf("Output To:\tinitresp_invalid_result");

                printf("3) Valid DbvSearchResponse()\n");
                printf("Input From:\tsrchresp_valid\n");
                tprintf("Output To:\tsrchresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tsrchresp_valid_received");

                printf("4) Invalid DbvSearchResponse()\n");
                printf("Input From:\tsrchresp_invalid\n");
                tprintf("Output To:\tsrchresp_invalid_result");

                printf("5) Valid DbvPresentResponse()\n");
                printf("Input From:\tpresresp_valid\n");
                tprintf("Output To:\tpresresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tpresresp_valid_received");

                printf("6) Invalid DbvPresentResponse()\n");
                printf("Input From:\tpresresp_invalid\n");
                tprintf("Output To:\tpresresp_invalid_result");

                printf("7) Valid DbvDeleteResultSetResponse()\n");
                printf("Input From:\tdelresp_valid\n");
                tprintf("Output To:\tdelresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tdelresp_valid_received");

                printf("8) Invalid DbvDeleteResultSetResponse()\n");
                printf("Input From:\tdelresp_invalid\n");
                tprintf("Output To:\tdelresp_invalid_result");

                printf("9) Valid DbvScanResponse()\n");
                printf("Input From:\tscanresp_valid\n");
                tprintf("Output To:\tscanresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tscanresp_valid_received");

                printf("10) Invalid DbvScanResponse()\n");
                printf("Input From:\tscanresp_invalid\n");
                tprintf("Output To:\tinitresp_invalid_result");

                printf("11) Valid DbvResourceReportResponse()\n");
                printf("Input From:\tresresp_valid\n");
                tprintf("Output To:\tresresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tresresp_valid_received");

                printf("12) Invalid DbvResourceReportResponse()\n");
                printf("Input From:\tresresp_invalid\n");
                tprintf("Output To:\tresresp_invalid_result");

                printf("13) Valid DbvSortResponse()\n");
                printf("Input From:\tsortresp_valid\n");
                tprintf("Output To:\tsortresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tsortresp_valid_received");

                printf("14) Invalid DbvSortResponse()\n");
                printf("Input From:\tsortresp_invalid\n");
                tprintf("Output To:\tsortresp_invalid_result");

                printf("15) Valid DbvSegmentationRequest()\n");
                printf("Input From:\tsegresp_valid\n");
                tprintf("Output To:\tsegresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tsegresp_valid_received");

                printf("16) Invalid DbvSegmentationRequest()\n");
                printf("Input From:\tsegresp_invalid\n");
                tprintf("Output To:\tsegresp_invalid_result");

                printf("17) Valid DbvExtendedServicesResponse()\n");
                printf("Input From:\textresp_valid\n");
                tprintf("Output To:\textresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\textresp_valid_received");

                printf("18) Invalid DbvExtendedServicesResponse()\n");
                printf("Input From:\textresp_invalid\n");
                tprintf("Output To:\textresp_invalid_result");

                printf("19) Valid DbvCloseResponse()\n");
                printf("Input From:\tcloseresp_valid\n");
                tprintf("Output To:\tcloseresp_valid_result");
		printf("Response At Origin To:\n");
		tprintf("\tcloseresp_valid_received");

                printf("20) Invalid DbvCloseResponse()\n");
                printf("Input From:\tcloseresp_invalid\n");
                tprintf("Output To:\tcloseresp_invalid_result");

                printf("21) Valid DbvCloseRequest()\n");
                printf("Input From:\tclosereq_valid\n");
                tprintf("Output To:\tclosereq_valid_result");
		printf("Request At Origin To:\n");
		tprintf("\tclosereq_valid_received");
		printf("\tOrigin Response From:\n");
		tprintf("\t\tcloseresp_valid");
		printf("\tResponse From origin To:\n");
		tprintf("\t\tcloseresp_valid_received");

                printf("22) Invalid DbvCloseRequest()\n");
                printf("Input From:\tclosereq_invalid\n");
                tprintf("Output To:\tclosereq_invalid_result");



                printf("23) Valid DbvAccessControlRequest()\n");
                printf("Input From:\taccessreq_valid\n");
                tprintf("Output To:\taccessreq_valid_result");
		printf("Request At Origin To:\n");
		tprintf("\taccessreq_valid_received");
		printf("\tOrigin Response From:\n");
		tprintf("\t\taccessresp_valid");
		printf("\tResponse From origin To:\n");
		tprintf("\t\taccessresp_valid_received");

                printf("24) Invalid DbvAccessControlRequest()\n");
                printf("Input From:\taccessreq_invalid\n");
                tprintf("Output To:\taccessreq_invalid_result");

                printf("25) Valid DbvResourceControlRequest()\n");
                printf("Input From:\trescntrlreq_valid\n");
                tprintf("Output To:\trescntrlreq_valid_result");
		printf("Request At Origin To:\n");
		tprintf("\trescntrlreq_valid_received");
		printf("\tOrigin Response From:\n");
		tprintf("\t\trescntrlresp_valid");
		printf("\tResponse From origin To:\n");
		tprintf("\t\trescntrlresp_valid_received");

                printf("26) Invalid DbvResourceControlRequest()\n");
                printf("Input From:\trescntrlreq_invalid\n");
                tprintf("Output To:\trescntrlreq_invalid_result");



		printf("-------------------------------------------------------------------------------\n");
 
		printf("Communication Related Utility Functions\n\n");
                printf("33) Valid SIGetEventLocation()\n");
                printf("Input From:\ttarget_sievent_valid\n");
                tprintf("Output To:\ttarget_sievent_valid_result");

                printf("34) Invalid SIGetEventLocation()\n");
                printf("Input From:\ttarget_sievent_invalid\n");
                tprintf("Output To:\ttarget_sievent_invalid_result");

                printf("35) Valid DbvReceiveDataTarget()\n");
                printf("Input From:\ttarget_receive_valid\n");
                tprintf("Output To:\ttarget_receive_valid_result");
		
                printf("36) Invalid DbvReceiveDataTarget()\n");
                printf("Input From:\ttarget_receive_invalid\n");
                tprintf("Output To:\ttarget_receive_invalid_result");

		printf("37) Previous Menu\n");
                printf("Choice (1/2/3/4/5/6/7/8/9/10/11/12/13/14/15/16/17/18/19/20/21/22/23/24/25/26/33/34/35/36/37) :");
                fflush(stdout);
                gets(choice_string);
                choice = atoi(choice_string);

                switch(choice)
                {
                case 1:
        		do_DbvInitializeResponse(Valid);
			choice = -1;
                        break;
                case 2:
        		do_DbvInitializeResponse(Invalid);
			choice = -1;
                        break;
                case 3:
        		do_DbvSearchResponse(Valid);
			choice = -1;
                        break;
                case 4:
        		do_DbvSearchResponse(Invalid);
			choice = -1;
                        break;
                case 5:
        		do_DbvPresentResponse(Valid);
			choice = -1;
                        break;
                case 6:
        		do_DbvPresentResponse(Invalid);
			choice = -1;
                        break;
                case 7:
        		do_DbvDeleteResultSetResponse(Valid);
			choice = -1;
                        break;
                case 8:
        		do_DbvDeleteResultSetResponse(Invalid);
			choice = -1;
                        break;
                case 9:
        		do_DbvScanResponse(Valid);
			choice = -1;
                        break;
                case 10:
        		do_DbvScanResponse(Invalid);
			choice = -1;
                        break;
                case 11:
        		do_DbvResourceReportResponse(Valid);
			choice = -1;
                        break;
                case 12:
        		do_DbvResourceReportResponse(Invalid);
			choice = -1;
                        break;
                case 13:
        		do_DbvSortResponse(Valid);
			choice = -1;
                        break;
                case 14:
        		do_DbvSortResponse(Invalid);
			choice = -1;
                        break;
                case 15:
        		do_DbvSegmentRequest(Valid);
			choice = -1;
                        break;
                case 16:
        		do_DbvSegmentRequest(Invalid);
			choice = -1;
                        break;
                case 17:
        		do_DbvExtendedServicesResponse(Valid);
			choice = -1;
                        break;
                case 18:
        		do_DbvExtendedServicesResponse(Invalid);
			choice = -1;
                        break;
                case 19:
        		do_DbvCloseResponse(Valid);
			choice = -1;
                        break;
                case 20:
        		do_DbvCloseResponse(Invalid);
			choice = -1;
                        break;
                case 21:
        		do_DbvCloseRequest(Valid);
			choice = -1;
                        break;
                case 22:
        		do_DbvCloseRequest(Invalid);
			choice = -1;
                        break;

                case 23:
        		do_DbvAccessControlRequest(Valid);
			choice = -1;
                        break;
                case 24:
        		do_DbvAccessControlRequest(Invalid);
			choice = -1;
                        break;

                case 25:
        		do_DbvResourceControlRequest(Valid);
			choice = -1;
                        break;
                case 26:
        		do_DbvResourceControlRequest(Invalid);
			choice = -1;
                        break;

		case 33:
			do_SIGetEventLocation(Valid);
			choice = -1;
			break;
		case 34:
			do_SIGetEventLocation(Invalid);
			choice = -1;
			break;
		case 35:
			do_DbvReceiveDataTarget(Valid);
			choice = -1;
			break;
		case 36:
			do_DbvReceiveDataTarget(Invalid);
			choice = -1;
			break;

                case 37:
                        return_value = SI_PREV;
                        break;
                default:
                        printf("Not currently Implemented!\n");
                        choice = -1;
                        break;
                }
		if(return_value!=SI_PREV)
		{
                	printf("Press enter\n");
                	fflush(stdin);
                	gets(string);
		}

        }
 
	return return_value;
}

int test_non_comms(void)
{
	int return_value = 0;
	int choice = 0;
        char choice_string[100];

        while(choice < 1 || choice > 9)
        {
                printf("Choose Target Non Communication Utility Function:\n");
 
                printf("1) Valid DbvTargetInitialize()\n");
                printf("Input From:\ttargetinit_valid\n");
                tprintf("Output To:\ttargetinit_valid_result");
 
                printf("2) Invalid DbvTargetInitialize()\n");
                printf("Input From:\ttargetinit_invalid\n");
                tprintf("Output To:\ttargetinit_invalid_result");

                printf("3) Valid SISetACSEFile()\n");
                printf("Input From:\ttarget_acse_valid\n");
                tprintf("Output To:\ttarget_acse_valid_result");
 
                printf("4) Invalid SISetACSEFile()\n");
                printf("Input From:\ttarget_acse_invalid\n");
                tprintf("Output To:\ttarget_acse_invalid_result");
 
                printf("5) Valid SIWhichStack()\n");
                printf("Input From:\ttarget_stack_valid\n");
                tprintf("Output To:\ttarget_stack_valid_result");
 
                printf("6) Invalid SIWhichStack()\n");
                printf("Input From:\ttarget_stack_invalid\n");
                tprintf("Output To:\ttarget_stack_invalid_result");
 
                printf("7) Valid SISetTraceParameter()\n");
                printf("Input From:\ttarget_trace_valid\n");
                tprintf("Output To:\ttarget_trace_valid_result");
 
                printf("8) Invalid SISetTraceParameter()\n");
                printf("Input From:\ttarget_trace_invalid\n");
                tprintf("Output To:\ttarget_trace_invalid_result");
 
                printf("9) Previous Menu\n");
                printf("Choice (1/2/3/4/5/6/7/8/9) :");
                fflush(stdout);
                gets(choice_string);
                choice = atoi(choice_string);
 
                switch(choice)
                {
                case 1:
			do_DbvTargetInitialize(Valid);
			choice = -1;
                        break;
		case 2:
			do_DbvTargetInitialize(Invalid);
			choice = -1;
			break;
		case 3:
                        do_SISetACSEFile(Valid);
                        choice = -1;
                        break;
                case 4:
                        do_SISetACSEFile(Invalid);
                        choice = -1;
                        break;
                case 5:
                        do_SIWhichStack(Valid);
                        choice = -1;
                        break;
                case 6:
                        do_SIWhichStack(Invalid);
                        choice = -1;
                        break;
                case 7:
                        do_SISetTraceParameter(Valid);
                        choice = -1;
                        break;
                case 8:
                        do_SISetTraceParameter(Invalid);
                        choice = -1;
                        break;

                case 9:
                        return_value = SI_PREV;
                        break;
                default:
                        printf("Not currently Implemented!\n");
                        choice = -1;
                        break;
                }
                printf("Press enter\n");
                fflush(stdin);
                gets(string);
        }

	return return_value;
}

void tprintf(char *data)
{
        printf("%s%s\n",data, pcFileExtension);
}
 
/* Used for silent Targets */
void process_target_fields(DbvTargetData *psTargetData, DbvAssocId assoc_id)
{
	TBL ResultOutTable;
	char pcResult[128];
 
        /* InitRequest from client  */
        switch (psTargetData->eTargetEvent) {
        case TA_InitializeInd:
		open_result_table("initreq", &ResultOutTable, pcResult);
		printf("Writing InitializeRequest to %s\n",pcResult);
        	write_DbvInitializeRequest(psTargetData->uTargetData.psInitializeReq, ResultOutTable);
        	printf("Sending InitializeResponse\n");
        	do_DbvInitializeResponse(Valid);
        	break;
        default:
                printf("Error! Error! I can't deal with this\n");
                exit(-1);
        }
        TBL_Destruct(ResultOutTable);
}

int do_DbvTargetInitialize(TestType eTestType)
{
	DbvError *psError=NULL;
        int return_value = SI_OK;
 
        printf("Press enter to execute DbvTargetInitialize\n");
        printf("Please note that this function does not read from any files at present\n");
        fflush(stdin);
        get_return();
 
        return_value = DbvTargetInitialize(Gargc, Gargv, &psError);
	if(return_value == SI_OK)
	{
		printf("DbvTargetInitialize Ok\n");
	}
	else
		printf("DbvTargetInitialize failed\n");
 
	return return_value;
}

int do_DbvReceiveAssociateRequest(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int place = 0;
 
        int intfield;
        char *stringfield;
        int out_record = 1;
        int return_value = SI_OK;
 
        DbvAssocId         lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        DbvSecs            lSeconds;
	char 		*pcOriginAddress = (char *)NULL;
	DbvApplicationProtocol eApplicationProtocol = Unidentified_Protocol;
	DbvProtocolAndSyntaxes *psProtocolAndSyntaxes = NULL;
        DbvError         *psError=NULL;
 
        return_value = open_assoc_tables("recassocreq", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvReceiveAssociateRequest\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
        printf("Press enter to execute DbvReceiveAssociateRequest\n");
        fflush(stdin);
        get_return();

        return_value = DbvReceiveAssociateRequest(&lAssocId, lSeconds,
                &pcOriginAddress, &psProtocolAndSyntaxes, &psError);
 
                printf("Writing output to %s\n", OutFilename);
		write_comment_field(" Association Id: ", OutTable, T_Return);
                write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssocId(lAssocId, OutTable);
                write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvSecs(lSeconds, OutTable);
		write_comment_field(" OriginAddress: ", OutTable, T_Return);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field(pcOriginAddress, OutTable, T_Return);
                TBL_PutRecord(OutTable);

                write_comment_field(" ProtocolAndSyntaxes structure", OutTable, T_Return);
/* */
                stringfield = NULL;
                convert_DbvApplicationProtocol(&stringfield, 
			psProtocolAndSyntaxes->peApplicationProtocol);
                write_string_field(stringfield, OutTable,T_Return);
                DbvFree(stringfield);
                stringfield = NULL;
                convert_DbvAbstractSyntax(&stringfield, 
			psProtocolAndSyntaxes->peAbstractSyntax);
                write_string_field(stringfield, OutTable, T_Return);
                DbvFree(stringfield);
                TBL_PutRecord(OutTable);

                stringfield = NULL;
                intfield = 0;
                if(psProtocolAndSyntaxes->ppcCommonSyntaxes == NULL)
                        write_string_field("NULL",OutTable, T_Return);
                else
                {
                while(psProtocolAndSyntaxes->ppcCommonSyntaxes[intfield] 
			!= NULL)
                {
                        write_string_field(psProtocolAndSyntaxes->
                                ppcCommonSyntaxes[intfield], OutTable,
                                T_Return);
                        intfield++;
                }
                }

                TBL_PutRecord(OutTable);

        if(return_value == SI_OK)
        {
                printf("DbvReceiveAssociateRequest Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_DbvAssociateResponse(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int place = 0;
 
        int intfield;
        char *stringfield;
        int out_record = 1;
        int return_value = SI_OK;

        DbvAssocId         lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
	DbvAssociateRsp		eAssociateRsp;
	DbvError		*psError=NULL;

        return_value = open_assoc_tables("assocresp", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvAssociateResponse\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
	do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
	stringfield=get_string_field(InTable, T_Return);
	convert_DbvAssociateRsp(&stringfield, &eAssociateRsp);
	DbvFree(stringfield);
		
        printf("Press enter to execute DbvAssociateResponse\n");
        fflush(stdin);
        get_return();

        return_value = DbvAssociateResponse(lAssocId, eAssociateRsp, &psError);
 
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssociateRsp(eAssociateRsp, OutTable);
        if(return_value == SI_OK)
        {
        	printf("DbvAssociateResponse Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
	}
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_DbvReleaseResponse(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int place = 0;
 
        int intfield;
        char *stringfield;
        int out_record = 1;
        int return_value = SI_OK;

        DbvAssocId      lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
	DbvReleaseRsp	*psReleaseRsp=NULL;
	DbvError	*psError=NULL;

        return_value = open_assoc_tables("relresp", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvReleaseResponse\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
	do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
/* peReleaseRspReason */
        intfield=get_int_field(InTable,T_Return);
	stringfield = get_string_field(InTable, T_Return);
	
	if(strcmp(stringfield, NOTHING) != 0)
	{
		psReleaseRsp = (DbvReleaseRsp *)DbvAlloc(sizeof(DbvReleaseRsp));
		if(strcmp(stringfield, "Normal") == 0)
		{
			psReleaseRsp->peReleaseRspReason = 
				(DbvReleaseRspReason *)
				DbvAlloc(sizeof(DbvReleaseRspReason));
			*psReleaseRsp->peReleaseRspReason = Normal;
		}
		else if(strcmp(stringfield, "Not_Finished") == 0)
		{
			psReleaseRsp->peReleaseRspReason = 
				(DbvReleaseRspReason *)
				DbvAlloc(sizeof(DbvReleaseRspReason));
			*psReleaseRsp->peReleaseRspReason = Not_Finished;
		}
		else
		{
			printf("Could not convert DbvReleaseRsp->peReleaseRspReason\n");
        		TBL_Destruct(InTable);
        		TBL_Destruct(OutTable);
			return SI_NOTOK;
		}
        	DbvFree(stringfield);
        	stringfield = get_string_field(InTable, T_Return);
        	if(strcmp(stringfield, "Affirmative") == 0)
        	{
                        	psReleaseRsp->eReleaseRspResult = Affirmative;
        	}
        	else if(strcmp(stringfield, "Negative") == 0)
        	{
                        	psReleaseRsp->eReleaseRspResult = Negative;
        	}
        	else
        	{
                	printf("Could not convert DbvReleaseRsp->eReleaseRspResult\n");
                	TBL_Destruct(InTable);
                	TBL_Destruct(OutTable);
                	return SI_NOTOK;
        	}
        	DbvFree(stringfield);

	}
	
        printf("Press enter to execute DbvReleaseResponse\n");
        fflush(stdin);
        get_return();

        return_value = DbvReleaseResponse(lAssocId, psReleaseRsp, &psError);
 
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
        TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvReleaseRsp(psReleaseRsp, OutTable);

        if(return_value == SI_OK)
        {
        	printf("DbvReleaseResponse Ok\n");
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
	}
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        out_record = 1;
        get_return();
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int PrivateErrorHandling(DbvError **ppsError)
{
        DbvError *psError=NULL;
 
        psError = *ppsError;
        if(psError == NULL)
        {
                printf("Error structure was not filled out!\n");
                return SI_OK;
        }
        else
        {
                printf("psError->eErrorClass ==");
                if(psError->eErrorClass == SI_Communication_Error)
                        printf("SI_Communication_Error\n");
                if(psError->eErrorClass == SI_Interface_Error)
                        printf("SI_Interface_Error\n");
                if(psError->eErrorClass == SI_OperatingSystem_Error)
                {
                        printf("SI_OperatingSystem_Error\n");
                }
                else
                {
 
                        printf("psError->uErrorCode.psSIAbort->eAbortSource == %d\n",psError->uErrorCode.psAbort->eAbortSource);
                        printf("psError->uErrorCode.psSIAbort->lAbortReason == %d\n",psError->uErrorCode.psAbort->lAbortReason);
                        printf("->pcDiagnostics == %s\n", psError->uErrorCode.psAbort->pcDiagnostics);
                        printf("psError->eAssociationState == ");
 
                        if(psError->eAssociationState == Association_Open)
                                printf("Association_Open\n");
                        if(psError->eAssociationState == Association_Closed)
                                printf("Association_Closed\n");
                        if(psError->eAssociationState == Association_Unknown_State)
                                printf("Association_Unknown_State\n");
 
                }
        }
        return SI_OK;
}
 
 
