/* Target Functions */

#include "testy_calls.h"
#include "target_batch.h"

extern char *pcBatchFile;
extern char pcBatchFilename[SZ_FILENAME];
extern TTMode eMode;

int batch_DoTargetBatchProcessing(void)
{
        TBL psBatchTable = NULL;
        char pcInput[3];
 
        if(eMode != Batch)
                return SI_NOTOK;
 
        printf("Target Batch Mode\n");
        psBatchTable = TBL_Create(pcBatchFile, TBL_Read);
        if(psBatchTable == NULL)
        {
                printf("Could not open file %s\n", pcBatchFile);
                config_TargetQuit();
        }
 
        batch_ParseTargetBatchFile(psBatchTable);
        TBL_Destruct(psBatchTable);
 
        batch_ExecuteTargetBatchFile();
 
        while(*pcInput != 'm' && *pcInput != 'q')
        {
                printf("\n\nBatch run has completed. Do you wish to:\n");
                printf("Revert to (m)enu mode or (q)uit\n");
                fgets(pcInput, 2, stdin);
        }
        if(*pcInput == 'q')
                config_TargetQuit();
 
        eMode = Menu;
 
        return SI_OK;
}

void batch_ParseTargetBatchFile(TBL psBatchTable)
{
        DbvBool fFoundEntry = DBV_FALSE;
        char *pcEntry;
        char *pcFilename;
        long lSub = 0;
 
        printf("Parsing Batch File\n");
        while(TBL_GetRecord(psBatchTable) != DBV_FALSE)
        {
                pcEntry = config_GetCharField(psBatchTable);
                if(pcEntry == NULL)
                        config_TargetQuit();
                lSub = 0;
		fFoundEntry = DBV_FALSE;
                while( fFoundEntry == DBV_FALSE && lSub < NO_TARGET_FUNCTIONS)
                {
                        if((strcmp(psTargetFunctions[lSub].Name, pcEntry)) == 0)
                                fFoundEntry = DBV_TRUE;
                        lSub ++;
                }
                if(fFoundEntry == DBV_FALSE)
                        fFoundEntry = batch_ParseUtilityEntries(pcEntry);
 
                if(fFoundEntry == DBV_FALSE)
                {
                        printf("Entry %s is invalid\n", pcEntry);
                        config_TargetQuit();
                }
                else
                {
                        pcFilename = config_GetStringField(psBatchTable);
                        if(pcFilename != NULL && 
			    (strcmp(pcEntry,"DbvTargetInitialize")!=0) )
                                batch_CheckFile(pcFilename);
                        else
                        {
                                printf("Entry %s has no filename\n", pcEntry);
                        }
                }
                free(pcFilename);
                free(pcEntry);
        }
        printf("Batch File Ok\n");
}

/* Execute the target test functions */
void batch_ExecuteTargetBatchFile(void)
{
        TBL psBatchTable;
        char *pcEntry;
        long lSub = 0;
        DbvBool fFoundEntry;
        int return_value = SI_OK;
 
        psBatchTable = TBL_Create(pcBatchFile, TBL_Read);
 
        printf("\nExecuting Batch file\n");
        while(TBL_GetRecord(psBatchTable) != DBV_FALSE)
        {
                lSub = 0; fFoundEntry = DBV_FALSE;
                pcEntry = config_GetCharField(psBatchTable);
                if(pcEntry == NULL)
                        config_TargetQuit();
                while( fFoundEntry == DBV_FALSE && lSub < NO_TARGET_FUNCTIONS)
                {
                        if((strcmp(psTargetFunctions[lSub].Name, pcEntry)) == 0)
                                fFoundEntry = DBV_TRUE;
                        else
                                lSub ++;
                }
		if(fFoundEntry == DBV_FALSE)
			batch_ExecuteUtility(pcEntry, 
                                config_GetStringField(psBatchTable));
		else
		{
                	return_value = psTargetFunctions[lSub].function(
                        	config_GetStringField(psBatchTable));
                	if(return_value == SI_NOTOK)
                        	return_value = batch_QuitOrContinue();
		}
                free(pcEntry);
                sleep(1);
        }
 
}
 
int batch_ExecDbvTargetInitialize(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvTargetInitialize()\n");
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvTargetInitialize(Valid);
 
        return return_value;
}
int batch_ExecDbvReceiveAssociateRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvReceiveAssociateRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvReceiveAssociateRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvAssociateResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvAssociateResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvAssociateResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvReleaseResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvReleaseResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvReleaseResponse(Valid);
 
        return return_value;
}
 
int batch_ExecDbvInitializeResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvInitializeResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvInitializeResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvSearchResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvSearchResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvSearchResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvPresentResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvPresentResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvPresentResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvDeleteResultSetResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvDbvDeleteResultSetResponse() [%s]\n",
                pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvDeleteResultSetResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvScanResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvScanResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvScanResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvResourceReportResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvResourceReportResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvResourceReportResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvSortResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvSortResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvSortResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvSegmentationRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvSegmentationRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvSegmentRequest(Valid);
 
        return return_value;
}
int batch_ExecDbvExtendedServicesResponse(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvExtendedServicesResponse() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvExtendedServicesResponse(Valid);
 
        return return_value;
}
int batch_ExecDbvReceiveDataTarget(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvReceiveDataTarget() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvReceiveDataTarget(Valid);
 
        return return_value;
}


int batch_ExecDbvAccessControlRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvAccessControlRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvAccessControlRequest(Valid);
 
        return return_value;
}



int batch_ExecDbvResourceControlRequest(char *pcFilename)
{
        int return_value;
 
        printf("***Executing DbvResourceControlRequest() [%s]\n", pcFilename);
        strcpy(pcBatchFilename, pcFilename);
        free(pcFilename);
        return_value = do_DbvResourceControlRequest(Valid);
 
        return return_value;
}


