/*  Module Name: @(#)target_funcs.c	7.1
    Version: 11/15/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write results/read arguments to/from target specific tbl 
	files associated with the Z39.50 functions. For use with the testtool.
 
*/
 
#include "target_funcs.h"

extern TBL_Return T_Return;
extern char string[];

int do_DbvReceiveDataTarget(TestType eTestType)
{
        char InFilename[128];
        char OutFilename[128];
        TBL InTable, OutTable;
        int place = 0;
 
    	int intfield;
        char *stringfield;
        int out_record = 1;
        int return_value = SI_OK;
 
        DbvAssocId      lAssocId;
        DbvSecs         lSeconds;
        DbvTargetData       *psTargetData = NULL;
        DbvError        *psSIError = NULL;
        return_value = open_tables("target_receive", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvReceiveDataTarget\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId = get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
        printf("Press enter to execute DbvReceiveDataTarget\n");
        fflush(stdin);
        get_return();
        return_value = DbvReceiveDataTarget(lAssocId, lSeconds,
                &psTargetData, &psSIError);
 
        if(return_value == SI_OK)
        {
                printf("DbvReceiveDataTarget Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM", OutTable, T_Return);
                TBL_PutRecord(OutTable);
	        if(psTargetData==NULL)
        	{
            		write_int_field(out_record, OutTable, T_Return);
            		write_string_field("NULL", OutTable, T_Return);
            		write_comment_field("Function timed out", OutTable, 
				T_Return);
            		TBL_PutRecord(OutTable);
        	}
        	else
                	write_DbvTargetData(out_record, psTargetData, OutTable);

                out_record++;
                write_int_field(out_record, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
		FreeDbvTargetData(psTargetData);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psSIError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}
 
void write_DbvTargetData(int out_record, DbvTargetData *psTargetData, TBL OutTable)
{
        TBL ResultOutTable;
        char pcResult[128];
 
        switch (psTargetData->eTargetEvent)
        {
        /* InitRequest from server */
        case TA_InitializeInd:
		open_result_table("initreq", &ResultOutTable, pcResult);
                printf("Writing TA_InitializeInd PDU result to %s\n",pcResult);
                write_comment_field(" InitializeRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvInitializeRequest(psTargetData->uTargetData.psInitializeReq, ResultOutTable);
                break;

        case TA_SearchInd:
		open_result_table("srchreq", &ResultOutTable, pcResult);
                printf("Writing TA_SearchInd PDU result to %s\n",pcResult);
                write_comment_field(" SearchRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvSearchRequest(psTargetData->uTargetData.psSearchReq, ResultOutTable);
                break;
     case TA_PresentInd:
		open_result_table("presreq", &ResultOutTable, pcResult);
                printf("Writing TA_PresentInd PDU result to %s\n",pcResult);
                write_comment_field(" PresentRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvPresentRequest(psTargetData->uTargetData.psPresentReq, ResultOutTable);
                break;


       case TA_DeleteResultSetInd:
		open_result_table("delreq", &ResultOutTable, pcResult);
                printf("Writing TA_DeleteResultSetInd PDU result to %s\n",pcResult);
                write_comment_field(" DeleteResultSetRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvDeleteResultSetRequest(psTargetData->uTargetData.psDeleteResultSetReq, ResultOutTable);
                break;

        case TA_ScanInd:
		open_result_table("scanreq", &ResultOutTable, pcResult);
                printf("Writing TA_ScanInd PDU result to %s\n",pcResult);
                write_comment_field(" ScanRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvScanRequest(psTargetData->uTargetData.psScanReq, ResultOutTable);
                break;
        case TA_ResourceReportInd:
		open_result_table("rescreq", &ResultOutTable, pcResult);
                printf("Writing TA_ResourceReportInd PDU result to %s\n",pcResult);
                write_comment_field(" ResourceReportRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvResourceReportRequest(psTargetData->uTargetData.psResourceReportReq, ResultOutTable);
                break;
 
        case TA_SortInd:
		open_result_table("sortreq", &ResultOutTable, pcResult);
                printf("Writing TA_SortInd PDU result to %s\n",pcResult);
                write_comment_field(" SortRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvSortRequest(psTargetData->uTargetData.psSortReq, ResultOutTable);
                break;

        case TA_ExtendedServicesInd:
		open_result_table("extreq", &ResultOutTable, pcResult);
                printf("Writing TA_ExtendedServicesInd PDU result to %s\n",pcResult);
                write_comment_field(" ExtendedServicesRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvExtendedServicesRequest(psTargetData->uTargetData.psExtendedServicesReq, ResultOutTable);
                break;

        case TA_CloseInd:
		open_result_table("closereq", &ResultOutTable, pcResult);
                printf("Writing TA_CloseInd PDU result to %s\n",pcResult);
                write_comment_field(" CloseRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvCloseRequest(psTargetData->uTargetData.psCloseReq, ResultOutTable);
                break;
        case TA_CloseConf:
                open_result_table("closeresp", &ResultOutTable, pcResult);
                printf("Writing TA_CloseConf PDU result to %s\n",pcResult);
                write_comment_field(" CloseResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvCloseResponse(psTargetData->uTargetData.psCloseReq, ResultOutTable);
                break;

	case TA_ReleaseInd:
		open_result_table("relreq", &ResultOutTable, pcResult);
                printf("Writing TA_ReleaseInd PDU result to %s\n",pcResult);
                write_comment_field(" ReleaseRequest received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvReleaseRequest(psTargetData->uTargetData.peReleaseReq, ResultOutTable);
                break;
        case TA_Invalid_APDU:
           break;

        case TA_AccessCntrlConf:
                open_result_table("accessresp", &ResultOutTable, pcResult);
                printf("Writing TA_AccessCntrlConf PDU result to %s\n",pcResult);
                write_comment_field(" AccessControlResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvAccessControlResponse(psTargetData->uTargetData.psAccessControlRsp, ResultOutTable);
		break;

        case TA_ResourCntrlConf:
                open_result_table("rescntrlresp", &ResultOutTable, pcResult);
                printf("Writing TA_ResourCntrlConf PDU result to %s\n",pcResult);
                write_comment_field(" ResourceControlResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvResourceControlResponse(psTargetData->uTargetData.psResourceControlRsp, ResultOutTable);
		break;

        case TA_TrigResourCntrlInd:
		open_result_table("trigrescntrlreq", &ResultOutTable, pcResult);
                printf("Writing TA_TrigResourCntrlInd PDU result to %s\n",pcResult);
                write_comment_field(" TriggerResourceControlResponse received ", OutTable, T_Return);
                write_int_field(out_record, OutTable, T_Return);
                write_DbvTriggerResourceControlRequest(psTargetData->uTargetData.psTriggerResourceControlReq, ResultOutTable);
                break;

        default:
                break;
        }
 
        TBL_PutRecord(OutTable);
	if(ResultOutTable != NULL)
        	TBL_Destruct(ResultOutTable);
}

int DbvInitializeRequest(       DbvAssocId lAssocId,
                                DbvInitializeReq *psInitializeReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvSearchRequest(           DbvAssocId lAssocId,
                                DbvSearchReq *psSearchReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvPresentRequest(          DbvAssocId lAssocId,
                                DbvPresentReq *psPresentReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvDeleteResultSetRequest(  DbvAssocId lAssocId,
                                DbvDeleteResultSetReq *psDeleteResultSetReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvScanRequest(             DbvAssocId lAssocId,
                                DbvScanReq *psScanReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvResourceReportRequest(   DbvAssocId lAssocId,
                                DbvResourceReportReq *psResourceReportReq,
                                DbvError **ppsDbvError)
{

}

int DbvSortRequest(             DbvAssocId lAssocId,
                                DbvSortReq *psSortReq,
                                DbvError **ppsDbvError)
{

}
 
int DbvExtendedServicesRequest( DbvAssocId lAssocId,
                                DbvExtendedServicesReq *psExtendedServicesReq,
                                DbvError **ppsDbvError)
{

}

