/*  Module Name: @(#)testy_calls.c	7.3
    Version: 2/27/97
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        DBV-OSI II Origin Test Tool utility functions. Covers the conversion
	of strings to enumerated types and reading and writing to/from tbl 
	files. Lots of things, really. Some variables as well.

	Used by the origin and target.

    Modification History.

    14-02-96	D.B.Maskell	Added supprot for new externals
*/

#include "testy_calls.h"
#include <errno.h>

extern char *sys_errlist[];
extern int errno;

char pcFileExtension[12];
char pcTBLDirectory[STRING_LEN];
char string[STRING_LEN];
int eOriginOrTarget = 0;
DbvBool bPrintInput;
DbvBool bLocalResult;
 
TBL_Return T_Return;

extern TTMode eMode;
extern char *pcOWorkingDir;
extern char *pcOAssocOutput;
extern char *pcOCompletedStructures;
extern char *pcOUtilityOutput;
extern char *pcOReceivePDUs;
 
extern char *pcTWorkingDir;
extern char *pcTAssocOutput;
extern char *pcTCompletedStructures;
extern char *pcTUtilityOutput;
extern char *pcTReceivePDUs;
extern char *pcBatchFile;

char pcBatchFilename[SZ_FILENAME];

int open_result_table(char *Filename, TBL *OutTable, char *OutFilename)
{
        int return_value = SI_OK;
        char buffer[STRING_LEN];
 
        if(eOriginOrTarget == ORIGIN)
	{
		if(bLocalResult == DBV_TRUE)
		{
        		sprintf(OutFilename,"%s%s",pcTBLDirectory, 
				pcOCompletedStructures);
        		sprintf(buffer,"\n%s%s\n",pcTBLDirectory, 
				pcOCompletedStructures);
		}
		else
		{
        		sprintf(OutFilename,"%s%s",pcTBLDirectory, 
				pcOReceivePDUs);
        		sprintf(buffer,"\n%s%s\n",pcTBLDirectory, 
				pcOReceivePDUs);
		}
	}
        else
	{
		if(bLocalResult == DBV_TRUE)
		{
        		sprintf(OutFilename,"%s%s",pcTBLDirectory, 
				pcTCompletedStructures);
        		sprintf(buffer,"\n%s%s\n",pcTBLDirectory, 
				pcTCompletedStructures);
		}
		else
		{
        		sprintf(OutFilename,"%s%s",pcTBLDirectory, 
				pcTReceivePDUs);
        		sprintf(buffer,"\n%s%s\n",pcTBLDirectory, 
				pcTReceivePDUs);
		}
	}

        *OutTable = TBL_Create(OutFilename, TBL_Append);
        if(*OutTable == NULL)
        {
                printf("Error: Could not open file %s\n", OutFilename);
                return SI_NOTOK;
        }
	else if(bLocalResult == DBV_TRUE)
	{
		printf("Writing Completed PDU to: %s\n",OutFilename);
	}
	write_char_field("\n!!\n", *OutTable, T_Return);
	TBL_PutRecord(*OutTable);
        write_comment_field(OutFilename, *OutTable, T_Return);

        return return_value;
}

int open_tables(char *Filename, TestType eTestType, char *InFilename,
                char *OutFilename, TBL *InTable, TBL *OutTable)
{
        int return_value  = SI_OK;
        char buffer[STRING_LEN];
 
        if(eTestType == Valid)
        {
                sprintf(InFilename,"%s%s_valid",pcTBLDirectory, Filename);
                sprintf(OutFilename,"%s%s_valid_result%s",pcTBLDirectory, Filename, pcFileExtension);
                sprintf(buffer,"\n%s%s_valid_result%s\n", pcTBLDirectory, Filename, pcFileExtension);
        }
        else
        if(eTestType == Invalid)
        {
                sprintf(InFilename,"%s%s_invalid",pcTBLDirectory, Filename);
                sprintf(OutFilename,"%s%s_invalid_result%s",pcTBLDirectory, Filename, pcFileExtension);
                sprintf(buffer,"\n%s%s_invalid_result%s\n", pcTBLDirectory, Filename, pcFileExtension);
        }
        else
	{
                printf("TestType MUST be either valid or invalid\n");
		return SI_NOTOK;
	}
	get_alternative_filename(InFilename);
 
        *InTable = TBL_Create(InFilename, TBL_Read);
        if(*InTable == NULL)
        {
                printf("Error: Could not open file %s\n", InFilename);
                return SI_NOTOK;
        }
 
        *OutTable = TBL_Create(OutFilename, TBL_Append);
        if(*OutTable == NULL)
        {
                printf("Error: Could not open file %s\n", OutFilename);
                return SI_NOTOK;
        }
        write_char_field("\n!!\n", *OutTable, T_Return);
	TBL_PutRecord(*OutTable);
        write_comment_field(buffer, *OutTable, T_Return);
 
        return return_value;
}

int open_assoc_tables(char *Filename, TestType eTestType, char *InFilename,
                char *OutFilename, TBL *InTable, TBL *OutTable)
{
        int return_value  = SI_OK;
        char buffer[STRING_LEN];
	char *pcResult;

	if(eOriginOrTarget == ORIGIN)
		pcResult = pcOAssocOutput;
	else
		pcResult = pcTAssocOutput;
 
        sprintf(OutFilename,"%s%s", pcResult, pcFileExtension);
        sprintf(buffer,"\n%s%s\n", pcResult, pcFileExtension);

        if(eTestType == Valid)
        {
                sprintf(InFilename,"%s%s_valid",pcTBLDirectory, Filename);
        }
        else
        if(eTestType == Invalid)
        {
                sprintf(InFilename,"%s%s_invalid",pcTBLDirectory, Filename);
        }
        else
        {
                printf("TestType MUST be either valid or invalid\n");
                return SI_NOTOK;
        }
	get_alternative_filename(InFilename);
 
        *InTable = TBL_Create(InFilename, TBL_Read);
        if(*InTable == NULL)
        {
                printf("Error: Could not open file %s\n", InFilename);
                return SI_NOTOK;
        }
 
        *OutTable = TBL_Create(OutFilename, TBL_Append);
        if(*OutTable == NULL)
        {
                printf("Error: Could not open file %s\n", OutFilename);
                return SI_NOTOK;
        }
        write_char_field("\n!!\n", *OutTable, T_Return);
	TBL_PutRecord(*OutTable);
        write_comment_field(buffer, *OutTable, T_Return);
 
        return return_value;
}

int open_utility_tables(char *Filename, TestType eTestType, char *InFilename,
                char *OutFilename, TBL *InTable, TBL *OutTable)
{
        int return_value  = SI_OK;
        char buffer[STRING_LEN];
        char *pcResult;
 
        if(eOriginOrTarget == ORIGIN)
                pcResult = pcOUtilityOutput;
        else
                pcResult = pcTUtilityOutput;
 
        sprintf(OutFilename,"%s%s", pcResult, pcFileExtension);
        sprintf(buffer,"\n%s%s\n", pcResult, pcFileExtension);
 
        if(eTestType == Valid)
        {
                sprintf(InFilename,"%s%s_valid",pcTBLDirectory, Filename);
        }
        else
        if(eTestType == Invalid)
        {
                sprintf(InFilename,"%s%s_invalid",pcTBLDirectory, Filename);
        }
        else
        {
                printf("TestType MUST be either valid or invalid\n");
                return SI_NOTOK;
        }
	get_alternative_filename(InFilename);
 
        *InTable = TBL_Create(InFilename, TBL_Read);
        if(*InTable == NULL)
        {
                printf("Error: Could not open file %s\n", InFilename);
                return SI_NOTOK;
        }
 
        *OutTable = TBL_Create(OutFilename, TBL_Append);
        if(*OutTable == NULL)
        {
                printf("Error: Could not open file %s\n", OutFilename);
                return SI_NOTOK;
        }
        write_char_field("\n!!\n", *OutTable, T_Return);
	TBL_PutRecord(*OutTable);
        write_comment_field(buffer, *OutTable, T_Return);
 
        return return_value;
}

void get_alternative_filename(char *InFilename)
{
	char pcNewFilename[STRING_LEN];
	DbvBool fError = DBV_TRUE;
	int fd;

	if(eMode == Batch)
	{
		strcpy(InFilename, pcBatchFilename);
		return;
	}

	while( fError == DBV_TRUE)
	{
		printf("Input File [%s]:", InFilename);
		gets(pcNewFilename);
		if(pcNewFilename[0] == '\0')
			fError = DBV_FALSE;
		else	
		{
			if((fd = open(pcNewFilename, O_RDONLY)) == -1)
			{
				printf("%s does not exist\n",pcNewFilename);
			}
			else
			{
				strcpy(InFilename, pcNewFilename);
				close(fd);
				fError = DBV_FALSE;
			}

		}
	}
}

void get_return(void)
{
	char string[10];
	
	if(eMode == Menu)
		fgets(string, 10, stdin);
}

int remove_old_results(char *pcDirectory)
{
	struct dirent *dp;
	DIR *dfd=NULL;
	struct stat stbuf;
	int lNoOfFiles = 0;
	
	char choice_string[2];
	char *pcFile=NULL;

	if(stat(pcDirectory, &stbuf) == -1)
	{
		fprintf(stderr, "remove_results: can't access %s\n", pcDirectory);
		return SI_NOTOK;
	}
	if(!S_ISDIR(stbuf.st_mode))
	{
		fprintf(stderr, "remove_results: %s is not a directory'n",pcDirectory);
		return SI_NOTOK;
	}

	dfd=opendir(pcDirectory);
	if(dfd != NULL)
	{
		printf("Directory listing\n");
		while((dp = readdir(dfd)) != NULL)
		{
			if((strstr(dp->d_name, "result")!=NULL) ||
				(strstr(dp->d_name, "_received")!=NULL))
			{
				if(lNoOfFiles==0)
					printf("Result Files:\n");
				printf("\t%s%s\n", pcDirectory, dp->d_name);
				lNoOfFiles++;
			}
		}
	}
	if(lNoOfFiles>0)
	{
		printf("There are old result files in the directory. Do you want them deleted (y/n)?");
		gets(choice_string);
		if(*choice_string=='y' || *choice_string=='Y')
		{
			rewinddir(dfd);
			while((dp = readdir(dfd)) != NULL)
			{
				if((strstr(dp->d_name, "result")!=NULL) ||
					(strstr(dp->d_name, "_received")!=NULL))
				{
					pcFile=(char *)DbvAlloc(strlen(pcDirectory)+strlen(dp->d_name)+1);
					sprintf(pcFile,"%s%s", pcDirectory, dp->d_name);
					printf("Removing: %s\n", pcFile);
					unlink(pcFile);
					DbvFree(pcFile);
				}
			}
		}
	}
	else
		printf("There are no existing result files\n");

	closedir(dfd);
	
	return SI_OK;
}

/* This is common to both the origin and target */
int do_DbvAbortRequest(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
 
        DbvAssocId         lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        DbvError         *psError=NULL;
 
        return_value = open_assoc_tables("abortreq", eTestType, InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of DbvAbortRequest\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=intfield=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        printf("Press enter to execute DbvAbortRequest\n");
        fflush(stdin);
	get_return();
 
        return_value = DbvAbortRequest(lAssocId, &psError);
 
        printf("DbvAbortRequest Ok\n");
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssocId(lAssocId, OutTable);
        TBL_PutRecord(OutTable);
        if(return_value == SI_OK)
        {
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        out_record = 1;
	get_return();
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_SIGetEventLocation(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
        TBL InTable, OutTable;
        int place = 0;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
        char buffer[SML_BUFFER];
 
        DbvAssocId         	*pplAssocId[50];
        DbvSecs                 lSeconds;
        DbvFds          	*ppulFds[50];
        DbvEventLocation        *psEventLocation = NULL;
        DbvError         	*psError;
 
	if(eOriginOrTarget == ORIGIN)
        	return_value = open_utility_tables("origin_sievent", eTestType, InFilename, OutFilename, &InTable, &OutTable);
	else
        	return_value = open_utility_tables("target_sievent", eTestType, InFilename, OutFilename, &InTable, &OutTable);

        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of SIGetEventLocation\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
 
        while((stringfield=get_string_field(InTable,T_Return))!=NULL)
        {
                intfield = atol(stringfield);
                DbvFree(stringfield);
                pplAssocId[place] = (DbvAssocId *)DbvAlloc(sizeof(DbvAssocId));
                *pplAssocId[place] = intfield;
                printf("pplAssocId[%d] == %d\n", place, *pplAssocId[place]);
                place++;
        }
        pplAssocId[place] = (DbvAssocId *)NULL; /* Termintate list */
	ppulFds[0] = (DbvFds *)NULL; /* Not testing this */
        TBL_GetRecord(InTable);
        intfield=get_int_field(InTable,T_Return);
        lSeconds = convert_seconds(stringfield=get_string_field(InTable,T_Return));
        place=0;
 
        printf("Press enter to execute SIGetEventLocation\n");
        fflush(stdin);
	get_return();
 
        return_value = SIGetEventLocation(pplAssocId, lSeconds, ppulFds, 
		&psEventLocation, &psError);
       
                printf("SIGetEventLocation Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
		TT_writeDbvAssocIdList(pplAssocId, OutTable, T_Return);
                write_int_field(out_record++, OutTable, T_Return);
		TT_writeDbvSecs(lSeconds, OutTable);
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("IN_PARAM ppulFileDescriptor not tested", 
			OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_char_field("4 (ProviderEvent)",OutTable, T_Return);
                place=0;
                if(psEventLocation != NULL)
                {
                while(psEventLocation->pplProviderEvent[place] != NULL)
                {
                        sprintf(buffer,"%d",*(psEventLocation->pplProviderEvent[place]));
                        place ++;
                        write_string_field(buffer, OutTable, T_Return);
       
                }
                }
                if(place==0)
                {
                        write_string_field("NULL", OutTable, T_Return);
                }
                TBL_PutRecord(OutTable);
                write_char_field("4 (UserEvent)",OutTable, T_Return);
                place=0;
                if(psEventLocation !=NULL)
                {
                while(psEventLocation->ppulUserEvent[place] != NULL)
                {
                        sprintf(buffer,"%d",*(psEventLocation->ppulUserEvent[place]));
                        place++;
                        write_string_field(buffer, OutTable, T_Return);
                }
                }
                if(place==0)
                {
                        write_string_field("NULL", OutTable, T_Return);
                }
                TBL_PutRecord(OutTable);
        if(return_value == SI_OK)
        {
                out_record++;
                write_int_field(out_record, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1; place = 0;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_SIWhichStack(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
 
        DbvAssocId         	lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        DbvProtocolStack        eProtocolStack = Unknown_Stack;
        DbvError         	*psError=NULL;
 
        return_value = open_utility_tables("origin_stack", eTestType, 
		InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of SIWhichStack\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        printf("Press enter to execute SIWhichStack\n");
        fflush(stdin);
	get_return();
 
        return_value = SIWhichStack(lAssocId, &eProtocolStack, &psError);
 
        printf("SIWhichStack Ok\n");
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
        stringfield=NULL;
        convert_DbvProtocolStack(&stringfield, &eProtocolStack);
        write_string_field(stringfield,OutTable, T_Return);
        DbvFree(stringfield);
        TBL_PutRecord(OutTable);

        if(return_value == SI_OK)
        {
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_SISetACSEFile(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
 
        char        	*pcACSEFile;
        DbvError        *psError=NULL;
 
        return_value = open_utility_tables("origin_setacse", eTestType,
                InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of SISetACSEFile\n", OutTable, T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        pcACSEFile=get_string_field(InTable,T_Return);
        TBL_GetRecord(InTable);
        printf("Press enter to execute SISetACSEFile\n");
        fflush(stdin);
	get_return();
 
        return_value = SISetACSEFile(pcACSEFile, &psError);
 
                printf("SISetACSEFile Ok\n");
                printf("Writing output to %s\n", OutFilename);
                write_int_field(out_record++, OutTable, T_Return);
                write_char_field(pcACSEFile, OutTable, T_Return);
                TBL_PutRecord(OutTable);
        if(return_value == SI_OK)
        {
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
	DbvFree(pcACSEFile);
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

int do_SISetTraceParameter(TestType eTestType)
{
        char InFilename[STRING_LEN];
        char OutFilename[STRING_LEN];
	char pcBuffer[STRING_LEN];
        TBL InTable, OutTable;
        int test;
 
        int intfield;
        int out_record = 1;
        int return_value = SI_OK;
        char *stringfield;
 
        DbvAssocId      lAssocId = NO_ASSOCIATION_ID_ASSIGNED;
        char            *pcTraceFile;
	FILE 		*psTraceFp;
	DbvTraceLevel	eTraceLevel;
        DbvError        *psError=NULL;
 
        return_value = open_utility_tables("origin_trace", eTestType,
                InFilename, OutFilename, &InTable, &OutTable);
        if(return_value == SI_NOTOK)
                return return_value;
        write_comment_field("\nResults of SISetTraceParameter\n", OutTable, 
		T_Return);
        TBL_GetRecord(InTable);
        do {
        intfield=get_int_field(InTable,T_Return);
        lAssocId=get_int_field(InTable,T_Return);
        TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
        pcTraceFile=get_string_field(InTable,T_Return);
	if((psTraceFp=fopen(pcTraceFile, "a")) == NULL)
	{
		printf("Couldn't open trace file %s\n", pcTraceFile);
		printf("Error:%s", sys_errlist[errno]);
		return SI_NOTOK;
	}
	else
	{
		/* Don't worry about SI_OK */
		printf("Opening file %s as trace file. Trace information will be appended\n", pcTraceFile);
	}

        TBL_GetRecord(InTable);
	intfield=get_int_field(InTable,T_Return);
        stringfield=get_string_field(InTable,T_Return);
	convert_DbvTraceLevel(&stringfield, &eTraceLevel);
	DbvFree(stringfield);
	
        printf("Press enter to execute SISetTraceParameter\n");

        fflush(stdin);
	get_return();
 
        return_value = SISetTraceParameter(lAssocId, psTraceFp, eTraceLevel, 
		&psError);

        printf("SISetTraceParameter Ok\n");
        printf("Writing output to %s\n", OutFilename);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvAssocId(lAssocId, OutTable);
        write_int_field(out_record++, OutTable, T_Return);
	sprintf(pcBuffer,"Filename: %s\n", pcTraceFile);
	write_char_field(pcBuffer, OutTable, T_Return);
        write_int_field(out_record++, OutTable, T_Return);
	TT_writeDbvTraceLevel(eTraceLevel,OutTable);

        if(return_value == SI_OK)
        {
                write_int_field(out_record++, OutTable, T_Return);
                write_string_field("NULL",OutTable, T_Return);
                TBL_PutRecord(OutTable);
 
        }
        else
        {
                printf("Call failed\n");
                write_DbvError(&psError, OutTable);
        }
        TBL_GetRecord(InTable);
        out_record = 1;
        }
        while(TBL_GetRecord(InTable)!=FALSE);
 
        TBL_Destruct(InTable);
        TBL_Destruct(OutTable);
 
        return return_value;
}

char *get_string_field(TBL Table, TBL_Return Return)
{
 
        char    *item = NULL;
 
        /* Get the first field in the record read */
        if( TBL_GetField(Table, Return) == FALSE ) {
                return NULL;
        }
        if(Return->Tag != TBL_QText)
        {       
                /* Use an internal DBV function which is a bit naughty,
                        I suppose */
                printf("***Error in script file: Required string, read %s\n",
                        DBVGetFieldTypeString(Return->Tag));
		sleep(2);
                return NULL;
        }
        /* Use the memory malloc function in memcheck.c */
        item  = Return->Value.QText;

 	if(bPrintInput == DBV_TRUE)
		fprintf(stderr, "String: %s\n",item);
        return item;
}

char *get_char_field(TBL Table, TBL_Return Return)
{
 
	/* This function has been re-written to read in the whole char field,
	not just a word at a time. */
	char 	*char_ptr;
        char    *item = NULL;
 
        /* Get the first field in the record read */
        if( TBL_GetField(Table, Return) == FALSE ) {
                return NULL;
        }
        if(Return->Tag != TBL_UText)
        {       
                /* Use an internal DBV function which is a bit naughty,
                        I suppose */
                printf("***Error in script file: Required characters, read %s\n",
                        DBVGetFieldTypeString(Return->Tag));
		sleep(2);
                return NULL;
        }
        /* Use the memory malloc function in memcheck.c */
        item  = Return->Value.UText;

 	if(bPrintInput == DBV_TRUE)
		fprintf(stderr, "Char: %s\n",item);
        return item;
 
}
 
int get_int_field(TBL Table, TBL_Return Return)
{
        TBL_GetField(Table, Return);
        if(Return->Tag != TBL_Int)
        {       
                /* Use an internal DBV function which is a bit naughty,
                        I suppose */
                printf("***Error in script file: Required integer, read %s\n",
                        DBVGetFieldTypeString(Return->Tag));
		sleep(2);
                return 0;
        }
 	if(bPrintInput == DBV_TRUE)
		fprintf(stderr, "Integer: %d\n",Return->Value.Int);
        return(Return->Value.Int);
}

int *get_integer_string(TBL InTable)
{
	char *stringfield;
	int *intfield=NULL;

	if(strcmp((stringfield=get_string_field(InTable, T_Return)), NOTHING) != 0)
	{
		intfield=(int *)DbvAlloc(sizeof(int));
		*intfield=atoi(stringfield);
	}
	DbvFree(stringfield);
	return intfield;
}

void get_Authentication(DbvAuthentication **ppsAuthentication, TBL InTable)
{
        char *stringfield;
        int intfield;
        DbvAuthentication *psAuthentication;
        DbvAuthenticationType eAuthenticationType;
 
        stringfield=get_char_field(InTable, T_Return);
	DbvFree(stringfield);
        if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))!=0)
        {
        *ppsAuthentication = (DbvAuthentication *)
		DbvAlloc(sizeof(DbvAuthentication));
        psAuthentication = *ppsAuthentication;
        convert_DbvAuthenticationType(&stringfield, &eAuthenticationType);
        switch(eAuthenticationType)
        {
        case AT_Open:
                printf("AT_Open, Chief\n");
		TBL_GetRecord(InTable);
                psAuthentication->eAuthenticationType = AT_Open;
		get_DbvVisibleString(&(psAuthentication->uAuthentication.
			psOpen), InTable);
                break;
        case AT_IdPassword:
                printf("AT_IdPassword, Chief\n");
		TBL_GetRecord(InTable);
                psAuthentication->eAuthenticationType = AT_IdPassword;
		psAuthentication->uAuthentication.psIdPassword=(DbvIdPassword *)DbvAlloc(sizeof(DbvIdPassword));
		get_DbvInternationalString(&(psAuthentication->uAuthentication.
			psIdPassword->psGroupId), InTable);
		get_DbvInternationalString(&(psAuthentication->uAuthentication.
			psIdPassword->psUserId), InTable);
		get_DbvInternationalString(&(psAuthentication->uAuthentication.
			psIdPassword->psPassword), InTable);
                break;
        case AT_Anonymous:
                psAuthentication->eAuthenticationType = AT_Anonymous;
                break;
        case AT_Other:
                psAuthentication->eAuthenticationType = AT_Other;
		/* Open the external file */
		get_DbvASN1External(&(psAuthentication->uAuthentication.psOther), InTable);
                break;
        default:
                printf("AuthenticationType is invalid!\n");
                break;
 
        }
        if(stringfield != NULL)
        DbvFree(stringfield);
        }
}

void get_DbvInternationalString(DbvInternationalString **ppsInternationalString,
	TBL InTable)
{
	get_DbvOctetString((DbvOctetString **)ppsInternationalString, InTable);
}

void get_DbvOctetString(DbvOctetString **ppsOctetString, TBL InTable)
{
	int intfield;
	char *stringfield;
	intfield=get_int_field(InTable, T_Return);
	stringfield=get_string_field(InTable, T_Return);
	/* If ulLength is specified as -1, then work it out automatically */
	if(intfield==-1)
		intfield = strlen(stringfield);

	if((strcmp(stringfield, NOTHING)) == 0)
		*ppsOctetString = NULL;
	else
	{
		*ppsOctetString = (DbvOctetString *)
			DbvAlloc(sizeof(DbvOctetString));
                (*ppsOctetString)->ulLength = intfield;
                (*ppsOctetString)->pcValue = stringfield;
	}
}

/* Get a compulsory structure. If the pcValue is set to the value of NOTHING
	(usually "NULL"), then the structure will not be filled out. This should
	always make the API return with an error. */
void get_compulsory_DbvOctetString(DbvOctetString *psOctetString, TBL InTable)
{
        int intfield;
        char *stringfield;
        intfield=get_int_field(InTable, T_Return);
        stringfield=get_string_field(InTable, T_Return);

        /* If ulLength is specified as -1, then work it out automatically */
        if(intfield==-1)
                intfield = strlen(stringfield);
        if((strcmp(stringfield, NOTHING)) != 0)
        {
                psOctetString->ulLength = intfield;
                psOctetString->pcValue = stringfield;
        }
}

void get_compulsory_DbvInternationalString(DbvInternationalString 
	*psInternationalString, TBL InTable)
{
	get_compulsory_DbvOctetString((DbvOctetString *)psInternationalString, 
		InTable);
}

DbvBool get_DbvBool(TBL InTable)
{
	char *stringfield;
	DbvBool fReturn=DBV_FALSE;

	stringfield=get_string_field(InTable, T_Return);
	if(strcmp(stringfield,"DBV_TRUE") == 0)
		fReturn=DBV_TRUE;
	else if(strcmp(stringfield,"DBV_FALSE") == 0)
		fReturn=DBV_FALSE;
	else
		fprintf(stderr,"WARNING: Invalid DbvBool value %s\n",stringfield);

	DbvFree(stringfield);
	return fReturn;
}

void get_DbvOtherInformation(DbvOtherInformation **ppsOtherInformation, 
	TBL InTable)
{
	char *stringfield;
	int intfield;
	DbvOtherInformation *psOtherInformation;

	/* Check for NULL */
	TBL_GetRecord(InTable);
	/* Ignore the title */
	stringfield = get_char_field( InTable, T_Return );
	DbvFree( stringfield );
	stringfield = get_string_field(InTable, T_Return );
	if( (strcmp( stringfield, NOTHING )) == 0 )
	{
		DbvFree(stringfield);
		return;
	}
	DbvFree(stringfield);
	/* Malloc the space */
	*ppsOtherInformation=(DbvOtherInformation *)DbvAlloc(sizeof(DbvOtherInformation));
	psOtherInformation = *ppsOtherInformation;

	TBL_GetRecord(InTable);
	/* Ignore the title */
	stringfield=get_char_field(InTable, T_Return);
	DbvFree(stringfield);
	stringfield=get_string_field(InTable, T_Return);
	if((strcmp(stringfield,NOTHING))!=0)
	{
		DbvFree(stringfield);
		psOtherInformation->psInfoCategory=(DbvInfoCategory *)DbvAlloc(sizeof(DbvInfoCategory));
		stringfield=get_string_field(InTable, T_Return);
		if((strcmp(stringfield,NOTHING))!=0)
		{
			psOtherInformation->psInfoCategory->pcCategoryTypeId=stringfield;
		}
		psOtherInformation->psInfoCategory->ulCategoryValue=
			get_int_field(InTable, T_Return);
	}
	else
		DbvFree(stringfield);
	TBL_GetRecord(InTable);
	stringfield=get_char_field(InTable, T_Return);
	DbvFree(stringfield);
	stringfield=get_string_field(InTable, T_Return);
	convert_DbvInformationType(&stringfield, &(psOtherInformation->eInformationType));
	switch(psOtherInformation->eInformationType)
	{
	case IT_CharacterInfo:
		TBL_GetRecord(InTable);
		stringfield=get_char_field(InTable, T_Return);
		DbvFree(stringfield);
		get_DbvInternationalString(&(psOtherInformation->uInformation.psCharacterInfo), InTable);
		break;
	case IT_BinaryInfo:
		TBL_GetRecord(InTable);
		stringfield=get_char_field(InTable, T_Return);
		DbvFree(stringfield);
		get_DbvOctetString(&(psOtherInformation->uInformation.psBinaryInfo), InTable);
		break;
	case IT_ExternalInfo:
		get_DbvASN1External(&(psOtherInformation->uInformation.psExternallyDefinedInfo), InTable);
		break;
	case IT_ObjectIdentifier:
		TBL_GetRecord(InTable);
		get_DbvObjectIdentifier(&psOtherInformation->uInformation.
			pcObjectIdentifier, InTable);
		break;
		
	}

}

/* ********* WRITE FUNCTIONS ********* */
void write_string_field(char *text, TBL Table, TBL_Return Return)
{
	char nullstr[]="NULL";

        Return->Tag = TBL_QText;
	if(text == NULL)
        	Return->Value.QText = nullstr;
	else
        	Return->Value.QText = text;
        TBL_PutField(Table, Return);
}
 
void write_comment_field(char *text, TBL Table, TBL_Return Return)
{
 
        Return->Tag = TBL_Comment;
        Return->Value.Comment = text;
        TBL_PutField(Table, Return);
}

void write_char_field(char *item, TBL Table, TBL_Return Return)
{
	char nullstr[]="NULL";

        Return->Tag = TBL_UText;
	if(item == NULL)
        	Return->Value.UText = nullstr;
	else
        	Return->Value.UText = item;
        TBL_PutField(Table, Return);
}
 
void write_int_field(int item, TBL Table, TBL_Return Return)
{
 
        Return->Tag = TBL_Int;
        Return->Value.Int = item;
        TBL_PutField(Table, Return);
}
/* Write an integer to the result TBL file. "NULL" will be written
	if the pointer is NULL */
void write_title_int(char *title, int *item, TBL Table)
{
	char *temp;

	if(item == NULL)
		write_title_char("NULL", title, Table);
	else
	{
		/* +12 for a 32-bit integer (plus an extra bit )*/
		temp = DbvAlloc(strlen(title)+20);
		sprintf(temp,"%s: >%d<",title, *item);
		write_char_field(temp, Table, T_Return);
		DbvFree(temp);
	}
	TBL_PutRecord(Table);
}

/* Write an unquoted text field with title. Handles NULL correctly. */
void write_title_char(char *text, char *title, TBL Table)
{
	char pcNull[]="NULL";
	char *temp;

	if(text==NULL)
		text=pcNull;
	temp=DbvAlloc((strlen(title)+strlen(text)+5));
	sprintf(temp,"%s: >%s<",title, text);
	write_char_field(temp, Table, T_Return);
	DbvFree(temp);
}

void write_DbvBool(char *title, DbvBool fBool, TBL OutTable)
{
	if(fBool == DBV_TRUE)
		write_title_char("DBV_TRUE", title, OutTable);
	else
		write_title_char("DBV_FALSE", title, OutTable);
}

void write_DbvError(DbvError **ppsError, TBL Table)
{
        char *stringfield;
        char buffer[256];
        DbvError *psError;
 
        psError = *ppsError;
 
	printf("Writing Error structure\n");
        write_comment_field(" The API function returned SI_NOTOK\n\tError structure is detailed below", Table, T_Return);
        if(psError == NULL)
        {
                write_comment_field("Error structure was not filled out!", Table
, T_Return);
        }
        else
        {
                if(psError->eErrorClass == SI_Communication_Error)
                        write_comment_field("Error.eErrorClass == SI_Communication_Error", Table, T_Return);
                if(psError->eErrorClass == SI_Interface_Error)
                        write_comment_field("Error.eErrorClass == SI_Interface_Error", Table, T_Return);
                if(psError->eErrorClass == SI_OperatingSystem_Error)
                        write_comment_field("Error.eErrorClass == SI_OperatingSystem_Error", Table, T_Return);
                if(psError->eErrorClass == SI_Abort)
                {
                        write_comment_field("Error.eErrorClass == SI_Abort", Table, T_Return);
                        stringfield = NULL;
                        convert_DbvAbortSource(&stringfield, &(psError->uErrorCode.psAbort->eAbortSource));
                        sprintf(buffer, "Error.uErrorCode.psAbort->eAbortSource == %s",stringfield);
                        free(stringfield);
                        write_comment_field(buffer, Table, T_Return);
                        sprintf(buffer, "Error.uErrorCode.psAbort->lAbortReason == %d",psError->uErrorCode.psAbort->lAbortReason);
                        write_comment_field(buffer, Table, T_Return);
                        sprintf(buffer, "Error.uErrorCode.psAbort->pcDiagnostics == %s", psError->uErrorCode.psAbort->pcDiagnostics);
                        write_comment_field(buffer, Table, T_Return);
		}
		else
		{
                        sprintf(buffer,"Error.uErrorCode.lIfOsCoError == %d", psError->uErrorCode.lIfOsCoError);
                        write_comment_field(buffer, Table, T_Return);
		}
 
                if(psError->eAssociationState == Association_Open)
                        write_comment_field("Error.eAssociationState == Association_Open", Table, T_Return);
                if(psError->eAssociationState == Association_Closed)
                        write_comment_field("Error.eAssociationState == Association_Closed", Table, T_Return);
                if(psError->eAssociationState == Association_Unknown_State)
                        write_comment_field("Error.eAssociationState == Association_Unknown_State", Table, T_Return);
        }
        TBL_PutRecord(Table);
 
        free_DbvError(ppsError);
}

void write_DbvOctetString(char *name, DbvOctetString *psOctetString, 
	TBL OutTable)
{
        char *temp = NULL;
 
        if (psOctetString != NULL)
        {
		temp = (char *)DbvAlloc(strlen(name)+20+
			psOctetString->ulLength);
                sprintf(temp, "%s->pcValue: >", name);
                if ( psOctetString->pcValue )
                   strncat(temp, psOctetString->pcValue, 
			   psOctetString->ulLength);
                else
                   printf( "ERROR : Null Pointer in DbvOctetString" );

                strcat(temp,"<");
                write_char_field(temp, OutTable, T_Return);
                TBL_PutRecord(OutTable);
                sprintf(temp, "%s->ulLength: %d", name, psOctetString->ulLength);
        }
        else
        {
		temp = (char *)DbvAlloc(10+strlen(name));
                sprintf(temp, "%s : NULL",name);
        }
        write_char_field(temp, OutTable, T_Return);
	DbvFree(temp);
        TBL_PutRecord(OutTable);
}

void write_DbvInternationalString(char *name, DbvInternationalString
        *psInternationalString, TBL OutTable)
{
 
        write_DbvOctetString(name, (DbvOctetString *)psInternationalString,
                OutTable);
}

void write_DbvAuthentication(DbvAuthentication *psAuthentication, TBL OutTable)
{
	char temp[STRING_LEN];

	strcpy(temp, "psAuthentication: ");

	if(psAuthentication==NULL)
	{
		write_char_field("psAuthentication: NULL", OutTable, T_Return);
	}
	else
	{
        switch(psAuthentication->eAuthenticationType)
        {
        case AT_Open:
		write_char_field("eAuthenticationType : AT_Open", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_DbvVisibleString("uAuthentication.psOpen",psAuthentication->uAuthentication.psOpen, OutTable);
                /*TBL_PutRecord(OutTable);*/
                break;
        case AT_IdPassword:
		write_char_field("eAuthenticationType : AT_IdPassword", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_DbvInternationalString("uAuthentication.psIdPassword->psGroupid",psAuthentication->uAuthentication.psIdPassword->psGroupId, OutTable);
                TBL_PutRecord(OutTable);
                write_DbvInternationalString("uAuthentication.psIdPassword->psUserId",psAuthentication->uAuthentication.psIdPassword->psUserId, OutTable);
                TBL_PutRecord(OutTable);
                write_DbvInternationalString("uAuthentication.psIdPassword->psPassword",psAuthentication->uAuthentication.psIdPassword->psPassword, OutTable);
                TBL_PutRecord(OutTable);
                break;
        case AT_Anonymous:
		write_char_field("eAuthenticationType : AT_Anonymous", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                break;
        case AT_Other:
		write_char_field("eAuthenticationType : AT_Other", OutTable, T_Return);
                TBL_PutRecord(OutTable);
		write_DbvASN1External(psAuthentication->uAuthentication.psOther, OutTable);
                break;
        default:
                printf("This will never appear, honest!\n");
                break;
 
        }
	}

}

void write_DbvOtherInformation(DbvOtherInformation *psOtherInformation, TBL OutTable)
{
	char *stringfield=NULL;

	if(psOtherInformation == NULL)
	{
		write_title_char("NULL","DbvOtherInformation", OutTable);
		TBL_PutRecord(OutTable);
		return;
	}

	if(psOtherInformation->psInfoCategory!=NULL)
	{
		write_DbvObjectIdentifier("psInfoCategory->pcCategoryTypeId",psOtherInformation->psInfoCategory->pcCategoryTypeId, OutTable);
	TBL_PutRecord(OutTable);
		write_title_int("psInfoCategory->ulCategoryValue",(int *)&(psOtherInformation->psInfoCategory->ulCategoryValue), OutTable);
	}
	else
		write_title_char("psOtherInformation->psInfoCategory", "NULL", OutTable);
	TBL_PutRecord(OutTable);
	convert_DbvInformationType(&stringfield, &(psOtherInformation->eInformationType));
	write_title_char(stringfield, "eInformationType", OutTable);
	DbvFree(stringfield);
	TBL_PutRecord(OutTable);
	switch(psOtherInformation->eInformationType)
	{
	case IT_CharacterInfo:
		write_DbvInternationalString("uInformation.psCharacterInfo", psOtherInformation->uInformation.psCharacterInfo, OutTable);
		break;
	case IT_BinaryInfo:
		write_DbvOctetString("uInformation.psBinaryInfo", psOtherInformation->uInformation.psBinaryInfo, OutTable);
		break;
	case IT_ExternalInfo:
		write_DbvASN1External(psOtherInformation->uInformation.psExternallyDefinedInfo, OutTable);
		break;
	case IT_ObjectIdentifier:
		write_title_char(psOtherInformation->uInformation.pcObjectIdentifier,"uInformation.pcObjectIdentifier", OutTable);
		break;
	
	}
	TBL_PutRecord(OutTable);
}

int convert_seconds(char *seconds)
{
        int return_value;
 
        if(strcmp(seconds, "CALL_BLOCKING") == 0)
                return CALL_BLOCKING;
        if(strcmp(seconds, "CALL_ASYNCHRONOUS") == 0)
                return CALL_ASYNCHRONOUS;
 
        return atoi(seconds);
}
 
/* Conversion routines to convert from string to enumerated type and
        vice versa */
 
void convert_DbvAuthenticationType(char **text,
        DbvAuthenticationType *peAuthenticationType)
{
        if(*text != NULL)
        {
                if(strcmp("AT_Open", *text) == 0)
                        *peAuthenticationType = AT_Open;
                else
                if(strcmp("AT_IdPassword", *text) == 0)
                        *peAuthenticationType = AT_IdPassword;
                else
                if(strcmp("AT_Anonymous", *text) == 0)
                        *peAuthenticationType = AT_Anonymous;
                else
                if(strcmp("AT_Other", *text) == 0)
                        *peAuthenticationType = AT_Other;
                else
                        printf("Warning: DbvAuthenticationType string could not be converted: %s\n",*text);
 
        }
}
void convert_DbvAssociateRsp(char **text, DbvAssociateRsp *peAssociateRsp)
{
        char temp[SML_BUFFER];

	if(*text != NULL)
	{
		if(strcmp("Accepted", *text) == 0)
			*peAssociateRsp=Accepted;
		else
		if(strcmp("Rejected_Perm", *text) == 0)
			*peAssociateRsp=Rejected_Perm;
		else
		if(strcmp("Rejected_Tran", *text) == 0)
			*peAssociateRsp=Rejected_Tran;
		else
                        printf("Warning: DbvAssociateRsp string could not be converted: %s\n",*text);
	}
	else
	{
                if(*peAssociateRsp == Accepted)
                        strcpy(temp, "Accepted");
                if(*peAssociateRsp == Rejected_Perm)
                        strcpy(temp, "Rejected_Perm");
                if(*peAssociateRsp == Rejected_Tran)
                        strcpy(temp, "Rejected_Tran");
                if(strlen(temp)==0)
                        printf("Warning: DbvAssociateRsp enum could not be converted !\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}
}
 
void convert_DbvAbortSource(char **text, DbvAbortSource *peAbortSource)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                /* Convert enum type to string */
                if(*peAbortSource == SI_ACSE_User)
                        strcpy(temp, "SI_ACSE_User");
                if(*peAbortSource == SI_ACSE_Provider)
                        strcpy(temp, "SI_ACSE_Provider");
                if(*peAbortSource == SI_Presentation_User)
                        strcpy(temp, "SI_Presentation_User");
                if(*peAbortSource == SI_Presentation_Provider)
                        strcpy(temp, "SI_Presentation_Provider");
                if(*peAbortSource == SI_Transport_Provider)
                        strcpy(temp, "SI_Transport_Provider");
                if(strlen(temp)==0)
                        printf("Warning: DbvAbortSource enum could not be converted !\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
}

void convert_DbvApplicationProtocol(char **text, DbvApplicationProtocol *peApplicationProtocol)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                /* Convert enum type to string */
                if(*peApplicationProtocol == Unknown_Protocol)
                        strcpy(temp, "Unknown_Protocol");
                if(*peApplicationProtocol == Unidentified_Protocol)
                        strcpy(temp, "Unidentified_Protocol");
                if(*peApplicationProtocol == SR_Protocol)
                        strcpy(temp, "SR_Protocol");
                if(*peApplicationProtocol == Z3950_Protocol)
                        strcpy(temp, "Z3950_Protocol");
 
                if(strlen(temp)==0)
                        printf("Warning: DbvApplicationProtocol enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
        else
        {
                /* Convert string to enum */
                if(strcmp("Unknown_Protocol" , *text) == 0)
                        *peApplicationProtocol = Unknown_Protocol;
                else
                if(strcmp("Unidentified_Protocol" , *text) == 0)
                        *peApplicationProtocol = Unidentified_Protocol;
                else
                if(strcmp("SR_Protocol" , *text) == 0)
                        *peApplicationProtocol = SR_Protocol;
                else
                if(strcmp("Z3950_Protocol" , *text) == 0)
                        *peApplicationProtocol = Z3950_Protocol;
                else
                        printf("Warning: DbvApplicationProtocol string could not be converted: %s\n", *text);
        }
}

void convert_DbvAssociateState( char **text, 
	DbvAssociateState *peAssociateState)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                if(*peAssociateState == Associate_Null)
                        strcpy(temp, "Associate_Null");
                if(*peAssociateState == Associate_Accepted)
                        strcpy(temp, "Associate_Accepted");
                if(*peAssociateState == Associate_Rejected)
                        strcpy(temp, "Associate_Rejected");
                if(*peAssociateState == Associate_Pending)
                        strcpy(temp, "Associate_Pending");
                if(strlen(temp)==0)
                        printf("Warning: DbvAssociateState enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
 
}

void convert_DbvAbstractSyntax( char **text, 
	DbvAbstractSyntax *peAbstractSyntax)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                if(*peAbstractSyntax == No_Syntaxes_Provided)
                        strcpy(temp, "No_Syntaxes_Provided");
                if(*peAbstractSyntax == No_Common_Syntax)
                        strcpy(temp, "No_Common_Syntax");
                if(*peAbstractSyntax == Common_Syntaxes)
                        strcpy(temp, "Common_Syntaxes");
                if(strlen(temp)==0)
                        printf("Warning: DbvAbstractSyntax enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
        else
        {
                if(strcmp("No_Syntaxes_Provided", *text) == 0)
                        *peAbstractSyntax = No_Syntaxes_Provided;
                else
                if(strcmp("No_Common_Syntax", *text) == 0)
                        *peAbstractSyntax = No_Common_Syntax;
                else
                if(strcmp("Common_Syntaxes", *text) == 0)
                        *peAbstractSyntax = Common_Syntaxes;
                else
                        printf("Warning: DbvAbstractSyntax string could not be converted: %s\n",*text);
        }
 
}
 
void convert_CommonSyntaxes( char **text, char **ppcCommonSyntaxes)
{
        /* Only converts from ppcCommonSyntaxes to text */
        char temp[SML_BUFFER];
 
        strcpy(temp, "Not defined in spec");
        *text = (char *)malloc(strlen(temp)+1);
        strcpy(*text, temp);
}
 
void convert_DbvOsiOrInternetAddress(char **text,
        DbvOsiOrInternetAddress *peOsiOrInternetAddress)
{
        if(*text != NULL)
        {
                if(strcmp("Osi_Address", *text) == 0)
                        *peOsiOrInternetAddress = Osi_Address;
                else
                if(strcmp("Internet_Address", *text) == 0)
                        *peOsiOrInternetAddress = Internet_Address;
                else
                        printf("Warning: DbvOsiOrInternet string could not be converted: %s\n", *text);
        }
 
}
 
void convert_DbvTargetNameOrAddress( char **text, DbvTargetNameOrAddress 
        *peTargetNameOrAddress)
{
	if(*text != NULL)
        {
                if(strcmp("Target_Name", *text) == 0)
                        *peTargetNameOrAddress = Target_Name;
		else
                if(strcmp("Target_Address", *text) == 0)
                        *peTargetNameOrAddress = Target_Address;
                else
                        printf("Warning: DbvTargetNameOrAddress string could not be converted: %s\n",*text);
	}
}

void convert_DbvReleaseState(char **text, DbvReleaseState *peReleaseState)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                if(*peReleaseState == Release_Null)
                        strcpy(temp, "Release_Null");
                if(*peReleaseState == Release_Accepted)
                        strcpy(temp, "Release_Accepted");
                if(*peReleaseState == Release_Rejected)
                        strcpy(temp, "Release_Rejected");
                if(*peReleaseState == Release_Pending)
                        strcpy(temp, "Release_Pending");
                if(strlen(temp)==0)
                        printf("Warning: DbvReleaseState enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
}
 
void convert_DbvProtocolStack(char **text, DbvProtocolStack *peProtocolStack)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                if(*peProtocolStack == OSI_Stack)
                        strcpy(temp, "OSI_Stack");
                if(*peProtocolStack == TCP_Stack)
                        strcpy(temp, "TCP_Stack");
                if(*peProtocolStack == Unknown_Stack)
                        strcpy(temp, "Unknown_Stack");
                if(strlen(temp)==0)
                        printf("Warning: DbvProtocolStack enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
}

void convert_DbvExternalValueType(char **text, DbvExternalValueType *peValueType)
{
	char temp[SML_BUFFER];

	if(*text != NULL)
	{
                if(strcmp("EVT_SingleASN1Type", *text) == 0)
                        *peValueType = EVT_SingleASN1Type;
                else
                if(strcmp("EVT_OctetAligned", *text) == 0)
                        *peValueType = EVT_OctetAligned;
                else
                if(strcmp("EVT_Arbitrary", *text) == 0)
                        *peValueType = EVT_Arbitrary;
		else
                        printf("Warning: DbvExternalValueType string could not be converted: %s\n", *text);

        }
	else
	{
		temp[ 0 ] = '\0';
                if(*peValueType == EVT_SingleASN1Type)
                        strcpy(temp, "EVT_SingleASN1Type");
                if(*peValueType == EVT_OctetAligned)
                        strcpy(temp, "EVT_OctetAligned");
                if(*peValueType == EVT_Arbitrary)
                        strcpy(temp, "EVT_Arbitrary");
                if(strlen(temp)==0)
                        printf("Warning: DbvExternalValueType enum could not be converted!\n" );
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}
}

void convert_DbvASN1Type(char **text, DbvASN1Type *peASN1Type)
{
        char temp[SML_BUFFER];
 
        if(*text != NULL)
        {
                if(strcmp("AT_UnstructuredText", *text) == 0)
                        *peASN1Type = AT_UnstructuredText;
		else
                if(strcmp("AT_OpacRecord", *text) == 0)
                        *peASN1Type = AT_OpacRecord;
		else
                if(strcmp("AT_SummaryRecord", *text) == 0)
                        *peASN1Type = AT_SummaryRecord;
		else
                if(strcmp("AT_ResourceReport1", *text) == 0)
                        *peASN1Type = AT_ResourceReport1;
		else
                if(strcmp("AT_ResourceReport2", *text) == 0)
                        *peASN1Type = AT_ResourceReport2;
		else
                if(strcmp("AT_EsTaskPackage", *text) == 0)
                        *peASN1Type = AT_EsTaskPackage;
		else
                if(strcmp("AT_UpdateExtendedService", *text) == 0)
                        *peASN1Type = AT_UpdateExtendedService;
		else
                if(strcmp("AT_UnknownType", *text) == 0)
                        *peASN1Type = AT_UnknownType;
		else
                if(strcmp("AT_ItemOrderExtendedService", *text) == 0)
                        *peASN1Type = AT_ItemOrderExtendedService;
		else
                if(strcmp("AT_IllPdu", *text) == 0)
                        *peASN1Type = AT_IllPdu;
		else

                if(strcmp("AT_ExplainRecord", *text) == 0)
                        *peASN1Type = AT_ExplainRecord;
                else

#ifdef NOT_YET_IMPLEMENTED
                if(strcmp("AT_GenericRecord", *text) == 0)
                        *peASN1Type = AT_GenericRecord;
                else
#endif

                if(strcmp("AT_PromptObject", *text) == 0)
                        *peASN1Type = AT_PromptObject;

                else

                if(strcmp("AT_CharSetAndLanguage", *text) == 0)
                        *peASN1Type = AT_CharSetAndLanguage;

                else

#ifdef NOT_YET_IMPLEMENTED
                if(strcmp("AT_AccessControlDes1", *text) == 0)
                        *peASN1Type = AT_AccessControlDes1;
                else
                if(strcmp("AT_AccessControlKrb1", *text) == 0)
                        *peASN1Type = AT_AccessControlKrb1;
                else
                if(strcmp("AT_PerResSetExtendedService", *text) == 0)
                        *peASN1Type = AT_PerResSetExtendedService;
                else
                if(strcmp("AT_PerQueryExtendedService", *text) == 0)
                        *peASN1Type = AT_PerQueryExtendedService;
                else
                if(strcmp("AT_PerQuerySchExtendedService", *text) == 0)
                        *peASN1Type = AT_PerQuerySchExtendedService;
                else
                if(strcmp("AT_ExpSpecExtendedService", *text) == 0)
                        *peASN1Type = AT_ExpSpecExtendedService;
                else
                if(strcmp("AT_ExpInvoExtendedService", *text) == 0)
                        *peASN1Type = AT_ExpInvoExtendedService;
                else
                if(strcmp("AT_UserInfoSearchResult", *text) == 0)
                        *peASN1Type = AT_UserInfoSearchResult;
                else
                if(strcmp("AT_ElementSpec1", *text) == 0)
                        *peASN1Type = AT_ElementSpec1;
                else
#endif  /* End NOT_YET_IMPLEMENTED */
                        printf("Warning: DbvASN1Type string could not be converted: %s\n",*text);
	}
	else
	{
		temp[ 0 ] = '\0';
                if(*peASN1Type == AT_UnstructuredText)
                        strcpy(temp, "AT_UnstructuredText");

                if(*peASN1Type == AT_OpacRecord)
                        strcpy(temp, "AT_OpacRecord");

                if(*peASN1Type == AT_SummaryRecord)
                        strcpy(temp, "AT_SummaryRecord");

                if(*peASN1Type == AT_ResourceReport1)
                        strcpy(temp, "AT_ResourceReport1");

                if(*peASN1Type == AT_ResourceReport2)
                        strcpy(temp, "AT_ResourceReport2");

                if(*peASN1Type == AT_EsTaskPackage)
                        strcpy(temp, "AT_EsTaskPackage");

                if(*peASN1Type == AT_UpdateExtendedService)
                        strcpy(temp, "AT_UpdateExtendedService");

                if(*peASN1Type == AT_UnknownType)
                        strcpy(temp, "AT_UnknownType");

                if(*peASN1Type == AT_ItemOrderExtendedService)
                        strcpy(temp, "AT_ItemOrderExtendedService");

                if(*peASN1Type == AT_IllPdu)
                        strcpy(temp, "AT_IllPdu");

                if(*peASN1Type == AT_ExplainRecord)
                        strcpy(temp, "AT_ExplainRecord");


#ifdef NOT_YET_IMPLEMENTED
                if(*peASN1Type == AT_GenericRecord)
                        strcpy(temp, "AT_GenericRecord");
#endif

                if(*peASN1Type == AT_PromptObject)
                        strcpy(temp, "AT_PromptObject");


                if(*peASN1Type == AT_CharSetAndLanguage)
                        strcpy(temp, "AT_CharSetAndLanguage");



#ifdef NOT_YET_IMPLEMENTED
                if(*peASN1Type == AT_AccessControlDes1)
                        strcpy(temp, "AT_AccessControlDes1");

                if(*peASN1Type == AT_AccessControlKrb1)
                        strcpy(temp, "AT_AccessControlKrb1");

                if(*peASN1Type == AT_PerResSetExtendedService)
                        strcpy(temp, "AT_PerResSetExtendedService");

                if(*peASN1Type == AT_PerQueryExtendedService)
                        strcpy(temp, "AT_PerQueryExtendedService");

                if(*peASN1Type == AT_PerQuerySchExtendedService)
                        strcpy(temp, "AT_PerQuerySchExtendedService");

                if(*peASN1Type == AT_ExpSpecExtendedService)
                        strcpy(temp, "AT_ExpSpecExtendedService");

                if(*peASN1Type == AT_ExpInvoExtendedService)
                        strcpy(temp, "AT_ExpInvoExtendedService");

                if(*peASN1Type == AT_UserInfoSearchResult)
                        strcpy(temp, "AT_UserInfoSearchResult");

                if(*peASN1Type == AT_ElementSpec1)
                        strcpy(temp, "AT_ElementSpec1");

#endif  /* End NOT_YET_IMPLEMENTED */
                if(strlen(temp)==0)
                        printf("Warning: DbvASN1Type enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}

}

void convert_DbvHoldingsRecType(char **text, DbvHoldingsRecType *peHoldingsRecType)
{
        char temp[SML_BUFFER];
 
        if(*text != NULL)
        {
                if(strcmp("HRT_MarcHoldingsRecord", *text) == 0)
                        *peHoldingsRecType = HRT_MarcHoldingsRecord;
		else
                if(strcmp("HRT_HoldingsAndCirc", *text) == 0)
                        *peHoldingsRecType = HRT_HoldingsAndCirc;
		else
                        printf("Warning: DbvHoldingsRecType string could not be converted: %s\n", *text);
	}
	else
	{
                if(*peHoldingsRecType == HRT_MarcHoldingsRecord)
                        strcpy(temp, "HRT_MarcHoldingsRecord");
                if(*peHoldingsRecType == HRT_HoldingsAndCirc)
                        strcpy(temp, "HRT_HoldingsAndCirc");
                if(strlen(temp)==0)
                        printf("Warning: DbvHoldingsRecType enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}
}

void convert_DbvEstimateType(char **text, DbvEstimateType *peEstimateType)
{
	char temp[SML_BUFFER];

	if(*text != NULL)
	{
                if(strcmp("ET_CurrentSearchRecords", *text) == 0)
                        *peEstimateType = ET_CurrentSearchRecords;
                else
                if(strcmp("ET_FinalSearchRecords", *text) == 0)
                        *peEstimateType = ET_FinalSearchRecords;
                else
                if(strcmp("ET_CurrentPresentRecords", *text) == 0)
                        *peEstimateType = ET_CurrentPresentRecords;
                else
                if(strcmp("ET_FinalPresentRecords", *text) == 0)
                        *peEstimateType = ET_FinalPresentRecords;
                else
                if(strcmp("ET_CurrentOpTimeProcessing", *text) == 0)
                        *peEstimateType = ET_CurrentOpTimeProcessing;
                else
                if(strcmp("ET_FinalOpTimeProcessing", *text) == 0)
                        *peEstimateType = ET_FinalOpTimeProcessing;
                else
                if(strcmp("ET_CurrentAssocTime", *text) == 0)
                        *peEstimateType = ET_CurrentAssocTime;
                else
                if(strcmp("ET_CurrentOperationCost", *text) == 0)
                        *peEstimateType = ET_CurrentOperationCost;
                else
                if(strcmp("ET_FinalOperationCost", *text) == 0)
                        *peEstimateType = ET_FinalOperationCost;
                else
                if(strcmp("ET_CurrentAssocCost", *text) == 0)
                        *peEstimateType = ET_CurrentAssocCost;
                else
                if(strcmp("ET_FinalOpTimeElapsed", *text) == 0)
                        *peEstimateType = ET_FinalOpTimeElapsed;
                else
                if(strcmp("ET_PercentComplete", *text) == 0)
                        *peEstimateType = ET_PercentComplete;
                else
                if(strcmp("ET_CurrentSearchAssocCost", *text) == 0)
                        *peEstimateType = ET_CurrentSearchAssocCost;
                else
                if(strcmp("ET_CurrentPresentAssocCost", *text) == 0)
                        *peEstimateType = ET_CurrentPresentAssocCost;
                else
                if(strcmp("ET_CurrentConnectAssocCost", *text) == 0)
                        *peEstimateType = ET_CurrentConnectAssocCost;
                else
                if(strcmp("ET_CurrentOtherAssocCost", *text) == 0)
                        *peEstimateType = ET_CurrentOtherAssocCost;
                else
                if(strcmp("ET_Invalid", *text) == 0)
                        *peEstimateType = ET_Invalid;
                else
                        printf("Warning: DbvEstimateType string could not be converted! >%s\n",*text);
	}
	else
	{
                if(*peEstimateType == ET_CurrentSearchRecords)
                        strcpy(temp, "ET_CurrentSearchRecords");
                if(*peEstimateType == ET_FinalSearchRecords)
                        strcpy(temp, "ET_FinalSearchRecords");
                if(*peEstimateType == ET_CurrentPresentRecords)
                        strcpy(temp, "ET_CurrentPresentRecords");
                if(*peEstimateType == ET_FinalPresentRecords)
                        strcpy(temp, "ET_FinalPresentRecords");
                if(*peEstimateType == ET_CurrentOpTimeProcessing)
                        strcpy(temp, "ET_CurrentOpTimeProcessing");
                if(*peEstimateType == ET_FinalOpTimeProcessing)
                        strcpy(temp, "ET_FinalOpTimeProcessing");
                if(*peEstimateType == ET_CurrentAssocTime)
                        strcpy(temp, "ET_CurrentAssocTime");
                if(*peEstimateType == ET_CurrentOperationCost)
                        strcpy(temp, "ET_CurrentOperationCost");
                if(*peEstimateType == ET_FinalOperationCost)
                        strcpy(temp, "ET_FinalOperationCost");
                if(*peEstimateType == ET_CurrentAssocCost)
                        strcpy(temp, "ET_CurrentAssocCost");
                if(*peEstimateType == ET_FinalOpTimeElapsed)
                        strcpy(temp, "ET_FinalOpTimeElapsed");
                if(*peEstimateType == ET_PercentComplete)
                        strcpy(temp, "ET_PercentComplete");
                if(*peEstimateType == ET_CurrentSearchAssocCost)
                        strcpy(temp, "ET_CurrentSearchAssocCost");
                if(*peEstimateType == ET_CurrentPresentAssocCost)
                        strcpy(temp, "ET_CurrentPresentAssocCost");
                if(*peEstimateType == ET_CurrentConnectAssocCost)
                        strcpy(temp, "ET_CurrentConnectAssocCost");
                if(*peEstimateType == ET_CurrentOtherAssocCost)
                        strcpy(temp, "ET_CurrentOtherAssocCost");
                if(*peEstimateType == ET_Invalid)
                        strcpy(temp, "ET_Invalid");

                if(strlen(temp)==0)
                        printf("Warning: DbvEstimateType enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}

}

void convert_DbvInformationType(char **text, DbvInformationType *peInformationType)
{
        char temp[SML_BUFFER];
 
        if(*text != NULL)
        {
                if(strcmp("IT_CharacterInfo", *text) == 0)
                        *peInformationType = IT_CharacterInfo;
		else
                if(strcmp("IT_BinaryInfo", *text) == 0)
                        *peInformationType = IT_BinaryInfo;
		else
                if(strcmp("IT_ExternalInfo", *text) == 0)
                        *peInformationType = IT_ExternalInfo;
		else
                if(strcmp("IT_ObjectIdentifier", *text) == 0)
                        *peInformationType = IT_ObjectIdentifier;
		else
                        printf("Warning: DbvInformationType string could not be converted: %s\n", *text);
	}
	else
	{
                if(*peInformationType == IT_CharacterInfo)
                        strcpy(temp, "IT_CharacterInfo");
                if(*peInformationType == IT_BinaryInfo)
                        strcpy(temp, "IT_BinaryInfo");
                if(*peInformationType == IT_ExternalInfo)
                        strcpy(temp, "IT_ExternalInfo");
                if(*peInformationType == IT_ObjectIdentifier)
                        strcpy(temp, "IT_ObjectIdentifier");

                if(strlen(temp)==0)
                        printf("Warning: DbvInformationType enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
	}
}

void convert_DbvResourceReportStatus(char **text, DbvResourceReportStatus *peResourceReportStatus)
{
        char temp[SML_BUFFER];
 
        if(*text == NULL)
        {
                /* Convert enum type to string */
                if(*peResourceReportStatus == RRS_Success)
                        strcpy(temp, "RRS_Success");
                if(*peResourceReportStatus == RRS_Partial)
                        strcpy(temp, "RRS_Partial");
                if(*peResourceReportStatus == RRS_Failure_1)
                        strcpy(temp, "RRS_Failure_1");
                if(*peResourceReportStatus == RRS_Failure_2)
                        strcpy(temp, "RRS_Failure_2");
                if(*peResourceReportStatus == RRS_Failure_3)
                        strcpy(temp, "RRS_Failure_3");
                if(*peResourceReportStatus == RRS_Failure_4)
                        strcpy(temp, "RRS_Failure_4");
                if(*peResourceReportStatus == RRS_Failure_5)
                        strcpy(temp, "RRS_Failure_5");
                if(*peResourceReportStatus == RRS_Failure_6)
                        strcpy(temp, "RRS_Failure_6");
                if(*peResourceReportStatus == RRS_Invalid)
                        strcpy(temp, "RRS_Invalid");

                if(strlen(temp)==0)
                        printf("Warning: DbvResourceReportStatus enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
        else
        {
                /* Convert string to enum */
                if(strcmp("RRS_Success" , *text) == 0)
                        *peResourceReportStatus  = RRS_Success;
                else
                if(strcmp("RRS_Partial" , *text) == 0)
                        *peResourceReportStatus  = RRS_Partial;
                else
                 if(strcmp("RRS_Failure_1" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_1;
                else
                 if(strcmp("RRS_Failure_2" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_2;
                else
                 if(strcmp("RRS_Failure_3" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_3;
                else
                 if(strcmp("RRS_Failure_4" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_4;
                else
                 if(strcmp("RRS_Failure_5" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_5;
                else
                 if(strcmp("RRS_Failure_6" , *text) == 0)
                        *peResourceReportStatus  = RRS_Failure_6;
                else
                 if(strcmp("RRS_Invalid" , *text) == 0)
                        *peResourceReportStatus  = RRS_Invalid;
                else
                        printf("Warning: DbvResourceReport string could not be converted: %s\n", *text);
        }
}





void convert_DbvCloseReason(char **text, DbvCloseReason *peCloseReason)
{
        char temp[SML_BUFFER];
        int length; 
        if(*text == NULL)

        {
                /* Convert enum type to string */
                if(*peCloseReason == CR_Finished)
                        strcpy(temp, "CR_Finished");
                if(*peCloseReason == CR_ShutDown)
                        strcpy(temp, "CR_ShutDown");
                if(*peCloseReason == CR_SystemProblem)
                        strcpy(temp, "CR_SystemProblem");
                if(*peCloseReason == CR_CostLimit)
                        strcpy(temp, "CR_CostLimit");
                if(*peCloseReason == CR_Resources)
                        strcpy(temp, "CR_Resources");
                if(*peCloseReason == CR_SecurityViolation)
                        strcpy(temp, "CR_SecurityViolation");
                if(*peCloseReason == CR_ProtocolError)
                        strcpy(temp, "CR_ProtocolError");
                if(*peCloseReason == CR_LackOfActivity)
                        strcpy(temp, "CR_LackOfActivity");
                if(*peCloseReason == CR_PeerAbort)
                        strcpy(temp, "CR_PeerAbort");
                if(*peCloseReason == CR_Unspecified)
                        strcpy(temp, "CR_Unspecified");
                if(*peCloseReason == CR_Invalid)
                        strcpy(temp, "CR_Invalid");
                if(strlen(temp)==0)
                        printf("Warning: DbvCloseReason enum could not be converted!\n");
                *text = (char *)malloc(strlen(temp)+1);
                strcpy(*text, temp);
        }
        else
        {
                /* Convert string to enum */
                if(strcmp("CR_Finished" , *text) == 0)
                        *peCloseReason = CR_Finished;
                else
                if(strcmp("CR_ShutDown" , *text) == 0)
                        *peCloseReason = CR_ShutDown;
                else
                if(strcmp("CR_SystemProblem" , *text) == 0)
                        *peCloseReason = CR_SystemProblem;
                else
                if(strcmp("CR_CostLimit" , *text) == 0)
                        *peCloseReason = CR_CostLimit;
                else
               if(strcmp("CR_Resources" , *text) == 0)
                        *peCloseReason = CR_Resources;
                else
               if(strcmp("CR_SecurityViolation" , *text) == 0)
                        *peCloseReason = CR_SecurityViolation;
                else
               if(strcmp("CR_ProtocolError" , *text) == 0)
                        *peCloseReason = CR_ProtocolError;
                else
               if(strcmp("CR_LackOfActivity" , *text) == 0)
                        *peCloseReason = CR_LackOfActivity;
                else
               if(strcmp("CR_PeerAbort" , *text) == 0)
                        *peCloseReason = CR_PeerAbort;
                else
               if(strcmp("CR_Unspecified" , *text) == 0)
                        *peCloseReason = CR_Unspecified;
                else
               if(strcmp("CR_Invalid" , *text) == 0)
                        *peCloseReason = CR_Invalid;
                else
                        printf("Warning: DbvCloseReason string could not be converted: %s\n", *text);
        }
}


/* Functions to free allocated structures */
void free_DbvError(DbvError **ppsError)
{
        DbvError *psError;
 
        psError = *ppsError;
        if(psError != NULL)
        {
                if(psError->eErrorClass == SI_Abort)
                {
                        free(psError->uErrorCode.psAbort->pcDiagnostics);
                        free(psError->uErrorCode.psAbort);
                }
                free(psError);
                *ppsError = (DbvError *)NULL;
        }
}

