/*  Module Name: @(#)testy_calls.h	7.2
    Version: 2/27/97
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        DBV-OSI II Origin Test Tool utility functions header file.
 
*/

#ifndef TESTY_CALLS_H 
#define TESTY_CALLS_H 

typedef enum DBVRpnQueryType {
	RPN_Q1 = 1,
	RPN_Q2 = 2,
	RPN_Q3 = 3
} DBVRpnQueryType;

#include <stdio.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "sidefs.h"
#include "sitypes.h"
#include "sitypes2.h"
#include "dbvproto.h"
#include "iord_ext.h"
#include "itemordr.h"

#include "sl_list.h"
#include "memcheck.h"
#include "tbl.h"
#include "str_hand.h"
#include "batch.h"

#include "apifree.h"
#include "Free_DbvFuncs.h"
#include "config.h"

#include "external_processing.h"
#include "write_DbvFuncs.h"
#include "get_DbvFuncs.h"
#include "convert_DbvFuncs.h"

extern DbvBool bLocalResult;

#define SI_PREV 2
/* I don't want to fill in an optional PDU field */
#define NOTHING "NULL"
#define STRING_LEN  	512	
#define SZ_FILENAME STRING_LEN
#define SML_BUFFER	64

#define ORIGIN 1
#define TARGET 2

typedef enum TestType {
        Valid = 0,
        Invalid = 1
} TestType;

/* TBL file control functions (for open) */
int open_result_table(char *Filename, TBL *OutTable, char *OutFilename);
int open_tables(char *Filename, TestType eTestType, char *InFilename,
        char *OutFilename, TBL *InTable, TBL *OutTable);
int open_assoc_tables(char *Filename, TestType eTestType, char *InFilename,
        char *OutFilename, TBL *InTable, TBL *OutTable);
int open_utility_tables(char *Filename, TestType eTestType, char *InFilename,
        char *OutFilename, TBL *InTable, TBL *OutTable);

void get_alternative_filename(char *InFilename);
void get_return(void);

/* Common Association functions */
int do_AbortRequest(TestType eTestType);

/* Functions to test API functions */
/* Utility functions */
int do_SIGetEventLocation(TestType eTestType);
int do_SIWhichStack(TestType eTestType);
int do_SIGetEventLocation(TestType eTestType);
int do_SISetTraceParameter(TestType eTestType);

/* Functions to get and write from/to tbl files */
char *get_string_field(TBL Table, TBL_Return Return);
char *get_char_field(TBL Table, TBL_Return Return);
int get_int_field(TBL Table, TBL_Return Return);
int *get_integer_string(TBL InTable);
DbvBool get_DbvBool(TBL InTable);
void get_DbvAuthentication(DbvAuthentication **ppsAuthentication, TBL InTable);
void get_DbvOctetString(DbvOctetString **ppsOctetString, TBL InTable);
void get_DbvInternationalString(DbvInternationalString **ppsInternationString, 
	TBL InTable);
void get_compulsory_DbvOctetString(DbvOctetString *psOctetString, TBL InTable);
void get_compulsory_DbvInternationalString(DbvInternationalString 
        *psInternationalString, TBL InTable);
void get_DbvOtherInformation(DbvOtherInformation **ppsOtherInformation, 
	TBL InTable);


/* Write structures to tbl files */
void write_string_field(char *text, TBL Table, TBL_Return Return);
void write_comment_field(char *text, TBL Table, TBL_Return Return);
void write_int_field(int item, TBL Table, TBL_Return Return);
void write_title_int(char *title, int *item, TBL Table);
void write_char_field(char *item, TBL Table, TBL_Return Return);
void write_title_char(char *text, char *title, TBL Table);
void write_DbvBool(char *title, DbvBool fBool, TBL OutTable);
void write_DbvError(DbvError **ppsSIError, TBL Table);
void write_DbvOctetString(char *name, DbvOctetString *psOctetString, TBL OutTable);
void write_DbvInternationalString(char *name, DbvInternationalString
        *psInternationalString, TBL OutTable);
void write_DbvAuthentication(DbvAuthentication *psAuthentication, TBL OutTable);
void write_DbvOtherInformation(DbvOtherInformation *psOtherInformation, TBL OutTable);

/* Enumerated type conversion routines */
void convert_DbvAuthenticationType(char **text,
        DbvAuthenticationType *peAuthenticationType);
void convert_DbvAssociateRsp(char **text, DbvAssociateRsp *peAssociateRsp);
void convert_DbvAbortSource(char **text, DbvAbortSource *peAbortSource);
void convert_DbvApplicationProtocol(char **text,
        DbvApplicationProtocol *peApplicationProtocol);
void convert_DbvAssociateState( char **text, DbvAssociateState *peAssociateState);
void convert_DbvAbstractSyntax( char **text, DbvAbstractSyntax *peAbstractSyntax);
void convert_DbvCommonSyntaxes( char **text, char **ppcCommonSyntaxes);
void convert_DbvOsiOrInternetAddress(char **text,
        DbvOsiOrInternetAddress *peOsiOrInternetAddress);
void convert_DbvTargetNameOrAddress( char **text, DbvTargetNameOrAddress 
	*peTargetNameOrAddress);
void convert_DbvReleaseState(char **text, DbvReleaseState *peReleaseState);
void convert_DbvProtocolStack(char **text, DbvProtocolStack *peProtocolStack);
void convert_DbvBool(char **text, DbvBool *pfBool);
void convert_DbvExternalValueType(char **text, DbvExternalValueType *peValueType);
void convert_DbvASN1Type(char **text, DbvASN1Type *peASN1Type);
void convert_DbvHoldingsRecType(char **text, DbvHoldingsRecType *peHoldingsRecType);
void convert_DbvEstimateType(char **text, DbvEstimateType *peEstimateType);
void convert_DbvInformationType(char **text, DbvInformationType *peInformationType);
void convert_DbvDeleteFunction(char **text, 
	DbvDeleteFunction *peDeleteFunction);
void convert_DbvDeleteSetStatus(char **text, DbvDeleteSetStatus *peDeleteSetStatus );
void convert_DbvResourceReportStatus(char **text,
                         DbvResourceReportStatus *peResourceReportStatus );
void convert_DbvCloseReason(char **text, DbvCloseReason *peCloseReason);

void free_DbvError(DbvError **ppsError);

#endif

