/*  Module Name: @(#)write_DbvFuncs.c	7.4
    Version: 4/28/97
    Author: Dan Maskell/Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write structure types to the result script files.
        Used by the origin and target.
 
*/
 
#include "write_DbvFuncs.h"

extern TBL_Return T_Return;

void write_DbvAdditionalInfo(DbvAdditionalInfo *psAdditionalInfo, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psAdditionalInfo == NULL)
   {
           write_title_char("NULL","DbvAdditionalInfo", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psAdditionalInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Protocol Version In Force */
   convert_DbvProtocolVersionInForce( &pc_string, 
				   &psAdditionalInfo->eProtocolVersionInForce );
   sprintf( temp, "eProtocolVersionInForce : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Additional Info */

   switch( psAdditionalInfo->eProtocolVersionInForce )
      {
      case PVIF_Version2:
	 write_DbvVisibleString( "psV2 :", psAdditionalInfo->
				     uAdditionalInfo.pcV2, OutTable );
	 break;

      case PVIF_Version3:
	 write_DbvInternationalString( "psV3 :", psAdditionalInfo->
				uAdditionalInfo.psV3, OutTable );
	 break;

      default:
	 break;
      } /* End switch */
  } /* End write_DbvAdditionalInfo() */

void write_DbvDefaultDiagFormat(DbvDefaultDiagFormat *psDefaultDiagFormat, 
				TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psDefaultDiagFormat == NULL)
   {
           write_title_char("NULL","DbvDefaultDiagFormat", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psDefaultDiagFormat", "Structure", OutTable );
   TBL_PutRecord( OutTable );

   /* Object Identifier */

   write_DbvObjectIdentifier( "pcDiagnosticSetId:",
				psDefaultDiagFormat->pcDiagnosticSetId, 
				OutTable );

   /* Condition */

   sprintf( temp, "ulCondition : %ld", psDefaultDiagFormat->ulCondition );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Additional Info */ 

   write_DbvAdditionalInfo( &psDefaultDiagFormat->sAdditionalInfo, OutTable );
   } /* End write_DbvDefaultDiagFormat() */

void write_DbvDiagRec(DbvDiagRec *psDiagRec, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psDiagRec == NULL)
   {
           write_title_char("NULL","DbvDiagRec", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psDiagRec", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   if ( psDiagRec == NULL )
      {
      write_char_field( "psDiagRec : NULL", OutTable, T_Return );
      TBL_PutRecord( OutTable );
      return;
      } /* End if Null */
   /* Diagnostic Record type */
   convert_DbvDiagnosticsType( &pc_string, &psDiagRec->eDiagnosticsType );
   sprintf( temp, "eDiagnosticsType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Dbv Diag Record */

   switch( psDiagRec->eDiagnosticsType )
      {
      case DT_DefaultFormat:
	 write_DbvDefaultDiagFormat( psDiagRec->
				     uDiagnostics.psDefault, OutTable );
	 break;

      case DT_ExternallyDefined:
	 write_DbvASN1External( psDiagRec->
				uDiagnostics.psExternallyDefined, OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvDiagRec() */

void write_DbvElementSetNames( DbvElementSetNames *psElementSetNames, 
				TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;
   DbvElementSetsDbSpecific  **ppsElementSetsDbSpecific;

   /* Write the title out */ 

   if(psElementSetNames == NULL)
   {
	write_title_char("NULL","DbvElementSetNames", OutTable );
	return;
   }

   /* Element Set type */

   convert_DbvElementSetType( &pc_string, &psElementSetNames->eElementSetType );
   sprintf( temp, "eElementSetType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uElementSet */

   switch( psElementSetNames->eElementSetType )
      {
      case EST_Generic:
	 write_DbvInternationalString( "psGeneric",
			psElementSetNames->uElementSet.psGeneric,
			OutTable );
	 break;

      case EST_DatabaseSpecific:
         ppsElementSetsDbSpecific = psElementSetNames->
				   uElementSet.ppsElementSetsDbSpecific;

         while ( ppsElementSetsDbSpecific && *ppsElementSetsDbSpecific )
            {
            write_DbvInternationalString( "psDatabaseName:",
			(*ppsElementSetsDbSpecific)->psDatabaseName, OutTable );
            write_DbvInternationalString( "psElementSetName:",
			(*ppsElementSetsDbSpecific)->psElementSetName,OutTable);
            ppsElementSetsDbSpecific++;
            } /* End while */
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvElementSetNames() */

void write_DbvFragmentSyntax(DbvFragmentSyntax *psFragmentSyntax, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psFragmentSyntax == NULL)
   {
           write_title_char("NULL","DbvFragmentSyntax", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psFragmentSyntax", "Structure", OutTable);
   TBL_PutRecord(OutTable);

   /* Fragment syntax type */
   convert_DbvFragmentSyntaxType( &pc_string, 
				  &psFragmentSyntax->eFragmentSyntaxType );
   sprintf( temp, "eFragmentSyntaxType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uRecord */

   switch( psFragmentSyntax->eFragmentSyntaxType )
      {
      case FST_ExternallyTagged:
	 write_DbvASN1External( psFragmentSyntax->
				uFragmentSyntax.psExternallyTagged,
				OutTable );
	 break;

      case FST_NotExternallyTagged:
	 write_DbvOctetString(  "DbvOctetString:",
				psFragmentSyntax->
				uFragmentSyntax.psNotExternallyTagged,
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */
  } /* End write_DbvFragmentSyntax() */

void write_DbvNamePlusRecord(DbvNamePlusRecord *psNamePlusRecord, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psNamePlusRecord == NULL)
   {
           write_title_char("NULL","DbvNamePlusRecord", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psNamePlusRecord", "Structure", OutTable);
   TBL_PutRecord(OutTable);

   /* Write the database name */

   write_DbvInternationalString( "psDatabaseName", 
				 psNamePlusRecord->psDatabaseName,
				 OutTable );
   /* Write the record type */
   convert_DbvDbRecordType( &pc_string, &psNamePlusRecord->eDbRecordType );
   sprintf( temp, "eDbRecordType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uRecord */

   switch( psNamePlusRecord->eDbRecordType )
      {
      case DRT_Retrieval:
	 write_DbvASN1External( psNamePlusRecord->uDbRecord.psRetrievalRecord,
				OutTable );
	 break;

      case DRT_SurrogateDiagnostics:
	 write_DbvDiagRec( psNamePlusRecord->uDbRecord.psSurrogateDiagnostic,
				OutTable );
	 break;

      case DRT_StartingFragment:
	 write_DbvFragmentSyntax( psNamePlusRecord->
				uDbRecord.psStartingFragment,
				OutTable );
	 break;

      case DRT_IntermediateFragment:
	 write_DbvFragmentSyntax( psNamePlusRecord->
				uDbRecord.psIntermediateFragment,
				OutTable );
	 break;

      case DRT_FinalFragment:
	 write_DbvFragmentSyntax( psNamePlusRecord->uDbRecord.psFinalFragment,
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvNamePlusRecord() */

void write_DbvObjectIdentifier(char *name, DbvObjectIdentifier
        psObjectIdentifier, TBL OutTable)
   {

   char temp[ STRING_LEN ];

   if ( psObjectIdentifier != NULL )
      {
      sprintf(temp, "%s: >%s<", name, psObjectIdentifier );
      } /* End if */
   else
      sprintf(temp, "%s : NULL", name);

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvObjectIdentifier() */

void write_DbvQuery( DbvQuery *psQuery, TBL OutTable )
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psQuery == NULL)
   {
           write_title_char("NULL","DbvQuery", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psQuery", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Query type */

   convert_DbvQueryType( &pc_string, &psQuery->eQueryType );
   sprintf( temp, "eQueryType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvQuery */

   switch( psQuery->eQueryType )
      {
      case QT_Private:
	 write_DbvOctetString( "psPrivateQuery:" , 
				psQuery->uQuery.psPrivateQuery, 
				OutTable );
	 break;

      case QT_Rpn:
         write_DbvRPNQuery("Rpn:", psQuery->uQuery.psRpnQuery, OutTable);
	 break;

      case QT_Iso8777:
	 write_DbvOctetString( "psIso8777Query:" , 
				psQuery->uQuery.psIso8777Query, 
				OutTable );
	 break;

      case QT_Z3958:
	 write_DbvOctetString( "psZ3958Query:" , 
				psQuery->uQuery.psZ3958Query, 
				OutTable );
	 break;

      case QT_Erpn:
         write_DbvRPNQuery("Erpn:", psQuery->uQuery.psErpnQuery, OutTable);

	 break;

      case QT_NaturalLanguage:
	 write_DbvOctetString( "psNaturalLanguage:" , 
				psQuery->uQuery.psNaturalLanguageQuery, 
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvQuery() */
void write_DbvRecords(DbvRecords *psRecords, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;
   DbvNamePlusRecord **ppsResponseRecord;
   DbvDiagRec	     **ppsMultipleNonSurrogateDiag;

   /* Write the title out */ 

   if(psRecords == NULL)
   {
           write_title_char("NULL","DbvRecords", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psRecords", "Structure", OutTable);
   TBL_PutRecord(OutTable);

   /* Write the record type */
   convert_DbvRecordType( &pc_string, &psRecords->eRecordType );
   sprintf( temp, "eRecordType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );


   switch( psRecords->eRecordType )
      {
      case RT_Response:

         ppsResponseRecord = psRecords->uRecord.ppsResponseRecord;

         while ( ppsResponseRecord && *ppsResponseRecord )
            {
            write_DbvNamePlusRecord( *ppsResponseRecord, OutTable );
            ppsResponseRecord++;
            } /* End while */

	 break;

      case RT_NonSurrDiagnostics:
         write_DbvDefaultDiagFormat( psRecords->uRecord.psNonSurrogateDiag, 
				     OutTable );
	 break;

      case RT_MultipleNonSurrDiagnostics:

         ppsMultipleNonSurrogateDiag = psRecords->
					uRecord.ppsMultipleNonSurrogateDiag;

         while ( ppsMultipleNonSurrogateDiag && *ppsMultipleNonSurrogateDiag )
            {
            write_DbvDiagRec( *ppsMultipleNonSurrogateDiag, OutTable );
            ppsMultipleNonSurrogateDiag++;
            } /* End while */

	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvRecords() */
void write_DbvVisibleString(char *name, DbvVisibleString
        psVisibleString, TBL OutTable)
   {

   char temp[ STRING_LEN ];

   if ( psVisibleString != NULL )
      {
      sprintf(temp, "%s: >%s<", name, psVisibleString );
      } /* End if */
   else
      sprintf(temp, "%s : NULL", name);

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvVisibleString() */


void write_DbvSortKeySpec(DbvSortKeySpec *psSortKeySpec, TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psSortKeySpec == NULL)
   {
           write_title_char("NULL","DbvSortKeySpec", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psSortKeySpec", "Structure", OutTable);
   TBL_PutRecord(OutTable);

   /* Sort Element */

   write_comment_field( "START of sSortElement", OutTable, T_Return );
   write_DbvSortElement( &psSortKeySpec->sSortElement, OutTable );
   write_comment_field( "End of sSortElement", OutTable, T_Return );

   /* Sort Relation Type */

   convert_DbvSortRelationType( &pc_string, &psSortKeySpec->eSortRelationType );
   sprintf( temp, "eSortRelationType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );
   pc_string=NULL;
	
   /* Sort Case Type */

   convert_DbvSortCaseType( &pc_string, &psSortKeySpec->eSortCaseType );
   sprintf( temp, "eSortCaseType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Write the Missing Value Action */
 
   write_comment_field( "START of psMVActionType", OutTable, T_Return );
   write_DbvSortMVAction( psSortKeySpec->psMVActionType, OutTable );
   write_comment_field( "END of psMVActionType", OutTable, T_Return );

   } /* End write_DbvSortKeySpec() */

void write_DbvSortElement( DbvSortElement *psSortElement, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   DbvDbSpecificSortKey  	**ppsDatabaseSpecific;
   
   /* Write the title out */ 

   if(psSortElement == NULL)
   {
           write_title_char("NULL","DbvSortElement", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psSortElement", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* SortElement type */

   convert_DbvSortElementType( &pc_string, &psSortElement->eSortElementType );
   sprintf( temp, "eSortElementType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvSortElement */

   switch( psSortElement->eSortElementType )
      {
      case SET_Generic:
	 write_DbvSortKey( psSortElement->uSortElement.psGeneric, OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case SET_DbSpecific:

         ppsDatabaseSpecific = psSortElement->uSortElement.ppsDatabaseSpecific;

         while ( ppsDatabaseSpecific && *ppsDatabaseSpecific )
            {
            write_DbvDbSpecificSortKey( *ppsDatabaseSpecific, OutTable );
            ppsDatabaseSpecific++;
            } /* End while */

	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvSortElement() */

void write_DbvSortKey( DbvSortKey *psSortKey, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   DbvDbSpecificSortKey  	**ppsDatabaseSpecific;
   
   /* Write the title out */ 

   if(psSortKey == NULL)
   {
           write_title_char("NULL","DbvSortKey", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psSortKey", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* SortKey type */

   convert_DbvSortKeyType( &pc_string, &psSortKey->eSortKeyType );
   sprintf( temp, "eSortKeyType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvSortKey */

   switch( psSortKey->eSortKeyType )
      {
      case SKT_SortField:
	 write_DbvInternationalString( "psSortField",
					psSortKey->uSortElement.psSortField, 
					OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case SKT_ElementSpec:
	 write_DbvSpecification( psSortKey->uSortElement.psElementSpec, 
					OutTable );
	 break;

      case SKT_SortAttributes:
	 write_DbvSortAttributes( psSortKey->uSortElement.psSortAttributes, 
					OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvSortKey() */

void write_DbvSpecification( DbvSpecification *psSpecification, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   
   /* Write the title out */ 

   if(psSpecification == NULL)
   {
           write_title_char("NULL","DbvSpecification", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psSpecification", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSpecification", OutTable, T_Return ) ;


   /* Schema */
   write_DbvObjectIdentifier( "pcSchema:", psSpecification->pcSchema,
			       OutTable );
								  
   /* psElementSpec */
   write_DbvElementSpec( psSpecification->psElementSpec,
			       OutTable );

   write_comment_field( "END of DbvSpecification", OutTable, T_Return );
   } /* End write_DbvSpecification() */

void write_DbvElementSpec( DbvElementSpec *psElementSpec, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   
   /* Write the title out */ 

   if(psElementSpec == NULL)
   {
           write_title_char("NULL","DbvElementSpec", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psElementSpec", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementSpec", OutTable, T_Return ) ;

   convert_DbvElementSpecType( &pc_string, &psElementSpec->eElementSpecType );
   sprintf( temp, "eElementSpecType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvSpecification */

   switch( psElementSpec->eElementSpecType )
      {
      case EST_Primitive:
	 write_DbvInternationalString( "psPrimitive",
					psElementSpec->
					uElementSpec.psPrimitive, 
					OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case EST_External:
	 write_DbvASN1External( psElementSpec->uElementSpec.psExternal,
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   write_comment_field( "END of DbvElementSpec", OutTable, T_Return );
   } /* End write_DbvElementSpec() */

void write_DbvSortAttributes( DbvSortAttributes *psSortAttributes, 
			      TBL OutTable )
   {
   DbvAttributeElement   **ppsAttributeElement; 

   /* Write the title out */ 

   if(psSortAttributes == NULL)
   {
           write_title_char("NULL","DbvSortAttributes", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }
   write_title_char("psSortAttributes", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* AttributeSetId */
   write_DbvObjectIdentifier( "pcAttributeSetId:", 
			       psSortAttributes->pcAttributeSetId,
			       OutTable );

   write_comment_field( "START of psAttributeElement", OutTable, T_Return );
   ppsAttributeElement = psSortAttributes->ppsAttributeElement;

   while ( ppsAttributeElement && *ppsAttributeElement )
      {
      write_DbvAttributeElement( *ppsAttributeElement, OutTable );
      ppsAttributeElement++;
      } /* End while */

   write_comment_field( "END of psAttributeElement", OutTable, T_Return );
   } /* End write_DbvSortAttributes() */

void write_DbvAttributeElement( DbvAttributeElement *psAttributeElement, 
			      TBL OutTable )
   {
   char temp[ STRING_LEN ];

   DbvAttributeElement   **ppsAttributeElement; 

   /* Write the title out */ 

   if(psAttributeElement == NULL)
   {
           write_title_char("NULL","DbvAttributeElement", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psAttributeElement", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* AttributeSetId */
   write_DbvObjectIdentifier( "pcAttributeSetId:", 
			       psAttributeElement->pcAttributeSetId,
			       OutTable );

   /* Attribute Type */

   sprintf(temp, "ulAttributeType: %ld", 
			psAttributeElement->ulAttributeType);
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Attribute Value */

   write_DbvAttributeValue( &psAttributeElement->sAttributeValue, OutTable );

   } /* End write_DbvAttributeElement() */

void write_DbvAttributeValue( DbvAttributeValue *psAttributeValue, 
				TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;
   
   /* Write the title out */ 

   if(psAttributeValue == NULL)
   {
           write_title_char("NULL","DbvAttributeValue", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psAttributeValue", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Attribute Value type */

   convert_DbvAttributeValueType( &pc_string, 
				  &psAttributeValue->eAttributeValueType );
   sprintf( temp, "eAttributeValueType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvAttributeValue */

   switch( psAttributeValue->eAttributeValueType )
      {
      case AVT_Numeric:
         /* AttributeValue type */

         sprintf( temp, "eAttributeValue->uAttributeValue.ulNumeric : %ld",
		psAttributeValue->uAttributeValue.ulNumeric );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
	 break;

      case AVT_Complex:
	 write_DbvComplexValue( psAttributeValue->
				uAttributeValue.psComplexValue, 
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvAttributeValue() */

void write_DbvComplexValue( DbvComplexValue *psComplexValue, TBL OutTable )
   {
   char temp[ STRING_LEN ];

   DbvStringOrNumeric	**ppsList;
   Ulong		**ppulSemanticAction;

   /* Write the title out */ 

   if(psComplexValue == NULL)
   {
           write_title_char("NULL","DbvComplexValue", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psComplexValue", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* List of string or numerics */

   write_comment_field( "START of ppsList", OutTable, T_Return );
   ppsList = psComplexValue->ppsList;

   while ( ppsList && *ppsList )
      {
      write_DbvStringOrNumeric( *ppsList, OutTable );
      ppsList++;
      } /* End while */

   /* Semantic Actions */

   write_comment_field( "END of ppsList", OutTable, T_Return );

   write_comment_field( "START of ppulSemanticAction", OutTable, T_Return );
   ppulSemanticAction = psComplexValue->ppulSemanticAction;

   while ( ppulSemanticAction && *ppulSemanticAction )
      {
      sprintf(temp, "**ppulSemanticAction: %ld", **ppulSemanticAction );
      write_char_field( temp, OutTable, T_Return );
      TBL_PutRecord( OutTable );
      ppulSemanticAction++;
      } /* End while */

   write_comment_field( "END of ppulSemanticAction", OutTable, T_Return );

   } /* End write_DbvComplexValue() */

void write_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric, 
			       TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   /* StringOrNumeric type */
   if(psStringOrNumeric == NULL)
   {
           write_title_char("NULL","DbvStringOrNumeric", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   convert_DbvStringOrNumericType( &pc_string, 
				   &psStringOrNumeric->eStringOrNumeric );
   sprintf( temp, "eStringOrNumericType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   DbvFree( pc_string );
   TBL_PutRecord( OutTable );

   /* DbvStringOrNumeric */

   switch( psStringOrNumeric->eStringOrNumeric )
      {
      case SNT_String:
	 write_DbvInternationalString( "psString:" , 
				psStringOrNumeric->uStringOrNumeric.psString, 
				OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case SNT_Numeric:
         sprintf( temp, "pcNumeric: %ld", 
			psStringOrNumeric->uStringOrNumeric.ulNumeric );
         write_char_field( temp, OutTable, T_Return );
         TBL_PutRecord( OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End write_DbvStringOrNumeric() */
void write_DbvDbSpecificSortKey( DbvDbSpecificSortKey *psDbSpecificSortKey, 
			       TBL OutTable )
   {

   /* Write the title out */

   if(psDbSpecificSortKey == NULL)
   {
           write_title_char("NULL","DbvDbSpecificSortKey", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psDbSpecificSortKey", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Database Name */

   write_DbvInternationalString( "sDatabaseName:" , 
				&psDbSpecificSortKey->sDatabaseName, 
				OutTable );
   TBL_PutRecord( OutTable );

   /* Write the sort key */

   write_DbvSortKey( &psDbSpecificSortKey->sDbSort, OutTable );

   } /* End write_DbvDbSpecificSortKey() */

void write_DbvSortMVAction( DbvSortMVAction *psSortMVAction, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if ( psSortMVAction == NULL )
      {
      write_title_char("NULL","DbvSortMVAction", OutTable);
      TBL_PutRecord( OutTable );
      return;
      } /* End if */
   else
      write_title_char("psSortMVAction", "Structure", OutTable);

   TBL_PutRecord( OutTable );

   /* SortMVAction type */

   convert_DbvSortMVActionType( &pc_string, &psSortMVAction->eSortMVActionType);
   sprintf( temp, "eSortMVActionType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvSortMVAction */

   switch( psSortMVAction->eSortMVActionType )
      {
      case SMT_Abort:
	 break;

      case SMT_Null:
	 break;

      case SMT_Constant:
	 write_DbvOctetString( "psOctetString:", psSortMVAction->psOctetString,
				OutTable );
         TBL_PutRecord( OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvSortMVAction() */

void write_integer_string( char *name, int *pul, TBL OutTable)
   {
   char temp[ STRING_LEN ];
   if ( pul )
      sprintf(temp, "%s : %ld", name, *pul);
   else
      sprintf(temp, "%s : NULL", name );

   write_char_field( temp, OutTable, T_Return );

   } /* End write_integer_string() */


void write_DbvEntry( DbvEntry *psEntry, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psEntry == NULL)
   {
           write_title_char("NULL","DbvEntry", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psEntry", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Entry type */

   convert_DbvEntryType( &pc_string, &psEntry->eEntryType );
   sprintf( temp, "eEntryType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvEntry */

   switch( psEntry->eEntryType )
      {
      case ET_TermInfo:
         write_comment_field( "START of psTermInfo", OutTable, T_Return );
	 write_DbvTermInfo( psEntry->uEntry.psTermInfo, OutTable );
       	 write_comment_field( "END of psTermInfo", OutTable, T_Return );
	 break;

      case ET_SurrogateDiagnostic:

         write_comment_field( "START of psSurrogateDiagnostic",
							OutTable, T_Return );
	 write_DbvDiagRec( psEntry->uEntry.psSurrogateDiagnostic, OutTable );
       	 write_comment_field( "END of psSurrogateDiagnostic",
							OutTable, T_Return );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvEntry() */

void write_DbvListEntries( DbvListEntries *psListEntries, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   DbvEntry	**ppsEntry;
   DbvDiagRec 	**ppsNonSurrogateDiagnostics;

   /* Write the title out */ 
   if(psListEntries == NULL)
   {
           write_title_char("NULL","DbvListEntries", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psListEntries", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* ListEntries type */

#ifdef OLD
convert_DbvListEntriesType( &pc_string, &psListEntries->eListEntryType );
   sprintf( temp, "eListEntryType : %s", pc_string ); 
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );
   /* DbvListEntries */

   switch( psListEntries->eListEntryType )
      {
      case LET_Entries:
#endif
         write_comment_field( "START of ppsEntry", OutTable, T_Return );
	 ppsEntry = psListEntries->ppsEntry;

	 while ( ppsEntry && *ppsEntry )
	    {
	    write_DbvEntry( *ppsEntry, OutTable );
	    ppsEntry++;
	    } /* End while */

       	 write_comment_field( "END of ppsEntry", OutTable, T_Return );
#ifdef OLD
	 break;

      case LET_NonSurrogateDiagnostics:
#endif

         write_comment_field( "START of ppsNonSurrogateDiagnostics", 
					OutTable, T_Return );
	 ppsNonSurrogateDiagnostics = psListEntries->
				      ppsNonSurrogateDiagnostics;

	 while ( ppsNonSurrogateDiagnostics && *ppsNonSurrogateDiagnostics )
	    {
	    write_DbvDiagRec( *ppsNonSurrogateDiagnostics, OutTable );
	    ppsNonSurrogateDiagnostics++;
	    } /* End while */

       	 write_comment_field( "END of ppsNonSurrogateDiagnostics", 
					OutTable, T_Return );
#ifdef OLD
	 break;

      default:
	 break;
      } /* End switch */
#endif
   } /* End write_DbvListEntries() */

void write_DbvTermInfo( DbvTermInfo *psTermInfo, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   DbvAttributeElement   	**ppsSuggestedAttribute;
   DbvAttributesPlusTerm	**ppsAlternativeTerm;

   /* Write the title out */ 

   if(psTermInfo == NULL)
   {
           write_title_char("NULL","DbvTermInfo", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psTermInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Term */

   write_DbvTerm( &psTermInfo->sTerm, OutTable );

   write_DbvInternationalString( "psDisplayTerm :" , 
	psTermInfo->psDisplayTerm, OutTable );
   TBL_PutRecord( OutTable );

   /* AttributeElement */
   write_comment_field( "START of ppsSuggestedAttribute", OutTable, T_Return );

   ppsSuggestedAttribute = psTermInfo->ppsSuggestedAttribute;
   while ( ppsSuggestedAttribute && *ppsSuggestedAttribute )
      {
      write_DbvAttributeElement( *ppsSuggestedAttribute, OutTable );
      ppsSuggestedAttribute++;
      } /* End while */

   write_comment_field( "END of ppsSuggestedAttribute", OutTable, T_Return );

   /* AlternativeTerm */

   write_comment_field( "START of ppsAlternativeTerm", OutTable, T_Return );

   ppsAlternativeTerm = psTermInfo->ppsAlternativeTerm;
   while ( ppsAlternativeTerm && *ppsAlternativeTerm )
      {
      write_DbvAttributesPlusTerm( *ppsAlternativeTerm, OutTable );
      ppsAlternativeTerm++;
      } /* End while */

   write_comment_field( "END of ppsAlternativeTerm", OutTable, T_Return );

   /* GlobalOccurrences */

   write_integer_string( "pulGlobalOccurrences",
		          (int*)psTermInfo->pulGlobalOccurrences,
			  OutTable );
   TBL_PutRecord( OutTable );

   /* ByAttributes */

   writeN_DbvOccurrenceByAttributes( psTermInfo->ppsByAttributes, OutTable );

   /* OtherInformation */
   if(psTermInfo->ppsOtherInformation==NULL)
   {
     write_char_field("psTermInfo->ppsOtherInformation: NULL", OutTable, T_Return);
     TBL_PutRecord( OutTable ); 
  }
   else
   writeN_DbvOtherInformation(psTermInfo->ppsOtherInformation, OutTable);



   } /* End write_DbvTermInfo() */

void write_DbvTerm( DbvTerm *psTerm, TBL OutTable )
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

   /* Write the title out */ 

   if(psTerm == NULL)
   {
           write_title_char("NULL","DbvTerm", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psTerm", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Term type */

   convert_DbvTermType( &pc_string, &psTerm->eTermType );
   sprintf( temp, "eTermType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvTerm */

   switch( psTerm->eTermType )
      {
      case TT_General:
	 write_DbvOctetString( "psGeneral :" , psTerm->uTerm.psGeneral, 
				OutTable );
	 break;

      case TT_Numeric:
	 sprintf( temp, "ulInteger : %ld", psTerm->uTerm.ulInteger ); 
         write_char_field(temp, OutTable, T_Return );
         TBL_PutRecord( OutTable );
	 break;

      case TT_SimpleCharacter:
	 write_DbvInternationalString( "psSimpleCharacter :" , 
				psTerm->uTerm.psSimpleCharacter, 
				OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case TT_ObjectIdentifier:
	 write_DbvObjectIdentifier( "pcObjectIdentifier :", 
				psTerm->uTerm.pcObjectIdentifier, 
				OutTable );
	 break;

      case TT_DateTime:
	 write_DbvGeneralizedTime( "pcDateTime", 
				   psTerm->uTerm.pcDateTime, 
				   OutTable );
	 break;

      case TT_External:
	 write_DbvASN1External( psTerm->uTerm.psExternal, OutTable );
	 break;

      case TT_IntegerAndUnit:
	 write_DbvIntUnit( psTerm->uTerm.psIntegerAndUnit, OutTable );
	 break;

      case TT_Null:
	 break;

      default:
	 break;
      } /* End switch */

   } /* End write_DbvTerm() */

void write_DbvGeneralizedTime(char *name, DbvGeneralizedTime
        psGeneralizedTime, TBL OutTable)
   {

   char temp[ STRING_LEN ];

   if ( psGeneralizedTime != NULL )
      {
      sprintf(temp, "%s: >%s<", name, psGeneralizedTime );
      } /* End if */
   else
      sprintf(temp, "%s : NULL", name);

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvGeneralizedTime() */

void write_DbvIntUnit( DbvIntUnit *psIntUnit, TBL OutTable)
   {

   char temp[ STRING_LEN ];

   /* Write the title out */

   if(psIntUnit == NULL)
   {
           write_title_char("NULL","DbvIntUnit", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psIntUnit :", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Value */

   sprintf( temp, "ulValue: %ld", psIntUnit->ulValue );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* Unit */

   write_DbvUnit( &psIntUnit->sUnitUsed, OutTable );

   } /* End write_DbvIntUnit() */

void write_DbvUnit( DbvUnit *psUnit, TBL OutTable )
   {

   char temp[ STRING_LEN ];

   /* Write the title out */

   if(psUnit == NULL)
   {
           write_title_char("NULL","DbvUnit", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psUnit :", "Structure", OutTable );
   TBL_PutRecord( OutTable );

   /* Unit System */

   write_DbvInternationalString( "psUnitSystem", psUnit->psUnitSystem,
				  OutTable );

   TBL_PutRecord( OutTable );

   /* Unit Type */

   if ( !psUnit->psUnitType )
      {
      sprintf( temp, "psUnitType : NULL" );
      write_char_field( temp, OutTable, T_Return );
      TBL_PutRecord( OutTable );
      } /* End if */
   else
      write_DbvStringOrNumeric( psUnit->psUnitType, OutTable );

   /* Unit */

   if ( !psUnit->psUnit )
      {
      sprintf( temp, "psUnit : NULL" );
      write_char_field( temp, OutTable, T_Return );
      TBL_PutRecord( OutTable );
      } /* End if */
   else
      write_DbvStringOrNumeric( psUnit->psUnit, OutTable );

   /* Scale Factor */

   write_integer_string( "pulScaleFactor", (int *)psUnit->pulScaleFactor, 
	OutTable );
   TBL_PutRecord( OutTable );

   } /* End write_DbvUnit() */

void write_DbvAttributesPlusTerm( DbvAttributesPlusTerm *psAttributesPlusTerm,
				  TBL OutTable )
   {
   char temp[ STRING_LEN ];

   DbvAttributeElement   	**ppsAttributeElement;

   /* Write the title out */ 

   if(psAttributesPlusTerm == NULL)
   {
           write_title_char("NULL","DbvAttributesPlusTerm", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_comment_field( "START of psAttributesPlusTerm", OutTable, T_Return );
   TBL_PutRecord( OutTable );

   /* AttributeElement */

   write_comment_field( "START of ppsAttributeElement", OutTable, T_Return );

   ppsAttributeElement = psAttributesPlusTerm->ppsAttributeElement;
   while ( ppsAttributeElement && *ppsAttributeElement )
      {
      write_DbvAttributeElement( *ppsAttributeElement, OutTable );
      ppsAttributeElement++;
      } /* End while */

   write_comment_field( "END of ppsAttributeElement", OutTable, T_Return );

   /* Term */

   write_DbvTerm( &psAttributesPlusTerm->sTerm, OutTable );

   write_comment_field( "END of psAttributesPlusTerm", OutTable, T_Return );
   } /* End write_DbvAttributesPlusTerm() */

void write_DbvOccurrenceByAttributes( DbvOccurrenceByAttributes 
	*psOccurrenceByAttributes, TBL OutTable )
   {
   DbvAttributeElement   	**ppsAttributeElement;

   /* Write the title out */ 

   if(psOccurrenceByAttributes == NULL)
   {
           write_title_char("NULL","DbvOccurrenceByAttributes", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psOccurrenceByAttributes", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* AttributeElement */

   write_comment_field( "START of ppsAttributeElement", OutTable, T_Return );

   ppsAttributeElement = psOccurrenceByAttributes->ppsAttributeElement;
   while ( ppsAttributeElement && *ppsAttributeElement )
      {
      write_DbvAttributeElement( *ppsAttributeElement, OutTable );
      ppsAttributeElement++;
      } /* End while */

   write_comment_field( "END of ppsAttributeElement", OutTable, T_Return );

   /* Occurance */

   write_DbvOccurrence( psOccurrenceByAttributes->psOccurrence, OutTable );

   /* Other Information */
   if(psOccurrenceByAttributes->ppsOtherInformation==NULL)
   {
     write_char_field("psOccurrenceByAttributes->ppsOtherInformation: NULL", OutTable, T_Return);
     TBL_PutRecord( OutTable );  
   }
   else
   writeN_DbvOtherInformation( psOccurrenceByAttributes->ppsOtherInformation, 
				OutTable );

   } /* End write_DbvOccurrenceByAttributes() */

void write_DbvOccurrence( DbvOccurrence *psOccurrence, TBL OutTable )
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
   DbvOccurrenceByDb	**ppsOccurrenceByDb;

   /* Write the title out */ 

   if(psOccurrence == NULL)
   {
           write_title_char("NULL","DbvOccurrence", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char("psOccurrence", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* Occurence */

   convert_DbvOccurrenceType( &pc_string, 
				   &psOccurrence->eOccurrenceType );
   sprintf( temp, "eOccurrenceType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   switch( psOccurrence->eOccurrenceType )
      {
      case OT_Global:
         sprintf( temp, "ulGlobal: %ld", psOccurrence->uOccurence.ulGlobal );
	 write_char_field( temp, OutTable, T_Return );
	 TBL_PutRecord( OutTable );
	 break;

      case OT_ByDatabase:
         write_comment_field( "START of ppsOccurrenceByDb", 
						OutTable, T_Return );
	 ppsOccurrenceByDb = psOccurrence->uOccurence.ppsOccurrenceByDb;

	 while ( ppsOccurrenceByDb && *ppsOccurrenceByDb )
	    {
	    write_DbvOccurrenceByDb( *ppsOccurrenceByDb, OutTable );
	    ppsOccurrenceByDb++;
	    } /* End while */

       	 write_comment_field( "END of ppsOccurrenceByDb", OutTable, T_Return );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End write_DbvOccurrence() */

void write_DbvOccurrenceByDb( DbvOccurrenceByDb *psOccurrenceByDb, TBL OutTable)
   {

   /* Write the title out */ 

   if(psOccurrenceByDb == NULL)
   {
           write_title_char("NULL","DbvOccurrenceByDb", OutTable);
           TBL_PutRecord(OutTable);
           return;
   }

   write_title_char( "psOccurrenceByDb", "Structure", OutTable );
   TBL_PutRecord( OutTable );

   /* DatabaseName */

   write_DbvInternationalString( "sDatabaseName",
				 &psOccurrenceByDb->sDatabaseName,
				 OutTable );
   TBL_PutRecord( OutTable );

   /* Number */

   write_integer_string( "pulNumber", (int *)psOccurrenceByDb->pulNumber, 
	OutTable );

   /* OtherInformation */
   if(psOccurrenceByDb->ppsOtherInformation==NULL)
   {
     write_char_field("psOccurrenceByDb->ppsOtherInformation: NULL", OutTable, T_Return);
     TBL_PutRecord( OutTable );
   }
   else
   writeN_DbvOtherInformation( psOccurrenceByDb->ppsOtherInformation , OutTable );
   
   } /* End write_DbvOccurrenceByDb */

void write_DbvEsTaskPackage(DbvEsTaskPackage *psEsTaskPackage, 
					TBL OutTable)
   {

   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvEsTaskPackage Record", OutTable, T_Return);
   if( psEsTaskPackage != NULL)
      {

      /* pcPackageType */

      write_DbvObjectIdentifier(  "pcPackageType:", 
				   psEsTaskPackage->pcPackageType, 
				   OutTable );

      /* psPackageName */

      write_DbvInternationalString( "psPackageName", 
				 psEsTaskPackage->psPackageName, 
				 OutTable );

      /* psUserId */

      write_DbvInternationalString( "psUserId", 
				 psEsTaskPackage->psUserId, 
				 OutTable );

      /* psRetentionTime */

      write_DbvIntUnit( psEsTaskPackage->psRetentionTime, OutTable );

      /* ppsPermissions */

      writeN_DbvPermissions( psEsTaskPackage->ppsPermissions, OutTable );

      /* psDescription */

      write_DbvInternationalString( "psDescription", 
				 psEsTaskPackage->psDescription, 
				 OutTable );

      /* psTargetReference */

      write_DbvOctetString(  "psTargetReference:", 
			     psEsTaskPackage->psTargetReference, 
			     OutTable );

      /* pcCreationDateTime */

      write_DbvGeneralizedTime( "pcCreationDateTime", 
		psEsTaskPackage->pcCreationDateTime, OutTable );

      /* eExtServicesTaskStatus */

      convert_DbvExtServicesTaskStatus( &pc_string, 
					&psEsTaskPackage->
					eExtServicesTaskStatus );

      sprintf( temp, "eExtServicesTaskStatus: %s", pc_string );
      write_char_field(temp, OutTable, T_Return);
      TBL_PutRecord( OutTable );
      DbvFree( pc_string );

      /* ppsPackageDiagnostics */

       writeN_DbvDiagRec( psEsTaskPackage->ppsPackageDiagnostics, OutTable );

      /* sTaskSpecificParameters */

      write_DbvASN1External( &psEsTaskPackage->sTaskSpecificParameters, 
				OutTable);
      } /* End if */

   write_comment_field("END of DbvEsTaskPackage Record", OutTable, T_Return);

   } /* End  write_DbvEsTaskPackage() */

void write_DbvEstimate2(DbvEstimate2 *psEstimate2, 
					TBL OutTable)
   {
   write_comment_field("START of DbvEstimate2", OutTable, T_Return);
   if( psEstimate2 != NULL)
      {
      write_DbvStringOrNumeric( &psEstimate2->sType, OutTable);
      write_DbvIntUnit( &psEstimate2->sValue, OutTable);
      } /* End if */

   write_comment_field("END of DbvEstimate2", OutTable, T_Return);

   } /* End  write_DbvEstimate2() */

void write_DbvPermAllowableFunctions(DbvPermAllowableFunctions 
				*pePermAllowableFunctions, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   if( pePermAllowableFunctions != NULL)
      {
      convert_DbvPermAllowableFunctions( &pc_string, pePermAllowableFunctions );
      sprintf( temp, "ePermAllowableFunctions : %s", pc_string );

      write_char_field(temp, OutTable, T_Return);
      TBL_PutRecord( OutTable );

      DbvFree( pc_string );
      } /* End if */
      else
        write_title_char("NULL","DbvPermAllowableFunctions", OutTable);


   } /* End  write_DbvPermAllowableFunctions() */

void write_DbvPermissions(DbvPermissions *psPermissions, TBL OutTable)
   {

   write_comment_field("START of DbvPermissions Record", OutTable, T_Return );
   if( psPermissions != NULL)
      {
      write_DbvInternationalString( "sUserId", &psPermissions->sUserId, 
				OutTable);
      writeN_DbvPermAllowableFunctions( psPermissions->ppeAllowableFunctions, 
					OutTable);
      } /* End if */

   write_comment_field("END of DbvPermissions Record", OutTable, T_Return);

   } /* End  write_DbvPermissions() */

void write_DbvResourceReport2(DbvResourceReport2 *psResourceReport2, 
					TBL OutTable)
   {

   write_comment_field("START of DbvResourceReport2 Record", 
						OutTable, T_Return );

   if(psResourceReport2 != NULL)
   {
   if(psResourceReport2->ppsEstimate != NULL)
      writeN_DbvEstimate2( psResourceReport2->ppsEstimate, OutTable);
   else
      {
      write_title_char("NULL", "ppsEstimate", OutTable);
      TBL_PutRecord(OutTable);
      } /* End else */

   write_DbvInternationalString( "psMessage", psResourceReport2->psMessage, 
				OutTable);

   } /* End if */
   TBL_PutRecord(OutTable);

   write_comment_field("END of DbvResourceReport2 Record", OutTable, T_Return );

   } /* End  write_DbvResourceReport2() */

void write_DbvUpdateExtendedService(DbvUpdateExtendedService 
					*psUpdateExtendedService, 
					TBL OutTable)
   {
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvUpdateExtendedService Record", 
				OutTable, T_Return );

   if(psUpdateExtendedService != NULL)
   {
   convert_DbvUpdateReqOrRsp( &pc_string, 
				 &psUpdateExtendedService->eUpdateReqRsp );
   sprintf( temp, "eUpdateReqRsp : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* DbvUpdateReqOrRsp */

   switch ( psUpdateExtendedService->eUpdateReqRsp )
      {
      case UROR_Request:
	 write_DbvUpdateRequestPackage( psUpdateExtendedService->
					uUpdateReqOrRsp.psUpdateRequest, 
					OutTable );
	 break;

      case UROR_Response:
	 write_DbvUpdateTaskPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateResponse, 
				     OutTable );
	 break;

      default:
	 break;
      } /* End switch */
   } /* End if */

   write_comment_field("END of DbvUpdateExtendedService Record", 
				OutTable, T_Return );

   } /* End  write_DbvUpdateExtendedService() */

void write_DbvUpdateRequestPackage( DbvUpdateRequestPackage *psUpdateRequest,
        TBL OutTable )
{
   write_comment_field("START of DbvUpdateRequestPackage", OutTable, T_Return );

   write_DbvOriginPartToKeep(&psUpdateRequest->sOriginPartToKeep, OutTable);
   writeN_DbvSuppliedRecord(psUpdateRequest->ppsSuppliedRecord, OutTable);

   write_comment_field("END of DbvUpdateRequestPackage", OutTable, T_Return );
}

void write_DbvOriginPartToKeep( DbvOriginPartToKeep *psOriginPartToKeep,
	TBL OutTable)
{
	
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvOriginPartToKeep Record",OutTable,T_Return);

   convert_DbvUpdtAction( &pc_string, 
				 &psOriginPartToKeep->eAction );
   sprintf( temp, "eAction : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_DbvInternationalString( "sDatabaseName", 
	&psOriginPartToKeep->sDatabaseName, OutTable);

   write_DbvObjectIdentifier("pcSchema:", psOriginPartToKeep->pcSchema, 
	OutTable);

   write_DbvInternationalString( "psElementSetName", 
	psOriginPartToKeep->psElementSetName, OutTable);

   write_comment_field("END of DbvOriginPartToKeep Record",OutTable,T_Return);
}

void write_DbvSuppliedRecord(DbvSuppliedRecord *psSuppliedRecord, TBL OutTable)
{
   write_comment_field("START of DbvSuppliedRecord Record",OutTable,T_Return);

   if(psSuppliedRecord != NULL)
   {
   write_DbvUpdtRecordId(psSuppliedRecord->psUpdtRecordId, OutTable);
   write_DbvUpdtSupplementalId(psSuppliedRecord->psUpdtSupplementalId, OutTable);
   write_DbvCorrelationInfo(psSuppliedRecord->psCorrelationInfo, OutTable);
   write_DbvASN1External( &psSuppliedRecord->sRecord, OutTable);

   }
   else
	write_title_char("NULL", "DbvSuppliedRecord", OutTable);

   write_comment_field("END of DbvSuppliedRecord Record",OutTable,T_Return);
}

void write_DbvUpdtRecordId(DbvUpdtRecordId *psUpdtRecordId, TBL OutTable)
{
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvUpdtRecordId Record",OutTable,T_Return);
   
   if(psUpdtRecordId != NULL)
   {
   convert_DbvUpdtRecordIdType(&pc_string, &psUpdtRecordId->eRecordIdType);
   sprintf( temp, "eRecordIdType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   switch( psUpdtRecordId->eRecordIdType )
   {
   case URIT_Number:
	write_integer_string("uUpdtRecordId.ulNumber", (int *)&psUpdtRecordId->
		uUpdtRecordId.ulNumber, OutTable);
	break;
   case URIT_String:
   	write_DbvInternationalString( "uUpdtRecordId.psString", 
		psUpdtRecordId->uUpdtRecordId.psString, OutTable);
	break;
   case URIT_Opaque:
   	write_DbvOctetString( "uUpdtRecordId.psOpaque", 
		psUpdtRecordId->uUpdtRecordId.psOpaque, OutTable);
	break;

   }
   }
   else
	write_title_char("NULL", "DbvUpdtRecordId", OutTable);

   write_comment_field("END of DbvUpdtRecordId Record",OutTable,T_Return);
}

void write_DbvUpdtSupplementalId(DbvUpdtSupplementalId *psUpdtSupplementalId,
	TBL OutTable)
{
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvUpdtSupplementalId Record",OutTable,T_Return);

   if(psUpdtSupplementalId != NULL)
   {
   convert_DbvUpdtSupplementalIdType(&pc_string, &psUpdtSupplementalId->
	eSupplementalIdType);
   sprintf( temp, "eSupplementalIdType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   switch( psUpdtSupplementalId->eSupplementalIdType )
   {
   case USIT_Time:
	write_DbvGeneralizedTime("uUpdtSupplementalId.pcTime", 
		psUpdtSupplementalId->uUpdtSupplementalId.pcTime, OutTable);
	break;
   case USIT_Version:
	write_DbvInternationalString("uUpdtSupplementalId.psVersion", 
		psUpdtSupplementalId->uUpdtSupplementalId.psVersion, OutTable);
	break;
   case USIT_PreviousVersion:
	write_DbvASN1External(psUpdtSupplementalId->uUpdtSupplementalId.
		psPreviousVersion, OutTable);
	break;
   }
   }
   else
	write_title_char("NULL", "DbvUpdtSupplementalId", OutTable);
	
   write_comment_field("END of DbvUpdtSupplementalId Record",OutTable,T_Return);
}

void write_DbvCorrelationInfo(DbvCorrelationInfo *psCorrelationInfo, 
	TBL OutTable)
{
   write_comment_field("START of DbvCorrelationInfo Record",OutTable,T_Return);
   
   if(psCorrelationInfo != NULL)
   {
   write_DbvInternationalString("psNote", psCorrelationInfo->psNote, OutTable);
   write_integer_string("pulId", (int *)psCorrelationInfo->pulId, OutTable);
   TBL_PutRecord( OutTable );
   }
   else
	write_title_char("NULL", "DbvCorrelationInfo", OutTable);
   
   write_comment_field("END of DbvCorrelationInfo Record",OutTable,T_Return);
}

void write_DbvUpdateTaskPackage( DbvUpdateTaskPackage *psUpdateResponse, 
     TBL OutTable ) 
{
   write_comment_field("START of DbvUpdateTaskPackage", OutTable, T_Return );
   
   if(psUpdateResponse != NULL)
   {
   write_DbvOriginPartToKeep(&psUpdateResponse->sOriginPartToKeep, OutTable);
   write_DbvUpdateTargetPart(&psUpdateResponse->sTargetPart, OutTable);
   }
   else
	write_title_char("NULL", "DbvUpdateTaskPackage", OutTable);

   write_comment_field("END of DbvUpdateTaskPackage", OutTable, T_Return );
}

void write_DbvUpdateTargetPart( DbvUpdateTargetPart *psTargetPart, 
	TBL OutTable)
{
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvUpdateTargetPart Record",OutTable,T_Return);

   if(psTargetPart != NULL)
   {
   convert_DbvUpdtTaskStatus(&pc_string, &psTargetPart->eTaskStatus);
   sprintf( temp, "eTaskStatus : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   writeN_DbvDiagRec(psTargetPart->ppsGlobalDiagnostics, OutTable);
   writeN_DbvTaskPackageRecord(psTargetPart->ppsTaskPackageRecord, OutTable);
   }
   else
	write_title_char("NULL", "DbvUpdateTargetPart", OutTable);

   write_comment_field("END of DbvUpdateTargetPart Record",OutTable,T_Return);
}

void write_DbvTaskPackageRecord(DbvTaskPackageRecord *psTaskPackageRecord, 
	TBL OutTable)
{
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvTaskPackage Record",OutTable,
	T_Return);

   if(psTaskPackageRecord != NULL)
   {
   write_DbvUpdtRecordOrDiag(psTaskPackageRecord->psUpdtRecordOrDiag, OutTable);
   write_DbvCorrelationInfo(psTaskPackageRecord->psCorrelationInfo, OutTable);

   convert_DbvUpdtRecordStatus(&pc_string, &psTaskPackageRecord->
	eUpdtRecordStatus);
   sprintf( temp, "eUpdtRecordStatus : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );
   }
   else
	write_title_char("NULL", "DbvUpdateTargetPart", OutTable);

   write_comment_field("END of DbvTaskPackageRecord Record",OutTable,T_Return);
}

void write_DbvUpdtRecordOrDiag(DbvUpdtRecordOrDiag *psUpdtRecordOrDiag,
	TBL OutTable)
{
   char temp[ SML_BUFFER ];
   char *pc_string = NULL;

   write_comment_field("START of DbvUpdtRecordOrDiag Record",OutTable,
	T_Return);
   
   if(psUpdtRecordOrDiag != NULL)
   {
   convert_DbvUpdtRecordOrDiagType(&pc_string, &psUpdtRecordOrDiag->
	eUpdtRecordOrDiag);
   sprintf( temp, "eUpdtRecordOrDiag : %s", pc_string );
   write_char_field(temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   switch( psUpdtRecordOrDiag->eUpdtRecordOrDiag )
   {
   case URDT_Record:
	write_DbvASN1External(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord,
		OutTable);
	break;
   case URDT_Diagnostic:
	write_DbvDiagRec(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic,
		OutTable);
	break;
   }
   }
   else
	write_title_char("NULL", "DbvUpdtRecordOrDiag", OutTable);

   write_comment_field("End of DbvUpdtRecordOrDiag Record",OutTable,
	T_Return);
}

void write_DbvRange(DbvRange *psAdditionalRanges, TBL OutTable)
   {

   write_comment_field("START of DbvRange", OutTable, T_Return );
   if( psAdditionalRanges != NULL)
      {
	write_title_int( "ulStartingPosition",(int *)&(psAdditionalRanges->ulStartingPosition), OutTable);
	TBL_PutRecord(OutTable);
	write_title_int( "ulNumberOfRecords", (int *)&(psAdditionalRanges->ulNumberOfRecords), OutTable);
	TBL_PutRecord(OutTable);
 
      } /* End if */

   write_comment_field("END of DbvRange ", OutTable, T_Return);

   } /* End  write_DbvRange() */

void write_DbvRecordComposition(DbvRecordComposition *psRecordComposition, TBL OutTable)
   {
	char *pcRecordCompositionType = NULL;

   write_comment_field("START of DbvRecordComposition ", OutTable, T_Return );
   if( psRecordComposition != NULL)
      {
/* eRecordCompositionType */
        convert_DbvRecordCompositionType(&pcRecordCompositionType,&(psRecordComposition->eRecordCompositionType));
	write_title_char(pcRecordCompositionType, "eRecordCompositionType", OutTable);
	DbvFree(pcRecordCompositionType);
	TBL_PutRecord(OutTable);
  	switch ( psRecordComposition->eRecordCompositionType )
      	{
      	case RCT_ElementSetNames:
		write_DbvElementSetNames(psRecordComposition->
			uRecordComposition.psElementSetNames, OutTable);
		break;
	case RCT_CompSpec:
		write_DbvCompSpec(psRecordComposition->uRecordComposition.
			psCompSpec, OutTable);
		break;
	} /* End case */
      } /* End if */

   write_comment_field("END of DbvRecordComposition", OutTable, T_Return);

   } /* End  write_DbvRecordComposition() */

void write_DbvCompSpec(DbvCompSpec *psCompSpec, TBL OutTable)
{
   char *stringfield = NULL;
   int intfield;
   DbvDbSpecificComp **ppsDbSpecific;
   DbvObjectIdentifier *ppcRecordSyntax;
   write_comment_field("START of DbvCompSpec ", OutTable, T_Return );
   if( psCompSpec != NULL)
   {
	write_DbvBool("fSelectAlternativeSyntax", psCompSpec->
		fSelectAlternativeSyntax, OutTable);
	TBL_PutRecord(OutTable);
	write_DbvSpecification(psCompSpec->psGeneric, OutTable);
	TBL_PutRecord(OutTable);
         ppsDbSpecific = psCompSpec->ppsDbSpecific;
         while (*ppsDbSpecific )
            {
            write_DbvDbSpecificComp( *ppsDbSpecific, OutTable );
            ppsDbSpecific++;
            } /* End while */
	TBL_PutRecord(OutTable);
         ppcRecordSyntax = psCompSpec->ppcRecordSyntax; 
        while (*ppcRecordSyntax)
        {
       write_DbvObjectIdentifier("pcRecordSyntax:", 
		*ppcRecordSyntax, OutTable);
       ppcRecordSyntax++;	
       TBL_PutRecord(OutTable);	
         }  
}
   else
	write_title_char("NULL", "DbvCompSpec", OutTable);

}


void write_DbvDbSpecificComp(DbvDbSpecificComp *psDbSpecific, TBL OutTable)

   {
   write_comment_field("START of DbvDbSpecificComp", OutTable, T_Return );
   if( psDbSpecific != NULL)
      {
        write_DbvInternationalString( "sDatabaseName", &psDbSpecific->sDatabaseName,OutTable);
	TBL_PutRecord(OutTable);
	write_DbvSpecification(&psDbSpecific->sSpecification, OutTable);
	TBL_PutRecord(OutTable);

      } /* End if */

   write_comment_field("END of DbvDbSpecificComp", OutTable, T_Return);

   } /* End  write_DbvDbSpecificComp() */

void write_DbvRPNQuery(char *query_type, DbvRPNQuery *psDbvRPNQuery, TBL OutTable)
{
	write_DbvObjectIdentifier("pcAttributeSetId", psDbvRPNQuery->pcAttributeSetId, OutTable);
 
        write_char_field(query_type, OutTable, T_Return);
        TBL_PutRecord(OutTable);

        write_DbvRPNStructure(&psDbvRPNQuery->sRpnStructure, OutTable);
}

void write_DbvRPNStructure(DbvRPNStructure *psDbvRPNStructure, TBL OutTable)
{
 
        switch (psDbvRPNStructure->eRpnStructureType) {
        case RST_RpnLopRopOperator:
 
                write_char_field("RST_RpnLopRopOperator: ", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_DbvRPNLopRopOperator(psDbvRPNStructure->uRPNStructure.
			psRpnLopRopOperator, OutTable);
                break;
        case RST_RpnOperand:
                write_char_field("RST_RpnOperand: ", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_DbvOperand(psDbvRPNStructure->uRPNStructure.psRpnOperand,
			OutTable);
                break;
        }
}

void write_DbvRPNLopRopOperator(DbvRPNLopRopOperator *psDbvRPNLopRopOperator, 
	TBL OutTable)
{
 
        if (psDbvRPNLopRopOperator->psLeftOperand != NULL)
                write_DbvRPNStructure(psDbvRPNLopRopOperator->psLeftOperand, 
			OutTable);
 
        if (psDbvRPNLopRopOperator->psRightOperand != NULL)
                write_DbvRPNStructure(psDbvRPNLopRopOperator->psRightOperand, 
			OutTable);
 
        write_DbvOperator(psDbvRPNLopRopOperator->sOperator, OutTable);
}

void write_DbvOperator(DbvOperator sOperator, TBL OutTable)
{
        write_char_field("sOperator: ", OutTable, T_Return);
        TBL_PutRecord(OutTable);
 
        switch (sOperator.eOperatorType) {
        case OT_Or:
                write_char_field("OT_Or", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                break;
        case OT_And:
                write_char_field("OT_And", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                break;
        case OT_AndNot:
                write_char_field("OT_AndNot", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                break;
        case OT_Proximity:
                write_char_field("OT_Proximity", OutTable, T_Return);
                TBL_PutRecord(OutTable);
                write_DbvProximityOperator(sOperator.psProximityOperator,
                        OutTable);
                break;
        default:
                printf("Unknown eOperatorType type");
                break;
        }
}

void write_DbvOperand(DbvOperand *psRpnOperand, TBL OutTable)
{
 
        switch (psRpnOperand->eOperandType) {
        case OT_AttributePlusTerm:
                write_DbvAttributesPlusTerm(psRpnOperand->uOperand.
			psAttributesPlusTerm, OutTable);
                break;
        }
 
}

void write_DbvProximityOperator(DbvProximityOperator *psProximityOperator,
        TBL OutTable)
{
	char *stringfield = NULL;
	char temp[ SML_BUFFER ];

   	write_comment_field("START of DbvProximityOperator ", OutTable, 
		T_Return);

        if(psProximityOperator != NULL)
	{
		if(psProximityOperator->pfExclusion != NULL)
			write_DbvBool("pfExclusion", *psProximityOperator->
				pfExclusion, OutTable);
		else
           		write_title_char("NULL","pfExclusion", OutTable);
		TBL_PutRecord( OutTable );
		write_integer_string("ulDistance", (int *)&psProximityOperator->
			ulDistance, OutTable);
		TBL_PutRecord( OutTable );
		write_DbvBool("fOrdered", psProximityOperator->fOrdered, 
			OutTable);
		TBL_PutRecord( OutTable );
		convert_DbvProximityRelationType(&stringfield, 
			&psProximityOperator->eProximityRelationType);
		sprintf(temp, "eProximityRelationType : %s", stringfield);
		write_char_field(temp, OutTable, T_Return);
		TBL_PutRecord( OutTable );
		DbvFree(stringfield);
		write_DbvProximityUnitCode(&psProximityOperator->
			sProximityUnitCode, OutTable);
	}
   write_comment_field("END of DbvProximityOperator ", OutTable, T_Return );
}

void write_DbvProximityUnitCode(DbvProximityUnitCode *psProximityUnitCode,
	TBL OutTable)
{
   char *stringfield = NULL;
   char temp[ SML_BUFFER ];

   write_comment_field("START of DbvProximityUnitCode ", OutTable, T_Return );
   if(psProximityUnitCode != NULL)
   {
	convert_DbvProximityUnitType(&stringfield, &psProximityUnitCode->
		eProximityUnitType);
	sprintf(temp, "eProximityUnitType : %s", stringfield);
	write_char_field(temp, OutTable, T_Return);
	TBL_PutRecord( OutTable );
	DbvFree(stringfield);
	
	switch(psProximityUnitCode->eProximityUnitType)
	{
	case PUT_Known:
		stringfield = NULL;
		convert_DbvKnownProximityUnit(&stringfield, 
			&psProximityUnitCode->uProximityUnit.
			eKnownProximityUnit);
		sprintf(temp, "uProximityUnit.eKnownProximityUnit : %s", 
			stringfield);
		write_char_field(temp, OutTable, T_Return);
		TBL_PutRecord( OutTable );
		DbvFree(stringfield);
		break;
	case PUT_Private:
		write_integer_string("uProximityUnit.ulPrivate", 
			(int *)&psProximityUnitCode->uProximityUnit.ulPrivate, 
			OutTable);
		TBL_PutRecord( OutTable );
		break;
	default:
		break;
	}
   }
   write_comment_field("END of DbvProximityUnitCode ", OutTable, T_Return );
}

/* Functions beyond this point where added by Dan Maskell 8-2-96 */


void write_DbvGeneralString(char *name, DbvGeneralString
        *psGeneralString, TBL OutTable)
   {
   write_DbvOctetString(name, (DbvOctetString *)psGeneralString, OutTable);
   } /* End write_DbvGeneralString() */

void write_DbvSystemId(DbvSystemId *psSystemId, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psSystemId == NULL)
      {
      write_title_char("NULL","DbvSystemId", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_comment_field("START of DbvSystemId", OutTable, T_Return );

   write_title_char("psSystemId", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* psSymbol */

   write_DbvPersonOrInstitutionSymbol( psSystemId->psSymbol, OutTable );

   /* psName */

   write_DbvNameOfPersonOrInstitution( psSystemId->psName, OutTable );

   write_comment_field("END of DbvSystemId", OutTable, T_Return );
   } /* End write_DbvSystemId() */


void write_DbvIllString( char *name, DbvIllString *psIllString, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psIllString == NULL)
      {
      write_title_char("NULL", name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char( name, "Structure", OutTable);
   TBL_PutRecord( OutTable );

   /* ILL String type */

   convert_DbvIllStringType( &pc_string, &psIllString->eStringType );
   sprintf( temp, "eStringType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Ill String */

   switch( psIllString->eStringType )
      {
      case ST_GeneralString:
	 write_DbvGeneralString( "psGeneralString",
				 psIllString->uString.psGeneralString,
				 OutTable );
	 break;

      case ST_EdifactString:
	 write_DbvVisibleString( "pcVisibleString",
				 psIllString->uString.pcVisibleString,
			 	 OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End write_DbvIllString() */


void write_DbvPersonOrInstitutionSymbol(
		DbvPersonOrInstitutionSymbol *psPersonOrInstitutionSymbol, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psPersonOrInstitutionSymbol == NULL)
      {
      write_title_char("NULL","DbvPersonOrInstitutionSymbol", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPersonOrInstitutionSymbol", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of PersonOrInstitutionSymbol", 
			OutTable, T_Return );

   /* eSymbolType */
   convert_DbvPersOrInstSymbolType( &pc_string, 
			&psPersonOrInstitutionSymbol->eSymbolType);
   sprintf( temp, "eSymbolType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* sSymbol */

   write_DbvIllString( "sSymbol", 
		       &psPersonOrInstitutionSymbol->sSymbol,
		       OutTable );

   write_comment_field( "END of PersonOrInstitutionSymbol", 
			OutTable, T_Return );
   } /* End write_DbvPersonOrInstitutionSymbol() */


void write_DbvNameOfPersonOrInstitution(
		DbvNameOfPersonOrInstitution *psNameOfPersonOrInstitution, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psNameOfPersonOrInstitution == NULL)
      {
      write_title_char("NULL","DbvNameOfPersonOrInstitution", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psNameOfPersonOrInstitution", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of NameOfPersonOrInstitution", 
			OutTable, T_Return );

   /* eNameType */

   convert_DbvNameOfPersOrInstType( &pc_string, 
			&psNameOfPersonOrInstitution->eNameType);
   sprintf( temp, "eNameType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* sName */

   write_DbvIllString( "sName", &psNameOfPersonOrInstitution->sName, OutTable );

   write_comment_field( "END of NameOfPersonOrInstitution", 
			OutTable, T_Return );
   } /* End write_DbvNameOfPersonOrInstitution() */


void write_DbvTransactionId(DbvTransactionId *psTransactionId, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psTransactionId == NULL)
      {
      write_title_char("NULL","DbvTransactionId", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTransactionId", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of TransactionId", OutTable, T_Return );

   /* psInitialRequesterId */

   write_DbvSystemId( psTransactionId->psInitialRequesterId, OutTable );

   /* sTransactionGroupQualifier */

   write_DbvIllString( "sTransactionGroupQualifier",
		       &psTransactionId->sTransactionGroupQualifier, OutTable );

   /* sTransactionQualifier */

   write_DbvIllString( "sTransactionQualifier",
		       &psTransactionId->sTransactionQualifier, OutTable );

   /* psSubTransactionQualifier */

   write_DbvIllString( "psSubTransactionQualifier",
		       psTransactionId->psSubTransactionQualifier, OutTable );

   write_comment_field( "END of TransactionId", OutTable, T_Return );
   } /* End write_DbvTransactionId() */


void write_DbvDateTime(DbvDateTime *psDateTime, TBL OutTable)
   {

   /* Write the title out */ 
   if(psDateTime == NULL)
      {
      write_title_char("NULL","DbvDateTime", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDateTime", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DateTime", OutTable, T_Return );

   /* pcDate */

   write_DbvVisibleString( "pcDate", psDateTime->pcDate, OutTable );

   /* pcTime */

   write_DbvVisibleString( "pcTime", psDateTime->pcTime, OutTable );

   write_comment_field( "END of DateTime", OutTable, T_Return );
   } /* End write_DbvDateTime() */


void write_DbvServiceDateTime(DbvServiceDateTime *psServiceDateTime, 
			      TBL OutTable)
   {

   /* Write the title out */ 
   if(psServiceDateTime == NULL)
      {
      write_title_char("NULL","DbvServiceDateTime", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psServiceDateTime", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of ServiceDateTime", OutTable, T_Return );

   /* sThisService */

   write_char_field( "sThisService", OutTable, T_Return );
   TBL_PutRecord( OutTable );
   write_DbvDateTime( &psServiceDateTime->sThisService, OutTable );

   /* psOriginalService */

   write_char_field( "psOriginalService", OutTable, T_Return );
   TBL_PutRecord( OutTable );
   write_DbvDateTime( psServiceDateTime->psOriginalService, OutTable );


   write_comment_field( "END of ServiceDateTime", OutTable, T_Return );
   } /* End write_DbvServiceDateTime() */


void write_DbvPostalAddress(DbvPostalAddress *psPostalAddress, 
			      TBL OutTable)
   {

   /* Write the title out */ 
   if(psPostalAddress == NULL)
      {
      write_title_char("NULL","DbvPostalAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPostalAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of PostalAddress", OutTable, T_Return );

   /* psNameOfPersonOrInstitution */

   write_DbvNameOfPersonOrInstitution( 
			psPostalAddress->psNameOfPersonOrInstitution, 
			OutTable );

   /* psExtendedPostalDeliveryAddress */

   write_DbvIllString( "psExtendedPostalDeliveryAddress", 
			psPostalAddress->psExtendedPostalDeliveryAddress,
			OutTable );

   /* psStreetAndNumber */

   write_DbvIllString( "psStreetAndNumber", 
			psPostalAddress->psStreetAndNumber, OutTable );

   /* psPostOfficeBox */

   write_DbvIllString( "psPostOfficeBox",
			psPostalAddress->psPostOfficeBox, OutTable );

   /* psCity */

   write_DbvIllString( "psCity", 
			psPostalAddress->psCity, OutTable );

   /* psRegion */

   write_DbvIllString( "psRegion", psPostalAddress->psRegion, OutTable );

   /* psCountry */

   write_DbvIllString( "psCountry", psPostalAddress->psCountry, OutTable );

   /* psPostalCode */

   write_DbvIllString( "psPostalCode",
			psPostalAddress->psPostalCode, OutTable );

   write_comment_field( "END of PostalAddress", OutTable, T_Return );

   } /* End write_DbvPostalAddress() */


void write_DbvSystemAddress(DbvSystemAddress *psSystemAddress, 
			      TBL OutTable)
   {

   /* Write the title out */ 
   if(psSystemAddress == NULL)
      {
      write_title_char("NULL","DbvSystemAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSystemAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSystemAddress", OutTable, T_Return );

   /* psTelecomServiceIdentifier */

   write_DbvIllString( "psTelecomServiceIdentifier ", 
			psSystemAddress->psTelecomServiceIdentifier ,
			OutTable );

   /* psTelecomServiceAddress */

   write_DbvIllString( "psTelecomServiceAddress", 
			psSystemAddress->psTelecomServiceAddress,
			OutTable );

   write_comment_field( "END of DbvSystemAddress", OutTable, T_Return );

   } /* End write_DbvSystemAddress() */


void write_DbvDeliveryAddress(DbvDeliveryAddress *psDeliveryAddress, 
			      TBL OutTable)
   {

   /* Write the title out */ 

   if(psDeliveryAddress == NULL)
      {
      write_title_char("NULL","DbvDeliveryAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDeliveryAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDeliveryAddress", OutTable, T_Return );

   /* psPostalAddress */

   write_DbvPostalAddress( psDeliveryAddress->psPostalAddress, OutTable );

   /* psElectronicAddress */

   write_DbvSystemAddress( psDeliveryAddress->psElectronicAddress, OutTable );

   write_comment_field( "END of DbvDeliveryAddress", OutTable, T_Return );

   } /* End write_DbvDeliveryAddress() */


void write_DbvElDeliveryDetails(DbvElDeliveryDetails *psElDeliveryDetails, 
				TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psElDeliveryDetails == NULL)
      {
      write_title_char("NULL","DbvElDeliveryDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElDeliveryDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElDeliveryDetails", OutTable, T_Return );

   /* DbvElDeliveryDetailsType */

   convert_DbvElDeliveryDetailsType( &pc_string, 
				&psElDeliveryDetails->eDeliveryDetailsType );

   sprintf( temp, "eDeliveryDetailsType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uDeliveryDetails */

   switch( psElDeliveryDetails->eDeliveryDetailsType )
      {
      case DDT_SystemAddress:
	 write_DbvSystemAddress( psElDeliveryDetails->
				 uDeliveryDetails.psSystemAddress, OutTable );
	 break;

      case DDT_SystemId:
	 write_DbvSystemId( psElDeliveryDetails->
			    uDeliveryDetails.psSystemId, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvElDeliveryDetails", OutTable, T_Return );
   } /* End write_DbvElDeliveryDetails() */


void write_DbvElDeliveryParameter( 
		DbvElDeliveryParameter *psElDeliveryParameter, 
		TBL OutTable)
   {
   /* Write the title out */ 
   if(psElDeliveryParameter == NULL)
      {
      write_title_char("NULL","DbvElDeliveryParameter", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElDeliveryParameter", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElDeliveryParameter", OutTable, T_Return );

   /* pcDeliveryMode */

   write_DbvObjectIdentifier( "pcDeliveryMode",
			      psElDeliveryParameter->pcDeliveryMode,
			      OutTable );

   /* puDeliveryParameter */

   if ( psElDeliveryParameter->puDeliveryParameter == NULL ||
        psElDeliveryParameter->puDeliveryParameter->psDummy == NULL )
      {
      write_char_field("puDeliveryParameter: NULL", OutTable, T_Return );
      TBL_PutRecord( OutTable );
      } /* End if */
   else
      write_DbvASN1External( psElDeliveryParameter->
			     puDeliveryParameter->
			     psDummy, OutTable );

   write_comment_field( "END of DbvElDeliveryParameter", OutTable, T_Return );
   } /* End write_DbvElDeliveryParameter() */


void write_DbvDocumentTypeParameter( 
		DbvDocumentTypeParameter *psDocumentTypeParameter, 
		TBL OutTable)
   {
   /* Write the title out */ 
   if(psDocumentTypeParameter == NULL)
      {
      write_title_char("NULL","DbvDocumentTypeParameter", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDocumentTypeParameter", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDocumentTypeParameter", 
			OutTable, T_Return );

   /* pcDocumentTypeId */

   write_DbvObjectIdentifier( "pcDocumentTypeId",
			      psDocumentTypeParameter->pcDocumentTypeId,
			      OutTable );

   /* puDocumentTypeParameter */

   if ( psDocumentTypeParameter->puDocumentTypeParameter == NULL ||
        psDocumentTypeParameter->puDocumentTypeParameter->psDummy == NULL )
      {
      write_char_field("puDocumentTypeParameter: NULL", OutTable, T_Return );
      TBL_PutRecord( OutTable );
      } /* End if */
   else
      write_DbvASN1External( psDocumentTypeParameter->
			     puDocumentTypeParameter->
			     psDummy, OutTable );

   write_comment_field( "END of DbvDocumentTypeParameter", OutTable, T_Return );
   } /* End write_DbvDocumentTypeParameter() */


void write_DbvElectronicDeliveryService(
		DbvElectronicDeliveryService *psElectronicDeliveryService, 
		TBL OutTable)
   {

   /* Write the title out */ 
   if(psElectronicDeliveryService == NULL)
      {
      write_title_char("NULL","DbvElectronicDeliveryService", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElectronicDeliveryService", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElectronicDeliveryService",
			OutTable, T_Return );

   /* psDeliveryParameter */

   write_DbvElDeliveryParameter( psElectronicDeliveryService->
				 psDeliveryParameter,
				 OutTable );
   /* psDocumentTypeParameter */

   write_DbvDocumentTypeParameter( psElectronicDeliveryService->
				   psDocumentTypeParameter,
				   OutTable );

   /* psDeliveryDescription */

   write_DbvIllString( "psDeliveryDescription",
		       psElectronicDeliveryService->
		       psDeliveryDescription,
		       OutTable );

   /* sDeliveryDetails */

   write_DbvElDeliveryDetails( &psElectronicDeliveryService->sDeliveryDetails,
			       OutTable );
   /* psNameOrCode */

   write_DbvIllString( "psNameOrCode",
		       psElectronicDeliveryService->
		       psNameOrCode,
		       OutTable );

   /* pcDeliveryTime */

   write_DbvVisibleString( "pcDeliveryTime",
			    psElectronicDeliveryService->
			    pcDeliveryTime,
			    OutTable );

   write_comment_field( "END of DbvElectronicDeliveryService",
			OutTable, T_Return );
   } /* End write_DbvElectronicDeliveryService() */


void write_DbvDeliveryService( DbvDeliveryService *psDeliveryService, 
			       TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psDeliveryService == NULL)
      {
      write_title_char("NULL","DbvDeliveryService", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDeliveryService", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDeliveryService", OutTable, T_Return );

   /* eDeliveryServiceType */

   convert_DbvDeliveryServiceType( &pc_string, 
				   &psDeliveryService->eDeliveryServiceType );

   sprintf( temp, "eDeliveryServiceType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uDeliveryService */

   switch( psDeliveryService->eDeliveryServiceType )
      {
      case DST_PhysicalDelivery:
	 write_DbvIllString( "psPhysicalDelivery",
			      psDeliveryService->
			      uDeliveryService.psPhysicalDelivery, OutTable );
	 break;

      case DST_ElectronicDelivery:
	 writeN_DbvElectronicDeliveryService( psDeliveryService->
				uDeliveryService.ppsElectronicDelivery,
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvDeliveryService", OutTable, T_Return );
   } /* End write_DbvDeliveryService() */


void write_DbvRequesterOptionalMessages( 
		DbvRequesterOptionalMessages *psRequesterOptionalMessages, 
		TBL OutTable)

   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psRequesterOptionalMessages == NULL)
      {
      write_title_char("NULL","DbvRequesterOptionalMessages", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRequesterOptionalMessages", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRequesterOptionalMessages", 
			OutTable, T_Return );

   /* fCanSendReceived */

   write_DbvBool("fCanSendReceived", 
		 psRequesterOptionalMessages->fCanSendReceived,
		 OutTable );

   TBL_PutRecord( OutTable );

   /* fCanSendReturned */

   write_DbvBool( "fCanSendReturned", 
		  psRequesterOptionalMessages->fCanSendReturned,
		  OutTable );
   TBL_PutRecord( OutTable );

   /* eRequesterShipped */

   convert_DbvOptionalMsgRequirements( &pc_string, 
			&psRequesterOptionalMessages->eRequesterShipped );

   sprintf( temp, "eRequesterShipped: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* eRequesterCheckedIn */

   pc_string = NULL;
   convert_DbvOptionalMsgRequirements( &pc_string, 
			&psRequesterOptionalMessages->eRequesterCheckedIn );

   sprintf( temp, "eRequesterCheckedIn: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* eRequesterCheckedIn */

   write_comment_field( "END of DbvRequesterOptionalMessages", 
			OutTable, T_Return );
   } /* End write_DbvRequesterOptionalMessages() */



void write_DbvSearchType( DbvSearchType *psSearchType, TBL OutTable)

   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psSearchType == NULL)
      {
      write_title_char("NULL","DbvSearchType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSearchType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSearchType", OutTable, T_Return );

   /* psLevelOfService */

   write_DbvIllString( "psLevelOfService", psSearchType->psLevelOfService, 
			OutTable );

   /* pcNeedBeforeDate */

   write_DbvVisibleString( "pcNeedBeforeDate", psSearchType->pcNeedBeforeDate, 
			OutTable );
   /* peExpiryFlag */

   if (psSearchType->peExpiryFlag) {
   	convert_DbvExpiry( &pc_string, psSearchType->peExpiryFlag);

   	sprintf( temp, "peExpiryFlag: %s", pc_string );
   } else
   	sprintf( temp, "peExpiryFlag: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* pcExpiryDate */

   write_DbvVisibleString( "pcExpiryDate", psSearchType->pcExpiryDate,
			   OutTable );

   write_comment_field( "END of DbvSearchType", OutTable, T_Return );
   } /* End write_DbvSearchType() */



void write_DbvSupplyMediumInfo( DbvSupplyMediumInfo *psSupplyMediumInfo,
				TBL OutTable)

   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psSupplyMediumInfo == NULL)
      {
      write_title_char("NULL","DbvSupplyMediumInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSupplyMediumInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSupplyMediumInfo", OutTable, T_Return );

   /* eSupplyMediumType */

   convert_DbvSupplyMediumType( &pc_string,
				&psSupplyMediumInfo->eSupplyMediumType);

   sprintf( temp, "eSupplyMediumType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psMediumCharacteristics */

   write_DbvIllString( "psMediumCharacteristics",
			psSupplyMediumInfo->psMediumCharacteristics, 
			OutTable );

   write_comment_field( "END of DbvSupplyMediumInfo", OutTable, T_Return );
   } /* End write_DbvSupplyMediumInfo() */



void write_DbvClientId( DbvClientId *psClientId, TBL OutTable)
   {

   /* Write the title out */ 
   if(psClientId == NULL)
      {
      write_title_char("NULL","DbvClientId", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psClientId", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvClientId", OutTable, T_Return );

   /* psClientName */

   write_DbvIllString( "psClientName", psClientId->psClientName, 
			OutTable );

   /* psClientStatus */

   write_DbvIllString( "psClientStatus", psClientId->psClientStatus, 
			OutTable );

   /* psClientIdentifier */

   write_DbvIllString( "psClientIdentifier", psClientId->psClientIdentifier, 
			OutTable );

   write_comment_field( "END of DbvClientId", OutTable, T_Return );
   } /* End write_DbvClientId() */


void write_DbvItemId( DbvItemId *psItemId, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psItemId == NULL)
      {
      write_title_char("NULL","DbvItemId", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psItemId", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvItemId", OutTable, T_Return );

   /* peItemType */
   if (psItemId->peItemType) {
   	convert_DbvItemType( &pc_string, psItemId->peItemType );

   	sprintf( temp, "peItemType: %s", pc_string );
   } else
   	sprintf( temp, "peItemType: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* peMediumType */

   pc_string = NULL;

   if (psItemId->peMediumType) {
   	convert_DbvMediumType( &pc_string, psItemId->peMediumType);

   	sprintf( temp, "peMediumType: %s", pc_string );
   } else
   	sprintf( temp, "peMediumType: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psCallNumber */

   write_DbvIllString( "psCallNumber", psItemId->psCallNumber, OutTable );

   /* psAuthor */

   write_DbvIllString( "psAuthor", psItemId->psAuthor, OutTable );

   /* psTitle */

   write_DbvIllString( "psTitle", psItemId->psTitle, OutTable );

   /* psSubTitle */

   write_DbvIllString( "psSubTitle", psItemId->psSubTitle, OutTable );

   /* psSponsoringBody */

   write_DbvIllString( "psSponsoringBody", 
			psItemId->psSponsoringBody, OutTable );

   /* psPlaceOfPublication */

   write_DbvIllString( "psPlaceOfPublication",
			psItemId->psPlaceOfPublication, OutTable );

   /* psPublisher */

   write_DbvIllString( "psPublisher", psItemId->psPublisher, OutTable );

   /* psSeriesTitleNumber; */

   write_DbvIllString( "psSeriesTitleNumber",
			psItemId->psSeriesTitleNumber, OutTable );

   /* psVolumeIssue */

   write_DbvIllString( "psVolumeIssue", psItemId->psVolumeIssue, OutTable );

   /* psEdition */

   write_DbvIllString( "psEdition", psItemId->psEdition, OutTable );

   /* psPublicationDate */

   write_DbvIllString( "psPublicationDate",
			psItemId->psPublicationDate, OutTable );

   /* psPublicationDateOfComponent */

   write_DbvIllString( "psPublicationDateOfComponent",
			psItemId->psPublicationDateOfComponent, OutTable );

   /* psAuthorOfArticle */

   write_DbvIllString( "psAuthorOfArticle", 
			psItemId->psAuthorOfArticle, OutTable );

   /* psTitleOfArticle */

   write_DbvIllString( "psTitleOfArticle",
			psItemId->psTitleOfArticle, OutTable );

   /* psPagination */

   write_DbvIllString( "psPagination",
			psItemId->psPagination, OutTable );

   /* psNationalBibliographyNo */

   write_DbvASN1External( psItemId->psNationalBibliographyNo, OutTable );

   /* psIsbn */

   write_DbvIllString( "psIsbn", psItemId->psIsbn, OutTable );

   /* psIssn */

   write_DbvIllString( "psIssn", psItemId->psIssn, OutTable );

   /* psSystemNo */

   write_DbvASN1External( psItemId->psSystemNo, OutTable );

   /* psAdditionalNoLetters */

   write_DbvIllString( "psAdditionalNoLetters",
			psItemId->psAdditionalNoLetters, OutTable );

   /* psVerificationReferenceSource */

   write_DbvIllString( "psVerificationReferenceSource", 
			psItemId->psVerificationReferenceSource, OutTable );

   write_comment_field( "END of DbvItemId", OutTable, T_Return );
   } /* End write_DbvItemId() */


void write_DbvAmount( DbvAmount *psAmount, TBL OutTable)
   {

   /* Write the title out */ 
   if(psAmount == NULL)
      {
      write_title_char("NULL","DbvAmount", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAmount", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAmount", OutTable, T_Return );

   /* pcCurrencyCode */

   write_DbvPrintableString( "pcCurrencyCode", 
			     psAmount->pcCurrencyCode, OutTable );

   /* pcMonetaryValue */

   write_DbvPrintableString( "pcMonetaryValue",
			     psAmount->pcMonetaryValue, OutTable );

   write_comment_field( "END of DbvAmount", OutTable, T_Return );
   } /* End write_DbvAmount() */



void write_DbvCostInfo( DbvCostInfo *psCostInfo, TBL OutTable)
   {

   /* Write the title out */ 
   if(psCostInfo == NULL)
      {
      write_title_char("NULL","DbvCostInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCostInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCostInfo", OutTable, T_Return );

   /* psAccountNumber */

   write_DbvIllString( "psAccountNumber", psCostInfo->psAccountNumber,
			OutTable );

   /* psMaximumCost */

   write_DbvAmount( psCostInfo->psMaximumCost,  OutTable );

   /* pfReciprocalAgreement */
   if (psCostInfo->pfReciprocalAgreement)
   	write_DbvBool( "pfReciprocalAgreement", *(psCostInfo->pfReciprocalAgreement), OutTable );
   else 
   	write_char_field("pfReciprocalAgreement: NULL", OutTable, T_Return);

   TBL_PutRecord( OutTable );

   /* pfWillPayFee */
   if (psCostInfo->pfWillPayFee)
   	write_DbvBool( "pfWillPayFee", *(psCostInfo->pfWillPayFee), OutTable );
   else
   	write_char_field("pfWillPayFee: NULL", OutTable, T_Return);

   TBL_PutRecord( OutTable );


   /* pfPaymentProvided */
   if (psCostInfo->pfWillPayFee)
   	write_DbvBool( "pfPaymentProvided", *(psCostInfo->pfPaymentProvided), OutTable );
   else
   	write_char_field("pfPaymentProvided: NULL", OutTable, T_Return);


   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvCostInfo", OutTable, T_Return );
   } /* End write_DbvCostInfo() */



void write_DbvSendToList( DbvSendToList *psSendToList, TBL OutTable)
   {

   /* Write the title out */ 
   if(psSendToList == NULL)
      {
      write_title_char("NULL","DbvSendToList", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSendToList", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSendToList", OutTable, T_Return );

   /* sSystemId */

   write_DbvSystemId( &psSendToList->sSystemId, OutTable );

   /* psAccountNumber */
   write_DbvIllString( "psAccountNumber", psSendToList->psAccountNumber,
			OutTable );

   /* psSystemAddress */

   write_DbvSystemAddress( psSendToList->psSystemAddress, OutTable );

   write_comment_field( "END of DbvSendToList", OutTable, T_Return );
   } /* End write_DbvSendToList() */


void write_DbvThirdPartyInfo( DbvThirdPartyInfo *psThirdPartyInfo, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psThirdPartyInfo == NULL)
      {
      write_title_char("NULL","DbvThirdPartyInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psThirdPartyInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvThirdPartyInfo", OutTable, T_Return );

   /* pfPermissionToForward */
   if (psThirdPartyInfo->pfPermissionToForward)
   	write_DbvBool( "pfPermissionToForward", 
			  *(psThirdPartyInfo->pfPermissionToForward),
			  OutTable );
   else
   	write_char_field("pfPermissionToForward: NULL", OutTable, T_Return);
   TBL_PutRecord( OutTable );


   /* pfPermissionToChain */
   if (psThirdPartyInfo->pfPermissionToChain) 
   	write_DbvBool( "pfPermissionToChain", 
		  	*(psThirdPartyInfo->pfPermissionToChain),
		  	OutTable );
    else
   	write_char_field("pfPermissionToChain: NULL", OutTable, T_Return);
    TBL_PutRecord( OutTable );


   /* pfPermissionToPartition */
   if (psThirdPartyInfo->pfPermissionToPartition) 
   	write_DbvBool( "pfPermissionToPartition", 
		  	*(psThirdPartyInfo->pfPermissionToPartition),
		  	OutTable );
   else
   	write_char_field("pfPermissionToPartition: NULL", OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* pfPermissionToChangeSendToList */
   if (psThirdPartyInfo->pfPermissionToChangeSendToList) 
   	write_DbvBool( "pfPermissionToChangeSendToList", 
			  *(psThirdPartyInfo->pfPermissionToChangeSendToList),
			  OutTable );
   else
   	write_char_field("pfPermissionToChangeSendToList: NULL", OutTable, T_Return);
   
   TBL_PutRecord( OutTable );

   /* psInitialRequesterAddress */

   write_DbvSystemAddress( psThirdPartyInfo->psInitialRequesterAddress, 
			   OutTable ); 

   /* pePreference */
   if (psThirdPartyInfo->pePreference) {
   	convert_DbvPreference( &pc_string, psThirdPartyInfo->pePreference );
   	sprintf( temp, "pePreference : %s", pc_string );
   } else
	sprintf( temp, "pePreference : %s", "NULL");

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* ppsSendToList */

   writeN_DbvSendToList( psThirdPartyInfo->ppsSendToList, OutTable );

   /* ppsAlreadyTriedList */

   writeN_DbvSystemId( psThirdPartyInfo->ppsAlreadyTriedList, OutTable );

   write_comment_field( "END of DbvThirdPartyInfo", OutTable, T_Return );
   } /* End write_DbvThirdPartyInfo() */










void write_DbvAddRequesterInfo(DbvAddRequesterInfo *psAddRequesterInfo, TBL OutTable )
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psAddRequesterInfo == NULL)
      {
      write_title_char("NULL","DbvAddRequesterInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAddRequesterInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAddRequesterInfo", OutTable, T_Return );

   write_title_char("psAddRequesterInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAddRequesterInfo", OutTable, T_Return );

   /* peRequesterGroup */
   if (psAddRequesterInfo->peRequesterGroup) {
   	convert_DbvRequesterGroup( &pc_string,
			   	psAddRequesterInfo->peRequesterGroup);
   	sprintf( temp, "peRequesterGroup: %s", pc_string );
   } else
   	sprintf( temp, "peRequesterGroup: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_DbvVisibleString( "pcRequesterEmailAddress ",
			psAddRequesterInfo->pcRequesterEmailAddress, OutTable );

   write_comment_field( "END of DbvAddRequesterInfo", OutTable, T_Return );

   } /* End write_DbvAddRequesterInfo() */



void write_DbvAddDeliveryInfo(DbvAddDeliveryInfo *psAddDeliveryInfo, TBL OutTable )
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psAddDeliveryInfo == NULL)
      {
      write_title_char("NULL","DbvAddDeliveryInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAddDeliveryInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAddDeliveryInfo", OutTable, T_Return );

   /* peStatusLevelUser */
   if (psAddDeliveryInfo->peStatusLevelUser) {
   	convert_DbvStatusLevelUser( &pc_string,
			   	psAddDeliveryInfo->peStatusLevelUser);
   	sprintf( temp, "peStatusLevelUser: %s", pc_string );
   } else
   	sprintf( temp, "peStatusLevelUser: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );


   /* peStatusLevelOrigin */

   /* reset pc_string so that convert_DbvStatusLevelRequester()
      works correctly */
   pc_string = NULL;

   if (psAddDeliveryInfo->peStatusLevelOrigin) {
   	convert_DbvStatusLevelRequester( &pc_string,
				   psAddDeliveryInfo->peStatusLevelOrigin);
   	sprintf( temp, "peStatusLevelUser: %s", pc_string );
   } else
   	sprintf( temp, "peStatusLevelUser: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );


   /* pcDeliveryNotes */
   write_DbvVisibleString( "pcDeliveryNotes ",
			psAddDeliveryInfo->pcDeliveryNotes, OutTable );

   /* pcCountryDeliveryTarget */
   write_DbvVisibleString( "pcCountryDeliveryTarget ",
			psAddDeliveryInfo->pcCountryDeliveryTarget, OutTable );



   /* pfAlternativePostalDelivery */
   if (psAddDeliveryInfo->pfAlternativePostalDelivery) {
   	write_DbvBool( "pfAlternativePostalDelivery", 
			  *(psAddDeliveryInfo->pfAlternativePostalDelivery),
			  OutTable );
   } else
	write_char_field( "pfAlternativePostalDelivery: NULL", OutTable, T_Return );

   TBL_PutRecord( OutTable );


   write_comment_field( "END of DbvAddDeliveryInfo", OutTable, T_Return );

   } /* End write_DbvAddDeliveryInfo() */



void write_DbvBillingBankInfo(DbvBillingBankInfo *psBillingBankInfo, TBL OutTable)
{
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psBillingBankInfo == NULL)
      {
      write_title_char("NULL","DbvBillingBankInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psBillingBankInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvBillingBankInfo", OutTable, T_Return );

   /* pcBankAccountHouse */
   write_DbvVisibleString( "pcBankAccountHouse ",
			psBillingBankInfo->pcBankAccountHouse, OutTable );


   /* pcBankAccountRoutingSymbol */
   write_DbvVisibleString( "pcBankAccountRoutingSymbol ",
			psBillingBankInfo->pcBankAccountRoutingSymbol, OutTable );


   /* pcBankAccountNo */
   write_DbvVisibleString( "pcBankAccountNo ",
			psBillingBankInfo->pcBankAccountNo, OutTable );


   /* fBankAuthority */
   write_DbvBool( "fBankAuthority", 
		  psBillingBankInfo->fBankAuthority,
		  OutTable );
   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvBillingBankInfo", OutTable, T_Return );

   } /* End write_DbvBillingBankInfo() */



void write_DbvBillingCreditCardInfo(DbvBillingCreditCardInfo *psBillingCreditCardInfo, TBL OutTable)
{
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psBillingCreditCardInfo == NULL)
      {
      write_title_char("NULL","DbvBillingCreditCardInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psBillingCreditCardInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvBillingCreditCardInfo", OutTable, T_Return );


   /* pcCardCompany */
   write_DbvVisibleString( "pcCardCompany ",
			psBillingCreditCardInfo->pcCardCompany, OutTable );



   /* pcCardNo */
   write_DbvVisibleString( "pcCardNo ",
			psBillingCreditCardInfo->pcCardNo, OutTable );



   /* pcCardValidFrom */
   write_DbvVisibleString( "pcCardValidFrom ",
			psBillingCreditCardInfo->pcCardValidFrom, OutTable );



   /* pcCardValidThru */
   write_DbvVisibleString( "pcCardValidThru ",
			psBillingCreditCardInfo->pcCardValidThru, OutTable );


   write_comment_field( "END of DbvBillingCreditCardInfo", OutTable, T_Return );
   } /* End write_DbvBillingCreditCardInfo() */



void write_DbvAddBillingInfo(DbvAddBillingInfo *psAddBillingInfo, TBL OutTable )
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psAddBillingInfo == NULL)
      {
      write_title_char("NULL","DbvAddBillingInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAddBillingInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAddBillingInfo", OutTable, T_Return );

   /* uAddBillingInfo */

   switch( psAddBillingInfo->eAddBillingType )
      {
      case ABT_BankInfo:
	 write_DbvBillingBankInfo( psAddBillingInfo->
				 uAddBillingInfo.psBillingBankInfo, OutTable );
	 break;

      case ABT_CreditCardInfo:
	 write_DbvBillingCreditCardInfo( psAddBillingInfo->
			    uAddBillingInfo.psBillingCreditCardInfo, OutTable );
	 break;

      case ABT_InvoiceType:
   
        /* peStatusLevelOrigin */
  	pc_string = NULL;
   	convert_DbvBillingInvoiceType( &pc_string,
		   &psAddBillingInfo->uAddBillingInfo.eBillingInvoiceType);
   	sprintf( temp, "eBillingInvoiceType: %s", pc_string );
        write_char_field(temp, OutTable, T_Return);
        TBL_PutRecord( OutTable );
        DbvFree( pc_string );
 	break;

      case ABT_PrepaidType:
        write_DbvVisibleString( "pcPrepaidId ",
			psAddBillingInfo->uAddBillingInfo.pcPrepaidId, OutTable );
	break;

      case ABT_DepositType:
        write_DbvVisibleString( "pcDepositId ",
			psAddBillingInfo->uAddBillingInfo.pcDepositId, OutTable );
	break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvAddBillingInfo", OutTable, T_Return );
   } /* End write_DbvAddBillingInfo() */




/* DbvSubitoAdditionalInfo is defined in iord_ext.h */ 
void write_DbvSubitoAdditionalInfo(DbvSubitoAdditionalInfo *psAddIllReqInfo, TBL OutTable )
   {
   /* Write the title out */ 
   if(psAddIllReqInfo == NULL)
      {
      write_title_char("NULL","DbvSubitoAdditionalInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */


   write_title_char("psAddIllReqInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSubitoAdditionalInfo", OutTable, T_Return );

   /* pfCustomerInfoToKeep */
   if (psAddIllReqInfo->pfCustomerInfoToKeep)
   	write_DbvBool( "pfCustomerInfoToKeep", 
		  	*(psAddIllReqInfo->pfCustomerInfoToKeep),
		  	OutTable );
   else
   	write_char_field("pfCustomerInfoToKeep: NULL", OutTable, T_Return);

   TBL_PutRecord( OutTable );

   /* psAddRequesterInfo */
   write_DbvAddRequesterInfo( psAddIllReqInfo->psAddRequesterInfo, 
			   OutTable ); 

   /* psAddDeliveryInfo */
   write_DbvAddDeliveryInfo( psAddIllReqInfo->psAddDeliveryInfo, 
			   OutTable ); 


   /* psAddBillingInfo */
   write_DbvAddBillingInfo( psAddIllReqInfo->psAddBillingInfo, 
			   OutTable ); 

   write_comment_field( "END of DbvSubitoAdditionalInfo", OutTable, T_Return );

   } /* End write_DbvSubitoAdditionalInfo() */



void write_DbvExtension( DbvExtension *psExtension, TBL OutTable)
   {
   /* Write the title out */ 
   if(psExtension == NULL)
      {
      write_title_char("NULL","DbvExtension", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psExtension", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvExtension", OutTable, T_Return );

   /* ulIdentifier */

   write_integer_string( "ulIdentifier", (int*)(&psExtension->ulIdentifier),
			 OutTable );
   TBL_PutRecord( OutTable );

   /* pfCritical */
   if (psExtension->pfCritical)
   	write_DbvBool( "pfCritical", *(psExtension->pfCritical), OutTable );
   else
        write_char_field("pfCritical: NULL", OutTable, T_Return);

   TBL_PutRecord( OutTable );

   /* uItem */
   if ( psExtension->uItem.psAddIllReqInfo == NULL )
      {
      write_char_field("NULL uItem", OutTable, T_Return );
      TBL_PutRecord( OutTable );
      } 
   else
      write_DbvSubitoAdditionalInfo( psExtension->uItem.psAddIllReqInfo, OutTable );

   write_comment_field( "END of DbvExtension", OutTable, T_Return );
   } /* End write_DbvExtension() */


void write_DbvIllRequest( DbvIllRequest *psIllRequest, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psIllRequest == NULL)
      {
      write_title_char("NULL","DbvIllRequest", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psIllRequest", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvIllRequest", OutTable, T_Return );

   /* eProtocolVersion */

   convert_DbvIllProtocolVersion( &pc_string, &psIllRequest->eProtocolVersion);
   sprintf( temp, "eProtocolVersion: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* sTransactionId */

   write_DbvTransactionId( &psIllRequest->sTransactionId, OutTable );

   /* sServiceDateTime */

   write_DbvServiceDateTime( &psIllRequest->sServiceDateTime, OutTable );

   /* psRequesterId */

   write_DbvSystemId( psIllRequest->psRequesterId, OutTable );

   /* psResponderId */

   write_DbvSystemId( psIllRequest->psResponderId, OutTable );

   /* peTransactionType */

   pc_string = NULL;

   if (psIllRequest->peTransactionType) {
   	convert_DbvTransactionType( &pc_string, psIllRequest->peTransactionType );
   	sprintf( temp, "peTransactionType: %s", pc_string );
   } else
   	sprintf( temp, "peTransactionType: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psDeliveryAddress */

   write_DbvDeliveryAddress( psIllRequest->psDeliveryAddress, OutTable );

   /* psDeliveryService */

   write_DbvDeliveryService( psIllRequest->psDeliveryService, OutTable );

   /* psBillingAddress */

   write_DbvDeliveryAddress( psIllRequest->psBillingAddress, OutTable );

   /* ppeServiceType */

   writeN_DbvServiceType( psIllRequest->ppeServiceType, OutTable );

   /* psResponderSpecificService */

   write_DbvASN1External( psIllRequest->psResponderSpecificService, OutTable );

   /* sRequesterOptionalMessages */

   write_DbvRequesterOptionalMessages( 
			&psIllRequest->sRequesterOptionalMessages,
			OutTable );

   /* psSearchType */

   write_DbvSearchType( psIllRequest->psSearchType, OutTable );

   /* ppsSupplyMediumInfo */

   writeN_DbvSupplyMediumInfo( psIllRequest->ppsSupplyMediumInfo, OutTable );

   /* pePlaceOnHold */

   pc_string = NULL;

   if (psIllRequest->pePlaceOnHold) {
   	convert_DbvPlaceOnHold( &pc_string, psIllRequest->pePlaceOnHold );
   	sprintf( temp, "pePlaceOnHold: %s", pc_string );
   } else
   	sprintf( temp, "pePlaceOnHold: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psClientId */

   write_DbvClientId( psIllRequest->psClientId, OutTable );

   /* sItemId */

   write_DbvItemId( &psIllRequest->sItemId, OutTable );

   /* ppsSupplementalItemDescription */

   writeN_DbvASN1External( psIllRequest->ppsSupplementalItemDescription,
			   OutTable );

   /* psCostInfo */

   write_DbvCostInfo( psIllRequest->psCostInfo, OutTable );

   /* psCopyrightCompliance */

   write_DbvIllString( "psCopyrightCompliance",
		       psIllRequest->psCopyrightCompliance,
		       OutTable );

   /* psThirdPartyInfo */

   write_DbvThirdPartyInfo( psIllRequest->psThirdPartyInfo, OutTable  );

   /* pfRetry */
   if (psIllRequest->pfRetry)
   	write_DbvBool( "pfRetry", *(psIllRequest->pfRetry), OutTable );
   else
   	write_char_field("pfRetry: NULL", OutTable, T_Return);

   TBL_PutRecord( OutTable );

   /* pfForward */
   if (psIllRequest->pfForward)
   	write_DbvBool( "pfForward", *(psIllRequest->pfForward), OutTable );
   else
   	write_char_field("pfForward: NULL", OutTable, T_Return);
		

   TBL_PutRecord( OutTable );

   /* psRequesterNote */

   write_DbvIllString( "psRequesterNote", psIllRequest->psRequesterNote,
		       OutTable );

   /* psForwardNote */

   write_DbvIllString( "psForwardNote", psIllRequest->psForwardNote, OutTable );

   /* ppsRequestExtension */

   writeN_DbvExtension( psIllRequest->ppsRequestExtension, OutTable );

   write_comment_field( "END of DbvIllRequest", OutTable, T_Return );
   } /* End write_DbvIllRequest() */


void write_DbvServiceType( DbvServiceType *peServiceType, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(peServiceType == NULL)
      {
      write_title_char("NULL","DbvServiceType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* peServiceType */

   convert_DbvServiceType( &pc_string, peServiceType);
   sprintf( temp, "peServiceType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvServiceType() */


void write_DbvHistoryReport(DbvHistoryReport *psHistoryReport, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psHistoryReport == NULL)
      {
      write_title_char("NULL","DbvHistoryReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psHistoryReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvHistoryReport", OutTable, T_Return );

   /* pcDateRequested */

   write_DbvVisibleString( "pcDateRequested",
			psHistoryReport->pcDateRequested, OutTable );

   /* psAuthor */

   write_DbvIllString( "psAuthor", psHistoryReport->psAuthor, OutTable );

   /* psTitle */

   write_DbvIllString( "psTitle", psHistoryReport->psTitle, OutTable );

   /* psAuthorOfArticle */

   write_DbvIllString( "psAuthorOfArticle",
			psHistoryReport->psAuthorOfArticle, OutTable );

   /* psTitleOfArticle */

   write_DbvIllString( "psTitleOfArticle",
			psHistoryReport->psTitleOfArticle, OutTable );

   /* pcDateOfLastTransition */

   write_DbvVisibleString( "pcDateOfLastTransition",
			psHistoryReport->pcDateOfLastTransition, OutTable );

   /* eMostRecentService */

   convert_DbvMostRecentService( &pc_string,
				 &psHistoryReport->eMostRecentService);
   sprintf( temp, "eMostRecentService: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* pcDateOfMostRecentService */

   write_DbvVisibleString( "pcDateOfMostRecentService ",
			psHistoryReport->pcDateOfMostRecentService, OutTable );

   /* sSystemId */

   write_DbvSystemId( &psHistoryReport->sSystemId, OutTable );

   /* peShippedServiceType */

   pc_string = NULL;
   if (psHistoryReport->peShippedServiceType) {
   	convert_DbvShippedServiceType( &pc_string,
				 psHistoryReport->peShippedServiceType);
   	sprintf( temp, "peShippedServiceType: %s", pc_string );
   } else
        sprintf( temp, "peShippedServiceType: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* peTransactionResults */

   pc_string = NULL;
   if (psHistoryReport->peTransactionResults) {
   	convert_DbvTransactionResults( &pc_string,
				 psHistoryReport->peTransactionResults);
   	sprintf( temp, "peTransactionResults: %s", pc_string );
   } else
        sprintf( temp, "peTransactionResults: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psMostRecentServiceNote */

   write_DbvIllString( "psMostRecentServiceNote",
			psHistoryReport->psMostRecentServiceNote, OutTable );

   write_comment_field( "END of DbvHistoryReport", OutTable, T_Return );
   } /* End write_DbvHistoryReport() */


void write_DbvStateTransitionProhibited(
		DbvStateTransitionProhibited *psStateTransitionProhibited,
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psStateTransitionProhibited == NULL)
      {
      write_title_char("NULL","DbvStateTransitionProhibited", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psStateTransitionProhibited", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvStateTransitionProhibited", 
			OutTable, T_Return );

   /* eAPDUType */

   convert_DbvIllAPDUType( &pc_string,
			   &psStateTransitionProhibited->eAPDUType);
   sprintf( temp, "eAPDUType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* eCurrentState */
   pc_string = NULL;
   convert_DbvCurrentState( &pc_string, 
			    &psStateTransitionProhibited->eCurrentState);
   sprintf( temp, "eCurrentState: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_comment_field( "END of DbvStateTransitionProhibited", 
			OutTable, T_Return );
   } /* End write_DbvStateTransitionProhibited() */


void write_DbvAlreadyForwarded( DbvAlreadyForwarded *psAlreadyForwarded,
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psAlreadyForwarded == NULL)
      {
      write_title_char("NULL","DbvAlreadyForwarded", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAlreadyForwarded", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAlreadyForwarded", OutTable, T_Return );

   /* sResponderId */

   write_DbvSystemId( &psAlreadyForwarded->sResponderId, OutTable );

   /* psResponderAddress */

   write_DbvSystemAddress( psAlreadyForwarded->psResponderAddress, OutTable );

   write_comment_field( "END of DbvAlreadyForwarded", OutTable, T_Return );
   } /* End write_DbvAlreadyForwarded() */


void write_DbvProviderErrorReport(
		DbvProviderErrorReport *psProviderErrorReport, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psProviderErrorReport == NULL)
      {
      write_title_char("NULL","DbvProviderErrorReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psProviderErrorReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvProviderErrorReport", OutTable, T_Return );

   /* eReportType*/

   convert_DbvProviderErrorReportType( &pc_string,
				 &psProviderErrorReport->eReportType );
   sprintf( temp, "eReportType : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   pc_string = NULL;

   /* uProviderErrorReport */

   switch( psProviderErrorReport->eReportType )
      {
      case PER_GeneralProblem:
         convert_DbvGeneralProblem( &pc_string,
	 	&psProviderErrorReport->uProviderErrorReport.eGeneralProblem  );
         sprintf( temp, "eGeneralProblem: %s", pc_string );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
         DbvFree( pc_string );
	 break;

      case PER_TransactionIdProblem:
         convert_DbvTransactionIdProblem( &pc_string,
	 	&psProviderErrorReport->
		uProviderErrorReport.eTransactionIdProblem );
         sprintf( temp, "eTransactionIdProblem: %s", pc_string );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
         DbvFree( pc_string );
	 break;

      case PER_StateTransitionProhibited:
	 write_DbvStateTransitionProhibited(
			psProviderErrorReport->
			uProviderErrorReport.psStateTransitionProblem, 
			OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvProviderErrorReport", OutTable, T_Return );
   } /* End write_DbvProviderErrorReport() */


void write_DbvUserErrorReport( DbvUserErrorReport *psUserErrorReport, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psUserErrorReport == NULL)
      {
      write_title_char("NULL","DbvUserErrorReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUserErrorReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUserErrorReport", OutTable, T_Return );

   /* eReportType*/

   convert_DbvUserErrorReportType( &pc_string,
				 &psUserErrorReport->eReportType );
   sprintf( temp, "eReportType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   pc_string = NULL;

   /* uUserErrorReport */

   switch( psUserErrorReport->eReportType )
      {
      case UER_AlreadyForwarded:
	 write_DbvAlreadyForwarded( psUserErrorReport->
				    uUserErrorReport.psAlreadyForwarded,
				    OutTable );
	 break;

      case UER_IntermediaryProblem:
         convert_DbvIntermediaryProblem( &pc_string,
	 	&psUserErrorReport->uUserErrorReport.eIntermediaryProblem);
         sprintf( temp, "eIntermediaryProblem: %s", pc_string );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
         DbvFree( pc_string );
	 break;

      case UER_SecurityProblem:
	 write_DbvIllString( "psSecurityProblem",
			psUserErrorReport->
			uUserErrorReport.psSecurityProblem, 
			OutTable );
	 break;

      case UER_UnableToPerform:
         convert_DbvUnableToPerform( &pc_string,
	 	&psUserErrorReport->uUserErrorReport.eUnableToPerform);
         sprintf( temp, "eUnableToPerform: %s", pc_string );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
         DbvFree( pc_string );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvUserErrorReport", OutTable, T_Return );
   } /* End write_DbvUserErrorReport() */


void write_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psUserOrProviderErrorReport == NULL)
      {
      write_title_char("NULL","DbvUserOrProviderErrorReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUserOrProviderErrorReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUserOrProviderErrorReport", 
			OutTable, T_Return );

   /* eReportSource */

   convert_DbvReportSource( &pc_string,
			    &psUserOrProviderErrorReport->eReportSource);
   sprintf( temp, "eReportSource: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uReport */

   switch( psUserOrProviderErrorReport->eReportSource )
      {
      case RS_Provider:
	 write_DbvProviderErrorReport( psUserOrProviderErrorReport->
				    uReport.psProviderErrorReport,
				    OutTable );
	 break;

      case RS_User:
	 write_DbvUserErrorReport( psUserOrProviderErrorReport->
				    uReport.psUserErrorReport,
				    OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvUserOrProviderErrorReport", 
			OutTable, T_Return );
   } /* End write_DbvUserOrProviderErrorReport() */


void write_DbvErrorReport( DbvErrorReport *psErrorReport, TBL OutTable)
   {

   /* Write the title out */ 
   if(psErrorReport == NULL)
      {
      write_title_char("NULL","DbvErrorReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psErrorReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvErrorReport", OutTable, T_Return );

   /* sCorrelationInfo */

   write_DbvIllString( "sCorrelationInfo", &psErrorReport->sCorrelationInfo,
			OutTable );
   /* sReport */

   write_DbvUserOrProviderErrorReport( &psErrorReport->sReport, OutTable );

   write_comment_field( "END of DbvErrorReport", OutTable, T_Return );
   } /* End write_DbvErrorReport() */


void write_DbvStatusReport( DbvStatusReport *psStatusReport, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psStatusReport == NULL)
      {
      write_title_char("NULL","DbvStatusReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psStatusReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvStatusReport", OutTable, T_Return );

   /* sUserStatusReport*/

   write_DbvHistoryReport( &psStatusReport->sUserStatusReport, OutTable );

   /* eProviderStatusReport */

   convert_DbvCurrentState( &pc_string, &psStatusReport->eProviderStatusReport);
   sprintf( temp, "eProviderStatusReport: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_comment_field( "END of DbvStatusReport", OutTable, T_Return );
   } /* End write_DbvStatusReport() */


void write_DbvStatusOrErrorReport( 
		DbvStatusOrErrorReport *psStatusOrErrorReport, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psStatusOrErrorReport == NULL)
      {
      write_title_char("NULL","DbvStatusOrErrorReport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psStatusOrErrorReport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvStatusOrErrorReport", OutTable, T_Return );

   /* eProtocolVersion */

   convert_DbvIllProtocolVersion( &pc_string, 
				  &psStatusOrErrorReport->eProtocolVersion);
   sprintf( temp, "eProtocolVersion: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* sTransactionId */

   write_DbvTransactionId( &psStatusOrErrorReport->sTransactionId, OutTable );

   /* sServiceDateTime */

   write_DbvServiceDateTime( &psStatusOrErrorReport->sServiceDateTime, 
			     OutTable );

   /* psRequesterId */

   write_DbvSystemId( psStatusOrErrorReport->psRequesterId, OutTable );

   /* psResponderId */

   write_DbvSystemId( psStatusOrErrorReport->psResponderId, OutTable );

   /* peReasonNoReport */

   pc_string = NULL;
   if (psStatusOrErrorReport->peReasonNoReport) {
   	convert_DbvReasonNoReport( &pc_string, 
				      psStatusOrErrorReport->peReasonNoReport );

   	sprintf( temp, "peReasonNoReport: %s", pc_string );
   } else
   	sprintf( temp, "peReasonNoReport: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* psStatusReport */

   write_DbvStatusReport( psStatusOrErrorReport->psStatusReport, OutTable );

   /* psErrorReport */

   write_DbvErrorReport( psStatusOrErrorReport->psErrorReport,  OutTable );

   /* psNote */

   write_DbvIllString( "psNote",
		       psStatusOrErrorReport->psNote,
		       OutTable );

   /* ppsStOrErReportExtension */

   writeN_DbvExtension( psStatusOrErrorReport->ppsStOrErReportExtension,
			OutTable );

   write_comment_field( "END of DbvStatusOrErrorReport", OutTable, T_Return );
   } /* End write_DbvStatusOrErrorReport() */


void write_DbvIllPdu(DbvIllPdu *psIllPdu, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psIllPdu == NULL)
      {
      write_title_char("NULL","DbvIllPdu", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psIllPdu", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of IllPdu", OutTable, T_Return );

   /* eIllPduType */

   convert_DbvIllPduType( &pc_string, &psIllPdu->eIllPduType );
   sprintf( temp, "eIllPduType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uIllPdu */

   switch( psIllPdu->eIllPduType )
      {
      case ILL_IllRequest:
	 write_DbvIllRequest( psIllPdu->uIllPdu.psIllRequest, OutTable);
	 break;

      case ILL_StatusOrErrorReport:
	 write_DbvStatusOrErrorReport( psIllPdu->
				       uIllPdu.psStatusOrErrorReport,
				       OutTable);
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of IllPdu", OutTable, T_Return );
   } /* End write_DbvIllPdu() */


void write_DbvPrintableString( char *name, 
			       DbvPrintableString pcPrintableString,
			       TBL OutTable)
   {
   char temp[ STRING_LEN ];

   if ( pcPrintableString != NULL )
      sprintf(temp, "%s: >%s<", name, pcPrintableString );
   else
      sprintf(temp, "%s : NULL", name);

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord(OutTable);

   } /* End write_DbvPrintableString() */


void write_DbvEsIoTargetPart(DbvEsIoTargetPart *psEsIoTargetPart, 
			     TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psEsIoTargetPart == NULL)
      {
      write_title_char("NULL","DbvEsIoTargetPart", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoTargetPart", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoTargetPart", OutTable, T_Return );

   /* psItemRequest */

   write_DbvASN1External( psEsIoTargetPart->psItemRequest, OutTable );

   /* psStatusOrErrorReport */

   write_DbvASN1External( psEsIoTargetPart->psStatusOrErrorReport, OutTable );

   /* peAuxiliaryStatus */
   if (psEsIoTargetPart->peAuxiliaryStatus) {
   	convert_DbvEsIoAuxiliaryStatus( &pc_string, 
				   	psEsIoTargetPart->peAuxiliaryStatus );
   	sprintf( temp, "peAuxiliaryStatus: %s", pc_string );
   } else
   	sprintf( temp, "peAuxiliaryStatus: %s", "NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_comment_field( "END of DbvEsIoTargetPart", OutTable, T_Return );
   } /* End write_DbvEsIoTargetPart() */


void write_DbvEsIoResultSetItem( DbvEsIoResultSetItem *psEsIoResultSetItem, 
			     TBL OutTable)
   {
   /* Write the title out */ 
   if(psEsIoResultSetItem == NULL)
      {
      write_title_char("NULL","DbvEsIoResultSetItem", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoResultSetItem", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoResultSetItem", OutTable, T_Return );

   /* sResultSetId */

   write_DbvInternationalString( "sResultSetId",
				 &psEsIoResultSetItem->sResultSetId,
				 OutTable );
   /* ulItem */

   write_integer_string( "ulItem", 
			 (int*)(&psEsIoResultSetItem->ulItem),
			 OutTable );
   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvEsIoResultSetItem", OutTable, T_Return );
   } /* End write_DbvEsIoResultSetItem() */


void write_DbvEsIoOriginPartNotToKeep( 
			DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep, 
			TBL OutTable)
   {
   /* Write the title out */ 
   if(psEsIoOriginPartNotToKeep == NULL)
      {
      write_title_char("NULL","DbvEsIoOriginPartNotToKeep", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoOriginPartNotToKeep", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoOriginPartNotToKeep", 
			OutTable, T_Return );

   /* psResultSetItem */

   write_DbvEsIoResultSetItem( psEsIoOriginPartNotToKeep->psResultSetItem,
				 OutTable );
   /* psItemRequest */

   write_DbvASN1External( psEsIoOriginPartNotToKeep->psItemRequest, OutTable );
   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvEsIoOriginPartNotToKeep", 
			OutTable, T_Return );
   } /* End write_DbvEsIoOriginPartNotToKeep() */


void write_DbvEsIoCreditCardInfo( DbvEsIoCreditCardInfo *psEsIoCreditCardInfo, 
			TBL OutTable)
   {
   /* Write the title out */ 
   if(psEsIoCreditCardInfo == NULL)
      {
      write_title_char("NULL","DbvEsIoCreditCardInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoCreditCardInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoCreditCardInfo", 
			OutTable, T_Return );

   /* sNameOnCard */

   write_DbvInternationalString( "sNameOnCard",
				 &psEsIoCreditCardInfo->sNameOnCard, 
				 OutTable );
   /* sExpirationDate */

   write_DbvInternationalString( "sExpirationDate",
				 &psEsIoCreditCardInfo->sExpirationDate, 
				 OutTable );

   /* sCardNumber */

   write_DbvInternationalString( "sCardNumber",
				 &psEsIoCreditCardInfo->sCardNumber, 
				 OutTable );

   write_comment_field( "END of DbvEsIoCreditCardInfo", 
			OutTable, T_Return );
   } /* End write_DbvEsIoCreditCardInfo() */


void write_DbvEsIoPaymentMethod(
			DbvEsIoPaymentMethod *psEsIoPaymentMethod, 
			TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psEsIoPaymentMethod == NULL)
      {
      write_title_char("NULL","DbvEsIoPaymentMethod", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoPaymentMethod", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoPaymentMethod", OutTable, T_Return );

   /* ePaymentMethodType */
   convert_DbvEsIoPaymentMethodType( &pc_string,
				     &psEsIoPaymentMethod->ePaymentMethodType);
   sprintf( temp, "ePaymentMethodType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* puPaymentMethod */

   if ( psEsIoPaymentMethod->puPaymentMethod == NULL )
      {
      write_char_field("NULL puPaymentMethod", OutTable, T_Return );
      TBL_PutRecord(OutTable);
      write_comment_field( "END of DbvEsIoPaymentMethod", OutTable, T_Return );
      return;
      } /* End if */

   switch( psEsIoPaymentMethod->ePaymentMethodType )
      {
      case IOPM_CreditCard:
	 write_DbvEsIoCreditCardInfo( psEsIoPaymentMethod->
				      puPaymentMethod->
				      psCreditCardInfo, OutTable );
	 break;

      case IOPM_PrivateNotKnown:
	 write_DbvASN1External( psEsIoPaymentMethod->
				puPaymentMethod->
				psPrivateNotKnown, OutTable );
	 break;

      case IOPM_BillInvoice:
      case IOPM_Prepay:
      case IOPM_DepositAccount:
      case IOPM_CardInfoPrevSupplied:
      case IOPM_PrivateKnown:
      case IOPM_Invalid:
      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvEsIoPaymentMethod", OutTable, T_Return );
   } /* End write_DbvEsIoPaymentMethod() */


void write_DbvEsIoAdditionalBilling(
		DbvEsIoAdditionalBilling *psEsIoAdditionalBilling, 
		TBL OutTable)
   {

   /* Write the title out */ 
   if(psEsIoAdditionalBilling == NULL)
      {
      write_title_char("NULL","DbvEsIoAdditionalBilling", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoAdditionalBilling", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoAdditionalBilling", 
			OutTable, T_Return );

   /* sPaymentMethod */

   write_DbvEsIoPaymentMethod( &psEsIoAdditionalBilling->sPaymentMethod,
			       OutTable );
   /* psCustomerReference */

   write_DbvInternationalString( "psCustomerReference",
				 psEsIoAdditionalBilling->psCustomerReference,
				 OutTable );

   /* psCustomerPONumber */

   write_DbvInternationalString( "psCustomerPONumber",
				 psEsIoAdditionalBilling->psCustomerPONumber,
				 OutTable );


   write_comment_field( "END of DbvEsIoAdditionalBilling", OutTable, T_Return );
   } /* End write_DbvEsIoAdditionalBilling() */


void write_DbvEsIoContact( DbvEsIoContact *psEsIoContact, TBL OutTable)
   {

   /* Write the title out */ 
   if(psEsIoContact == NULL)
      {
      write_title_char("NULL","DbvEsIoContact", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoContact", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoContact", OutTable, T_Return );

   /* psName */

   write_DbvInternationalString( "psName", psEsIoContact->psName, OutTable );

   /* psPhone */

   write_DbvInternationalString( "psPhone", psEsIoContact->psPhone, OutTable );

   /* psEmail */

   write_DbvInternationalString( "psEmail", psEsIoContact->psEmail, OutTable );

   write_comment_field( "END of DbvEsIoContact", OutTable, T_Return );
   } /* End write_DbvEsIoContact() */


void write_DbvEsIoOriginPartToKeep( 
		DbvEsIoOriginPartToKeep *psEsIoOriginPartToKeep, 
		TBL OutTable)
   {

   /* Write the title out */ 
   if(psEsIoOriginPartToKeep == NULL)
      {
      write_title_char("NULL","DbvEsIoOriginPartToKeep", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoOriginPartToKeep", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoOriginPartToKeep", 
			OutTable, T_Return );

   /* psSupplementalDescription */

   write_DbvASN1External( psEsIoOriginPartToKeep->psSupplementalDescription,
			  OutTable );

   /* psContact */

   write_DbvEsIoContact( psEsIoOriginPartToKeep->psContact, OutTable );

   /* psAdditionalBilling */

   write_DbvEsIoAdditionalBilling( psEsIoOriginPartToKeep->psAdditionalBilling,
				   OutTable );


   write_comment_field( "END of DbvEsIoOriginPartToKeep", OutTable, T_Return );
   } /* End write_DbvEsIoOriginPartToKeep() */


void write_DbvEsIoRequest( DbvEsIoRequest *psEsIoRequest, TBL OutTable)
   {

   /* Write the title out */ 
   if(psEsIoRequest == NULL)
      {
      write_title_char("NULL","DbvEsIoRequest", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoRequest", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoRequest", OutTable, T_Return );

   /* psToKeep */

   write_DbvEsIoOriginPartToKeep( psEsIoRequest->psToKeep, OutTable );

   /* sNotToKeep */

   write_DbvEsIoOriginPartNotToKeep( &psEsIoRequest->sNotToKeep, OutTable );

   write_comment_field( "END of DbvEsIoRequest", OutTable, T_Return );
   } /* End write_DbvEsIoRequest() */


void write_DbvEsIoTaskPackage( DbvEsIoTaskPackage *psEsIoTaskPackage, 
		TBL OutTable)
   {

   /* Write the title out */ 
   if(psEsIoTaskPackage == NULL)
      {
      write_title_char("NULL","DbvEsIoTaskPackage", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEsIoTaskPackage", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEsIoTaskPackage", OutTable, T_Return );

   /* psOriginPart */

   write_DbvEsIoOriginPartToKeep( psEsIoTaskPackage->psOriginPart, OutTable );

   /* sTargetPart */

   write_DbvEsIoTargetPart( &psEsIoTaskPackage->sTargetPart, OutTable );

   write_comment_field( "END of DbvEsIoTaskPackage", OutTable, T_Return );
   } /* End write_DbvEsIoTaskPackage() */


void write_DbvItemOrderExtendedService(
		DbvItemOrderExtendedService *psItemOrderExtendedService, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psItemOrderExtendedService == NULL)
      {
      write_title_char("NULL","DbvItemOrderExtendedService", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psItemOrderExtendedService", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of ItemOrderExtendedService", 
			OutTable, T_Return );

   /* eItemOrderReqRsp */
   convert_DbvItemOrderReqOrRsp( &pc_string, 
				 &psItemOrderExtendedService->eItemOrderReqRsp);
   sprintf( temp, "eItemOrderReqRsp: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uItemOrderReqOrRsp */

   switch( psItemOrderExtendedService->eItemOrderReqRsp )
      {
      case IOROR_Request:
	 write_DbvEsIoRequest( psItemOrderExtendedService->
			       uItemOrderReqOrRsp.psItemOrderRequest,
			       OutTable );
	 break;

      case IOROR_Response:
	 write_DbvEsIoTaskPackage( psItemOrderExtendedService->
				   uItemOrderReqOrRsp.psItemOrderResponse,
				   OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of ItemOrderExtendedService", OutTable, T_Return );
   } /* End write_DbvItemOrderExtendedService() */

