/*  Module Name: @(#)write_DbvFuncs2.c	7.2
    Version: 2/27/97
    Author: Dan Maskell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write structure types to the result script files.
        Used by the origin and target.
 
*/

#include "write_DbvFuncs.h"



extern TBL_Return T_Return;

void write_DbvCommonInfo( DbvCommonInfo *psCommonInfo, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
#ifdef DEBUG
   printf( "D - write_DbvCommonInfo()\n" );
#endif

   /* Write the title out */ 
   if(psCommonInfo == NULL)
      {
      write_title_char("NULL","DbvCommonInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCommonInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCommonInfo", OutTable, T_Return );

   /* pcDateAdded */
   write_DbvGeneralizedTime( "pcDateAdded", psCommonInfo->pcDateAdded,
			     OutTable );
   /* pcDateChanged */
   write_DbvGeneralizedTime( "pcDateChanged", psCommonInfo->pcDateChanged,
			     OutTable );

   /* pcExpiry */
   write_DbvGeneralizedTime( "pcExpiry", psCommonInfo->pcExpiry,
			     OutTable );

   /* psHumanStringLanguage */
   write_DbvLanguageCode( "psHumanStringLanguage", 
			   psCommonInfo->psHumanStringLanguage, 
			   OutTable );

   /* ppsOtherInformation */
   
   if( psCommonInfo->ppsOtherInformation == NULL )
      {
      write_char_field("psOccurrenceByDb->ppsOtherInformation: NULL", 
			OutTable, T_Return);
      TBL_PutRecord( OutTable );
      }
   else
      writeN_DbvOtherInformation( psCommonInfo->ppsOtherInformation , 
				  OutTable);

   write_comment_field( "END of DbvCommonInfo", OutTable, T_Return );
   } /* End write_DbvCommonInfo() */

void write_DbvHumanString( DbvHumanString *psHumanString, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvHumanString()\n" );
#endif

   /* Write the title out */ 
   if(psHumanString == NULL)
      {
      write_title_char("NULL","DbvHumanString", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psHumanString", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvHumanString", OutTable, T_Return );

   /* psLanguage */
   write_DbvLanguageCode( "psLanguage", psHumanString->psLanguage, OutTable );

   /* sText */
   write_DbvInternationalString( "sText", &psHumanString->sText, OutTable );

   write_comment_field( "END of DbvHumanString", OutTable, T_Return );
   } /* End write_DbvHumanString() */

void write_DbvBodyType( DbvBodyType *psBodyType, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvBodyType()\n" );
#endif


   /* Write the title out */ 
   if(psBodyType == NULL)
      {
      write_title_char("NULL","DbvBodyType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psBodyType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvBodyType", OutTable, T_Return );

   /* eBodyTypeType */
   convert_DbvBodyTypeType( &pc_string, &psBodyType->eBodyTypeType);
   sprintf( temp, "eBodyType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uBodyType */
   switch( psBodyType->eBodyTypeType )
      {
      case BTT_IanaType:
	 write_DbvInternationalString( "psIanaType",
				       psBodyType->uBodyType.psIanaType,
				       OutTable );
				 
	 break;

      case BTT_Z3950Type:
	 write_DbvInternationalString( "psIanaType",
				       psBodyType->uBodyType.psZ3950Type,
				       OutTable );
	 break;

      case BTT_OtherType:
	 write_DbvInternationalString( "psOtherType",
				       psBodyType->uBodyType.psOtherType,
				       OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvBodyType", OutTable, T_Return );
   } /* End write_DbvBodyType() */

void write_DbvIconObject( DbvIconObject *psIconObject, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvIconObject()\n" );
#endif

   /* Write the title out */ 
   if(psIconObject == NULL)
      {
      write_title_char("NULL","DbvIconObject", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psIconObject", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvIconObject", OutTable, T_Return );

   /* sBodyType */
   write_DbvBodyType( &psIconObject->sBodyType, OutTable );
				
   /* sContent */
   write_DbvOctetString( "sContent", &psIconObject->sContent, OutTable );

   write_comment_field( "END of DbvIconObject", OutTable, T_Return );
   } /* End write_DbvIconObject() */

void write_DbvContactInfo( DbvContactInfo *psContactInfo, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvContactInfo()\n" );
#endif

   /* Write the title out */ 
   if(psContactInfo == NULL)
      {
      write_title_char("NULL","DbvContactInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psContactInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvContactInfo", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psContactInfo->psName, OutTable );
				
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription",
			  psContactInfo->ppsDescription, OutTable );

   /* ppsAddress */
   writeN_DbvHumanString( "ppsAddress",
			  psContactInfo->ppsAddress, OutTable );

   /* psEmail */
   write_DbvInternationalString( "psEmail", psContactInfo->psEmail, OutTable );

   /* psPhone */
   write_DbvInternationalString( "psEmail", psContactInfo->psPhone, OutTable );

   write_comment_field( "END of DbvContactInfo", OutTable, T_Return );
   } /* End write_DbvContactInfo() */

void write_DbvDatabaseList( DbvDatabaseList *psDatabaseList, TBL OutTable)
   {
   /* Write the title out */ 
   if(psDatabaseList == NULL)
      {
      write_title_char("NULL","DbvDatabaseList", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDatabaseList", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDatabaseList", OutTable, T_Return );

   /* ppsInnerDatabases */
   writeN_DbvInternationalString( "ppsInnerDatabases",
				  psDatabaseList->ppsInnerDatabases,
				  OutTable );

   write_comment_field( "END of DbvDatabaseList", OutTable, T_Return );
   } /* End write_DbvDatabaseList() */

void write_DbvInterAddress( DbvInterAddress *psInterAddress, TBL OutTable)
   {
   char temp[ SML_BUFFER ];
#ifdef DEBUG
   printf( "D - write_DbvInterAddress()\n" );
#endif


   /* Write the title out */ 
   if(psInterAddress == NULL)
      {
      write_title_char("NULL","DbvInterAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psInterAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvInterAddress", OutTable, T_Return );

   /* sHostAddress */
   write_DbvInternationalString( "sHostAddress", 
				 &psInterAddress->sHostAddress,
				 OutTable );
   /* ulPort */

   sprintf( temp, "ulPort: %ld", psInterAddress->ulPort  );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvInterAddress", OutTable, T_Return );
   } /* End write_DbvInterAddress() */

void write_DbvOsiPresentationAddress( 
		DbvOsiPresentationAddress *psOsiPresentationAddress, 
		TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvOsiPresentationAddress()\n" );
#endif

   /* Write the title out */ 
   if(psOsiPresentationAddress == NULL)
      {
      write_title_char("NULL","DbvOsiPresentationAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOsiPresentationAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOsiPresentationAddress", 
			OutTable, T_Return );

   /* sPsel */
   write_DbvInternationalString( "sPsel", &psOsiPresentationAddress->sPsel,
				 OutTable );
   /* psSSel */
   write_DbvInternationalString( "psSSel", psOsiPresentationAddress->psSSel,
				OutTable );
   /* psTSel */
   write_DbvInternationalString( "psTSel", psOsiPresentationAddress->psTSel,
				OutTable );
   /* sNSap */
   write_DbvInternationalString( "sNSap", &psOsiPresentationAddress->sNSap,
				OutTable );

   write_comment_field( "END of DbvOsiPresentationAddress",
			OutTable, T_Return );
   } /* End write_DbvOsiPresentationAddress() */

void write_DbvOtherAddress( DbvOtherAddress *psOtherAddress, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvOtherAddress()\n" );
#endif

   /* Write the title out */ 
   if(psOtherAddress == NULL)
      {
      write_title_char("NULL","DbvOtherAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOtherAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOtherAddress", OutTable, T_Return );

   /* sType */
   write_DbvInternationalString( "sType", &psOtherAddress->sType, OutTable );

   /* sAddress */
   write_DbvInternationalString( "sAddress", &psOtherAddress->sAddress,
				OutTable );

   write_comment_field( "END of DbvOtherAddress",
			OutTable, T_Return );
   } /* End write_DbvOtherAddress() */

void write_DbvNetworkAddress( DbvNetworkAddress *psNetworkAddress, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvNetworkAddress()\n" );
#endif


   /* Write the title out */ 
   if(psNetworkAddress == NULL)
      {
      write_title_char("NULL","DbvNetworkAddress", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psNetworkAddress", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvNetworkAddress", OutTable, T_Return );

   /* eNetworkAddressType */
   convert_DbvNetworkAddressType( &pc_string, 
				  &psNetworkAddress->eNetworkAddressType);
   sprintf( temp, "eNetworkAddress: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uNetworkAddress */

   switch( psNetworkAddress->eNetworkAddressType )
      {
      case NAT_InternetAddress:
	 write_DbvInterAddress( psNetworkAddress->
				uNetworkAddress.psInternetAddress, OutTable );
	 break;

      case NAT_OsiPresentationAddress:
	 write_DbvOsiPresentationAddress( psNetworkAddress->
				uNetworkAddress.psOsiPresentationAddress,
				OutTable );
	 break;

      case NAT_OtherAddress:
	 write_DbvOtherAddress( psNetworkAddress->
				uNetworkAddress.psOtherAddress, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvNetworkAddress", OutTable, T_Return );
   } /* End write_DbvNetworkAddress() */

void write_DbvOperators( DbvOperators *psOperators, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvOperators()\n" );
#endif

   /* Write the title out */ 
   if(psOperators == NULL)
      {
      write_title_char("NULL","DbvOperators", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOperators", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOperators", OutTable, T_Return );

   /* sOperator */
   write_DbvInternationalString( "sOperator", &psOperators->sOperator,
				 OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psOperators->ppsDescription,
				OutTable );

   write_comment_field( "END of DbvOperators", OutTable, T_Return );
   } /* End write_DbvOperators() */

void write_DbvSearchKey( DbvSearchKey *psSearchKey, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvSearchKey()\n" );
#endif

   /* Write the title out */ 
   if(psSearchKey == NULL)
      {
      write_title_char("NULL","DbvSearchKey", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSearchKey", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSearchKey", OutTable, T_Return );

   /* sSearchKey */
   write_DbvInternationalString( "sSearchKey", &psSearchKey->sSearchKey,
				 OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psSearchKey->ppsDescription,
				OutTable );

   write_comment_field( "END of DbvSearchKey", OutTable, T_Return );
   } /* End write_DbvSearchKey() */



void write_DbvHumanStringSeq( DbvHumanStringSeq *psDescription, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvHumanStringSeq()\n" );
#endif

   /* Write the title out */ 
   if(psDescription == NULL)
      {
      write_title_char("NULL","DbvHumanStringSeq", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDescription", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvHumanStringSeq", OutTable, T_Return );

   /* ppsHumanString */
   writeN_DbvHumanString( "ppsHumanString", psDescription->ppsHumanString,
				 OutTable );

   write_comment_field( "END of DbvHumanStringSeq", OutTable, T_Return );
   } /* End write_DbvHumanStringSeq() */


void write_DbvPrivateCapabilities( 
		DbvPrivateCapabilities *psPrivateCapabilities, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D -write_DbvPrivateCapabilities()\n" );
#endif

   /* Write the title out */ 
   if(psPrivateCapabilities == NULL)
      {
      write_title_char("NULL","DbvPrivateCapabilities", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPrivateCapabilities", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPrivateCapabilities", OutTable, T_Return );

   /* ppsOperators */
   writeN_DbvOperators( psPrivateCapabilities->ppsOperators, OutTable );

   /* ppsSearchKeys */
   writeN_DbvSearchKey( psPrivateCapabilities->ppsSearchKeys, OutTable );

   /* ppsDescription */
   writeN_DbvHumanStringSeq( psPrivateCapabilities->ppsDescription, OutTable );

   write_comment_field( "END of DbvPrivateCapabilities", OutTable, T_Return );
   } /* End write_DbvPrivateCapabilities() */

void write_DbvPrivate( DbvPrivate *psPrivate, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
#ifdef DEBUG
   printf( "D - write_DbvPrivate()\n" );
#endif

   /* Write the title out */ 
   if(psPrivate == NULL)
      {
      write_title_char("NULL","DbvPrivate", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPrivate", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPrivate", OutTable, T_Return );

   /* ulUnit */
   sprintf( temp, "ulUnit: %ld", psPrivate->ulUnit );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psPrivate->ppsDescription,
			  OutTable );

   write_comment_field( "END of DbvPrivate", OutTable, T_Return );

   } /* End write_DbvPrivate() */

void write_DbvUnitsSupported( DbvUnitsSupported *psUnitsSupported, 
			      TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvUnitsSupported()\n" );
#endif


   /* Write the title out */ 
   if(psUnitsSupported == NULL)
      {
      write_title_char("NULL","DbvUnitsSupported", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUnitsSupported", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUnitsSupported", OutTable, T_Return );

   /* eUnitsSupportedType */
   convert_DbvUnitsSupportedType( &pc_string,
				  &psUnitsSupported->eUnitsSupportedType);
   sprintf( temp, "eUnitsSupported: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uUnitsSupported */

   switch( psUnitsSupported->eUnitsSupportedType )
      {
      case UST_Known:
         sprintf( temp, "ulKnown: %ld", 
		  psUnitsSupported->uUnitsSupported.ulKnown );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
	 break;

      case UST_Private:
	 write_DbvPrivate( psUnitsSupported->uUnitsSupported.psPrivate, 
				OutTable);
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvUnitsSupported", OutTable, T_Return );
   } /* End write_DbvUnitsSupported() */

void write_DbvProximitySupport( DbvProximitySupport *psProximitySupport, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvProximitySupport()\n" );
#endif

   /* Write the title out */ 
   if(psProximitySupport == NULL)
      {
      write_title_char("NULL","DbvProximitySupport", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psProximitySupport", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvProximitySupport", OutTable, T_Return );

   /* fAnySupport */
   write_DbvBool("fAnySupport", psProximitySupport->fAnySupport, OutTable);
   TBL_PutRecord(OutTable);

   /* ppsUnitsSupported */
   writeN_DbvUnitsSupported( psProximitySupport->ppsUnitsSupported, OutTable);

   write_comment_field( "END of DbvProximitySupport", OutTable, T_Return );
   } /* End write_DbvProximitySupport() */

void write_DbvRpnCapabilities( DbvRpnCapabilities *psRpnCapabilities, 
			       TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvRpnCapabilities()\n" );
#endif

   /* Write the title out */ 
   if(psRpnCapabilities == NULL)
      {
      write_title_char("NULL","DbvRpnCapabilities", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRpnCapabilities", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRpnCapabilities", OutTable, T_Return );

   /* ppulOperators */
   writeN_Ulong( "ppulOperators", psRpnCapabilities->ppulOperators, OutTable );

   /* fResultSetAsOperandSupported */
   write_DbvBool("fResultSetAsOperandSupported",
		 psRpnCapabilities->fResultSetAsOperandSupported,
		 OutTable);
   TBL_PutRecord(OutTable);

   /* fRestrictionOperandSupported */
   write_DbvBool("fRestrictionOperandSupported", 
		 psRpnCapabilities->fRestrictionOperandSupported, 
		 OutTable);
   TBL_PutRecord(OutTable);

   /* psProximity */
   write_DbvProximitySupport( psRpnCapabilities->psProximity, OutTable );

   write_comment_field( "END of DbvRpnCapabilities", OutTable, T_Return );
   } /* End write_DbvRpnCapabilities() */

void write_DbvIso8777Capabilities( 
			DbvIso8777Capabilities *psIso8777Capabilities, 
			TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvIso8777Capabilities()\n" );
#endif

   /* Write the title out */ 
   if(psIso8777Capabilities == NULL)
      {
      write_title_char("NULL","DbvIso8777Capabilities", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psIso8777Capabilities", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvIso8777Capabilities", OutTable, T_Return );

   /* ppsSearchKeys */
   writeN_DbvSearchKey( psIso8777Capabilities->ppsSearchKeys, OutTable );

   /* ppsRestrictions */
   writeN_DbvHumanString( "ppsRestrictions", 
			  psIso8777Capabilities->ppsRestrictions, 
			  OutTable );

   write_comment_field( "END of DbvIso8777Capabilities", OutTable, T_Return );
   } /* End write_DbvIso8777Capabilities() */

void write_DbvQueryTypeDetails( 
		DbvQueryTypeDetails *psQueryTypeDetails, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvQueryTypeDetails()\n" );
#endif

   /* Write the title out */ 
   if(psQueryTypeDetails == NULL)
      {
      write_title_char("NULL","DbvQueryTypeDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psQueryTypeDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvQueryTypeDetails", OutTable, T_Return );

   /* eQueryTypeDetailsType */
   convert_DbvQueryTypeDetailsType( &pc_string, 
				    &psQueryTypeDetails->eQueryTypeDetailsType);
   sprintf( temp, "eQueryTypeDetails: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uQueryTypeDetails */

   switch( psQueryTypeDetails->eQueryTypeDetailsType )
      {
      case QTDT_Private:
	 write_DbvPrivateCapabilities( psQueryTypeDetails->
				       uQueryTypeDetails.psPrivate , OutTable );
	 break;

      case QTDT_Rpn:
	 write_DbvRpnCapabilities( psQueryTypeDetails->
				   uQueryTypeDetails.psRpn, OutTable );
	 break;

      case QTDT_Iso8777:
	 write_DbvIso8777Capabilities( psQueryTypeDetails->
				       uQueryTypeDetails.psIso8777, OutTable );
	 break;

      case QTDT_Z39_58:
	 writeN_DbvHumanString( "ppsZ39_58",
				psQueryTypeDetails->uQueryTypeDetails.ppsZ39_58,
				OutTable );
	 break;

      case QTDT_Erpn:
	 write_DbvRpnCapabilities( psQueryTypeDetails->
				   uQueryTypeDetails.psErpn, OutTable );
	 break;

      case QTDT_RankedList:
	 writeN_DbvHumanString( "ppsRankedList",
			       psQueryTypeDetails->
			       uQueryTypeDetails.ppsRankedList, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvQueryTypeDetails", OutTable, T_Return );
   } /* End write_DbvQueryTypeDetails() */

void write_DbvAccessRestrictions( DbvAccessRestrictions *psAccessRestrictions, 
				  TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvAccessRestrictions()\n" );
#endif

   /* Write the title out */ 
   if(psAccessRestrictions == NULL)
      {
      write_title_char("NULL","DbvAccessRestrictions", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAccessRestrictions", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAccessRestrictions", OutTable, T_Return );

   /* eAccessType */
   convert_DbvAccessType( &pc_string, &psAccessRestrictions->eAccessType);
   sprintf( temp, "eAccessType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* ppsAccessText */
   writeN_DbvHumanString( "ppsAccessText", psAccessRestrictions->ppsAccessText,
			  OutTable );

   /* ppcAccessChallenges */
   writeN_DbvObjectIdentifier( "ppcAccessChallenges", 
				psAccessRestrictions->ppcAccessChallenges, 
				OutTable );

   write_comment_field( "END of DbvAccessRestrictions", OutTable, T_Return );
   } /* End write_DbvAccessRestrictions() */

void write_DbvCharge( DbvCharge *psCharge, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvCharge()\n" );
#endif

   /* Write the title out */ 
   if(psCharge == NULL)
      {
      write_title_char("NULL","DbvCharge", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCharge", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCharge", OutTable, T_Return );

   /* sCost */
   write_DbvIntUnit( &psCharge->sCost, OutTable );

   /* psPerWhat */
   write_DbvUnit( psCharge->psPerWhat, OutTable );

   /* ppsText */
   writeN_DbvHumanString( "ppsText", psCharge->ppsText, OutTable );

   write_comment_field( "END of DbvCharge", OutTable, T_Return );
   } /* End write_DbvCharge() */

void write_DbvOtherCharge( DbvOtherCharge *psOtherCharge, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvOtherCharge()\n" );
#endif

   /* Write the title out */ 
   if(psOtherCharge == NULL)
      {
      write_title_char("NULL","DbvOtherCharge", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOtherCharge", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOtherCharge", OutTable, T_Return );

   /* ppsForWhat */
   writeN_DbvHumanString( "ppsForWhat", psOtherCharge->ppsForWhat, OutTable );

   /* sCharge */
   write_DbvCharge( &psOtherCharge->sCharge, OutTable );

   write_comment_field( "END of DbvOtherCharge", OutTable, T_Return );
   } /* End write_DbvOtherCharge() */

void write_DbvCosts( DbvCosts *psCosts, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvCosts()\n" );
#endif

   /* Write the title out */ 
   if(psCosts == NULL)
      {
      write_title_char("NULL","DbvCosts", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCosts", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCosts", OutTable, T_Return );

   /* psConnectCharge */
   write_DbvCharge( psCosts->psConnectCharge, OutTable );

   /* psConnectTime */
   write_DbvCharge( psCosts->psConnectTime, OutTable );

   /* psDisplayCharge */
   write_DbvCharge( psCosts->psDisplayCharge, OutTable );

   /* psSearchCharge */
   write_DbvCharge( psCosts->psSearchCharge, OutTable );

   /* psSubscriptCharge */
   write_DbvCharge( psCosts->psSubscriptCharge, OutTable );

   /* ppsOtherCharge */
   writeN_DbvOtherCharge( psCosts->ppsOtherCharge, OutTable );

   write_comment_field( "END of DbvCosts", OutTable, T_Return );
   } /* End write_DbvCosts() */

void write_DbvAccessInfo( DbvAccessInfo *psAccessInfo, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvAccessInfo()\n" );
#endif

   /* Write the title out */ 
   if(psAccessInfo == NULL)
      {
      write_title_char("NULL","DbvAccessInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAccessInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAccessInfo", OutTable, T_Return );

   /* ppsQueryTypesSupported */
   writeN_DbvQueryTypeDetails( psAccessInfo->ppsQueryTypesSupported, OutTable );

   /* ppcDiagnosticsSets */
   writeN_DbvObjectIdentifier( "ppcDiagnosticsSets", 
				psAccessInfo->ppcDiagnosticsSets, 
				OutTable );

   /* ppcAttributeSetIds */
   writeN_DbvObjectIdentifier( "ppcAttributeSetIds",
				psAccessInfo->ppcAttributeSetIds, 
				OutTable );

   /* ppcSchemas */
   writeN_DbvObjectIdentifier( "ppcSchemas", psAccessInfo->ppcSchemas, 
				OutTable );

   /* ppcRecordSyntaxes */
   writeN_DbvObjectIdentifier( "ppcRecordSyntaxes", 
				psAccessInfo->ppcRecordSyntaxes, 
				OutTable );

   /* ppcResourceChallenges */
   writeN_DbvObjectIdentifier( "ppcResourceChallenges",
				psAccessInfo->ppcResourceChallenges, 
				OutTable );

   /* ppsRestrictedAccess */
   writeN_DbvAccessRestrictions( psAccessInfo->ppsRestrictedAccess, OutTable );

   /* psCostInfo */
   write_DbvCosts( psAccessInfo->psCostInfo, OutTable );

   /* ppcVariantSets */
   writeN_DbvObjectIdentifier( "ppcVariantSets", 
				psAccessInfo->ppcVariantSets, OutTable );

   /* ppsElementSetNames */
   writeN_DbvInternationalString( "ppsElementSetNames",
				  psAccessInfo->ppsElementSetNames,
				  OutTable );
   /* ppsUnitSystems */
   writeN_DbvInternationalString( "ppsUnitSystems", 
				  psAccessInfo->ppsUnitSystems,
				  OutTable );

   write_comment_field( "END of DbvAccessInfo", OutTable, T_Return );
   } /* End write_DbvAccessInfo() */

void write_DbvTargetInfo( DbvTargetInfo *psTargetInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTargetInfo()\n" );
#endif

   /* Write the title out */ 
   if(psTargetInfo == NULL)
      {
      write_title_char("NULL","DbvTargetInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTargetInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTargetInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psTargetInfo->psCommonInfo, OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psTargetInfo->sName,
				  OutTable );
   /* ppsRecentNews */
   writeN_DbvHumanString( "ppsRecentNews", psTargetInfo->ppsRecentNews,
			  OutTable );

   /* ppsIcon */
   writeN_DbvIconObject( psTargetInfo->ppsIcon, OutTable );

   /* fNamedResultSets */
   write_DbvBool("fNamedResultSets", psTargetInfo->fNamedResultSets, 
			OutTable);
   TBL_PutRecord(OutTable);

   /* fMultipleDBSearch */
   write_DbvBool("fMultipleDBSearch", psTargetInfo->fMultipleDBSearch,
			OutTable);
   TBL_PutRecord(OutTable);

   /* pulMaxResultSets */
   write_integer_string("pulMaxResultSets", 
			(int *)psTargetInfo->pulMaxResultSets, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulMaxResultSize */
   write_integer_string("pulMaxResultSize", 
			(int *)psTargetInfo->pulMaxResultSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulMaxTerms */
   write_integer_string("pulMaxTerms", (int *)psTargetInfo->pulMaxTerms, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* psTimeoutInterval */
   write_DbvIntUnit( psTargetInfo->psTimeoutInterval, OutTable );

   /* ppsWelcomeMessage */
   writeN_DbvHumanString( "ppsWelcomeMessage", 
			  psTargetInfo->ppsWelcomeMessage,
			  OutTable );

   /* psContactInfo */
   write_DbvContactInfo( psTargetInfo->psContactInfo, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psTargetInfo->ppsDescription,
			  OutTable );

   /* ppsNicknames */
   writeN_DbvInternationalString( "ppsNicknames",
			  psTargetInfo->ppsNicknames,
			  OutTable );
   /* ppsUsageRest */
   writeN_DbvHumanString( "ppsUsageRest", psTargetInfo->ppsUsageRest,
			  OutTable );

   /* ppsPaymentAddr */
   writeN_DbvHumanString( "ppsPaymentAddr", psTargetInfo->ppsPaymentAddr,
			  OutTable );

   /* ppsHours */
   writeN_DbvHumanString( "ppsHours", psTargetInfo->ppsHours,
			  OutTable );

   /* ppsDbCombinations */
   writeN_DbvDatabaseList( psTargetInfo->ppsDbCombinations,
			   OutTable );
   /* ppsAddresses */
   writeN_DbvNetworkAddress( psTargetInfo->ppsAddresses, OutTable );

   /* ppsLanguages */
   writeN_DbvInternationalString( "ppsLanguages",
			  psTargetInfo->ppsLanguages,
			  OutTable );

   /* psCommonAccessInfo */
   write_DbvAccessInfo( psTargetInfo->psCommonAccessInfo, OutTable );

   write_comment_field( "END of DbvTargetInfo", OutTable, T_Return );
   } /* End write_DbvTargetInfo() */

void write_DbvRecordCount( DbvRecordCount *psRecordCount, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvRecordCount()\n" );
#endif

   /* Write the title out */ 
   if(psRecordCount == NULL)
      {
      write_title_char("NULL","DbvRecordCount", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRecordCount", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRecordCount", OutTable, T_Return );

   /* eRecordCountType */
   convert_DbvRecordCountType( &pc_string, &psRecordCount->eRecordCountType);
   sprintf( temp, "eRecordCount: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uRecordCount */

   switch( psRecordCount->eRecordCountType )
      {
      case RCT_ActualNumber:
         sprintf( temp, "ulActualNumber: %ld",
			psRecordCount->uRecordCount.ulActualNumber  );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
	 break;

      case RCT_ApproxNumber:
         sprintf( temp, "ulApproxNumber: %ld",
			psRecordCount->uRecordCount.ulApproxNumber  );
         write_char_field(temp, OutTable, T_Return);
         TBL_PutRecord( OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvRecordCount", OutTable, T_Return );
   } /* End write_DbvRecordCount() */

void write_DbvDatabaseInfo( DbvDatabaseInfo *psDatabaseInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvDatabaseInfo()\n" );
#endif

   /* Write the title out */ 
   if(psDatabaseInfo == NULL)
      {
      write_title_char("NULL","DbvDatabaseInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDatabaseInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDatabaseInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psDatabaseInfo->psCommonInfo, OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psDatabaseInfo->sName,
				  OutTable );

   /* pvExplainDatabase */
   write_DbvNull( "pvExplainDatabase",
		  psDatabaseInfo->pvExplainDatabase, OutTable );

   /* ppsNicknames */
   writeN_DbvInternationalString( "ppsNicknames",
				psDatabaseInfo->ppsNicknames, OutTable );

   /* ppsIcon */
   writeN_DbvIconObject( psDatabaseInfo->ppsIcon, OutTable );

   /* fUserFee */
   write_DbvBool("fUserFee", psDatabaseInfo->fUserFee, OutTable);
   TBL_PutRecord(OutTable);

   /* fAvailable */
   write_DbvBool("fAvailable", psDatabaseInfo->fAvailable, OutTable);
   TBL_PutRecord(OutTable);

   /* ppsTitleString */
   writeN_DbvHumanString( "ppsTitleString", psDatabaseInfo->ppsTitleString,
			  OutTable );
   /* ppsKeywords */
   writeN_DbvDataBaseInfoSeq( psDatabaseInfo->ppsKeywords, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psDatabaseInfo->ppsDescription,
			  OutTable );

   /* ppsAssociatedDbs */
   writeN_DbvInternationalString( "ppsAssociatedDbs",
				psDatabaseInfo->ppsAssociatedDbs, OutTable );
   /* ppsSubDbs */
   writeN_DbvInternationalString( "ppsSubDbs",
				psDatabaseInfo->ppsSubDbs, OutTable );
   /* ppsDisclaimers */
   writeN_DbvHumanString( "ppsDisclaimers", psDatabaseInfo->ppsDisclaimers,
			  OutTable );

   /* ppsNews */
   writeN_DbvHumanString( "ppsNews", psDatabaseInfo->ppsNews, OutTable );

   /* psRecordCount */
   write_DbvRecordCount( psDatabaseInfo->psRecordCount, OutTable );

   /* ppsDefaultOrder */
   writeN_DbvHumanString( "ppsDefaultOrder", psDatabaseInfo->ppsDefaultOrder,
			  OutTable );

   /* pulAvRecordSize */
   write_integer_string("pulAvRecordSize", 
			(int *)psDatabaseInfo->pulAvRecordSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulMaxRecordSize */
   write_integer_string("pulMaxRecordSize", 
			(int *)psDatabaseInfo->pulMaxRecordSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* ppsHours */
   writeN_DbvHumanString( "ppsHours", psDatabaseInfo->ppsHours, OutTable );

   /* ppsBestTime */
   writeN_DbvHumanString( "ppsBestTime",psDatabaseInfo->ppsBestTime, OutTable );

   /* pcLastUpdate */
   write_DbvGeneralizedTime( "pcLastUpdate", psDatabaseInfo->pcLastUpdate, 
			     OutTable );

   /* psUpdateInterval */
   write_DbvIntUnit( psDatabaseInfo->psUpdateInterval, OutTable );

   /* ppsCoverage */
   writeN_DbvHumanString( "ppsCoverage",psDatabaseInfo->ppsCoverage, OutTable );

   /* pfProprietary */
   if ( psDatabaseInfo->pfProprietary )
      write_DbvBool("pfProprietary", *psDatabaseInfo->pfProprietary, OutTable);
   else
      write_title_char("NULL","pfProprietary", OutTable);
   TBL_PutRecord(OutTable);

   /* ppsCopyrightText */
   writeN_DbvHumanString( "ppsCopyrightText", psDatabaseInfo->ppsCopyrightText,
				OutTable );
   /* ppsCopyrightNotice */
   writeN_DbvHumanString( "ppsCopyrightNotice",
				psDatabaseInfo->ppsCopyrightNotice,
				OutTable );
   /* psProducerContactInfo */
   write_DbvContactInfo( psDatabaseInfo->psProducerContactInfo, OutTable );

   /* psSupplierContactInfo */
   write_DbvContactInfo( psDatabaseInfo->psSupplierContactInfo, OutTable );

   /* psSubmissionContactInfo */
   write_DbvContactInfo( psDatabaseInfo->psSubmissionContactInfo, OutTable );

   /* psAccessInfo */
   write_DbvAccessInfo( psDatabaseInfo->psAccessInfo, OutTable );

   write_comment_field( "END of DbvDatabaseInfo", OutTable, T_Return );
   } /* End write_DbvDatabaseInfo() */

void write_DbvTagTypeMapping( DbvTagTypeMapping *psTagTypeMapping, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTagTypeMapping()\n" );
#endif

   /* Write the title out */ 
   if(psTagTypeMapping == NULL)
      {
      write_title_char("NULL","DbvTagTypeMapping", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTagTypeMapping", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTagTypeMapping", OutTable, T_Return );

   /* ulTagType */
   write_integer_string("ulTagType", (int *)&psTagTypeMapping->ulTagType, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pcTagSet */
   write_DbvObjectIdentifier( "pcTagSet", psTagTypeMapping->pcTagSet, OutTable);

   /* pvDefaultTagType */
   write_DbvNull( "pvDefaultTagType",
		  psTagTypeMapping->pvDefaultTagType, OutTable );

   write_comment_field( "END of DbvTagTypeMapping", OutTable, T_Return );
   } /* End write_DbvTagTypeMapping() */

void write_DbvPath( DbvPath *psPath, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvPath()\n" );
#endif

   /* Write the title out */ 
   if(psPath == NULL)
      {
      write_title_char("NULL","DbvPath", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPath", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPath", OutTable, T_Return );

   /* ulTagType */
   write_integer_string("ulTagType", (int *)&psPath->ulTagType, OutTable);
   TBL_PutRecord( OutTable );

   /* sTagValue */
   write_DbvStringOrNumeric( &psPath->sTagValue, OutTable);

   write_comment_field( "END of DbvPath", OutTable, T_Return );
   } /* End write_DbvPath() */

void write_DbvPathA( DbvPathA *psPathA, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvPathA()\n" );
#endif

   /* Write the title out */ 
   if(psPathA == NULL)
      {
      write_title_char("NULL","DbvPathA", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("ppsSchemaTags", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPathA", OutTable, T_Return );

   /* ppsSchemaTags */
   writeN_DbvPath( psPathA->ppsSchemaTags, OutTable );

   write_comment_field( "END of DbvPathA", OutTable, T_Return );
   } /* End write_DbvPathA() */

void write_DbvElementInfo( DbvElementInfo *psElementInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvElementInfo()\n" );
#endif

   /* Write the title out */ 
   if(psElementInfo == NULL)
      {
      write_title_char("NULL","DbvElementInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElementInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementInfo", OutTable, T_Return );

   /* sElementName */
   write_DbvInternationalString( "sElementName", &psElementInfo->sElementName,
				OutTable );

   /* ppsElementTagPath */
   writeN_DbvPath( psElementInfo->ppsElementTagPath, OutTable );

   /* psDataType */
   write_DbvElementDataType( psElementInfo->psDataType, OutTable );

   /* fRequired */
   write_DbvBool("fRequired", psElementInfo->fRequired, OutTable);
   TBL_PutRecord(OutTable);

   /* fRepeatable */
   write_DbvBool("fRepeatable", psElementInfo->fRepeatable, OutTable);
   TBL_PutRecord(OutTable);

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psElementInfo->ppsDescription,
		OutTable );

   write_comment_field( "END of DbvElementInfo", OutTable, T_Return );
   } /* End write_DbvElementInfo() */

void write_DbvElementDataType( DbvElementDataType *psElementDataType, 
				TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvElementDataType()\n" );
#endif

   /* Write the title out */ 
   if(psElementDataType == NULL)
      {
      write_title_char("NULL","DbvElementDataType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElementDataType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementDataType", OutTable, T_Return );

   /* eElementDataTypeType */
   convert_DbvElementDataTypeType( &pc_string, 
				   &psElementDataType->eElementDataTypeType);
   sprintf( temp, "eElementDataType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uElementDataType */

   switch( psElementDataType->eElementDataTypeType )
      {
      case EDTT_Primitive:
	 write_DbvPrimitiveDataType( "ePrimitive", 
				     &psElementDataType->
				     uElementDataType.ePrimitive, OutTable );
	 break;

      case EDTT_Structured:
	 writeN_DbvElementInfo( psElementDataType->
				uElementDataType.ppsStructured, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvElementDataType", OutTable, T_Return );
   } /* End write_DbvElementDataType() */

void write_DbvSchemaInfo( DbvSchemaInfo *psSchemaInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvSchemaInfo()\n" );
#endif

   /* Write the title out */ 
   if(psSchemaInfo == NULL)
      {
      write_title_char("NULL","DbvSchemaInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSchemaInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSchemaInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psSchemaInfo->psCommonInfo, OutTable );

   /* pcSchema */
   write_DbvObjectIdentifier( "pcSchema", psSchemaInfo->pcSchema, OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psSchemaInfo->sName, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psSchemaInfo->ppsDescription,
			  OutTable );

   /* ppsTagTypeMapping */
   writeN_DbvTagTypeMapping( psSchemaInfo->ppsTagTypeMapping, OutTable );

   /* ppsRecordStructure */
   writeN_DbvElementInfo( psSchemaInfo->ppsRecordStructure,
			OutTable );

   write_comment_field( "END of DbvSchemaInfo", OutTable, T_Return );
   } /* End write_DbvSchemaInfo() */

void write_DbvElements( DbvElements *psElements, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvElements()\n" );
#endif

   /* Write the title out */ 
   if(psElements == NULL)
      {
      write_title_char("NULL","DbvElements", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElements", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElements", OutTable, T_Return );

   /* sElementName */
   write_DbvInternationalString( "sElementName", 
				 &psElements->sElementName, OutTable ); 
   /* ppsNicknames */
   writeN_DbvInternationalString( "ppsNicknames", 
				  psElements->ppsNicknames,
				  OutTable );
   /* sElementTag */
   write_DbvStringOrNumeric( &psElements->sElementTag, OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psElements->ppsDescription,
			  OutTable );

   /* peDataType */
   write_DbvPrimitiveDataType( "peDataType",psElements->peDataType, OutTable );

   /* ppsOtherInformation */
   writeN_DbvOtherInformation( psElements->ppsOtherInformation, OutTable );

   write_comment_field( "END of DbvElements", OutTable, T_Return );
   } /* End write_DbvElements() */

void write_DbvPrimitiveDataType( char *name,
				 DbvPrimitiveDataType *pePrimitiveDataType, 
				 TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvPrimitiveDataType()\n" );
#endif

   /* Write the title out */ 
   if(pePrimitiveDataType == NULL)
      {
      write_title_char("NULL","DbvPrimitiveDataType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* ePrimitiveDataType */
   convert_DbvPrimitiveDataType( &pc_string, pePrimitiveDataType );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvPrimitiveDataType() */

void write_DbvTagSetInfo( DbvTagSetInfo *psTagSetInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTagSetInfo()\n" );
#endif

   /* Write the title out */ 
   if(psTagSetInfo == NULL)
      {
      write_title_char("NULL","DbvTagSetInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTagSetInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTagSetInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psTagSetInfo->psCommonInfo, OutTable );

   /* pcTagSet */
   write_DbvObjectIdentifier( "pcTagSet", psTagSetInfo->pcTagSet, OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psTagSetInfo->sName, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psTagSetInfo->ppsDescription,
			  OutTable );

   /* ppsElements */
   writeN_DbvElements( psTagSetInfo->ppsElements, OutTable );

   write_comment_field( "END of DbvTagSetInfo", OutTable, T_Return );
   } /* End write_DbvTagSetInfo() */

void write_DbvRecordSyntaxInfo( DbvRecordSyntaxInfo *psRecordSyntaxInfo, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvRecordSyntaxInfo()\n" );
#endif

   /* Write the title out */ 
   if(psRecordSyntaxInfo == NULL)
      {
      write_title_char("NULL","DbvRecordSyntaxInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRecordSyntaxInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRecordSyntaxInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psRecordSyntaxInfo->psCommonInfo, OutTable );

   /* pcRecordSyntax */
   write_DbvObjectIdentifier( "pcRecordSyntax",
			      psRecordSyntaxInfo->pcRecordSyntax, OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psRecordSyntaxInfo->sName,OutTable );

   /* ppcTransferSyntaxes */
   writeN_DbvObjectIdentifier( "ppcTransferSyntaxes", 
			       psRecordSyntaxInfo->ppcTransferSyntaxes,
			       OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psRecordSyntaxInfo->ppsDescription,
			  OutTable );

   /* psAsn1Module */
   write_DbvInternationalString( "psAsn1Module", 
				  psRecordSyntaxInfo->psAsn1Module,
				  OutTable );
   /* ppsAbstractStructure */

   writeN_DbvElementInfo( psRecordSyntaxInfo->ppsAbstractStructure, OutTable );

   write_comment_field( "END of DbvRecordSyntaxInfo", OutTable, T_Return );
   } /* End write_DbvRecordSyntaxInfo() */

void write_DbvAttributeDescription( 
			DbvAttributeDescription *psAttributeDescription, 
			TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeDescription()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeDescription == NULL)
      {
      write_title_char("NULL","DbvAttributeDescription", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeDescription", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeDescription",
						OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", 
				psAttributeDescription->psName,OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", 
			  psAttributeDescription->ppsDescription,
			  OutTable );

   /* sAttributeValue */
   write_DbvStringOrNumeric( &psAttributeDescription->sAttributeValue,
				  OutTable );
   /* ppsEquivalentAttributes */
   writeN_DbvStringOrNumeric( psAttributeDescription->ppsEquivalentAttributes,
				OutTable );

   write_comment_field( "END of DbvAttributeDescription", OutTable, T_Return );
   } /* End write_DbvAttributeDescription() */

void write_DbvAttributeType( DbvAttributeType *psAttributeType, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeType()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeType == NULL)
      {
      write_title_char("NULL","DbvAttributeType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeType", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psAttributeType->psName, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psAttributeType->ppsDescription,
			  OutTable );
   /* ulAttributeType */

   write_integer_string( "ulAttributeType", 
			(int*)&psAttributeType->ulAttributeType,
			OutTable );
   TBL_PutRecord( OutTable );

   /* ppsAttributeValues */
   writeN_DbvAttributeDescription( psAttributeType->ppsAttributeValues,
			       OutTable );

   write_comment_field( "END of DbvAttributeType", OutTable, T_Return );
   } /* End write_DbvAttributeType() */

void write_DbvAttributeSetInfo( DbvAttributeSetInfo *psAttributeSetInfo, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeSetInfo()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeSetInfo == NULL)
      {
      write_title_char("NULL","DbvAttributeSetInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeSetInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeSetInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psAttributeSetInfo->psCommonInfo, OutTable );

   /* pcAttributeSetId */
   write_DbvObjectIdentifier( "pcAttributeSetId", 
				psAttributeSetInfo->pcAttributeSetId,
				OutTable );
   /* sName */
   write_DbvInternationalString( "sName", &psAttributeSetInfo->sName,
			  OutTable );
   /* ppsAttributes */
   writeN_DbvAttributeType( psAttributeSetInfo->ppsAttributes, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psAttributeSetInfo->ppsDescription,
			  OutTable );

   write_comment_field( "END of DbvAttributeSetInfo", OutTable, T_Return );
   } /* End write_DbvAttributeSetInfo() */

void write_DbvTermList( DbvTermList *psTermList, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTermList()\n" );
#endif

   /* Write the title out */ 
   if(psTermList == NULL)
      {
      write_title_char("NULL","DbvTermList", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTermList", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTermList", OutTable, T_Return );

   /* sName */
   write_DbvInternationalString( "sName", &psTermList->sName, OutTable );

   /* ppsTitle */
   writeN_DbvHumanString( "ppsTitle", psTermList->ppsTitle, OutTable );

   /* peSearchCost */
   write_DbvSearchCost( "peSearchCost", psTermList->peSearchCost, OutTable );

   /* fScanable */
   write_DbvBool("fScanable", psTermList->fScanable, OutTable);
   TBL_PutRecord(OutTable);

   /* ppsBroader */
   writeN_DbvInternationalString( "ppsBroader", psTermList->ppsBroader,
			  OutTable );
   /* ppsNarrower */
   writeN_DbvInternationalString( "ppsNarrower", psTermList->ppsNarrower,
			  OutTable );

   write_comment_field( "END of DbvTermList", OutTable, T_Return );
   } /* End write_DbvTermList() */

void write_DbvTermListInfo( DbvTermListInfo *psTermListInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTermListInfo()\n" );
#endif

   /* Write the title out */ 
   if(psTermListInfo == NULL)
      {
      write_title_char("NULL","DbvTermListInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTermListInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTermListInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psTermListInfo->psCommonInfo, OutTable );

   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", 
				 &psTermListInfo->sDatabaseName,
				 OutTable );
   /* ppsTermLists */
   writeN_DbvTermList( psTermListInfo->ppsTermLists, OutTable );

   write_comment_field( "END of DbvTermListInfo", OutTable, T_Return );
   } /* End write_DbvTermListInfo() */

void write_DbvExtendedServicesInfo( 
			DbvExtendedServicesInfo *psExtendedServicesInfo, 
			TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvExtendedServicesInfo()\n" );
#endif

   /* Write the title out */ 
   if(psExtendedServicesInfo == NULL)
      {
      write_title_char("NULL","DbvExtendedServicesInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psExtendedServicesInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvExtendedServicesInfo", 
							OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psExtendedServicesInfo->psCommonInfo, OutTable );

   /* pcType */
    write_DbvObjectIdentifier( "pcType", psExtendedServicesInfo->pcType,
				OutTable );

   /* psName */
   write_DbvInternationalString( "psName", psExtendedServicesInfo->psName,
				  OutTable );

   /* fPrivateType */
   write_DbvBool("fPrivateType", psExtendedServicesInfo->fPrivateType, 
			OutTable);
   TBL_PutRecord(OutTable);

   /* fRestrictionsApply */
   write_DbvBool("fRestrictionsApply", 
			psExtendedServicesInfo->fRestrictionsApply, 
			OutTable);
   TBL_PutRecord(OutTable);

   /* fFeeApply */
   write_DbvBool("fFeeApply", psExtendedServicesInfo->fFeeApply, OutTable);
   TBL_PutRecord(OutTable);

   /* fAvailable */
   write_DbvBool("fAvailable", psExtendedServicesInfo->fAvailable, OutTable);
   TBL_PutRecord(OutTable);

   /* fRetentionSupported */
   write_DbvBool("fRetentionSupported",
			psExtendedServicesInfo->fRetentionSupported, 
			OutTable);
   TBL_PutRecord(OutTable);

   /* eWaitAction */
   write_DbvExtendedServicesInfoInt( &psExtendedServicesInfo->eWaitAction,
					OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", 
			  psExtendedServicesInfo->ppsDescription,
			  OutTable );

   /* psSpecificExplain */
   write_DbvASN1External( psExtendedServicesInfo->psSpecificExplain, OutTable );

   /* psEsASN */
   write_DbvInternationalString( "psEsASN", psExtendedServicesInfo->psEsASN,
				  OutTable );

   write_comment_field( "END of DbvExtendedServicesInfo", OutTable, T_Return );
   } /* End write_DbvExtendedServicesInfo() */

void write_DbvExtendedServicesInfoInt( 
	DbvExtendedServicesInfoInt *peExtendedServicesInfoInt, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvExtendedServicesInfoInt()\n" );
#endif

   /* Write the title out */ 
   if(peExtendedServicesInfoInt == NULL)
      {
      write_title_char("NULL","DbvExtendedServicesInfoInt", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eExtendedServicesInfoInt */
   convert_DbvExtendedServicesInfoInt( &pc_string, peExtendedServicesInfoInt );
   sprintf( temp, "eExtendedServicesInfoInt : %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvExtendedServicesInfoInt() */
void write_DbvOmittedAttributeInterpretation( 
	DbvOmittedAttributeInterpretation *psOmittedAttributeInterpretation, 
	TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvOmittedAttributeInterpretation()\n" );
#endif

   /* Write the title out */ 
   if(psOmittedAttributeInterpretation == NULL)
      {
      write_title_char("NULL","DbvOmittedAttributeInterpretation", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOmittedAttributeInterpretation", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOmittedAttributeInterpretation", 
				OutTable, T_Return );

   /* psDefaultValue */
   write_DbvStringOrNumeric( psOmittedAttributeInterpretation->psDefaultValue,
				OutTable );
   /* ppsDefaultDescription */
   writeN_DbvHumanString( "ppsDefaultDescription", 
		  psOmittedAttributeInterpretation->ppsDefaultDescription,
		  OutTable );

   write_comment_field( "END of DbvOmittedAttributeInterpretation",
		OutTable, T_Return );
   } /* End write_DbvOmittedAttributeInterpretation() */

void write_DbvSearchCost( char* name, DbvSearchCost *peSearchCost, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvSearchCost()\n" );
#endif

   /* Write the title out */ 
   if(peSearchCost == NULL)
      {
      write_title_char("NULL","DbvSearchCost", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eSearchCost */
   convert_DbvSearchCost( &pc_string, peSearchCost );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvSearchCost() */

void write_DbvAttributeValues( DbvAttributeValues *psAttributeValues, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeValues()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeValues == NULL)
      {
      write_title_char("NULL","DbvAttributeValues", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeValues", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeValues", OutTable, T_Return );

   /* sValue */
   write_DbvStringOrNumeric( &psAttributeValues->sValue, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psAttributeValues->ppsDescription,
			  OutTable );

   /* ppsSubAttributes */
   writeN_DbvStringOrNumeric( psAttributeValues->ppsSubAttributes, OutTable );

   /* ppsSuperAttributes */
   writeN_DbvStringOrNumeric( psAttributeValues->ppsSuperAttributes, OutTable );

   /* pvPartialSupport */
   write_DbvNull( "pvPartialSupport",
		  psAttributeValues->pvPartialSupport, OutTable );

   write_comment_field( "END of DbvAttributeValues", OutTable, T_Return );
   } /* End write_DbvAttributeValues() */

void write_DbvAttributesByType( DbvAttributesByType *psAttributesByType, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributesByType()\n" );
#endif

   /* Write the title out */ 
   if(psAttributesByType == NULL)
      {
      write_title_char("NULL","DbvAttributesByType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributesByType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributesByType", OutTable, T_Return );

   /* ulAttributeType */
   write_integer_string("ulAttributeType", 
			(int *)&psAttributesByType->ulAttributeType, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* psDefaultIfOmitted */
   write_DbvOmittedAttributeInterpretation( psAttributesByType->psDefaultIfOmitted,
					    OutTable);

   /* ppsAttributeValues */
   writeN_DbvAttributeValues( psAttributesByType->ppsAttributeValues, OutTable);

   write_comment_field( "END of DbvAttributesByType", OutTable, T_Return );
   } /* End write_DbvAttributesByType() */

void write_DbvAttributesBySet( DbvAttributesBySet *psAttributesBySet, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributesBySet()\n" );
#endif

   /* Write the title out */ 
   if(psAttributesBySet == NULL)
      {
      write_title_char("NULL","DbvAttributesBySet", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributesBySet", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributesBySet", OutTable, T_Return );

   /* pcAttributeSet */
   write_DbvObjectIdentifier( "pcAttributeSet", 
			      psAttributesBySet->pcAttributeSet,
			      OutTable);
   /* ppsAttributesByType */
   writeN_DbvAttributesByType( psAttributesBySet->ppsAttributesByType, 
				OutTable);

   write_comment_field( "END of DbvAttributesBySet", OutTable, T_Return );
   } /* End write_DbvAttributesBySet() */

void write_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice, 
		TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psAttributeOccurrenceChoice == NULL)
      {
      write_title_char("NULL","DbvAttributeOccurrenceChoice", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeOccurrenceChoice", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeOccurrenceChoice", 
							OutTable, T_Return );

   /* eAttributeOccurrenceChoiceType */
   convert_DbvAttributeOccurrenceChoiceType( &pc_string,
		&psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType);
   sprintf( temp, "eAttributeOccurrenceChoiceType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uAttributeOccurrenceChoice */

   switch( psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType )
      {
      case AOT_AnyOrNone:
	 write_DbvNull( "pvAnyOrNone",
			psAttributeOccurrenceChoice->
			uAttributeOccurrenceChoiceType.pvAnyOrNone, OutTable );
	 break;

      case AOT_Specific:
	 writeN_DbvStringOrNumeric( psAttributeOccurrenceChoice->
			uAttributeOccurrenceChoiceType.ppsSpecific, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvAttributeOccurrenceChoice", 
							OutTable, T_Return );
   } /* End write_DbvAttributeOccurrenceChoice() */

void write_DbvAttributeOccurrence( 
		DbvAttributeOccurrence *psAttributeOccurrence, 
		TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeOccurrence()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeOccurrence == NULL)
      {
      write_title_char("NULL","DbvAttributeOccurrence", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeOccurrence", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeOccurrence", OutTable, T_Return );

   /* pcAttributeSet */
   write_DbvObjectIdentifier( "pcAttributeSet", 
				psAttributeOccurrence->pcAttributeSet, 
				OutTable);

   /* ulAttributeType */
   write_integer_string( "ulAttributeType", 
			 (int *)&psAttributeOccurrence->ulAttributeType, 
			 OutTable);
   TBL_PutRecord( OutTable );

   /* pvMustBeSupplied */
   write_DbvNull( "pvMustBeSupplied",
		  psAttributeOccurrence->pvMustBeSupplied, OutTable );

   /* sAttributeValues */
   write_DbvAttributeOccurrenceChoice( &psAttributeOccurrence->sAttributeValues,
				       OutTable );

   write_comment_field( "END of DbvAttributeOccurrence", OutTable, T_Return );
   } /* End write_DbvAttributeOccurrence() */

void write_DbvAttributeCombination( 
		DbvAttributeCombination *psAttributeCombination, TBL OutTable)
   {

   /* Write the title out */ 
   if(psAttributeCombination == NULL)
      {
      write_title_char("NULL","DbvAttributeCombination", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeCombination", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeCombination", 
							OutTable, T_Return );

   /* ppsInnerDatabases */
   writeN_DbvAttributeOccurrence( psAttributeCombination->
				  ppsAttributeOccurrence,
				  OutTable );

   write_comment_field( "END of DbvAttributeCombination", OutTable, T_Return );
   } /* End write_DbvAttributeCombination() */

void write_DbvAttributeCombinations( 
			DbvAttributeCombinations *psAttributeCombinations, 
			TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeCombinations()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeCombinations == NULL)
      {
      write_title_char("NULL","DbvAttributeCombinations", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeCombinations", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeCombinations", 
				OutTable, T_Return );

   /* pcDefaultAttributeSet */
   write_DbvObjectIdentifier( "pcDefaultAttributeSet", 
				psAttributeCombinations->pcDefaultAttributeSet,
				OutTable );


   /* ppsLegalCombinations */
   writeN_DbvAttributeCombination( psAttributeCombinations->
				ppsLegalCombinations,
				OutTable );

   write_comment_field( "END of DbvAttributeCombinations", OutTable, T_Return );
   } /* End write_DbvAttributeCombinations() */

void write_DbvAttributeDetails( DbvAttributeDetails *psAttributeDetails, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvAttributeDetails()\n" );
#endif

   /* Write the title out */ 
   if(psAttributeDetails == NULL)
      {
      write_title_char("NULL","DbvAttributeDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAttributeDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAttributeDetails", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psAttributeDetails->psCommonInfo, OutTable );


   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", 
				  &psAttributeDetails->sDatabaseName,
				  OutTable );
   /* ppsAttributesBySet */

   writeN_DbvAttributesBySet( psAttributeDetails->ppsAttributesBySet, 
				OutTable );

   /* psAttributeCombinations */
   write_DbvAttributeCombinations( psAttributeDetails->psAttributeCombinations,
				OutTable );

   write_comment_field( "END of DbvAttributeDetails", OutTable, T_Return );
   } /* End write_DbvAttributeDetails() */

void write_DbvScanInfo( DbvScanInfo *psScanInfo, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvScanInfo()\n" );
#endif

   /* Write the title out */ 
   if(psScanInfo == NULL)
      {
      write_title_char("NULL","DbvScanInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psScanInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvScanInfo", OutTable, T_Return );

   /* pulMaxStepSize */
   write_integer_string("pulMaxStepSize", (int *)psScanInfo->pulMaxStepSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* ppsCollatingSequence */
   writeN_DbvHumanString( "ppsCollatingSequence",
			  psScanInfo->ppsCollatingSequence,
			  OutTable );
   /* pfIncreasing */
   if ( psScanInfo->pfIncreasing )
      write_DbvBool("pfIncreasing", *psScanInfo->pfIncreasing, OutTable);
   else
      write_title_char("NULL","pfIncreasing", OutTable);
   TBL_PutRecord(OutTable);

   write_comment_field( "END of DbvScanInfo", OutTable, T_Return );
   } /* End write_DbvScanInfo() */

void write_DbvTermListDetails( DbvTermListDetails *psTermListDetails, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvTermListDetails()\n" );
#endif

   /* Write the title out */ 
   if(psTermListDetails == NULL)
      {
      write_title_char("NULL","DbvTermListDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTermListDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTermListDetails", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psTermListDetails->psCommonInfo, OutTable );

   /* sTermListName */
   write_DbvInternationalString( "sTermListName",
				  &psTermListDetails->sTermListName,
				  OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psTermListDetails->ppsDescription,
			  OutTable );

   /* psAttributes */
   write_DbvAttributeCombinations( psTermListDetails->psAttributes, OutTable );

   /* psScanInfo */
   write_DbvScanInfo( psTermListDetails->psScanInfo, OutTable );

   /* pulEstNumberTerms */
   write_integer_string("pulEstNumberTerms", 
			(int *)psTermListDetails->pulEstNumberTerms, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* ppsSampleTerms */
   writeN_DbvTerm( psTermListDetails->ppsSampleTerms, OutTable );

   write_comment_field( "END of DbvTermListDetails", OutTable, T_Return );
   } /* End write_DbvTermListDetails() */

void write_DbvRecordTag( DbvRecordTag *psRecordTag, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvRecordTag()\n" );
#endif

   /* Write the title out */ 
   if(psRecordTag == NULL)
      {
      write_title_char("NULL","DbvRecordTag", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRecordTag", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRecordTag", OutTable, T_Return );

   /* psQualifier */
   write_DbvStringOrNumeric( psRecordTag->psQualifier, OutTable );

   /* sTagValue */
   write_DbvStringOrNumeric( &psRecordTag->sTagValue, OutTable );

   write_comment_field( "END of DbvRecordTag", OutTable, T_Return );
   } /* End write_DbvRecordTag() */

void write_DbvDetailsPerElement( DbvDetailsPerElement *psDetailsPerElement, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvDetailsPerElement()\n" );
#endif

   /* Write the title out */ 
   if(psDetailsPerElement == NULL)
      {
      write_title_char("NULL","DbvDetailsPerElement", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psDetailsPerElement", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDetailsPerElement", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psDetailsPerElement->psName,
				  OutTable );
   /* psRecordTag */
   write_DbvRecordTag( psDetailsPerElement->psRecordTag, OutTable );

   /* ppsSchemaTags */
   writeN_DbvPathA( psDetailsPerElement->ppsSchemaTags, OutTable );

   /* pulMaxSize */
   write_integer_string("pulMaxSize", (int *)psDetailsPerElement->pulMaxSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulMinSize */
   write_integer_string("pulMinSize", (int *)psDetailsPerElement->pulMinSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulAvgSize */
   write_integer_string("pulAvgSize", (int *)psDetailsPerElement->pulAvgSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* pulFixedSize */
   write_integer_string("pulFixedSize", 
			(int *)psDetailsPerElement->pulFixedSize, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* fRepeatable */
   write_DbvBool("fRepeatable", psDetailsPerElement->fRepeatable, OutTable);
   TBL_PutRecord(OutTable);

   /* fRequired */
   write_DbvBool("fRequired", psDetailsPerElement->fRequired, OutTable);
   TBL_PutRecord(OutTable);

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psDetailsPerElement->ppsDescription,
			  OutTable );
   /* ppsContents */
   writeN_DbvHumanString( "ppsContents", psDetailsPerElement->ppsContents,
			  OutTable );

   /* ppsBillingInfo */
   writeN_DbvHumanString( "ppsBillingInfo", psDetailsPerElement->ppsBillingInfo,
			  OutTable );
   /* ppsRestrictions */
   writeN_DbvHumanString( "ppsRestrictions",
			  psDetailsPerElement->ppsRestrictions,
			  OutTable );

   /* ppsAlternateNames */
   writeN_DbvInternationalString( "ppsAlternateNames",
			  psDetailsPerElement->ppsAlternateNames,
			  OutTable );
   /* ppsGenericNames */
   writeN_DbvInternationalString( "ppsGenericNames",
			  psDetailsPerElement->ppsGenericNames,
			  OutTable );
   /* psSearchAccess */
   write_DbvAttributeCombinations( psDetailsPerElement->psSearchAccess,
				   OutTable ); 

   write_comment_field( "END of DbvDetailsPerElement", OutTable, T_Return );
   } /* End write_DbvDetailsPerElement() */

void write_DbvElementSetDetails( DbvElementSetDetails *psElementSetDetails, 
				 TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvElementSetDetails()\n" );
#endif

   /* Write the title out */ 
   if(psElementSetDetails == NULL)
      {
      write_title_char("NULL","DbvElementSetDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElementSetDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementSetDetails", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psElementSetDetails->psCommonInfo, OutTable );

   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", 
				  &psElementSetDetails->sDatabaseName,
				  OutTable );

   /* sElementSetName */
   write_DbvInternationalString( "sElementSetName",
				 &psElementSetDetails->sElementSetName, 
				 OutTable);

   /* pcRecordSyntax */
   write_DbvObjectIdentifier( "pcRecordSyntax", 
				psElementSetDetails->pcRecordSyntax,
				OutTable );
   /* pcSchema */
   write_DbvObjectIdentifier( "pcSchema", psElementSetDetails->pcSchema,
				OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psElementSetDetails->ppsDescription,
				OutTable );
   /* ppsDetailsPerElement */
   writeN_DbvDetailsPerElement( psElementSetDetails->ppsDetailsPerElement,
				OutTable );

   write_comment_field( "END of DbvElementSetDetails", OutTable, T_Return );
   } /* End write_DbvElementSetDetails() */

void write_DbvRetrievalRecordDetails( 
			DbvRetrievalRecordDetails *psRetrievalRecordDetails, 
			TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvRetrievalRecordDetails()\n" );
#endif

   /* Write the title out */ 
   if(psRetrievalRecordDetails == NULL)
      {
      write_title_char("NULL","DbvRetrievalRecordDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psRetrievalRecordDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvRetrievalRecordDetails", 
				OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psRetrievalRecordDetails->psCommonInfo, OutTable );

   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", 
				  &psRetrievalRecordDetails->sDatabaseName,
				  OutTable );

   /* pcSchema */
   write_DbvObjectIdentifier( "pcSchema",
			      psRetrievalRecordDetails->pcSchema,
			      OutTable );
   /* pcRecordSyntax */
   write_DbvObjectIdentifier( "pcRecordSyntax",
			      psRetrievalRecordDetails->pcRecordSyntax,
			      OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", 
			  psRetrievalRecordDetails->ppsDescription,
			  OutTable );

   /* ppsDetailsPerElement */
   writeN_DbvDetailsPerElement( psRetrievalRecordDetails->ppsDetailsPerElement,
				OutTable );

   write_comment_field( "END of DbvRetrievalRecordDetails", 
			OutTable, T_Return );
   } /* End write_DbvRetrievalRecordDetails() */

void write_DbvSortType( DbvSortType *psSortType, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvSortType()\n" );
#endif

   /* Write the title out */ 
   if(psSortType == NULL)
      {
      write_title_char("NULL","DbvSortType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSortType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSortType", OutTable, T_Return );

   /* eSortTypeType */
   convert_DbvSortTypeType( &pc_string, &psSortType->eSortTypeType);
   sprintf( temp, "eSortType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uSortType */

   switch( psSortType->eSortTypeType )
      {
      case STT_Structured:
	 writeN_DbvHumanString( "ppsStructured", 
				psSortType->uSortType.ppsStructured, 
				OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvSortType", OutTable, T_Return );
   } /* End write_DbvSortType() */

void write_DbvSortKeys( DbvSortKeys *psSortKeys, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvSortKeys()\n" );
#endif

   /* Write the title out */ 
   if(psSortKeys == NULL)
      {
      write_title_char("NULL","DbvSortKeys", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSortKeys", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSortKeys", OutTable, T_Return );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psSortKeys->ppsDescription,
				  OutTable );

   /* ppsElementSpecifications */
   writeN_DbvSpecification( psSortKeys->ppsElementSpecifications, OutTable );

   /* psAttributeSpecifications */
   write_DbvAttributeCombinations( psSortKeys->psAttributeSpecifications,
				   OutTable );

   /* psSortType */
   write_DbvSortType( psSortKeys->psSortType, OutTable );

   /* peCaseSensitivity */
   if ( psSortKeys->peCaseSensitivity )
      {
      convert_DbvCaseSensitivity( &pc_string, psSortKeys->peCaseSensitivity );
      sprintf( temp, "peCaseSensitivity: %s", pc_string );
      }
   else
      sprintf( temp, "peCaseSensitivity: NULL" );

   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   write_comment_field( "END of DbvSortKeys", OutTable, T_Return );
   } /* End write_DbvSortKeys() */

void write_DbvSortDetails( DbvSortDetails *psSortDetails, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvSortDetails()\n" );
#endif

   /* Write the title out */ 
   if(psSortDetails == NULL)
      {
      write_title_char("NULL","DbvSortDetails", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSortDetails", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSortDetails", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psSortDetails->psCommonInfo, OutTable );

   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", &psSortDetails->sDatabaseName,
				  OutTable );
   /* ppsSortKeys */
   writeN_DbvSortKeys( psSortDetails->ppsSortKeys, OutTable );

   write_comment_field( "END of DbvSortDetails", OutTable, T_Return );
   } /* End write_DbvSortDetails() */

void write_DbvProcessingInformation( 
			DbvProcessingInformation *psProcessingInformation, 
			TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvProcessingInformation()\n" );
#endif

   /* Write the title out */ 
   if(psProcessingInformation == NULL)
      {
      write_title_char("NULL","DbvProcessingInformation", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psProcessingInformation", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvProcessingInformation", 
				OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psProcessingInformation->psCommonInfo, OutTable );

   /* sDatabaseName */
   write_DbvInternationalString( "sDatabaseName", 
				 &psProcessingInformation->sDatabaseName,
				  OutTable );
   /* eProcessingContext */

   convert_DbvProcessingContext( &pc_string, 
				    &psProcessingInformation->
				    eProcessingContext);
   sprintf( temp, "eDbvProcessingContext: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );


   /* sName */
   write_DbvInternationalString( "sName", &psProcessingInformation->sName,
				  OutTable );
   /* pcOid */
   write_DbvObjectIdentifier( "pcOid", psProcessingInformation->pcOid,
	 			OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", 
			  psProcessingInformation->ppsDescription,
			  OutTable );
   /* psInstructions */

   write_DbvASN1External( psProcessingInformation->psInstructions, OutTable );

   write_comment_field( "END of DbvProcessingInformation", OutTable, T_Return );
   } /* End write_DbvProcessingInformation() */

void write_DbvValueDescription( DbvValueDescription *psValueDescription, 
				TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvValueDescription()\n" );
#endif

   /* Write the title out */ 
   if(psValueDescription == NULL)
      {
      write_title_char("NULL","DbvValueDescription", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psValueDescription", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvValueDescription", OutTable, T_Return );

   /* eValueDescriptionType */
   convert_DbvValueDescriptionType( &pc_string, 
				    &psValueDescription->eValueDescriptionType);
   sprintf( temp, "eValueDescription: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uValueDescription */

   switch( psValueDescription->eValueDescriptionType )
      {
      case VDT_Integer:
	 write_integer_string( "ulInteger", (int*)&psValueDescription->
				uValueDescription.ulInteger, OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case VDT_String:
	 write_DbvInternationalString( "psString", psValueDescription->
				uValueDescription.psString, OutTable );
	 break;

      case VDT_Octets:
	 write_DbvOctetString( "psOctets", psValueDescription->
				uValueDescription.psOctets, OutTable );
	 break;

      case VDT_Oid:
	 write_DbvObjectIdentifier( "pcOid", psValueDescription->
				uValueDescription.pcOid, OutTable );
	 break;

      case VDT_Unit:
	 write_DbvUnit( psValueDescription->uValueDescription.psUnit,
			OutTable );
	 break;

      case VDT_ValueAndUnit:
	 write_DbvIntUnit( psValueDescription->uValueDescription.psValueAndUnit,
			OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvValueDescription", OutTable, T_Return );
   } /* End write_DbvValueDescription() */

void write_DbvValueRange( DbvValueRange *psValueRange, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvValueRange()\n" );
#endif

   /* Write the title out */ 
   if(psValueRange == NULL)
      {
      write_title_char("NULL","DbvValueRange", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psValueRange", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvValueRange", OutTable, T_Return );

   /* psLower */
   write_DbvValueDescription( psValueRange->psLower, OutTable );

   /* psUpper */
   write_DbvValueDescription( psValueRange->psUpper, OutTable );

   write_comment_field( "END of DbvValueRange", OutTable, T_Return );
   } /* End write_DbvValueRange() */

void write_DbvValueSet( DbvValueSet *psValueSet, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvValues()\n" );
#endif

   /* Write the title out */ 
   if(psValueSet == NULL)
      {
      write_title_char("NULL","DbvValueSet", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psValueSet", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvValueSet", OutTable, T_Return );

   /* eValueSetType */
   convert_DbvValueSetType( &pc_string, &psValueSet->eValueSetType );
   sprintf( temp, "eValueSetType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uValueSet */

   switch( psValueSet->eValueSetType )
      {
      case VT_Range:
	 write_DbvValueRange( psValueSet->uValueSet.psRange, OutTable );
	 break;

      case VT_Enumerated:
	 writeN_DbvValueDescription( psValueSet->uValueSet.ppsEnumerated, 
					OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvValueSet", OutTable, T_Return );
   } /* End write_DbvValues() */

void write_DbvVariantValue( DbvVariantValue *psVariantValue, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvVariantValue()\n" );
#endif

   /* Write the title out */ 
   if(psVariantValue == NULL)
      {
      write_title_char("NULL","DbvVariantValue", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psVariantValue", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvVariantValue", OutTable, T_Return );

   /* eDataType */
   write_DbvPrimitiveDataType( "eDataType", 
				&psVariantValue->eDataType, OutTable );
   /* psValues */

   write_DbvValueSet( psVariantValue->psValues, OutTable );

   write_comment_field( "END of DbvVariantValue", OutTable, T_Return );
   } /* End write_DbvVariantValue() */

void write_DbvVariantTypes( DbvVariantTypes *psVariantTypes, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvVariantTypes()\n" );
#endif

   /* Write the title out */ 
   if(psVariantTypes == NULL)
      {
      write_title_char("NULL","DbvVariantTypes", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psVariantTypes", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvVariantTypes", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psVariantTypes->psName,
				  OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psVariantTypes->ppsDescription,
			  OutTable );

   /* ulVariantType */
   write_integer_string("ulVariantType", (int *)&psVariantTypes->ulVariantType, 
			OutTable);
   TBL_PutRecord( OutTable );
   /* psVariantValue */
   write_DbvVariantValue( psVariantTypes->psVariantValue, OutTable );

   write_comment_field( "END of DbvVariantTypes", OutTable, T_Return );
   } /* End write_DbvVariantTypes() */

void write_DbvVariantClass( DbvVariantClass *psVariantClass, TBL OutTable)
   {

   /* Write the title out */ 
   if(psVariantClass == NULL)
      {
      write_title_char("NULL","DbvVariantClass", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psVariantClass", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvVariantClass", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psVariantClass->psName, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psVariantClass->ppsDescription,
			  OutTable );

   /* ulVariantClass */
   write_integer_string("ulVariantClass", 
			(int *)&psVariantClass->ulVariantClass, 
			OutTable);
   TBL_PutRecord( OutTable );

   /* ppsVariantTypes */
   writeN_DbvVariantTypes( psVariantClass->ppsVariantTypes, OutTable );

   write_comment_field( "END of DbvVariantClass", OutTable, T_Return );
   } /* End write_DbvVariantClass() */

void write_DbvVariantSetInfo( DbvVariantSetInfo *psVariantSetInfo, TBL OutTable)
   {

   /* Write the title out */ 
   if(psVariantSetInfo == NULL)
      {
      write_title_char("NULL","DbvVariantSetInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psVariantSetInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvVariantSetInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psVariantSetInfo->psCommonInfo, OutTable );

   /* pcVariantSet */
   write_DbvObjectIdentifier( "pcVariantSet", psVariantSetInfo->pcVariantSet,
				OutTable );

   /* sName */
   write_DbvInternationalString( "sName", &psVariantSetInfo->sName, OutTable );

   /* ppsVariants */
   writeN_DbvVariantClass( psVariantSetInfo->ppsVariants, OutTable );

   write_comment_field( "END of DbvVariantSetInfo", OutTable, T_Return );
   } /* End write_DbvVariantSetInfo() */

void write_DbvUnits( DbvUnits *psUnits, TBL OutTable)
   {

   /* Write the title out */ 
   if(psUnits == NULL)
      {
      write_title_char("NULL","DbvUnits", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUnits", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUnits", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psUnits->psName, OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psUnits->ppsDescription, OutTable );

   /* sUnit */
   write_DbvStringOrNumeric( &psUnits->sUnit, OutTable );

   write_comment_field( "END of DbvUnits", OutTable, T_Return );
   } /* End write_DbvUnits() */

void write_DbvUnitType( DbvUnitType *psUnitType, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvUnitType()\n" );
#endif

   /* Write the title out */ 
   if(psUnitType == NULL)
      {
      write_title_char("NULL","DbvUnitType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUnitType", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUnitType", OutTable, T_Return );

   /* psName */
   write_DbvInternationalString( "psName", psUnitType->psName, OutTable);

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psUnitType->ppsDescription, 
								OutTable);
   /* sUnitType */
   write_DbvStringOrNumeric( &psUnitType->sUnitType, OutTable);

   /* ppsUnits */
   writeN_DbvUnits( psUnitType->ppsUnits, OutTable );

   write_comment_field( "END of DbvUnitType", OutTable, T_Return );
   } /* End write_DbvUnitType() */

void write_DbvUnitInfo( DbvUnitInfo *psUnitInfo, TBL OutTable)
   {

   /* Write the title out */ 
   if(psUnitInfo == NULL)
      {
      write_title_char("NULL","DbvUnitInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUnitInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUnitInfo", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psUnitInfo->psCommonInfo, OutTable );

   /* sUnitSystem */
   write_DbvInternationalString( "sUnitSystem", &psUnitInfo->sUnitSystem,
				  OutTable );
   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psUnitInfo->ppsDescription,
			  OutTable );

   /* ppsUnits */
   writeN_DbvUnitType( psUnitInfo->ppsUnitTypes, OutTable );

   write_comment_field( "END of DbvUnitInfo", OutTable, T_Return );
   } /* End write_DbvUnitInfo() */

void write_DbvCategoryInfo( DbvCategoryInfo *psCategoryInfo, TBL OutTable)
   {

   /* Write the title out */ 
   if(psCategoryInfo == NULL)
      {
      write_title_char("NULL","DbvCategoryInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCategoryInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCategoryInfo", OutTable, T_Return );

   /* sCategory */
   write_DbvInternationalString( "sCategory", &psCategoryInfo->sCategory,
				  OutTable );

   /* psOriginalCategory */
   write_DbvInternationalString( "psOriginalCategory", 
				  psCategoryInfo->psOriginalCategory,
				  OutTable );

   /* ppsDescription */
   writeN_DbvHumanString( "ppsDescription", psCategoryInfo->ppsDescription,
				OutTable );
			
   /* psAsn1Module */
   write_DbvInternationalString( "psAsn1Module", psCategoryInfo->psAsn1Module,
				  OutTable );
   write_comment_field( "END of DbvCategoryInfo", OutTable, T_Return );
   } /* End write_DbvCategoryInfo() */

void write_DbvCategoryList( DbvCategoryList *psCategoryList, TBL OutTable)
   {

   /* Write the title out */ 
   if(psCategoryList == NULL)
      {
      write_title_char("NULL","DbvCategoryList", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psCategoryList", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvCategoryList", OutTable, T_Return );

   /* psCommonInfo */
   write_DbvCommonInfo( psCategoryList->psCommonInfo, OutTable );

   /* ppsCategories */
   writeN_DbvCategoryInfo( psCategoryList->ppsCategories, OutTable );

   write_comment_field( "END of DbvCategoryList", OutTable, T_Return );
   } /* End write_DbvCategoryList() */

void write_DbvExplainRecord( DbvExplainRecord *psExplainRecord, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

   /* Write the title out */ 
   if(psExplainRecord == NULL)
      {
      write_title_char("NULL","DbvExplainRecord", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psExplainRecord", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvExplainRecord", OutTable, T_Return );

   /* eExplainRecordType */
   convert_DbvExplainRecordType( &pc_string, 
				&psExplainRecord->eExplainRecordType);
   sprintf( temp, "eExplainRecord: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uExplainRecord */

   switch( psExplainRecord->eExplainRecordType )
      {
      case ERT_TargetInfo:
	 write_DbvTargetInfo( psExplainRecord->uExplainRecord.psTargetInfo,
				OutTable );
	 break;

      case ERT_DatabaseInfo:
	 write_DbvDatabaseInfo( psExplainRecord->
				uExplainRecord.psDatabaseInfo, OutTable );
	 break;

      case ERT_SchemaInfo:
	 write_DbvSchemaInfo( psExplainRecord->
				uExplainRecord.psSchemaInfo, OutTable );
	 break;

      case ERT_TagSetInfo:
	 write_DbvTagSetInfo( psExplainRecord->
				uExplainRecord.psTagSetInfo, OutTable );
	 break;

      case ERT_RecordSyntaxInfo:
	 write_DbvRecordSyntaxInfo( psExplainRecord->
				uExplainRecord.psRecordSyntaxInfo, OutTable );
	 break;

      case ERT_AttributeSetInfo:
	 write_DbvAttributeSetInfo( psExplainRecord->
				uExplainRecord.psAttributeSetInfo, OutTable );
	 break;

      case ERT_TermListInfo:
	 write_DbvTermListInfo( psExplainRecord->
				uExplainRecord.psTermListInfo, OutTable );
	 break;

      case ERT_ExtendedServicesInfo:
	 write_DbvExtendedServicesInfo( psExplainRecord->
				uExplainRecord.psExtendedServicesInfo, 
				OutTable );
	 break;

      case ERT_AttributeDetails:
	 write_DbvAttributeDetails( psExplainRecord->
				uExplainRecord.psAttributeDetails, OutTable );
	 break;

      case ERT_TermListDetails:
	 write_DbvTermListDetails( psExplainRecord->
				uExplainRecord.psTermListDetails, OutTable );
	 break;

      case ERT_ElementSetDetails:
	 write_DbvElementSetDetails( psExplainRecord->
				uExplainRecord.psElementSetDetails, OutTable );
	 break;

      case ERT_RetrievalRecordDetails:
	 write_DbvRetrievalRecordDetails( psExplainRecord->
				uExplainRecord.psRetrievalRecordDetails,
				OutTable );
	 break;

      case ERT_SortDetails:
	 write_DbvSortDetails( psExplainRecord->
				uExplainRecord.psSortDetails, OutTable );
	 break;

      case ERT_Processing:
	 write_DbvProcessingInformation( psExplainRecord->
				uExplainRecord.psProcessing, OutTable );
	 break;

      case ERT_Variants:
	 write_DbvVariantSetInfo( psExplainRecord->
				uExplainRecord.psVariants, OutTable );
	 break;

      case ERT_Units:
	 write_DbvUnitInfo( psExplainRecord->
				uExplainRecord.psUnits, OutTable );
	 break;

      case ERT_CategoryList:
	 write_DbvCategoryList( psExplainRecord->
				uExplainRecord.psCategoryList, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvExplainRecord", OutTable, T_Return );
   } /* End write_DbvExplainRecord() */

void write_DbvLanguageCode( char *name,DbvLanguageCode *psLanguageCode, 
			    TBL OutTable)
   {

   write_DbvOctetString( name, (DbvOctetString *)psLanguageCode, OutTable);

   } /* End write_DbvLanguageCode() */

void write_DbvNull( char *name, DbvNull *pvNull, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvNull()\n" );
#endif

   if(pvNull == NULL)
      write_title_char("NULL","DbvNull", OutTable);
   else
      write_title_char("Value Defined","DbvNull", OutTable);

   TBL_PutRecord(OutTable);
   return;
   } /* End write_DbvNull() */

void write_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq *psDataBaseInfoSeq, 
								TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvDataBaseInfoSeq()\n" );
#endif

   /* Write the title out */ 
   if(psDataBaseInfoSeq == NULL)
      {
      write_title_char("NULL","DbvDataBaseInfoSeq", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("ppsKeywords", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvDataBaseInfoSeq", OutTable, T_Return );

   /* ppsDataBaseInfo */
   writeN_DbvHumanString( "ppsDataDaseInfo",
			  psDataBaseInfoSeq->ppsDataBaseInfo, OutTable );

   write_comment_field( "END of DbvDataBaseInfoSeq", OutTable, T_Return );
   } /* End write_DbvDataBaseInfoSeq() */


