/*  Module Name: @(#)write_DbvFuncs3.c	7.1
    Version: 11/15/96
    Author: Dan Maskell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write structure types to the result script files.
        Used by the origin and target.
 
*/

#include "write_DbvFuncs.h"


#ifdef NOT_YET_IMPLEMENTED


extern TBL_Return T_Return;

void write_DbvUsageIntType( char *name, DbvUsageIntType *peUsageIntType, 
				TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvUsageIntType()\n" );
#endif

   /* Write the title out */ 
   if(peUsageIntType == NULL)
      {
      write_title_char("NULL", name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eType */
   convert_DbvUsageIntType( &pc_string, peUsageIntType);
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvUsageIntType() */

void write_DbvOrder( DbvOrder *psOrder, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvOrder()\n" );
#endif

   /* Write the title out */ 
   if(psOrder == NULL)
      {
      write_title_char("NULL","DbvOrder", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psOrder", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvOrder", OutTable, T_Return );

   /* fAscending */
   write_DbvBool("fAscending", psOrder->fAscending, OutTable);
   TBL_PutRecord( OutTable );

   /* ulOrder */
   write_integer_string("ulOrder", (int*)&psOrder->ulOrder, OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "END of DbvOrder", OutTable, T_Return );
   } /* End write_DbvOrder() */

void write_DbvUsage( DbvUsage *psUsage, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvUsage()\n" );
#endif

   /* Write the title out */ 
   if(psUsage == NULL)
      {
      write_title_char("NULL","DbvUsage", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psUsage", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvUsage", OutTable, T_Return );

   /* eType */
   write_DbvUsageIntType( "eType", &psUsage->eType, OutTable);

   /* psRestriction */
   write_DbvInternationalString( "psRestriction",  psUsage->psRestriction,
				OutTable);

   write_comment_field( "END of DbvUsage", OutTable, T_Return );
   } /* End write_DbvUsage() */

void write_DbvHits( DbvHits *psHits, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvHits()\n" );
#endif

   /* Write the title out */ 
   if(psHits == NULL)
      {
      write_title_char("NULL","DbvHits", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psHits", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvHits", OutTable, T_Return );

   /* psSatisfier */
   write_DbvTerm( psHits->psSatisfier, OutTable );

   /* psOffsetIntoElement */
   write_DbvIntUnit( psHits->psOffsetIntoElement, OutTable );

   /* psLength */
   write_DbvIntUnit( psHits->psLength, OutTable );

   /* pulHitRank */
   write_integer_string("pulHitRank", (int *)psHits->pulHitRank, OutTable);
   TBL_PutRecord( OutTable );

   /* psTargetToken */
   write_DbvOctetString( "psTargetToken", psHits->psTargetToken, OutTable);

   write_comment_field( "END of DbvHits", OutTable, T_Return );
   } /* End write_DbvHits() */

void write_DbvValue( DbvValue *psValue, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvValue()\n" );
#endif

   /* Write the title out */ 
   if(psValue == NULL)
      {
      write_title_char("NULL","DbvValue", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psValue", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvValue", OutTable, T_Return );

   /* eValueType */
   convert_DbvVariantValueType( &pc_string, &psValue->eVariantValueType);
   sprintf( temp, "eVariantValueType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uValue */

   switch( psValue->eVariantValueType )
      {
      case VVT_Int:
         write_integer_string("ulInt1", (int *)&psValue->
				uVariantValueType.ulInt1, OutTable);
         TBL_PutRecord( OutTable );
	 break;

      case VVT_InternationalString:
	 write_DbvInternationalString( "psInternationalString",
					psValue->uVariantValueType.
					psInternationalString, OutTable );
	 break;

      case VVT_Octs:
	 write_DbvOctetString( "psOcts", psValue->uVariantValueType.psOcts,
				OutTable );
	 break;

      case VVT_Oid:
	 write_DbvObjectIdentifier( "pcOid", psValue->uVariantValueType.pcOid,
					 OutTable );
	 break;

      case VVT_Bool:
         write_DbvBool("fBool", psValue->uVariantValueType.fBool, OutTable);
	 TBL_PutRecord(OutTable);
	 break;

      case VVT_Null:
	 break;

      case VVT_Unit:
	 write_DbvUnit( psValue->uVariantValueType.psUnit, OutTable );
	 break;

      case VVT_ValueAndUnit:
	 write_DbvIntUnit( psValue->uVariantValueType.psValueAndUnit, OutTable);
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvValue", OutTable, T_Return );
   } /* End write_DbvValue() */

void write_DbvTriples( DbvTriples *psTriples, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvTriples()\n" );
#endif
   /* Write the title out */ 
   if(psTriples == NULL)
      {
      write_title_char("NULL","DbvTriples", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTriples", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTriples", OutTable, T_Return );

   /* pcVariantSetId */
   write_DbvObjectIdentifier( "pcVariantSetId", psTriples->pcVariantSetId,
				   OutTable );

   /* ulClass */
   write_integer_string("ulClass", (int *)&psTriples->ulClass, OutTable);
   TBL_PutRecord( OutTable );

   /* ulType */
   write_integer_string("ulType", (int *)&psTriples->ulType, OutTable);
   TBL_PutRecord( OutTable );

   /* ppsValue */
   writeN_DbvValue( psTriples->ppsValue, OutTable );

   write_comment_field( "END of DbvTriples", OutTable, T_Return );
   } /* End write_DbvTriples() */

void write_DbvVariant( DbvVariant *psVariant, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvVariant()\n" );
#endif
   /* Write the title out */ 
   if(psVariant == NULL)
      {
      write_title_char("NULL","DbvVariant", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psVariant", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvVariant", OutTable, T_Return );

   /* pcGlobalVariantSetId */
   write_DbvObjectIdentifier( "pcGlobalVariantSetId",
			      psVariant->pcGlobalVariantSetId,
			      OutTable );
   /* ppsTriples */
   writeN_DbvTriples( psVariant->ppsTriples, OutTable );

   write_comment_field( "END of DbvVariant", OutTable, T_Return );
   } /* End write_DbvVariant() */

void write_DbvTagPath( DbvTagPath *psTagPath, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvTagPath()\n" );
#endif
   /* Write the title out */ 
   if(psTagPath == NULL)
      {
      write_title_char("NULL","DbvTagPath", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTagPath", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTagPath", OutTable, T_Return );

   /* pulTagType */
   write_integer_string("pulTagType", (int *)psTagPath->pulTagType, OutTable);
   TBL_PutRecord( OutTable );

   /* sTagValue */
   write_DbvStringOrNumeric( &psTagPath->sTagValue, OutTable);

   /* pulTagOccurrence */
   write_integer_string("pulTagOccurrence", (int *)psTagPath->pulTagOccurrence,
			 OutTable);
   TBL_PutRecord( OutTable );
   
   write_comment_field( "END of DbvTagPath", OutTable, T_Return );
   } /* End write_DbvTagPath() */

void write_DbvElementMetaData( DbvElementMetaData *psElementMetaData, 
				 TBL OutTable)
   {

   /* Write the title out */ 
   if(psElementMetaData == NULL)
      {
      write_title_char("NULL","DbvElementMetaData", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElementMetaData", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementMetaData", OutTable, T_Return );

   /* psSeriesOrder */
   write_DbvOrder( psElementMetaData->psSeriesOrder, OutTable );

   /* psUsageRight */
   write_DbvUsage( psElementMetaData->psUsageRight, OutTable );

   /* ppsHits */
   writeN_DbvHits( psElementMetaData->ppsHits, OutTable );

   /* psDisplayName */
   write_DbvInternationalString( "psDisplayName",
				 psElementMetaData->psDisplayName, 
				 OutTable );

   /* ppsSupportedVariants */
   writeN_DbvVariant( psElementMetaData->ppsSupportedVariants, OutTable );

   /* psMessage */
   write_DbvInternationalString( "psMessage", psElementMetaData->psMessage,
				 OutTable );

   /* psElementDescriptor */
   write_DbvOctetString( "psElementDescriptor", 
			 psElementMetaData->psElementDescriptor, OutTable );

   /* ppsSurrogateFor */
   writeN_DbvTagPath( psElementMetaData->ppsSurrogateFor, OutTable );

   /* ppsSurrogateElement */
   writeN_DbvTagPath( psElementMetaData->ppsSurrogateElement, OutTable );

   /* psOther */
   write_DbvASN1External( psElementMetaData->psOther, OutTable );

   write_comment_field( "END of DbvElementMetaData", OutTable, T_Return );
   } /* End write_DbvElementMetaData() */

void write_DbvElementDataG( DbvElementDataG *psElementDataG, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvElementDataG()\n" );
#endif
   /* Write the title out */ 
   if(psElementDataG == NULL)
      {
      write_title_char("NULL","DbvElementDataG", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psElementDataG", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvElementDataG", OutTable, T_Return );

   /* eElementDataGType */
   convert_DbvElementDataGType( &pc_string, &psElementDataG->eElementDataGType);
   sprintf( temp, "eElementGData: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uElementDataG */

   switch( psElementDataG->eElementDataGType )
      {
      case EDT_Octets:
	 write_DbvOctetString( "psOctets", 
				psElementDataG->uElementDataG.psOctets,
				OutTable );
	 break;

      case EDT_Numeric:
         write_integer_string( "ulNumeric", (int*)&psElementDataG->
					 uElementDataG.ulNumeric, OutTable );
         TBL_PutRecord( OutTable );
	 break;

      case EDT_Date:
	 write_DbvGeneralizedTime( "pcDate", psElementDataG->
				     uElementDataG.pcDate, OutTable );
	 break;

      case EDT_Ext:
	 write_DbvASN1External( psElementDataG->uElementDataG.psExt, OutTable );
	 break;

      case EDT_String:
	 write_DbvInternationalString( "psString",
					psElementDataG->uElementDataG.psString,
					OutTable );
	 break;

      case EDT_TrueOrFalse:
	 write_DbvBool("fTrueOrFalse",
			psElementDataG->uElementDataG.fTrueOrFalse,
			OutTable );
	 TBL_PutRecord(OutTable);
	 break;

      case EDT_Oid:
	 write_DbvObjectIdentifier( "pcOid",
				     psElementDataG->uElementDataG.pcOid,
				     OutTable );
	 break;

      case EDT_IntUnit:
	 write_DbvIntUnit( psElementDataG->uElementDataG.psIntUnit, OutTable );
	 break;

      case EDT_ElementNotThere:
      case EDT_ElementEmpty:
      case EDT_NoDataRequested:
	 break;

      case EDT_Diagnostic:
	 write_DbvASN1External( psElementDataG->uElementDataG.psDiagnostic, 
				OutTable );
	 break;

      case EDT_Subtree:
	 writeN_DbvTaggedElement( psElementDataG->uElementDataG.ppsSubTree,
				   OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvElementDataG", OutTable, T_Return );
   } /* End write_DbvElementDataG() */

void write_DbvTaggedElement( DbvTaggedElement *psTaggedElement, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvTaggedElement()\n" );
#endif
   /* Write the title out */ 
   if(psTaggedElement == NULL)
      {
      write_title_char("NULL","DbvTaggedElement", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psTaggedElement", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvTaggedElement", OutTable, T_Return );

   /* pulTagType */
   write_integer_string("pulTagType", (int *)psTaggedElement->pulTagType, 
			 OutTable);
   TBL_PutRecord( OutTable );

   /* sTagValue */
   write_DbvStringOrNumeric( &psTaggedElement->sTagValue, OutTable);

   /* pulTagOccurrence */
   write_integer_string( "pulTagOccurrence", 
			 (int *)psTaggedElement->pulTagOccurrence,
			 OutTable);
   TBL_PutRecord( OutTable );
  
   /* sContent */
   write_DbvElementDataG( &psTaggedElement->sContent, OutTable );

   /* psMetaData */
   write_DbvElementMetaData( psTaggedElement->psMetaData, OutTable );

   /* psAppliedVariant */
   write_DbvVariant( psTaggedElement->psAppliedVariant, OutTable );

   write_comment_field( "END of DbvTaggedElement", OutTable, T_Return );
   } /* End write_DbvTaggedElement() */

void write_DbvGenericRecord( DbvGenericRecord *psGenericRecord, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvGenericRecord()\n" );
#endif

   /* Write the title out */ 
   if(psGenericRecord == NULL)
      {
      write_title_char("NULL","DbvGenericRecord", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psGenericRecord", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvGenericRecord", OutTable, T_Return );

   /* ppsTaggedElement */
   writeN_DbvTaggedElement( psGenericRecord->ppsTaggedElement, OutTable );

   write_comment_field( "END of DbvGenericRecord", OutTable, T_Return );
   } /* End write_DbvGenericRecord() */


#endif
