/*  Module Name: @(#)write_DbvFuncs4.c	7.1
    Version: 11/15/96
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write structure types to the result script files.
        Used by the origin and target.
 
*/

#include "write_DbvFuncs.h"



extern TBL_Return T_Return;


/* char *name parameter only with integers, bools, enumerations
   octet and inter strings  */


/* ------------------    ACCESSCONTROL   --------------------------------- */


void write_DbvSecurityChallengeReq( DbvSecurityChallengeReq *psSecurityChallengeReq, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvSecurityChallengeReq()\n" );
#endif

   /* Write the title out */ 
   if(psSecurityChallengeReq == NULL)
      {
      write_title_char("NULL","DbvSecurityChallengeReq", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSecurityChallengeReq", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSecurityChallengeReq", OutTable, T_Return );

   /* eSecurityChallengeReqType */
   convert_DbvSecurityChallengeType( &pc_string, &psSecurityChallengeReq->eSecurityChallengeReqType);
   sprintf( temp, "eSecurityChallengeReqType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uSecurityChallengeReq */

   switch( psSecurityChallengeReq->eSecurityChallengeReqType )
      {
      case SECT_SimpleForm:
         write_DbvOctetString( "psSimpleForm", psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm, OutTable);
	 break;

      case SECT_ExternallyDefined:
         write_DbvASN1External( psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvSecurityChallengeReq", OutTable, T_Return );
   } /* End write_DbvSecurityChallengeReq() */



void write_DbvSecurityChallengeRsp( DbvSecurityChallengeRsp *psSecurityChallengeRsp, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvSecurityChallengeRsp()\n" );
#endif

   /* Write the title out */ 
   if(psSecurityChallengeRsp == NULL)
      {
      write_title_char("NULL","DbvSecurityChallengeRsp", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psSecurityChallengeRsp", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvSecurityChallengeRsp", OutTable, T_Return );

   /* eSecurityChallengeRspType */
   convert_DbvSecurityChallengeType( &pc_string, &psSecurityChallengeRsp->eSecurityChallengeRspType);
   sprintf( temp, "eSecurityChallengeRspType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uSecurityChallengeRsp */

   switch( psSecurityChallengeRsp->eSecurityChallengeRspType )
      {
      case SECT_SimpleForm:
         write_DbvOctetString( "psSimpleForm", psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm, OutTable);
	 break;

      case SECT_ExternallyDefined:
         write_DbvASN1External( psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvSecurityChallengeRsp", OutTable, T_Return );
   } /* End write_DbvSecurityChallengeRsp() */



/* -------------- Access Control Format "Prompt 1" ----------------*/

void write_DbvEPromptIdType( char* name, DbvEPromptIdType *peEPromptIdType, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvEPromptIdType()\n" );
#endif
   
   /* Write the title out */
   if(peEPromptIdType == NULL)
      {
      write_title_char("NULL","DbvEPromptIdType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eEPromptIdType */
   convert_DbvEPromptIdType( &pc_string, peEPromptIdType );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvEPromptIdType() */



void write_DbvEnumeratedPrompt( DbvEnumeratedPrompt *psEnumeratedPrompt, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvEnumeratedPrompt()\n" );
#endif

   /* Write the title out */ 
   if(psEnumeratedPrompt == NULL)
      {
      write_title_char("NULL","DbvEnumeratedPrompt", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEnumeratedPrompt", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEnumeratedPrompt", OutTable, T_Return );

   /* eType */
   write_DbvEPromptIdType( "eType", &psEnumeratedPrompt->eType, OutTable);

   /* psSuggestedString */
   write_DbvInternationalString( "psSuggestedString", psEnumeratedPrompt->psSuggestedString, OutTable);

   write_comment_field( "END of DbvEnumeratedPrompt", OutTable, T_Return );
   } /* End write_DbvEnumeratedPrompt() */



void write_DbvPromptId( DbvPromptId *psPromptId, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvPromptId()\n" );
#endif

   /* Write the title out */ 
   if(psPromptId == NULL)
      {
      write_title_char("NULL","DbvPromptId", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPromptId", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPromptId", OutTable, T_Return );

   /* ePromptIdType */
   convert_DbvPromptIdType( &pc_string, &psPromptId->ePromptIdType);
   sprintf( temp, "ePromptIdType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uPromptId */

   switch( psPromptId->ePromptIdType )
      {
      case PIT_EnumeratedPrompt:
         write_DbvEnumeratedPrompt( psPromptId->uPromptId.psEnumeratedPrompt, OutTable);
	 break;

      case PIT_NoneEnumeratedPrompt:
         write_DbvInternationalString( "psNonEnumeratedPrompt", psPromptId->uPromptId.psNonEnumeratedPrompt, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvPromptId", OutTable, T_Return );
   } /* End write_DbvPromptId() */





void write_DbvEncryption( DbvEncryption *psEncryption, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvEncryption()\n" );
#endif

   /* Write the title out */ 
   if(psEncryption == NULL)
      {
      write_title_char("NULL","DbvEncryption", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psEncryption", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvEncryption", OutTable, T_Return );

   /* psCryptType */
   write_DbvOctetString( "psCryptType", psEncryption->psCryptType, OutTable);

   /* psCredential */
   write_DbvOctetString( "psCredential", psEncryption->psCredential, OutTable);

   /* sData */
   write_DbvOctetString( "sData", &psEncryption->sData, OutTable);

   write_comment_field( "END of DbvEncryption", OutTable, T_Return );
   } /* End write_DbvEncryption() */



void write_DbvPromptInfo( DbvPromptInfo *psPromptInfo, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvPromptInfo()\n" );
#endif

   /* Write the title out */ 
   if(psPromptInfo == NULL)
      {
      write_title_char("NULL","DbvPromptInfo", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPromptInfo", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPromptInfo", OutTable, T_Return );

   /* ePromptInfoType */
   convert_DbvPromptInfoType( &pc_string, &psPromptInfo->ePromptInfoType);
   sprintf( temp, "ePromptInfoType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uPromptInfo */

   switch( psPromptInfo->ePromptInfoType )
      {
      case PIIT_Character:
         write_DbvInternationalString( "psCharacter", psPromptInfo->uPromptInfo.psCharacter, OutTable);
	 break;

      case PIIT_Encrypted:
         write_DbvEncryption( psPromptInfo->uPromptInfo.psEncrypted, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvPromptInfo", OutTable, T_Return );
   } /* End write_DbvPromptInfo() */





void write_DbvDataType( char* name, DbvDataType *peDataType, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUg
   printf( "D - write_DbvDataType()\n" );
#endif
   
   /* Write the title out */
   if(peDataType == NULL)
      {
      write_title_char("NULL","DbvDataType", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eDataType */
   convert_DbvDataType( &pc_string, peDataType );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvDataType() */



void write_DbvChallenge( DbvChallenge *psChallenge, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvChallenge()\n" );
#endif

   /* Write the title out */ 
   if(psChallenge == NULL)
      {
      write_title_char("NULL","DbvChallenge", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */


   write_title_char("psChallenge", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvChallenge", OutTable, T_Return );

   /* sPromptId */
   write_DbvPromptId( &psChallenge->sPromptId, OutTable);

   /* psDefaultResponse */
   write_DbvInternationalString( "psDefaultResponse", psChallenge->psDefaultResponse, OutTable);

   /* psPromptInfo */
   write_DbvPromptInfo( psChallenge->psPromptInfo, OutTable);

   /* psRegExpr */
   write_DbvInternationalString( "psRegExpr", psChallenge->psRegExpr, OutTable);
   /* pvResponseRequired */
   write_DbvNull( "pvResponseRequired", psChallenge->pvResponseRequired, OutTable);

   /* psRegExpr */
   writeN_DbvInternationalString( "ppsAllowedValues", psChallenge->ppsAllowedValues, OutTable);

   /* pvShouldSave */
   write_DbvNull( "pvShouldSave", psChallenge->pvShouldSave, OutTable);

   /* peDataType */
   write_DbvDataType( "peDataType", psChallenge->peDataType, OutTable);

   /* psDiagnostic */
   write_DbvASN1External( psChallenge->psDiagnostic, OutTable);

   write_comment_field( "END Of DbvChallenge", OutTable, T_Return );
   } /* End write_DbvChallenge() */



void write_DbvPromptResponse( DbvPromptResponse *psPromptResponse, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvPromptResponse()\n" );
#endif

   /* Write the title out */ 
   if(psPromptResponse == NULL)
      {
      write_title_char("NULL","DbvPromptResponse", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPromptResponse", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPromptResponse", OutTable, T_Return );

   /* eResponseType */
   convert_DbvResponseType( &pc_string, &psPromptResponse->eResponseType);
   sprintf( temp, "eResponseType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uResponse */

   switch( psPromptResponse->eResponseType )
      {
      case RT_String:
         write_DbvInternationalString( "psString", psPromptResponse->uResponse.psString, OutTable);
	 break;

      case RT_Accept:
         write_DbvBool( "fAccept", psPromptResponse->uResponse.fAccept, OutTable);
         TBL_PutRecord(OutTable);
	 break;

      case RT_Acknowledge:
	 break;

      case RT_Diagnostic:
         write_DbvDiagRec( psPromptResponse->uResponse.psDiagnostic, OutTable );
	 break;

      case RT_Encrypted:
         write_DbvEncryption( psPromptResponse->uResponse.psEncryption, OutTable );
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvPromptResponse", OutTable, T_Return );
   } /* End write_DbvPromptResponse() */



void write_DbvResponse( DbvResponse *psResponse, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvResponse()\n" );
#endif

   /* Write the title out */ 
   if(psResponse == NULL)
      {
      write_title_char("NULL","DbvResponse", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psResponse", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvResponse", OutTable, T_Return );

   /* sPromptId */
   write_DbvPromptId( &psResponse->sPromptId, OutTable);

   /* sPromptResponse */
   write_DbvPromptResponse( &psResponse->sPromptResponse, OutTable);

   write_comment_field( "END of DbvResponse", OutTable, T_Return );
   } /* End write_DbvResponse() */



void write_DbvPromptObject( DbvPromptObject *psPromptObject, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;
#ifdef DEBUG
   printf( "D - write_DbvPromptObject()\n" );
#endif

   /* Write the title out */ 
   if(psPromptObject == NULL)
      {
      write_title_char("NULL","DbvPromptObject", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psPromptObject", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvPromptObject", OutTable, T_Return );

   /* ePromptObjectType */
   convert_DbvPromptObjectType( &pc_string, &psPromptObject->ePromptObjectType);
   sprintf( temp, "ePromptObjectType: %s", pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* uPromptObject */

   switch( psPromptObject->ePromptObjectType )
      {
      case POT_Challenge:
         writeN_DbvChallenge( psPromptObject->uPromptObject.ppsChallenge, OutTable);
	 break;

      case POT_Response:
         writeN_DbvResponse( psPromptObject->uPromptObject.ppsResponse, OutTable);
	 break;

      default:
	 break;
      } /* End switch */

   write_comment_field( "END of DbvPromptObject", OutTable, T_Return );
   } /* End write_DbvPromptObject() */




/* ------------------    RESOURCECONTROL   --------------------------------- */

void write_DbvPartialResultsAvailable( char* name, DbvPartialResultsAvailable *pePartialResultsAvailable, TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUg
   printf( "D - write_DbvPartialResultsAvailable()\n" );
#endif
   
   /* Write the title out */
   if(pePartialResultsAvailable == NULL)
      {
      write_title_char("NULL","DbvPartialResultsAvailable", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* ePartialResultsAvailable */
   convert_DbvPartialResultsAvailable( &pc_string, pePartialResultsAvailable );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvPartialResultsAvailable() */



/* -----------    TRIGGERRESOURCECONTROL   -------------------------- */


void write_DbvRequestedAction( char *name,
                                 DbvRequestedAction *peRequestedAction,
                                 TBL OutTable)
   {
   char temp[ STRING_LEN  ];
   char *pc_string = NULL;

#ifdef DEBUG
   printf( "D - write_DbvRequestedAction()\n" );
#endif
      
   /* Write the title out */
   if(peRequestedAction == NULL)
      {
      write_title_char("NULL","DbvRequestedAction", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   /* eRequestedAction */
   convert_DbvRequestedAction( &pc_string, peRequestedAction );
   sprintf( temp, "%s : %s", name, pc_string );
   write_char_field(temp, OutTable, T_Return);
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   } /* End write_DbvRequestedAction() */




/*+---------------------------------------------------------------------+*/
/*! write_DbvAccessControlResponse()                                    !*/
/*+---------------------------------------------------------------------+*/

void write_DbvAccessControlResponse( DbvAccessControlRsp *psAccessControlRsp, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvAccessControlResponse()\n" );
#endif

   /* Write the title out */
   if(psAccessControlRsp == NULL)
      {
      write_title_char("NULL","DbvAccessControlRsp", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAccessControlRsp", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAccessControlRsp", OutTable, T_Return );

   /* psReferenceId */
   write_DbvOctetString( "psReferenceId", psAccessControlRsp->psReferenceId, OutTable);

   /* psSecurityChallengeRsp */
   write_DbvSecurityChallengeRsp( psAccessControlRsp->psSecurityChallengeRsp, OutTable);

   /* psDiagnostic */
   write_DbvDiagRec( psAccessControlRsp->psDiagnostic, OutTable);


   /* ppsOtherInformation */
   writeN_DbvOtherInformation( psAccessControlRsp->ppsOtherInformation, OutTable );
   write_comment_field( "END of DbvAccessControlRsp", OutTable, T_Return );
   TBL_PutRecord( OutTable );

   } /* End write_DbvAccessControlResponse() */






/*+---------------------------------------------------------------------+*/
/*! write_DbvResourceControlResponse()                                  !*/
/*+---------------------------------------------------------------------+*/

void write_DbvResourceControlResponse( DbvResourceControlRsp *psResourceControlRsp, TBL OutTable)
   {

#ifdef DEBUG
   printf( "D - write_DbvResourceControlResponse()\n" );
#endif

   /* Write the title out */
   if(psResourceControlRsp == NULL)
      {
      write_title_char("NULL","DbvResourceControlRsp", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
   write_title_char("psResourceControlRsp", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvResourceControlRsp", OutTable, T_Return );

   /* psReferenceId */
   write_DbvOctetString( "psReferenceId", psResourceControlRsp->psReferenceId, OutTable);

   /* fContinueFlag */
   write_DbvBool( "fContinueFlag", psResourceControlRsp->fContinueFlag, OutTable);
   TBL_PutRecord(OutTable);

   /* pfResultSetWanted */
   if ( psResourceControlRsp->pfResultSetWanted )
      write_DbvBool( "pfResultSetWanted", *psResourceControlRsp->pfResultSetWanted, OutTable);
   else
      write_title_char("NULL","pfResultSetWanted", OutTable);
   TBL_PutRecord(OutTable);


   /* ppsOtherInformation */
   writeN_DbvOtherInformation( psResourceControlRsp->ppsOtherInformation, OutTable );

   write_comment_field( "END of DbvResourceControlRsp", OutTable, T_Return );
   TBL_PutRecord( OutTable );

   } /* End write_DbvResourceControlResponse() */




/*+---------------------------------------------------------------------+*/
/*! write_DbvAccessControlRequest()                                     !*/
/*+---------------------------------------------------------------------+*/

void write_DbvAccessControlRequest( DbvAccessControlReq *psAccessControlReq, TBL OutTable)
   {
#ifdef DEBUG
   printf( "D - write_DbvAccessControlRequest()\n" );
#endif

   /* Write the title out */
   if(psAccessControlReq == NULL)
      {
      write_title_char("NULL","DbvAccessControlReq", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psAccessControlReq", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvAccessControlReq", OutTable, T_Return );


   /* psReferenceId */
   write_DbvOctetString( "psReferenceId", psAccessControlReq->psReferenceId, OutTable);

   /* sSecurityChallengeReq */
   write_DbvSecurityChallengeReq( &psAccessControlReq->sSecurityChallengeReq, OutTable);

   /* ppsOtherInformation */
   writeN_DbvOtherInformation( psAccessControlReq->ppsOtherInformation, OutTable );

  
   write_comment_field( "END of DbvAccessControlReq", OutTable, T_Return );
   TBL_PutRecord( OutTable );

   } /* End write_DbvAccessControlRequest() */





/*+---------------------------------------------------------------------+*/
/*! write_DbvResourceControlRequest()                                   !*/
/*+---------------------------------------------------------------------+*/

void write_DbvResourceControlRequest( DbvResourceControlReq *psResourceControlReq, TBL OutTable)
   { 
#ifdef DEBUG
   printf( "D - write_DbvResourceControlRequest()\n" );
#endif

   /* Write the title out */
   if(psResourceControlReq == NULL)
      {
      write_title_char("NULL","DbvResourceControlReq", OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */

   write_title_char("psResourceControlReq", "Structure", OutTable);
   TBL_PutRecord( OutTable );

   write_comment_field( "START of DbvResourceControlReq", OutTable, T_Return );

   /* psReferenceId */
   write_DbvOctetString( "psReferenceId", psResourceControlReq->psReferenceId, OutTable);

   /* pfSuspendedFlag */
   if ( psResourceControlReq->pfSuspendedFlag )
      write_DbvBool( "pfSuspendedFlag", *psResourceControlReq->pfSuspendedFlag, OutTable);
   else
      write_title_char("NULL","pfSuspendedFlag", OutTable);
   TBL_PutRecord(OutTable);



   /* psResourceReport */
   write_DbvASN1External( psResourceControlReq->psResourceReport, OutTable );


   /* pePartialResultsAvailable */
   write_DbvPartialResultsAvailable( "pePartialResultsAvailable", psResourceControlReq->pePartialResultsAvailable, OutTable );


   /* fResponseRequired */
   write_DbvBool( "fResponseRequired", psResourceControlReq->fResponseRequired, OutTable);
   TBL_PutRecord(OutTable);


   /* pfTriggeredRequestFlag */
   if ( psResourceControlReq->pfTriggeredRequestFlag )
      write_DbvBool( "pfTriggeredRequestFlag", *psResourceControlReq->pfTriggeredRequestFlag, OutTable);
   else
      write_title_char("NULL","pfTriggeredRequestFlag", OutTable);
   TBL_PutRecord(OutTable);



   /* ppsOtherInformation */
   writeN_DbvOtherInformation( psResourceControlReq->ppsOtherInformation, OutTable );

   write_comment_field( "END of DbvResourceControlReq", OutTable, T_Return );
   TBL_PutRecord( OutTable );

   } /* End write_DbvResourceControlRequset() */



