/*  Module Name: @(#)
    Version: 
    Author: Dan Maskell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to write structure types to the result script files.
        Used by the origin and target.
 
*/

#include "write_DbvFuncs.h"

extern TBL_Return T_Return;
/*+-----------------------------------------------+*/
/*! write_DbvCharSetAndLanguage                   !*/ 
/*+-----------------------------------------------+*/
void write_DbvCharSetAndLanguage( char *name, 
	   DbvCharSetAndLanguage *psDbvCharSetAndLanguage,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvCharSetAndLanguage == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvCharSetAndLanguage", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvCharSetAndLanguage", OutTable, T_Return );
 
 
   /* eCharSetAndLanguageType */
   pc_string = NULL;
   convert_DbvCharSetAndLanguageType( &pc_string, &psDbvCharSetAndLanguage->eCharSetAndLanguageType );
   sprintf( temp, "eCharSetAndLanguageType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Union uCharSetAndLanguage */
   switch ( psDbvCharSetAndLanguage->eCharSetAndLanguageType )
      {
      case CSALT_Proposal:
         write_DbvOriginProposal( "psProposal", psDbvCharSetAndLanguage->uCharSetAndLanguage.psProposal, OutTable);
         break;

      case CSALT_Response:
         write_DbvTargetResponse( "psResponse", psDbvCharSetAndLanguage->uCharSetAndLanguage.psResponse, OutTable);
         break;

      default:
	 break;
      } /* End switch */
   
   write_comment_field( "END of DbvCharSetAndLanguage", OutTable, T_Return );
   } /* End write_DbvCharSetAndLanguage() */
 
/*+-----------------------------------------------+*/
/*! write_DbvTargetResponse                       !*/ 
/*+-----------------------------------------------+*/
void write_DbvTargetResponse( char *name, 
	   DbvTargetResponse *psDbvTargetResponse,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvTargetResponse == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvTargetResponse", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvTargetResponse", OutTable, T_Return );
 
 
   /* psSelectedCharSets */
   write_DbvSelectedCharSet( "psSelectedCharSets", psDbvTargetResponse->psSelectedCharSets, OutTable);
 
   /* psSelectedLanguage */
   write_DbvGeneralString( "psSelectedLanguage", psDbvTargetResponse->psSelectedLanguage, OutTable );

  /* pfRecordsInSelectedCharSets */
  if ( psDbvTargetResponse->pfRecordsInSelectedCharSets )
      write_DbvBool( "pfRecordsInSelectedCharSets", *psDbvTargetResponse->pfRecordsInSelectedCharSets, OutTable);
   else
      write_title_char("NULL","pfRecordsInSelectedCharSets", OutTable);
   TBL_PutRecord(OutTable);

   
   write_comment_field( "END of DbvTargetResponse", OutTable, T_Return );
   } /* End write_DbvTargetResponse() */
 
/*+-----------------------------------------------+*/
/*! write_DbvSelectedCharSet                      !*/ 
/*+-----------------------------------------------+*/
void write_DbvSelectedCharSet( char *name, 
	   DbvSelectedCharSet *psDbvSelectedCharSet,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvSelectedCharSet == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvSelectedCharSet", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvSelectedCharSet", OutTable, T_Return );
 
 
   /* eSelectedCharSetType */
   pc_string = NULL;
   convert_DbvSelectedCharSetType( &pc_string, &psDbvSelectedCharSet->eSelectedCharSetType );
   sprintf( temp, "eSelectedCharSetType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Union uSelectedCharSet */
   switch ( psDbvSelectedCharSet->eSelectedCharSetType )
      {
      case SCST_Iso2022:
         write_DbvIso2022( "psIso2022", psDbvSelectedCharSet->uSelectedCharSet.psIso2022, OutTable);
         break;

      case SCST_Iso10646:
         write_DbvIso10646( "psIso10646", psDbvSelectedCharSet->uSelectedCharSet.psIso10646, OutTable);
         break;

      case SCST_Private:
         write_DbvPrivateCharacterSet( "psPrivate", psDbvSelectedCharSet->uSelectedCharSet.psPrivate, OutTable);
         break;

      default:
	 break;
      } /* End switch */
   
   write_comment_field( "END of DbvSelectedCharSet", OutTable, T_Return );
   } /* End write_DbvSelectedCharSet() */
 
/*+-----------------------------------------------+*/
/*! write_DbvOriginProposal                       !*/ 
/*+-----------------------------------------------+*/
void write_DbvOriginProposal( char *name, 
	   DbvOriginProposal *psDbvOriginProposal,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvOriginProposal == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvOriginProposal", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvOriginProposal", OutTable, T_Return );
 
 
   /* ppsProposedCharSets */
   writeN_DbvProposedCharSet( "ppsProposedCharSets", psDbvOriginProposal->ppsProposedCharSets, OutTable );
 
   /* ppsProposedLanguages */
   writeN_DbvLanguageCode( "ppsProposedLanguages", psDbvOriginProposal->ppsProposedLanguages, OutTable );
   
  /* pfRecordsInSelectedCharSets */
  if ( psDbvOriginProposal->pfRecordsInSelectedCharSets )
      write_DbvBool( "pfRecordsInSelectedCharSets", *psDbvOriginProposal->pfRecordsInSelectedCharSets, OutTable);
   else
      write_title_char("NULL","pfRecordsInSelectedCharSets", OutTable);
   TBL_PutRecord(OutTable);


   write_comment_field( "END of DbvOriginProposal", OutTable, T_Return );
   } /* End write_DbvOriginProposal() */
 
/*+-----------------------------------------------+*/
/*! write_DbvProposedCharSet                      !*/ 
/*+-----------------------------------------------+*/
void write_DbvProposedCharSet( char *name, 
	   DbvProposedCharSet *psDbvProposedCharSet,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvProposedCharSet == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvProposedCharSet", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvProposedCharSet", OutTable, T_Return );
 
 
   /* eProposedCharSetType */
   pc_string = NULL;
   convert_DbvProposedCharSetType( &pc_string, &psDbvProposedCharSet->eProposedCharSetType );
   sprintf( temp, "eProposedCharSetType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Union uProposedCharSet */
   switch ( psDbvProposedCharSet->eProposedCharSetType )
      {
      case PRCST_Iso2022:
         write_DbvIso2022( "psIso2022", psDbvProposedCharSet->uProposedCharSet.psIso2022, OutTable);
         break;

      case PRCST_Iso10646:
         write_DbvIso10646( "psIso10646", psDbvProposedCharSet->uProposedCharSet.psIso10646, OutTable);
         break;

      case PRCST_Private:
         write_DbvPrivateCharacterSet( "psPrivate", psDbvProposedCharSet->uProposedCharSet.psPrivate, OutTable);
         break;

      default:
	 break;
      } /* End switch */
   
   write_comment_field( "END of DbvProposedCharSet", OutTable, T_Return );
   } /* End write_DbvProposedCharSet() */
 
/*+-----------------------------------------------+*/
/*! write_DbvPrivateCharacterSet                  !*/ 
/*+-----------------------------------------------+*/
void write_DbvPrivateCharacterSet( char *name, 
	   DbvPrivateCharacterSet *psDbvPrivateCharacterSet,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvPrivateCharacterSet == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvPrivateCharacterSet", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvPrivateCharacterSet", OutTable, T_Return );
 
 
   /* ePrivateCharacterSetType */
   pc_string = NULL;
   convert_DbvPrivateCharacterSetType( &pc_string, &psDbvPrivateCharacterSet->ePrivateCharacterSetType );
   sprintf( temp, "ePrivateCharacterSetType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Union uPrivateCharacterSet */
   switch ( psDbvPrivateCharacterSet->ePrivateCharacterSetType )
      {
      /* struct_typedef_p2.s1 */
      case PCST_ViaOid:
         writeN_DbvObjectIdentifier( "ppcViaOid", psDbvPrivateCharacterSet->uPrivateCharacterSet.ppcViaOid, OutTable);
         break;

      /* struct_typedef_p1.s1 */
      case PCST_ExternallySpecified:
         write_DbvASN1External( psDbvPrivateCharacterSet->uPrivateCharacterSet.psExternallySpecified, OutTable);
         break;

      default:
	 break;
      } /* End switch */
   
   write_comment_field( "END of DbvPrivateCharacterSet", OutTable, T_Return );
   } /* End write_DbvPrivateCharacterSet() */
 
/*+-----------------------------------------------+*/
/*! write_DbvIso10646                             !*/ 
/*+-----------------------------------------------+*/
void write_DbvIso10646( char *name, 
	   DbvIso10646 *psDbvIso10646,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvIso10646 == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvIso10646", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvIso10646", OutTable, T_Return );
 
 
   /* pcCollections */
   write_DbvObjectIdentifier( "pcCollections", psDbvIso10646->pcCollections, OutTable);
 
   /* pcEncodingLevel */
   write_DbvObjectIdentifier( "pcEncodingLevel", psDbvIso10646->pcEncodingLevel, OutTable);
   
   write_comment_field( "END of DbvIso10646", OutTable, T_Return );
   } /* End write_DbvIso10646() */
 
/*+-----------------------------------------------+*/
/*! write_DbvIso2022                              !*/ 
/*+-----------------------------------------------+*/
void write_DbvIso2022( char *name, 
	   DbvIso2022 *psDbvIso2022,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvIso2022 == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvIso2022", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvIso2022", OutTable, T_Return );
 
 
   /* eIso2022Type */
   pc_string = NULL;
   convert_DbvIso2022Type( &pc_string, &psDbvIso2022->eIso2022Type );
   sprintf( temp, "eIso2022Type : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   /* Union uIso2022Type */
   switch ( psDbvIso2022->eIso2022Type )
      {
      case I2022T_OriginProposal:
         write_DbvIso2022Proposal( "psOriginProposal", psDbvIso2022->uIso2022Type.psOriginProposal, OutTable);
         break;

      case I2022T_TargetResponse:
         write_DbvIso2022Response( "psTargetResponse", psDbvIso2022->uIso2022Type.psTargetResponse, OutTable);
         break;

      default:
	 break;
      } /* End switch */
   
   write_comment_field( "END of DbvIso2022", OutTable, T_Return );
   } /* End write_DbvIso2022() */
 
/*+-----------------------------------------------+*/
/*! write_DbvIso2022Response                      !*/ 
/*+-----------------------------------------------+*/
void write_DbvIso2022Response( char *name, 
	   DbvIso2022Response *psDbvIso2022Response,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvIso2022Response == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvIso2022Response", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvIso2022Response", OutTable, T_Return );
 
 
   /* psSelectedEnvironment */
   write_DbvEnvironment( "psSelectedEnvironment", psDbvIso2022Response->psSelectedEnvironment, OutTable);
 
   /* ppulSelectedSets */
   writeN_Ulong( "ppulSelectedSets", psDbvIso2022Response->ppulSelectedSets, OutTable);
 
   /* psSelectedInitialSet */
   write_DbvInitialSet( "psSelectedInitialSet", psDbvIso2022Response->psSelectedInitialSet, OutTable);
 
   /* psSelectedLeftAndRight */
   write_DbvLeftAndRight( "psSelectedLeftAndRight", psDbvIso2022Response->psSelectedLeftAndRight, OutTable);
   
   write_comment_field( "END of DbvIso2022Response", OutTable, T_Return );
   } /* End write_DbvIso2022Response() */
 
/*+-----------------------------------------------+*/
/*! write_DbvIso2022Proposal                      !*/ 
/*+-----------------------------------------------+*/
void write_DbvIso2022Proposal( char *name, 
	   DbvIso2022Proposal *psDbvIso2022Proposal,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvIso2022Proposal == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvIso2022Proposal", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvIso2022Proposal", OutTable, T_Return );
 
 
   /* psProposedEnvironment */
   write_DbvEnvironment( "psProposedEnvironment", psDbvIso2022Proposal->psProposedEnvironment, OutTable);
 
   /* ppulProposedSets */
   writeN_Ulong( "ppulProposedSets", psDbvIso2022Proposal->ppulProposedSets, OutTable);
 
   /* ppsProposedInitialSets */
   writeN_DbvInitialSet( "ppsProposedInitialSets", psDbvIso2022Proposal->ppsProposedInitialSets, OutTable );
 
   /* psProposedLeftAndRight */
   write_DbvLeftAndRight( "psProposedLeftAndRight", psDbvIso2022Proposal->psProposedLeftAndRight, OutTable);
   
   write_comment_field( "END of DbvIso2022Proposal", OutTable, T_Return );
   } /* End write_DbvIso2022Proposal() */
 
/*+-----------------------------------------------+*/
/*! write_DbvLeftAndRight                         !*/ 
/*+-----------------------------------------------+*/
void write_DbvLeftAndRight( char *name, 
	   DbvLeftAndRight *psDbvLeftAndRight,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvLeftAndRight == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvLeftAndRight", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvLeftAndRight", OutTable, T_Return );
 
 
   /* ulGLeft */
   write_integer_string( "ulGLeft", (int*)&psDbvLeftAndRight->ulGLeft, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulGRight */
   write_integer_string( "ulGRight", (int*)&psDbvLeftAndRight->ulGRight, OutTable);
   TBL_PutRecord( OutTable );
   
   write_comment_field( "END of DbvLeftAndRight", OutTable, T_Return );
   } /* End write_DbvLeftAndRight() */
 
/*+-----------------------------------------------+*/
/*! write_DbvInitialSet                           !*/ 
/*+-----------------------------------------------+*/
void write_DbvInitialSet( char *name, 
	   DbvInitialSet *psDbvInitialSet,
	   TBL OutTable)
   {
  
   /* Write the title out */ 
   if(psDbvInitialSet == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvInitialSet", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvInitialSet", OutTable, T_Return );
 
 
   /* ulG0 */
   write_integer_string( "ulG0", (int*)&psDbvInitialSet->ulG0, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulG1 */
   write_integer_string( "ulG1", (int*)&psDbvInitialSet->ulG1, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulG2 */
   write_integer_string( "ulG2", (int*)&psDbvInitialSet->ulG2, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulG3 */
   write_integer_string( "ulG3", (int*)&psDbvInitialSet->ulG3, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulC0 */
   write_integer_string( "ulC0", (int*)&psDbvInitialSet->ulC0, OutTable);
   TBL_PutRecord( OutTable );
 
   /* ulC1 */
   write_integer_string( "ulC1", (int*)&psDbvInitialSet->ulC1, OutTable);
   TBL_PutRecord( OutTable );
   
   write_comment_field( "END of DbvInitialSet", OutTable, T_Return );
   } /* End write_DbvInitialSet() */
 
/*+-----------------------------------------------+*/
/*! write_DbvEnvironment                          !*/ 
/*+-----------------------------------------------+*/
void write_DbvEnvironment( char *name, 
	   DbvEnvironment *psDbvEnvironment,
	   TBL OutTable)
   {
   char temp[ STRING_LEN ];
   char *pc_string;
  
   /* Write the title out */ 
   if(psDbvEnvironment == NULL)
      {
      write_title_char("NULL",name, OutTable);
      TBL_PutRecord(OutTable);
      return;
      } /* End if */
  
   write_title_char( "psDbvEnvironment", "Structure", OutTable);
   TBL_PutRecord( OutTable );
   write_comment_field( "START of DbvEnvironment", OutTable, T_Return );
 
 
   /* eEnvironmentType */
   pc_string = NULL;
   convert_DbvEnvironmentType( &pc_string, &psDbvEnvironment->eEnvironmentType );
   sprintf( temp, "eEnvironmentType : %s", pc_string );
   write_char_field( temp, OutTable, T_Return );
   TBL_PutRecord( OutTable );
   DbvFree( pc_string );

   
   write_comment_field( "END of DbvEnvironment", OutTable, T_Return );
   } /* End write_DbvEnvironment() */
 
