/*  Module Name: @(#)free_code.c	7.10
    Version: 8/8/97
    Author: 
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
	Routines to free DBV PDU structures
        Used by the origin and target.
*/




#include "free_code.h" 


/* Origin end freeing function for structure returned by
   DbvReceiveDataOrigin() */
int SIFreeDbvOriginData(DbvOriginData **ppsOriginData)
{

	if (ppsOriginData == NULL)
		return SI_OK;

	if (*ppsOriginData == NULL) 
		return SI_OK;

	switch((*ppsOriginData)->eOriginEvent)
	{
	case OR_InitializeConf:
		FreeDbvInitializeRsp((*ppsOriginData)->uOriginData.psInitializeRsp);
		break;
	case OR_SearchConf:
		Free_DbvSearchRsp((*ppsOriginData)->uOriginData.psSearchRsp);
		break;
	case OR_PresentConf:
		Free_DbvPresentRsp((*ppsOriginData)->uOriginData.psPresentRsp);
		break;
	case OR_DeleteResultSetConf:
		Free_DbvDeleteResultSetRsp((*ppsOriginData)->uOriginData.
			psDeleteResultSetRsp);
		break;
	case OR_ScanConf:
		Free_DbvScanRsp((*ppsOriginData)->uOriginData.psScanRsp);
		break;
	case OR_ResourceReportConf:
		Free_DbvResourceReportRsp((*ppsOriginData)->uOriginData.
			psResourceReportRsp);
		break;
	case OR_SegmentationInd:
		Free_DbvSegmentReq((*ppsOriginData)->uOriginData.psSegmentReq);
		break;
	case OR_SortConf:
		Free_DbvSortRsp((*ppsOriginData)->uOriginData.psSortRsp);
		break;
	case OR_ExtendedServicesConf:
		Free_DbvExtendedServicesRsp((*ppsOriginData)->uOriginData.
			psExtendedServicesRsp);
		break;
	case OR_CloseConf:
		Free_DbvCloseReq((*ppsOriginData)->uOriginData.psCloseReq);
		break;
	case OR_CloseInd:
		Free_DbvCloseReq((*ppsOriginData)->uOriginData.psCloseReq);
		break;
	case OR_AccessCntrlInd:
		Free_DbvAccessControlReq((*ppsOriginData)->uOriginData.psAccessControlReq);
		break;
	case OR_ResourCntrlInd:
		Free_DbvResourceControlReq((*ppsOriginData)->uOriginData.psResourceControlReq);
		break;
	default:
		break;
	}

	DbvFree(*ppsOriginData);
        *ppsOriginData = NULL;

	return SI_OK;
	
}



/* Target end freeing function for structure returned by
   DbvReceiveDataTarget() */
int SIFreeDbvTargetData(DbvTargetData **ppsTargetData)
{
	if (ppsTargetData == NULL)
		return SI_OK;

	if (*ppsTargetData == NULL)
		return SI_OK;

	switch((*ppsTargetData)->eTargetEvent)
	{
	case TA_InitializeInd:
		FreeDbvInitializeReq((*ppsTargetData)->uTargetData.psInitializeReq);
		break;
	case TA_SearchInd:
		Free_DbvSearchReq((*ppsTargetData)->uTargetData.psSearchReq);
		break;
	case TA_PresentInd: 
		Free_DbvPresentReq((*ppsTargetData)->uTargetData.psPresentReq);
		break;
	case TA_DeleteResultSetInd: 
		Free_DbvDeleteResultSetReq((*ppsTargetData)->uTargetData.
			psDeleteResultSetReq);
		break;
	case TA_ScanInd:
		Free_DbvScanReq((*ppsTargetData)->uTargetData.psScanReq);
		break;
	case TA_ResourceReportInd: 
		Free_DbvResourceReportReq((*ppsTargetData)->uTargetData.
			psResourceReportReq);
		break;
	case TA_SortInd: 
		Free_DbvSortReq((*ppsTargetData)->uTargetData.psSortReq);
		break;
	case TA_ExtendedServicesInd: 
		Free_DbvExtendedServicesReq((*ppsTargetData)->uTargetData.
			psExtendedServicesReq);
		break;
	case TA_CloseInd:
                Free_DbvCloseReq((*ppsTargetData)->uTargetData.psCloseReq);
		break;
	case TA_CloseConf:
                Free_DbvCloseReq((*ppsTargetData)->uTargetData.psCloseReq);
		break;
	case TA_ReleaseInd:
		DbvFree((*ppsTargetData)->uTargetData.peReleaseReq);
		break;
	case TA_AccessCntrlConf:
		Free_DbvAccessControlRsp((*ppsTargetData)->uTargetData.psAccessControlRsp);
		break;
	case TA_ResourCntrlConf:
		Free_DbvResourceControlRsp((*ppsTargetData)->uTargetData.psResourceControlRsp);
		break;
	case TA_TrigResourCntrlInd:
		Free_DbvTriggerResourceControlReq((*ppsTargetData)->uTargetData.psTriggerResourceControlReq);
		break;
	default:
		break;
	}

	DbvFree(*ppsTargetData);
        *ppsTargetData = NULL;


	return SI_OK;
	
}



/************************************************************/
/* Origin end freeing functions for PDUs sent to a target */
int SIFreeDbvInitializeReq(DbvInitializeReq **ppsInitializeReq)
{
	if (ppsInitializeReq == NULL)
		return SI_OK;

	if (*ppsInitializeReq == NULL)
		return SI_OK;

	FreeDbvInitializeReq(*ppsInitializeReq);

	*ppsInitializeReq = NULL;

	return SI_OK;
	
}



int SIFreeDbvSearchReq(DbvSearchReq **ppsSearchReq)
{
	if (ppsSearchReq == NULL)
		return SI_OK;

	if (*ppsSearchReq == NULL)
		return SI_OK;

	Free_DbvSearchReq(*ppsSearchReq);

	*ppsSearchReq = NULL;

	return SI_OK;
	
}



int SIFreeDbvPresentReq(DbvPresentReq **ppsPresentReq)
{
	if (ppsPresentReq == NULL)
		return SI_OK;

	if (*ppsPresentReq == NULL)
		return SI_OK;

	Free_DbvPresentReq(*ppsPresentReq);

	*ppsPresentReq = NULL;

	return SI_OK;
}



int SIFreeDbvDeleteResultSetReq(DbvDeleteResultSetReq **ppsDeleteResultSetReq)
{
	if (ppsDeleteResultSetReq == NULL)
		return SI_OK;

	if (*ppsDeleteResultSetReq == NULL)
		return SI_OK;

	Free_DbvDeleteResultSetReq(*ppsDeleteResultSetReq);

	*ppsDeleteResultSetReq = NULL;

	return SI_OK;
}



int SIFreeDbvScanReq(DbvScanReq **ppsScanReq)
{
	if (ppsScanReq == NULL)
		return SI_OK;

	if (*ppsScanReq == NULL)
		return SI_OK;

	Free_DbvScanReq(*ppsScanReq);

	*ppsScanReq = NULL;

	return SI_OK;
}



int SIFreeDbvResourceReportReq(DbvResourceReportReq **ppsResourceReportReq)
{
	if (ppsResourceReportReq == NULL)
		return SI_OK;

	if (*ppsResourceReportReq == NULL)
		return SI_OK;

	Free_DbvResourceReportReq(*ppsResourceReportReq);

	*ppsResourceReportReq = NULL;

	return SI_OK;
}




int SIFreeDbvSortReq(DbvSortReq **ppsSortReq)
{
	if (ppsSortReq == NULL)
		return SI_OK;

	if (*ppsSortReq == NULL)
		return SI_OK;

	Free_DbvSortReq(*ppsSortReq);

	*ppsSortReq = NULL;

	return SI_OK;
}



int SIFreeDbvExtendedServicesReq(DbvExtendedServicesReq **ppsExtendedServicesReq)
{
	if (ppsExtendedServicesReq == NULL)
		return SI_OK;

	if (*ppsExtendedServicesReq == NULL)
		return SI_OK;

	Free_DbvExtendedServicesReq(*ppsExtendedServicesReq);

	*ppsExtendedServicesReq = NULL;

	return SI_OK;
}




int SIFreeDbvCloseReq(DbvCloseReq **ppsCloseReq)
{
	if (ppsCloseReq == NULL)
		return SI_OK;

	if (*ppsCloseReq == NULL)
		return SI_OK;

	Free_DbvCloseReq(*ppsCloseReq);

	*ppsCloseReq = NULL;

	return SI_OK;
}



int SIFreeDbvOctetString(DbvOctetString **ppsOctetString)
{
	if (ppsOctetString == NULL)
		return SI_OK;

	if (*ppsOctetString == NULL)
		return SI_OK;

	FreeDbvOctetString(*ppsOctetString);

	*ppsOctetString = NULL;

	return SI_OK;
}






int SIFreeDbvAccessControlRsp(DbvAccessControlRsp **ppsAccessControlRsp)
{
	if (ppsAccessControlRsp == NULL)
		return SI_OK;

	if (*ppsAccessControlRsp == NULL)
		return SI_OK;

	Free_DbvAccessControlRsp(*ppsAccessControlRsp);

	*ppsAccessControlRsp = NULL;

	return SI_OK;
}




int SIFreeDbvResourceControlRsp(DbvResourceControlRsp **ppsResourceControlRsp)
{
	if (ppsResourceControlRsp == NULL)
		return SI_OK;

	if (*ppsResourceControlRsp == NULL)
		return SI_OK;

	Free_DbvResourceControlRsp(*ppsResourceControlRsp);

	*ppsResourceControlRsp = NULL;

	return SI_OK;
}



int SIFreeDbvTriggerResourceControlReq(DbvTriggerResourceControlReq **ppsTriggerResourceControlReq)
{
	if (ppsTriggerResourceControlReq == NULL)
		return SI_OK;

	if (*ppsTriggerResourceControlReq == NULL)
		return SI_OK;

	Free_DbvTriggerResourceControlReq(*ppsTriggerResourceControlReq);

	*ppsTriggerResourceControlReq = NULL;

	return SI_OK;
}



/***************************************************************/
/* Target end freeing functions for PDUs sent to an origin */
int SIFreeDbvInitializeRsp(DbvInitializeRsp **ppsInitializeRsp)
{
	if (ppsInitializeRsp == NULL)
		return SI_OK;

	if (*ppsInitializeRsp == NULL)
		return SI_OK;

	FreeDbvInitializeRsp(*ppsInitializeRsp);

	*ppsInitializeRsp = NULL;

	return SI_OK;
}



int SIFreeDbvSearchRsp(DbvSearchRsp **ppsSearchRsp)
{
	if (ppsSearchRsp == NULL)
		return SI_OK;

	if (*ppsSearchRsp == NULL)
		return SI_OK;

	Free_DbvSearchRsp(*ppsSearchRsp);

	*ppsSearchRsp = NULL;

	return SI_OK;
}



int SIFreeDbvPresentRsp(DbvPresentRsp **ppsPresentRsp)
{
	if (ppsPresentRsp == NULL)
		return SI_OK;

	if (*ppsPresentRsp == NULL)
		return SI_OK;

	Free_DbvPresentRsp(*ppsPresentRsp);

	*ppsPresentRsp = NULL;

	return SI_OK;
}



int SIFreeDbvDeleteResultSetRsp(DbvDeleteResultSetRsp **ppsDeleteResultSetRsp)
{
	if (ppsDeleteResultSetRsp == NULL)
		return SI_OK;

	if (*ppsDeleteResultSetRsp == NULL)
		return SI_OK;

	Free_DbvDeleteResultSetRsp(*ppsDeleteResultSetRsp);

	*ppsDeleteResultSetRsp = NULL;

	return SI_OK;
}



int SIFreeDbvScanRsp(DbvScanRsp **ppsScanRsp)
{
	if (ppsScanRsp == NULL)
		return SI_OK;

	if (*ppsScanRsp == NULL)
		return SI_OK;

	Free_DbvScanRsp(*ppsScanRsp);

	*ppsScanRsp = NULL;

	return SI_OK;
}



int SIFreeDbvResourceReportRsp(DbvResourceReportRsp **ppsResourceReportRsp)
{
	if (ppsResourceReportRsp == NULL)
		return SI_OK;

	if (*ppsResourceReportRsp == NULL)
		return SI_OK;

	Free_DbvResourceReportRsp(*ppsResourceReportRsp);

	*ppsResourceReportRsp = NULL;

	return SI_OK;
}



int SIFreeDbvSegmentReq(DbvSegmentReq **ppsSegmentReq)
{
	if (ppsSegmentReq == NULL)
		return SI_OK;

	if (*ppsSegmentReq == NULL)
		return SI_OK;

	Free_DbvSegmentReq(*ppsSegmentReq);

	*ppsSegmentReq = NULL;

	return SI_OK;
}



int SIFreeDbvSortRsp(DbvSortRsp **ppsSortRsp)
{
	if (ppsSortRsp == NULL)
		return SI_OK;

	if (*ppsSortRsp == NULL)
		return SI_OK;

	Free_DbvSortRsp(*ppsSortRsp);

	*ppsSortRsp = NULL;

	return SI_OK;
}



int SIFreeDbvExtendedServicesRsp(DbvExtendedServicesRsp **ppsExtendedServicesRsp)
{
	if (ppsExtendedServicesRsp == NULL)
		return SI_OK;

	if (*ppsExtendedServicesRsp == NULL)
		return SI_OK;

	Free_DbvExtendedServicesRsp(*ppsExtendedServicesRsp);

	*ppsExtendedServicesRsp = NULL;

	return SI_OK;
}



int SIFreeDbvAccessControlReq(DbvAccessControlReq **ppsAccessControlReq)
{
	if (ppsAccessControlReq == NULL)
		return SI_OK;

	if (*ppsAccessControlReq == NULL)
		return SI_OK;

	Free_DbvAccessControlReq(*ppsAccessControlReq);

	*ppsAccessControlReq = NULL;

	return SI_OK;
}



int SIFreeDbvResourceControlReq(DbvResourceControlReq **ppsResourceControlReq)
{
	if (ppsResourceControlReq == NULL)
		return SI_OK;

	if (*ppsResourceControlReq == NULL)
		return SI_OK;

	Free_DbvResourceControlReq(*ppsResourceControlReq);

	*ppsResourceControlReq = NULL;

	return SI_OK;
}




/* Now for the imported testtool stuff ... */
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/


#define Free_compulsory_DbvRequesterOptionalMessages( a )


void FreeDbvOriginData(DbvOriginData *psOriginData)
{
	printf("Freeing the APDU received from the target\n");

	if(psOriginData == NULL)
		return;

	switch(psOriginData->eOriginEvent)
	{
	case OR_InitializeConf:
		FreeDbvInitializeRsp(psOriginData->uOriginData.psInitializeRsp);
		break;
	case OR_SearchConf:
		Free_DbvSearchRsp(psOriginData->uOriginData.psSearchRsp);
		break;
	case OR_PresentConf:
		Free_DbvPresentRsp(psOriginData->uOriginData.psPresentRsp);
		break;
	case OR_DeleteResultSetConf:
		Free_DbvDeleteResultSetRsp(psOriginData->uOriginData.
			psDeleteResultSetRsp);
		break;
	case OR_ScanConf:
		Free_DbvScanRsp(psOriginData->uOriginData.psScanRsp);
		break;
	case OR_ResourceReportConf:
		Free_DbvResourceReportRsp(psOriginData->uOriginData.
			psResourceReportRsp);
		break;
	case OR_SegmentationInd:
		Free_DbvSegmentReq(psOriginData->uOriginData.psSegmentReq);
		break;
	case OR_SortConf:
		Free_DbvSortRsp(psOriginData->uOriginData.psSortRsp);
		break;
	case OR_ExtendedServicesConf:
		Free_DbvExtendedServicesRsp(psOriginData->uOriginData.
			psExtendedServicesRsp);
		break;
	case OR_CloseConf:
		Free_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_CloseInd:
		Free_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_AccessCntrlInd:
		Free_DbvAccessControlReq(psOriginData->uOriginData.psAccessControlReq);
		break;
	case OR_ResourCntrlInd:
		Free_DbvResourceControlReq(psOriginData->uOriginData.psResourceControlReq);
		break;
	default:
		printf("apifree: Not done yet!\n");
		break;
	}

	printf("Completed the free\n");
	
}

void FreeDbvTargetData(DbvTargetData *psTargetData)
{
	printf("Freeing the APDU received from the target\n");
	if(psTargetData == NULL)
		return;

	switch(psTargetData->eTargetEvent)
	{
	case TA_InitializeInd:
		FreeDbvInitializeReq(psTargetData->uTargetData.psInitializeReq);
		break;
	case TA_SearchInd:
		Free_DbvSearchReq(psTargetData->uTargetData.psSearchReq);
		break;
	case TA_PresentInd: 
		Free_DbvPresentReq(psTargetData->uTargetData.psPresentReq);
		break;
	case TA_DeleteResultSetInd: 
		Free_DbvDeleteResultSetReq(psTargetData->uTargetData.
			psDeleteResultSetReq);
		break;
	case TA_ScanInd:
		Free_DbvScanReq(psTargetData->uTargetData.psScanReq);
		break;
	case TA_ResourceReportInd: 
		Free_DbvResourceReportReq(psTargetData->uTargetData.
			psResourceReportReq);
		break;
	case TA_SortInd: 
		Free_DbvSortReq(psTargetData->uTargetData.psSortReq);
		break;
	case TA_ExtendedServicesInd: 
		Free_DbvExtendedServicesReq(psTargetData->uTargetData.
			psExtendedServicesReq);
		break;
	case TA_CloseInd:
                Free_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_CloseConf:
                Free_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_ReleaseInd:
		DbvFree(psTargetData->uTargetData.peReleaseReq);
		break;
	case TA_AccessCntrlConf:
		Free_DbvAccessControlRsp(psTargetData->uTargetData.psAccessControlRsp);
		break;
	case TA_ResourCntrlConf:
		Free_DbvResourceControlRsp(psTargetData->uTargetData.psResourceControlRsp);
		break;
	case TA_TrigResourCntrlInd:
		Free_DbvTriggerResourceControlReq(psTargetData->uTargetData.psTriggerResourceControlReq);
		break;
	default:
		printf("Not done yet!\n");
		break;
	}
	
	printf("Completed the free\n");
}

void FreeDbvInitializeRsp(DbvInitializeRsp *psInitializeRsp)
{
	if(psInitializeRsp->psReferenceId != NULL)
		FreeDbvOctetString(psInitializeRsp->psReferenceId);
	FreeDbvProtocolVersion(psInitializeRsp->pcProtocolVersion);
	FreeDbvOptions(psInitializeRsp->pcOptions);
	if(psInitializeRsp->psImplementationId != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationId);
	if(psInitializeRsp->psImplementationName != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationName);
	if(psInitializeRsp->psImplementationVersion != NULL)
		FreeDbvInternationalString(psInitializeRsp->psImplementationVersion);
	if(psInitializeRsp->psUserInformationField != NULL)
		FreeDbvASN1External(psInitializeRsp->psUserInformationField);
	if(psInitializeRsp->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeRsp->ppsOtherInformation);
        DbvFree(psInitializeRsp);
}

void FreeDbvInitializeReq(DbvInitializeReq *psInitializeReq)
{
	if(psInitializeReq->psReferenceId != NULL)
		FreeDbvOctetString(psInitializeReq->psReferenceId);
	FreeDbvProtocolVersion(psInitializeReq->pcProtocolVersion);
	FreeDbvOptions(psInitializeReq->pcOptions);
	if(psInitializeReq->psAuthentication != NULL)
		FreeDbvAuthentication(psInitializeReq->psAuthentication);
	if(psInitializeReq->psImplementationId != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationId);
	if(psInitializeReq->psImplementationName != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationName);
	if(psInitializeReq->psImplementationVersion != NULL)
		FreeDbvInternationalString(psInitializeReq->psImplementationVersion);
	if(psInitializeReq->psUserInformationField != NULL)
		FreeDbvASN1External(psInitializeReq->psUserInformationField);
	if(psInitializeReq->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeReq->ppsOtherInformation);
	DbvFree(psInitializeReq);
}

void FreeDbvOctetString(DbvOctetString *psOctetString)
{
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
		DbvFree(psOctetString);
	}
}



void Free_compulsory_DbvOctetString(DbvOctetString *psOctetString)
{   
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
	}
}



void FreeDbvProtocolVersion(DbvProtocolVersion pcProtocolVersion)
{
	if(pcProtocolVersion != NULL)
		DbvFree(pcProtocolVersion);
}

void FreeDbvOptions(DbvOptions pcOptions)
{
	if(pcOptions != NULL)
		DbvFree(pcOptions);
}

void FreeDbvInternationalString(DbvInternationalString *psInternationalString)
{
	if(psInternationalString != NULL)
	{
		if(psInternationalString->pcValue != NULL)
			DbvFree(psInternationalString->pcValue);
		DbvFree(psInternationalString);
	}
}

void FreeDbvObjectIdentifier(DbvObjectIdentifier pcObjectIdentifier)
{
	if(pcObjectIdentifier != NULL)
		DbvFree(pcObjectIdentifier);
}

void FreeDbvOtherInformation(DbvOtherInformation *psOtherInformation)
{
	if(psOtherInformation == NULL)
		return;

	switch(psOtherInformation->eInformationType)
	{
	case IT_CharacterInfo:
		FreeDbvInternationalString(psOtherInformation->uInformation.psCharacterInfo);
		break;
	case IT_BinaryInfo:
		FreeDbvOctetString(psOtherInformation->uInformation.psBinaryInfo);
		break;
	case IT_ExternalInfo:
		FreeDbvASN1External(psOtherInformation->uInformation.psExternallyDefinedInfo);
		break;
	case IT_ObjectIdentifier:
		FreeDbvObjectIdentifier(psOtherInformation->uInformation.pcObjectIdentifier);
		break;
	default:
		printf("OtherInformation: error\n");
		break;
	}
	if(psOtherInformation->psInfoCategory != NULL)
		FreeDbvInfoCategory(psOtherInformation->psInfoCategory);
	DbvFree(psOtherInformation);
}

void FreeDbvInfoCategory(DbvInfoCategory *psInfoCategory)
{
	FreeDbvObjectIdentifier(psInfoCategory->pcCategoryTypeId);
	DbvFree(psInfoCategory);
}

void FreeDbvIdPassword(DbvIdPassword *psIdPassword)
{
	if(psIdPassword != NULL)
	{
		FreeDbvInternationalString(psIdPassword->psGroupId);
		FreeDbvInternationalString(psIdPassword->psUserId);
		FreeDbvInternationalString(psIdPassword->psPassword);
	}
}

void FreeDbvAuthentication(DbvAuthentication *psAuthentication)
{
	if(psAuthentication == NULL)
		return;

	switch(psAuthentication->eAuthenticationType)
	{
	case AT_Open:
		DbvFree(psAuthentication->uAuthentication.psOpen);
		break;
	case AT_IdPassword:
		FreeDbvIdPassword(psAuthentication->uAuthentication.psIdPassword);
		break;
	case AT_Anonymous:
		break;
	case  AT_Other:
		FreeDbvASN1External(psAuthentication->uAuthentication.psOther);
		break;
	default:
		printf("FreeAuthentication: error\n");
		break;
	}

	DbvFree(psAuthentication);
}




void Free_compulsory_DbvAdditionalInfo( DbvAdditionalInfo *psAdditionalInfo )
   {

   if ( psAdditionalInfo == NULL )
      return;

   switch(  psAdditionalInfo->eProtocolVersionInForce )
      {
      case PVIF_Version2:
         Free_DbvVisibleString( psAdditionalInfo->uAdditionalInfo.pcV2 );
	 break;

      case PVIF_Version3:
         FreeDbvInternationalString( psAdditionalInfo->uAdditionalInfo.psV3 );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvAdditionalInfo() */ 

void Free_compulsory_DbvAttributeValue( DbvAttributeValue *psAttributeValue )
   {

   if ( psAttributeValue == NULL )
      return;

   switch(  psAttributeValue->eAttributeValueType )
      {
      case AVT_Numeric:
	 /* No space allocated */
	 break;

      case AVT_Complex:
         Free_DbvComplexValue( psAttributeValue->
			       uAttributeValue.psComplexValue );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvAttributeValue() */ 

void Free_compulsory_DbvHoldingsAndCircData( DbvHoldingsAndCircData 
					*psHoldingsAndCircData )
   {

   /* psTypeOfRecord */

   if ( psHoldingsAndCircData->psTypeOfRecord != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psTypeOfRecord );

   /* psEncodingLevel */

   if ( psHoldingsAndCircData->psEncodingLevel != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psEncodingLevel );

   /* psFormat */

   if ( psHoldingsAndCircData->psFormat != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psFormat );

   /* psReceiptAcqStatus */

   if ( psHoldingsAndCircData->psReceiptAcqStatus != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psReceiptAcqStatus );

   /* psGeneralRetention */

   if ( psHoldingsAndCircData->psGeneralRetention != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psGeneralRetention );

   /* psCompleteness */

   if ( psHoldingsAndCircData->psCompleteness != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCompleteness );

   /* psDateOfReport */

   if ( psHoldingsAndCircData->psDateOfReport != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psDateOfReport );

   /* psNucCode */

   if ( psHoldingsAndCircData->psNucCode != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psNucCode );

   /* psLocalLocation */

   if ( psHoldingsAndCircData->psLocalLocation != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psLocalLocation );

   /* psShelvingLocation */

   if ( psHoldingsAndCircData->psShelvingLocation != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psShelvingLocation );

   /* psCallNumber */

   if ( psHoldingsAndCircData->psCallNumber != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCallNumber );

   /* psShelvingData */

   if ( psHoldingsAndCircData->psShelvingData != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psShelvingData );

   /* psCopyNumber */

   if ( psHoldingsAndCircData->psCopyNumber != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psCopyNumber );

   /* psPublicNote */

   if ( psHoldingsAndCircData->psPublicNote != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psPublicNote );

   /* psReproductionNote */

   if ( psHoldingsAndCircData->psReproductionNote != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psReproductionNote );

   /* psTermsUseRepro */

   if ( psHoldingsAndCircData->psTermsUseRepro != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psTermsUseRepro );

   /* psEnumAndChron */

   if ( psHoldingsAndCircData->psEnumAndChron != NULL )
      FreeDbvInternationalString( psHoldingsAndCircData->psEnumAndChron );

   /* ppsVolume */

   FreeN_DbvVolume( psHoldingsAndCircData->ppsVolume );

   /* ppsCircRecord*/

   FreeN_DbvCircRecord( psHoldingsAndCircData->ppsCircRecord );

   /* Note No final free as compulsory */

   } /* End Free_DbvHoldingsAndCircData() */ 

void Free_compulsory_DbvInternationalString( DbvInternationalString 
					*psInternationalString)
   {
   DbvFree(psInternationalString->pcValue);
   } /* Free_compulsory_DbvInternationalString() */

void Free_compulsory_DbvOriginPartToKeep( DbvOriginPartToKeep 
						*psOriginPartToKeep )
   {

   if ( psOriginPartToKeep == NULL )
      return;

   /* sDatabaseName */

   Free_compulsory_DbvInternationalString( &psOriginPartToKeep->sDatabaseName ); 
   /* pcSchema */

   if ( psOriginPartToKeep->pcSchema != NULL )
      FreeDbvObjectIdentifier( psOriginPartToKeep->pcSchema );

   /* psElementSetName */

   if ( psOriginPartToKeep->psElementSetName != NULL )
      FreeDbvInternationalString( psOriginPartToKeep->psElementSetName ); 


   } /* End Free_compulsory_DbvOriginPartToKeep() */ 

void Free_compulsory_DbvTerm( DbvTerm *psTerm )
   {

   if ( psTerm == NULL )
      return;

   switch( psTerm->eTermType )
      {
      case TT_General:
         if ( psTerm->uTerm.psGeneral != NULL )
	    FreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
         /* No Space allocated */
	 break;

      case TT_SimpleCharacter:
         if ( psTerm->uTerm.psSimpleCharacter != NULL )
            FreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter ); 
	 break;

      case TT_ObjectIdentifier:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            FreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            Free_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
         FreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 Free_DbvIntUnit(  psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_compulsory_DbvTerm() */ 

void Free_compulsory_DbvUnit( DbvUnit *psUnit )
   {

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */

   if ( psUnit->psUnitSystem != NULL )
      FreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */

   if ( psUnit->psUnitType != NULL )
      Free_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */

   if ( psUnit->psUnit != NULL )
      Free_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */

   if ( psUnit->pulScaleFactor != NULL )
      Free_Ulong( psUnit->pulScaleFactor );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End Free_compulsory_DbvUnit() */

void Free_compulsory_DbvUpdateTargetPart( DbvUpdateTargetPart 
						*psUpdateTargetPart )
   {

   if ( psUpdateTargetPart == NULL )
      return;

   /* ppsGlobalDiagnostics */

   FreeN_DbvDiagRec( psUpdateTargetPart->ppsGlobalDiagnostics );

   /* ppsTaskPackageRecord */

   FreeN_DbvTaskPackageRecord( psUpdateTargetPart->ppsTaskPackageRecord );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End Free_compulsory_DbvUpdateTargetPart() */



void Free_compulsory_DbvSpecification(DbvSpecification *psSpecification)
{
	FreeDbvObjectIdentifier(psSpecification->pcSchema);

	/* optional */
	Free_DbvElementSpec(psSpecification->psElementSpec);
} /* End Free_compulsory_DbvSpecification */


/* 12-2-96 The following function where added by Dan Maskell */

void Free_compulsory_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      do_Free_DbvEsIoTargetPart( psEsIoTargetPart );

   } /* End Free_compulsory_DbvEsIoTargetPart() */ 

void Free_compulsory_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      do_Free_DbvIllString( psIllString );

   } /* End Free_compulsory_DbvIllString() */ 

void Free_compulsory_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      do_Free_DbvDateTime( psDateTime );

   } /* End Free_compulsory_DbvDateTime() */ 


void Free_compulsory_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      do_Free_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );

   } /* End Free_compulsory_DbvUserOrProviderErrorReport() */ 

void Free_compulsory_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      do_Free_DbvHistoryReport( psHistoryReport );

   } /* End Free_compulsory_DbvHistoryReport() */ 

void Free_compulsory_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      do_Free_DbvItemId( psItemId );

   } /* End Free_compulsory_DbvItemId() */ 

void Free_compulsory_DbvElDeliveryDetails( 
		DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      do_Free_DbvElDeliveryDetails( psElDeliveryDetails );

   } /* End Free_compulsory_DbvElDeliveryDetails() */ 

void Free_compulsory_DbvSystemId( DbvSystemId *psSystemId )
   {

   if ( psSystemId != NULL )
      do_Free_DbvSystemId( psSystemId );

   } /* End Free_compulsory_DbvSystemId() */ 

void Free_compulsory_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      do_Free_DbvTransactionId( psTransactionId );

   } /* End Free_compulsory_DbvTransactionId() */ 

void Free_compulsory_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      do_Free_DbvServiceDateTime( psServiceDateTime );

   } /* End Free_compulsory_DbvServiceDateTime() */ 


void Free_compulsory_DbvEsIoPaymentMethod( 
		DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod != NULL )
      do_Free_DbvEsIoPaymentMethod( psEsIoPaymentMethod );

   } /* End Free_compulsory_DbvEsIoPaymentMethod() */ 

void Free_compulsory_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      do_Free_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );

   } /* End Free_compulsory_DbvEsIoOriginPartNotToKeep() */ 





/* Explain Record Support */


void Free_compulsory_DbvCharge( DbvCharge *psCharge )
   {

   if ( psCharge != NULL )
      do_Free_DbvCharge( psCharge );

   } /* End Free_compulsory_DbvCharge() */ 

void Free_compulsory_DbvBodyType( DbvBodyType *psBodyType )
   {

   if ( psBodyType != NULL )
      do_Free_DbvBodyType( psBodyType );

   } /* End Free_compulsory_DbvBodyType() */ 

void Free_compulsory_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {

   if ( psAttributeOccurrenceChoice != NULL )
      do_Free_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );

   } /* End Free_compulsory_DbvAttributeOccurrenceChoice() */ 






/*******************************************************/
/* Access, Resource and Trigger Control */


void Free_compulsory_DbvSecurityChallengeReq( DbvSecurityChallengeReq *psSecurityChallengeReq )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSecurityChallengeReq()\n" );
#endif

   if ( psSecurityChallengeReq == NULL )
      return;

   /* uSecurityChallengeReq */

   switch( psSecurityChallengeReq->eSecurityChallengeReqType ) 
      {
      case SECT_SimpleForm:
   	 FreeDbvOctetString( psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 FreeDbvASN1External( psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvSecurityChallengeReq */



void Free_compulsory_DbvPromptId( DbvPromptId *psPromptId )
   {
#ifdef DEBUGZ
   printf( "D - Free_compulsory_DbvPromptId()\n" );
#endif

   if ( psPromptId == NULL )
      return;


   /* uPromptId */

   switch( psPromptId->ePromptIdType ) 
      {
      case PIT_EnumeratedPrompt:
   	 Free_DbvEnumeratedPrompt( psPromptId->uPromptId.psEnumeratedPrompt );
	 break;

      case PIT_NoneEnumeratedPrompt:
         FreeDbvInternationalString( psPromptId->uPromptId.psNonEnumeratedPrompt );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvPromptId */



void Free_compulsory_DbvPromptResponse( DbvPromptResponse *psPromptResponse )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPromptResponse()\n" );
#endif

   if ( psPromptResponse == NULL )
      return;


   /* uResponse */

   switch( psPromptResponse->eResponseType ) 
      {
      case RT_String:
   	 FreeDbvInternationalString( psPromptResponse->uResponse.psString );
	 break;

      case RT_Accept:
	 break;

      case RT_Acknowledge:
	 break;

      case RT_Diagnostic:
   	 Free_DbvDiagRec( psPromptResponse->uResponse.psDiagnostic );
	 break;

      case RT_Encrypted:
   	 Free_DbvEncryption( psPromptResponse->uResponse.psEncryption );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End Free_compulsory_DbvPromptResponse() */




void FreeN_DbvAttributeElement( DbvAttributeElement **ppsAttributeElement )
   {

   DbvAttributeElement **pps_base;

   if ( ppsAttributeElement == NULL )
      return;

   pps_base = ppsAttributeElement;

   while( *ppsAttributeElement )
      {
      Free_DbvAttributeElement( *ppsAttributeElement );
      ppsAttributeElement++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeElement() */ 

void FreeN_DbvAttributesPlusTerm( DbvAttributesPlusTerm 
					**ppsAttributesPlusTerm )
   {

   DbvAttributesPlusTerm **pps_base;

   if ( ppsAttributesPlusTerm == NULL )
      return;

   pps_base = ppsAttributesPlusTerm;

   while( *ppsAttributesPlusTerm )
      {
      Free_DbvAttributesPlusTerm( *ppsAttributesPlusTerm );
      ppsAttributesPlusTerm++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesPlusTerm() */ 

void FreeN_DbvCircRecord( DbvCircRecord **ppsCircRecord )
   {

   DbvCircRecord **pps_base;

   if ( ppsCircRecord == NULL )
      return;

   pps_base = ppsCircRecord;

   while( *ppsCircRecord )
      {
      Free_DbvCircRecord( *ppsCircRecord );
      ppsCircRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvCircRecord() */ 

void FreeN_DbvDiagRec( DbvDiagRec **ppsDiagRec )
   {

   DbvDiagRec **pps_base;

   if ( ppsDiagRec == NULL )
      return;

   pps_base = ppsDiagRec;

   while( *ppsDiagRec )
      {
      Free_DbvDiagRec( *ppsDiagRec );
      ppsDiagRec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDiagRec() */ 

void FreeN_DbvElementSetsDbSpecific( DbvElementSetsDbSpecific 
					**ppsElementSetsDbSpecific )
   {

   DbvElementSetsDbSpecific **pps_base;

   if ( ppsElementSetsDbSpecific == NULL )
      return;

   pps_base = ppsElementSetsDbSpecific;

   while( *ppsElementSetsDbSpecific )
      {
      Free_DbvElementSetsDbSpecific( *ppsElementSetsDbSpecific );
      ppsElementSetsDbSpecific++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementSetsDbSpecific() */ 

void FreeN_DbvEntry( DbvEntry **ppsEntry )
   {

   DbvEntry **pps_base;

   if ( ppsEntry == NULL )
      return;

   pps_base = ppsEntry;

   while( *ppsEntry )
      {
      Free_DbvEntry( *ppsEntry );
      ppsEntry++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEntry() */ 

void FreeN_DbvEstimate1( DbvEstimate1 **ppsEstimate1 )
   {

   DbvEstimate1 **pps_base;

   if ( ppsEstimate1 == NULL )
      return;

   pps_base = ppsEstimate1;

   while( *ppsEstimate1 )
      {
      Free_DbvEstimate1( *ppsEstimate1 );
      ppsEstimate1++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEstimate1() */ 

void FreeN_DbvEstimate2( DbvEstimate2 **ppsEstimate2 )
   {

   DbvEstimate2 **pps_base;

   if ( ppsEstimate2 == NULL )
      return;

   pps_base = ppsEstimate2;

   while( *ppsEstimate2 )
      {
      Free_DbvEstimate2( *ppsEstimate2 );
      ppsEstimate2++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEstimate2() */ 

void FreeN_DbvFormatSpec( DbvFormatSpec **ppsFormatSpec )
   {

   DbvFormatSpec **pps_base;

   if ( ppsFormatSpec == NULL )
      return;

   pps_base = ppsFormatSpec;

   while( *ppsFormatSpec )
      {
      Free_DbvFormatSpec( *ppsFormatSpec );
      ppsFormatSpec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvFormatSpec() */ 

void FreeN_DbvHoldingsRecord( DbvHoldingsRecord **ppsHoldingsRecord )
   {

   DbvHoldingsRecord **pps_base;

   if ( ppsHoldingsRecord == NULL )
      return;

   pps_base = ppsHoldingsRecord;

   while( *ppsHoldingsRecord )
      {
      Free_DbvHoldingsRecord( *ppsHoldingsRecord );
      ppsHoldingsRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHoldingsRecord() */ 

void FreeN_DbvInternationalString( DbvInternationalString 
					**ppsInternationalString )
   {

   DbvInternationalString **pps_base;

   if ( ppsInternationalString == NULL )
      return;

   pps_base = ppsInternationalString;

   while( *ppsInternationalString )
      {
      FreeDbvInternationalString( *ppsInternationalString );
      ppsInternationalString++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvInternationalString() */ 

void FreeN_DbvListStatus( DbvListStatus **ppsListStatus )
   {

   DbvListStatus **pps_base;

   if ( ppsListStatus == NULL )
      return;

   pps_base = ppsListStatus;

   while( *ppsListStatus )
      {
      Free_DbvListStatus( *ppsListStatus );
      ppsListStatus++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvListStatus() */ 

void FreeN_DbvNamePlusRecord( DbvNamePlusRecord **ppsNamePlusRecord )
   {

   DbvNamePlusRecord **pps_base;

   if ( ppsNamePlusRecord == NULL )
      return;

   pps_base = ppsNamePlusRecord;

   while( *ppsNamePlusRecord )
      {
      Free_DbvNamePlusRecord( *ppsNamePlusRecord );
      ppsNamePlusRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvNamePlusRecord() */ 

void FreeN_DbvOccurrenceByDb( DbvOccurrenceByDb **ppsOccurrenceByDb )
   {

   DbvOccurrenceByDb **pps_base;

   if ( ppsOccurrenceByDb == NULL )
      return;

   pps_base = ppsOccurrenceByDb;

   while( *ppsOccurrenceByDb )
      {
      Free_DbvOccurrenceByDb( *ppsOccurrenceByDb );
      ppsOccurrenceByDb++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOccurrenceByDb() */ 

void FreeN_DbvPermAllowableFunctions( DbvPermAllowableFunctions 
					**ppePermAllowableFunctions )
   {

   DbvPermAllowableFunctions **pps_base;

   if ( ppePermAllowableFunctions == NULL )
      return;

   pps_base = ppePermAllowableFunctions;

   while( *ppePermAllowableFunctions )
      {
      Free_DbvPermAllowableFunctions( *ppePermAllowableFunctions );
      ppePermAllowableFunctions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPermAllowableFunctions() */ 

void FreeN_DbvPermissions( DbvPermissions **ppsPermissions )
   {

   DbvPermissions **pps_base;

   if ( ppsPermissions == NULL )
      return;

   pps_base = ppsPermissions;

   while( *ppsPermissions )
      {
      Free_DbvPermissions( *ppsPermissions );
      ppsPermissions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPermissions() */ 

void FreeN_DbvRange( DbvRange **ppsRange )
   {

   DbvRange **pps_base;

   if ( ppsRange == NULL )
      return;

   pps_base = ppsRange;

   while( *ppsRange )
      {
      Free_DbvRange( *ppsRange );
      ppsRange++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvRange() */ 

void FreeN_DbvSortKeySpec( DbvSortKeySpec **ppsSortKeySpec )
   {

   DbvSortKeySpec **pps_base;

   if ( ppsSortKeySpec == NULL )
      return;

   pps_base = ppsSortKeySpec;

   while( *ppsSortKeySpec )
      {
      Free_DbvSortKeySpec( *ppsSortKeySpec );
      ppsSortKeySpec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSortKeySpec() */ 

void FreeN_DbvStringOrNumeric( DbvStringOrNumeric **ppsStringOrNumeric )
   {

   DbvStringOrNumeric **pps_base;

   if ( ppsStringOrNumeric == NULL )
      return;

   pps_base = ppsStringOrNumeric;

   while( *ppsStringOrNumeric )
      {
      Free_DbvStringOrNumeric( *ppsStringOrNumeric );
      ppsStringOrNumeric++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvStringOrNumeric() */ 

void FreeN_DbvSuppliedRecord( DbvSuppliedRecord **ppsSuppliedRecord )
   {

   DbvSuppliedRecord **pps_base;

   if ( ppsSuppliedRecord == NULL )
      return;

   pps_base = ppsSuppliedRecord;

   while( *ppsSuppliedRecord )
      {
      Free_DbvSuppliedRecord( *ppsSuppliedRecord );
      ppsSuppliedRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSuppliedRecord() */ 

void FreeN_DbvTaskPackageRecord( DbvTaskPackageRecord **ppsTaskPackageRecord )
   {

   DbvTaskPackageRecord **pps_base;

   if ( ppsTaskPackageRecord == NULL )
      return;

   pps_base = ppsTaskPackageRecord;

   while( *ppsTaskPackageRecord )
      {
      Free_DbvTaskPackageRecord( *ppsTaskPackageRecord );
      ppsTaskPackageRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTaskPackageRecord() */ 

void FreeN_DbvVolume( DbvVolume **ppsVolume )
   {

   DbvVolume **pps_base;

   if ( ppsVolume == NULL )
      return;

   pps_base = ppsVolume;

   while( *ppsVolume )
      {
      Free_DbvVolume( *ppsVolume );
      ppsVolume++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVolume() */ 

void FreeN_Ulong( Ulong **ppUlong )
   {

   Ulong **pp_base;

   if ( ppUlong == NULL )
      return;

   pp_base = ppUlong;

   while( *ppUlong )
      {
      Free_Ulong( *ppUlong );
      ppUlong++;
      } /* End while */

   DbvFree( pp_base );
   } /* End FreeN_Ulong() */ 

void FreeN_DbvOtherInformation( DbvOtherInformation **ppsOtherInformation )
   {
 
   DbvOtherInformation **pps_base;
 
   if ( ppsOtherInformation == NULL )
      return;
 
   pps_base = ppsOtherInformation;
 
   while( *ppsOtherInformation )
      {
      FreeDbvOtherInformation( *ppsOtherInformation );
      ppsOtherInformation++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvOtherInformation() */
 
void FreeN_DbvOccurrenceByAttributes(DbvOccurrenceByAttributes 
	**ppsByAttributes )
   {
 
   DbvOccurrenceByAttributes **pps_base;
 
   if ( ppsByAttributes == NULL )
      return;
 
   pps_base = ppsByAttributes;
 
   while( *ppsByAttributes )
      {
      Free_DbvOccurrenceByAttributes( *ppsByAttributes );
      ppsByAttributes++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvOccurrenceByAttributes() */
 
void FreeN_DbvDbSpecificComp(DbvDbSpecificComp **ppsDbSpecific )
   {
 
   DbvDbSpecificComp **pps_base;
 
   if ( ppsDbSpecific == NULL )
      return;
 
   pps_base = ppsDbSpecific;
 
   while( *ppsDbSpecific )
      {
      Free_DbvDbSpecificComp( *ppsDbSpecific );
      ppsDbSpecific++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvDbSpecificComp() */

void FreeN_DbvObjectIdentifier(DbvObjectIdentifier *ppcObjectIdentifier)
   {
 
   DbvObjectIdentifier *ppc_base;
 
   if ( ppcObjectIdentifier == NULL )
      return;
 
   ppc_base = ppcObjectIdentifier;
 
   while( *ppcObjectIdentifier )
      {
      FreeDbvObjectIdentifier( *ppcObjectIdentifier );
      ppcObjectIdentifier++;
      } /* End while */
 
   DbvFree( ppc_base );
   } /* End FreeN_DbvObjectIdentifier() */

/* 5-2-96 The following function where added by Dan Maskell  */

void FreeN_DbvSystemId( DbvSystemId **ppsSystemId )
   {

   DbvSystemId **pps_base;

   if ( ppsSystemId == NULL )
      return;

   pps_base = ppsSystemId;

   while( *ppsSystemId )
      {
      Free_DbvSystemId( *ppsSystemId );
      ppsSystemId++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSystemId() */ 

void FreeN_DbvElectronicDeliveryService( 
	DbvElectronicDeliveryService **ppsElectronicDeliveryService )
   {

   DbvElectronicDeliveryService **pps_base;

   if ( ppsElectronicDeliveryService == NULL )
      return;

   pps_base = ppsElectronicDeliveryService;

   while( *ppsElectronicDeliveryService )
      {
      Free_DbvElectronicDeliveryService( *ppsElectronicDeliveryService );
      ppsElectronicDeliveryService++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElectronicDeliveryService() */ 

void FreeN_DbvSupplyMediumInfo( DbvSupplyMediumInfo **ppsSupplyMediumInfo )
   {

   DbvSupplyMediumInfo **pps_base;

   if ( ppsSupplyMediumInfo == NULL )
      return;

   pps_base = ppsSupplyMediumInfo;

   while( *ppsSupplyMediumInfo )
      {
      Free_DbvSupplyMediumInfo( *ppsSupplyMediumInfo );
      ppsSupplyMediumInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSupplyMediumInfo() */ 

void FreeN_DbvSendToList( DbvSendToList **ppsSendToList )
   {

   DbvSendToList **pps_base;

   if ( ppsSendToList == NULL )
      return;

   pps_base = ppsSendToList;

   while( *ppsSendToList )
      {
      Free_DbvSendToList( *ppsSendToList );
      ppsSendToList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSendToList() */ 

void FreeN_DbvExtension( DbvExtension **ppsExtension )
   {

   DbvExtension **pps_base;

   if ( ppsExtension == NULL )
      return;

   pps_base = ppsExtension;

   while( *ppsExtension )
      {
      Free_DbvExtension( *ppsExtension );
      ppsExtension++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvExtension() */ 

void FreeN_DbvServiceType( DbvServiceType **ppsServiceType )
   {

   DbvServiceType **pps_base;

   if ( ppsServiceType == NULL )
      return;

   pps_base = ppsServiceType;

   while( *ppsServiceType )
      {
      Free_DbvServiceType( *ppsServiceType );
      ppsServiceType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvServiceType() */ 

void FreeN_DbvASN1External( DbvASN1External **ppsASN1External )
   {

   DbvASN1External **pps_base;

   if ( ppsASN1External == NULL )
      return;

   pps_base = ppsASN1External;

   while( *ppsASN1External )
      {
      FreeDbvASN1External( *ppsASN1External );
      ppsASN1External++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvASN1External() */ 


/* Explain Record Support */


void FreeN_DbvHumanString( DbvHumanString **ppsHumanString )
   {
   DbvHumanString **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvHumanString()\n" );
#endif

   if ( ppsHumanString == NULL )
      return;

   pps_base = ppsHumanString;

   while( *ppsHumanString )
      {
      Free_DbvHumanString( *ppsHumanString );
      ppsHumanString++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHumanString() */ 

void FreeN_DbvIconObject( DbvIconObject **ppsIconObject )
   {
   DbvIconObject **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvIconObject()\n" );
#endif

   if ( ppsIconObject == NULL )
      return;

   pps_base = ppsIconObject;

   while( *ppsIconObject )
      {
      Free_DbvIconObject( *ppsIconObject );
      ppsIconObject++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvIconObject() */ 

void FreeN_DbvDatabaseList( DbvDatabaseList **ppsDatabaseList )
   {
   DbvDatabaseList **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvDatabaseList()\n" );
#endif

   if ( ppsDatabaseList == NULL )
      return;

   pps_base = ppsDatabaseList;

   while( *ppsDatabaseList )
      {
      Free_DbvDatabaseList( *ppsDatabaseList );
      ppsDatabaseList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDatabaseList() */ 

void FreeN_DbvNetworkAddress( DbvNetworkAddress **ppsNetworkAddress )
   {
   DbvNetworkAddress **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvNetworkAddress()\n" );
#endif

   if ( ppsNetworkAddress == NULL )
      return;

   pps_base = ppsNetworkAddress;

   while( *ppsNetworkAddress )
      {
      Free_DbvNetworkAddress( *ppsNetworkAddress );
      ppsNetworkAddress++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvNetworkAddress() */ 

void FreeN_DbvOperators( DbvOperators **ppsOperators )
   {
   DbvOperators **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvOperators()\n" );
#endif

   if ( ppsOperators == NULL )
      return;

   pps_base = ppsOperators;

   while( *ppsOperators )
      {
      Free_DbvOperators( *ppsOperators );
      ppsOperators++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOperators() */ 

void FreeN_DbvSearchKey( DbvSearchKey **ppsSearchKey )
   {
   DbvSearchKey **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvSearchKey()\n" );
#endif

   if ( ppsSearchKey == NULL )
      return;

   pps_base = ppsSearchKey;

   while( *ppsSearchKey )
      {
      Free_DbvSearchKey( *ppsSearchKey );
      ppsSearchKey++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSearchKey() */ 


void FreeN_DbvHumanStringSeq( DbvHumanStringSeq **ppsDescription )
   {
   DbvHumanStringSeq **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvHumanStringSeq()\n" );
#endif

   if ( ppsDescription == NULL )
      return;

   pps_base = ppsDescription;

   while( *ppsDescription )
      {
      Free_DbvHumanStringSeq( *ppsDescription );
      ppsDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHumanStringSeq() */ 



void FreeN_DbvUnitType( DbvUnitType **ppsUnitType )
   {
   DbvUnitType **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvUnitType()\n" );
#endif

   if ( ppsUnitType == NULL )
      return;

   pps_base = ppsUnitType;

   while( *ppsUnitType )
      {
      Free_DbvUnitType( *ppsUnitType );
      ppsUnitType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnitType() */ 

void FreeN_DbvUnitsSupported( DbvUnitsSupported **ppsUnitsSupported )
   {
   DbvUnitsSupported **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvUnitsSupported()\n" );
#endif

   if ( ppsUnitsSupported == NULL )
      return;

   pps_base = ppsUnitsSupported;

   while( *ppsUnitsSupported )
      {
      Free_DbvUnitsSupported( *ppsUnitsSupported );
      ppsUnitsSupported++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnitsSupported() */ 

void FreeN_DbvQueryTypeDetails( DbvQueryTypeDetails **ppsQueryTypeDetails )
   {
   DbvQueryTypeDetails **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvQueryTypeDetails()\n" );
#endif

   if ( ppsQueryTypeDetails == NULL )
      return;

   pps_base = ppsQueryTypeDetails;

   while( *ppsQueryTypeDetails )
      {
      Free_DbvQueryTypeDetails( *ppsQueryTypeDetails );
      ppsQueryTypeDetails++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvQueryTypeDetails() */ 

void FreeN_DbvOtherCharge( DbvOtherCharge **ppsOtherCharge )
   {
   DbvOtherCharge **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvOtherCharge()\n" );
#endif

   if ( ppsOtherCharge == NULL )
      return;

   pps_base = ppsOtherCharge;

   while( *ppsOtherCharge )
      {
      Free_DbvOtherCharge( *ppsOtherCharge );
      ppsOtherCharge++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOtherCharge() */ 

void FreeN_DbvElementSetNames( DbvElementSetNames **ppsElementSetNames )
   {
   DbvElementSetNames **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvElementSetNames()\n" );
#endif

   if ( ppsElementSetNames == NULL )
      return;

   pps_base = ppsElementSetNames;

   while( *ppsElementSetNames )
      {
      Free_DbvElementSetNames( *ppsElementSetNames );
      ppsElementSetNames++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementSetNames() */ 

void FreeN_DbvAccessRestrictions( 
			DbvAccessRestrictions **ppsAccessRestrictions )
   {
   DbvAccessRestrictions **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAccessRestrictions()\n" );
#endif

   if ( ppsAccessRestrictions == NULL )
      return;

   pps_base = ppsAccessRestrictions;

   while( *ppsAccessRestrictions )
      {
      Free_DbvAccessRestrictions( *ppsAccessRestrictions );
      ppsAccessRestrictions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAccessRestrictions() */ 


void FreeN_DbvTagTypeMapping( DbvTagTypeMapping **ppsTagTypeMapping )
   {
   DbvTagTypeMapping **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvTagTypeMapping()\n" );
#endif

   if ( ppsTagTypeMapping == NULL )
      return;

   pps_base = ppsTagTypeMapping;

   while( *ppsTagTypeMapping )
      {
      Free_DbvTagTypeMapping( *ppsTagTypeMapping );
      ppsTagTypeMapping++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTagTypeMapping() */ 

void FreeN_DbvPath( DbvPath **ppsPath )
   {
   DbvPath **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvPath()\n" );
#endif

   if ( ppsPath == NULL )
      return;

   pps_base = ppsPath;

   while( *ppsPath )
      {
      Free_DbvPath( *ppsPath );
      ppsPath++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPath() */ 

void FreeN_DbvPathA( DbvPathA **ppsPathA )
   {
   DbvPathA **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvPathA()\n" );
#endif

   if ( ppsPathA == NULL )
      return;

   pps_base = ppsPathA;

   while( *ppsPathA )
      {
      Free_DbvPathA( *ppsPathA );
      ppsPathA++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPathA() */ 

void FreeN_DbvElementInfo( DbvElementInfo **ppsElementInfo )
   {
   DbvElementInfo **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvElementInfo()\n" );
#endif

   if ( ppsElementInfo == NULL )
      return;

   pps_base = ppsElementInfo;

   while( *ppsElementInfo )
      {
      Free_DbvElementInfo( *ppsElementInfo );
      ppsElementInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementInfo() */ 

void FreeN_DbvElements( DbvElements **ppsElements )
   {
   DbvElements **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvElements()\n" );
#endif

   if ( ppsElements == NULL )
      return;

   pps_base = ppsElements;

   while( *ppsElements )
      {
      Free_DbvElements( *ppsElements );
      ppsElements++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElements() */ 

void FreeN_DbvAttributeDescription( 
		DbvAttributeDescription **ppsAttributeDescription )
   {
   DbvAttributeDescription **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeDescription()\n" );
#endif

   if ( ppsAttributeDescription == NULL )
      return;

   pps_base = ppsAttributeDescription;

   while( *ppsAttributeDescription )
      {
      Free_DbvAttributeDescription( *ppsAttributeDescription );
      ppsAttributeDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeDescription() */ 

void FreeN_DbvAttributeType( DbvAttributeType **ppsAttributeType )
   {
   DbvAttributeType **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeType()\n" );
#endif

   if ( ppsAttributeType == NULL )
      return;

   pps_base = ppsAttributeType;

   while( *ppsAttributeType )
      {
      Free_DbvAttributeType( *ppsAttributeType );
      ppsAttributeType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeType() */ 

void FreeN_DbvTermList( DbvTermList **ppsTermList )
   {
   DbvTermList **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvTermList()\n" );
#endif

   if ( ppsTermList == NULL )
      return;

   pps_base = ppsTermList;

   while( *ppsTermList )
      {
      Free_DbvTermList( *ppsTermList );
      ppsTermList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTermList() */ 

void FreeN_DbvValueDescription( DbvValueDescription **ppsValueDescription )
   {
   DbvValueDescription **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvValueDescription()\n" );
#endif

   if ( ppsValueDescription == NULL )
      return;

   pps_base = ppsValueDescription;

   while( *ppsValueDescription )
      {
      Free_DbvValueDescription( *ppsValueDescription );
      ppsValueDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvValueDescription() */ 

void FreeN_DbvAttributeValues( DbvAttributeValues **ppsAttributeValues )
   {
   DbvAttributeValues **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeValues()\n" );
#endif

   if ( ppsAttributeValues == NULL )
      return;

   pps_base = ppsAttributeValues;

   while( *ppsAttributeValues )
      {
      Free_DbvAttributeValues( *ppsAttributeValues );
      ppsAttributeValues++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeValues() */ 

void FreeN_DbvAttributesByType( DbvAttributesByType **ppsAttributesByType )
   {
   DbvAttributesByType **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributesByType()\n" );
#endif

   if ( ppsAttributesByType == NULL )
      return;

   pps_base = ppsAttributesByType;

   while( *ppsAttributesByType )
      {
      Free_DbvAttributesByType( *ppsAttributesByType );
      ppsAttributesByType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesByType() */ 

void FreeN_DbvAttributesBySet( DbvAttributesBySet **ppsAttributesBySet )
   {
   DbvAttributesBySet **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributesBySet()\n" );
#endif

   if ( ppsAttributesBySet == NULL )
      return;

   pps_base = ppsAttributesBySet;

   while( *ppsAttributesBySet )
      {
      Free_DbvAttributesBySet( *ppsAttributesBySet );
      ppsAttributesBySet++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesBySet() */ 

void FreeN_DbvAttributeDetails( DbvAttributeDetails **ppsAttributeDetails )
   {
   DbvAttributeDetails **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeDetails()\n" );
#endif

   if ( ppsAttributeDetails == NULL )
      return;

   pps_base = ppsAttributeDetails;

   while( *ppsAttributeDetails )
      {
      Free_DbvAttributeDetails( *ppsAttributeDetails );
      ppsAttributeDetails++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeDetails() */ 

void FreeN_DbvTerm( DbvTerm **ppsTerm )
   {
   DbvTerm **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvTerm()\n" );
#endif

   if ( ppsTerm == NULL )
      return;

   pps_base = ppsTerm;

   while( *ppsTerm )
      {
      Free_DbvTerm( *ppsTerm );
      ppsTerm++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTerm() */ 

void FreeN_DbvDetailsPerElement( DbvDetailsPerElement **ppsDetailsPerElement )
   {
   DbvDetailsPerElement **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvDetailsPerElement()\n" );
#endif

   if ( ppsDetailsPerElement == NULL )
      return;

   pps_base = ppsDetailsPerElement;

   while( *ppsDetailsPerElement )
      {
      Free_DbvDetailsPerElement( *ppsDetailsPerElement );
      ppsDetailsPerElement++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDetailsPerElement() */ 

void FreeN_DbvSortKeys( DbvSortKeys **ppsSortKeys )
   {
   DbvSortKeys **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvSortKeys()\n" );
#endif

   if ( ppsSortKeys == NULL )
      return;

   pps_base = ppsSortKeys;

   while( *ppsSortKeys )
      {
      Free_DbvSortKeys( *ppsSortKeys );
      ppsSortKeys++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSortKeys() */ 

void FreeN_DbvVariantTypes( DbvVariantTypes **ppsVariantTypes )
   {
   DbvVariantTypes **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvVariantTypes()\n" );
#endif

   if ( ppsVariantTypes == NULL )
      return;

   pps_base = ppsVariantTypes;

   while( *ppsVariantTypes )
      {
      Free_DbvVariantTypes( *ppsVariantTypes );
      ppsVariantTypes++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVariantTypes() */ 

void FreeN_DbvVariantClass( DbvVariantClass **ppsVariantClass )
   {
   DbvVariantClass **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvVariantClass()\n" );
#endif

   if ( ppsVariantClass == NULL )
      return;

   pps_base = ppsVariantClass;

   while( *ppsVariantClass )
      {
      Free_DbvVariantClass( *ppsVariantClass );
      ppsVariantClass++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVariantClass() */ 

void FreeN_DbvUnits( DbvUnits **ppsUnits )
   {
   DbvUnits **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvUnits()\n" );
#endif

   if ( ppsUnits == NULL )
      return;

   pps_base = ppsUnits;

   while( *ppsUnits )
      {
      Free_DbvUnits( *ppsUnits );
      ppsUnits++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnits() */ 

void FreeN_DbvUnit( DbvUnit **ppsUnit )
   {
   DbvUnit **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvUnit()\n" );
#endif

   if ( ppsUnit == NULL )
      return;

   pps_base = ppsUnit;

   while( *ppsUnit )
      {
      Free_DbvUnit( *ppsUnit );
      ppsUnit++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnit() */ 

void FreeN_DbvCategoryInfo( DbvCategoryInfo **ppsCategoryInfo )
   {
   DbvCategoryInfo **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvCategoryInfo()\n" );
#endif

   if ( ppsCategoryInfo == NULL )
      return;

   pps_base = ppsCategoryInfo;

   while( *ppsCategoryInfo )
      {
      Free_DbvCategoryInfo( *ppsCategoryInfo );
      ppsCategoryInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvCategoryInfo() */ 

void FreeN_DbvAttributeOccurrence( 
			DbvAttributeOccurrence **ppsAttributeOccurrence )
   {
   DbvAttributeOccurrence **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeOccurrence()\n" );
#endif

   if ( ppsAttributeOccurrence == NULL )
      return;

   pps_base = ppsAttributeOccurrence;

   while( *ppsAttributeOccurrence )
      {
      Free_DbvAttributeOccurrence( *ppsAttributeOccurrence );
      ppsAttributeOccurrence++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeOccurrence() */ 

void FreeN_DbvAttributeCombination( 
		DbvAttributeCombination **ppsAttributeCombination )
   {
   DbvAttributeCombination **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeCombination()\n" );
#endif

   if ( ppsAttributeCombination == NULL )
      return;

   pps_base = ppsAttributeCombination;

   while( *ppsAttributeCombination )
      {
      Free_DbvAttributeCombination( *ppsAttributeCombination );
      ppsAttributeCombination++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeCombination() */ 

void FreeN_DbvAttributeCombinations( 
			DbvAttributeCombinations **ppsAttributeCombinations )
   {
   DbvAttributeCombinations **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvAttributeCombinations()\n" );
#endif

   if ( ppsAttributeCombinations == NULL )
      return;

   pps_base = ppsAttributeCombinations;

   while( *ppsAttributeCombinations )
      {
      Free_DbvAttributeCombinations( *ppsAttributeCombinations );
      ppsAttributeCombinations++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeCombinations() */ 

void FreeN_DbvSpecification( DbvSpecification **ppsSpecification )
   {
   DbvSpecification **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvSpecification()\n" );
#endif

   if ( ppsSpecification == NULL )
      return;

   pps_base = ppsSpecification;

   while( *ppsSpecification )
      {
      Free_DbvSpecification( *ppsSpecification );
      ppsSpecification++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSpecification() */ 






void FreeN_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq **ppsDbvDataBaseInfoSeq )
   {
   DbvDataBaseInfoSeq **pps_base;

#ifdef DEBUGZ
   printf( "D - FreeN_DbvDataBaseInfoSeq()\n" );
#endif

   if ( ppsDbvDataBaseInfoSeq == NULL )
      return;

   pps_base = ppsDbvDataBaseInfoSeq;

   while( *ppsDbvDataBaseInfoSeq )
      {
      Free_DbvDataBaseInfoSeq( *ppsDbvDataBaseInfoSeq );
      ppsDbvDataBaseInfoSeq++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDataBaseInfoSeq() */ 




/*******************************************************/
/* Access, Resource and Trigger Control */


void FreeN_DbvChallenge( DbvChallenge **ppsChallenge )
   {
   DbvChallenge **pps_base;

   if ( ppsChallenge == NULL )
      return;

   pps_base = ppsChallenge;

   while( *ppsChallenge )
      {
      Free_DbvChallenge( *ppsChallenge );
      ppsChallenge++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvChallenge() */



void FreeN_DbvResponse( DbvResponse **ppsResponse )
   {
   DbvResponse **pps_base;

   if ( ppsResponse == NULL )
      return;

   pps_base = ppsResponse;

   while( *ppsResponse )
      {
      Free_DbvResponse( *ppsResponse );
      ppsResponse++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvResponse() */



/* ---------------------------------------------------------------------*/
/* ---- User Info Format "Character Set And Language Negotiation" ----- */

void FreeN_DbvInitialSet( DbvInitialSet **ppsInitialSet )
   {
   DbvInitialSet **pps_base;

   if ( ppsInitialSet == NULL )
      return;

   pps_base = ppsInitialSet;

   while( *ppsInitialSet )
      {
      Free_DbvInitialSet( *ppsInitialSet );
      ppsInitialSet++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvInitialSet() */



void FreeN_DbvLanguageCode( DbvLanguageCode **ppsLanguageCode )
   {
   DbvLanguageCode **pps_base;

   if ( ppsLanguageCode == NULL )
      return;

   pps_base = ppsLanguageCode;

   while( *ppsLanguageCode )
      {
      Free_DbvLanguageCode( *ppsLanguageCode );
      ppsLanguageCode++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvLanguageCode() */



void FreeN_DbvProposedCharSet( DbvProposedCharSet **ppsProposedCharSet )
   {
   DbvProposedCharSet **pps_base;

   if ( ppsProposedCharSet == NULL )
      return;

   pps_base = ppsProposedCharSet;

   while( *ppsProposedCharSet )
      {
      Free_DbvProposedCharSet( *ppsProposedCharSet );
      ppsProposedCharSet++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvProposedCharSet() */




void Free_DbvAttributeElement( DbvAttributeElement *psAttributeElement )
   {

   if ( psAttributeElement == NULL )
      return;

   /* pcAttributeSetId */

   if ( psAttributeElement->pcAttributeSetId != NULL )
      FreeDbvObjectIdentifier( psAttributeElement->pcAttributeSetId );

   /* sAttributeValue */

   Free_compulsory_DbvAttributeValue( &psAttributeElement->sAttributeValue );

   DbvFree( psAttributeElement );
   } /* End Free_DbvAttributeElement() */ 



void Free_DbvAttributesPlusTerm( DbvAttributesPlusTerm *psAttributesPlusTerm )
   {

   if ( psAttributesPlusTerm == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psAttributesPlusTerm->ppsAttributeElement );

   /* sTerm */

   Free_compulsory_DbvTerm( &psAttributesPlusTerm->sTerm );

   DbvFree( psAttributesPlusTerm );
   } /* End Free_DbvAttributesPlusTerm() */ 



void Free_DbvCircRecord( DbvCircRecord *psCircRecord )
   {

   if ( psCircRecord == NULL )
      return;

   /* psAvailabilityDate */

   if ( psCircRecord->psAvailabilityDate != NULL )
      FreeDbvInternationalString( psCircRecord->psAvailabilityDate );

   /* psAvailableThru */

   if ( psCircRecord->psAvailableThru != NULL )
      FreeDbvInternationalString( psCircRecord->psAvailableThru );

   /* psRestrictions */

   if ( psCircRecord->psRestrictions != NULL )
      FreeDbvInternationalString( psCircRecord->psRestrictions );

   /* psItemId */

   if ( psCircRecord->psItemId != NULL )
      FreeDbvInternationalString( psCircRecord->psItemId );

   /* psEnumAndChron */

   if ( psCircRecord->psEnumAndChron != NULL )
      FreeDbvInternationalString( psCircRecord->psEnumAndChron );

   /* psMidspine */

   if ( psCircRecord->psMidspine != NULL )
      FreeDbvInternationalString( psCircRecord->psMidspine );

   /* psTemporaryLocation */

   if ( psCircRecord->psTemporaryLocation != NULL )
      FreeDbvInternationalString( psCircRecord->psTemporaryLocation );

   DbvFree( psCircRecord );
   } /* End Free_DbvCircRecord() */ 


void Free_DbvCloseReq( DbvCloseReq *psCloseReq )
   {

   /* psReferenceId */

   if( psCloseReq->psReferenceId != NULL)
      FreeDbvOctetString( psCloseReq->psReferenceId );

   /* psDiagnosticInformation */

   if( psCloseReq->psDiagnosticInformation != NULL)
      FreeDbvInternationalString( psCloseReq->psDiagnosticInformation );

   /* pcResourceReportFormat */

   if( psCloseReq->pcResourceReportFormat != NULL)
      FreeDbvObjectIdentifier( psCloseReq->pcResourceReportFormat );

   /* psResourceReport */

   FreeDbvASN1External( psCloseReq->psResourceReport );

   /* ppsOtherInformation */

   if( psCloseReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psCloseReq->ppsOtherInformation );

   DbvFree( psCloseReq );
   } /* End Free_DbvCloseReq() */ 


void Free_DbvComplexValue( DbvComplexValue *psComplexValue )
   {

   if ( psComplexValue == NULL )
      return;

   /* ppsList */

   FreeN_DbvStringOrNumeric( psComplexValue->ppsList );

   /* ppulSemanticAction */

   FreeN_Ulong( psComplexValue->ppulSemanticAction );

   DbvFree( psComplexValue );
   } /* End Free_DbvComplexValue() */ 

void Free_DbvCorrelationInfo( DbvCorrelationInfo *psCorrelationInfo )
   {

   if ( psCorrelationInfo == NULL )
      return;

   /* psNote */

   if ( psCorrelationInfo->psNote != NULL )
      FreeDbvInternationalString( psCorrelationInfo->psNote );

   /* pulId */

   if ( psCorrelationInfo->pulId != NULL )
      Free_Ulong( psCorrelationInfo->pulId );

   DbvFree( psCorrelationInfo );
   } /* End Free_DbvCorrelationInfo() */ 

void Free_DbvDefaultDiagFormat( DbvDefaultDiagFormat *psDefaultDiagFormat )
   {

   if ( psDefaultDiagFormat == NULL )
      return;

   if ( psDefaultDiagFormat->pcDiagnosticSetId != NULL )
      FreeDbvObjectIdentifier( psDefaultDiagFormat->pcDiagnosticSetId );

   Free_compulsory_DbvAdditionalInfo( &psDefaultDiagFormat->sAdditionalInfo );

   DbvFree( psDefaultDiagFormat );
   } /* End Free_DbvDefaultDiagFormat() */ 

void Free_DbvDeleteResultSetReq( DbvDeleteResultSetReq *psDeleteResultSetReq )
   {

   /* psReferenceId */
   if( psDeleteResultSetReq->psReferenceId != NULL)
      FreeDbvOctetString(psDeleteResultSetReq->psReferenceId);
			 
   /* ppsResultSetId */

   FreeN_DbvInternationalString( psDeleteResultSetReq->ppsResultSetId );

   /* ppsOtherInformation */

   if( psDeleteResultSetReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetReq->ppsOtherInformation );

   DbvFree( psDeleteResultSetReq );
   } /* End Free_DbvDeleteResultSetReq() */ 

void Free_DbvDeleteResultSetRsp( DbvDeleteResultSetRsp *psDeleteResultSetRsp )
   {

   /* psReferenceId */

   if( psDeleteResultSetRsp->psReferenceId != NULL)
      FreeDbvOctetString(psDeleteResultSetRsp->psReferenceId);
			 
   /* ppsDeleteListStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsDeleteListStatuses );

   /* pulNumberNotDeleted */

   if ( psDeleteResultSetRsp->pulNumberNotDeleted != NULL )
      Free_Ulong( psDeleteResultSetRsp->pulNumberNotDeleted );

   /* ppsBulkStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsBulkStatuses );

   /* psDeleteMessage */

   if ( psDeleteResultSetRsp->psDeleteMessage != NULL )
      FreeDbvInternationalString( psDeleteResultSetRsp->psDeleteMessage );

   /* ppsOtherInformation */
   if( psDeleteResultSetRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetRsp->ppsOtherInformation );

   DbvFree( psDeleteResultSetRsp );
   } /* End Free_DbvDeleteResultSetRsp() */ 

void Free_DbvDiagRec( DbvDiagRec *psDiagRec )
   {

   if ( psDiagRec == NULL )
      return;

   switch( psDiagRec->eDiagnosticsType )
      {
      case DT_DefaultFormat:
	 Free_DbvDefaultDiagFormat( psDiagRec->uDiagnostics.psDefault );
	 break;

      case DT_ExternallyDefined:
	 FreeDbvASN1External( psDiagRec->uDiagnostics.psExternallyDefined );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDiagRec );
   } /* End Free_DbvDiagRec() */ 

void Free_DbvElementSetNames( DbvElementSetNames *psElementSetNames )
   {

   if ( psElementSetNames == NULL )
      return;

   switch( psElementSetNames->eElementSetType )
      {
      case EST_Generic:
	 FreeDbvInternationalString( psElementSetNames->uElementSet.psGeneric );
	 break;

      case EST_DatabaseSpecific:
	 FreeN_DbvElementSetsDbSpecific( psElementSetNames->
					uElementSet.ppsElementSetsDbSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementSetNames );
   } /* End Free_DbvElementSetNames() */ 

void Free_DbvElementSetsDbSpecific( DbvElementSetsDbSpecific 
						*psElementSetsDbSpecific )
   {

   /* psDatabaseName */

   if ( psElementSetsDbSpecific->psDatabaseName != NULL )
      FreeDbvInternationalString( psElementSetsDbSpecific->psDatabaseName );

   /* psElementSetName */

   if ( psElementSetsDbSpecific->psElementSetName != NULL )
      FreeDbvInternationalString( psElementSetsDbSpecific->psElementSetName );

   DbvFree( psElementSetsDbSpecific );
   } /* End Free_DbvElementSetsDbSpecific() */ 

void Free_DbvEntry( DbvEntry *psEntry )
   {

   if ( psEntry == NULL )
      return;

   switch( psEntry->eEntryType )
      {
      case ET_TermInfo:
	 Free_DbvTermInfo( psEntry->uEntry.psTermInfo );
	 break;

      case ET_SurrogateDiagnostic:
	 Free_DbvDiagRec( psEntry->uEntry.psSurrogateDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psEntry );
   } /* End Free_DbvEntry() */ 

void Free_DbvEsTaskPackage( DbvEsTaskPackage *psEsTaskPackage )
   {

   if ( psEsTaskPackage == NULL )
      return;

   /* pcPackageType */

   FreeDbvObjectIdentifier( psEsTaskPackage->pcPackageType );

   /* psPackageName */

   if ( psEsTaskPackage->psPackageName != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psPackageName );

   /* psUserId */

   if ( psEsTaskPackage->psUserId != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psUserId );

   /* psRetentionTime */

   Free_DbvIntUnit( psEsTaskPackage->psRetentionTime );

   /* ppsPermissions */

   FreeN_DbvPermissions( psEsTaskPackage->ppsPermissions );

   /* psDescription */

   if ( psEsTaskPackage->psDescription != NULL )
      FreeDbvInternationalString( psEsTaskPackage->psDescription );

   /* psTargetReference */

   if( psEsTaskPackage->psTargetReference != NULL)
      FreeDbvOctetString( psEsTaskPackage->psTargetReference );

   /* pcCreationDateTime */

   Free_DbvGeneralizedTime( psEsTaskPackage->pcCreationDateTime );

   /* ppsPackageDiagnostics */

   FreeN_DbvDiagRec( psEsTaskPackage->ppsPackageDiagnostics );

   /* sTaskSpecificParameters */

   Free_compulsory_DbvASN1External( &psEsTaskPackage->sTaskSpecificParameters );

   DbvFree( psEsTaskPackage );
   } /* End Free_DbvEsTaskPackage() */ 


void Free_DbvEstimate1( DbvEstimate1 *psEstimate1 )
   {

   if ( psEstimate1 == NULL )
	 return;

   /* pulCurrencyCode */

   if ( psEstimate1->pulCurrencyCode != NULL )
      Free_Ulong( psEstimate1->pulCurrencyCode );
	   
   DbvFree( psEstimate1 );
   } /* End Free_DbvEstimate1() */ 


void Free_DbvEstimate2( DbvEstimate2 *psEstimate2 )
   {

   if ( psEstimate2 == NULL )
	 return;

   Free_compulsory_DbvStringOrNumeric(&psEstimate2->sType);
   Free_compulsory_DbvIntUnit(&psEstimate2->sValue);

   DbvFree( psEstimate2 );
   } /* End Free_DbvEstimate2() */ 

void Free_DbvFormatSpec( DbvFormatSpec *psFormatSpec )
   {

   if ( psFormatSpec == NULL )
	 return;

   Free_compulsory_DbvInternationalString( &psFormatSpec->sType );

   /* pulSize */

   if ( psFormatSpec->pulSize != NULL )
      Free_Ulong( psFormatSpec->pulSize );
	   
   /* pulBestPosn */

   if ( psFormatSpec->pulBestPosn != NULL )
      Free_Ulong( psFormatSpec->pulBestPosn );
	   
   DbvFree( psFormatSpec );
   } /* End Free_DbvFormatSpec() */ 

void Free_DbvFragmentSyntax( DbvFragmentSyntax *psFragmentSyntax )
   {

   if ( psFragmentSyntax == NULL )
      return;

   switch( psFragmentSyntax->eFragmentSyntaxType )
      {
      case FST_ExternallyTagged:
	 FreeDbvASN1External( psFragmentSyntax->
			      uFragmentSyntax.psExternallyTagged );
	 break;

      case FST_NotExternallyTagged:
	 if ( psFragmentSyntax-> uFragmentSyntax.psNotExternallyTagged != NULL )
	    FreeDbvOctetString( psFragmentSyntax->
			     uFragmentSyntax.psNotExternallyTagged );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psFragmentSyntax );
   } /* End Free_DbvFragmentSyntax() */ 

void Free_DbvGeneralizedTime( DbvGeneralizedTime psGeneralizedTime )
   {

   if ( psGeneralizedTime == NULL )
      return;

   DbvFree( psGeneralizedTime );
   } /* End Free_DbvGeneralizedTime() */ 

void Free_DbvHoldingsRecord( DbvHoldingsRecord *psHoldingsRecord )
   {

   if ( psHoldingsRecord == NULL )
      return;

   switch( psHoldingsRecord->eHoldingsRecType )
      {
      case HRT_MarcHoldingsRecord:
         Free_compulsory_DbvASN1External( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sMarcHoldingsRecord );
	 break;

      case HRT_HoldingsAndCirc:
         Free_compulsory_DbvHoldingsAndCircData( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sHoldingsAndCirc );
	 break;
      default:
	 break;
      } /* End switch */

   DbvFree( psHoldingsRecord );
   } /* End Free_DbvHoldingsRecord() */ 

void Free_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      {
      do_Free_DbvIntUnit( psIntUnit );
      DbvFree( psIntUnit );
      } /* End if */

   } /* End Free_DbvIntUnit() */

void Free_compulsory_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      do_Free_DbvIntUnit( psIntUnit );

   } /* End Free_compulsory_DbvIntUnit() */


void do_Free_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   Free_compulsory_DbvUnit( &psIntUnit->sUnitUsed );
   } /* End Free_DbvIntUnit() */ 

void Free_DbvListEntries( DbvListEntries *psListEntries )
   {

   if ( psListEntries == NULL )
      return;

   FreeN_DbvEntry( psListEntries->ppsEntry );
   FreeN_DbvDiagRec( psListEntries->ppsNonSurrogateDiagnostics );

   DbvFree( psListEntries );
   } /* End Free_DbvListEntries() */ 

void Free_DbvListStatus( DbvListStatus *psListStatus )
   {
   Free_compulsory_DbvInternationalString( &psListStatus->sResultSetId );
   DbvFree( psListStatus );
   } /* End Free_DbvListStatus() */ 

void Free_DbvNamePlusRecord( DbvNamePlusRecord *psNamePlusRecord )
   {

   if ( psNamePlusRecord == NULL )
      return;

   if ( psNamePlusRecord->psDatabaseName != NULL )
      FreeDbvInternationalString( psNamePlusRecord->psDatabaseName );

   switch( psNamePlusRecord->eDbRecordType )
      {
      case DRT_Retrieval:
	 FreeDbvASN1External( psNamePlusRecord->uDbRecord.psRetrievalRecord );
	 break;

      case DRT_SurrogateDiagnostics:
	 Free_DbvDiagRec( psNamePlusRecord->uDbRecord.psSurrogateDiagnostic );
	 break;

      case DRT_StartingFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psStartingFragment );
	 break;

      case DRT_IntermediateFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psIntermediateFragment );
	 break;

      case DRT_FinalFragment:
	 Free_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psFinalFragment);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNamePlusRecord );
   } /* End Free_DbvNamePlusRecord() */ 

void Free_DbvOccurrence( DbvOccurrence *psOccurrence )
   {

   if ( psOccurrence == NULL )
      return;

   switch( psOccurrence->eOccurrenceType )
      {
      case OT_Global:
	 /* No space allocated */
	 break;

      case OT_ByDatabase:
	 FreeN_DbvOccurrenceByDb( psOccurrence->uOccurence.ppsOccurrenceByDb );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psOccurrence );
   } /* End Free_DbvOccurrence() */ 


void Free_DbvOccurrenceByAttributes( DbvOccurrenceByAttributes 
			*psOccurrenceByAttributes )
   {

   if ( psOccurrenceByAttributes == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psOccurrenceByAttributes->ppsAttributeElement );

   /* psOccurrence */

   Free_DbvOccurrence( psOccurrenceByAttributes->psOccurrence );

   /* ppsOtherInformation */
   if( psOccurrenceByAttributes->ppsOtherInformation != NULL)
      FreeN_DbvOtherInformation(psOccurrenceByAttributes->ppsOtherInformation );


   DbvFree( psOccurrenceByAttributes );
   } /* End Free_DbvOccurrenceByAttributes() */ 


void Free_DbvOccurrenceByDb( DbvOccurrenceByDb *psOccurrenceByDb )
   {

   if ( psOccurrenceByDb == NULL )
      return;

   /* sDatabaseName */

   Free_compulsory_DbvInternationalString( &psOccurrenceByDb->sDatabaseName );

   /* pulNumber */

   if ( psOccurrenceByDb->pulNumber ) 
      Free_Ulong( psOccurrenceByDb->pulNumber );

   /* ppsOtherInformation */
   if ( psOccurrenceByDb->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psOccurrenceByDb->ppsOtherInformation );

   DbvFree( psOccurrenceByDb );
   } /* End Free_DbvOccurrenceByDb() */ 


void Free_DbvOpacRecord( DbvOpacRecord *psOpacRecord )
   {

   if ( psOpacRecord == NULL )
      return;

   FreeDbvASN1External( psOpacRecord->psBibliographicRecord );
   FreeN_DbvHoldingsRecord( psOpacRecord->ppsHoldingsData );

   DbvFree( psOpacRecord );
   } /* End Free_DbvOpacRecord() */ 

void Free_DbvPermAllowableFunctions( DbvPermAllowableFunctions 
					*psPermAllowableFunctions )
   {

   if ( psPermAllowableFunctions == NULL )
      return;

   DbvFree( psPermAllowableFunctions );
   } /* End Free_DbvPermAllowableFunctions() */ 

void Free_DbvPermissions( DbvPermissions *psPermissions )
   {

   if ( psPermissions == NULL )
      return;

   Free_compulsory_DbvInternationalString( &psPermissions->sUserId );
   FreeN_DbvPermAllowableFunctions( psPermissions->ppeAllowableFunctions );

   DbvFree( psPermissions );
   } /* End Free_DbvPermissions() */ 

void Free_DbvPresentReq( DbvPresentReq *psPresentReq )
   {
   /* psReferenceId */
   if( psPresentReq->psReferenceId != NULL)
      FreeDbvOctetString(psPresentReq->psReferenceId);

   /* sResultSetId */

   Free_compulsory_DbvInternationalString( &psPresentReq->sResultSetId );

   /* ppsAdditionalRanges */

   FreeN_DbvRange( psPresentReq->ppsAdditionalRanges );

   /* DbvRecordComposition */

   if( psPresentReq->psRecordComposition != NULL)
      Free_DbvRecordComposition( psPresentReq->psRecordComposition );

   /* pcPreferredRecordSyntax */

   if( psPresentReq->pcPreferredRecordSyntax != NULL)
      FreeDbvObjectIdentifier( psPresentReq->pcPreferredRecordSyntax );

   /* pulMaxSegmentCount */

   if( psPresentReq->pulMaxSegmentCount != NULL)
      Free_Ulong( psPresentReq->pulMaxSegmentCount );

   /* pulMaxRecordSize */

   if( psPresentReq->pulMaxRecordSize!= NULL)
      Free_Ulong( psPresentReq->pulMaxRecordSize );

   /* pulMaxSegmentSize */

   if( psPresentReq->pulMaxSegmentSize != NULL)
      Free_Ulong( psPresentReq->pulMaxSegmentSize );

   /* ppsOtherInformation */

   if( psPresentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentReq->ppsOtherInformation );

   DbvFree( psPresentReq );
   } /* End Free_DbvPresentReq() */ 

void Free_DbvPresentRsp( DbvPresentRsp *psPresentRsp )
   {
   /* psReferenceId */
   if( psPresentRsp->psReferenceId != NULL)
      FreeDbvOctetString( psPresentRsp->psReferenceId );

   /* psRecords */

   Free_DbvRecords( psPresentRsp->psRecords );

   /* ppsOtherInformation */

   if( psPresentRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentRsp->ppsOtherInformation );

   DbvFree( psPresentRsp );
   } /* End Free_DbvPresentRsp() */ 

void Free_DbvPresentStatus( DbvPresentStatus *pePresentStatus )
   {

   if ( pePresentStatus == NULL )
      return;

   DbvFree( pePresentStatus );
   } /* End Free_DbvPresentStatus() */ 

void Free_DbvQuery( DbvQuery *psQuery )
   {

   if ( psQuery == NULL )
      return;

   switch( psQuery->eQueryType )
      {
      case QT_Private:
	 if ( psQuery->uQuery.psPrivateQuery )
	    FreeDbvOctetString( psQuery->uQuery.psPrivateQuery);
	 break;

      case QT_Iso8777:
	 if ( psQuery->uQuery.psIso8777Query )
	    FreeDbvOctetString( psQuery->uQuery.psIso8777Query);
	 break;

      case QT_Z3958:
	 if ( psQuery->uQuery.psZ3958Query )
	    FreeDbvOctetString( psQuery->uQuery.psZ3958Query);
	 break;

      case QT_NaturalLanguage:
	 if ( psQuery->uQuery.psNaturalLanguageQuery )
	    FreeDbvOctetString( psQuery->uQuery.psNaturalLanguageQuery);
	 break;

      case QT_Erpn:
         if (psQuery->uQuery.psErpnQuery != NULL)
                Free_DbvRPNQuery(psQuery->uQuery.psErpnQuery);
         break;

      case QT_Rpn:
         if (psQuery->uQuery.psRpnQuery != NULL)
                Free_DbvRPNQuery(psQuery->uQuery.psRpnQuery);
 
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQuery );
   } /* End Free_DbvQuery() */ 

void Free_DbvRange( DbvRange *psRange )
   {
   DbvFree( psRange );
   } /* End Free_DbvRange() */ 

void Free_DbvRecords( DbvRecords *psRecords )
   {

   if ( psRecords == NULL )
      return;

   switch( psRecords->eRecordType )
      {
      case RT_Response:
	 FreeN_DbvNamePlusRecord( psRecords->uRecord.ppsResponseRecord );
	 break;

      case RT_NonSurrDiagnostics:
	 Free_DbvDefaultDiagFormat( psRecords->uRecord.psNonSurrogateDiag );
	 break;

      case RT_MultipleNonSurrDiagnostics:
	 FreeN_DbvDiagRec( psRecords->uRecord.ppsMultipleNonSurrogateDiag );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psRecords );
   } /* End Free_DbvRecord() */ 

void Free_DbvResourceReport1( DbvResourceReport1 *psResourceReport1 )
   {

   if ( psResourceReport1 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate1( psResourceReport1->ppsEstimate );

   /* sMessage */

   Free_compulsory_DbvInternationalString( &psResourceReport1->sMessage );

   DbvFree( psResourceReport1 );
   } /* End Free_DbvResourceReport1() */ 

void Free_DbvResourceReport2( DbvResourceReport2 *psResourceReport2 )
   {

   if ( psResourceReport2 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate2( psResourceReport2->ppsEstimate );

   /* psMessage */

   if ( psResourceReport2->psMessage != NULL )
      FreeDbvInternationalString( psResourceReport2->psMessage );

   DbvFree( psResourceReport2 );
   } /* End Free_DbvResourceReport2() */ 

void Free_DbvResourceReportReq( DbvResourceReportReq *psResourceReportReq )
   {

   /* psReferenceId */

   if( psResourceReportReq->psReferenceId != NULL)
      FreeDbvOctetString(psResourceReportReq->psReferenceId);

   /* psOperationId */

   if( psResourceReportReq->psOperationId != NULL)
      FreeDbvOctetString(psResourceReportReq->psOperationId );

   /* pcPrefResourceReportFormat */

   if( psResourceReportReq->pcPrefResourceReportFormat != NULL)
      FreeDbvObjectIdentifier(psResourceReportReq->pcPrefResourceReportFormat );

   /* ppsOtherInformation */

   if( psResourceReportReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportReq->ppsOtherInformation );

   DbvFree( psResourceReportReq );
   } /* End Free_DbvResourceReportReq() */ 

void Free_DbvResourceReportRsp( DbvResourceReportRsp *psResourceReportRsp )
   {

   /* psReferenceId */

   if( psResourceReportRsp->psReferenceId != NULL)
      FreeDbvOctetString(psResourceReportRsp->psReferenceId );

   /* psResourceReport*/

   FreeDbvASN1External( psResourceReportRsp->psResourceReport );

   /* ppsOtherInformation */

   if( psResourceReportRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportRsp->ppsOtherInformation );

   DbvFree( psResourceReportRsp );
   } /* End Free_DbvResourceReportRsp() */ 

void Free_DbvResultSetStatus( DbvResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End Free_DbvResultSetStatus() */ 

void Free_DbvScanReq( DbvScanReq *psScanReq )
   {
   /* psReferenceId */
   if( psScanReq->psReferenceId != NULL)
      FreeDbvOctetString(psScanReq->psReferenceId);
			 
   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psScanReq->ppsDatabaseNames );

   /* pcAttributeSetId  */

   if( psScanReq->pcAttributeSetId != NULL)
      FreeDbvObjectIdentifier( psScanReq->pcAttributeSetId );

   /* pulStepSize */

   if( psScanReq->pulStepSize != NULL)
      Free_Ulong( psScanReq->pulStepSize );

   /* pulPreferredPositionInResponse */

   if( psScanReq->pulPreferredPositionInResponse != NULL)
      Free_Ulong( psScanReq->pulPreferredPositionInResponse );

   /* ppsOtherInformation */

   if( psScanReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanReq->ppsOtherInformation );

   DbvFree( psScanReq );
   } /* End Free_DbvScanReq() */ 

void Free_DbvScanRsp( DbvScanRsp *psScanRsp )
   {

   /* psReferenceId */

   if( psScanRsp->psReferenceId != NULL )
      FreeDbvOctetString(psScanRsp->psReferenceId );

   /* pulStepSize */

   if ( psScanRsp->pulStepSize )
      Free_Ulong( psScanRsp->pulStepSize );

   /* pulPositionOfTerm */

   if ( psScanRsp->pulPositionOfTerm )
      Free_Ulong( psScanRsp->pulPositionOfTerm );

   /* psEntries */

   Free_DbvListEntries( psScanRsp->psEntries );

   /* pcAttributeSetId */

   if( psScanRsp->pcAttributeSetId != NULL)
      FreeDbvObjectIdentifier( psScanRsp->pcAttributeSetId );

   /* ppsOtherInformation */

   if( psScanRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanRsp->ppsOtherInformation );

   DbvFree( psScanRsp );
   } /* End Free_DbvScanRsp() */ 

void Free_DbvSearchReq( DbvSearchReq *psSearchReq )
   {

   /* psReferenceId */

   if( psSearchReq->psReferenceId != NULL)
      FreeDbvOctetString(psSearchReq->psReferenceId);

   /* sResultSetName */

   Free_compulsory_DbvInternationalString( &psSearchReq->sResultSetName );

   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psSearchReq->ppsDatabaseNames );

   /* psSmallSetElementNames */

   Free_DbvElementSetNames( psSearchReq->psSmallSetElementNames );

   /* psMediumSetElementNames */

   Free_DbvElementSetNames( psSearchReq->psMediumSetElementNames );

   /* pcPreferredRecordSyntax */

   if( psSearchReq->pcPreferredRecordSyntax != NULL)
      FreeDbvObjectIdentifier( psSearchReq->pcPreferredRecordSyntax );

   /* psQuery */

   Free_DbvQuery( psSearchReq->psQuery );

   /* psAdditionalSearchInfo */

   if( psSearchReq->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsOtherInformation );

   DbvFree( psSearchReq );
   } /* End Free_DbvSearchReq() */ 

void Free_DbvSearchRsp( DbvSearchRsp *psSearchRsp )
   {

   /* psReferenceId */

   if( psSearchRsp->psReferenceId != NULL )
      FreeDbvOctetString(psSearchRsp->psReferenceId );

   /* peResultSetStatus */

   if ( psSearchRsp->peResultSetStatus )
      Free_DbvResultSetStatus( psSearchRsp->peResultSetStatus );

   /* pePresentStatus */

   if ( psSearchRsp->pePresentStatus )
      Free_DbvPresentStatus( psSearchRsp->pePresentStatus );

   /* psRecords */

   Free_DbvRecords( psSearchRsp->psRecords );

   /* psAdditionalSearchInfo */

   if( psSearchRsp->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsOtherInformation );

   DbvFree( psSearchRsp );
   } /* End Free_DbvSearchRsp() */ 

void Free_DbvSegmentReq( DbvSegmentReq *psSegmentReq )
   {

   /* psReferenceId */

   if( psSegmentReq->psReferenceId != NULL)
      FreeDbvOctetString(psSegmentReq->psReferenceId);

   /* ppsSegmentRecords */

   FreeN_DbvNamePlusRecord( psSegmentReq->ppsSegmentRecords );

   /* ppsOtherInformation */

   if( psSegmentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSegmentReq->ppsOtherInformation );

   DbvFree( psSegmentReq );
   } /* End Free_DbvSegmentReq() */ 

void Free_DbvSingleASN1Type( DbvSingleASN1Type *psSingleASN1Type )
   {

   if ( psSingleASN1Type == NULL )
      return;

   switch( psSingleASN1Type->eASN1Type )
      {
      case AT_UnstructuredText:
	 if ( psSingleASN1Type-> uASN1.psUnstructuredText != NULL )
	    FreeDbvInternationalString( psSingleASN1Type->
				 uASN1.psUnstructuredText );
	 break;

      case AT_OpacRecord:
	 Free_DbvOpacRecord( psSingleASN1Type->uASN1.psOpacRecord );
	 break;

      case AT_SummaryRecord:
	 Free_DbvSummaryRecord( psSingleASN1Type->uASN1.psSummaryRecord );
	 break;

      case AT_ResourceReport1:
	 Free_DbvResourceReport1( psSingleASN1Type->uASN1.psResourceReport1 );
	 break;

      case AT_ResourceReport2:
	 Free_DbvResourceReport2( psSingleASN1Type->uASN1.psResourceReport2 );
	 break;

      case AT_EsTaskPackage:
	 Free_DbvEsTaskPackage( psSingleASN1Type->uASN1.psEsTaskPackage );
	 break;

      case AT_ItemOrderExtendedService:
	 Free_DbvItemOrderExtendedService( psSingleASN1Type->
					   uASN1.psItemOrderExtendedService );
	 break;

      case AT_UpdateExtendedService:
	 Free_DbvUpdateExtendedService( psSingleASN1Type->
					uASN1.psUpdateExtendedService );
	 break;

      case AT_IllPdu:
	 Free_DbvIllPdu( psSingleASN1Type->uASN1.psIllPdu );
	 break;

	
      case AT_ExplainRecord:
	 Free_DbvExplainRecord( psSingleASN1Type->uASN1.psExplainRecord );
         break;




      case AT_PromptObject:
         Free_DbvPromptObject( psSingleASN1Type->uASN1.psPromptObject );
         break;

      case AT_CharSetAndLanguage:
	 if ( psSingleASN1Type->uASN1.psCharSetAndLanguage != NULL )
	    Free_DbvCharSetAndLanguage( psSingleASN1Type->uASN1.psCharSetAndLanguage );
         break;

      case AT_UnknownType:
	 if ( psSingleASN1Type->uASN1.psUnknownType != NULL )
	    FreeDbvOctetString( psSingleASN1Type->uASN1.psUnknownType );
         break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSingleASN1Type );
   } /* End Free_DbvSingleASN1Type() */ 

void Free_DbvSortKeySpec( DbvSortKeySpec *psSortKeySpec )
   {

   if ( psSortKeySpec->psMVActionType != NULL )
      Free_DbvSortMVAction( psSortKeySpec->psMVActionType );

   DbvFree( psSortKeySpec );
   } /* End Free_DbvSortKeySpec() */ 

void Free_DbvSortMVAction( DbvSortMVAction *psSortMVAction )
   {

   if ( psSortMVAction->psOctetString!= NULL )
      FreeDbvOctetString( psSortMVAction->psOctetString );

   DbvFree( psSortMVAction );
   } /* End Free_DbvSortMVAction() */ 

void Free_DbvSortReq( DbvSortReq *psSortReq )
   {
   /* psReferenceId */
   if( psSortReq->psReferenceId != NULL)
      FreeDbvOctetString(psSortReq->psReferenceId);
			 
   /* ppsInputResultSetName */

   FreeN_DbvInternationalString( psSortReq->ppsInputResultSetName );

   /* sSortedResultSetName */

   Free_compulsory_DbvInternationalString( &psSortReq->sSortedResultSetName );

   /* ppsSortSequence */

   FreeN_DbvSortKeySpec( psSortReq->ppsSortSequence );

   /* ppsOtherInformation */

   if( psSortReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortReq->ppsOtherInformation );

   DbvFree( psSortReq );
   } /* End Free_DbvSortReq() */ 

void Free_DbvSortResultSetStatus( DbvSortResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End Free_DbvSortResultSetStatus() */ 

void Free_DbvSortRsp( DbvSortRsp *psSortRsp )
   {

   /* psReferenceId */

   if( psSortRsp->psReferenceId != NULL )
      FreeDbvOctetString(psSortRsp->psReferenceId );

   /* peResultSetStatus  */

   Free_DbvSortResultSetStatus( psSortRsp->peResultSetStatus );

   /* ppsDiagnostics */

   FreeN_DbvDiagRec( psSortRsp->ppsDiagnostics );

   /* ppsOtherInformation */

   if( psSortRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortRsp->ppsOtherInformation );

   DbvFree( psSortRsp );
   } /* End Free_DbvSortRsp() */ 

void Free_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      {
      do_Free_DbvStringOrNumeric( psStringOrNumeric );
      DbvFree( psStringOrNumeric );
      } /* End if */

   } /* End FreeDbvStringOrNumeric() */

void Free_compulsory_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      do_Free_DbvStringOrNumeric( psStringOrNumeric );

   } /* End Free_compulsory_DbvStringOrNumeric() */


void do_Free_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {


   switch( psStringOrNumeric->eStringOrNumeric )
      {
      case SNT_String:
	 FreeDbvInternationalString( psStringOrNumeric->
				     uStringOrNumeric.psString);
	 break;

      case SNT_Numeric:
      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvStringOrNumeric() */ 

void Free_DbvSummaryRecord( DbvSummaryRecord *psSummaryRecord )
   {

   if ( psSummaryRecord == NULL )
      return;

   /* sTitle */

   Free_compulsory_DbvInternationalString( &psSummaryRecord->sTitle );

   /* psAuthor */

   if ( psSummaryRecord->psAuthor != NULL )
      FreeDbvInternationalString( psSummaryRecord->psAuthor );

   /* psCallNumber */

   if ( psSummaryRecord->psCallNumber != NULL )
      FreeDbvInternationalString( psSummaryRecord->psCallNumber );

   /* psRecordType */

   if ( psSummaryRecord->psRecordType != NULL )
      FreeDbvInternationalString( psSummaryRecord->psRecordType );

   /* psBibliographicLevel */

   if ( psSummaryRecord->psBibliographicLevel != NULL )
      FreeDbvInternationalString( psSummaryRecord->psBibliographicLevel );

   /* ppsFormat */

   FreeN_DbvFormatSpec( psSummaryRecord->ppsFormat );

   /* psPublicationPlace */

   if ( psSummaryRecord->psPublicationPlace != NULL )
      FreeDbvInternationalString( psSummaryRecord->psPublicationPlace );

   /* psPublicationDate */

   if ( psSummaryRecord->psPublicationDate != NULL )
      FreeDbvInternationalString( psSummaryRecord->psPublicationDate );

   /* psTargetSystemKey */

   if ( psSummaryRecord->psTargetSystemKey != NULL )
      FreeDbvInternationalString( psSummaryRecord->psTargetSystemKey );

   /* psSatisfyingElement */

   if ( psSummaryRecord->psSatisfyingElement != NULL )
      FreeDbvInternationalString( psSummaryRecord->psSatisfyingElement );

   /* pulRank */

   if ( psSummaryRecord->pulRank )
      Free_Ulong( psSummaryRecord->pulRank );

   /* psDocumentId */

   if ( psSummaryRecord->psDocumentId != NULL )
      FreeDbvInternationalString( psSummaryRecord->psDocumentId );

   /* psAbstract */

   if ( psSummaryRecord->psAbstract != NULL )
      FreeDbvInternationalString( psSummaryRecord->psAbstract );

   /* ppsOtherInformation */

   if ( psSummaryRecord->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psSummaryRecord->ppsOtherInformation );

   DbvFree( psSummaryRecord );
   } /* End Free_DbvSummaryRecord() */ 

void Free_DbvSuppliedRecord( DbvSuppliedRecord *psSuppliedRecord )
   {

   if ( psSuppliedRecord == NULL )
      return;

   /* psUpdtRecordId  */

   Free_DbvUpdtRecordId( psSuppliedRecord->psUpdtRecordId ); 

   /* psUpdtSupplementalId */

   Free_DbvUpdtSupplementalId( psSuppliedRecord->psUpdtSupplementalId  );

   /* psCorrelationInfo */

   Free_DbvCorrelationInfo( psSuppliedRecord->psCorrelationInfo  );

   /* sRecord */

   Free_compulsory_DbvASN1External( &psSuppliedRecord->sRecord );

   DbvFree( psSuppliedRecord );

   } /* End Free_compulsory_DbvSuppliedRecord() */ 

void Free_DbvTaskPackageRecord( DbvTaskPackageRecord *psTaskPackageRecord )
   {

   if ( psTaskPackageRecord == NULL )
      return;

   /* psUpdtRecordOrDiag */

   Free_DbvUpdtRecordOrDiag( psTaskPackageRecord->psUpdtRecordOrDiag );

   /* psCorrelationInfo */

   Free_DbvCorrelationInfo( psTaskPackageRecord->psCorrelationInfo );

   DbvFree( psTaskPackageRecord );
   } /* End Free_DbvTaskPackageRecord() */ 

void Free_DbvTermInfo( DbvTermInfo *psTermInfo )
   {

   if ( psTermInfo == NULL )
      return;

   /* sTerm */

   Free_compulsory_DbvTerm( &psTermInfo->sTerm  );

   /* psDisplayTerm */
   FreeDbvInternationalString( psTermInfo->psDisplayTerm );

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psTermInfo->ppsSuggestedAttribute );

   /* ppsAlternativeTerm */

   FreeN_DbvAttributesPlusTerm( psTermInfo->ppsAlternativeTerm );

   /* pulGlobalOccurrences */

   if ( psTermInfo->pulGlobalOccurrences != NULL )
      Free_Ulong( psTermInfo->pulGlobalOccurrences );

   /* psByAttributes */

   FreeN_DbvOccurrenceByAttributes( psTermInfo->ppsByAttributes );

   /* ppsOtherInformation */

   if ( psTermInfo->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psTermInfo->ppsOtherInformation );

   DbvFree( psTermInfo );
   } /* End Free_DbvTermInfo() */ 

void Free_DbvUpdateExtendedService( DbvUpdateExtendedService 
						*psUpdateExtendedService )
   {

   if ( psUpdateExtendedService == NULL )
      return;

   switch( psUpdateExtendedService->eUpdateReqRsp )
      {
      case UROR_Request:
	 Free_DbvUpdateRequestPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateRequest );
	 break;

      case UROR_Response:
	 Free_DbvUpdateTaskPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateResponse );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psUpdateExtendedService );
   } /* End Free_DbvUpdateExtendedService() */

void Free_DbvUpdateRequestPackage( DbvUpdateRequestPackage 
						*psUpdateRequestPackage )
   {

   if ( psUpdateRequestPackage == NULL )
      return;

   Free_compulsory_DbvOriginPartToKeep( &psUpdateRequestPackage->
							sOriginPartToKeep );
   FreeN_DbvSuppliedRecord( psUpdateRequestPackage->ppsSuppliedRecord );

   DbvFree( psUpdateRequestPackage );
   } /* End Free_DbvUpdateRequestPackage() */ 

void Free_DbvUpdateTaskPackage( DbvUpdateTaskPackage 
						*psUpdateTaskPackage )
   {

   if ( psUpdateTaskPackage == NULL )
      return;

   Free_compulsory_DbvOriginPartToKeep( &psUpdateTaskPackage->
							sOriginPartToKeep );
   Free_compulsory_DbvUpdateTargetPart( &psUpdateTaskPackage->sTargetPart );

   DbvFree( psUpdateTaskPackage );
   } /* End Free_DbvUpdateTaskPackage() */ 

void Free_DbvUpdtRecordId( DbvUpdtRecordId *psUpdtRecordId )
   {

   if ( psUpdtRecordId == NULL )
      return;

   switch( psUpdtRecordId->eRecordIdType )
      {
      case URIT_Number:
	 /* No Space allocated */
	 break;

      case URIT_String:
	 if ( psUpdtRecordId->uUpdtRecordId.psString != NULL )
	    FreeDbvInternationalString( psUpdtRecordId->
					uUpdtRecordId.psString );
	 break;

      case URIT_Opaque:
	 if ( psUpdtRecordId->uUpdtRecordId.psOpaque != NULL )
	    FreeDbvOctetString( psUpdtRecordId-> uUpdtRecordId.psOpaque);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordId );
   } /* End Free_DbvUpdtRecordId() */ 

void Free_DbvUpdtRecordOrDiag( DbvUpdtRecordOrDiag *psUpdtRecordOrDiag )
   {

   if ( psUpdtRecordOrDiag == NULL )
      return;

   switch( psUpdtRecordOrDiag->eUpdtRecordOrDiag )
      {
      case URDT_Record:
	 FreeDbvASN1External( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord );
	 break;

      case URDT_Diagnostic:
	 Free_DbvDiagRec( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordOrDiag );
   } /* End Free_DbvUpdtRecordOrDiag() */ 

void Free_DbvUpdtSupplementalId( DbvUpdtSupplementalId *psUpdtSupplementalId )
   {

   if ( psUpdtSupplementalId == NULL )
      return;

   switch( psUpdtSupplementalId->eSupplementalIdType )
      {
      case USIT_Time:
	 Free_DbvGeneralizedTime( psUpdtSupplementalId->
					uUpdtSupplementalId.pcTime );
	 break;

      case USIT_Version:
	 if ( psUpdtSupplementalId->uUpdtSupplementalId.psVersion != NULL )
	    FreeDbvInternationalString( psUpdtSupplementalId->
					uUpdtSupplementalId.psVersion );
	 break;

      case USIT_PreviousVersion:
	 FreeDbvASN1External( psUpdtSupplementalId->
					uUpdtSupplementalId.psPreviousVersion );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtSupplementalId );
   } /* End Free_DbvUpdtSupplementalId() */ 

void Free_DbvVisibleString( DbvVisibleString psVisibleString )
   {

   if ( psVisibleString == NULL )
      return;

   DbvFree( psVisibleString );
   } /* End Free_DbvVisibleString() */ 

void Free_DbvVolume( DbvVolume *psVolume )
   {

   if ( psVolume == NULL )
	 return;

   /* psEnumeration */

   if ( psVolume->psEnumeration != NULL )
      FreeDbvInternationalString( psVolume->psEnumeration );

   /* psChronology */

   if ( psVolume->psChronology != NULL )
      FreeDbvInternationalString( psVolume->psChronology );

   /* psEnumAndChron */

   if ( psVolume->psEnumAndChron != NULL )
      FreeDbvInternationalString( psVolume->psEnumAndChron );

   DbvFree( psVolume );

   } /* End Free_DbvVolume() */ 

void Free_Ulong( Ulong *pulong )
   {
   if (pulong)
   	DbvFree( pulong );
   } /* End Free_Ulong() */ 


void FreeDbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      {
      do_FreeDbvASN1External( psASN1External );
      DbvFree( psASN1External );
      } /* End if */

   } /* End FreeDbvASN1External() */

void Free_compulsory_DbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      do_FreeDbvASN1External( psASN1External );

   } /* End Free_compulsory_DbvASN1External() */


void do_FreeDbvASN1External( DbvASN1External *psASN1External )
   {


   if ( psASN1External->pcObjectIdentifier != NULL )
      FreeDbvObjectIdentifier( psASN1External->pcObjectIdentifier );

   switch( psASN1External->eValueType )
      {
      case EVT_SingleASN1Type:
	 Free_DbvSingleASN1Type( psASN1External->
				 uExternalValue.psSingleASN1Type );
	 break;

      case EVT_OctetAligned:
	 FreeDbvOctetString( psASN1External->
			     uExternalValue.psOctetAligned );
	 break;

      case EVT_Arbitrary:
	 FreeDbvOctetString( psASN1External->
			     uExternalValue.psArbitrary );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End FreeDbvASN1External() */ 

void Free_DbvRecordComposition(DbvRecordComposition *psRecordComposition)
   {

   if(psRecordComposition != NULL)
   {
	switch(psRecordComposition->eRecordCompositionType)
	{
	case RCT_ElementSetNames:
		Free_DbvElementSetNames(psRecordComposition->uRecordComposition.
			psElementSetNames);
		break;
	case RCT_CompSpec:
		break;
	default:
		Free_DbvCompSpec(psRecordComposition->uRecordComposition.
			psCompSpec);
		break;
	} /* End switch */
	DbvFree(psRecordComposition);
   } /* End if */

   } /* End Free_DbvRecordComposition */

void Free_DbvCompSpec(DbvCompSpec *psCompSpec)
   {
   if(psCompSpec != NULL)
	{
	Free_DbvSpecification(psCompSpec->psGeneric);
	FreeN_DbvDbSpecificComp(psCompSpec->ppsDbSpecific);
	FreeN_DbvObjectIdentifier(psCompSpec->ppcRecordSyntax);
	DbvFree(psCompSpec);
	}
   } /* End Free_DbvCompSpec */

void Free_DbvSpecification(DbvSpecification *psSpecification)
{
	if(psSpecification != NULL)
	{
		Free_compulsory_DbvSpecification(psSpecification);
		DbvFree(psSpecification);
	}
} /* End Free_DbvSpecification */

void Free_DbvDbSpecificComp(DbvDbSpecificComp *psDbSpecific)
{
	if(psDbSpecific != NULL)
	{
		Free_compulsory_DbvInternationalString(&psDbSpecific->
			sDatabaseName);
		Free_compulsory_DbvSpecification(&psDbSpecific->
			sSpecification);
	} /* End if */
} /* End Free_DbvDbSpecificComp */

void Free_DbvRPNQuery(DbvRPNQuery *psDbvRpnQuery)
{
	if(psDbvRpnQuery->pcAttributeSetId != NULL)
		DbvFree(psDbvRpnQuery->pcAttributeSetId);

        Free_rpn_structure(&psDbvRpnQuery->sRpnStructure);
        DbvFree(psDbvRpnQuery);
}

void Free_rpn_structure(DbvRPNStructure *psDbvRPNStructure)
{
 
        switch (psDbvRPNStructure->eRpnStructureType) {
        case RST_RpnLopRopOperator:
 
                if (psDbvRPNStructure->uRPNStructure.psRpnLopRopOperator != NULL)
                        Free_rpnlopropoperator(psDbvRPNStructure->uRPNStructure.
				psRpnLopRopOperator);
                break;
        case RST_RpnOperand:
                if (psDbvRPNStructure->uRPNStructure.psRpnOperand != NULL)
 
                        Free_rpnoperand(psDbvRPNStructure->uRPNStructure.
				psRpnOperand);
                break;
        }
 
}

void Free_rpnlopropoperator(DbvRPNLopRopOperator *psDbvRPNLopRopOperator)
{
 
        if (psDbvRPNLopRopOperator->psLeftOperand != NULL)
                Free_rpn_structure(psDbvRPNLopRopOperator->psLeftOperand);
 
        if (psDbvRPNLopRopOperator->psRightOperand != NULL)
                Free_rpn_structure(psDbvRPNLopRopOperator->psRightOperand);
 
	Free_DbvOperator(&psDbvRPNLopRopOperator->sOperator);
 
        DbvFree(psDbvRPNLopRopOperator);
}

void Free_rpnoperand(DbvOperand *psRpnOperand)
{
 
        switch (psRpnOperand->eOperandType) {
        case OT_AttributePlusTerm:
 
                if (psRpnOperand->uOperand.psAttributesPlusTerm != NULL)
                        Free_attsplusterm(psRpnOperand->uOperand.
				psAttributesPlusTerm);
                break;
        }
 
        DbvFree(psRpnOperand);
}

void Free_attsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm)
{
        int index = 0;
 
 
        /* Print all attributes */
 
        while (psAttributesPlusTerm->ppsAttributeElement[index] != NULL) {
                Free_att_element(psAttributesPlusTerm->
			ppsAttributeElement[index]);
                index++;
        }
 
        /* then the term */
        Free_att_term(psAttributesPlusTerm->sTerm);
 
 
        DbvFree(psAttributesPlusTerm);
}

void Free_att_element(DbvAttributeElement *psAttributeElement)
{
        if (psAttributeElement->pcAttributeSetId != NULL)
                DbvFree(psAttributeElement->pcAttributeSetId);
 
        DbvFree(psAttributeElement);
}
 
void Free_att_term(DbvTerm sTerm)
{
        /* Only general catered for !! */
        switch (sTerm.eTermType) {
        case TT_General:
 
                if (sTerm.uTerm.psGeneral != NULL)
                        DbvFree(sTerm.uTerm.psGeneral);
                break;
        }
 
}

void Free_DbvOperator(DbvOperator *psOperator)
{
	
	/* Operator is not dynamically allocated */
	if(psOperator->psProximityOperator != NULL)
	{
		if(psOperator->psProximityOperator->pfExclusion != NULL)
			DbvFree(psOperator->psProximityOperator->pfExclusion);
		DbvFree(psOperator->psProximityOperator);
	}
}

void Free_DbvExtendedServicesReq(DbvExtendedServicesReq *psExtendedServicesReq)
{
   /* psReferenceId */

   if( psExtendedServicesReq->psReferenceId != NULL)
      FreeDbvOctetString(psExtendedServicesReq->psReferenceId);

   /* pcPackageType */
   FreeDbvObjectIdentifier( psExtendedServicesReq->pcPackageType);

   /* psPackageName */
   FreeDbvInternationalString(psExtendedServicesReq->psPackageName);

   /* psUserId */
   FreeDbvInternationalString(psExtendedServicesReq->psUserId);

   /* psRetentionTime */
   Free_DbvIntUnit(psExtendedServicesReq->psRetentionTime);

   /* ppsPermissions */
   FreeN_DbvPermissions(psExtendedServicesReq->ppsPermissions);

   /* psDescription */
   FreeDbvInternationalString(psExtendedServicesReq->psDescription);

   /* psOriginSpecificParameter */
   FreeDbvASN1External(psExtendedServicesReq->psOriginSpecificParameter);

   /* psElements */
   FreeDbvInternationalString(psExtendedServicesReq->psElements);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesReq->ppsOtherInformation);

   DbvFree(psExtendedServicesReq);
} /* End Free_DbvExtendedServicesRsp */

void Free_DbvExtendedServicesRsp(DbvExtendedServicesRsp *psExtendedServicesRsp)
{
   /* psReferenceId */
 
   FreeDbvOctetString(psExtendedServicesRsp->psReferenceId);
 
   /* ppsDiagnostics */
   FreeN_DbvDiagRec(psExtendedServicesRsp->ppsDiagnostics);

   /* psTaskPackage */
   FreeDbvASN1External(psExtendedServicesRsp->psTaskPackage);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesRsp->ppsOtherInformation);

} /* End Free_DbvExtendedServicesRsp */

void Free_DbvAssociateResult(DbvAssociateResult *puAssociateResult,
        DbvAssociateState eAssociateState)
{
        int place = 0;
        if(eAssociateState == Associate_Null || eAssociateState ==
          Associate_Pending)
                return;
 
        if(eAssociateState == Associate_Accepted)
        {
                if(puAssociateResult->psProtocolAndSyntaxes->
                  peApplicationProtocol != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peApplicationProtocol);
                if(puAssociateResult->psProtocolAndSyntaxes->peAbstractSyntax 
                  != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peAbstractSyntax);
                if(puAssociateResult->psProtocolAndSyntaxes->
			ppcCommonSyntaxes != NULL)
                {
                while(puAssociateResult->psProtocolAndSyntaxes->
                  ppcCommonSyntaxes[place] != NULL)
                {
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          ppcCommonSyntaxes[place]);
                        place++;
                }
		}
                DbvFree(puAssociateResult->psProtocolAndSyntaxes);
        }
        else
        {       
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectSource != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectSource);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectResult != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectResult);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectReason != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectReason);
                DbvFree(puAssociateResult->psAssociateReject);

        }
        DbvFree(puAssociateResult);
}

/* 12-2-96 The following function where added by Dan Maskell  */

void Free_DbvGeneralString( DbvGeneralString *psGeneralString )
   {
   if( psGeneralString == NULL)
      return;

   if(psGeneralString->pcValue != NULL )
      DbvFree( psGeneralString->pcValue );

   DbvFree( psGeneralString );
   } /* End Free_DbvGeneralString() */
 
 
void Free_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      {
      do_Free_DbvEsIoTargetPart( psEsIoTargetPart );
      DbvFree( psEsIoTargetPart );
      } /* End if */

   } /* End Free_DbvEsIoTargetPart() */ 

void do_Free_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   /* psItemRequest */
   FreeDbvASN1External( psEsIoTargetPart->psItemRequest );

   /* psStatusOrErrorReport */
   FreeDbvASN1External( psEsIoTargetPart->psStatusOrErrorReport );

   /* peAuxiliaryStatus */
   Free_DbvEsIoAuxiliaryStatus( psEsIoTargetPart->peAuxiliaryStatus );

   } /* End do_Free_DbvEsIoTargetPart() */ 

void Free_DbvEsIoResultSetItem( DbvEsIoResultSetItem *psEsIoResultSetItem )
   {

   if ( psEsIoResultSetItem == NULL )
      return;

   /* sResultSetId */
   Free_compulsory_DbvInternationalString( &psEsIoResultSetItem->sResultSetId);

   DbvFree( psEsIoResultSetItem );
   } /* End Free_DbvEsIoResultSetItem() */ 

void Free_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      {
      do_Free_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );
      DbvFree( psEsIoOriginPartNotToKeep );
      } /* End if */

   } /* End Free_DbvEsIoOriginPartNotToKeep() */ 

void Free_DbvEsIoCreditCardInfo( DbvEsIoCreditCardInfo *psEsIoCreditCardInfo)
   {
   if( psEsIoCreditCardInfo == NULL)
      return;

   /* sNameOnCard */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sNameOnCard );

   /* sExpirationDate */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->
					   sExpirationDate );

   /* sCardNumber */
   Free_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sCardNumber );

   DbvFree( psEsIoCreditCardInfo );
   } /* End Free_DbvEsIoCreditCardInfo() */
 
 
void do_Free_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   /* psResultSetItem */
   Free_DbvEsIoResultSetItem( psEsIoOriginPartNotToKeep->psResultSetItem );

   /* psItemRequest */
   FreeDbvASN1External( psEsIoOriginPartNotToKeep->psItemRequest );

   } /* End do_Free_DbvEsIoOriginPartNotToKeep() */ 

void Free_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      {
      do_Free_DbvIllString( psIllString );
      DbvFree( psIllString );
      } /* End if */

   } /* End Free_DbvIllString() */ 


void do_Free_DbvIllString( DbvIllString *psIllString )
   {

   switch( psIllString->eStringType)
      {
      case ST_GeneralString:
	 Free_DbvGeneralString( psIllString->uString.psGeneralString);
	 break;

      case ST_EdifactString:
	 Free_DbvVisibleString( psIllString->uString.pcVisibleString);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvIllString() */ 

void Free_DbvPersonOrInstitutionSymbol( DbvPersonOrInstitutionSymbol 
					*psPersonOrInstitutionSymbol )
   {
   if ( psPersonOrInstitutionSymbol == NULL )
      return;

   /* sSymbol */
   Free_compulsory_DbvIllString( &psPersonOrInstitutionSymbol->sSymbol );

   DbvFree( psPersonOrInstitutionSymbol );
   } /* End Free_DbvPersonOrInstitutionSymbol() */ 

void Free_DbvNameOfPersonOrInstitution( DbvNameOfPersonOrInstitution 
					*psNameOfPersonOrInstitution )
   {
   if ( psNameOfPersonOrInstitution == NULL )
      return;

   /* sName */
   Free_compulsory_DbvIllString( &psNameOfPersonOrInstitution->sName );

   DbvFree( psNameOfPersonOrInstitution );
   } /* End Free_DbvNameOfPersonOrInstitution() */ 

void Free_DbvSystemId( DbvSystemId *psSystemId )
   {
   if ( psSystemId != NULL )
      {
      do_Free_DbvSystemId( psSystemId );
      DbvFree( psSystemId );
      } /* End if */

   } /* End Free_DbvSystemId() */ 

void do_Free_DbvSystemId( DbvSystemId *psSystemId )
   {
   /* psSymbol */
   Free_DbvPersonOrInstitutionSymbol( psSystemId->psSymbol );

   /* psName */
   Free_DbvNameOfPersonOrInstitution( psSystemId->psName );

   } /* End do_Free_DbvSystemId() */ 

void Free_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      {
      do_Free_DbvTransactionId( psTransactionId );
      DbvFree( psTransactionId );
      } /* End if */

   } /* End Free_DbvTransactionId() */ 

void do_Free_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   /* psInitialRequesterId */
   Free_DbvSystemId( psTransactionId->psInitialRequesterId );

   /* sTransactionGroupQualifier */
   Free_compulsory_DbvIllString( &psTransactionId->sTransactionGroupQualifier );

   /* sTransactionQualifier */
   Free_compulsory_DbvIllString( &psTransactionId->sTransactionQualifier );

   /* psSubTransactionQualifier */
   Free_DbvIllString( psTransactionId->psSubTransactionQualifier );

   } /* End do_Free_DbvTransactionId() */ 

void Free_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      {
      do_Free_DbvDateTime(  psDateTime );
      DbvFree( psDateTime );
      } /* End if */
   } /* End Free_DbvDateTime() */ 

void do_Free_DbvDateTime( DbvDateTime *psDateTime )
   {

   /* pcDate */
   Free_DbvVisibleString( psDateTime->pcDate );

   /* pcTime */
   Free_DbvVisibleString( psDateTime->pcTime );

   } /* End do_Free_DbvDateTime() */ 
void Free_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      {
      do_Free_DbvServiceDateTime( psServiceDateTime );
      DbvFree( psServiceDateTime );
      } /* End if */

   } /* End Free_DbvServiceDateTime() */ 

void do_Free_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   /* sThisService */
   Free_compulsory_DbvDateTime( &psServiceDateTime->sThisService );

   /* psOriginalService */
   Free_DbvDateTime( psServiceDateTime->psOriginalService );

   } /* End do_Free_DbvServiceDateTime() */ 


void Free_DbvPostalAddress( DbvPostalAddress *psPostalAddress )
   {

   if ( psPostalAddress == NULL )
      return;

   /* psNameOfPersonOrInstitution */
   Free_DbvNameOfPersonOrInstitution( psPostalAddress->
				      psNameOfPersonOrInstitution );

   /* psExtendedPostalDeliveryAddress */
   Free_DbvIllString( psPostalAddress->psExtendedPostalDeliveryAddress );

   /* psStreetAndNumber */
   Free_DbvIllString( psPostalAddress->psStreetAndNumber );

   /* psPostOfficeBox */
   Free_DbvIllString( psPostalAddress->psPostOfficeBox );

   /* psCity */
   Free_DbvIllString( psPostalAddress->psCity );

   /* psRegion */
   Free_DbvIllString( psPostalAddress->psRegion );

   /* psCountry */
   Free_DbvIllString( psPostalAddress->psCountry );

   /* psPostalCode */
   Free_DbvIllString( psPostalAddress->psPostalCode );

   DbvFree( psPostalAddress );
   } /* End Free_DbvPostalAddress() */ 

void Free_DbvSystemAddress( DbvSystemAddress *psSystemAddress )
   {

   if ( psSystemAddress == NULL )
      return;

   /* psTelecomServiceIdentifier  */
   Free_DbvIllString( psSystemAddress->psTelecomServiceIdentifier );

   /* psTelecomServiceAddress */
   Free_DbvIllString( psSystemAddress->psTelecomServiceAddress );

   DbvFree( psSystemAddress );
   } /* End Free_DbvSystemAddress() */ 

void Free_DbvDeliveryAddress( DbvDeliveryAddress *psDeliveryAddress )
   {

   if ( psDeliveryAddress == NULL )
      return;

   /* psPostalAddress */
   Free_DbvPostalAddress( psDeliveryAddress->psPostalAddress );

   /* psElectronicAddress */
   Free_DbvSystemAddress( psDeliveryAddress->psElectronicAddress);

   DbvFree( psDeliveryAddress );
   } /* End Free_DbvDeliveryAddress() */ 

void Free_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      {
      do_Free_DbvElDeliveryDetails( psElDeliveryDetails );
      DbvFree( psElDeliveryDetails );
      } /* End if */

   } /* End Free_DbvElDeliveryDetails() */ 

void do_Free_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   switch( psElDeliveryDetails->eDeliveryDetailsType )
      {
      case DDT_SystemAddress:
	 Free_DbvSystemAddress( psElDeliveryDetails->
				uDeliveryDetails.psSystemAddress);
	 break;

      case DDT_SystemId:
	 Free_DbvSystemId( psElDeliveryDetails->
			   uDeliveryDetails.psSystemId);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_Free_DbvElDeliveryDetails() */ 

void Free_DbvElDeliveryParameter( 
			DbvElDeliveryParameter *psElDeliveryParameter )
   {

   if ( psElDeliveryParameter == NULL )
      return;

   /* pcDeliveryMode */
   FreeDbvObjectIdentifier( psElDeliveryParameter->pcDeliveryMode );

   /* psDummy */
   if ( psElDeliveryParameter->puDeliveryParameter )
      {
/*
      FreeDbvASN1External( psElDeliveryParameter->
			   puDeliveryParameter->psDummy );
*/

      /* puDeliveryParameter */
      DbvFree( psElDeliveryParameter->puDeliveryParameter );
      } /* End if */

   DbvFree( psElDeliveryParameter );
   } /* End Free_DbvElDeliveryParameter() */ 


void Free_DbvDocumentTypeParameter( 
			DbvDocumentTypeParameter *psDocumentTypeParameter )
   {

   if ( psDocumentTypeParameter == NULL )
      return;

   /* pcDeliveryMode */
   FreeDbvObjectIdentifier( psDocumentTypeParameter->pcDocumentTypeId );

   /* psDummy */
   if ( psDocumentTypeParameter->puDocumentTypeParameter)
      {
      FreeDbvASN1External( psDocumentTypeParameter->
			   puDocumentTypeParameter->psDummy );

      /* puDeliveryParameter */
      DbvFree( psDocumentTypeParameter->puDocumentTypeParameter);
      } /* End if */

   DbvFree( psDocumentTypeParameter );
   } /* End Free_DbvDocumentTypeParameter() */ 


void Free_DbvElectronicDeliveryService( 
		DbvElectronicDeliveryService *psElectronicDeliveryService )
   {

   if ( psElectronicDeliveryService == NULL )
      return;

   /* psDeliveryParameter */
   Free_DbvElDeliveryParameter( psElectronicDeliveryService->
				psDeliveryParameter);

   /* psDocumentTypeParameter */
   Free_DbvDocumentTypeParameter( psElectronicDeliveryService->
				  psDocumentTypeParameter );
   /* psDeliveryDescription */
   Free_DbvIllString( psElectronicDeliveryService->psDeliveryDescription );

   /* sDeliveryDetails */
   Free_compulsory_DbvElDeliveryDetails( &psElectronicDeliveryService->
					 sDeliveryDetails );


   /* psNameOrCode */
   Free_DbvIllString( psElectronicDeliveryService->psNameOrCode );

   /* pcDeliveryTime */
   Free_DbvVisibleString( psElectronicDeliveryService->pcDeliveryTime );
   
   DbvFree( psElectronicDeliveryService );
   } /* End Free_DbvElectronicDeliveryService() */ 


void Free_DbvDeliveryService( DbvDeliveryService *psDeliveryService )
   {

   if ( psDeliveryService == NULL )
      return;

   switch( psDeliveryService->eDeliveryServiceType )
      {
      case DST_PhysicalDelivery:
	 Free_DbvIllString( psDeliveryService->
			    uDeliveryService.psPhysicalDelivery);
	 break;

      case DST_ElectronicDelivery:
	 FreeN_DbvElectronicDeliveryService( psDeliveryService->
				uDeliveryService.ppsElectronicDelivery);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDeliveryService );
   } /* End Free_DbvDeliveryService() */ 

void Free_DbvRequesterOptionalMessages( 
		DbvRequesterOptionalMessages *psRequesterOptionalMessages )
   {

   if ( psRequesterOptionalMessages == NULL )
      return;

   DbvFree( psRequesterOptionalMessages );
   } /* End Free_DbvRequesterOptionalMessages() */ 

void Free_DbvSearchType( DbvSearchType *psSearchType )
   {

   if ( psSearchType == NULL )
      return;

   /* psLevelOfService */
   Free_DbvIllString( psSearchType->psLevelOfService );

   /* pcNeedBeforeDate */
   Free_DbvVisibleString( psSearchType->pcNeedBeforeDate );

   /* peExpiryFlag */
   Free_DbvExpiry( psSearchType->peExpiryFlag );

   /* pcExpiryDate */
   Free_DbvVisibleString( psSearchType->pcExpiryDate );

   DbvFree( psSearchType );
   } /* End Free_DbvSearchType() */ 

void Free_DbvSupplyMediumInfo( DbvSupplyMediumInfo *psSupplyMediumInfo )
   {

   if ( psSupplyMediumInfo == NULL )
      return;

   /* psMediumCharacteristics */
   Free_DbvIllString( psSupplyMediumInfo->psMediumCharacteristics );

   DbvFree( psSupplyMediumInfo );
   } /* End Free_DbvSupplyMediumInfo() */ 

void Free_DbvClientId( DbvClientId *psClientId )
   {

   if ( psClientId == NULL )
      return;

   /* psClientName */
   Free_DbvIllString( psClientId->psClientName);

   /* psClientStatus */
   Free_DbvIllString( psClientId->psClientStatus);

   /* psClientIdentifier */
   Free_DbvIllString( psClientId->psClientIdentifier);

   DbvFree( psClientId );
   } /* End Free_DbvClientId() */ 

void Free_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      {

      /* Free the fields in psItemId */
      do_Free_DbvItemId( psItemId );

      /* Dont attempt a DbvFree on psItemId as psItemId points to a structure
	 allocated at compile time, not one malloced at run time */

      } /* End if */

   } /* End Free_DbvItemId() */ 

void do_Free_DbvItemId( DbvItemId *psItemId )
   {

   /* peItemType */
   Free_DbvItemType( psItemId->peItemType );

   /* peMediumType */
   Free_DbvMediumType( psItemId->peMediumType );

   /* psCallNumber */
   Free_DbvIllString( psItemId->psCallNumber );

   /* psAuthor */
   Free_DbvIllString( psItemId->psAuthor );

   /* psTitle */
   Free_DbvIllString( psItemId->psTitle );

   /* psSubTitle */
   Free_DbvIllString( psItemId->psSubTitle );

   /* psSponsoringBody */
   Free_DbvIllString( psItemId->psSponsoringBody );

   /* psPlaceOfPublication */
   Free_DbvIllString( psItemId->psPlaceOfPublication );

   /* psPublisher */
   Free_DbvIllString( psItemId->psPublisher);

   /* psSeriesTitleNumber */
   Free_DbvIllString( psItemId->psSeriesTitleNumber);

   /* psVolumeIssue */
   Free_DbvIllString( psItemId->psVolumeIssue );

   /* psEdition */
   Free_DbvIllString( psItemId->psEdition );

   /* psPublicationDate */
   Free_DbvIllString( psItemId->psPublicationDate );

   /* psPublicationDateOfComponent */
   Free_DbvIllString( psItemId->psPublicationDateOfComponent );

   /* psAuthorOfArticle */
   Free_DbvIllString( psItemId->psAuthorOfArticle );

   /* psTitleOfArticle */
   Free_DbvIllString( psItemId->psTitleOfArticle );

   /* psPagination */
   Free_DbvIllString( psItemId->psPagination );

   /* psNationalBibliographyNo */
   FreeDbvASN1External( psItemId->psNationalBibliographyNo );

   /* psIsbn */
   Free_DbvIllString( psItemId->psIsbn );

   /* psIssn */
   Free_DbvIllString( psItemId->psIssn );

   /* psSystemNo */
   FreeDbvASN1External( psItemId->psSystemNo );

   /* psAdditionalNoLetters */
   Free_DbvIllString( psItemId->psAdditionalNoLetters );

   /* psVerificationReferenceSource */
   Free_DbvIllString( psItemId->psVerificationReferenceSource );

   } /* End do_Free_DbvItemId() */ 

void Free_DbvAmount( DbvAmount *psAmount )
   {

   if ( psAmount == NULL )
      return;

   /* pcCurrencyCode */
   Free_DbvPrintableString( psAmount->pcCurrencyCode );

   /* pcMonetaryValue */
   Free_DbvPrintableString( psAmount->pcMonetaryValue );

   DbvFree( psAmount );
   } /* End Free_DbvAmount() */ 

void Free_DbvCostInfo( DbvCostInfo *psCostInfo )
   {

   if ( psCostInfo == NULL )
      return;

   /* psAccountNumber */
   Free_DbvIllString( psCostInfo->psAccountNumber );

   /* psMaximumCost */
   Free_DbvAmount( psCostInfo->psMaximumCost );

   /* pfReciprocalAgreement */
   Free_DbvBool( psCostInfo->pfReciprocalAgreement );

   /* pfWillPayFee */
   Free_DbvBool( psCostInfo->pfWillPayFee );

   /* pfPaymentProvided */
   Free_DbvBool( psCostInfo->pfPaymentProvided );

   DbvFree( psCostInfo );
   } /* End Free_DbvCostInfo() */ 

void Free_DbvSendToList( DbvSendToList *psSendToList )
   {

   if ( psSendToList == NULL )
      return;

   /* sSystemId */
   Free_compulsory_DbvSystemId( &psSendToList->sSystemId );

   /* psAccountNumber */
   Free_DbvIllString( psSendToList->psAccountNumber );

   /* psSystemAddress */
   Free_DbvSystemAddress( psSendToList->psSystemAddress );

   DbvFree( psSendToList );
   } /* End Free_DbvSendToList() */ 

void Free_DbvThirdPartyInfo( DbvThirdPartyInfo *psThirdPartyInfo )
   {

   if ( psThirdPartyInfo == NULL )
      return;

   /* pfPermissionToForward */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToForward );

   /* pfPermissionToChain */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToChain );

   /* pfPermissionToPartition */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToPartition );

   /* pfPermissionToChangeSendToList */
   Free_DbvBool( psThirdPartyInfo->pfPermissionToChangeSendToList );

   /* psInitialRequesterAddress */
   Free_DbvSystemAddress( psThirdPartyInfo->psInitialRequesterAddress );

   /* pePreference */
   Free_DbvPreference( psThirdPartyInfo->pePreference );

   /* ppsSendToList */
   FreeN_DbvSendToList( psThirdPartyInfo->ppsSendToList );

   /* ppsAlreadyTriedList */
   FreeN_DbvSystemId( psThirdPartyInfo->ppsAlreadyTriedList );

   DbvFree( psThirdPartyInfo );
   } /* End Free_DbvThirdPartyInfo() */ 


void Free_DbvRequesterGroup( DbvRequesterGroup *peRequesterGroup )
{
   if ( peRequesterGroup == NULL )
      return;

   DbvFree( peRequesterGroup );

} /* End Free_DbvRequesterGroup() */



void Free_DbvAddRequesterInfo( DbvAddRequesterInfo *psAddRequesterInfo )
{
   if ( psAddRequesterInfo == NULL )
      return;

   /* peRequesterGroup */
   Free_DbvRequesterGroup( psAddRequesterInfo->peRequesterGroup );

   /* pcRequesterEmailAddress */
   Free_DbvVisibleString( psAddRequesterInfo->pcRequesterEmailAddress );

   DbvFree( psAddRequesterInfo );
 
} /* End Free_DbvAddRequesterInfo() */


void Free_DbvStatusLevelUser( DbvStatusLevelUser *peStatusLevelUser )
{
   if ( peStatusLevelUser == NULL )
      return;

   DbvFree( peStatusLevelUser );
 
} /* End Free_DbvStatusLevelUser() */


 
void Free_DbvStatusLevelRequester( DbvStatusLevelRequester *peStatusLevelOrigin )
{
   if ( peStatusLevelOrigin == NULL )
      return;

   DbvFree( peStatusLevelOrigin );
 
} /* End Free_DbvStatusLevelRequester() */


 
void Free_DbvAddDeliveryInfo( DbvAddDeliveryInfo *psAddDeliveryInfo )
{
   if ( psAddDeliveryInfo == NULL )
      return;

   /* peStatusLevelUser */
   Free_DbvStatusLevelUser( psAddDeliveryInfo->peStatusLevelUser );

   /* peStatusLevelOrigin */
   Free_DbvStatusLevelRequester( psAddDeliveryInfo->peStatusLevelOrigin );

   /* pcDeliveryNotes */
   Free_DbvVisibleString( psAddDeliveryInfo->pcDeliveryNotes );

   /* pcCountryDeliveryTarget */
   Free_DbvVisibleString( psAddDeliveryInfo->pcCountryDeliveryTarget );

   /* pfAlternativePostalDelivery */
   Free_DbvBool( psAddDeliveryInfo->pfAlternativePostalDelivery );
    
   DbvFree( psAddDeliveryInfo );
 
} /* End Free_DbvAddDeliveryInfo() */



void Free_DbvBillingBankInfo( DbvBillingBankInfo *psBillingBankInfo)
{
   if ( psBillingBankInfo == NULL )
      return;

   /* pcBankAccountHouse */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountHouse );

   /* pcBankAccountRoutingSymbol */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountRoutingSymbol );

   /* pcBankAccountNo */
   Free_DbvVisibleString( psBillingBankInfo->pcBankAccountNo );

   /* fBankAuthority ok */

   DbvFree( psBillingBankInfo );

} /* End Free_DbvBillingBankInfo */



void Free_DbvBillingCreditCardInfo( DbvBillingCreditCardInfo *psBillingCreditCardInfo)
{
   if ( psBillingCreditCardInfo == NULL )
      return;

   /* pcCardCompany */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardCompany );

   /* pcCardNo */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardNo );

   /* pcCardValidFrom */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardValidFrom );

   /* pcCardValidThru */
   Free_DbvVisibleString( psBillingCreditCardInfo->pcCardValidThru );


   DbvFree( psBillingCreditCardInfo );

} /* End Free_DbvBillingCreditCardInfo */



void Free_DbvAddBillingInfo( DbvAddBillingInfo *psAddBillingInfo )
{
   if ( psAddBillingInfo == NULL )
      return;

   switch( psAddBillingInfo->eAddBillingType )
      {
      case ABT_BankInfo:
	 Free_DbvBillingBankInfo( psAddBillingInfo->
			    uAddBillingInfo.psBillingBankInfo);
	 break;

      case ABT_CreditCardInfo:
	 Free_DbvBillingCreditCardInfo( psAddBillingInfo->
				uAddBillingInfo.psBillingCreditCardInfo);
	 break;

      case ABT_InvoiceType:

         /* nothing required here because no pointer */
	 break;

      case ABT_PrepaidType:
         Free_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcPrepaidId); 
	 break;

      case ABT_DepositType:
         Free_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcDepositId); 
	 break;


      default:
	 break;
      } /* End switch */

   DbvFree( psAddBillingInfo );
 
} /* End Free_DbvAddBillingInfo() */



 


void Free_DbvSubitoAdditionalInfo( DbvSubitoAdditionalInfo *psAddIllReqInfo )
{
   if ( psAddIllReqInfo == NULL )
      return;

   /* pfCustomerInfoToKeep */
   Free_DbvBool( psAddIllReqInfo->pfCustomerInfoToKeep );

   /* psAddRequesterInfo */
   Free_DbvAddRequesterInfo( psAddIllReqInfo->psAddRequesterInfo );

 
   /* psAddDeliveryInfo */
   Free_DbvAddDeliveryInfo( psAddIllReqInfo->psAddDeliveryInfo );

 
   /* psAddBillingInfo */
   Free_DbvAddBillingInfo( psAddIllReqInfo->psAddBillingInfo );

 

   DbvFree( psAddIllReqInfo );
   } /* End Free_DbvSubitoAdditionalInfo() */ 


void Free_DbvExtension( DbvExtension *psExtension )
   {

   if ( psExtension == NULL )
      return;

   /* pfCritical */
   Free_DbvBool( psExtension->pfCritical );

   /* uItem.psAddIllReqInfo */

   Free_DbvSubitoAdditionalInfo( psExtension->uItem.psAddIllReqInfo );

   DbvFree( psExtension );
   } /* End Free_DbvExtension() */ 


void Free_DbvIllRequest( DbvIllRequest *psIllRequest )
   {

   if ( psIllRequest == NULL )
      return;

   /* eProtocolVersion : No Free required */
   /* sTransactionId */
   Free_compulsory_DbvTransactionId( &psIllRequest->sTransactionId );

   /* sServiceDateTime */
   Free_compulsory_DbvServiceDateTime( &psIllRequest->sServiceDateTime );

   /* psRequesterId */
   Free_DbvSystemId( psIllRequest->psRequesterId );

   /* psResponderId */
   Free_DbvSystemId( psIllRequest->psResponderId );

   /* peTransactionType */
   Free_DbvTransactionType( psIllRequest->peTransactionType );

   /* psDeliveryAddress */
   Free_DbvDeliveryAddress( psIllRequest->psDeliveryAddress );

   /* psDeliveryService */
   Free_DbvDeliveryService( psIllRequest->psDeliveryService );

   /* psBillingAddress */
   Free_DbvDeliveryAddress( psIllRequest->psBillingAddress );

   /* ppeServiceType */
   FreeN_DbvServiceType( psIllRequest->ppeServiceType );

   /* psResponderSpecificService */
   FreeDbvASN1External( psIllRequest->psResponderSpecificService );

   /* sRequesterOptionalMessages : This is currently defined as a    */
   /* macro which converts to a null as the stucture has no pointers */
   Free_compulsory_DbvRequesterOptionalMessages( 
			&psIllRequest->sRequesterOptionalMessages );

   /* psSearchType */
   Free_DbvSearchType( psIllRequest->psSearchType );

   /* ppsSupplyMediumInfo */
   FreeN_DbvSupplyMediumInfo( psIllRequest->ppsSupplyMediumInfo );

   /* pePlaceOnHold */
   Free_DbvPlaceOnHold( psIllRequest->pePlaceOnHold );

   /* psClientId */
   Free_DbvClientId( psIllRequest->psClientId );

   /* sItemId */
   Free_compulsory_DbvItemId( &psIllRequest->sItemId );

   /* ppsSupplementalItemDescription */
   FreeN_DbvASN1External( psIllRequest->ppsSupplementalItemDescription );

   /* psCostInfo */
   Free_DbvCostInfo( psIllRequest->psCostInfo );

   /* psCopyrightCompliance */
   Free_DbvIllString( psIllRequest->psCopyrightCompliance );

   /* psThirdPartyInfo */
   Free_DbvThirdPartyInfo( psIllRequest->psThirdPartyInfo );

   /* pfRetry */
   Free_DbvBool( psIllRequest->pfRetry );

   /* pfForward */
   Free_DbvBool( psIllRequest->pfForward );

   /* psRequesterNote */
   Free_DbvIllString( psIllRequest->psRequesterNote );

   /* psForwardNote */
   Free_DbvIllString( psIllRequest->psForwardNote );

   /* ppsRequestExtension */
   FreeN_DbvExtension( psIllRequest->ppsRequestExtension );

   DbvFree( psIllRequest );
   } /* End Free_DbvIllRequest() */ 


void Free_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      {
      do_Free_DbvHistoryReport( psHistoryReport );
      DbvFree( psHistoryReport );
      } /* End if */

   } /* End Free_DbvHistoryReport() */ 

void do_Free_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   /* pcDateRequested  */
   Free_DbvVisibleString( psHistoryReport->pcDateRequested );

   /* psAuthor */
   Free_DbvIllString( psHistoryReport->psAuthor );

   /* psTitle */
   Free_DbvIllString( psHistoryReport->psTitle );

   /* psAuthorOfArticle */
   Free_DbvIllString( psHistoryReport->psAuthorOfArticle );

   /* psTitleOfArticle */
   Free_DbvIllString( psHistoryReport->psTitleOfArticle );

   /* pcDateOfLastTransition */
   Free_DbvVisibleString( psHistoryReport->pcDateOfLastTransition );

   /* pcDateOfMostRecentService */
   Free_DbvVisibleString( psHistoryReport->pcDateOfMostRecentService );

   /* sSystemId */
   Free_compulsory_DbvSystemId( &psHistoryReport->sSystemId );

   /* peShippedServiceType */
   Free_DbvShippedServiceType( psHistoryReport->peShippedServiceType );

   /* peTransactionResults */
   Free_DbvTransactionResults( psHistoryReport->peTransactionResults );

   /* psMostRecentServiceNote */
   Free_DbvIllString( psHistoryReport->psMostRecentServiceNote );

   } /* End do_Free_DbvHistoryReport() */ 

void Free_DbvStateTransitionProhibited( 
		DbvStateTransitionProhibited *psStateTransitionProhibited )
   {

   if ( psStateTransitionProhibited == NULL )
      return;

   DbvFree( psStateTransitionProhibited );
   } /* End Free_DbvStateTransitionProhibited() */ 


void Free_DbvAlreadyForwarded( DbvAlreadyForwarded *psAlreadyForwarded )
   {

   if ( psAlreadyForwarded == NULL )
      return;

   /* sResponderId */
   Free_compulsory_DbvSystemId( &psAlreadyForwarded->sResponderId );

   /* psResponderAddress */
   Free_DbvSystemAddress( psAlreadyForwarded->psResponderAddress );


   DbvFree( psAlreadyForwarded );
   } /* End Free_DbvAlreadyForwarded() */ 


void Free_DbvProviderErrorReport( 
			DbvProviderErrorReport *psProviderErrorReport )
   {

   if ( psProviderErrorReport == NULL )
      return;

   /* uProviderErrorReport */

   switch ( psProviderErrorReport->eReportType )
      {
      case PER_GeneralProblem:
      case PER_TransactionIdProblem:
	 break;

      case PER_StateTransitionProhibited:
	 Free_DbvStateTransitionProhibited( psProviderErrorReport->
					    uProviderErrorReport.
					    psStateTransitionProblem );
         break;

      default:
	 break;

      } /* End if */
   DbvFree( psProviderErrorReport );
   } /* End Free_DbvProviderErrorReport() */ 

void Free_DbvUserErrorReport( DbvUserErrorReport *psUserErrorReport )
   {

   if ( psUserErrorReport == NULL )
      return;

   /* uUserErrorReport */

   switch ( psUserErrorReport->eReportType )
      {
      case UER_AlreadyForwarded:
	 Free_DbvAlreadyForwarded( psUserErrorReport->
				   uUserErrorReport.
				   psAlreadyForwarded );
	 break;
      case UER_IntermediaryProblem:
	 break;

      case UER_SecurityProblem:
	 Free_DbvIllString( psUserErrorReport->
			    uUserErrorReport.
			    psSecurityProblem );
	 break;

      case UER_UnableToPerform:
      default:
	 break;

      } /* End if */
   DbvFree( psUserErrorReport );
   } /* End Free_DbvUserErrorReport() */ 

void Free_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      {
      do_Free_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );
      DbvFree( psUserOrProviderErrorReport );
      } /* End if */

   } /* End Free_DbvUserOrProviderErrorReport() */ 

void do_Free_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   /* uReport */

   switch ( psUserOrProviderErrorReport->eReportSource )
      {
      case RS_Provider:
	 Free_DbvProviderErrorReport( psUserOrProviderErrorReport->
				      uReport.psProviderErrorReport);
	 break;

      case RS_User:
	 Free_DbvUserErrorReport( psUserOrProviderErrorReport->
			    uReport.psUserErrorReport );
	 break;

      default:
	 break;

      } /* End if */

   } /* End do_Free_DbvUserOrProviderErrorReport() */ 

void Free_DbvErrorReport( DbvErrorReport *psErrorReport )
   {

   if ( psErrorReport == NULL )
      return;

   /* sCorrelationInfo */
   Free_compulsory_DbvIllString( &psErrorReport->sCorrelationInfo );

   /* sReport */
   Free_compulsory_DbvUserOrProviderErrorReport(  &psErrorReport->sReport );

   DbvFree( psErrorReport );
   } /* End Free_DbvErrorReport() */ 

void Free_DbvStatusReport( DbvStatusReport *psStatusReport )
   {

   if ( psStatusReport == NULL )
      return;

   /* sUserStatusReport */
   Free_compulsory_DbvHistoryReport( &psStatusReport->sUserStatusReport );

   DbvFree( psStatusReport );
   } /* End Free_DbvStatusReport() */ 

void Free_DbvStatusOrErrorReport( 
			DbvStatusOrErrorReport *psStatusOrErrorReport )
   {

   if ( psStatusOrErrorReport == NULL )
      return;

   /* eProtocolVersion */

   /* sTransactionId */
   Free_compulsory_DbvTransactionId( &psStatusOrErrorReport->sTransactionId );

   /* sServiceDateTime */
   Free_compulsory_DbvServiceDateTime( 
				&psStatusOrErrorReport->sServiceDateTime );
   /* psRequesterId */
   Free_DbvSystemId( psStatusOrErrorReport->psRequesterId );

   /* psResponderId */
   Free_DbvSystemId( psStatusOrErrorReport->psResponderId );

   /* peReasonNoReport */
   Free_DbvReasonNoReport( psStatusOrErrorReport->peReasonNoReport );

   /* psStatusReport */
   Free_DbvStatusReport( psStatusOrErrorReport->psStatusReport );

   /* psErrorReport */
   Free_DbvErrorReport( psStatusOrErrorReport->psErrorReport );

   /* psNote */
   Free_DbvIllString( psStatusOrErrorReport->psNote );

   /* ppsStOrErReportExtension */
   FreeN_DbvExtension( psStatusOrErrorReport->ppsStOrErReportExtension );

   DbvFree( psStatusOrErrorReport );
   } /* End Free_DbvStatusOrErrorReport() */ 


void Free_DbvIllPdu( DbvIllPdu *psIllPdu )
   {

   if ( psIllPdu == NULL )
      return;

   /* uIllPdu */

   switch( psIllPdu->eIllPduType )
      {
      case ILL_IllRequest:
	 Free_DbvIllRequest( psIllPdu->uIllPdu.psIllRequest );
	 break;

      case ILL_StatusOrErrorReport:
	 Free_DbvStatusOrErrorReport( psIllPdu->
				      uIllPdu.psStatusOrErrorReport);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psIllPdu );
   } /* End Free_DbvIllPdu() */ 

void Free_DbvBool( DbvBool *peBool )
   {

   if ( peBool == NULL )
      return;

   DbvFree( peBool );
   } /* End Free_DbvBool() */ 

void Free_DbvPreference( DbvPreference *pePreference )
   {

   if ( pePreference == NULL )
      return;

   DbvFree( pePreference );
   } /* End Free_DbvPreference() */ 

void Free_DbvTransactionType( DbvTransactionType *peTransactionType )
   {

   if ( peTransactionType == NULL )
      return;

   DbvFree( peTransactionType );
   } /* End Free_DbvTransactionType() */ 

void Free_DbvReasonNoReport( DbvReasonNoReport *peReasonNoReport )
   {

   if ( peReasonNoReport == NULL )
      return;

   DbvFree( peReasonNoReport );
   } /* End Free_DbvReasonNoReport() */ 

void Free_DbvPrintableString( DbvPrintableString pcPrintableString )
   {

   if ( pcPrintableString == NULL )
      return;

   DbvFree( pcPrintableString );
   } /* End Free_DbvPrintableString() */ 

void Free_DbvItemType( DbvItemType *peItemType )
   {

   if ( peItemType == NULL )
      return;

   DbvFree( peItemType );
   } /* End Free_DbvItemType() */ 

void Free_DbvExpiry( DbvExpiry *peExpiry )
   {

   if ( peExpiry == NULL )
      return;

   DbvFree( peExpiry );
   } /* End Free_DbvExpiry() */ 

void Free_DbvMediumType( DbvMediumType *peMediumType )
   {

   if ( peMediumType == NULL )
      return;

   DbvFree( peMediumType );
   } /* End Free_DbvMediumType() */ 

void Free_DbvServiceType( DbvServiceType *peServiceType )
   {

   if ( peServiceType == NULL )
      return;

   DbvFree( peServiceType );
   } /* End Free_DbvServiceType() */ 

void Free_DbvPlaceOnHold( DbvPlaceOnHold *pePlaceOnHold )
   {

   if ( pePlaceOnHold == NULL )
      return;

   DbvFree( pePlaceOnHold );
   } /* End Free_DbvPlaceOnHold() */ 

void Free_DbvShippedServiceType( DbvShippedServiceType *peShippedServiceType )
   {

   if ( peShippedServiceType == NULL )
      return;

   DbvFree( peShippedServiceType );
   } /* End Free_DbvShippedServiceType() */ 

void Free_DbvTransactionResults( DbvTransactionResults *peTransactionResults )
   {

   if ( peTransactionResults == NULL )
      return;

   DbvFree( peTransactionResults );
   } /* End Free_DbvTransactionResults() */ 

void Free_DbvEsIoAuxiliaryStatus( 
			DbvEsIoAuxiliaryStatus *peEsIoAuxiliaryStatus )
   {

   if ( peEsIoAuxiliaryStatus == NULL )
      return;

   DbvFree( peEsIoAuxiliaryStatus );
   } /* End Free_DbvEsIoAuxiliaryStatus() */ 

void Free_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod == NULL )
      {
      do_Free_DbvEsIoPaymentMethod( psEsIoPaymentMethod );
      DbvFree( psEsIoPaymentMethod );
      } /* End if */

   } /* End Free_DbvEsIoPaymentMethod() */ 


void do_Free_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod->puPaymentMethod )
      {
      switch( psEsIoPaymentMethod->ePaymentMethodType )
         {
         case IOPM_CreditCard:
	    Free_DbvEsIoCreditCardInfo( psEsIoPaymentMethod->
				       puPaymentMethod->psCreditCardInfo );
	    break;
   
         case IOPM_PrivateNotKnown:
	    FreeDbvASN1External( psEsIoPaymentMethod->
				 puPaymentMethod->psPrivateNotKnown );
	    break;
   
	 case IOPM_BillInvoice:
	 case IOPM_Prepay:
	 case IOPM_DepositAccount:
	 case IOPM_CardInfoPrevSupplied:
	 case IOPM_PrivateKnown:
	 case IOPM_Invalid:
         default:
	    break;
         } /* End switch */
      } /* End if */

   } /* End do_Free_DbvEsIoPaymentMethod() */ 


void Free_DbvEsIoAdditionalBilling( 
			DbvEsIoAdditionalBilling *psEsIoAdditionalBilling )
   {

   if ( psEsIoAdditionalBilling == NULL )
      return;

   /* sPaymentMethod */

   Free_compulsory_DbvEsIoPaymentMethod( 
				&psEsIoAdditionalBilling->sPaymentMethod );
   /* psCustomerReference */
   if ( psEsIoAdditionalBilling->psCustomerReference != NULL )
      FreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerReference);

   /* psCustomerPONumber */
   if ( psEsIoAdditionalBilling->psCustomerPONumber != NULL )
      FreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerPONumber);

   DbvFree( psEsIoAdditionalBilling );
   } /* End Free_DbvEsIoAdditionalBilling() */ 


void Free_DbvEsIoContact( DbvEsIoContact *psEsIoContact )
   {

   if ( psEsIoContact == NULL )
      return;

   /* psName */
   if ( psEsIoContact->psName != NULL )
      FreeDbvInternationalString( psEsIoContact->psName );

   /* psPhone */
   if ( psEsIoContact->psPhone != NULL )
      FreeDbvInternationalString( psEsIoContact->psPhone );

   /* psEmail */
   if ( psEsIoContact->psEmail != NULL )
      FreeDbvInternationalString( psEsIoContact->psEmail );

   DbvFree( psEsIoContact );
   } /* End Free_DbvEsIoContact() */ 

void Free_DbvEsIoOriginPartToKeep( 
			DbvEsIoOriginPartToKeep *psEsIoOriginPartToKeep )
   {

   if ( psEsIoOriginPartToKeep == NULL )
      return;

   /* psSupplementalDescription  */
   FreeDbvASN1External( psEsIoOriginPartToKeep->psSupplementalDescription );

   /* psContact */
   Free_DbvEsIoContact( psEsIoOriginPartToKeep->psContact );

   /* psAdditionalBilling */
   Free_DbvEsIoAdditionalBilling( psEsIoOriginPartToKeep->psAdditionalBilling );

   DbvFree( psEsIoOriginPartToKeep );
   } /* End Free_DbvEsIoOriginPartToKeep() */ 

void Free_DbvEsIoRequest( DbvEsIoRequest *psEsIoRequest )
   {

   if ( psEsIoRequest == NULL )
      return;

   /* psToKeep */
   Free_DbvEsIoOriginPartToKeep( psEsIoRequest->psToKeep );

   /* sNotToKeep */
   Free_compulsory_DbvEsIoOriginPartNotToKeep( &psEsIoRequest->sNotToKeep );

   DbvFree( psEsIoRequest );
   } /* End Free_DbvEsIoRequest() */ 

void Free_DbvEsIoTaskPackage( DbvEsIoTaskPackage *psEsIoTaskPackage )
   {

   if ( psEsIoTaskPackage == NULL )
      return;

   /* psOriginPart */
   Free_DbvEsIoOriginPartToKeep( psEsIoTaskPackage->psOriginPart );

   /* sTargetPart */
   Free_compulsory_DbvEsIoTargetPart( &psEsIoTaskPackage->sTargetPart );

   DbvFree( psEsIoTaskPackage );
   } /* End Free_DbvEsIoTaskPackage() */ 

void Free_DbvItemOrderExtendedService( 
		DbvItemOrderExtendedService *psItemOrderExtendedService )
   {

   if ( psItemOrderExtendedService == NULL )
      return;

   /* uItemOrderReqOrRsp */

   switch( psItemOrderExtendedService->eItemOrderReqRsp )
      {
      case IOROR_Request:
	 Free_DbvEsIoRequest( psItemOrderExtendedService->
			      uItemOrderReqOrRsp.psItemOrderRequest);
	 break;

      case IOROR_Response:
	 Free_DbvEsIoTaskPackage( psItemOrderExtendedService->
				  uItemOrderReqOrRsp.psItemOrderResponse);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psItemOrderExtendedService );
   } /* End Free_DbvItemOrderExtendedService() */ 




void Free_DbvCommonInfo( DbvCommonInfo *psCommonInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCommonInfo()\n" );
#endif

   if ( psCommonInfo == NULL )
      return;

   /* pcDateAdded */
   Free_DbvGeneralizedTime( psCommonInfo->pcDateAdded );

   /* pcDateChanged */
   Free_DbvGeneralizedTime( psCommonInfo->pcDateChanged );

   /* pcExpiry */
   Free_DbvGeneralizedTime( psCommonInfo->pcExpiry );

   /* psHumanStringLanguage */
   Free_DbvLanguageCode( psCommonInfo->psHumanStringLanguage );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psCommonInfo->ppsOtherInformation );

   DbvFree( psCommonInfo );
   } /* End Free_DbvCommonInfo() */ 

void Free_DbvHumanString( DbvHumanString *psHumanString )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvHumanString()\n" );
#endif

   if ( psHumanString == NULL )
      return;

   /* psLanguage */
   Free_DbvLanguageCode( psHumanString->psLanguage );

   /* sText */
   Free_compulsory_DbvInternationalString( &psHumanString->sText );

   DbvFree( psHumanString );
   } /* End Free_DbvHumanString() */ 

void Free_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvBodyType()\n" );
#endif

   if ( psBodyType != NULL )
      {
      do_Free_DbvBodyType( psBodyType );
      DbvFree( psBodyType );
      } /* End if */

   } /* End Free_DbvBodyType() */ 

void do_Free_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUGZ
   printf( "D - do_Free_DbvBodyType()\n" );
#endif

   if ( psBodyType == NULL )
      return;

   /* uBodyTypeType */

   switch( psBodyType->eBodyTypeType )
      {
      case BTT_IanaType:
	 FreeDbvInternationalString( psBodyType->uBodyType.psIanaType );
	 break;

      case BTT_Z3950Type:
	 FreeDbvInternationalString( psBodyType->uBodyType.psZ3950Type );
	 break;

      case BTT_OtherType:
	 FreeDbvInternationalString( psBodyType->uBodyType.psOtherType );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_DbvBodyType() */ 

void Free_DbvIconObject( DbvIconObject *psIconObject )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIconObject()\n" );
#endif

   if ( psIconObject == NULL )
      return;

   /* sBodyType */
   Free_compulsory_DbvBodyType( &psIconObject->sBodyType );

   /* sContent */
   Free_compulsory_DbvOctetString( &psIconObject->sContent );

   DbvFree( psIconObject );
   } /* End Free_DbvIconObject() */ 

void Free_DbvContactInfo( DbvContactInfo *psContactInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvContactInfo()\n" );
#endif

   if ( psContactInfo == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psContactInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psContactInfo->ppsDescription );

   /* ppsAddress */
   FreeN_DbvHumanString( psContactInfo->ppsAddress );

   /* psEmail */
   FreeDbvInternationalString( psContactInfo->psEmail );

   /* psPhone */
   FreeDbvInternationalString( psContactInfo->psPhone );

   DbvFree( psContactInfo );
   } /* End Free_DbvContactInfo() */ 

void Free_DbvDatabaseList( DbvDatabaseList *psDatabaseList )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvDatabaseList()\n" );
#endif

   if ( psDatabaseList == NULL )
      return;

   /* ppsInnerDatabases */
   FreeN_DbvInternationalString( psDatabaseList->ppsInnerDatabases );

   DbvFree( psDatabaseList );
   } /* End Free_DbvDatabaseList() */ 

void Free_DbvInterAddress( DbvInterAddress *psInterAddress )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvInterAddress()\n" );
#endif

   if ( psInterAddress == NULL )
      return;

   /* sHostAddress */
   Free_compulsory_DbvInternationalString( &psInterAddress->sHostAddress );

   DbvFree( psInterAddress );
   } /* End Free_DbvInterAddress() */ 

void Free_DbvOsiPresentationAddress( 
		DbvOsiPresentationAddress *psOsiPresentationAddress )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOsiPresentationAddress()\n" );
#endif

   if ( psOsiPresentationAddress == NULL )
      return;

   /* sPsel */
   Free_compulsory_DbvInternationalString( &psOsiPresentationAddress->sPsel );

   /* psSSel */
   FreeDbvInternationalString( psOsiPresentationAddress->psSSel );

   /* psTSel */
   FreeDbvInternationalString( psOsiPresentationAddress->psTSel );

   /* sNSap */
   Free_compulsory_DbvInternationalString( &psOsiPresentationAddress->sNSap );

   DbvFree( psOsiPresentationAddress );
   } /* End Free_DbvOsiPresentationAddress() */ 

void Free_DbvOtherAddress( DbvOtherAddress *psOtherAddress )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOtherAddress()\n" );
#endif

   if ( psOtherAddress == NULL )
      return;

   /* sType */
   Free_compulsory_DbvInternationalString( &psOtherAddress->sType );

   /* sAddress */
   Free_compulsory_DbvInternationalString( &psOtherAddress->sAddress );

   DbvFree( psOtherAddress );
   } /* End Free_DbvOtherAddress() */ 

void Free_DbvNetworkAddress( DbvNetworkAddress *psNetworkAddress )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvNetworkAddress()\n" );
#endif

   if ( psNetworkAddress == NULL )
      return;

   /* uNetworkAddressType */

   switch( psNetworkAddress->eNetworkAddressType )
      {
      case NAT_InternetAddress:
	 Free_DbvInterAddress( psNetworkAddress->
				uNetworkAddress.psInternetAddress );
	 break;

      case NAT_OsiPresentationAddress:
	 Free_DbvOsiPresentationAddress( psNetworkAddress->
				uNetworkAddress.psOsiPresentationAddress );
	 break;

      case NAT_OtherAddress:
	 Free_DbvOtherAddress( psNetworkAddress->
				uNetworkAddress.psOtherAddress );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNetworkAddress );
   } /* End Free_DbvNetworkAddress() */ 

void Free_DbvOperators( DbvOperators *psOperators )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOperators()\n" );
#endif

   if ( psOperators == NULL )
      return;

   /* sOperator */
   Free_compulsory_DbvInternationalString( &psOperators->sOperator );

   /* ppsDescription */
   FreeN_DbvHumanString( psOperators->ppsDescription );

   DbvFree( psOperators );
   } /* End Free_DbvOperators() */ 

void Free_DbvSearchKey( DbvSearchKey *psSearchKey )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSearchKey()\n" );
#endif

   if ( psSearchKey == NULL )
      return;

   /* sSearchKey */
   Free_compulsory_DbvInternationalString( &psSearchKey->sSearchKey );

   /* ppsDescription */
   FreeN_DbvHumanString( psSearchKey->ppsDescription );

   DbvFree( psSearchKey );
   } /* End Free_DbvSearchKey() */ 


void Free_DbvHumanStringSeq( DbvHumanStringSeq *psDescription )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvHumanStringSeq()\n" );
#endif

   if ( psDescription == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psDescription->ppsHumanString );

   DbvFree( psDescription );
   } /* End Free_DbvHumanStringSeq() */ 



void Free_DbvPrivateCapabilities( 
				DbvPrivateCapabilities *psPrivateCapabilities )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPrivateCapabilities()\n" );
#endif

   if ( psPrivateCapabilities == NULL )
      return;

   /* ppsOperators */
   FreeN_DbvOperators( psPrivateCapabilities->ppsOperators );

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psPrivateCapabilities->ppsSearchKeys );

   /* ppsDescription */
   FreeN_DbvHumanStringSeq( psPrivateCapabilities->ppsDescription );

   DbvFree( psPrivateCapabilities );
   } /* End Free_DbvPrivateCapabilities() */ 

void Free_DbvPrivate( DbvPrivate *psPrivate )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPrivate()\n" );
#endif

   if ( psPrivate == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psPrivate->ppsDescription );

   DbvFree( psPrivate );
   } /* End Free_DbvPrivate() */ 

void Free_DbvUnitsSupported( DbvUnitsSupported *psUnitsSupported )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvUnitsSupported()\n" );
#endif

   if ( psUnitsSupported == NULL )
      return;

   /* uUnitsSupportedType */

   switch( psUnitsSupported->eUnitsSupportedType )
      {
      case UST_Private:
	 Free_DbvPrivate( psUnitsSupported->uUnitsSupported.psPrivate );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUnitsSupported );
   } /* End Free_DbvUnitsSupported() */ 

void Free_DbvProximitySupport( DbvProximitySupport *psProximitySupport )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvProximitySupport()\n" );
#endif

   if ( psProximitySupport == NULL )
      return;

   FreeN_DbvUnitsSupported( psProximitySupport->ppsUnitsSupported );

   DbvFree( psProximitySupport );
   } /* End Free_DbvProximitySupport() */ 

void Free_DbvRpnCapabilities( DbvRpnCapabilities *psRpnCapabilities )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvRpnCapabilities()\n" );
#endif

   if ( psRpnCapabilities == NULL )
      return;

   /* ppulOperators */
   FreeN_Ulong( psRpnCapabilities->ppulOperators );

   /* psProximity */
   Free_DbvProximitySupport( psRpnCapabilities->psProximity );

   DbvFree( psRpnCapabilities );
   } /* End Free_DbvRpnCapabilities() */ 

void Free_DbvIso8777Capabilities(
			DbvIso8777Capabilities *psIso8777Capabilities )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIso8777Capabilities()\n" );
#endif

   if ( psIso8777Capabilities == NULL )
      return;

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psIso8777Capabilities->ppsSearchKeys );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psIso8777Capabilities->ppsRestrictions );

   DbvFree( psIso8777Capabilities );
   } /* End Free_DbvIso8777Capabilities() */ 

void Free_DbvQueryTypeDetails( DbvQueryTypeDetails *psQueryTypeDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvQueryTypeDetails()\n" );
#endif

   if ( psQueryTypeDetails == NULL )
      return;

   /* uQueryTypeDetailsType */

   switch( psQueryTypeDetails->eQueryTypeDetailsType )
      {
      case QTDT_Private:
	 Free_DbvPrivateCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psPrivate );
	 break;

      case QTDT_Rpn:
	 Free_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psRpn );
	 break;

      case QTDT_Iso8777:
	 Free_DbvIso8777Capabilities( psQueryTypeDetails->
				uQueryTypeDetails.psIso8777 );
	 break;

      case QTDT_Z39_58:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsZ39_58 );
	 break;

      case QTDT_Erpn:
	 Free_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psErpn );
	 break;

      case QTDT_RankedList:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsRankedList );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQueryTypeDetails );
   } /* End Free_DbvQueryTypeDetails() */ 

void Free_DbvAccessRestrictions( DbvAccessRestrictions *psAccessRestrictions )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAccessRestrictions()\n" );
#endif

   if ( psAccessRestrictions == NULL )
      return;

   /* ppsAccessText */
   FreeN_DbvHumanString( psAccessRestrictions->ppsAccessText );

   /* ppcAccessChallenges */
   FreeN_DbvObjectIdentifier( psAccessRestrictions->ppcAccessChallenges );

   DbvFree( psAccessRestrictions );
   } /* End Free_DbvAccessRestrictions() */ 

void Free_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCharge()\n" );
#endif

   if ( psCharge != NULL )
      {
      do_Free_DbvCharge( psCharge );
      DbvFree( psCharge );
      } /* End if */

   } /* End Free_DbvCharge() */ 

void do_Free_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUGZ
   printf( "D - do_Free_DbvCharge()\n" );
#endif

   /* sCost */
   Free_compulsory_DbvIntUnit( &psCharge->sCost );

   /* psPerWhat */
   Free_DbvUnit( psCharge->psPerWhat );

   /* ppsText */
   FreeN_DbvHumanString( psCharge->ppsText );

   } /* End do_Free_DbvCharge() */ 

void Free_DbvOtherCharge( DbvOtherCharge *psOtherCharge )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOtherCharge()\n" );
#endif

   if ( psOtherCharge == NULL )
      return;

   /* ppsForWhat */
   FreeN_DbvHumanString( psOtherCharge->ppsForWhat );

   /* sCharge */
   Free_compulsory_DbvCharge( &psOtherCharge->sCharge );

   DbvFree( psOtherCharge );
   } /* End Free_DbvOtherCharge() */ 

void Free_DbvCosts( DbvCosts *psCosts )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCosts()\n" );
#endif

   if ( psCosts == NULL )
      return;

   /* ppsForWhat */
   Free_DbvCharge( psCosts->psConnectCharge );

   /* psConnectTime */
   Free_DbvCharge( psCosts->psConnectTime );

   /* psDisplayCharge */
   Free_DbvCharge( psCosts->psDisplayCharge );

   /* psSearchCharge */
   Free_DbvCharge( psCosts->psSearchCharge );

   /* psSubscriptCharge */
   Free_DbvCharge( psCosts->psSubscriptCharge );

   /* ppsOtherCharge */
   FreeN_DbvOtherCharge( psCosts->ppsOtherCharge );

   DbvFree( psCosts );
   } /* End Free_DbvCosts() */ 

void Free_DbvAccessInfo( DbvAccessInfo *psAccessInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAccessInfo()\n" );
#endif

   if ( psAccessInfo == NULL )
      return;

   /* ppsQueryTypesSupported */
   FreeN_DbvQueryTypeDetails( psAccessInfo->ppsQueryTypesSupported );

   /* ppcDiagnosticsSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcDiagnosticsSets );

   /* ppcAttributeSetIds */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcAttributeSetIds );

   /* ppcSchemas */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcSchemas );

   /* ppcRecordSyntaxes */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcRecordSyntaxes );

   /* ppcResourceChallenges */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcResourceChallenges );

   /* ppsRestrictedAccess */
   FreeN_DbvAccessRestrictions( psAccessInfo->ppsRestrictedAccess );

   /* psCostInfo */
   Free_DbvCosts( psAccessInfo->psCostInfo );

   /* ppcVariantSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcVariantSets );

   /* ppsElementSetNames */
   FreeN_DbvInternationalString( psAccessInfo->ppsElementSetNames );

   /* ppsUnitSystems */
   FreeN_DbvInternationalString( psAccessInfo->ppsUnitSystems );

   DbvFree( psAccessInfo );
   } /* End Free_DbvAccessInfo() */ 

void Free_DbvTargetInfo( DbvTargetInfo *psTargetInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTargetInfo()\n" );
#endif

   if ( psTargetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTargetInfo->psCommonInfo );

   /* sName */
   Free_compulsory_DbvInternationalString( &psTargetInfo->sName );

   /* ppsRecentNews */
   FreeN_DbvHumanString( psTargetInfo->ppsRecentNews );

   /* ppsIcon */
   FreeN_DbvIconObject( psTargetInfo->ppsIcon );

   /* pulMaxResultSets */
   Free_Ulong( psTargetInfo->pulMaxResultSets );

   /* pulMaxResultSize */
   Free_Ulong( psTargetInfo->pulMaxResultSize );

   /* pulMaxTerms */
   Free_Ulong( psTargetInfo->pulMaxTerms );

   /* psTimeoutInterval */
   Free_DbvIntUnit( psTargetInfo->psTimeoutInterval );

   /* ppsWelcomeMessage */
   FreeN_DbvHumanString( psTargetInfo->ppsWelcomeMessage );

   /* psContactInfo */
   Free_DbvContactInfo( psTargetInfo->psContactInfo );

   /* ppsDescription */
   FreeN_DbvHumanString( psTargetInfo->ppsDescription );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psTargetInfo->ppsNicknames );

   /* ppsUsageRest */
   FreeN_DbvHumanString( psTargetInfo->ppsUsageRest );

   /* ppsPaymentAddr */
   FreeN_DbvHumanString( psTargetInfo->ppsPaymentAddr );

   /* ppsHours */
   FreeN_DbvHumanString( psTargetInfo->ppsHours  );

   /* ppsDbCombinations */
   FreeN_DbvDatabaseList( psTargetInfo->ppsDbCombinations );

   /* ppsAddresses */
   FreeN_DbvNetworkAddress( psTargetInfo->ppsAddresses );

   /* ppsLanguages */
   FreeN_DbvInternationalString( psTargetInfo->ppsLanguages );

   /* psCommonAccessInfo */
   Free_DbvAccessInfo( psTargetInfo->psCommonAccessInfo );

   DbvFree( psTargetInfo );
   } /* End Free_DbvTargetInfo() */ 

void Free_DbvRecordCount( DbvRecordCount *psRecordCount )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvRecordCount()\n" );
#endif

   if ( psRecordCount == NULL )
      return;

   DbvFree( psRecordCount );
   } /* End Free_DbvRecordCount() */ 

void Free_DbvDatabaseInfo( DbvDatabaseInfo *psDatabaseInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvDatabaseInfo()\n" );
#endif

   if ( psDatabaseInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psDatabaseInfo->psCommonInfo );

   /* sName */
   Free_compulsory_DbvInternationalString( &psDatabaseInfo->sName );

   /* pvExplainDatabase */
   Free_DbvNull( psDatabaseInfo->pvExplainDatabase );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsNicknames );

   /* ppsIcon */
   FreeN_DbvIconObject( psDatabaseInfo->ppsIcon );

   /* ppsTitleString */
   FreeN_DbvHumanString( psDatabaseInfo->ppsTitleString );

   /* ppsKeywords */
   FreeN_DbvDataBaseInfoSeq( psDatabaseInfo->ppsKeywords );

   /* ppsDescription */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDescription );

   /* ppsAssociatedDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsAssociatedDbs );

   /* ppsSubDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsSubDbs );

   /* ppsDisclaimers */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDisclaimers );

   /* ppsNews */
   FreeN_DbvHumanString( psDatabaseInfo->ppsNews );

   /* psRecordCount */
   Free_DbvRecordCount( psDatabaseInfo->psRecordCount );

   /* ppsDefaultOrder */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDefaultOrder );

   /* pulAvRecordSize */
   Free_Ulong( psDatabaseInfo->pulAvRecordSize );

   /* pulMaxRecordSize */
   Free_Ulong( psDatabaseInfo->pulMaxRecordSize );

   /* ppsHours */
   FreeN_DbvHumanString( psDatabaseInfo->ppsHours );

   /* ppsBestTime */
   FreeN_DbvHumanString( psDatabaseInfo->ppsBestTime );

   /* pcLastUpdate */
   Free_DbvGeneralizedTime( psDatabaseInfo->pcLastUpdate );

   /* psUpdateInterval */
   Free_DbvIntUnit( psDatabaseInfo->psUpdateInterval );

   /* ppsCoverage */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCoverage );

   /* pfProprietary */
   Free_DbvBool( psDatabaseInfo->pfProprietary );

   /* ppsCopyrightText */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightText );

   /* ppsCopyrightNotice */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightNotice );

   /* psProducerContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psProducerContactInfo );

   /* psSupplierContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psSupplierContactInfo );

   /* psSubmissionContactInfo */
   Free_DbvContactInfo( psDatabaseInfo->psSubmissionContactInfo );

   /* psAccessInfo */
   Free_DbvAccessInfo( psDatabaseInfo->psAccessInfo );

   DbvFree( psDatabaseInfo );
   } /* End Free_DbvDatabaseInfo() */ 

void Free_DbvTagTypeMapping( DbvTagTypeMapping *psTagTypeMapping )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTagTypeMapping()\n" );
#endif

   if ( psTagTypeMapping == NULL )
      return;

   /* pcTagSet */
   FreeDbvObjectIdentifier( psTagTypeMapping->pcTagSet );

   /* pvDefaultTagType */
   Free_DbvNull( psTagTypeMapping->pvDefaultTagType );

   DbvFree( psTagTypeMapping );
   } /* End Free_DbvTagTypeMapping() */ 

void Free_DbvPath( DbvPath *psPath )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPath()\n" );
#endif

   if ( psPath == NULL )
      return;

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psPath->sTagValue );

   DbvFree( psPath );
   } /* End Free_DbvPath() */ 

void Free_DbvPathA( DbvPathA *psPathA )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPathA()\n" );
#endif

   if ( psPathA == NULL )
      return;

   /* ppsSchemaTags */
   FreeN_DbvPath( psPathA->ppsSchemaTags );

   DbvFree( psPathA );
   } /* End Free_DbvPathA() */ 

void Free_DbvElementInfo( DbvElementInfo *psElementInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvElementInfo()\n" );
#endif

   if ( psElementInfo == NULL )
      return;

   /* sElementName */
   Free_compulsory_DbvInternationalString( &psElementInfo->sElementName );

   /* ppsElementTagPath */
   FreeN_DbvPath( psElementInfo->ppsElementTagPath );

   /* psDataType */
   Free_DbvElementDataType( psElementInfo->psDataType );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementInfo->ppsDescription );

   DbvFree( psElementInfo );
   } /* End Free_DbvElementInfo() */ 

void Free_DbvElementDataType( DbvElementDataType *psElementDataType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvElementDataType()\n" );
#endif

   if ( psElementDataType == NULL )
      return;

   /* uElementDataTypeType */

   switch( psElementDataType->eElementDataTypeType )
      {
      case EDTT_Primitive:
	 break;

      case EDTT_Structured:
	 FreeN_DbvElementInfo( psElementDataType->
				uElementDataType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementDataType );
   } /* End Free_DbvElementDataType() */ 

void Free_DbvSchemaInfo( DbvSchemaInfo *psSchemaInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSchemaInfo()\n" );
#endif

   if ( psSchemaInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psSchemaInfo->psCommonInfo );

   /* pcSchema */
   FreeDbvObjectIdentifier( psSchemaInfo->pcSchema );

   /* sName */
   Free_compulsory_DbvInternationalString( &psSchemaInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psSchemaInfo->ppsDescription );

   /* ppsTagTypeMapping */
   FreeN_DbvTagTypeMapping( psSchemaInfo->ppsTagTypeMapping );

   /* ppsRecordStructure */
   FreeN_DbvElementInfo( psSchemaInfo->ppsRecordStructure );

   DbvFree( psSchemaInfo );
   } /* End Free_DbvSchemaInfo() */ 

void Free_DbvElements( DbvElements *psElements )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvElements()\n" );
#endif

   if ( psElements == NULL )
      return;

   /* sElementName */
   Free_compulsory_DbvInternationalString( &psElements->sElementName );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psElements->ppsNicknames );

   /* sElementTag */
   Free_compulsory_DbvStringOrNumeric( &psElements->sElementTag );

   /* ppsDescription */
   FreeN_DbvHumanString( psElements->ppsDescription );

   /* peDataType */
   Free_DbvPrimitiveDataType( psElements->peDataType );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psElements->ppsOtherInformation );

   DbvFree( psElements );
   } /* End Free_DbvElements() */ 

void Free_DbvPrimitiveDataType( DbvPrimitiveDataType *pePrimitiveDataType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPrimitiveDataType()\n" );
#endif

   if ( pePrimitiveDataType == NULL )
      return;

   DbvFree( pePrimitiveDataType );
   } /* End Free_DbvPrimitiveDataType() */ 

void Free_DbvTagSetInfo( DbvTagSetInfo *psTagSetInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTagSetInfo()\n" );
#endif

   if ( psTagSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTagSetInfo->psCommonInfo );

   /* pcTagSet */
   FreeDbvObjectIdentifier( psTagSetInfo->pcTagSet );

   /* sName */
   Free_compulsory_DbvInternationalString( &psTagSetInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTagSetInfo->ppsDescription );

   /* ppsElements */
   FreeN_DbvElements( psTagSetInfo->ppsElements );

   DbvFree( psTagSetInfo );
   } /* End Free_DbvTagSetInfo() */ 

void Free_DbvRecordSyntaxInfo( DbvRecordSyntaxInfo *psRecordSyntaxInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvRecordSyntaxInfo()\n" );
#endif

   if ( psRecordSyntaxInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psRecordSyntaxInfo->psCommonInfo );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psRecordSyntaxInfo->pcRecordSyntax );

   /* sName */
   Free_compulsory_DbvInternationalString( &psRecordSyntaxInfo->sName );

   /* ppcTransferSyntaxes */
   FreeN_DbvObjectIdentifier( psRecordSyntaxInfo->ppcTransferSyntaxes );

   /* ppsDescription */
   FreeN_DbvHumanString( psRecordSyntaxInfo->ppsDescription );

   /* psAsn1Module */
   FreeDbvInternationalString( psRecordSyntaxInfo->psAsn1Module );

   /* ppsAbstractStructure */
   FreeN_DbvElementInfo( psRecordSyntaxInfo->ppsAbstractStructure );

   DbvFree( psRecordSyntaxInfo );
   } /* End Free_DbvRecordSyntaxInfo() */ 

void Free_DbvAttributeDescription( 
		DbvAttributeDescription *psAttributeDescription )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeDescription()\n" );
#endif

   if ( psAttributeDescription == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psAttributeDescription->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeDescription->ppsDescription );

   /* sAttributeValue */
   Free_compulsory_DbvStringOrNumeric( 
				&psAttributeDescription->sAttributeValue );

   /* ppsEquivalentAttributes */
   FreeN_DbvStringOrNumeric( psAttributeDescription->ppsEquivalentAttributes );

   DbvFree( psAttributeDescription );
   } /* End Free_DbvAttributeDescription() */ 

void Free_DbvAttributeType( DbvAttributeType *psAttributeType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeType()\n" );
#endif

   if ( psAttributeType == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psAttributeType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeType->ppsDescription );

   /* ppsAttributeValues */
   FreeN_DbvAttributeDescription( psAttributeType->ppsAttributeValues );

   DbvFree( psAttributeType );
   } /* End Free_DbvAttributeType() */ 

void Free_DbvAttributeSetInfo( DbvAttributeSetInfo *psAttributeSetInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeSetInfo()\n" );
#endif

   if ( psAttributeSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psAttributeSetInfo->psCommonInfo );

   /* pcAttributeSetId */
   FreeDbvObjectIdentifier( psAttributeSetInfo->pcAttributeSetId );

   /* sName */
   Free_compulsory_DbvInternationalString( &psAttributeSetInfo->sName );

   /* ppsAttributes */
   FreeN_DbvAttributeType( psAttributeSetInfo->ppsAttributes );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeSetInfo->ppsDescription );

   DbvFree( psAttributeSetInfo );
   } /* End Free_DbvAttributeSetInfo() */ 

void Free_DbvTermList( DbvTermList *psTermList )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTermList()\n" );
#endif

   if ( psTermList == NULL )
      return;

   /* sName */
   Free_compulsory_DbvInternationalString( &psTermList->sName );

   /* ppsTitle */
   FreeN_DbvHumanString( psTermList->ppsTitle );

   /* peSearchCost */
   Free_DbvSearchCost( psTermList->peSearchCost );

   /* ppsBroader */
   FreeN_DbvInternationalString( psTermList->ppsBroader );

   /* ppsNarrower */
   FreeN_DbvInternationalString( psTermList->ppsNarrower );

   DbvFree( psTermList );
   } /* End Free_DbvTermList() */ 

void Free_DbvTermListInfo( DbvTermListInfo *psTermListInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTermListInfo()\n" );
#endif

   if ( psTermListInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTermListInfo->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psTermListInfo->sDatabaseName );

   /* ppsTermLists */
   FreeN_DbvTermList( psTermListInfo->ppsTermLists );

   DbvFree( psTermListInfo );
   } /* End Free_DbvTermListInfo() */ 

void Free_DbvSearchCost( DbvSearchCost *peSearchCost )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSearchCost()\n" );
#endif

   if ( peSearchCost == NULL )
      return;

   DbvFree( peSearchCost );
   } /* End Free_DbvSearchCost() */ 

void Free_DbvExtendedServicesInfo( 
			DbvExtendedServicesInfo *psExtendedServicesInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvExtendedServicesInfo()\n" );
#endif

   if ( psExtendedServicesInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psExtendedServicesInfo->psCommonInfo );

   /* pcType */
   FreeDbvObjectIdentifier( psExtendedServicesInfo->pcType );

   /* psName */
   FreeDbvInternationalString( psExtendedServicesInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psExtendedServicesInfo->ppsDescription );

   /* psSpecificExplain */
   FreeDbvASN1External( psExtendedServicesInfo->psSpecificExplain );

   /* psEsASN */
   FreeDbvInternationalString( psExtendedServicesInfo->psEsASN );

   DbvFree( psExtendedServicesInfo );
   } /* End Free_DbvExtendedServicesInfo() */ 

void Free_DbvOmittedAttributeInterpretation( 
	DbvOmittedAttributeInterpretation *psOmittedAttributeInterpretation )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOmittedAttributeInterpretation()\n" );
#endif

   if ( psOmittedAttributeInterpretation == NULL )
      return;

   /* psDefaultValue */
   Free_DbvStringOrNumeric( psOmittedAttributeInterpretation->
			    psDefaultValue );

   /* ppsDefaultDescription */
   FreeN_DbvHumanString( psOmittedAttributeInterpretation->
			 ppsDefaultDescription );

   DbvFree( psOmittedAttributeInterpretation );
   } /* End Free_DbvOmittedAttributeInterpretation() */ 

void Free_DbvAttributeValues( DbvAttributeValues *psAttributeValues )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeValues()\n" );
#endif

   if ( psAttributeValues == NULL )
      return;

   /* sValue */
   Free_compulsory_DbvStringOrNumeric( &psAttributeValues->sValue );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeValues->ppsDescription );

   /* ppsSubAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSubAttributes );

   /* ppsSuperAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSuperAttributes );

   /* pvPartialSupport */
   Free_DbvNull( psAttributeValues->pvPartialSupport );

   DbvFree( psAttributeValues );
   } /* End Free_DbvAttributeValues() */ 

void Free_DbvAttributesByType( DbvAttributesByType *psAttributesByType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributesByType()\n" );
#endif

   if ( psAttributesByType == NULL )
      return;

   /* psDefaultIfOmitted */
   Free_DbvOmittedAttributeInterpretation( psAttributesByType->psDefaultIfOmitted );

   /* ppsAttributeValues */
   FreeN_DbvAttributeValues( psAttributesByType->ppsAttributeValues );

   DbvFree( psAttributesByType );
   } /* End Free_DbvAttributesByType() */ 

void Free_DbvAttributesBySet( DbvAttributesBySet *psAttributesBySet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributesBySet()\n" );
#endif

   if ( psAttributesBySet == NULL )
      return;

   /* pcAttributeSet */
   FreeDbvObjectIdentifier( psAttributesBySet->pcAttributeSet );

   /* ppsAttributesByType */
   FreeN_DbvAttributesByType( psAttributesBySet->ppsAttributesByType );

   DbvFree( psAttributesBySet );
   } /* End Free_DbvAttributesBySet() */ 

void Free_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeOccurrenceChoice()\n" );
#endif

   if ( psAttributeOccurrenceChoice != NULL )
      {
      do_Free_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );
      DbvFree( psAttributeOccurrenceChoice );
      } /* End if */

   } /* End Free_DbvAttributeOccurrenceChoice() */ 

void do_Free_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUGZ
   printf( "D - do_Free_DbvAttributeOccurrenceChoice()\n" );
#endif

   if ( psAttributeOccurrenceChoice == NULL )
      return;

   /* uAttributeOccurrenceChoiceType */

   switch( psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType )
      {
      case AOT_AnyOrNone:
	 Free_DbvNull( psAttributeOccurrenceChoice->
		       uAttributeOccurrenceChoiceType.
		       pvAnyOrNone );
	 break;

      case AOT_Specific:
	 FreeN_DbvStringOrNumeric( psAttributeOccurrenceChoice->
				       uAttributeOccurrenceChoiceType.
				       ppsSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End Free_DbvAttributeOccurrenceChoice() */ 

void Free_DbvAttributeOccurrence(
			DbvAttributeOccurrence *psAttributeOccurrence )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeOccurrence()\n" );
#endif

   if ( psAttributeOccurrence == NULL )
      return;

   /* pcAttributeSet */
   FreeDbvObjectIdentifier( psAttributeOccurrence->pcAttributeSet );

   /* pvMustBeSupplied */
   Free_DbvNull( psAttributeOccurrence->pvMustBeSupplied );

   /* sAttributeValues */
   Free_compulsory_DbvAttributeOccurrenceChoice(
				&psAttributeOccurrence->sAttributeValues );

   DbvFree( psAttributeOccurrence );
   } /* End Free_DbvAttributeOccurrence() */ 

void Free_DbvAttributeCombination( 
			DbvAttributeCombination *psAttributeCombination )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeCombination()\n" );
#endif

   if ( psAttributeCombination == NULL )
      return;

   /* ppsAttributeOccurrence */
   FreeN_DbvAttributeOccurrence( 
			psAttributeCombination->ppsAttributeOccurrence );

   DbvFree( psAttributeCombination );
   } /* End Free_DbvAttributeCombination() */ 

void Free_DbvAttributeCombinations( 
			DbvAttributeCombinations *psAttributeCombinations )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeCombinations()\n" );
#endif

   if ( psAttributeCombinations == NULL )
      return;

   /* pcDefaultAttributeSet */
   FreeDbvObjectIdentifier( psAttributeCombinations->pcDefaultAttributeSet );

   /* ppsLegalCombinations */
   FreeN_DbvAttributeCombination( psAttributeCombinations->
				   ppsLegalCombinations );

   DbvFree( psAttributeCombinations );
   } /* End Free_DbvAttributeCombinations() */ 

void Free_DbvAttributeDetails( DbvAttributeDetails *psAttributeDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAttributeDetails()\n" );
#endif

   if ( psAttributeDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psAttributeDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psAttributeDetails->sDatabaseName );

   /* ppsAttributesBySet */
   FreeN_DbvAttributesBySet( psAttributeDetails->ppsAttributesBySet );

   /* psAttributeCombinations */
   Free_DbvAttributeCombinations( psAttributeDetails->psAttributeCombinations );

   DbvFree( psAttributeDetails );
   } /* End Free_DbvAttributeDetails() */ 

void Free_DbvScanInfo( DbvScanInfo *psScanInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvScanInfo()\n" );
#endif

   if ( psScanInfo == NULL )
      return;

   /* pulMaxStepSize */
   Free_Ulong( psScanInfo->pulMaxStepSize );

   /* ppsCollatingSequence */
   FreeN_DbvHumanString( psScanInfo->ppsCollatingSequence );

   /* pfIncreasing */
   Free_DbvBool( psScanInfo->pfIncreasing );

   DbvFree( psScanInfo );
   } /* End Free_DbvScanInfo() */ 

void Free_DbvTermListDetails( DbvTermListDetails *psTermListDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTermListDetails()\n" );
#endif

   if ( psTermListDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psTermListDetails->psCommonInfo );

   /* sTermListName */
   Free_compulsory_DbvInternationalString( &psTermListDetails->sTermListName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTermListDetails->ppsDescription );

   /* psAttributes */
   Free_DbvAttributeCombinations( psTermListDetails->psAttributes );

   /* psScanInfo */
   Free_DbvScanInfo( psTermListDetails->psScanInfo );

   /* pulEstNumberTerms */
   Free_Ulong( psTermListDetails->pulEstNumberTerms );

   /* ppsSampleTerms */
   FreeN_DbvTerm( psTermListDetails->ppsSampleTerms );

   DbvFree( psTermListDetails );
   } /* End Free_DbvTermListDetails() */ 

void Free_DbvRecordTag( DbvRecordTag *psRecordTag )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvRecordTag()\n" );
#endif

   if ( psRecordTag == NULL )
      return;

   /* psQualifier */
   Free_DbvStringOrNumeric( psRecordTag->psQualifier );

   /* sTagValue */
   Free_compulsory_DbvStringOrNumeric( &psRecordTag->sTagValue );

   DbvFree( psRecordTag );
   } /* End Free_DbvRecordTag() */ 

void Free_DbvDetailsPerElement( DbvDetailsPerElement *psDetailsPerElement )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvDetailsPerElement()\n" );
#endif

   if ( psDetailsPerElement == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psDetailsPerElement->psName );

   /* psRecordTag */
   Free_DbvRecordTag( psDetailsPerElement->psRecordTag );

   /* ppsSchemaTags */
   FreeN_DbvPathA( psDetailsPerElement->ppsSchemaTags );

   /* pulMaxSize */
   Free_Ulong(  psDetailsPerElement->pulMaxSize );

   /* pulMinSize */
   Free_Ulong(  psDetailsPerElement->pulMinSize );

   /* pulAvgSize */
   Free_Ulong(  psDetailsPerElement->pulAvgSize );

   /* pulFixedSize */
   Free_Ulong(  psDetailsPerElement->pulFixedSize );

   /* ppsDescription*/
   FreeN_DbvHumanString( psDetailsPerElement->ppsDescription );

   /* ppsContents */
   FreeN_DbvHumanString( psDetailsPerElement->ppsContents );

   /* ppsBillingInfo */
   FreeN_DbvHumanString( psDetailsPerElement->ppsBillingInfo );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psDetailsPerElement->ppsRestrictions );

   /* ppsAlternateNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsAlternateNames );

   /* ppsGenericNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsGenericNames );

   /* psSearchAccess */
   Free_DbvAttributeCombinations( psDetailsPerElement->psSearchAccess );

   DbvFree( psDetailsPerElement );
   } /* End Free_DbvDetailsPerElement() */ 

void Free_DbvElementSetDetails( DbvElementSetDetails *psElementSetDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvElementSetDetails()\n" );
#endif

   if ( psElementSetDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psElementSetDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psElementSetDetails->sDatabaseName);

   /* sElementSetName */
   Free_compulsory_DbvInternationalString( 
					&psElementSetDetails->sElementSetName );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psElementSetDetails->pcRecordSyntax );

   /* pcSchema */
   FreeDbvObjectIdentifier( psElementSetDetails->pcSchema );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementSetDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psElementSetDetails->ppsDetailsPerElement );

   DbvFree( psElementSetDetails );
   } /* End Free_DbvElementSetDetails() */ 

void Free_DbvRetrievalRecordDetails( 
			DbvRetrievalRecordDetails *psRetrievalRecordDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvRetrievalRecordDetails()\n" );
#endif

   if ( psRetrievalRecordDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psRetrievalRecordDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psRetrievalRecordDetails->
					   sDatabaseName);
   /* pcSchema */
   FreeDbvObjectIdentifier( psRetrievalRecordDetails->pcSchema );

   /* pcRecordSyntax */
   FreeDbvObjectIdentifier( psRetrievalRecordDetails->pcRecordSyntax );

   /* ppsDescription */
   FreeN_DbvHumanString( psRetrievalRecordDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psRetrievalRecordDetails->ppsDetailsPerElement );

   DbvFree( psRetrievalRecordDetails );
   } /* End Free_DbvRetrievalRecordDetails() */ 

void Free_DbvSortType( DbvSortType *psSortType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSortType()\n" );
#endif

   if ( psSortType == NULL )
      return;

   /* uSortTypeType */

   switch( psSortType->eSortTypeType )
      {
      case STT_Structured:
	 FreeN_DbvHumanString( psSortType->uSortType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSortType );
   } /* End Free_DbvSortType() */ 

void Free_DbvCaseSensitivity( DbvCaseSensitivity *peCaseSensitivity )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCaseSensitivity()\n" );
#endif

   if ( peCaseSensitivity == NULL )
      return;

   DbvFree( peCaseSensitivity );
   } /* End Free_DbvCaseSensitivity() */ 

void Free_DbvSortKeys( DbvSortKeys *psSortKeys )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSortKeys()\n" );
#endif

   if ( psSortKeys == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psSortKeys->ppsDescription );

   /* ppsElementSpecifications */
   FreeN_DbvSpecification( psSortKeys->ppsElementSpecifications );

   /* psAttributeSpecifications */
   Free_DbvAttributeCombinations( psSortKeys->psAttributeSpecifications );

   /* psSortType */
   Free_DbvSortType( psSortKeys->psSortType );

   /* peCaseSensitivity */
   Free_DbvCaseSensitivity( psSortKeys->peCaseSensitivity );
   
   DbvFree( psSortKeys );
   } /* End Free_DbvSortKeys() */ 

void Free_DbvSortDetails( DbvSortDetails *psSortDetails )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSortDetails()\n" );
#endif

   if ( psSortDetails == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psSortDetails->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( &psSortDetails->sDatabaseName );

   /* ppsSortKeys */
   FreeN_DbvSortKeys( psSortDetails->ppsSortKeys );

   DbvFree( psSortDetails );
   } /* End Free_DbvSortDetails() */ 

void Free_DbvProcessingInformation( 
			DbvProcessingInformation *psProcessingInformation )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvProcessingInformation()\n" );
#endif

   if ( psProcessingInformation == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psProcessingInformation->psCommonInfo );

   /* sDatabaseName */
   Free_compulsory_DbvInternationalString( 
				&psProcessingInformation->sDatabaseName );
   /* sName */
   Free_compulsory_DbvInternationalString( &psProcessingInformation->sName );

   /* pcOid */
   FreeDbvObjectIdentifier( psProcessingInformation->pcOid );

   /* ppsDescription */
   FreeN_DbvHumanString( psProcessingInformation->ppsDescription );

   /* psInstructions */
   FreeDbvASN1External( psProcessingInformation->psInstructions );

   DbvFree( psProcessingInformation );
   } /* End Free_DbvProcessingInformation() */ 

void Free_DbvValueDescription( DbvValueDescription *psValueDescription )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvValueDescription()\n" );
#endif

   if ( psValueDescription == NULL )
      return;

   /* uValueDescriptionType */

   switch( psValueDescription->eValueDescriptionType )
      {
      case VDT_Integer:
	 break;

      case VDT_String:
	 FreeDbvInternationalString( psValueDescription->
					uValueDescription.psString );
	 break;

      case VDT_Octets:
	 FreeDbvOctetString( psValueDescription->
					uValueDescription.psOctets );
	 break;

      case VDT_Oid:
	 FreeDbvObjectIdentifier( psValueDescription->
					uValueDescription.pcOid );
	 break;

      case VDT_Unit:
	 Free_DbvUnit( psValueDescription->
					uValueDescription.psUnit );
	 break;

      case VDT_ValueAndUnit:
	 Free_DbvIntUnit( psValueDescription->
					uValueDescription.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueDescription );
   } /* End Free_DbvValueDescription() */ 

void Free_DbvValueRange( DbvValueRange *psValueRange )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvValueRange()\n" );
#endif

   if ( psValueRange == NULL )
      return;

   /* psLower */
   Free_DbvValueDescription( psValueRange->psLower );

   /* psUpper */
   Free_DbvValueDescription( psValueRange->psUpper );

   DbvFree( psValueRange );
   } /* End Free_DbvValueRange() */ 

void Free_DbvValueSet( DbvValueSet *psValueSet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvValueSet()\n" );
#endif

   if ( psValueSet == NULL )
      return;

   /* uValueSetType */

   switch( psValueSet->eValueSetType )
      {
      case VT_Range:
	 Free_DbvValueRange( psValueSet->uValueSet.psRange );
	 break;

      case VT_Enumerated:
	 FreeN_DbvValueDescription( psValueSet->uValueSet.ppsEnumerated );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueSet );
   } /* End Free_DbvValueSet() */ 

void Free_DbvVariantValue( DbvVariantValue *psVariantValue )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvVariantValue()\n" );
#endif

   if ( psVariantValue == NULL )
      return;

   /* psValues */
   Free_DbvValueSet( psVariantValue->psValues );

   DbvFree( psVariantValue );
   } /* End Free_DbvVariantValue() */ 

void Free_DbvVariantTypes( DbvVariantTypes *psVariantTypes )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvVariantTypes()\n" );
#endif

   if ( psVariantTypes == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psVariantTypes->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantTypes->ppsDescription );

   /* psVariantValue */
   Free_DbvVariantValue( psVariantTypes->psVariantValue );

   DbvFree( psVariantTypes );
   } /* End Free_DbvVariantTypes() */ 

void Free_DbvVariantClass( DbvVariantClass *psVariantClass )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvVariantClass()\n" );
#endif

   if ( psVariantClass == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psVariantClass->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantClass->ppsDescription );

   /* ppsVariantTypes */
   FreeN_DbvVariantTypes( psVariantClass->ppsVariantTypes );

   DbvFree( psVariantClass );
   } /* End Free_DbvVariants() */ 

void Free_DbvVariantSetInfo( DbvVariantSetInfo *psVariantSetInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvVariantSetInfo()\n" );
#endif

   if ( psVariantSetInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psVariantSetInfo->psCommonInfo );

   /* pcVariantSet */
   FreeDbvObjectIdentifier( psVariantSetInfo->pcVariantSet );

   /* sName */
   Free_compulsory_DbvInternationalString( &psVariantSetInfo->sName );

   /* ppsVariants */
   FreeN_DbvVariantClass( psVariantSetInfo->ppsVariants );

   DbvFree( psVariantSetInfo );
   } /* End Free_DbvVariantSetInfo() */ 

void Free_DbvUnits( DbvUnits *psUnits )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvUnits()\n" );
#endif

   if ( psUnits == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psUnits->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnits->ppsDescription );

   /* sUnit */
   Free_compulsory_DbvStringOrNumeric( &psUnits->sUnit );

   DbvFree( psUnits );
   } /* End Free_DbvUnits() */ 

void Free_DbvUnitType( DbvUnitType *psUnitType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvUnitType()\n" );
#endif

   if ( psUnitType == NULL )
      return;

   /* psName */
   FreeDbvInternationalString( psUnitType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitType->ppsDescription );

   /* sUnitType */
   Free_compulsory_DbvStringOrNumeric( &psUnitType->sUnitType );

   /* ppsUnits */
   FreeN_DbvUnits( psUnitType->ppsUnits );

   DbvFree( psUnitType );
   } /* End Free_DbvUnitType() */ 

void Free_DbvUnitInfo( DbvUnitInfo *psUnitInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvUnitInfo()\n" );
#endif

   if ( psUnitInfo == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psUnitInfo->psCommonInfo );

   /* sUnitSystem */
   Free_compulsory_DbvInternationalString( &psUnitInfo->sUnitSystem );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitInfo->ppsDescription );

   /* ppsUnitTypes */
   FreeN_DbvUnitType( psUnitInfo->ppsUnitTypes );

   DbvFree( psUnitInfo );
   } /* End Free_DbvUnitInfo() */ 

void Free_DbvCategoryInfo( DbvCategoryInfo *psCategoryInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCategoryInfo()\n" );
#endif

   if ( psCategoryInfo == NULL )
      return;

   /* sCategory */
   Free_compulsory_DbvInternationalString( &psCategoryInfo->sCategory );

   /* psOriginalCategory */
   FreeDbvInternationalString( psCategoryInfo->psOriginalCategory );

   /* ppsDescription */
   FreeN_DbvHumanString( psCategoryInfo->ppsDescription );

   /* psAsn1Module */
   FreeDbvInternationalString( psCategoryInfo->psAsn1Module );

   DbvFree( psCategoryInfo );
   } /* End Free_DbvCategorInfo() */ 

void Free_DbvCategoryList( DbvCategoryList *psCategoryList )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCategoryList()\n" );
#endif

   if ( psCategoryList == NULL )
      return;

   /* psCommonInfo */
   Free_DbvCommonInfo( psCategoryList->psCommonInfo );

   /* ppsCategories */
   FreeN_DbvCategoryInfo( psCategoryList->ppsCategories );

   DbvFree( psCategoryList );
   } /* End Free_DbvCategoryList() */ 

void Free_DbvExplainRecord( DbvExplainRecord *psExplainRecord )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvExplainRecord()\n" );
#endif

   if ( psExplainRecord == NULL )
      return;

   /* uExplainRecordType */

   switch( psExplainRecord->eExplainRecordType )
      {
      case ERT_TargetInfo:
	 Free_DbvTargetInfo( psExplainRecord->uExplainRecord.psTargetInfo);
	 break;

      case ERT_DatabaseInfo:
	 Free_DbvDatabaseInfo( psExplainRecord->
					uExplainRecord.psDatabaseInfo);
	 break;

      case ERT_SchemaInfo:
	 Free_DbvSchemaInfo( psExplainRecord->
					uExplainRecord.psSchemaInfo);
	 break;

      case ERT_TagSetInfo:
	 Free_DbvTagSetInfo( psExplainRecord->
					uExplainRecord.psTagSetInfo);
	 break;

      case ERT_RecordSyntaxInfo:
	 Free_DbvRecordSyntaxInfo( psExplainRecord->
					uExplainRecord.psRecordSyntaxInfo);
	 break;

      case ERT_AttributeSetInfo:
	 Free_DbvAttributeSetInfo( psExplainRecord->
					uExplainRecord.psAttributeSetInfo);
	 break;

      case ERT_TermListInfo:
	 Free_DbvTermListInfo( psExplainRecord->
					uExplainRecord.psTermListInfo);
	 break;

      case ERT_ExtendedServicesInfo:
	 Free_DbvExtendedServicesInfo( psExplainRecord->
					uExplainRecord.psExtendedServicesInfo);
	 break;

      case ERT_AttributeDetails:
	 Free_DbvAttributeDetails( psExplainRecord->
					uExplainRecord.psAttributeDetails);
	 break;

      case ERT_TermListDetails:
	 Free_DbvTermListDetails( psExplainRecord->
					uExplainRecord.psTermListDetails);
	 break;

      case ERT_ElementSetDetails:
	 Free_DbvElementSetDetails( psExplainRecord->
					uExplainRecord.psElementSetDetails);
	 break;

      case ERT_RetrievalRecordDetails:
	 Free_DbvRetrievalRecordDetails( psExplainRecord->
			 	uExplainRecord.psRetrievalRecordDetails);
	 break;

      case ERT_SortDetails:
	 Free_DbvSortDetails( psExplainRecord->
					uExplainRecord.psSortDetails);
	 break;

      case ERT_Processing:
	 Free_DbvProcessingInformation( psExplainRecord->
					uExplainRecord.psProcessing);
	 break;

      case ERT_Variants:
	 Free_DbvVariantSetInfo( psExplainRecord->
					uExplainRecord.psVariants);
	 break;

      case ERT_Units:
	 Free_DbvUnitInfo( psExplainRecord->
					uExplainRecord.psUnits);
	 break;

      case ERT_CategoryList:
	 Free_DbvCategoryList( psExplainRecord->
					uExplainRecord.psCategoryList);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psExplainRecord );
   } /* End Free_DbvExplainRecord() */ 

void Free_DbvLanguageCode( DbvLanguageCode *psLanguageCode )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvLanguageCode()\n" );
#endif

   FreeDbvOctetString( (DbvOctetString*)psLanguageCode );

   } /* End Free_DbvLanguageCode() */ 

void Free_DbvNull( DbvNull *psNull )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvNull()\n" );
#endif

   if ( psNull == NULL )
      return;

   DbvFree( psNull );

   } /* End Free_DbvNull() */ 

void Free_DbvUnit( DbvUnit *psUnit )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvUnit()\n" );
#endif

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */
   FreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */
   Free_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */
   Free_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */
   Free_Ulong( psUnit->pulScaleFactor );

   DbvFree( psUnit );
   } /* End Free_DbvUnit() */ 

void Free_DbvTerm( DbvTerm *psTerm )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTerm()\n" );
#endif

   if ( psTerm == NULL )
      return;

   /* uTermType */

   switch( psTerm->eTermType )
      {
      case TT_General:
	 FreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
	 break;

      case TT_SimpleCharacter:
	 FreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter );
	 break;

      case TT_ObjectIdentifier:
	 FreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
	 Free_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
	 FreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 Free_DbvIntUnit( psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
      default:
	 break;
      } /* End switch */

   DbvFree( psTerm );
   } /* End Free_DbvTerm() */ 

void Free_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq *psDataBaseInfoSeq )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvDataBaseInfoSeq()\n" );
#endif

   if ( psDataBaseInfoSeq == NULL )
      return;

   /* ppsDataBaseInfo */
   FreeN_DbvHumanString( psDataBaseInfoSeq->ppsDataBaseInfo );

   DbvFree( psDataBaseInfoSeq );
   } /* End Free_DbvDataBaseInfoSeq() */ 










/*-------------------- ACCESSCONTROL --------------------*/

void Free_DbvSecurityChallengeRsp( DbvSecurityChallengeRsp *psSecurityChallengeRsp )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSecurityChallengeRsp()\n" );
#endif

   if ( psSecurityChallengeRsp == NULL )
      return;

   /* uSecurityChallengeRsp */

   switch( psSecurityChallengeRsp->eSecurityChallengeRspType ) 
      {
      case SECT_SimpleForm:
   	 FreeDbvOctetString( psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 FreeDbvASN1External( psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSecurityChallengeRsp );
   } /* End Free_DbvSecurityChallengeRsp */



void Free_DbvAccessControlReq( DbvAccessControlReq *psAccessControlReq )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAccessControlReq()\n" );
#endif

   if ( psAccessControlReq == NULL )
      return;

   /* psReferenceId */  
   FreeDbvOctetString( psAccessControlReq->psReferenceId );

   /* sSecurityChallengeReq */
   Free_compulsory_DbvSecurityChallengeReq( &psAccessControlReq->sSecurityChallengeReq );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlReq->ppsOtherInformation );

   DbvFree( psAccessControlReq );
   } /* End Free_DbvAccessControlReq() */ 



void Free_DbvAccessControlRsp( DbvAccessControlRsp *psAccessControlRsp )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvAccessControlRsp()\n" );
#endif

   if ( psAccessControlRsp == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psAccessControlRsp->psReferenceId );

   /* psSecurityChallengeRsp */
   Free_DbvSecurityChallengeRsp( psAccessControlRsp->psSecurityChallengeRsp );

   /* psDiagnostic */
   Free_DbvDiagRec( psAccessControlRsp->psDiagnostic );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlRsp->ppsOtherInformation );

   DbvFree( psAccessControlRsp );
   } /* End Free_DbvAccessControlRsp() */ 



/* -------------- Access Control Format "Prompt 1" ----------------*/


void Free_DbvEnumeratedPrompt( DbvEnumeratedPrompt *psEnumeratedPrompt )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvEnumeratedPrompt()\n" );
#endif

   if ( psEnumeratedPrompt == NULL )
      return;


   /* compulsory eType */

   /* psSuggestedString */
   FreeDbvInternationalString( psEnumeratedPrompt->psSuggestedString );

   DbvFree( psEnumeratedPrompt );
   } /* End Free_DbvEnumeratedPrompt() */ 



void Free_DbvEncryption( DbvEncryption *psEncryption )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvEncryption()\n" );
#endif

   if ( psEncryption == NULL )
      return;

   /* psCryptType */
   FreeDbvOctetString( psEncryption->psCryptType );

   /* psCredential */
   FreeDbvOctetString( psEncryption->psCredential );

   /* sData */
   Free_compulsory_DbvOctetString( &psEncryption->sData);

   DbvFree( psEncryption );
   } /* End Free_DbvEncryption() */ 



void Free_DbvPromptInfo( DbvPromptInfo *psPromptInfo )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPromptInfo()\n" );
#endif

   if ( psPromptInfo == NULL )
      return;


   /* uPromptInfo */

   switch( psPromptInfo->ePromptInfoType ) 
      {
      case PIIT_Character:
   	 FreeDbvInternationalString( psPromptInfo->uPromptInfo.psCharacter );
	 break;

      case PIIT_Encrypted:
         Free_DbvEncryption( psPromptInfo->uPromptInfo.psEncrypted );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptInfo );
   } /* End Free_DbvPromptInfo */



void Free_DbvDataType( DbvDataType *peDataType )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvDataType()\n" );
#endif
     
   if ( peDataType == NULL )
      return;

   DbvFree( peDataType );
   } /* End Free_DbvDataType() */



void Free_DbvChallenge( DbvChallenge *psChallenge )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvChallenge()\n" );
#endif

   if ( psChallenge == NULL )
      return;

   /* sPromptId */
   Free_compulsory_DbvPromptId( &psChallenge->sPromptId );

   /* psDefaultResponse */
   FreeDbvInternationalString( psChallenge->psDefaultResponse );

   /* psPromptInfo */
   Free_DbvPromptInfo( psChallenge->psPromptInfo );

   /* psRegExpr */
   FreeDbvInternationalString( psChallenge->psRegExpr );

   /* pvResponseRequired */
   Free_DbvNull( psChallenge->pvResponseRequired );

   /* ppsAllowedValues */
   FreeN_DbvInternationalString( psChallenge->ppsAllowedValues );

   /* pvShouldSave */
   Free_DbvNull( psChallenge->pvShouldSave );

   /* peDataType */
   Free_DbvDataType( psChallenge->peDataType );

   /* psDiagnostic */
   FreeDbvASN1External( psChallenge->psDiagnostic );

   DbvFree( psChallenge );
   } /* End Free_DbvChallenge() */ 



void Free_DbvResponse( DbvResponse *psResponse )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvResponse()\n" );
#endif

   if ( psResponse == NULL )
      return;

   /* sPromptId */
   Free_compulsory_DbvPromptId( &psResponse->sPromptId );

   /* sPromptResponse */
   Free_compulsory_DbvPromptResponse( &psResponse->sPromptResponse );

   DbvFree( psResponse );
   } /* Free_DbvResponse() */



void Free_DbvPromptObject( DbvPromptObject *psPromptObject )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPromptObject()\n" );
#endif

   if ( psPromptObject == NULL )
      return;


   /* uPromptObject */

   switch( psPromptObject->ePromptObjectType ) 
      {
      case POT_Challenge:
   	 FreeN_DbvChallenge( psPromptObject->uPromptObject.ppsChallenge );
	 break;

      case POT_Response:
   	 FreeN_DbvResponse( psPromptObject->uPromptObject.ppsResponse );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptObject );
   } /* End Free_DbvPromptObject() */



/*-------------------- RESOURCECONTROL --------------------*/


void Free_DbvPartialResultsAvailable( DbvPartialResultsAvailable *pePartialResultsAvailable )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPartialResultsAvailable()\n" );
#endif
     
   if ( pePartialResultsAvailable == NULL )
      return;

   DbvFree( pePartialResultsAvailable );
   } /* End Free_DbvPartialResultsAvailable() */



void Free_DbvResourceControlReq( DbvResourceControlReq *psResourceControlReq )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvResourceControlReq()\n" );
#endif

   if ( psResourceControlReq == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psResourceControlReq->psReferenceId );

   /* pfSuspendedFlag */
   Free_DbvBool( psResourceControlReq->pfSuspendedFlag );

   /* psResourceReport */
   FreeDbvASN1External( psResourceControlReq->psResourceReport );

   /* pePartialResultsAvailable */
   Free_DbvPartialResultsAvailable( psResourceControlReq->pePartialResultsAvailable );

   /* compulsory fResponseRequired */

   /* pfTriggeredRequestFlag */
   Free_DbvBool( psResourceControlReq->pfTriggeredRequestFlag );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlReq->ppsOtherInformation );

   DbvFree( psResourceControlReq );
   } /* End Free_DbvResourceControlReq() */ 




void Free_DbvResourceControlRsp( DbvResourceControlRsp *psResourceControlRsp )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvResourceControlRsp()\n" );
#endif

   if ( psResourceControlRsp == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psResourceControlRsp->psReferenceId );

   /* compulsory fContinueFlag */

   /* pfResultSetWanted */
   Free_DbvBool( psResourceControlRsp->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlRsp->ppsOtherInformation );

   DbvFree( psResourceControlRsp );
   } /* End Free_DbvResourceControlRsp() */ 




/*-------------------- TRIGGERRESOURCECONTROL --------------------*/


void Free_DbvTriggerResourceControlReq( DbvTriggerResourceControlReq *psTriggerResourceControlReq )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTriggerResourceControlReq()\n" );
#endif

   if ( psTriggerResourceControlReq == NULL )
      return;

   /* psReferenceId */
   FreeDbvOctetString( psTriggerResourceControlReq->psReferenceId );

   /* compulsory eRequestedAction */

   /* pcPreferredResourceReportFormat */
   FreeDbvObjectIdentifier( psTriggerResourceControlReq->pcPreferredResourceReportFormat );

   /* pfResultSetWanted */
   Free_DbvBool( psTriggerResourceControlReq->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psTriggerResourceControlReq->ppsOtherInformation );

   DbvFree( psTriggerResourceControlReq );
   } /* End Free_DbvTriggerResourceControlReq() */ 


/*------------------------------------*/


void Free_DbvElementSpec(DbvElementSpec *psElementSpec)                 
{
#ifdef DEBUGZ
   printf( "D - Free_DbvElementSpec()\n" );
#endif

   if ( psElementSpec == NULL )
      return;

   switch(psElementSpec->eElementSpecType)
   {
   case EST_Primitive:
           FreeDbvInternationalString(psElementSpec->
                   uElementSpec.psPrimitive);
           break;
   case EST_External:
           FreeDbvASN1External(psElementSpec->
                   uElementSpec.psExternal);
           break;
   default:
           break;
   }
} /* Free_DbvElementSpec() */





/* ---------------------------------------------------------------------*/
/* ---- User Info Format "Character Set And Language Negotiation" ----- */



void Free_DbvEnvironment( DbvEnvironment *psEnvironment )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvEnvironment()\n" );
#endif

   if ( psEnvironment == NULL )
      return;

   DbvFree( psEnvironment );
   } /* End Free_DbvEnvironment */



void Free_DbvInitialSet( DbvInitialSet *psInitialSet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvInitialSet()\n" );
#endif

   if ( psInitialSet == NULL )
      return;

   DbvFree( psInitialSet );
   } /* End Free_DbvInitialSet */



void Free_DbvLeftAndRight( DbvLeftAndRight *psLeftAndRight )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvLeftAndRight()\n" );
#endif

   if ( psLeftAndRight == NULL )
      return;

   DbvFree( psLeftAndRight );
   } /* End Free_DbvLeftAndRight */



void Free_DbvIso2022Proposal( DbvIso2022Proposal *psIso2022Proposal )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIso2022Proposal()\n" );
#endif

   if ( psIso2022Proposal == NULL )
      return;


   /* psProposedEnvironment */  
   Free_DbvEnvironment( psIso2022Proposal->psProposedEnvironment );

   /* ppulProposedSets */
   FreeN_Ulong(psIso2022Proposal->ppulProposedSets);

   /* ppsProposedInitialSets */  
   FreeN_DbvInitialSet( psIso2022Proposal->ppsProposedInitialSets );

   /* psProposedLeftAndRight */
   Free_DbvLeftAndRight(psIso2022Proposal->psProposedLeftAndRight);


   DbvFree( psIso2022Proposal );
   } /* End Free_DbvIso2022Proposal() */ 



void Free_DbvIso2022Response( DbvIso2022Response *psIso2022Response )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIso2022Response()\n" );
#endif

   if ( psIso2022Response == NULL )
      return;

   /* psSelectedEnvironment */  
   Free_DbvEnvironment( psIso2022Response->psSelectedEnvironment );

   /* ppulSelectedSets */
   FreeN_Ulong(psIso2022Response->ppulSelectedSets);

   /* psSelectedInitialSet */  
   Free_DbvInitialSet( psIso2022Response->psSelectedInitialSet );

   /* psSelectedLeftAndRight */
   Free_DbvLeftAndRight(psIso2022Response->psSelectedLeftAndRight);


   DbvFree( psIso2022Response );
   } /* End Free_DbvIso2022Response() */ 




void Free_DbvIso2022( DbvIso2022 *psIso2022 )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIso2022()\n" );
#endif

   if ( psIso2022 == NULL )
      return;


   /* uIso2022Type */

   switch( psIso2022->eIso2022Type ) 
      {
      case I2022T_OriginProposal:
   	 Free_DbvIso2022Proposal( psIso2022->uIso2022Type.psOriginProposal );
	 break;

      case I2022T_TargetResponse:
   	 Free_DbvIso2022Response( psIso2022->uIso2022Type.psTargetResponse );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psIso2022 );
   } /* End Free_DbvIso2022 */



void Free_DbvIso10646( DbvIso10646 *psIso10646 )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvIso10646()\n" );
#endif

   if ( psIso10646 == NULL )
      return;

   /* pcCollections */  
   FreeDbvObjectIdentifier( psIso10646->pcCollections );

   /* pcEncodingLevel */
   FreeDbvObjectIdentifier( psIso10646->pcEncodingLevel );


   DbvFree( psIso10646 );
   } /* End Free_DbvIso10646() */ 



void Free_DbvPrivateCharacterSet( DbvPrivateCharacterSet *psPrivateCharacterSet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvPrivateCharacterSet()\n" );
#endif

   if ( psPrivateCharacterSet == NULL )
      return;


   /* uPrivateCharacterSet */

   switch( psPrivateCharacterSet->ePrivateCharacterSetType ) 
      {
      case PCST_ViaOid:
   	 FreeN_DbvObjectIdentifier( psPrivateCharacterSet->uPrivateCharacterSet.ppcViaOid );
	 break;

      case PCST_ExternallySpecified:
   	 FreeDbvASN1External( psPrivateCharacterSet->uPrivateCharacterSet.psExternallySpecified );
	 break;

      case PCST_PreviouslyAgreedUpon:
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPrivateCharacterSet );
   } /* End Free_DbvPrivateCharacterSet */



void Free_DbvProposedCharSet( DbvProposedCharSet *psProposedCharSet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvProposedCharSet()\n" );
#endif

   if ( psProposedCharSet == NULL )
      return;


   /* uProposedCharSet */

   switch( psProposedCharSet->eProposedCharSetType ) 
      {
      case PRCST_Iso2022:
   	 Free_DbvIso2022( psProposedCharSet->uProposedCharSet.psIso2022);
	 break;

      case PRCST_Iso10646:
   	 Free_DbvIso10646( psProposedCharSet->uProposedCharSet.psIso10646);
	 break;

      case PRCST_Private:
   	 Free_DbvPrivateCharacterSet( psProposedCharSet->uProposedCharSet.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psProposedCharSet );
   } /* End Free_DbvProposedCharSet */



void Free_DbvOriginProposal( DbvOriginProposal *psOriginProposal )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvOriginProposal()\n" );
#endif

   if ( psOriginProposal == NULL )
      return;

   /* ppsProposedCharSets */  
   FreeN_DbvProposedCharSet( psOriginProposal->ppsProposedCharSets );

   /* ppsProposedLanguages */
   FreeN_DbvLanguageCode( psOriginProposal->ppsProposedLanguages );

   /* pfRecordsInSelectedCharSets */
   Free_DbvBool( psOriginProposal->pfRecordsInSelectedCharSets );

   DbvFree( psOriginProposal );
   } /* End Free_DbvOriginProposal() */ 



void Free_DbvSelectedCharSet( DbvSelectedCharSet *psSelectedCharSet )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvSelectedCharSet()\n" );
#endif

   if ( psSelectedCharSet == NULL )
      return;


   /* uSelectedCharSet */

   switch( psSelectedCharSet->eSelectedCharSetType ) 
      {
      case SCST_Iso2022:
   	 Free_DbvIso2022( psSelectedCharSet->uSelectedCharSet.psIso2022);
	 break;

      case SCST_Iso10646:
   	 Free_DbvIso10646( psSelectedCharSet->uSelectedCharSet.psIso10646);
	 break;

      case SCST_Private:
   	 Free_DbvPrivateCharacterSet( psSelectedCharSet->uSelectedCharSet.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSelectedCharSet );
   } /* End Free_DbvSelectedCharSet */



void Free_DbvTargetResponse( DbvTargetResponse *psTargetResponse )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvTargetResponse()\n" );
#endif

   if ( psTargetResponse == NULL )
      return;

   /* psSelectedCharSets */  
   Free_DbvSelectedCharSet( psTargetResponse->psSelectedCharSets );

   /* psSelectedLanguage */
   Free_DbvGeneralString( psTargetResponse->psSelectedLanguage );

   /* pfRecordsInSelectedCharSets */
   Free_DbvBool( psTargetResponse->pfRecordsInSelectedCharSets );

   DbvFree( psTargetResponse );
   } /* End Free_DbvTargetResponse() */ 



void Free_DbvCharSetAndLanguage( DbvCharSetAndLanguage *psCharSetAndLanguage )
   {
#ifdef DEBUGZ
   printf( "D - Free_DbvCharSetAndLanguage()\n" );
#endif

   if ( psCharSetAndLanguage == NULL )
      return;


   /* uCharSetAndLanguage */

   switch( psCharSetAndLanguage->eCharSetAndLanguageType ) 
      {
      case CSALT_Proposal:
   	 Free_DbvOriginProposal( psCharSetAndLanguage->uCharSetAndLanguage.psProposal);
	 break;

      case CSALT_Response:
   	 Free_DbvTargetResponse( psCharSetAndLanguage->uCharSetAndLanguage.psResponse);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psCharSetAndLanguage );
   } /* End Free_DbvCharSetAndLanguage */



