/*  Module Name: @(#)get_DbvFuncs.c	7.1
    Version: 11/15/96
    Author: Dan Maskell/Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Functions to get structure types from the script files. 
	Used by the origin and target.

    31-01-96 Dan Maskell Changed Resourse Report 2 to psMessage from sMessage
    05-02-96 Dan Maskell Added Functions for ILL & Item order stuff
*/
 
#include "get_DbvFuncs.h"

#undef DEBUG

extern TBL_Return T_Return;

void get_DbvAdditionalInfo(DbvAdditionalInfo **ppsAdditionalInfo, TBL InTable)
   {
   char *stringfield;
   DbvAdditionalInfo *psAdditionalInfo;
   DbvProtocolVersionInForce  eProtocolVersionInForce;

   /* Malloc the space */

   *ppsAdditionalInfo = (DbvAdditionalInfo*)DbvAlloc( 
					sizeof( DbvAdditionalInfo ) );
   psAdditionalInfo = *ppsAdditionalInfo;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvProtocolVersionInForce( &stringfield, &eProtocolVersionInForce );
   psAdditionalInfo->eProtocolVersionInForce = eProtocolVersionInForce;
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   { 
   DbvFree( stringfield );

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   switch( eProtocolVersionInForce )
      {
      case PVIF_Version2:
         get_DbvVisibleString(&psAdditionalInfo->uAdditionalInfo.pcV2,
				InTable);
         break;

      case PVIF_Version3:
         get_DbvInternationalString(&psAdditionalInfo->uAdditionalInfo.psV3,
				InTable);
         break;

      default:
	 break;
      } /* End switch */
     }
     else
     DbvFree( stringfield );
   } /* End get_DbvAdditionalInfo() */

void get_DbvDefaultDiagFormat( DbvDefaultDiagFormat **ppsDefaultDiagFormat, 
				TBL InTable )
   {
   DbvDefaultDiagFormat		*psDefaultDiagFormat;
   DbvAdditionalInfo		*psAdditionalInfo;
   char				*stringfield;

   /* Malloc the space */

   *ppsDefaultDiagFormat = (DbvDefaultDiagFormat*)DbvAlloc( 
					sizeof( DbvDefaultDiagFormat ) );
   psDefaultDiagFormat = *ppsDefaultDiagFormat;

   TBL_GetRecord(InTable);

   /* Object Identifier */
   get_DbvObjectIdentifier( &psDefaultDiagFormat->pcDiagnosticSetId, InTable );

   /* Condition */

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   psDefaultDiagFormat->ulCondition = get_int_field( InTable,T_Return );

    /* Additional Info */

   get_DbvAdditionalInfo( &psAdditionalInfo, InTable );

   psDefaultDiagFormat->sAdditionalInfo.eProtocolVersionInForce =
   				psAdditionalInfo->eProtocolVersionInForce;

   switch( psAdditionalInfo->eProtocolVersionInForce ) 
      {
      case PVIF_Version2:
         psDefaultDiagFormat->sAdditionalInfo.uAdditionalInfo.pcV2 =
   				psAdditionalInfo->uAdditionalInfo.pcV2;
	 
	 break;

      case PVIF_Version3:
         psDefaultDiagFormat->sAdditionalInfo.uAdditionalInfo.psV3 =
   				psAdditionalInfo->uAdditionalInfo.psV3;
	 break;

      default:
      printf( "Missing case statement in %s at %d\n", __FILE__, __LINE__ );
	 break;
      } /* End switch */

   /* Free just the main block as this has been copied */

   DbvFree( psAdditionalInfo );

   } /* End get_DbvDefaultDiagFormat() */

void get_DbvDiagRec( DbvDiagRec **ppsDiagRec, TBL InTable )
   {
   char *stringfield;
   DbvDiagRec		*psDiagRec;
   DbvDiagnosticsType	eDiagnosticsType;


   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvDiagnosticsType( &stringfield, &eDiagnosticsType );
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   { 
   DbvFree( stringfield );

   /* Malloc the space */

   *ppsDiagRec = (DbvDiagRec*)DbvAlloc( sizeof( DbvDiagRec ) );
   psDiagRec = *ppsDiagRec;
   psDiagRec->eDiagnosticsType = eDiagnosticsType;

   switch( eDiagnosticsType )
      {
      case DT_DefaultFormat:
         get_DbvDefaultDiagFormat( &psDiagRec->uDiagnostics.psDefault, InTable);
         break;

      case DT_ExternallyDefined:
         get_DbvASN1External( &psDiagRec->uDiagnostics.psExternallyDefined, 
				InTable );
         break;

      default:
	 break;
      } /* End switch */
     }
    else
     DbvFree( stringfield );
   } /* End get_DbvDiagRec() */

void get_DbvElementSetNames(DbvElementSetNames **ppsElementSetNames, 
			    TBL InTable)
   {
   char *stringfield;
   int intfield;
   DbvElementSetNames *psElementSetNames;
   DbvElementSetType  eElementSetType;
   DbvElementSetsDbSpecific **ppsElementSetsDbSpecific;

   /* Malloc the space */

   *ppsElementSetNames = (DbvElementSetNames*)DbvAlloc( 
						sizeof(DbvElementSetNames));
   psElementSetNames = *ppsElementSetNames;

   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvElementSetType( &stringfield, &eElementSetType );
   psElementSetNames->eElementSetType = eElementSetType;
   DbvFree(stringfield);
   TBL_GetRecord(InTable);
   switch( eElementSetType )
      {
      case EST_Generic:
	 /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);

         get_DbvInternationalString( &psElementSetNames->uElementSet.psGeneric,
				     InTable );
         break;

      case EST_DatabaseSpecific:

         /* Get the Number of Database Names */

         stringfield = get_char_field(InTable, T_Return); 
         DbvFree(stringfield);

         intfield = get_int_field(InTable,T_Return);

         /* Database Names */

         ppsElementSetsDbSpecific = ( DbvElementSetsDbSpecific **)DbvAlloc( 
			( sizeof( DbvElementSetsDbSpecific* )*(intfield + 1)));
         psElementSetNames->uElementSet.ppsElementSetsDbSpecific = ppsElementSetsDbSpecific;

         for ( ; intfield; intfield-- )
	    {
	    /* Ignore the title */
            TBL_GetRecord(InTable);
            stringfield = get_char_field(InTable, T_Return); 
            DbvFree(stringfield);
            *ppsElementSetsDbSpecific = ( DbvElementSetsDbSpecific *)DbvAlloc( 
			sizeof( DbvElementSetsDbSpecific ));

            get_DbvInternationalString( &(*ppsElementSetsDbSpecific)->
					psDatabaseName, InTable );
	    /* Ignore the title */
            TBL_GetRecord(InTable);
            stringfield = get_char_field(InTable, T_Return); 
            DbvFree(stringfield);
            get_DbvInternationalString( &(*ppsElementSetsDbSpecific)->
					psElementSetName, InTable );
	    ppsElementSetsDbSpecific++;
	    } /* End for */
 
         *ppsElementSetsDbSpecific = NULL;
         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvElementSetNames() */
void get_DbvFragmentSyntax( DbvFragmentSyntax **ppsFragmentSyntax, TBL InTable )
   {
   char *stringfield;
   DbvFragmentSyntax		*psFragmentSyntax;
   DbvFragmentSyntaxType	eFragmentSyntaxType;

   /* Malloc the space */

   *ppsFragmentSyntax = (DbvFragmentSyntax*)DbvAlloc( 
					sizeof( DbvFragmentSyntax ) );
   psFragmentSyntax = *ppsFragmentSyntax;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvFragmentSyntaxType( &stringfield, &eFragmentSyntaxType );
   DbvFree( stringfield );

   psFragmentSyntax->eFragmentSyntaxType = eFragmentSyntaxType;

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   { 
   DbvFree( stringfield );
   switch( eFragmentSyntaxType )
      {
      case FST_ExternallyTagged:
         get_DbvASN1External( &psFragmentSyntax->
				uFragmentSyntax.psExternallyTagged, 
				InTable);
         break;

      case FST_NotExternallyTagged:
        TBL_GetRecord( InTable );
        /* Ignore title */
        stringfield = get_char_field( InTable, T_Return );
        DbvFree(stringfield);
        get_DbvOctetString( &psFragmentSyntax->
				uFragmentSyntax.psNotExternallyTagged, 
				InTable);
         break;

      default:
	 break;
      } /* End switch */
      }/* End If */
    else
      DbvFree( stringfield );
   } /* End get_DbvFragmentSyntax() */
void get_DbvListStatus(DbvListStatus **ppsListStatus, TBL InTable)
   {
   char *stringfield;
   int intfield;
   DbvListStatus *psListStatus;
   DbvDeleteSetStatus eDeleteSetStatus;
   DbvInternationalString *psResultSetId;

   /* Malloc the space */

   *ppsListStatus = (DbvListStatus*)DbvAlloc( sizeof( DbvListStatus ) );
   psListStatus = *ppsListStatus;

   TBL_GetRecord( InTable );
   /* ignore title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvInternationalString( &psResultSetId, InTable );
   psListStatus->sResultSetId.ulLength = psResultSetId->ulLength;
   psListStatus->sResultSetId.pcValue  = psResultSetId->pcValue;

   /* Free doesn't free ->pcValue */
   DbvFree( psResultSetId );

   /* Delete Set Status */

   TBL_GetRecord( InTable );
   /* ignore title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvDeleteSetStatus( &stringfield, &eDeleteSetStatus );
   DbvFree( stringfield );
   psListStatus->eDeleteSetStatus = eDeleteSetStatus;

   } /* End get_DbvListStatus() */
void get_DbvNamePlusRecord( DbvNamePlusRecord **ppsResponseRecord, TBL InTable )
   {
   char *stringfield;
   DbvNamePlusRecord	*psResponseRecord;
   DbvDbRecordType	eDbRecordType;

   /* Malloc the space */

   *ppsResponseRecord = (DbvNamePlusRecord*)DbvAlloc( 
					sizeof( DbvNamePlusRecord ) );
   psResponseRecord = *ppsResponseRecord;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvInternationalString(&(*ppsResponseRecord)->psDatabaseName, InTable);

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvDbRecordType( &stringfield, &eDbRecordType );
   psResponseRecord->eDbRecordType = eDbRecordType;
   DbvFree( stringfield );


   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   { 
   DbvFree(stringfield);
   switch( eDbRecordType )
      {
      case DRT_Retrieval:
         get_DbvASN1External( &psResponseRecord->uDbRecord.psRetrievalRecord, 
				InTable);
         break;

      case DRT_SurrogateDiagnostics:

         get_DbvDiagRec( &psResponseRecord->uDbRecord.psSurrogateDiagnostic, 
				InTable);
         break;

      case DRT_StartingFragment:
         get_DbvFragmentSyntax( &psResponseRecord->uDbRecord.psStartingFragment,
				InTable);
         break;

      case DRT_IntermediateFragment:
         get_DbvFragmentSyntax( &psResponseRecord->
				uDbRecord.psIntermediateFragment,
				InTable);
         break;

      case DRT_FinalFragment:
         get_DbvFragmentSyntax( &psResponseRecord->uDbRecord.psFinalFragment,
				InTable);
         break;

      default:
	 break;
      } /* End switch */
     } /*END if */
     else
     DbvFree(stringfield);
   } /* End get_DbvNamePlusRecord() */

void get_DbvObjectIdentifier( DbvObjectIdentifier *pcObjectIdentifier, 
				TBL InTable )
   {
   char *stringfield;

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   /* Object Identifier */
   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   { 
   *pcObjectIdentifier = stringfield;
   }
   else
     DbvFree( stringfield );   
   } /* End get_DbvObjectIdentifier() */

void get_DbvQuery(DbvQuery **ppsQuery, TBL InTable)
   {
   char *stringfield;
   DbvQuery *psQuery;
   DbvQueryType  eQueryType;

   /* Malloc the space */

   *ppsQuery = (DbvQuery*)DbvAlloc( sizeof( DbvQuery ) );
   psQuery = *ppsQuery;

   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvQueryType( &stringfield, &eQueryType );
   DbvFree( stringfield );
   psQuery->eQueryType = eQueryType;
   TBL_GetRecord(InTable);

   switch( eQueryType )
      {
      case QT_Private:
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvOctetString(&(psQuery->uQuery.psPrivateQuery), InTable);
         break;

      case QT_Rpn:
	 get_DbvRPNQuery(&(psQuery->uQuery.psRpnQuery), InTable);
         break;

      case QT_Iso8777:
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvOctetString(&(psQuery->uQuery.psIso8777Query), InTable);
         break;

      case QT_Z3958:
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvOctetString(&(psQuery->uQuery.psZ3958Query), InTable);
         break;

      case QT_Erpn:
	 get_DbvRPNQuery(&(psQuery->uQuery.psRpnQuery), InTable);
         break;

      case QT_NaturalLanguage:
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvOctetString(&(psQuery->uQuery.psNaturalLanguageQuery), InTable);
         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvQuery() */
void get_DbvRecords( DbvRecords **ppsRecords, TBL InTable )
   {
   int  	intfield;
   char 	*stringfield;
   DbvRecords 	*psRecords;
   DbvRecordType 	eRecordType;
   DbvNamePlusRecord 	**ppsResponseRecord;
   DbvDiagRec		**ppsMultipleNonSurrogateDiag;

   /* Malloc the space */

   *ppsRecords = (DbvRecords*)DbvAlloc( sizeof( DbvRecords ) );
   psRecords = *ppsRecords;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvRecordType( &stringfield, &eRecordType );
   DbvFree( stringfield );
   psRecords->eRecordType = eRecordType;

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
   {
   DbvFree( stringfield );
   switch( eRecordType )
      {
      case RT_Response:

         /* Get the number of Response Records */
         TBL_GetRecord( InTable ); 

         stringfield = get_char_field( InTable, T_Return ); 
         DbvFree( stringfield );

         intfield = get_int_field( InTable, T_Return );

         /* ResponseRecord list  */

         ppsResponseRecord = ( DbvNamePlusRecord **)DbvAlloc( 
			( sizeof( DbvNamePlusRecord* ) * (intfield + 1 )));
         psRecords->uRecord.ppsResponseRecord = ppsResponseRecord;

         for ( ; intfield; intfield-- )
	    {
            get_DbvNamePlusRecord( ppsResponseRecord, InTable );
	    ppsResponseRecord++;
	    } /* End for */
 
         *ppsResponseRecord = NULL;

         break;

      case RT_NonSurrDiagnostics:
         get_DbvDefaultDiagFormat( &psRecords->uRecord.psNonSurrogateDiag, 
				   InTable );
         break;

      case RT_MultipleNonSurrDiagnostics:

         /* Get the number of Response Records */
        TBL_GetRecord( InTable );

        stringfield = get_char_field(InTable, T_Return ); 
        DbvFree( stringfield );

        intfield = get_int_field(InTable, T_Return );

         /* MultipleNonSurrogateDiag list  */

         ppsMultipleNonSurrogateDiag = ( DbvDiagRec **)DbvAlloc( 
			( sizeof( DbvDiagRec* ) * (intfield + 1 )));
         psRecords->uRecord.ppsMultipleNonSurrogateDiag = 
			ppsMultipleNonSurrogateDiag;

         for ( ; intfield; intfield-- )
	    {
            get_DbvDiagRec( ppsMultipleNonSurrogateDiag, InTable );
	    ppsMultipleNonSurrogateDiag++;
	    } /* End for */
 
         *ppsMultipleNonSurrogateDiag = NULL;

         break;

      default:
	 break;
      } /* End switch */
    } /* End if */
    else
      DbvFree( stringfield );
   } /* End get_DbvRecords() */

void get_DbvVisibleString( DbvVisibleString *pcVisibleString, TBL InTable )
   {
#ifdef DEBUG
   printf( "D - get_DbvVisibleString()\n" );
#endif
   /* The Visible String */
   *pcVisibleString = get_string_field( InTable,T_Return );
   } /* End get_DbvVisibleString() */


void get_DbvSortKeySpec(DbvSortKeySpec **ppsSortKeySpec, TBL InTable )
   {
   char *stringfield;
   DbvSortKeySpec 	*psSortKeySpec;
   DbvSortElement 	*psSortElement;
   DbvSortRelationType	eSortRelationType;
   DbvSortCaseType	eSortCaseType;

   /* Malloc the space if required */

   if ( *ppsSortKeySpec == NULL )
      *ppsSortKeySpec = (DbvSortKeySpec*)DbvAlloc( sizeof( DbvSortKeySpec ) );

   psSortKeySpec = *ppsSortKeySpec;

   /* Sort Element */

   psSortElement = &psSortKeySpec->sSortElement;
   get_DbvSortElement( &psSortElement, InTable );

   /* Sort Relation Type */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSortRelationType( &stringfield, &eSortRelationType );
   DbvFree( stringfield );
   psSortKeySpec->eSortRelationType = eSortRelationType;

   /* Sort Case Type */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSortCaseType( &stringfield, &eSortCaseType );
   DbvFree( stringfield );
   psSortKeySpec->eSortCaseType = eSortCaseType;

   /* Sort MV Action */
   /* Note : Will pass a Null to get as space was calloced */

   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);
   if((strcmp(stringfield=get_string_field(InTable, T_Return),NOTHING))!=0)
      get_DbvSortMVAction( &psSortKeySpec->psMVActionType, InTable );

   } /* End get_DbvSortKeySpec() */

void get_DbvSortElement(DbvSortElement **ppsSortElement, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvSortElement 	*psSortElement;
   DbvSortElementType 	eSortElementType;
   DbvDbSpecificSortKey **ppsDatabaseSpecific;

   /* Malloc the space if required */

   if ( *ppsSortElement == NULL )
      *ppsSortElement = (DbvSortElement*)DbvAlloc( sizeof( DbvSortElement ) );

   psSortElement = *ppsSortElement;

   /* Sort Element Type */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSortElementType( &stringfield, &eSortElementType );
   DbvFree( stringfield );
   psSortElement->eSortElementType = eSortElementType;

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   {
   DbvFree(stringfield);
   switch( eSortElementType )
      {
      case SET_Generic:
	 get_DbvSortKey( &psSortElement->uSortElement.psGeneric, InTable );
         break;

      case SET_DbSpecific:

        /* Get the number of Sort keys */
        TBL_GetRecord( InTable );

        stringfield = get_char_field(InTable, T_Return ); 
        DbvFree( stringfield );

        intfield = get_int_field(InTable, T_Return );

         /* DatabaseSpecific list  */

	 if ( intfield )
	    {
            ppsDatabaseSpecific = ( DbvDbSpecificSortKey **)DbvAlloc( 
			( sizeof( DbvDbSpecificSortKey* ) * (intfield + 1 )));
            psSortElement->uSortElement.ppsDatabaseSpecific = 
			ppsDatabaseSpecific;

            for ( ; intfield; intfield-- )
	       {
               get_DbvDbSpecificSortKey( ppsDatabaseSpecific, InTable );
	       ppsDatabaseSpecific++;
	       } /* End for */
 
            *ppsDatabaseSpecific = NULL;
	    } /* End if */

         break;

      default:
	 break;
      } /* End switch */
   }
   else
   	DbvFree(stringfield);

   } /* End get_DbvSortElement() */

void get_DbvSortKey( DbvSortKey **ppsSortKey, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvSortKey 		*psSortKey;
   DbvSortKeyType 	eSortKeyType;

   /* Malloc the space if required */

   if ( *ppsSortKey == NULL )
      *ppsSortKey = (DbvSortKey*)DbvAlloc( sizeof( DbvSortKey ) );

   psSortKey = *ppsSortKey;

   /* Sort Key Type */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSortKeyType( &stringfield, &eSortKeyType );
   DbvFree( stringfield );
   psSortKey->eSortKeyType = eSortKeyType;
   TBL_GetRecord( InTable );

   switch( eSortKeyType )
      {
      case SKT_SortField:

         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );

	 get_DbvInternationalString( &psSortKey->
					uSortElement.psSortField,
					InTable );
         break;

      case SKT_ElementSpec:
         get_DbvSpecification( &psSortKey->uSortElement.psElementSpec, 
				InTable);
         break;

      case SKT_SortAttributes:
         get_DbvSortAttributes( &psSortKey->uSortElement.psSortAttributes, 
				InTable);
         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvSortKey() */

void get_DbvSpecification( DbvSpecification **ppsSpecification, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvSpecification 		*psSpecification;

   /* Malloc the space if required */

   if ( *ppsSpecification == NULL )
      *ppsSpecification = (DbvSpecification*)DbvAlloc( sizeof( DbvSpecification ) );

   psSpecification = *ppsSpecification;

   /* Object Identifier */

   get_DbvObjectIdentifier( &psSpecification->pcSchema, InTable );

   /* psElementSpec */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvElementSpec( &psSpecification->psElementSpec, InTable );
   DbvFree( stringfield );

} /* End get_DbvSpecification() */



void get_DbvElementSpec( DbvElementSpec **ppsElementSpec, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvElementSpec 		*psElementSpec;
   DbvElementSpecType 	eElementSpecType;

   /* Malloc the space if required */

   if ( *ppsElementSpec == NULL )
      *ppsElementSpec = (DbvElementSpec*)DbvAlloc( sizeof( DbvElementSpec ) );

   psElementSpec = *ppsElementSpec;

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvElementSpecType( &stringfield, &eElementSpecType );
   DbvFree( stringfield );
   psElementSpec->eElementSpecType = eElementSpecType;

   switch( eElementSpecType )
      {
      case EST_Primitive:

         TBL_GetRecord( InTable );
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );

	 get_DbvInternationalString( &psElementSpec->
					uElementSpec.psPrimitive,
					InTable );
         break;

      case EST_External:
         get_DbvASN1External( &psElementSpec->uElementSpec.psExternal, 
				InTable);
         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvElementSpec() */

void get_DbvDbSpecificSortKey( DbvDbSpecificSortKey **ppsDbSpecificSortKey, 
			       TBL InTable )
   {
   char 			*stringfield = NULL;
   DbvDbSpecificSortKey 	*psDbSpecificSortKey;
   DbvElementSpecType 		eElementSpecType;
   DbvInternationalString	*psDatabaseName;
   DbvSortKey			*psDbSort;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsDbSpecificSortKey == NULL )
      *ppsDbSpecificSortKey = (DbvDbSpecificSortKey*)DbvAlloc( 
					sizeof( DbvDbSpecificSortKey ) );

   psDbSpecificSortKey = *ppsDbSpecificSortKey;

   /* DatabaseName */

   /* If get_DbvInternationalString is moded the cluge bellow will go */

   TBL_GetRecord( InTable );
   /* Ignore the titile */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   get_DbvInternationalString( &psDatabaseName, InTable );
   psDbSpecificSortKey->sDatabaseName.ulLength = psDatabaseName->ulLength;
   psDbSpecificSortKey->sDatabaseName.pcValue  = psDatabaseName->pcValue;

   DbvFree( psDatabaseName ); 

   /* Sort Key */

   psDbSort = &psDbSpecificSortKey->sDbSort;
   get_DbvSortKey( &psDbSort, InTable );

   } /* End get_DbvDbSpecificSortKey() */

void get_DbvSortAttributes( DbvSortAttributes **ppsSortAttributes, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvSortAttributes 		*psSortAttributes;
   DbvElementSpecType 		eElementSpecType;
   DbvInternationalString	*psDatabaseName;
   DbvSortKey			*psDbSort;
   DbvAttributeElement		**ppsAttributeElement;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsSortAttributes == NULL )
      *ppsSortAttributes = (DbvSortAttributes*)DbvAlloc( 
					sizeof( DbvSortAttributes ) );

   psSortAttributes = *ppsSortAttributes;

   /* Attribute Set Id */

   get_DbvObjectIdentifier( &psSortAttributes->pcAttributeSetId, InTable );

   /* Attribute Element */

   /* Get the Number of Database Names */

   TBL_GetRecord(InTable);
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);

   intfield = get_int_field( InTable, T_Return );

   /* Database Names */

   if ( intfield )
      {
      ppsAttributeElement = ( DbvAttributeElement **)DbvAlloc( 
			( sizeof( DbvAttributeElement* ) * (intfield + 1 )));
      psSortAttributes->ppsAttributeElement = ppsAttributeElement;

      for ( ; intfield; intfield-- )
         {
         get_DbvAttributeElement( ppsAttributeElement, InTable );
         ppsAttributeElement++;
         } /* End for */
 
      *ppsAttributeElement = NULL;
      } /* End if */
   } /* End get_DbvSortAttributes() */

void get_DbvAttributeElement( DbvAttributeElement **ppsAttributeElement, 
				TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvAttributeElement 		*psAttributeElement;
   DbvElementSpecType 		eElementSpecType;
   DbvInternationalString	*psDatabaseName;
   DbvSortKey			*psDbSort;
   DbvAttributeValue		*psAttributeValue;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsAttributeElement == NULL )
      *ppsAttributeElement = (DbvAttributeElement*)DbvAlloc( 
					sizeof( DbvAttributeElement ) );

   psAttributeElement = *ppsAttributeElement;

   /* Attribute Set Id */
   TBL_GetRecord(InTable);
   get_DbvObjectIdentifier( &psAttributeElement->pcAttributeSetId, InTable );

   /* Attribute Type */

   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   psAttributeElement->ulAttributeType = get_int_field( InTable, T_Return );

   /* Attribute Value */

   psAttributeValue = &psAttributeElement->sAttributeValue;
   get_DbvAttributeValue( &psAttributeValue, InTable );
   } /* End get_DbvAttributeElement() */

void get_DbvAttributeValue(DbvAttributeValue **ppsAttributeValue, TBL InTable)
   {
   char *stringfield;
   DbvAttributeValue *psAttributeValue;
   DbvAttributeValueType  eAttributeValueType;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsAttributeValue == NULL )
      *ppsAttributeValue = (DbvAttributeValue*)DbvAlloc( 
						sizeof( DbvAttributeValue ) );
   psAttributeValue = *ppsAttributeValue;

   /* Title */
   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvAttributeValueType( &stringfield, &eAttributeValueType );
   DbvFree( stringfield );
   psAttributeValue->eAttributeValueType = eAttributeValueType;

   switch( eAttributeValueType )
      {
      case AVT_Numeric:
	 
         TBL_GetRecord(InTable);
         /* Ignore Title  */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );

	 psAttributeValue->uAttributeValue.ulNumeric = get_int_field( InTable,
								T_Return );

         break;

      case AVT_Complex:
	 get_DbvComplexValue( &psAttributeValue->uAttributeValue.psComplexValue,
					InTable );
         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvAttributeValue() */

void get_DbvComplexValue(DbvComplexValue **ppsComplexValue, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvComplexValue 	*psComplexValue;
   DbvStringOrNumeric	**ppsList;
   Ulong		**ppulSemanticAction;
   Ulong		*pulSemanticAction;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsComplexValue == NULL )
      *ppsComplexValue = (DbvComplexValue*)DbvAlloc( sizeof( DbvComplexValue ));
   psComplexValue = *ppsComplexValue;

   /* Get the Number of String/Numerics */

   TBL_GetRecord(InTable);
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);

   intfield = get_int_field( InTable, T_Return );

   /* String Or Numeric Values */

   if ( intfield )
      {
      ppsList = ( DbvStringOrNumeric **)DbvAlloc( 
			( sizeof( DbvStringOrNumeric* ) * (intfield + 1 )));
      psComplexValue->ppsList = ppsList;

      for ( ; intfield; intfield-- )
         {
         get_DbvStringOrNumeric( ppsList, InTable );
         ppsList++;
         } /* End for */
 
      *ppsList = NULL;
      } /* End if */

   /* Get the Number of SemanticActions */

   TBL_GetRecord(InTable);
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);

   intfield = get_int_field( InTable, T_Return );

   /* Check for zero Actions */

   if ( !intfield )
      {
      psComplexValue->ppulSemanticAction = NULL;
      return;
      } /* End if no actions */

   /* Allocate Space for semantic actions */

   ppulSemanticAction = ( Ulong **)DbvAlloc(( sizeof(Ulong*) * (intfield + 1)));
   psComplexValue->ppulSemanticAction = ppulSemanticAction;

   /* Ignore title for Semantic Actions */

   TBL_GetRecord(InTable);
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);

   for ( ; intfield; intfield-- )
      {
      pulSemanticAction = ( Ulong *)DbvAlloc(sizeof( Ulong ));
      *pulSemanticAction = (Ulong)get_int_field( InTable, T_Return ); 
      *ppulSemanticAction = pulSemanticAction;
      ppulSemanticAction++;
      } /* End for */
 
   *ppulSemanticAction = NULL;
   
   } /* End get_DbvComplexValue() */

void get_DbvStringOrNumeric( DbvStringOrNumeric **ppsStringOrNumeric, 
			TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvStringOrNumericType	eStringOrNumeric;
   DbvStringOrNumeric		*psStringOrNumeric;

   /* Malloc the space if required : Must be initialised to Null  */

   if (*ppsStringOrNumeric == NULL )
       *ppsStringOrNumeric = (DbvStringOrNumeric*)DbvAlloc( 
					sizeof( DbvStringOrNumeric ));
   psStringOrNumeric = *ppsStringOrNumeric;


   /* Get StringOrNumericType */

   TBL_GetRecord( InTable );

   /* Ignore title */

   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvStringOrNumericType( &stringfield, &eStringOrNumeric );
   DbvFree( stringfield );
   psStringOrNumeric->eStringOrNumeric = eStringOrNumeric;

   TBL_GetRecord( InTable );

   /* Ignore Title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   switch( eStringOrNumeric )
      {
      case SNT_String:
         get_DbvInternationalString(&psStringOrNumeric->
					uStringOrNumeric.psString,
					InTable );
         break;

      case SNT_Numeric:
         psStringOrNumeric->uStringOrNumeric.ulNumeric = get_int_field( InTable,
								T_Return );
         break;

      default:
	 break;
      } /* End switch */
   } /* End get_DbvStringOrNumeric() */

void get_DbvSortMVAction( DbvSortMVAction **ppsDbvSortMVAction, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvSortMVActionType	eSortMVActionType;
   DbvSortMVAction	*psDbvSortMVAction;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsDbvSortMVAction == NULL )
      *ppsDbvSortMVAction = (DbvSortMVAction*)DbvAlloc( 
					sizeof( DbvSortMVAction ));
   psDbvSortMVAction = *ppsDbvSortMVAction;


   /* Get DbvSortMVActionType */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSortMVActionType( &stringfield, &eSortMVActionType );
   DbvFree( stringfield );
   psDbvSortMVAction->eSortMVActionType = eSortMVActionType;

   switch( eSortMVActionType )
      {
      case SMT_Abort:
         break;
      case SMT_Null:
         break;

      case SMT_Constant:
   	 TBL_GetRecord( InTable );
   	 /* Ignore the title */
   	 stringfield = get_char_field( InTable, T_Return );
   	 DbvFree(stringfield);
	 get_DbvOctetString( &psDbvSortMVAction->psOctetString, InTable );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvSortMVAction() */

void get_DbvAttributesPlusTerm( DbvAttributesPlusTerm **ppsAttributesPlusTerm, 
				TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvAttributesPlusTerm 	*psAttributesPlusTerm;
   DbvAttributeElement		**ppsAttributeElement;
   DbvTerm			*psTerm;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsAttributesPlusTerm == NULL )
      *ppsAttributesPlusTerm = (DbvAttributesPlusTerm*)DbvAlloc( 
					sizeof( DbvAttributesPlusTerm ) );

   psAttributesPlusTerm = *ppsAttributesPlusTerm;

   /* Attribute Elements */

   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield );

   intfield = get_int_field( InTable, T_Return );

   /* Attribute Elements */

   if ( intfield )
      {
      ppsAttributeElement = ( DbvAttributeElement **)DbvAlloc( 
			( sizeof( DbvAttributeElement* ) * (intfield + 1 )));
      psAttributesPlusTerm->ppsAttributeElement = ppsAttributeElement;

      for ( ; intfield; intfield-- )
         {
         get_DbvAttributeElement( ppsAttributeElement, InTable );
         ppsAttributeElement++;
         } /* End for */
 
      *ppsAttributeElement = NULL;
      } /* End if */

   /* Term */

   psTerm = &psAttributesPlusTerm->sTerm;
   get_DbvTerm( &psTerm, InTable );

   } /* End get_DbvAttributesPlusTerm() */

void get_DbvTerm(DbvTerm **ppsTerm, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvTerm 	*psTerm;
   DbvTermType	eTermType;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsTerm == NULL )
      *ppsTerm = (DbvTerm*)DbvAlloc( sizeof( DbvTerm ));
   psTerm = *ppsTerm;

   /* Term Type */ 
   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvTermType( &stringfield, &eTermType );
   DbvFree( stringfield );
   psTerm->eTermType = eTermType;

   switch( eTermType )
      {
      case TT_General:
         TBL_GetRecord(InTable);
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvOctetString( &psTerm->uTerm.psGeneral, InTable );
	 break;

      case TT_Numeric:
         TBL_GetRecord(InTable);
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );
         psTerm->uTerm.ulInteger = get_int_field( InTable, T_Return );
	 break;

      case TT_SimpleCharacter:
         TBL_GetRecord(InTable);
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         get_DbvInternationalString( &psTerm->uTerm.psSimpleCharacter, InTable);
	 break;

      case TT_ObjectIdentifier:
         TBL_GetRecord(InTable);
         get_DbvObjectIdentifier( &psTerm->uTerm.pcObjectIdentifier, InTable );
	 break;

      case TT_DateTime:
         TBL_GetRecord(InTable);
         get_DbvGeneralizedTime( &psTerm->uTerm.pcDateTime, InTable );
	 break;

      case TT_External:
         get_DbvASN1External( &psTerm->uTerm.psExternal, InTable );
	 break;

      case TT_IntegerAndUnit:
         get_DbvIntUnit( &psTerm->uTerm.psIntegerAndUnit, InTable );
	 break;

      case TT_Null:
         break;

      default:
	 break;
      } /* End switch */
   } /* End get_DbvTerm() */

void get_DbvGeneralizedTime( DbvGeneralizedTime *pcGeneralizedTime, 
				TBL InTable )
   {
   char *stringfield;

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( InTable,T_Return );
   if(strcmp(stringfield, NOTHING) != 0)
   {
      *pcGeneralizedTime = stringfield;
   } /* End if */
   else
   	DbvFree(stringfield);

   } /* End get_DbvGeneralizedTime() */

void get_DbvIntUnit(DbvIntUnit **ppsIntUnit, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvIntUnit 	*psIntUnit;
   DbvUnit	*psUnitUsed;
#ifdef DEBUG
   printf( "D - get_DbvIntUnit()\n" );
#endif
   /* Malloc the space if required : Must be initialised to Null  */

   if (*ppsIntUnit == NULL )
      *ppsIntUnit = (DbvIntUnit*)DbvAlloc( sizeof( DbvIntUnit ));
   psIntUnit = *ppsIntUnit;

   /* IntUnit Type */ 

   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   intfield = get_int_field( InTable, T_Return );
   psIntUnit->ulValue = intfield;

   /* Unit */

   psUnitUsed = &psIntUnit->sUnitUsed;
   get_DbvUnit( &psUnitUsed, InTable );

   } /* End get_DbvIntUnit() */

void get_DbvUnit(DbvUnit **ppsUnit, TBL InTable )
   {
   char *stringfield;
   DbvUnit 	*psUnit;
#ifdef DEBUG
   printf( "D - get_DbvUnit()\n" );
#endif

   /* Malloc the space if required : Must be initialised to Null  */

   if (*ppsUnit == NULL )
          *ppsUnit = (DbvUnit*)DbvAlloc( sizeof( DbvUnit ));
   psUnit = *ppsUnit;

   /* Unit  System */

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   
   stringfield = get_string_field( InTable, T_Return ); 
   if (( strcmp(stringfield, NOTHING )) != 0 ) 
   {
   DbvFree(stringfield);

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   get_DbvInternationalString( &psUnit->psUnitSystem, InTable );


   /* Unit type */

   TBL_GetRecord( InTable );

   /* Ignore title */

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

    stringfield = get_string_field( InTable, T_Return );
    if (( strcmp(stringfield, NOTHING )) != 0 ) 
     get_DbvStringOrNumeric( &psUnit->psUnitType, InTable );
    DbvFree(stringfield);

    /* Unit */

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if (( strcmp(stringfield, NOTHING )) != 0 )
    get_DbvStringOrNumeric( &psUnit->psUnit, InTable );
   DbvFree(stringfield);

   /* Scale Factor */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psUnit->pulScaleFactor = (Ulong*)get_integer_string( InTable );
   } /* end if */
   else
     DbvFree(stringfield);
   } /* End get_DbvUnit() */

void get_DbvListEntries(DbvListEntries **ppsListEntries, TBL InTable)
   {
   char *stringfield;
   int intfield;
   DbvListEntries 	*psListEntries;
/* DbvListEntryType 	 eListEntryType; */
   DbvEntry 		**ppsEntry;
   DbvDiagRec  		**ppsNonSurrogateDiagnostics;

   /* Malloc the space */

   if ( *ppsListEntries == NULL )
      *ppsListEntries = (DbvListEntries*)DbvAlloc( sizeof(DbvListEntries));

   psListEntries = *ppsListEntries;

   getN_DbvEntry( &psListEntries->ppsEntry, InTable );

   getN_DbvDiagRec( &psListEntries->ppsNonSurrogateDiagnostics, InTable );

   } /* End get_DbvListEntries() */

void get_DbvEntry( DbvEntry **ppsEntry, TBL InTable )
   {
   char *stringfield;
   DbvEntry 	*psEntry;
   DbvEntryType  eEntryType;

   /* Malloc the space */

   if ( *ppsEntry == NULL )
      *ppsEntry = (DbvEntry*)DbvAlloc( sizeof( DbvEntry ));

   psEntry = *ppsEntry;

   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvEntryType( &stringfield, &eEntryType );

   psEntry->eEntryType = eEntryType;
   stringfield = get_string_field( InTable, T_Return );

   if( strcmp( stringfield, NOTHING ) != 0 )
   { 
   DbvFree(stringfield);
   switch( eEntryType )
      {
      case ET_TermInfo:
         get_DbvTermInfo( &psEntry->uEntry.psTermInfo, InTable );
         break;

      case ET_SurrogateDiagnostic:
         get_DbvDiagRec( &psEntry->uEntry.psSurrogateDiagnostic, InTable );
         break;

      default:
	 break;
      } /* End switch */
     }/* end if */
    else
     DbvFree(stringfield);
   } /* End get_DbvEntry() */

void get_DbvTermInfo( DbvTermInfo **ppsTermInfo, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvTermInfo 	*psTermInfo;
   DbvTerm	         *psTerm;
   DbvAttributeElement   **ppsAttributeElement;
   DbvAttributesPlusTerm **ppsAlternativeTerm;

   /* Malloc the space */

   if ( *ppsTermInfo == NULL )
      *ppsTermInfo = (DbvTermInfo*)DbvAlloc( sizeof( DbvTermInfo ));

   psTermInfo = *ppsTermInfo;

   /* Term */

   psTerm = &psTermInfo->sTerm;
   get_DbvTerm( &psTerm, InTable );

   /* Display Term */
   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvInternationalString( &psTermInfo->psDisplayTerm, InTable );

   /* Attribute Elements */

   getN_DbvAttributeElement( &psTermInfo->ppsSuggestedAttribute, InTable );

   /* Attribute Plus Term */

   TBL_GetRecord( InTable );

   /* Ignore title */
   stringfield = get_char_field(InTable, T_Return); 
   DbvFree(stringfield);

   intfield = get_int_field( InTable, T_Return );

   /* Alternative Terms */

   if ( intfield )
      {
      ppsAlternativeTerm = ( DbvAttributesPlusTerm **)DbvAlloc( 
			( sizeof( DbvAttributesPlusTerm* ) * intfield + 1 ));
      psTermInfo->ppsAlternativeTerm = ppsAlternativeTerm;

      for ( ; intfield; intfield-- )
         {
         get_DbvAttributesPlusTerm( ppsAlternativeTerm, InTable );
         ppsAlternativeTerm++;
         } /* End for */
 
      *ppsAlternativeTerm = NULL;
      } /* End if */

   /* GlobalOccurrences */

   TBL_GetRecord( InTable );

   /* Ignore title */
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree(stringfield);
    
   psTermInfo->pulGlobalOccurrences = (Ulong*)get_integer_string( InTable );

   /* ByAttributes */
   getN_DbvOccurrenceByAttributes( &psTermInfo->ppsByAttributes, InTable );

   /* OtherInformation */
   getN_DbvOtherInformation( &psTermInfo->ppsOtherInformation, InTable);

   } /* End get_DbvTermInfo() */

void get_DbvOccurrenceByAttributes( DbvOccurrenceByAttributes 
				**ppsOccurrenceByAttributes, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvOccurrenceByAttributes 	*psOccurrenceByAttributes;

   TBL_GetRecord( InTable );
   /* Ignore title */
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield ); 
   stringfield = get_string_field(InTable, T_Return );
   if( (strcmp( stringfield, NOTHING )) == 0 )
   {
   	DbvFree(stringfield);
	return;
   }
   else
	DbvFree(stringfield);

   /* Malloc the space */

   if ( *ppsOccurrenceByAttributes == NULL )
      *ppsOccurrenceByAttributes = (DbvOccurrenceByAttributes*)DbvAlloc( 
					sizeof( DbvOccurrenceByAttributes ));

   psOccurrenceByAttributes = *ppsOccurrenceByAttributes;

   /* Get N Attribute Elements */

   getN_DbvAttributeElement( &psOccurrenceByAttributes->ppsAttributeElement,
				InTable );

   /* GlobalOccurrences */

   get_DbvOccurrence( &psOccurrenceByAttributes->psOccurrence, InTable );

   /* OtherInformation */
   getN_DbvOtherInformation( &psOccurrenceByAttributes->ppsOtherInformation, 
		InTable);
   } /* End get_DbvOccurrenceByAttributes() */

void get_DbvOccurrence( DbvOccurrence **ppsOccurrence, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvOccurrence 	*psOccurrence;
   DbvOccurrenceType	eOccurrenceType;

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsOccurrence == NULL )
      *ppsOccurrence = (DbvOccurrence*)DbvAlloc( sizeof( DbvOccurrence ));
   psOccurrence = *ppsOccurrence;

   /* Occurrence Type */ 
   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvOccurrenceType( &stringfield, &eOccurrenceType );
   psOccurrence->eOccurrenceType = eOccurrenceType;

   switch( eOccurrenceType )
      {
      case OT_Global:
         TBL_GetRecord(InTable);
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         psOccurrence->uOccurence.ulGlobal = get_int_field( InTable, T_Return);
	 break;

      case OT_ByDatabase:
	 getN_DbvOccurrenceByDb( &psOccurrence->uOccurence.ppsOccurrenceByDb,
				 InTable );
	 break;

      default:
	 break;
  
     } /* End switch */

   } /* End get_DbvOccurrence() */
void get_DbvOccurrenceByDb( DbvOccurrenceByDb 
				**ppsOccurrenceByDb, TBL InTable )
   {
   char *stringfield;
   int  intfield;
   DbvInternationalString *psDatabaseName;
   DbvOccurrenceByDb 	  *psOccurrenceByDb;

   /* Malloc the space */

   if ( *ppsOccurrenceByDb == NULL )
      *ppsOccurrenceByDb = (DbvOccurrenceByDb*)DbvAlloc( 
					sizeof( DbvOccurrenceByDb ));

   psOccurrenceByDb = *ppsOccurrenceByDb;

   /* Database */

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield );

   get_DbvInternationalString( &psDatabaseName, InTable );
   psOccurrenceByDb->sDatabaseName.ulLength = psDatabaseName->ulLength;
   psOccurrenceByDb->sDatabaseName.pcValue  = psDatabaseName->pcValue;

   DbvFree( psDatabaseName ); 

   /* Number */

   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield );
   psOccurrenceByDb->pulNumber = (Ulong*)get_integer_string( InTable );

   /* OtherInformation */
   getN_DbvOtherInformation( &psOccurrenceByDb->ppsOtherInformation, InTable);

   } /* End get_DbvOccurrenceByDb() */

void get_DbvRange(DbvRange **ppsRange, TBL InTable )
   {
   char *stringfield;
   DbvRange 	*psRange;
   DbvUnit	*psUnitUsed;

   /* Is the structure completed ? */
   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) == 0 )
   {
	DbvFree(stringfield);
	return;
   }
   else
	DbvFree(stringfield);

   /* Malloc the space if required : Must be initialised to Null  */

   if ( *ppsRange == NULL )
      *ppsRange = (DbvRange*)DbvAlloc( sizeof( DbvRange ));
   psRange = *ppsRange;

   /* ulStartingPosition */ 
   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield );
   psRange->ulStartingPosition = get_int_field( InTable, T_Return );

   /* ulNumberOfRecords */
   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return ); 
   DbvFree( stringfield );
   psRange->ulNumberOfRecords  = get_int_field( InTable, T_Return );

   } /* End get_DbvRange() */

void get_DbvRecordComposition(DbvRecordComposition **ppsRecordComposition, 
				TBL InTable)
   {
   char *stringfield;
   DbvRecordComposition *psRecordComposition;
   DbvRecordCompositionType  eRecordCompositionType;

   TBL_GetRecord(InTable);
   stringfield = get_char_field( InTable, T_Return );
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) == 0 )
   {
	DbvFree(stringfield);
        return;
   }
   else
	DbvFree(stringfield);

   /* Malloc the space */

   if ( *ppsRecordComposition == NULL )
      *ppsRecordComposition = (DbvRecordComposition*)DbvAlloc( 
					sizeof( DbvRecordComposition ) );
   psRecordComposition = *ppsRecordComposition;


   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvRecordCompositionType( &stringfield, &eRecordCompositionType );
   DbvFree( stringfield );
   psRecordComposition->eRecordCompositionType = eRecordCompositionType;

   switch( eRecordCompositionType )
      {
      case RCT_ElementSetNames:
	   TBL_GetRecord(InTable);
	   /* Ignore the title */
	   stringfield = get_char_field( InTable, T_Return );
	   DbvFree(stringfield);
	   stringfield = get_string_field( InTable, T_Return );
	   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
	   { 
 	       DbvFree(stringfield);
	       get_DbvElementSetNames( &psRecordComposition->
                uRecordComposition.psElementSetNames,InTable);
	   }
	   else
	   DbvFree(stringfield);
         break;

      case RCT_CompSpec:
         get_DbvCompSpec( &psRecordComposition-> uRecordComposition.psCompSpec,
			  InTable);
         break;

      default:
	 break;
      } /* End switch */
   } /* End get_DbvRecordComposition() */

void get_DbvCompSpec(DbvCompSpec **ppsCompSpec, TBL InTable )
   {
   char *stringfield;
   DbvCompSpec *psCompSpec;

   /* Malloc the space */

   if ( *ppsCompSpec == NULL )
      *ppsCompSpec = (DbvCompSpec*)DbvAlloc( 
					sizeof( DbvCompSpec ) );
   psCompSpec = *ppsCompSpec;

   TBL_GetRecord(InTable);

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psCompSpec->fSelectAlternativeSyntax = get_DbvBool( InTable );

   /* Generic */

   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );
   TBL_GetRecord(InTable);
   if( strcmp(stringfield, NOTHING) != 0 )
      get_DbvSpecification( &psCompSpec->psGeneric, InTable );

   DbvFree( stringfield );

   /* DbSpecific */

   getN_DbvDbSpecificComp( &psCompSpec->ppsDbSpecific, InTable );

   /* Record Syntax */
   getN_DbvObjectIdentifier( &psCompSpec->ppcRecordSyntax, InTable );

   } /* End get_DbvCompSpec() */
void get_DbvDbSpecificComp(DbvDbSpecificComp **ppsDbSpecificComp, TBL InTable )
   {
   char *stringfield;
   DbvDbSpecificComp 	  *psDbSpecificComp;
   DbvInternationalString *psDatabaseName;
   DbvSpecification	  *psSpecification;

   /* Malloc the space */

   if ( *ppsDbSpecificComp == NULL )
      *ppsDbSpecificComp = (DbvDbSpecificComp*)DbvAlloc( 
					sizeof( DbvDbSpecificComp ) );
   psDbSpecificComp = *ppsDbSpecificComp;


   TBL_GetRecord(InTable);
   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
    get_DbvInternationalString( &psDatabaseName, InTable );
   psDbSpecificComp->sDatabaseName.ulLength = psDatabaseName->ulLength;
   psDbSpecificComp->sDatabaseName.pcValue  = psDatabaseName->pcValue;
   DbvFree( psDatabaseName );

   /* DbSpecific */
   psSpecification = &psDbSpecificComp->sSpecification;
   TBL_GetRecord(InTable);
   get_DbvSpecification( &psSpecification, InTable );

   } /* End get_DbvDbSpecificComp() */

void get_DbvCorrelationInfo( DbvCorrelationInfo 
                            **ppsCorrelationInfo, TBL ExternalTable )
   {
   char *stringfield;
   DbvCorrelationInfo *psCorrelationInfo;

   if ( *ppsCorrelationInfo == NULL )
      *ppsCorrelationInfo = (DbvCorrelationInfo *)DbvAlloc(
					sizeof( DbvCorrelationInfo ));
   psCorrelationInfo = *ppsCorrelationInfo;

   /* Note */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvInternationalString( &psCorrelationInfo->psNote, ExternalTable );

   /* Id */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   psCorrelationInfo->pulId = (Ulong*)get_integer_string( ExternalTable );

   } /* End get_DbvCorrelationInfo() */

void get_DbvEsTaskPackage( DbvEsTaskPackage **ppsEsTaskPackage, 
			     TBL ExternalTable)
   {
   char *stringfield;
   DbvEsTaskPackage *psEsTaskPackage;
   DbvASN1External  *psTaskSpecificParameters;

   if ( *ppsEsTaskPackage == NULL )
      *ppsEsTaskPackage = (DbvEsTaskPackage *)DbvAlloc(
					sizeof( DbvEsTaskPackage ));
   psEsTaskPackage = *ppsEsTaskPackage;

   /* Package Type */

   TBL_GetRecord(ExternalTable);
   get_DbvObjectIdentifier( &psEsTaskPackage->pcPackageType, ExternalTable );

   /* Package Name*/
   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvInternationalString( &psEsTaskPackage->psPackageName, ExternalTable );

   /* User Id */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvInternationalString( &psEsTaskPackage->psUserId, ExternalTable );

   /* Retention Time */

   TBL_GetRecord(ExternalTable);
   stringfield= get_char_field(ExternalTable, T_Return);
   DbvFree(stringfield);
   if((strcmp(stringfield=get_string_field(ExternalTable, T_Return),NOTHING))
	!=0)
   {
	DbvFree(stringfield);
	get_DbvIntUnit( &psEsTaskPackage->psRetentionTime, ExternalTable );
   }
   else
	DbvFree(stringfield);

   /* Permissions */

   getN_DbvPermissions( &psEsTaskPackage->ppsPermissions, ExternalTable );

   /* Description */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvInternationalString( &psEsTaskPackage->psDescription, ExternalTable );

   /* Target Reference */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvOctetString( &psEsTaskPackage->psTargetReference, ExternalTable );

   /* Creation Date Time */

   TBL_GetRecord( ExternalTable );
   get_DbvGeneralizedTime( &psEsTaskPackage->pcCreationDateTime, 
			   ExternalTable );

   /* Ext Services Task Status */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvExtServicesTaskStatus( &stringfield, 
				&psEsTaskPackage->eExtServicesTaskStatus );
   DbvFree( stringfield );

   /* Package Diagnostics */

   getN_DbvDiagRec( &psEsTaskPackage->ppsPackageDiagnostics, ExternalTable );

   /* Task Specific Parameters */

   psTaskSpecificParameters = &psEsTaskPackage->sTaskSpecificParameters;
   get_DbvASN1External( &psTaskSpecificParameters, ExternalTable );
		
   } /* End get_DbvEsTaskPackage() */

void get_DbvEstimate2(DbvEstimate2 **ppsEstimate, TBL ExternalTable)
   {
   char *stringfield;
   DbvEstimate2 	*psEstimate;
   DbvStringOrNumeric	*psType;
   DbvIntUnit		*psValue;

   if ( *ppsEstimate == NULL )
      *ppsEstimate = (DbvEstimate2*)DbvAlloc( sizeof( DbvEstimate2 ) );

   psEstimate = *ppsEstimate;

   /* sType */

   psType = &psEstimate->sType;
   get_DbvStringOrNumeric( &psType, ExternalTable );

   /* psValue */
  
   psValue = &psEstimate->sValue;
   get_DbvIntUnit( &psValue, ExternalTable );
   } /* End get_DbvEstimate2() */

void get_DbvOriginPartToKeep(DbvOriginPartToKeep **ppsOriginPartToKeep, 
				TBL ExternalTable)
   {
   char *stringfield;
   DbvOriginPartToKeep 	*psOriginPartToKeep;
   DbvUpdtAction 	eAction;

   if ( *ppsOriginPartToKeep == NULL )
      *ppsOriginPartToKeep = (DbvOriginPartToKeep*)DbvAlloc( 
		sizeof( DbvOriginPartToKeep ) );

   psOriginPartToKeep = *ppsOriginPartToKeep;

   /* eAction */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtAction( &stringfield, &eAction );
   psOriginPartToKeep->eAction = eAction;

   /* sDatabaseName */
   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_compulsory_DbvInternationalString( &psOriginPartToKeep->sDatabaseName, 
					  ExternalTable) ;
		

   /* pcSchema */

   TBL_GetRecord( ExternalTable );
   get_DbvObjectIdentifier( &psOriginPartToKeep->pcSchema, ExternalTable );

   /* psElementSetName */
  
   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   get_DbvInternationalString( &psOriginPartToKeep->psElementSetName,
				ExternalTable );

   } /* End get_DbvOriginPartToKeep() */

void get_DbvPermAllowableFunctions(DbvPermAllowableFunctions 
			**ppsPermAllowableFunctions, TBL ExternalTable)
   {
   char *stringfield;
   DbvPermAllowableFunctions 	*pePermAllowableFunctions;

   TBL_GetRecord( ExternalTable );
   /* Ignore title */
   stringfield = get_char_field(ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field(ExternalTable, T_Return );
   if(strcmp( stringfield, NOTHING ) == 0)
   {
   	DbvFree(stringfield);
	return;
   }

   if ( *ppsPermAllowableFunctions == NULL )
      *ppsPermAllowableFunctions = (DbvPermAllowableFunctions*)DbvAlloc( 
				sizeof( DbvPermAllowableFunctions ) );

   pePermAllowableFunctions = *ppsPermAllowableFunctions;

   stringfield = get_string_field(  ExternalTable, T_Return );
   convert_DbvPermAllowableFunctions( &stringfield, pePermAllowableFunctions );
   DbvFree( stringfield );

   } /* End get_DbvPermAllowableFunctions() */

void get_DbvPermissions(DbvPermissions **ppsPermissions, TBL ExternalTable)
   {
   char *stringfield;
   DbvPermissions 	*psPermissions;

   if ( *ppsPermissions == NULL )
      *ppsPermissions = (DbvPermissions*)DbvAlloc( sizeof( DbvPermissions ) );

   psPermissions = *ppsPermissions;

   /* User Id */

     TBL_GetRecord( ExternalTable );
   /* Ignore title */
   stringfield = get_char_field(ExternalTable, T_Return );
   DbvFree( stringfield );

   get_compulsory_DbvInternationalString( &psPermissions->sUserId, 
					  ExternalTable );
   /* Allowable Functions */

   getN_DbvPermAllowableFunctions( &psPermissions->ppeAllowableFunctions,
				   ExternalTable );
   } /* End get_DbvPermissions() */

void get_DbvResourceReport2( DbvResourceReport2 **ppsResourceReport2, 
			     TBL ExternalTable)
   {
   char *stringfield;
   DbvResourceReport2 *psResourceReport2;

   if ( *ppsResourceReport2 == NULL )
      *ppsResourceReport2 = (DbvResourceReport2 *)DbvAlloc(
					sizeof( DbvResourceReport2 ));
   psResourceReport2 = *ppsResourceReport2;

   getN_DbvEstimate2( &psResourceReport2->ppsEstimate, ExternalTable );

   /* psMessage */

   TBL_GetRecord(ExternalTable);
   /* Ignore the title */
   stringfield=get_char_field(ExternalTable, T_Return);
   DbvFree(stringfield);

  /* psMessage */

   get_DbvInternationalString( &psResourceReport2->psMessage, ExternalTable);
		
   } /* End get_DbvResourceReport2() */

void get_DbvSuppliedRecord(DbvSuppliedRecord **ppsSuppliedRecord, 
					TBL ExternalTable)
   {
   char *stringfield;
   DbvSuppliedRecord 	*psSuppliedRecord;
   DbvASN1External	*psRecord;

   if ( *ppsSuppliedRecord == NULL )
      *ppsSuppliedRecord = (DbvSuppliedRecord*)DbvAlloc( 
						sizeof( DbvSuppliedRecord ) );

   psSuppliedRecord = *ppsSuppliedRecord;

   /* Updt Record Id */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field(ExternalTable, T_Return);
   if (( strcmp( stringfield, NOTHING )) != 0 )
      get_DbvUpdtRecordId( &psSuppliedRecord->psUpdtRecordId, ExternalTable );

   DbvFree( stringfield );

   /* Updt Supplemental Id */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field(ExternalTable, T_Return);
   if (( strcmp( stringfield, NOTHING )) != 0 )
      get_DbvUpdtSupplementalId( &psSuppliedRecord->psUpdtSupplementalId, 
   			     ExternalTable );

   DbvFree( stringfield );

   /* psCorrelationInfo */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field(ExternalTable, T_Return);
   if (( strcmp( stringfield, NOTHING )) != 0 )
      get_DbvCorrelationInfo( &psSuppliedRecord->psCorrelationInfo, 
			     ExternalTable );

   DbvFree( stringfield );

   /* Record */

   psRecord = &psSuppliedRecord->sRecord;
   get_DbvASN1External( &psRecord, ExternalTable );
  
   } /* End get_DbvSuppliedRecord() */

void get_DbvTaskPackageRecord(DbvTaskPackageRecord **ppsTaskPackageRecord, 
				TBL ExternalTable)
   {
   char *stringfield;
   DbvTaskPackageRecord *psTaskPackageRecord;
   DbvUpdtRecordStatus 	eUpdtRecordStatus;

   if ( *ppsTaskPackageRecord == NULL )
      *ppsTaskPackageRecord = (DbvTaskPackageRecord*)DbvAlloc( 
					sizeof( DbvTaskPackageRecord ) );

   psTaskPackageRecord = *ppsTaskPackageRecord;

   /* Updt Record Or Diag */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );
   stringfield = get_string_field( ExternalTable, T_Return );

   if( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvUpdtRecordOrDiag( &psTaskPackageRecord->psUpdtRecordOrDiag, 
			       ExternalTable );
   DbvFree( stringfield );

   /* Correlation Info */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );
   stringfield = get_string_field( ExternalTable, T_Return );

   if( strcmp( stringfield, NOTHING ) != 0 )
     get_DbvCorrelationInfo( &psTaskPackageRecord->psCorrelationInfo,
				ExternalTable );
   DbvFree( stringfield );

   /* Updt Record Status */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtRecordStatus( &stringfield, &eUpdtRecordStatus );
   psTaskPackageRecord->eUpdtRecordStatus = eUpdtRecordStatus;

   } /* End get_DbvTaskPackageRecord() */

void get_DbvUpdateExtendedService( DbvUpdateExtendedService 
                            **ppsUpdateExtendedService, TBL ExternalTable )
   {
   char *stringfield;
   DbvUpdateReqOrRsp eUpdateReqRsp;
   DbvUpdateExtendedService *psUpdateExtendedService;

   if ( *ppsUpdateExtendedService == NULL )
      *ppsUpdateExtendedService = (DbvUpdateExtendedService *)DbvAlloc(
					sizeof( DbvUpdateExtendedService ));
   psUpdateExtendedService = *ppsUpdateExtendedService;

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdateReqOrRsp( &stringfield, &eUpdateReqRsp );
   psUpdateExtendedService->eUpdateReqRsp = eUpdateReqRsp;
   DbvFree( stringfield );

   switch( eUpdateReqRsp )
      {
      case UROR_Request:
         get_DbvUpdateRequestPackage( &psUpdateExtendedService->
				      uUpdateReqOrRsp.psUpdateRequest, 
				      ExternalTable);
         break;

      case UROR_Response:
         get_DbvUpdateTaskPackage( &psUpdateExtendedService->
				   uUpdateReqOrRsp.psUpdateResponse, 
				   ExternalTable );
         break;

      default:
         break;
      } /* End switch */

   } /* End get_DbvUpdateExtendedService() */

void get_DbvUpdateRequestPackage( DbvUpdateRequestPackage 
                            **ppsUpdateRequestPackage, TBL ExternalTable )
   {
   char *stringfield;
   DbvOriginPartToKeep *psOriginPartToKeep;
   DbvUpdateRequestPackage *psUpdateRequestPackage;

   if ( *ppsUpdateRequestPackage == NULL )
      *ppsUpdateRequestPackage = (DbvUpdateRequestPackage *)DbvAlloc(
					sizeof( DbvUpdateRequestPackage ));
   psUpdateRequestPackage = *ppsUpdateRequestPackage;
	
   psOriginPartToKeep = &psUpdateRequestPackage->sOriginPartToKeep;
   get_DbvOriginPartToKeep( &psOriginPartToKeep, ExternalTable);

   getN_DbvSuppliedRecord( &psUpdateRequestPackage->ppsSuppliedRecord, 
				   ExternalTable );

   } /* End get_DbvUpdateRequestPackage() */

void get_DbvUpdateTargetPart(DbvUpdateTargetPart **ppsUpdateTargetPart, 
				TBL ExternalTable)
   {
   char *stringfield;
   DbvUpdateTargetPart *psUpdateTargetPart;
   DbvUpdtTaskStatus 	eTaskStatus;

   if ( *ppsUpdateTargetPart == NULL )
      *ppsUpdateTargetPart = (DbvUpdateTargetPart*)DbvAlloc( 
						sizeof( DbvUpdateTargetPart ) );

   psUpdateTargetPart = *ppsUpdateTargetPart;

   /* Task Status */

   TBL_GetRecord( ExternalTable );
   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtTaskStatus( &stringfield, &eTaskStatus );
   psUpdateTargetPart->eTaskStatus = eTaskStatus;

   /* Global Diagnostics */

   getN_DbvDiagRec( &psUpdateTargetPart->ppsGlobalDiagnostics, ExternalTable );

   /* Task Package Record */

   getN_DbvTaskPackageRecord( &psUpdateTargetPart->ppsTaskPackageRecord, 
			      ExternalTable );

   } /* End get_DbvUpdateTargetPart() */

void get_DbvUpdateTaskPackage( DbvUpdateTaskPackage 
                            **ppsUpdateTaskPackage, TBL ExternalTable )
   {
   DbvOriginPartToKeep *psOriginPartToKeep;
   DbvUpdateTargetPart *psTargetPart;
   DbvUpdateTaskPackage *psUpdateTaskPackage;

   if ( *ppsUpdateTaskPackage == NULL )
      *ppsUpdateTaskPackage = (DbvUpdateTaskPackage *)DbvAlloc(
					sizeof( DbvUpdateTaskPackage ));
   psUpdateTaskPackage = *ppsUpdateTaskPackage;

   /* Origin Part To Keep */

   psOriginPartToKeep = &psUpdateTaskPackage->sOriginPartToKeep;
   get_DbvOriginPartToKeep( &psOriginPartToKeep, ExternalTable);

   /* Target Part */

   psTargetPart = &psUpdateTaskPackage->sTargetPart;
   get_DbvUpdateTargetPart( &psTargetPart, ExternalTable );

   } /* End get_DbvUpdateTaskPackage() */

void get_DbvUpdtRecordId( DbvUpdtRecordId 
                            **ppsUpdtRecordId, TBL ExternalTable )
   {
   char *stringfield;
   DbvUpdtRecordIdType eRecordIdType;
   DbvUpdtRecordId *psUpdtRecordId;

   if ( *ppsUpdtRecordId == NULL )
      *ppsUpdtRecordId = (DbvUpdtRecordId *)DbvAlloc(
					sizeof( DbvUpdtRecordId ));
   psUpdtRecordId = *ppsUpdtRecordId;

   /* Record Id Type */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtRecordIdType( &stringfield, &eRecordIdType );
   psUpdtRecordId->eRecordIdType = eRecordIdType;
   DbvFree( stringfield );

   switch( eRecordIdType )
      {
      case URIT_Number:
         TBL_GetRecord( ExternalTable );

         /* Ignore the title */
         stringfield = get_char_field( ExternalTable, T_Return );
         DbvFree( stringfield );

         psUpdtRecordId->uUpdtRecordId.ulNumber = get_int_field( ExternalTable,
								 T_Return );
         break;

      case URIT_String:
         TBL_GetRecord( ExternalTable );

         /* Ignore the title */
         stringfield = get_char_field( ExternalTable, T_Return );
         DbvFree( stringfield );

	 /* Note this IntStr should not be optional */

         get_DbvInternationalString( &psUpdtRecordId->
				     uUpdtRecordId.psString, 
				     ExternalTable );
         break;

      case URIT_Opaque:
         TBL_GetRecord( ExternalTable );

         /* Ignore the title */
         stringfield = get_char_field( ExternalTable, T_Return );
         DbvFree( stringfield );

	 /* Note this OctStr should not be optional */

         get_DbvOctetString( &psUpdtRecordId->uUpdtRecordId.psOpaque, 
				     ExternalTable );

         break;

      default:
         break;
      } /* End switch */

   } /* End get_DbvUpdtRecordId() */

void get_DbvUpdtRecordOrDiag( DbvUpdtRecordOrDiag **ppsUpdtRecordOrDiag, 
			      TBL ExternalTable )
   {
   char *stringfield;
   DbvUpdtRecordOrDiagType eUpdtRecordOrDiag;
   DbvUpdtRecordOrDiag *psUpdtRecordOrDiag;

   if ( *ppsUpdtRecordOrDiag == NULL )
      *ppsUpdtRecordOrDiag = (DbvUpdtRecordOrDiag *)DbvAlloc(
					sizeof( DbvUpdtRecordOrDiag ));
   psUpdtRecordOrDiag = *ppsUpdtRecordOrDiag;

   /* Updt Record Or Diag */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtRecordOrDiagType( &stringfield, &eUpdtRecordOrDiag );
   psUpdtRecordOrDiag->eUpdtRecordOrDiag = eUpdtRecordOrDiag;
   DbvFree( stringfield );

   switch( eUpdtRecordOrDiag )
      {
      case URDT_Record:
         get_DbvASN1External( &psUpdtRecordOrDiag->
			      uUpdtRecordOrDiag.psRecord, 
			      ExternalTable );
         break;

      case URDT_Diagnostic:
         get_DbvDiagRec( &psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic, 
				     ExternalTable );
         break;

      default:
         break;
      } /* End switch */

   } /* End get_DbvUpdtRecordOrDiag() */

void get_DbvUpdtSupplementalId( DbvUpdtSupplementalId 
                            **ppsUpdtSupplementalId, TBL ExternalTable )
   {
   char *stringfield;
   DbvUpdtSupplementalIdType eSupplementalIdType;
   DbvUpdtSupplementalId *psUpdtSupplementalId;

   if ( *ppsUpdtSupplementalId == NULL )
      *ppsUpdtSupplementalId = (DbvUpdtSupplementalId *)DbvAlloc(
					sizeof( DbvUpdtSupplementalId ));
   psUpdtSupplementalId = *ppsUpdtSupplementalId;

   /* Supplemental Id Type */

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree( stringfield );

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvUpdtSupplementalIdType( &stringfield, &eSupplementalIdType );
   psUpdtSupplementalId->eSupplementalIdType = eSupplementalIdType;
   DbvFree( stringfield );

   switch( eSupplementalIdType )
      {
      case USIT_Time:
         TBL_GetRecord( ExternalTable );
	 get_DbvGeneralizedTime( &psUpdtSupplementalId->
				 uUpdtSupplementalId.pcTime,
				 ExternalTable );
         break;

      case USIT_Version:
         TBL_GetRecord( ExternalTable );

         /* Ignore the title */
         stringfield = get_char_field( ExternalTable, T_Return );
         DbvFree( stringfield );

	 /* Note this IntStr should not be optional */

         get_DbvInternationalString( &psUpdtSupplementalId->
				     uUpdtSupplementalId.psVersion, 
				     ExternalTable );
         break;

      case USIT_PreviousVersion:
         get_DbvASN1External( &psUpdtSupplementalId->
			      uUpdtSupplementalId.psPreviousVersion, 
			      ExternalTable );

         break;

      default:
         break;
      } /* End switch */

   } /* End get_DbvUpdtSupplementalId() */

void get_DbvRPNQuery(DbvRPNQuery **ppsRpnQuery, TBL InTable)
{
        char *rpnquerytype = NULL;
	char *stringfield = NULL;
 
        /* Operands */
        char *opnd1 = NULL, *opnd2 = NULL, *opnd3 = NULL;
 
        /* Attributes */
        char *att1 = NULL, *att2 = NULL, *att3 = NULL;
 
        /* Operators */
        char *oper1 = NULL, *oper2 = NULL;

	/* Proximity operators */
	DbvProximityOperator *psProximityOperator1 = NULL, 
		*psProximityOperator2 = NULL;
 
        DBVRpnQueryType eRpnQueryType;
        DbvRPNStructure *rpnstruct = NULL;
 
	*ppsRpnQuery = (DbvRPNQuery *)DbvAlloc(sizeof(DbvRPNQuery));
 
	get_DbvObjectIdentifier(&(*ppsRpnQuery)->pcAttributeSetId, InTable);

        TBL_GetRecord(InTable); 

	/* Ignore the title */
	stringfield=get_char_field(InTable, T_Return );
	if(stringfield != NULL)
		DbvFree(stringfield);

        rpnquerytype = get_string_field( InTable, T_Return );
        convert_DbvRpnQueryType(&rpnquerytype, &eRpnQueryType );
 
        if (rpnquerytype != NULL)
                DbvFree(rpnquerytype);
 
        TBL_GetRecord(InTable); 

        switch (eRpnQueryType) {
 
        /* "john", "1,2:3,5:5,6:"; */
        case RPN_Q1:
                opnd1 = get_string_field( InTable, T_Return );
                att1 = get_string_field( InTable, T_Return );
                rpnstruct = form_dbv_rpn_1(opnd1, att1);
                DbvFree(opnd1);
                DbvFree(att1);
                break;
 
 
        /* "john", "1,2:3,5:5,6:", "AND", "smith", "5,6:1,2:7,8:" */
        case RPN_Q2:
                opnd1 = get_string_field( InTable, T_Return );
                att1 = get_string_field( InTable, T_Return );
                oper1 = get_string_field( InTable, T_Return );
		if(strcmp(oper1, "OT_Proximity") == 0)
			get_DbvProximityOperator(&psProximityOperator1,InTable);
                opnd2 = get_string_field( InTable, T_Return );
                att2 = get_string_field( InTable, T_Return );
                rpnstruct = form_dbv_rpn_2(opnd1, att1, oper1, 
			psProximityOperator1, opnd2, att2);
                DbvFree(opnd1);
                DbvFree(att1);
                DbvFree(opnd2);
                DbvFree(att2);
                break;
 
        /* "chris", "7,7:8,8:", "AND", "smith", "5,6:1,2:7,8:", "OR", "pandora",
 "5,7:3,3:4,4:" */
        case RPN_Q3:
                opnd1 = get_string_field( InTable, T_Return );
                att1 = get_string_field( InTable, T_Return );
                oper1 = get_string_field( InTable, T_Return );
		if(strcmp(oper1, "OT_Proximity") == 0)
			get_DbvProximityOperator(&psProximityOperator1,InTable);
                opnd2 = get_string_field( InTable, T_Return );
                att2 = get_string_field( InTable, T_Return );
                oper2 = get_string_field( InTable, T_Return );
		if(strcmp(oper2, "OT_Proximity") == 0)
			get_DbvProximityOperator(&psProximityOperator2,InTable);
                opnd3 = get_string_field( InTable, T_Return );
                att3 = get_string_field( InTable, T_Return );
                rpnstruct = form_dbv_rpn_3(opnd1, att1, oper1, 
			psProximityOperator1, opnd2, att2, oper2, 
			psProximityOperator2, opnd3, att3);
                DbvFree(opnd1);
                DbvFree(att1);
                DbvFree(opnd2);
                DbvFree(att2);
                DbvFree(opnd3);
                DbvFree(att3);
                break;
        }

        /*psQuery->uQuery.psRpnQuery = rpnstruct;*/

	(*ppsRpnQuery)->sRpnStructure.eRpnStructureType = rpnstruct->
		eRpnStructureType;

	switch(rpnstruct->eRpnStructureType)
	{
	case RST_RpnLopRopOperator:
		(*ppsRpnQuery)->sRpnStructure.uRPNStructure.
			psRpnLopRopOperator = rpnstruct->uRPNStructure.
			psRpnLopRopOperator;
		break;

	case RST_RpnOperand:
		(*ppsRpnQuery)->sRpnStructure.uRPNStructure.psRpnOperand =
			rpnstruct->uRPNStructure.psRpnOperand;
		break;
	default:
		printf("In get_DbvRPNQuery eRpnStructureType type is invalid\n");
		sleep(5);
		break;
	}
 
}

DbvRPNStructure *form_dbv_rpn_1(char *operand_string, char *atts)
{
 
        DbvRPNStructure *rpnstruct = NULL;
 
        rpnstruct = (DbvRPNStructure *) DbvAlloc (sizeof(DbvRPNStructure));
        rpnstruct->eRpnStructureType = RST_RpnOperand;
 
        rpnstruct->uRPNStructure.psRpnOperand = (DbvOperand *) DbvAlloc 
		(sizeof(DbvOperand));
 
        /* Fill the operand */
        fill_dbv_operand(rpnstruct->uRPNStructure.psRpnOperand, operand_string,
		atts);
 
        return rpnstruct;
}
 

void fill_dbv_operand(DbvOperand *psRpnOperand, char *operand_string, char *atts)
{
        /* Attributes plus term */
        psRpnOperand->eOperandType = OT_AttributePlusTerm;
        psRpnOperand->uOperand.psAttributesPlusTerm = (DbvAttributesPlusTerm *) 
		DbvAlloc (sizeof(DbvAttributesPlusTerm));
        fill_dbv_attrsplusterm(psRpnOperand->uOperand.psAttributesPlusTerm, 
		operand_string, atts);
}

void fill_dbv_attrsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm, 
	char *operand_string, char *atts)
{
 
        /* attributes */
 
        /* 6 attributes + 1 NULL */
        psAttributesPlusTerm->ppsAttributeElement = (DbvAttributeElement **) 
		DbvAlloc (sizeof(DbvAttributeElement *) * 7);
 
        fill_dbv_attterm(psAttributesPlusTerm->ppsAttributeElement, atts);
 
        /* Now the term  - make it a general  */
        psAttributesPlusTerm->sTerm.eTermType = TT_General;
        psAttributesPlusTerm->sTerm.uTerm.psGeneral = (DbvOctetString *) 
		DbvAlloc (sizeof (DbvOctetString));
        psAttributesPlusTerm->sTerm.uTerm.psGeneral->ulLength = 
		strlen(operand_string);
        psAttributesPlusTerm->sTerm.uTerm.psGeneral->pcValue = (char *) 
		DbvAlloc(strlen(operand_string));
        memcpy(psAttributesPlusTerm->sTerm.uTerm.psGeneral->pcValue, 
		operand_string, strlen(operand_string));
}

void fill_dbv_attterm(DbvAttributeElement **ppsAttributeElement, char *attributes)
{
        int index = 0;
        char *ptr = NULL;
        unsigned long atttype = 0, attval = 0;
        char attributes_copy[MAXVAL];
	char *oid = NULL;
 
        strcpy(attributes_copy, attributes);
 
        ptr = (char *) strtok (attributes_copy, ",");
 
        while (ptr != NULL) {
		oid = ptr;
                atttype= atol((char *) strtok(NULL, ","));
                attval = atol((char *) strtok(NULL, ":"));
 
                ppsAttributeElement[index] = (DbvAttributeElement *) DbvAlloc 
			(sizeof(DbvAttributeElement));
 
                /* DONT need an object identifier for each attribute pair */
 
		if(strcmp(oid, NOTHING) != 0)
		{
			
			ppsAttributeElement[index]->pcAttributeSetId = 
				(char *)DbvAlloc(strlen(oid) + 1);
			strcpy(ppsAttributeElement[index]->pcAttributeSetId,
				oid);
		}

                ppsAttributeElement[index]->ulAttributeType = atttype;
                ppsAttributeElement[index]->sAttributeValue.eAttributeValueType
			= AVT_Numeric;
                ppsAttributeElement[index]->sAttributeValue.uAttributeValue.
			ulNumeric = attval;
 
                ptr = (char *) strtok (NULL, ",");
 
                index++;
        }
 
        /* Terminate the list */
        ppsAttributeElement[index] = NULL;
}
 
DbvRPNStructure *form_dbv_rpn_2(char *operand_string1, char *atts1, 
	char *operator, DbvProximityOperator *psProximityOperator1, 
	char *operand_string2, char *atts2)
{
 
        DbvRPNStructure *rpnstruct = NULL;
        DbvRPNStructure *rpnstruct1 = NULL;
        DbvRPNStructure *rpnstruct2 = NULL;
 
 
        /* Left side of lop */
        rpnstruct1 = form_dbv_rpn_1(operand_string1, atts1);
 
        /* Right side of lop */
        rpnstruct2 = form_dbv_rpn_1(operand_string2, atts2);
 
 
        /* Now form that rpn  */
        rpnstruct = (DbvRPNStructure *) DbvAlloc (sizeof(DbvRPNStructure));
        /* A lop roper */
        rpnstruct->eRpnStructureType = RST_RpnLopRopOperator;
 
        rpnstruct->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *) 
		DbvAlloc (sizeof(DbvRPNLopRopOperator));
        rpnstruct->uRPNStructure.psRpnLopRopOperator->psLeftOperand = rpnstruct1;
        rpnstruct->uRPNStructure.psRpnLopRopOperator->psRightOperand = rpnstruct2;
 
        fill_operator(&rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator,
		operator);

        if(rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator.
            eOperatorType == OT_Proximity)
		rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator.
			psProximityOperator = psProximityOperator1;
 
        return rpnstruct;
}

void fill_operator(DbvOperator *psOperator, char *operator_string)
{
        if (strcmp(operator_string, "OT_And") == 0)
                psOperator->eOperatorType = OT_And;
 
        else if (strcmp(operator_string, "OT_Or") == 0)
                psOperator->eOperatorType = OT_Or;
 
        else if (strcmp(operator_string, "OT_AndNot") == 0)
                psOperator->eOperatorType = OT_AndNot;

	else if (strcmp(operator_string, "OT_Proximity") == 0)
		psOperator->eOperatorType = OT_Proximity;
	else
		printf("psOperator->eOperatorType type unknown!\n");

}
 
 
DbvRPNStructure *form_dbv_rpn_3(char *operand_string1, char *atts1, 
	char *operator1, DbvProximityOperator *psProximityOperator1, 
	char *operand_string2, char *atts2, char *operator2, 
	DbvProximityOperator *psProximityOperator2, char *operand_string3, 
	char *atts3)
{
 
        DbvRPNStructure *rpnstruct = NULL;
        DbvRPNStructure *rpnstruct1 = NULL;
        DbvRPNStructure *rpnstruct2 = NULL;
 
 
        /* Left side of lop */
        rpnstruct1 = form_dbv_rpn_2(operand_string1, atts1, operator1, 
		psProximityOperator1, operand_string2, atts2);
 
        /* Right side of lop */
        rpnstruct2 = form_dbv_rpn_1(operand_string3, atts3);
 
 
        /* Now form that rpn  */
        rpnstruct = (DbvRPNStructure *) DbvAlloc (sizeof(DbvRPNStructure));
        /* A lop roper */
        rpnstruct->eRpnStructureType = RST_RpnLopRopOperator;
 
        rpnstruct->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *)
		DbvAlloc (sizeof(DbvRPNLopRopOperator));
        rpnstruct->uRPNStructure.psRpnLopRopOperator->psLeftOperand = 
		rpnstruct1;
        rpnstruct->uRPNStructure.psRpnLopRopOperator->psRightOperand = 
		rpnstruct2;
        fill_operator(&rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator,
		operator2);

	if(rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator.
            eOperatorType == OT_Proximity)
		rpnstruct->uRPNStructure.psRpnLopRopOperator->sOperator.
			psProximityOperator = psProximityOperator2;
 
        return rpnstruct;
}
 

void get_DbvProximityOperator(DbvProximityOperator **ppsProximityOperator,
	TBL InTable)
{
   char *stringfield = NULL;

   TBL_GetRecord( InTable );

   *ppsProximityOperator = (DbvProximityOperator *)
	DbvAlloc(sizeof(DbvProximityOperator));

   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);
   stringfield = get_string_field(InTable, T_Return);

   if(strcmp(stringfield, NOTHING) != 0)
   {
	(*ppsProximityOperator)->pfExclusion = (DbvBool *)
		DbvAlloc(sizeof(DbvBool));
	*(*ppsProximityOperator)->pfExclusion = get_DbvBool(InTable);
   }

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   (*ppsProximityOperator)->ulDistance = get_int_field(InTable, T_Return);

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   (*ppsProximityOperator)->fOrdered = get_DbvBool(InTable);

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   stringfield = get_string_field(InTable, T_Return);
   convert_DbvProximityRelationType(&stringfield, &(*ppsProximityOperator)->
	eProximityRelationType);
   DbvFree(stringfield);

   get_DbvProximityUnitCode(&(*ppsProximityOperator)->sProximityUnitCode,
	InTable);

   TBL_GetRecord( InTable );

} /* End get_DbvProximityOperator */

/* This one's just a pointer */
void get_DbvProximityUnitCode(DbvProximityUnitCode *psProximityUnitCode,
	TBL InTable)
{

   char *stringfield = NULL;


   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   stringfield = get_string_field(InTable, T_Return);
   convert_DbvProximityUnitType( &stringfield, &psProximityUnitCode->
	eProximityUnitType);
   DbvFree(stringfield);

   TBL_GetRecord( InTable );
   /* Ignore the title */
   stringfield = get_char_field(InTable, T_Return);
   DbvFree(stringfield);

   switch(psProximityUnitCode->eProximityUnitType)
   {
   case PUT_Known:
   	stringfield = get_string_field(InTable, T_Return);
	convert_DbvKnownProximityUnit(&stringfield, 
		&psProximityUnitCode->uProximityUnit.eKnownProximityUnit);
	DbvFree(stringfield);
	break;

   case PUT_Private:
	psProximityUnitCode->uProximityUnit.ulPrivate = get_int_field(InTable,
		T_Return);
	break;

   default:
	break;
   }

} /* End get_DbvProximityUnitCode */

/* 5-2-96 The following function where added by Dan Maskell  */


void get_DbvIllString( DbvIllString **ppsIllString, TBL InTable )
   {
   char *stringfield;
   DbvIllString		*psIllString;
   DbvIllStringType	eStringType;

#ifdef DEBUG
   printf( "D - get_DbvIllString()\n" );
#endif

   /* Malloc the space */

   if ( *ppsIllString == NULL )
      *ppsIllString = (DbvIllString*)DbvAlloc( sizeof( DbvIllString ) );
   psIllString = *ppsIllString;

   /*   TBL_GetRecord( InTable ); To remain consistant with 	*/
   /* get_DbvInternationalString() this is commented out 	*/

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvIllStringType( &stringfield, &eStringType );
   psIllString->eStringType = eStringType;
   DbvFree( stringfield );

   switch( eStringType )
      {
      case ST_GeneralString:
         get_DbvGeneralString( &psIllString->uString.psGeneralString, InTable );
         break;

      case ST_EdifactString:
         get_DbvVisibleString( &psIllString->uString.pcVisibleString, InTable );
         break;

      default:
	 printf( "Warning: DbvIllString() : unknown type\n" );
	 break;
      } /* End switch */
     
   } /* End get_DbvIllString() */


void get_DbvPersonOrInstitutionSymbol(
	DbvPersonOrInstitutionSymbol **ppsPersonOrInstitutionSymbol, 
	TBL InTable )
   {
   char *stringfield;
   DbvPersonOrInstitutionSymbol		*psPersonOrInstitutionSymbol;
   DbvPersOrInstSymbolType		ePersonOrInstitutionSymbolType;
   DbvIllString				*psSymbol;

#ifdef DEBUG
   printf( "D - get_DbvPersonOrInstitutionSymbol()\n" );
#endif
   /* Malloc the space */

   if ( *ppsPersonOrInstitutionSymbol == NULL )
      *ppsPersonOrInstitutionSymbol = (DbvPersonOrInstitutionSymbol*)DbvAlloc(
			sizeof( DbvPersonOrInstitutionSymbol ) );
   psPersonOrInstitutionSymbol = *ppsPersonOrInstitutionSymbol;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvPersOrInstSymbolType( &stringfield, 
					&ePersonOrInstitutionSymbolType );
   psPersonOrInstitutionSymbol->eSymbolType = ePersonOrInstitutionSymbolType;
   DbvFree( stringfield );

   /* This reads the symbol regardless of symbol type */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   psSymbol = &psPersonOrInstitutionSymbol->sSymbol;
   get_DbvIllString( &psSymbol, InTable );

   } /* End get_DbvPersonOrInstitutionSymbol() */


void get_DbvNameOfPersonOrInstitution(
	DbvNameOfPersonOrInstitution **ppsNameOfPersonOrInstitution, 
	TBL InTable )
   {
   char *stringfield;
   DbvNameOfPersonOrInstitution	*psNameOfPersonOrInstitution;
   DbvNameOfPersOrInstType	eNameType;
   DbvIllString			*psName;

#ifdef DEBUG
   printf( "D - get_DbvNameOfPersonOrInstitution()\n" );
#endif
   /* Malloc the space */

   if ( *ppsNameOfPersonOrInstitution == NULL )
      *ppsNameOfPersonOrInstitution = (DbvNameOfPersonOrInstitution*)DbvAlloc(
			sizeof( DbvNameOfPersonOrInstitution ) );
   psNameOfPersonOrInstitution = *ppsNameOfPersonOrInstitution;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvNameOfPersOrInstType( &stringfield, &eNameType );
   psNameOfPersonOrInstitution->eNameType = eNameType;
   DbvFree( stringfield );

   /* This reads the name regardless of type */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psName = &psNameOfPersonOrInstitution->sName;
   get_DbvIllString( &psName, InTable );

   } /* End get_DbvNameOfPersonOrInstitution() */


void get_DbvSystemId( DbvSystemId **ppsSystemId, TBL InTable )
   {
   char *stringfield;
   DbvSystemId	*psSystemId;
   DbvNameOfPersOrInstType	eNameType;
   DbvIllString			*psName;

#ifdef DEBUG
   printf( "D - get_DbvSystemId()\n" );
#endif
   /* Malloc the space */

   if ( *ppsSystemId == NULL )
      *ppsSystemId = (DbvSystemId*)DbvAlloc(
			sizeof( DbvSystemId ) );
   psSystemId = *ppsSystemId;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvPersonOrInstitutionSymbol( &psSystemId->psSymbol, InTable );
   DbvFree(stringfield);

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvNameOfPersonOrInstitution( &psSystemId->psName, InTable );
   DbvFree(stringfield);

   } /* End get_DbvSystemId() */


void get_DbvTransactionId( DbvTransactionId **ppsTransactionId, TBL InTable )
   {
   char *stringfield;
   DbvTransactionId	*psTransactionId;
   DbvIllString		*psTransactionGroupQualifier;
   DbvIllString		*psTransactionQualifier;

#ifdef DEBUG
   printf( "D - get_DbvTransactionId()\n" );
#endif
   /* Malloc the space */

   if ( *ppsTransactionId == NULL )
      *ppsTransactionId = (DbvTransactionId*)DbvAlloc(sizeof(DbvTransactionId));

   psTransactionId = *ppsTransactionId;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvSystemId( &psTransactionId->psInitialRequesterId, InTable );
   DbvFree(stringfield);

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   /* sTransactionGroupQualifier */
   psTransactionGroupQualifier = &psTransactionId->sTransactionGroupQualifier;
   get_DbvIllString( &psTransactionGroupQualifier, InTable );

   /* sTransactionQualifier */
   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psTransactionQualifier= &psTransactionId->sTransactionQualifier;
   get_DbvIllString( &psTransactionQualifier, InTable );

   /* psSubTransactionQualifier */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psTransactionId->psSubTransactionQualifier, InTable );
   DbvFree(stringfield);

   } /* End get_DbvTransactionId() */


void get_DbvDateTime( DbvDateTime **ppsDateTime, TBL InTable )
   {
   char *stringfield;
   DbvDateTime	*psDateTime;

#ifdef DEBUG
   printf( "D - get_DbvDateTime()\n" );
#endif
   /* Malloc the space */

   if ( *ppsDateTime == NULL )
      *ppsDateTime = (DbvDateTime*)DbvAlloc(sizeof(DbvDateTime));

   psDateTime = *ppsDateTime;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   get_DbvVisibleString( &psDateTime->pcDate, InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvVisibleString( &psDateTime->pcTime, InTable );
     
   DbvFree(stringfield);

   } /* End get_DbvDateTime() */


void get_DbvServiceDateTime( DbvServiceDateTime **ppsServiceDateTime, 
			     TBL InTable )
   {
   char *stringfield;
   DbvServiceDateTime	*psServiceDateTime;
   DbvDateTime		*psThisService;

#ifdef DEBUG
   printf( "D - get_DbvServiceDateTime()\n" );
#endif
   /* Malloc the space */

   if ( *ppsServiceDateTime == NULL )
      *ppsServiceDateTime = (DbvServiceDateTime*)DbvAlloc(sizeof(DbvServiceDateTime));

   psServiceDateTime = *ppsServiceDateTime;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psThisService = &psServiceDateTime->sThisService;
   get_DbvDateTime( &psThisService, InTable );

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvDateTime( &psServiceDateTime->psOriginalService, InTable );
     
   DbvFree(stringfield);

   } /* End get_DbvServiceDateTime() */


void get_DbvPostalAddress( DbvPostalAddress **ppsPostalAddress, TBL InTable )
   {
   char *stringfield;
   DbvPostalAddress	*psPostalAddress;
   DbvDateTime		*psThisService;

#ifdef DEBUG
   printf( "D - get_DbvPostalAddress()\n" );
#endif
   /* Malloc the space */

   if ( *ppsPostalAddress == NULL )
      *ppsPostalAddress = (DbvPostalAddress*)DbvAlloc(sizeof(DbvPostalAddress));

   psPostalAddress = *ppsPostalAddress;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   /* psNameOfPersonOrInstitution */

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvNameOfPersonOrInstitution( &psPostalAddress->
					psNameOfPersonOrInstitution,
					InTable );
   DbvFree(stringfield);

   /* psExtendedPostalDeliveryAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psExtendedPostalDeliveryAddress, 
			 InTable );
     
   DbvFree(stringfield);

   /* psStreetAndNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psStreetAndNumber, InTable );
     
   DbvFree(stringfield);

   /* psPostOfficeBox */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psPostOfficeBox, InTable );
     
   DbvFree(stringfield);

   /* psCity */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psCity, InTable );
     
   DbvFree(stringfield);

   /* psRegion */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psRegion, InTable );
     
   DbvFree(stringfield);

   /* psCountry */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psCountry, InTable );
     
   DbvFree(stringfield);

   /* psPostalCode */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psPostalAddress->psPostalCode, InTable );
     
   DbvFree(stringfield);

   } /* End get_DbvPostalAddress() */


void get_DbvSystemAddress( DbvSystemAddress **ppsSystemAddress, TBL InTable )
   {
   char *stringfield;
   DbvSystemAddress	*psSystemAddress;
   DbvDateTime		*psThisService;

#ifdef DEBUG
   printf( "D - get_DbvSystemAddress()\n" );
#endif
   /* Malloc the space */

   if ( *ppsSystemAddress == NULL )
      *ppsSystemAddress = (DbvSystemAddress*)DbvAlloc(sizeof(DbvSystemAddress));

   psSystemAddress = *ppsSystemAddress;

   /* psTelecomServiceIdentifier */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psSystemAddress->psTelecomServiceIdentifier, 
			 InTable );
     
   DbvFree(stringfield);

   /* psTelecomServiceAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvIllString( &psSystemAddress->psTelecomServiceAddress, 
			 InTable );
     
   DbvFree(stringfield);

   } /* End get_DbvSystemAddress() */


void get_DbvDeliveryAddress( DbvDeliveryAddress **ppsDeliveryAddress, 
			     TBL InTable )
   {
   char *stringfield;
   DbvDeliveryAddress	*psDeliveryAddress;

   /* Malloc the space */

   if ( *ppsDeliveryAddress == NULL )
      *ppsDeliveryAddress = (DbvDeliveryAddress*)DbvAlloc(
					sizeof(DbvDeliveryAddress));

   psDeliveryAddress = *ppsDeliveryAddress;

   TBL_GetRecord( InTable );

   /* psPostalAddress */

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvPostalAddress( &psDeliveryAddress->psPostalAddress, InTable );
     
   DbvFree(stringfield);

   /* psElectronicAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
      get_DbvSystemAddress( &psDeliveryAddress->psElectronicAddress, InTable );
     
   DbvFree(stringfield);

   } /* End get_DbvDeliveryAddress() */


void get_DbvElDeliveryDetails( DbvElDeliveryDetails **ppsElDeliveryDetails, 
			       TBL InTable )
   {
   char *stringfield;
   DbvElDeliveryDetails		*psElDeliveryDetails;
   DbvElDeliveryDetailsType	eElDeliveryDetailsType;

   /* Malloc the space */

   if ( *ppsElDeliveryDetails == NULL )
      *ppsElDeliveryDetails = (DbvElDeliveryDetails*)DbvAlloc( 
				sizeof( DbvElDeliveryDetails ) );
   psElDeliveryDetails = *ppsElDeliveryDetails;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvElDeliveryDetailsType( &stringfield, &eElDeliveryDetailsType );
   psElDeliveryDetails->eDeliveryDetailsType = eElDeliveryDetailsType;
   DbvFree( stringfield );

   switch( eElDeliveryDetailsType )
      {
      case DDT_SystemAddress:
         get_DbvSystemAddress( &psElDeliveryDetails->
				uDeliveryDetails.psSystemAddress,
				InTable);
         break;

      case DDT_SystemId:

	 /* Get that System Id pre stuff */

	TBL_GetRecord( InTable );
   	stringfield = get_char_field( InTable, T_Return );
   	DbvFree(stringfield);

   	stringfield = get_string_field( InTable, T_Return );
   	if( (strcmp(stringfield, NOTHING)) != 0 )

		/* Now get the actual system id */
		get_DbvSystemId( &psElDeliveryDetails->uDeliveryDetails.psSystemId, 
				InTable );

	DbvFree(stringfield);

         break;

      default:
	 break;
      } /* End switch */

   } /* End get_DbvElDeliveryDetails() */


void get_DbvElectronicDeliveryService( 
	DbvElectronicDeliveryService **ppsElectronicDeliveryService,
	TBL InTable )
   {
   char *stringfield;
   DbvElectronicDeliveryService	*psElectronicDeliveryService;
   DbvElDeliveryDetails		*psDeliveryDetails;

   /* Malloc the space */

   if ( *ppsElectronicDeliveryService == NULL )
      *ppsElectronicDeliveryService = (DbvElectronicDeliveryService*)DbvAlloc( 
				sizeof( DbvElectronicDeliveryService ) );
   psElectronicDeliveryService = *ppsElectronicDeliveryService;

   /* psDeliveryParameter */
   TBL_GetRecord( InTable );

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      get_DbvElDeliveryParameter( &psElectronicDeliveryService->
				  psDeliveryParameter,
				  InTable );
   DbvFree( stringfield );

   /* psDocumentTypeParameter */
   TBL_GetRecord( InTable );

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      get_DbvDocumentTypeParameter( &psElectronicDeliveryService->
				  psDocumentTypeParameter,
				  InTable );
   DbvFree( stringfield );
   
   /* psDeliveryDescription */

   TBL_GetRecord( InTable );

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      get_DbvIllString( &psElectronicDeliveryService->
				  psDeliveryDescription,
				  InTable );
   DbvFree( stringfield );

   /* sDeliveryDetails */

   psDeliveryDetails = &psElectronicDeliveryService->sDeliveryDetails;
   get_DbvElDeliveryDetails( &psDeliveryDetails, InTable );

   /* psNameOrCode */

   TBL_GetRecord( InTable );

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      get_DbvIllString( &psElectronicDeliveryService->
				  psNameOrCode,
				  InTable );
   DbvFree( stringfield );

   /* pcDeliveryTime */

   TBL_GetRecord( InTable );

   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      get_DbvVisibleString( &psElectronicDeliveryService->
				  pcDeliveryTime,
				  InTable );
   DbvFree( stringfield );

   } /* End get_DbvElectronicDeliveryService() */


void get_DbvDeliveryService( DbvDeliveryService **ppsDeliveryService, 
			     TBL InTable )
   {
   char *stringfield;
   DbvDeliveryService		*psDeliveryService;
   DbvDeliveryServiceType	eDeliveryServiceType;

   /* Malloc the space */

   if ( *ppsDeliveryService == NULL )
      *ppsDeliveryService = (DbvDeliveryService*)DbvAlloc( 
					sizeof( DbvDeliveryService ) );
   psDeliveryService = *ppsDeliveryService;

   TBL_GetRecord( InTable );

   /* Ignore the title - (e.g. type  ) */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);


   /* Get "DST_PhysicalDelivery" | "DST_ElectronicDelivery" */

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvDeliveryServiceType( &stringfield, &eDeliveryServiceType );
   psDeliveryService->eDeliveryServiceType = eDeliveryServiceType;
   DbvFree( stringfield );


#ifdef NO
   if( ( strcmp( stringfield, NOTHING ) ) != 0 )
      { 
      DbvFree(stringfield);
#endif

      switch( eDeliveryServiceType )
         {
         case DST_PhysicalDelivery:
            TBL_GetRecord( InTable );
            stringfield = get_char_field( InTable, T_Return );
            DbvFree(stringfield);
   	    stringfield = get_string_field( InTable, T_Return );
   	    if( ( strcmp( stringfield, NOTHING ) ) != 0 )
            	get_DbvIllString( &psDeliveryService->
				      uDeliveryService.psPhysicalDelivery,
				      InTable);
            break;

         case DST_ElectronicDelivery:
            getN_DbvElectronicDeliveryService(
				&psDeliveryService->
				uDeliveryService.ppsElectronicDelivery, 
	   			InTable );
            break;

         default:
	    break;
         } /* End switch */

#ifdef NO
      } /* End if */
   else
      DbvFree(stringfield);

#endif


   } /* End get_DbvDeliveryService() */


void get_DbvRequesterOptionalMessages( 
		DbvRequesterOptionalMessages **ppsRequesterOptionalMessages, 
		TBL InTable )
   {
   char *stringfield;
   DbvOptionalMsgRequirements eOptionalMsgRequirements;
   DbvRequesterOptionalMessages  *psRequesterOptionalMessages;

   /* Malloc the space */

   if ( *ppsRequesterOptionalMessages == NULL )
      *ppsRequesterOptionalMessages = (DbvRequesterOptionalMessages*)DbvAlloc( 
					sizeof( DbvRequesterOptionalMessages ));

   psRequesterOptionalMessages = *ppsRequesterOptionalMessages;

   /* fCanSendReceived */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psRequesterOptionalMessages->fCanSendReceived = get_DbvBool( InTable );

   /* fCanSendReturned */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psRequesterOptionalMessages->fCanSendReturned = get_DbvBool( InTable );

   /* eRequesterShipped */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvOptionalMsgRequirements( &stringfield, &eOptionalMsgRequirements);
   psRequesterOptionalMessages->eRequesterShipped = eOptionalMsgRequirements;
   DbvFree( stringfield );

   /* eRequesterCheckedIn */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvOptionalMsgRequirements( &stringfield, &eOptionalMsgRequirements);
   psRequesterOptionalMessages->eRequesterCheckedIn = eOptionalMsgRequirements;
   DbvFree( stringfield );

   } /* End get_DbvRequesterOptionalMessages() */


void get_DbvSearchType( DbvSearchType **ppsSearchType, TBL InTable )
   {
   char *stringfield;
   DbvSearchType *psSearchType;

   /* Malloc the space */

   if ( *ppsSearchType == NULL )
      *ppsSearchType = (DbvSearchType*)DbvAlloc( sizeof( DbvSearchType ));

   psSearchType = *ppsSearchType;

   /* psLevelOfService */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psSearchType->psLevelOfService, InTable );

   DbvFree(stringfield);

   /* pcNeedBeforeDate */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvVisibleString( &psSearchType->pcNeedBeforeDate, InTable );

   DbvFree(stringfield);

   /* peExpiryFlag  - default */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvExpiry( &psSearchType->peExpiryFlag, InTable );
   DbvFree(stringfield);


   /* pcExpiryDate */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvVisibleString( &psSearchType->pcExpiryDate, InTable );

   DbvFree(stringfield);
   } /* End get_DbvSearchType() */


void get_DbvExpiry( DbvExpiry **ppsExpiry, TBL InTable )
   {
   char *stringfield;
   DbvExpiry *peExpiry;

   /* Malloc the space */

   if ( *ppsExpiry == NULL )
      *ppsExpiry = (DbvExpiry*)DbvAlloc( sizeof( DbvExpiry ));

   peExpiry = *ppsExpiry;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvExpiry( &stringfield, peExpiry);
   DbvFree( stringfield );

   } /* End get_DbvExpiry() */


void get_DbvSupplyMediumInfo( DbvSupplyMediumInfo **ppsSupplyMediumInfo, 
			      TBL InTable )
   {
   char *stringfield;
   DbvSupplyMediumType eSupplyMediumType;
   DbvSupplyMediumInfo *psSupplyMediumInfo;

   /* Malloc the space */

   if ( *ppsSupplyMediumInfo == NULL )
      *ppsSupplyMediumInfo = (DbvSupplyMediumInfo*)DbvAlloc( 
					sizeof( DbvSupplyMediumInfo ));

   psSupplyMediumInfo = *ppsSupplyMediumInfo;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvSupplyMediumType( &stringfield, &eSupplyMediumType );
   DbvFree( stringfield );

   psSupplyMediumInfo->eSupplyMediumType = eSupplyMediumType;
   /* psMediumCharacteristics */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psSupplyMediumInfo->psMediumCharacteristics, InTable );

   DbvFree(stringfield);
   } /* End get_DbvSupplyMediumInfo() */


void get_DbvClientId( DbvClientId **ppsClientId, TBL InTable )
   {
   char *stringfield;
   DbvClientId *psClientId;

   /* Malloc the space */

   if ( *ppsClientId == NULL )
      *ppsClientId = (DbvClientId*)DbvAlloc( sizeof( DbvClientId ));

   psClientId = *ppsClientId;

   /* psClientName */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psClientId->psClientName, InTable );

   DbvFree(stringfield);

   /* psClientStatus */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psClientId->psClientStatus, InTable );

   DbvFree(stringfield);

   /* psClientIdentifier */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psClientId->psClientIdentifier, InTable );

   DbvFree(stringfield);

   } /* End get_DbvClientId() */


void get_DbvItemId( DbvItemId **ppsItemId, TBL InTable )
   {
   char *stringfield;
   DbvItemId *psItemId;

   /* Malloc the space */

   if ( *ppsItemId == NULL )
      *ppsItemId = (DbvItemId*)DbvAlloc( sizeof( DbvItemId ));

   psItemId = *ppsItemId;

   /* peItemType */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvItemType( &psItemId->peItemType, InTable );

   DbvFree(stringfield);

   /* peMediumType */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvMediumType( &psItemId->peMediumType, InTable );

   DbvFree(stringfield);

   /* psCallNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psCallNumber, InTable );

   DbvFree(stringfield);

   /* psAuthor */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psAuthor, InTable );

   DbvFree(stringfield);

   /* psTitle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psTitle, InTable );

   DbvFree(stringfield);

   /* psSubTitle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psSubTitle, InTable );

   DbvFree(stringfield);

   /* psSponsoringBody */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psSponsoringBody, InTable );

   DbvFree(stringfield);

   /* psPlaceOfPublication */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psPlaceOfPublication, InTable );

   DbvFree(stringfield);

   /* psPublisher */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psPublisher, InTable );

   DbvFree(stringfield);

   /* psSeriesTitleNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psSeriesTitleNumber, InTable );

   DbvFree(stringfield);

   /* psVolumeIssue */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psVolumeIssue, InTable );

   DbvFree(stringfield);

   /* psEdition */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psEdition, InTable );

   DbvFree(stringfield);

   /* psPublicationDate */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psPublicationDate, InTable );

   DbvFree(stringfield);

   /* psPublicationDateOfComponent */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psPublicationDateOfComponent, InTable );

   DbvFree(stringfield);

   /* psAuthorOfArticle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psAuthorOfArticle, InTable );

   DbvFree(stringfield);

   /* psTitleOfArticle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psTitleOfArticle, InTable );

   DbvFree(stringfield);

   /* psPagination */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psPagination, InTable );

   DbvFree(stringfield);

   /* psNationalBibliographyNo */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvASN1External( &psItemId->psNationalBibliographyNo, InTable );

   DbvFree(stringfield);

   /* psIsbn */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psIsbn, InTable );

   DbvFree(stringfield);

   /* psIssn */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psIssn, InTable );

   DbvFree(stringfield);

   /* psSystemNo */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvASN1External( &psItemId->psSystemNo, InTable );

   DbvFree(stringfield);

   /* psAdditionalNoLetters */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psAdditionalNoLetters, InTable );

   DbvFree(stringfield);

   /* psVerificationReferenceSource */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psItemId->psVerificationReferenceSource, InTable );

   DbvFree(stringfield);

   } /* End get_DbvItemId() */


void get_DbvItemType( DbvItemType **ppeItemType, TBL InTable )
   {
   char *stringfield;
   DbvItemType *peItemType;

   /* Malloc the space */

   if ( *ppeItemType == NULL )
      *ppeItemType = (DbvItemType*)DbvAlloc( sizeof( DbvItemType ));

   peItemType = *ppeItemType;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvItemType( &stringfield, peItemType);
   DbvFree( stringfield );

   } /* End get_DbvItemType() */


void get_DbvMediumType( DbvMediumType **ppeMediumType, TBL InTable )
   {
   char *stringfield;
   DbvMediumType *peMediumType;

   /* Malloc the space */

   if ( *ppeMediumType == NULL )
      *ppeMediumType = (DbvMediumType*)DbvAlloc( sizeof( DbvMediumType ));

   peMediumType = *ppeMediumType;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvMediumType( &stringfield, peMediumType);
   DbvFree( stringfield );

   } /* End get_DbvMediumType() */


void get_DbvAmount( DbvAmount **ppsAmount, TBL InTable )
   {
   char *stringfield;
   DbvAmount *psAmount;

   /* Malloc the space */

   if ( *ppsAmount == NULL )
      *ppsAmount = (DbvAmount*)DbvAlloc( sizeof( DbvAmount ));

   psAmount = *ppsAmount;

   /* pcCurrencyCode */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvPrintableString( &psAmount->pcCurrencyCode, InTable );

   DbvFree(stringfield);

   /* pcMonetaryValue */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   get_DbvPrintableString( &psAmount->pcMonetaryValue, InTable );

   } /* End get_DbvAmount() */

void get_DbvPrintableString( DbvPrintableString *pcPrintableString, 
			     TBL InTable )
   {
   /* The Printable String */

   /* Character set FROM 1 2 3 4 5 6 7 8 9 0 . , and space  */

   *pcPrintableString = get_string_field( InTable,T_Return );
   } /* End get_DbvPrintableString() */


void get_DbvCostInfo( DbvCostInfo **ppsCostInfo, TBL InTable )
   {
   char *stringfield;
   DbvCostInfo *psCostInfo;
   DbvBool     *pBool;

   /* Malloc the space */

   if ( *ppsCostInfo == NULL )
      *ppsCostInfo = (DbvCostInfo*)DbvAlloc( sizeof( DbvCostInfo ));

   psCostInfo = *ppsCostInfo;

   /* psAccountNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psCostInfo->psAccountNumber, InTable );

   DbvFree(stringfield);

   /* psMaximumCost */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvAmount( &psCostInfo->psMaximumCost, InTable );

   DbvFree(stringfield);


   /* pfReciprocalAgreement - default */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psCostInfo->pfReciprocalAgreement = pBool;
   }
   DbvFree(stringfield);



   /* pfWillPayFee - default */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {


   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psCostInfo->pfWillPayFee= pBool;
    }
    DbvFree(stringfield);




   /* pfPaymentProvided - default */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {


   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psCostInfo->pfPaymentProvided = pBool;
   }
   DbvFree(stringfield);


   } /* End get_DbvCostInfo() */


void get_DbvSendToList( DbvSendToList **ppsSendToList, TBL InTable )
   {
   char *stringfield;
   DbvSendToList *psSendToList;
   DbvSystemId   *psSystemId;

   /* Malloc the space */

   if ( *ppsSendToList == NULL )
      *ppsSendToList = (DbvSendToList*)DbvAlloc( sizeof( DbvSendToList ));

   psSendToList = *ppsSendToList;

   /* sSystemId */

   psSystemId = &psSendToList->sSystemId;
   get_DbvSystemId( &psSystemId, InTable );

   /* psAccountNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psSendToList->psAccountNumber, InTable );

   DbvFree(stringfield);

   /* psSystemAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemAddress( &psSendToList->psSystemAddress, InTable );

   DbvFree(stringfield);

   } /* End get_DbvSendToList() */


void get_DbvThirdPartyInfo( DbvThirdPartyInfo **ppsThirdPartyInfo, 
			    TBL InTable )
   {
   char *stringfield;
   DbvThirdPartyInfo *psThirdPartyInfo;
   DbvSystemId   *psSystemId;
   DbvBool	 *pBool;

   /* Malloc the space */

   if ( *ppsThirdPartyInfo == NULL )
      *ppsThirdPartyInfo = (DbvThirdPartyInfo*)DbvAlloc( 
					sizeof( DbvThirdPartyInfo ));

   psThirdPartyInfo = *ppsThirdPartyInfo;

   /* pfPermissionToForward */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psThirdPartyInfo->pfPermissionToForward = pBool;
   }
   DbvFree(stringfield);


   /* pfPermissionToChain */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psThirdPartyInfo->pfPermissionToChain = pBool;
   }
   DbvFree(stringfield);


   /* pfPermissionToPartition */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psThirdPartyInfo->pfPermissionToPartition = pBool;
   }
   DbvFree(stringfield);


   /* pfPermissionToChangeSendToList */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psThirdPartyInfo->pfPermissionToChangeSendToList = pBool;
   }
   DbvFree(stringfield);


   /* psInitialRequesterAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemAddress( &psThirdPartyInfo->
			    psInitialRequesterAddress, InTable);
   DbvFree(stringfield);

   /* pePreference */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvPreference( &psThirdPartyInfo->pePreference, InTable );
   DbvFree(stringfield);


   /* ppsSendToList */

   getN_DbvSendToList( &psThirdPartyInfo->ppsSendToList, InTable );

   /* ppsAlreadyTriedList */

   getN_DbvSystemId( &psThirdPartyInfo->ppsAlreadyTriedList, InTable );
   } /* End get_DbvThirdPartyInfo() */



void get_DbvPreference( DbvPreference **ppsPreference, TBL InTable )
   {
   char *stringfield;
   DbvPreference *pePreference;

   /* Malloc the space */

   if ( *ppsPreference == NULL )
      *ppsPreference = (DbvPreference*)DbvAlloc( sizeof( DbvPreference ));

   pePreference = *ppsPreference;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvPreference( &stringfield, pePreference);
   DbvFree(stringfield);

   } /* End get_DbvPreference() */



void get_DbvIllRequest( DbvIllRequest **ppsIllRequest, TBL InTable )
   {
   char *stringfield;
   DbvIllRequest		*psIllRequest;
   DbvTransactionId 		*psTransactionId;
   DbvServiceDateTime 		*psServiceDateTime;
   DbvRequesterOptionalMessages *psRequesterOptionalMessages;
   DbvItemId			*psItemId;
   DbvBool			*pBool;

   /* Malloc the space */

   if ( *ppsIllRequest == NULL )
      *ppsIllRequest = (DbvIllRequest*)DbvAlloc( sizeof( DbvIllRequest ));

   psIllRequest = *ppsIllRequest;

   /* eProtocolVersion */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvIllProtocolVersion( &stringfield, 
				  &psIllRequest->eProtocolVersion);
   DbvFree( stringfield );

   /* sTransactionId */

   psTransactionId = &psIllRequest->sTransactionId;
   get_DbvTransactionId( &psTransactionId, InTable );

   /* sServiceDateTime */

   psServiceDateTime = &psIllRequest->sServiceDateTime;
   get_DbvServiceDateTime( &psServiceDateTime, InTable );

   /* psRequesterId */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemId( &psIllRequest->psRequesterId, InTable) ;

   DbvFree(stringfield);

   /* psResponderId */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemId( &psIllRequest->psResponderId, InTable) ;

   DbvFree(stringfield);

   /* peTransactionType */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvTransactionType( &psIllRequest->peTransactionType, InTable );
   DbvFree(stringfield);


   /* psDeliveryAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvDeliveryAddress( &psIllRequest->psDeliveryAddress, InTable );
   
   DbvFree(stringfield);

   /* psDeliveryService */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvDeliveryService( &psIllRequest->psDeliveryService, InTable );
   
   DbvFree(stringfield);

   /* psBillingAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvDeliveryAddress( &psIllRequest->psBillingAddress, InTable );
   
   DbvFree(stringfield);

   /* ppeServiceType */

   getN_DbvServiceType( &psIllRequest->ppeServiceType, InTable );

   /* psResponderSpecificService */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvASN1External( &psIllRequest->psResponderSpecificService, InTable );
   
   DbvFree(stringfield);

   /* sRequesterOptionalMessages */

   psRequesterOptionalMessages = &psIllRequest->sRequesterOptionalMessages;
   get_DbvRequesterOptionalMessages( &psRequesterOptionalMessages, InTable );

   /* psSearchType */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSearchType( &psIllRequest->psSearchType, InTable );
   
   DbvFree(stringfield);

   /* ppsSupplyMediumInfo */

   getN_DbvSupplyMediumInfo( &psIllRequest->ppsSupplyMediumInfo, InTable );

   /* pePlaceOnHold */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvPlaceOnHold( &psIllRequest->pePlaceOnHold, InTable );
   DbvFree(stringfield);


   /* psClientId */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvClientId( &psIllRequest->psClientId, InTable );
   
   DbvFree(stringfield);

   /* sItemId */
   psItemId = &psIllRequest->sItemId;
   get_DbvItemId( &psItemId, InTable );

   /* ppsSupplementalItemDescription */

   getN_DbvASN1External( &psIllRequest->ppsSupplementalItemDescription, 
			 InTable );
   /* psCostInfo */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvCostInfo( &psIllRequest->psCostInfo, InTable );
   
   DbvFree(stringfield);

   /* psCopyrightCompliance */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psIllRequest->psCopyrightCompliance, InTable );
   
   DbvFree(stringfield);

   /* psThirdPartyInfo */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvThirdPartyInfo( &psIllRequest->psThirdPartyInfo, InTable );
   
   DbvFree(stringfield);


   /* pfRetry - default */
   
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psIllRequest->pfRetry = pBool;
   }
   DbvFree(stringfield);


   /* pfForward - default */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psIllRequest->pfForward = pBool;
   }
   DbvFree(stringfield);


   /* psRequesterNote */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psIllRequest->psRequesterNote, InTable );
   
   DbvFree(stringfield);

   /* psForwardNote */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psIllRequest->psForwardNote, InTable );
   
   DbvFree(stringfield);

   /* ppsRequestExtension */

   getN_DbvExtension( &psIllRequest->ppsRequestExtension, InTable );

#ifdef DEBUG
   printf( "D - Returning get_DbvIllRequest()\n" );
#endif
   } /* End get_DbvIllRequest() */


void get_DbvServiceType( DbvServiceType **ppsServiceType, TBL InTable )
   {
   char *stringfield;
   DbvServiceType *peServiceType;

   /* Malloc the space */

   if ( *ppsServiceType == NULL )
      *ppsServiceType = (DbvServiceType*)DbvAlloc( sizeof( DbvServiceType ));

   peServiceType = *ppsServiceType;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvServiceType( &stringfield, peServiceType);
   DbvFree( stringfield );

   } /* End get_DbvServiceType() */


void get_DbvTransactionType( DbvTransactionType **ppsTransactionType, 
			     TBL InTable )
   {
   char *stringfield;
   DbvTransactionType *peTransactionType;

   /* Malloc the space */

   if ( *ppsTransactionType == NULL )
      *ppsTransactionType = (DbvTransactionType*)DbvAlloc( 
						sizeof( DbvTransactionType ));

   peTransactionType = *ppsTransactionType;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvTransactionType( &stringfield, peTransactionType);
   DbvFree( stringfield );

   } /* End get_DbvTransactionType() */


void get_DbvPlaceOnHold( DbvPlaceOnHold **ppePlaceOnHold, TBL InTable )
   {
   char *stringfield;
   DbvPlaceOnHold *pePlaceOnHold;

   /* Malloc the space */

   if ( *ppePlaceOnHold == NULL )
      *ppePlaceOnHold = (DbvPlaceOnHold*)DbvAlloc( sizeof( DbvPlaceOnHold ));

   pePlaceOnHold = *ppePlaceOnHold;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvPlaceOnHold( &stringfield, pePlaceOnHold);
   DbvFree( stringfield );

   } /* End get_DbvPlaceOnHold() */


void get_DbvHistoryReport( DbvHistoryReport **ppsHistoryReport, TBL InTable )
   {
   char *stringfield;
   DbvHistoryReport 	*psHistoryReport;
   DbvSystemId		*psSystemId;
   /* Malloc the space */

   if ( *ppsHistoryReport == NULL )
      *ppsHistoryReport = (DbvHistoryReport*)DbvAlloc( 
					sizeof( DbvHistoryReport ));

   psHistoryReport = *ppsHistoryReport;

   /* pcDateRequested */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvVisibleString( &psHistoryReport->pcDateRequested, InTable );
   
   DbvFree(stringfield);

   /* psAuthor */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psHistoryReport->psAuthor, InTable );
   
   DbvFree(stringfield);

   /* psTitle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psHistoryReport->psTitle, InTable );
   
   DbvFree(stringfield);

   /* psAuthorOfArticle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psHistoryReport->psAuthorOfArticle, InTable );
   
   DbvFree(stringfield);

   /* psTitleOfArticle */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psHistoryReport->psTitleOfArticle, InTable );
   
   DbvFree(stringfield);

   /* pcDateOfLastTransition */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvVisibleString( &psHistoryReport->pcDateOfLastTransition, InTable );
   
   DbvFree(stringfield);

   /* eMostRecentService */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvMostRecentService( &stringfield, 
				  &psHistoryReport->eMostRecentService );
   DbvFree( stringfield );

   /* pcDateOfMostRecentService */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   get_DbvVisibleString( &psHistoryReport->pcDateOfMostRecentService, InTable );
   
   /* sSystemId */

   psSystemId = &psHistoryReport->sSystemId;
   get_DbvSystemId( &psSystemId, InTable );

   /* peShippedServiceType */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvShippedServiceType( &psHistoryReport->peShippedServiceType, 
				 InTable );
   
   DbvFree(stringfield);

   /* peTransactionResults */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvTransactionResults( &psHistoryReport->peTransactionResults, 
				 InTable );
   
   DbvFree(stringfield);

   /* psMostRecentServiceNote */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psHistoryReport->psMostRecentServiceNote, InTable );
   
   DbvFree(stringfield);

   } /* End get_DbvHistoryReport() */



void get_DbvShippedServiceType( DbvShippedServiceType **ppeShippedServiceType, 
				TBL InTable )
   {
   char *stringfield;
   DbvShippedServiceType *peShippedServiceType;

   /* Malloc the space */

   if ( *ppeShippedServiceType == NULL )
      *ppeShippedServiceType = (DbvShippedServiceType*)DbvAlloc( 
					sizeof( DbvShippedServiceType ));

   peShippedServiceType = *ppeShippedServiceType;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvShippedServiceType( &stringfield, peShippedServiceType);
   DbvFree( stringfield );

   } /* End get_DbvShippedServiceType() */



void get_DbvTransactionResults( DbvTransactionResults **ppeTransactionResults, 
				TBL InTable )
   {
   char *stringfield;
   DbvTransactionResults *peTransactionResults;

   /* Malloc the space */

   if ( *ppeTransactionResults == NULL )
      *ppeTransactionResults = (DbvTransactionResults*)DbvAlloc( 
					sizeof( DbvTransactionResults ));

   peTransactionResults = *ppeTransactionResults;

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvTransactionResults( &stringfield, peTransactionResults);
   DbvFree( stringfield );

   } /* End get_DbvTransactionResults() */


void get_DbvStateTransitionProhibited( 
		DbvStateTransitionProhibited **ppsStateTransitionProhibited, 
		TBL InTable )
   {
   char *stringfield;
   DbvStateTransitionProhibited *psStateTransitionProhibited;
   /* Malloc the space */

   if ( *ppsStateTransitionProhibited == NULL )
      *ppsStateTransitionProhibited = (DbvStateTransitionProhibited*)DbvAlloc( 
					sizeof( DbvStateTransitionProhibited ));

   psStateTransitionProhibited = *ppsStateTransitionProhibited;

   /* eAPDUType */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvIllAPDUType( &stringfield, 
			  &psStateTransitionProhibited->eAPDUType);
   DbvFree( stringfield );

   /* eCurrentState */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvCurrentState( &stringfield, 
			  &psStateTransitionProhibited->eCurrentState);
   DbvFree( stringfield );

   } /* End get_DbvStateTransitionProhibited() */


void get_DbvAlreadyForwarded( DbvAlreadyForwarded **ppsAlreadyForwarded, 
			      TBL InTable )
   {
   char *stringfield;
   DbvSystemId *psResponderId;
   DbvAlreadyForwarded *psAlreadyForwarded;

   /* Malloc the space */

   if ( *ppsAlreadyForwarded == NULL )
      *ppsAlreadyForwarded = (DbvAlreadyForwarded*)DbvAlloc( 
					sizeof( DbvAlreadyForwarded ));

   psAlreadyForwarded = *ppsAlreadyForwarded;

   /* sResponderId */

   psResponderId = &psAlreadyForwarded->sResponderId;
   get_DbvSystemId( &psResponderId, InTable );

   /* psResponderAddress */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemAddress( &psAlreadyForwarded->psResponderAddress, InTable );
   
   DbvFree(stringfield);
   } /* End get_DbvAlreadyForwarded() */


void get_DbvProviderErrorReport( 
		DbvProviderErrorReport **ppsProviderErrorReport,
		TBL InTable )
   {
   char *stringfield;
   DbvProviderErrorReport	*psProviderErrorReport;
   DbvProviderErrorReportType	eReportType;

   /* Malloc the space */

   if ( *ppsProviderErrorReport == NULL )
      *ppsProviderErrorReport = (DbvProviderErrorReport*)DbvAlloc( 
				sizeof( DbvProviderErrorReport ) );
   psProviderErrorReport = *ppsProviderErrorReport;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvProviderErrorReportType( &stringfield, &eReportType );
   psProviderErrorReport->eReportType = eReportType;
   DbvFree( stringfield );

   switch( eReportType )
      {
      case PER_GeneralProblem:
         TBL_GetRecord( InTable );
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         stringfield = get_string_field( InTable, T_Return );
         convert_DbvGeneralProblem( &stringfield, 
				    &psProviderErrorReport->
				    uProviderErrorReport.eGeneralProblem);
         DbvFree( stringfield );
         break;

      case PER_TransactionIdProblem:
         TBL_GetRecord( InTable );
         /* Ignore the title */
         stringfield = get_char_field( InTable, T_Return );
         DbvFree(stringfield);
         stringfield = get_string_field( InTable, T_Return );
         convert_DbvTransactionIdProblem( &stringfield, 
				    &psProviderErrorReport->
				    uProviderErrorReport.eTransactionIdProblem);
         DbvFree( stringfield );
         break;

      case PER_StateTransitionProhibited:
	 get_DbvStateTransitionProhibited( 
				&psProviderErrorReport->
				uProviderErrorReport.psStateTransitionProblem, 
				InTable );
	 break;

      default:
	 break;
      } /* End switch */
     
   } /* End get_DbvProviderErrorReport() */


void get_DbvUserErrorReport( DbvUserErrorReport **ppsUserErrorReport,
		TBL InTable )
   {
   char *stringfield;
   DbvUserErrorReport		*psUserErrorReport;
   DbvUserErrorReportType	eReportType;

   /* Malloc the space */

   if ( *ppsUserErrorReport == NULL )
      *ppsUserErrorReport = (DbvUserErrorReport*)DbvAlloc( 
				sizeof( DbvUserErrorReport ) );
   psUserErrorReport = *ppsUserErrorReport;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvUserErrorReportType( &stringfield, &eReportType );
   psUserErrorReport->eReportType = eReportType;
   DbvFree( stringfield );

   switch( eReportType )
      {
      case UER_AlreadyForwarded:
	 get_DbvAlreadyForwarded( &psUserErrorReport->
				  uUserErrorReport.psAlreadyForwarded,
				  InTable );
         break;

      case UER_IntermediaryProblem:
         TBL_GetRecord( InTable );
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );
         stringfield = get_string_field( InTable, T_Return );
         convert_DbvIntermediaryProblem( &stringfield, 
				    &psUserErrorReport->
				    uUserErrorReport.eIntermediaryProblem);
         DbvFree( stringfield );
         break;

      case UER_SecurityProblem:
         TBL_GetRecord( InTable );
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );
	 get_DbvIllString( &psUserErrorReport->
			   uUserErrorReport.psSecurityProblem, 
			   InTable );
	 break;

      case UER_UnableToPerform:
         TBL_GetRecord( InTable );
         stringfield = get_char_field( InTable, T_Return );
         DbvFree( stringfield );
         stringfield = get_string_field( InTable, T_Return );
         convert_DbvUnableToPerform( &stringfield, 
				    &psUserErrorReport->
				    uUserErrorReport.eUnableToPerform );
         DbvFree( stringfield );
	 break;

      default:
	 break;
      } /* End switch */
     
   } /* End get_DbvUserErrorReport() */



void get_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport **ppsUserOrProviderErrorReport,
		TBL InTable )
   {
   char *stringfield;
   DbvUserOrProviderErrorReport	*psUserOrProviderErrorReport;
   DbvReportSource		eReportSource;

   /* Malloc the space */

   if ( *ppsUserOrProviderErrorReport == NULL ) 
      *ppsUserOrProviderErrorReport = (DbvUserOrProviderErrorReport*)DbvAlloc( 
				sizeof( DbvUserOrProviderErrorReport ) );
   psUserOrProviderErrorReport = *ppsUserOrProviderErrorReport;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvReportSource( &stringfield, &eReportSource );
   psUserOrProviderErrorReport->eReportSource = eReportSource;
   DbvFree( stringfield );

   switch( eReportSource )
      {
      case RS_Provider:
	 get_DbvProviderErrorReport( &psUserOrProviderErrorReport->
				  uReport.psProviderErrorReport,
				  InTable );
         break;

      case RS_User:
	 get_DbvUserErrorReport( &psUserOrProviderErrorReport->
				  uReport.psUserErrorReport,
				  InTable );
         break;

      default:
	 break;
      } /* End switch */
     
   } /* End get_DbvUserOrProviderErrorReport() */


void get_DbvErrorReport( DbvErrorReport **ppsErrorReport, TBL InTable )
   {
   char *stringfield;
   DbvErrorReport		*psErrorReport;
   DbvIllString			*psCorrelationInfo;
   DbvUserOrProviderErrorReport *psReport;

   /* Malloc the space */

   if ( *ppsErrorReport == NULL )
      *ppsErrorReport = (DbvErrorReport*)DbvAlloc( sizeof( DbvErrorReport ) );
   psErrorReport = *ppsErrorReport;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   /* sCorrelationInfo */

   psCorrelationInfo = &psErrorReport->sCorrelationInfo;
   get_DbvIllString( &psCorrelationInfo, InTable );

   /* sReport */

   psReport = &psErrorReport->sReport;
   get_DbvUserOrProviderErrorReport( &psReport, InTable );

   } /* End get_DbvErrorReport() */


void get_DbvStatusReport( DbvStatusReport **ppsStatusReport, TBL InTable )
   {
   char *stringfield;
   DbvHistoryReport *psUserStatusReport;
   DbvStatusReport *psStatusReport;


   /* Malloc the space */

   if ( *ppsStatusReport == NULL )
      *ppsStatusReport = (DbvStatusReport*)DbvAlloc( sizeof( DbvStatusReport ));
   psStatusReport = *ppsStatusReport;

   /* sUserStatusReport */

   psUserStatusReport = &psStatusReport->sUserStatusReport;
   get_DbvHistoryReport( &psUserStatusReport, InTable );

   /* eProviderStatusReport */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvCurrentState( &stringfield, 
			    &psStatusReport->eProviderStatusReport);
   DbvFree( stringfield );

   } /* End get_DbvStatusReport() */


void get_DbvStatusOrErrorReport( 
		DbvStatusOrErrorReport **ppsStatusOrErrorReport, 
		TBL InTable )
   {
   char *stringfield;
   DbvHistoryReport 		*psUserStatusOrErrorReport;
   DbvStatusOrErrorReport 	*psStatusOrErrorReport;
   DbvTransactionId 		*psTransactionId;
   DbvServiceDateTime		*psServiceDateTime;

   /* Malloc the space */

   if ( *ppsStatusOrErrorReport == NULL )
      *ppsStatusOrErrorReport = (DbvStatusOrErrorReport*)DbvAlloc( 
					sizeof( DbvStatusOrErrorReport ) );
   psStatusOrErrorReport = *ppsStatusOrErrorReport;


   /* eProtocolVersion */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvIllProtocolVersion( &stringfield,
   &psStatusOrErrorReport->eProtocolVersion);
   DbvFree( stringfield );

   /* sTransactionId */
   psTransactionId = &psStatusOrErrorReport->sTransactionId;
   get_DbvTransactionId( &psTransactionId, InTable );

   /* sServiceDateTime */

   psServiceDateTime = &psStatusOrErrorReport->sServiceDateTime;
   get_DbvServiceDateTime( &psServiceDateTime, InTable );

   /* psRequesterId */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemId( &psStatusOrErrorReport->psRequesterId, InTable);
   DbvFree(stringfield);

   /* psResponderId */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvSystemId( &psStatusOrErrorReport->psResponderId, InTable);
   DbvFree(stringfield);

   /* peReasonNoReport */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvReasonNoReport( &psStatusOrErrorReport->peReasonNoReport,  InTable);
   DbvFree(stringfield);


   /* psStatusReport */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvStatusReport( &psStatusOrErrorReport->psStatusReport, InTable);
   DbvFree(stringfield);

   /* psErrorReport */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvErrorReport( &psStatusOrErrorReport->psErrorReport, InTable);
   DbvFree(stringfield);

   /* psNote */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvIllString( &psStatusOrErrorReport->psNote, InTable);
   DbvFree(stringfield);

   /* ppsStOrErReportExtension */

   getN_DbvExtension( &psStatusOrErrorReport->ppsStOrErReportExtension, 
		      InTable );

   } /* End get_DbvStatusOrErrorReport() */


void get_DbvReasonNoReport( DbvReasonNoReport **ppeReasonNoReport, TBL InTable )
   {
   char *stringfield;
   DbvReasonNoReport *peReasonNoReport;

   /* Malloc the space */

   if ( *ppeReasonNoReport == NULL )
      *ppeReasonNoReport = (DbvReasonNoReport*)DbvAlloc( 
					sizeof( DbvReasonNoReport ));

   peReasonNoReport = *ppeReasonNoReport;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvReasonNoReport( &stringfield, peReasonNoReport);
   DbvFree( stringfield );

   } /* End get_DbvReasonNoReport() */


void get_DbvIllPdu( DbvIllPdu **ppsIllPdu, TBL ExternalTable )
   {
   char *stringfield;
   DbvIllPdu		*psIllPdu;
   DbvIllPduType	eIllPduType;

   /* Malloc the space */

   if ( *ppsIllPdu == NULL )
      *ppsIllPdu = (DbvIllPdu*)DbvAlloc( sizeof( DbvIllPdu ) );
   psIllPdu = *ppsIllPdu;

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvIllPduType( &stringfield, &eIllPduType );
   psIllPdu->eIllPduType = eIllPduType;
   DbvFree( stringfield );
   stringfield = get_string_field( ExternalTable, T_Return );

   switch( eIllPduType )
      {
      case ILL_IllRequest:
         get_DbvIllRequest( &psIllPdu->uIllPdu.psIllRequest, 
			    ExternalTable);
         break;

      case ILL_StatusOrErrorReport:
         get_DbvStatusOrErrorReport( &psIllPdu->
				uIllPdu.psStatusOrErrorReport, 
				ExternalTable );
         break;

      default:
	 break;
      } /* End switch */
   } /* End get_DbvIllPdu() */


void get_DbvItemOrderExtendedService( 
		DbvItemOrderExtendedService **ppsItemOrderExtendedService, 
		TBL ExternalTable )
   {
   char *stringfield;
   DbvItemOrderExtendedService	*psItemOrderExtendedService;
   DbvItemOrderReqOrRsp		eItemOrderReqRsp;

   /* Malloc the space */

   if ( *ppsItemOrderExtendedService == NULL )
      *ppsItemOrderExtendedService = (DbvItemOrderExtendedService*)DbvAlloc(
				sizeof( DbvItemOrderExtendedService ) );
   psItemOrderExtendedService = *ppsItemOrderExtendedService;

   TBL_GetRecord( ExternalTable );

   /* Ignore the title */
   stringfield = get_char_field( ExternalTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( ExternalTable, T_Return );
   convert_DbvItemOrderReqOrRsp( &stringfield, &eItemOrderReqRsp );
   psItemOrderExtendedService->eItemOrderReqRsp = eItemOrderReqRsp;
   DbvFree( stringfield );
   stringfield = get_string_field( ExternalTable, T_Return );

   switch( eItemOrderReqRsp )
      {
      case IOROR_Request:
         get_DbvEsIoRequest( &psItemOrderExtendedService->
			     uItemOrderReqOrRsp.psItemOrderRequest, 
			     ExternalTable);
         break;

      case IOROR_Response:
         get_DbvEsIoTaskPackage( &psItemOrderExtendedService->
				uItemOrderReqOrRsp.psItemOrderResponse,
				ExternalTable );
         break;

      default:
	 break;
      } /* End switch */
   
   } /* End get_DbvItemOrderExtendedService() */



void get_DbvEsIoTaskPackage( DbvEsIoTaskPackage **ppsEsIoTaskPackage, 
		TBL InTable )
   {
   char *stringfield;
   DbvEsIoTaskPackage	*psEsIoTaskPackage;
   DbvEsIoTargetPart	*psTargetPart;

   /* Malloc the space */

   if ( *ppsEsIoTaskPackage == NULL )
      *ppsEsIoTaskPackage = (DbvEsIoTaskPackage*)DbvAlloc(
				sizeof( DbvEsIoTaskPackage ) );
   psEsIoTaskPackage = *ppsEsIoTaskPackage;

   /* psOriginPart */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvEsIoOriginPartToKeep( &psEsIoTaskPackage->psOriginPart, InTable );
   
   DbvFree(stringfield);

   /* sTargetPart */

   psTargetPart = &psEsIoTaskPackage->sTargetPart;
   get_DbvEsIoTargetPart( &psTargetPart, InTable );

   } /* End get_DbvEsIoTaskPackage() */



void get_DbvEsIoRequest( DbvEsIoRequest **ppsEsIoRequest, TBL InTable )
   {
   char *stringfield;
   DbvEsIoRequest	*psEsIoRequest;
   DbvEsIoOriginPartNotToKeep *psNotToKeep;

   /* Malloc the space */

   if ( *ppsEsIoRequest == NULL )
      *ppsEsIoRequest = (DbvEsIoRequest*)DbvAlloc(
				sizeof( DbvEsIoRequest ) );
   psEsIoRequest = *ppsEsIoRequest;

   /* psToKeep */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvEsIoOriginPartToKeep( &psEsIoRequest->psToKeep, InTable );
   
   DbvFree(stringfield);

   /* sNotToKeep */

   psNotToKeep = &psEsIoRequest->sNotToKeep;
   get_DbvEsIoOriginPartNotToKeep( &psNotToKeep, InTable );

   } /* End get_DbvEsIoRequest() */


void get_DbvEsIoOriginPartToKeep( 
		DbvEsIoOriginPartToKeep **ppsEsIoOriginPartToKeep, 
		TBL InTable )
   {
   char *stringfield;
   DbvEsIoOriginPartToKeep *psEsIoOriginPartToKeep;

   /* Malloc the space */

   if ( *ppsEsIoOriginPartToKeep == NULL )
      *ppsEsIoOriginPartToKeep = (DbvEsIoOriginPartToKeep*)DbvAlloc(
				sizeof( DbvEsIoOriginPartToKeep ) );
   psEsIoOriginPartToKeep = *ppsEsIoOriginPartToKeep;

   /* psSupplementalDescription */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvASN1External( &psEsIoOriginPartToKeep->psSupplementalDescription,
			   InTable );
   
   DbvFree(stringfield);

   /* psContact */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvEsIoContact( &psEsIoOriginPartToKeep->psContact, InTable );
   
   DbvFree(stringfield);

   /* psAdditionalBilling */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvEsIoAdditionalBilling( &psEsIoOriginPartToKeep->
				    psAdditionalBilling,
				    InTable );
   
   DbvFree(stringfield);

   } /* End get_DbvEsIoOriginPartToKeep() */


void get_DbvEsIoContact( DbvEsIoContact **ppsEsIoContact, TBL InTable )
   {
   char *stringfield;
   DbvEsIoContact *psEsIoContact;

   /* Malloc the space */

   if ( *ppsEsIoContact == NULL )
      *ppsEsIoContact = (DbvEsIoContact*)DbvAlloc(
				sizeof( DbvEsIoContact ) );
   psEsIoContact = *ppsEsIoContact;

   /* psName */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvInternationalString( &psEsIoContact->psName, InTable );

   DbvFree(stringfield);

   /* psPhone */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvInternationalString( &psEsIoContact->psPhone, InTable );

   DbvFree(stringfield);

   /* psEmail */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvInternationalString( &psEsIoContact->psEmail, InTable );

   DbvFree(stringfield);

   } /* End get_DbvEsIoContact() */


void get_DbvEsIoAdditionalBilling( 
	DbvEsIoAdditionalBilling **ppsEsIoAdditionalBilling, 
	TBL InTable )
   {
   char *stringfield;
   DbvEsIoAdditionalBilling *psEsIoAdditionalBilling;
   DbvEsIoPaymentMethod *psPaymentMethod;

#ifdef DEBUG
   printf( "D - get_DbvEsIoAdditionalBilling()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoAdditionalBilling == NULL )
      *ppsEsIoAdditionalBilling = (DbvEsIoAdditionalBilling*)DbvAlloc(
				sizeof( DbvEsIoAdditionalBilling ) );
   psEsIoAdditionalBilling = *ppsEsIoAdditionalBilling;

   /* sPaymentMethod */

   psPaymentMethod = &psEsIoAdditionalBilling->sPaymentMethod;
   get_DbvEsIoPaymentMethod( &psPaymentMethod, InTable );

   /* psCustomerReference */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvInternationalString( &psEsIoAdditionalBilling->
				  psCustomerReference, InTable );

   DbvFree(stringfield);

   /* psCustomerPONumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvInternationalString( &psEsIoAdditionalBilling->
				  psCustomerPONumber, InTable );

   DbvFree(stringfield);

   } /* End get_DbvEsIoAdditionalBilling() */


void get_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod **ppsEsIoPaymentMethod, 
			       TBL InTable )
   {
   char *stringfield;
   DbvEsIoPaymentMethod		*psEsIoPaymentMethod;
   DbvEsIoPaymentMethodType	ePaymentMethodType;

#ifdef DEBUG
   printf( "D - get_DbvEsIoPaymentMethod()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoPaymentMethod == NULL )
      *ppsEsIoPaymentMethod = (DbvEsIoPaymentMethod*)DbvAlloc( 
					sizeof( DbvEsIoPaymentMethod ) );
   psEsIoPaymentMethod = *ppsEsIoPaymentMethod;

   TBL_GetRecord( InTable );

   /* Ignore the title */
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvEsIoPaymentMethodType( &stringfield, &ePaymentMethodType );
   psEsIoPaymentMethod->ePaymentMethodType = ePaymentMethodType;
   DbvFree( stringfield );
   stringfield = get_string_field( InTable, T_Return );

   switch( ePaymentMethodType )
      {
      case IOPM_CreditCard:
	 psEsIoPaymentMethod->puPaymentMethod = ( union uPaymentMethod* )
 	 			DbvAlloc( sizeof( union uPaymentMethod  ));

         get_DbvEsIoCreditCardInfo( &psEsIoPaymentMethod->puPaymentMethod->
				    psCreditCardInfo, InTable);
         break;

      case IOPM_PrivateNotKnown:
	 psEsIoPaymentMethod->puPaymentMethod = ( union uPaymentMethod* )
 	 			DbvAlloc( sizeof( union uPaymentMethod  ));

         get_DbvASN1External( &psEsIoPaymentMethod->puPaymentMethod->
			      psPrivateNotKnown, InTable);
         break;

      case IOPM_BillInvoice:
      case IOPM_Prepay:
      case IOPM_DepositAccount:
      case IOPM_CardInfoPrevSupplied:
      case IOPM_PrivateKnown:
      case IOPM_Invalid:
      default:
	 break;
      } /* End switch */

   } /* End get_DbvEsIoPaymentMethod() */


void get_DbvEsIoCreditCardInfo( DbvEsIoCreditCardInfo **ppsEsIoCreditCardInfo,
				TBL InTable )
   {
   char *stringfield;
   DbvEsIoCreditCardInfo *psEsIoCreditCardInfo;
   DbvInternationalString *psInternationalString;

#ifdef DEBUG
   printf( "D - get_DbvEsIoCreditCardInfo()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoCreditCardInfo == NULL )
      *ppsEsIoCreditCardInfo = (DbvEsIoCreditCardInfo*)DbvAlloc(
				sizeof( DbvEsIoCreditCardInfo ) );
   psEsIoCreditCardInfo = *ppsEsIoCreditCardInfo;

   /* sNameOnCard */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psInternationalString = &psEsIoCreditCardInfo->sNameOnCard;
   get_compulsory_DbvInternationalString( psInternationalString, InTable );

   /* sExpirationDate */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psInternationalString = &psEsIoCreditCardInfo->sExpirationDate;
   get_compulsory_DbvInternationalString( psInternationalString, InTable );

   /* sCardNumber */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psInternationalString = &psEsIoCreditCardInfo->sCardNumber;
   get_compulsory_DbvInternationalString( psInternationalString, InTable );

   } /* End get_DbvEsIoCreditCardInfo() */


void get_DbvEsIoOriginPartNotToKeep( 
			DbvEsIoOriginPartNotToKeep **ppsEsIoOriginPartNotToKeep,
			TBL InTable )
   {
   char *stringfield;
   DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep;

#ifdef DEBUG
   printf( "D - get_DbvEsIoOriginPartNotToKeep()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoOriginPartNotToKeep == NULL )
      *ppsEsIoOriginPartNotToKeep = (DbvEsIoOriginPartNotToKeep*)DbvAlloc(
				sizeof( DbvEsIoOriginPartNotToKeep ) );
   psEsIoOriginPartNotToKeep = *ppsEsIoOriginPartNotToKeep;

   /* psResultSetItem */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvEsIoResultSetItem( &psEsIoOriginPartNotToKeep-> psResultSetItem,
				 InTable );
   
   DbvFree(stringfield);

   /* psItemRequest */

   get_DbvASN1External( &psEsIoOriginPartNotToKeep->psItemRequest, InTable );

   } /* End get_DbvEsIoOriginPartNotToKeep() */


void get_DbvEsIoResultSetItem( DbvEsIoResultSetItem **ppsEsIoResultSetItem,
			TBL InTable )
   {
   char *stringfield;
   DbvEsIoResultSetItem		*psEsIoResultSetItem;
   DbvInternationalString    	*psResultSetId;

#ifdef DEBUG
   printf( "D - get_DbvEsIoResultSetItem()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoResultSetItem == NULL )
      *ppsEsIoResultSetItem = (DbvEsIoResultSetItem*)DbvAlloc(
				sizeof( DbvEsIoResultSetItem ) );
   psEsIoResultSetItem = *ppsEsIoResultSetItem;

   /* sResultSetId */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psResultSetId = &psEsIoResultSetItem->sResultSetId;
   get_compulsory_DbvInternationalString( psResultSetId, InTable );
   

   /* ulItem */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psEsIoResultSetItem->ulItem = (Ulong)get_int_field( InTable, T_Return);

   } /* End get_DbvEsIoResultSetItem() */


void get_DbvEsIoTargetPart( DbvEsIoTargetPart **ppsEsIoTargetPart, TBL InTable )
   {
   char *stringfield;
   DbvEsIoTargetPart	*psEsIoTargetPart;

#ifdef DEBUG
   printf( "D - get_DbvEsIoTargetPart()\n" );
#endif
   /* Malloc the space */

   if ( *ppsEsIoTargetPart == NULL )
      *ppsEsIoTargetPart = (DbvEsIoTargetPart*)DbvAlloc(
				sizeof( DbvEsIoTargetPart ) );

   psEsIoTargetPart = *ppsEsIoTargetPart;

   /* psItemRequest */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
      get_DbvASN1External( &psEsIoTargetPart->psItemRequest, InTable );
   
   DbvFree(stringfield);

   /* psStatusOrErrorReport */

   get_DbvASN1External( &psEsIoTargetPart->psStatusOrErrorReport, InTable );
   
   /* peAuxiliaryStatus */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 )
   	get_DbvEsIoAuxiliaryStatus(  &psEsIoTargetPart->peAuxiliaryStatus, InTable );
   DbvFree(stringfield);



   } /* End get_DbvEsIoTargetPart() */


void get_DbvEsIoAuxiliaryStatus( 
		DbvEsIoAuxiliaryStatus **ppeEsIoAuxiliaryStatus, 
		TBL InTable )
   {
   char *stringfield;
   DbvEsIoAuxiliaryStatus *peEsIoAuxiliaryStatus;

#ifdef DEBUG
   printf( "D - get_DbvEsIoAuxiliaryStatus()\n" );
#endif
   /* Malloc the space */

   if ( *ppeEsIoAuxiliaryStatus == NULL )
      *ppeEsIoAuxiliaryStatus = (DbvEsIoAuxiliaryStatus*)DbvAlloc( 
					sizeof( DbvEsIoAuxiliaryStatus ));

   peEsIoAuxiliaryStatus = *ppeEsIoAuxiliaryStatus;

   stringfield = get_string_field( InTable, T_Return );
   convert_DbvEsIoAuxiliaryStatus( &stringfield, peEsIoAuxiliaryStatus);
   DbvFree( stringfield );

   } /* End get_DbvEsIoAuxiliaryStatus() */


void get_DbvGeneralString(DbvGeneralString **ppsGeneralString, TBL InTable)
   {
   get_DbvOctetString((DbvOctetString **)ppsGeneralString, InTable);
   }



void get_DbvAddRequesterInfo( DbvAddRequesterInfo **ppsAddRequesterInfo, TBL InTable )
{
   char				*stringfield = NULL;
   DbvAddRequesterInfo		*psAddRequesterInfo = NULL;


   if ( *ppsAddRequesterInfo == NULL )
      *ppsAddRequesterInfo = (DbvAddRequesterInfo*)DbvAlloc(
				sizeof( DbvAddRequesterInfo ) );
   psAddRequesterInfo = *ppsAddRequesterInfo;


   /* default - peRequesterGroup: peRequesterGroup, "COMPLETED", "RG_UserGroup1" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {
   	stringfield = get_string_field( InTable, T_Return );
   	psAddRequesterInfo->peRequesterGroup = (DbvRequesterGroup *) DbvAlloc (sizeof (DbvRequesterGroup));
   	convert_DbvRequesterGroup( &stringfield, psAddRequesterInfo->peRequesterGroup );
   }
   DbvFree(stringfield);


   /* o - pcRequesterEmailAddress: pcRequesterEmailAddress, "NULL" 
				   pcRequesterEmailAddress, "COMPLETED", "john@test.com" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
   	get_DbvVisibleString( &psAddRequesterInfo->pcRequesterEmailAddress, InTable );
   DbvFree(stringfield);


} /* End get_DbvAddRequesterInfo() */



void get_DbvAddDeliveryInfo( DbvAddDeliveryInfo **ppsAddDeliveryInfo, TBL InTable )
{
   char 			*stringfield = NULL;
   DbvAddDeliveryInfo		*psAddDeliveryInfo = NULL;


   if ( *ppsAddDeliveryInfo == NULL )
      *ppsAddDeliveryInfo = (DbvAddDeliveryInfo*)DbvAlloc(
				sizeof( DbvAddDeliveryInfo ) );
   psAddDeliveryInfo = *ppsAddDeliveryInfo;


   /* default - peStatusLevelUser: peStatusLevelUser, "COMPLETED", "SLU_Negative"  */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {
   	stringfield = get_string_field( InTable, T_Return );
   	psAddDeliveryInfo->peStatusLevelUser = (DbvStatusLevelUser *) DbvAlloc (sizeof (DbvStatusLevelUser));
   	convert_DbvStatusLevelUser( &stringfield, psAddDeliveryInfo->peStatusLevelUser);
   }
   DbvFree(stringfield);



   /* default - peStatusLevelOrigin: peStatusLevelOrigin, "COMPLETED", "SLO_Negative" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {
	stringfield = get_string_field( InTable, T_Return );
	psAddDeliveryInfo->peStatusLevelOrigin = (DbvStatusLevelRequester *) DbvAlloc(sizeof (DbvStatusLevelRequester));
	convert_DbvStatusLevelRequester( &stringfield, psAddDeliveryInfo->peStatusLevelOrigin );

   }
   DbvFree(stringfield);




   /* o - pcDeliveryNotes: pcDeliveryNotes, "COMPLETED", "notes"  */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) 
   	get_DbvVisibleString( &psAddDeliveryInfo->pcDeliveryNotes, InTable );
   DbvFree(stringfield);

   /* o - pcCountryDeliveryTarget */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) 
   	get_DbvVisibleString( &psAddDeliveryInfo->pcCountryDeliveryTarget, InTable );
   DbvFree(stringfield);





   /* default - pfAlternativePostalDelivery: pfAlternativePostalDelivery, "COMPLETED", "DBV_TRUE" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {
   	psAddDeliveryInfo->pfAlternativePostalDelivery = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*psAddDeliveryInfo->pfAlternativePostalDelivery = get_DbvBool( InTable );
   }
   DbvFree(stringfield);


  

} /* End get_DbvAddDeliveryInfo() */



void get_DbvBillingBankInfo(DbvBillingBankInfo **ppsBillingBankInfo, TBL InTable)
{
   char 			*stringfield = NULL;
   DbvBillingBankInfo		*psBillingBankInfo = NULL;


   if ( *ppsBillingBankInfo == NULL )
      *ppsBillingBankInfo = (DbvBillingBankInfo*)DbvAlloc(
				sizeof( DbvBillingBankInfo ) );
   psBillingBankInfo = *ppsBillingBankInfo;


   /* c - pcBankAccountHouse: pcBankAccountHouse, "bank account house"   */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingBankInfo->pcBankAccountHouse, InTable );


   /* c - pcBankAccountRoutingSymbol: pcBankAccountRoutingSymbol, "routing symbol"   */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingBankInfo->pcBankAccountRoutingSymbol, InTable );


   /* c - pcBankAccountNo: pcBankAccountNo, "bank account number"  */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingBankInfo->pcBankAccountNo, InTable );


   /* c - fBankAuthority: fBankAuthority, "bank authority"  */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   psBillingBankInfo->fBankAuthority = get_DbvBool( InTable );


} /* End get_DbvBillingBankInfo() */



void get_DbvBillingCreditCardInfo(DbvBillingCreditCardInfo **ppsBillingCreditCardInfo, TBL InTable)
{
   char 			*stringfield = NULL;
   DbvBillingCreditCardInfo	*psBillingCreditCardInfo = NULL;


   if ( *ppsBillingCreditCardInfo == NULL )
      *ppsBillingCreditCardInfo = (DbvBillingCreditCardInfo*)DbvAlloc(
				sizeof( DbvBillingCreditCardInfo ) );
   psBillingCreditCardInfo = *ppsBillingCreditCardInfo;


   /* c - pcCardCompany: pcCardCompany, "card company" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingCreditCardInfo->pcCardCompany, InTable );


   /* c - pcCardNo: pcCardNo, "card number" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingCreditCardInfo->pcCardNo, InTable );


   /* o - pcCardValidFrom */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) 
   	get_DbvVisibleString( &psBillingCreditCardInfo->pcCardValidFrom, InTable );
   DbvFree(stringfield);


   /* c - pcCardValidThru: pcCardValidThru, "valid thru"  */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   get_DbvVisibleString( &psBillingCreditCardInfo->pcCardValidThru, InTable );


} /* End get_DbvBillingCreditCardInfo() */



void get_DbvAddBillingInfo( DbvAddBillingInfo **ppsAddBillingInfo, TBL InTable )
{
   char 			*stringfield = NULL;
   DbvAddBillingInfo		*psAddBillingInfo = NULL;


   if ( *ppsAddBillingInfo == NULL )
      *ppsAddBillingInfo = (DbvAddBillingInfo*)DbvAlloc(
				sizeof( DbvAddBillingInfo ) );
   psAddBillingInfo = *ppsAddBillingInfo;

   /* c - eAddBillingType: eAddBillingType, "ABT_CreditCardInfo" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   convert_DbvAddBillingType( &stringfield, &psAddBillingInfo->eAddBillingType );
   DbvFree( stringfield );

   /* c - uAddBillingInfo */

   switch( psAddBillingInfo->eAddBillingType )
      {
      case ABT_BankInfo:
         get_DbvBillingBankInfo(&psAddBillingInfo->uAddBillingInfo.psBillingBankInfo, InTable);
         break;

      case ABT_CreditCardInfo:
         get_DbvBillingCreditCardInfo(&psAddBillingInfo->uAddBillingInfo.psBillingCreditCardInfo, InTable);
         break;

      case ABT_InvoiceType:

	/* eBillingInvoiceType: "eBillingInvoiceType", "BIT_Collective" */
	TBL_GetRecord( InTable );
	stringfield = get_char_field( InTable, T_Return );
	DbvFree(stringfield);
	stringfield = get_string_field( InTable, T_Return );
	convert_DbvBillingInvoiceType( &stringfield, &psAddBillingInfo->uAddBillingInfo.eBillingInvoiceType );
	DbvFree( stringfield );

	break;

      case ABT_PrepaidType:
        TBL_GetRecord( InTable );
        stringfield = get_char_field( InTable, T_Return );
        DbvFree(stringfield);
        get_DbvVisibleString( &psAddBillingInfo->uAddBillingInfo.pcPrepaidId, InTable );
        break;

      case ABT_DepositType:
        TBL_GetRecord( InTable );
        stringfield = get_char_field( InTable, T_Return );
        DbvFree(stringfield);
        get_DbvVisibleString( &psAddBillingInfo->uAddBillingInfo.pcDepositId, InTable );
        break;

      default:
	 break;
      } /* End switch */

} /* End get_DbvAddBillingInfo() */



void get_DbvSubitoAdditionalInfo( DbvSubitoAdditionalInfo **ppsAddIllReqInfo, TBL InTable )
{
   char 			*stringfield = NULL;
   DbvSubitoAdditionalInfo	*psAddIllReqInfo = NULL;

   if ( *ppsAddIllReqInfo == NULL )
      *ppsAddIllReqInfo = (DbvSubitoAdditionalInfo*)DbvAlloc(
				sizeof( DbvSubitoAdditionalInfo ) );
   psAddIllReqInfo = *ppsAddIllReqInfo;


   /* default - pfCustomerInfoToKeep: pfCustomerInfoToKeep, "COMPLETED", "DBV_FALSE" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if( (strcmp(stringfield, NOTHING)) != 0 ) {
   	psAddIllReqInfo->pfCustomerInfoToKeep = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*psAddIllReqInfo->pfCustomerInfoToKeep = get_DbvBool( InTable );
   }
   DbvFree(stringfield);






   /* o - psAddRequesterInfo: psAddRequesterInfo, "COMPLETED" 						     psAddRequesterInfo, "NULL" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
   	get_DbvAddRequesterInfo( &psAddIllReqInfo->psAddRequesterInfo, InTable );
   DbvFree(stringfield);



   /* o - psAddDeliveryInfo: psAddDeliveryInfo, "COMPLETED" 						     psAddDeliveryInfo, "NULL" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
   	get_DbvAddDeliveryInfo( &psAddIllReqInfo->psAddDeliveryInfo, InTable );
   DbvFree(stringfield);



   /* o - psAddBillingInfo: psAddBillingInfo, "COMPLETED" 						     psAddBillingInfo, "NULL" */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);
   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 )
   	get_DbvAddBillingInfo( &psAddIllReqInfo->psAddBillingInfo, InTable );
   DbvFree(stringfield);


} /* End get_DbvSubitoAdditionalInfo() */


void get_DbvExtension( DbvExtension **ppsExtension, TBL InTable )
   {
   char *stringfield;
   DbvExtension		*psExtension;
   DbvBool		*pBool;

   /* Malloc the space */

   if ( *ppsExtension == NULL )
      *ppsExtension = (DbvExtension*)DbvAlloc(
				sizeof( DbvExtension ) );
   psExtension = *ppsExtension;

   /* ulIdentifier */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   psExtension->ulIdentifier = (Ulong)get_int_field( InTable, T_Return);

   /* pfCritical - default */

   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	pBool = (DbvBool*)DbvAlloc( sizeof( DbvBool ));
   	*pBool = get_DbvBool( InTable );
   	psExtension->pfCritical = pBool;
   } 
   DbvFree(stringfield);

   /* uItem */

   get_DbvSubitoAdditionalInfo( &psExtension->uItem.psAddIllReqInfo, InTable );

   } /* End get_DbvExtension() */


void get_DbvElDeliveryParameter( 
		DbvElDeliveryParameter **ppsElDeliveryParameter, 
		TBL InTable )
   {
   char *stringfield;
   DbvElDeliveryParameter	*psElDeliveryParameter;
   union uDeliveryParameter 	*puDeliveryParameter;

#ifdef DEBUG
   printf( "D - get_DbvElDeliveryParameter()\n" );
#endif
   /* Malloc the space */

   if ( *ppsElDeliveryParameter == NULL )
      *ppsElDeliveryParameter = (DbvElDeliveryParameter*)DbvAlloc(
				sizeof( DbvElDeliveryParameter ) );
   psElDeliveryParameter = *ppsElDeliveryParameter;

   /* pcDeliveryMode */
   TBL_GetRecord( InTable );
   get_DbvObjectIdentifier( &psElDeliveryParameter->pcDeliveryMode, InTable );

   /* puDeliveryParameter */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	puDeliveryParameter = (union uDeliveryParameter*)DbvAlloc( 
					sizeof( union uDeliveryParameter ));
   	get_DbvASN1External( &puDeliveryParameter->psDummy,  InTable );
   	psElDeliveryParameter->puDeliveryParameter = puDeliveryParameter;
   } 
   DbvFree(stringfield);



   } /* End get_DbvElDeliveryParameter() */

void get_DbvDocumentTypeParameter( 
		DbvDocumentTypeParameter **ppsDocumentTypeParameter, 
		TBL InTable )
   {
   char *stringfield;
   DbvDocumentTypeParameter	*psDocumentTypeParameter;
   union uDocumentTypeParameter *puDocumentTypeParameter;

#ifdef DEBUG
   printf( "D - get_DbvDocumentTypeParameter()\n" );
#endif
   /* Malloc the space */

   if ( *ppsDocumentTypeParameter == NULL )
      *ppsDocumentTypeParameter = (DbvDocumentTypeParameter*)DbvAlloc(
				sizeof( DbvDocumentTypeParameter ) );
   psDocumentTypeParameter = *ppsDocumentTypeParameter;

   /* pcDocumentTypeId */
   TBL_GetRecord( InTable );
   get_DbvObjectIdentifier( &psDocumentTypeParameter->pcDocumentTypeId, InTable );

   /* puDeliveryParameter */
   TBL_GetRecord( InTable );
   stringfield = get_char_field( InTable, T_Return );
   DbvFree(stringfield);

   stringfield = get_string_field( InTable, T_Return );
   if ( strcmp( stringfield, NOTHING ) != 0 ) {

   	/* Note : We Need to Malloc some space here ( Yuk ) */
   	puDocumentTypeParameter = (union uDocumentTypeParameter*)DbvAlloc( 
					sizeof( union uDocumentTypeParameter ));
   	get_DbvASN1External( &puDocumentTypeParameter->psDummy,  InTable );
   	psDocumentTypeParameter->puDocumentTypeParameter = puDocumentTypeParameter;
	
   } 
   DbvFree(stringfield);



   } /* End get_DbvDocumentTypeParameter() */
